function main=main()
% command line: 
%      'KaDE' 'fceri_ji.ka''--print-efficiency'
%% THINGS THAT ARE KNOWN FROM KAPPA FILE AND KaSim OPTIONS:
%% 
%% init - the initial abundances of each species and token
%% tinit - the initial simulation time (likely 0)
%% tend - the final simulation time 
%% initialstep - initial time step at the beginning of numerical integration
%% maxstep - maximal time step for numerical integration
%% reltol - relative error tolerance;
%% abstol - absolute error tolerance;
%% period - the time period between points to return
%%
%% variables (init(i),y(i)) denote numbers of embeddings 
%% rule rates are corrected by the number of automorphisms in the lhs of rules


tinit=0;
tend=1;
initialstep=1e-05;
maxstep=0.02;
reltol=0.001;
abstol=0.001;
period=0.01;
nonnegative=false;

global nodevar
nodevar=655;
global max_stoc_coef
max_stoc_coef=0;
nvar=30;
nobs=1;
nrules=36;

global var
var=zeros(nvar,1);
global init
init=sparse(nodevar,1);
stoc=zeros(nrules,max_stoc_coef);
global k
global kd
global kun
global kdun
global stoc

k=zeros(nrules,1);
kd=zeros(nrules,1);
kun=zeros(nrules,1);
kdun=zeros(nrules,1);
global jacvar
jacvar=sparse(nvar,nodevar);
global jack
global jackd
global jackun
global jackund
global jacstoc

jack=zeros(nrules,nodevar);
jackd=zeros(nrules,nodevar);
jackun=zeros(nrules,nodevar);
jackund=zeros(nrules,nodevar);

t = 0.000000;

init(655)=t;
var(2)=400; % Rec_tot
init(1)=var(2); % Rec(a, b~Y, g~Y)
var(4)=400; % Syk_tot
init(2)=var(4); % Syk(tSH2, l~Y, a~Y)
var(3)=28; % Lyn_tot
init(3)=var(3); % Lyn(U, SH2)
var(1)=6000; % Lig_tot
init(4)=var(1); % Lig(l1, l2)
var(26)=20; % dc
var(25)=20; % dm
var(24)=200; % pSSs
var(23)=100; % pSS
var(22)=100; % pLSs
var(21)=30; % pLS
var(20)=3; % pLgs
var(19)=1; % pLg
var(18)=100; % pLbs
var(17)=30; % pLb
var(16)=0.13; % kmSs
var(15)=0.06; % kpSs
var(14)=0.13; % kmS
var(13)=0.06; % kpS
var(12)=0.12; % kmLs
var(11)=0.05; % kpLs
var(10)=20; % kmL
var(9)=0.05; % kpL
var(8)=0; % km2
var(7)=0.25; % kp2
var(6)=0; % km1
var(5)=1.32845e-07; % kp1


uiIsOctave = false;
uiIsMatlab = false;
LIC = license('inuse');
for elem = 1:numel(LIC)
    envStr = LIC(elem).feature
    if strcmpi(envStr,'octave')
       LICname=envStr;
       uiIsOctave = true;
       break
    end
    if strcmpi(envStr,'matlab')
       LICname=envStr
       uiIsMatlab = true;
       break
    end
end


if nonnegative 
   options = odeset('RelTol', reltol, ...
                    'AbsTol', abstol, ...
                    'InitialStep', initialstep, ...
                    'MaxStep', maxstep, ...
                    'Jacobian', @ode_jacobian, ...
                   'NonNegative', [1:1:654]);
else
   options = odeset('RelTol', reltol, ...
                    'AbsTol', abstol, ...
                    'InitialStep', initialstep, ...
                    'MaxStep', maxstep, ...
                    'Jacobian', @ode_jacobian);
end


if nonnegative
   if uiIsMatlab
      soln =  ode15s(@ode_aux,[tinit tend],ode_init(),options);
      soln.y=soln.y';
      vt = soln.x;
      vy = soln.y;
   elseif uiIsOctave
      [vt,vy] = ode23s(@ode_aux,[tinit tend],ode_init(),options);
   end
else
   if uiIsMatlab
      soln =  ode15s(@ode_aux,[tinit tend],ode_init(),options);
      soln.y=soln.y';
      vt = soln.x;
      vy = soln.y;
   elseif uiIsOctave
      soln = ode2r(@ode_aux,[tinit tend],ode_init(),options);
      vt = soln.x;
      vy = soln.y;
   end
end;


nrows = length(vt);

tmp = zeros(nodevar,1);

n_points = floor ((tend-tinit)/period)+1;
t = linspace(tinit, tend, n_points);
obs = zeros(nrows,nobs);

for j=1:nrows
    for i=1:nodevar
        z(i)=vy(j,i);
    end
    h=ode_obs(z);
    for i=1:nobs
        obs(j,i)=h(i);
    end
end
if nobs==1
   y = interp1(vt, obs, t, 'pchip')';
else
   y = interp1(vt, obs, t, 'pchip');
end


filename = 'data.csv';
fid = fopen (filename,'w');
fprintf(fid,'# KaDE fceri_ji.ka --print-efficiency\n')
fprintf(fid,'# ')
fprintf(fid,'[T],')
fprintf(fid,'\n')
for j=1:n_points
    for i=1:nobs
        fprintf(fid,'%f,',y(j,i));
    end
    fprintf(fid,'\n');
end
fclose(fid);


end



function Init=ode_init()

global nodevar
global init
Init=zeros(nodevar,1);

Init(1) = init(1); % Rec(a, b~Y, g~Y)
Init(2) = init(2); % Syk(tSH2, l~Y, a~Y)
Init(3) = init(3); % Lyn(U, SH2)
Init(4) = init(4); % Lig(l1, l2)
Init(5) = init(5); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2)
Init(6) = init(6); % Rec(a!1, b~Y, g~Y), Lig(l1, l2!1)
Init(7) = init(7); % Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2)
Init(8) = init(8); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2)
Init(9) = init(9); % Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2)
Init(10) = init(10); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2)
Init(11) = init(11); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2)
Init(12) = init(12); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y)
Init(13) = init(13); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2)
Init(14) = init(14); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2)
Init(15) = init(15); % Rec(a, b~Y, g~pY)
Init(16) = init(16); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2)
Init(17) = init(17); % Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2)
Init(18) = init(18); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y)
Init(19) = init(19); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2)
Init(20) = init(20); % Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y)
Init(21) = init(21); % Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y)
Init(22) = init(22); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y)
Init(23) = init(23); % Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2)
Init(24) = init(24); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y)
Init(25) = init(25); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y)
Init(26) = init(26); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y)
Init(27) = init(27); % Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y)
Init(28) = init(28); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y)
Init(29) = init(29); % Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2)
Init(30) = init(30); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y)
Init(31) = init(31); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y)
Init(32) = init(32); % Syk(tSH2, l~pY, a~Y)
Init(33) = init(33); % Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y)
Init(34) = init(34); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y)
Init(35) = init(35); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y)
Init(36) = init(36); % Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4)
Init(37) = init(37); % Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y)
Init(38) = init(38); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y)
Init(39) = init(39); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y)
Init(40) = init(40); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2)
Init(41) = init(41); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2)
Init(42) = init(42); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2)
Init(43) = init(43); % Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y)
Init(44) = init(44); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y)
Init(45) = init(45); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y)
Init(46) = init(46); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2)
Init(47) = init(47); % Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2)
Init(48) = init(48); % Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2)
Init(49) = init(49); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y)
Init(50) = init(50); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y)
Init(51) = init(51); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2)
Init(52) = init(52); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2)
Init(53) = init(53); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2)
Init(54) = init(54); % Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1)
Init(55) = init(55); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2)
Init(56) = init(56); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2)
Init(57) = init(57); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2)
Init(58) = init(58); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2)
Init(59) = init(59); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2)
Init(60) = init(60); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2)
Init(61) = init(61); % Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1)
Init(62) = init(62); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2)
Init(63) = init(63); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2)
Init(64) = init(64); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2)
Init(65) = init(65); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2)
Init(66) = init(66); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2)
Init(67) = init(67); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2)
Init(68) = init(68); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2)
Init(69) = init(69); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y)
Init(70) = init(70); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2)
Init(71) = init(71); % Rec(a!1, b~pY, g~Y), Lig(l1, l2!1)
Init(72) = init(72); % Rec(a, b~pY, g~Y)
Init(73) = init(73); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4)
Init(74) = init(74); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y)
Init(75) = init(75); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2)
Init(76) = init(76); % Rec(a!1, b~pY, g~pY), Lig(l1, l2!1)
Init(77) = init(77); % Rec(a, b~pY, g~pY)
Init(78) = init(78); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4)
Init(79) = init(79); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY)
Init(80) = init(80); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y)
Init(81) = init(81); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y)
Init(82) = init(82); % Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y)
Init(83) = init(83); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4)
Init(84) = init(84); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y)
Init(85) = init(85); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y)
Init(86) = init(86); % Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y)
Init(87) = init(87); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y)
Init(88) = init(88); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y)
Init(89) = init(89); % Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y)
Init(90) = init(90); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y)
Init(91) = init(91); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2)
Init(92) = init(92); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y)
Init(93) = init(93); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY)
Init(94) = init(94); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y)
Init(95) = init(95); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y)
Init(96) = init(96); % Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y)
Init(97) = init(97); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY)
Init(98) = init(98); % Rec(a!1, b~Y, g~pY), Lig(l1, l2!1)
Init(99) = init(99); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2)
Init(100) = init(100); % Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2)
Init(101) = init(101); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y)
Init(102) = init(102); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y)
Init(103) = init(103); % Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y)
Init(104) = init(104); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2)
Init(105) = init(105); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y)
Init(106) = init(106); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y)
Init(107) = init(107); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4)
Init(108) = init(108); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2)
Init(109) = init(109); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y)
Init(110) = init(110); % Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y)
Init(111) = init(111); % Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y)
Init(112) = init(112); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y)
Init(113) = init(113); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4)
Init(114) = init(114); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y)
Init(115) = init(115); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y)
Init(116) = init(116); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4)
Init(117) = init(117); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4)
Init(118) = init(118); % Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2)
Init(119) = init(119); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3)
Init(120) = init(120); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4)
Init(121) = init(121); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4)
Init(122) = init(122); % Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2)
Init(123) = init(123); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3)
Init(124) = init(124); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2)
Init(125) = init(125); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4)
Init(126) = init(126); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2)
Init(127) = init(127); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3)
Init(128) = init(128); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2)
Init(129) = init(129); % Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2)
Init(130) = init(130); % Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2)
Init(131) = init(131); % Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y)
Init(132) = init(132); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y)
Init(133) = init(133); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y)
Init(134) = init(134); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5)
Init(135) = init(135); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y)
Init(136) = init(136); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y)
Init(137) = init(137); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y)
Init(138) = init(138); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y)
Init(139) = init(139); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y)
Init(140) = init(140); % Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y)
Init(141) = init(141); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y)
Init(142) = init(142); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y)
Init(143) = init(143); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y)
Init(144) = init(144); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y)
Init(145) = init(145); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y)
Init(146) = init(146); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2)
Init(147) = init(147); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2)
Init(148) = init(148); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2)
Init(149) = init(149); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2)
Init(150) = init(150); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY)
Init(151) = init(151); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2)
Init(152) = init(152); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y)
Init(153) = init(153); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y)
Init(154) = init(154); % Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y)
Init(155) = init(155); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2)
Init(156) = init(156); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y)
Init(157) = init(157); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(158) = init(158); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(159) = init(159); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2)
Init(160) = init(160); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y)
Init(161) = init(161); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(162) = init(162); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(163) = init(163); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(164) = init(164); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(165) = init(165); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2)
Init(166) = init(166); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y)
Init(167) = init(167); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(168) = init(168); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(169) = init(169); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(170) = init(170); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(171) = init(171); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2)
Init(172) = init(172); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y)
Init(173) = init(173); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(174) = init(174); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(175) = init(175); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(176) = init(176); % Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY)
Init(177) = init(177); % Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY)
Init(178) = init(178); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(179) = init(179); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY)
Init(180) = init(180); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(181) = init(181); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY)
Init(182) = init(182); % Syk(tSH2, l~Y, a~pY)
Init(183) = init(183); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(184) = init(184); % Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY)
Init(185) = init(185); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY)
Init(186) = init(186); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(187) = init(187); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY)
Init(188) = init(188); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(189) = init(189); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY)
Init(190) = init(190); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(191) = init(191); % Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY)
Init(192) = init(192); % Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY)
Init(193) = init(193); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(194) = init(194); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY)
Init(195) = init(195); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(196) = init(196); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(197) = init(197); % Syk(tSH2, l~pY, a~pY)
Init(198) = init(198); % Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(199) = init(199); % Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(200) = init(200); % Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2)
Init(201) = init(201); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4)
Init(202) = init(202); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(203) = init(203); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY)
Init(204) = init(204); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2)
Init(205) = init(205); % Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4)
Init(206) = init(206); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2)
Init(207) = init(207); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(208) = init(208); % Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(209) = init(209); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY)
Init(210) = init(210); % Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2)
Init(211) = init(211); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY)
Init(212) = init(212); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY)
Init(213) = init(213); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY)
Init(214) = init(214); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY)
Init(215) = init(215); % Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2)
Init(216) = init(216); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY)
Init(217) = init(217); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY)
Init(218) = init(218); % Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY)
Init(219) = init(219); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY)
Init(220) = init(220); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY)
Init(221) = init(221); % Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4)
Init(222) = init(222); % Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY)
Init(223) = init(223); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY)
Init(224) = init(224); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY)
Init(225) = init(225); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2)
Init(226) = init(226); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2)
Init(227) = init(227); % Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY)
Init(228) = init(228); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY)
Init(229) = init(229); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY)
Init(230) = init(230); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY)
Init(231) = init(231); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY)
Init(232) = init(232); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY)
Init(233) = init(233); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2)
Init(234) = init(234); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y)
Init(235) = init(235); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2)
Init(236) = init(236); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4)
Init(237) = init(237); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y)
Init(238) = init(238); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2)
Init(239) = init(239); % Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y)
Init(240) = init(240); % Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y)
Init(241) = init(241); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5)
Init(242) = init(242); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y)
Init(243) = init(243); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y)
Init(244) = init(244); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5)
Init(245) = init(245); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y)
Init(246) = init(246); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(247) = init(247); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(248) = init(248); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(249) = init(249); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y)
Init(250) = init(250); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y)
Init(251) = init(251); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5)
Init(252) = init(252); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y)
Init(253) = init(253); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(254) = init(254); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(255) = init(255); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(256) = init(256); % Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY)
Init(257) = init(257); % Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY)
Init(258) = init(258); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5)
Init(259) = init(259); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY)
Init(260) = init(260); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY)
Init(261) = init(261); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(262) = init(262); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(263) = init(263); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(264) = init(264); % Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY)
Init(265) = init(265); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5)
Init(266) = init(266); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY)
Init(267) = init(267); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY)
Init(268) = init(268); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(269) = init(269); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(270) = init(270); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(271) = init(271); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY)
Init(272) = init(272); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY)
Init(273) = init(273); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(274) = init(274); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(275) = init(275); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y)
Init(276) = init(276); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y)
Init(277) = init(277); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(278) = init(278); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(279) = init(279); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2)
Init(280) = init(280); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y)
Init(281) = init(281); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(282) = init(282); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY)
Init(283) = init(283); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(284) = init(284); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2)
Init(285) = init(285); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y)
Init(286) = init(286); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(287) = init(287); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y)
Init(288) = init(288); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(289) = init(289); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(290) = init(290); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY)
Init(291) = init(291); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY)
Init(292) = init(292); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(293) = init(293); % Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(294) = init(294); % Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(295) = init(295); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY)
Init(296) = init(296); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(297) = init(297); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY)
Init(298) = init(298); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2)
Init(299) = init(299); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY)
Init(300) = init(300); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY)
Init(301) = init(301); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4)
Init(302) = init(302); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY)
Init(303) = init(303); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY)
Init(304) = init(304); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY)
Init(305) = init(305); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y)
Init(306) = init(306); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y)
Init(307) = init(307); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5)
Init(308) = init(308); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y)
Init(309) = init(309); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3)
Init(310) = init(310); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3)
Init(311) = init(311); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y)
Init(312) = init(312); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY)
Init(313) = init(313); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3)
Init(314) = init(314); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3)
Init(315) = init(315); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2)
Init(316) = init(316); % Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y)
Init(317) = init(317); % Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y)
Init(318) = init(318); % Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY)
Init(319) = init(319); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY)
Init(320) = init(320); % Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4)
Init(321) = init(321); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY)
Init(322) = init(322); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY)
Init(323) = init(323); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY)
Init(324) = init(324); % Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4)
Init(325) = init(325); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY)
Init(326) = init(326); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY)
Init(327) = init(327); % Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2)
Init(328) = init(328); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY)
Init(329) = init(329); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y)
Init(330) = init(330); % Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY)
Init(331) = init(331); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y)
Init(332) = init(332); % Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(333) = init(333); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y)
Init(334) = init(334); % Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(335) = init(335); % Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY)
Init(336) = init(336); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY)
Init(337) = init(337); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY)
Init(338) = init(338); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY)
Init(339) = init(339); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5)
Init(340) = init(340); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(341) = init(341); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(342) = init(342); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(343) = init(343); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY)
Init(344) = init(344); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY)
Init(345) = init(345); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5)
Init(346) = init(346); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(347) = init(347); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(348) = init(348); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y)
Init(349) = init(349); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5)
Init(350) = init(350); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y)
Init(351) = init(351); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y)
Init(352) = init(352); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4)
Init(353) = init(353); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y)
Init(354) = init(354); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y)
Init(355) = init(355); % Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(356) = init(356); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y)
Init(357) = init(357); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5)
Init(358) = init(358); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(359) = init(359); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY)
Init(360) = init(360); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5)
Init(361) = init(361); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY)
Init(362) = init(362); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(363) = init(363); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(364) = init(364); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(365) = init(365); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(366) = init(366); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY)
Init(367) = init(367); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(368) = init(368); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(369) = init(369); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY)
Init(370) = init(370); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(371) = init(371); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(372) = init(372); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2)
Init(373) = init(373); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y)
Init(374) = init(374); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(375) = init(375); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y)
Init(376) = init(376); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY)
Init(377) = init(377); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y)
Init(378) = init(378); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y)
Init(379) = init(379); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(380) = init(380); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y)
Init(381) = init(381); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2)
Init(382) = init(382); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(383) = init(383); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y)
Init(384) = init(384); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y)
Init(385) = init(385); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(386) = init(386); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(387) = init(387); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(388) = init(388); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y)
Init(389) = init(389); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(390) = init(390); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(391) = init(391); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY)
Init(392) = init(392); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(393) = init(393); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY)
Init(394) = init(394); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(395) = init(395); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(396) = init(396); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY)
Init(397) = init(397); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(398) = init(398); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(399) = init(399); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY)
Init(400) = init(400); % Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2)
Init(401) = init(401); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY)
Init(402) = init(402); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(403) = init(403); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(404) = init(404); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2)
Init(405) = init(405); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y)
Init(406) = init(406); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(407) = init(407); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY)
Init(408) = init(408); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2)
Init(409) = init(409); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(410) = init(410); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(411) = init(411); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(412) = init(412); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(413) = init(413); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(414) = init(414); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2)
Init(415) = init(415); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(416) = init(416); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY)
Init(417) = init(417); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2)
Init(418) = init(418); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY)
Init(419) = init(419); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY)
Init(420) = init(420); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY)
Init(421) = init(421); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY)
Init(422) = init(422); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5)
Init(423) = init(423); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(424) = init(424); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y)
Init(425) = init(425); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y)
Init(426) = init(426); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(427) = init(427); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y)
Init(428) = init(428); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5)
Init(429) = init(429); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(430) = init(430); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY)
Init(431) = init(431); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY)
Init(432) = init(432); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(433) = init(433); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(434) = init(434); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(435) = init(435); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(436) = init(436); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(437) = init(437); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y)
Init(438) = init(438); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY)
Init(439) = init(439); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(440) = init(440); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(441) = init(441); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y)
Init(442) = init(442); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y)
Init(443) = init(443); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y)
Init(444) = init(444); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(445) = init(445); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(446) = init(446); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(447) = init(447); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(448) = init(448); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(449) = init(449); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y)
Init(450) = init(450); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y)
Init(451) = init(451); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY)
Init(452) = init(452); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y)
Init(453) = init(453); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY)
Init(454) = init(454); % Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY)
Init(455) = init(455); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY)
Init(456) = init(456); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY)
Init(457) = init(457); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2)
Init(458) = init(458); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2)
Init(459) = init(459); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2)
Init(460) = init(460); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2)
Init(461) = init(461); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2)
Init(462) = init(462); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2)
Init(463) = init(463); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2)
Init(464) = init(464); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(465) = init(465); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(466) = init(466); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(467) = init(467); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2)
Init(468) = init(468); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y)
Init(469) = init(469); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5)
Init(470) = init(470); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y)
Init(471) = init(471); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(472) = init(472); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y)
Init(473) = init(473); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5)
Init(474) = init(474); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(475) = init(475); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(476) = init(476); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(477) = init(477); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(478) = init(478); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(479) = init(479); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(480) = init(480); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2)
Init(481) = init(481); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(482) = init(482); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y)
Init(483) = init(483); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(484) = init(484); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(485) = init(485); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(486) = init(486); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(487) = init(487); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY)
Init(488) = init(488); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(489) = init(489); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5)
Init(490) = init(490); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY)
Init(491) = init(491); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY)
Init(492) = init(492); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(493) = init(493); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(494) = init(494); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5)
Init(495) = init(495); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY)
Init(496) = init(496); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(497) = init(497); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY)
Init(498) = init(498); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY)
Init(499) = init(499); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y)
Init(500) = init(500); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(501) = init(501); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y)
Init(502) = init(502); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y)
Init(503) = init(503); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y)
Init(504) = init(504); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(505) = init(505); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y)
Init(506) = init(506); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y)
Init(507) = init(507); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(508) = init(508); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y)
Init(509) = init(509); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y)
Init(510) = init(510); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY)
Init(511) = init(511); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(512) = init(512); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY)
Init(513) = init(513); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY)
Init(514) = init(514); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(515) = init(515); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY)
Init(516) = init(516); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(517) = init(517); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(518) = init(518); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY)
Init(519) = init(519); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY)
Init(520) = init(520); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY)
Init(521) = init(521); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(522) = init(522); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(523) = init(523); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(524) = init(524); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y)
Init(525) = init(525); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(526) = init(526); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(527) = init(527); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y)
Init(528) = init(528); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(529) = init(529); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY)
Init(530) = init(530); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y)
Init(531) = init(531); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y)
Init(532) = init(532); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY)
Init(533) = init(533); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y)
Init(534) = init(534); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y)
Init(535) = init(535); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(536) = init(536); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(537) = init(537); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3)
Init(538) = init(538); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y)
Init(539) = init(539); % Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2)
Init(540) = init(540); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y)
Init(541) = init(541); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(542) = init(542); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(543) = init(543); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY)
Init(544) = init(544); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2)
Init(545) = init(545); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY)
Init(546) = init(546); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(547) = init(547); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY)
Init(548) = init(548); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(549) = init(549); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(550) = init(550); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY)
Init(551) = init(551); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(552) = init(552); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(553) = init(553); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(554) = init(554); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y)
Init(555) = init(555); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(556) = init(556); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y)
Init(557) = init(557); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(558) = init(558); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(559) = init(559); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y)
Init(560) = init(560); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y)
Init(561) = init(561); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5)
Init(562) = init(562); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y)
Init(563) = init(563); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y)
Init(564) = init(564); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY)
Init(565) = init(565); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(566) = init(566); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(567) = init(567); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5)
Init(568) = init(568); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y)
Init(569) = init(569); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(570) = init(570); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y)
Init(571) = init(571); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5)
Init(572) = init(572); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y)
Init(573) = init(573); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y)
Init(574) = init(574); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y)
Init(575) = init(575); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y)
Init(576) = init(576); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY)
Init(577) = init(577); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY)
Init(578) = init(578); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(579) = init(579); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY)
Init(580) = init(580); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY)
Init(581) = init(581); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY)
Init(582) = init(582); % Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y)
Init(583) = init(583); % Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY)
Init(584) = init(584); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY)
Init(585) = init(585); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(586) = init(586); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(587) = init(587); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(588) = init(588); % Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY)
Init(589) = init(589); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY)
Init(590) = init(590); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY)
Init(591) = init(591); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY)
Init(592) = init(592); % Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y)
Init(593) = init(593); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2)
Init(594) = init(594); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY)
Init(595) = init(595); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4)
Init(596) = init(596); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2)
Init(597) = init(597); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(598) = init(598); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4)
Init(599) = init(599); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(600) = init(600); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2)
Init(601) = init(601); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2)
Init(602) = init(602); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5)
Init(603) = init(603); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5)
Init(604) = init(604); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5)
Init(605) = init(605); % Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(606) = init(606); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY)
Init(607) = init(607); % Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY)
Init(608) = init(608); % Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y)
Init(609) = init(609); % Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY)
Init(610) = init(610); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY)
Init(611) = init(611); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY)
Init(612) = init(612); % Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y)
Init(613) = init(613); % Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(614) = init(614); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY)
Init(615) = init(615); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y)
Init(616) = init(616); % Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y)
Init(617) = init(617); % Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y)
Init(618) = init(618); % Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY)
Init(619) = init(619); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY)
Init(620) = init(620); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY)
Init(621) = init(621); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y)
Init(622) = init(622); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5)
Init(623) = init(623); % Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY)
Init(624) = init(624); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5)
Init(625) = init(625); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y)
Init(626) = init(626); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY)
Init(627) = init(627); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4)
Init(628) = init(628); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4)
Init(629) = init(629); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y)
Init(630) = init(630); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5)
Init(631) = init(631); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5)
Init(632) = init(632); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5)
Init(633) = init(633); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4)
Init(634) = init(634); % Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y)
Init(635) = init(635); % Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY)
Init(636) = init(636); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5)
Init(637) = init(637); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5)
Init(638) = init(638); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5)
Init(639) = init(639); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5)
Init(640) = init(640); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5)
Init(641) = init(641); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5)
Init(642) = init(642); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5)
Init(643) = init(643); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5)
Init(644) = init(644); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY)
Init(645) = init(645); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY)
Init(646) = init(646); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY)
Init(647) = init(647); % Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4)
Init(648) = init(648); % Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY)
Init(649) = init(649); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2)
Init(650) = init(650); % Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y)
Init(651) = init(651); % Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2)
Init(652) = init(652); % Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY)
Init(653) = init(653); % Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y)
Init(654) = init(654); % Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5)
Init(655) = init(655); % t
end


function dydt=ode_aux(t,y)

global nodevar
global max_stoc_coef
global var
global k
global kd
global kun
global kdun
global stoc


k(1)=var(5);
k(2)=var(6);
k(3)=var(5);
k(4)=var(6);
k(5)=var(7);
k(6)=var(8);
k(7)=var(7);
k(8)=var(8);
k(9)=var(9);
k(10)=var(10);
k(11)=var(17);
k(12)=var(17);
k(13)=var(19);
k(14)=var(19);
k(15)=var(11);
k(16)=var(12);
k(17)=var(18);
k(18)=var(18);
k(19)=var(20);
k(20)=var(20);
k(21)=var(13);
k(22)=var(14);
k(23)=var(21);
k(24)=var(21);
k(25)=var(22);
k(26)=var(22);
k(27)=var(23);
k(28)=var(23);
k(29)=var(24);
k(30)=var(24);
k(31)=var(25);
k(32)=var(25);
k(33)=var(25);
k(34)=var(25);
k(35)=var(26);
k(36)=var(26);

dydt=zeros(nodevar,1);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(5);
dydt(5)=dydt(5)-k(9)*y(3)*y(5);
dydt(8)=dydt(8)+k(9)*y(3)*y(5);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(5);
dydt(5)=dydt(5)-k(7)*y(1)*y(5);
dydt(12)=dydt(12)+k(7)*y(1)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(5)=dydt(5)-k(7)*y(5)*y(7);
dydt(7)=dydt(7)-k(7)*y(5)*y(7);
dydt(11)=dydt(11)+k(7)*y(5)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(5)=dydt(5)-k(7)*y(5)*y(335);
dydt(335)=dydt(335)-k(7)*y(5)*y(335);
dydt(413)=dydt(413)+k(7)*y(5)*y(335);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(5)=dydt(5)-k(7)*y(5)*y(257);
dydt(257)=dydt(257)-k(7)*y(5)*y(257);
dydt(587)=dydt(587)+k(7)*y(5)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(5)=dydt(5)-k(7)*y(5)*y(227);
dydt(227)=dydt(227)-k(7)*y(5)*y(227);
dydt(589)=dydt(589)+k(7)*y(5)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(5)=dydt(5)-k(7)*y(5)*y(222);
dydt(222)=dydt(222)-k(7)*y(5)*y(222);
dydt(591)=dydt(591)+k(7)*y(5)*y(222);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(5)=dydt(5)-k(7)*y(5)*y(218);
dydt(218)=dydt(218)-k(7)*y(5)*y(218);
dydt(304)=dydt(304)+k(7)*y(5)*y(218);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(5)=dydt(5)-k(7)*y(5)*y(192);
dydt(192)=dydt(192)-k(7)*y(5)*y(192);
dydt(202)=dydt(202)+k(7)*y(5)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(5)=dydt(5)-k(7)*y(5)*y(184);
dydt(184)=dydt(184)-k(7)*y(5)*y(184);
dydt(416)=dydt(416)+k(7)*y(5)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(5)=dydt(5)-k(7)*y(5)*y(177);
dydt(177)=dydt(177)-k(7)*y(5)*y(177);
dydt(418)=dydt(418)+k(7)*y(5)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(5)=dydt(5)-k(7)*y(5)*y(140);
dydt(140)=dydt(140)-k(7)*y(5)*y(140);
dydt(95)=dydt(95)+k(7)*y(5)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(5)=dydt(5)-k(7)*y(5)*y(131);
dydt(131)=dydt(131)-k(7)*y(5)*y(131);
dydt(102)=dydt(102)+k(7)*y(5)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(5)=dydt(5)-k(7)*y(5)*y(111);
dydt(111)=dydt(111)-k(7)*y(5)*y(111);
dydt(112)=dydt(112)+k(7)*y(5)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(5);
dydt(5)=dydt(5)-k(7)*y(77)*y(5);
dydt(590)=dydt(590)+k(7)*y(77)*y(5);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(5);
dydt(5)=dydt(5)-k(7)*y(72)*y(5);
dydt(333)=dydt(333)+k(7)*y(72)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(5)=dydt(5)-k(7)*y(5)*y(61);
dydt(61)=dydt(61)-k(7)*y(5)*y(61);
dydt(123)=dydt(123)+k(7)*y(5)*y(61);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(5)=dydt(5)-k(7)*y(5)*y(54);
dydt(54)=dydt(54)-k(7)*y(5)*y(54);
dydt(119)=dydt(119)+k(7)*y(5)*y(54);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(5)=dydt(5)-k(7)*y(5)*y(48);
dydt(48)=dydt(48)-k(7)*y(5)*y(48);
dydt(99)=dydt(99)+k(7)*y(5)*y(48);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(5)=dydt(5)-k(7)*y(5)*y(43);
dydt(43)=dydt(43)-k(7)*y(5)*y(43);
dydt(115)=dydt(115)+k(7)*y(5)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(5)=dydt(5)-k(7)*y(5)*y(37);
dydt(37)=dydt(37)-k(7)*y(5)*y(37);
dydt(592)=dydt(592)+k(7)*y(5)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(5)=dydt(5)-k(7)*y(5)*y(33);
dydt(33)=dydt(33)-k(7)*y(5)*y(33);
dydt(106)=dydt(106)+k(7)*y(5)*y(33);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(5)=dydt(5)-k(7)*y(5)*y(27);
dydt(27)=dydt(27)-k(7)*y(5)*y(27);
dydt(101)=dydt(101)+k(7)*y(5)*y(27);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(5)=dydt(5)-k(7)*y(5)*y(21);
dydt(21)=dydt(21)-k(7)*y(5)*y(21);
dydt(94)=dydt(94)+k(7)*y(5)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(5);
dydt(5)=dydt(5)-k(7)*y(15)*y(5);
dydt(97)=dydt(97)+k(7)*y(15)*y(5);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a, b~Y, g~Y) + Lig(l1, l2) 

dydt(5)=dydt(5)-k(2)*y(5);
dydt(4)=dydt(4)+k(2)*y(5);
dydt(1)=dydt(1)+k(2)*y(5);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(6);
dydt(6)=dydt(6)-k(9)*y(3)*y(6);
dydt(9)=dydt(9)+k(9)*y(3)*y(6);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(1)=dydt(1)-k(5)*y(1)*y(6);
dydt(6)=dydt(6)-k(5)*y(1)*y(6);
dydt(12)=dydt(12)+k(5)*y(1)*y(6);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(6)=dydt(6)-k(5)*y(6)*y(7);
dydt(7)=dydt(7)-k(5)*y(6)*y(7);
dydt(148)=dydt(148)+k(5)*y(6)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(6)=dydt(6)-k(5)*y(6)*y(335);
dydt(335)=dydt(335)-k(5)*y(6)*y(335);
dydt(199)=dydt(199)+k(5)*y(6)*y(335);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(6)=dydt(6)-k(5)*y(6)*y(257);
dydt(257)=dydt(257)-k(5)*y(6)*y(257);
dydt(230)=dydt(230)+k(5)*y(6)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(6)=dydt(6)-k(5)*y(6)*y(227);
dydt(227)=dydt(227)-k(5)*y(6)*y(227);
dydt(229)=dydt(229)+k(5)*y(6)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(6)=dydt(6)-k(5)*y(6)*y(222);
dydt(222)=dydt(222)-k(5)*y(6)*y(222);
dydt(224)=dydt(224)+k(5)*y(6)*y(222);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(6)=dydt(6)-k(5)*y(6)*y(218);
dydt(218)=dydt(218)-k(5)*y(6)*y(218);
dydt(220)=dydt(220)+k(5)*y(6)*y(218);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(6)=dydt(6)-k(5)*y(6)*y(192);
dydt(192)=dydt(192)-k(5)*y(6)*y(192);
dydt(211)=dydt(211)+k(5)*y(6)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(6)=dydt(6)-k(5)*y(6)*y(184);
dydt(184)=dydt(184)-k(5)*y(6)*y(184);
dydt(652)=dydt(652)+k(5)*y(6)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(6)=dydt(6)-k(5)*y(6)*y(177);
dydt(177)=dydt(177)-k(5)*y(6)*y(177);
dydt(216)=dydt(216)+k(5)*y(6)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(6)=dydt(6)-k(5)*y(6)*y(140);
dydt(140)=dydt(140)-k(5)*y(6)*y(140);
dydt(144)=dydt(144)+k(5)*y(6)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(6)=dydt(6)-k(5)*y(6)*y(131);
dydt(131)=dydt(131)-k(5)*y(6)*y(131);
dydt(138)=dydt(138)+k(5)*y(6)*y(131);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(6)=dydt(6)-k(5)*y(6)*y(111);
dydt(111)=dydt(111)-k(5)*y(6)*y(111);
dydt(621)=dydt(621)+k(5)*y(6)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(77)=dydt(77)-k(5)*y(77)*y(6);
dydt(6)=dydt(6)-k(5)*y(77)*y(6);
dydt(234)=dydt(234)+k(5)*y(77)*y(6);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(72)=dydt(72)-k(5)*y(72)*y(6);
dydt(6)=dydt(6)-k(5)*y(72)*y(6);
dydt(69)=dydt(69)+k(5)*y(72)*y(6);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(6)=dydt(6)-k(5)*y(6)*y(61);
dydt(61)=dydt(61)-k(5)*y(6)*y(61);
dydt(68)=dydt(68)+k(5)*y(6)*y(61);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(6)=dydt(6)-k(5)*y(6)*y(54);
dydt(54)=dydt(54)-k(5)*y(6)*y(54);
dydt(233)=dydt(233)+k(5)*y(6)*y(54);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(6)=dydt(6)-k(5)*y(6)*y(48);
dydt(48)=dydt(48)-k(5)*y(6)*y(48);
dydt(146)=dydt(146)+k(5)*y(6)*y(48);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(6)=dydt(6)-k(5)*y(6)*y(43);
dydt(43)=dydt(43)-k(5)*y(6)*y(43);
dydt(45)=dydt(45)+k(5)*y(6)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(6)=dydt(6)-k(5)*y(6)*y(37);
dydt(37)=dydt(37)-k(5)*y(6)*y(37);
dydt(39)=dydt(39)+k(5)*y(6)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(6)=dydt(6)-k(5)*y(6)*y(33);
dydt(33)=dydt(33)-k(5)*y(6)*y(33);
dydt(35)=dydt(35)+k(5)*y(6)*y(33);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(6)=dydt(6)-k(5)*y(6)*y(27);
dydt(27)=dydt(27)-k(5)*y(6)*y(27);
dydt(30)=dydt(30)+k(5)*y(6)*y(27);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(6)=dydt(6)-k(5)*y(6)*y(21);
dydt(21)=dydt(21)-k(5)*y(6)*y(21);
dydt(24)=dydt(24)+k(5)*y(6)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(15)=dydt(15)-k(5)*y(15)*y(6);
dydt(6)=dydt(6)-k(5)*y(15)*y(6);
dydt(18)=dydt(18)+k(5)*y(15)*y(6);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a, b~Y, g~Y) + Lig(l1, l2) 

dydt(6)=dydt(6)-k(4)*y(6);
dydt(4)=dydt(4)+k(4)*y(6);
dydt(1)=dydt(1)+k(4)*y(6);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Lyn(U, SH2) 

dydt(8)=dydt(8)-k(10)*y(8);
dydt(3)=dydt(3)+k(10)*y(8);
dydt(5)=dydt(5)+k(10)*y(8);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(1)=dydt(1)-k(7)*y(1)*y(8);
dydt(8)=dydt(8)-k(7)*y(1)*y(8);
dydt(148)=dydt(148)+k(7)*y(1)*y(8);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(7)*y(7)*y(8);
dydt(8)=dydt(8)-k(7)*y(7)*y(8);
dydt(10)=dydt(10)+k(7)*y(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(8)=dydt(8)-k(7)*y(8)*y(335);
dydt(335)=dydt(335)-k(7)*y(8)*y(335);
dydt(414)=dydt(414)+k(7)*y(8)*y(335);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(8)=dydt(8)-k(7)*y(8)*y(257);
dydt(257)=dydt(257)-k(7)*y(8)*y(257);
dydt(588)=dydt(588)+k(7)*y(8)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(8)=dydt(8)-k(7)*y(8)*y(227);
dydt(227)=dydt(227)-k(7)*y(8)*y(227);
dydt(201)=dydt(201)+k(7)*y(8)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(8)=dydt(8)-k(7)*y(8)*y(222);
dydt(222)=dydt(222)-k(7)*y(8)*y(222);
dydt(593)=dydt(593)+k(7)*y(8)*y(222);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(8)=dydt(8)-k(7)*y(8)*y(218);
dydt(218)=dydt(218)-k(7)*y(8)*y(218);
dydt(594)=dydt(594)+k(7)*y(8)*y(218);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(8)=dydt(8)-k(7)*y(8)*y(192);
dydt(192)=dydt(192)-k(7)*y(8)*y(192);
dydt(607)=dydt(607)+k(7)*y(8)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(8)=dydt(8)-k(7)*y(8)*y(184);
dydt(184)=dydt(184)-k(7)*y(8)*y(184);
dydt(417)=dydt(417)+k(7)*y(8)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(8)=dydt(8)-k(7)*y(8)*y(177);
dydt(177)=dydt(177)-k(7)*y(8)*y(177);
dydt(606)=dydt(606)+k(7)*y(8)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(8)=dydt(8)-k(7)*y(8)*y(140);
dydt(140)=dydt(140)-k(7)*y(8)*y(140);
dydt(462)=dydt(462)+k(7)*y(8)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(8)=dydt(8)-k(7)*y(8)*y(131);
dydt(131)=dydt(131)-k(7)*y(8)*y(131);
dydt(104)=dydt(104)+k(7)*y(8)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(8)=dydt(8)-k(7)*y(8)*y(111);
dydt(111)=dydt(111)-k(7)*y(8)*y(111);
dydt(109)=dydt(109)+k(7)*y(8)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(77)=dydt(77)-k(7)*y(77)*y(8);
dydt(8)=dydt(8)-k(7)*y(77)*y(8);
dydt(108)=dydt(108)+k(7)*y(77)*y(8);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(72)=dydt(72)-k(7)*y(72)*y(8);
dydt(8)=dydt(8)-k(7)*y(72)*y(8);
dydt(124)=dydt(124)+k(7)*y(72)*y(8);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(7)*y(61)*y(8);
dydt(8)=dydt(8)-k(7)*y(61)*y(8);
dydt(121)=dydt(121)+k(7)*y(61)*y(8);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(54)=dydt(54)-k(7)*y(54)*y(8);
dydt(8)=dydt(8)-k(7)*y(54)*y(8);
dydt(117)=dydt(117)+k(7)*y(54)*y(8);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(8);
dydt(8)=dydt(8)-k(7)*y(48)*y(8);
dydt(463)=dydt(463)+k(7)*y(48)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(8)=dydt(8)-k(7)*y(8)*y(43);
dydt(43)=dydt(43)-k(7)*y(8)*y(43);
dydt(113)=dydt(113)+k(7)*y(8)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(8)=dydt(8)-k(7)*y(8)*y(37);
dydt(37)=dydt(37)-k(7)*y(8)*y(37);
dydt(107)=dydt(107)+k(7)*y(8)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(8)=dydt(8)-k(7)*y(8)*y(33);
dydt(33)=dydt(33)-k(7)*y(8)*y(33);
dydt(105)=dydt(105)+k(7)*y(8)*y(33);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(8)=dydt(8)-k(7)*y(8)*y(27);
dydt(27)=dydt(27)-k(7)*y(8)*y(27);
dydt(103)=dydt(103)+k(7)*y(8)*y(27);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(8)=dydt(8)-k(7)*y(8)*y(21);
dydt(21)=dydt(21)-k(7)*y(8)*y(21);
dydt(96)=dydt(96)+k(7)*y(8)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(15)=dydt(15)-k(7)*y(15)*y(8);
dydt(8)=dydt(8)-k(7)*y(15)*y(8);
dydt(100)=dydt(100)+k(7)*y(15)*y(8);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Lig(l1, l2) 

dydt(8)=dydt(8)-k(2)*y(8);
dydt(4)=dydt(4)+k(2)*y(8);
dydt(7)=dydt(7)+k(2)*y(8);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(10)=dydt(10)-k(14)*y(10);
dydt(463)=dydt(463)+k(14)*y(10);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(10)=dydt(10)-k(13)*y(10);
dydt(17)=dydt(17)+k(13)*y(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) 

dydt(10)=dydt(10)-k(10)*y(10);
dydt(3)=dydt(3)+k(10)*y(10);
dydt(148)=dydt(148)+k(10)*y(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) 

dydt(10)=dydt(10)-k(10)*y(10);
dydt(3)=dydt(3)+k(10)*y(10);
dydt(11)=dydt(11)+k(10)*y(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(10)=dydt(10)-k(8)*y(10);
dydt(7)=dydt(7)+k(8)*y(10);
dydt(8)=dydt(8)+k(8)*y(10);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(10)=dydt(10)-k(6)*y(10);
dydt(7)=dydt(7)+k(6)*y(10);
dydt(9)=dydt(9)+k(6)*y(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(12);
dydt(12)=dydt(12)-k(9)*y(3)*y(12);
dydt(148)=dydt(148)+k(9)*y(3)*y(12);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(12);
dydt(12)=dydt(12)-k(9)*y(3)*y(12);
dydt(11)=dydt(11)+k(9)*y(3)*y(12);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) 

dydt(12)=dydt(12)-k(8)*y(12);
dydt(1)=dydt(1)+k(8)*y(12);
dydt(5)=dydt(5)+k(8)*y(12);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) 

dydt(12)=dydt(12)-k(6)*y(12);
dydt(1)=dydt(1)+k(6)*y(12);
dydt(6)=dydt(6)+k(6)*y(12);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(13)=dydt(13)-k(31)*y(13);
dydt(11)=dydt(11)+k(31)*y(13);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(13);
dydt(13)=dydt(13)-k(15)*y(3)*y(13);
dydt(67)=dydt(67)+k(15)*y(3)*y(13);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(13)=dydt(13)-k(13)*y(13);
dydt(19)=dydt(19)+k(13)*y(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

dydt(13)=dydt(13)-k(10)*y(13);
dydt(3)=dydt(3)+k(10)*y(13);
dydt(69)=dydt(69)+k(10)*y(13);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(13)=dydt(13)-k(8)*y(13);
dydt(91)=dydt(91)+k(8)*y(13);
dydt(7)=dydt(7)+k(8)*y(13);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) 

dydt(13)=dydt(13)-k(6)*y(13);
dydt(72)=dydt(72)+k(6)*y(13);
dydt(9)=dydt(9)+k(6)*y(13);

% rule    : R15
% reaction: Rec(a, b~Y, g~pY) -> Rec(a, b~Y, g~Y) 

dydt(15)=dydt(15)-k(32)*y(15);
dydt(1)=dydt(1)+k(32)*y(15);

% rule    : R9
% reaction: Rec(a, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(15);
dydt(15)=dydt(15)-k(21)*y(182)*y(15);
dydt(177)=dydt(177)+k(21)*y(182)*y(15);

% rule    : R9
% reaction: Rec(a, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(15);
dydt(15)=dydt(15)-k(21)*y(197)*y(15);
dydt(192)=dydt(192)+k(21)*y(197)*y(15);

% rule    : R9
% reaction: Rec(a, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(15);
dydt(15)=dydt(15)-k(21)*y(32)*y(15);
dydt(27)=dydt(27)+k(21)*y(32)*y(15);

% rule    : R9
% reaction: Rec(a, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(15);
dydt(15)=dydt(15)-k(21)*y(2)*y(15);
dydt(21)=dydt(21)+k(21)*y(2)*y(15);

% rule    : R3
% reaction: Rec(a, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(15);
dydt(15)=dydt(15)-k(9)*y(3)*y(15);
dydt(48)=dydt(48)+k(9)*y(3)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(16);
dydt(16)=dydt(16)-k(7)*y(15)*y(16);
dydt(150)=dydt(150)+k(7)*y(15)*y(16);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(15)=dydt(15)-k(7)*y(15)*y(22);
dydt(22)=dydt(22)-k(7)*y(15)*y(22);
dydt(452)=dydt(452)+k(7)*y(15)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(15)=dydt(15)-k(7)*y(15)*y(28);
dydt(28)=dydt(28)-k(7)*y(15)*y(28);
dydt(156)=dydt(156)+k(7)*y(15)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(15)=dydt(15)-k(7)*y(15)*y(34);
dydt(34)=dydt(34)-k(7)*y(15)*y(34);
dydt(617)=dydt(617)+k(7)*y(15)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(15)=dydt(15)-k(7)*y(15)*y(38);
dydt(38)=dydt(38)-k(7)*y(15)*y(38);
dydt(249)=dydt(249)+k(7)*y(15)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(15)=dydt(15)-k(7)*y(15)*y(44);
dydt(44)=dydt(44)-k(7)*y(15)*y(44);
dydt(49)=dydt(49)+k(7)*y(15)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(15)=dydt(15)-k(7)*y(15)*y(55);
dydt(55)=dydt(55)-k(7)*y(15)*y(55);
dydt(56)=dydt(56)+k(7)*y(15)*y(55);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(15)=dydt(15)-k(7)*y(15)*y(62);
dydt(62)=dydt(62)-k(7)*y(15)*y(62);
dydt(63)=dydt(63)+k(7)*y(15)*y(62);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(91);
dydt(91)=dydt(91)-k(7)*y(15)*y(91);
dydt(93)=dydt(93)+k(7)*y(15)*y(91);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) 

dydt(15)=dydt(15)-k(7)*y(15)*y(126);
dydt(126)=dydt(126)-k(7)*y(15)*y(126);
dydt(147)=dydt(147)+k(7)*y(15)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(15)=dydt(15)-k(7)*y(15)*y(132);
dydt(132)=dydt(132)-k(7)*y(15)*y(132);
dydt(154)=dydt(154)+k(7)*y(15)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(15)=dydt(15)-k(7)*y(15)*y(141);
dydt(141)=dydt(141)-k(7)*y(15)*y(141);
dydt(145)=dydt(145)+k(7)*y(15)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(185);
dydt(185)=dydt(185)-k(7)*y(15)*y(185);
dydt(189)=dydt(189)+k(7)*y(15)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(209);
dydt(209)=dydt(209)-k(7)*y(15)*y(209);
dydt(376)=dydt(376)+k(7)*y(15)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(214);
dydt(214)=dydt(214)-k(7)*y(15)*y(214);
dydt(451)=dydt(451)+k(7)*y(15)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(219);
dydt(219)=dydt(219)-k(7)*y(15)*y(219);
dydt(431)=dydt(431)+k(7)*y(15)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(223);
dydt(223)=dydt(223)-k(7)*y(15)*y(223);
dydt(282)=dydt(282)+k(7)*y(15)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(228);
dydt(228)=dydt(228)-k(7)*y(15)*y(228);
dydt(232)=dydt(232)+k(7)*y(15)*y(228);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(315);
dydt(315)=dydt(315)-k(7)*y(15)*y(315);
dydt(614)=dydt(614)+k(7)*y(15)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(319);
dydt(319)=dydt(319)-k(7)*y(15)*y(319);
dydt(618)=dydt(618)+k(7)*y(15)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(15)=dydt(15)-k(7)*y(15)*y(336);
dydt(336)=dydt(336)-k(7)*y(15)*y(336);
dydt(198)=dydt(198)+k(7)*y(15)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(15)=dydt(15)-k(7)*y(15)*y(437);
dydt(437)=dydt(437)-k(7)*y(15)*y(437);
dydt(616)=dydt(616)+k(7)*y(15)*y(437);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(15)=dydt(15)-k(5)*y(15)*y(47);
dydt(47)=dydt(47)-k(5)*y(15)*y(47);
dydt(404)=dydt(404)+k(5)*y(15)*y(47);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(15)=dydt(15)-k(5)*y(15)*y(71);
dydt(71)=dydt(71)-k(5)*y(15)*y(71);
dydt(331)=dydt(331)+k(5)*y(15)*y(71);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(15)=dydt(15)-k(5)*y(15)*y(76);
dydt(76)=dydt(76)-k(5)*y(15)*y(76);
dydt(610)=dydt(610)+k(5)*y(15)*y(76);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(15)=dydt(15)-k(5)*y(15)*y(82);
dydt(82)=dydt(82)-k(5)*y(15)*y(82);
dydt(351)=dydt(351)+k(5)*y(15)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(15)=dydt(15)-k(5)*y(15)*y(86);
dydt(86)=dydt(86)-k(5)*y(15)*y(86);
dydt(450)=dydt(450)+k(5)*y(15)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(15)=dydt(15)-k(5)*y(15)*y(89);
dydt(89)=dydt(89)-k(5)*y(15)*y(89);
dydt(375)=dydt(375)+k(5)*y(15)*y(89);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(15)=dydt(15)-k(5)*y(15)*y(98);
dydt(98)=dydt(98)-k(5)*y(15)*y(98);
dydt(150)=dydt(150)+k(5)*y(15)*y(98);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(15)=dydt(15)-k(5)*y(15)*y(110);
dydt(110)=dydt(110)-k(5)*y(15)*y(110);
dydt(354)=dydt(354)+k(5)*y(15)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(15)=dydt(15)-k(5)*y(15)*y(114);
dydt(114)=dydt(114)-k(5)*y(15)*y(114);
dydt(524)=dydt(524)+k(5)*y(15)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(15)=dydt(15)-k(5)*y(15)*y(118);
dydt(118)=dydt(118)-k(5)*y(15)*y(118);
dydt(537)=dydt(537)+k(5)*y(15)*y(118);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(15)=dydt(15)-k(5)*y(15)*y(122);
dydt(122)=dydt(122)-k(5)*y(15)*y(122);
dydt(127)=dydt(127)+k(5)*y(15)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(15)=dydt(15)-k(5)*y(15)*y(176);
dydt(176)=dydt(176)-k(5)*y(15)*y(176);
dydt(648)=dydt(648)+k(5)*y(15)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(15)=dydt(15)-k(5)*y(15)*y(191);
dydt(191)=dydt(191)-k(5)*y(15)*y(191);
dydt(409)=dydt(409)+k(5)*y(15)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(15)=dydt(15)-k(5)*y(15)*y(256);
dydt(256)=dydt(256)-k(5)*y(15)*y(256);
dydt(585)=dydt(585)+k(5)*y(15)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(15)=dydt(15)-k(5)*y(15)*y(264);
dydt(264)=dydt(264)-k(5)*y(15)*y(264);
dydt(512)=dydt(512)+k(5)*y(15)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(15)=dydt(15)-k(5)*y(15)*y(380);
dydt(380)=dydt(380)-k(5)*y(15)*y(380);
dydt(383)=dydt(383)+k(5)*y(15)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(15)=dydt(15)-k(5)*y(15)*y(391);
dydt(391)=dydt(391)-k(5)*y(15)*y(391);
dydt(393)=dydt(393)+k(5)*y(15)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(15)=dydt(15)-k(5)*y(15)*y(407);
dydt(407)=dydt(407)-k(5)*y(15)*y(407);
dydt(406)=dydt(406)+k(5)*y(15)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(15)=dydt(15)-k(5)*y(15)*y(441);
dydt(441)=dydt(441)-k(5)*y(15)*y(441);
dydt(405)=dydt(405)+k(5)*y(15)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(15)=dydt(15)-k(5)*y(15)*y(515);
dydt(515)=dydt(515)-k(5)*y(15)*y(515);
dydt(518)=dydt(518)+k(5)*y(15)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(15)=dydt(15)-k(5)*y(15)*y(543);
dydt(543)=dydt(543)-k(5)*y(15)*y(543);
dydt(547)=dydt(547)+k(5)*y(15)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(15)=dydt(15)-k(5)*y(15)*y(554);
dydt(554)=dydt(554)-k(5)*y(15)*y(554);
dydt(556)=dydt(556)+k(5)*y(15)*y(554);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(15)=dydt(15)-k(5)*y(15)*y(9);
dydt(9)=dydt(9)-k(5)*y(15)*y(9);
dydt(14)=dydt(14)+k(5)*y(15)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(4)=dydt(4)-k(3)*y(4)*y(15);
dydt(15)=dydt(15)-k(3)*y(4)*y(15);
dydt(98)=dydt(98)+k(3)*y(4)*y(15);

% rule    : R1
% reaction: Rec(a, b~Y, g~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(4)=dydt(4)-k(1)*y(4)*y(15);
dydt(15)=dydt(15)-k(1)*y(4)*y(15);
dydt(16)=dydt(16)+k(1)*y(4)*y(15);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(16)=dydt(16)-k(32)*y(16);
dydt(5)=dydt(5)+k(32)*y(16);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(16);
dydt(16)=dydt(16)-k(21)*y(182)*y(16);
dydt(214)=dydt(214)+k(21)*y(182)*y(16);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(16);
dydt(16)=dydt(16)-k(21)*y(197)*y(16);
dydt(209)=dydt(209)+k(21)*y(197)*y(16);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(16);
dydt(16)=dydt(16)-k(21)*y(32)*y(16);
dydt(28)=dydt(28)+k(21)*y(32)*y(16);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(16);
dydt(16)=dydt(16)-k(21)*y(2)*y(16);
dydt(22)=dydt(22)+k(21)*y(2)*y(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(16);
dydt(16)=dydt(16)-k(9)*y(3)*y(16);
dydt(126)=dydt(126)+k(9)*y(3)*y(16);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(16);
dydt(16)=dydt(16)-k(7)*y(1)*y(16);
dydt(18)=dydt(18)+k(7)*y(1)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(16)=dydt(16)-k(7)*y(16)*y(7);
dydt(7)=dydt(7)-k(7)*y(16)*y(7);
dydt(14)=dydt(14)+k(7)*y(16)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(16)=dydt(16)-k(7)*y(16)*y(335);
dydt(335)=dydt(335)-k(7)*y(16)*y(335);
dydt(406)=dydt(406)+k(7)*y(16)*y(335);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(16)=dydt(16)-k(7)*y(16)*y(257);
dydt(257)=dydt(257)-k(7)*y(16)*y(257);
dydt(585)=dydt(585)+k(7)*y(16)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(16)=dydt(16)-k(7)*y(16)*y(227);
dydt(227)=dydt(227)-k(7)*y(16)*y(227);
dydt(547)=dydt(547)+k(7)*y(16)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(16)=dydt(16)-k(7)*y(16)*y(222);
dydt(222)=dydt(222)-k(7)*y(16)*y(222);
dydt(518)=dydt(518)+k(7)*y(16)*y(222);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(16)=dydt(16)-k(7)*y(16)*y(218);
dydt(218)=dydt(218)-k(7)*y(16)*y(218);
dydt(512)=dydt(512)+k(7)*y(16)*y(218);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(16)=dydt(16)-k(7)*y(16)*y(192);
dydt(192)=dydt(192)-k(7)*y(16)*y(192);
dydt(409)=dydt(409)+k(7)*y(16)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(16)=dydt(16)-k(7)*y(16)*y(184);
dydt(184)=dydt(184)-k(7)*y(16)*y(184);
dydt(393)=dydt(393)+k(7)*y(16)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(16)=dydt(16)-k(7)*y(16)*y(177);
dydt(177)=dydt(177)-k(7)*y(16)*y(177);
dydt(648)=dydt(648)+k(7)*y(16)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(16)=dydt(16)-k(7)*y(16)*y(140);
dydt(140)=dydt(140)-k(7)*y(16)*y(140);
dydt(383)=dydt(383)+k(7)*y(16)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(16)=dydt(16)-k(7)*y(16)*y(131);
dydt(131)=dydt(131)-k(7)*y(16)*y(131);
dydt(405)=dydt(405)+k(7)*y(16)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(16)=dydt(16)-k(7)*y(16)*y(111);
dydt(111)=dydt(111)-k(7)*y(16)*y(111);
dydt(354)=dydt(354)+k(7)*y(16)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(16);
dydt(16)=dydt(16)-k(7)*y(77)*y(16);
dydt(610)=dydt(610)+k(7)*y(77)*y(16);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(16);
dydt(16)=dydt(16)-k(7)*y(72)*y(16);
dydt(331)=dydt(331)+k(7)*y(72)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(16)=dydt(16)-k(7)*y(16)*y(61);
dydt(61)=dydt(61)-k(7)*y(16)*y(61);
dydt(127)=dydt(127)+k(7)*y(16)*y(61);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(16)=dydt(16)-k(7)*y(16)*y(54);
dydt(54)=dydt(54)-k(7)*y(16)*y(54);
dydt(537)=dydt(537)+k(7)*y(16)*y(54);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(16)=dydt(16)-k(7)*y(16)*y(48);
dydt(48)=dydt(48)-k(7)*y(16)*y(48);
dydt(404)=dydt(404)+k(7)*y(16)*y(48);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(16)=dydt(16)-k(7)*y(16)*y(43);
dydt(43)=dydt(43)-k(7)*y(16)*y(43);
dydt(524)=dydt(524)+k(7)*y(16)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(16)=dydt(16)-k(7)*y(16)*y(37);
dydt(37)=dydt(37)-k(7)*y(16)*y(37);
dydt(556)=dydt(556)+k(7)*y(16)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(16)=dydt(16)-k(7)*y(16)*y(33);
dydt(33)=dydt(33)-k(7)*y(16)*y(33);
dydt(351)=dydt(351)+k(7)*y(16)*y(33);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(16)=dydt(16)-k(7)*y(16)*y(27);
dydt(27)=dydt(27)-k(7)*y(16)*y(27);
dydt(450)=dydt(450)+k(7)*y(16)*y(27);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(16)=dydt(16)-k(7)*y(16)*y(21);
dydt(21)=dydt(21)-k(7)*y(16)*y(21);
dydt(375)=dydt(375)+k(7)*y(16)*y(21);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a, b~Y, g~pY) + Lig(l1, l2) 

dydt(16)=dydt(16)-k(2)*y(16);
dydt(4)=dydt(4)+k(2)*y(16);
dydt(15)=dydt(15)+k(2)*y(16);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(17)=dydt(17)-k(32)*y(17);
dydt(10)=dydt(10)+k(32)*y(17);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(17);
dydt(17)=dydt(17)-k(21)*y(182)*y(17);
dydt(215)=dydt(215)+k(21)*y(182)*y(17);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(17);
dydt(17)=dydt(17)-k(21)*y(197)*y(17);
dydt(210)=dydt(210)+k(21)*y(197)*y(17);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(17);
dydt(17)=dydt(17)-k(21)*y(32)*y(17);
dydt(29)=dydt(29)+k(21)*y(32)*y(17);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(17);
dydt(17)=dydt(17)-k(21)*y(2)*y(17);
dydt(23)=dydt(23)+k(21)*y(2)*y(17);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(17)=dydt(17)-k(14)*y(17);
dydt(149)=dydt(149)+k(14)*y(17);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) 

dydt(17)=dydt(17)-k(10)*y(17);
dydt(3)=dydt(3)+k(10)*y(17);
dydt(14)=dydt(14)+k(10)*y(17);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) 

dydt(17)=dydt(17)-k(10)*y(17);
dydt(3)=dydt(3)+k(10)*y(17);
dydt(146)=dydt(146)+k(10)*y(17);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(17)=dydt(17)-k(8)*y(17);
dydt(7)=dydt(7)+k(8)*y(17);
dydt(126)=dydt(126)+k(8)*y(17);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(17)=dydt(17)-k(6)*y(17);
dydt(48)=dydt(48)+k(6)*y(17);
dydt(9)=dydt(9)+k(6)*y(17);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(18)=dydt(18)-k(32)*y(18);
dydt(12)=dydt(12)+k(32)*y(18);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(18);
dydt(18)=dydt(18)-k(21)*y(182)*y(18);
dydt(216)=dydt(216)+k(21)*y(182)*y(18);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(18);
dydt(18)=dydt(18)-k(21)*y(197)*y(18);
dydt(211)=dydt(211)+k(21)*y(197)*y(18);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(18);
dydt(18)=dydt(18)-k(21)*y(32)*y(18);
dydt(30)=dydt(30)+k(21)*y(32)*y(18);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(18);
dydt(18)=dydt(18)-k(21)*y(2)*y(18);
dydt(24)=dydt(24)+k(21)*y(2)*y(18);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(18);
dydt(18)=dydt(18)-k(9)*y(3)*y(18);
dydt(146)=dydt(146)+k(9)*y(3)*y(18);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(18);
dydt(18)=dydt(18)-k(9)*y(3)*y(18);
dydt(14)=dydt(14)+k(9)*y(3)*y(18);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) 

dydt(18)=dydt(18)-k(8)*y(18);
dydt(1)=dydt(1)+k(8)*y(18);
dydt(16)=dydt(16)+k(8)*y(18);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) 

dydt(18)=dydt(18)-k(6)*y(18);
dydt(15)=dydt(15)+k(6)*y(18);
dydt(6)=dydt(6)+k(6)*y(18);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(19)=dydt(19)-k(32)*y(19);
dydt(13)=dydt(13)+k(32)*y(19);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(19)=dydt(19)-k(31)*y(19);
dydt(14)=dydt(14)+k(31)*y(19);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(19);
dydt(19)=dydt(19)-k(21)*y(182)*y(19);
dydt(217)=dydt(217)+k(21)*y(182)*y(19);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(19);
dydt(19)=dydt(19)-k(21)*y(197)*y(19);
dydt(212)=dydt(212)+k(21)*y(197)*y(19);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(19);
dydt(19)=dydt(19)-k(21)*y(32)*y(19);
dydt(31)=dydt(31)+k(21)*y(32)*y(19);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(19);
dydt(19)=dydt(19)-k(21)*y(2)*y(19);
dydt(25)=dydt(25)+k(21)*y(2)*y(19);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(19);
dydt(19)=dydt(19)-k(15)*y(3)*y(19);
dydt(41)=dydt(41)+k(15)*y(3)*y(19);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

dydt(19)=dydt(19)-k(10)*y(19);
dydt(3)=dydt(3)+k(10)*y(19);
dydt(234)=dydt(234)+k(10)*y(19);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(19)=dydt(19)-k(8)*y(19);
dydt(315)=dydt(315)+k(8)*y(19);
dydt(7)=dydt(7)+k(8)*y(19);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) 

dydt(19)=dydt(19)-k(6)*y(19);
dydt(77)=dydt(77)+k(6)*y(19);
dydt(9)=dydt(9)+k(6)*y(19);

% rule    : R9_op
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(21)=dydt(21)-k(22)*y(21);
dydt(2)=dydt(2)+k(22)*y(21);
dydt(15)=dydt(15)+k(22)*y(21);

% rule    : R3
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(21);
dydt(21)=dydt(21)-k(9)*y(3)*y(21);
dydt(140)=dydt(140)+k(9)*y(3)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(22);
dydt(22)=dydt(22)-k(7)*y(21)*y(22);
dydt(172)=dydt(172)+k(7)*y(21)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(28);
dydt(28)=dydt(28)-k(7)*y(21)*y(28);
dydt(378)=dydt(378)+k(7)*y(21)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(34);
dydt(34)=dydt(34)-k(7)*y(21)*y(34);
dydt(482)=dydt(482)+k(7)*y(21)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(38);
dydt(38)=dydt(38)-k(7)*y(21)*y(38);
dydt(481)=dydt(481)+k(7)*y(21)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(44);
dydt(44)=dydt(44)-k(7)*y(21)*y(44);
dydt(274)=dydt(274)+k(7)*y(21)*y(44);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(55)=dydt(55)-k(7)*y(55)*y(21);
dydt(21)=dydt(21)-k(7)*y(55)*y(21);
dydt(276)=dydt(276)+k(7)*y(55)*y(21);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(62)=dydt(62)-k(7)*y(62)*y(21);
dydt(21)=dydt(21)-k(7)*y(62)*y(21);
dydt(88)=dydt(88)+k(7)*y(62)*y(21);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(91)=dydt(91)-k(7)*y(91)*y(21);
dydt(21)=dydt(21)-k(7)*y(91)*y(21);
dydt(90)=dydt(90)+k(7)*y(91)*y(21);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(126)=dydt(126)-k(7)*y(126)*y(21);
dydt(21)=dydt(21)-k(7)*y(126)*y(21);
dydt(152)=dydt(152)+k(7)*y(126)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(132);
dydt(132)=dydt(132)-k(7)*y(21)*y(132);
dydt(157)=dydt(157)+k(7)*y(21)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(141);
dydt(141)=dydt(141)-k(7)*y(21)*y(141);
dydt(170)=dydt(170)+k(7)*y(21)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(185);
dydt(185)=dydt(185)-k(7)*y(21)*y(185);
dydt(174)=dydt(174)+k(7)*y(21)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(209);
dydt(209)=dydt(209)-k(7)*y(21)*y(209);
dydt(373)=dydt(373)+k(7)*y(21)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(214);
dydt(214)=dydt(214)-k(7)*y(21)*y(214);
dydt(377)=dydt(377)+k(7)*y(21)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(219);
dydt(219)=dydt(219)-k(7)*y(21)*y(219);
dydt(280)=dydt(280)+k(7)*y(21)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(223);
dydt(223)=dydt(223)-k(7)*y(21)*y(223);
dydt(278)=dydt(278)+k(7)*y(21)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(228);
dydt(228)=dydt(228)-k(7)*y(21)*y(228);
dydt(484)=dydt(484)+k(7)*y(21)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(315)=dydt(315)-k(7)*y(315)*y(21);
dydt(21)=dydt(21)-k(7)*y(315)*y(21);
dydt(425)=dydt(425)+k(7)*y(315)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(319);
dydt(319)=dydt(319)-k(7)*y(21)*y(319);
dydt(427)=dydt(427)+k(7)*y(21)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(336);
dydt(336)=dydt(336)-k(7)*y(21)*y(336);
dydt(371)=dydt(371)+k(7)*y(21)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(21)=dydt(21)-k(7)*y(21)*y(437);
dydt(437)=dydt(437)-k(7)*y(21)*y(437);
dydt(275)=dydt(275)+k(7)*y(21)*y(437);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(47)=dydt(47)-k(5)*y(47)*y(21);
dydt(21)=dydt(21)-k(5)*y(47)*y(21);
dydt(388)=dydt(388)+k(5)*y(47)*y(21);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(71)=dydt(71)-k(5)*y(71)*y(21);
dydt(21)=dydt(21)-k(5)*y(71)*y(21);
dydt(142)=dydt(142)+k(5)*y(71)*y(21);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(76)=dydt(76)-k(5)*y(76)*y(21);
dydt(21)=dydt(21)-k(5)*y(76)*y(21);
dydt(508)=dydt(508)+k(5)*y(76)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(21)=dydt(21)-k(5)*y(21)*y(82);
dydt(82)=dydt(82)-k(5)*y(21)*y(82);
dydt(575)=dydt(575)+k(5)*y(21)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(21)=dydt(21)-k(5)*y(21)*y(86);
dydt(86)=dydt(86)-k(5)*y(21)*y(86);
dydt(166)=dydt(166)+k(5)*y(21)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(21)=dydt(21)-k(5)*y(21)*y(89);
dydt(89)=dydt(89)-k(5)*y(21)*y(89);
dydt(172)=dydt(172)+k(5)*y(21)*y(89);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(98)=dydt(98)-k(5)*y(98)*y(21);
dydt(21)=dydt(21)-k(5)*y(98)*y(21);
dydt(452)=dydt(452)+k(5)*y(98)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(21)=dydt(21)-k(5)*y(21)*y(110);
dydt(110)=dydt(110)-k(5)*y(21)*y(110);
dydt(506)=dydt(506)+k(5)*y(21)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(5)*y(21)*y(114);
dydt(114)=dydt(114)-k(5)*y(21)*y(114);
dydt(507)=dydt(507)+k(5)*y(21)*y(114);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(118)=dydt(118)-k(5)*y(118)*y(21);
dydt(21)=dydt(21)-k(5)*y(118)*y(21);
dydt(538)=dydt(538)+k(5)*y(118)*y(21);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(122)=dydt(122)-k(5)*y(122)*y(21);
dydt(21)=dydt(21)-k(5)*y(122)*y(21);
dydt(329)=dydt(329)+k(5)*y(122)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(21)=dydt(21)-k(5)*y(21)*y(176);
dydt(176)=dydt(176)-k(5)*y(21)*y(176);
dydt(179)=dydt(179)+k(5)*y(21)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(21)=dydt(21)-k(5)*y(21)*y(191);
dydt(191)=dydt(191)-k(5)*y(21)*y(191);
dydt(453)=dydt(453)+k(5)*y(21)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(21)=dydt(21)-k(5)*y(21)*y(256);
dydt(256)=dydt(256)-k(5)*y(21)*y(256);
dydt(545)=dydt(545)+k(5)*y(21)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(21)=dydt(21)-k(5)*y(21)*y(264);
dydt(264)=dydt(264)-k(5)*y(21)*y(264);
dydt(510)=dydt(510)+k(5)*y(21)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(5)*y(21)*y(380);
dydt(380)=dydt(380)-k(5)*y(21)*y(380);
dydt(386)=dydt(386)+k(5)*y(21)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(21)=dydt(21)-k(5)*y(21)*y(391);
dydt(391)=dydt(391)-k(5)*y(21)*y(391);
dydt(390)=dydt(390)+k(5)*y(21)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(21)=dydt(21)-k(5)*y(21)*y(407);
dydt(407)=dydt(407)-k(5)*y(21)*y(407);
dydt(411)=dydt(411)+k(5)*y(21)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(21)=dydt(21)-k(5)*y(21)*y(441);
dydt(441)=dydt(441)-k(5)*y(21)*y(441);
dydt(445)=dydt(445)+k(5)*y(21)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(21)=dydt(21)-k(5)*y(21)*y(515);
dydt(515)=dydt(515)-k(5)*y(21)*y(515);
dydt(511)=dydt(511)+k(5)*y(21)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(21)=dydt(21)-k(5)*y(21)*y(543);
dydt(543)=dydt(543)-k(5)*y(21)*y(543);
dydt(542)=dydt(542)+k(5)*y(21)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(21)=dydt(21)-k(5)*y(21)*y(554);
dydt(554)=dydt(554)-k(5)*y(21)*y(554);
dydt(541)=dydt(541)+k(5)*y(21)*y(554);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(9)=dydt(9)-k(5)*y(9)*y(21);
dydt(21)=dydt(21)-k(5)*y(9)*y(21);
dydt(20)=dydt(20)+k(5)*y(9)*y(21);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(4)=dydt(4)-k(3)*y(4)*y(21);
dydt(21)=dydt(21)-k(3)*y(4)*y(21);
dydt(89)=dydt(89)+k(3)*y(4)*y(21);

% rule    : R1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(4)=dydt(4)-k(1)*y(4)*y(21);
dydt(21)=dydt(21)-k(1)*y(4)*y(21);
dydt(22)=dydt(22)+k(1)*y(4)*y(21);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~Y) 

dydt(22)=dydt(22)-k(22)*y(22);
dydt(2)=dydt(2)+k(22)*y(22);
dydt(16)=dydt(16)+k(22)*y(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(22);
dydt(22)=dydt(22)-k(9)*y(3)*y(22);
dydt(141)=dydt(141)+k(9)*y(3)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(22);
dydt(22)=dydt(22)-k(7)*y(1)*y(22);
dydt(24)=dydt(24)+k(7)*y(1)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(7)=dydt(7)-k(7)*y(7)*y(22);
dydt(22)=dydt(22)-k(7)*y(7)*y(22);
dydt(20)=dydt(20)+k(7)*y(7)*y(22);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(22)=dydt(22)-k(7)*y(22)*y(335);
dydt(335)=dydt(335)-k(7)*y(22)*y(335);
dydt(411)=dydt(411)+k(7)*y(22)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(22);
dydt(22)=dydt(22)-k(7)*y(257)*y(22);
dydt(545)=dydt(545)+k(7)*y(257)*y(22);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(22)=dydt(22)-k(7)*y(22)*y(227);
dydt(227)=dydt(227)-k(7)*y(22)*y(227);
dydt(542)=dydt(542)+k(7)*y(22)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(22)=dydt(22)-k(7)*y(22)*y(222);
dydt(222)=dydt(222)-k(7)*y(22)*y(222);
dydt(511)=dydt(511)+k(7)*y(22)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(22);
dydt(22)=dydt(22)-k(7)*y(218)*y(22);
dydt(510)=dydt(510)+k(7)*y(218)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(22);
dydt(22)=dydt(22)-k(7)*y(192)*y(22);
dydt(453)=dydt(453)+k(7)*y(192)*y(22);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(22)=dydt(22)-k(7)*y(22)*y(184);
dydt(184)=dydt(184)-k(7)*y(22)*y(184);
dydt(390)=dydt(390)+k(7)*y(22)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(22);
dydt(22)=dydt(22)-k(7)*y(177)*y(22);
dydt(179)=dydt(179)+k(7)*y(177)*y(22);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(22)=dydt(22)-k(7)*y(22)*y(140);
dydt(140)=dydt(140)-k(7)*y(22)*y(140);
dydt(386)=dydt(386)+k(7)*y(22)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(22)=dydt(22)-k(7)*y(22)*y(131);
dydt(131)=dydt(131)-k(7)*y(22)*y(131);
dydt(445)=dydt(445)+k(7)*y(22)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(22);
dydt(22)=dydt(22)-k(7)*y(111)*y(22);
dydt(506)=dydt(506)+k(7)*y(111)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(77)=dydt(77)-k(7)*y(77)*y(22);
dydt(22)=dydt(22)-k(7)*y(77)*y(22);
dydt(508)=dydt(508)+k(7)*y(77)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(22);
dydt(22)=dydt(22)-k(7)*y(72)*y(22);
dydt(142)=dydt(142)+k(7)*y(72)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(61)=dydt(61)-k(7)*y(61)*y(22);
dydt(22)=dydt(22)-k(7)*y(61)*y(22);
dydt(329)=dydt(329)+k(7)*y(61)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(54)=dydt(54)-k(7)*y(54)*y(22);
dydt(22)=dydt(22)-k(7)*y(54)*y(22);
dydt(538)=dydt(538)+k(7)*y(54)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(48)=dydt(48)-k(7)*y(48)*y(22);
dydt(22)=dydt(22)-k(7)*y(48)*y(22);
dydt(388)=dydt(388)+k(7)*y(48)*y(22);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(22)=dydt(22)-k(7)*y(22)*y(43);
dydt(43)=dydt(43)-k(7)*y(22)*y(43);
dydt(507)=dydt(507)+k(7)*y(22)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(22)=dydt(22)-k(7)*y(22)*y(37);
dydt(37)=dydt(37)-k(7)*y(22)*y(37);
dydt(541)=dydt(541)+k(7)*y(22)*y(37);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(22);
dydt(22)=dydt(22)-k(7)*y(33)*y(22);
dydt(575)=dydt(575)+k(7)*y(33)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(22);
dydt(22)=dydt(22)-k(7)*y(27)*y(22);
dydt(166)=dydt(166)+k(7)*y(27)*y(22);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) 

dydt(22)=dydt(22)-k(2)*y(22);
dydt(4)=dydt(4)+k(2)*y(22);
dydt(21)=dydt(21)+k(2)*y(22);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(23)=dydt(23)-k(23)*y(23);
dydt(29)=dydt(29)+k(23)*y(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(23)=dydt(23)-k(22)*y(23);
dydt(2)=dydt(2)+k(22)*y(23);
dydt(17)=dydt(17)+k(22)*y(23);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(23)=dydt(23)-k(14)*y(23);
dydt(467)=dydt(467)+k(14)*y(23);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(23)=dydt(23)-k(10)*y(23);
dydt(3)=dydt(3)+k(10)*y(23);
dydt(20)=dydt(20)+k(10)*y(23);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(23)=dydt(23)-k(10)*y(23);
dydt(3)=dydt(3)+k(10)*y(23);
dydt(144)=dydt(144)+k(10)*y(23);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(23)=dydt(23)-k(8)*y(23);
dydt(7)=dydt(7)+k(8)*y(23);
dydt(141)=dydt(141)+k(8)*y(23);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(23)=dydt(23)-k(6)*y(23);
dydt(9)=dydt(9)+k(6)*y(23);
dydt(140)=dydt(140)+k(6)*y(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(24)=dydt(24)-k(22)*y(24);
dydt(2)=dydt(2)+k(22)*y(24);
dydt(18)=dydt(18)+k(22)*y(24);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(24);
dydt(24)=dydt(24)-k(9)*y(3)*y(24);
dydt(144)=dydt(144)+k(9)*y(3)*y(24);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(24);
dydt(24)=dydt(24)-k(9)*y(3)*y(24);
dydt(20)=dydt(20)+k(9)*y(3)*y(24);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

dydt(24)=dydt(24)-k(8)*y(24);
dydt(1)=dydt(1)+k(8)*y(24);
dydt(22)=dydt(22)+k(8)*y(24);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(24)=dydt(24)-k(6)*y(24);
dydt(6)=dydt(6)+k(6)*y(24);
dydt(21)=dydt(21)+k(6)*y(24);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(25)=dydt(25)-k(31)*y(25);
dydt(20)=dydt(20)+k(31)*y(25);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(25)=dydt(25)-k(23)*y(25);
dydt(31)=dydt(31)+k(23)*y(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(25)=dydt(25)-k(22)*y(25);
dydt(2)=dydt(2)+k(22)*y(25);
dydt(19)=dydt(19)+k(22)*y(25);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(25);
dydt(25)=dydt(25)-k(15)*y(3)*y(25);
dydt(42)=dydt(42)+k(15)*y(3)*y(25);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(25)=dydt(25)-k(10)*y(25);
dydt(3)=dydt(3)+k(10)*y(25);
dydt(621)=dydt(621)+k(10)*y(25);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(25)=dydt(25)-k(8)*y(25);
dydt(7)=dydt(7)+k(8)*y(25);
dydt(437)=dydt(437)+k(8)*y(25);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(25)=dydt(25)-k(6)*y(25);
dydt(9)=dydt(9)+k(6)*y(25);
dydt(111)=dydt(111)+k(6)*y(25);

% rule    : R16
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(27)=dydt(27)-k(33)*y(27);
dydt(21)=dydt(21)+k(33)*y(27);

% rule    : R9_op
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(27)=dydt(27)-k(22)*y(27);
dydt(32)=dydt(32)+k(22)*y(27);
dydt(15)=dydt(15)+k(22)*y(27);

% rule    : R3
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(27);
dydt(27)=dydt(27)-k(9)*y(3)*y(27);
dydt(131)=dydt(131)+k(9)*y(3)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(28);
dydt(28)=dydt(28)-k(7)*y(27)*y(28);
dydt(160)=dydt(160)+k(7)*y(27)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(34);
dydt(34)=dydt(34)-k(7)*y(27)*y(34);
dydt(468)=dydt(468)+k(7)*y(27)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(38);
dydt(38)=dydt(38)-k(7)*y(27)*y(38);
dydt(483)=dydt(483)+k(7)*y(27)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(44);
dydt(44)=dydt(44)-k(7)*y(27)*y(44);
dydt(277)=dydt(277)+k(7)*y(27)*y(44);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(55)=dydt(55)-k(7)*y(55)*y(27);
dydt(27)=dydt(27)-k(7)*y(55)*y(27);
dydt(287)=dydt(287)+k(7)*y(55)*y(27);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(62)=dydt(62)-k(7)*y(62)*y(27);
dydt(27)=dydt(27)-k(7)*y(62)*y(27);
dydt(85)=dydt(85)+k(7)*y(62)*y(27);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(91)=dydt(91)-k(7)*y(91)*y(27);
dydt(27)=dydt(27)-k(7)*y(91)*y(27);
dydt(87)=dydt(87)+k(7)*y(91)*y(27);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(126)=dydt(126)-k(7)*y(126)*y(27);
dydt(27)=dydt(27)-k(7)*y(126)*y(27);
dydt(153)=dydt(153)+k(7)*y(126)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(132);
dydt(132)=dydt(132)-k(7)*y(27)*y(132);
dydt(158)=dydt(158)+k(7)*y(27)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(141);
dydt(141)=dydt(141)-k(7)*y(27)*y(141);
dydt(164)=dydt(164)+k(7)*y(27)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(185);
dydt(185)=dydt(185)-k(7)*y(27)*y(185);
dydt(168)=dydt(168)+k(7)*y(27)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(27)=dydt(27)-k(7)*y(27)*y(209);
dydt(209)=dydt(209)-k(7)*y(27)*y(209);
dydt(454)=dydt(454)+k(7)*y(27)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(214);
dydt(214)=dydt(214)-k(7)*y(27)*y(214);
dydt(449)=dydt(449)+k(7)*y(27)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(219);
dydt(219)=dydt(219)-k(7)*y(27)*y(219);
dydt(285)=dydt(285)+k(7)*y(27)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(223);
dydt(223)=dydt(223)-k(7)*y(27)*y(223);
dydt(283)=dydt(283)+k(7)*y(27)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(228);
dydt(228)=dydt(228)-k(7)*y(27)*y(228);
dydt(474)=dydt(474)+k(7)*y(27)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(315)=dydt(315)-k(7)*y(315)*y(27);
dydt(27)=dydt(27)-k(7)*y(315)*y(27);
dydt(615)=dydt(615)+k(7)*y(315)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(319);
dydt(319)=dydt(319)-k(7)*y(27)*y(319);
dydt(472)=dydt(472)+k(7)*y(27)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(336);
dydt(336)=dydt(336)-k(7)*y(27)*y(336);
dydt(162)=dydt(162)+k(7)*y(27)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(27)=dydt(27)-k(7)*y(27)*y(437);
dydt(437)=dydt(437)-k(7)*y(27)*y(437);
dydt(442)=dydt(442)+k(7)*y(27)*y(437);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(47)=dydt(47)-k(5)*y(47)*y(27);
dydt(27)=dydt(27)-k(5)*y(47)*y(27);
dydt(384)=dydt(384)+k(5)*y(47)*y(27);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(71)=dydt(71)-k(5)*y(71)*y(27);
dydt(27)=dydt(27)-k(5)*y(71)*y(27);
dydt(136)=dydt(136)+k(5)*y(71)*y(27);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(76)=dydt(76)-k(5)*y(76)*y(27);
dydt(27)=dydt(27)-k(5)*y(76)*y(27);
dydt(533)=dydt(533)+k(5)*y(76)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(27)=dydt(27)-k(5)*y(27)*y(82);
dydt(82)=dydt(82)-k(5)*y(27)*y(82);
dydt(612)=dydt(612)+k(5)*y(27)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(27)=dydt(27)-k(5)*y(27)*y(86);
dydt(86)=dydt(86)-k(5)*y(27)*y(86);
dydt(160)=dydt(160)+k(5)*y(27)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(27)=dydt(27)-k(5)*y(27)*y(89);
dydt(89)=dydt(89)-k(5)*y(27)*y(89);
dydt(378)=dydt(378)+k(5)*y(27)*y(89);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(98)=dydt(98)-k(5)*y(98)*y(27);
dydt(27)=dydt(27)-k(5)*y(98)*y(27);
dydt(156)=dydt(156)+k(5)*y(98)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(27)=dydt(27)-k(5)*y(27)*y(110);
dydt(110)=dydt(110)-k(5)*y(27)*y(110);
dydt(356)=dydt(356)+k(5)*y(27)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(27)=dydt(27)-k(5)*y(27)*y(114);
dydt(114)=dydt(114)-k(5)*y(27)*y(114);
dydt(530)=dydt(530)+k(5)*y(27)*y(114);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(118)=dydt(118)-k(5)*y(118)*y(27);
dydt(27)=dydt(27)-k(5)*y(118)*y(27);
dydt(531)=dydt(531)+k(5)*y(118)*y(27);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(122)=dydt(122)-k(5)*y(122)*y(27);
dydt(27)=dydt(27)-k(5)*y(122)*y(27);
dydt(133)=dydt(133)+k(5)*y(122)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(27)=dydt(27)-k(5)*y(27)*y(176);
dydt(176)=dydt(176)-k(5)*y(27)*y(176);
dydt(366)=dydt(366)+k(5)*y(27)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(27)=dydt(27)-k(5)*y(27)*y(191);
dydt(191)=dydt(191)-k(5)*y(27)*y(191);
dydt(456)=dydt(456)+k(5)*y(27)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(27)=dydt(27)-k(5)*y(27)*y(256);
dydt(256)=dydt(256)-k(5)*y(27)*y(256);
dydt(608)=dydt(608)+k(5)*y(27)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(27)=dydt(27)-k(5)*y(27)*y(264);
dydt(264)=dydt(264)-k(5)*y(27)*y(264);
dydt(359)=dydt(359)+k(5)*y(27)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(27)=dydt(27)-k(5)*y(27)*y(380);
dydt(380)=dydt(380)-k(5)*y(27)*y(380);
dydt(379)=dydt(379)+k(5)*y(27)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(27)=dydt(27)-k(5)*y(27)*y(391);
dydt(391)=dydt(391)-k(5)*y(27)*y(391);
dydt(385)=dydt(385)+k(5)*y(27)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(5)*y(27)*y(407);
dydt(407)=dydt(407)-k(5)*y(27)*y(407);
dydt(412)=dydt(412)+k(5)*y(27)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(5)*y(27)*y(441);
dydt(441)=dydt(441)-k(5)*y(27)*y(441);
dydt(446)=dydt(446)+k(5)*y(27)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(27)=dydt(27)-k(5)*y(27)*y(515);
dydt(515)=dydt(515)-k(5)*y(27)*y(515);
dydt(532)=dydt(532)+k(5)*y(27)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(27)=dydt(27)-k(5)*y(27)*y(543);
dydt(543)=dydt(543)-k(5)*y(27)*y(543);
dydt(536)=dydt(536)+k(5)*y(27)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(27)=dydt(27)-k(5)*y(27)*y(554);
dydt(554)=dydt(554)-k(5)*y(27)*y(554);
dydt(535)=dydt(535)+k(5)*y(27)*y(554);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(9)=dydt(9)-k(5)*y(9)*y(27);
dydt(27)=dydt(27)-k(5)*y(9)*y(27);
dydt(26)=dydt(26)+k(5)*y(9)*y(27);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(4)=dydt(4)-k(3)*y(4)*y(27);
dydt(27)=dydt(27)-k(3)*y(4)*y(27);
dydt(86)=dydt(86)+k(3)*y(4)*y(27);

% rule    : R1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(4)=dydt(4)-k(1)*y(4)*y(27);
dydt(27)=dydt(27)-k(1)*y(4)*y(27);
dydt(28)=dydt(28)+k(1)*y(4)*y(27);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(28)=dydt(28)-k(33)*y(28);
dydt(22)=dydt(22)+k(33)*y(28);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~Y) 

dydt(28)=dydt(28)-k(22)*y(28);
dydt(32)=dydt(32)+k(22)*y(28);
dydt(16)=dydt(16)+k(22)*y(28);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(28);
dydt(28)=dydt(28)-k(9)*y(3)*y(28);
dydt(132)=dydt(132)+k(9)*y(3)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(28);
dydt(28)=dydt(28)-k(7)*y(1)*y(28);
dydt(30)=dydt(30)+k(7)*y(1)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(7)=dydt(7)-k(7)*y(7)*y(28);
dydt(28)=dydt(28)-k(7)*y(7)*y(28);
dydt(26)=dydt(26)+k(7)*y(7)*y(28);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(28)=dydt(28)-k(7)*y(28)*y(335);
dydt(335)=dydt(335)-k(7)*y(28)*y(335);
dydt(412)=dydt(412)+k(7)*y(28)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(257)=dydt(257)-k(7)*y(257)*y(28);
dydt(28)=dydt(28)-k(7)*y(257)*y(28);
dydt(608)=dydt(608)+k(7)*y(257)*y(28);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(28)=dydt(28)-k(7)*y(28)*y(227);
dydt(227)=dydt(227)-k(7)*y(28)*y(227);
dydt(536)=dydt(536)+k(7)*y(28)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(28)=dydt(28)-k(7)*y(28)*y(222);
dydt(222)=dydt(222)-k(7)*y(28)*y(222);
dydt(532)=dydt(532)+k(7)*y(28)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(28);
dydt(28)=dydt(28)-k(7)*y(218)*y(28);
dydt(359)=dydt(359)+k(7)*y(218)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(28);
dydt(28)=dydt(28)-k(7)*y(192)*y(28);
dydt(456)=dydt(456)+k(7)*y(192)*y(28);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(28)=dydt(28)-k(7)*y(28)*y(184);
dydt(184)=dydt(184)-k(7)*y(28)*y(184);
dydt(385)=dydt(385)+k(7)*y(28)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(28);
dydt(28)=dydt(28)-k(7)*y(177)*y(28);
dydt(366)=dydt(366)+k(7)*y(177)*y(28);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(28)=dydt(28)-k(7)*y(28)*y(140);
dydt(140)=dydt(140)-k(7)*y(28)*y(140);
dydt(379)=dydt(379)+k(7)*y(28)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(28)=dydt(28)-k(7)*y(28)*y(131);
dydt(131)=dydt(131)-k(7)*y(28)*y(131);
dydt(446)=dydt(446)+k(7)*y(28)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(28);
dydt(28)=dydt(28)-k(7)*y(111)*y(28);
dydt(356)=dydt(356)+k(7)*y(111)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(77)=dydt(77)-k(7)*y(77)*y(28);
dydt(28)=dydt(28)-k(7)*y(77)*y(28);
dydt(533)=dydt(533)+k(7)*y(77)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(28);
dydt(28)=dydt(28)-k(7)*y(72)*y(28);
dydt(136)=dydt(136)+k(7)*y(72)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(61)=dydt(61)-k(7)*y(61)*y(28);
dydt(28)=dydt(28)-k(7)*y(61)*y(28);
dydt(133)=dydt(133)+k(7)*y(61)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(54)=dydt(54)-k(7)*y(54)*y(28);
dydt(28)=dydt(28)-k(7)*y(54)*y(28);
dydt(531)=dydt(531)+k(7)*y(54)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(48)=dydt(48)-k(7)*y(48)*y(28);
dydt(28)=dydt(28)-k(7)*y(48)*y(28);
dydt(384)=dydt(384)+k(7)*y(48)*y(28);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(28)=dydt(28)-k(7)*y(28)*y(43);
dydt(43)=dydt(43)-k(7)*y(28)*y(43);
dydt(530)=dydt(530)+k(7)*y(28)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(28)=dydt(28)-k(7)*y(28)*y(37);
dydt(37)=dydt(37)-k(7)*y(28)*y(37);
dydt(535)=dydt(535)+k(7)*y(28)*y(37);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(28);
dydt(28)=dydt(28)-k(7)*y(33)*y(28);
dydt(612)=dydt(612)+k(7)*y(33)*y(28);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) 

dydt(28)=dydt(28)-k(2)*y(28);
dydt(4)=dydt(4)+k(2)*y(28);
dydt(27)=dydt(27)+k(2)*y(28);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(29)=dydt(29)-k(33)*y(29);
dydt(23)=dydt(23)+k(33)*y(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(29)=dydt(29)-k(22)*y(29);
dydt(32)=dydt(32)+k(22)*y(29);
dydt(17)=dydt(17)+k(22)*y(29);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(29)=dydt(29)-k(14)*y(29);
dydt(155)=dydt(155)+k(14)*y(29);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(29)=dydt(29)-k(10)*y(29);
dydt(3)=dydt(3)+k(10)*y(29);
dydt(26)=dydt(26)+k(10)*y(29);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(29)=dydt(29)-k(10)*y(29);
dydt(3)=dydt(3)+k(10)*y(29);
dydt(138)=dydt(138)+k(10)*y(29);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(29)=dydt(29)-k(8)*y(29);
dydt(7)=dydt(7)+k(8)*y(29);
dydt(132)=dydt(132)+k(8)*y(29);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(29)=dydt(29)-k(6)*y(29);
dydt(9)=dydt(9)+k(6)*y(29);
dydt(131)=dydt(131)+k(6)*y(29);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(30)=dydt(30)-k(33)*y(30);
dydt(24)=dydt(24)+k(33)*y(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(30)=dydt(30)-k(22)*y(30);
dydt(32)=dydt(32)+k(22)*y(30);
dydt(18)=dydt(18)+k(22)*y(30);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(30);
dydt(30)=dydt(30)-k(9)*y(3)*y(30);
dydt(138)=dydt(138)+k(9)*y(3)*y(30);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(30);
dydt(30)=dydt(30)-k(9)*y(3)*y(30);
dydt(26)=dydt(26)+k(9)*y(3)*y(30);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

dydt(30)=dydt(30)-k(8)*y(30);
dydt(1)=dydt(1)+k(8)*y(30);
dydt(28)=dydt(28)+k(8)*y(30);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(30)=dydt(30)-k(6)*y(30);
dydt(6)=dydt(6)+k(6)*y(30);
dydt(27)=dydt(27)+k(6)*y(30);

% rule    : R16
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(33)=dydt(33)-k(33)*y(33);
dydt(111)=dydt(111)+k(33)*y(33);

% rule    : R14
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(33)=dydt(33)-k(31)*y(33);
dydt(27)=dydt(27)+k(31)*y(33);

% rule    : R9_op
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(33)=dydt(33)-k(22)*y(33);
dydt(32)=dydt(32)+k(22)*y(33);
dydt(77)=dydt(77)+k(22)*y(33);

% rule    : R6
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(33);
dydt(33)=dydt(33)-k(15)*y(3)*y(33);
dydt(37)=dydt(37)+k(15)*y(3)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(34);
dydt(34)=dydt(34)-k(7)*y(33)*y(34);
dydt(568)=dydt(568)+k(7)*y(33)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(38);
dydt(38)=dydt(38)-k(7)*y(33)*y(38);
dydt(247)=dydt(247)+k(7)*y(33)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(44);
dydt(44)=dydt(44)-k(7)*y(33)*y(44);
dydt(254)=dydt(254)+k(7)*y(33)*y(44);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(55)=dydt(55)-k(7)*y(55)*y(33);
dydt(33)=dydt(33)-k(7)*y(55)*y(33);
dydt(634)=dydt(634)+k(7)*y(55)*y(33);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(62)=dydt(62)-k(7)*y(62)*y(33);
dydt(33)=dydt(33)-k(7)*y(62)*y(33);
dydt(81)=dydt(81)+k(7)*y(62)*y(33);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(91)=dydt(91)-k(7)*y(91)*y(33);
dydt(33)=dydt(33)-k(7)*y(91)*y(33);
dydt(84)=dydt(84)+k(7)*y(91)*y(33);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(126)=dydt(126)-k(7)*y(126)*y(33);
dydt(33)=dydt(33)-k(7)*y(126)*y(33);
dydt(350)=dydt(350)+k(7)*y(126)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(132);
dydt(132)=dydt(132)-k(7)*y(33)*y(132);
dydt(161)=dydt(161)+k(7)*y(33)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(141);
dydt(141)=dydt(141)-k(7)*y(33)*y(141);
dydt(167)=dydt(167)+k(7)*y(33)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(185);
dydt(185)=dydt(185)-k(7)*y(33)*y(185);
dydt(347)=dydt(347)+k(7)*y(33)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(209);
dydt(209)=dydt(209)-k(7)*y(33)*y(209);
dydt(559)=dydt(559)+k(7)*y(33)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(214);
dydt(214)=dydt(214)-k(7)*y(33)*y(214);
dydt(348)=dydt(348)+k(7)*y(33)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(219);
dydt(219)=dydt(219)-k(7)*y(33)*y(219);
dydt(574)=dydt(574)+k(7)*y(33)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(223);
dydt(223)=dydt(223)-k(7)*y(33)*y(223);
dydt(286)=dydt(286)+k(7)*y(33)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(33)=dydt(33)-k(7)*y(33)*y(228);
dydt(228)=dydt(228)-k(7)*y(33)*y(228);
dydt(635)=dydt(635)+k(7)*y(33)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(315)=dydt(315)-k(7)*y(315)*y(33);
dydt(33)=dydt(33)-k(7)*y(315)*y(33);
dydt(573)=dydt(573)+k(7)*y(315)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(319);
dydt(319)=dydt(319)-k(7)*y(33)*y(319);
dydt(562)=dydt(562)+k(7)*y(33)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(336);
dydt(336)=dydt(336)-k(7)*y(33)*y(336);
dydt(341)=dydt(341)+k(7)*y(33)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(7)*y(33)*y(437);
dydt(437)=dydt(437)-k(7)*y(33)*y(437);
dydt(572)=dydt(572)+k(7)*y(33)*y(437);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(47)=dydt(47)-k(5)*y(47)*y(33);
dydt(33)=dydt(33)-k(5)*y(47)*y(33);
dydt(470)=dydt(470)+k(5)*y(47)*y(33);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(71)=dydt(71)-k(5)*y(71)*y(33);
dydt(33)=dydt(33)-k(5)*y(71)*y(33);
dydt(242)=dydt(242)+k(5)*y(71)*y(33);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(76)=dydt(76)-k(5)*y(76)*y(33);
dydt(33)=dydt(33)-k(5)*y(76)*y(33);
dydt(245)=dydt(245)+k(5)*y(76)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(5)*y(33)*y(82);
dydt(82)=dydt(82)-k(5)*y(33)*y(82);
dydt(568)=dydt(568)+k(5)*y(33)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(5)*y(33)*y(86);
dydt(86)=dydt(86)-k(5)*y(33)*y(86);
dydt(468)=dydt(468)+k(5)*y(33)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(33)=dydt(33)-k(5)*y(33)*y(89);
dydt(89)=dydt(89)-k(5)*y(33)*y(89);
dydt(482)=dydt(482)+k(5)*y(33)*y(89);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(98)=dydt(98)-k(5)*y(98)*y(33);
dydt(33)=dydt(33)-k(5)*y(98)*y(33);
dydt(617)=dydt(617)+k(5)*y(98)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(33)=dydt(33)-k(5)*y(33)*y(110);
dydt(110)=dydt(110)-k(5)*y(33)*y(110);
dydt(499)=dydt(499)+k(5)*y(33)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(33)=dydt(33)-k(5)*y(33)*y(114);
dydt(114)=dydt(114)-k(5)*y(33)*y(114);
dydt(500)=dydt(500)+k(5)*y(33)*y(114);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(118)=dydt(118)-k(5)*y(118)*y(33);
dydt(33)=dydt(33)-k(5)*y(118)*y(33);
dydt(534)=dydt(534)+k(5)*y(118)*y(33);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(122)=dydt(122)-k(5)*y(122)*y(33);
dydt(33)=dydt(33)-k(5)*y(122)*y(33);
dydt(317)=dydt(317)+k(5)*y(122)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(33)=dydt(33)-k(5)*y(33)*y(176);
dydt(176)=dydt(176)-k(5)*y(33)*y(176);
dydt(498)=dydt(498)+k(5)*y(33)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(33)=dydt(33)-k(5)*y(33)*y(191);
dydt(191)=dydt(191)-k(5)*y(33)*y(191);
dydt(620)=dydt(620)+k(5)*y(33)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(33)=dydt(33)-k(5)*y(33)*y(256);
dydt(256)=dydt(256)-k(5)*y(33)*y(256);
dydt(582)=dydt(582)+k(5)*y(33)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(33)=dydt(33)-k(5)*y(33)*y(264);
dydt(264)=dydt(264)-k(5)*y(33)*y(264);
dydt(495)=dydt(495)+k(5)*y(33)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(33)=dydt(33)-k(5)*y(33)*y(380);
dydt(380)=dydt(380)-k(5)*y(33)*y(380);
dydt(382)=dydt(382)+k(5)*y(33)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(33)=dydt(33)-k(5)*y(33)*y(391);
dydt(391)=dydt(391)-k(5)*y(33)*y(391);
dydt(423)=dydt(423)+k(5)*y(33)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(33)=dydt(33)-k(5)*y(33)*y(407);
dydt(407)=dydt(407)-k(5)*y(33)*y(407);
dydt(439)=dydt(439)+k(5)*y(33)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(33)=dydt(33)-k(5)*y(33)*y(441);
dydt(441)=dydt(441)-k(5)*y(33)*y(441);
dydt(444)=dydt(444)+k(5)*y(33)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(33)=dydt(33)-k(5)*y(33)*y(515);
dydt(515)=dydt(515)-k(5)*y(33)*y(515);
dydt(496)=dydt(496)+k(5)*y(33)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(33)=dydt(33)-k(5)*y(33)*y(543);
dydt(543)=dydt(543)-k(5)*y(33)*y(543);
dydt(569)=dydt(569)+k(5)*y(33)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(33)=dydt(33)-k(5)*y(33)*y(554);
dydt(554)=dydt(554)-k(5)*y(33)*y(554);
dydt(566)=dydt(566)+k(5)*y(33)*y(554);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(9)=dydt(9)-k(5)*y(9)*y(33);
dydt(33)=dydt(33)-k(5)*y(9)*y(33);
dydt(31)=dydt(31)+k(5)*y(9)*y(33);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(4)=dydt(4)-k(3)*y(4)*y(33);
dydt(33)=dydt(33)-k(3)*y(4)*y(33);
dydt(82)=dydt(82)+k(3)*y(4)*y(33);

% rule    : R1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(4)=dydt(4)-k(1)*y(4)*y(33);
dydt(33)=dydt(33)-k(1)*y(4)*y(33);
dydt(34)=dydt(34)+k(1)*y(4)*y(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(34)=dydt(34)-k(33)*y(34);
dydt(437)=dydt(437)+k(33)*y(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(34)=dydt(34)-k(31)*y(34);
dydt(28)=dydt(28)+k(31)*y(34);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~Y) 

dydt(34)=dydt(34)-k(22)*y(34);
dydt(32)=dydt(32)+k(22)*y(34);
dydt(315)=dydt(315)+k(22)*y(34);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(34);
dydt(34)=dydt(34)-k(15)*y(3)*y(34);
dydt(38)=dydt(38)+k(15)*y(3)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(34);
dydt(34)=dydt(34)-k(7)*y(1)*y(34);
dydt(35)=dydt(35)+k(7)*y(1)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(7)=dydt(7)-k(7)*y(7)*y(34);
dydt(34)=dydt(34)-k(7)*y(7)*y(34);
dydt(31)=dydt(31)+k(7)*y(7)*y(34);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(34)=dydt(34)-k(7)*y(34)*y(335);
dydt(335)=dydt(335)-k(7)*y(34)*y(335);
dydt(439)=dydt(439)+k(7)*y(34)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(257)=dydt(257)-k(7)*y(257)*y(34);
dydt(34)=dydt(34)-k(7)*y(257)*y(34);
dydt(582)=dydt(582)+k(7)*y(257)*y(34);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(34)=dydt(34)-k(7)*y(34)*y(227);
dydt(227)=dydt(227)-k(7)*y(34)*y(227);
dydt(569)=dydt(569)+k(7)*y(34)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(34)=dydt(34)-k(7)*y(34)*y(222);
dydt(222)=dydt(222)-k(7)*y(34)*y(222);
dydt(496)=dydt(496)+k(7)*y(34)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(34);
dydt(34)=dydt(34)-k(7)*y(218)*y(34);
dydt(495)=dydt(495)+k(7)*y(218)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(34);
dydt(34)=dydt(34)-k(7)*y(192)*y(34);
dydt(620)=dydt(620)+k(7)*y(192)*y(34);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(34)=dydt(34)-k(7)*y(34)*y(184);
dydt(184)=dydt(184)-k(7)*y(34)*y(184);
dydt(423)=dydt(423)+k(7)*y(34)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(34);
dydt(34)=dydt(34)-k(7)*y(177)*y(34);
dydt(498)=dydt(498)+k(7)*y(177)*y(34);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(34)=dydt(34)-k(7)*y(34)*y(140);
dydt(140)=dydt(140)-k(7)*y(34)*y(140);
dydt(382)=dydt(382)+k(7)*y(34)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(34)=dydt(34)-k(7)*y(34)*y(131);
dydt(131)=dydt(131)-k(7)*y(34)*y(131);
dydt(444)=dydt(444)+k(7)*y(34)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(34);
dydt(34)=dydt(34)-k(7)*y(111)*y(34);
dydt(499)=dydt(499)+k(7)*y(111)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(77)=dydt(77)-k(7)*y(77)*y(34);
dydt(34)=dydt(34)-k(7)*y(77)*y(34);
dydt(245)=dydt(245)+k(7)*y(77)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(34);
dydt(34)=dydt(34)-k(7)*y(72)*y(34);
dydt(242)=dydt(242)+k(7)*y(72)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(61)=dydt(61)-k(7)*y(61)*y(34);
dydt(34)=dydt(34)-k(7)*y(61)*y(34);
dydt(317)=dydt(317)+k(7)*y(61)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(54)=dydt(54)-k(7)*y(54)*y(34);
dydt(34)=dydt(34)-k(7)*y(54)*y(34);
dydt(534)=dydt(534)+k(7)*y(54)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(48)=dydt(48)-k(7)*y(48)*y(34);
dydt(34)=dydt(34)-k(7)*y(48)*y(34);
dydt(470)=dydt(470)+k(7)*y(48)*y(34);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(34)=dydt(34)-k(7)*y(34)*y(43);
dydt(43)=dydt(43)-k(7)*y(34)*y(43);
dydt(500)=dydt(500)+k(7)*y(34)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(34)=dydt(34)-k(7)*y(34)*y(37);
dydt(37)=dydt(37)-k(7)*y(34)*y(37);
dydt(566)=dydt(566)+k(7)*y(34)*y(37);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) 

dydt(34)=dydt(34)-k(2)*y(34);
dydt(4)=dydt(4)+k(2)*y(34);
dydt(33)=dydt(33)+k(2)*y(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(35)=dydt(35)-k(33)*y(35);
dydt(621)=dydt(621)+k(33)*y(35);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(35)=dydt(35)-k(31)*y(35);
dydt(30)=dydt(30)+k(31)*y(35);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(35)=dydt(35)-k(22)*y(35);
dydt(32)=dydt(32)+k(22)*y(35);
dydt(234)=dydt(234)+k(22)*y(35);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(35);
dydt(35)=dydt(35)-k(15)*y(3)*y(35);
dydt(39)=dydt(39)+k(15)*y(3)*y(35);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(35);
dydt(35)=dydt(35)-k(9)*y(3)*y(35);
dydt(31)=dydt(31)+k(9)*y(3)*y(35);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

dydt(35)=dydt(35)-k(8)*y(35);
dydt(1)=dydt(1)+k(8)*y(35);
dydt(34)=dydt(34)+k(8)*y(35);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(35)=dydt(35)-k(6)*y(35);
dydt(6)=dydt(6)+k(6)*y(35);
dydt(33)=dydt(33)+k(6)*y(35);

% rule    : R16
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(37)=dydt(37)-k(33)*y(37);
dydt(43)=dydt(43)+k(33)*y(37);

% rule    : R9_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~pY, a~Y) 

dydt(37)=dydt(37)-k(22)*y(37);
dydt(32)=dydt(32)+k(22)*y(37);
dydt(54)=dydt(54)+k(22)*y(37);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lyn(U, SH2) 

dydt(37)=dydt(37)-k(16)*y(37);
dydt(3)=dydt(3)+k(16)*y(37);
dydt(33)=dydt(33)+k(16)*y(37);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(7)*y(37)*y(38);
dydt(38)=dydt(38)-k(7)*y(37)*y(38);
dydt(567)=dydt(567)+k(7)*y(37)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(7)*y(37)*y(44);
dydt(44)=dydt(44)-k(7)*y(37)*y(44);
dydt(571)=dydt(571)+k(7)*y(37)*y(44);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(55)=dydt(55)-k(7)*y(55)*y(37);
dydt(37)=dydt(37)-k(7)*y(55)*y(37);
dydt(633)=dydt(633)+k(7)*y(55)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(62)=dydt(62)-k(7)*y(62)*y(37);
dydt(37)=dydt(37)-k(7)*y(62)*y(37);
dydt(83)=dydt(83)+k(7)*y(62)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(91)=dydt(91)-k(7)*y(91)*y(37);
dydt(37)=dydt(37)-k(7)*y(91)*y(37);
dydt(311)=dydt(311)+k(7)*y(91)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(126)=dydt(126)-k(7)*y(126)*y(37);
dydt(37)=dydt(37)-k(7)*y(126)*y(37);
dydt(352)=dydt(352)+k(7)*y(126)*y(37);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(7)*y(37)*y(132);
dydt(132)=dydt(132)-k(7)*y(37)*y(132);
dydt(602)=dydt(602)+k(7)*y(37)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(7)*y(37)*y(141);
dydt(141)=dydt(141)-k(7)*y(37)*y(141);
dydt(603)=dydt(603)+k(7)*y(37)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(7)*y(37)*y(185);
dydt(185)=dydt(185)-k(7)*y(37)*y(185);
dydt(349)=dydt(349)+k(7)*y(37)*y(185);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(209)=dydt(209)-k(7)*y(209)*y(37);
dydt(37)=dydt(37)-k(7)*y(209)*y(37);
dydt(557)=dydt(557)+k(7)*y(209)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(214)=dydt(214)-k(7)*y(214)*y(37);
dydt(37)=dydt(37)-k(7)*y(214)*y(37);
dydt(553)=dydt(553)+k(7)*y(214)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(219)=dydt(219)-k(7)*y(219)*y(37);
dydt(37)=dydt(37)-k(7)*y(219)*y(37);
dydt(555)=dydt(555)+k(7)*y(219)*y(37);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(7)*y(37)*y(223);
dydt(223)=dydt(223)-k(7)*y(37)*y(223);
dydt(639)=dydt(639)+k(7)*y(37)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(7)*y(37)*y(228);
dydt(228)=dydt(228)-k(7)*y(37)*y(228);
dydt(561)=dydt(561)+k(7)*y(37)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(315)=dydt(315)-k(7)*y(315)*y(37);
dydt(37)=dydt(37)-k(7)*y(315)*y(37);
dydt(563)=dydt(563)+k(7)*y(315)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(319)=dydt(319)-k(7)*y(319)*y(37);
dydt(37)=dydt(37)-k(7)*y(319)*y(37);
dydt(560)=dydt(560)+k(7)*y(319)*y(37);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(37)=dydt(37)-k(7)*y(37)*y(336);
dydt(336)=dydt(336)-k(7)*y(37)*y(336);
dydt(558)=dydt(558)+k(7)*y(37)*y(336);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(437)=dydt(437)-k(7)*y(437)*y(37);
dydt(37)=dydt(37)-k(7)*y(437)*y(37);
dydt(570)=dydt(570)+k(7)*y(437)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(47)=dydt(47)-k(5)*y(47)*y(37);
dydt(37)=dydt(37)-k(5)*y(47)*y(37);
dydt(40)=dydt(40)+k(5)*y(47)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(71)=dydt(71)-k(5)*y(71)*y(37);
dydt(37)=dydt(37)-k(5)*y(71)*y(37);
dydt(240)=dydt(240)+k(5)*y(71)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(76)=dydt(76)-k(5)*y(76)*y(37);
dydt(37)=dydt(37)-k(5)*y(76)*y(37);
dydt(243)=dydt(243)+k(5)*y(76)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(82)=dydt(82)-k(5)*y(82)*y(37);
dydt(37)=dydt(37)-k(5)*y(82)*y(37);
dydt(247)=dydt(247)+k(5)*y(82)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(86)=dydt(86)-k(5)*y(86)*y(37);
dydt(37)=dydt(37)-k(5)*y(86)*y(37);
dydt(483)=dydt(483)+k(5)*y(86)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(89)=dydt(89)-k(5)*y(89)*y(37);
dydt(37)=dydt(37)-k(5)*y(89)*y(37);
dydt(481)=dydt(481)+k(5)*y(89)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(98)=dydt(98)-k(5)*y(98)*y(37);
dydt(37)=dydt(37)-k(5)*y(98)*y(37);
dydt(249)=dydt(249)+k(5)*y(98)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(110)=dydt(110)-k(5)*y(110)*y(37);
dydt(37)=dydt(37)-k(5)*y(110)*y(37);
dydt(246)=dydt(246)+k(5)*y(110)*y(37);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(5)*y(37)*y(114);
dydt(114)=dydt(114)-k(5)*y(37)*y(114);
dydt(631)=dydt(631)+k(5)*y(37)*y(114);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(118)=dydt(118)-k(5)*y(118)*y(37);
dydt(37)=dydt(37)-k(5)*y(118)*y(37);
dydt(244)=dydt(244)+k(5)*y(118)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(122)=dydt(122)-k(5)*y(122)*y(37);
dydt(37)=dydt(37)-k(5)*y(122)*y(37);
dydt(241)=dydt(241)+k(5)*y(122)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(176)=dydt(176)-k(5)*y(176)*y(37);
dydt(37)=dydt(37)-k(5)*y(176)*y(37);
dydt(479)=dydt(479)+k(5)*y(176)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(191)=dydt(191)-k(5)*y(191)*y(37);
dydt(37)=dydt(37)-k(5)*y(191)*y(37);
dydt(293)=dydt(293)+k(5)*y(191)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(256)=dydt(256)-k(5)*y(256)*y(37);
dydt(37)=dydt(37)-k(5)*y(256)*y(37);
dydt(248)=dydt(248)+k(5)*y(256)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(264)=dydt(264)-k(5)*y(264)*y(37);
dydt(37)=dydt(37)-k(5)*y(264)*y(37);
dydt(493)=dydt(493)+k(5)*y(264)*y(37);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(37)=dydt(37)-k(5)*y(37)*y(380);
dydt(380)=dydt(380)-k(5)*y(37)*y(380);
dydt(480)=dydt(480)+k(5)*y(37)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(37)=dydt(37)-k(5)*y(37)*y(391);
dydt(391)=dydt(391)-k(5)*y(37)*y(391);
dydt(478)=dydt(478)+k(5)*y(37)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(37)=dydt(37)-k(5)*y(37)*y(407);
dydt(407)=dydt(407)-k(5)*y(37)*y(407);
dydt(477)=dydt(477)+k(5)*y(37)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(5)*y(37)*y(441);
dydt(441)=dydt(441)-k(5)*y(37)*y(441);
dydt(469)=dydt(469)+k(5)*y(37)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(5)*y(37)*y(515);
dydt(515)=dydt(515)-k(5)*y(37)*y(515);
dydt(494)=dydt(494)+k(5)*y(37)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(5)*y(37)*y(543);
dydt(543)=dydt(543)-k(5)*y(37)*y(543);
dydt(643)=dydt(643)+k(5)*y(37)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(37)=dydt(37)-k(5)*y(37)*y(554);
dydt(554)=dydt(554)-k(5)*y(37)*y(554);
dydt(567)=dydt(567)+k(5)*y(37)*y(554);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(9)=dydt(9)-k(5)*y(9)*y(37);
dydt(37)=dydt(37)-k(5)*y(9)*y(37);
dydt(36)=dydt(36)+k(5)*y(9)*y(37);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(4)=dydt(4)-k(3)*y(4)*y(37);
dydt(37)=dydt(37)-k(3)*y(4)*y(37);
dydt(554)=dydt(554)+k(3)*y(4)*y(37);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(4)=dydt(4)-k(1)*y(4)*y(37);
dydt(37)=dydt(37)-k(1)*y(4)*y(37);
dydt(38)=dydt(38)+k(1)*y(4)*y(37);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(38)=dydt(38)-k(33)*y(38);
dydt(44)=dydt(44)+k(33)*y(38);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

dydt(38)=dydt(38)-k(22)*y(38);
dydt(32)=dydt(32)+k(22)*y(38);
dydt(55)=dydt(55)+k(22)*y(38);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(38)=dydt(38)-k(16)*y(38);
dydt(3)=dydt(3)+k(16)*y(38);
dydt(34)=dydt(34)+k(16)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(38);
dydt(38)=dydt(38)-k(7)*y(1)*y(38);
dydt(39)=dydt(39)+k(7)*y(1)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(7)=dydt(7)-k(7)*y(7)*y(38);
dydt(38)=dydt(38)-k(7)*y(7)*y(38);
dydt(36)=dydt(36)+k(7)*y(7)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(7)*y(335)*y(38);
dydt(38)=dydt(38)-k(7)*y(335)*y(38);
dydt(477)=dydt(477)+k(7)*y(335)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(38);
dydt(38)=dydt(38)-k(7)*y(257)*y(38);
dydt(248)=dydt(248)+k(7)*y(257)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(227)=dydt(227)-k(7)*y(227)*y(38);
dydt(38)=dydt(38)-k(7)*y(227)*y(38);
dydt(643)=dydt(643)+k(7)*y(227)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(222)=dydt(222)-k(7)*y(222)*y(38);
dydt(38)=dydt(38)-k(7)*y(222)*y(38);
dydt(494)=dydt(494)+k(7)*y(222)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(38);
dydt(38)=dydt(38)-k(7)*y(218)*y(38);
dydt(493)=dydt(493)+k(7)*y(218)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(192)=dydt(192)-k(7)*y(192)*y(38);
dydt(38)=dydt(38)-k(7)*y(192)*y(38);
dydt(293)=dydt(293)+k(7)*y(192)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(7)*y(184)*y(38);
dydt(38)=dydt(38)-k(7)*y(184)*y(38);
dydt(478)=dydt(478)+k(7)*y(184)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(38);
dydt(38)=dydt(38)-k(7)*y(177)*y(38);
dydt(479)=dydt(479)+k(7)*y(177)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(7)*y(140)*y(38);
dydt(38)=dydt(38)-k(7)*y(140)*y(38);
dydt(480)=dydt(480)+k(7)*y(140)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(131)=dydt(131)-k(7)*y(131)*y(38);
dydt(38)=dydt(38)-k(7)*y(131)*y(38);
dydt(469)=dydt(469)+k(7)*y(131)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(38);
dydt(38)=dydt(38)-k(7)*y(111)*y(38);
dydt(246)=dydt(246)+k(7)*y(111)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(77)=dydt(77)-k(7)*y(77)*y(38);
dydt(38)=dydt(38)-k(7)*y(77)*y(38);
dydt(243)=dydt(243)+k(7)*y(77)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(38);
dydt(38)=dydt(38)-k(7)*y(72)*y(38);
dydt(240)=dydt(240)+k(7)*y(72)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(61)=dydt(61)-k(7)*y(61)*y(38);
dydt(38)=dydt(38)-k(7)*y(61)*y(38);
dydt(241)=dydt(241)+k(7)*y(61)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(54)=dydt(54)-k(7)*y(54)*y(38);
dydt(38)=dydt(38)-k(7)*y(54)*y(38);
dydt(244)=dydt(244)+k(7)*y(54)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(38);
dydt(38)=dydt(38)-k(7)*y(48)*y(38);
dydt(40)=dydt(40)+k(7)*y(48)*y(38);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(43)=dydt(43)-k(7)*y(43)*y(38);
dydt(38)=dydt(38)-k(7)*y(43)*y(38);
dydt(631)=dydt(631)+k(7)*y(43)*y(38);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) 

dydt(38)=dydt(38)-k(2)*y(38);
dydt(4)=dydt(4)+k(2)*y(38);
dydt(37)=dydt(37)+k(2)*y(38);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(39)=dydt(39)-k(33)*y(39);
dydt(45)=dydt(45)+k(33)*y(39);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

dydt(39)=dydt(39)-k(22)*y(39);
dydt(32)=dydt(32)+k(22)*y(39);
dydt(233)=dydt(233)+k(22)*y(39);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(39)=dydt(39)-k(20)*y(39);
dydt(249)=dydt(249)+k(20)*y(39);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(39)=dydt(39)-k(18)*y(39);
dydt(240)=dydt(240)+k(18)*y(39);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(39)=dydt(39)-k(16)*y(39);
dydt(3)=dydt(3)+k(16)*y(39);
dydt(35)=dydt(35)+k(16)*y(39);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(39);
dydt(39)=dydt(39)-k(9)*y(3)*y(39);
dydt(36)=dydt(36)+k(9)*y(3)*y(39);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

dydt(39)=dydt(39)-k(8)*y(39);
dydt(1)=dydt(1)+k(8)*y(39);
dydt(38)=dydt(38)+k(8)*y(39);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(39)=dydt(39)-k(6)*y(39);
dydt(6)=dydt(6)+k(6)*y(39);
dydt(37)=dydt(37)+k(6)*y(39);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(40)=dydt(40)-k(33)*y(40);
dydt(46)=dydt(46)+k(33)*y(40);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(40)=dydt(40)-k(32)*y(40);
dydt(36)=dydt(36)+k(32)*y(40);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(40)=dydt(40)-k(22)*y(40);
dydt(32)=dydt(32)+k(22)*y(40);
dydt(53)=dydt(53)+k(22)*y(40);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(40);
dydt(40)=dydt(40)-k(21)*y(182)*y(40);
dydt(478)=dydt(478)+k(21)*y(182)*y(40);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(40);
dydt(40)=dydt(40)-k(21)*y(197)*y(40);
dydt(477)=dydt(477)+k(21)*y(197)*y(40);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(40);
dydt(40)=dydt(40)-k(21)*y(32)*y(40);
dydt(469)=dydt(469)+k(21)*y(32)*y(40);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(40);
dydt(40)=dydt(40)-k(21)*y(2)*y(40);
dydt(480)=dydt(480)+k(21)*y(2)*y(40);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(40)=dydt(40)-k(16)*y(40);
dydt(3)=dydt(3)+k(16)*y(40);
dydt(470)=dydt(470)+k(16)*y(40);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(40)=dydt(40)-k(10)*y(40);
dydt(3)=dydt(3)+k(10)*y(40);
dydt(249)=dydt(249)+k(10)*y(40);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(40)=dydt(40)-k(8)*y(40);
dydt(48)=dydt(48)+k(8)*y(40);
dydt(38)=dydt(38)+k(8)*y(40);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(40)=dydt(40)-k(6)*y(40);
dydt(47)=dydt(47)+k(6)*y(40);
dydt(37)=dydt(37)+k(6)*y(40);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(41)=dydt(41)-k(32)*y(41);
dydt(67)=dydt(67)+k(32)*y(41);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(182)=dydt(182)-k(21)*y(182)*y(41);
dydt(41)=dydt(41)-k(21)*y(182)*y(41);
dydt(221)=dydt(221)+k(21)*y(182)*y(41);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(41);
dydt(41)=dydt(41)-k(21)*y(197)*y(41);
dydt(226)=dydt(226)+k(21)*y(197)*y(41);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(32)=dydt(32)-k(21)*y(32)*y(41);
dydt(41)=dydt(41)-k(21)*y(32)*y(41);
dydt(36)=dydt(36)+k(21)*y(32)*y(41);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(41);
dydt(41)=dydt(41)-k(21)*y(2)*y(41);
dydt(42)=dydt(42)+k(21)*y(2)*y(41);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(41)=dydt(41)-k(20)*y(41);
dydt(53)=dydt(53)+k(20)*y(41);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) 

dydt(41)=dydt(41)-k(16)*y(41);
dydt(3)=dydt(3)+k(16)*y(41);
dydt(19)=dydt(19)+k(16)*y(41);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) 

dydt(41)=dydt(41)-k(10)*y(41);
dydt(3)=dydt(3)+k(10)*y(41);
dydt(233)=dydt(233)+k(10)*y(41);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(41)=dydt(41)-k(8)*y(41);
dydt(7)=dydt(7)+k(8)*y(41);
dydt(55)=dydt(55)+k(8)*y(41);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(41)=dydt(41)-k(6)*y(41);
dydt(54)=dydt(54)+k(6)*y(41);
dydt(9)=dydt(9)+k(6)*y(41);

% rule    : R9_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~Y, a~Y) 

dydt(43)=dydt(43)-k(22)*y(43);
dydt(2)=dydt(2)+k(22)*y(43);
dydt(54)=dydt(54)+k(22)*y(43);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lyn(U, SH2) 

dydt(43)=dydt(43)-k(16)*y(43);
dydt(3)=dydt(3)+k(16)*y(43);
dydt(111)=dydt(111)+k(16)*y(43);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(43)=dydt(43)-k(7)*y(43)*y(44);
dydt(44)=dydt(44)-k(7)*y(43)*y(44);
dydt(630)=dydt(630)+k(7)*y(43)*y(44);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(55)=dydt(55)-k(7)*y(55)*y(43);
dydt(43)=dydt(43)-k(7)*y(55)*y(43);
dydt(628)=dydt(628)+k(7)*y(55)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(62)=dydt(62)-k(7)*y(62)*y(43);
dydt(43)=dydt(43)-k(7)*y(62)*y(43);
dydt(307)=dydt(307)+k(7)*y(62)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(91)=dydt(91)-k(7)*y(91)*y(43);
dydt(43)=dydt(43)-k(7)*y(91)*y(43);
dydt(306)=dydt(306)+k(7)*y(91)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(126)=dydt(126)-k(7)*y(126)*y(43);
dydt(43)=dydt(43)-k(7)*y(126)*y(43);
dydt(116)=dydt(116)+k(7)*y(126)*y(43);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(43)=dydt(43)-k(7)*y(43)*y(132);
dydt(132)=dydt(132)-k(7)*y(43)*y(132);
dydt(357)=dydt(357)+k(7)*y(43)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(43)=dydt(43)-k(7)*y(43)*y(141);
dydt(141)=dydt(141)-k(7)*y(43)*y(141);
dydt(604)=dydt(604)+k(7)*y(43)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(43)=dydt(43)-k(7)*y(43)*y(185);
dydt(185)=dydt(185)-k(7)*y(43)*y(185);
dydt(522)=dydt(522)+k(7)*y(43)*y(185);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(209)=dydt(209)-k(7)*y(209)*y(43);
dydt(43)=dydt(43)-k(7)*y(209)*y(43);
dydt(525)=dydt(525)+k(7)*y(209)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(214)=dydt(214)-k(7)*y(214)*y(43);
dydt(43)=dydt(43)-k(7)*y(214)*y(43);
dydt(521)=dydt(521)+k(7)*y(214)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(219)=dydt(219)-k(7)*y(219)*y(43);
dydt(43)=dydt(43)-k(7)*y(219)*y(43);
dydt(523)=dydt(523)+k(7)*y(219)*y(43);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(43)=dydt(43)-k(7)*y(43)*y(223);
dydt(223)=dydt(223)-k(7)*y(43)*y(223);
dydt(637)=dydt(637)+k(7)*y(43)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(43)=dydt(43)-k(7)*y(43)*y(228);
dydt(228)=dydt(228)-k(7)*y(43)*y(228);
dydt(632)=dydt(632)+k(7)*y(43)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(315)=dydt(315)-k(7)*y(315)*y(43);
dydt(43)=dydt(43)-k(7)*y(315)*y(43);
dydt(308)=dydt(308)+k(7)*y(315)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(319)=dydt(319)-k(7)*y(319)*y(43);
dydt(43)=dydt(43)-k(7)*y(319)*y(43);
dydt(528)=dydt(528)+k(7)*y(319)*y(43);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(43)=dydt(43)-k(7)*y(43)*y(336);
dydt(336)=dydt(336)-k(7)*y(43)*y(336);
dydt(526)=dydt(526)+k(7)*y(43)*y(336);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(437)=dydt(437)-k(7)*y(437)*y(43);
dydt(43)=dydt(43)-k(7)*y(437)*y(43);
dydt(504)=dydt(504)+k(7)*y(437)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(47)=dydt(47)-k(5)*y(47)*y(43);
dydt(43)=dydt(43)-k(5)*y(47)*y(43);
dydt(46)=dydt(46)+k(5)*y(47)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(71)=dydt(71)-k(5)*y(71)*y(43);
dydt(43)=dydt(43)-k(5)*y(71)*y(43);
dydt(239)=dydt(239)+k(5)*y(71)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(76)=dydt(76)-k(5)*y(76)*y(43);
dydt(43)=dydt(43)-k(5)*y(76)*y(43);
dydt(250)=dydt(250)+k(5)*y(76)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(82)=dydt(82)-k(5)*y(82)*y(43);
dydt(43)=dydt(43)-k(5)*y(82)*y(43);
dydt(254)=dydt(254)+k(5)*y(82)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(86)=dydt(86)-k(5)*y(86)*y(43);
dydt(43)=dydt(43)-k(5)*y(86)*y(43);
dydt(277)=dydt(277)+k(5)*y(86)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(89)=dydt(89)-k(5)*y(89)*y(43);
dydt(43)=dydt(43)-k(5)*y(89)*y(43);
dydt(274)=dydt(274)+k(5)*y(89)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(98)=dydt(98)-k(5)*y(98)*y(43);
dydt(43)=dydt(43)-k(5)*y(98)*y(43);
dydt(49)=dydt(49)+k(5)*y(98)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(110)=dydt(110)-k(5)*y(110)*y(43);
dydt(43)=dydt(43)-k(5)*y(110)*y(43);
dydt(253)=dydt(253)+k(5)*y(110)*y(43);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(43)=dydt(43)-k(5)*y(43)*y(114);
dydt(114)=dydt(114)-k(5)*y(43)*y(114);
dydt(630)=dydt(630)+k(5)*y(43)*y(114);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(118)=dydt(118)-k(5)*y(118)*y(43);
dydt(43)=dydt(43)-k(5)*y(118)*y(43);
dydt(251)=dydt(251)+k(5)*y(118)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(122)=dydt(122)-k(5)*y(122)*y(43);
dydt(43)=dydt(43)-k(5)*y(122)*y(43);
dydt(654)=dydt(654)+k(5)*y(122)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(176)=dydt(176)-k(5)*y(176)*y(43);
dydt(43)=dydt(43)-k(5)*y(176)*y(43);
dydt(269)=dydt(269)+k(5)*y(176)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(191)=dydt(191)-k(5)*y(191)*y(43);
dydt(43)=dydt(43)-k(5)*y(191)*y(43);
dydt(262)=dydt(262)+k(5)*y(191)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(256)=dydt(256)-k(5)*y(256)*y(43);
dydt(43)=dydt(43)-k(5)*y(256)*y(43);
dydt(255)=dydt(255)+k(5)*y(256)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(264)=dydt(264)-k(5)*y(264)*y(43);
dydt(43)=dydt(43)-k(5)*y(264)*y(43);
dydt(263)=dydt(263)+k(5)*y(264)*y(43);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(43)=dydt(43)-k(5)*y(43)*y(380);
dydt(380)=dydt(380)-k(5)*y(43)*y(380);
dydt(51)=dydt(51)+k(5)*y(43)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(43)=dydt(43)-k(5)*y(43)*y(391);
dydt(391)=dydt(391)-k(5)*y(43)*y(391);
dydt(270)=dydt(270)+k(5)*y(43)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(43)=dydt(43)-k(5)*y(43)*y(407);
dydt(407)=dydt(407)-k(5)*y(43)*y(407);
dydt(207)=dydt(207)+k(5)*y(43)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(43)=dydt(43)-k(5)*y(43)*y(441);
dydt(441)=dydt(441)-k(5)*y(43)*y(441);
dydt(52)=dydt(52)+k(5)*y(43)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(43)=dydt(43)-k(5)*y(43)*y(515);
dydt(515)=dydt(515)-k(5)*y(43)*y(515);
dydt(265)=dydt(265)+k(5)*y(43)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(43)=dydt(43)-k(5)*y(43)*y(543);
dydt(543)=dydt(543)-k(5)*y(43)*y(543);
dydt(258)=dydt(258)+k(5)*y(43)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(43)=dydt(43)-k(5)*y(43)*y(554);
dydt(554)=dydt(554)-k(5)*y(43)*y(554);
dydt(571)=dydt(571)+k(5)*y(43)*y(554);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(9)=dydt(9)-k(5)*y(9)*y(43);
dydt(43)=dydt(43)-k(5)*y(9)*y(43);
dydt(42)=dydt(42)+k(5)*y(9)*y(43);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(4)=dydt(4)-k(3)*y(4)*y(43);
dydt(43)=dydt(43)-k(3)*y(4)*y(43);
dydt(114)=dydt(114)+k(3)*y(4)*y(43);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(4)=dydt(4)-k(1)*y(4)*y(43);
dydt(43)=dydt(43)-k(1)*y(4)*y(43);
dydt(44)=dydt(44)+k(1)*y(4)*y(43);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

dydt(44)=dydt(44)-k(22)*y(44);
dydt(2)=dydt(2)+k(22)*y(44);
dydt(55)=dydt(55)+k(22)*y(44);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(44)=dydt(44)-k(16)*y(44);
dydt(3)=dydt(3)+k(16)*y(44);
dydt(437)=dydt(437)+k(16)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(44);
dydt(44)=dydt(44)-k(7)*y(1)*y(44);
dydt(45)=dydt(45)+k(7)*y(1)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(7)=dydt(7)-k(7)*y(7)*y(44);
dydt(44)=dydt(44)-k(7)*y(7)*y(44);
dydt(42)=dydt(42)+k(7)*y(7)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(7)*y(335)*y(44);
dydt(44)=dydt(44)-k(7)*y(335)*y(44);
dydt(207)=dydt(207)+k(7)*y(335)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(44);
dydt(44)=dydt(44)-k(7)*y(257)*y(44);
dydt(255)=dydt(255)+k(7)*y(257)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(227)=dydt(227)-k(7)*y(227)*y(44);
dydt(44)=dydt(44)-k(7)*y(227)*y(44);
dydt(258)=dydt(258)+k(7)*y(227)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(222)=dydt(222)-k(7)*y(222)*y(44);
dydt(44)=dydt(44)-k(7)*y(222)*y(44);
dydt(265)=dydt(265)+k(7)*y(222)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(44);
dydt(44)=dydt(44)-k(7)*y(218)*y(44);
dydt(263)=dydt(263)+k(7)*y(218)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(44);
dydt(44)=dydt(44)-k(7)*y(192)*y(44);
dydt(262)=dydt(262)+k(7)*y(192)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(7)*y(184)*y(44);
dydt(44)=dydt(44)-k(7)*y(184)*y(44);
dydt(270)=dydt(270)+k(7)*y(184)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(44);
dydt(44)=dydt(44)-k(7)*y(177)*y(44);
dydt(269)=dydt(269)+k(7)*y(177)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(7)*y(140)*y(44);
dydt(44)=dydt(44)-k(7)*y(140)*y(44);
dydt(51)=dydt(51)+k(7)*y(140)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(7)*y(131)*y(44);
dydt(44)=dydt(44)-k(7)*y(131)*y(44);
dydt(52)=dydt(52)+k(7)*y(131)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(44);
dydt(44)=dydt(44)-k(7)*y(111)*y(44);
dydt(253)=dydt(253)+k(7)*y(111)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(77)=dydt(77)-k(7)*y(77)*y(44);
dydt(44)=dydt(44)-k(7)*y(77)*y(44);
dydt(250)=dydt(250)+k(7)*y(77)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(44);
dydt(44)=dydt(44)-k(7)*y(72)*y(44);
dydt(239)=dydt(239)+k(7)*y(72)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(61)=dydt(61)-k(7)*y(61)*y(44);
dydt(44)=dydt(44)-k(7)*y(61)*y(44);
dydt(654)=dydt(654)+k(7)*y(61)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(54)=dydt(54)-k(7)*y(54)*y(44);
dydt(44)=dydt(44)-k(7)*y(54)*y(44);
dydt(251)=dydt(251)+k(7)*y(54)*y(44);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(44);
dydt(44)=dydt(44)-k(7)*y(48)*y(44);
dydt(46)=dydt(46)+k(7)*y(48)*y(44);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) 

dydt(44)=dydt(44)-k(2)*y(44);
dydt(4)=dydt(4)+k(2)*y(44);
dydt(43)=dydt(43)+k(2)*y(44);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

dydt(45)=dydt(45)-k(22)*y(45);
dydt(2)=dydt(2)+k(22)*y(45);
dydt(233)=dydt(233)+k(22)*y(45);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(45)=dydt(45)-k(20)*y(45);
dydt(49)=dydt(49)+k(20)*y(45);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(45)=dydt(45)-k(18)*y(45);
dydt(239)=dydt(239)+k(18)*y(45);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(45)=dydt(45)-k(16)*y(45);
dydt(3)=dydt(3)+k(16)*y(45);
dydt(621)=dydt(621)+k(16)*y(45);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(45);
dydt(45)=dydt(45)-k(9)*y(3)*y(45);
dydt(42)=dydt(42)+k(9)*y(3)*y(45);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

dydt(45)=dydt(45)-k(8)*y(45);
dydt(1)=dydt(1)+k(8)*y(45);
dydt(44)=dydt(44)+k(8)*y(45);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(45)=dydt(45)-k(6)*y(45);
dydt(6)=dydt(6)+k(6)*y(45);
dydt(43)=dydt(43)+k(6)*y(45);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(47)=dydt(47)-k(32)*y(47);
dydt(9)=dydt(9)+k(32)*y(47);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(47);
dydt(47)=dydt(47)-k(21)*y(182)*y(47);
dydt(391)=dydt(391)+k(21)*y(182)*y(47);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(47);
dydt(47)=dydt(47)-k(21)*y(197)*y(47);
dydt(407)=dydt(407)+k(21)*y(197)*y(47);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(47);
dydt(47)=dydt(47)-k(21)*y(32)*y(47);
dydt(441)=dydt(441)+k(21)*y(32)*y(47);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(47);
dydt(47)=dydt(47)-k(21)*y(2)*y(47);
dydt(380)=dydt(380)+k(21)*y(2)*y(47);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Lyn(U, SH2) 

dydt(47)=dydt(47)-k(10)*y(47);
dydt(3)=dydt(3)+k(10)*y(47);
dydt(98)=dydt(98)+k(10)*y(47);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(1)=dydt(1)-k(5)*y(1)*y(47);
dydt(47)=dydt(47)-k(5)*y(1)*y(47);
dydt(99)=dydt(99)+k(5)*y(1)*y(47);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(5)*y(7)*y(47);
dydt(47)=dydt(47)-k(5)*y(7)*y(47);
dydt(463)=dydt(463)+k(5)*y(7)*y(47);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(47)=dydt(47)-k(5)*y(47)*y(335);
dydt(335)=dydt(335)-k(5)*y(47)*y(335);
dydt(400)=dydt(400)+k(5)*y(47)*y(335);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(47)=dydt(47)-k(5)*y(47)*y(257);
dydt(257)=dydt(257)-k(5)*y(47)*y(257);
dydt(401)=dydt(401)+k(5)*y(47)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(47)=dydt(47)-k(5)*y(47)*y(227);
dydt(227)=dydt(227)-k(5)*y(47)*y(227);
dydt(205)=dydt(205)+k(5)*y(47)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(47)=dydt(47)-k(5)*y(47)*y(222);
dydt(222)=dydt(222)-k(5)*y(47)*y(222);
dydt(225)=dydt(225)+k(5)*y(47)*y(222);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(47)=dydt(47)-k(5)*y(47)*y(218);
dydt(218)=dydt(218)-k(5)*y(47)*y(218);
dydt(430)=dydt(430)+k(5)*y(47)*y(218);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(47)=dydt(47)-k(5)*y(47)*y(192);
dydt(192)=dydt(192)-k(5)*y(47)*y(192);
dydt(399)=dydt(399)+k(5)*y(47)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(47)=dydt(47)-k(5)*y(47)*y(184);
dydt(184)=dydt(184)-k(5)*y(47)*y(184);
dydt(457)=dydt(457)+k(5)*y(47)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(47)=dydt(47)-k(5)*y(47)*y(177);
dydt(177)=dydt(177)-k(5)*y(47)*y(177);
dydt(396)=dydt(396)+k(5)*y(47)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(47)=dydt(47)-k(5)*y(47)*y(140);
dydt(140)=dydt(140)-k(5)*y(47)*y(140);
dydt(467)=dydt(467)+k(5)*y(47)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(47)=dydt(47)-k(5)*y(47)*y(131);
dydt(131)=dydt(131)-k(5)*y(47)*y(131);
dydt(155)=dydt(155)+k(5)*y(47)*y(131);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(47)=dydt(47)-k(5)*y(47)*y(111);
dydt(111)=dydt(111)-k(5)*y(47)*y(111);
dydt(50)=dydt(50)+k(5)*y(47)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(77)=dydt(77)-k(5)*y(77)*y(47);
dydt(47)=dydt(47)-k(5)*y(77)*y(47);
dydt(57)=dydt(57)+k(5)*y(77)*y(47);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(72)=dydt(72)-k(5)*y(72)*y(47);
dydt(47)=dydt(47)-k(5)*y(72)*y(47);
dydt(64)=dydt(64)+k(5)*y(72)*y(47);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(61)=dydt(61)-k(5)*y(61)*y(47);
dydt(47)=dydt(47)-k(5)*y(61)*y(47);
dydt(60)=dydt(60)+k(5)*y(61)*y(47);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(54)=dydt(54)-k(5)*y(54)*y(47);
dydt(47)=dydt(47)-k(5)*y(54)*y(47);
dydt(53)=dydt(53)+k(5)*y(54)*y(47);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(48)=dydt(48)-k(5)*y(48)*y(47);
dydt(47)=dydt(47)-k(5)*y(48)*y(47);
dydt(149)=dydt(149)+k(5)*y(48)*y(47);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Lig(l1, l2) 

dydt(47)=dydt(47)-k(4)*y(47);
dydt(4)=dydt(4)+k(4)*y(47);
dydt(48)=dydt(48)+k(4)*y(47);

% rule    : R15
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(48)=dydt(48)-k(32)*y(48);
dydt(7)=dydt(7)+k(32)*y(48);

% rule    : R9
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(48);
dydt(48)=dydt(48)-k(21)*y(182)*y(48);
dydt(184)=dydt(184)+k(21)*y(182)*y(48);

% rule    : R9
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(48);
dydt(48)=dydt(48)-k(21)*y(197)*y(48);
dydt(335)=dydt(335)+k(21)*y(197)*y(48);

% rule    : R9
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(48);
dydt(48)=dydt(48)-k(21)*y(32)*y(48);
dydt(131)=dydt(131)+k(21)*y(32)*y(48);

% rule    : R9
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(48);
dydt(48)=dydt(48)-k(21)*y(2)*y(48);
dydt(140)=dydt(140)+k(21)*y(2)*y(48);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a, b~Y, g~pY) + Lyn(U, SH2) 

dydt(48)=dydt(48)-k(10)*y(48);
dydt(3)=dydt(3)+k(10)*y(48);
dydt(15)=dydt(15)+k(10)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(55);
dydt(55)=dydt(55)-k(7)*y(48)*y(55);
dydt(53)=dydt(53)+k(7)*y(48)*y(55);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(62);
dydt(62)=dydt(62)-k(7)*y(48)*y(62);
dydt(60)=dydt(60)+k(7)*y(48)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(91)=dydt(91)-k(7)*y(91)*y(48);
dydt(48)=dydt(48)-k(7)*y(91)*y(48);
dydt(64)=dydt(64)+k(7)*y(91)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(126);
dydt(126)=dydt(126)-k(7)*y(48)*y(126);
dydt(149)=dydt(149)+k(7)*y(48)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(132);
dydt(132)=dydt(132)-k(7)*y(48)*y(132);
dydt(155)=dydt(155)+k(7)*y(48)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(141);
dydt(141)=dydt(141)-k(7)*y(48)*y(141);
dydt(467)=dydt(467)+k(7)*y(48)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(185);
dydt(185)=dydt(185)-k(7)*y(48)*y(185);
dydt(457)=dydt(457)+k(7)*y(48)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(48)=dydt(48)-k(7)*y(48)*y(209);
dydt(209)=dydt(209)-k(7)*y(48)*y(209);
dydt(399)=dydt(399)+k(7)*y(48)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(48)=dydt(48)-k(7)*y(48)*y(214);
dydt(214)=dydt(214)-k(7)*y(48)*y(214);
dydt(396)=dydt(396)+k(7)*y(48)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(48)=dydt(48)-k(7)*y(48)*y(219);
dydt(219)=dydt(219)-k(7)*y(48)*y(219);
dydt(430)=dydt(430)+k(7)*y(48)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(223);
dydt(223)=dydt(223)-k(7)*y(48)*y(223);
dydt(225)=dydt(225)+k(7)*y(48)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(48)=dydt(48)-k(7)*y(48)*y(228);
dydt(228)=dydt(228)-k(7)*y(48)*y(228);
dydt(205)=dydt(205)+k(7)*y(48)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(315)=dydt(315)-k(7)*y(315)*y(48);
dydt(48)=dydt(48)-k(7)*y(315)*y(48);
dydt(57)=dydt(57)+k(7)*y(315)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(48)=dydt(48)-k(7)*y(48)*y(319);
dydt(319)=dydt(319)-k(7)*y(48)*y(319);
dydt(401)=dydt(401)+k(7)*y(48)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(48)=dydt(48)-k(7)*y(48)*y(336);
dydt(336)=dydt(336)-k(7)*y(48)*y(336);
dydt(400)=dydt(400)+k(7)*y(48)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(48)=dydt(48)-k(7)*y(48)*y(437);
dydt(437)=dydt(437)-k(7)*y(48)*y(437);
dydt(50)=dydt(50)+k(7)*y(48)*y(437);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(71)=dydt(71)-k(5)*y(71)*y(48);
dydt(48)=dydt(48)-k(5)*y(71)*y(48);
dydt(128)=dydt(128)+k(5)*y(71)*y(48);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(76)=dydt(76)-k(5)*y(76)*y(48);
dydt(48)=dydt(48)-k(5)*y(76)*y(48);
dydt(151)=dydt(151)+k(5)*y(76)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(48)=dydt(48)-k(5)*y(48)*y(82);
dydt(82)=dydt(82)-k(5)*y(48)*y(82);
dydt(350)=dydt(350)+k(5)*y(48)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(48)=dydt(48)-k(5)*y(48)*y(86);
dydt(86)=dydt(86)-k(5)*y(48)*y(86);
dydt(153)=dydt(153)+k(5)*y(48)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(48)=dydt(48)-k(5)*y(48)*y(89);
dydt(89)=dydt(89)-k(5)*y(48)*y(89);
dydt(152)=dydt(152)+k(5)*y(48)*y(89);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) 

dydt(98)=dydt(98)-k(5)*y(98)*y(48);
dydt(48)=dydt(48)-k(5)*y(98)*y(48);
dydt(147)=dydt(147)+k(5)*y(98)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(48)=dydt(48)-k(5)*y(48)*y(110);
dydt(110)=dydt(110)-k(5)*y(48)*y(110);
dydt(353)=dydt(353)+k(5)*y(48)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(48)=dydt(48)-k(5)*y(48)*y(114);
dydt(114)=dydt(114)-k(5)*y(48)*y(114);
dydt(116)=dydt(116)+k(5)*y(48)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(48)=dydt(48)-k(5)*y(48)*y(118);
dydt(118)=dydt(118)-k(5)*y(48)*y(118);
dydt(120)=dydt(120)+k(5)*y(48)*y(118);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(48)=dydt(48)-k(5)*y(48)*y(122);
dydt(122)=dydt(122)-k(5)*y(48)*y(122);
dydt(125)=dydt(125)+k(5)*y(48)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(48)=dydt(48)-k(5)*y(48)*y(176);
dydt(176)=dydt(176)-k(5)*y(48)*y(176);
dydt(181)=dydt(181)+k(5)*y(48)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(48)=dydt(48)-k(5)*y(48)*y(191);
dydt(191)=dydt(191)-k(5)*y(48)*y(191);
dydt(196)=dydt(196)+k(5)*y(48)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(48)=dydt(48)-k(5)*y(48)*y(256);
dydt(256)=dydt(256)-k(5)*y(48)*y(256);
dydt(599)=dydt(599)+k(5)*y(48)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(48)=dydt(48)-k(5)*y(48)*y(264);
dydt(264)=dydt(264)-k(5)*y(48)*y(264);
dydt(361)=dydt(361)+k(5)*y(48)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(48)=dydt(48)-k(5)*y(48)*y(380);
dydt(380)=dydt(380)-k(5)*y(48)*y(380);
dydt(461)=dydt(461)+k(5)*y(48)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(48)=dydt(48)-k(5)*y(48)*y(391);
dydt(391)=dydt(391)-k(5)*y(48)*y(391);
dydt(651)=dydt(651)+k(5)*y(48)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(48)=dydt(48)-k(5)*y(48)*y(407);
dydt(407)=dydt(407)-k(5)*y(48)*y(407);
dydt(408)=dydt(408)+k(5)*y(48)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(48)=dydt(48)-k(5)*y(48)*y(441);
dydt(441)=dydt(441)-k(5)*y(48)*y(441);
dydt(458)=dydt(458)+k(5)*y(48)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(48)=dydt(48)-k(5)*y(48)*y(515);
dydt(515)=dydt(515)-k(5)*y(48)*y(515);
dydt(595)=dydt(595)+k(5)*y(48)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(48)=dydt(48)-k(5)*y(48)*y(543);
dydt(543)=dydt(543)-k(5)*y(48)*y(543);
dydt(598)=dydt(598)+k(5)*y(48)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(48)=dydt(48)-k(5)*y(48)*y(554);
dydt(554)=dydt(554)-k(5)*y(48)*y(554);
dydt(352)=dydt(352)+k(5)*y(48)*y(554);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(48)=dydt(48)-k(5)*y(48)*y(9);
dydt(9)=dydt(9)-k(5)*y(48)*y(9);
dydt(17)=dydt(17)+k(5)*y(48)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(4)=dydt(4)-k(3)*y(4)*y(48);
dydt(48)=dydt(48)-k(3)*y(4)*y(48);
dydt(47)=dydt(47)+k(3)*y(4)*y(48);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(4)=dydt(4)-k(1)*y(4)*y(48);
dydt(48)=dydt(48)-k(1)*y(4)*y(48);
dydt(126)=dydt(126)+k(1)*y(4)*y(48);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(49)=dydt(49)-k(32)*y(49);
dydt(45)=dydt(45)+k(32)*y(49);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

dydt(49)=dydt(49)-k(22)*y(49);
dydt(2)=dydt(2)+k(22)*y(49);
dydt(56)=dydt(56)+k(22)*y(49);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(49);
dydt(49)=dydt(49)-k(21)*y(182)*y(49);
dydt(269)=dydt(269)+k(21)*y(182)*y(49);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(49);
dydt(49)=dydt(49)-k(21)*y(197)*y(49);
dydt(262)=dydt(262)+k(21)*y(197)*y(49);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(49);
dydt(49)=dydt(49)-k(21)*y(32)*y(49);
dydt(277)=dydt(277)+k(21)*y(32)*y(49);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(49);
dydt(49)=dydt(49)-k(21)*y(2)*y(49);
dydt(274)=dydt(274)+k(21)*y(2)*y(49);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(49)=dydt(49)-k(18)*y(49);
dydt(250)=dydt(250)+k(18)*y(49);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(49)=dydt(49)-k(16)*y(49);
dydt(3)=dydt(3)+k(16)*y(49);
dydt(616)=dydt(616)+k(16)*y(49);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(49);
dydt(49)=dydt(49)-k(9)*y(3)*y(49);
dydt(46)=dydt(46)+k(9)*y(3)*y(49);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

dydt(49)=dydt(49)-k(8)*y(49);
dydt(15)=dydt(15)+k(8)*y(49);
dydt(44)=dydt(44)+k(8)*y(49);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(49)=dydt(49)-k(6)*y(49);
dydt(98)=dydt(98)+k(6)*y(49);
dydt(43)=dydt(43)+k(6)*y(49);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(50)=dydt(50)-k(32)*y(50);
dydt(25)=dydt(25)+k(32)*y(50);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(50)=dydt(50)-k(31)*y(50);
dydt(388)=dydt(388)+k(31)*y(50);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(50)=dydt(50)-k(23)*y(50);
dydt(470)=dydt(470)+k(23)*y(50);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(50)=dydt(50)-k(22)*y(50);
dydt(2)=dydt(2)+k(22)*y(50);
dydt(57)=dydt(57)+k(22)*y(50);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(50);
dydt(50)=dydt(50)-k(21)*y(182)*y(50);
dydt(392)=dydt(392)+k(21)*y(182)*y(50);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(50);
dydt(50)=dydt(50)-k(21)*y(197)*y(50);
dydt(436)=dydt(436)+k(21)*y(197)*y(50);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(50);
dydt(50)=dydt(50)-k(21)*y(32)*y(50);
dydt(440)=dydt(440)+k(21)*y(32)*y(50);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(50);
dydt(50)=dydt(50)-k(21)*y(2)*y(50);
dydt(387)=dydt(387)+k(21)*y(2)*y(50);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(50);
dydt(50)=dydt(50)-k(15)*y(3)*y(50);
dydt(46)=dydt(46)+k(15)*y(3)*y(50);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(50)=dydt(50)-k(10)*y(50);
dydt(3)=dydt(3)+k(10)*y(50);
dydt(616)=dydt(616)+k(10)*y(50);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(50)=dydt(50)-k(8)*y(50);
dydt(48)=dydt(48)+k(8)*y(50);
dydt(437)=dydt(437)+k(8)*y(50);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(50)=dydt(50)-k(6)*y(50);
dydt(47)=dydt(47)+k(6)*y(50);
dydt(111)=dydt(111)+k(6)*y(50);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(51)=dydt(51)-k(28)*y(51);
dydt(270)=dydt(270)+k(28)*y(51);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(51)=dydt(51)-k(27)*y(51);
dydt(279)=dydt(279)+k(27)*y(51);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(51)=dydt(51)-k(26)*y(51);
dydt(52)=dydt(52)+k(26)*y(51);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(51)=dydt(51)-k(23)*y(51);
dydt(480)=dydt(480)+k(23)*y(51);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(51)=dydt(51)-k(22)*y(51);
dydt(2)=dydt(2)+k(22)*y(51);
dydt(46)=dydt(46)+k(22)*y(51);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(51)=dydt(51)-k(22)*y(51);
dydt(2)=dydt(2)+k(22)*y(51);
dydt(58)=dydt(58)+k(22)*y(51);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(51)=dydt(51)-k(16)*y(51);
dydt(3)=dydt(3)+k(16)*y(51);
dydt(387)=dydt(387)+k(16)*y(51);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(51)=dydt(51)-k(10)*y(51);
dydt(3)=dydt(3)+k(10)*y(51);
dydt(274)=dydt(274)+k(10)*y(51);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(51)=dydt(51)-k(8)*y(51);
dydt(140)=dydt(140)+k(8)*y(51);
dydt(44)=dydt(44)+k(8)*y(51);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(51)=dydt(51)-k(6)*y(51);
dydt(43)=dydt(43)+k(6)*y(51);
dydt(380)=dydt(380)+k(6)*y(51);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(52)=dydt(52)-k(33)*y(52);
dydt(51)=dydt(51)+k(33)*y(52);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(52)=dydt(52)-k(28)*y(52);
dydt(207)=dydt(207)+k(28)*y(52);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(52)=dydt(52)-k(27)*y(52);
dydt(284)=dydt(284)+k(27)*y(52);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(52)=dydt(52)-k(23)*y(52);
dydt(469)=dydt(469)+k(23)*y(52);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(52)=dydt(52)-k(22)*y(52);
dydt(32)=dydt(32)+k(22)*y(52);
dydt(46)=dydt(46)+k(22)*y(52);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(52)=dydt(52)-k(22)*y(52);
dydt(2)=dydt(2)+k(22)*y(52);
dydt(59)=dydt(59)+k(22)*y(52);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(52)=dydt(52)-k(16)*y(52);
dydt(3)=dydt(3)+k(16)*y(52);
dydt(440)=dydt(440)+k(16)*y(52);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(52)=dydt(52)-k(10)*y(52);
dydt(3)=dydt(3)+k(10)*y(52);
dydt(277)=dydt(277)+k(10)*y(52);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(52)=dydt(52)-k(8)*y(52);
dydt(131)=dydt(131)+k(8)*y(52);
dydt(44)=dydt(44)+k(8)*y(52);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(52)=dydt(52)-k(6)*y(52);
dydt(43)=dydt(43)+k(6)*y(52);
dydt(441)=dydt(441)+k(6)*y(52);

% rule    : R15
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(54)=dydt(54)-k(32)*y(54);
dydt(61)=dydt(61)+k(32)*y(54);

% rule    : R9
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(54);
dydt(54)=dydt(54)-k(21)*y(182)*y(54);
dydt(222)=dydt(222)+k(21)*y(182)*y(54);

% rule    : R9
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(54);
dydt(54)=dydt(54)-k(21)*y(197)*y(54);
dydt(227)=dydt(227)+k(21)*y(197)*y(54);

% rule    : R9
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(54);
dydt(54)=dydt(54)-k(21)*y(32)*y(54);
dydt(37)=dydt(37)+k(21)*y(32)*y(54);

% rule    : R9
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(54);
dydt(54)=dydt(54)-k(21)*y(2)*y(54);
dydt(43)=dydt(43)+k(21)*y(2)*y(54);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a, b~pY, g~pY) + Lyn(U, SH2) 

dydt(54)=dydt(54)-k(16)*y(54);
dydt(3)=dydt(3)+k(16)*y(54);
dydt(77)=dydt(77)+k(16)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(54)=dydt(54)-k(7)*y(54)*y(55);
dydt(55)=dydt(55)-k(7)*y(54)*y(55);
dydt(627)=dydt(627)+k(7)*y(54)*y(55);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(54)=dydt(54)-k(7)*y(54)*y(62);
dydt(62)=dydt(62)-k(7)*y(54)*y(62);
dydt(78)=dydt(78)+k(7)*y(54)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(91)=dydt(91)-k(7)*y(91)*y(54);
dydt(54)=dydt(54)-k(7)*y(91)*y(54);
dydt(309)=dydt(309)+k(7)*y(91)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(54)=dydt(54)-k(7)*y(54)*y(126);
dydt(126)=dydt(126)-k(7)*y(54)*y(126);
dydt(120)=dydt(120)+k(7)*y(54)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(54)=dydt(54)-k(7)*y(54)*y(132);
dydt(132)=dydt(132)-k(7)*y(54)*y(132);
dydt(134)=dydt(134)+k(7)*y(54)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(54)=dydt(54)-k(7)*y(54)*y(141);
dydt(141)=dydt(141)-k(7)*y(54)*y(141);
dydt(539)=dydt(539)+k(7)*y(54)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(54)=dydt(54)-k(7)*y(54)*y(185);
dydt(185)=dydt(185)-k(7)*y(54)*y(185);
dydt(345)=dydt(345)+k(7)*y(54)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(54)=dydt(54)-k(7)*y(54)*y(209);
dydt(209)=dydt(209)-k(7)*y(54)*y(209);
dydt(529)=dydt(529)+k(7)*y(54)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(54)=dydt(54)-k(7)*y(54)*y(214);
dydt(214)=dydt(214)-k(7)*y(54)*y(214);
dydt(519)=dydt(519)+k(7)*y(54)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(54)=dydt(54)-k(7)*y(54)*y(219);
dydt(219)=dydt(219)-k(7)*y(54)*y(219);
dydt(644)=dydt(644)+k(7)*y(54)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(54)=dydt(54)-k(7)*y(54)*y(223);
dydt(223)=dydt(223)-k(7)*y(54)*y(223);
dydt(638)=dydt(638)+k(7)*y(54)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(54)=dydt(54)-k(7)*y(54)*y(228);
dydt(228)=dydt(228)-k(7)*y(54)*y(228);
dydt(636)=dydt(636)+k(7)*y(54)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(315)=dydt(315)-k(7)*y(315)*y(54);
dydt(54)=dydt(54)-k(7)*y(315)*y(54);
dydt(310)=dydt(310)+k(7)*y(315)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(54)=dydt(54)-k(7)*y(54)*y(319);
dydt(319)=dydt(319)-k(7)*y(54)*y(319);
dydt(564)=dydt(564)+k(7)*y(54)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(54)=dydt(54)-k(7)*y(54)*y(336);
dydt(336)=dydt(336)-k(7)*y(54)*y(336);
dydt(339)=dydt(339)+k(7)*y(54)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(54)=dydt(54)-k(7)*y(54)*y(437);
dydt(437)=dydt(437)-k(7)*y(54)*y(437);
dydt(540)=dydt(540)+k(7)*y(54)*y(437);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(71)=dydt(71)-k(5)*y(71)*y(54);
dydt(54)=dydt(54)-k(5)*y(71)*y(54);
dydt(235)=dydt(235)+k(5)*y(71)*y(54);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(76)=dydt(76)-k(5)*y(76)*y(54);
dydt(54)=dydt(54)-k(5)*y(76)*y(54);
dydt(238)=dydt(238)+k(5)*y(76)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(54)=dydt(54)-k(5)*y(54)*y(82);
dydt(82)=dydt(82)-k(5)*y(54)*y(82);
dydt(634)=dydt(634)+k(5)*y(54)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(54)=dydt(54)-k(5)*y(54)*y(86);
dydt(86)=dydt(86)-k(5)*y(54)*y(86);
dydt(287)=dydt(287)+k(5)*y(54)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(54)=dydt(54)-k(5)*y(54)*y(89);
dydt(89)=dydt(89)-k(5)*y(54)*y(89);
dydt(276)=dydt(276)+k(5)*y(54)*y(89);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(98)=dydt(98)-k(5)*y(98)*y(54);
dydt(54)=dydt(54)-k(5)*y(98)*y(54);
dydt(56)=dydt(56)+k(5)*y(98)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(54)=dydt(54)-k(5)*y(54)*y(110);
dydt(110)=dydt(110)-k(5)*y(54)*y(110);
dydt(629)=dydt(629)+k(5)*y(54)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(54)=dydt(54)-k(5)*y(54)*y(114);
dydt(114)=dydt(114)-k(5)*y(54)*y(114);
dydt(628)=dydt(628)+k(5)*y(54)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(54)=dydt(54)-k(5)*y(54)*y(118);
dydt(118)=dydt(118)-k(5)*y(54)*y(118);
dydt(627)=dydt(627)+k(5)*y(54)*y(118);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(54)=dydt(54)-k(5)*y(54)*y(122);
dydt(122)=dydt(122)-k(5)*y(54)*y(122);
dydt(236)=dydt(236)+k(5)*y(54)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(54)=dydt(54)-k(5)*y(54)*y(176);
dydt(176)=dydt(176)-k(5)*y(54)*y(176);
dydt(272)=dydt(272)+k(5)*y(54)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(54)=dydt(54)-k(5)*y(54)*y(191);
dydt(191)=dydt(191)-k(5)*y(54)*y(191);
dydt(291)=dydt(291)+k(5)*y(54)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(54)=dydt(54)-k(5)*y(54)*y(256);
dydt(256)=dydt(256)-k(5)*y(54)*y(256);
dydt(260)=dydt(260)+k(5)*y(54)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(54)=dydt(54)-k(5)*y(54)*y(264);
dydt(264)=dydt(264)-k(5)*y(54)*y(264);
dydt(267)=dydt(267)+k(5)*y(54)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(54)=dydt(54)-k(5)*y(54)*y(380);
dydt(380)=dydt(380)-k(5)*y(54)*y(380);
dydt(58)=dydt(58)+k(5)*y(54)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(54)=dydt(54)-k(5)*y(54)*y(391);
dydt(391)=dydt(391)-k(5)*y(54)*y(391);
dydt(422)=dydt(422)+k(5)*y(54)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(54)=dydt(54)-k(5)*y(54)*y(407);
dydt(407)=dydt(407)-k(5)*y(54)*y(407);
dydt(206)=dydt(206)+k(5)*y(54)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(54)=dydt(54)-k(5)*y(54)*y(441);
dydt(441)=dydt(441)-k(5)*y(54)*y(441);
dydt(59)=dydt(59)+k(5)*y(54)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(54)=dydt(54)-k(5)*y(54)*y(515);
dydt(515)=dydt(515)-k(5)*y(54)*y(515);
dydt(624)=dydt(624)+k(5)*y(54)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(54)=dydt(54)-k(5)*y(54)*y(543);
dydt(543)=dydt(543)-k(5)*y(54)*y(543);
dydt(622)=dydt(622)+k(5)*y(54)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(54)=dydt(54)-k(5)*y(54)*y(554);
dydt(554)=dydt(554)-k(5)*y(54)*y(554);
dydt(633)=dydt(633)+k(5)*y(54)*y(554);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(54)=dydt(54)-k(5)*y(54)*y(9);
dydt(9)=dydt(9)-k(5)*y(54)*y(9);
dydt(41)=dydt(41)+k(5)*y(54)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(4)=dydt(4)-k(3)*y(4)*y(54);
dydt(54)=dydt(54)-k(3)*y(4)*y(54);
dydt(118)=dydt(118)+k(3)*y(4)*y(54);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(4)=dydt(4)-k(1)*y(4)*y(54);
dydt(54)=dydt(54)-k(1)*y(4)*y(54);
dydt(55)=dydt(55)+k(1)*y(4)*y(54);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(55)=dydt(55)-k(32)*y(55);
dydt(62)=dydt(62)+k(32)*y(55);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(55);
dydt(55)=dydt(55)-k(21)*y(182)*y(55);
dydt(223)=dydt(223)+k(21)*y(182)*y(55);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(55);
dydt(55)=dydt(55)-k(21)*y(197)*y(55);
dydt(228)=dydt(228)+k(21)*y(197)*y(55);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(55);
dydt(55)=dydt(55)-k(21)*y(32)*y(55);
dydt(38)=dydt(38)+k(21)*y(32)*y(55);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(55);
dydt(55)=dydt(55)-k(21)*y(2)*y(55);
dydt(44)=dydt(44)+k(21)*y(2)*y(55);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Lyn(U, SH2) 

dydt(55)=dydt(55)-k(16)*y(55);
dydt(3)=dydt(3)+k(16)*y(55);
dydt(315)=dydt(315)+k(16)*y(55);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(1)=dydt(1)-k(7)*y(1)*y(55);
dydt(55)=dydt(55)-k(7)*y(1)*y(55);
dydt(233)=dydt(233)+k(7)*y(1)*y(55);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(7)*y(7)*y(55);
dydt(55)=dydt(55)-k(7)*y(7)*y(55);
dydt(41)=dydt(41)+k(7)*y(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(55)=dydt(55)-k(7)*y(55)*y(335);
dydt(335)=dydt(335)-k(7)*y(55)*y(335);
dydt(206)=dydt(206)+k(7)*y(55)*y(335);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(55)=dydt(55)-k(7)*y(55)*y(257);
dydt(257)=dydt(257)-k(7)*y(55)*y(257);
dydt(260)=dydt(260)+k(7)*y(55)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(55)=dydt(55)-k(7)*y(55)*y(227);
dydt(227)=dydt(227)-k(7)*y(55)*y(227);
dydt(622)=dydt(622)+k(7)*y(55)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(55)=dydt(55)-k(7)*y(55)*y(222);
dydt(222)=dydt(222)-k(7)*y(55)*y(222);
dydt(624)=dydt(624)+k(7)*y(55)*y(222);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(55)=dydt(55)-k(7)*y(55)*y(218);
dydt(218)=dydt(218)-k(7)*y(55)*y(218);
dydt(267)=dydt(267)+k(7)*y(55)*y(218);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(55)=dydt(55)-k(7)*y(55)*y(192);
dydt(192)=dydt(192)-k(7)*y(55)*y(192);
dydt(291)=dydt(291)+k(7)*y(55)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(55)=dydt(55)-k(7)*y(55)*y(184);
dydt(184)=dydt(184)-k(7)*y(55)*y(184);
dydt(422)=dydt(422)+k(7)*y(55)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(55)=dydt(55)-k(7)*y(55)*y(177);
dydt(177)=dydt(177)-k(7)*y(55)*y(177);
dydt(272)=dydt(272)+k(7)*y(55)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(55)=dydt(55)-k(7)*y(55)*y(140);
dydt(140)=dydt(140)-k(7)*y(55)*y(140);
dydt(58)=dydt(58)+k(7)*y(55)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(55)=dydt(55)-k(7)*y(55)*y(131);
dydt(131)=dydt(131)-k(7)*y(55)*y(131);
dydt(59)=dydt(59)+k(7)*y(55)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(55)=dydt(55)-k(7)*y(55)*y(111);
dydt(111)=dydt(111)-k(7)*y(55)*y(111);
dydt(629)=dydt(629)+k(7)*y(55)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(77)=dydt(77)-k(7)*y(77)*y(55);
dydt(55)=dydt(55)-k(7)*y(77)*y(55);
dydt(238)=dydt(238)+k(7)*y(77)*y(55);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(72)=dydt(72)-k(7)*y(72)*y(55);
dydt(55)=dydt(55)-k(7)*y(72)*y(55);
dydt(235)=dydt(235)+k(7)*y(72)*y(55);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(7)*y(61)*y(55);
dydt(55)=dydt(55)-k(7)*y(61)*y(55);
dydt(236)=dydt(236)+k(7)*y(61)*y(55);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Lig(l1, l2) 

dydt(55)=dydt(55)-k(2)*y(55);
dydt(4)=dydt(4)+k(2)*y(55);
dydt(54)=dydt(54)+k(2)*y(55);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(56)=dydt(56)-k(32)*y(56);
dydt(233)=dydt(233)+k(32)*y(56);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(56)=dydt(56)-k(32)*y(56);
dydt(63)=dydt(63)+k(32)*y(56);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(56);
dydt(56)=dydt(56)-k(21)*y(182)*y(56);
dydt(282)=dydt(282)+k(21)*y(182)*y(56);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(56);
dydt(56)=dydt(56)-k(21)*y(197)*y(56);
dydt(232)=dydt(232)+k(21)*y(197)*y(56);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(56);
dydt(56)=dydt(56)-k(21)*y(32)*y(56);
dydt(249)=dydt(249)+k(21)*y(32)*y(56);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(56);
dydt(56)=dydt(56)-k(21)*y(2)*y(56);
dydt(49)=dydt(49)+k(21)*y(2)*y(56);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(56);
dydt(56)=dydt(56)-k(21)*y(182)*y(56);
dydt(272)=dydt(272)+k(21)*y(182)*y(56);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(56);
dydt(56)=dydt(56)-k(21)*y(197)*y(56);
dydt(291)=dydt(291)+k(21)*y(197)*y(56);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(56);
dydt(56)=dydt(56)-k(21)*y(32)*y(56);
dydt(287)=dydt(287)+k(21)*y(32)*y(56);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(56);
dydt(56)=dydt(56)-k(21)*y(2)*y(56);
dydt(276)=dydt(276)+k(21)*y(2)*y(56);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(56)=dydt(56)-k(18)*y(56);
dydt(238)=dydt(238)+k(18)*y(56);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

dydt(56)=dydt(56)-k(16)*y(56);
dydt(3)=dydt(3)+k(16)*y(56);
dydt(614)=dydt(614)+k(16)*y(56);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(56);
dydt(56)=dydt(56)-k(9)*y(3)*y(56);
dydt(53)=dydt(53)+k(9)*y(3)*y(56);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) 

dydt(56)=dydt(56)-k(8)*y(56);
dydt(15)=dydt(15)+k(8)*y(56);
dydt(55)=dydt(55)+k(8)*y(56);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(56)=dydt(56)-k(6)*y(56);
dydt(98)=dydt(98)+k(6)*y(56);
dydt(54)=dydt(54)+k(6)*y(56);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(57)=dydt(57)-k(32)*y(57);
dydt(19)=dydt(19)+k(32)*y(57);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(57)=dydt(57)-k(32)*y(57);
dydt(64)=dydt(64)+k(32)*y(57);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(57)=dydt(57)-k(31)*y(57);
dydt(404)=dydt(404)+k(31)*y(57);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(57);
dydt(57)=dydt(57)-k(21)*y(182)*y(57);
dydt(430)=dydt(430)+k(21)*y(182)*y(57);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(57);
dydt(57)=dydt(57)-k(21)*y(197)*y(57);
dydt(401)=dydt(401)+k(21)*y(197)*y(57);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(57);
dydt(57)=dydt(57)-k(21)*y(32)*y(57);
dydt(470)=dydt(470)+k(21)*y(32)*y(57);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(57);
dydt(57)=dydt(57)-k(21)*y(2)*y(57);
dydt(50)=dydt(50)+k(21)*y(2)*y(57);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(57);
dydt(57)=dydt(57)-k(21)*y(182)*y(57);
dydt(420)=dydt(420)+k(21)*y(182)*y(57);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(57);
dydt(57)=dydt(57)-k(21)*y(197)*y(57);
dydt(410)=dydt(410)+k(21)*y(197)*y(57);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(57);
dydt(57)=dydt(57)-k(21)*y(32)*y(57);
dydt(443)=dydt(443)+k(21)*y(32)*y(57);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(57);
dydt(57)=dydt(57)-k(21)*y(2)*y(57);
dydt(424)=dydt(424)+k(21)*y(2)*y(57);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(57);
dydt(57)=dydt(57)-k(15)*y(3)*y(57);
dydt(53)=dydt(53)+k(15)*y(3)*y(57);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

dydt(57)=dydt(57)-k(10)*y(57);
dydt(3)=dydt(3)+k(10)*y(57);
dydt(614)=dydt(614)+k(10)*y(57);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(57)=dydt(57)-k(8)*y(57);
dydt(315)=dydt(315)+k(8)*y(57);
dydt(48)=dydt(48)+k(8)*y(57);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) 

dydt(57)=dydt(57)-k(6)*y(57);
dydt(77)=dydt(77)+k(6)*y(57);
dydt(47)=dydt(47)+k(6)*y(57);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(58)=dydt(58)-k(32)*y(58);
dydt(65)=dydt(65)+k(32)*y(58);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(58)=dydt(58)-k(26)*y(58);
dydt(59)=dydt(59)+k(26)*y(58);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(58)=dydt(58)-k(22)*y(58);
dydt(2)=dydt(2)+k(22)*y(58);
dydt(53)=dydt(53)+k(22)*y(58);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(58);
dydt(58)=dydt(58)-k(21)*y(182)*y(58);
dydt(279)=dydt(279)+k(21)*y(182)*y(58);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(58);
dydt(58)=dydt(58)-k(21)*y(197)*y(58);
dydt(428)=dydt(428)+k(21)*y(197)*y(58);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(58);
dydt(58)=dydt(58)-k(21)*y(32)*y(58);
dydt(480)=dydt(480)+k(21)*y(32)*y(58);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(58);
dydt(58)=dydt(58)-k(21)*y(2)*y(58);
dydt(51)=dydt(51)+k(21)*y(2)*y(58);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(58)=dydt(58)-k(16)*y(58);
dydt(3)=dydt(3)+k(16)*y(58);
dydt(424)=dydt(424)+k(16)*y(58);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(58)=dydt(58)-k(10)*y(58);
dydt(3)=dydt(3)+k(10)*y(58);
dydt(276)=dydt(276)+k(10)*y(58);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(58)=dydt(58)-k(8)*y(58);
dydt(55)=dydt(55)+k(8)*y(58);
dydt(140)=dydt(140)+k(8)*y(58);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(58)=dydt(58)-k(6)*y(58);
dydt(54)=dydt(54)+k(6)*y(58);
dydt(380)=dydt(380)+k(6)*y(58);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(59)=dydt(59)-k(33)*y(59);
dydt(58)=dydt(58)+k(33)*y(59);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(59)=dydt(59)-k(32)*y(59);
dydt(66)=dydt(66)+k(32)*y(59);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(59)=dydt(59)-k(22)*y(59);
dydt(32)=dydt(32)+k(22)*y(59);
dydt(53)=dydt(53)+k(22)*y(59);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(59);
dydt(59)=dydt(59)-k(21)*y(182)*y(59);
dydt(284)=dydt(284)+k(21)*y(182)*y(59);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(59);
dydt(59)=dydt(59)-k(21)*y(197)*y(59);
dydt(473)=dydt(473)+k(21)*y(197)*y(59);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(59);
dydt(59)=dydt(59)-k(21)*y(32)*y(59);
dydt(469)=dydt(469)+k(21)*y(32)*y(59);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(59);
dydt(59)=dydt(59)-k(21)*y(2)*y(59);
dydt(52)=dydt(52)+k(21)*y(2)*y(59);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(59)=dydt(59)-k(16)*y(59);
dydt(3)=dydt(3)+k(16)*y(59);
dydt(443)=dydt(443)+k(16)*y(59);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(59)=dydt(59)-k(10)*y(59);
dydt(3)=dydt(3)+k(10)*y(59);
dydt(287)=dydt(287)+k(10)*y(59);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(59)=dydt(59)-k(8)*y(59);
dydt(55)=dydt(55)+k(8)*y(59);
dydt(131)=dydt(131)+k(8)*y(59);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(59)=dydt(59)-k(6)*y(59);
dydt(54)=dydt(54)+k(6)*y(59);
dydt(441)=dydt(441)+k(6)*y(59);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a, b~pY, g~Y) + Lyn(U, SH2) 

dydt(61)=dydt(61)-k(16)*y(61);
dydt(3)=dydt(3)+k(16)*y(61);
dydt(72)=dydt(72)+k(16)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(7)*y(61)*y(62);
dydt(62)=dydt(62)-k(7)*y(61)*y(62);
dydt(73)=dydt(73)+k(7)*y(61)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(91)=dydt(91)-k(7)*y(91)*y(61);
dydt(61)=dydt(61)-k(7)*y(91)*y(61);
dydt(313)=dydt(313)+k(7)*y(91)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(7)*y(61)*y(126);
dydt(126)=dydt(126)-k(7)*y(61)*y(126);
dydt(125)=dydt(125)+k(7)*y(61)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(61)=dydt(61)-k(7)*y(61)*y(132);
dydt(132)=dydt(132)-k(7)*y(61)*y(132);
dydt(130)=dydt(130)+k(7)*y(61)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(61)=dydt(61)-k(7)*y(61)*y(141);
dydt(141)=dydt(141)-k(7)*y(61)*y(141);
dydt(129)=dydt(129)+k(7)*y(61)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(61)=dydt(61)-k(7)*y(61)*y(185);
dydt(185)=dydt(185)-k(7)*y(61)*y(185);
dydt(327)=dydt(327)+k(7)*y(61)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(61)=dydt(61)-k(7)*y(61)*y(209);
dydt(209)=dydt(209)-k(7)*y(61)*y(209);
dydt(322)=dydt(322)+k(7)*y(61)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(61)=dydt(61)-k(7)*y(61)*y(214);
dydt(214)=dydt(214)-k(7)*y(61)*y(214);
dydt(326)=dydt(326)+k(7)*y(61)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(61)=dydt(61)-k(7)*y(61)*y(219);
dydt(219)=dydt(219)-k(7)*y(61)*y(219);
dydt(323)=dydt(323)+k(7)*y(61)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(7)*y(61)*y(223);
dydt(223)=dydt(223)-k(7)*y(61)*y(223);
dydt(324)=dydt(324)+k(7)*y(61)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(7)*y(61)*y(228);
dydt(228)=dydt(228)-k(7)*y(61)*y(228);
dydt(320)=dydt(320)+k(7)*y(61)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(315)=dydt(315)-k(7)*y(315)*y(61);
dydt(61)=dydt(61)-k(7)*y(315)*y(61);
dydt(314)=dydt(314)+k(7)*y(315)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(61)=dydt(61)-k(7)*y(61)*y(319);
dydt(319)=dydt(319)-k(7)*y(61)*y(319);
dydt(318)=dydt(318)+k(7)*y(61)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(61)=dydt(61)-k(7)*y(61)*y(336);
dydt(336)=dydt(336)-k(7)*y(61)*y(336);
dydt(200)=dydt(200)+k(7)*y(61)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(61)=dydt(61)-k(7)*y(61)*y(437);
dydt(437)=dydt(437)-k(7)*y(61)*y(437);
dydt(316)=dydt(316)+k(7)*y(61)*y(437);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(71)=dydt(71)-k(5)*y(71)*y(61);
dydt(61)=dydt(61)-k(5)*y(71)*y(61);
dydt(70)=dydt(70)+k(5)*y(71)*y(61);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(76)=dydt(76)-k(5)*y(76)*y(61);
dydt(61)=dydt(61)-k(5)*y(76)*y(61);
dydt(75)=dydt(75)+k(5)*y(76)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(61)=dydt(61)-k(5)*y(61)*y(82);
dydt(82)=dydt(82)-k(5)*y(61)*y(82);
dydt(81)=dydt(81)+k(5)*y(61)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(61)=dydt(61)-k(5)*y(61)*y(86);
dydt(86)=dydt(86)-k(5)*y(61)*y(86);
dydt(85)=dydt(85)+k(5)*y(61)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(61)=dydt(61)-k(5)*y(61)*y(89);
dydt(89)=dydt(89)-k(5)*y(61)*y(89);
dydt(88)=dydt(88)+k(5)*y(61)*y(89);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(98)=dydt(98)-k(5)*y(98)*y(61);
dydt(61)=dydt(61)-k(5)*y(98)*y(61);
dydt(63)=dydt(63)+k(5)*y(98)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(61)=dydt(61)-k(5)*y(61)*y(110);
dydt(110)=dydt(110)-k(5)*y(61)*y(110);
dydt(80)=dydt(80)+k(5)*y(61)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(61)=dydt(61)-k(5)*y(61)*y(114);
dydt(114)=dydt(114)-k(5)*y(61)*y(114);
dydt(307)=dydt(307)+k(5)*y(61)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(5)*y(61)*y(118);
dydt(118)=dydt(118)-k(5)*y(61)*y(118);
dydt(78)=dydt(78)+k(5)*y(61)*y(118);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(5)*y(61)*y(122);
dydt(122)=dydt(122)-k(5)*y(61)*y(122);
dydt(73)=dydt(73)+k(5)*y(61)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(61)=dydt(61)-k(5)*y(61)*y(176);
dydt(176)=dydt(176)-k(5)*y(61)*y(176);
dydt(297)=dydt(297)+k(5)*y(61)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(61)=dydt(61)-k(5)*y(61)*y(191);
dydt(191)=dydt(191)-k(5)*y(61)*y(191);
dydt(295)=dydt(295)+k(5)*y(61)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(61)=dydt(61)-k(5)*y(61)*y(256);
dydt(256)=dydt(256)-k(5)*y(61)*y(256);
dydt(203)=dydt(203)+k(5)*y(61)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(61)=dydt(61)-k(5)*y(61)*y(264);
dydt(264)=dydt(264)-k(5)*y(61)*y(264);
dydt(300)=dydt(300)+k(5)*y(61)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(61)=dydt(61)-k(5)*y(61)*y(380);
dydt(380)=dydt(380)-k(5)*y(61)*y(380);
dydt(65)=dydt(65)+k(5)*y(61)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(61)=dydt(61)-k(5)*y(61)*y(391);
dydt(391)=dydt(391)-k(5)*y(61)*y(391);
dydt(298)=dydt(298)+k(5)*y(61)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(61)=dydt(61)-k(5)*y(61)*y(407);
dydt(407)=dydt(407)-k(5)*y(61)*y(407);
dydt(204)=dydt(204)+k(5)*y(61)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(61)=dydt(61)-k(5)*y(61)*y(441);
dydt(441)=dydt(441)-k(5)*y(61)*y(441);
dydt(66)=dydt(66)+k(5)*y(61)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(5)*y(61)*y(515);
dydt(515)=dydt(515)-k(5)*y(61)*y(515);
dydt(301)=dydt(301)+k(5)*y(61)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(5)*y(61)*y(543);
dydt(543)=dydt(543)-k(5)*y(61)*y(543);
dydt(647)=dydt(647)+k(5)*y(61)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(61)=dydt(61)-k(5)*y(61)*y(554);
dydt(554)=dydt(554)-k(5)*y(61)*y(554);
dydt(83)=dydt(83)+k(5)*y(61)*y(554);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(61)=dydt(61)-k(5)*y(61)*y(9);
dydt(9)=dydt(9)-k(5)*y(61)*y(9);
dydt(67)=dydt(67)+k(5)*y(61)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(4)=dydt(4)-k(3)*y(4)*y(61);
dydt(61)=dydt(61)-k(3)*y(4)*y(61);
dydt(122)=dydt(122)+k(3)*y(4)*y(61);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(4)=dydt(4)-k(1)*y(4)*y(61);
dydt(61)=dydt(61)-k(1)*y(4)*y(61);
dydt(62)=dydt(62)+k(1)*y(4)*y(61);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Lyn(U, SH2) 

dydt(62)=dydt(62)-k(16)*y(62);
dydt(3)=dydt(3)+k(16)*y(62);
dydt(91)=dydt(91)+k(16)*y(62);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(1)=dydt(1)-k(7)*y(1)*y(62);
dydt(62)=dydt(62)-k(7)*y(1)*y(62);
dydt(68)=dydt(68)+k(7)*y(1)*y(62);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(7)*y(7)*y(62);
dydt(62)=dydt(62)-k(7)*y(7)*y(62);
dydt(67)=dydt(67)+k(7)*y(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(62)=dydt(62)-k(7)*y(62)*y(335);
dydt(335)=dydt(335)-k(7)*y(62)*y(335);
dydt(204)=dydt(204)+k(7)*y(62)*y(335);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(62)=dydt(62)-k(7)*y(62)*y(257);
dydt(257)=dydt(257)-k(7)*y(62)*y(257);
dydt(203)=dydt(203)+k(7)*y(62)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(62)=dydt(62)-k(7)*y(62)*y(227);
dydt(227)=dydt(227)-k(7)*y(62)*y(227);
dydt(647)=dydt(647)+k(7)*y(62)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(62)=dydt(62)-k(7)*y(62)*y(222);
dydt(222)=dydt(222)-k(7)*y(62)*y(222);
dydt(301)=dydt(301)+k(7)*y(62)*y(222);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(62)=dydt(62)-k(7)*y(62)*y(218);
dydt(218)=dydt(218)-k(7)*y(62)*y(218);
dydt(300)=dydt(300)+k(7)*y(62)*y(218);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(62)=dydt(62)-k(7)*y(62)*y(192);
dydt(192)=dydt(192)-k(7)*y(62)*y(192);
dydt(295)=dydt(295)+k(7)*y(62)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(62)=dydt(62)-k(7)*y(62)*y(184);
dydt(184)=dydt(184)-k(7)*y(62)*y(184);
dydt(298)=dydt(298)+k(7)*y(62)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(62)=dydt(62)-k(7)*y(62)*y(177);
dydt(177)=dydt(177)-k(7)*y(62)*y(177);
dydt(297)=dydt(297)+k(7)*y(62)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(62)=dydt(62)-k(7)*y(62)*y(140);
dydt(140)=dydt(140)-k(7)*y(62)*y(140);
dydt(65)=dydt(65)+k(7)*y(62)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(62)=dydt(62)-k(7)*y(62)*y(131);
dydt(131)=dydt(131)-k(7)*y(62)*y(131);
dydt(66)=dydt(66)+k(7)*y(62)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(62)=dydt(62)-k(7)*y(62)*y(111);
dydt(111)=dydt(111)-k(7)*y(62)*y(111);
dydt(80)=dydt(80)+k(7)*y(62)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(77)=dydt(77)-k(7)*y(77)*y(62);
dydt(62)=dydt(62)-k(7)*y(77)*y(62);
dydt(75)=dydt(75)+k(7)*y(77)*y(62);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(72)=dydt(72)-k(7)*y(72)*y(62);
dydt(62)=dydt(62)-k(7)*y(72)*y(62);
dydt(70)=dydt(70)+k(7)*y(72)*y(62);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Lig(l1, l2) 

dydt(62)=dydt(62)-k(2)*y(62);
dydt(4)=dydt(4)+k(2)*y(62);
dydt(61)=dydt(61)+k(2)*y(62);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(63)=dydt(63)-k(32)*y(63);
dydt(68)=dydt(68)+k(32)*y(63);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(63);
dydt(63)=dydt(63)-k(21)*y(182)*y(63);
dydt(297)=dydt(297)+k(21)*y(182)*y(63);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(63);
dydt(63)=dydt(63)-k(21)*y(197)*y(63);
dydt(295)=dydt(295)+k(21)*y(197)*y(63);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(63);
dydt(63)=dydt(63)-k(21)*y(32)*y(63);
dydt(85)=dydt(85)+k(21)*y(32)*y(63);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(63);
dydt(63)=dydt(63)-k(21)*y(2)*y(63);
dydt(88)=dydt(88)+k(21)*y(2)*y(63);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(63)=dydt(63)-k(18)*y(63);
dydt(75)=dydt(75)+k(18)*y(63);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

dydt(63)=dydt(63)-k(16)*y(63);
dydt(3)=dydt(3)+k(16)*y(63);
dydt(93)=dydt(93)+k(16)*y(63);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(63);
dydt(63)=dydt(63)-k(9)*y(3)*y(63);
dydt(60)=dydt(60)+k(9)*y(3)*y(63);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) 

dydt(63)=dydt(63)-k(8)*y(63);
dydt(15)=dydt(15)+k(8)*y(63);
dydt(62)=dydt(62)+k(8)*y(63);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(63)=dydt(63)-k(6)*y(63);
dydt(98)=dydt(98)+k(6)*y(63);
dydt(61)=dydt(61)+k(6)*y(63);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(64)=dydt(64)-k(32)*y(64);
dydt(13)=dydt(13)+k(32)*y(64);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(64)=dydt(64)-k(31)*y(64);
dydt(99)=dydt(99)+k(31)*y(64);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(64);
dydt(64)=dydt(64)-k(21)*y(182)*y(64);
dydt(419)=dydt(419)+k(21)*y(182)*y(64);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(64);
dydt(64)=dydt(64)-k(21)*y(197)*y(64);
dydt(415)=dydt(415)+k(21)*y(197)*y(64);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(64);
dydt(64)=dydt(64)-k(21)*y(32)*y(64);
dydt(650)=dydt(650)+k(21)*y(32)*y(64);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(64);
dydt(64)=dydt(64)-k(21)*y(2)*y(64);
dydt(92)=dydt(92)+k(21)*y(2)*y(64);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(64);
dydt(64)=dydt(64)-k(15)*y(3)*y(64);
dydt(60)=dydt(60)+k(15)*y(3)*y(64);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(64)=dydt(64)-k(13)*y(64);
dydt(57)=dydt(57)+k(13)*y(64);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

dydt(64)=dydt(64)-k(10)*y(64);
dydt(3)=dydt(3)+k(10)*y(64);
dydt(93)=dydt(93)+k(10)*y(64);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(64)=dydt(64)-k(8)*y(64);
dydt(91)=dydt(91)+k(8)*y(64);
dydt(48)=dydt(48)+k(8)*y(64);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) 

dydt(64)=dydt(64)-k(6)*y(64);
dydt(72)=dydt(72)+k(6)*y(64);
dydt(47)=dydt(47)+k(6)*y(64);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(65)=dydt(65)-k(26)*y(65);
dydt(66)=dydt(66)+k(26)*y(65);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(65)=dydt(65)-k(22)*y(65);
dydt(2)=dydt(2)+k(22)*y(65);
dydt(60)=dydt(60)+k(22)*y(65);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(65)=dydt(65)-k(16)*y(65);
dydt(3)=dydt(3)+k(16)*y(65);
dydt(92)=dydt(92)+k(16)*y(65);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(65)=dydt(65)-k(13)*y(65);
dydt(58)=dydt(58)+k(13)*y(65);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(65)=dydt(65)-k(10)*y(65);
dydt(3)=dydt(3)+k(10)*y(65);
dydt(88)=dydt(88)+k(10)*y(65);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(65)=dydt(65)-k(8)*y(65);
dydt(62)=dydt(62)+k(8)*y(65);
dydt(140)=dydt(140)+k(8)*y(65);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(65)=dydt(65)-k(6)*y(65);
dydt(61)=dydt(61)+k(6)*y(65);
dydt(380)=dydt(380)+k(6)*y(65);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(66)=dydt(66)-k(33)*y(66);
dydt(65)=dydt(65)+k(33)*y(66);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(66)=dydt(66)-k(22)*y(66);
dydt(32)=dydt(32)+k(22)*y(66);
dydt(60)=dydt(60)+k(22)*y(66);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(66)=dydt(66)-k(16)*y(66);
dydt(3)=dydt(3)+k(16)*y(66);
dydt(650)=dydt(650)+k(16)*y(66);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(66)=dydt(66)-k(13)*y(66);
dydt(59)=dydt(59)+k(13)*y(66);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(66)=dydt(66)-k(10)*y(66);
dydt(3)=dydt(3)+k(10)*y(66);
dydt(85)=dydt(85)+k(10)*y(66);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(66)=dydt(66)-k(8)*y(66);
dydt(62)=dydt(62)+k(8)*y(66);
dydt(131)=dydt(131)+k(8)*y(66);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(66)=dydt(66)-k(6)*y(66);
dydt(61)=dydt(61)+k(6)*y(66);
dydt(441)=dydt(441)+k(6)*y(66);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(69)=dydt(69)-k(31)*y(69);
dydt(12)=dydt(12)+k(31)*y(69);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(69);
dydt(69)=dydt(69)-k(15)*y(3)*y(69);
dydt(68)=dydt(68)+k(15)*y(3)*y(69);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(69);
dydt(69)=dydt(69)-k(9)*y(3)*y(69);
dydt(13)=dydt(13)+k(9)*y(3)*y(69);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) 

dydt(69)=dydt(69)-k(8)*y(69);
dydt(1)=dydt(1)+k(8)*y(69);
dydt(91)=dydt(91)+k(8)*y(69);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) 

dydt(69)=dydt(69)-k(6)*y(69);
dydt(72)=dydt(72)+k(6)*y(69);
dydt(6)=dydt(6)+k(6)*y(69);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(71)=dydt(71)-k(31)*y(71);
dydt(6)=dydt(6)+k(31)*y(71);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(71);
dydt(71)=dydt(71)-k(15)*y(3)*y(71);
dydt(122)=dydt(122)+k(15)*y(3)*y(71);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(1)=dydt(1)-k(5)*y(1)*y(71);
dydt(71)=dydt(71)-k(5)*y(1)*y(71);
dydt(333)=dydt(333)+k(5)*y(1)*y(71);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(71)=dydt(71)-k(5)*y(71)*y(7);
dydt(7)=dydt(7)-k(5)*y(71)*y(7);
dydt(124)=dydt(124)+k(5)*y(71)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(71)=dydt(71)-k(5)*y(71)*y(335);
dydt(335)=dydt(335)-k(5)*y(71)*y(335);
dydt(334)=dydt(334)+k(5)*y(71)*y(335);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(71)=dydt(71)-k(5)*y(71)*y(257);
dydt(257)=dydt(257)-k(5)*y(71)*y(257);
dydt(321)=dydt(321)+k(5)*y(71)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(71)=dydt(71)-k(5)*y(71)*y(227);
dydt(227)=dydt(227)-k(5)*y(71)*y(227);
dydt(231)=dydt(231)+k(5)*y(71)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(71)=dydt(71)-k(5)*y(71)*y(222);
dydt(222)=dydt(222)-k(5)*y(71)*y(222);
dydt(646)=dydt(646)+k(5)*y(71)*y(222);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(71)=dydt(71)-k(5)*y(71)*y(218);
dydt(218)=dydt(218)-k(5)*y(71)*y(218);
dydt(325)=dydt(325)+k(5)*y(71)*y(218);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(71)=dydt(71)-k(5)*y(71)*y(192);
dydt(192)=dydt(192)-k(5)*y(71)*y(192);
dydt(332)=dydt(332)+k(5)*y(71)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(71)=dydt(71)-k(5)*y(71)*y(184);
dydt(184)=dydt(184)-k(5)*y(71)*y(184);
dydt(330)=dydt(330)+k(5)*y(71)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(71)=dydt(71)-k(5)*y(71)*y(177);
dydt(177)=dydt(177)-k(5)*y(71)*y(177);
dydt(328)=dydt(328)+k(5)*y(71)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(71)=dydt(71)-k(5)*y(71)*y(140);
dydt(140)=dydt(140)-k(5)*y(71)*y(140);
dydt(139)=dydt(139)+k(5)*y(71)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(71)=dydt(71)-k(5)*y(71)*y(131);
dydt(131)=dydt(131)-k(5)*y(71)*y(131);
dydt(135)=dydt(135)+k(5)*y(71)*y(131);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(71)=dydt(71)-k(5)*y(71)*y(111);
dydt(111)=dydt(111)-k(5)*y(71)*y(111);
dydt(653)=dydt(653)+k(5)*y(71)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(77)=dydt(77)-k(5)*y(77)*y(71);
dydt(71)=dydt(71)-k(5)*y(77)*y(71);
dydt(237)=dydt(237)+k(5)*y(77)*y(71);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(72)=dydt(72)-k(5)*y(72)*y(71);
dydt(71)=dydt(71)-k(5)*y(72)*y(71);
dydt(74)=dydt(74)+k(5)*y(72)*y(71);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a, b~pY, g~Y) + Lig(l1, l2) 

dydt(71)=dydt(71)-k(4)*y(71);
dydt(4)=dydt(4)+k(4)*y(71);
dydt(72)=dydt(72)+k(4)*y(71);

% rule    : R14
% reaction: Rec(a, b~pY, g~Y) -> Rec(a, b~Y, g~Y) 

dydt(72)=dydt(72)-k(31)*y(72);
dydt(1)=dydt(1)+k(31)*y(72);

% rule    : R6
% reaction: Rec(a, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(3)=dydt(3)-k(15)*y(3)*y(72);
dydt(72)=dydt(72)-k(15)*y(3)*y(72);
dydt(61)=dydt(61)+k(15)*y(3)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(91);
dydt(91)=dydt(91)-k(7)*y(72)*y(91);
dydt(74)=dydt(74)+k(7)*y(72)*y(91);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(72)=dydt(72)-k(7)*y(72)*y(126);
dydt(126)=dydt(126)-k(7)*y(72)*y(126);
dydt(128)=dydt(128)+k(7)*y(72)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(132);
dydt(132)=dydt(132)-k(7)*y(72)*y(132);
dydt(135)=dydt(135)+k(7)*y(72)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(141);
dydt(141)=dydt(141)-k(7)*y(72)*y(141);
dydt(139)=dydt(139)+k(7)*y(72)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(72)=dydt(72)-k(7)*y(72)*y(185);
dydt(185)=dydt(185)-k(7)*y(72)*y(185);
dydt(330)=dydt(330)+k(7)*y(72)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(72)=dydt(72)-k(7)*y(72)*y(209);
dydt(209)=dydt(209)-k(7)*y(72)*y(209);
dydt(332)=dydt(332)+k(7)*y(72)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(72)=dydt(72)-k(7)*y(72)*y(214);
dydt(214)=dydt(214)-k(7)*y(72)*y(214);
dydt(328)=dydt(328)+k(7)*y(72)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(72)=dydt(72)-k(7)*y(72)*y(219);
dydt(219)=dydt(219)-k(7)*y(72)*y(219);
dydt(325)=dydt(325)+k(7)*y(72)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(72)=dydt(72)-k(7)*y(72)*y(223);
dydt(223)=dydt(223)-k(7)*y(72)*y(223);
dydt(646)=dydt(646)+k(7)*y(72)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(72)=dydt(72)-k(7)*y(72)*y(228);
dydt(228)=dydt(228)-k(7)*y(72)*y(228);
dydt(231)=dydt(231)+k(7)*y(72)*y(228);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(315);
dydt(315)=dydt(315)-k(7)*y(72)*y(315);
dydt(237)=dydt(237)+k(7)*y(72)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(72)=dydt(72)-k(7)*y(72)*y(319);
dydt(319)=dydt(319)-k(7)*y(72)*y(319);
dydt(321)=dydt(321)+k(7)*y(72)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(72)=dydt(72)-k(7)*y(72)*y(336);
dydt(336)=dydt(336)-k(7)*y(72)*y(336);
dydt(334)=dydt(334)+k(7)*y(72)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(72)=dydt(72)-k(7)*y(72)*y(437);
dydt(437)=dydt(437)-k(7)*y(72)*y(437);
dydt(653)=dydt(653)+k(7)*y(72)*y(437);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(72)=dydt(72)-k(5)*y(72)*y(76);
dydt(76)=dydt(76)-k(5)*y(72)*y(76);
dydt(79)=dydt(79)+k(5)*y(72)*y(76);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(72)=dydt(72)-k(5)*y(72)*y(82);
dydt(82)=dydt(82)-k(5)*y(72)*y(82);
dydt(84)=dydt(84)+k(5)*y(72)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(72)=dydt(72)-k(5)*y(72)*y(86);
dydt(86)=dydt(86)-k(5)*y(72)*y(86);
dydt(87)=dydt(87)+k(5)*y(72)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(72)=dydt(72)-k(5)*y(72)*y(89);
dydt(89)=dydt(89)-k(5)*y(72)*y(89);
dydt(90)=dydt(90)+k(5)*y(72)*y(89);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(72)=dydt(72)-k(5)*y(72)*y(98);
dydt(98)=dydt(98)-k(5)*y(72)*y(98);
dydt(93)=dydt(93)+k(5)*y(72)*y(98);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(72)=dydt(72)-k(5)*y(72)*y(110);
dydt(110)=dydt(110)-k(5)*y(72)*y(110);
dydt(305)=dydt(305)+k(5)*y(72)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(72)=dydt(72)-k(5)*y(72)*y(114);
dydt(114)=dydt(114)-k(5)*y(72)*y(114);
dydt(306)=dydt(306)+k(5)*y(72)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(72)=dydt(72)-k(5)*y(72)*y(118);
dydt(118)=dydt(118)-k(5)*y(72)*y(118);
dydt(309)=dydt(309)+k(5)*y(72)*y(118);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(72)=dydt(72)-k(5)*y(72)*y(122);
dydt(122)=dydt(122)-k(5)*y(72)*y(122);
dydt(313)=dydt(313)+k(5)*y(72)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(72)=dydt(72)-k(5)*y(72)*y(176);
dydt(176)=dydt(176)-k(5)*y(72)*y(176);
dydt(299)=dydt(299)+k(5)*y(72)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(72)=dydt(72)-k(5)*y(72)*y(191);
dydt(191)=dydt(191)-k(5)*y(72)*y(191);
dydt(296)=dydt(296)+k(5)*y(72)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(72)=dydt(72)-k(5)*y(72)*y(256);
dydt(256)=dydt(256)-k(5)*y(72)*y(256);
dydt(586)=dydt(586)+k(5)*y(72)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(72)=dydt(72)-k(5)*y(72)*y(264);
dydt(264)=dydt(264)-k(5)*y(72)*y(264);
dydt(302)=dydt(302)+k(5)*y(72)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(72)=dydt(72)-k(5)*y(72)*y(380);
dydt(380)=dydt(380)-k(5)*y(72)*y(380);
dydt(92)=dydt(92)+k(5)*y(72)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(72)=dydt(72)-k(5)*y(72)*y(391);
dydt(391)=dydt(391)-k(5)*y(72)*y(391);
dydt(419)=dydt(419)+k(5)*y(72)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(72)=dydt(72)-k(5)*y(72)*y(407);
dydt(407)=dydt(407)-k(5)*y(72)*y(407);
dydt(415)=dydt(415)+k(5)*y(72)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(72)=dydt(72)-k(5)*y(72)*y(441);
dydt(441)=dydt(441)-k(5)*y(72)*y(441);
dydt(650)=dydt(650)+k(5)*y(72)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(72)=dydt(72)-k(5)*y(72)*y(515);
dydt(515)=dydt(515)-k(5)*y(72)*y(515);
dydt(303)=dydt(303)+k(5)*y(72)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(72)=dydt(72)-k(5)*y(72)*y(543);
dydt(543)=dydt(543)-k(5)*y(72)*y(543);
dydt(312)=dydt(312)+k(5)*y(72)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(72)=dydt(72)-k(5)*y(72)*y(554);
dydt(554)=dydt(554)-k(5)*y(72)*y(554);
dydt(311)=dydt(311)+k(5)*y(72)*y(554);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(72)=dydt(72)-k(5)*y(72)*y(9);
dydt(9)=dydt(9)-k(5)*y(72)*y(9);
dydt(13)=dydt(13)+k(5)*y(72)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(4)=dydt(4)-k(3)*y(4)*y(72);
dydt(72)=dydt(72)-k(3)*y(4)*y(72);
dydt(71)=dydt(71)+k(3)*y(4)*y(72);

% rule    : R1
% reaction: Rec(a, b~pY, g~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(4)=dydt(4)-k(1)*y(4)*y(72);
dydt(72)=dydt(72)-k(1)*y(4)*y(72);
dydt(91)=dydt(91)+k(1)*y(4)*y(72);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(73)=dydt(73)-k(20)*y(73);
dydt(78)=dydt(78)+k(20)*y(73);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(73)=dydt(73)-k(19)*y(73);
dydt(236)=dydt(236)+k(19)*y(73);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) 

dydt(73)=dydt(73)-k(16)*y(73);
dydt(3)=dydt(3)+k(16)*y(73);
dydt(70)=dydt(70)+k(16)*y(73);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) 

dydt(73)=dydt(73)-k(16)*y(73);
dydt(3)=dydt(3)+k(16)*y(73);
dydt(313)=dydt(313)+k(16)*y(73);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(73)=dydt(73)-k(8)*y(73);
dydt(61)=dydt(61)+k(8)*y(73);
dydt(62)=dydt(62)+k(8)*y(73);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(73)=dydt(73)-k(6)*y(73);
dydt(61)=dydt(61)+k(6)*y(73);
dydt(122)=dydt(122)+k(6)*y(73);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(74)=dydt(74)-k(31)*y(74);
dydt(69)=dydt(69)+k(31)*y(74);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(74)=dydt(74)-k(31)*y(74);
dydt(333)=dydt(333)+k(31)*y(74);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(74);
dydt(74)=dydt(74)-k(15)*y(3)*y(74);
dydt(70)=dydt(70)+k(15)*y(3)*y(74);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(3)=dydt(3)-k(15)*y(3)*y(74);
dydt(74)=dydt(74)-k(15)*y(3)*y(74);
dydt(313)=dydt(313)+k(15)*y(3)*y(74);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) 

dydt(74)=dydt(74)-k(8)*y(74);
dydt(72)=dydt(72)+k(8)*y(74);
dydt(91)=dydt(91)+k(8)*y(74);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) 

dydt(74)=dydt(74)-k(6)*y(74);
dydt(72)=dydt(72)+k(6)*y(74);
dydt(71)=dydt(71)+k(6)*y(74);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(76)=dydt(76)-k(32)*y(76);
dydt(71)=dydt(71)+k(32)*y(76);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(76)=dydt(76)-k(31)*y(76);
dydt(98)=dydt(98)+k(31)*y(76);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(76);
dydt(76)=dydt(76)-k(21)*y(182)*y(76);
dydt(264)=dydt(264)+k(21)*y(182)*y(76);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(76);
dydt(76)=dydt(76)-k(21)*y(197)*y(76);
dydt(256)=dydt(256)+k(21)*y(197)*y(76);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(76);
dydt(76)=dydt(76)-k(21)*y(32)*y(76);
dydt(82)=dydt(82)+k(21)*y(32)*y(76);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(76);
dydt(76)=dydt(76)-k(21)*y(2)*y(76);
dydt(110)=dydt(110)+k(21)*y(2)*y(76);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(76);
dydt(76)=dydt(76)-k(15)*y(3)*y(76);
dydt(118)=dydt(118)+k(15)*y(3)*y(76);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(1)=dydt(1)-k(5)*y(1)*y(76);
dydt(76)=dydt(76)-k(5)*y(1)*y(76);
dydt(590)=dydt(590)+k(5)*y(1)*y(76);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(76)=dydt(76)-k(5)*y(76)*y(7);
dydt(7)=dydt(7)-k(5)*y(76)*y(7);
dydt(108)=dydt(108)+k(5)*y(76)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(76)=dydt(76)-k(5)*y(76)*y(335);
dydt(335)=dydt(335)-k(5)*y(76)*y(335);
dydt(337)=dydt(337)+k(5)*y(76)*y(335);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(76)=dydt(76)-k(5)*y(76)*y(257);
dydt(257)=dydt(257)-k(5)*y(76)*y(257);
dydt(613)=dydt(613)+k(5)*y(76)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(76)=dydt(76)-k(5)*y(76)*y(227);
dydt(227)=dydt(227)-k(5)*y(76)*y(227);
dydt(491)=dydt(491)+k(5)*y(76)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(76)=dydt(76)-k(5)*y(76)*y(222);
dydt(222)=dydt(222)-k(5)*y(76)*y(222);
dydt(645)=dydt(645)+k(5)*y(76)*y(222);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(76)=dydt(76)-k(5)*y(76)*y(218);
dydt(218)=dydt(218)-k(5)*y(76)*y(218);
dydt(576)=dydt(576)+k(5)*y(76)*y(218);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) 

dydt(76)=dydt(76)-k(5)*y(76)*y(192);
dydt(192)=dydt(192)-k(5)*y(76)*y(192);
dydt(338)=dydt(338)+k(5)*y(76)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(76)=dydt(76)-k(5)*y(76)*y(184);
dydt(184)=dydt(184)-k(5)*y(76)*y(184);
dydt(343)=dydt(343)+k(5)*y(76)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(76)=dydt(76)-k(5)*y(76)*y(177);
dydt(177)=dydt(177)-k(5)*y(76)*y(177);
dydt(344)=dydt(344)+k(5)*y(76)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(76)=dydt(76)-k(5)*y(76)*y(140);
dydt(140)=dydt(140)-k(5)*y(76)*y(140);
dydt(143)=dydt(143)+k(5)*y(76)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(76)=dydt(76)-k(5)*y(76)*y(131);
dydt(131)=dydt(131)-k(5)*y(76)*y(131);
dydt(137)=dydt(137)+k(5)*y(76)*y(131);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(76)=dydt(76)-k(5)*y(76)*y(111);
dydt(111)=dydt(111)-k(5)*y(76)*y(111);
dydt(252)=dydt(252)+k(5)*y(76)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(77)=dydt(77)-k(5)*y(77)*y(76);
dydt(76)=dydt(76)-k(5)*y(77)*y(76);
dydt(584)=dydt(584)+k(5)*y(77)*y(76);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a, b~pY, g~pY) + Lig(l1, l2) 

dydt(76)=dydt(76)-k(4)*y(76);
dydt(4)=dydt(4)+k(4)*y(76);
dydt(77)=dydt(77)+k(4)*y(76);

% rule    : R15
% reaction: Rec(a, b~pY, g~pY) -> Rec(a, b~pY, g~Y) 

dydt(77)=dydt(77)-k(32)*y(77);
dydt(72)=dydt(72)+k(32)*y(77);

% rule    : R14
% reaction: Rec(a, b~pY, g~pY) -> Rec(a, b~Y, g~pY) 

dydt(77)=dydt(77)-k(31)*y(77);
dydt(15)=dydt(15)+k(31)*y(77);

% rule    : R9
% reaction: Rec(a, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(77);
dydt(77)=dydt(77)-k(21)*y(182)*y(77);
dydt(218)=dydt(218)+k(21)*y(182)*y(77);

% rule    : R9
% reaction: Rec(a, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(77);
dydt(77)=dydt(77)-k(21)*y(197)*y(77);
dydt(257)=dydt(257)+k(21)*y(197)*y(77);

% rule    : R9
% reaction: Rec(a, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(77);
dydt(77)=dydt(77)-k(21)*y(32)*y(77);
dydt(33)=dydt(33)+k(21)*y(32)*y(77);

% rule    : R9
% reaction: Rec(a, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(77);
dydt(77)=dydt(77)-k(21)*y(2)*y(77);
dydt(111)=dydt(111)+k(21)*y(2)*y(77);

% rule    : R6
% reaction: Rec(a, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(3)=dydt(3)-k(15)*y(3)*y(77);
dydt(77)=dydt(77)-k(15)*y(3)*y(77);
dydt(54)=dydt(54)+k(15)*y(3)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(91);
dydt(91)=dydt(91)-k(7)*y(77)*y(91);
dydt(79)=dydt(79)+k(7)*y(77)*y(91);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(77)=dydt(77)-k(7)*y(77)*y(126);
dydt(126)=dydt(126)-k(7)*y(77)*y(126);
dydt(151)=dydt(151)+k(7)*y(77)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(77)=dydt(77)-k(7)*y(77)*y(132);
dydt(132)=dydt(132)-k(7)*y(77)*y(132);
dydt(137)=dydt(137)+k(7)*y(77)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(77)=dydt(77)-k(7)*y(77)*y(141);
dydt(141)=dydt(141)-k(7)*y(77)*y(141);
dydt(143)=dydt(143)+k(7)*y(77)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(185);
dydt(185)=dydt(185)-k(7)*y(77)*y(185);
dydt(343)=dydt(343)+k(7)*y(77)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(209);
dydt(209)=dydt(209)-k(7)*y(77)*y(209);
dydt(338)=dydt(338)+k(7)*y(77)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(214);
dydt(214)=dydt(214)-k(7)*y(77)*y(214);
dydt(344)=dydt(344)+k(7)*y(77)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(219);
dydt(219)=dydt(219)-k(7)*y(77)*y(219);
dydt(576)=dydt(576)+k(7)*y(77)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(223);
dydt(223)=dydt(223)-k(7)*y(77)*y(223);
dydt(645)=dydt(645)+k(7)*y(77)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(228);
dydt(228)=dydt(228)-k(7)*y(77)*y(228);
dydt(491)=dydt(491)+k(7)*y(77)*y(228);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(315);
dydt(315)=dydt(315)-k(7)*y(77)*y(315);
dydt(584)=dydt(584)+k(7)*y(77)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(319);
dydt(319)=dydt(319)-k(7)*y(77)*y(319);
dydt(613)=dydt(613)+k(7)*y(77)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(77)=dydt(77)-k(7)*y(77)*y(336);
dydt(336)=dydt(336)-k(7)*y(77)*y(336);
dydt(337)=dydt(337)+k(7)*y(77)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(77)=dydt(77)-k(7)*y(77)*y(437);
dydt(437)=dydt(437)-k(7)*y(77)*y(437);
dydt(252)=dydt(252)+k(7)*y(77)*y(437);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(77)=dydt(77)-k(5)*y(77)*y(82);
dydt(82)=dydt(82)-k(5)*y(77)*y(82);
dydt(573)=dydt(573)+k(5)*y(77)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(77)=dydt(77)-k(5)*y(77)*y(86);
dydt(86)=dydt(86)-k(5)*y(77)*y(86);
dydt(615)=dydt(615)+k(5)*y(77)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(77)=dydt(77)-k(5)*y(77)*y(89);
dydt(89)=dydt(89)-k(5)*y(77)*y(89);
dydt(425)=dydt(425)+k(5)*y(77)*y(89);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(77)=dydt(77)-k(5)*y(77)*y(98);
dydt(98)=dydt(98)-k(5)*y(77)*y(98);
dydt(614)=dydt(614)+k(5)*y(77)*y(98);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(77)=dydt(77)-k(5)*y(77)*y(110);
dydt(110)=dydt(110)-k(5)*y(77)*y(110);
dydt(501)=dydt(501)+k(5)*y(77)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(77)=dydt(77)-k(5)*y(77)*y(114);
dydt(114)=dydt(114)-k(5)*y(77)*y(114);
dydt(308)=dydt(308)+k(5)*y(77)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(77)=dydt(77)-k(5)*y(77)*y(118);
dydt(118)=dydt(118)-k(5)*y(77)*y(118);
dydt(310)=dydt(310)+k(5)*y(77)*y(118);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(77)=dydt(77)-k(5)*y(77)*y(122);
dydt(122)=dydt(122)-k(5)*y(77)*y(122);
dydt(314)=dydt(314)+k(5)*y(77)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(77)=dydt(77)-k(5)*y(77)*y(176);
dydt(176)=dydt(176)-k(5)*y(77)*y(176);
dydt(421)=dydt(421)+k(5)*y(77)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(77)=dydt(77)-k(5)*y(77)*y(191);
dydt(191)=dydt(191)-k(5)*y(77)*y(191);
dydt(292)=dydt(292)+k(5)*y(77)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(77)=dydt(77)-k(5)*y(77)*y(256);
dydt(256)=dydt(256)-k(5)*y(77)*y(256);
dydt(578)=dydt(578)+k(5)*y(77)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(77)=dydt(77)-k(5)*y(77)*y(264);
dydt(264)=dydt(264)-k(5)*y(77)*y(264);
dydt(497)=dydt(497)+k(5)*y(77)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(77)=dydt(77)-k(5)*y(77)*y(380);
dydt(380)=dydt(380)-k(5)*y(77)*y(380);
dydt(424)=dydt(424)+k(5)*y(77)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(77)=dydt(77)-k(5)*y(77)*y(391);
dydt(391)=dydt(391)-k(5)*y(77)*y(391);
dydt(420)=dydt(420)+k(5)*y(77)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(77)=dydt(77)-k(5)*y(77)*y(407);
dydt(407)=dydt(407)-k(5)*y(77)*y(407);
dydt(410)=dydt(410)+k(5)*y(77)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(77)=dydt(77)-k(5)*y(77)*y(441);
dydt(441)=dydt(441)-k(5)*y(77)*y(441);
dydt(443)=dydt(443)+k(5)*y(77)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(77)=dydt(77)-k(5)*y(77)*y(515);
dydt(515)=dydt(515)-k(5)*y(77)*y(515);
dydt(623)=dydt(623)+k(5)*y(77)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(77)=dydt(77)-k(5)*y(77)*y(543);
dydt(543)=dydt(543)-k(5)*y(77)*y(543);
dydt(581)=dydt(581)+k(5)*y(77)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(77)=dydt(77)-k(5)*y(77)*y(554);
dydt(554)=dydt(554)-k(5)*y(77)*y(554);
dydt(563)=dydt(563)+k(5)*y(77)*y(554);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(77)=dydt(77)-k(5)*y(77)*y(9);
dydt(9)=dydt(9)-k(5)*y(77)*y(9);
dydt(19)=dydt(19)+k(5)*y(77)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(4)=dydt(4)-k(3)*y(4)*y(77);
dydt(77)=dydt(77)-k(3)*y(4)*y(77);
dydt(76)=dydt(76)+k(3)*y(4)*y(77);

% rule    : R1
% reaction: Rec(a, b~pY, g~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(4)=dydt(4)-k(1)*y(4)*y(77);
dydt(77)=dydt(77)-k(1)*y(4)*y(77);
dydt(315)=dydt(315)+k(1)*y(4)*y(77);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(78)=dydt(78)-k(32)*y(78);
dydt(73)=dydt(73)+k(32)*y(78);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(182)=dydt(182)-k(21)*y(182)*y(78);
dydt(78)=dydt(78)-k(21)*y(182)*y(78);
dydt(301)=dydt(301)+k(21)*y(182)*y(78);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(197)=dydt(197)-k(21)*y(197)*y(78);
dydt(78)=dydt(78)-k(21)*y(197)*y(78);
dydt(647)=dydt(647)+k(21)*y(197)*y(78);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(32)=dydt(32)-k(21)*y(32)*y(78);
dydt(78)=dydt(78)-k(21)*y(32)*y(78);
dydt(83)=dydt(83)+k(21)*y(32)*y(78);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(78);
dydt(78)=dydt(78)-k(21)*y(2)*y(78);
dydt(307)=dydt(307)+k(21)*y(2)*y(78);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(78)=dydt(78)-k(19)*y(78);
dydt(627)=dydt(627)+k(19)*y(78);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) 

dydt(78)=dydt(78)-k(16)*y(78);
dydt(3)=dydt(3)+k(16)*y(78);
dydt(75)=dydt(75)+k(16)*y(78);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) 

dydt(78)=dydt(78)-k(16)*y(78);
dydt(3)=dydt(3)+k(16)*y(78);
dydt(309)=dydt(309)+k(16)*y(78);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(78)=dydt(78)-k(8)*y(78);
dydt(54)=dydt(54)+k(8)*y(78);
dydt(62)=dydt(62)+k(8)*y(78);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(78)=dydt(78)-k(6)*y(78);
dydt(61)=dydt(61)+k(6)*y(78);
dydt(118)=dydt(118)+k(6)*y(78);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(79)=dydt(79)-k(32)*y(79);
dydt(74)=dydt(74)+k(32)*y(79);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(79)=dydt(79)-k(31)*y(79);
dydt(93)=dydt(93)+k(31)*y(79);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(79)=dydt(79)-k(31)*y(79);
dydt(590)=dydt(590)+k(31)*y(79);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(79);
dydt(79)=dydt(79)-k(21)*y(182)*y(79);
dydt(302)=dydt(302)+k(21)*y(182)*y(79);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(79);
dydt(79)=dydt(79)-k(21)*y(197)*y(79);
dydt(586)=dydt(586)+k(21)*y(197)*y(79);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(79);
dydt(79)=dydt(79)-k(21)*y(32)*y(79);
dydt(84)=dydt(84)+k(21)*y(32)*y(79);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(79);
dydt(79)=dydt(79)-k(21)*y(2)*y(79);
dydt(305)=dydt(305)+k(21)*y(2)*y(79);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(79);
dydt(79)=dydt(79)-k(15)*y(3)*y(79);
dydt(75)=dydt(75)+k(15)*y(3)*y(79);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(3)=dydt(3)-k(15)*y(3)*y(79);
dydt(79)=dydt(79)-k(15)*y(3)*y(79);
dydt(309)=dydt(309)+k(15)*y(3)*y(79);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) 

dydt(79)=dydt(79)-k(8)*y(79);
dydt(77)=dydt(77)+k(8)*y(79);
dydt(91)=dydt(91)+k(8)*y(79);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) 

dydt(79)=dydt(79)-k(6)*y(79);
dydt(72)=dydt(72)+k(6)*y(79);
dydt(76)=dydt(76)+k(6)*y(79);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(80)=dydt(80)-k(31)*y(80);
dydt(88)=dydt(88)+k(31)*y(80);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(80)=dydt(80)-k(26)*y(80);
dydt(81)=dydt(81)+k(26)*y(80);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

dydt(80)=dydt(80)-k(22)*y(80);
dydt(2)=dydt(2)+k(22)*y(80);
dydt(75)=dydt(75)+k(22)*y(80);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(80)=dydt(80)-k(16)*y(80);
dydt(3)=dydt(3)+k(16)*y(80);
dydt(305)=dydt(305)+k(16)*y(80);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(80);
dydt(80)=dydt(80)-k(15)*y(3)*y(80);
dydt(307)=dydt(307)+k(15)*y(3)*y(80);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(80)=dydt(80)-k(8)*y(80);
dydt(62)=dydt(62)+k(8)*y(80);
dydt(111)=dydt(111)+k(8)*y(80);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(80)=dydt(80)-k(6)*y(80);
dydt(61)=dydt(61)+k(6)*y(80);
dydt(110)=dydt(110)+k(6)*y(80);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(82)=dydt(82)-k(33)*y(82);
dydt(110)=dydt(110)+k(33)*y(82);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(82)=dydt(82)-k(31)*y(82);
dydt(86)=dydt(86)+k(31)*y(82);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~Y) 

dydt(82)=dydt(82)-k(22)*y(82);
dydt(32)=dydt(32)+k(22)*y(82);
dydt(76)=dydt(76)+k(22)*y(82);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(82);
dydt(82)=dydt(82)-k(15)*y(3)*y(82);
dydt(554)=dydt(554)+k(15)*y(3)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(1)=dydt(1)-k(5)*y(1)*y(82);
dydt(82)=dydt(82)-k(5)*y(1)*y(82);
dydt(106)=dydt(106)+k(5)*y(1)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(7)=dydt(7)-k(5)*y(7)*y(82);
dydt(82)=dydt(82)-k(5)*y(7)*y(82);
dydt(105)=dydt(105)+k(5)*y(7)*y(82);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(82)=dydt(82)-k(5)*y(82)*y(335);
dydt(335)=dydt(335)-k(5)*y(82)*y(335);
dydt(341)=dydt(341)+k(5)*y(82)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(257)=dydt(257)-k(5)*y(257)*y(82);
dydt(82)=dydt(82)-k(5)*y(257)*y(82);
dydt(562)=dydt(562)+k(5)*y(257)*y(82);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(82)=dydt(82)-k(5)*y(82)*y(227);
dydt(227)=dydt(227)-k(5)*y(82)*y(227);
dydt(635)=dydt(635)+k(5)*y(82)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(82)=dydt(82)-k(5)*y(82)*y(222);
dydt(222)=dydt(222)-k(5)*y(82)*y(222);
dydt(286)=dydt(286)+k(5)*y(82)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(218)=dydt(218)-k(5)*y(218)*y(82);
dydt(82)=dydt(82)-k(5)*y(218)*y(82);
dydt(574)=dydt(574)+k(5)*y(218)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(192)=dydt(192)-k(5)*y(192)*y(82);
dydt(82)=dydt(82)-k(5)*y(192)*y(82);
dydt(559)=dydt(559)+k(5)*y(192)*y(82);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(82)=dydt(82)-k(5)*y(82)*y(184);
dydt(184)=dydt(184)-k(5)*y(82)*y(184);
dydt(347)=dydt(347)+k(5)*y(82)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(177)=dydt(177)-k(5)*y(177)*y(82);
dydt(82)=dydt(82)-k(5)*y(177)*y(82);
dydt(348)=dydt(348)+k(5)*y(177)*y(82);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(82)=dydt(82)-k(5)*y(82)*y(140);
dydt(140)=dydt(140)-k(5)*y(82)*y(140);
dydt(167)=dydt(167)+k(5)*y(82)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(82)=dydt(82)-k(5)*y(82)*y(131);
dydt(131)=dydt(131)-k(5)*y(82)*y(131);
dydt(161)=dydt(161)+k(5)*y(82)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(111)=dydt(111)-k(5)*y(111)*y(82);
dydt(82)=dydt(82)-k(5)*y(111)*y(82);
dydt(572)=dydt(572)+k(5)*y(111)*y(82);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) 

dydt(82)=dydt(82)-k(4)*y(82);
dydt(4)=dydt(4)+k(4)*y(82);
dydt(33)=dydt(33)+k(4)*y(82);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(83)=dydt(83)-k(33)*y(83);
dydt(307)=dydt(307)+k(33)*y(83);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(83)=dydt(83)-k(22)*y(83);
dydt(32)=dydt(32)+k(22)*y(83);
dydt(78)=dydt(78)+k(22)*y(83);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(83)=dydt(83)-k(19)*y(83);
dydt(633)=dydt(633)+k(19)*y(83);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(83)=dydt(83)-k(16)*y(83);
dydt(3)=dydt(3)+k(16)*y(83);
dydt(81)=dydt(81)+k(16)*y(83);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(83)=dydt(83)-k(16)*y(83);
dydt(3)=dydt(3)+k(16)*y(83);
dydt(311)=dydt(311)+k(16)*y(83);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(83)=dydt(83)-k(8)*y(83);
dydt(62)=dydt(62)+k(8)*y(83);
dydt(37)=dydt(37)+k(8)*y(83);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(83)=dydt(83)-k(6)*y(83);
dydt(61)=dydt(61)+k(6)*y(83);
dydt(554)=dydt(554)+k(6)*y(83);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(84)=dydt(84)-k(33)*y(84);
dydt(305)=dydt(305)+k(33)*y(84);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(84)=dydt(84)-k(31)*y(84);
dydt(87)=dydt(87)+k(31)*y(84);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(84)=dydt(84)-k(31)*y(84);
dydt(106)=dydt(106)+k(31)*y(84);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(84)=dydt(84)-k(22)*y(84);
dydt(32)=dydt(32)+k(22)*y(84);
dydt(79)=dydt(79)+k(22)*y(84);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(84);
dydt(84)=dydt(84)-k(15)*y(3)*y(84);
dydt(81)=dydt(81)+k(15)*y(3)*y(84);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(84);
dydt(84)=dydt(84)-k(15)*y(3)*y(84);
dydt(311)=dydt(311)+k(15)*y(3)*y(84);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(84)=dydt(84)-k(8)*y(84);
dydt(91)=dydt(91)+k(8)*y(84);
dydt(33)=dydt(33)+k(8)*y(84);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

dydt(84)=dydt(84)-k(6)*y(84);
dydt(72)=dydt(72)+k(6)*y(84);
dydt(82)=dydt(82)+k(6)*y(84);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(86)=dydt(86)-k(33)*y(86);
dydt(89)=dydt(89)+k(33)*y(86);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~Y) 

dydt(86)=dydt(86)-k(22)*y(86);
dydt(32)=dydt(32)+k(22)*y(86);
dydt(98)=dydt(98)+k(22)*y(86);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(86);
dydt(86)=dydt(86)-k(9)*y(3)*y(86);
dydt(441)=dydt(441)+k(9)*y(3)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(1)=dydt(1)-k(5)*y(1)*y(86);
dydt(86)=dydt(86)-k(5)*y(1)*y(86);
dydt(101)=dydt(101)+k(5)*y(1)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(7)=dydt(7)-k(5)*y(7)*y(86);
dydt(86)=dydt(86)-k(5)*y(7)*y(86);
dydt(103)=dydt(103)+k(5)*y(7)*y(86);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(86)=dydt(86)-k(5)*y(86)*y(335);
dydt(335)=dydt(335)-k(5)*y(86)*y(335);
dydt(162)=dydt(162)+k(5)*y(86)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(257)=dydt(257)-k(5)*y(257)*y(86);
dydt(86)=dydt(86)-k(5)*y(257)*y(86);
dydt(472)=dydt(472)+k(5)*y(257)*y(86);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(86)=dydt(86)-k(5)*y(86)*y(227);
dydt(227)=dydt(227)-k(5)*y(86)*y(227);
dydt(474)=dydt(474)+k(5)*y(86)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(86)=dydt(86)-k(5)*y(86)*y(222);
dydt(222)=dydt(222)-k(5)*y(86)*y(222);
dydt(283)=dydt(283)+k(5)*y(86)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(218)=dydt(218)-k(5)*y(218)*y(86);
dydt(86)=dydt(86)-k(5)*y(218)*y(86);
dydt(285)=dydt(285)+k(5)*y(218)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(5)*y(192)*y(86);
dydt(86)=dydt(86)-k(5)*y(192)*y(86);
dydt(454)=dydt(454)+k(5)*y(192)*y(86);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(86)=dydt(86)-k(5)*y(86)*y(184);
dydt(184)=dydt(184)-k(5)*y(86)*y(184);
dydt(168)=dydt(168)+k(5)*y(86)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(177)=dydt(177)-k(5)*y(177)*y(86);
dydt(86)=dydt(86)-k(5)*y(177)*y(86);
dydt(449)=dydt(449)+k(5)*y(177)*y(86);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(86)=dydt(86)-k(5)*y(86)*y(140);
dydt(140)=dydt(140)-k(5)*y(86)*y(140);
dydt(164)=dydt(164)+k(5)*y(86)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(86)=dydt(86)-k(5)*y(86)*y(131);
dydt(131)=dydt(131)-k(5)*y(86)*y(131);
dydt(158)=dydt(158)+k(5)*y(86)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(111)=dydt(111)-k(5)*y(111)*y(86);
dydt(86)=dydt(86)-k(5)*y(111)*y(86);
dydt(442)=dydt(442)+k(5)*y(111)*y(86);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) 

dydt(86)=dydt(86)-k(4)*y(86);
dydt(4)=dydt(4)+k(4)*y(86);
dydt(27)=dydt(27)+k(4)*y(86);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(87)=dydt(87)-k(33)*y(87);
dydt(90)=dydt(90)+k(33)*y(87);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(87)=dydt(87)-k(31)*y(87);
dydt(101)=dydt(101)+k(31)*y(87);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(87)=dydt(87)-k(22)*y(87);
dydt(32)=dydt(32)+k(22)*y(87);
dydt(93)=dydt(93)+k(22)*y(87);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(87);
dydt(87)=dydt(87)-k(15)*y(3)*y(87);
dydt(85)=dydt(85)+k(15)*y(3)*y(87);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(87);
dydt(87)=dydt(87)-k(9)*y(3)*y(87);
dydt(650)=dydt(650)+k(9)*y(3)*y(87);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(87)=dydt(87)-k(8)*y(87);
dydt(91)=dydt(91)+k(8)*y(87);
dydt(27)=dydt(27)+k(8)*y(87);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

dydt(87)=dydt(87)-k(6)*y(87);
dydt(72)=dydt(72)+k(6)*y(87);
dydt(86)=dydt(86)+k(6)*y(87);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~Y) 

dydt(89)=dydt(89)-k(22)*y(89);
dydt(2)=dydt(2)+k(22)*y(89);
dydt(98)=dydt(98)+k(22)*y(89);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(89);
dydt(89)=dydt(89)-k(9)*y(3)*y(89);
dydt(380)=dydt(380)+k(9)*y(3)*y(89);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(1)=dydt(1)-k(5)*y(1)*y(89);
dydt(89)=dydt(89)-k(5)*y(1)*y(89);
dydt(94)=dydt(94)+k(5)*y(1)*y(89);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(7)=dydt(7)-k(5)*y(7)*y(89);
dydt(89)=dydt(89)-k(5)*y(7)*y(89);
dydt(96)=dydt(96)+k(5)*y(7)*y(89);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(89)=dydt(89)-k(5)*y(89)*y(335);
dydt(335)=dydt(335)-k(5)*y(89)*y(335);
dydt(371)=dydt(371)+k(5)*y(89)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(257)=dydt(257)-k(5)*y(257)*y(89);
dydt(89)=dydt(89)-k(5)*y(257)*y(89);
dydt(427)=dydt(427)+k(5)*y(257)*y(89);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(89)=dydt(89)-k(5)*y(89)*y(227);
dydt(227)=dydt(227)-k(5)*y(89)*y(227);
dydt(484)=dydt(484)+k(5)*y(89)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(89)=dydt(89)-k(5)*y(89)*y(222);
dydt(222)=dydt(222)-k(5)*y(89)*y(222);
dydt(278)=dydt(278)+k(5)*y(89)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(218)=dydt(218)-k(5)*y(218)*y(89);
dydt(89)=dydt(89)-k(5)*y(218)*y(89);
dydt(280)=dydt(280)+k(5)*y(218)*y(89);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(192)=dydt(192)-k(5)*y(192)*y(89);
dydt(89)=dydt(89)-k(5)*y(192)*y(89);
dydt(373)=dydt(373)+k(5)*y(192)*y(89);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(89)=dydt(89)-k(5)*y(89)*y(184);
dydt(184)=dydt(184)-k(5)*y(89)*y(184);
dydt(174)=dydt(174)+k(5)*y(89)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(177)=dydt(177)-k(5)*y(177)*y(89);
dydt(89)=dydt(89)-k(5)*y(177)*y(89);
dydt(377)=dydt(377)+k(5)*y(177)*y(89);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(89)=dydt(89)-k(5)*y(89)*y(140);
dydt(140)=dydt(140)-k(5)*y(89)*y(140);
dydt(170)=dydt(170)+k(5)*y(89)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(89)=dydt(89)-k(5)*y(89)*y(131);
dydt(131)=dydt(131)-k(5)*y(89)*y(131);
dydt(157)=dydt(157)+k(5)*y(89)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(111)=dydt(111)-k(5)*y(111)*y(89);
dydt(89)=dydt(89)-k(5)*y(111)*y(89);
dydt(275)=dydt(275)+k(5)*y(111)*y(89);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) 

dydt(89)=dydt(89)-k(4)*y(89);
dydt(4)=dydt(4)+k(4)*y(89);
dydt(21)=dydt(21)+k(4)*y(89);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(91)=dydt(91)-k(31)*y(91);
dydt(5)=dydt(5)+k(31)*y(91);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(91);
dydt(91)=dydt(91)-k(15)*y(3)*y(91);
dydt(62)=dydt(62)+k(15)*y(3)*y(91);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(91);
dydt(91)=dydt(91)-k(7)*y(1)*y(91);
dydt(69)=dydt(69)+k(7)*y(1)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(91)=dydt(91)-k(7)*y(91)*y(7);
dydt(7)=dydt(7)-k(7)*y(91)*y(7);
dydt(13)=dydt(13)+k(7)*y(91)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(91)=dydt(91)-k(7)*y(91)*y(335);
dydt(335)=dydt(335)-k(7)*y(91)*y(335);
dydt(415)=dydt(415)+k(7)*y(91)*y(335);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(91)=dydt(91)-k(7)*y(91)*y(257);
dydt(257)=dydt(257)-k(7)*y(91)*y(257);
dydt(586)=dydt(586)+k(7)*y(91)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(91)=dydt(91)-k(7)*y(91)*y(227);
dydt(227)=dydt(227)-k(7)*y(91)*y(227);
dydt(312)=dydt(312)+k(7)*y(91)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(91)=dydt(91)-k(7)*y(91)*y(222);
dydt(222)=dydt(222)-k(7)*y(91)*y(222);
dydt(303)=dydt(303)+k(7)*y(91)*y(222);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(91)=dydt(91)-k(7)*y(91)*y(218);
dydt(218)=dydt(218)-k(7)*y(91)*y(218);
dydt(302)=dydt(302)+k(7)*y(91)*y(218);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(91)=dydt(91)-k(7)*y(91)*y(192);
dydt(192)=dydt(192)-k(7)*y(91)*y(192);
dydt(296)=dydt(296)+k(7)*y(91)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(91)=dydt(91)-k(7)*y(91)*y(184);
dydt(184)=dydt(184)-k(7)*y(91)*y(184);
dydt(419)=dydt(419)+k(7)*y(91)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(91)=dydt(91)-k(7)*y(91)*y(177);
dydt(177)=dydt(177)-k(7)*y(91)*y(177);
dydt(299)=dydt(299)+k(7)*y(91)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(91)=dydt(91)-k(7)*y(91)*y(140);
dydt(140)=dydt(140)-k(7)*y(91)*y(140);
dydt(92)=dydt(92)+k(7)*y(91)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(91)=dydt(91)-k(7)*y(91)*y(131);
dydt(131)=dydt(131)-k(7)*y(91)*y(131);
dydt(650)=dydt(650)+k(7)*y(91)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(91)=dydt(91)-k(7)*y(91)*y(111);
dydt(111)=dydt(111)-k(7)*y(91)*y(111);
dydt(305)=dydt(305)+k(7)*y(91)*y(111);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a, b~pY, g~Y) + Lig(l1, l2) 

dydt(91)=dydt(91)-k(2)*y(91);
dydt(4)=dydt(4)+k(2)*y(91);
dydt(72)=dydt(72)+k(2)*y(91);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(92)=dydt(92)-k(31)*y(92);
dydt(95)=dydt(95)+k(31)*y(92);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(92)=dydt(92)-k(22)*y(92);
dydt(2)=dydt(2)+k(22)*y(92);
dydt(64)=dydt(64)+k(22)*y(92);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(92);
dydt(92)=dydt(92)-k(15)*y(3)*y(92);
dydt(65)=dydt(65)+k(15)*y(3)*y(92);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(92)=dydt(92)-k(13)*y(92);
dydt(424)=dydt(424)+k(13)*y(92);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(92)=dydt(92)-k(10)*y(92);
dydt(3)=dydt(3)+k(10)*y(92);
dydt(90)=dydt(90)+k(10)*y(92);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(92)=dydt(92)-k(8)*y(92);
dydt(91)=dydt(91)+k(8)*y(92);
dydt(140)=dydt(140)+k(8)*y(92);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

dydt(92)=dydt(92)-k(6)*y(92);
dydt(72)=dydt(72)+k(6)*y(92);
dydt(380)=dydt(380)+k(6)*y(92);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(93)=dydt(93)-k(32)*y(93);
dydt(69)=dydt(69)+k(32)*y(93);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(93)=dydt(93)-k(31)*y(93);
dydt(97)=dydt(97)+k(31)*y(93);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(93);
dydt(93)=dydt(93)-k(21)*y(182)*y(93);
dydt(299)=dydt(299)+k(21)*y(182)*y(93);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(93);
dydt(93)=dydt(93)-k(21)*y(197)*y(93);
dydt(296)=dydt(296)+k(21)*y(197)*y(93);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(93);
dydt(93)=dydt(93)-k(21)*y(32)*y(93);
dydt(87)=dydt(87)+k(21)*y(32)*y(93);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(93);
dydt(93)=dydt(93)-k(21)*y(2)*y(93);
dydt(90)=dydt(90)+k(21)*y(2)*y(93);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(93);
dydt(93)=dydt(93)-k(15)*y(3)*y(93);
dydt(63)=dydt(63)+k(15)*y(3)*y(93);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(93);
dydt(93)=dydt(93)-k(9)*y(3)*y(93);
dydt(64)=dydt(64)+k(9)*y(3)*y(93);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) 

dydt(93)=dydt(93)-k(8)*y(93);
dydt(15)=dydt(15)+k(8)*y(93);
dydt(91)=dydt(91)+k(8)*y(93);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) 

dydt(93)=dydt(93)-k(6)*y(93);
dydt(72)=dydt(72)+k(6)*y(93);
dydt(98)=dydt(98)+k(6)*y(93);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(95)=dydt(95)-k(22)*y(95);
dydt(2)=dydt(2)+k(22)*y(95);
dydt(99)=dydt(99)+k(22)*y(95);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(95)=dydt(95)-k(13)*y(95);
dydt(383)=dydt(383)+k(13)*y(95);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(95)=dydt(95)-k(11)*y(95);
dydt(92)=dydt(92)+k(11)*y(95);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(95)=dydt(95)-k(10)*y(95);
dydt(3)=dydt(3)+k(10)*y(95);
dydt(94)=dydt(94)+k(10)*y(95);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(95);
dydt(95)=dydt(95)-k(9)*y(3)*y(95);
dydt(462)=dydt(462)+k(9)*y(3)*y(95);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(95)=dydt(95)-k(8)*y(95);
dydt(5)=dydt(5)+k(8)*y(95);
dydt(140)=dydt(140)+k(8)*y(95);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

dydt(95)=dydt(95)-k(6)*y(95);
dydt(1)=dydt(1)+k(6)*y(95);
dydt(380)=dydt(380)+k(6)*y(95);

% rule    : R10_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(96)=dydt(96)-k(24)*y(96);
dydt(103)=dydt(103)+k(24)*y(96);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(96)=dydt(96)-k(22)*y(96);
dydt(2)=dydt(2)+k(22)*y(96);
dydt(100)=dydt(100)+k(22)*y(96);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(96)=dydt(96)-k(12)*y(96);
dydt(109)=dydt(109)+k(12)*y(96);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(96)=dydt(96)-k(10)*y(96);
dydt(3)=dydt(3)+k(10)*y(96);
dydt(94)=dydt(94)+k(10)*y(96);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(96);
dydt(96)=dydt(96)-k(9)*y(3)*y(96);
dydt(462)=dydt(462)+k(9)*y(3)*y(96);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(96)=dydt(96)-k(8)*y(96);
dydt(8)=dydt(8)+k(8)*y(96);
dydt(21)=dydt(21)+k(8)*y(96);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(96)=dydt(96)-k(6)*y(96);
dydt(7)=dydt(7)+k(6)*y(96);
dydt(89)=dydt(89)+k(6)*y(96);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(98)=dydt(98)-k(32)*y(98);
dydt(6)=dydt(6)+k(32)*y(98);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(98);
dydt(98)=dydt(98)-k(21)*y(182)*y(98);
dydt(176)=dydt(176)+k(21)*y(182)*y(98);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(98);
dydt(98)=dydt(98)-k(21)*y(197)*y(98);
dydt(191)=dydt(191)+k(21)*y(197)*y(98);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(98);
dydt(98)=dydt(98)-k(21)*y(32)*y(98);
dydt(86)=dydt(86)+k(21)*y(32)*y(98);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(98);
dydt(98)=dydt(98)-k(21)*y(2)*y(98);
dydt(89)=dydt(89)+k(21)*y(2)*y(98);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(98);
dydt(98)=dydt(98)-k(9)*y(3)*y(98);
dydt(47)=dydt(47)+k(9)*y(3)*y(98);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(1)=dydt(1)-k(5)*y(1)*y(98);
dydt(98)=dydt(98)-k(5)*y(1)*y(98);
dydt(97)=dydt(97)+k(5)*y(1)*y(98);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(98)=dydt(98)-k(5)*y(98)*y(7);
dydt(7)=dydt(7)-k(5)*y(98)*y(7);
dydt(100)=dydt(100)+k(5)*y(98)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(98)=dydt(98)-k(5)*y(98)*y(335);
dydt(335)=dydt(335)-k(5)*y(98)*y(335);
dydt(198)=dydt(198)+k(5)*y(98)*y(335);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(98)=dydt(98)-k(5)*y(98)*y(257);
dydt(257)=dydt(257)-k(5)*y(98)*y(257);
dydt(618)=dydt(618)+k(5)*y(98)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(98)=dydt(98)-k(5)*y(98)*y(227);
dydt(227)=dydt(227)-k(5)*y(98)*y(227);
dydt(232)=dydt(232)+k(5)*y(98)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(98)=dydt(98)-k(5)*y(98)*y(222);
dydt(222)=dydt(222)-k(5)*y(98)*y(222);
dydt(282)=dydt(282)+k(5)*y(98)*y(222);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(98)=dydt(98)-k(5)*y(98)*y(218);
dydt(218)=dydt(218)-k(5)*y(98)*y(218);
dydt(431)=dydt(431)+k(5)*y(98)*y(218);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) 

dydt(98)=dydt(98)-k(5)*y(98)*y(192);
dydt(192)=dydt(192)-k(5)*y(98)*y(192);
dydt(376)=dydt(376)+k(5)*y(98)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(98)=dydt(98)-k(5)*y(98)*y(184);
dydt(184)=dydt(184)-k(5)*y(98)*y(184);
dydt(189)=dydt(189)+k(5)*y(98)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(98)=dydt(98)-k(5)*y(98)*y(177);
dydt(177)=dydt(177)-k(5)*y(98)*y(177);
dydt(451)=dydt(451)+k(5)*y(98)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(98)=dydt(98)-k(5)*y(98)*y(140);
dydt(140)=dydt(140)-k(5)*y(98)*y(140);
dydt(145)=dydt(145)+k(5)*y(98)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(98)=dydt(98)-k(5)*y(98)*y(131);
dydt(131)=dydt(131)-k(5)*y(98)*y(131);
dydt(154)=dydt(154)+k(5)*y(98)*y(131);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(98)=dydt(98)-k(5)*y(98)*y(111);
dydt(111)=dydt(111)-k(5)*y(98)*y(111);
dydt(616)=dydt(616)+k(5)*y(98)*y(111);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a, b~Y, g~pY) + Lig(l1, l2) 

dydt(98)=dydt(98)-k(4)*y(98);
dydt(4)=dydt(4)+k(4)*y(98);
dydt(15)=dydt(15)+k(4)*y(98);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(99)=dydt(99)-k(32)*y(99);
dydt(11)=dydt(11)+k(32)*y(99);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(99);
dydt(99)=dydt(99)-k(21)*y(182)*y(99);
dydt(416)=dydt(416)+k(21)*y(182)*y(99);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(99);
dydt(99)=dydt(99)-k(21)*y(197)*y(99);
dydt(413)=dydt(413)+k(21)*y(197)*y(99);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(99);
dydt(99)=dydt(99)-k(21)*y(32)*y(99);
dydt(102)=dydt(102)+k(21)*y(32)*y(99);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(99);
dydt(99)=dydt(99)-k(21)*y(2)*y(99);
dydt(95)=dydt(95)+k(21)*y(2)*y(99);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(99)=dydt(99)-k(13)*y(99);
dydt(404)=dydt(404)+k(13)*y(99);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(99)=dydt(99)-k(11)*y(99);
dydt(64)=dydt(64)+k(11)*y(99);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

dydt(99)=dydt(99)-k(10)*y(99);
dydt(3)=dydt(3)+k(10)*y(99);
dydt(97)=dydt(97)+k(10)*y(99);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(99);
dydt(99)=dydt(99)-k(9)*y(3)*y(99);
dydt(463)=dydt(463)+k(9)*y(3)*y(99);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(99)=dydt(99)-k(8)*y(99);
dydt(5)=dydt(5)+k(8)*y(99);
dydt(48)=dydt(48)+k(8)*y(99);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) 

dydt(99)=dydt(99)-k(6)*y(99);
dydt(1)=dydt(1)+k(6)*y(99);
dydt(47)=dydt(47)+k(6)*y(99);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(100)=dydt(100)-k(32)*y(100);
dydt(148)=dydt(148)+k(32)*y(100);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(100);
dydt(100)=dydt(100)-k(21)*y(182)*y(100);
dydt(606)=dydt(606)+k(21)*y(182)*y(100);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(100);
dydt(100)=dydt(100)-k(21)*y(197)*y(100);
dydt(607)=dydt(607)+k(21)*y(197)*y(100);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(100);
dydt(100)=dydt(100)-k(21)*y(32)*y(100);
dydt(103)=dydt(103)+k(21)*y(32)*y(100);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(100);
dydt(100)=dydt(100)-k(21)*y(2)*y(100);
dydt(96)=dydt(96)+k(21)*y(2)*y(100);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(100)=dydt(100)-k(12)*y(100);
dydt(108)=dydt(108)+k(12)*y(100);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

dydt(100)=dydt(100)-k(10)*y(100);
dydt(3)=dydt(3)+k(10)*y(100);
dydt(97)=dydt(97)+k(10)*y(100);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(100);
dydt(100)=dydt(100)-k(9)*y(3)*y(100);
dydt(463)=dydt(463)+k(9)*y(3)*y(100);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) 

dydt(100)=dydt(100)-k(8)*y(100);
dydt(15)=dydt(15)+k(8)*y(100);
dydt(8)=dydt(8)+k(8)*y(100);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(100)=dydt(100)-k(6)*y(100);
dydt(98)=dydt(98)+k(6)*y(100);
dydt(7)=dydt(7)+k(6)*y(100);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(102)=dydt(102)-k(33)*y(102);
dydt(95)=dydt(95)+k(33)*y(102);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(102)=dydt(102)-k(22)*y(102);
dydt(32)=dydt(32)+k(22)*y(102);
dydt(99)=dydt(99)+k(22)*y(102);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(102)=dydt(102)-k(13)*y(102);
dydt(405)=dydt(405)+k(13)*y(102);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(102)=dydt(102)-k(11)*y(102);
dydt(650)=dydt(650)+k(11)*y(102);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(102)=dydt(102)-k(10)*y(102);
dydt(3)=dydt(3)+k(10)*y(102);
dydt(101)=dydt(101)+k(10)*y(102);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(102);
dydt(102)=dydt(102)-k(9)*y(3)*y(102);
dydt(104)=dydt(104)+k(9)*y(3)*y(102);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(102)=dydt(102)-k(8)*y(102);
dydt(5)=dydt(5)+k(8)*y(102);
dydt(131)=dydt(131)+k(8)*y(102);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

dydt(102)=dydt(102)-k(6)*y(102);
dydt(1)=dydt(1)+k(6)*y(102);
dydt(441)=dydt(441)+k(6)*y(102);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(104)=dydt(104)-k(33)*y(104);
dydt(462)=dydt(462)+k(33)*y(104);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(104)=dydt(104)-k(22)*y(104);
dydt(32)=dydt(32)+k(22)*y(104);
dydt(463)=dydt(463)+k(22)*y(104);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(104)=dydt(104)-k(13)*y(104);
dydt(458)=dydt(458)+k(13)*y(104);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(104)=dydt(104)-k(10)*y(104);
dydt(3)=dydt(3)+k(10)*y(104);
dydt(103)=dydt(103)+k(10)*y(104);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(104)=dydt(104)-k(10)*y(104);
dydt(3)=dydt(3)+k(10)*y(104);
dydt(102)=dydt(102)+k(10)*y(104);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(104)=dydt(104)-k(8)*y(104);
dydt(8)=dydt(8)+k(8)*y(104);
dydt(131)=dydt(131)+k(8)*y(104);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(104)=dydt(104)-k(6)*y(104);
dydt(7)=dydt(7)+k(6)*y(104);
dydt(441)=dydt(441)+k(6)*y(104);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(106)=dydt(106)-k(33)*y(106);
dydt(112)=dydt(112)+k(33)*y(106);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(106)=dydt(106)-k(31)*y(106);
dydt(101)=dydt(101)+k(31)*y(106);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(106)=dydt(106)-k(22)*y(106);
dydt(32)=dydt(32)+k(22)*y(106);
dydt(590)=dydt(590)+k(22)*y(106);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(106);
dydt(106)=dydt(106)-k(15)*y(3)*y(106);
dydt(592)=dydt(592)+k(15)*y(3)*y(106);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(106);
dydt(106)=dydt(106)-k(9)*y(3)*y(106);
dydt(105)=dydt(105)+k(9)*y(3)*y(106);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(106)=dydt(106)-k(8)*y(106);
dydt(5)=dydt(5)+k(8)*y(106);
dydt(33)=dydt(33)+k(8)*y(106);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

dydt(106)=dydt(106)-k(6)*y(106);
dydt(1)=dydt(1)+k(6)*y(106);
dydt(82)=dydt(82)+k(6)*y(106);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(107)=dydt(107)-k(33)*y(107);
dydt(113)=dydt(113)+k(33)*y(107);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(107)=dydt(107)-k(22)*y(107);
dydt(32)=dydt(32)+k(22)*y(107);
dydt(117)=dydt(117)+k(22)*y(107);

% rule    : R8
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(107)=dydt(107)-k(19)*y(107);
dydt(352)=dydt(352)+k(19)*y(107);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(107)=dydt(107)-k(16)*y(107);
dydt(3)=dydt(3)+k(16)*y(107);
dydt(105)=dydt(105)+k(16)*y(107);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(107)=dydt(107)-k(10)*y(107);
dydt(3)=dydt(3)+k(10)*y(107);
dydt(592)=dydt(592)+k(10)*y(107);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(107)=dydt(107)-k(8)*y(107);
dydt(8)=dydt(8)+k(8)*y(107);
dydt(37)=dydt(37)+k(8)*y(107);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(107)=dydt(107)-k(6)*y(107);
dydt(7)=dydt(7)+k(6)*y(107);
dydt(554)=dydt(554)+k(6)*y(107);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(108)=dydt(108)-k(32)*y(108);
dydt(124)=dydt(124)+k(32)*y(108);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(108)=dydt(108)-k(31)*y(108);
dydt(100)=dydt(100)+k(31)*y(108);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(108);
dydt(108)=dydt(108)-k(21)*y(182)*y(108);
dydt(594)=dydt(594)+k(21)*y(182)*y(108);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(108);
dydt(108)=dydt(108)-k(21)*y(197)*y(108);
dydt(588)=dydt(588)+k(21)*y(197)*y(108);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(108);
dydt(108)=dydt(108)-k(21)*y(32)*y(108);
dydt(105)=dydt(105)+k(21)*y(32)*y(108);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(108);
dydt(108)=dydt(108)-k(21)*y(2)*y(108);
dydt(109)=dydt(109)+k(21)*y(2)*y(108);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(108);
dydt(108)=dydt(108)-k(15)*y(3)*y(108);
dydt(117)=dydt(117)+k(15)*y(3)*y(108);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

dydt(108)=dydt(108)-k(10)*y(108);
dydt(3)=dydt(3)+k(10)*y(108);
dydt(590)=dydt(590)+k(10)*y(108);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) 

dydt(108)=dydt(108)-k(8)*y(108);
dydt(77)=dydt(77)+k(8)*y(108);
dydt(8)=dydt(8)+k(8)*y(108);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(108)=dydt(108)-k(6)*y(108);
dydt(76)=dydt(76)+k(6)*y(108);
dydt(7)=dydt(7)+k(6)*y(108);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(110)=dydt(110)-k(31)*y(110);
dydt(89)=dydt(89)+k(31)*y(110);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~Y) 

dydt(110)=dydt(110)-k(22)*y(110);
dydt(2)=dydt(2)+k(22)*y(110);
dydt(76)=dydt(76)+k(22)*y(110);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(110);
dydt(110)=dydt(110)-k(15)*y(3)*y(110);
dydt(114)=dydt(114)+k(15)*y(3)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(1)=dydt(1)-k(5)*y(1)*y(110);
dydt(110)=dydt(110)-k(5)*y(1)*y(110);
dydt(112)=dydt(112)+k(5)*y(1)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(7)=dydt(7)-k(5)*y(7)*y(110);
dydt(110)=dydt(110)-k(5)*y(7)*y(110);
dydt(109)=dydt(109)+k(5)*y(7)*y(110);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(110)=dydt(110)-k(5)*y(110)*y(335);
dydt(335)=dydt(335)-k(5)*y(110)*y(335);
dydt(340)=dydt(340)+k(5)*y(110)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(257)=dydt(257)-k(5)*y(257)*y(110);
dydt(110)=dydt(110)-k(5)*y(257)*y(110);
dydt(502)=dydt(502)+k(5)*y(257)*y(110);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(110)=dydt(110)-k(5)*y(110)*y(227);
dydt(227)=dydt(227)-k(5)*y(110)*y(227);
dydt(485)=dydt(485)+k(5)*y(110)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(110)=dydt(110)-k(5)*y(110)*y(222);
dydt(222)=dydt(222)-k(5)*y(110)*y(222);
dydt(281)=dydt(281)+k(5)*y(110)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(218)=dydt(218)-k(5)*y(218)*y(110);
dydt(110)=dydt(110)-k(5)*y(218)*y(110);
dydt(505)=dydt(505)+k(5)*y(218)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(192)=dydt(192)-k(5)*y(192)*y(110);
dydt(110)=dydt(110)-k(5)*y(192)*y(110);
dydt(527)=dydt(527)+k(5)*y(192)*y(110);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(110)=dydt(110)-k(5)*y(110)*y(184);
dydt(184)=dydt(184)-k(5)*y(110)*y(184);
dydt(346)=dydt(346)+k(5)*y(110)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(177)=dydt(177)-k(5)*y(177)*y(110);
dydt(110)=dydt(110)-k(5)*y(177)*y(110);
dydt(509)=dydt(509)+k(5)*y(177)*y(110);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(110)=dydt(110)-k(5)*y(110)*y(140);
dydt(140)=dydt(140)-k(5)*y(110)*y(140);
dydt(173)=dydt(173)+k(5)*y(110)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(110)=dydt(110)-k(5)*y(110)*y(131);
dydt(131)=dydt(131)-k(5)*y(110)*y(131);
dydt(355)=dydt(355)+k(5)*y(110)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(111)=dydt(111)-k(5)*y(111)*y(110);
dydt(110)=dydt(110)-k(5)*y(111)*y(110);
dydt(503)=dydt(503)+k(5)*y(111)*y(110);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) 

dydt(110)=dydt(110)-k(4)*y(110);
dydt(4)=dydt(4)+k(4)*y(110);
dydt(111)=dydt(111)+k(4)*y(110);

% rule    : R14
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(111)=dydt(111)-k(31)*y(111);
dydt(21)=dydt(21)+k(31)*y(111);

% rule    : R9_op
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(111)=dydt(111)-k(22)*y(111);
dydt(2)=dydt(2)+k(22)*y(111);
dydt(77)=dydt(77)+k(22)*y(111);

% rule    : R6
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(111);
dydt(111)=dydt(111)-k(15)*y(3)*y(111);
dydt(43)=dydt(43)+k(15)*y(3)*y(111);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(126)=dydt(126)-k(7)*y(126)*y(111);
dydt(111)=dydt(111)-k(7)*y(126)*y(111);
dydt(353)=dydt(353)+k(7)*y(126)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(132);
dydt(132)=dydt(132)-k(7)*y(111)*y(132);
dydt(355)=dydt(355)+k(7)*y(111)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(141);
dydt(141)=dydt(141)-k(7)*y(111)*y(141);
dydt(173)=dydt(173)+k(7)*y(111)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(185);
dydt(185)=dydt(185)-k(7)*y(111)*y(185);
dydt(346)=dydt(346)+k(7)*y(111)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(209);
dydt(209)=dydt(209)-k(7)*y(111)*y(209);
dydt(527)=dydt(527)+k(7)*y(111)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(214);
dydt(214)=dydt(214)-k(7)*y(111)*y(214);
dydt(509)=dydt(509)+k(7)*y(111)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(219);
dydt(219)=dydt(219)-k(7)*y(111)*y(219);
dydt(505)=dydt(505)+k(7)*y(111)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(223);
dydt(223)=dydt(223)-k(7)*y(111)*y(223);
dydt(281)=dydt(281)+k(7)*y(111)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(228);
dydt(228)=dydt(228)-k(7)*y(111)*y(228);
dydt(485)=dydt(485)+k(7)*y(111)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(315)=dydt(315)-k(7)*y(315)*y(111);
dydt(111)=dydt(111)-k(7)*y(315)*y(111);
dydt(501)=dydt(501)+k(7)*y(315)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(319);
dydt(319)=dydt(319)-k(7)*y(111)*y(319);
dydt(502)=dydt(502)+k(7)*y(111)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(336);
dydt(336)=dydt(336)-k(7)*y(111)*y(336);
dydt(340)=dydt(340)+k(7)*y(111)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(111)=dydt(111)-k(7)*y(111)*y(437);
dydt(437)=dydt(437)-k(7)*y(111)*y(437);
dydt(503)=dydt(503)+k(7)*y(111)*y(437);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(111)=dydt(111)-k(5)*y(111)*y(114);
dydt(114)=dydt(114)-k(5)*y(111)*y(114);
dydt(504)=dydt(504)+k(5)*y(111)*y(114);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(118)=dydt(118)-k(5)*y(118)*y(111);
dydt(111)=dydt(111)-k(5)*y(118)*y(111);
dydt(540)=dydt(540)+k(5)*y(118)*y(111);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(122)=dydt(122)-k(5)*y(122)*y(111);
dydt(111)=dydt(111)-k(5)*y(122)*y(111);
dydt(316)=dydt(316)+k(5)*y(122)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(111)=dydt(111)-k(5)*y(111)*y(176);
dydt(176)=dydt(176)-k(5)*y(111)*y(176);
dydt(271)=dydt(271)+k(5)*y(111)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(111)=dydt(111)-k(5)*y(111)*y(191);
dydt(191)=dydt(191)-k(5)*y(111)*y(191);
dydt(438)=dydt(438)+k(5)*y(111)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(111)=dydt(111)-k(5)*y(111)*y(256);
dydt(256)=dydt(256)-k(5)*y(111)*y(256);
dydt(259)=dydt(259)+k(5)*y(111)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(111)=dydt(111)-k(5)*y(111)*y(264);
dydt(264)=dydt(264)-k(5)*y(111)*y(264);
dydt(266)=dydt(266)+k(5)*y(111)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(111)=dydt(111)-k(5)*y(111)*y(380);
dydt(380)=dydt(380)-k(5)*y(111)*y(380);
dydt(387)=dydt(387)+k(5)*y(111)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(111)=dydt(111)-k(5)*y(111)*y(391);
dydt(391)=dydt(391)-k(5)*y(111)*y(391);
dydt(392)=dydt(392)+k(5)*y(111)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(111)=dydt(111)-k(5)*y(111)*y(407);
dydt(407)=dydt(407)-k(5)*y(111)*y(407);
dydt(436)=dydt(436)+k(5)*y(111)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(111)=dydt(111)-k(5)*y(111)*y(441);
dydt(441)=dydt(441)-k(5)*y(111)*y(441);
dydt(440)=dydt(440)+k(5)*y(111)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(111)=dydt(111)-k(5)*y(111)*y(515);
dydt(515)=dydt(515)-k(5)*y(111)*y(515);
dydt(625)=dydt(625)+k(5)*y(111)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(111)=dydt(111)-k(5)*y(111)*y(543);
dydt(543)=dydt(543)-k(5)*y(111)*y(543);
dydt(546)=dydt(546)+k(5)*y(111)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(111)=dydt(111)-k(5)*y(111)*y(554);
dydt(554)=dydt(554)-k(5)*y(111)*y(554);
dydt(570)=dydt(570)+k(5)*y(111)*y(554);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(9)=dydt(9)-k(5)*y(9)*y(111);
dydt(111)=dydt(111)-k(5)*y(9)*y(111);
dydt(25)=dydt(25)+k(5)*y(9)*y(111);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(4)=dydt(4)-k(3)*y(4)*y(111);
dydt(111)=dydt(111)-k(3)*y(4)*y(111);
dydt(110)=dydt(110)+k(3)*y(4)*y(111);

% rule    : R1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(4)=dydt(4)-k(1)*y(4)*y(111);
dydt(111)=dydt(111)-k(1)*y(4)*y(111);
dydt(437)=dydt(437)+k(1)*y(4)*y(111);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(112)=dydt(112)-k(31)*y(112);
dydt(94)=dydt(94)+k(31)*y(112);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(112)=dydt(112)-k(22)*y(112);
dydt(2)=dydt(2)+k(22)*y(112);
dydt(590)=dydt(590)+k(22)*y(112);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(112);
dydt(112)=dydt(112)-k(15)*y(3)*y(112);
dydt(115)=dydt(115)+k(15)*y(3)*y(112);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(112);
dydt(112)=dydt(112)-k(9)*y(3)*y(112);
dydt(109)=dydt(109)+k(9)*y(3)*y(112);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(112)=dydt(112)-k(8)*y(112);
dydt(5)=dydt(5)+k(8)*y(112);
dydt(111)=dydt(111)+k(8)*y(112);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

dydt(112)=dydt(112)-k(6)*y(112);
dydt(1)=dydt(1)+k(6)*y(112);
dydt(110)=dydt(110)+k(6)*y(112);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

dydt(114)=dydt(114)-k(22)*y(114);
dydt(2)=dydt(2)+k(22)*y(114);
dydt(118)=dydt(118)+k(22)*y(114);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(114)=dydt(114)-k(16)*y(114);
dydt(3)=dydt(3)+k(16)*y(114);
dydt(110)=dydt(110)+k(16)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(1)=dydt(1)-k(5)*y(1)*y(114);
dydt(114)=dydt(114)-k(5)*y(1)*y(114);
dydt(115)=dydt(115)+k(5)*y(1)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(7)=dydt(7)-k(5)*y(7)*y(114);
dydt(114)=dydt(114)-k(5)*y(7)*y(114);
dydt(113)=dydt(113)+k(5)*y(7)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(5)*y(335)*y(114);
dydt(114)=dydt(114)-k(5)*y(335)*y(114);
dydt(526)=dydt(526)+k(5)*y(335)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(257)=dydt(257)-k(5)*y(257)*y(114);
dydt(114)=dydt(114)-k(5)*y(257)*y(114);
dydt(528)=dydt(528)+k(5)*y(257)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(227)=dydt(227)-k(5)*y(227)*y(114);
dydt(114)=dydt(114)-k(5)*y(227)*y(114);
dydt(632)=dydt(632)+k(5)*y(227)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(222)=dydt(222)-k(5)*y(222)*y(114);
dydt(114)=dydt(114)-k(5)*y(222)*y(114);
dydt(637)=dydt(637)+k(5)*y(222)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(218)=dydt(218)-k(5)*y(218)*y(114);
dydt(114)=dydt(114)-k(5)*y(218)*y(114);
dydt(523)=dydt(523)+k(5)*y(218)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(192)=dydt(192)-k(5)*y(192)*y(114);
dydt(114)=dydt(114)-k(5)*y(192)*y(114);
dydt(525)=dydt(525)+k(5)*y(192)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(5)*y(184)*y(114);
dydt(114)=dydt(114)-k(5)*y(184)*y(114);
dydt(522)=dydt(522)+k(5)*y(184)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(177)=dydt(177)-k(5)*y(177)*y(114);
dydt(114)=dydt(114)-k(5)*y(177)*y(114);
dydt(521)=dydt(521)+k(5)*y(177)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(140)=dydt(140)-k(5)*y(140)*y(114);
dydt(114)=dydt(114)-k(5)*y(140)*y(114);
dydt(604)=dydt(604)+k(5)*y(140)*y(114);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(131)=dydt(131)-k(5)*y(131)*y(114);
dydt(114)=dydt(114)-k(5)*y(131)*y(114);
dydt(357)=dydt(357)+k(5)*y(131)*y(114);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) 

dydt(114)=dydt(114)-k(4)*y(114);
dydt(4)=dydt(4)+k(4)*y(114);
dydt(43)=dydt(43)+k(4)*y(114);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

dydt(115)=dydt(115)-k(22)*y(115);
dydt(2)=dydt(2)+k(22)*y(115);
dydt(119)=dydt(119)+k(22)*y(115);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(115)=dydt(115)-k(19)*y(115);
dydt(524)=dydt(524)+k(19)*y(115);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(115)=dydt(115)-k(17)*y(115);
dydt(306)=dydt(306)+k(17)*y(115);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(115)=dydt(115)-k(16)*y(115);
dydt(3)=dydt(3)+k(16)*y(115);
dydt(112)=dydt(112)+k(16)*y(115);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(115);
dydt(115)=dydt(115)-k(9)*y(3)*y(115);
dydt(113)=dydt(113)+k(9)*y(3)*y(115);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(115)=dydt(115)-k(8)*y(115);
dydt(5)=dydt(5)+k(8)*y(115);
dydt(43)=dydt(43)+k(8)*y(115);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

dydt(115)=dydt(115)-k(6)*y(115);
dydt(1)=dydt(1)+k(6)*y(115);
dydt(114)=dydt(114)+k(6)*y(115);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(116)=dydt(116)-k(32)*y(116);
dydt(113)=dydt(113)+k(32)*y(116);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(116)=dydt(116)-k(24)*y(116);
dydt(352)=dydt(352)+k(24)*y(116);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(116)=dydt(116)-k(22)*y(116);
dydt(2)=dydt(2)+k(22)*y(116);
dydt(120)=dydt(120)+k(22)*y(116);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(116);
dydt(116)=dydt(116)-k(21)*y(182)*y(116);
dydt(522)=dydt(522)+k(21)*y(182)*y(116);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(116);
dydt(116)=dydt(116)-k(21)*y(197)*y(116);
dydt(526)=dydt(526)+k(21)*y(197)*y(116);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(116);
dydt(116)=dydt(116)-k(21)*y(32)*y(116);
dydt(357)=dydt(357)+k(21)*y(32)*y(116);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(116);
dydt(116)=dydt(116)-k(21)*y(2)*y(116);
dydt(604)=dydt(604)+k(21)*y(2)*y(116);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(116)=dydt(116)-k(16)*y(116);
dydt(3)=dydt(3)+k(16)*y(116);
dydt(353)=dydt(353)+k(16)*y(116);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(116)=dydt(116)-k(10)*y(116);
dydt(3)=dydt(3)+k(10)*y(116);
dydt(524)=dydt(524)+k(10)*y(116);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(116)=dydt(116)-k(8)*y(116);
dydt(126)=dydt(126)+k(8)*y(116);
dydt(43)=dydt(43)+k(8)*y(116);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(116)=dydt(116)-k(6)*y(116);
dydt(48)=dydt(48)+k(6)*y(116);
dydt(114)=dydt(114)+k(6)*y(116);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(118)=dydt(118)-k(32)*y(118);
dydt(122)=dydt(122)+k(32)*y(118);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(118);
dydt(118)=dydt(118)-k(21)*y(182)*y(118);
dydt(515)=dydt(515)+k(21)*y(182)*y(118);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(118);
dydt(118)=dydt(118)-k(21)*y(197)*y(118);
dydt(543)=dydt(543)+k(21)*y(197)*y(118);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(118);
dydt(118)=dydt(118)-k(21)*y(32)*y(118);
dydt(554)=dydt(554)+k(21)*y(32)*y(118);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(118);
dydt(118)=dydt(118)-k(21)*y(2)*y(118);
dydt(114)=dydt(114)+k(21)*y(2)*y(118);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Lyn(U, SH2) 

dydt(118)=dydt(118)-k(16)*y(118);
dydt(3)=dydt(3)+k(16)*y(118);
dydt(76)=dydt(76)+k(16)*y(118);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(1)=dydt(1)-k(5)*y(1)*y(118);
dydt(118)=dydt(118)-k(5)*y(1)*y(118);
dydt(119)=dydt(119)+k(5)*y(1)*y(118);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(7)=dydt(7)-k(5)*y(7)*y(118);
dydt(118)=dydt(118)-k(5)*y(7)*y(118);
dydt(117)=dydt(117)+k(5)*y(7)*y(118);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(118)=dydt(118)-k(5)*y(118)*y(335);
dydt(335)=dydt(335)-k(5)*y(118)*y(335);
dydt(339)=dydt(339)+k(5)*y(118)*y(335);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(118)=dydt(118)-k(5)*y(118)*y(257);
dydt(257)=dydt(257)-k(5)*y(118)*y(257);
dydt(564)=dydt(564)+k(5)*y(118)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(118)=dydt(118)-k(5)*y(118)*y(227);
dydt(227)=dydt(227)-k(5)*y(118)*y(227);
dydt(636)=dydt(636)+k(5)*y(118)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(118)=dydt(118)-k(5)*y(118)*y(222);
dydt(222)=dydt(222)-k(5)*y(118)*y(222);
dydt(638)=dydt(638)+k(5)*y(118)*y(222);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(118)=dydt(118)-k(5)*y(118)*y(218);
dydt(218)=dydt(218)-k(5)*y(118)*y(218);
dydt(644)=dydt(644)+k(5)*y(118)*y(218);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(118)=dydt(118)-k(5)*y(118)*y(192);
dydt(192)=dydt(192)-k(5)*y(118)*y(192);
dydt(529)=dydt(529)+k(5)*y(118)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(118)=dydt(118)-k(5)*y(118)*y(184);
dydt(184)=dydt(184)-k(5)*y(118)*y(184);
dydt(345)=dydt(345)+k(5)*y(118)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(118)=dydt(118)-k(5)*y(118)*y(177);
dydt(177)=dydt(177)-k(5)*y(118)*y(177);
dydt(519)=dydt(519)+k(5)*y(118)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(118)=dydt(118)-k(5)*y(118)*y(140);
dydt(140)=dydt(140)-k(5)*y(118)*y(140);
dydt(539)=dydt(539)+k(5)*y(118)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(118)=dydt(118)-k(5)*y(118)*y(131);
dydt(131)=dydt(131)-k(5)*y(118)*y(131);
dydt(134)=dydt(134)+k(5)*y(118)*y(131);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Lig(l1, l2) 

dydt(118)=dydt(118)-k(4)*y(118);
dydt(4)=dydt(4)+k(4)*y(118);
dydt(54)=dydt(54)+k(4)*y(118);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(119)=dydt(119)-k(32)*y(119);
dydt(123)=dydt(123)+k(32)*y(119);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(119);
dydt(119)=dydt(119)-k(21)*y(182)*y(119);
dydt(591)=dydt(591)+k(21)*y(182)*y(119);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(119);
dydt(119)=dydt(119)-k(21)*y(197)*y(119);
dydt(589)=dydt(589)+k(21)*y(197)*y(119);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(119);
dydt(119)=dydt(119)-k(21)*y(32)*y(119);
dydt(592)=dydt(592)+k(21)*y(32)*y(119);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(119);
dydt(119)=dydt(119)-k(21)*y(2)*y(119);
dydt(115)=dydt(115)+k(21)*y(2)*y(119);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(119)=dydt(119)-k(19)*y(119);
dydt(537)=dydt(537)+k(19)*y(119);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(119)=dydt(119)-k(17)*y(119);
dydt(309)=dydt(309)+k(17)*y(119);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

dydt(119)=dydt(119)-k(16)*y(119);
dydt(3)=dydt(3)+k(16)*y(119);
dydt(590)=dydt(590)+k(16)*y(119);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(119);
dydt(119)=dydt(119)-k(9)*y(3)*y(119);
dydt(117)=dydt(117)+k(9)*y(3)*y(119);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(119)=dydt(119)-k(8)*y(119);
dydt(5)=dydt(5)+k(8)*y(119);
dydt(54)=dydt(54)+k(8)*y(119);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) 

dydt(119)=dydt(119)-k(6)*y(119);
dydt(1)=dydt(1)+k(6)*y(119);
dydt(118)=dydt(118)+k(6)*y(119);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(120)=dydt(120)-k(32)*y(120);
dydt(125)=dydt(125)+k(32)*y(120);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(120)=dydt(120)-k(32)*y(120);
dydt(117)=dydt(117)+k(32)*y(120);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(120);
dydt(120)=dydt(120)-k(21)*y(182)*y(120);
dydt(345)=dydt(345)+k(21)*y(182)*y(120);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(120);
dydt(120)=dydt(120)-k(21)*y(197)*y(120);
dydt(339)=dydt(339)+k(21)*y(197)*y(120);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(120);
dydt(120)=dydt(120)-k(21)*y(32)*y(120);
dydt(134)=dydt(134)+k(21)*y(32)*y(120);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(120);
dydt(120)=dydt(120)-k(21)*y(2)*y(120);
dydt(539)=dydt(539)+k(21)*y(2)*y(120);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(182)=dydt(182)-k(21)*y(182)*y(120);
dydt(120)=dydt(120)-k(21)*y(182)*y(120);
dydt(595)=dydt(595)+k(21)*y(182)*y(120);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(197)=dydt(197)-k(21)*y(197)*y(120);
dydt(120)=dydt(120)-k(21)*y(197)*y(120);
dydt(598)=dydt(598)+k(21)*y(197)*y(120);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(32)=dydt(32)-k(21)*y(32)*y(120);
dydt(120)=dydt(120)-k(21)*y(32)*y(120);
dydt(352)=dydt(352)+k(21)*y(32)*y(120);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(2)=dydt(2)-k(21)*y(2)*y(120);
dydt(120)=dydt(120)-k(21)*y(2)*y(120);
dydt(116)=dydt(116)+k(21)*y(2)*y(120);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) 

dydt(120)=dydt(120)-k(16)*y(120);
dydt(3)=dydt(3)+k(16)*y(120);
dydt(151)=dydt(151)+k(16)*y(120);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) 

dydt(120)=dydt(120)-k(10)*y(120);
dydt(3)=dydt(3)+k(10)*y(120);
dydt(537)=dydt(537)+k(10)*y(120);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(120)=dydt(120)-k(8)*y(120);
dydt(54)=dydt(54)+k(8)*y(120);
dydt(126)=dydt(126)+k(8)*y(120);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(120)=dydt(120)-k(6)*y(120);
dydt(48)=dydt(48)+k(6)*y(120);
dydt(118)=dydt(118)+k(6)*y(120);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Lyn(U, SH2) 

dydt(122)=dydt(122)-k(16)*y(122);
dydt(3)=dydt(3)+k(16)*y(122);
dydt(71)=dydt(71)+k(16)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(1)=dydt(1)-k(5)*y(1)*y(122);
dydt(122)=dydt(122)-k(5)*y(1)*y(122);
dydt(123)=dydt(123)+k(5)*y(1)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(7)=dydt(7)-k(5)*y(7)*y(122);
dydt(122)=dydt(122)-k(5)*y(7)*y(122);
dydt(121)=dydt(121)+k(5)*y(7)*y(122);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(122)=dydt(122)-k(5)*y(122)*y(335);
dydt(335)=dydt(335)-k(5)*y(122)*y(335);
dydt(200)=dydt(200)+k(5)*y(122)*y(335);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(122)=dydt(122)-k(5)*y(122)*y(257);
dydt(257)=dydt(257)-k(5)*y(122)*y(257);
dydt(318)=dydt(318)+k(5)*y(122)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(122)=dydt(122)-k(5)*y(122)*y(227);
dydt(227)=dydt(227)-k(5)*y(122)*y(227);
dydt(320)=dydt(320)+k(5)*y(122)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(122)=dydt(122)-k(5)*y(122)*y(222);
dydt(222)=dydt(222)-k(5)*y(122)*y(222);
dydt(324)=dydt(324)+k(5)*y(122)*y(222);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(122)=dydt(122)-k(5)*y(122)*y(218);
dydt(218)=dydt(218)-k(5)*y(122)*y(218);
dydt(323)=dydt(323)+k(5)*y(122)*y(218);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(122)=dydt(122)-k(5)*y(122)*y(192);
dydt(192)=dydt(192)-k(5)*y(122)*y(192);
dydt(322)=dydt(322)+k(5)*y(122)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(122)=dydt(122)-k(5)*y(122)*y(184);
dydt(184)=dydt(184)-k(5)*y(122)*y(184);
dydt(327)=dydt(327)+k(5)*y(122)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(122)=dydt(122)-k(5)*y(122)*y(177);
dydt(177)=dydt(177)-k(5)*y(122)*y(177);
dydt(326)=dydt(326)+k(5)*y(122)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(122)=dydt(122)-k(5)*y(122)*y(140);
dydt(140)=dydt(140)-k(5)*y(122)*y(140);
dydt(129)=dydt(129)+k(5)*y(122)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(122)=dydt(122)-k(5)*y(122)*y(131);
dydt(131)=dydt(131)-k(5)*y(122)*y(131);
dydt(130)=dydt(130)+k(5)*y(122)*y(131);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Lig(l1, l2) 

dydt(122)=dydt(122)-k(4)*y(122);
dydt(4)=dydt(4)+k(4)*y(122);
dydt(61)=dydt(61)+k(4)*y(122);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(123)=dydt(123)-k(19)*y(123);
dydt(127)=dydt(127)+k(19)*y(123);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(123)=dydt(123)-k(17)*y(123);
dydt(313)=dydt(313)+k(17)*y(123);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

dydt(123)=dydt(123)-k(16)*y(123);
dydt(3)=dydt(3)+k(16)*y(123);
dydt(333)=dydt(333)+k(16)*y(123);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(123);
dydt(123)=dydt(123)-k(9)*y(3)*y(123);
dydt(121)=dydt(121)+k(9)*y(3)*y(123);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(123)=dydt(123)-k(8)*y(123);
dydt(5)=dydt(5)+k(8)*y(123);
dydt(61)=dydt(61)+k(8)*y(123);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) 

dydt(123)=dydt(123)-k(6)*y(123);
dydt(1)=dydt(1)+k(6)*y(123);
dydt(122)=dydt(122)+k(6)*y(123);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(124)=dydt(124)-k(31)*y(124);
dydt(148)=dydt(148)+k(31)*y(124);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(124);
dydt(124)=dydt(124)-k(15)*y(3)*y(124);
dydt(121)=dydt(121)+k(15)*y(3)*y(124);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(124)=dydt(124)-k(14)*y(124);
dydt(108)=dydt(108)+k(14)*y(124);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

dydt(124)=dydt(124)-k(10)*y(124);
dydt(3)=dydt(3)+k(10)*y(124);
dydt(333)=dydt(333)+k(10)*y(124);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) 

dydt(124)=dydt(124)-k(8)*y(124);
dydt(72)=dydt(72)+k(8)*y(124);
dydt(8)=dydt(8)+k(8)*y(124);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(124)=dydt(124)-k(6)*y(124);
dydt(71)=dydt(71)+k(6)*y(124);
dydt(7)=dydt(7)+k(6)*y(124);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(126)=dydt(126)-k(32)*y(126);
dydt(8)=dydt(8)+k(32)*y(126);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(126);
dydt(126)=dydt(126)-k(21)*y(182)*y(126);
dydt(185)=dydt(185)+k(21)*y(182)*y(126);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(126);
dydt(126)=dydt(126)-k(21)*y(197)*y(126);
dydt(336)=dydt(336)+k(21)*y(197)*y(126);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(126);
dydt(126)=dydt(126)-k(21)*y(32)*y(126);
dydt(132)=dydt(132)+k(21)*y(32)*y(126);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(126);
dydt(126)=dydt(126)-k(21)*y(2)*y(126);
dydt(141)=dydt(141)+k(21)*y(2)*y(126);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Lyn(U, SH2) 

dydt(126)=dydt(126)-k(10)*y(126);
dydt(3)=dydt(3)+k(10)*y(126);
dydt(16)=dydt(16)+k(10)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(1)=dydt(1)-k(7)*y(1)*y(126);
dydt(126)=dydt(126)-k(7)*y(1)*y(126);
dydt(146)=dydt(146)+k(7)*y(1)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(7)*y(7)*y(126);
dydt(126)=dydt(126)-k(7)*y(7)*y(126);
dydt(17)=dydt(17)+k(7)*y(7)*y(126);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(126)=dydt(126)-k(7)*y(126)*y(335);
dydt(335)=dydt(335)-k(7)*y(126)*y(335);
dydt(408)=dydt(408)+k(7)*y(126)*y(335);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(126)=dydt(126)-k(7)*y(126)*y(257);
dydt(257)=dydt(257)-k(7)*y(126)*y(257);
dydt(599)=dydt(599)+k(7)*y(126)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(126)=dydt(126)-k(7)*y(126)*y(227);
dydt(227)=dydt(227)-k(7)*y(126)*y(227);
dydt(598)=dydt(598)+k(7)*y(126)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(126)=dydt(126)-k(7)*y(126)*y(222);
dydt(222)=dydt(222)-k(7)*y(126)*y(222);
dydt(595)=dydt(595)+k(7)*y(126)*y(222);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(126)=dydt(126)-k(7)*y(126)*y(218);
dydt(218)=dydt(218)-k(7)*y(126)*y(218);
dydt(361)=dydt(361)+k(7)*y(126)*y(218);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(126)=dydt(126)-k(7)*y(126)*y(192);
dydt(192)=dydt(192)-k(7)*y(126)*y(192);
dydt(196)=dydt(196)+k(7)*y(126)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(126)=dydt(126)-k(7)*y(126)*y(184);
dydt(184)=dydt(184)-k(7)*y(126)*y(184);
dydt(651)=dydt(651)+k(7)*y(126)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(126)=dydt(126)-k(7)*y(126)*y(177);
dydt(177)=dydt(177)-k(7)*y(126)*y(177);
dydt(181)=dydt(181)+k(7)*y(126)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(126)=dydt(126)-k(7)*y(126)*y(140);
dydt(140)=dydt(140)-k(7)*y(126)*y(140);
dydt(461)=dydt(461)+k(7)*y(126)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(126)=dydt(126)-k(7)*y(126)*y(131);
dydt(131)=dydt(131)-k(7)*y(126)*y(131);
dydt(458)=dydt(458)+k(7)*y(126)*y(131);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Lig(l1, l2) 

dydt(126)=dydt(126)-k(2)*y(126);
dydt(4)=dydt(4)+k(2)*y(126);
dydt(48)=dydt(48)+k(2)*y(126);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(127)=dydt(127)-k(32)*y(127);
dydt(123)=dydt(123)+k(32)*y(127);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(127);
dydt(127)=dydt(127)-k(21)*y(182)*y(127);
dydt(326)=dydt(326)+k(21)*y(182)*y(127);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(127);
dydt(127)=dydt(127)-k(21)*y(197)*y(127);
dydt(322)=dydt(322)+k(21)*y(197)*y(127);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(127);
dydt(127)=dydt(127)-k(21)*y(32)*y(127);
dydt(133)=dydt(133)+k(21)*y(32)*y(127);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(127);
dydt(127)=dydt(127)-k(21)*y(2)*y(127);
dydt(329)=dydt(329)+k(21)*y(2)*y(127);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(127)=dydt(127)-k(17)*y(127);
dydt(314)=dydt(314)+k(17)*y(127);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

dydt(127)=dydt(127)-k(16)*y(127);
dydt(3)=dydt(3)+k(16)*y(127);
dydt(331)=dydt(331)+k(16)*y(127);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(127);
dydt(127)=dydt(127)-k(9)*y(3)*y(127);
dydt(125)=dydt(125)+k(9)*y(3)*y(127);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(127)=dydt(127)-k(8)*y(127);
dydt(16)=dydt(16)+k(8)*y(127);
dydt(61)=dydt(61)+k(8)*y(127);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) 

dydt(127)=dydt(127)-k(6)*y(127);
dydt(15)=dydt(15)+k(6)*y(127);
dydt(122)=dydt(122)+k(6)*y(127);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(128)=dydt(128)-k(32)*y(128);
dydt(124)=dydt(124)+k(32)*y(128);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(128)=dydt(128)-k(31)*y(128);
dydt(146)=dydt(146)+k(31)*y(128);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(128);
dydt(128)=dydt(128)-k(21)*y(182)*y(128);
dydt(330)=dydt(330)+k(21)*y(182)*y(128);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(128);
dydt(128)=dydt(128)-k(21)*y(197)*y(128);
dydt(334)=dydt(334)+k(21)*y(197)*y(128);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(128);
dydt(128)=dydt(128)-k(21)*y(32)*y(128);
dydt(135)=dydt(135)+k(21)*y(32)*y(128);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(128);
dydt(128)=dydt(128)-k(21)*y(2)*y(128);
dydt(139)=dydt(139)+k(21)*y(2)*y(128);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(128);
dydt(128)=dydt(128)-k(15)*y(3)*y(128);
dydt(125)=dydt(125)+k(15)*y(3)*y(128);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(128)=dydt(128)-k(14)*y(128);
dydt(151)=dydt(151)+k(14)*y(128);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

dydt(128)=dydt(128)-k(10)*y(128);
dydt(3)=dydt(3)+k(10)*y(128);
dydt(331)=dydt(331)+k(10)*y(128);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) 

dydt(128)=dydt(128)-k(8)*y(128);
dydt(72)=dydt(72)+k(8)*y(128);
dydt(126)=dydt(126)+k(8)*y(128);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(128)=dydt(128)-k(6)*y(128);
dydt(71)=dydt(71)+k(6)*y(128);
dydt(48)=dydt(48)+k(6)*y(128);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(129)=dydt(129)-k(25)*y(129);
dydt(130)=dydt(130)+k(25)*y(129);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(129)=dydt(129)-k(22)*y(129);
dydt(2)=dydt(2)+k(22)*y(129);
dydt(125)=dydt(125)+k(22)*y(129);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(129)=dydt(129)-k(16)*y(129);
dydt(3)=dydt(3)+k(16)*y(129);
dydt(139)=dydt(139)+k(16)*y(129);

% rule    : R5_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(129)=dydt(129)-k(14)*y(129);
dydt(539)=dydt(539)+k(14)*y(129);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(129)=dydt(129)-k(10)*y(129);
dydt(3)=dydt(3)+k(10)*y(129);
dydt(329)=dydt(329)+k(10)*y(129);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(129)=dydt(129)-k(8)*y(129);
dydt(61)=dydt(61)+k(8)*y(129);
dydt(141)=dydt(141)+k(8)*y(129);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(129)=dydt(129)-k(6)*y(129);
dydt(122)=dydt(122)+k(6)*y(129);
dydt(140)=dydt(140)+k(6)*y(129);

% rule    : R16
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(131)=dydt(131)-k(33)*y(131);
dydt(140)=dydt(140)+k(33)*y(131);

% rule    : R9_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(131)=dydt(131)-k(22)*y(131);
dydt(32)=dydt(32)+k(22)*y(131);
dydt(48)=dydt(48)+k(22)*y(131);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lyn(U, SH2) 

dydt(131)=dydt(131)-k(10)*y(131);
dydt(3)=dydt(3)+k(10)*y(131);
dydt(27)=dydt(27)+k(10)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(7)*y(131)*y(132);
dydt(132)=dydt(132)-k(7)*y(131)*y(132);
dydt(159)=dydt(159)+k(7)*y(131)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(7)*y(131)*y(141);
dydt(141)=dydt(141)-k(7)*y(131)*y(141);
dydt(165)=dydt(165)+k(7)*y(131)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(7)*y(131)*y(185);
dydt(185)=dydt(185)-k(7)*y(131)*y(185);
dydt(448)=dydt(448)+k(7)*y(131)*y(185);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(209)=dydt(209)-k(7)*y(209)*y(131);
dydt(131)=dydt(131)-k(7)*y(209)*y(131);
dydt(402)=dydt(402)+k(7)*y(209)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(214)=dydt(214)-k(7)*y(214)*y(131);
dydt(131)=dydt(131)-k(7)*y(214)*y(131);
dydt(447)=dydt(447)+k(7)*y(214)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(219)=dydt(219)-k(7)*y(219)*y(131);
dydt(131)=dydt(131)-k(7)*y(219)*y(131);
dydt(432)=dydt(432)+k(7)*y(219)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(7)*y(131)*y(223);
dydt(223)=dydt(223)-k(7)*y(131)*y(223);
dydt(284)=dydt(284)+k(7)*y(131)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(131)=dydt(131)-k(7)*y(131)*y(228);
dydt(228)=dydt(228)-k(7)*y(131)*y(228);
dydt(473)=dydt(473)+k(7)*y(131)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(315)=dydt(315)-k(7)*y(315)*y(131);
dydt(131)=dydt(131)-k(7)*y(315)*y(131);
dydt(443)=dydt(443)+k(7)*y(315)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(319)=dydt(319)-k(7)*y(319)*y(131);
dydt(131)=dydt(131)-k(7)*y(319)*y(131);
dydt(471)=dydt(471)+k(7)*y(319)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(7)*y(131)*y(336);
dydt(336)=dydt(336)-k(7)*y(131)*y(336);
dydt(459)=dydt(459)+k(7)*y(131)*y(336);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(437)=dydt(437)-k(7)*y(437)*y(131);
dydt(131)=dydt(131)-k(7)*y(437)*y(131);
dydt(440)=dydt(440)+k(7)*y(437)*y(131);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(176)=dydt(176)-k(5)*y(176)*y(131);
dydt(131)=dydt(131)-k(5)*y(176)*y(131);
dydt(364)=dydt(364)+k(5)*y(176)*y(131);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(191)=dydt(191)-k(5)*y(191)*y(131);
dydt(131)=dydt(131)-k(5)*y(191)*y(131);
dydt(163)=dydt(163)+k(5)*y(191)*y(131);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(256)=dydt(256)-k(5)*y(256)*y(131);
dydt(131)=dydt(131)-k(5)*y(256)*y(131);
dydt(362)=dydt(362)+k(5)*y(256)*y(131);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(264)=dydt(264)-k(5)*y(264)*y(131);
dydt(131)=dydt(131)-k(5)*y(264)*y(131);
dydt(358)=dydt(358)+k(5)*y(264)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(5)*y(131)*y(380);
dydt(380)=dydt(380)-k(5)*y(131)*y(380);
dydt(381)=dydt(381)+k(5)*y(131)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(5)*y(131)*y(391);
dydt(391)=dydt(391)-k(5)*y(131)*y(391);
dydt(365)=dydt(365)+k(5)*y(131)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(5)*y(131)*y(407);
dydt(407)=dydt(407)-k(5)*y(131)*y(407);
dydt(465)=dydt(465)+k(5)*y(131)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(5)*y(131)*y(441);
dydt(441)=dydt(441)-k(5)*y(131)*y(441);
dydt(159)=dydt(159)+k(5)*y(131)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(131)=dydt(131)-k(5)*y(131)*y(515);
dydt(515)=dydt(515)-k(5)*y(131)*y(515);
dydt(360)=dydt(360)+k(5)*y(131)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(131)=dydt(131)-k(5)*y(131)*y(543);
dydt(543)=dydt(543)-k(5)*y(131)*y(543);
dydt(600)=dydt(600)+k(5)*y(131)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(131)=dydt(131)-k(5)*y(131)*y(554);
dydt(554)=dydt(554)-k(5)*y(131)*y(554);
dydt(602)=dydt(602)+k(5)*y(131)*y(554);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(9)=dydt(9)-k(5)*y(9)*y(131);
dydt(131)=dydt(131)-k(5)*y(9)*y(131);
dydt(29)=dydt(29)+k(5)*y(9)*y(131);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(4)=dydt(4)-k(3)*y(4)*y(131);
dydt(131)=dydt(131)-k(3)*y(4)*y(131);
dydt(441)=dydt(441)+k(3)*y(4)*y(131);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(4)=dydt(4)-k(1)*y(4)*y(131);
dydt(131)=dydt(131)-k(1)*y(4)*y(131);
dydt(132)=dydt(132)+k(1)*y(4)*y(131);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(132)=dydt(132)-k(33)*y(132);
dydt(141)=dydt(141)+k(33)*y(132);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(132)=dydt(132)-k(22)*y(132);
dydt(32)=dydt(32)+k(22)*y(132);
dydt(126)=dydt(126)+k(22)*y(132);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(132)=dydt(132)-k(10)*y(132);
dydt(3)=dydt(3)+k(10)*y(132);
dydt(28)=dydt(28)+k(10)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(132);
dydt(132)=dydt(132)-k(7)*y(1)*y(132);
dydt(138)=dydt(138)+k(7)*y(1)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(7)*y(7)*y(132);
dydt(132)=dydt(132)-k(7)*y(7)*y(132);
dydt(29)=dydt(29)+k(7)*y(7)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(7)*y(335)*y(132);
dydt(132)=dydt(132)-k(7)*y(335)*y(132);
dydt(465)=dydt(465)+k(7)*y(335)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(132);
dydt(132)=dydt(132)-k(7)*y(257)*y(132);
dydt(362)=dydt(362)+k(7)*y(257)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(227)=dydt(227)-k(7)*y(227)*y(132);
dydt(132)=dydt(132)-k(7)*y(227)*y(132);
dydt(600)=dydt(600)+k(7)*y(227)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(222)=dydt(222)-k(7)*y(222)*y(132);
dydt(132)=dydt(132)-k(7)*y(222)*y(132);
dydt(360)=dydt(360)+k(7)*y(222)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(132);
dydt(132)=dydt(132)-k(7)*y(218)*y(132);
dydt(358)=dydt(358)+k(7)*y(218)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(132);
dydt(132)=dydt(132)-k(7)*y(192)*y(132);
dydt(163)=dydt(163)+k(7)*y(192)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(7)*y(184)*y(132);
dydt(132)=dydt(132)-k(7)*y(184)*y(132);
dydt(365)=dydt(365)+k(7)*y(184)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(132);
dydt(132)=dydt(132)-k(7)*y(177)*y(132);
dydt(364)=dydt(364)+k(7)*y(177)*y(132);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(7)*y(140)*y(132);
dydt(132)=dydt(132)-k(7)*y(140)*y(132);
dydt(381)=dydt(381)+k(7)*y(140)*y(132);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) 

dydt(132)=dydt(132)-k(2)*y(132);
dydt(4)=dydt(4)+k(2)*y(132);
dydt(131)=dydt(131)+k(2)*y(132);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(133)=dydt(133)-k(33)*y(133);
dydt(329)=dydt(329)+k(33)*y(133);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

dydt(133)=dydt(133)-k(22)*y(133);
dydt(32)=dydt(32)+k(22)*y(133);
dydt(127)=dydt(127)+k(22)*y(133);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(133)=dydt(133)-k(17)*y(133);
dydt(317)=dydt(317)+k(17)*y(133);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(133)=dydt(133)-k(16)*y(133);
dydt(3)=dydt(3)+k(16)*y(133);
dydt(136)=dydt(136)+k(16)*y(133);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(133);
dydt(133)=dydt(133)-k(9)*y(3)*y(133);
dydt(130)=dydt(130)+k(9)*y(3)*y(133);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(133)=dydt(133)-k(8)*y(133);
dydt(61)=dydt(61)+k(8)*y(133);
dydt(28)=dydt(28)+k(8)*y(133);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(133)=dydt(133)-k(6)*y(133);
dydt(122)=dydt(122)+k(6)*y(133);
dydt(27)=dydt(27)+k(6)*y(133);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(134)=dydt(134)-k(33)*y(134);
dydt(539)=dydt(539)+k(33)*y(134);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(134)=dydt(134)-k(32)*y(134);
dydt(130)=dydt(130)+k(32)*y(134);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(134)=dydt(134)-k(22)*y(134);
dydt(32)=dydt(32)+k(22)*y(134);
dydt(120)=dydt(120)+k(22)*y(134);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(134);
dydt(134)=dydt(134)-k(21)*y(182)*y(134);
dydt(360)=dydt(360)+k(21)*y(182)*y(134);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(134);
dydt(134)=dydt(134)-k(21)*y(197)*y(134);
dydt(600)=dydt(600)+k(21)*y(197)*y(134);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(134);
dydt(134)=dydt(134)-k(21)*y(32)*y(134);
dydt(602)=dydt(602)+k(21)*y(32)*y(134);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(134);
dydt(134)=dydt(134)-k(21)*y(2)*y(134);
dydt(357)=dydt(357)+k(21)*y(2)*y(134);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(134)=dydt(134)-k(16)*y(134);
dydt(3)=dydt(3)+k(16)*y(134);
dydt(137)=dydt(137)+k(16)*y(134);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(134)=dydt(134)-k(10)*y(134);
dydt(3)=dydt(3)+k(10)*y(134);
dydt(531)=dydt(531)+k(10)*y(134);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(134)=dydt(134)-k(8)*y(134);
dydt(54)=dydt(54)+k(8)*y(134);
dydt(132)=dydt(132)+k(8)*y(134);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(134)=dydt(134)-k(6)*y(134);
dydt(118)=dydt(118)+k(6)*y(134);
dydt(131)=dydt(131)+k(6)*y(134);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(136)=dydt(136)-k(33)*y(136);
dydt(142)=dydt(142)+k(33)*y(136);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(136)=dydt(136)-k(31)*y(136);
dydt(30)=dydt(30)+k(31)*y(136);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(136)=dydt(136)-k(22)*y(136);
dydt(32)=dydt(32)+k(22)*y(136);
dydt(331)=dydt(331)+k(22)*y(136);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(136);
dydt(136)=dydt(136)-k(15)*y(3)*y(136);
dydt(133)=dydt(133)+k(15)*y(3)*y(136);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(136);
dydt(136)=dydt(136)-k(9)*y(3)*y(136);
dydt(135)=dydt(135)+k(9)*y(3)*y(136);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

dydt(136)=dydt(136)-k(8)*y(136);
dydt(72)=dydt(72)+k(8)*y(136);
dydt(28)=dydt(28)+k(8)*y(136);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(136)=dydt(136)-k(6)*y(136);
dydt(71)=dydt(71)+k(6)*y(136);
dydt(27)=dydt(27)+k(6)*y(136);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(137)=dydt(137)-k(33)*y(137);
dydt(143)=dydt(143)+k(33)*y(137);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(137)=dydt(137)-k(32)*y(137);
dydt(135)=dydt(135)+k(32)*y(137);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(137)=dydt(137)-k(31)*y(137);
dydt(154)=dydt(154)+k(31)*y(137);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(137)=dydt(137)-k(22)*y(137);
dydt(32)=dydt(32)+k(22)*y(137);
dydt(151)=dydt(151)+k(22)*y(137);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(137);
dydt(137)=dydt(137)-k(21)*y(182)*y(137);
dydt(358)=dydt(358)+k(21)*y(182)*y(137);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(137);
dydt(137)=dydt(137)-k(21)*y(197)*y(137);
dydt(362)=dydt(362)+k(21)*y(197)*y(137);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(137);
dydt(137)=dydt(137)-k(21)*y(32)*y(137);
dydt(161)=dydt(161)+k(21)*y(32)*y(137);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(137);
dydt(137)=dydt(137)-k(21)*y(2)*y(137);
dydt(355)=dydt(355)+k(21)*y(2)*y(137);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(137);
dydt(137)=dydt(137)-k(15)*y(3)*y(137);
dydt(134)=dydt(134)+k(15)*y(3)*y(137);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(137)=dydt(137)-k(10)*y(137);
dydt(3)=dydt(3)+k(10)*y(137);
dydt(533)=dydt(533)+k(10)*y(137);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

dydt(137)=dydt(137)-k(8)*y(137);
dydt(77)=dydt(77)+k(8)*y(137);
dydt(132)=dydt(132)+k(8)*y(137);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(137)=dydt(137)-k(6)*y(137);
dydt(76)=dydt(76)+k(6)*y(137);
dydt(131)=dydt(131)+k(6)*y(137);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(138)=dydt(138)-k(33)*y(138);
dydt(144)=dydt(144)+k(33)*y(138);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(138)=dydt(138)-k(22)*y(138);
dydt(32)=dydt(32)+k(22)*y(138);
dydt(146)=dydt(146)+k(22)*y(138);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(138)=dydt(138)-k(14)*y(138);
dydt(154)=dydt(154)+k(14)*y(138);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(138)=dydt(138)-k(12)*y(138);
dydt(135)=dydt(135)+k(12)*y(138);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(138)=dydt(138)-k(10)*y(138);
dydt(3)=dydt(3)+k(10)*y(138);
dydt(30)=dydt(30)+k(10)*y(138);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(138);
dydt(138)=dydt(138)-k(9)*y(3)*y(138);
dydt(29)=dydt(29)+k(9)*y(3)*y(138);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

dydt(138)=dydt(138)-k(8)*y(138);
dydt(1)=dydt(1)+k(8)*y(138);
dydt(132)=dydt(132)+k(8)*y(138);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(138)=dydt(138)-k(6)*y(138);
dydt(6)=dydt(6)+k(6)*y(138);
dydt(131)=dydt(131)+k(6)*y(138);

% rule    : R9_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(140)=dydt(140)-k(22)*y(140);
dydt(2)=dydt(2)+k(22)*y(140);
dydt(48)=dydt(48)+k(22)*y(140);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lyn(U, SH2) 

dydt(140)=dydt(140)-k(10)*y(140);
dydt(3)=dydt(3)+k(10)*y(140);
dydt(21)=dydt(21)+k(10)*y(140);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(7)*y(140)*y(141);
dydt(141)=dydt(141)-k(7)*y(140)*y(141);
dydt(171)=dydt(171)+k(7)*y(140)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(7)*y(140)*y(185);
dydt(185)=dydt(185)-k(7)*y(140)*y(185);
dydt(460)=dydt(460)+k(7)*y(140)*y(185);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(209)=dydt(209)-k(7)*y(209)*y(140);
dydt(140)=dydt(140)-k(7)*y(209)*y(140);
dydt(374)=dydt(374)+k(7)*y(209)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(214)=dydt(214)-k(7)*y(214)*y(140);
dydt(140)=dydt(140)-k(7)*y(214)*y(140);
dydt(389)=dydt(389)+k(7)*y(214)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(219)=dydt(219)-k(7)*y(219)*y(140);
dydt(140)=dydt(140)-k(7)*y(219)*y(140);
dydt(429)=dydt(429)+k(7)*y(219)*y(140);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(7)*y(140)*y(223);
dydt(223)=dydt(223)-k(7)*y(140)*y(223);
dydt(279)=dydt(279)+k(7)*y(140)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(140)=dydt(140)-k(7)*y(140)*y(228);
dydt(228)=dydt(228)-k(7)*y(140)*y(228);
dydt(428)=dydt(428)+k(7)*y(140)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(315)=dydt(315)-k(7)*y(315)*y(140);
dydt(140)=dydt(140)-k(7)*y(315)*y(140);
dydt(424)=dydt(424)+k(7)*y(315)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(319)=dydt(319)-k(7)*y(319)*y(140);
dydt(140)=dydt(140)-k(7)*y(319)*y(140);
dydt(426)=dydt(426)+k(7)*y(319)*y(140);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(7)*y(140)*y(336);
dydt(336)=dydt(336)-k(7)*y(140)*y(336);
dydt(372)=dydt(372)+k(7)*y(140)*y(336);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(437)=dydt(437)-k(7)*y(437)*y(140);
dydt(140)=dydt(140)-k(7)*y(437)*y(140);
dydt(387)=dydt(387)+k(7)*y(437)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(176)=dydt(176)-k(5)*y(176)*y(140);
dydt(140)=dydt(140)-k(5)*y(176)*y(140);
dydt(175)=dydt(175)+k(5)*y(176)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(191)=dydt(191)-k(5)*y(191)*y(140);
dydt(140)=dydt(140)-k(5)*y(191)*y(140);
dydt(169)=dydt(169)+k(5)*y(191)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(256)=dydt(256)-k(5)*y(256)*y(140);
dydt(140)=dydt(140)-k(5)*y(256)*y(140);
dydt(605)=dydt(605)+k(5)*y(256)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(264)=dydt(264)-k(5)*y(264)*y(140);
dydt(140)=dydt(140)-k(5)*y(264)*y(140);
dydt(180)=dydt(180)+k(5)*y(264)*y(140);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(5)*y(140)*y(380);
dydt(380)=dydt(380)-k(5)*y(140)*y(380);
dydt(171)=dydt(171)+k(5)*y(140)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(5)*y(140)*y(391);
dydt(391)=dydt(391)-k(5)*y(140)*y(391);
dydt(178)=dydt(178)+k(5)*y(140)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(5)*y(140)*y(407);
dydt(407)=dydt(407)-k(5)*y(140)*y(407);
dydt(466)=dydt(466)+k(5)*y(140)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(5)*y(140)*y(441);
dydt(441)=dydt(441)-k(5)*y(140)*y(441);
dydt(165)=dydt(165)+k(5)*y(140)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(5)*y(140)*y(515);
dydt(515)=dydt(515)-k(5)*y(140)*y(515);
dydt(596)=dydt(596)+k(5)*y(140)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(140)=dydt(140)-k(5)*y(140)*y(543);
dydt(543)=dydt(543)-k(5)*y(140)*y(543);
dydt(544)=dydt(544)+k(5)*y(140)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(140)=dydt(140)-k(5)*y(140)*y(554);
dydt(554)=dydt(554)-k(5)*y(140)*y(554);
dydt(603)=dydt(603)+k(5)*y(140)*y(554);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(9)=dydt(9)-k(5)*y(9)*y(140);
dydt(140)=dydt(140)-k(5)*y(9)*y(140);
dydt(23)=dydt(23)+k(5)*y(9)*y(140);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(4)=dydt(4)-k(3)*y(4)*y(140);
dydt(140)=dydt(140)-k(3)*y(4)*y(140);
dydt(380)=dydt(380)+k(3)*y(4)*y(140);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(4)=dydt(4)-k(1)*y(4)*y(140);
dydt(140)=dydt(140)-k(1)*y(4)*y(140);
dydt(141)=dydt(141)+k(1)*y(4)*y(140);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(141)=dydt(141)-k(22)*y(141);
dydt(2)=dydt(2)+k(22)*y(141);
dydt(126)=dydt(126)+k(22)*y(141);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(141)=dydt(141)-k(10)*y(141);
dydt(3)=dydt(3)+k(10)*y(141);
dydt(22)=dydt(22)+k(10)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(141);
dydt(141)=dydt(141)-k(7)*y(1)*y(141);
dydt(144)=dydt(144)+k(7)*y(1)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(7)*y(7)*y(141);
dydt(141)=dydt(141)-k(7)*y(7)*y(141);
dydt(23)=dydt(23)+k(7)*y(7)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(7)*y(335)*y(141);
dydt(141)=dydt(141)-k(7)*y(335)*y(141);
dydt(466)=dydt(466)+k(7)*y(335)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(257)=dydt(257)-k(7)*y(257)*y(141);
dydt(141)=dydt(141)-k(7)*y(257)*y(141);
dydt(605)=dydt(605)+k(7)*y(257)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(227)=dydt(227)-k(7)*y(227)*y(141);
dydt(141)=dydt(141)-k(7)*y(227)*y(141);
dydt(544)=dydt(544)+k(7)*y(227)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(222)=dydt(222)-k(7)*y(222)*y(141);
dydt(141)=dydt(141)-k(7)*y(222)*y(141);
dydt(596)=dydt(596)+k(7)*y(222)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(141);
dydt(141)=dydt(141)-k(7)*y(218)*y(141);
dydt(180)=dydt(180)+k(7)*y(218)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(141);
dydt(141)=dydt(141)-k(7)*y(192)*y(141);
dydt(169)=dydt(169)+k(7)*y(192)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(7)*y(184)*y(141);
dydt(141)=dydt(141)-k(7)*y(184)*y(141);
dydt(178)=dydt(178)+k(7)*y(184)*y(141);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(141);
dydt(141)=dydt(141)-k(7)*y(177)*y(141);
dydt(175)=dydt(175)+k(7)*y(177)*y(141);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) 

dydt(141)=dydt(141)-k(2)*y(141);
dydt(4)=dydt(4)+k(2)*y(141);
dydt(140)=dydt(140)+k(2)*y(141);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(142)=dydt(142)-k(31)*y(142);
dydt(24)=dydt(24)+k(31)*y(142);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(142)=dydt(142)-k(22)*y(142);
dydt(2)=dydt(2)+k(22)*y(142);
dydt(331)=dydt(331)+k(22)*y(142);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(142);
dydt(142)=dydt(142)-k(15)*y(3)*y(142);
dydt(329)=dydt(329)+k(15)*y(3)*y(142);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(142);
dydt(142)=dydt(142)-k(9)*y(3)*y(142);
dydt(139)=dydt(139)+k(9)*y(3)*y(142);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

dydt(142)=dydt(142)-k(8)*y(142);
dydt(72)=dydt(72)+k(8)*y(142);
dydt(22)=dydt(22)+k(8)*y(142);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(142)=dydt(142)-k(6)*y(142);
dydt(71)=dydt(71)+k(6)*y(142);
dydt(21)=dydt(21)+k(6)*y(142);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(143)=dydt(143)-k(32)*y(143);
dydt(139)=dydt(139)+k(32)*y(143);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(143)=dydt(143)-k(31)*y(143);
dydt(145)=dydt(145)+k(31)*y(143);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(143)=dydt(143)-k(22)*y(143);
dydt(2)=dydt(2)+k(22)*y(143);
dydt(151)=dydt(151)+k(22)*y(143);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(143);
dydt(143)=dydt(143)-k(21)*y(182)*y(143);
dydt(180)=dydt(180)+k(21)*y(182)*y(143);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(143);
dydt(143)=dydt(143)-k(21)*y(197)*y(143);
dydt(605)=dydt(605)+k(21)*y(197)*y(143);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(143);
dydt(143)=dydt(143)-k(21)*y(32)*y(143);
dydt(167)=dydt(167)+k(21)*y(32)*y(143);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(143);
dydt(143)=dydt(143)-k(21)*y(2)*y(143);
dydt(173)=dydt(173)+k(21)*y(2)*y(143);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(143);
dydt(143)=dydt(143)-k(15)*y(3)*y(143);
dydt(539)=dydt(539)+k(15)*y(3)*y(143);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(143)=dydt(143)-k(10)*y(143);
dydt(3)=dydt(3)+k(10)*y(143);
dydt(508)=dydt(508)+k(10)*y(143);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

dydt(143)=dydt(143)-k(8)*y(143);
dydt(77)=dydt(77)+k(8)*y(143);
dydt(141)=dydt(141)+k(8)*y(143);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(143)=dydt(143)-k(6)*y(143);
dydt(76)=dydt(76)+k(6)*y(143);
dydt(140)=dydt(140)+k(6)*y(143);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(145)=dydt(145)-k(32)*y(145);
dydt(144)=dydt(144)+k(32)*y(145);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(145)=dydt(145)-k(22)*y(145);
dydt(2)=dydt(2)+k(22)*y(145);
dydt(147)=dydt(147)+k(22)*y(145);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(145);
dydt(145)=dydt(145)-k(21)*y(182)*y(145);
dydt(175)=dydt(175)+k(21)*y(182)*y(145);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(145);
dydt(145)=dydt(145)-k(21)*y(197)*y(145);
dydt(169)=dydt(169)+k(21)*y(197)*y(145);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(145);
dydt(145)=dydt(145)-k(21)*y(32)*y(145);
dydt(164)=dydt(164)+k(21)*y(32)*y(145);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(145);
dydt(145)=dydt(145)-k(21)*y(2)*y(145);
dydt(170)=dydt(170)+k(21)*y(2)*y(145);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(145)=dydt(145)-k(12)*y(145);
dydt(143)=dydt(143)+k(12)*y(145);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(145)=dydt(145)-k(10)*y(145);
dydt(3)=dydt(3)+k(10)*y(145);
dydt(452)=dydt(452)+k(10)*y(145);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(145);
dydt(145)=dydt(145)-k(9)*y(3)*y(145);
dydt(467)=dydt(467)+k(9)*y(3)*y(145);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

dydt(145)=dydt(145)-k(8)*y(145);
dydt(15)=dydt(15)+k(8)*y(145);
dydt(141)=dydt(141)+k(8)*y(145);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(145)=dydt(145)-k(6)*y(145);
dydt(98)=dydt(98)+k(6)*y(145);
dydt(140)=dydt(140)+k(6)*y(145);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(149)=dydt(149)-k(32)*y(149);
dydt(17)=dydt(17)+k(32)*y(149);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(149)=dydt(149)-k(32)*y(149);
dydt(463)=dydt(463)+k(32)*y(149);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(149);
dydt(149)=dydt(149)-k(21)*y(182)*y(149);
dydt(457)=dydt(457)+k(21)*y(182)*y(149);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(149);
dydt(149)=dydt(149)-k(21)*y(197)*y(149);
dydt(400)=dydt(400)+k(21)*y(197)*y(149);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(149);
dydt(149)=dydt(149)-k(21)*y(32)*y(149);
dydt(155)=dydt(155)+k(21)*y(32)*y(149);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(149);
dydt(149)=dydt(149)-k(21)*y(2)*y(149);
dydt(467)=dydt(467)+k(21)*y(2)*y(149);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(149);
dydt(149)=dydt(149)-k(21)*y(182)*y(149);
dydt(651)=dydt(651)+k(21)*y(182)*y(149);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(149);
dydt(149)=dydt(149)-k(21)*y(197)*y(149);
dydt(408)=dydt(408)+k(21)*y(197)*y(149);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(149);
dydt(149)=dydt(149)-k(21)*y(32)*y(149);
dydt(458)=dydt(458)+k(21)*y(32)*y(149);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(149);
dydt(149)=dydt(149)-k(21)*y(2)*y(149);
dydt(461)=dydt(461)+k(21)*y(2)*y(149);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) 

dydt(149)=dydt(149)-k(10)*y(149);
dydt(3)=dydt(3)+k(10)*y(149);
dydt(147)=dydt(147)+k(10)*y(149);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) 

dydt(149)=dydt(149)-k(10)*y(149);
dydt(3)=dydt(3)+k(10)*y(149);
dydt(404)=dydt(404)+k(10)*y(149);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(149)=dydt(149)-k(8)*y(149);
dydt(48)=dydt(48)+k(8)*y(149);
dydt(126)=dydt(126)+k(8)*y(149);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(149)=dydt(149)-k(6)*y(149);
dydt(48)=dydt(48)+k(6)*y(149);
dydt(47)=dydt(47)+k(6)*y(149);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(150)=dydt(150)-k(32)*y(150);
dydt(18)=dydt(18)+k(32)*y(150);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(150)=dydt(150)-k(32)*y(150);
dydt(97)=dydt(97)+k(32)*y(150);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(150);
dydt(150)=dydt(150)-k(21)*y(182)*y(150);
dydt(451)=dydt(451)+k(21)*y(182)*y(150);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(150);
dydt(150)=dydt(150)-k(21)*y(197)*y(150);
dydt(376)=dydt(376)+k(21)*y(197)*y(150);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(150);
dydt(150)=dydt(150)-k(21)*y(32)*y(150);
dydt(156)=dydt(156)+k(21)*y(32)*y(150);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(150);
dydt(150)=dydt(150)-k(21)*y(2)*y(150);
dydt(452)=dydt(452)+k(21)*y(2)*y(150);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(150);
dydt(150)=dydt(150)-k(21)*y(182)*y(150);
dydt(648)=dydt(648)+k(21)*y(182)*y(150);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(150);
dydt(150)=dydt(150)-k(21)*y(197)*y(150);
dydt(409)=dydt(409)+k(21)*y(197)*y(150);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(150);
dydt(150)=dydt(150)-k(21)*y(32)*y(150);
dydt(450)=dydt(450)+k(21)*y(32)*y(150);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(150);
dydt(150)=dydt(150)-k(21)*y(2)*y(150);
dydt(375)=dydt(375)+k(21)*y(2)*y(150);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(150);
dydt(150)=dydt(150)-k(9)*y(3)*y(150);
dydt(147)=dydt(147)+k(9)*y(3)*y(150);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(150);
dydt(150)=dydt(150)-k(9)*y(3)*y(150);
dydt(404)=dydt(404)+k(9)*y(3)*y(150);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) 

dydt(150)=dydt(150)-k(8)*y(150);
dydt(15)=dydt(15)+k(8)*y(150);
dydt(16)=dydt(16)+k(8)*y(150);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) 

dydt(150)=dydt(150)-k(6)*y(150);
dydt(15)=dydt(15)+k(6)*y(150);
dydt(98)=dydt(98)+k(6)*y(150);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(151)=dydt(151)-k(32)*y(151);
dydt(128)=dydt(128)+k(32)*y(151);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(151)=dydt(151)-k(32)*y(151);
dydt(108)=dydt(108)+k(32)*y(151);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) 

dydt(151)=dydt(151)-k(31)*y(151);
dydt(147)=dydt(147)+k(31)*y(151);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(151);
dydt(151)=dydt(151)-k(21)*y(182)*y(151);
dydt(343)=dydt(343)+k(21)*y(182)*y(151);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(151);
dydt(151)=dydt(151)-k(21)*y(197)*y(151);
dydt(337)=dydt(337)+k(21)*y(197)*y(151);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(151);
dydt(151)=dydt(151)-k(21)*y(32)*y(151);
dydt(137)=dydt(137)+k(21)*y(32)*y(151);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(151);
dydt(151)=dydt(151)-k(21)*y(2)*y(151);
dydt(143)=dydt(143)+k(21)*y(2)*y(151);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(151);
dydt(151)=dydt(151)-k(21)*y(182)*y(151);
dydt(361)=dydt(361)+k(21)*y(182)*y(151);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(151);
dydt(151)=dydt(151)-k(21)*y(197)*y(151);
dydt(599)=dydt(599)+k(21)*y(197)*y(151);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(151);
dydt(151)=dydt(151)-k(21)*y(32)*y(151);
dydt(350)=dydt(350)+k(21)*y(32)*y(151);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(151);
dydt(151)=dydt(151)-k(21)*y(2)*y(151);
dydt(353)=dydt(353)+k(21)*y(2)*y(151);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(151);
dydt(151)=dydt(151)-k(15)*y(3)*y(151);
dydt(120)=dydt(120)+k(15)*y(3)*y(151);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

dydt(151)=dydt(151)-k(10)*y(151);
dydt(3)=dydt(3)+k(10)*y(151);
dydt(610)=dydt(610)+k(10)*y(151);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) 

dydt(151)=dydt(151)-k(8)*y(151);
dydt(77)=dydt(77)+k(8)*y(151);
dydt(126)=dydt(126)+k(8)*y(151);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(151)=dydt(151)-k(6)*y(151);
dydt(76)=dydt(76)+k(6)*y(151);
dydt(48)=dydt(48)+k(6)*y(151);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(152)=dydt(152)-k(32)*y(152);
dydt(96)=dydt(96)+k(32)*y(152);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(152)=dydt(152)-k(24)*y(152);
dydt(153)=dydt(153)+k(24)*y(152);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(152)=dydt(152)-k(22)*y(152);
dydt(2)=dydt(2)+k(22)*y(152);
dydt(147)=dydt(147)+k(22)*y(152);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(152);
dydt(152)=dydt(152)-k(21)*y(182)*y(152);
dydt(174)=dydt(174)+k(21)*y(182)*y(152);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(152);
dydt(152)=dydt(152)-k(21)*y(197)*y(152);
dydt(371)=dydt(371)+k(21)*y(197)*y(152);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(152);
dydt(152)=dydt(152)-k(21)*y(32)*y(152);
dydt(157)=dydt(157)+k(21)*y(32)*y(152);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(152);
dydt(152)=dydt(152)-k(21)*y(2)*y(152);
dydt(170)=dydt(170)+k(21)*y(2)*y(152);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(152)=dydt(152)-k(12)*y(152);
dydt(353)=dydt(353)+k(12)*y(152);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(152)=dydt(152)-k(10)*y(152);
dydt(3)=dydt(3)+k(10)*y(152);
dydt(375)=dydt(375)+k(10)*y(152);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(152);
dydt(152)=dydt(152)-k(9)*y(3)*y(152);
dydt(461)=dydt(461)+k(9)*y(3)*y(152);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(152)=dydt(152)-k(8)*y(152);
dydt(126)=dydt(126)+k(8)*y(152);
dydt(21)=dydt(21)+k(8)*y(152);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(152)=dydt(152)-k(6)*y(152);
dydt(48)=dydt(48)+k(6)*y(152);
dydt(89)=dydt(89)+k(6)*y(152);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(153)=dydt(153)-k(33)*y(153);
dydt(152)=dydt(152)+k(33)*y(153);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(153)=dydt(153)-k(32)*y(153);
dydt(103)=dydt(103)+k(32)*y(153);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(153)=dydt(153)-k(22)*y(153);
dydt(32)=dydt(32)+k(22)*y(153);
dydt(147)=dydt(147)+k(22)*y(153);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(153);
dydt(153)=dydt(153)-k(21)*y(182)*y(153);
dydt(168)=dydt(168)+k(21)*y(182)*y(153);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(153);
dydt(153)=dydt(153)-k(21)*y(197)*y(153);
dydt(162)=dydt(162)+k(21)*y(197)*y(153);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(153);
dydt(153)=dydt(153)-k(21)*y(32)*y(153);
dydt(158)=dydt(158)+k(21)*y(32)*y(153);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(153);
dydt(153)=dydt(153)-k(21)*y(2)*y(153);
dydt(164)=dydt(164)+k(21)*y(2)*y(153);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(153)=dydt(153)-k(12)*y(153);
dydt(350)=dydt(350)+k(12)*y(153);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(153)=dydt(153)-k(10)*y(153);
dydt(3)=dydt(3)+k(10)*y(153);
dydt(450)=dydt(450)+k(10)*y(153);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(153);
dydt(153)=dydt(153)-k(9)*y(3)*y(153);
dydt(458)=dydt(458)+k(9)*y(3)*y(153);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(153)=dydt(153)-k(8)*y(153);
dydt(126)=dydt(126)+k(8)*y(153);
dydt(27)=dydt(27)+k(8)*y(153);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(153)=dydt(153)-k(6)*y(153);
dydt(48)=dydt(48)+k(6)*y(153);
dydt(86)=dydt(86)+k(6)*y(153);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(155)=dydt(155)-k(33)*y(155);
dydt(467)=dydt(467)+k(33)*y(155);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(155)=dydt(155)-k(32)*y(155);
dydt(29)=dydt(29)+k(32)*y(155);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(155)=dydt(155)-k(22)*y(155);
dydt(32)=dydt(32)+k(22)*y(155);
dydt(149)=dydt(149)+k(22)*y(155);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(155);
dydt(155)=dydt(155)-k(21)*y(182)*y(155);
dydt(365)=dydt(365)+k(21)*y(182)*y(155);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(155);
dydt(155)=dydt(155)-k(21)*y(197)*y(155);
dydt(465)=dydt(465)+k(21)*y(197)*y(155);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(155);
dydt(155)=dydt(155)-k(21)*y(32)*y(155);
dydt(159)=dydt(159)+k(21)*y(32)*y(155);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(155);
dydt(155)=dydt(155)-k(21)*y(2)*y(155);
dydt(381)=dydt(381)+k(21)*y(2)*y(155);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(155)=dydt(155)-k(10)*y(155);
dydt(3)=dydt(3)+k(10)*y(155);
dydt(154)=dydt(154)+k(10)*y(155);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(155)=dydt(155)-k(10)*y(155);
dydt(3)=dydt(3)+k(10)*y(155);
dydt(384)=dydt(384)+k(10)*y(155);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(155)=dydt(155)-k(8)*y(155);
dydt(48)=dydt(48)+k(8)*y(155);
dydt(132)=dydt(132)+k(8)*y(155);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(155)=dydt(155)-k(6)*y(155);
dydt(47)=dydt(47)+k(6)*y(155);
dydt(131)=dydt(131)+k(6)*y(155);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(156)=dydt(156)-k(33)*y(156);
dydt(452)=dydt(452)+k(33)*y(156);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(156)=dydt(156)-k(32)*y(156);
dydt(30)=dydt(30)+k(32)*y(156);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(156)=dydt(156)-k(22)*y(156);
dydt(32)=dydt(32)+k(22)*y(156);
dydt(150)=dydt(150)+k(22)*y(156);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(156);
dydt(156)=dydt(156)-k(21)*y(182)*y(156);
dydt(366)=dydt(366)+k(21)*y(182)*y(156);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(156);
dydt(156)=dydt(156)-k(21)*y(197)*y(156);
dydt(456)=dydt(456)+k(21)*y(197)*y(156);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(156);
dydt(156)=dydt(156)-k(21)*y(32)*y(156);
dydt(160)=dydt(160)+k(21)*y(32)*y(156);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(156);
dydt(156)=dydt(156)-k(21)*y(2)*y(156);
dydt(378)=dydt(378)+k(21)*y(2)*y(156);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(156);
dydt(156)=dydt(156)-k(9)*y(3)*y(156);
dydt(154)=dydt(154)+k(9)*y(3)*y(156);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(156);
dydt(156)=dydt(156)-k(9)*y(3)*y(156);
dydt(384)=dydt(384)+k(9)*y(3)*y(156);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

dydt(156)=dydt(156)-k(8)*y(156);
dydt(15)=dydt(15)+k(8)*y(156);
dydt(28)=dydt(28)+k(8)*y(156);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(156)=dydt(156)-k(6)*y(156);
dydt(98)=dydt(98)+k(6)*y(156);
dydt(27)=dydt(27)+k(6)*y(156);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(157)=dydt(157)-k(33)*y(157);
dydt(170)=dydt(170)+k(33)*y(157);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(157)=dydt(157)-k(28)*y(157);
dydt(364)=dydt(364)+k(28)*y(157);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(157)=dydt(157)-k(27)*y(157);
dydt(371)=dydt(371)+k(27)*y(157);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(157)=dydt(157)-k(24)*y(157);
dydt(158)=dydt(158)+k(24)*y(157);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(157)=dydt(157)-k(22)*y(157);
dydt(2)=dydt(2)+k(22)*y(157);
dydt(154)=dydt(154)+k(22)*y(157);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(157)=dydt(157)-k(22)*y(157);
dydt(32)=dydt(32)+k(22)*y(157);
dydt(152)=dydt(152)+k(22)*y(157);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(157)=dydt(157)-k(12)*y(157);
dydt(355)=dydt(355)+k(12)*y(157);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(157)=dydt(157)-k(10)*y(157);
dydt(3)=dydt(3)+k(10)*y(157);
dydt(378)=dydt(378)+k(10)*y(157);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(157);
dydt(157)=dydt(157)-k(9)*y(3)*y(157);
dydt(381)=dydt(381)+k(9)*y(3)*y(157);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(157)=dydt(157)-k(8)*y(157);
dydt(21)=dydt(21)+k(8)*y(157);
dydt(132)=dydt(132)+k(8)*y(157);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(157)=dydt(157)-k(6)*y(157);
dydt(89)=dydt(89)+k(6)*y(157);
dydt(131)=dydt(131)+k(6)*y(157);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(159)=dydt(159)-k(33)*y(159);
dydt(381)=dydt(381)+k(33)*y(159);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(159)=dydt(159)-k(33)*y(159);
dydt(165)=dydt(165)+k(33)*y(159);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(159)=dydt(159)-k(28)*y(159);
dydt(465)=dydt(465)+k(28)*y(159);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(159)=dydt(159)-k(27)*y(159);
dydt(459)=dydt(459)+k(27)*y(159);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(159)=dydt(159)-k(22)*y(159);
dydt(32)=dydt(32)+k(22)*y(159);
dydt(155)=dydt(155)+k(22)*y(159);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(159)=dydt(159)-k(22)*y(159);
dydt(32)=dydt(32)+k(22)*y(159);
dydt(458)=dydt(458)+k(22)*y(159);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(159)=dydt(159)-k(10)*y(159);
dydt(3)=dydt(3)+k(10)*y(159);
dydt(158)=dydt(158)+k(10)*y(159);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(159)=dydt(159)-k(10)*y(159);
dydt(3)=dydt(3)+k(10)*y(159);
dydt(446)=dydt(446)+k(10)*y(159);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(159)=dydt(159)-k(8)*y(159);
dydt(131)=dydt(131)+k(8)*y(159);
dydt(132)=dydt(132)+k(8)*y(159);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(159)=dydt(159)-k(6)*y(159);
dydt(131)=dydt(131)+k(6)*y(159);
dydt(441)=dydt(441)+k(6)*y(159);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(160)=dydt(160)-k(33)*y(160);
dydt(378)=dydt(378)+k(33)*y(160);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(160)=dydt(160)-k(33)*y(160);
dydt(166)=dydt(166)+k(33)*y(160);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(160)=dydt(160)-k(28)*y(160);
dydt(456)=dydt(456)+k(28)*y(160);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(160)=dydt(160)-k(27)*y(160);
dydt(454)=dydt(454)+k(27)*y(160);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(160)=dydt(160)-k(22)*y(160);
dydt(32)=dydt(32)+k(22)*y(160);
dydt(156)=dydt(156)+k(22)*y(160);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(160)=dydt(160)-k(22)*y(160);
dydt(32)=dydt(32)+k(22)*y(160);
dydt(450)=dydt(450)+k(22)*y(160);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(160);
dydt(160)=dydt(160)-k(9)*y(3)*y(160);
dydt(158)=dydt(158)+k(9)*y(3)*y(160);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(160);
dydt(160)=dydt(160)-k(9)*y(3)*y(160);
dydt(446)=dydt(446)+k(9)*y(3)*y(160);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(160)=dydt(160)-k(8)*y(160);
dydt(27)=dydt(27)+k(8)*y(160);
dydt(28)=dydt(28)+k(8)*y(160);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(160)=dydt(160)-k(6)*y(160);
dydt(27)=dydt(27)+k(6)*y(160);
dydt(86)=dydt(86)+k(6)*y(160);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(161)=dydt(161)-k(33)*y(161);
dydt(355)=dydt(355)+k(33)*y(161);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(161)=dydt(161)-k(33)*y(161);
dydt(167)=dydt(167)+k(33)*y(161);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(161)=dydt(161)-k(31)*y(161);
dydt(158)=dydt(158)+k(31)*y(161);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(161)=dydt(161)-k(28)*y(161);
dydt(362)=dydt(362)+k(28)*y(161);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(161)=dydt(161)-k(27)*y(161);
dydt(341)=dydt(341)+k(27)*y(161);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(161)=dydt(161)-k(22)*y(161);
dydt(32)=dydt(32)+k(22)*y(161);
dydt(137)=dydt(137)+k(22)*y(161);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(161)=dydt(161)-k(22)*y(161);
dydt(32)=dydt(32)+k(22)*y(161);
dydt(350)=dydt(350)+k(22)*y(161);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(161);
dydt(161)=dydt(161)-k(15)*y(3)*y(161);
dydt(602)=dydt(602)+k(15)*y(3)*y(161);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(161)=dydt(161)-k(10)*y(161);
dydt(3)=dydt(3)+k(10)*y(161);
dydt(612)=dydt(612)+k(10)*y(161);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(161)=dydt(161)-k(8)*y(161);
dydt(33)=dydt(33)+k(8)*y(161);
dydt(132)=dydt(132)+k(8)*y(161);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(161)=dydt(161)-k(6)*y(161);
dydt(82)=dydt(82)+k(6)*y(161);
dydt(131)=dydt(131)+k(6)*y(161);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(162)=dydt(162)-k(34)*y(162);
dydt(158)=dydt(158)+k(34)*y(162);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(162)=dydt(162)-k(33)*y(162);
dydt(371)=dydt(371)+k(33)*y(162);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(162)=dydt(162)-k(33)*y(162);
dydt(168)=dydt(168)+k(33)*y(162);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(162)=dydt(162)-k(30)*y(162);
dydt(370)=dydt(370)+k(30)*y(162);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(162)=dydt(162)-k(22)*y(162);
dydt(32)=dydt(32)+k(22)*y(162);
dydt(198)=dydt(198)+k(22)*y(162);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(162)=dydt(162)-k(22)*y(162);
dydt(197)=dydt(197)+k(22)*y(162);
dydt(153)=dydt(153)+k(22)*y(162);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(162)=dydt(162)-k(12)*y(162);
dydt(341)=dydt(341)+k(12)*y(162);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(162)=dydt(162)-k(10)*y(162);
dydt(3)=dydt(3)+k(10)*y(162);
dydt(454)=dydt(454)+k(10)*y(162);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(162);
dydt(162)=dydt(162)-k(9)*y(3)*y(162);
dydt(459)=dydt(459)+k(9)*y(3)*y(162);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(162)=dydt(162)-k(8)*y(162);
dydt(27)=dydt(27)+k(8)*y(162);
dydt(336)=dydt(336)+k(8)*y(162);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(162)=dydt(162)-k(6)*y(162);
dydt(86)=dydt(86)+k(6)*y(162);
dydt(335)=dydt(335)+k(6)*y(162);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(163)=dydt(163)-k(34)*y(163);
dydt(158)=dydt(158)+k(34)*y(163);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(163)=dydt(163)-k(33)*y(163);
dydt(364)=dydt(364)+k(33)*y(163);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(163)=dydt(163)-k(33)*y(163);
dydt(169)=dydt(169)+k(33)*y(163);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(163)=dydt(163)-k(29)*y(163);
dydt(370)=dydt(370)+k(29)*y(163);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(163)=dydt(163)-k(22)*y(163);
dydt(197)=dydt(197)+k(22)*y(163);
dydt(154)=dydt(154)+k(22)*y(163);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(163)=dydt(163)-k(22)*y(163);
dydt(32)=dydt(32)+k(22)*y(163);
dydt(196)=dydt(196)+k(22)*y(163);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(163)=dydt(163)-k(12)*y(163);
dydt(362)=dydt(362)+k(12)*y(163);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(163)=dydt(163)-k(10)*y(163);
dydt(3)=dydt(3)+k(10)*y(163);
dydt(456)=dydt(456)+k(10)*y(163);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(163);
dydt(163)=dydt(163)-k(9)*y(3)*y(163);
dydt(465)=dydt(465)+k(9)*y(3)*y(163);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(163)=dydt(163)-k(8)*y(163);
dydt(192)=dydt(192)+k(8)*y(163);
dydt(132)=dydt(132)+k(8)*y(163);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(163)=dydt(163)-k(6)*y(163);
dydt(191)=dydt(191)+k(6)*y(163);
dydt(131)=dydt(131)+k(6)*y(163);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(165)=dydt(165)-k(33)*y(165);
dydt(171)=dydt(171)+k(33)*y(165);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(165)=dydt(165)-k(28)*y(165);
dydt(466)=dydt(466)+k(28)*y(165);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(165)=dydt(165)-k(27)*y(165);
dydt(448)=dydt(448)+k(27)*y(165);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(165)=dydt(165)-k(23)*y(165);
dydt(159)=dydt(159)+k(23)*y(165);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(165)=dydt(165)-k(22)*y(165);
dydt(32)=dydt(32)+k(22)*y(165);
dydt(467)=dydt(467)+k(22)*y(165);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(165)=dydt(165)-k(22)*y(165);
dydt(2)=dydt(2)+k(22)*y(165);
dydt(458)=dydt(458)+k(22)*y(165);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(165)=dydt(165)-k(10)*y(165);
dydt(3)=dydt(3)+k(10)*y(165);
dydt(164)=dydt(164)+k(10)*y(165);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(165)=dydt(165)-k(10)*y(165);
dydt(3)=dydt(3)+k(10)*y(165);
dydt(445)=dydt(445)+k(10)*y(165);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(165)=dydt(165)-k(8)*y(165);
dydt(131)=dydt(131)+k(8)*y(165);
dydt(141)=dydt(141)+k(8)*y(165);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(165)=dydt(165)-k(6)*y(165);
dydt(140)=dydt(140)+k(6)*y(165);
dydt(441)=dydt(441)+k(6)*y(165);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(166)=dydt(166)-k(33)*y(166);
dydt(172)=dydt(172)+k(33)*y(166);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(166)=dydt(166)-k(28)*y(166);
dydt(453)=dydt(453)+k(28)*y(166);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(166)=dydt(166)-k(27)*y(166);
dydt(449)=dydt(449)+k(27)*y(166);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(166)=dydt(166)-k(22)*y(166);
dydt(32)=dydt(32)+k(22)*y(166);
dydt(452)=dydt(452)+k(22)*y(166);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(166)=dydt(166)-k(22)*y(166);
dydt(2)=dydt(2)+k(22)*y(166);
dydt(450)=dydt(450)+k(22)*y(166);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(166);
dydt(166)=dydt(166)-k(9)*y(3)*y(166);
dydt(164)=dydt(164)+k(9)*y(3)*y(166);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(166);
dydt(166)=dydt(166)-k(9)*y(3)*y(166);
dydt(445)=dydt(445)+k(9)*y(3)*y(166);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(166)=dydt(166)-k(8)*y(166);
dydt(27)=dydt(27)+k(8)*y(166);
dydt(22)=dydt(22)+k(8)*y(166);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(166)=dydt(166)-k(6)*y(166);
dydt(21)=dydt(21)+k(6)*y(166);
dydt(86)=dydt(86)+k(6)*y(166);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(167)=dydt(167)-k(33)*y(167);
dydt(173)=dydt(173)+k(33)*y(167);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(167)=dydt(167)-k(31)*y(167);
dydt(164)=dydt(164)+k(31)*y(167);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(167)=dydt(167)-k(28)*y(167);
dydt(605)=dydt(605)+k(28)*y(167);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(167)=dydt(167)-k(27)*y(167);
dydt(347)=dydt(347)+k(27)*y(167);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(167)=dydt(167)-k(22)*y(167);
dydt(32)=dydt(32)+k(22)*y(167);
dydt(143)=dydt(143)+k(22)*y(167);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(167)=dydt(167)-k(22)*y(167);
dydt(2)=dydt(2)+k(22)*y(167);
dydt(350)=dydt(350)+k(22)*y(167);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(167);
dydt(167)=dydt(167)-k(15)*y(3)*y(167);
dydt(603)=dydt(603)+k(15)*y(3)*y(167);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(167)=dydt(167)-k(10)*y(167);
dydt(3)=dydt(3)+k(10)*y(167);
dydt(575)=dydt(575)+k(10)*y(167);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(167)=dydt(167)-k(8)*y(167);
dydt(33)=dydt(33)+k(8)*y(167);
dydt(141)=dydt(141)+k(8)*y(167);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(167)=dydt(167)-k(6)*y(167);
dydt(82)=dydt(82)+k(6)*y(167);
dydt(140)=dydt(140)+k(6)*y(167);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(168)=dydt(168)-k(34)*y(168);
dydt(164)=dydt(164)+k(34)*y(168);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(168)=dydt(168)-k(33)*y(168);
dydt(174)=dydt(174)+k(33)*y(168);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(168)=dydt(168)-k(30)*y(168);
dydt(190)=dydt(190)+k(30)*y(168);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(168)=dydt(168)-k(22)*y(168);
dydt(32)=dydt(32)+k(22)*y(168);
dydt(189)=dydt(189)+k(22)*y(168);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(168)=dydt(168)-k(22)*y(168);
dydt(182)=dydt(182)+k(22)*y(168);
dydt(153)=dydt(153)+k(22)*y(168);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(168)=dydt(168)-k(12)*y(168);
dydt(347)=dydt(347)+k(12)*y(168);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(168)=dydt(168)-k(10)*y(168);
dydt(3)=dydt(3)+k(10)*y(168);
dydt(449)=dydt(449)+k(10)*y(168);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(168);
dydt(168)=dydt(168)-k(9)*y(3)*y(168);
dydt(448)=dydt(448)+k(9)*y(3)*y(168);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(168)=dydt(168)-k(8)*y(168);
dydt(27)=dydt(27)+k(8)*y(168);
dydt(185)=dydt(185)+k(8)*y(168);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(168)=dydt(168)-k(6)*y(168);
dydt(86)=dydt(86)+k(6)*y(168);
dydt(184)=dydt(184)+k(6)*y(168);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(169)=dydt(169)-k(34)*y(169);
dydt(164)=dydt(164)+k(34)*y(169);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(169)=dydt(169)-k(33)*y(169);
dydt(175)=dydt(175)+k(33)*y(169);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(169)=dydt(169)-k(29)*y(169);
dydt(190)=dydt(190)+k(29)*y(169);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(169)=dydt(169)-k(22)*y(169);
dydt(197)=dydt(197)+k(22)*y(169);
dydt(145)=dydt(145)+k(22)*y(169);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(169)=dydt(169)-k(22)*y(169);
dydt(2)=dydt(2)+k(22)*y(169);
dydt(196)=dydt(196)+k(22)*y(169);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(169)=dydt(169)-k(12)*y(169);
dydt(605)=dydt(605)+k(12)*y(169);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(169)=dydt(169)-k(10)*y(169);
dydt(3)=dydt(3)+k(10)*y(169);
dydt(453)=dydt(453)+k(10)*y(169);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(169);
dydt(169)=dydt(169)-k(9)*y(3)*y(169);
dydt(466)=dydt(466)+k(9)*y(3)*y(169);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(169)=dydt(169)-k(8)*y(169);
dydt(192)=dydt(192)+k(8)*y(169);
dydt(141)=dydt(141)+k(8)*y(169);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(169)=dydt(169)-k(6)*y(169);
dydt(191)=dydt(191)+k(6)*y(169);
dydt(140)=dydt(140)+k(6)*y(169);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(171)=dydt(171)-k(28)*y(171);
dydt(178)=dydt(178)+k(28)*y(171);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(171)=dydt(171)-k(27)*y(171);
dydt(460)=dydt(460)+k(27)*y(171);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(171)=dydt(171)-k(24)*y(171);
dydt(165)=dydt(165)+k(24)*y(171);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(171)=dydt(171)-k(23)*y(171);
dydt(381)=dydt(381)+k(23)*y(171);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(171)=dydt(171)-k(22)*y(171);
dydt(2)=dydt(2)+k(22)*y(171);
dydt(467)=dydt(467)+k(22)*y(171);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(171)=dydt(171)-k(22)*y(171);
dydt(2)=dydt(2)+k(22)*y(171);
dydt(461)=dydt(461)+k(22)*y(171);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(171)=dydt(171)-k(10)*y(171);
dydt(3)=dydt(3)+k(10)*y(171);
dydt(170)=dydt(170)+k(10)*y(171);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(171)=dydt(171)-k(10)*y(171);
dydt(3)=dydt(3)+k(10)*y(171);
dydt(386)=dydt(386)+k(10)*y(171);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(171)=dydt(171)-k(8)*y(171);
dydt(140)=dydt(140)+k(8)*y(171);
dydt(141)=dydt(141)+k(8)*y(171);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(171)=dydt(171)-k(6)*y(171);
dydt(140)=dydt(140)+k(6)*y(171);
dydt(380)=dydt(380)+k(6)*y(171);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(172)=dydt(172)-k(28)*y(172);
dydt(179)=dydt(179)+k(28)*y(172);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(172)=dydt(172)-k(27)*y(172);
dydt(377)=dydt(377)+k(27)*y(172);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(172)=dydt(172)-k(22)*y(172);
dydt(2)=dydt(2)+k(22)*y(172);
dydt(452)=dydt(452)+k(22)*y(172);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(172)=dydt(172)-k(22)*y(172);
dydt(2)=dydt(2)+k(22)*y(172);
dydt(375)=dydt(375)+k(22)*y(172);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(172);
dydt(172)=dydt(172)-k(9)*y(3)*y(172);
dydt(170)=dydt(170)+k(9)*y(3)*y(172);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(172);
dydt(172)=dydt(172)-k(9)*y(3)*y(172);
dydt(386)=dydt(386)+k(9)*y(3)*y(172);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(172)=dydt(172)-k(8)*y(172);
dydt(21)=dydt(21)+k(8)*y(172);
dydt(22)=dydt(22)+k(8)*y(172);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(172)=dydt(172)-k(6)*y(172);
dydt(21)=dydt(21)+k(6)*y(172);
dydt(89)=dydt(89)+k(6)*y(172);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(173)=dydt(173)-k(31)*y(173);
dydt(170)=dydt(170)+k(31)*y(173);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(173)=dydt(173)-k(28)*y(173);
dydt(180)=dydt(180)+k(28)*y(173);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(173)=dydt(173)-k(27)*y(173);
dydt(346)=dydt(346)+k(27)*y(173);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(173)=dydt(173)-k(24)*y(173);
dydt(167)=dydt(167)+k(24)*y(173);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(173)=dydt(173)-k(22)*y(173);
dydt(2)=dydt(2)+k(22)*y(173);
dydt(143)=dydt(143)+k(22)*y(173);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(173)=dydt(173)-k(22)*y(173);
dydt(2)=dydt(2)+k(22)*y(173);
dydt(353)=dydt(353)+k(22)*y(173);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(173);
dydt(173)=dydt(173)-k(15)*y(3)*y(173);
dydt(604)=dydt(604)+k(15)*y(3)*y(173);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(173)=dydt(173)-k(10)*y(173);
dydt(3)=dydt(3)+k(10)*y(173);
dydt(506)=dydt(506)+k(10)*y(173);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(173)=dydt(173)-k(8)*y(173);
dydt(111)=dydt(111)+k(8)*y(173);
dydt(141)=dydt(141)+k(8)*y(173);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(173)=dydt(173)-k(6)*y(173);
dydt(110)=dydt(110)+k(6)*y(173);
dydt(140)=dydt(140)+k(6)*y(173);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(174)=dydt(174)-k(34)*y(174);
dydt(170)=dydt(170)+k(34)*y(174);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(174)=dydt(174)-k(30)*y(174);
dydt(183)=dydt(183)+k(30)*y(174);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(174)=dydt(174)-k(24)*y(174);
dydt(168)=dydt(168)+k(24)*y(174);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(174)=dydt(174)-k(22)*y(174);
dydt(2)=dydt(2)+k(22)*y(174);
dydt(189)=dydt(189)+k(22)*y(174);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(174)=dydt(174)-k(22)*y(174);
dydt(182)=dydt(182)+k(22)*y(174);
dydt(152)=dydt(152)+k(22)*y(174);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(174)=dydt(174)-k(12)*y(174);
dydt(346)=dydt(346)+k(12)*y(174);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(174)=dydt(174)-k(10)*y(174);
dydt(3)=dydt(3)+k(10)*y(174);
dydt(377)=dydt(377)+k(10)*y(174);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(174);
dydt(174)=dydt(174)-k(9)*y(3)*y(174);
dydt(460)=dydt(460)+k(9)*y(3)*y(174);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(174)=dydt(174)-k(8)*y(174);
dydt(21)=dydt(21)+k(8)*y(174);
dydt(185)=dydt(185)+k(8)*y(174);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(174)=dydt(174)-k(6)*y(174);
dydt(89)=dydt(89)+k(6)*y(174);
dydt(184)=dydt(184)+k(6)*y(174);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(176)=dydt(176)-k(34)*y(176);
dydt(89)=dydt(89)+k(34)*y(176);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~pY) 

dydt(176)=dydt(176)-k(22)*y(176);
dydt(182)=dydt(182)+k(22)*y(176);
dydt(98)=dydt(98)+k(22)*y(176);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(176);
dydt(176)=dydt(176)-k(9)*y(3)*y(176);
dydt(391)=dydt(391)+k(9)*y(3)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(1)=dydt(1)-k(5)*y(1)*y(176);
dydt(176)=dydt(176)-k(5)*y(1)*y(176);
dydt(418)=dydt(418)+k(5)*y(1)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(7)=dydt(7)-k(5)*y(7)*y(176);
dydt(176)=dydt(176)-k(5)*y(7)*y(176);
dydt(606)=dydt(606)+k(5)*y(7)*y(176);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(176)=dydt(176)-k(5)*y(176)*y(335);
dydt(335)=dydt(335)-k(5)*y(176)*y(335);
dydt(367)=dydt(367)+k(5)*y(176)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(257)=dydt(257)-k(5)*y(257)*y(176);
dydt(176)=dydt(176)-k(5)*y(257)*y(176);
dydt(487)=dydt(487)+k(5)*y(257)*y(176);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(176)=dydt(176)-k(5)*y(176)*y(227);
dydt(227)=dydt(227)-k(5)*y(176)*y(227);
dydt(486)=dydt(486)+k(5)*y(176)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(176)=dydt(176)-k(5)*y(176)*y(222);
dydt(222)=dydt(222)-k(5)*y(176)*y(222);
dydt(273)=dydt(273)+k(5)*y(176)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(218)=dydt(218)-k(5)*y(218)*y(176);
dydt(176)=dydt(176)-k(5)*y(218)*y(176);
dydt(580)=dydt(580)+k(5)*y(218)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(192)=dydt(192)-k(5)*y(192)*y(176);
dydt(176)=dydt(176)-k(5)*y(192)*y(176);
dydt(369)=dydt(369)+k(5)*y(192)*y(176);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(176)=dydt(176)-k(5)*y(176)*y(184);
dydt(184)=dydt(184)-k(5)*y(176)*y(184);
dydt(183)=dydt(183)+k(5)*y(176)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(177)=dydt(177)-k(5)*y(177)*y(176);
dydt(176)=dydt(176)-k(5)*y(177)*y(176);
dydt(187)=dydt(187)+k(5)*y(177)*y(176);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) 

dydt(176)=dydt(176)-k(4)*y(176);
dydt(4)=dydt(4)+k(4)*y(176);
dydt(177)=dydt(177)+k(4)*y(176);

% rule    : R17
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(177)=dydt(177)-k(34)*y(177);
dydt(21)=dydt(21)+k(34)*y(177);

% rule    : R9_op
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(177)=dydt(177)-k(22)*y(177);
dydt(182)=dydt(182)+k(22)*y(177);
dydt(15)=dydt(15)+k(22)*y(177);

% rule    : R3
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(177);
dydt(177)=dydt(177)-k(9)*y(3)*y(177);
dydt(184)=dydt(184)+k(9)*y(3)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(185);
dydt(185)=dydt(185)-k(7)*y(177)*y(185);
dydt(183)=dydt(183)+k(7)*y(177)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(209);
dydt(209)=dydt(209)-k(7)*y(177)*y(209);
dydt(369)=dydt(369)+k(7)*y(177)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(214);
dydt(214)=dydt(214)-k(7)*y(177)*y(214);
dydt(187)=dydt(187)+k(7)*y(177)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(219);
dydt(219)=dydt(219)-k(7)*y(177)*y(219);
dydt(580)=dydt(580)+k(7)*y(177)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(223);
dydt(223)=dydt(223)-k(7)*y(177)*y(223);
dydt(273)=dydt(273)+k(7)*y(177)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(228);
dydt(228)=dydt(228)-k(7)*y(177)*y(228);
dydt(486)=dydt(486)+k(7)*y(177)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(315)=dydt(315)-k(7)*y(315)*y(177);
dydt(177)=dydt(177)-k(7)*y(315)*y(177);
dydt(421)=dydt(421)+k(7)*y(315)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(319);
dydt(319)=dydt(319)-k(7)*y(177)*y(319);
dydt(487)=dydt(487)+k(7)*y(177)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(336);
dydt(336)=dydt(336)-k(7)*y(177)*y(336);
dydt(367)=dydt(367)+k(7)*y(177)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(177)=dydt(177)-k(7)*y(177)*y(437);
dydt(437)=dydt(437)-k(7)*y(177)*y(437);
dydt(271)=dydt(271)+k(7)*y(177)*y(437);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(177)=dydt(177)-k(5)*y(177)*y(191);
dydt(191)=dydt(191)-k(5)*y(177)*y(191);
dydt(194)=dydt(194)+k(5)*y(177)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(177)=dydt(177)-k(5)*y(177)*y(256);
dydt(256)=dydt(256)-k(5)*y(177)*y(256);
dydt(550)=dydt(550)+k(5)*y(177)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(177)=dydt(177)-k(5)*y(177)*y(264);
dydt(264)=dydt(264)-k(5)*y(177)*y(264);
dydt(513)=dydt(513)+k(5)*y(177)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(177)=dydt(177)-k(5)*y(177)*y(380);
dydt(380)=dydt(380)-k(5)*y(177)*y(380);
dydt(389)=dydt(389)+k(5)*y(177)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(177)=dydt(177)-k(5)*y(177)*y(391);
dydt(391)=dydt(391)-k(5)*y(177)*y(391);
dydt(394)=dydt(394)+k(5)*y(177)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(177)=dydt(177)-k(5)*y(177)*y(407);
dydt(407)=dydt(407)-k(5)*y(177)*y(407);
dydt(435)=dydt(435)+k(5)*y(177)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(177)=dydt(177)-k(5)*y(177)*y(441);
dydt(441)=dydt(441)-k(5)*y(177)*y(441);
dydt(447)=dydt(447)+k(5)*y(177)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(177)=dydt(177)-k(5)*y(177)*y(515);
dydt(515)=dydt(515)-k(5)*y(177)*y(515);
dydt(514)=dydt(514)+k(5)*y(177)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(177)=dydt(177)-k(5)*y(177)*y(543);
dydt(543)=dydt(543)-k(5)*y(177)*y(543);
dydt(548)=dydt(548)+k(5)*y(177)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(177)=dydt(177)-k(5)*y(177)*y(554);
dydt(554)=dydt(554)-k(5)*y(177)*y(554);
dydt(553)=dydt(553)+k(5)*y(177)*y(554);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(9)=dydt(9)-k(5)*y(9)*y(177);
dydt(177)=dydt(177)-k(5)*y(9)*y(177);
dydt(213)=dydt(213)+k(5)*y(9)*y(177);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(4)=dydt(4)-k(3)*y(4)*y(177);
dydt(177)=dydt(177)-k(3)*y(4)*y(177);
dydt(176)=dydt(176)+k(3)*y(4)*y(177);

% rule    : R1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(4)=dydt(4)-k(1)*y(4)*y(177);
dydt(177)=dydt(177)-k(1)*y(4)*y(177);
dydt(214)=dydt(214)+k(1)*y(4)*y(177);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(178)=dydt(178)-k(34)*y(178);
dydt(171)=dydt(171)+k(34)*y(178);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(178)=dydt(178)-k(29)*y(178);
dydt(186)=dydt(186)+k(29)*y(178);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(178)=dydt(178)-k(24)*y(178);
dydt(466)=dydt(466)+k(24)*y(178);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(178)=dydt(178)-k(23)*y(178);
dydt(365)=dydt(365)+k(23)*y(178);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(178)=dydt(178)-k(22)*y(178);
dydt(182)=dydt(182)+k(22)*y(178);
dydt(467)=dydt(467)+k(22)*y(178);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(178)=dydt(178)-k(22)*y(178);
dydt(2)=dydt(2)+k(22)*y(178);
dydt(651)=dydt(651)+k(22)*y(178);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(178)=dydt(178)-k(10)*y(178);
dydt(3)=dydt(3)+k(10)*y(178);
dydt(175)=dydt(175)+k(10)*y(178);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(178)=dydt(178)-k(10)*y(178);
dydt(3)=dydt(3)+k(10)*y(178);
dydt(390)=dydt(390)+k(10)*y(178);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(178)=dydt(178)-k(8)*y(178);
dydt(184)=dydt(184)+k(8)*y(178);
dydt(141)=dydt(141)+k(8)*y(178);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(178)=dydt(178)-k(6)*y(178);
dydt(140)=dydt(140)+k(6)*y(178);
dydt(391)=dydt(391)+k(6)*y(178);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(179)=dydt(179)-k(34)*y(179);
dydt(172)=dydt(172)+k(34)*y(179);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(179)=dydt(179)-k(29)*y(179);
dydt(187)=dydt(187)+k(29)*y(179);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(179)=dydt(179)-k(22)*y(179);
dydt(182)=dydt(182)+k(22)*y(179);
dydt(452)=dydt(452)+k(22)*y(179);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(179)=dydt(179)-k(22)*y(179);
dydt(2)=dydt(2)+k(22)*y(179);
dydt(648)=dydt(648)+k(22)*y(179);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(179);
dydt(179)=dydt(179)-k(9)*y(3)*y(179);
dydt(175)=dydt(175)+k(9)*y(3)*y(179);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(179);
dydt(179)=dydt(179)-k(9)*y(3)*y(179);
dydt(390)=dydt(390)+k(9)*y(3)*y(179);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(179)=dydt(179)-k(8)*y(179);
dydt(177)=dydt(177)+k(8)*y(179);
dydt(22)=dydt(22)+k(8)*y(179);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(179)=dydt(179)-k(6)*y(179);
dydt(21)=dydt(21)+k(6)*y(179);
dydt(176)=dydt(176)+k(6)*y(179);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(180)=dydt(180)-k(34)*y(180);
dydt(173)=dydt(173)+k(34)*y(180);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(180)=dydt(180)-k(31)*y(180);
dydt(175)=dydt(175)+k(31)*y(180);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(180)=dydt(180)-k(29)*y(180);
dydt(188)=dydt(188)+k(29)*y(180);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(180)=dydt(180)-k(24)*y(180);
dydt(605)=dydt(605)+k(24)*y(180);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(180)=dydt(180)-k(22)*y(180);
dydt(182)=dydt(182)+k(22)*y(180);
dydt(143)=dydt(143)+k(22)*y(180);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(180)=dydt(180)-k(22)*y(180);
dydt(2)=dydt(2)+k(22)*y(180);
dydt(361)=dydt(361)+k(22)*y(180);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(180);
dydt(180)=dydt(180)-k(15)*y(3)*y(180);
dydt(596)=dydt(596)+k(15)*y(3)*y(180);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(180)=dydt(180)-k(10)*y(180);
dydt(3)=dydt(3)+k(10)*y(180);
dydt(510)=dydt(510)+k(10)*y(180);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(180)=dydt(180)-k(8)*y(180);
dydt(218)=dydt(218)+k(8)*y(180);
dydt(141)=dydt(141)+k(8)*y(180);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(180)=dydt(180)-k(6)*y(180);
dydt(264)=dydt(264)+k(6)*y(180);
dydt(140)=dydt(140)+k(6)*y(180);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(181)=dydt(181)-k(34)*y(181);
dydt(152)=dydt(152)+k(34)*y(181);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(181)=dydt(181)-k(32)*y(181);
dydt(606)=dydt(606)+k(32)*y(181);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(181)=dydt(181)-k(24)*y(181);
dydt(196)=dydt(196)+k(24)*y(181);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(181)=dydt(181)-k(22)*y(181);
dydt(182)=dydt(182)+k(22)*y(181);
dydt(147)=dydt(147)+k(22)*y(181);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(181);
dydt(181)=dydt(181)-k(21)*y(182)*y(181);
dydt(183)=dydt(183)+k(21)*y(182)*y(181);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(181);
dydt(181)=dydt(181)-k(21)*y(197)*y(181);
dydt(367)=dydt(367)+k(21)*y(197)*y(181);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(181);
dydt(181)=dydt(181)-k(21)*y(32)*y(181);
dydt(364)=dydt(364)+k(21)*y(32)*y(181);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(181);
dydt(181)=dydt(181)-k(21)*y(2)*y(181);
dydt(175)=dydt(175)+k(21)*y(2)*y(181);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(181)=dydt(181)-k(12)*y(181);
dydt(361)=dydt(361)+k(12)*y(181);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(181)=dydt(181)-k(10)*y(181);
dydt(3)=dydt(3)+k(10)*y(181);
dydt(648)=dydt(648)+k(10)*y(181);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(181);
dydt(181)=dydt(181)-k(9)*y(3)*y(181);
dydt(651)=dydt(651)+k(9)*y(3)*y(181);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(181)=dydt(181)-k(8)*y(181);
dydt(126)=dydt(126)+k(8)*y(181);
dydt(177)=dydt(177)+k(8)*y(181);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(181)=dydt(181)-k(6)*y(181);
dydt(48)=dydt(48)+k(6)*y(181);
dydt(176)=dydt(176)+k(6)*y(181);

% rule    : R19
% reaction: Syk(tSH2, l~Y, a~pY) -> Syk(tSH2, l~Y, a~Y) 

dydt(182)=dydt(182)-k(36)*y(182);
dydt(2)=dydt(2)+k(36)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(14);
dydt(14)=dydt(14)-k(21)*y(182)*y(14);
dydt(213)=dydt(213)+k(21)*y(182)*y(14);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(46);
dydt(46)=dydt(46)-k(21)*y(182)*y(46);
dydt(270)=dydt(270)+k(21)*y(182)*y(46);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(53);
dydt(53)=dydt(53)-k(21)*y(182)*y(53);
dydt(422)=dydt(422)+k(21)*y(182)*y(53);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(53);
dydt(53)=dydt(53)-k(21)*y(182)*y(53);
dydt(225)=dydt(225)+k(21)*y(182)*y(53);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(60);
dydt(60)=dydt(60)-k(21)*y(182)*y(60);
dydt(298)=dydt(298)+k(21)*y(182)*y(60);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(75);
dydt(75)=dydt(75)-k(21)*y(182)*y(75);
dydt(300)=dydt(300)+k(21)*y(182)*y(75);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(97);
dydt(97)=dydt(97)-k(21)*y(182)*y(97);
dydt(418)=dydt(418)+k(21)*y(182)*y(97);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(117);
dydt(117)=dydt(117)-k(21)*y(182)*y(117);
dydt(593)=dydt(593)+k(21)*y(182)*y(117);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(125);
dydt(125)=dydt(125)-k(21)*y(182)*y(125);
dydt(327)=dydt(327)+k(21)*y(182)*y(125);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(146);
dydt(146)=dydt(146)-k(21)*y(182)*y(146);
dydt(652)=dydt(652)+k(21)*y(182)*y(146);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(147);
dydt(147)=dydt(147)-k(21)*y(182)*y(147);
dydt(181)=dydt(181)+k(21)*y(182)*y(147);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(147);
dydt(147)=dydt(147)-k(21)*y(182)*y(147);
dydt(189)=dydt(189)+k(21)*y(182)*y(147);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(154);
dydt(154)=dydt(154)-k(21)*y(182)*y(154);
dydt(364)=dydt(364)+k(21)*y(182)*y(154);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(233);
dydt(233)=dydt(233)-k(21)*y(182)*y(233);
dydt(224)=dydt(224)+k(21)*y(182)*y(233);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(235);
dydt(235)=dydt(235)-k(21)*y(182)*y(235);
dydt(646)=dydt(646)+k(21)*y(182)*y(235);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(243);
dydt(243)=dydt(243)-k(21)*y(182)*y(243);
dydt(493)=dydt(493)+k(21)*y(182)*y(243);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(250);
dydt(250)=dydt(250)-k(21)*y(182)*y(250);
dydt(263)=dydt(263)+k(21)*y(182)*y(250);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(291);
dydt(291)=dydt(291)-k(21)*y(182)*y(291);
dydt(288)=dydt(288)+k(21)*y(182)*y(291);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(309);
dydt(309)=dydt(309)-k(21)*y(182)*y(309);
dydt(303)=dydt(303)+k(21)*y(182)*y(309);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(314);
dydt(314)=dydt(314)-k(21)*y(182)*y(314);
dydt(323)=dydt(323)+k(21)*y(182)*y(314);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(331);
dydt(331)=dydt(331)-k(21)*y(182)*y(331);
dydt(328)=dydt(328)+k(21)*y(182)*y(331);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(337);
dydt(337)=dydt(337)-k(21)*y(182)*y(337);
dydt(363)=dydt(363)+k(21)*y(182)*y(337);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(343);
dydt(343)=dydt(343)-k(21)*y(182)*y(343);
dydt(188)=dydt(188)+k(21)*y(182)*y(343);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(350);
dydt(350)=dydt(350)-k(21)*y(182)*y(350);
dydt(347)=dydt(347)+k(21)*y(182)*y(350);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(353);
dydt(353)=dydt(353)-k(21)*y(182)*y(353);
dydt(346)=dydt(346)+k(21)*y(182)*y(353);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(399);
dydt(399)=dydt(399)-k(21)*y(182)*y(399);
dydt(397)=dydt(397)+k(21)*y(182)*y(399);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(404);
dydt(404)=dydt(404)-k(21)*y(182)*y(404);
dydt(393)=dydt(393)+k(21)*y(182)*y(404);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(404);
dydt(404)=dydt(404)-k(21)*y(182)*y(404);
dydt(396)=dydt(396)+k(21)*y(182)*y(404);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(406);
dydt(406)=dydt(406)-k(21)*y(182)*y(406);
dydt(435)=dydt(435)+k(21)*y(182)*y(406);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(420);
dydt(420)=dydt(420)-k(21)*y(182)*y(420);
dydt(395)=dydt(395)+k(21)*y(182)*y(420);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(424);
dydt(424)=dydt(424)-k(21)*y(182)*y(424);
dydt(429)=dydt(429)+k(21)*y(182)*y(424);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(430);
dydt(430)=dydt(430)-k(21)*y(182)*y(430);
dydt(395)=dydt(395)+k(21)*y(182)*y(430);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(451);
dydt(451)=dydt(451)-k(21)*y(182)*y(451);
dydt(187)=dydt(187)+k(21)*y(182)*y(451);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(452);
dydt(452)=dydt(452)-k(21)*y(182)*y(452);
dydt(179)=dydt(179)+k(21)*y(182)*y(452);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(450);
dydt(450)=dydt(450)-k(21)*y(182)*y(450);
dydt(449)=dydt(449)+k(21)*y(182)*y(450);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(461);
dydt(461)=dydt(461)-k(21)*y(182)*y(461);
dydt(460)=dydt(460)+k(21)*y(182)*y(461);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(463);
dydt(463)=dydt(463)-k(21)*y(182)*y(463);
dydt(417)=dydt(417)+k(21)*y(182)*y(463);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(467);
dydt(467)=dydt(467)-k(21)*y(182)*y(467);
dydt(178)=dydt(178)+k(21)*y(182)*y(467);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(458);
dydt(458)=dydt(458)-k(21)*y(182)*y(458);
dydt(448)=dydt(448)+k(21)*y(182)*y(458);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(457);
dydt(457)=dydt(457)-k(21)*y(182)*y(457);
dydt(186)=dydt(186)+k(21)*y(182)*y(457);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(531);
dydt(531)=dydt(531)-k(21)*y(182)*y(531);
dydt(532)=dydt(532)+k(21)*y(182)*y(531);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(538);
dydt(538)=dydt(538)-k(21)*y(182)*y(538);
dydt(511)=dydt(511)+k(21)*y(182)*y(538);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(578);
dydt(578)=dydt(578)-k(21)*y(182)*y(578);
dydt(577)=dydt(577)+k(21)*y(182)*y(578);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(590);
dydt(590)=dydt(590)-k(21)*y(182)*y(590);
dydt(304)=dydt(304)+k(21)*y(182)*y(590);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(595);
dydt(595)=dydt(595)-k(21)*y(182)*y(595);
dydt(516)=dydt(516)+k(21)*y(182)*y(595);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(598);
dydt(598)=dydt(598)-k(21)*y(182)*y(598);
dydt(549)=dydt(549)+k(21)*y(182)*y(598);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(599);
dydt(599)=dydt(599)-k(21)*y(182)*y(599);
dydt(195)=dydt(195)+k(21)*y(182)*y(599);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(585);
dydt(585)=dydt(585)-k(21)*y(182)*y(585);
dydt(550)=dydt(550)+k(21)*y(182)*y(585);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(610);
dydt(610)=dydt(610)-k(21)*y(182)*y(610);
dydt(512)=dydt(512)+k(21)*y(182)*y(610);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(610);
dydt(610)=dydt(610)-k(21)*y(182)*y(610);
dydt(344)=dydt(344)+k(21)*y(182)*y(610);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(584);
dydt(584)=dydt(584)-k(21)*y(182)*y(584);
dydt(497)=dydt(497)+k(21)*y(182)*y(584);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(584);
dydt(584)=dydt(584)-k(21)*y(182)*y(584);
dydt(576)=dydt(576)+k(21)*y(182)*y(584);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(614);
dydt(614)=dydt(614)-k(21)*y(182)*y(614);
dydt(421)=dydt(421)+k(21)*y(182)*y(614);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(614);
dydt(614)=dydt(614)-k(21)*y(182)*y(614);
dydt(431)=dydt(431)+k(21)*y(182)*y(614);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(618);
dydt(618)=dydt(618)-k(21)*y(182)*y(618);
dydt(487)=dydt(487)+k(21)*y(182)*y(618);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(617);
dydt(617)=dydt(617)-k(21)*y(182)*y(617);
dydt(498)=dydt(498)+k(21)*y(182)*y(617);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(616);
dydt(616)=dydt(616)-k(21)*y(182)*y(616);
dydt(271)=dydt(271)+k(21)*y(182)*y(616);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(615);
dydt(615)=dydt(615)-k(21)*y(182)*y(615);
dydt(285)=dydt(285)+k(21)*y(182)*y(615);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(613);
dydt(613)=dydt(613)-k(21)*y(182)*y(613);
dydt(490)=dydt(490)+k(21)*y(182)*y(613);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(581);
dydt(581)=dydt(581)-k(21)*y(182)*y(581);
dydt(551)=dydt(551)+k(21)*y(182)*y(581);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(623);
dydt(623)=dydt(623)-k(21)*y(182)*y(623);
dydt(517)=dydt(517)+k(21)*y(182)*y(623);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(624);
dydt(624)=dydt(624)-k(21)*y(182)*y(624);
dydt(640)=dydt(640)+k(21)*y(182)*y(624);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(628);
dydt(628)=dydt(628)-k(21)*y(182)*y(628);
dydt(637)=dydt(637)+k(21)*y(182)*y(628);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(633);
dydt(633)=dydt(633)-k(21)*y(182)*y(633);
dydt(639)=dydt(639)+k(21)*y(182)*y(633);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(634);
dydt(634)=dydt(634)-k(21)*y(182)*y(634);
dydt(286)=dydt(286)+k(21)*y(182)*y(634);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(638);
dydt(638)=dydt(638)-k(21)*y(182)*y(638);
dydt(640)=dydt(640)+k(21)*y(182)*y(638);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(645);
dydt(645)=dydt(645)-k(21)*y(182)*y(645);
dydt(268)=dydt(268)+k(21)*y(182)*y(645);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(644);
dydt(644)=dydt(644)-k(21)*y(182)*y(644);
dydt(517)=dydt(517)+k(21)*y(182)*y(644);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(636);
dydt(636)=dydt(636)-k(21)*y(182)*y(636);
dydt(489)=dydt(489)+k(21)*y(182)*y(636);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(629);
dydt(629)=dydt(629)-k(21)*y(182)*y(629);
dydt(281)=dydt(281)+k(21)*y(182)*y(629);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(627);
dydt(627)=dydt(627)-k(21)*y(182)*y(627);
dydt(624)=dydt(624)+k(21)*y(182)*y(627);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(627);
dydt(627)=dydt(627)-k(21)*y(182)*y(627);
dydt(638)=dydt(638)+k(21)*y(182)*y(627);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(622);
dydt(622)=dydt(622)-k(21)*y(182)*y(622);
dydt(641)=dydt(641)+k(21)*y(182)*y(622);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(576);
dydt(576)=dydt(576)-k(21)*y(182)*y(576);
dydt(579)=dydt(579)+k(21)*y(182)*y(576);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(573);
dydt(573)=dydt(573)-k(21)*y(182)*y(573);
dydt(574)=dydt(574)+k(21)*y(182)*y(573);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(564);
dydt(564)=dydt(564)-k(21)*y(182)*y(564);
dydt(626)=dydt(626)+k(21)*y(182)*y(564);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(563);
dydt(563)=dydt(563)-k(21)*y(182)*y(563);
dydt(555)=dydt(555)+k(21)*y(182)*y(563);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(556);
dydt(556)=dydt(556)-k(21)*y(182)*y(556);
dydt(553)=dydt(553)+k(21)*y(182)*y(556);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(547);
dydt(547)=dydt(547)-k(21)*y(182)*y(547);
dydt(548)=dydt(548)+k(21)*y(182)*y(547);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(540);
dydt(540)=dydt(540)-k(21)*y(182)*y(540);
dydt(625)=dydt(625)+k(21)*y(182)*y(540);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(539);
dydt(539)=dydt(539)-k(21)*y(182)*y(539);
dydt(596)=dydt(596)+k(21)*y(182)*y(539);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(537);
dydt(537)=dydt(537)-k(21)*y(182)*y(537);
dydt(518)=dydt(518)+k(21)*y(182)*y(537);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(537);
dydt(537)=dydt(537)-k(21)*y(182)*y(537);
dydt(519)=dydt(519)+k(21)*y(182)*y(537);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(534);
dydt(534)=dydt(534)-k(21)*y(182)*y(534);
dydt(496)=dydt(496)+k(21)*y(182)*y(534);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(533);
dydt(533)=dydt(533)-k(21)*y(182)*y(533);
dydt(359)=dydt(359)+k(21)*y(182)*y(533);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(529);
dydt(529)=dydt(529)-k(21)*y(182)*y(529);
dydt(520)=dydt(520)+k(21)*y(182)*y(529);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(524);
dydt(524)=dydt(524)-k(21)*y(182)*y(524);
dydt(521)=dydt(521)+k(21)*y(182)*y(524);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(519);
dydt(519)=dydt(519)-k(21)*y(182)*y(519);
dydt(514)=dydt(514)+k(21)*y(182)*y(519);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(518);
dydt(518)=dydt(518)-k(21)*y(182)*y(518);
dydt(514)=dydt(514)+k(21)*y(182)*y(518);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(512);
dydt(512)=dydt(512)-k(21)*y(182)*y(512);
dydt(513)=dydt(513)+k(21)*y(182)*y(512);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(648);
dydt(648)=dydt(648)-k(21)*y(182)*y(648);
dydt(187)=dydt(187)+k(21)*y(182)*y(648);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(508);
dydt(508)=dydt(508)-k(21)*y(182)*y(508);
dydt(510)=dydt(510)+k(21)*y(182)*y(508);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(501);
dydt(501)=dydt(501)-k(21)*y(182)*y(501);
dydt(505)=dydt(505)+k(21)*y(182)*y(501);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(497);
dydt(497)=dydt(497)-k(21)*y(182)*y(497);
dydt(579)=dydt(579)+k(21)*y(182)*y(497);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(491);
dydt(491)=dydt(491)-k(21)*y(182)*y(491);
dydt(488)=dydt(488)+k(21)*y(182)*y(491);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(470);
dydt(470)=dydt(470)-k(21)*y(182)*y(470);
dydt(423)=dydt(423)+k(21)*y(182)*y(470);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(443);
dydt(443)=dydt(443)-k(21)*y(182)*y(443);
dydt(432)=dydt(432)+k(21)*y(182)*y(443);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(431);
dydt(431)=dydt(431)-k(21)*y(182)*y(431);
dydt(580)=dydt(580)+k(21)*y(182)*y(431);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(425);
dydt(425)=dydt(425)-k(21)*y(182)*y(425);
dydt(280)=dydt(280)+k(21)*y(182)*y(425);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(422);
dydt(422)=dydt(422)-k(21)*y(182)*y(422);
dydt(649)=dydt(649)+k(21)*y(182)*y(422);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(421);
dydt(421)=dydt(421)-k(21)*y(182)*y(421);
dydt(580)=dydt(580)+k(21)*y(182)*y(421);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(651);
dydt(651)=dydt(651)-k(21)*y(182)*y(651);
dydt(186)=dydt(186)+k(21)*y(182)*y(651);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(410);
dydt(410)=dydt(410)-k(21)*y(182)*y(410);
dydt(433)=dydt(433)+k(21)*y(182)*y(410);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(409);
dydt(409)=dydt(409)-k(21)*y(182)*y(409);
dydt(194)=dydt(194)+k(21)*y(182)*y(409);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(408);
dydt(408)=dydt(408)-k(21)*y(182)*y(408);
dydt(193)=dydt(193)+k(21)*y(182)*y(408);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(405);
dydt(405)=dydt(405)-k(21)*y(182)*y(405);
dydt(447)=dydt(447)+k(21)*y(182)*y(405);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(401);
dydt(401)=dydt(401)-k(21)*y(182)*y(401);
dydt(398)=dydt(398)+k(21)*y(182)*y(401);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(400);
dydt(400)=dydt(400)-k(21)*y(182)*y(400);
dydt(368)=dydt(368)+k(21)*y(182)*y(400);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(396);
dydt(396)=dydt(396)-k(21)*y(182)*y(396);
dydt(394)=dydt(394)+k(21)*y(182)*y(396);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(393);
dydt(393)=dydt(393)-k(21)*y(182)*y(393);
dydt(394)=dydt(394)+k(21)*y(182)*y(393);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(388);
dydt(388)=dydt(388)-k(21)*y(182)*y(388);
dydt(390)=dydt(390)+k(21)*y(182)*y(388);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(384);
dydt(384)=dydt(384)-k(21)*y(182)*y(384);
dydt(385)=dydt(385)+k(21)*y(182)*y(384);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(383);
dydt(383)=dydt(383)-k(21)*y(182)*y(383);
dydt(389)=dydt(389)+k(21)*y(182)*y(383);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(376);
dydt(376)=dydt(376)-k(21)*y(182)*y(376);
dydt(369)=dydt(369)+k(21)*y(182)*y(376);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(375);
dydt(375)=dydt(375)-k(21)*y(182)*y(375);
dydt(377)=dydt(377)+k(21)*y(182)*y(375);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(361);
dydt(361)=dydt(361)-k(21)*y(182)*y(361);
dydt(188)=dydt(188)+k(21)*y(182)*y(361);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(354);
dydt(354)=dydt(354)-k(21)*y(182)*y(354);
dydt(509)=dydt(509)+k(21)*y(182)*y(354);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(352);
dydt(352)=dydt(352)-k(21)*y(182)*y(352);
dydt(349)=dydt(349)+k(21)*y(182)*y(352);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(351);
dydt(351)=dydt(351)-k(21)*y(182)*y(351);
dydt(348)=dydt(348)+k(21)*y(182)*y(351);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(345);
dydt(345)=dydt(345)-k(21)*y(182)*y(345);
dydt(516)=dydt(516)+k(21)*y(182)*y(345);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(344);
dydt(344)=dydt(344)-k(21)*y(182)*y(344);
dydt(513)=dydt(513)+k(21)*y(182)*y(344);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(339);
dydt(339)=dydt(339)-k(21)*y(182)*y(339);
dydt(597)=dydt(597)+k(21)*y(182)*y(339);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(338);
dydt(338)=dydt(338)-k(21)*y(182)*y(338);
dydt(611)=dydt(611)+k(21)*y(182)*y(338);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(315);
dydt(315)=dydt(315)-k(21)*y(182)*y(315);
dydt(219)=dydt(219)+k(21)*y(182)*y(315);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(310);
dydt(310)=dydt(310)-k(21)*y(182)*y(310);
dydt(623)=dydt(623)+k(21)*y(182)*y(310);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(310);
dydt(310)=dydt(310)-k(21)*y(182)*y(310);
dydt(644)=dydt(644)+k(21)*y(182)*y(310);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(308);
dydt(308)=dydt(308)-k(21)*y(182)*y(308);
dydt(523)=dydt(523)+k(21)*y(182)*y(308);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(292);
dydt(292)=dydt(292)-k(21)*y(182)*y(292);
dydt(290)=dydt(290)+k(21)*y(182)*y(292);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(287);
dydt(287)=dydt(287)-k(21)*y(182)*y(287);
dydt(283)=dydt(283)+k(21)*y(182)*y(287);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(282);
dydt(282)=dydt(282)-k(21)*y(182)*y(282);
dydt(273)=dydt(273)+k(21)*y(182)*y(282);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(182)=dydt(182)-k(21)*y(182)*y(276);
dydt(276)=dydt(276)-k(21)*y(182)*y(276);
dydt(278)=dydt(278)+k(21)*y(182)*y(276);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(272);
dydt(272)=dydt(272)-k(21)*y(182)*y(272);
dydt(273)=dydt(273)+k(21)*y(182)*y(272);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(267);
dydt(267)=dydt(267)-k(21)*y(182)*y(267);
dydt(268)=dydt(268)+k(21)*y(182)*y(267);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(260);
dydt(260)=dydt(260)-k(21)*y(182)*y(260);
dydt(261)=dydt(261)+k(21)*y(182)*y(260);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(252);
dydt(252)=dydt(252)-k(21)*y(182)*y(252);
dydt(266)=dydt(266)+k(21)*y(182)*y(252);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(251);
dydt(251)=dydt(251)-k(21)*y(182)*y(251);
dydt(265)=dydt(265)+k(21)*y(182)*y(251);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(249);
dydt(249)=dydt(249)-k(21)*y(182)*y(249);
dydt(479)=dydt(479)+k(21)*y(182)*y(249);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(245);
dydt(245)=dydt(245)-k(21)*y(182)*y(245);
dydt(495)=dydt(495)+k(21)*y(182)*y(245);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(182)=dydt(182)-k(21)*y(182)*y(244);
dydt(244)=dydt(244)-k(21)*y(182)*y(244);
dydt(494)=dydt(494)+k(21)*y(182)*y(244);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(238);
dydt(238)=dydt(238)-k(21)*y(182)*y(238);
dydt(267)=dydt(267)+k(21)*y(182)*y(238);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(238);
dydt(238)=dydt(238)-k(21)*y(182)*y(238);
dydt(645)=dydt(645)+k(21)*y(182)*y(238);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(237);
dydt(237)=dydt(237)-k(21)*y(182)*y(237);
dydt(325)=dydt(325)+k(21)*y(182)*y(237);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(182)=dydt(182)-k(21)*y(182)*y(236);
dydt(236)=dydt(236)-k(21)*y(182)*y(236);
dydt(324)=dydt(324)+k(21)*y(182)*y(236);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(234);
dydt(234)=dydt(234)-k(21)*y(182)*y(234);
dydt(220)=dydt(220)+k(21)*y(182)*y(234);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(232);
dydt(232)=dydt(232)-k(21)*y(182)*y(232);
dydt(486)=dydt(486)+k(21)*y(182)*y(232);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(225);
dydt(225)=dydt(225)-k(21)*y(182)*y(225);
dydt(649)=dydt(649)+k(21)*y(182)*y(225);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(206);
dydt(206)=dydt(206)-k(21)*y(182)*y(206);
dydt(289)=dydt(289)+k(21)*y(182)*y(206);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(182)=dydt(182)-k(21)*y(182)*y(205);
dydt(205)=dydt(205)-k(21)*y(182)*y(205);
dydt(476)=dydt(476)+k(21)*y(182)*y(205);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(198);
dydt(198)=dydt(198)-k(21)*y(182)*y(198);
dydt(367)=dydt(367)+k(21)*y(182)*y(198);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(196);
dydt(196)=dydt(196)-k(21)*y(182)*y(196);
dydt(190)=dydt(190)+k(21)*y(182)*y(196);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(182)=dydt(182)-k(21)*y(182)*y(189);
dydt(189)=dydt(189)-k(21)*y(182)*y(189);
dydt(183)=dydt(183)+k(21)*y(182)*y(189);

% rule    : R17
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(184)=dydt(184)-k(34)*y(184);
dydt(140)=dydt(140)+k(34)*y(184);

% rule    : R9_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(184)=dydt(184)-k(22)*y(184);
dydt(182)=dydt(182)+k(22)*y(184);
dydt(48)=dydt(48)+k(22)*y(184);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lyn(U, SH2) 

dydt(184)=dydt(184)-k(10)*y(184);
dydt(3)=dydt(3)+k(10)*y(184);
dydt(177)=dydt(177)+k(10)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(7)*y(184)*y(185);
dydt(185)=dydt(185)-k(7)*y(184)*y(185);
dydt(186)=dydt(186)+k(7)*y(184)*y(185);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(209)=dydt(209)-k(7)*y(209)*y(184);
dydt(184)=dydt(184)-k(7)*y(209)*y(184);
dydt(397)=dydt(397)+k(7)*y(209)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(214)=dydt(214)-k(7)*y(214)*y(184);
dydt(184)=dydt(184)-k(7)*y(214)*y(184);
dydt(394)=dydt(394)+k(7)*y(214)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(219)=dydt(219)-k(7)*y(219)*y(184);
dydt(184)=dydt(184)-k(7)*y(219)*y(184);
dydt(395)=dydt(395)+k(7)*y(219)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(7)*y(184)*y(223);
dydt(223)=dydt(223)-k(7)*y(184)*y(223);
dydt(649)=dydt(649)+k(7)*y(184)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(7)*y(184)*y(228);
dydt(228)=dydt(228)-k(7)*y(184)*y(228);
dydt(476)=dydt(476)+k(7)*y(184)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(315)=dydt(315)-k(7)*y(315)*y(184);
dydt(184)=dydt(184)-k(7)*y(315)*y(184);
dydt(420)=dydt(420)+k(7)*y(315)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(319)=dydt(319)-k(7)*y(319)*y(184);
dydt(184)=dydt(184)-k(7)*y(319)*y(184);
dydt(398)=dydt(398)+k(7)*y(319)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(7)*y(184)*y(336);
dydt(336)=dydt(336)-k(7)*y(184)*y(336);
dydt(368)=dydt(368)+k(7)*y(184)*y(336);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(437)=dydt(437)-k(7)*y(437)*y(184);
dydt(184)=dydt(184)-k(7)*y(437)*y(184);
dydt(392)=dydt(392)+k(7)*y(437)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(191)=dydt(191)-k(5)*y(191)*y(184);
dydt(184)=dydt(184)-k(5)*y(191)*y(184);
dydt(190)=dydt(190)+k(5)*y(191)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(256)=dydt(256)-k(5)*y(256)*y(184);
dydt(184)=dydt(184)-k(5)*y(256)*y(184);
dydt(195)=dydt(195)+k(5)*y(256)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(264)=dydt(264)-k(5)*y(264)*y(184);
dydt(184)=dydt(184)-k(5)*y(264)*y(184);
dydt(188)=dydt(188)+k(5)*y(264)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(5)*y(184)*y(380);
dydt(380)=dydt(380)-k(5)*y(184)*y(380);
dydt(460)=dydt(460)+k(5)*y(184)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(5)*y(184)*y(391);
dydt(391)=dydt(391)-k(5)*y(184)*y(391);
dydt(186)=dydt(186)+k(5)*y(184)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(5)*y(184)*y(407);
dydt(407)=dydt(407)-k(5)*y(184)*y(407);
dydt(193)=dydt(193)+k(5)*y(184)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(5)*y(184)*y(441);
dydt(441)=dydt(441)-k(5)*y(184)*y(441);
dydt(448)=dydt(448)+k(5)*y(184)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(5)*y(184)*y(515);
dydt(515)=dydt(515)-k(5)*y(184)*y(515);
dydt(516)=dydt(516)+k(5)*y(184)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(184)=dydt(184)-k(5)*y(184)*y(543);
dydt(543)=dydt(543)-k(5)*y(184)*y(543);
dydt(549)=dydt(549)+k(5)*y(184)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(184)=dydt(184)-k(5)*y(184)*y(554);
dydt(554)=dydt(554)-k(5)*y(184)*y(554);
dydt(349)=dydt(349)+k(5)*y(184)*y(554);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(9)=dydt(9)-k(5)*y(9)*y(184);
dydt(184)=dydt(184)-k(5)*y(9)*y(184);
dydt(215)=dydt(215)+k(5)*y(9)*y(184);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(4)=dydt(4)-k(3)*y(4)*y(184);
dydt(184)=dydt(184)-k(3)*y(4)*y(184);
dydt(391)=dydt(391)+k(3)*y(4)*y(184);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(4)=dydt(4)-k(1)*y(4)*y(184);
dydt(184)=dydt(184)-k(1)*y(4)*y(184);
dydt(185)=dydt(185)+k(1)*y(4)*y(184);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(185)=dydt(185)-k(34)*y(185);
dydt(141)=dydt(141)+k(34)*y(185);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(185)=dydt(185)-k(22)*y(185);
dydt(182)=dydt(182)+k(22)*y(185);
dydt(126)=dydt(126)+k(22)*y(185);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(185)=dydt(185)-k(10)*y(185);
dydt(3)=dydt(3)+k(10)*y(185);
dydt(214)=dydt(214)+k(10)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(1)=dydt(1)-k(7)*y(1)*y(185);
dydt(185)=dydt(185)-k(7)*y(1)*y(185);
dydt(652)=dydt(652)+k(7)*y(1)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(7)*y(7)*y(185);
dydt(185)=dydt(185)-k(7)*y(7)*y(185);
dydt(215)=dydt(215)+k(7)*y(7)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(7)*y(335)*y(185);
dydt(185)=dydt(185)-k(7)*y(335)*y(185);
dydt(193)=dydt(193)+k(7)*y(335)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(185);
dydt(185)=dydt(185)-k(7)*y(257)*y(185);
dydt(195)=dydt(195)+k(7)*y(257)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(227)=dydt(227)-k(7)*y(227)*y(185);
dydt(185)=dydt(185)-k(7)*y(227)*y(185);
dydt(549)=dydt(549)+k(7)*y(227)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(222)=dydt(222)-k(7)*y(222)*y(185);
dydt(185)=dydt(185)-k(7)*y(222)*y(185);
dydt(516)=dydt(516)+k(7)*y(222)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(185);
dydt(185)=dydt(185)-k(7)*y(218)*y(185);
dydt(188)=dydt(188)+k(7)*y(218)*y(185);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(185);
dydt(185)=dydt(185)-k(7)*y(192)*y(185);
dydt(190)=dydt(190)+k(7)*y(192)*y(185);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) 

dydt(185)=dydt(185)-k(2)*y(185);
dydt(4)=dydt(4)+k(2)*y(185);
dydt(184)=dydt(184)+k(2)*y(185);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(186)=dydt(186)-k(34)*y(186);
dydt(460)=dydt(460)+k(34)*y(186);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(186)=dydt(186)-k(34)*y(186);
dydt(178)=dydt(178)+k(34)*y(186);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(186)=dydt(186)-k(24)*y(186);
dydt(193)=dydt(193)+k(24)*y(186);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(186)=dydt(186)-k(23)*y(186);
dydt(368)=dydt(368)+k(23)*y(186);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(186)=dydt(186)-k(22)*y(186);
dydt(182)=dydt(182)+k(22)*y(186);
dydt(457)=dydt(457)+k(22)*y(186);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(186)=dydt(186)-k(22)*y(186);
dydt(182)=dydt(182)+k(22)*y(186);
dydt(651)=dydt(651)+k(22)*y(186);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(186)=dydt(186)-k(10)*y(186);
dydt(3)=dydt(3)+k(10)*y(186);
dydt(183)=dydt(183)+k(10)*y(186);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(186)=dydt(186)-k(10)*y(186);
dydt(3)=dydt(3)+k(10)*y(186);
dydt(394)=dydt(394)+k(10)*y(186);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(186)=dydt(186)-k(8)*y(186);
dydt(184)=dydt(184)+k(8)*y(186);
dydt(185)=dydt(185)+k(8)*y(186);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(186)=dydt(186)-k(6)*y(186);
dydt(184)=dydt(184)+k(6)*y(186);
dydt(391)=dydt(391)+k(6)*y(186);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(187)=dydt(187)-k(34)*y(187);
dydt(377)=dydt(377)+k(34)*y(187);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(187)=dydt(187)-k(34)*y(187);
dydt(179)=dydt(179)+k(34)*y(187);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(187)=dydt(187)-k(22)*y(187);
dydt(182)=dydt(182)+k(22)*y(187);
dydt(451)=dydt(451)+k(22)*y(187);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(187)=dydt(187)-k(22)*y(187);
dydt(182)=dydt(182)+k(22)*y(187);
dydt(648)=dydt(648)+k(22)*y(187);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(187);
dydt(187)=dydt(187)-k(9)*y(3)*y(187);
dydt(183)=dydt(183)+k(9)*y(3)*y(187);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(187);
dydt(187)=dydt(187)-k(9)*y(3)*y(187);
dydt(394)=dydt(394)+k(9)*y(3)*y(187);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(187)=dydt(187)-k(8)*y(187);
dydt(177)=dydt(177)+k(8)*y(187);
dydt(214)=dydt(214)+k(8)*y(187);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(187)=dydt(187)-k(6)*y(187);
dydt(177)=dydt(177)+k(6)*y(187);
dydt(176)=dydt(176)+k(6)*y(187);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(188)=dydt(188)-k(34)*y(188);
dydt(346)=dydt(346)+k(34)*y(188);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(188)=dydt(188)-k(34)*y(188);
dydt(180)=dydt(180)+k(34)*y(188);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(188)=dydt(188)-k(31)*y(188);
dydt(183)=dydt(183)+k(31)*y(188);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(188)=dydt(188)-k(24)*y(188);
dydt(195)=dydt(195)+k(24)*y(188);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(188)=dydt(188)-k(22)*y(188);
dydt(182)=dydt(182)+k(22)*y(188);
dydt(343)=dydt(343)+k(22)*y(188);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(188)=dydt(188)-k(22)*y(188);
dydt(182)=dydt(182)+k(22)*y(188);
dydt(361)=dydt(361)+k(22)*y(188);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(188);
dydt(188)=dydt(188)-k(15)*y(3)*y(188);
dydt(516)=dydt(516)+k(15)*y(3)*y(188);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(188)=dydt(188)-k(10)*y(188);
dydt(3)=dydt(3)+k(10)*y(188);
dydt(513)=dydt(513)+k(10)*y(188);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(188)=dydt(188)-k(8)*y(188);
dydt(218)=dydt(218)+k(8)*y(188);
dydt(185)=dydt(185)+k(8)*y(188);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(188)=dydt(188)-k(6)*y(188);
dydt(264)=dydt(264)+k(6)*y(188);
dydt(184)=dydt(184)+k(6)*y(188);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(189)=dydt(189)-k(34)*y(189);
dydt(145)=dydt(145)+k(34)*y(189);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(189)=dydt(189)-k(32)*y(189);
dydt(652)=dydt(652)+k(32)*y(189);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(189)=dydt(189)-k(22)*y(189);
dydt(182)=dydt(182)+k(22)*y(189);
dydt(147)=dydt(147)+k(22)*y(189);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(189);
dydt(189)=dydt(189)-k(21)*y(197)*y(189);
dydt(190)=dydt(190)+k(21)*y(197)*y(189);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(189);
dydt(189)=dydt(189)-k(21)*y(32)*y(189);
dydt(168)=dydt(168)+k(21)*y(32)*y(189);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(189);
dydt(189)=dydt(189)-k(21)*y(2)*y(189);
dydt(174)=dydt(174)+k(21)*y(2)*y(189);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(189)=dydt(189)-k(12)*y(189);
dydt(343)=dydt(343)+k(12)*y(189);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(189)=dydt(189)-k(10)*y(189);
dydt(3)=dydt(3)+k(10)*y(189);
dydt(451)=dydt(451)+k(10)*y(189);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(189);
dydt(189)=dydt(189)-k(9)*y(3)*y(189);
dydt(457)=dydt(457)+k(9)*y(3)*y(189);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

dydt(189)=dydt(189)-k(8)*y(189);
dydt(15)=dydt(15)+k(8)*y(189);
dydt(185)=dydt(185)+k(8)*y(189);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(189)=dydt(189)-k(6)*y(189);
dydt(98)=dydt(98)+k(6)*y(189);
dydt(184)=dydt(184)+k(6)*y(189);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(191)=dydt(191)-k(34)*y(191);
dydt(86)=dydt(86)+k(34)*y(191);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(191)=dydt(191)-k(33)*y(191);
dydt(176)=dydt(176)+k(33)*y(191);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~pY) 

dydt(191)=dydt(191)-k(22)*y(191);
dydt(197)=dydt(197)+k(22)*y(191);
dydt(98)=dydt(98)+k(22)*y(191);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(191);
dydt(191)=dydt(191)-k(9)*y(3)*y(191);
dydt(407)=dydt(407)+k(9)*y(3)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(1)=dydt(1)-k(5)*y(1)*y(191);
dydt(191)=dydt(191)-k(5)*y(1)*y(191);
dydt(202)=dydt(202)+k(5)*y(1)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(7)=dydt(7)-k(5)*y(7)*y(191);
dydt(191)=dydt(191)-k(5)*y(7)*y(191);
dydt(607)=dydt(607)+k(5)*y(7)*y(191);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(191)=dydt(191)-k(5)*y(191)*y(335);
dydt(335)=dydt(335)-k(5)*y(191)*y(335);
dydt(370)=dydt(370)+k(5)*y(191)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(257)=dydt(257)-k(5)*y(257)*y(191);
dydt(191)=dydt(191)-k(5)*y(257)*y(191);
dydt(619)=dydt(619)+k(5)*y(257)*y(191);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(191)=dydt(191)-k(5)*y(191)*y(227);
dydt(227)=dydt(227)-k(5)*y(191)*y(227);
dydt(294)=dydt(294)+k(5)*y(191)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(191)=dydt(191)-k(5)*y(191)*y(222);
dydt(222)=dydt(222)-k(5)*y(191)*y(222);
dydt(288)=dydt(288)+k(5)*y(191)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(218)=dydt(218)-k(5)*y(218)*y(191);
dydt(191)=dydt(191)-k(5)*y(218)*y(191);
dydt(290)=dydt(290)+k(5)*y(218)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(5)*y(192)*y(191);
dydt(191)=dydt(191)-k(5)*y(192)*y(191);
dydt(455)=dydt(455)+k(5)*y(192)*y(191);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) 

dydt(191)=dydt(191)-k(4)*y(191);
dydt(4)=dydt(4)+k(4)*y(191);
dydt(192)=dydt(192)+k(4)*y(191);

% rule    : R17
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(192)=dydt(192)-k(34)*y(192);
dydt(27)=dydt(27)+k(34)*y(192);

% rule    : R16
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(192)=dydt(192)-k(33)*y(192);
dydt(177)=dydt(177)+k(33)*y(192);

% rule    : R9_op
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(192)=dydt(192)-k(22)*y(192);
dydt(197)=dydt(197)+k(22)*y(192);
dydt(15)=dydt(15)+k(22)*y(192);

% rule    : R3
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(192);
dydt(192)=dydt(192)-k(9)*y(3)*y(192);
dydt(335)=dydt(335)+k(9)*y(3)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(209);
dydt(209)=dydt(209)-k(7)*y(192)*y(209);
dydt(455)=dydt(455)+k(7)*y(192)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(214);
dydt(214)=dydt(214)-k(7)*y(192)*y(214);
dydt(194)=dydt(194)+k(7)*y(192)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(219);
dydt(219)=dydt(219)-k(7)*y(192)*y(219);
dydt(290)=dydt(290)+k(7)*y(192)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(223);
dydt(223)=dydt(223)-k(7)*y(192)*y(223);
dydt(288)=dydt(288)+k(7)*y(192)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(228);
dydt(228)=dydt(228)-k(7)*y(192)*y(228);
dydt(294)=dydt(294)+k(7)*y(192)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(315)=dydt(315)-k(7)*y(315)*y(192);
dydt(192)=dydt(192)-k(7)*y(315)*y(192);
dydt(292)=dydt(292)+k(7)*y(315)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(319);
dydt(319)=dydt(319)-k(7)*y(192)*y(319);
dydt(619)=dydt(619)+k(7)*y(192)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(336);
dydt(336)=dydt(336)-k(7)*y(192)*y(336);
dydt(370)=dydt(370)+k(7)*y(192)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(7)*y(192)*y(437);
dydt(437)=dydt(437)-k(7)*y(192)*y(437);
dydt(438)=dydt(438)+k(7)*y(192)*y(437);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(192)=dydt(192)-k(5)*y(192)*y(256);
dydt(256)=dydt(256)-k(5)*y(192)*y(256);
dydt(609)=dydt(609)+k(5)*y(192)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(192)=dydt(192)-k(5)*y(192)*y(264);
dydt(264)=dydt(264)-k(5)*y(192)*y(264);
dydt(611)=dydt(611)+k(5)*y(192)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(192)=dydt(192)-k(5)*y(192)*y(380);
dydt(380)=dydt(380)-k(5)*y(192)*y(380);
dydt(374)=dydt(374)+k(5)*y(192)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(192)=dydt(192)-k(5)*y(192)*y(391);
dydt(391)=dydt(391)-k(5)*y(192)*y(391);
dydt(397)=dydt(397)+k(5)*y(192)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(192)=dydt(192)-k(5)*y(192)*y(407);
dydt(407)=dydt(407)-k(5)*y(192)*y(407);
dydt(403)=dydt(403)+k(5)*y(192)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(192)=dydt(192)-k(5)*y(192)*y(441);
dydt(441)=dydt(441)-k(5)*y(192)*y(441);
dydt(402)=dydt(402)+k(5)*y(192)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(192)=dydt(192)-k(5)*y(192)*y(515);
dydt(515)=dydt(515)-k(5)*y(192)*y(515);
dydt(520)=dydt(520)+k(5)*y(192)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(192)=dydt(192)-k(5)*y(192)*y(543);
dydt(543)=dydt(543)-k(5)*y(192)*y(543);
dydt(552)=dydt(552)+k(5)*y(192)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(192)=dydt(192)-k(5)*y(192)*y(554);
dydt(554)=dydt(554)-k(5)*y(192)*y(554);
dydt(557)=dydt(557)+k(5)*y(192)*y(554);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(9)=dydt(9)-k(5)*y(9)*y(192);
dydt(192)=dydt(192)-k(5)*y(9)*y(192);
dydt(208)=dydt(208)+k(5)*y(9)*y(192);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(4)=dydt(4)-k(3)*y(4)*y(192);
dydt(192)=dydt(192)-k(3)*y(4)*y(192);
dydt(191)=dydt(191)+k(3)*y(4)*y(192);

% rule    : R1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(4)=dydt(4)-k(1)*y(4)*y(192);
dydt(192)=dydt(192)-k(1)*y(4)*y(192);
dydt(209)=dydt(209)+k(1)*y(4)*y(192);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(193)=dydt(193)-k(34)*y(193);
dydt(448)=dydt(448)+k(34)*y(193);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(193)=dydt(193)-k(34)*y(193);
dydt(466)=dydt(466)+k(34)*y(193);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(193)=dydt(193)-k(33)*y(193);
dydt(186)=dydt(186)+k(33)*y(193);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(193)=dydt(193)-k(23)*y(193);
dydt(464)=dydt(464)+k(23)*y(193);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(193)=dydt(193)-k(22)*y(193);
dydt(197)=dydt(197)+k(22)*y(193);
dydt(457)=dydt(457)+k(22)*y(193);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(193)=dydt(193)-k(22)*y(193);
dydt(182)=dydt(182)+k(22)*y(193);
dydt(408)=dydt(408)+k(22)*y(193);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(193)=dydt(193)-k(10)*y(193);
dydt(3)=dydt(3)+k(10)*y(193);
dydt(190)=dydt(190)+k(10)*y(193);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(193)=dydt(193)-k(10)*y(193);
dydt(3)=dydt(3)+k(10)*y(193);
dydt(435)=dydt(435)+k(10)*y(193);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(193)=dydt(193)-k(8)*y(193);
dydt(335)=dydt(335)+k(8)*y(193);
dydt(185)=dydt(185)+k(8)*y(193);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(193)=dydt(193)-k(6)*y(193);
dydt(184)=dydt(184)+k(6)*y(193);
dydt(407)=dydt(407)+k(6)*y(193);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(194)=dydt(194)-k(34)*y(194);
dydt(449)=dydt(449)+k(34)*y(194);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(194)=dydt(194)-k(34)*y(194);
dydt(453)=dydt(453)+k(34)*y(194);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(194)=dydt(194)-k(33)*y(194);
dydt(187)=dydt(187)+k(33)*y(194);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(194)=dydt(194)-k(22)*y(194);
dydt(197)=dydt(197)+k(22)*y(194);
dydt(451)=dydt(451)+k(22)*y(194);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(194)=dydt(194)-k(22)*y(194);
dydt(182)=dydt(182)+k(22)*y(194);
dydt(409)=dydt(409)+k(22)*y(194);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(194);
dydt(194)=dydt(194)-k(9)*y(3)*y(194);
dydt(190)=dydt(190)+k(9)*y(3)*y(194);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(194);
dydt(194)=dydt(194)-k(9)*y(3)*y(194);
dydt(435)=dydt(435)+k(9)*y(3)*y(194);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(194)=dydt(194)-k(8)*y(194);
dydt(192)=dydt(192)+k(8)*y(194);
dydt(214)=dydt(214)+k(8)*y(194);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(194)=dydt(194)-k(6)*y(194);
dydt(177)=dydt(177)+k(6)*y(194);
dydt(191)=dydt(191)+k(6)*y(194);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(195)=dydt(195)-k(34)*y(195);
dydt(347)=dydt(347)+k(34)*y(195);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(195)=dydt(195)-k(34)*y(195);
dydt(605)=dydt(605)+k(34)*y(195);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(195)=dydt(195)-k(33)*y(195);
dydt(188)=dydt(188)+k(33)*y(195);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(195)=dydt(195)-k(31)*y(195);
dydt(190)=dydt(190)+k(31)*y(195);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(195)=dydt(195)-k(22)*y(195);
dydt(197)=dydt(197)+k(22)*y(195);
dydt(343)=dydt(343)+k(22)*y(195);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(195)=dydt(195)-k(22)*y(195);
dydt(182)=dydt(182)+k(22)*y(195);
dydt(599)=dydt(599)+k(22)*y(195);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(195);
dydt(195)=dydt(195)-k(15)*y(3)*y(195);
dydt(549)=dydt(549)+k(15)*y(3)*y(195);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(195)=dydt(195)-k(10)*y(195);
dydt(3)=dydt(3)+k(10)*y(195);
dydt(550)=dydt(550)+k(10)*y(195);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(195)=dydt(195)-k(8)*y(195);
dydt(257)=dydt(257)+k(8)*y(195);
dydt(185)=dydt(185)+k(8)*y(195);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(195)=dydt(195)-k(6)*y(195);
dydt(256)=dydt(256)+k(6)*y(195);
dydt(184)=dydt(184)+k(6)*y(195);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(196)=dydt(196)-k(34)*y(196);
dydt(153)=dydt(153)+k(34)*y(196);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(196)=dydt(196)-k(33)*y(196);
dydt(181)=dydt(181)+k(33)*y(196);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(196)=dydt(196)-k(32)*y(196);
dydt(607)=dydt(607)+k(32)*y(196);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(196)=dydt(196)-k(22)*y(196);
dydt(197)=dydt(197)+k(22)*y(196);
dydt(147)=dydt(147)+k(22)*y(196);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(196);
dydt(196)=dydt(196)-k(21)*y(197)*y(196);
dydt(370)=dydt(370)+k(21)*y(197)*y(196);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(196);
dydt(196)=dydt(196)-k(21)*y(32)*y(196);
dydt(163)=dydt(163)+k(21)*y(32)*y(196);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(196);
dydt(196)=dydt(196)-k(21)*y(2)*y(196);
dydt(169)=dydt(169)+k(21)*y(2)*y(196);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(196)=dydt(196)-k(12)*y(196);
dydt(599)=dydt(599)+k(12)*y(196);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(196)=dydt(196)-k(10)*y(196);
dydt(3)=dydt(3)+k(10)*y(196);
dydt(409)=dydt(409)+k(10)*y(196);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(196);
dydt(196)=dydt(196)-k(9)*y(3)*y(196);
dydt(408)=dydt(408)+k(9)*y(3)*y(196);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(196)=dydt(196)-k(8)*y(196);
dydt(126)=dydt(126)+k(8)*y(196);
dydt(192)=dydt(192)+k(8)*y(196);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(196)=dydt(196)-k(6)*y(196);
dydt(48)=dydt(48)+k(6)*y(196);
dydt(191)=dydt(191)+k(6)*y(196);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(198)=dydt(198)-k(34)*y(198);
dydt(154)=dydt(154)+k(34)*y(198);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(198)=dydt(198)-k(33)*y(198);
dydt(189)=dydt(189)+k(33)*y(198);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(198)=dydt(198)-k(32)*y(198);
dydt(199)=dydt(199)+k(32)*y(198);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(198)=dydt(198)-k(22)*y(198);
dydt(197)=dydt(197)+k(22)*y(198);
dydt(147)=dydt(147)+k(22)*y(198);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(198);
dydt(198)=dydt(198)-k(21)*y(197)*y(198);
dydt(370)=dydt(370)+k(21)*y(197)*y(198);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(198);
dydt(198)=dydt(198)-k(21)*y(32)*y(198);
dydt(162)=dydt(162)+k(21)*y(32)*y(198);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(198);
dydt(198)=dydt(198)-k(21)*y(2)*y(198);
dydt(371)=dydt(371)+k(21)*y(2)*y(198);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(198)=dydt(198)-k(12)*y(198);
dydt(337)=dydt(337)+k(12)*y(198);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(198)=dydt(198)-k(10)*y(198);
dydt(3)=dydt(3)+k(10)*y(198);
dydt(376)=dydt(376)+k(10)*y(198);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(198);
dydt(198)=dydt(198)-k(9)*y(3)*y(198);
dydt(400)=dydt(400)+k(9)*y(3)*y(198);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

dydt(198)=dydt(198)-k(8)*y(198);
dydt(15)=dydt(15)+k(8)*y(198);
dydt(336)=dydt(336)+k(8)*y(198);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(198)=dydt(198)-k(6)*y(198);
dydt(98)=dydt(98)+k(6)*y(198);
dydt(335)=dydt(335)+k(6)*y(198);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(199)=dydt(199)-k(34)*y(199);
dydt(138)=dydt(138)+k(34)*y(199);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(199)=dydt(199)-k(33)*y(199);
dydt(652)=dydt(652)+k(33)*y(199);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(199)=dydt(199)-k(22)*y(199);
dydt(197)=dydt(197)+k(22)*y(199);
dydt(146)=dydt(146)+k(22)*y(199);

% rule    : R5_2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(199)=dydt(199)-k(14)*y(199);
dydt(198)=dydt(198)+k(14)*y(199);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(199)=dydt(199)-k(12)*y(199);
dydt(334)=dydt(334)+k(12)*y(199);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(199)=dydt(199)-k(10)*y(199);
dydt(3)=dydt(3)+k(10)*y(199);
dydt(211)=dydt(211)+k(10)*y(199);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(199);
dydt(199)=dydt(199)-k(9)*y(3)*y(199);
dydt(210)=dydt(210)+k(9)*y(3)*y(199);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

dydt(199)=dydt(199)-k(8)*y(199);
dydt(1)=dydt(1)+k(8)*y(199);
dydt(336)=dydt(336)+k(8)*y(199);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(199)=dydt(199)-k(6)*y(199);
dydt(6)=dydt(6)+k(6)*y(199);
dydt(335)=dydt(335)+k(6)*y(199);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(200)=dydt(200)-k(34)*y(200);
dydt(130)=dydt(130)+k(34)*y(200);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(200)=dydt(200)-k(33)*y(200);
dydt(327)=dydt(327)+k(33)*y(200);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(200)=dydt(200)-k(22)*y(200);
dydt(197)=dydt(197)+k(22)*y(200);
dydt(125)=dydt(125)+k(22)*y(200);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(200)=dydt(200)-k(16)*y(200);
dydt(3)=dydt(3)+k(16)*y(200);
dydt(334)=dydt(334)+k(16)*y(200);

% rule    : R5_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(200)=dydt(200)-k(14)*y(200);
dydt(339)=dydt(339)+k(14)*y(200);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(200)=dydt(200)-k(10)*y(200);
dydt(3)=dydt(3)+k(10)*y(200);
dydt(322)=dydt(322)+k(10)*y(200);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(200)=dydt(200)-k(8)*y(200);
dydt(61)=dydt(61)+k(8)*y(200);
dydt(336)=dydt(336)+k(8)*y(200);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(200)=dydt(200)-k(6)*y(200);
dydt(122)=dydt(122)+k(6)*y(200);
dydt(335)=dydt(335)+k(6)*y(200);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(201)=dydt(201)-k(34)*y(201);
dydt(107)=dydt(107)+k(34)*y(201);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(201)=dydt(201)-k(33)*y(201);
dydt(593)=dydt(593)+k(33)*y(201);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(201)=dydt(201)-k(22)*y(201);
dydt(197)=dydt(197)+k(22)*y(201);
dydt(117)=dydt(117)+k(22)*y(201);

% rule    : R8
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(201)=dydt(201)-k(19)*y(201);
dydt(598)=dydt(598)+k(19)*y(201);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(201)=dydt(201)-k(16)*y(201);
dydt(3)=dydt(3)+k(16)*y(201);
dydt(588)=dydt(588)+k(16)*y(201);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(201)=dydt(201)-k(10)*y(201);
dydt(3)=dydt(3)+k(10)*y(201);
dydt(589)=dydt(589)+k(10)*y(201);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(201)=dydt(201)-k(8)*y(201);
dydt(8)=dydt(8)+k(8)*y(201);
dydt(227)=dydt(227)+k(8)*y(201);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(201)=dydt(201)-k(6)*y(201);
dydt(7)=dydt(7)+k(6)*y(201);
dydt(543)=dydt(543)+k(6)*y(201);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(202)=dydt(202)-k(34)*y(202);
dydt(101)=dydt(101)+k(34)*y(202);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(202)=dydt(202)-k(33)*y(202);
dydt(418)=dydt(418)+k(33)*y(202);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(202)=dydt(202)-k(22)*y(202);
dydt(197)=dydt(197)+k(22)*y(202);
dydt(97)=dydt(97)+k(22)*y(202);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(202);
dydt(202)=dydt(202)-k(9)*y(3)*y(202);
dydt(607)=dydt(607)+k(9)*y(3)*y(202);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(202);
dydt(202)=dydt(202)-k(9)*y(3)*y(202);
dydt(413)=dydt(413)+k(9)*y(3)*y(202);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(202)=dydt(202)-k(8)*y(202);
dydt(5)=dydt(5)+k(8)*y(202);
dydt(192)=dydt(192)+k(8)*y(202);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

dydt(202)=dydt(202)-k(6)*y(202);
dydt(1)=dydt(1)+k(6)*y(202);
dydt(191)=dydt(191)+k(6)*y(202);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(203)=dydt(203)-k(34)*y(203);
dydt(81)=dydt(81)+k(34)*y(203);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(203)=dydt(203)-k(33)*y(203);
dydt(300)=dydt(300)+k(33)*y(203);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(203)=dydt(203)-k(31)*y(203);
dydt(295)=dydt(295)+k(31)*y(203);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

dydt(203)=dydt(203)-k(22)*y(203);
dydt(197)=dydt(197)+k(22)*y(203);
dydt(75)=dydt(75)+k(22)*y(203);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(203)=dydt(203)-k(16)*y(203);
dydt(3)=dydt(3)+k(16)*y(203);
dydt(586)=dydt(586)+k(16)*y(203);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(203);
dydt(203)=dydt(203)-k(15)*y(3)*y(203);
dydt(647)=dydt(647)+k(15)*y(3)*y(203);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(203)=dydt(203)-k(8)*y(203);
dydt(62)=dydt(62)+k(8)*y(203);
dydt(257)=dydt(257)+k(8)*y(203);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(203)=dydt(203)-k(6)*y(203);
dydt(61)=dydt(61)+k(6)*y(203);
dydt(256)=dydt(256)+k(6)*y(203);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(204)=dydt(204)-k(34)*y(204);
dydt(66)=dydt(66)+k(34)*y(204);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(204)=dydt(204)-k(33)*y(204);
dydt(298)=dydt(298)+k(33)*y(204);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(204)=dydt(204)-k(22)*y(204);
dydt(197)=dydt(197)+k(22)*y(204);
dydt(60)=dydt(60)+k(22)*y(204);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(204)=dydt(204)-k(16)*y(204);
dydt(3)=dydt(3)+k(16)*y(204);
dydt(415)=dydt(415)+k(16)*y(204);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(204)=dydt(204)-k(13)*y(204);
dydt(206)=dydt(206)+k(13)*y(204);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(204)=dydt(204)-k(10)*y(204);
dydt(3)=dydt(3)+k(10)*y(204);
dydt(295)=dydt(295)+k(10)*y(204);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(204)=dydt(204)-k(8)*y(204);
dydt(62)=dydt(62)+k(8)*y(204);
dydt(335)=dydt(335)+k(8)*y(204);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(204)=dydt(204)-k(6)*y(204);
dydt(61)=dydt(61)+k(6)*y(204);
dydt(407)=dydt(407)+k(6)*y(204);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(205)=dydt(205)-k(34)*y(205);
dydt(40)=dydt(40)+k(34)*y(205);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(205)=dydt(205)-k(33)*y(205);
dydt(225)=dydt(225)+k(33)*y(205);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(205)=dydt(205)-k(32)*y(205);
dydt(226)=dydt(226)+k(32)*y(205);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(205)=dydt(205)-k(22)*y(205);
dydt(197)=dydt(197)+k(22)*y(205);
dydt(53)=dydt(53)+k(22)*y(205);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(205);
dydt(205)=dydt(205)-k(21)*y(197)*y(205);
dydt(475)=dydt(475)+k(21)*y(197)*y(205);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(205);
dydt(205)=dydt(205)-k(21)*y(32)*y(205);
dydt(473)=dydt(473)+k(21)*y(32)*y(205);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(205);
dydt(205)=dydt(205)-k(21)*y(2)*y(205);
dydt(428)=dydt(428)+k(21)*y(2)*y(205);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(205)=dydt(205)-k(16)*y(205);
dydt(3)=dydt(3)+k(16)*y(205);
dydt(401)=dydt(401)+k(16)*y(205);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(205)=dydt(205)-k(10)*y(205);
dydt(3)=dydt(3)+k(10)*y(205);
dydt(232)=dydt(232)+k(10)*y(205);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(205)=dydt(205)-k(8)*y(205);
dydt(48)=dydt(48)+k(8)*y(205);
dydt(228)=dydt(228)+k(8)*y(205);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(205)=dydt(205)-k(6)*y(205);
dydt(47)=dydt(47)+k(6)*y(205);
dydt(227)=dydt(227)+k(6)*y(205);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(206)=dydt(206)-k(34)*y(206);
dydt(59)=dydt(59)+k(34)*y(206);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(206)=dydt(206)-k(33)*y(206);
dydt(422)=dydt(422)+k(33)*y(206);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(206)=dydt(206)-k(32)*y(206);
dydt(204)=dydt(204)+k(32)*y(206);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(206)=dydt(206)-k(22)*y(206);
dydt(197)=dydt(197)+k(22)*y(206);
dydt(53)=dydt(53)+k(22)*y(206);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(206);
dydt(206)=dydt(206)-k(21)*y(197)*y(206);
dydt(475)=dydt(475)+k(21)*y(197)*y(206);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(206);
dydt(206)=dydt(206)-k(21)*y(32)*y(206);
dydt(477)=dydt(477)+k(21)*y(32)*y(206);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(206);
dydt(206)=dydt(206)-k(21)*y(2)*y(206);
dydt(207)=dydt(207)+k(21)*y(2)*y(206);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(206)=dydt(206)-k(16)*y(206);
dydt(3)=dydt(3)+k(16)*y(206);
dydt(410)=dydt(410)+k(16)*y(206);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(206)=dydt(206)-k(10)*y(206);
dydt(3)=dydt(3)+k(10)*y(206);
dydt(291)=dydt(291)+k(10)*y(206);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(206)=dydt(206)-k(8)*y(206);
dydt(55)=dydt(55)+k(8)*y(206);
dydt(335)=dydt(335)+k(8)*y(206);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(206)=dydt(206)-k(6)*y(206);
dydt(54)=dydt(54)+k(6)*y(206);
dydt(407)=dydt(407)+k(6)*y(206);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(207)=dydt(207)-k(34)*y(207);
dydt(52)=dydt(52)+k(34)*y(207);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(207)=dydt(207)-k(33)*y(207);
dydt(270)=dydt(270)+k(33)*y(207);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(207)=dydt(207)-k(29)*y(207);
dydt(289)=dydt(289)+k(29)*y(207);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(207)=dydt(207)-k(23)*y(207);
dydt(477)=dydt(477)+k(23)*y(207);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(207)=dydt(207)-k(22)*y(207);
dydt(197)=dydt(197)+k(22)*y(207);
dydt(46)=dydt(46)+k(22)*y(207);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(207)=dydt(207)-k(22)*y(207);
dydt(2)=dydt(2)+k(22)*y(207);
dydt(206)=dydt(206)+k(22)*y(207);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(207)=dydt(207)-k(16)*y(207);
dydt(3)=dydt(3)+k(16)*y(207);
dydt(436)=dydt(436)+k(16)*y(207);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(207)=dydt(207)-k(10)*y(207);
dydt(3)=dydt(3)+k(10)*y(207);
dydt(262)=dydt(262)+k(10)*y(207);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(207)=dydt(207)-k(8)*y(207);
dydt(335)=dydt(335)+k(8)*y(207);
dydt(44)=dydt(44)+k(8)*y(207);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(207)=dydt(207)-k(6)*y(207);
dydt(43)=dydt(43)+k(6)*y(207);
dydt(407)=dydt(407)+k(6)*y(207);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(209)=dydt(209)-k(34)*y(209);
dydt(28)=dydt(28)+k(34)*y(209);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(209)=dydt(209)-k(33)*y(209);
dydt(214)=dydt(214)+k(33)*y(209);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~pY) 

dydt(209)=dydt(209)-k(22)*y(209);
dydt(197)=dydt(197)+k(22)*y(209);
dydt(16)=dydt(16)+k(22)*y(209);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(209);
dydt(209)=dydt(209)-k(9)*y(3)*y(209);
dydt(336)=dydt(336)+k(9)*y(3)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(1)=dydt(1)-k(7)*y(1)*y(209);
dydt(209)=dydt(209)-k(7)*y(1)*y(209);
dydt(211)=dydt(211)+k(7)*y(1)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(7)=dydt(7)-k(7)*y(7)*y(209);
dydt(209)=dydt(209)-k(7)*y(7)*y(209);
dydt(208)=dydt(208)+k(7)*y(7)*y(209);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(209)=dydt(209)-k(7)*y(209)*y(335);
dydt(335)=dydt(335)-k(7)*y(209)*y(335);
dydt(403)=dydt(403)+k(7)*y(209)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(209);
dydt(209)=dydt(209)-k(7)*y(257)*y(209);
dydt(609)=dydt(609)+k(7)*y(257)*y(209);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(209)=dydt(209)-k(7)*y(209)*y(227);
dydt(227)=dydt(227)-k(7)*y(209)*y(227);
dydt(552)=dydt(552)+k(7)*y(209)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(209)=dydt(209)-k(7)*y(209)*y(222);
dydt(222)=dydt(222)-k(7)*y(209)*y(222);
dydt(520)=dydt(520)+k(7)*y(209)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(209);
dydt(209)=dydt(209)-k(7)*y(218)*y(209);
dydt(611)=dydt(611)+k(7)*y(218)*y(209);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) 

dydt(209)=dydt(209)-k(2)*y(209);
dydt(4)=dydt(4)+k(2)*y(209);
dydt(192)=dydt(192)+k(2)*y(209);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(210)=dydt(210)-k(34)*y(210);
dydt(29)=dydt(29)+k(34)*y(210);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(210)=dydt(210)-k(33)*y(210);
dydt(215)=dydt(215)+k(33)*y(210);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(210)=dydt(210)-k(22)*y(210);
dydt(197)=dydt(197)+k(22)*y(210);
dydt(17)=dydt(17)+k(22)*y(210);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(210)=dydt(210)-k(14)*y(210);
dydt(400)=dydt(400)+k(14)*y(210);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(210)=dydt(210)-k(10)*y(210);
dydt(3)=dydt(3)+k(10)*y(210);
dydt(208)=dydt(208)+k(10)*y(210);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(210)=dydt(210)-k(10)*y(210);
dydt(3)=dydt(3)+k(10)*y(210);
dydt(199)=dydt(199)+k(10)*y(210);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(210)=dydt(210)-k(8)*y(210);
dydt(7)=dydt(7)+k(8)*y(210);
dydt(336)=dydt(336)+k(8)*y(210);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(210)=dydt(210)-k(6)*y(210);
dydt(9)=dydt(9)+k(6)*y(210);
dydt(335)=dydt(335)+k(6)*y(210);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(211)=dydt(211)-k(34)*y(211);
dydt(30)=dydt(30)+k(34)*y(211);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(211)=dydt(211)-k(33)*y(211);
dydt(216)=dydt(216)+k(33)*y(211);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(211)=dydt(211)-k(22)*y(211);
dydt(197)=dydt(197)+k(22)*y(211);
dydt(18)=dydt(18)+k(22)*y(211);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(211);
dydt(211)=dydt(211)-k(9)*y(3)*y(211);
dydt(199)=dydt(199)+k(9)*y(3)*y(211);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(211);
dydt(211)=dydt(211)-k(9)*y(3)*y(211);
dydt(208)=dydt(208)+k(9)*y(3)*y(211);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

dydt(211)=dydt(211)-k(8)*y(211);
dydt(1)=dydt(1)+k(8)*y(211);
dydt(209)=dydt(209)+k(8)*y(211);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(211)=dydt(211)-k(6)*y(211);
dydt(6)=dydt(6)+k(6)*y(211);
dydt(192)=dydt(192)+k(6)*y(211);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(212)=dydt(212)-k(34)*y(212);
dydt(31)=dydt(31)+k(34)*y(212);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(212)=dydt(212)-k(33)*y(212);
dydt(217)=dydt(217)+k(33)*y(212);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(212)=dydt(212)-k(31)*y(212);
dydt(208)=dydt(208)+k(31)*y(212);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(212)=dydt(212)-k(22)*y(212);
dydt(197)=dydt(197)+k(22)*y(212);
dydt(19)=dydt(19)+k(22)*y(212);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(212);
dydt(212)=dydt(212)-k(15)*y(3)*y(212);
dydt(226)=dydt(226)+k(15)*y(3)*y(212);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(212)=dydt(212)-k(10)*y(212);
dydt(3)=dydt(3)+k(10)*y(212);
dydt(230)=dydt(230)+k(10)*y(212);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(212)=dydt(212)-k(8)*y(212);
dydt(7)=dydt(7)+k(8)*y(212);
dydt(319)=dydt(319)+k(8)*y(212);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(212)=dydt(212)-k(6)*y(212);
dydt(9)=dydt(9)+k(6)*y(212);
dydt(257)=dydt(257)+k(6)*y(212);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(214)=dydt(214)-k(34)*y(214);
dydt(22)=dydt(22)+k(34)*y(214);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~pY) 

dydt(214)=dydt(214)-k(22)*y(214);
dydt(182)=dydt(182)+k(22)*y(214);
dydt(16)=dydt(16)+k(22)*y(214);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(214);
dydt(214)=dydt(214)-k(9)*y(3)*y(214);
dydt(185)=dydt(185)+k(9)*y(3)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(1)=dydt(1)-k(7)*y(1)*y(214);
dydt(214)=dydt(214)-k(7)*y(1)*y(214);
dydt(216)=dydt(216)+k(7)*y(1)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(7)=dydt(7)-k(7)*y(7)*y(214);
dydt(214)=dydt(214)-k(7)*y(7)*y(214);
dydt(213)=dydt(213)+k(7)*y(7)*y(214);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(214)=dydt(214)-k(7)*y(214)*y(335);
dydt(335)=dydt(335)-k(7)*y(214)*y(335);
dydt(435)=dydt(435)+k(7)*y(214)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(214);
dydt(214)=dydt(214)-k(7)*y(257)*y(214);
dydt(550)=dydt(550)+k(7)*y(257)*y(214);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(214)=dydt(214)-k(7)*y(214)*y(227);
dydt(227)=dydt(227)-k(7)*y(214)*y(227);
dydt(548)=dydt(548)+k(7)*y(214)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(214)=dydt(214)-k(7)*y(214)*y(222);
dydt(222)=dydt(222)-k(7)*y(214)*y(222);
dydt(514)=dydt(514)+k(7)*y(214)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(214);
dydt(214)=dydt(214)-k(7)*y(218)*y(214);
dydt(513)=dydt(513)+k(7)*y(218)*y(214);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) 

dydt(214)=dydt(214)-k(2)*y(214);
dydt(4)=dydt(4)+k(2)*y(214);
dydt(177)=dydt(177)+k(2)*y(214);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(215)=dydt(215)-k(34)*y(215);
dydt(23)=dydt(23)+k(34)*y(215);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(215)=dydt(215)-k(23)*y(215);
dydt(210)=dydt(210)+k(23)*y(215);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(215)=dydt(215)-k(22)*y(215);
dydt(182)=dydt(182)+k(22)*y(215);
dydt(17)=dydt(17)+k(22)*y(215);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(215)=dydt(215)-k(14)*y(215);
dydt(457)=dydt(457)+k(14)*y(215);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(215)=dydt(215)-k(10)*y(215);
dydt(3)=dydt(3)+k(10)*y(215);
dydt(213)=dydt(213)+k(10)*y(215);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(215)=dydt(215)-k(10)*y(215);
dydt(3)=dydt(3)+k(10)*y(215);
dydt(652)=dydt(652)+k(10)*y(215);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(215)=dydt(215)-k(8)*y(215);
dydt(7)=dydt(7)+k(8)*y(215);
dydt(185)=dydt(185)+k(8)*y(215);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(215)=dydt(215)-k(6)*y(215);
dydt(9)=dydt(9)+k(6)*y(215);
dydt(184)=dydt(184)+k(6)*y(215);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(216)=dydt(216)-k(34)*y(216);
dydt(24)=dydt(24)+k(34)*y(216);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(216)=dydt(216)-k(22)*y(216);
dydt(182)=dydt(182)+k(22)*y(216);
dydt(18)=dydt(18)+k(22)*y(216);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(216);
dydt(216)=dydt(216)-k(9)*y(3)*y(216);
dydt(652)=dydt(652)+k(9)*y(3)*y(216);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(216);
dydt(216)=dydt(216)-k(9)*y(3)*y(216);
dydt(213)=dydt(213)+k(9)*y(3)*y(216);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

dydt(216)=dydt(216)-k(8)*y(216);
dydt(1)=dydt(1)+k(8)*y(216);
dydt(214)=dydt(214)+k(8)*y(216);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(216)=dydt(216)-k(6)*y(216);
dydt(6)=dydt(6)+k(6)*y(216);
dydt(177)=dydt(177)+k(6)*y(216);

% rule    : R17
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(218)=dydt(218)-k(34)*y(218);
dydt(111)=dydt(111)+k(34)*y(218);

% rule    : R14
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(218)=dydt(218)-k(31)*y(218);
dydt(177)=dydt(177)+k(31)*y(218);

% rule    : R9_op
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(218)=dydt(218)-k(22)*y(218);
dydt(182)=dydt(182)+k(22)*y(218);
dydt(77)=dydt(77)+k(22)*y(218);

% rule    : R6
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(218);
dydt(218)=dydt(218)-k(15)*y(3)*y(218);
dydt(222)=dydt(222)+k(15)*y(3)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(219);
dydt(219)=dydt(219)-k(7)*y(218)*y(219);
dydt(579)=dydt(579)+k(7)*y(218)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(223);
dydt(223)=dydt(223)-k(7)*y(218)*y(223);
dydt(268)=dydt(268)+k(7)*y(218)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(228);
dydt(228)=dydt(228)-k(7)*y(218)*y(228);
dydt(488)=dydt(488)+k(7)*y(218)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(315)=dydt(315)-k(7)*y(315)*y(218);
dydt(218)=dydt(218)-k(7)*y(315)*y(218);
dydt(497)=dydt(497)+k(7)*y(315)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(319);
dydt(319)=dydt(319)-k(7)*y(218)*y(319);
dydt(490)=dydt(490)+k(7)*y(218)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(336);
dydt(336)=dydt(336)-k(7)*y(218)*y(336);
dydt(363)=dydt(363)+k(7)*y(218)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(218)=dydt(218)-k(7)*y(218)*y(437);
dydt(437)=dydt(437)-k(7)*y(218)*y(437);
dydt(266)=dydt(266)+k(7)*y(218)*y(437);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(218)=dydt(218)-k(5)*y(218)*y(256);
dydt(256)=dydt(256)-k(5)*y(218)*y(256);
dydt(577)=dydt(577)+k(5)*y(218)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(218)=dydt(218)-k(5)*y(218)*y(264);
dydt(264)=dydt(264)-k(5)*y(218)*y(264);
dydt(579)=dydt(579)+k(5)*y(218)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(218)=dydt(218)-k(5)*y(218)*y(380);
dydt(380)=dydt(380)-k(5)*y(218)*y(380);
dydt(429)=dydt(429)+k(5)*y(218)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(218)=dydt(218)-k(5)*y(218)*y(391);
dydt(391)=dydt(391)-k(5)*y(218)*y(391);
dydt(395)=dydt(395)+k(5)*y(218)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(218)=dydt(218)-k(5)*y(218)*y(407);
dydt(407)=dydt(407)-k(5)*y(218)*y(407);
dydt(433)=dydt(433)+k(5)*y(218)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(218)=dydt(218)-k(5)*y(218)*y(441);
dydt(441)=dydt(441)-k(5)*y(218)*y(441);
dydt(432)=dydt(432)+k(5)*y(218)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(218)=dydt(218)-k(5)*y(218)*y(515);
dydt(515)=dydt(515)-k(5)*y(218)*y(515);
dydt(517)=dydt(517)+k(5)*y(218)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(218)=dydt(218)-k(5)*y(218)*y(543);
dydt(543)=dydt(543)-k(5)*y(218)*y(543);
dydt(551)=dydt(551)+k(5)*y(218)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(218)=dydt(218)-k(5)*y(218)*y(554);
dydt(554)=dydt(554)-k(5)*y(218)*y(554);
dydt(555)=dydt(555)+k(5)*y(218)*y(554);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(9)=dydt(9)-k(5)*y(9)*y(218);
dydt(218)=dydt(218)-k(5)*y(9)*y(218);
dydt(217)=dydt(217)+k(5)*y(9)*y(218);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(4)=dydt(4)-k(3)*y(4)*y(218);
dydt(218)=dydt(218)-k(3)*y(4)*y(218);
dydt(264)=dydt(264)+k(3)*y(4)*y(218);

% rule    : R1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(4)=dydt(4)-k(1)*y(4)*y(218);
dydt(218)=dydt(218)-k(1)*y(4)*y(218);
dydt(219)=dydt(219)+k(1)*y(4)*y(218);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(219)=dydt(219)-k(34)*y(219);
dydt(437)=dydt(437)+k(34)*y(219);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(219)=dydt(219)-k(31)*y(219);
dydt(214)=dydt(214)+k(31)*y(219);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~pY) 

dydt(219)=dydt(219)-k(22)*y(219);
dydt(182)=dydt(182)+k(22)*y(219);
dydt(315)=dydt(315)+k(22)*y(219);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(219);
dydt(219)=dydt(219)-k(15)*y(3)*y(219);
dydt(223)=dydt(223)+k(15)*y(3)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(1)=dydt(1)-k(7)*y(1)*y(219);
dydt(219)=dydt(219)-k(7)*y(1)*y(219);
dydt(220)=dydt(220)+k(7)*y(1)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(7)=dydt(7)-k(7)*y(7)*y(219);
dydt(219)=dydt(219)-k(7)*y(7)*y(219);
dydt(217)=dydt(217)+k(7)*y(7)*y(219);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(219)=dydt(219)-k(7)*y(219)*y(335);
dydt(335)=dydt(335)-k(7)*y(219)*y(335);
dydt(433)=dydt(433)+k(7)*y(219)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(219);
dydt(219)=dydt(219)-k(7)*y(257)*y(219);
dydt(577)=dydt(577)+k(7)*y(257)*y(219);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(219)=dydt(219)-k(7)*y(219)*y(227);
dydt(227)=dydt(227)-k(7)*y(219)*y(227);
dydt(551)=dydt(551)+k(7)*y(219)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(219)=dydt(219)-k(7)*y(219)*y(222);
dydt(222)=dydt(222)-k(7)*y(219)*y(222);
dydt(517)=dydt(517)+k(7)*y(219)*y(222);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) 

dydt(219)=dydt(219)-k(2)*y(219);
dydt(4)=dydt(4)+k(2)*y(219);
dydt(218)=dydt(218)+k(2)*y(219);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(220)=dydt(220)-k(34)*y(220);
dydt(621)=dydt(621)+k(34)*y(220);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(220)=dydt(220)-k(31)*y(220);
dydt(216)=dydt(216)+k(31)*y(220);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(220)=dydt(220)-k(22)*y(220);
dydt(182)=dydt(182)+k(22)*y(220);
dydt(234)=dydt(234)+k(22)*y(220);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(220);
dydt(220)=dydt(220)-k(15)*y(3)*y(220);
dydt(224)=dydt(224)+k(15)*y(3)*y(220);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(220);
dydt(220)=dydt(220)-k(9)*y(3)*y(220);
dydt(217)=dydt(217)+k(9)*y(3)*y(220);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

dydt(220)=dydt(220)-k(8)*y(220);
dydt(1)=dydt(1)+k(8)*y(220);
dydt(219)=dydt(219)+k(8)*y(220);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(220)=dydt(220)-k(6)*y(220);
dydt(6)=dydt(6)+k(6)*y(220);
dydt(218)=dydt(218)+k(6)*y(220);

% rule    : R17
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(222)=dydt(222)-k(34)*y(222);
dydt(43)=dydt(43)+k(34)*y(222);

% rule    : R9_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~Y, a~pY) 

dydt(222)=dydt(222)-k(22)*y(222);
dydt(182)=dydt(182)+k(22)*y(222);
dydt(54)=dydt(54)+k(22)*y(222);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lyn(U, SH2) 

dydt(222)=dydt(222)-k(16)*y(222);
dydt(3)=dydt(3)+k(16)*y(222);
dydt(218)=dydt(218)+k(16)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(222)=dydt(222)-k(7)*y(222)*y(223);
dydt(223)=dydt(223)-k(7)*y(222)*y(223);
dydt(640)=dydt(640)+k(7)*y(222)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(222)=dydt(222)-k(7)*y(222)*y(228);
dydt(228)=dydt(228)-k(7)*y(222)*y(228);
dydt(489)=dydt(489)+k(7)*y(222)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(315)=dydt(315)-k(7)*y(315)*y(222);
dydt(222)=dydt(222)-k(7)*y(315)*y(222);
dydt(623)=dydt(623)+k(7)*y(315)*y(222);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(319)=dydt(319)-k(7)*y(319)*y(222);
dydt(222)=dydt(222)-k(7)*y(319)*y(222);
dydt(626)=dydt(626)+k(7)*y(319)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(222)=dydt(222)-k(7)*y(222)*y(336);
dydt(336)=dydt(336)-k(7)*y(222)*y(336);
dydt(597)=dydt(597)+k(7)*y(222)*y(336);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(437)=dydt(437)-k(7)*y(437)*y(222);
dydt(222)=dydt(222)-k(7)*y(437)*y(222);
dydt(625)=dydt(625)+k(7)*y(437)*y(222);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(256)=dydt(256)-k(5)*y(256)*y(222);
dydt(222)=dydt(222)-k(5)*y(256)*y(222);
dydt(261)=dydt(261)+k(5)*y(256)*y(222);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(264)=dydt(264)-k(5)*y(264)*y(222);
dydt(222)=dydt(222)-k(5)*y(264)*y(222);
dydt(268)=dydt(268)+k(5)*y(264)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(222)=dydt(222)-k(5)*y(222)*y(380);
dydt(380)=dydt(380)-k(5)*y(222)*y(380);
dydt(279)=dydt(279)+k(5)*y(222)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(222)=dydt(222)-k(5)*y(222)*y(391);
dydt(391)=dydt(391)-k(5)*y(222)*y(391);
dydt(649)=dydt(649)+k(5)*y(222)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(222)=dydt(222)-k(5)*y(222)*y(407);
dydt(407)=dydt(407)-k(5)*y(222)*y(407);
dydt(289)=dydt(289)+k(5)*y(222)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(222)=dydt(222)-k(5)*y(222)*y(441);
dydt(441)=dydt(441)-k(5)*y(222)*y(441);
dydt(284)=dydt(284)+k(5)*y(222)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(222)=dydt(222)-k(5)*y(222)*y(515);
dydt(515)=dydt(515)-k(5)*y(222)*y(515);
dydt(640)=dydt(640)+k(5)*y(222)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(222)=dydt(222)-k(5)*y(222)*y(543);
dydt(543)=dydt(543)-k(5)*y(222)*y(543);
dydt(641)=dydt(641)+k(5)*y(222)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(222)=dydt(222)-k(5)*y(222)*y(554);
dydt(554)=dydt(554)-k(5)*y(222)*y(554);
dydt(639)=dydt(639)+k(5)*y(222)*y(554);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(9)=dydt(9)-k(5)*y(9)*y(222);
dydt(222)=dydt(222)-k(5)*y(9)*y(222);
dydt(221)=dydt(221)+k(5)*y(9)*y(222);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(4)=dydt(4)-k(3)*y(4)*y(222);
dydt(222)=dydt(222)-k(3)*y(4)*y(222);
dydt(515)=dydt(515)+k(3)*y(4)*y(222);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(4)=dydt(4)-k(1)*y(4)*y(222);
dydt(222)=dydt(222)-k(1)*y(4)*y(222);
dydt(223)=dydt(223)+k(1)*y(4)*y(222);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(223)=dydt(223)-k(34)*y(223);
dydt(44)=dydt(44)+k(34)*y(223);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

dydt(223)=dydt(223)-k(22)*y(223);
dydt(182)=dydt(182)+k(22)*y(223);
dydt(55)=dydt(55)+k(22)*y(223);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(223)=dydt(223)-k(16)*y(223);
dydt(3)=dydt(3)+k(16)*y(223);
dydt(219)=dydt(219)+k(16)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(1)=dydt(1)-k(7)*y(1)*y(223);
dydt(223)=dydt(223)-k(7)*y(1)*y(223);
dydt(224)=dydt(224)+k(7)*y(1)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(7)=dydt(7)-k(7)*y(7)*y(223);
dydt(223)=dydt(223)-k(7)*y(7)*y(223);
dydt(221)=dydt(221)+k(7)*y(7)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(7)*y(335)*y(223);
dydt(223)=dydt(223)-k(7)*y(335)*y(223);
dydt(289)=dydt(289)+k(7)*y(335)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(223);
dydt(223)=dydt(223)-k(7)*y(257)*y(223);
dydt(261)=dydt(261)+k(7)*y(257)*y(223);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(227)=dydt(227)-k(7)*y(227)*y(223);
dydt(223)=dydt(223)-k(7)*y(227)*y(223);
dydt(641)=dydt(641)+k(7)*y(227)*y(223);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) 

dydt(223)=dydt(223)-k(2)*y(223);
dydt(4)=dydt(4)+k(2)*y(223);
dydt(222)=dydt(222)+k(2)*y(223);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(224)=dydt(224)-k(34)*y(224);
dydt(45)=dydt(45)+k(34)*y(224);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

dydt(224)=dydt(224)-k(22)*y(224);
dydt(182)=dydt(182)+k(22)*y(224);
dydt(233)=dydt(233)+k(22)*y(224);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(224)=dydt(224)-k(20)*y(224);
dydt(282)=dydt(282)+k(20)*y(224);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(224)=dydt(224)-k(18)*y(224);
dydt(646)=dydt(646)+k(18)*y(224);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(224)=dydt(224)-k(16)*y(224);
dydt(3)=dydt(3)+k(16)*y(224);
dydt(220)=dydt(220)+k(16)*y(224);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(224);
dydt(224)=dydt(224)-k(9)*y(3)*y(224);
dydt(221)=dydt(221)+k(9)*y(3)*y(224);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

dydt(224)=dydt(224)-k(8)*y(224);
dydt(1)=dydt(1)+k(8)*y(224);
dydt(223)=dydt(223)+k(8)*y(224);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(224)=dydt(224)-k(6)*y(224);
dydt(6)=dydt(6)+k(6)*y(224);
dydt(222)=dydt(222)+k(6)*y(224);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(225)=dydt(225)-k(34)*y(225);
dydt(46)=dydt(46)+k(34)*y(225);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(225)=dydt(225)-k(32)*y(225);
dydt(221)=dydt(221)+k(32)*y(225);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(225)=dydt(225)-k(23)*y(225);
dydt(205)=dydt(205)+k(23)*y(225);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(225)=dydt(225)-k(22)*y(225);
dydt(182)=dydt(182)+k(22)*y(225);
dydt(53)=dydt(53)+k(22)*y(225);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(225);
dydt(225)=dydt(225)-k(21)*y(197)*y(225);
dydt(289)=dydt(289)+k(21)*y(197)*y(225);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(225);
dydt(225)=dydt(225)-k(21)*y(32)*y(225);
dydt(284)=dydt(284)+k(21)*y(32)*y(225);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(225);
dydt(225)=dydt(225)-k(21)*y(2)*y(225);
dydt(279)=dydt(279)+k(21)*y(2)*y(225);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(225)=dydt(225)-k(16)*y(225);
dydt(3)=dydt(3)+k(16)*y(225);
dydt(430)=dydt(430)+k(16)*y(225);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(225)=dydt(225)-k(10)*y(225);
dydt(3)=dydt(3)+k(10)*y(225);
dydt(282)=dydt(282)+k(10)*y(225);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(225)=dydt(225)-k(8)*y(225);
dydt(48)=dydt(48)+k(8)*y(225);
dydt(223)=dydt(223)+k(8)*y(225);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(225)=dydt(225)-k(6)*y(225);
dydt(47)=dydt(47)+k(6)*y(225);
dydt(222)=dydt(222)+k(6)*y(225);

% rule    : R17
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(227)=dydt(227)-k(34)*y(227);
dydt(37)=dydt(37)+k(34)*y(227);

% rule    : R16
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(227)=dydt(227)-k(33)*y(227);
dydt(222)=dydt(222)+k(33)*y(227);

% rule    : R9_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~pY, a~pY) 

dydt(227)=dydt(227)-k(22)*y(227);
dydt(197)=dydt(197)+k(22)*y(227);
dydt(54)=dydt(54)+k(22)*y(227);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lyn(U, SH2) 

dydt(227)=dydt(227)-k(16)*y(227);
dydt(3)=dydt(3)+k(16)*y(227);
dydt(257)=dydt(257)+k(16)*y(227);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(227)=dydt(227)-k(7)*y(227)*y(228);
dydt(228)=dydt(228)-k(7)*y(227)*y(228);
dydt(642)=dydt(642)+k(7)*y(227)*y(228);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(315)=dydt(315)-k(7)*y(315)*y(227);
dydt(227)=dydt(227)-k(7)*y(315)*y(227);
dydt(581)=dydt(581)+k(7)*y(315)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(319)=dydt(319)-k(7)*y(319)*y(227);
dydt(227)=dydt(227)-k(7)*y(319)*y(227);
dydt(565)=dydt(565)+k(7)*y(319)*y(227);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(227)=dydt(227)-k(7)*y(227)*y(336);
dydt(336)=dydt(336)-k(7)*y(227)*y(336);
dydt(601)=dydt(601)+k(7)*y(227)*y(336);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(437)=dydt(437)-k(7)*y(437)*y(227);
dydt(227)=dydt(227)-k(7)*y(437)*y(227);
dydt(546)=dydt(546)+k(7)*y(437)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(256)=dydt(256)-k(5)*y(256)*y(227);
dydt(227)=dydt(227)-k(5)*y(256)*y(227);
dydt(492)=dydt(492)+k(5)*y(256)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(264)=dydt(264)-k(5)*y(264)*y(227);
dydt(227)=dydt(227)-k(5)*y(264)*y(227);
dydt(488)=dydt(488)+k(5)*y(264)*y(227);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(227)=dydt(227)-k(5)*y(227)*y(380);
dydt(380)=dydt(380)-k(5)*y(227)*y(380);
dydt(428)=dydt(428)+k(5)*y(227)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(227)=dydt(227)-k(5)*y(227)*y(391);
dydt(391)=dydt(391)-k(5)*y(227)*y(391);
dydt(476)=dydt(476)+k(5)*y(227)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(227)=dydt(227)-k(5)*y(227)*y(407);
dydt(407)=dydt(407)-k(5)*y(227)*y(407);
dydt(475)=dydt(475)+k(5)*y(227)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(227)=dydt(227)-k(5)*y(227)*y(441);
dydt(441)=dydt(441)-k(5)*y(227)*y(441);
dydt(473)=dydt(473)+k(5)*y(227)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(227)=dydt(227)-k(5)*y(227)*y(515);
dydt(515)=dydt(515)-k(5)*y(227)*y(515);
dydt(489)=dydt(489)+k(5)*y(227)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(227)=dydt(227)-k(5)*y(227)*y(543);
dydt(543)=dydt(543)-k(5)*y(227)*y(543);
dydt(642)=dydt(642)+k(5)*y(227)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(227)=dydt(227)-k(5)*y(227)*y(554);
dydt(554)=dydt(554)-k(5)*y(227)*y(554);
dydt(561)=dydt(561)+k(5)*y(227)*y(554);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(9)=dydt(9)-k(5)*y(9)*y(227);
dydt(227)=dydt(227)-k(5)*y(9)*y(227);
dydt(226)=dydt(226)+k(5)*y(9)*y(227);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(4)=dydt(4)-k(3)*y(4)*y(227);
dydt(227)=dydt(227)-k(3)*y(4)*y(227);
dydt(543)=dydt(543)+k(3)*y(4)*y(227);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(4)=dydt(4)-k(1)*y(4)*y(227);
dydt(227)=dydt(227)-k(1)*y(4)*y(227);
dydt(228)=dydt(228)+k(1)*y(4)*y(227);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(228)=dydt(228)-k(34)*y(228);
dydt(38)=dydt(38)+k(34)*y(228);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(228)=dydt(228)-k(33)*y(228);
dydt(223)=dydt(223)+k(33)*y(228);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

dydt(228)=dydt(228)-k(22)*y(228);
dydt(197)=dydt(197)+k(22)*y(228);
dydt(55)=dydt(55)+k(22)*y(228);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(228)=dydt(228)-k(16)*y(228);
dydt(3)=dydt(3)+k(16)*y(228);
dydt(319)=dydt(319)+k(16)*y(228);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(1)=dydt(1)-k(7)*y(1)*y(228);
dydt(228)=dydt(228)-k(7)*y(1)*y(228);
dydt(229)=dydt(229)+k(7)*y(1)*y(228);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(7)=dydt(7)-k(7)*y(7)*y(228);
dydt(228)=dydt(228)-k(7)*y(7)*y(228);
dydt(226)=dydt(226)+k(7)*y(7)*y(228);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(7)*y(335)*y(228);
dydt(228)=dydt(228)-k(7)*y(335)*y(228);
dydt(475)=dydt(475)+k(7)*y(335)*y(228);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(228);
dydt(228)=dydt(228)-k(7)*y(257)*y(228);
dydt(492)=dydt(492)+k(7)*y(257)*y(228);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) 

dydt(228)=dydt(228)-k(2)*y(228);
dydt(4)=dydt(4)+k(2)*y(228);
dydt(227)=dydt(227)+k(2)*y(228);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(230)=dydt(230)-k(34)*y(230);
dydt(35)=dydt(35)+k(34)*y(230);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(230)=dydt(230)-k(33)*y(230);
dydt(220)=dydt(220)+k(33)*y(230);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(230)=dydt(230)-k(31)*y(230);
dydt(211)=dydt(211)+k(31)*y(230);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(230)=dydt(230)-k(22)*y(230);
dydt(197)=dydt(197)+k(22)*y(230);
dydt(234)=dydt(234)+k(22)*y(230);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(230);
dydt(230)=dydt(230)-k(15)*y(3)*y(230);
dydt(229)=dydt(229)+k(15)*y(3)*y(230);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(230);
dydt(230)=dydt(230)-k(9)*y(3)*y(230);
dydt(212)=dydt(212)+k(9)*y(3)*y(230);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

dydt(230)=dydt(230)-k(8)*y(230);
dydt(1)=dydt(1)+k(8)*y(230);
dydt(319)=dydt(319)+k(8)*y(230);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(230)=dydt(230)-k(6)*y(230);
dydt(6)=dydt(6)+k(6)*y(230);
dydt(257)=dydt(257)+k(6)*y(230);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(231)=dydt(231)-k(34)*y(231);
dydt(240)=dydt(240)+k(34)*y(231);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(231)=dydt(231)-k(33)*y(231);
dydt(646)=dydt(646)+k(33)*y(231);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(231)=dydt(231)-k(31)*y(231);
dydt(229)=dydt(229)+k(31)*y(231);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

dydt(231)=dydt(231)-k(22)*y(231);
dydt(197)=dydt(197)+k(22)*y(231);
dydt(235)=dydt(235)+k(22)*y(231);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(231)=dydt(231)-k(20)*y(231);
dydt(491)=dydt(491)+k(20)*y(231);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(231)=dydt(231)-k(16)*y(231);
dydt(3)=dydt(3)+k(16)*y(231);
dydt(321)=dydt(321)+k(16)*y(231);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(231);
dydt(231)=dydt(231)-k(15)*y(3)*y(231);
dydt(320)=dydt(320)+k(15)*y(3)*y(231);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

dydt(231)=dydt(231)-k(8)*y(231);
dydt(72)=dydt(72)+k(8)*y(231);
dydt(228)=dydt(228)+k(8)*y(231);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(231)=dydt(231)-k(6)*y(231);
dydt(71)=dydt(71)+k(6)*y(231);
dydt(227)=dydt(227)+k(6)*y(231);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(232)=dydt(232)-k(34)*y(232);
dydt(249)=dydt(249)+k(34)*y(232);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(232)=dydt(232)-k(33)*y(232);
dydt(282)=dydt(282)+k(33)*y(232);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(232)=dydt(232)-k(32)*y(232);
dydt(229)=dydt(229)+k(32)*y(232);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

dydt(232)=dydt(232)-k(22)*y(232);
dydt(197)=dydt(197)+k(22)*y(232);
dydt(56)=dydt(56)+k(22)*y(232);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(232);
dydt(232)=dydt(232)-k(21)*y(197)*y(232);
dydt(294)=dydt(294)+k(21)*y(197)*y(232);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(232);
dydt(232)=dydt(232)-k(21)*y(32)*y(232);
dydt(474)=dydt(474)+k(21)*y(32)*y(232);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(232);
dydt(232)=dydt(232)-k(21)*y(2)*y(232);
dydt(484)=dydt(484)+k(21)*y(2)*y(232);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(232)=dydt(232)-k(18)*y(232);
dydt(491)=dydt(491)+k(18)*y(232);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(232)=dydt(232)-k(16)*y(232);
dydt(3)=dydt(3)+k(16)*y(232);
dydt(618)=dydt(618)+k(16)*y(232);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(232);
dydt(232)=dydt(232)-k(9)*y(3)*y(232);
dydt(205)=dydt(205)+k(9)*y(3)*y(232);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

dydt(232)=dydt(232)-k(8)*y(232);
dydt(15)=dydt(15)+k(8)*y(232);
dydt(228)=dydt(228)+k(8)*y(232);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(232)=dydt(232)-k(6)*y(232);
dydt(98)=dydt(98)+k(6)*y(232);
dydt(227)=dydt(227)+k(6)*y(232);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(234)=dydt(234)-k(32)*y(234);
dydt(69)=dydt(69)+k(32)*y(234);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(234)=dydt(234)-k(31)*y(234);
dydt(18)=dydt(18)+k(31)*y(234);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(234);
dydt(234)=dydt(234)-k(21)*y(197)*y(234);
dydt(230)=dydt(230)+k(21)*y(197)*y(234);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(234);
dydt(234)=dydt(234)-k(21)*y(32)*y(234);
dydt(35)=dydt(35)+k(21)*y(32)*y(234);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(234);
dydt(234)=dydt(234)-k(21)*y(2)*y(234);
dydt(621)=dydt(621)+k(21)*y(2)*y(234);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(234);
dydt(234)=dydt(234)-k(15)*y(3)*y(234);
dydt(233)=dydt(233)+k(15)*y(3)*y(234);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(234);
dydt(234)=dydt(234)-k(9)*y(3)*y(234);
dydt(19)=dydt(19)+k(9)*y(3)*y(234);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) 

dydt(234)=dydt(234)-k(8)*y(234);
dydt(1)=dydt(1)+k(8)*y(234);
dydt(315)=dydt(315)+k(8)*y(234);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) 

dydt(234)=dydt(234)-k(6)*y(234);
dydt(77)=dydt(77)+k(6)*y(234);
dydt(6)=dydt(6)+k(6)*y(234);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(236)=dydt(236)-k(32)*y(236);
dydt(73)=dydt(73)+k(32)*y(236);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(197)=dydt(197)-k(21)*y(197)*y(236);
dydt(236)=dydt(236)-k(21)*y(197)*y(236);
dydt(320)=dydt(320)+k(21)*y(197)*y(236);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(236);
dydt(236)=dydt(236)-k(21)*y(32)*y(236);
dydt(241)=dydt(241)+k(21)*y(32)*y(236);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(236);
dydt(236)=dydt(236)-k(21)*y(2)*y(236);
dydt(654)=dydt(654)+k(21)*y(2)*y(236);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(236)=dydt(236)-k(20)*y(236);
dydt(627)=dydt(627)+k(20)*y(236);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) 

dydt(236)=dydt(236)-k(16)*y(236);
dydt(3)=dydt(3)+k(16)*y(236);
dydt(235)=dydt(235)+k(16)*y(236);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) 

dydt(236)=dydt(236)-k(16)*y(236);
dydt(3)=dydt(3)+k(16)*y(236);
dydt(314)=dydt(314)+k(16)*y(236);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(236)=dydt(236)-k(8)*y(236);
dydt(61)=dydt(61)+k(8)*y(236);
dydt(55)=dydt(55)+k(8)*y(236);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(236)=dydt(236)-k(6)*y(236);
dydt(54)=dydt(54)+k(6)*y(236);
dydt(122)=dydt(122)+k(6)*y(236);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(237)=dydt(237)-k(32)*y(237);
dydt(74)=dydt(74)+k(32)*y(237);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(237)=dydt(237)-k(31)*y(237);
dydt(234)=dydt(234)+k(31)*y(237);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(237)=dydt(237)-k(31)*y(237);
dydt(331)=dydt(331)+k(31)*y(237);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(237);
dydt(237)=dydt(237)-k(21)*y(197)*y(237);
dydt(321)=dydt(321)+k(21)*y(197)*y(237);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(237);
dydt(237)=dydt(237)-k(21)*y(32)*y(237);
dydt(242)=dydt(242)+k(21)*y(32)*y(237);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(237);
dydt(237)=dydt(237)-k(21)*y(2)*y(237);
dydt(653)=dydt(653)+k(21)*y(2)*y(237);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(237);
dydt(237)=dydt(237)-k(15)*y(3)*y(237);
dydt(235)=dydt(235)+k(15)*y(3)*y(237);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(3)=dydt(3)-k(15)*y(3)*y(237);
dydt(237)=dydt(237)-k(15)*y(3)*y(237);
dydt(314)=dydt(314)+k(15)*y(3)*y(237);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) 

dydt(237)=dydt(237)-k(8)*y(237);
dydt(72)=dydt(72)+k(8)*y(237);
dydt(315)=dydt(315)+k(8)*y(237);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) 

dydt(237)=dydt(237)-k(6)*y(237);
dydt(77)=dydt(77)+k(6)*y(237);
dydt(71)=dydt(71)+k(6)*y(237);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(238)=dydt(238)-k(32)*y(238);
dydt(235)=dydt(235)+k(32)*y(238);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(238)=dydt(238)-k(32)*y(238);
dydt(75)=dydt(75)+k(32)*y(238);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(238)=dydt(238)-k(31)*y(238);
dydt(56)=dydt(56)+k(31)*y(238);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(238);
dydt(238)=dydt(238)-k(21)*y(197)*y(238);
dydt(491)=dydt(491)+k(21)*y(197)*y(238);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(238);
dydt(238)=dydt(238)-k(21)*y(32)*y(238);
dydt(243)=dydt(243)+k(21)*y(32)*y(238);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(238);
dydt(238)=dydt(238)-k(21)*y(2)*y(238);
dydt(250)=dydt(250)+k(21)*y(2)*y(238);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(238);
dydt(238)=dydt(238)-k(21)*y(197)*y(238);
dydt(260)=dydt(260)+k(21)*y(197)*y(238);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(238);
dydt(238)=dydt(238)-k(21)*y(32)*y(238);
dydt(634)=dydt(634)+k(21)*y(32)*y(238);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(238);
dydt(238)=dydt(238)-k(21)*y(2)*y(238);
dydt(629)=dydt(629)+k(21)*y(2)*y(238);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

dydt(238)=dydt(238)-k(16)*y(238);
dydt(3)=dydt(3)+k(16)*y(238);
dydt(584)=dydt(584)+k(16)*y(238);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(238);
dydt(238)=dydt(238)-k(15)*y(3)*y(238);
dydt(627)=dydt(627)+k(15)*y(3)*y(238);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) 

dydt(238)=dydt(238)-k(8)*y(238);
dydt(77)=dydt(77)+k(8)*y(238);
dydt(55)=dydt(55)+k(8)*y(238);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(238)=dydt(238)-k(6)*y(238);
dydt(76)=dydt(76)+k(6)*y(238);
dydt(54)=dydt(54)+k(6)*y(238);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(239)=dydt(239)-k(31)*y(239);
dydt(45)=dydt(45)+k(31)*y(239);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

dydt(239)=dydt(239)-k(22)*y(239);
dydt(2)=dydt(2)+k(22)*y(239);
dydt(235)=dydt(235)+k(22)*y(239);

% rule    : R8_2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(239)=dydt(239)-k(20)*y(239);
dydt(250)=dydt(250)+k(20)*y(239);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(239)=dydt(239)-k(16)*y(239);
dydt(3)=dydt(3)+k(16)*y(239);
dydt(653)=dydt(653)+k(16)*y(239);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(239);
dydt(239)=dydt(239)-k(15)*y(3)*y(239);
dydt(654)=dydt(654)+k(15)*y(3)*y(239);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

dydt(239)=dydt(239)-k(8)*y(239);
dydt(72)=dydt(72)+k(8)*y(239);
dydt(44)=dydt(44)+k(8)*y(239);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(239)=dydt(239)-k(6)*y(239);
dydt(71)=dydt(71)+k(6)*y(239);
dydt(43)=dydt(43)+k(6)*y(239);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(241)=dydt(241)-k(33)*y(241);
dydt(654)=dydt(654)+k(33)*y(241);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(241)=dydt(241)-k(22)*y(241);
dydt(32)=dydt(32)+k(22)*y(241);
dydt(236)=dydt(236)+k(22)*y(241);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(241)=dydt(241)-k(20)*y(241);
dydt(244)=dydt(244)+k(20)*y(241);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(241)=dydt(241)-k(16)*y(241);
dydt(3)=dydt(3)+k(16)*y(241);
dydt(240)=dydt(240)+k(16)*y(241);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(241)=dydt(241)-k(16)*y(241);
dydt(3)=dydt(3)+k(16)*y(241);
dydt(317)=dydt(317)+k(16)*y(241);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(241)=dydt(241)-k(8)*y(241);
dydt(61)=dydt(61)+k(8)*y(241);
dydt(38)=dydt(38)+k(8)*y(241);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(241)=dydt(241)-k(6)*y(241);
dydt(122)=dydt(122)+k(6)*y(241);
dydt(37)=dydt(37)+k(6)*y(241);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(242)=dydt(242)-k(33)*y(242);
dydt(653)=dydt(653)+k(33)*y(242);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(242)=dydt(242)-k(31)*y(242);
dydt(35)=dydt(35)+k(31)*y(242);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(242)=dydt(242)-k(31)*y(242);
dydt(136)=dydt(136)+k(31)*y(242);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(242)=dydt(242)-k(22)*y(242);
dydt(32)=dydt(32)+k(22)*y(242);
dydt(237)=dydt(237)+k(22)*y(242);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(242);
dydt(242)=dydt(242)-k(15)*y(3)*y(242);
dydt(240)=dydt(240)+k(15)*y(3)*y(242);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(242);
dydt(242)=dydt(242)-k(15)*y(3)*y(242);
dydt(317)=dydt(317)+k(15)*y(3)*y(242);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

dydt(242)=dydt(242)-k(8)*y(242);
dydt(72)=dydt(72)+k(8)*y(242);
dydt(34)=dydt(34)+k(8)*y(242);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(242)=dydt(242)-k(6)*y(242);
dydt(71)=dydt(71)+k(6)*y(242);
dydt(33)=dydt(33)+k(6)*y(242);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(244)=dydt(244)-k(33)*y(244);
dydt(251)=dydt(251)+k(33)*y(244);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(244)=dydt(244)-k(32)*y(244);
dydt(241)=dydt(241)+k(32)*y(244);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(244)=dydt(244)-k(22)*y(244);
dydt(32)=dydt(32)+k(22)*y(244);
dydt(627)=dydt(627)+k(22)*y(244);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(244);
dydt(244)=dydt(244)-k(21)*y(197)*y(244);
dydt(643)=dydt(643)+k(21)*y(197)*y(244);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(244);
dydt(244)=dydt(244)-k(21)*y(32)*y(244);
dydt(567)=dydt(567)+k(21)*y(32)*y(244);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(244);
dydt(244)=dydt(244)-k(21)*y(2)*y(244);
dydt(631)=dydt(631)+k(21)*y(2)*y(244);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(244)=dydt(244)-k(16)*y(244);
dydt(3)=dydt(3)+k(16)*y(244);
dydt(243)=dydt(243)+k(16)*y(244);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(244)=dydt(244)-k(16)*y(244);
dydt(3)=dydt(3)+k(16)*y(244);
dydt(534)=dydt(534)+k(16)*y(244);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(244)=dydt(244)-k(8)*y(244);
dydt(54)=dydt(54)+k(8)*y(244);
dydt(38)=dydt(38)+k(8)*y(244);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(244)=dydt(244)-k(6)*y(244);
dydt(118)=dydt(118)+k(6)*y(244);
dydt(37)=dydt(37)+k(6)*y(244);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(245)=dydt(245)-k(33)*y(245);
dydt(252)=dydt(252)+k(33)*y(245);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(245)=dydt(245)-k(32)*y(245);
dydt(242)=dydt(242)+k(32)*y(245);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(245)=dydt(245)-k(31)*y(245);
dydt(617)=dydt(617)+k(31)*y(245);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(245)=dydt(245)-k(31)*y(245);
dydt(533)=dydt(533)+k(31)*y(245);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(245)=dydt(245)-k(22)*y(245);
dydt(32)=dydt(32)+k(22)*y(245);
dydt(584)=dydt(584)+k(22)*y(245);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(245);
dydt(245)=dydt(245)-k(21)*y(197)*y(245);
dydt(582)=dydt(582)+k(21)*y(197)*y(245);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(245);
dydt(245)=dydt(245)-k(21)*y(32)*y(245);
dydt(568)=dydt(568)+k(21)*y(32)*y(245);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(245);
dydt(245)=dydt(245)-k(21)*y(2)*y(245);
dydt(499)=dydt(499)+k(21)*y(2)*y(245);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(245);
dydt(245)=dydt(245)-k(15)*y(3)*y(245);
dydt(243)=dydt(243)+k(15)*y(3)*y(245);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(245);
dydt(245)=dydt(245)-k(15)*y(3)*y(245);
dydt(534)=dydt(534)+k(15)*y(3)*y(245);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

dydt(245)=dydt(245)-k(8)*y(245);
dydt(77)=dydt(77)+k(8)*y(245);
dydt(34)=dydt(34)+k(8)*y(245);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(245)=dydt(245)-k(6)*y(245);
dydt(76)=dydt(76)+k(6)*y(245);
dydt(33)=dydt(33)+k(6)*y(245);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(246)=dydt(246)-k(33)*y(246);
dydt(253)=dydt(253)+k(33)*y(246);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(246)=dydt(246)-k(31)*y(246);
dydt(481)=dydt(481)+k(31)*y(246);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(246)=dydt(246)-k(28)*y(246);
dydt(493)=dydt(493)+k(28)*y(246);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(246)=dydt(246)-k(27)*y(246);
dydt(485)=dydt(485)+k(27)*y(246);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(246)=dydt(246)-k(26)*y(246);
dydt(247)=dydt(247)+k(26)*y(246);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(246)=dydt(246)-k(22)*y(246);
dydt(2)=dydt(2)+k(22)*y(246);
dydt(243)=dydt(243)+k(22)*y(246);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(246)=dydt(246)-k(22)*y(246);
dydt(32)=dydt(32)+k(22)*y(246);
dydt(629)=dydt(629)+k(22)*y(246);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(246)=dydt(246)-k(16)*y(246);
dydt(3)=dydt(3)+k(16)*y(246);
dydt(499)=dydt(499)+k(16)*y(246);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(246);
dydt(246)=dydt(246)-k(15)*y(3)*y(246);
dydt(631)=dydt(631)+k(15)*y(3)*y(246);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(246)=dydt(246)-k(8)*y(246);
dydt(111)=dydt(111)+k(8)*y(246);
dydt(38)=dydt(38)+k(8)*y(246);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(246)=dydt(246)-k(6)*y(246);
dydt(110)=dydt(110)+k(6)*y(246);
dydt(37)=dydt(37)+k(6)*y(246);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(247)=dydt(247)-k(33)*y(247);
dydt(246)=dydt(246)+k(33)*y(247);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(247)=dydt(247)-k(33)*y(247);
dydt(254)=dydt(254)+k(33)*y(247);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(247)=dydt(247)-k(31)*y(247);
dydt(483)=dydt(483)+k(31)*y(247);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(247)=dydt(247)-k(28)*y(247);
dydt(248)=dydt(248)+k(28)*y(247);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(247)=dydt(247)-k(27)*y(247);
dydt(635)=dydt(635)+k(27)*y(247);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(247)=dydt(247)-k(22)*y(247);
dydt(32)=dydt(32)+k(22)*y(247);
dydt(243)=dydt(243)+k(22)*y(247);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(247)=dydt(247)-k(22)*y(247);
dydt(32)=dydt(32)+k(22)*y(247);
dydt(634)=dydt(634)+k(22)*y(247);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(247)=dydt(247)-k(16)*y(247);
dydt(3)=dydt(3)+k(16)*y(247);
dydt(568)=dydt(568)+k(16)*y(247);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(247);
dydt(247)=dydt(247)-k(15)*y(3)*y(247);
dydt(567)=dydt(567)+k(15)*y(3)*y(247);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(247)=dydt(247)-k(8)*y(247);
dydt(33)=dydt(33)+k(8)*y(247);
dydt(38)=dydt(38)+k(8)*y(247);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(247)=dydt(247)-k(6)*y(247);
dydt(82)=dydt(82)+k(6)*y(247);
dydt(37)=dydt(37)+k(6)*y(247);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(248)=dydt(248)-k(34)*y(248);
dydt(247)=dydt(247)+k(34)*y(248);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(248)=dydt(248)-k(33)*y(248);
dydt(493)=dydt(493)+k(33)*y(248);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(248)=dydt(248)-k(33)*y(248);
dydt(255)=dydt(255)+k(33)*y(248);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(248)=dydt(248)-k(31)*y(248);
dydt(293)=dydt(293)+k(31)*y(248);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(248)=dydt(248)-k(29)*y(248);
dydt(492)=dydt(492)+k(29)*y(248);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(248)=dydt(248)-k(22)*y(248);
dydt(197)=dydt(197)+k(22)*y(248);
dydt(243)=dydt(243)+k(22)*y(248);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(248)=dydt(248)-k(22)*y(248);
dydt(32)=dydt(32)+k(22)*y(248);
dydt(260)=dydt(260)+k(22)*y(248);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(248)=dydt(248)-k(16)*y(248);
dydt(3)=dydt(3)+k(16)*y(248);
dydt(582)=dydt(582)+k(16)*y(248);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(248);
dydt(248)=dydt(248)-k(15)*y(3)*y(248);
dydt(643)=dydt(643)+k(15)*y(3)*y(248);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(248)=dydt(248)-k(8)*y(248);
dydt(257)=dydt(257)+k(8)*y(248);
dydt(38)=dydt(38)+k(8)*y(248);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(248)=dydt(248)-k(6)*y(248);
dydt(256)=dydt(256)+k(6)*y(248);
dydt(37)=dydt(37)+k(6)*y(248);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(249)=dydt(249)-k(33)*y(249);
dydt(49)=dydt(49)+k(33)*y(249);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(249)=dydt(249)-k(32)*y(249);
dydt(39)=dydt(39)+k(32)*y(249);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

dydt(249)=dydt(249)-k(22)*y(249);
dydt(32)=dydt(32)+k(22)*y(249);
dydt(56)=dydt(56)+k(22)*y(249);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(249);
dydt(249)=dydt(249)-k(21)*y(197)*y(249);
dydt(293)=dydt(293)+k(21)*y(197)*y(249);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(249);
dydt(249)=dydt(249)-k(21)*y(32)*y(249);
dydt(483)=dydt(483)+k(21)*y(32)*y(249);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(249);
dydt(249)=dydt(249)-k(21)*y(2)*y(249);
dydt(481)=dydt(481)+k(21)*y(2)*y(249);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(249)=dydt(249)-k(18)*y(249);
dydt(243)=dydt(243)+k(18)*y(249);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(249)=dydt(249)-k(16)*y(249);
dydt(3)=dydt(3)+k(16)*y(249);
dydt(617)=dydt(617)+k(16)*y(249);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(249);
dydt(249)=dydt(249)-k(9)*y(3)*y(249);
dydt(40)=dydt(40)+k(9)*y(3)*y(249);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

dydt(249)=dydt(249)-k(8)*y(249);
dydt(15)=dydt(15)+k(8)*y(249);
dydt(38)=dydt(38)+k(8)*y(249);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(249)=dydt(249)-k(6)*y(249);
dydt(98)=dydt(98)+k(6)*y(249);
dydt(37)=dydt(37)+k(6)*y(249);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(251)=dydt(251)-k(32)*y(251);
dydt(654)=dydt(654)+k(32)*y(251);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(251)=dydt(251)-k(25)*y(251);
dydt(244)=dydt(244)+k(25)*y(251);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(251)=dydt(251)-k(22)*y(251);
dydt(2)=dydt(2)+k(22)*y(251);
dydt(627)=dydt(627)+k(22)*y(251);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(251);
dydt(251)=dydt(251)-k(21)*y(197)*y(251);
dydt(258)=dydt(258)+k(21)*y(197)*y(251);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(251);
dydt(251)=dydt(251)-k(21)*y(32)*y(251);
dydt(571)=dydt(571)+k(21)*y(32)*y(251);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(251);
dydt(251)=dydt(251)-k(21)*y(2)*y(251);
dydt(630)=dydt(630)+k(21)*y(2)*y(251);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(251)=dydt(251)-k(16)*y(251);
dydt(3)=dydt(3)+k(16)*y(251);
dydt(250)=dydt(250)+k(16)*y(251);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(251)=dydt(251)-k(16)*y(251);
dydt(3)=dydt(3)+k(16)*y(251);
dydt(540)=dydt(540)+k(16)*y(251);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(251)=dydt(251)-k(8)*y(251);
dydt(54)=dydt(54)+k(8)*y(251);
dydt(44)=dydt(44)+k(8)*y(251);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(251)=dydt(251)-k(6)*y(251);
dydt(118)=dydt(118)+k(6)*y(251);
dydt(43)=dydt(43)+k(6)*y(251);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(252)=dydt(252)-k(32)*y(252);
dydt(653)=dydt(653)+k(32)*y(252);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(252)=dydt(252)-k(31)*y(252);
dydt(616)=dydt(616)+k(31)*y(252);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(252)=dydt(252)-k(31)*y(252);
dydt(508)=dydt(508)+k(31)*y(252);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(252)=dydt(252)-k(22)*y(252);
dydt(2)=dydt(2)+k(22)*y(252);
dydt(584)=dydt(584)+k(22)*y(252);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(252);
dydt(252)=dydt(252)-k(21)*y(197)*y(252);
dydt(259)=dydt(259)+k(21)*y(197)*y(252);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(252);
dydt(252)=dydt(252)-k(21)*y(32)*y(252);
dydt(572)=dydt(572)+k(21)*y(32)*y(252);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(252);
dydt(252)=dydt(252)-k(21)*y(2)*y(252);
dydt(503)=dydt(503)+k(21)*y(2)*y(252);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(252);
dydt(252)=dydt(252)-k(15)*y(3)*y(252);
dydt(250)=dydt(250)+k(15)*y(3)*y(252);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(252);
dydt(252)=dydt(252)-k(15)*y(3)*y(252);
dydt(540)=dydt(540)+k(15)*y(3)*y(252);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

dydt(252)=dydt(252)-k(8)*y(252);
dydt(77)=dydt(77)+k(8)*y(252);
dydt(437)=dydt(437)+k(8)*y(252);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(252)=dydt(252)-k(6)*y(252);
dydt(76)=dydt(76)+k(6)*y(252);
dydt(111)=dydt(111)+k(6)*y(252);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(253)=dydt(253)-k(31)*y(253);
dydt(274)=dydt(274)+k(31)*y(253);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(253)=dydt(253)-k(28)*y(253);
dydt(263)=dydt(263)+k(28)*y(253);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(253)=dydt(253)-k(27)*y(253);
dydt(281)=dydt(281)+k(27)*y(253);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(253)=dydt(253)-k(26)*y(253);
dydt(254)=dydt(254)+k(26)*y(253);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(253)=dydt(253)-k(22)*y(253);
dydt(2)=dydt(2)+k(22)*y(253);
dydt(250)=dydt(250)+k(22)*y(253);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(253)=dydt(253)-k(22)*y(253);
dydt(2)=dydt(2)+k(22)*y(253);
dydt(629)=dydt(629)+k(22)*y(253);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(253)=dydt(253)-k(16)*y(253);
dydt(3)=dydt(3)+k(16)*y(253);
dydt(503)=dydt(503)+k(16)*y(253);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(253);
dydt(253)=dydt(253)-k(15)*y(3)*y(253);
dydt(630)=dydt(630)+k(15)*y(3)*y(253);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(253)=dydt(253)-k(8)*y(253);
dydt(111)=dydt(111)+k(8)*y(253);
dydt(44)=dydt(44)+k(8)*y(253);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(253)=dydt(253)-k(6)*y(253);
dydt(110)=dydt(110)+k(6)*y(253);
dydt(43)=dydt(43)+k(6)*y(253);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(254)=dydt(254)-k(33)*y(254);
dydt(253)=dydt(253)+k(33)*y(254);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(254)=dydt(254)-k(31)*y(254);
dydt(277)=dydt(277)+k(31)*y(254);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(254)=dydt(254)-k(28)*y(254);
dydt(255)=dydt(255)+k(28)*y(254);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(254)=dydt(254)-k(27)*y(254);
dydt(286)=dydt(286)+k(27)*y(254);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(254)=dydt(254)-k(22)*y(254);
dydt(32)=dydt(32)+k(22)*y(254);
dydt(250)=dydt(250)+k(22)*y(254);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(254)=dydt(254)-k(22)*y(254);
dydt(2)=dydt(2)+k(22)*y(254);
dydt(634)=dydt(634)+k(22)*y(254);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(254)=dydt(254)-k(16)*y(254);
dydt(3)=dydt(3)+k(16)*y(254);
dydt(572)=dydt(572)+k(16)*y(254);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(254);
dydt(254)=dydt(254)-k(15)*y(3)*y(254);
dydt(571)=dydt(571)+k(15)*y(3)*y(254);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(254)=dydt(254)-k(8)*y(254);
dydt(33)=dydt(33)+k(8)*y(254);
dydt(44)=dydt(44)+k(8)*y(254);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(254)=dydt(254)-k(6)*y(254);
dydt(82)=dydt(82)+k(6)*y(254);
dydt(43)=dydt(43)+k(6)*y(254);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(256)=dydt(256)-k(34)*y(256);
dydt(82)=dydt(82)+k(34)*y(256);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(256)=dydt(256)-k(33)*y(256);
dydt(264)=dydt(264)+k(33)*y(256);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(256)=dydt(256)-k(31)*y(256);
dydt(191)=dydt(191)+k(31)*y(256);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~pY) 

dydt(256)=dydt(256)-k(22)*y(256);
dydt(197)=dydt(197)+k(22)*y(256);
dydt(76)=dydt(76)+k(22)*y(256);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(256);
dydt(256)=dydt(256)-k(15)*y(3)*y(256);
dydt(543)=dydt(543)+k(15)*y(3)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(1)=dydt(1)-k(5)*y(1)*y(256);
dydt(256)=dydt(256)-k(5)*y(1)*y(256);
dydt(587)=dydt(587)+k(5)*y(1)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(7)=dydt(7)-k(5)*y(7)*y(256);
dydt(256)=dydt(256)-k(5)*y(7)*y(256);
dydt(588)=dydt(588)+k(5)*y(7)*y(256);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(256)=dydt(256)-k(5)*y(256)*y(335);
dydt(335)=dydt(335)-k(5)*y(256)*y(335);
dydt(342)=dydt(342)+k(5)*y(256)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(257)=dydt(257)-k(5)*y(257)*y(256);
dydt(256)=dydt(256)-k(5)*y(257)*y(256);
dydt(583)=dydt(583)+k(5)*y(257)*y(256);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) 

dydt(256)=dydt(256)-k(4)*y(256);
dydt(4)=dydt(4)+k(4)*y(256);
dydt(257)=dydt(257)+k(4)*y(256);

% rule    : R17
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(257)=dydt(257)-k(34)*y(257);
dydt(33)=dydt(33)+k(34)*y(257);

% rule    : R16
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(257)=dydt(257)-k(33)*y(257);
dydt(218)=dydt(218)+k(33)*y(257);

% rule    : R14
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(257)=dydt(257)-k(31)*y(257);
dydt(192)=dydt(192)+k(31)*y(257);

% rule    : R9_op
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(257)=dydt(257)-k(22)*y(257);
dydt(197)=dydt(197)+k(22)*y(257);
dydt(77)=dydt(77)+k(22)*y(257);

% rule    : R6
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(257);
dydt(257)=dydt(257)-k(15)*y(3)*y(257);
dydt(227)=dydt(227)+k(15)*y(3)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(315)=dydt(315)-k(7)*y(315)*y(257);
dydt(257)=dydt(257)-k(7)*y(315)*y(257);
dydt(578)=dydt(578)+k(7)*y(315)*y(257);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(319);
dydt(319)=dydt(319)-k(7)*y(257)*y(319);
dydt(583)=dydt(583)+k(7)*y(257)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(336);
dydt(336)=dydt(336)-k(7)*y(257)*y(336);
dydt(342)=dydt(342)+k(7)*y(257)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(257)=dydt(257)-k(7)*y(257)*y(437);
dydt(437)=dydt(437)-k(7)*y(257)*y(437);
dydt(259)=dydt(259)+k(7)*y(257)*y(437);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(257)=dydt(257)-k(5)*y(257)*y(264);
dydt(264)=dydt(264)-k(5)*y(257)*y(264);
dydt(490)=dydt(490)+k(5)*y(257)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(5)*y(257)*y(380);
dydt(380)=dydt(380)-k(5)*y(257)*y(380);
dydt(426)=dydt(426)+k(5)*y(257)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(257)=dydt(257)-k(5)*y(257)*y(391);
dydt(391)=dydt(391)-k(5)*y(257)*y(391);
dydt(398)=dydt(398)+k(5)*y(257)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(5)*y(257)*y(407);
dydt(407)=dydt(407)-k(5)*y(257)*y(407);
dydt(434)=dydt(434)+k(5)*y(257)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(257)=dydt(257)-k(5)*y(257)*y(441);
dydt(441)=dydt(441)-k(5)*y(257)*y(441);
dydt(471)=dydt(471)+k(5)*y(257)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(5)*y(257)*y(515);
dydt(515)=dydt(515)-k(5)*y(257)*y(515);
dydt(626)=dydt(626)+k(5)*y(257)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(257)=dydt(257)-k(5)*y(257)*y(543);
dydt(543)=dydt(543)-k(5)*y(257)*y(543);
dydt(565)=dydt(565)+k(5)*y(257)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(257)=dydt(257)-k(5)*y(257)*y(554);
dydt(554)=dydt(554)-k(5)*y(257)*y(554);
dydt(560)=dydt(560)+k(5)*y(257)*y(554);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(9)=dydt(9)-k(5)*y(9)*y(257);
dydt(257)=dydt(257)-k(5)*y(9)*y(257);
dydt(212)=dydt(212)+k(5)*y(9)*y(257);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(4)=dydt(4)-k(3)*y(4)*y(257);
dydt(257)=dydt(257)-k(3)*y(4)*y(257);
dydt(256)=dydt(256)+k(3)*y(4)*y(257);

% rule    : R1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(4)=dydt(4)-k(1)*y(4)*y(257);
dydt(257)=dydt(257)-k(1)*y(4)*y(257);
dydt(319)=dydt(319)+k(1)*y(4)*y(257);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(258)=dydt(258)-k(34)*y(258);
dydt(571)=dydt(571)+k(34)*y(258);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(258)=dydt(258)-k(33)*y(258);
dydt(265)=dydt(265)+k(33)*y(258);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(258)=dydt(258)-k(29)*y(258);
dydt(641)=dydt(641)+k(29)*y(258);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(258)=dydt(258)-k(25)*y(258);
dydt(643)=dydt(643)+k(25)*y(258);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

dydt(258)=dydt(258)-k(22)*y(258);
dydt(197)=dydt(197)+k(22)*y(258);
dydt(251)=dydt(251)+k(22)*y(258);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

dydt(258)=dydt(258)-k(22)*y(258);
dydt(2)=dydt(2)+k(22)*y(258);
dydt(622)=dydt(622)+k(22)*y(258);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(258)=dydt(258)-k(16)*y(258);
dydt(3)=dydt(3)+k(16)*y(258);
dydt(255)=dydt(255)+k(16)*y(258);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(258)=dydt(258)-k(16)*y(258);
dydt(3)=dydt(3)+k(16)*y(258);
dydt(546)=dydt(546)+k(16)*y(258);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(258)=dydt(258)-k(8)*y(258);
dydt(227)=dydt(227)+k(8)*y(258);
dydt(44)=dydt(44)+k(8)*y(258);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(258)=dydt(258)-k(6)*y(258);
dydt(43)=dydt(43)+k(6)*y(258);
dydt(543)=dydt(543)+k(6)*y(258);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(259)=dydt(259)-k(34)*y(259);
dydt(572)=dydt(572)+k(34)*y(259);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(259)=dydt(259)-k(33)*y(259);
dydt(266)=dydt(266)+k(33)*y(259);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(259)=dydt(259)-k(31)*y(259);
dydt(438)=dydt(438)+k(31)*y(259);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(259)=dydt(259)-k(31)*y(259);
dydt(545)=dydt(545)+k(31)*y(259);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(259)=dydt(259)-k(29)*y(259);
dydt(577)=dydt(577)+k(29)*y(259);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(259)=dydt(259)-k(22)*y(259);
dydt(197)=dydt(197)+k(22)*y(259);
dydt(252)=dydt(252)+k(22)*y(259);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(259)=dydt(259)-k(22)*y(259);
dydt(2)=dydt(2)+k(22)*y(259);
dydt(578)=dydt(578)+k(22)*y(259);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(259);
dydt(259)=dydt(259)-k(15)*y(3)*y(259);
dydt(255)=dydt(255)+k(15)*y(3)*y(259);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(259);
dydt(259)=dydt(259)-k(15)*y(3)*y(259);
dydt(546)=dydt(546)+k(15)*y(3)*y(259);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(259)=dydt(259)-k(8)*y(259);
dydt(257)=dydt(257)+k(8)*y(259);
dydt(437)=dydt(437)+k(8)*y(259);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(259)=dydt(259)-k(6)*y(259);
dydt(111)=dydt(111)+k(6)*y(259);
dydt(256)=dydt(256)+k(6)*y(259);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(260)=dydt(260)-k(34)*y(260);
dydt(634)=dydt(634)+k(34)*y(260);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(260)=dydt(260)-k(33)*y(260);
dydt(267)=dydt(267)+k(33)*y(260);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(260)=dydt(260)-k(32)*y(260);
dydt(203)=dydt(203)+k(32)*y(260);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(260)=dydt(260)-k(31)*y(260);
dydt(291)=dydt(291)+k(31)*y(260);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

dydt(260)=dydt(260)-k(22)*y(260);
dydt(197)=dydt(197)+k(22)*y(260);
dydt(238)=dydt(238)+k(22)*y(260);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(260);
dydt(260)=dydt(260)-k(21)*y(197)*y(260);
dydt(492)=dydt(492)+k(21)*y(197)*y(260);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(260);
dydt(260)=dydt(260)-k(21)*y(32)*y(260);
dydt(248)=dydt(248)+k(21)*y(32)*y(260);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(260);
dydt(260)=dydt(260)-k(21)*y(2)*y(260);
dydt(255)=dydt(255)+k(21)*y(2)*y(260);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(260)=dydt(260)-k(16)*y(260);
dydt(3)=dydt(3)+k(16)*y(260);
dydt(578)=dydt(578)+k(16)*y(260);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(260);
dydt(260)=dydt(260)-k(15)*y(3)*y(260);
dydt(622)=dydt(622)+k(15)*y(3)*y(260);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(260)=dydt(260)-k(8)*y(260);
dydt(55)=dydt(55)+k(8)*y(260);
dydt(257)=dydt(257)+k(8)*y(260);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(260)=dydt(260)-k(6)*y(260);
dydt(54)=dydt(54)+k(6)*y(260);
dydt(256)=dydt(256)+k(6)*y(260);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(261)=dydt(261)-k(34)*y(261);
dydt(286)=dydt(286)+k(34)*y(261);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(261)=dydt(261)-k(34)*y(261);
dydt(255)=dydt(255)+k(34)*y(261);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(261)=dydt(261)-k(33)*y(261);
dydt(268)=dydt(268)+k(33)*y(261);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(261)=dydt(261)-k(31)*y(261);
dydt(288)=dydt(288)+k(31)*y(261);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(261)=dydt(261)-k(22)*y(261);
dydt(197)=dydt(197)+k(22)*y(261);
dydt(645)=dydt(645)+k(22)*y(261);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(261)=dydt(261)-k(22)*y(261);
dydt(182)=dydt(182)+k(22)*y(261);
dydt(260)=dydt(260)+k(22)*y(261);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(261)=dydt(261)-k(16)*y(261);
dydt(3)=dydt(3)+k(16)*y(261);
dydt(577)=dydt(577)+k(16)*y(261);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(261);
dydt(261)=dydt(261)-k(15)*y(3)*y(261);
dydt(641)=dydt(641)+k(15)*y(3)*y(261);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(261)=dydt(261)-k(8)*y(261);
dydt(257)=dydt(257)+k(8)*y(261);
dydt(223)=dydt(223)+k(8)*y(261);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(261)=dydt(261)-k(6)*y(261);
dydt(256)=dydt(256)+k(6)*y(261);
dydt(222)=dydt(222)+k(6)*y(261);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(262)=dydt(262)-k(34)*y(262);
dydt(277)=dydt(277)+k(34)*y(262);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(262)=dydt(262)-k(33)*y(262);
dydt(269)=dydt(269)+k(33)*y(262);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(262)=dydt(262)-k(29)*y(262);
dydt(288)=dydt(288)+k(29)*y(262);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(262)=dydt(262)-k(22)*y(262);
dydt(197)=dydt(197)+k(22)*y(262);
dydt(49)=dydt(49)+k(22)*y(262);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(262)=dydt(262)-k(22)*y(262);
dydt(2)=dydt(2)+k(22)*y(262);
dydt(291)=dydt(291)+k(22)*y(262);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(262)=dydt(262)-k(18)*y(262);
dydt(255)=dydt(255)+k(18)*y(262);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(262)=dydt(262)-k(16)*y(262);
dydt(3)=dydt(3)+k(16)*y(262);
dydt(438)=dydt(438)+k(16)*y(262);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(262);
dydt(262)=dydt(262)-k(9)*y(3)*y(262);
dydt(207)=dydt(207)+k(9)*y(3)*y(262);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(262)=dydt(262)-k(8)*y(262);
dydt(192)=dydt(192)+k(8)*y(262);
dydt(44)=dydt(44)+k(8)*y(262);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(262)=dydt(262)-k(6)*y(262);
dydt(191)=dydt(191)+k(6)*y(262);
dydt(43)=dydt(43)+k(6)*y(262);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(264)=dydt(264)-k(34)*y(264);
dydt(110)=dydt(110)+k(34)*y(264);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(264)=dydt(264)-k(31)*y(264);
dydt(176)=dydt(176)+k(31)*y(264);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~pY) 

dydt(264)=dydt(264)-k(22)*y(264);
dydt(182)=dydt(182)+k(22)*y(264);
dydt(76)=dydt(76)+k(22)*y(264);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(264);
dydt(264)=dydt(264)-k(15)*y(3)*y(264);
dydt(515)=dydt(515)+k(15)*y(3)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(1)=dydt(1)-k(5)*y(1)*y(264);
dydt(264)=dydt(264)-k(5)*y(1)*y(264);
dydt(304)=dydt(304)+k(5)*y(1)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(7)=dydt(7)-k(5)*y(7)*y(264);
dydt(264)=dydt(264)-k(5)*y(7)*y(264);
dydt(594)=dydt(594)+k(5)*y(7)*y(264);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(264)=dydt(264)-k(5)*y(264)*y(335);
dydt(335)=dydt(335)-k(5)*y(264)*y(335);
dydt(363)=dydt(363)+k(5)*y(264)*y(335);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) 

dydt(264)=dydt(264)-k(4)*y(264);
dydt(4)=dydt(4)+k(4)*y(264);
dydt(218)=dydt(218)+k(4)*y(264);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(265)=dydt(265)-k(34)*y(265);
dydt(630)=dydt(630)+k(34)*y(265);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(265)=dydt(265)-k(29)*y(265);
dydt(640)=dydt(640)+k(29)*y(265);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(265)=dydt(265)-k(26)*y(265);
dydt(258)=dydt(258)+k(26)*y(265);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(265)=dydt(265)-k(25)*y(265);
dydt(494)=dydt(494)+k(25)*y(265);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

dydt(265)=dydt(265)-k(22)*y(265);
dydt(182)=dydt(182)+k(22)*y(265);
dydt(251)=dydt(251)+k(22)*y(265);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

dydt(265)=dydt(265)-k(22)*y(265);
dydt(2)=dydt(2)+k(22)*y(265);
dydt(624)=dydt(624)+k(22)*y(265);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(265)=dydt(265)-k(16)*y(265);
dydt(3)=dydt(3)+k(16)*y(265);
dydt(263)=dydt(263)+k(16)*y(265);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(265)=dydt(265)-k(16)*y(265);
dydt(3)=dydt(3)+k(16)*y(265);
dydt(625)=dydt(625)+k(16)*y(265);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(265)=dydt(265)-k(8)*y(265);
dydt(222)=dydt(222)+k(8)*y(265);
dydt(44)=dydt(44)+k(8)*y(265);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(265)=dydt(265)-k(6)*y(265);
dydt(43)=dydt(43)+k(6)*y(265);
dydt(515)=dydt(515)+k(6)*y(265);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(266)=dydt(266)-k(34)*y(266);
dydt(503)=dydt(503)+k(34)*y(266);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(266)=dydt(266)-k(31)*y(266);
dydt(271)=dydt(271)+k(31)*y(266);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(266)=dydt(266)-k(31)*y(266);
dydt(510)=dydt(510)+k(31)*y(266);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(266)=dydt(266)-k(29)*y(266);
dydt(579)=dydt(579)+k(29)*y(266);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(266)=dydt(266)-k(22)*y(266);
dydt(182)=dydt(182)+k(22)*y(266);
dydt(252)=dydt(252)+k(22)*y(266);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(266)=dydt(266)-k(22)*y(266);
dydt(2)=dydt(2)+k(22)*y(266);
dydt(497)=dydt(497)+k(22)*y(266);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(266);
dydt(266)=dydt(266)-k(15)*y(3)*y(266);
dydt(263)=dydt(263)+k(15)*y(3)*y(266);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(266);
dydt(266)=dydt(266)-k(15)*y(3)*y(266);
dydt(625)=dydt(625)+k(15)*y(3)*y(266);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(266)=dydt(266)-k(8)*y(266);
dydt(218)=dydt(218)+k(8)*y(266);
dydt(437)=dydt(437)+k(8)*y(266);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(266)=dydt(266)-k(6)*y(266);
dydt(111)=dydt(111)+k(6)*y(266);
dydt(264)=dydt(264)+k(6)*y(266);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(267)=dydt(267)-k(34)*y(267);
dydt(629)=dydt(629)+k(34)*y(267);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(267)=dydt(267)-k(32)*y(267);
dydt(300)=dydt(300)+k(32)*y(267);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(267)=dydt(267)-k(31)*y(267);
dydt(272)=dydt(272)+k(31)*y(267);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(267)=dydt(267)-k(26)*y(267);
dydt(260)=dydt(260)+k(26)*y(267);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

dydt(267)=dydt(267)-k(22)*y(267);
dydt(182)=dydt(182)+k(22)*y(267);
dydt(238)=dydt(238)+k(22)*y(267);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(267);
dydt(267)=dydt(267)-k(21)*y(197)*y(267);
dydt(488)=dydt(488)+k(21)*y(197)*y(267);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(267);
dydt(267)=dydt(267)-k(21)*y(32)*y(267);
dydt(493)=dydt(493)+k(21)*y(32)*y(267);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(267);
dydt(267)=dydt(267)-k(21)*y(2)*y(267);
dydt(263)=dydt(263)+k(21)*y(2)*y(267);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(267)=dydt(267)-k(16)*y(267);
dydt(3)=dydt(3)+k(16)*y(267);
dydt(497)=dydt(497)+k(16)*y(267);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(267);
dydt(267)=dydt(267)-k(15)*y(3)*y(267);
dydt(624)=dydt(624)+k(15)*y(3)*y(267);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(267)=dydt(267)-k(8)*y(267);
dydt(55)=dydt(55)+k(8)*y(267);
dydt(218)=dydt(218)+k(8)*y(267);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(267)=dydt(267)-k(6)*y(267);
dydt(54)=dydt(54)+k(6)*y(267);
dydt(264)=dydt(264)+k(6)*y(267);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(268)=dydt(268)-k(34)*y(268);
dydt(281)=dydt(281)+k(34)*y(268);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(268)=dydt(268)-k(34)*y(268);
dydt(263)=dydt(263)+k(34)*y(268);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(268)=dydt(268)-k(31)*y(268);
dydt(273)=dydt(273)+k(31)*y(268);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(268)=dydt(268)-k(26)*y(268);
dydt(261)=dydt(261)+k(26)*y(268);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(268)=dydt(268)-k(22)*y(268);
dydt(182)=dydt(182)+k(22)*y(268);
dydt(645)=dydt(645)+k(22)*y(268);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(268)=dydt(268)-k(22)*y(268);
dydt(182)=dydt(182)+k(22)*y(268);
dydt(267)=dydt(267)+k(22)*y(268);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(268)=dydt(268)-k(16)*y(268);
dydt(3)=dydt(3)+k(16)*y(268);
dydt(579)=dydt(579)+k(16)*y(268);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(268);
dydt(268)=dydt(268)-k(15)*y(3)*y(268);
dydt(640)=dydt(640)+k(15)*y(3)*y(268);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(268)=dydt(268)-k(8)*y(268);
dydt(218)=dydt(218)+k(8)*y(268);
dydt(223)=dydt(223)+k(8)*y(268);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(268)=dydt(268)-k(6)*y(268);
dydt(264)=dydt(264)+k(6)*y(268);
dydt(222)=dydt(222)+k(6)*y(268);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(270)=dydt(270)-k(34)*y(270);
dydt(51)=dydt(51)+k(34)*y(270);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(270)=dydt(270)-k(29)*y(270);
dydt(649)=dydt(649)+k(29)*y(270);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(270)=dydt(270)-k(26)*y(270);
dydt(207)=dydt(207)+k(26)*y(270);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(270)=dydt(270)-k(23)*y(270);
dydt(478)=dydt(478)+k(23)*y(270);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(270)=dydt(270)-k(22)*y(270);
dydt(182)=dydt(182)+k(22)*y(270);
dydt(46)=dydt(46)+k(22)*y(270);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

dydt(270)=dydt(270)-k(22)*y(270);
dydt(2)=dydt(2)+k(22)*y(270);
dydt(422)=dydt(422)+k(22)*y(270);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(270)=dydt(270)-k(16)*y(270);
dydt(3)=dydt(3)+k(16)*y(270);
dydt(392)=dydt(392)+k(16)*y(270);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(270)=dydt(270)-k(10)*y(270);
dydt(3)=dydt(3)+k(10)*y(270);
dydt(269)=dydt(269)+k(10)*y(270);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(270)=dydt(270)-k(8)*y(270);
dydt(184)=dydt(184)+k(8)*y(270);
dydt(44)=dydt(44)+k(8)*y(270);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(270)=dydt(270)-k(6)*y(270);
dydt(43)=dydt(43)+k(6)*y(270);
dydt(391)=dydt(391)+k(6)*y(270);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(271)=dydt(271)-k(34)*y(271);
dydt(275)=dydt(275)+k(34)*y(271);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(271)=dydt(271)-k(31)*y(271);
dydt(179)=dydt(179)+k(31)*y(271);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(271)=dydt(271)-k(29)*y(271);
dydt(580)=dydt(580)+k(29)*y(271);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(271)=dydt(271)-k(22)*y(271);
dydt(182)=dydt(182)+k(22)*y(271);
dydt(616)=dydt(616)+k(22)*y(271);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(271)=dydt(271)-k(22)*y(271);
dydt(2)=dydt(2)+k(22)*y(271);
dydt(421)=dydt(421)+k(22)*y(271);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(271);
dydt(271)=dydt(271)-k(15)*y(3)*y(271);
dydt(269)=dydt(269)+k(15)*y(3)*y(271);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(271);
dydt(271)=dydt(271)-k(9)*y(3)*y(271);
dydt(392)=dydt(392)+k(9)*y(3)*y(271);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(271)=dydt(271)-k(8)*y(271);
dydt(177)=dydt(177)+k(8)*y(271);
dydt(437)=dydt(437)+k(8)*y(271);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(271)=dydt(271)-k(6)*y(271);
dydt(111)=dydt(111)+k(6)*y(271);
dydt(176)=dydt(176)+k(6)*y(271);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(272)=dydt(272)-k(34)*y(272);
dydt(276)=dydt(276)+k(34)*y(272);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(272)=dydt(272)-k(32)*y(272);
dydt(297)=dydt(297)+k(32)*y(272);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(272)=dydt(272)-k(26)*y(272);
dydt(291)=dydt(291)+k(26)*y(272);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

dydt(272)=dydt(272)-k(22)*y(272);
dydt(182)=dydt(182)+k(22)*y(272);
dydt(56)=dydt(56)+k(22)*y(272);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(272);
dydt(272)=dydt(272)-k(21)*y(197)*y(272);
dydt(486)=dydt(486)+k(21)*y(197)*y(272);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(272);
dydt(272)=dydt(272)-k(21)*y(32)*y(272);
dydt(479)=dydt(479)+k(21)*y(32)*y(272);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(272);
dydt(272)=dydt(272)-k(21)*y(2)*y(272);
dydt(269)=dydt(269)+k(21)*y(2)*y(272);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(272)=dydt(272)-k(18)*y(272);
dydt(267)=dydt(267)+k(18)*y(272);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(272)=dydt(272)-k(16)*y(272);
dydt(3)=dydt(3)+k(16)*y(272);
dydt(421)=dydt(421)+k(16)*y(272);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(272);
dydt(272)=dydt(272)-k(9)*y(3)*y(272);
dydt(422)=dydt(422)+k(9)*y(3)*y(272);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(272)=dydt(272)-k(8)*y(272);
dydt(55)=dydt(55)+k(8)*y(272);
dydt(177)=dydt(177)+k(8)*y(272);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(272)=dydt(272)-k(6)*y(272);
dydt(54)=dydt(54)+k(6)*y(272);
dydt(176)=dydt(176)+k(6)*y(272);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(273)=dydt(273)-k(34)*y(273);
dydt(278)=dydt(278)+k(34)*y(273);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(273)=dydt(273)-k(34)*y(273);
dydt(269)=dydt(269)+k(34)*y(273);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(273)=dydt(273)-k(26)*y(273);
dydt(288)=dydt(288)+k(26)*y(273);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(273)=dydt(273)-k(22)*y(273);
dydt(182)=dydt(182)+k(22)*y(273);
dydt(282)=dydt(282)+k(22)*y(273);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(273)=dydt(273)-k(22)*y(273);
dydt(182)=dydt(182)+k(22)*y(273);
dydt(272)=dydt(272)+k(22)*y(273);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(273)=dydt(273)-k(18)*y(273);
dydt(268)=dydt(268)+k(18)*y(273);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(273)=dydt(273)-k(16)*y(273);
dydt(3)=dydt(3)+k(16)*y(273);
dydt(580)=dydt(580)+k(16)*y(273);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(273);
dydt(273)=dydt(273)-k(9)*y(3)*y(273);
dydt(649)=dydt(649)+k(9)*y(3)*y(273);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(273)=dydt(273)-k(8)*y(273);
dydt(177)=dydt(177)+k(8)*y(273);
dydt(223)=dydt(223)+k(8)*y(273);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(273)=dydt(273)-k(6)*y(273);
dydt(176)=dydt(176)+k(6)*y(273);
dydt(222)=dydt(222)+k(6)*y(273);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(275)=dydt(275)-k(31)*y(275);
dydt(172)=dydt(172)+k(31)*y(275);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(275)=dydt(275)-k(28)*y(275);
dydt(271)=dydt(271)+k(28)*y(275);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(275)=dydt(275)-k(27)*y(275);
dydt(280)=dydt(280)+k(27)*y(275);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(275)=dydt(275)-k(22)*y(275);
dydt(2)=dydt(2)+k(22)*y(275);
dydt(616)=dydt(616)+k(22)*y(275);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(275)=dydt(275)-k(22)*y(275);
dydt(2)=dydt(2)+k(22)*y(275);
dydt(425)=dydt(425)+k(22)*y(275);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(275);
dydt(275)=dydt(275)-k(15)*y(3)*y(275);
dydt(274)=dydt(274)+k(15)*y(3)*y(275);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(275);
dydt(275)=dydt(275)-k(9)*y(3)*y(275);
dydt(387)=dydt(387)+k(9)*y(3)*y(275);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(275)=dydt(275)-k(8)*y(275);
dydt(21)=dydt(21)+k(8)*y(275);
dydt(437)=dydt(437)+k(8)*y(275);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(275)=dydt(275)-k(6)*y(275);
dydt(111)=dydt(111)+k(6)*y(275);
dydt(89)=dydt(89)+k(6)*y(275);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(276)=dydt(276)-k(32)*y(276);
dydt(88)=dydt(88)+k(32)*y(276);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(276)=dydt(276)-k(26)*y(276);
dydt(287)=dydt(287)+k(26)*y(276);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

dydt(276)=dydt(276)-k(22)*y(276);
dydt(2)=dydt(2)+k(22)*y(276);
dydt(56)=dydt(56)+k(22)*y(276);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(276);
dydt(276)=dydt(276)-k(21)*y(197)*y(276);
dydt(484)=dydt(484)+k(21)*y(197)*y(276);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(276);
dydt(276)=dydt(276)-k(21)*y(32)*y(276);
dydt(481)=dydt(481)+k(21)*y(32)*y(276);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(276);
dydt(276)=dydt(276)-k(21)*y(2)*y(276);
dydt(274)=dydt(274)+k(21)*y(2)*y(276);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(276)=dydt(276)-k(18)*y(276);
dydt(629)=dydt(629)+k(18)*y(276);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(276)=dydt(276)-k(16)*y(276);
dydt(3)=dydt(3)+k(16)*y(276);
dydt(425)=dydt(425)+k(16)*y(276);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(276);
dydt(276)=dydt(276)-k(9)*y(3)*y(276);
dydt(58)=dydt(58)+k(9)*y(3)*y(276);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(276)=dydt(276)-k(8)*y(276);
dydt(55)=dydt(55)+k(8)*y(276);
dydt(21)=dydt(21)+k(8)*y(276);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(276)=dydt(276)-k(6)*y(276);
dydt(54)=dydt(54)+k(6)*y(276);
dydt(89)=dydt(89)+k(6)*y(276);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(277)=dydt(277)-k(33)*y(277);
dydt(274)=dydt(274)+k(33)*y(277);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(277)=dydt(277)-k(28)*y(277);
dydt(262)=dydt(262)+k(28)*y(277);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(277)=dydt(277)-k(27)*y(277);
dydt(283)=dydt(283)+k(27)*y(277);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(277)=dydt(277)-k(22)*y(277);
dydt(32)=dydt(32)+k(22)*y(277);
dydt(49)=dydt(49)+k(22)*y(277);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(277)=dydt(277)-k(22)*y(277);
dydt(2)=dydt(2)+k(22)*y(277);
dydt(287)=dydt(287)+k(22)*y(277);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(277)=dydt(277)-k(18)*y(277);
dydt(254)=dydt(254)+k(18)*y(277);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(277)=dydt(277)-k(16)*y(277);
dydt(3)=dydt(3)+k(16)*y(277);
dydt(442)=dydt(442)+k(16)*y(277);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(277);
dydt(277)=dydt(277)-k(9)*y(3)*y(277);
dydt(52)=dydt(52)+k(9)*y(3)*y(277);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(277)=dydt(277)-k(8)*y(277);
dydt(27)=dydt(27)+k(8)*y(277);
dydt(44)=dydt(44)+k(8)*y(277);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(277)=dydt(277)-k(6)*y(277);
dydt(86)=dydt(86)+k(6)*y(277);
dydt(43)=dydt(43)+k(6)*y(277);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(279)=dydt(279)-k(34)*y(279);
dydt(51)=dydt(51)+k(34)*y(279);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(279)=dydt(279)-k(30)*y(279);
dydt(649)=dydt(649)+k(30)*y(279);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(279)=dydt(279)-k(26)*y(279);
dydt(284)=dydt(284)+k(26)*y(279);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(279)=dydt(279)-k(23)*y(279);
dydt(428)=dydt(428)+k(23)*y(279);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(279)=dydt(279)-k(22)*y(279);
dydt(2)=dydt(2)+k(22)*y(279);
dydt(225)=dydt(225)+k(22)*y(279);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(279)=dydt(279)-k(22)*y(279);
dydt(182)=dydt(182)+k(22)*y(279);
dydt(58)=dydt(58)+k(22)*y(279);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(279)=dydt(279)-k(16)*y(279);
dydt(3)=dydt(3)+k(16)*y(279);
dydt(429)=dydt(429)+k(16)*y(279);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(279)=dydt(279)-k(10)*y(279);
dydt(3)=dydt(3)+k(10)*y(279);
dydt(278)=dydt(278)+k(10)*y(279);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(279)=dydt(279)-k(8)*y(279);
dydt(140)=dydt(140)+k(8)*y(279);
dydt(223)=dydt(223)+k(8)*y(279);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(279)=dydt(279)-k(6)*y(279);
dydt(222)=dydt(222)+k(6)*y(279);
dydt(380)=dydt(380)+k(6)*y(279);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(280)=dydt(280)-k(34)*y(280);
dydt(275)=dydt(275)+k(34)*y(280);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(280)=dydt(280)-k(31)*y(280);
dydt(377)=dydt(377)+k(31)*y(280);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(280)=dydt(280)-k(30)*y(280);
dydt(580)=dydt(580)+k(30)*y(280);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(280)=dydt(280)-k(22)*y(280);
dydt(2)=dydt(2)+k(22)*y(280);
dydt(431)=dydt(431)+k(22)*y(280);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(280)=dydt(280)-k(22)*y(280);
dydt(182)=dydt(182)+k(22)*y(280);
dydt(425)=dydt(425)+k(22)*y(280);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(280);
dydt(280)=dydt(280)-k(15)*y(3)*y(280);
dydt(278)=dydt(278)+k(15)*y(3)*y(280);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(280);
dydt(280)=dydt(280)-k(9)*y(3)*y(280);
dydt(429)=dydt(429)+k(9)*y(3)*y(280);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(280)=dydt(280)-k(8)*y(280);
dydt(21)=dydt(21)+k(8)*y(280);
dydt(219)=dydt(219)+k(8)*y(280);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(280)=dydt(280)-k(6)*y(280);
dydt(218)=dydt(218)+k(6)*y(280);
dydt(89)=dydt(89)+k(6)*y(280);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(281)=dydt(281)-k(34)*y(281);
dydt(253)=dydt(253)+k(34)*y(281);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(281)=dydt(281)-k(31)*y(281);
dydt(278)=dydt(278)+k(31)*y(281);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(281)=dydt(281)-k(30)*y(281);
dydt(268)=dydt(268)+k(30)*y(281);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(281)=dydt(281)-k(26)*y(281);
dydt(286)=dydt(286)+k(26)*y(281);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(281)=dydt(281)-k(22)*y(281);
dydt(2)=dydt(2)+k(22)*y(281);
dydt(645)=dydt(645)+k(22)*y(281);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(281)=dydt(281)-k(22)*y(281);
dydt(182)=dydt(182)+k(22)*y(281);
dydt(629)=dydt(629)+k(22)*y(281);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(281)=dydt(281)-k(16)*y(281);
dydt(3)=dydt(3)+k(16)*y(281);
dydt(505)=dydt(505)+k(16)*y(281);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(281);
dydt(281)=dydt(281)-k(15)*y(3)*y(281);
dydt(637)=dydt(637)+k(15)*y(3)*y(281);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(281)=dydt(281)-k(8)*y(281);
dydt(111)=dydt(111)+k(8)*y(281);
dydt(223)=dydt(223)+k(8)*y(281);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(281)=dydt(281)-k(6)*y(281);
dydt(110)=dydt(110)+k(6)*y(281);
dydt(222)=dydt(222)+k(6)*y(281);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(282)=dydt(282)-k(34)*y(282);
dydt(49)=dydt(49)+k(34)*y(282);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(282)=dydt(282)-k(32)*y(282);
dydt(224)=dydt(224)+k(32)*y(282);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

dydt(282)=dydt(282)-k(22)*y(282);
dydt(182)=dydt(182)+k(22)*y(282);
dydt(56)=dydt(56)+k(22)*y(282);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(282);
dydt(282)=dydt(282)-k(21)*y(197)*y(282);
dydt(288)=dydt(288)+k(21)*y(197)*y(282);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(282);
dydt(282)=dydt(282)-k(21)*y(32)*y(282);
dydt(283)=dydt(283)+k(21)*y(32)*y(282);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(282);
dydt(282)=dydt(282)-k(21)*y(2)*y(282);
dydt(278)=dydt(278)+k(21)*y(2)*y(282);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(282)=dydt(282)-k(18)*y(282);
dydt(645)=dydt(645)+k(18)*y(282);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(282)=dydt(282)-k(16)*y(282);
dydt(3)=dydt(3)+k(16)*y(282);
dydt(431)=dydt(431)+k(16)*y(282);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(282);
dydt(282)=dydt(282)-k(9)*y(3)*y(282);
dydt(225)=dydt(225)+k(9)*y(3)*y(282);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

dydt(282)=dydt(282)-k(8)*y(282);
dydt(15)=dydt(15)+k(8)*y(282);
dydt(223)=dydt(223)+k(8)*y(282);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(282)=dydt(282)-k(6)*y(282);
dydt(98)=dydt(98)+k(6)*y(282);
dydt(222)=dydt(222)+k(6)*y(282);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(284)=dydt(284)-k(34)*y(284);
dydt(52)=dydt(52)+k(34)*y(284);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(284)=dydt(284)-k(33)*y(284);
dydt(279)=dydt(279)+k(33)*y(284);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(284)=dydt(284)-k(30)*y(284);
dydt(289)=dydt(289)+k(30)*y(284);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(284)=dydt(284)-k(23)*y(284);
dydt(473)=dydt(473)+k(23)*y(284);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(284)=dydt(284)-k(22)*y(284);
dydt(32)=dydt(32)+k(22)*y(284);
dydt(225)=dydt(225)+k(22)*y(284);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(284)=dydt(284)-k(22)*y(284);
dydt(182)=dydt(182)+k(22)*y(284);
dydt(59)=dydt(59)+k(22)*y(284);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(284)=dydt(284)-k(16)*y(284);
dydt(3)=dydt(3)+k(16)*y(284);
dydt(432)=dydt(432)+k(16)*y(284);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(284)=dydt(284)-k(10)*y(284);
dydt(3)=dydt(3)+k(10)*y(284);
dydt(283)=dydt(283)+k(10)*y(284);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(284)=dydt(284)-k(8)*y(284);
dydt(131)=dydt(131)+k(8)*y(284);
dydt(223)=dydt(223)+k(8)*y(284);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(284)=dydt(284)-k(6)*y(284);
dydt(222)=dydt(222)+k(6)*y(284);
dydt(441)=dydt(441)+k(6)*y(284);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(285)=dydt(285)-k(34)*y(285);
dydt(442)=dydt(442)+k(34)*y(285);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(285)=dydt(285)-k(33)*y(285);
dydt(280)=dydt(280)+k(33)*y(285);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(285)=dydt(285)-k(31)*y(285);
dydt(449)=dydt(449)+k(31)*y(285);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(285)=dydt(285)-k(30)*y(285);
dydt(290)=dydt(290)+k(30)*y(285);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(285)=dydt(285)-k(22)*y(285);
dydt(32)=dydt(32)+k(22)*y(285);
dydt(431)=dydt(431)+k(22)*y(285);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(285)=dydt(285)-k(22)*y(285);
dydt(182)=dydt(182)+k(22)*y(285);
dydt(615)=dydt(615)+k(22)*y(285);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(285);
dydt(285)=dydt(285)-k(15)*y(3)*y(285);
dydt(283)=dydt(283)+k(15)*y(3)*y(285);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(285);
dydt(285)=dydt(285)-k(9)*y(3)*y(285);
dydt(432)=dydt(432)+k(9)*y(3)*y(285);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(285)=dydt(285)-k(8)*y(285);
dydt(27)=dydt(27)+k(8)*y(285);
dydt(219)=dydt(219)+k(8)*y(285);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(285)=dydt(285)-k(6)*y(285);
dydt(218)=dydt(218)+k(6)*y(285);
dydt(86)=dydt(86)+k(6)*y(285);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(286)=dydt(286)-k(34)*y(286);
dydt(254)=dydt(254)+k(34)*y(286);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(286)=dydt(286)-k(33)*y(286);
dydt(281)=dydt(281)+k(33)*y(286);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(286)=dydt(286)-k(31)*y(286);
dydt(283)=dydt(283)+k(31)*y(286);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(286)=dydt(286)-k(30)*y(286);
dydt(261)=dydt(261)+k(30)*y(286);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(286)=dydt(286)-k(22)*y(286);
dydt(32)=dydt(32)+k(22)*y(286);
dydt(645)=dydt(645)+k(22)*y(286);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(286)=dydt(286)-k(22)*y(286);
dydt(182)=dydt(182)+k(22)*y(286);
dydt(634)=dydt(634)+k(22)*y(286);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(286)=dydt(286)-k(16)*y(286);
dydt(3)=dydt(3)+k(16)*y(286);
dydt(574)=dydt(574)+k(16)*y(286);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(286);
dydt(286)=dydt(286)-k(15)*y(3)*y(286);
dydt(639)=dydt(639)+k(15)*y(3)*y(286);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(286)=dydt(286)-k(8)*y(286);
dydt(33)=dydt(33)+k(8)*y(286);
dydt(223)=dydt(223)+k(8)*y(286);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(286)=dydt(286)-k(6)*y(286);
dydt(82)=dydt(82)+k(6)*y(286);
dydt(222)=dydt(222)+k(6)*y(286);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(287)=dydt(287)-k(33)*y(287);
dydt(276)=dydt(276)+k(33)*y(287);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(287)=dydt(287)-k(32)*y(287);
dydt(85)=dydt(85)+k(32)*y(287);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

dydt(287)=dydt(287)-k(22)*y(287);
dydt(32)=dydt(32)+k(22)*y(287);
dydt(56)=dydt(56)+k(22)*y(287);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(287);
dydt(287)=dydt(287)-k(21)*y(197)*y(287);
dydt(474)=dydt(474)+k(21)*y(197)*y(287);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(287);
dydt(287)=dydt(287)-k(21)*y(32)*y(287);
dydt(483)=dydt(483)+k(21)*y(32)*y(287);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(287);
dydt(287)=dydt(287)-k(21)*y(2)*y(287);
dydt(277)=dydt(277)+k(21)*y(2)*y(287);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(287)=dydt(287)-k(18)*y(287);
dydt(634)=dydt(634)+k(18)*y(287);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(287)=dydt(287)-k(16)*y(287);
dydt(3)=dydt(3)+k(16)*y(287);
dydt(615)=dydt(615)+k(16)*y(287);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(287);
dydt(287)=dydt(287)-k(9)*y(3)*y(287);
dydt(59)=dydt(59)+k(9)*y(3)*y(287);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(287)=dydt(287)-k(8)*y(287);
dydt(55)=dydt(55)+k(8)*y(287);
dydt(27)=dydt(27)+k(8)*y(287);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(287)=dydt(287)-k(6)*y(287);
dydt(54)=dydt(54)+k(6)*y(287);
dydt(86)=dydt(86)+k(6)*y(287);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(289)=dydt(289)-k(34)*y(289);
dydt(284)=dydt(284)+k(34)*y(289);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(289)=dydt(289)-k(34)*y(289);
dydt(207)=dydt(207)+k(34)*y(289);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(289)=dydt(289)-k(33)*y(289);
dydt(649)=dydt(649)+k(33)*y(289);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(289)=dydt(289)-k(23)*y(289);
dydt(475)=dydt(475)+k(23)*y(289);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(289)=dydt(289)-k(22)*y(289);
dydt(197)=dydt(197)+k(22)*y(289);
dydt(225)=dydt(225)+k(22)*y(289);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(289)=dydt(289)-k(22)*y(289);
dydt(182)=dydt(182)+k(22)*y(289);
dydt(206)=dydt(206)+k(22)*y(289);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(289)=dydt(289)-k(16)*y(289);
dydt(3)=dydt(3)+k(16)*y(289);
dydt(433)=dydt(433)+k(16)*y(289);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(289)=dydt(289)-k(10)*y(289);
dydt(3)=dydt(3)+k(10)*y(289);
dydt(288)=dydt(288)+k(10)*y(289);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(289)=dydt(289)-k(8)*y(289);
dydt(335)=dydt(335)+k(8)*y(289);
dydt(223)=dydt(223)+k(8)*y(289);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(289)=dydt(289)-k(6)*y(289);
dydt(222)=dydt(222)+k(6)*y(289);
dydt(407)=dydt(407)+k(6)*y(289);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(290)=dydt(290)-k(34)*y(290);
dydt(285)=dydt(285)+k(34)*y(290);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(290)=dydt(290)-k(34)*y(290);
dydt(438)=dydt(438)+k(34)*y(290);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(290)=dydt(290)-k(33)*y(290);
dydt(580)=dydt(580)+k(33)*y(290);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(290)=dydt(290)-k(31)*y(290);
dydt(194)=dydt(194)+k(31)*y(290);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(290)=dydt(290)-k(22)*y(290);
dydt(197)=dydt(197)+k(22)*y(290);
dydt(431)=dydt(431)+k(22)*y(290);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(290)=dydt(290)-k(22)*y(290);
dydt(182)=dydt(182)+k(22)*y(290);
dydt(292)=dydt(292)+k(22)*y(290);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(290);
dydt(290)=dydt(290)-k(15)*y(3)*y(290);
dydt(288)=dydt(288)+k(15)*y(3)*y(290);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(290);
dydt(290)=dydt(290)-k(9)*y(3)*y(290);
dydt(433)=dydt(433)+k(9)*y(3)*y(290);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(290)=dydt(290)-k(8)*y(290);
dydt(192)=dydt(192)+k(8)*y(290);
dydt(219)=dydt(219)+k(8)*y(290);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(290)=dydt(290)-k(6)*y(290);
dydt(218)=dydt(218)+k(6)*y(290);
dydt(191)=dydt(191)+k(6)*y(290);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(292)=dydt(292)-k(34)*y(292);
dydt(615)=dydt(615)+k(34)*y(292);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(292)=dydt(292)-k(33)*y(292);
dydt(421)=dydt(421)+k(33)*y(292);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(292)=dydt(292)-k(32)*y(292);
dydt(296)=dydt(296)+k(32)*y(292);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(292)=dydt(292)-k(31)*y(292);
dydt(409)=dydt(409)+k(31)*y(292);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(292)=dydt(292)-k(22)*y(292);
dydt(197)=dydt(197)+k(22)*y(292);
dydt(614)=dydt(614)+k(22)*y(292);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(292);
dydt(292)=dydt(292)-k(21)*y(197)*y(292);
dydt(619)=dydt(619)+k(21)*y(197)*y(292);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(292);
dydt(292)=dydt(292)-k(21)*y(32)*y(292);
dydt(620)=dydt(620)+k(21)*y(32)*y(292);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(292);
dydt(292)=dydt(292)-k(21)*y(2)*y(292);
dydt(438)=dydt(438)+k(21)*y(2)*y(292);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(292);
dydt(292)=dydt(292)-k(15)*y(3)*y(292);
dydt(291)=dydt(291)+k(15)*y(3)*y(292);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(292);
dydt(292)=dydt(292)-k(9)*y(3)*y(292);
dydt(410)=dydt(410)+k(9)*y(3)*y(292);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(292)=dydt(292)-k(8)*y(292);
dydt(315)=dydt(315)+k(8)*y(292);
dydt(192)=dydt(192)+k(8)*y(292);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

dydt(292)=dydt(292)-k(6)*y(292);
dydt(77)=dydt(77)+k(6)*y(292);
dydt(191)=dydt(191)+k(6)*y(292);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(293)=dydt(293)-k(34)*y(293);
dydt(483)=dydt(483)+k(34)*y(293);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(293)=dydt(293)-k(33)*y(293);
dydt(262)=dydt(262)+k(33)*y(293);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(293)=dydt(293)-k(33)*y(293);
dydt(479)=dydt(479)+k(33)*y(293);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(293)=dydt(293)-k(29)*y(293);
dydt(294)=dydt(294)+k(29)*y(293);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(293)=dydt(293)-k(22)*y(293);
dydt(32)=dydt(32)+k(22)*y(293);
dydt(291)=dydt(291)+k(22)*y(293);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(293)=dydt(293)-k(22)*y(293);
dydt(197)=dydt(197)+k(22)*y(293);
dydt(249)=dydt(249)+k(22)*y(293);

% rule    : R7_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(293)=dydt(293)-k(18)*y(293);
dydt(248)=dydt(248)+k(18)*y(293);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(293)=dydt(293)-k(16)*y(293);
dydt(3)=dydt(3)+k(16)*y(293);
dydt(620)=dydt(620)+k(16)*y(293);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(293);
dydt(293)=dydt(293)-k(9)*y(3)*y(293);
dydt(477)=dydt(477)+k(9)*y(3)*y(293);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(293)=dydt(293)-k(8)*y(293);
dydt(192)=dydt(192)+k(8)*y(293);
dydt(38)=dydt(38)+k(8)*y(293);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(293)=dydt(293)-k(6)*y(293);
dydt(191)=dydt(191)+k(6)*y(293);
dydt(37)=dydt(37)+k(6)*y(293);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(294)=dydt(294)-k(34)*y(294);
dydt(293)=dydt(293)+k(34)*y(294);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(294)=dydt(294)-k(34)*y(294);
dydt(474)=dydt(474)+k(34)*y(294);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(294)=dydt(294)-k(33)*y(294);
dydt(288)=dydt(288)+k(33)*y(294);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(294)=dydt(294)-k(33)*y(294);
dydt(486)=dydt(486)+k(33)*y(294);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(294)=dydt(294)-k(22)*y(294);
dydt(197)=dydt(197)+k(22)*y(294);
dydt(291)=dydt(291)+k(22)*y(294);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(294)=dydt(294)-k(22)*y(294);
dydt(197)=dydt(197)+k(22)*y(294);
dydt(232)=dydt(232)+k(22)*y(294);

% rule    : R7_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(294)=dydt(294)-k(18)*y(294);
dydt(492)=dydt(492)+k(18)*y(294);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(294)=dydt(294)-k(16)*y(294);
dydt(3)=dydt(3)+k(16)*y(294);
dydt(619)=dydt(619)+k(16)*y(294);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(294);
dydt(294)=dydt(294)-k(9)*y(3)*y(294);
dydt(475)=dydt(475)+k(9)*y(3)*y(294);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(294)=dydt(294)-k(8)*y(294);
dydt(192)=dydt(192)+k(8)*y(294);
dydt(228)=dydt(228)+k(8)*y(294);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(294)=dydt(294)-k(6)*y(294);
dydt(191)=dydt(191)+k(6)*y(294);
dydt(227)=dydt(227)+k(6)*y(294);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(296)=dydt(296)-k(34)*y(296);
dydt(87)=dydt(87)+k(34)*y(296);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(296)=dydt(296)-k(33)*y(296);
dydt(299)=dydt(299)+k(33)*y(296);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(296)=dydt(296)-k(31)*y(296);
dydt(202)=dydt(202)+k(31)*y(296);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(296)=dydt(296)-k(22)*y(296);
dydt(197)=dydt(197)+k(22)*y(296);
dydt(93)=dydt(93)+k(22)*y(296);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(296);
dydt(296)=dydt(296)-k(15)*y(3)*y(296);
dydt(295)=dydt(295)+k(15)*y(3)*y(296);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(296);
dydt(296)=dydt(296)-k(9)*y(3)*y(296);
dydt(415)=dydt(415)+k(9)*y(3)*y(296);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(296)=dydt(296)-k(8)*y(296);
dydt(91)=dydt(91)+k(8)*y(296);
dydt(192)=dydt(192)+k(8)*y(296);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

dydt(296)=dydt(296)-k(6)*y(296);
dydt(72)=dydt(72)+k(6)*y(296);
dydt(191)=dydt(191)+k(6)*y(296);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(298)=dydt(298)-k(34)*y(298);
dydt(65)=dydt(65)+k(34)*y(298);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(298)=dydt(298)-k(26)*y(298);
dydt(204)=dydt(204)+k(26)*y(298);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(298)=dydt(298)-k(22)*y(298);
dydt(182)=dydt(182)+k(22)*y(298);
dydt(60)=dydt(60)+k(22)*y(298);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(298)=dydt(298)-k(16)*y(298);
dydt(3)=dydt(3)+k(16)*y(298);
dydt(419)=dydt(419)+k(16)*y(298);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(298)=dydt(298)-k(13)*y(298);
dydt(422)=dydt(422)+k(13)*y(298);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(298)=dydt(298)-k(10)*y(298);
dydt(3)=dydt(3)+k(10)*y(298);
dydt(297)=dydt(297)+k(10)*y(298);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(298)=dydt(298)-k(8)*y(298);
dydt(62)=dydt(62)+k(8)*y(298);
dydt(184)=dydt(184)+k(8)*y(298);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(298)=dydt(298)-k(6)*y(298);
dydt(61)=dydt(61)+k(6)*y(298);
dydt(391)=dydt(391)+k(6)*y(298);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(299)=dydt(299)-k(34)*y(299);
dydt(90)=dydt(90)+k(34)*y(299);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(299)=dydt(299)-k(31)*y(299);
dydt(418)=dydt(418)+k(31)*y(299);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(299)=dydt(299)-k(22)*y(299);
dydt(182)=dydt(182)+k(22)*y(299);
dydt(93)=dydt(93)+k(22)*y(299);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(299);
dydt(299)=dydt(299)-k(15)*y(3)*y(299);
dydt(297)=dydt(297)+k(15)*y(3)*y(299);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(299);
dydt(299)=dydt(299)-k(9)*y(3)*y(299);
dydt(419)=dydt(419)+k(9)*y(3)*y(299);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(299)=dydt(299)-k(8)*y(299);
dydt(91)=dydt(91)+k(8)*y(299);
dydt(177)=dydt(177)+k(8)*y(299);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

dydt(299)=dydt(299)-k(6)*y(299);
dydt(72)=dydt(72)+k(6)*y(299);
dydt(176)=dydt(176)+k(6)*y(299);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(301)=dydt(301)-k(34)*y(301);
dydt(307)=dydt(307)+k(34)*y(301);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(301)=dydt(301)-k(26)*y(301);
dydt(647)=dydt(647)+k(26)*y(301);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(301)=dydt(301)-k(22)*y(301);
dydt(182)=dydt(182)+k(22)*y(301);
dydt(78)=dydt(78)+k(22)*y(301);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(301)=dydt(301)-k(19)*y(301);
dydt(624)=dydt(624)+k(19)*y(301);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(301)=dydt(301)-k(16)*y(301);
dydt(3)=dydt(3)+k(16)*y(301);
dydt(300)=dydt(300)+k(16)*y(301);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(301)=dydt(301)-k(16)*y(301);
dydt(3)=dydt(3)+k(16)*y(301);
dydt(303)=dydt(303)+k(16)*y(301);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(301)=dydt(301)-k(8)*y(301);
dydt(62)=dydt(62)+k(8)*y(301);
dydt(222)=dydt(222)+k(8)*y(301);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(301)=dydt(301)-k(6)*y(301);
dydt(61)=dydt(61)+k(6)*y(301);
dydt(515)=dydt(515)+k(6)*y(301);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(303)=dydt(303)-k(34)*y(303);
dydt(306)=dydt(306)+k(34)*y(303);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(303)=dydt(303)-k(31)*y(303);
dydt(591)=dydt(591)+k(31)*y(303);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

dydt(303)=dydt(303)-k(22)*y(303);
dydt(182)=dydt(182)+k(22)*y(303);
dydt(309)=dydt(309)+k(22)*y(303);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(303)=dydt(303)-k(19)*y(303);
dydt(623)=dydt(623)+k(19)*y(303);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(303)=dydt(303)-k(16)*y(303);
dydt(3)=dydt(3)+k(16)*y(303);
dydt(302)=dydt(302)+k(16)*y(303);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(303);
dydt(303)=dydt(303)-k(15)*y(3)*y(303);
dydt(301)=dydt(301)+k(15)*y(3)*y(303);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(303)=dydt(303)-k(8)*y(303);
dydt(91)=dydt(91)+k(8)*y(303);
dydt(222)=dydt(222)+k(8)*y(303);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

dydt(303)=dydt(303)-k(6)*y(303);
dydt(72)=dydt(72)+k(6)*y(303);
dydt(515)=dydt(515)+k(6)*y(303);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(304)=dydt(304)-k(34)*y(304);
dydt(112)=dydt(112)+k(34)*y(304);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(304)=dydt(304)-k(31)*y(304);
dydt(418)=dydt(418)+k(31)*y(304);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(304)=dydt(304)-k(22)*y(304);
dydt(182)=dydt(182)+k(22)*y(304);
dydt(590)=dydt(590)+k(22)*y(304);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(304);
dydt(304)=dydt(304)-k(15)*y(3)*y(304);
dydt(591)=dydt(591)+k(15)*y(3)*y(304);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(304);
dydt(304)=dydt(304)-k(9)*y(3)*y(304);
dydt(594)=dydt(594)+k(9)*y(3)*y(304);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(304)=dydt(304)-k(8)*y(304);
dydt(5)=dydt(5)+k(8)*y(304);
dydt(218)=dydt(218)+k(8)*y(304);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

dydt(304)=dydt(304)-k(6)*y(304);
dydt(1)=dydt(1)+k(6)*y(304);
dydt(264)=dydt(264)+k(6)*y(304);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(307)=dydt(307)-k(26)*y(307);
dydt(83)=dydt(83)+k(26)*y(307);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(307)=dydt(307)-k(22)*y(307);
dydt(2)=dydt(2)+k(22)*y(307);
dydt(78)=dydt(78)+k(22)*y(307);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(307)=dydt(307)-k(19)*y(307);
dydt(628)=dydt(628)+k(19)*y(307);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(307)=dydt(307)-k(16)*y(307);
dydt(3)=dydt(3)+k(16)*y(307);
dydt(306)=dydt(306)+k(16)*y(307);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(307)=dydt(307)-k(16)*y(307);
dydt(3)=dydt(3)+k(16)*y(307);
dydt(80)=dydt(80)+k(16)*y(307);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(307)=dydt(307)-k(8)*y(307);
dydt(62)=dydt(62)+k(8)*y(307);
dydt(43)=dydt(43)+k(8)*y(307);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(307)=dydt(307)-k(6)*y(307);
dydt(61)=dydt(61)+k(6)*y(307);
dydt(114)=dydt(114)+k(6)*y(307);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(308)=dydt(308)-k(32)*y(308);
dydt(306)=dydt(306)+k(32)*y(308);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(308)=dydt(308)-k(31)*y(308);
dydt(524)=dydt(524)+k(31)*y(308);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

dydt(308)=dydt(308)-k(22)*y(308);
dydt(2)=dydt(2)+k(22)*y(308);
dydt(310)=dydt(310)+k(22)*y(308);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(308);
dydt(308)=dydt(308)-k(21)*y(197)*y(308);
dydt(528)=dydt(528)+k(21)*y(197)*y(308);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(308);
dydt(308)=dydt(308)-k(21)*y(32)*y(308);
dydt(500)=dydt(500)+k(21)*y(32)*y(308);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(308);
dydt(308)=dydt(308)-k(21)*y(2)*y(308);
dydt(504)=dydt(504)+k(21)*y(2)*y(308);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(308)=dydt(308)-k(16)*y(308);
dydt(3)=dydt(3)+k(16)*y(308);
dydt(501)=dydt(501)+k(16)*y(308);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(308);
dydt(308)=dydt(308)-k(15)*y(3)*y(308);
dydt(628)=dydt(628)+k(15)*y(3)*y(308);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(308)=dydt(308)-k(8)*y(308);
dydt(315)=dydt(315)+k(8)*y(308);
dydt(43)=dydt(43)+k(8)*y(308);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

dydt(308)=dydt(308)-k(6)*y(308);
dydt(77)=dydt(77)+k(6)*y(308);
dydt(114)=dydt(114)+k(6)*y(308);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(310)=dydt(310)-k(32)*y(310);
dydt(314)=dydt(314)+k(32)*y(310);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(310)=dydt(310)-k(32)*y(310);
dydt(309)=dydt(309)+k(32)*y(310);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(310)=dydt(310)-k(31)*y(310);
dydt(537)=dydt(537)+k(31)*y(310);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(310);
dydt(310)=dydt(310)-k(21)*y(197)*y(310);
dydt(564)=dydt(564)+k(21)*y(197)*y(310);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(310);
dydt(310)=dydt(310)-k(21)*y(32)*y(310);
dydt(534)=dydt(534)+k(21)*y(32)*y(310);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(310);
dydt(310)=dydt(310)-k(21)*y(2)*y(310);
dydt(540)=dydt(540)+k(21)*y(2)*y(310);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(310);
dydt(310)=dydt(310)-k(21)*y(197)*y(310);
dydt(581)=dydt(581)+k(21)*y(197)*y(310);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(310);
dydt(310)=dydt(310)-k(21)*y(32)*y(310);
dydt(563)=dydt(563)+k(21)*y(32)*y(310);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(310);
dydt(310)=dydt(310)-k(21)*y(2)*y(310);
dydt(308)=dydt(308)+k(21)*y(2)*y(310);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

dydt(310)=dydt(310)-k(16)*y(310);
dydt(3)=dydt(3)+k(16)*y(310);
dydt(584)=dydt(584)+k(16)*y(310);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(310);
dydt(310)=dydt(310)-k(15)*y(3)*y(310);
dydt(627)=dydt(627)+k(15)*y(3)*y(310);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(310)=dydt(310)-k(8)*y(310);
dydt(315)=dydt(315)+k(8)*y(310);
dydt(54)=dydt(54)+k(8)*y(310);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) 

dydt(310)=dydt(310)-k(6)*y(310);
dydt(77)=dydt(77)+k(6)*y(310);
dydt(118)=dydt(118)+k(6)*y(310);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(311)=dydt(311)-k(33)*y(311);
dydt(306)=dydt(306)+k(33)*y(311);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(311)=dydt(311)-k(31)*y(311);
dydt(592)=dydt(592)+k(31)*y(311);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

dydt(311)=dydt(311)-k(22)*y(311);
dydt(32)=dydt(32)+k(22)*y(311);
dydt(309)=dydt(309)+k(22)*y(311);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(311)=dydt(311)-k(19)*y(311);
dydt(563)=dydt(563)+k(19)*y(311);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(311)=dydt(311)-k(16)*y(311);
dydt(3)=dydt(3)+k(16)*y(311);
dydt(84)=dydt(84)+k(16)*y(311);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(311);
dydt(311)=dydt(311)-k(15)*y(3)*y(311);
dydt(83)=dydt(83)+k(15)*y(3)*y(311);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(311)=dydt(311)-k(8)*y(311);
dydt(91)=dydt(91)+k(8)*y(311);
dydt(37)=dydt(37)+k(8)*y(311);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

dydt(311)=dydt(311)-k(6)*y(311);
dydt(72)=dydt(72)+k(6)*y(311);
dydt(554)=dydt(554)+k(6)*y(311);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(312)=dydt(312)-k(34)*y(312);
dydt(311)=dydt(311)+k(34)*y(312);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(312)=dydt(312)-k(33)*y(312);
dydt(303)=dydt(303)+k(33)*y(312);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(312)=dydt(312)-k(31)*y(312);
dydt(589)=dydt(589)+k(31)*y(312);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

dydt(312)=dydt(312)-k(22)*y(312);
dydt(197)=dydt(197)+k(22)*y(312);
dydt(309)=dydt(309)+k(22)*y(312);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(312)=dydt(312)-k(19)*y(312);
dydt(581)=dydt(581)+k(19)*y(312);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(312)=dydt(312)-k(16)*y(312);
dydt(3)=dydt(3)+k(16)*y(312);
dydt(586)=dydt(586)+k(16)*y(312);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(312);
dydt(312)=dydt(312)-k(15)*y(3)*y(312);
dydt(647)=dydt(647)+k(15)*y(3)*y(312);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(312)=dydt(312)-k(8)*y(312);
dydt(91)=dydt(91)+k(8)*y(312);
dydt(227)=dydt(227)+k(8)*y(312);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

dydt(312)=dydt(312)-k(6)*y(312);
dydt(72)=dydt(72)+k(6)*y(312);
dydt(543)=dydt(543)+k(6)*y(312);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(315)=dydt(315)-k(32)*y(315);
dydt(91)=dydt(91)+k(32)*y(315);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(315)=dydt(315)-k(31)*y(315);
dydt(16)=dydt(16)+k(31)*y(315);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(315);
dydt(315)=dydt(315)-k(21)*y(197)*y(315);
dydt(319)=dydt(319)+k(21)*y(197)*y(315);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(315);
dydt(315)=dydt(315)-k(21)*y(32)*y(315);
dydt(34)=dydt(34)+k(21)*y(32)*y(315);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(315);
dydt(315)=dydt(315)-k(21)*y(2)*y(315);
dydt(437)=dydt(437)+k(21)*y(2)*y(315);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(315);
dydt(315)=dydt(315)-k(15)*y(3)*y(315);
dydt(55)=dydt(55)+k(15)*y(3)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(315);
dydt(315)=dydt(315)-k(7)*y(1)*y(315);
dydt(234)=dydt(234)+k(7)*y(1)*y(315);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(315)=dydt(315)-k(7)*y(315)*y(7);
dydt(7)=dydt(7)-k(7)*y(315)*y(7);
dydt(19)=dydt(19)+k(7)*y(315)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(315)=dydt(315)-k(7)*y(315)*y(335);
dydt(335)=dydt(335)-k(7)*y(315)*y(335);
dydt(410)=dydt(410)+k(7)*y(315)*y(335);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a, b~pY, g~pY) + Lig(l1, l2) 

dydt(315)=dydt(315)-k(2)*y(315);
dydt(4)=dydt(4)+k(2)*y(315);
dydt(77)=dydt(77)+k(2)*y(315);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(316)=dydt(316)-k(31)*y(316);
dydt(329)=dydt(329)+k(31)*y(316);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(316)=dydt(316)-k(25)*y(316);
dydt(317)=dydt(317)+k(25)*y(316);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

dydt(316)=dydt(316)-k(22)*y(316);
dydt(2)=dydt(2)+k(22)*y(316);
dydt(314)=dydt(314)+k(22)*y(316);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(316)=dydt(316)-k(16)*y(316);
dydt(3)=dydt(3)+k(16)*y(316);
dydt(653)=dydt(653)+k(16)*y(316);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(316);
dydt(316)=dydt(316)-k(15)*y(3)*y(316);
dydt(654)=dydt(654)+k(15)*y(3)*y(316);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(316)=dydt(316)-k(8)*y(316);
dydt(61)=dydt(61)+k(8)*y(316);
dydt(437)=dydt(437)+k(8)*y(316);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(316)=dydt(316)-k(6)*y(316);
dydt(122)=dydt(122)+k(6)*y(316);
dydt(111)=dydt(111)+k(6)*y(316);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(317)=dydt(317)-k(33)*y(317);
dydt(316)=dydt(316)+k(33)*y(317);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(317)=dydt(317)-k(31)*y(317);
dydt(133)=dydt(133)+k(31)*y(317);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

dydt(317)=dydt(317)-k(22)*y(317);
dydt(32)=dydt(32)+k(22)*y(317);
dydt(314)=dydt(314)+k(22)*y(317);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(317)=dydt(317)-k(16)*y(317);
dydt(3)=dydt(3)+k(16)*y(317);
dydt(242)=dydt(242)+k(16)*y(317);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(317);
dydt(317)=dydt(317)-k(15)*y(3)*y(317);
dydt(241)=dydt(241)+k(15)*y(3)*y(317);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(317)=dydt(317)-k(8)*y(317);
dydt(61)=dydt(61)+k(8)*y(317);
dydt(34)=dydt(34)+k(8)*y(317);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(317)=dydt(317)-k(6)*y(317);
dydt(122)=dydt(122)+k(6)*y(317);
dydt(33)=dydt(33)+k(6)*y(317);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(319)=dydt(319)-k(34)*y(319);
dydt(34)=dydt(34)+k(34)*y(319);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(319)=dydt(319)-k(33)*y(319);
dydt(219)=dydt(219)+k(33)*y(319);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(319)=dydt(319)-k(31)*y(319);
dydt(209)=dydt(209)+k(31)*y(319);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~pY) 

dydt(319)=dydt(319)-k(22)*y(319);
dydt(197)=dydt(197)+k(22)*y(319);
dydt(315)=dydt(315)+k(22)*y(319);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(319);
dydt(319)=dydt(319)-k(15)*y(3)*y(319);
dydt(228)=dydt(228)+k(15)*y(3)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(1)=dydt(1)-k(7)*y(1)*y(319);
dydt(319)=dydt(319)-k(7)*y(1)*y(319);
dydt(230)=dydt(230)+k(7)*y(1)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(7)=dydt(7)-k(7)*y(7)*y(319);
dydt(319)=dydt(319)-k(7)*y(7)*y(319);
dydt(212)=dydt(212)+k(7)*y(7)*y(319);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(319)=dydt(319)-k(7)*y(319)*y(335);
dydt(335)=dydt(335)-k(7)*y(319)*y(335);
dydt(434)=dydt(434)+k(7)*y(319)*y(335);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) 

dydt(319)=dydt(319)-k(2)*y(319);
dydt(4)=dydt(4)+k(2)*y(319);
dydt(257)=dydt(257)+k(2)*y(319);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(320)=dydt(320)-k(34)*y(320);
dydt(241)=dydt(241)+k(34)*y(320);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(320)=dydt(320)-k(33)*y(320);
dydt(324)=dydt(324)+k(33)*y(320);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(320)=dydt(320)-k(22)*y(320);
dydt(197)=dydt(197)+k(22)*y(320);
dydt(236)=dydt(236)+k(22)*y(320);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(320)=dydt(320)-k(20)*y(320);
dydt(636)=dydt(636)+k(20)*y(320);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(320)=dydt(320)-k(16)*y(320);
dydt(3)=dydt(3)+k(16)*y(320);
dydt(318)=dydt(318)+k(16)*y(320);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(320)=dydt(320)-k(16)*y(320);
dydt(3)=dydt(3)+k(16)*y(320);
dydt(231)=dydt(231)+k(16)*y(320);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(320)=dydt(320)-k(8)*y(320);
dydt(61)=dydt(61)+k(8)*y(320);
dydt(228)=dydt(228)+k(8)*y(320);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(320)=dydt(320)-k(6)*y(320);
dydt(122)=dydt(122)+k(6)*y(320);
dydt(227)=dydt(227)+k(6)*y(320);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(321)=dydt(321)-k(34)*y(321);
dydt(242)=dydt(242)+k(34)*y(321);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(321)=dydt(321)-k(33)*y(321);
dydt(325)=dydt(325)+k(33)*y(321);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(321)=dydt(321)-k(31)*y(321);
dydt(230)=dydt(230)+k(31)*y(321);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(321)=dydt(321)-k(31)*y(321);
dydt(332)=dydt(332)+k(31)*y(321);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(321)=dydt(321)-k(22)*y(321);
dydt(197)=dydt(197)+k(22)*y(321);
dydt(237)=dydt(237)+k(22)*y(321);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(321);
dydt(321)=dydt(321)-k(15)*y(3)*y(321);
dydt(231)=dydt(231)+k(15)*y(3)*y(321);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(321);
dydt(321)=dydt(321)-k(15)*y(3)*y(321);
dydt(318)=dydt(318)+k(15)*y(3)*y(321);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

dydt(321)=dydt(321)-k(8)*y(321);
dydt(72)=dydt(72)+k(8)*y(321);
dydt(319)=dydt(319)+k(8)*y(321);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(321)=dydt(321)-k(6)*y(321);
dydt(71)=dydt(71)+k(6)*y(321);
dydt(257)=dydt(257)+k(6)*y(321);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(322)=dydt(322)-k(34)*y(322);
dydt(133)=dydt(133)+k(34)*y(322);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(322)=dydt(322)-k(33)*y(322);
dydt(326)=dydt(326)+k(33)*y(322);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

dydt(322)=dydt(322)-k(22)*y(322);
dydt(197)=dydt(197)+k(22)*y(322);
dydt(127)=dydt(127)+k(22)*y(322);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(322)=dydt(322)-k(17)*y(322);
dydt(318)=dydt(318)+k(17)*y(322);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(322)=dydt(322)-k(16)*y(322);
dydt(3)=dydt(3)+k(16)*y(322);
dydt(332)=dydt(332)+k(16)*y(322);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(322);
dydt(322)=dydt(322)-k(9)*y(3)*y(322);
dydt(200)=dydt(200)+k(9)*y(3)*y(322);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(322)=dydt(322)-k(8)*y(322);
dydt(61)=dydt(61)+k(8)*y(322);
dydt(209)=dydt(209)+k(8)*y(322);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(322)=dydt(322)-k(6)*y(322);
dydt(122)=dydt(122)+k(6)*y(322);
dydt(192)=dydt(192)+k(6)*y(322);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(654)=dydt(654)-k(25)*y(654);
dydt(241)=dydt(241)+k(25)*y(654);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(654)=dydt(654)-k(22)*y(654);
dydt(2)=dydt(2)+k(22)*y(654);
dydt(236)=dydt(236)+k(22)*y(654);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(654)=dydt(654)-k(20)*y(654);
dydt(251)=dydt(251)+k(20)*y(654);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(654)=dydt(654)-k(16)*y(654);
dydt(3)=dydt(3)+k(16)*y(654);
dydt(239)=dydt(239)+k(16)*y(654);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(654)=dydt(654)-k(16)*y(654);
dydt(3)=dydt(3)+k(16)*y(654);
dydt(316)=dydt(316)+k(16)*y(654);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(654)=dydt(654)-k(8)*y(654);
dydt(61)=dydt(61)+k(8)*y(654);
dydt(44)=dydt(44)+k(8)*y(654);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(654)=dydt(654)-k(6)*y(654);
dydt(122)=dydt(122)+k(6)*y(654);
dydt(43)=dydt(43)+k(6)*y(654);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(324)=dydt(324)-k(34)*y(324);
dydt(654)=dydt(654)+k(34)*y(324);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(324)=dydt(324)-k(25)*y(324);
dydt(320)=dydt(320)+k(25)*y(324);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(324)=dydt(324)-k(22)*y(324);
dydt(182)=dydt(182)+k(22)*y(324);
dydt(236)=dydt(236)+k(22)*y(324);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(324)=dydt(324)-k(20)*y(324);
dydt(638)=dydt(638)+k(20)*y(324);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(324)=dydt(324)-k(16)*y(324);
dydt(3)=dydt(3)+k(16)*y(324);
dydt(323)=dydt(323)+k(16)*y(324);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(324)=dydt(324)-k(16)*y(324);
dydt(3)=dydt(3)+k(16)*y(324);
dydt(646)=dydt(646)+k(16)*y(324);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(324)=dydt(324)-k(8)*y(324);
dydt(61)=dydt(61)+k(8)*y(324);
dydt(223)=dydt(223)+k(8)*y(324);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(324)=dydt(324)-k(6)*y(324);
dydt(122)=dydt(122)+k(6)*y(324);
dydt(222)=dydt(222)+k(6)*y(324);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(653)=dydt(653)-k(31)*y(653);
dydt(621)=dydt(621)+k(31)*y(653);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(653)=dydt(653)-k(31)*y(653);
dydt(142)=dydt(142)+k(31)*y(653);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(653)=dydt(653)-k(22)*y(653);
dydt(2)=dydt(2)+k(22)*y(653);
dydt(237)=dydt(237)+k(22)*y(653);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(653);
dydt(653)=dydt(653)-k(15)*y(3)*y(653);
dydt(239)=dydt(239)+k(15)*y(3)*y(653);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(653);
dydt(653)=dydt(653)-k(15)*y(3)*y(653);
dydt(316)=dydt(316)+k(15)*y(3)*y(653);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

dydt(653)=dydt(653)-k(8)*y(653);
dydt(72)=dydt(72)+k(8)*y(653);
dydt(437)=dydt(437)+k(8)*y(653);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(653)=dydt(653)-k(6)*y(653);
dydt(71)=dydt(71)+k(6)*y(653);
dydt(111)=dydt(111)+k(6)*y(653);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(325)=dydt(325)-k(34)*y(325);
dydt(653)=dydt(653)+k(34)*y(325);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(325)=dydt(325)-k(31)*y(325);
dydt(220)=dydt(220)+k(31)*y(325);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(325)=dydt(325)-k(31)*y(325);
dydt(328)=dydt(328)+k(31)*y(325);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(325)=dydt(325)-k(22)*y(325);
dydt(182)=dydt(182)+k(22)*y(325);
dydt(237)=dydt(237)+k(22)*y(325);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(325);
dydt(325)=dydt(325)-k(15)*y(3)*y(325);
dydt(646)=dydt(646)+k(15)*y(3)*y(325);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(325);
dydt(325)=dydt(325)-k(15)*y(3)*y(325);
dydt(323)=dydt(323)+k(15)*y(3)*y(325);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

dydt(325)=dydt(325)-k(8)*y(325);
dydt(72)=dydt(72)+k(8)*y(325);
dydt(219)=dydt(219)+k(8)*y(325);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(325)=dydt(325)-k(6)*y(325);
dydt(71)=dydt(71)+k(6)*y(325);
dydt(218)=dydt(218)+k(6)*y(325);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(327)=dydt(327)-k(34)*y(327);
dydt(129)=dydt(129)+k(34)*y(327);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(327)=dydt(327)-k(25)*y(327);
dydt(200)=dydt(200)+k(25)*y(327);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(327)=dydt(327)-k(22)*y(327);
dydt(182)=dydt(182)+k(22)*y(327);
dydt(125)=dydt(125)+k(22)*y(327);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(327)=dydt(327)-k(16)*y(327);
dydt(3)=dydt(3)+k(16)*y(327);
dydt(330)=dydt(330)+k(16)*y(327);

% rule    : R5_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(327)=dydt(327)-k(14)*y(327);
dydt(345)=dydt(345)+k(14)*y(327);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(327)=dydt(327)-k(10)*y(327);
dydt(3)=dydt(3)+k(10)*y(327);
dydt(326)=dydt(326)+k(10)*y(327);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(327)=dydt(327)-k(8)*y(327);
dydt(61)=dydt(61)+k(8)*y(327);
dydt(185)=dydt(185)+k(8)*y(327);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(327)=dydt(327)-k(6)*y(327);
dydt(122)=dydt(122)+k(6)*y(327);
dydt(184)=dydt(184)+k(6)*y(327);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(652)=dydt(652)-k(34)*y(652);
dydt(144)=dydt(144)+k(34)*y(652);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(652)=dydt(652)-k(22)*y(652);
dydt(182)=dydt(182)+k(22)*y(652);
dydt(146)=dydt(146)+k(22)*y(652);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(652)=dydt(652)-k(14)*y(652);
dydt(189)=dydt(189)+k(14)*y(652);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(652)=dydt(652)-k(12)*y(652);
dydt(330)=dydt(330)+k(12)*y(652);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(652)=dydt(652)-k(10)*y(652);
dydt(3)=dydt(3)+k(10)*y(652);
dydt(216)=dydt(216)+k(10)*y(652);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(652);
dydt(652)=dydt(652)-k(9)*y(3)*y(652);
dydt(215)=dydt(215)+k(9)*y(3)*y(652);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

dydt(652)=dydt(652)-k(8)*y(652);
dydt(1)=dydt(1)+k(8)*y(652);
dydt(185)=dydt(185)+k(8)*y(652);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(652)=dydt(652)-k(6)*y(652);
dydt(6)=dydt(6)+k(6)*y(652);
dydt(184)=dydt(184)+k(6)*y(652);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(330)=dydt(330)-k(34)*y(330);
dydt(139)=dydt(139)+k(34)*y(330);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(330)=dydt(330)-k(31)*y(330);
dydt(652)=dydt(652)+k(31)*y(330);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(330)=dydt(330)-k(22)*y(330);
dydt(182)=dydt(182)+k(22)*y(330);
dydt(128)=dydt(128)+k(22)*y(330);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(330);
dydt(330)=dydt(330)-k(15)*y(3)*y(330);
dydt(327)=dydt(327)+k(15)*y(3)*y(330);

% rule    : R5_2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(330)=dydt(330)-k(14)*y(330);
dydt(343)=dydt(343)+k(14)*y(330);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(330)=dydt(330)-k(10)*y(330);
dydt(3)=dydt(3)+k(10)*y(330);
dydt(328)=dydt(328)+k(10)*y(330);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

dydt(330)=dydt(330)-k(8)*y(330);
dydt(72)=dydt(72)+k(8)*y(330);
dydt(185)=dydt(185)+k(8)*y(330);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(330)=dydt(330)-k(6)*y(330);
dydt(71)=dydt(71)+k(6)*y(330);
dydt(184)=dydt(184)+k(6)*y(330);

% rule    : R17
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(335)=dydt(335)-k(34)*y(335);
dydt(131)=dydt(131)+k(34)*y(335);

% rule    : R16
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(335)=dydt(335)-k(33)*y(335);
dydt(184)=dydt(184)+k(33)*y(335);

% rule    : R9_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(335)=dydt(335)-k(22)*y(335);
dydt(197)=dydt(197)+k(22)*y(335);
dydt(48)=dydt(48)+k(22)*y(335);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lyn(U, SH2) 

dydt(335)=dydt(335)-k(10)*y(335);
dydt(3)=dydt(3)+k(10)*y(335);
dydt(192)=dydt(192)+k(10)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(7)*y(335)*y(336);
dydt(336)=dydt(336)-k(7)*y(335)*y(336);
dydt(464)=dydt(464)+k(7)*y(335)*y(336);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(437)=dydt(437)-k(7)*y(437)*y(335);
dydt(335)=dydt(335)-k(7)*y(437)*y(335);
dydt(436)=dydt(436)+k(7)*y(437)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(5)*y(335)*y(380);
dydt(380)=dydt(380)-k(5)*y(335)*y(380);
dydt(372)=dydt(372)+k(5)*y(335)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(5)*y(335)*y(391);
dydt(391)=dydt(391)-k(5)*y(335)*y(391);
dydt(368)=dydt(368)+k(5)*y(335)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(5)*y(335)*y(407);
dydt(407)=dydt(407)-k(5)*y(335)*y(407);
dydt(464)=dydt(464)+k(5)*y(335)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(5)*y(335)*y(441);
dydt(441)=dydt(441)-k(5)*y(335)*y(441);
dydt(459)=dydt(459)+k(5)*y(335)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(5)*y(335)*y(515);
dydt(515)=dydt(515)-k(5)*y(335)*y(515);
dydt(597)=dydt(597)+k(5)*y(335)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(5)*y(335)*y(543);
dydt(543)=dydt(543)-k(5)*y(335)*y(543);
dydt(601)=dydt(601)+k(5)*y(335)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(335)=dydt(335)-k(5)*y(335)*y(554);
dydt(554)=dydt(554)-k(5)*y(335)*y(554);
dydt(558)=dydt(558)+k(5)*y(335)*y(554);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(9)=dydt(9)-k(5)*y(9)*y(335);
dydt(335)=dydt(335)-k(5)*y(9)*y(335);
dydt(210)=dydt(210)+k(5)*y(9)*y(335);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(4)=dydt(4)-k(3)*y(4)*y(335);
dydt(335)=dydt(335)-k(3)*y(4)*y(335);
dydt(407)=dydt(407)+k(3)*y(4)*y(335);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(4)=dydt(4)-k(1)*y(4)*y(335);
dydt(335)=dydt(335)-k(1)*y(4)*y(335);
dydt(336)=dydt(336)+k(1)*y(4)*y(335);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(336)=dydt(336)-k(34)*y(336);
dydt(132)=dydt(132)+k(34)*y(336);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(336)=dydt(336)-k(33)*y(336);
dydt(185)=dydt(185)+k(33)*y(336);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(336)=dydt(336)-k(22)*y(336);
dydt(197)=dydt(197)+k(22)*y(336);
dydt(126)=dydt(126)+k(22)*y(336);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(336)=dydt(336)-k(10)*y(336);
dydt(3)=dydt(3)+k(10)*y(336);
dydt(209)=dydt(209)+k(10)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(1)=dydt(1)-k(7)*y(1)*y(336);
dydt(336)=dydt(336)-k(7)*y(1)*y(336);
dydt(199)=dydt(199)+k(7)*y(1)*y(336);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(7)*y(7)*y(336);
dydt(336)=dydt(336)-k(7)*y(7)*y(336);
dydt(210)=dydt(210)+k(7)*y(7)*y(336);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) 

dydt(336)=dydt(336)-k(2)*y(336);
dydt(4)=dydt(4)+k(2)*y(336);
dydt(335)=dydt(335)+k(2)*y(336);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(338)=dydt(338)-k(34)*y(338);
dydt(533)=dydt(533)+k(34)*y(338);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(338)=dydt(338)-k(33)*y(338);
dydt(344)=dydt(344)+k(33)*y(338);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(338)=dydt(338)-k(32)*y(338);
dydt(332)=dydt(332)+k(32)*y(338);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) 

dydt(338)=dydt(338)-k(31)*y(338);
dydt(376)=dydt(376)+k(31)*y(338);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(338)=dydt(338)-k(22)*y(338);
dydt(197)=dydt(197)+k(22)*y(338);
dydt(610)=dydt(610)+k(22)*y(338);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(338);
dydt(338)=dydt(338)-k(21)*y(197)*y(338);
dydt(609)=dydt(609)+k(21)*y(197)*y(338);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(338);
dydt(338)=dydt(338)-k(21)*y(32)*y(338);
dydt(559)=dydt(559)+k(21)*y(32)*y(338);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(338);
dydt(338)=dydt(338)-k(21)*y(2)*y(338);
dydt(527)=dydt(527)+k(21)*y(2)*y(338);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(338);
dydt(338)=dydt(338)-k(15)*y(3)*y(338);
dydt(529)=dydt(529)+k(15)*y(3)*y(338);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(338);
dydt(338)=dydt(338)-k(9)*y(3)*y(338);
dydt(337)=dydt(337)+k(9)*y(3)*y(338);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

dydt(338)=dydt(338)-k(8)*y(338);
dydt(77)=dydt(77)+k(8)*y(338);
dydt(209)=dydt(209)+k(8)*y(338);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(338)=dydt(338)-k(6)*y(338);
dydt(76)=dydt(76)+k(6)*y(338);
dydt(192)=dydt(192)+k(6)*y(338);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(339)=dydt(339)-k(34)*y(339);
dydt(134)=dydt(134)+k(34)*y(339);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(339)=dydt(339)-k(33)*y(339);
dydt(345)=dydt(345)+k(33)*y(339);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(339)=dydt(339)-k(32)*y(339);
dydt(200)=dydt(200)+k(32)*y(339);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(339)=dydt(339)-k(22)*y(339);
dydt(197)=dydt(197)+k(22)*y(339);
dydt(120)=dydt(120)+k(22)*y(339);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(339);
dydt(339)=dydt(339)-k(21)*y(197)*y(339);
dydt(601)=dydt(601)+k(21)*y(197)*y(339);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(339);
dydt(339)=dydt(339)-k(21)*y(32)*y(339);
dydt(558)=dydt(558)+k(21)*y(32)*y(339);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(339);
dydt(339)=dydt(339)-k(21)*y(2)*y(339);
dydt(526)=dydt(526)+k(21)*y(2)*y(339);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(339)=dydt(339)-k(16)*y(339);
dydt(3)=dydt(3)+k(16)*y(339);
dydt(337)=dydt(337)+k(16)*y(339);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(339)=dydt(339)-k(10)*y(339);
dydt(3)=dydt(3)+k(10)*y(339);
dydt(529)=dydt(529)+k(10)*y(339);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(339)=dydt(339)-k(8)*y(339);
dydt(54)=dydt(54)+k(8)*y(339);
dydt(336)=dydt(336)+k(8)*y(339);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(339)=dydt(339)-k(6)*y(339);
dydt(118)=dydt(118)+k(6)*y(339);
dydt(335)=dydt(335)+k(6)*y(339);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(340)=dydt(340)-k(34)*y(340);
dydt(355)=dydt(355)+k(34)*y(340);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(340)=dydt(340)-k(33)*y(340);
dydt(346)=dydt(346)+k(33)*y(340);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(340)=dydt(340)-k(31)*y(340);
dydt(371)=dydt(371)+k(31)*y(340);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(340)=dydt(340)-k(30)*y(340);
dydt(363)=dydt(363)+k(30)*y(340);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(340)=dydt(340)-k(24)*y(340);
dydt(341)=dydt(341)+k(24)*y(340);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(340)=dydt(340)-k(22)*y(340);
dydt(2)=dydt(2)+k(22)*y(340);
dydt(337)=dydt(337)+k(22)*y(340);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(340)=dydt(340)-k(22)*y(340);
dydt(197)=dydt(197)+k(22)*y(340);
dydt(353)=dydt(353)+k(22)*y(340);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(340);
dydt(340)=dydt(340)-k(15)*y(3)*y(340);
dydt(526)=dydt(526)+k(15)*y(3)*y(340);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(340)=dydt(340)-k(10)*y(340);
dydt(3)=dydt(3)+k(10)*y(340);
dydt(527)=dydt(527)+k(10)*y(340);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(340)=dydt(340)-k(8)*y(340);
dydt(111)=dydt(111)+k(8)*y(340);
dydt(336)=dydt(336)+k(8)*y(340);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(340)=dydt(340)-k(6)*y(340);
dydt(110)=dydt(110)+k(6)*y(340);
dydt(335)=dydt(335)+k(6)*y(340);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(341)=dydt(341)-k(34)*y(341);
dydt(161)=dydt(161)+k(34)*y(341);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(341)=dydt(341)-k(33)*y(341);
dydt(340)=dydt(340)+k(33)*y(341);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(341)=dydt(341)-k(33)*y(341);
dydt(347)=dydt(347)+k(33)*y(341);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(341)=dydt(341)-k(31)*y(341);
dydt(162)=dydt(162)+k(31)*y(341);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(341)=dydt(341)-k(30)*y(341);
dydt(342)=dydt(342)+k(30)*y(341);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(341)=dydt(341)-k(22)*y(341);
dydt(32)=dydt(32)+k(22)*y(341);
dydt(337)=dydt(337)+k(22)*y(341);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(341)=dydt(341)-k(22)*y(341);
dydt(197)=dydt(197)+k(22)*y(341);
dydt(350)=dydt(350)+k(22)*y(341);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(341);
dydt(341)=dydt(341)-k(15)*y(3)*y(341);
dydt(558)=dydt(558)+k(15)*y(3)*y(341);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(341)=dydt(341)-k(10)*y(341);
dydt(3)=dydt(3)+k(10)*y(341);
dydt(559)=dydt(559)+k(10)*y(341);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(341)=dydt(341)-k(8)*y(341);
dydt(33)=dydt(33)+k(8)*y(341);
dydt(336)=dydt(336)+k(8)*y(341);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(341)=dydt(341)-k(6)*y(341);
dydt(82)=dydt(82)+k(6)*y(341);
dydt(335)=dydt(335)+k(6)*y(341);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(342)=dydt(342)-k(34)*y(342);
dydt(341)=dydt(341)+k(34)*y(342);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(342)=dydt(342)-k(34)*y(342);
dydt(362)=dydt(362)+k(34)*y(342);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(342)=dydt(342)-k(33)*y(342);
dydt(363)=dydt(363)+k(33)*y(342);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(342)=dydt(342)-k(33)*y(342);
dydt(195)=dydt(195)+k(33)*y(342);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(342)=dydt(342)-k(31)*y(342);
dydt(370)=dydt(370)+k(31)*y(342);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(342)=dydt(342)-k(22)*y(342);
dydt(197)=dydt(197)+k(22)*y(342);
dydt(337)=dydt(337)+k(22)*y(342);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(342)=dydt(342)-k(22)*y(342);
dydt(197)=dydt(197)+k(22)*y(342);
dydt(599)=dydt(599)+k(22)*y(342);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(342);
dydt(342)=dydt(342)-k(15)*y(3)*y(342);
dydt(601)=dydt(601)+k(15)*y(3)*y(342);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(342)=dydt(342)-k(10)*y(342);
dydt(3)=dydt(3)+k(10)*y(342);
dydt(609)=dydt(609)+k(10)*y(342);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(342)=dydt(342)-k(8)*y(342);
dydt(257)=dydt(257)+k(8)*y(342);
dydt(336)=dydt(336)+k(8)*y(342);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(342)=dydt(342)-k(6)*y(342);
dydt(256)=dydt(256)+k(6)*y(342);
dydt(335)=dydt(335)+k(6)*y(342);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(344)=dydt(344)-k(34)*y(344);
dydt(508)=dydt(508)+k(34)*y(344);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(344)=dydt(344)-k(32)*y(344);
dydt(328)=dydt(328)+k(32)*y(344);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(344)=dydt(344)-k(31)*y(344);
dydt(451)=dydt(451)+k(31)*y(344);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(344)=dydt(344)-k(22)*y(344);
dydt(182)=dydt(182)+k(22)*y(344);
dydt(610)=dydt(610)+k(22)*y(344);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(344);
dydt(344)=dydt(344)-k(21)*y(197)*y(344);
dydt(550)=dydt(550)+k(21)*y(197)*y(344);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(344);
dydt(344)=dydt(344)-k(21)*y(32)*y(344);
dydt(348)=dydt(348)+k(21)*y(32)*y(344);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(344);
dydt(344)=dydt(344)-k(21)*y(2)*y(344);
dydt(509)=dydt(509)+k(21)*y(2)*y(344);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(344);
dydt(344)=dydt(344)-k(15)*y(3)*y(344);
dydt(519)=dydt(519)+k(15)*y(3)*y(344);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(344);
dydt(344)=dydt(344)-k(9)*y(3)*y(344);
dydt(343)=dydt(343)+k(9)*y(3)*y(344);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

dydt(344)=dydt(344)-k(8)*y(344);
dydt(77)=dydt(77)+k(8)*y(344);
dydt(214)=dydt(214)+k(8)*y(344);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(344)=dydt(344)-k(6)*y(344);
dydt(76)=dydt(76)+k(6)*y(344);
dydt(177)=dydt(177)+k(6)*y(344);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(345)=dydt(345)-k(34)*y(345);
dydt(539)=dydt(539)+k(34)*y(345);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(345)=dydt(345)-k(32)*y(345);
dydt(327)=dydt(327)+k(32)*y(345);

% rule    : R11
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(345)=dydt(345)-k(25)*y(345);
dydt(339)=dydt(339)+k(25)*y(345);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(345)=dydt(345)-k(22)*y(345);
dydt(182)=dydt(182)+k(22)*y(345);
dydt(120)=dydt(120)+k(22)*y(345);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(345);
dydt(345)=dydt(345)-k(21)*y(197)*y(345);
dydt(549)=dydt(549)+k(21)*y(197)*y(345);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(345);
dydt(345)=dydt(345)-k(21)*y(32)*y(345);
dydt(349)=dydt(349)+k(21)*y(32)*y(345);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(345);
dydt(345)=dydt(345)-k(21)*y(2)*y(345);
dydt(522)=dydt(522)+k(21)*y(2)*y(345);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(345)=dydt(345)-k(16)*y(345);
dydt(3)=dydt(3)+k(16)*y(345);
dydt(343)=dydt(343)+k(16)*y(345);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(345)=dydt(345)-k(10)*y(345);
dydt(3)=dydt(3)+k(10)*y(345);
dydt(519)=dydt(519)+k(10)*y(345);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(345)=dydt(345)-k(8)*y(345);
dydt(54)=dydt(54)+k(8)*y(345);
dydt(185)=dydt(185)+k(8)*y(345);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(345)=dydt(345)-k(6)*y(345);
dydt(118)=dydt(118)+k(6)*y(345);
dydt(184)=dydt(184)+k(6)*y(345);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(346)=dydt(346)-k(34)*y(346);
dydt(173)=dydt(173)+k(34)*y(346);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(346)=dydt(346)-k(31)*y(346);
dydt(174)=dydt(174)+k(31)*y(346);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(346)=dydt(346)-k(30)*y(346);
dydt(188)=dydt(188)+k(30)*y(346);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(346)=dydt(346)-k(24)*y(346);
dydt(347)=dydt(347)+k(24)*y(346);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(346)=dydt(346)-k(22)*y(346);
dydt(2)=dydt(2)+k(22)*y(346);
dydt(343)=dydt(343)+k(22)*y(346);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(346)=dydt(346)-k(22)*y(346);
dydt(182)=dydt(182)+k(22)*y(346);
dydt(353)=dydt(353)+k(22)*y(346);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(346);
dydt(346)=dydt(346)-k(15)*y(3)*y(346);
dydt(522)=dydt(522)+k(15)*y(3)*y(346);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(346)=dydt(346)-k(10)*y(346);
dydt(3)=dydt(3)+k(10)*y(346);
dydt(509)=dydt(509)+k(10)*y(346);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(346)=dydt(346)-k(8)*y(346);
dydt(111)=dydt(111)+k(8)*y(346);
dydt(185)=dydt(185)+k(8)*y(346);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(346)=dydt(346)-k(6)*y(346);
dydt(110)=dydt(110)+k(6)*y(346);
dydt(184)=dydt(184)+k(6)*y(346);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(348)=dydt(348)-k(34)*y(348);
dydt(575)=dydt(575)+k(34)*y(348);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(348)=dydt(348)-k(33)*y(348);
dydt(509)=dydt(509)+k(33)*y(348);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(348)=dydt(348)-k(31)*y(348);
dydt(449)=dydt(449)+k(31)*y(348);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(348)=dydt(348)-k(30)*y(348);
dydt(550)=dydt(550)+k(30)*y(348);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(348)=dydt(348)-k(22)*y(348);
dydt(32)=dydt(32)+k(22)*y(348);
dydt(344)=dydt(344)+k(22)*y(348);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(348)=dydt(348)-k(22)*y(348);
dydt(182)=dydt(182)+k(22)*y(348);
dydt(351)=dydt(351)+k(22)*y(348);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(348);
dydt(348)=dydt(348)-k(15)*y(3)*y(348);
dydt(553)=dydt(553)+k(15)*y(3)*y(348);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(348);
dydt(348)=dydt(348)-k(9)*y(3)*y(348);
dydt(347)=dydt(347)+k(9)*y(3)*y(348);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(348)=dydt(348)-k(8)*y(348);
dydt(33)=dydt(33)+k(8)*y(348);
dydt(214)=dydt(214)+k(8)*y(348);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(348)=dydt(348)-k(6)*y(348);
dydt(177)=dydt(177)+k(6)*y(348);
dydt(82)=dydt(82)+k(6)*y(348);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(349)=dydt(349)-k(34)*y(349);
dydt(603)=dydt(603)+k(34)*y(349);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(349)=dydt(349)-k(33)*y(349);
dydt(522)=dydt(522)+k(33)*y(349);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(349)=dydt(349)-k(30)*y(349);
dydt(549)=dydt(549)+k(30)*y(349);

% rule    : R11
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(349)=dydt(349)-k(25)*y(349);
dydt(558)=dydt(558)+k(25)*y(349);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

dydt(349)=dydt(349)-k(22)*y(349);
dydt(32)=dydt(32)+k(22)*y(349);
dydt(345)=dydt(345)+k(22)*y(349);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(349)=dydt(349)-k(22)*y(349);
dydt(182)=dydt(182)+k(22)*y(349);
dydt(352)=dydt(352)+k(22)*y(349);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(349)=dydt(349)-k(16)*y(349);
dydt(3)=dydt(3)+k(16)*y(349);
dydt(347)=dydt(347)+k(16)*y(349);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(349)=dydt(349)-k(10)*y(349);
dydt(3)=dydt(3)+k(10)*y(349);
dydt(553)=dydt(553)+k(10)*y(349);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(349)=dydt(349)-k(8)*y(349);
dydt(37)=dydt(37)+k(8)*y(349);
dydt(185)=dydt(185)+k(8)*y(349);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(349)=dydt(349)-k(6)*y(349);
dydt(184)=dydt(184)+k(6)*y(349);
dydt(554)=dydt(554)+k(6)*y(349);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(351)=dydt(351)-k(33)*y(351);
dydt(354)=dydt(354)+k(33)*y(351);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(351)=dydt(351)-k(32)*y(351);
dydt(106)=dydt(106)+k(32)*y(351);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(351)=dydt(351)-k(31)*y(351);
dydt(450)=dydt(450)+k(31)*y(351);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(351)=dydt(351)-k(22)*y(351);
dydt(32)=dydt(32)+k(22)*y(351);
dydt(610)=dydt(610)+k(22)*y(351);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(351);
dydt(351)=dydt(351)-k(21)*y(197)*y(351);
dydt(559)=dydt(559)+k(21)*y(197)*y(351);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(351);
dydt(351)=dydt(351)-k(21)*y(32)*y(351);
dydt(612)=dydt(612)+k(21)*y(32)*y(351);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(351);
dydt(351)=dydt(351)-k(21)*y(2)*y(351);
dydt(575)=dydt(575)+k(21)*y(2)*y(351);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(351);
dydt(351)=dydt(351)-k(15)*y(3)*y(351);
dydt(556)=dydt(556)+k(15)*y(3)*y(351);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(351);
dydt(351)=dydt(351)-k(9)*y(3)*y(351);
dydt(350)=dydt(350)+k(9)*y(3)*y(351);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(351)=dydt(351)-k(8)*y(351);
dydt(16)=dydt(16)+k(8)*y(351);
dydt(33)=dydt(33)+k(8)*y(351);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

dydt(351)=dydt(351)-k(6)*y(351);
dydt(15)=dydt(15)+k(6)*y(351);
dydt(82)=dydt(82)+k(6)*y(351);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(352)=dydt(352)-k(33)*y(352);
dydt(116)=dydt(116)+k(33)*y(352);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(352)=dydt(352)-k(32)*y(352);
dydt(107)=dydt(107)+k(32)*y(352);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(352)=dydt(352)-k(22)*y(352);
dydt(32)=dydt(32)+k(22)*y(352);
dydt(120)=dydt(120)+k(22)*y(352);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(352);
dydt(352)=dydt(352)-k(21)*y(197)*y(352);
dydt(558)=dydt(558)+k(21)*y(197)*y(352);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(352);
dydt(352)=dydt(352)-k(21)*y(32)*y(352);
dydt(602)=dydt(602)+k(21)*y(32)*y(352);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(352);
dydt(352)=dydt(352)-k(21)*y(2)*y(352);
dydt(603)=dydt(603)+k(21)*y(2)*y(352);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(352)=dydt(352)-k(16)*y(352);
dydt(3)=dydt(3)+k(16)*y(352);
dydt(350)=dydt(350)+k(16)*y(352);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(352)=dydt(352)-k(10)*y(352);
dydt(3)=dydt(3)+k(10)*y(352);
dydt(556)=dydt(556)+k(10)*y(352);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(352)=dydt(352)-k(8)*y(352);
dydt(126)=dydt(126)+k(8)*y(352);
dydt(37)=dydt(37)+k(8)*y(352);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(352)=dydt(352)-k(6)*y(352);
dydt(48)=dydt(48)+k(6)*y(352);
dydt(554)=dydt(554)+k(6)*y(352);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(354)=dydt(354)-k(32)*y(354);
dydt(112)=dydt(112)+k(32)*y(354);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(354)=dydt(354)-k(31)*y(354);
dydt(375)=dydt(375)+k(31)*y(354);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(354)=dydt(354)-k(22)*y(354);
dydt(2)=dydt(2)+k(22)*y(354);
dydt(610)=dydt(610)+k(22)*y(354);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(354);
dydt(354)=dydt(354)-k(21)*y(197)*y(354);
dydt(527)=dydt(527)+k(21)*y(197)*y(354);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(354);
dydt(354)=dydt(354)-k(21)*y(32)*y(354);
dydt(356)=dydt(356)+k(21)*y(32)*y(354);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(354);
dydt(354)=dydt(354)-k(21)*y(2)*y(354);
dydt(506)=dydt(506)+k(21)*y(2)*y(354);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(354);
dydt(354)=dydt(354)-k(15)*y(3)*y(354);
dydt(524)=dydt(524)+k(15)*y(3)*y(354);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(354);
dydt(354)=dydt(354)-k(9)*y(3)*y(354);
dydt(353)=dydt(353)+k(9)*y(3)*y(354);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(354)=dydt(354)-k(8)*y(354);
dydt(16)=dydt(16)+k(8)*y(354);
dydt(111)=dydt(111)+k(8)*y(354);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

dydt(354)=dydt(354)-k(6)*y(354);
dydt(15)=dydt(15)+k(6)*y(354);
dydt(110)=dydt(110)+k(6)*y(354);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(356)=dydt(356)-k(33)*y(356);
dydt(506)=dydt(506)+k(33)*y(356);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(356)=dydt(356)-k(31)*y(356);
dydt(378)=dydt(378)+k(31)*y(356);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(356)=dydt(356)-k(28)*y(356);
dydt(359)=dydt(359)+k(28)*y(356);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(356)=dydt(356)-k(27)*y(356);
dydt(527)=dydt(527)+k(27)*y(356);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(356)=dydt(356)-k(22)*y(356);
dydt(2)=dydt(2)+k(22)*y(356);
dydt(533)=dydt(533)+k(22)*y(356);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(356)=dydt(356)-k(22)*y(356);
dydt(32)=dydt(32)+k(22)*y(356);
dydt(354)=dydt(354)+k(22)*y(356);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(356);
dydt(356)=dydt(356)-k(15)*y(3)*y(356);
dydt(530)=dydt(530)+k(15)*y(3)*y(356);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(356);
dydt(356)=dydt(356)-k(9)*y(3)*y(356);
dydt(355)=dydt(355)+k(9)*y(3)*y(356);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(356)=dydt(356)-k(8)*y(356);
dydt(111)=dydt(111)+k(8)*y(356);
dydt(28)=dydt(28)+k(8)*y(356);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(356)=dydt(356)-k(6)*y(356);
dydt(27)=dydt(27)+k(6)*y(356);
dydt(110)=dydt(110)+k(6)*y(356);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(357)=dydt(357)-k(33)*y(357);
dydt(604)=dydt(604)+k(33)*y(357);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(357)=dydt(357)-k(28)*y(357);
dydt(360)=dydt(360)+k(28)*y(357);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(357)=dydt(357)-k(27)*y(357);
dydt(526)=dydt(526)+k(27)*y(357);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(357)=dydt(357)-k(24)*y(357);
dydt(602)=dydt(602)+k(24)*y(357);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

dydt(357)=dydt(357)-k(22)*y(357);
dydt(2)=dydt(2)+k(22)*y(357);
dydt(134)=dydt(134)+k(22)*y(357);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(357)=dydt(357)-k(22)*y(357);
dydt(32)=dydt(32)+k(22)*y(357);
dydt(116)=dydt(116)+k(22)*y(357);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(357)=dydt(357)-k(16)*y(357);
dydt(3)=dydt(3)+k(16)*y(357);
dydt(355)=dydt(355)+k(16)*y(357);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(357)=dydt(357)-k(10)*y(357);
dydt(3)=dydt(3)+k(10)*y(357);
dydt(530)=dydt(530)+k(10)*y(357);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(357)=dydt(357)-k(8)*y(357);
dydt(43)=dydt(43)+k(8)*y(357);
dydt(132)=dydt(132)+k(8)*y(357);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(357)=dydt(357)-k(6)*y(357);
dydt(131)=dydt(131)+k(6)*y(357);
dydt(114)=dydt(114)+k(6)*y(357);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(359)=dydt(359)-k(34)*y(359);
dydt(356)=dydt(356)+k(34)*y(359);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(359)=dydt(359)-k(33)*y(359);
dydt(510)=dydt(510)+k(33)*y(359);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(359)=dydt(359)-k(31)*y(359);
dydt(366)=dydt(366)+k(31)*y(359);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(359)=dydt(359)-k(29)*y(359);
dydt(611)=dydt(611)+k(29)*y(359);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(359)=dydt(359)-k(22)*y(359);
dydt(182)=dydt(182)+k(22)*y(359);
dydt(533)=dydt(533)+k(22)*y(359);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(359)=dydt(359)-k(22)*y(359);
dydt(32)=dydt(32)+k(22)*y(359);
dydt(512)=dydt(512)+k(22)*y(359);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(359);
dydt(359)=dydt(359)-k(15)*y(3)*y(359);
dydt(532)=dydt(532)+k(15)*y(3)*y(359);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(359);
dydt(359)=dydt(359)-k(9)*y(3)*y(359);
dydt(358)=dydt(358)+k(9)*y(3)*y(359);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(359)=dydt(359)-k(8)*y(359);
dydt(218)=dydt(218)+k(8)*y(359);
dydt(28)=dydt(28)+k(8)*y(359);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(359)=dydt(359)-k(6)*y(359);
dydt(27)=dydt(27)+k(6)*y(359);
dydt(264)=dydt(264)+k(6)*y(359);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(360)=dydt(360)-k(34)*y(360);
dydt(357)=dydt(357)+k(34)*y(360);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(360)=dydt(360)-k(33)*y(360);
dydt(596)=dydt(596)+k(33)*y(360);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(360)=dydt(360)-k(29)*y(360);
dydt(597)=dydt(597)+k(29)*y(360);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(360)=dydt(360)-k(24)*y(360);
dydt(600)=dydt(600)+k(24)*y(360);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

dydt(360)=dydt(360)-k(22)*y(360);
dydt(182)=dydt(182)+k(22)*y(360);
dydt(134)=dydt(134)+k(22)*y(360);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(360)=dydt(360)-k(22)*y(360);
dydt(32)=dydt(32)+k(22)*y(360);
dydt(595)=dydt(595)+k(22)*y(360);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(360)=dydt(360)-k(16)*y(360);
dydt(3)=dydt(3)+k(16)*y(360);
dydt(358)=dydt(358)+k(16)*y(360);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(360)=dydt(360)-k(10)*y(360);
dydt(3)=dydt(3)+k(10)*y(360);
dydt(532)=dydt(532)+k(10)*y(360);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(360)=dydt(360)-k(8)*y(360);
dydt(222)=dydt(222)+k(8)*y(360);
dydt(132)=dydt(132)+k(8)*y(360);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(360)=dydt(360)-k(6)*y(360);
dydt(131)=dydt(131)+k(6)*y(360);
dydt(515)=dydt(515)+k(6)*y(360);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(361)=dydt(361)-k(34)*y(361);
dydt(353)=dydt(353)+k(34)*y(361);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(361)=dydt(361)-k(32)*y(361);
dydt(594)=dydt(594)+k(32)*y(361);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(361)=dydt(361)-k(31)*y(361);
dydt(181)=dydt(181)+k(31)*y(361);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(361)=dydt(361)-k(24)*y(361);
dydt(599)=dydt(599)+k(24)*y(361);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(361)=dydt(361)-k(22)*y(361);
dydt(182)=dydt(182)+k(22)*y(361);
dydt(151)=dydt(151)+k(22)*y(361);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(361);
dydt(361)=dydt(361)-k(21)*y(197)*y(361);
dydt(363)=dydt(363)+k(21)*y(197)*y(361);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(361);
dydt(361)=dydt(361)-k(21)*y(32)*y(361);
dydt(358)=dydt(358)+k(21)*y(32)*y(361);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(361);
dydt(361)=dydt(361)-k(21)*y(2)*y(361);
dydt(180)=dydt(180)+k(21)*y(2)*y(361);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(361);
dydt(361)=dydt(361)-k(15)*y(3)*y(361);
dydt(595)=dydt(595)+k(15)*y(3)*y(361);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(361)=dydt(361)-k(10)*y(361);
dydt(3)=dydt(3)+k(10)*y(361);
dydt(512)=dydt(512)+k(10)*y(361);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(361)=dydt(361)-k(8)*y(361);
dydt(126)=dydt(126)+k(8)*y(361);
dydt(218)=dydt(218)+k(8)*y(361);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(361)=dydt(361)-k(6)*y(361);
dydt(48)=dydt(48)+k(6)*y(361);
dydt(264)=dydt(264)+k(6)*y(361);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(362)=dydt(362)-k(34)*y(362);
dydt(161)=dydt(161)+k(34)*y(362);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(362)=dydt(362)-k(33)*y(362);
dydt(358)=dydt(358)+k(33)*y(362);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(362)=dydt(362)-k(33)*y(362);
dydt(605)=dydt(605)+k(33)*y(362);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(362)=dydt(362)-k(31)*y(362);
dydt(163)=dydt(163)+k(31)*y(362);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(362)=dydt(362)-k(29)*y(362);
dydt(342)=dydt(342)+k(29)*y(362);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(362)=dydt(362)-k(22)*y(362);
dydt(197)=dydt(197)+k(22)*y(362);
dydt(137)=dydt(137)+k(22)*y(362);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(362)=dydt(362)-k(22)*y(362);
dydt(32)=dydt(32)+k(22)*y(362);
dydt(599)=dydt(599)+k(22)*y(362);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(362);
dydt(362)=dydt(362)-k(15)*y(3)*y(362);
dydt(600)=dydt(600)+k(15)*y(3)*y(362);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(362)=dydt(362)-k(10)*y(362);
dydt(3)=dydt(3)+k(10)*y(362);
dydt(608)=dydt(608)+k(10)*y(362);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(362)=dydt(362)-k(8)*y(362);
dydt(257)=dydt(257)+k(8)*y(362);
dydt(132)=dydt(132)+k(8)*y(362);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(362)=dydt(362)-k(6)*y(362);
dydt(256)=dydt(256)+k(6)*y(362);
dydt(131)=dydt(131)+k(6)*y(362);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(363)=dydt(363)-k(34)*y(363);
dydt(340)=dydt(340)+k(34)*y(363);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(363)=dydt(363)-k(34)*y(363);
dydt(358)=dydt(358)+k(34)*y(363);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(363)=dydt(363)-k(33)*y(363);
dydt(188)=dydt(188)+k(33)*y(363);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(363)=dydt(363)-k(31)*y(363);
dydt(367)=dydt(367)+k(31)*y(363);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(363)=dydt(363)-k(24)*y(363);
dydt(342)=dydt(342)+k(24)*y(363);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(363)=dydt(363)-k(22)*y(363);
dydt(182)=dydt(182)+k(22)*y(363);
dydt(337)=dydt(337)+k(22)*y(363);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(363)=dydt(363)-k(22)*y(363);
dydt(197)=dydt(197)+k(22)*y(363);
dydt(361)=dydt(361)+k(22)*y(363);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(363);
dydt(363)=dydt(363)-k(15)*y(3)*y(363);
dydt(597)=dydt(597)+k(15)*y(3)*y(363);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(363)=dydt(363)-k(10)*y(363);
dydt(3)=dydt(3)+k(10)*y(363);
dydt(611)=dydt(611)+k(10)*y(363);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(363)=dydt(363)-k(8)*y(363);
dydt(218)=dydt(218)+k(8)*y(363);
dydt(336)=dydt(336)+k(8)*y(363);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(363)=dydt(363)-k(6)*y(363);
dydt(264)=dydt(264)+k(6)*y(363);
dydt(335)=dydt(335)+k(6)*y(363);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(365)=dydt(365)-k(34)*y(365);
dydt(381)=dydt(381)+k(34)*y(365);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(365)=dydt(365)-k(33)*y(365);
dydt(178)=dydt(178)+k(33)*y(365);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(365)=dydt(365)-k(29)*y(365);
dydt(368)=dydt(368)+k(29)*y(365);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(365)=dydt(365)-k(24)*y(365);
dydt(465)=dydt(465)+k(24)*y(365);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(365)=dydt(365)-k(22)*y(365);
dydt(182)=dydt(182)+k(22)*y(365);
dydt(155)=dydt(155)+k(22)*y(365);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(365)=dydt(365)-k(22)*y(365);
dydt(32)=dydt(32)+k(22)*y(365);
dydt(651)=dydt(651)+k(22)*y(365);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(365)=dydt(365)-k(10)*y(365);
dydt(3)=dydt(3)+k(10)*y(365);
dydt(364)=dydt(364)+k(10)*y(365);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(365)=dydt(365)-k(10)*y(365);
dydt(3)=dydt(3)+k(10)*y(365);
dydt(385)=dydt(385)+k(10)*y(365);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(365)=dydt(365)-k(8)*y(365);
dydt(184)=dydt(184)+k(8)*y(365);
dydt(132)=dydt(132)+k(8)*y(365);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(365)=dydt(365)-k(6)*y(365);
dydt(131)=dydt(131)+k(6)*y(365);
dydt(391)=dydt(391)+k(6)*y(365);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(366)=dydt(366)-k(34)*y(366);
dydt(378)=dydt(378)+k(34)*y(366);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(366)=dydt(366)-k(33)*y(366);
dydt(179)=dydt(179)+k(33)*y(366);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(366)=dydt(366)-k(29)*y(366);
dydt(369)=dydt(369)+k(29)*y(366);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(366)=dydt(366)-k(22)*y(366);
dydt(182)=dydt(182)+k(22)*y(366);
dydt(156)=dydt(156)+k(22)*y(366);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(366)=dydt(366)-k(22)*y(366);
dydt(32)=dydt(32)+k(22)*y(366);
dydt(648)=dydt(648)+k(22)*y(366);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(366);
dydt(366)=dydt(366)-k(9)*y(3)*y(366);
dydt(364)=dydt(364)+k(9)*y(3)*y(366);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(366);
dydt(366)=dydt(366)-k(9)*y(3)*y(366);
dydt(385)=dydt(385)+k(9)*y(3)*y(366);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(366)=dydt(366)-k(8)*y(366);
dydt(177)=dydt(177)+k(8)*y(366);
dydt(28)=dydt(28)+k(8)*y(366);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(366)=dydt(366)-k(6)*y(366);
dydt(27)=dydt(27)+k(6)*y(366);
dydt(176)=dydt(176)+k(6)*y(366);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(368)=dydt(368)-k(34)*y(368);
dydt(372)=dydt(372)+k(34)*y(368);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(368)=dydt(368)-k(34)*y(368);
dydt(365)=dydt(365)+k(34)*y(368);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(368)=dydt(368)-k(33)*y(368);
dydt(186)=dydt(186)+k(33)*y(368);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(368)=dydt(368)-k(24)*y(368);
dydt(464)=dydt(464)+k(24)*y(368);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(368)=dydt(368)-k(22)*y(368);
dydt(182)=dydt(182)+k(22)*y(368);
dydt(400)=dydt(400)+k(22)*y(368);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(368)=dydt(368)-k(22)*y(368);
dydt(197)=dydt(197)+k(22)*y(368);
dydt(651)=dydt(651)+k(22)*y(368);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(368)=dydt(368)-k(10)*y(368);
dydt(3)=dydt(3)+k(10)*y(368);
dydt(367)=dydt(367)+k(10)*y(368);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(368)=dydt(368)-k(10)*y(368);
dydt(3)=dydt(3)+k(10)*y(368);
dydt(397)=dydt(397)+k(10)*y(368);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(368)=dydt(368)-k(8)*y(368);
dydt(184)=dydt(184)+k(8)*y(368);
dydt(336)=dydt(336)+k(8)*y(368);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(368)=dydt(368)-k(6)*y(368);
dydt(335)=dydt(335)+k(6)*y(368);
dydt(391)=dydt(391)+k(6)*y(368);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(369)=dydt(369)-k(34)*y(369);
dydt(373)=dydt(373)+k(34)*y(369);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(369)=dydt(369)-k(34)*y(369);
dydt(366)=dydt(366)+k(34)*y(369);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(369)=dydt(369)-k(33)*y(369);
dydt(187)=dydt(187)+k(33)*y(369);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(369)=dydt(369)-k(22)*y(369);
dydt(182)=dydt(182)+k(22)*y(369);
dydt(376)=dydt(376)+k(22)*y(369);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(369)=dydt(369)-k(22)*y(369);
dydt(197)=dydt(197)+k(22)*y(369);
dydt(648)=dydt(648)+k(22)*y(369);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(369);
dydt(369)=dydt(369)-k(9)*y(3)*y(369);
dydt(367)=dydt(367)+k(9)*y(3)*y(369);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(369);
dydt(369)=dydt(369)-k(9)*y(3)*y(369);
dydt(397)=dydt(397)+k(9)*y(3)*y(369);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(369)=dydt(369)-k(8)*y(369);
dydt(177)=dydt(177)+k(8)*y(369);
dydt(209)=dydt(209)+k(8)*y(369);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(369)=dydt(369)-k(6)*y(369);
dydt(192)=dydt(192)+k(6)*y(369);
dydt(176)=dydt(176)+k(6)*y(369);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(370)=dydt(370)-k(34)*y(370);
dydt(162)=dydt(162)+k(34)*y(370);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(370)=dydt(370)-k(34)*y(370);
dydt(163)=dydt(163)+k(34)*y(370);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(370)=dydt(370)-k(33)*y(370);
dydt(367)=dydt(367)+k(33)*y(370);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(370)=dydt(370)-k(33)*y(370);
dydt(190)=dydt(190)+k(33)*y(370);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(370)=dydt(370)-k(22)*y(370);
dydt(197)=dydt(197)+k(22)*y(370);
dydt(198)=dydt(198)+k(22)*y(370);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(370)=dydt(370)-k(22)*y(370);
dydt(197)=dydt(197)+k(22)*y(370);
dydt(196)=dydt(196)+k(22)*y(370);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(370)=dydt(370)-k(12)*y(370);
dydt(342)=dydt(342)+k(12)*y(370);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(370)=dydt(370)-k(10)*y(370);
dydt(3)=dydt(3)+k(10)*y(370);
dydt(455)=dydt(455)+k(10)*y(370);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(370);
dydt(370)=dydt(370)-k(9)*y(3)*y(370);
dydt(464)=dydt(464)+k(9)*y(3)*y(370);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(370)=dydt(370)-k(8)*y(370);
dydt(192)=dydt(192)+k(8)*y(370);
dydt(336)=dydt(336)+k(8)*y(370);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(370)=dydt(370)-k(6)*y(370);
dydt(191)=dydt(191)+k(6)*y(370);
dydt(335)=dydt(335)+k(6)*y(370);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(372)=dydt(372)-k(34)*y(372);
dydt(381)=dydt(381)+k(34)*y(372);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(372)=dydt(372)-k(33)*y(372);
dydt(460)=dydt(460)+k(33)*y(372);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(372)=dydt(372)-k(30)*y(372);
dydt(368)=dydt(368)+k(30)*y(372);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(372)=dydt(372)-k(24)*y(372);
dydt(459)=dydt(459)+k(24)*y(372);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(372)=dydt(372)-k(22)*y(372);
dydt(2)=dydt(2)+k(22)*y(372);
dydt(400)=dydt(400)+k(22)*y(372);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(372)=dydt(372)-k(22)*y(372);
dydt(197)=dydt(197)+k(22)*y(372);
dydt(461)=dydt(461)+k(22)*y(372);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(372)=dydt(372)-k(10)*y(372);
dydt(3)=dydt(3)+k(10)*y(372);
dydt(371)=dydt(371)+k(10)*y(372);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(372)=dydt(372)-k(10)*y(372);
dydt(3)=dydt(3)+k(10)*y(372);
dydt(374)=dydt(374)+k(10)*y(372);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(372)=dydt(372)-k(8)*y(372);
dydt(140)=dydt(140)+k(8)*y(372);
dydt(336)=dydt(336)+k(8)*y(372);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(372)=dydt(372)-k(6)*y(372);
dydt(335)=dydt(335)+k(6)*y(372);
dydt(380)=dydt(380)+k(6)*y(372);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(374)=dydt(374)-k(34)*y(374);
dydt(379)=dydt(379)+k(34)*y(374);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(374)=dydt(374)-k(33)*y(374);
dydt(389)=dydt(389)+k(33)*y(374);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(374)=dydt(374)-k(30)*y(374);
dydt(397)=dydt(397)+k(30)*y(374);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(374)=dydt(374)-k(22)*y(374);
dydt(2)=dydt(2)+k(22)*y(374);
dydt(399)=dydt(399)+k(22)*y(374);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(374)=dydt(374)-k(22)*y(374);
dydt(197)=dydt(197)+k(22)*y(374);
dydt(383)=dydt(383)+k(22)*y(374);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(374)=dydt(374)-k(11)*y(374);
dydt(426)=dydt(426)+k(11)*y(374);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(374)=dydt(374)-k(10)*y(374);
dydt(3)=dydt(3)+k(10)*y(374);
dydt(373)=dydt(373)+k(10)*y(374);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(374);
dydt(374)=dydt(374)-k(9)*y(3)*y(374);
dydt(372)=dydt(372)+k(9)*y(3)*y(374);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(374)=dydt(374)-k(8)*y(374);
dydt(209)=dydt(209)+k(8)*y(374);
dydt(140)=dydt(140)+k(8)*y(374);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(374)=dydt(374)-k(6)*y(374);
dydt(192)=dydt(192)+k(6)*y(374);
dydt(380)=dydt(380)+k(6)*y(374);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(375)=dydt(375)-k(32)*y(375);
dydt(94)=dydt(94)+k(32)*y(375);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(375)=dydt(375)-k(22)*y(375);
dydt(2)=dydt(2)+k(22)*y(375);
dydt(150)=dydt(150)+k(22)*y(375);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(375);
dydt(375)=dydt(375)-k(21)*y(197)*y(375);
dydt(373)=dydt(373)+k(21)*y(197)*y(375);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(375);
dydt(375)=dydt(375)-k(21)*y(32)*y(375);
dydt(378)=dydt(378)+k(21)*y(32)*y(375);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(375);
dydt(375)=dydt(375)-k(21)*y(2)*y(375);
dydt(172)=dydt(172)+k(21)*y(2)*y(375);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(375);
dydt(375)=dydt(375)-k(9)*y(3)*y(375);
dydt(152)=dydt(152)+k(9)*y(3)*y(375);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(375);
dydt(375)=dydt(375)-k(9)*y(3)*y(375);
dydt(383)=dydt(383)+k(9)*y(3)*y(375);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(375)=dydt(375)-k(8)*y(375);
dydt(16)=dydt(16)+k(8)*y(375);
dydt(21)=dydt(21)+k(8)*y(375);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

dydt(375)=dydt(375)-k(6)*y(375);
dydt(15)=dydt(15)+k(6)*y(375);
dydt(89)=dydt(89)+k(6)*y(375);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(376)=dydt(376)-k(34)*y(376);
dydt(156)=dydt(156)+k(34)*y(376);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(376)=dydt(376)-k(33)*y(376);
dydt(451)=dydt(451)+k(33)*y(376);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(376)=dydt(376)-k(32)*y(376);
dydt(211)=dydt(211)+k(32)*y(376);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(376)=dydt(376)-k(22)*y(376);
dydt(197)=dydt(197)+k(22)*y(376);
dydt(150)=dydt(150)+k(22)*y(376);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(376);
dydt(376)=dydt(376)-k(21)*y(197)*y(376);
dydt(455)=dydt(455)+k(21)*y(197)*y(376);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(376);
dydt(376)=dydt(376)-k(21)*y(32)*y(376);
dydt(454)=dydt(454)+k(21)*y(32)*y(376);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(376);
dydt(376)=dydt(376)-k(21)*y(2)*y(376);
dydt(373)=dydt(373)+k(21)*y(2)*y(376);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(376);
dydt(376)=dydt(376)-k(9)*y(3)*y(376);
dydt(198)=dydt(198)+k(9)*y(3)*y(376);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(376);
dydt(376)=dydt(376)-k(9)*y(3)*y(376);
dydt(399)=dydt(399)+k(9)*y(3)*y(376);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

dydt(376)=dydt(376)-k(8)*y(376);
dydt(15)=dydt(15)+k(8)*y(376);
dydt(209)=dydt(209)+k(8)*y(376);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(376)=dydt(376)-k(6)*y(376);
dydt(98)=dydt(98)+k(6)*y(376);
dydt(192)=dydt(192)+k(6)*y(376);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(377)=dydt(377)-k(34)*y(377);
dydt(172)=dydt(172)+k(34)*y(377);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(377)=dydt(377)-k(30)*y(377);
dydt(187)=dydt(187)+k(30)*y(377);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(377)=dydt(377)-k(22)*y(377);
dydt(2)=dydt(2)+k(22)*y(377);
dydt(451)=dydt(451)+k(22)*y(377);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(377)=dydt(377)-k(22)*y(377);
dydt(182)=dydt(182)+k(22)*y(377);
dydt(375)=dydt(375)+k(22)*y(377);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(377);
dydt(377)=dydt(377)-k(9)*y(3)*y(377);
dydt(174)=dydt(174)+k(9)*y(3)*y(377);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(377);
dydt(377)=dydt(377)-k(9)*y(3)*y(377);
dydt(389)=dydt(389)+k(9)*y(3)*y(377);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(377)=dydt(377)-k(8)*y(377);
dydt(21)=dydt(21)+k(8)*y(377);
dydt(214)=dydt(214)+k(8)*y(377);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(377)=dydt(377)-k(6)*y(377);
dydt(177)=dydt(177)+k(6)*y(377);
dydt(89)=dydt(89)+k(6)*y(377);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(380)=dydt(380)-k(22)*y(380);
dydt(2)=dydt(2)+k(22)*y(380);
dydt(47)=dydt(47)+k(22)*y(380);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(380)=dydt(380)-k(10)*y(380);
dydt(3)=dydt(3)+k(10)*y(380);
dydt(89)=dydt(89)+k(10)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(1)=dydt(1)-k(5)*y(1)*y(380);
dydt(380)=dydt(380)-k(5)*y(1)*y(380);
dydt(95)=dydt(95)+k(5)*y(1)*y(380);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(5)*y(7)*y(380);
dydt(380)=dydt(380)-k(5)*y(7)*y(380);
dydt(462)=dydt(462)+k(5)*y(7)*y(380);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) 

dydt(380)=dydt(380)-k(4)*y(380);
dydt(4)=dydt(4)+k(4)*y(380);
dydt(140)=dydt(140)+k(4)*y(380);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(381)=dydt(381)-k(33)*y(381);
dydt(171)=dydt(171)+k(33)*y(381);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(381)=dydt(381)-k(28)*y(381);
dydt(365)=dydt(365)+k(28)*y(381);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(381)=dydt(381)-k(27)*y(381);
dydt(372)=dydt(372)+k(27)*y(381);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(381)=dydt(381)-k(24)*y(381);
dydt(159)=dydt(159)+k(24)*y(381);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(381)=dydt(381)-k(22)*y(381);
dydt(32)=dydt(32)+k(22)*y(381);
dydt(461)=dydt(461)+k(22)*y(381);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(381)=dydt(381)-k(22)*y(381);
dydt(2)=dydt(2)+k(22)*y(381);
dydt(155)=dydt(155)+k(22)*y(381);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(381)=dydt(381)-k(10)*y(381);
dydt(3)=dydt(3)+k(10)*y(381);
dydt(379)=dydt(379)+k(10)*y(381);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(381)=dydt(381)-k(10)*y(381);
dydt(3)=dydt(3)+k(10)*y(381);
dydt(157)=dydt(157)+k(10)*y(381);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(381)=dydt(381)-k(8)*y(381);
dydt(140)=dydt(140)+k(8)*y(381);
dydt(132)=dydt(132)+k(8)*y(381);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(381)=dydt(381)-k(6)*y(381);
dydt(131)=dydt(131)+k(6)*y(381);
dydt(380)=dydt(380)+k(6)*y(381);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(382)=dydt(382)-k(33)*y(382);
dydt(387)=dydt(387)+k(33)*y(382);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(382)=dydt(382)-k(31)*y(382);
dydt(379)=dydt(379)+k(31)*y(382);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(382)=dydt(382)-k(28)*y(382);
dydt(423)=dydt(423)+k(28)*y(382);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(382)=dydt(382)-k(27)*y(382);
dydt(426)=dydt(426)+k(27)*y(382);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(382)=dydt(382)-k(22)*y(382);
dydt(2)=dydt(2)+k(22)*y(382);
dydt(470)=dydt(470)+k(22)*y(382);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(382)=dydt(382)-k(22)*y(382);
dydt(32)=dydt(32)+k(22)*y(382);
dydt(424)=dydt(424)+k(22)*y(382);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(382);
dydt(382)=dydt(382)-k(15)*y(3)*y(382);
dydt(480)=dydt(480)+k(15)*y(3)*y(382);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(382)=dydt(382)-k(10)*y(382);
dydt(3)=dydt(3)+k(10)*y(382);
dydt(482)=dydt(482)+k(10)*y(382);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(382)=dydt(382)-k(8)*y(382);
dydt(34)=dydt(34)+k(8)*y(382);
dydt(140)=dydt(140)+k(8)*y(382);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(382)=dydt(382)-k(6)*y(382);
dydt(33)=dydt(33)+k(6)*y(382);
dydt(380)=dydt(380)+k(6)*y(382);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(383)=dydt(383)-k(32)*y(383);
dydt(95)=dydt(95)+k(32)*y(383);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(383)=dydt(383)-k(22)*y(383);
dydt(2)=dydt(2)+k(22)*y(383);
dydt(404)=dydt(404)+k(22)*y(383);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(383);
dydt(383)=dydt(383)-k(21)*y(197)*y(383);
dydt(374)=dydt(374)+k(21)*y(197)*y(383);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(383);
dydt(383)=dydt(383)-k(21)*y(32)*y(383);
dydt(379)=dydt(379)+k(21)*y(32)*y(383);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(383);
dydt(383)=dydt(383)-k(21)*y(2)*y(383);
dydt(386)=dydt(386)+k(21)*y(2)*y(383);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(383)=dydt(383)-k(11)*y(383);
dydt(424)=dydt(424)+k(11)*y(383);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(383)=dydt(383)-k(10)*y(383);
dydt(3)=dydt(3)+k(10)*y(383);
dydt(375)=dydt(375)+k(10)*y(383);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(383);
dydt(383)=dydt(383)-k(9)*y(3)*y(383);
dydt(461)=dydt(461)+k(9)*y(3)*y(383);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(383)=dydt(383)-k(8)*y(383);
dydt(16)=dydt(16)+k(8)*y(383);
dydt(140)=dydt(140)+k(8)*y(383);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

dydt(383)=dydt(383)-k(6)*y(383);
dydt(15)=dydt(15)+k(6)*y(383);
dydt(380)=dydt(380)+k(6)*y(383);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(384)=dydt(384)-k(33)*y(384);
dydt(388)=dydt(388)+k(33)*y(384);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(384)=dydt(384)-k(32)*y(384);
dydt(26)=dydt(26)+k(32)*y(384);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(384)=dydt(384)-k(22)*y(384);
dydt(32)=dydt(32)+k(22)*y(384);
dydt(404)=dydt(404)+k(22)*y(384);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(384);
dydt(384)=dydt(384)-k(21)*y(197)*y(384);
dydt(412)=dydt(412)+k(21)*y(197)*y(384);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(384);
dydt(384)=dydt(384)-k(21)*y(32)*y(384);
dydt(446)=dydt(446)+k(21)*y(32)*y(384);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(384);
dydt(384)=dydt(384)-k(21)*y(2)*y(384);
dydt(379)=dydt(379)+k(21)*y(2)*y(384);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(384)=dydt(384)-k(11)*y(384);
dydt(470)=dydt(470)+k(11)*y(384);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(384)=dydt(384)-k(10)*y(384);
dydt(3)=dydt(3)+k(10)*y(384);
dydt(156)=dydt(156)+k(10)*y(384);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(384);
dydt(384)=dydt(384)-k(9)*y(3)*y(384);
dydt(155)=dydt(155)+k(9)*y(3)*y(384);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(384)=dydt(384)-k(8)*y(384);
dydt(48)=dydt(48)+k(8)*y(384);
dydt(28)=dydt(28)+k(8)*y(384);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(384)=dydt(384)-k(6)*y(384);
dydt(47)=dydt(47)+k(6)*y(384);
dydt(27)=dydt(27)+k(6)*y(384);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(385)=dydt(385)-k(34)*y(385);
dydt(379)=dydt(379)+k(34)*y(385);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(385)=dydt(385)-k(33)*y(385);
dydt(390)=dydt(390)+k(33)*y(385);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(385)=dydt(385)-k(29)*y(385);
dydt(397)=dydt(397)+k(29)*y(385);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(385)=dydt(385)-k(22)*y(385);
dydt(182)=dydt(182)+k(22)*y(385);
dydt(384)=dydt(384)+k(22)*y(385);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(385)=dydt(385)-k(22)*y(385);
dydt(32)=dydt(32)+k(22)*y(385);
dydt(393)=dydt(393)+k(22)*y(385);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(385)=dydt(385)-k(11)*y(385);
dydt(423)=dydt(423)+k(11)*y(385);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(385)=dydt(385)-k(10)*y(385);
dydt(3)=dydt(3)+k(10)*y(385);
dydt(366)=dydt(366)+k(10)*y(385);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(385);
dydt(385)=dydt(385)-k(9)*y(3)*y(385);
dydt(365)=dydt(365)+k(9)*y(3)*y(385);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(385)=dydt(385)-k(8)*y(385);
dydt(28)=dydt(28)+k(8)*y(385);
dydt(184)=dydt(184)+k(8)*y(385);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(385)=dydt(385)-k(6)*y(385);
dydt(27)=dydt(27)+k(6)*y(385);
dydt(391)=dydt(391)+k(6)*y(385);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(387)=dydt(387)-k(31)*y(387);
dydt(386)=dydt(386)+k(31)*y(387);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(387)=dydt(387)-k(28)*y(387);
dydt(392)=dydt(392)+k(28)*y(387);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(387)=dydt(387)-k(27)*y(387);
dydt(429)=dydt(429)+k(27)*y(387);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(387)=dydt(387)-k(23)*y(387);
dydt(382)=dydt(382)+k(23)*y(387);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(387)=dydt(387)-k(22)*y(387);
dydt(2)=dydt(2)+k(22)*y(387);
dydt(50)=dydt(50)+k(22)*y(387);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(387)=dydt(387)-k(22)*y(387);
dydt(2)=dydt(2)+k(22)*y(387);
dydt(424)=dydt(424)+k(22)*y(387);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(387);
dydt(387)=dydt(387)-k(15)*y(3)*y(387);
dydt(51)=dydt(51)+k(15)*y(3)*y(387);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(387)=dydt(387)-k(10)*y(387);
dydt(3)=dydt(3)+k(10)*y(387);
dydt(275)=dydt(275)+k(10)*y(387);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(387)=dydt(387)-k(8)*y(387);
dydt(437)=dydt(437)+k(8)*y(387);
dydt(140)=dydt(140)+k(8)*y(387);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(387)=dydt(387)-k(6)*y(387);
dydt(111)=dydt(111)+k(6)*y(387);
dydt(380)=dydt(380)+k(6)*y(387);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(388)=dydt(388)-k(32)*y(388);
dydt(20)=dydt(20)+k(32)*y(388);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(388)=dydt(388)-k(23)*y(388);
dydt(384)=dydt(384)+k(23)*y(388);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(388)=dydt(388)-k(22)*y(388);
dydt(2)=dydt(2)+k(22)*y(388);
dydt(404)=dydt(404)+k(22)*y(388);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(388);
dydt(388)=dydt(388)-k(21)*y(197)*y(388);
dydt(411)=dydt(411)+k(21)*y(197)*y(388);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(388);
dydt(388)=dydt(388)-k(21)*y(32)*y(388);
dydt(445)=dydt(445)+k(21)*y(32)*y(388);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(388);
dydt(388)=dydt(388)-k(21)*y(2)*y(388);
dydt(386)=dydt(386)+k(21)*y(2)*y(388);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(388)=dydt(388)-k(11)*y(388);
dydt(50)=dydt(50)+k(11)*y(388);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(388)=dydt(388)-k(10)*y(388);
dydt(3)=dydt(3)+k(10)*y(388);
dydt(452)=dydt(452)+k(10)*y(388);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(388);
dydt(388)=dydt(388)-k(9)*y(3)*y(388);
dydt(467)=dydt(467)+k(9)*y(3)*y(388);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(388)=dydt(388)-k(8)*y(388);
dydt(48)=dydt(48)+k(8)*y(388);
dydt(22)=dydt(22)+k(8)*y(388);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(388)=dydt(388)-k(6)*y(388);
dydt(47)=dydt(47)+k(6)*y(388);
dydt(21)=dydt(21)+k(6)*y(388);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(389)=dydt(389)-k(34)*y(389);
dydt(386)=dydt(386)+k(34)*y(389);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(389)=dydt(389)-k(30)*y(389);
dydt(394)=dydt(394)+k(30)*y(389);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(389)=dydt(389)-k(23)*y(389);
dydt(374)=dydt(374)+k(23)*y(389);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(389)=dydt(389)-k(22)*y(389);
dydt(2)=dydt(2)+k(22)*y(389);
dydt(396)=dydt(396)+k(22)*y(389);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(389)=dydt(389)-k(22)*y(389);
dydt(182)=dydt(182)+k(22)*y(389);
dydt(383)=dydt(383)+k(22)*y(389);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(389)=dydt(389)-k(11)*y(389);
dydt(429)=dydt(429)+k(11)*y(389);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(389)=dydt(389)-k(10)*y(389);
dydt(3)=dydt(3)+k(10)*y(389);
dydt(377)=dydt(377)+k(10)*y(389);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(389);
dydt(389)=dydt(389)-k(9)*y(3)*y(389);
dydt(460)=dydt(460)+k(9)*y(3)*y(389);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(389)=dydt(389)-k(8)*y(389);
dydt(214)=dydt(214)+k(8)*y(389);
dydt(140)=dydt(140)+k(8)*y(389);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(389)=dydt(389)-k(6)*y(389);
dydt(177)=dydt(177)+k(6)*y(389);
dydt(380)=dydt(380)+k(6)*y(389);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(391)=dydt(391)-k(34)*y(391);
dydt(380)=dydt(380)+k(34)*y(391);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(391)=dydt(391)-k(22)*y(391);
dydt(182)=dydt(182)+k(22)*y(391);
dydt(47)=dydt(47)+k(22)*y(391);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(391)=dydt(391)-k(10)*y(391);
dydt(3)=dydt(3)+k(10)*y(391);
dydt(176)=dydt(176)+k(10)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(1)=dydt(1)-k(5)*y(1)*y(391);
dydt(391)=dydt(391)-k(5)*y(1)*y(391);
dydt(416)=dydt(416)+k(5)*y(1)*y(391);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(7)=dydt(7)-k(5)*y(7)*y(391);
dydt(391)=dydt(391)-k(5)*y(7)*y(391);
dydt(417)=dydt(417)+k(5)*y(7)*y(391);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) 

dydt(391)=dydt(391)-k(4)*y(391);
dydt(4)=dydt(4)+k(4)*y(391);
dydt(184)=dydt(184)+k(4)*y(391);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(392)=dydt(392)-k(34)*y(392);
dydt(387)=dydt(387)+k(34)*y(392);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(392)=dydt(392)-k(31)*y(392);
dydt(390)=dydt(390)+k(31)*y(392);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(392)=dydt(392)-k(29)*y(392);
dydt(395)=dydt(395)+k(29)*y(392);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(392)=dydt(392)-k(23)*y(392);
dydt(423)=dydt(423)+k(23)*y(392);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(392)=dydt(392)-k(22)*y(392);
dydt(182)=dydt(182)+k(22)*y(392);
dydt(50)=dydt(50)+k(22)*y(392);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(392)=dydt(392)-k(22)*y(392);
dydt(2)=dydt(2)+k(22)*y(392);
dydt(420)=dydt(420)+k(22)*y(392);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(392);
dydt(392)=dydt(392)-k(15)*y(3)*y(392);
dydt(270)=dydt(270)+k(15)*y(3)*y(392);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(392)=dydt(392)-k(10)*y(392);
dydt(3)=dydt(3)+k(10)*y(392);
dydt(271)=dydt(271)+k(10)*y(392);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(392)=dydt(392)-k(8)*y(392);
dydt(437)=dydt(437)+k(8)*y(392);
dydt(184)=dydt(184)+k(8)*y(392);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(392)=dydt(392)-k(6)*y(392);
dydt(111)=dydt(111)+k(6)*y(392);
dydt(391)=dydt(391)+k(6)*y(392);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(393)=dydt(393)-k(34)*y(393);
dydt(383)=dydt(383)+k(34)*y(393);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(393)=dydt(393)-k(32)*y(393);
dydt(416)=dydt(416)+k(32)*y(393);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(393)=dydt(393)-k(22)*y(393);
dydt(182)=dydt(182)+k(22)*y(393);
dydt(404)=dydt(404)+k(22)*y(393);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(393);
dydt(393)=dydt(393)-k(21)*y(197)*y(393);
dydt(397)=dydt(397)+k(21)*y(197)*y(393);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(393);
dydt(393)=dydt(393)-k(21)*y(32)*y(393);
dydt(385)=dydt(385)+k(21)*y(32)*y(393);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(393);
dydt(393)=dydt(393)-k(21)*y(2)*y(393);
dydt(390)=dydt(390)+k(21)*y(2)*y(393);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(393)=dydt(393)-k(11)*y(393);
dydt(420)=dydt(420)+k(11)*y(393);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(393)=dydt(393)-k(10)*y(393);
dydt(3)=dydt(3)+k(10)*y(393);
dydt(648)=dydt(648)+k(10)*y(393);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(393);
dydt(393)=dydt(393)-k(9)*y(3)*y(393);
dydt(651)=dydt(651)+k(9)*y(3)*y(393);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(393)=dydt(393)-k(8)*y(393);
dydt(16)=dydt(16)+k(8)*y(393);
dydt(184)=dydt(184)+k(8)*y(393);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

dydt(393)=dydt(393)-k(6)*y(393);
dydt(15)=dydt(15)+k(6)*y(393);
dydt(391)=dydt(391)+k(6)*y(393);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(395)=dydt(395)-k(34)*y(395);
dydt(429)=dydt(429)+k(34)*y(395);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(395)=dydt(395)-k(34)*y(395);
dydt(392)=dydt(392)+k(34)*y(395);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(395)=dydt(395)-k(31)*y(395);
dydt(394)=dydt(394)+k(31)*y(395);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(395)=dydt(395)-k(23)*y(395);
dydt(398)=dydt(398)+k(23)*y(395);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(395)=dydt(395)-k(22)*y(395);
dydt(182)=dydt(182)+k(22)*y(395);
dydt(430)=dydt(430)+k(22)*y(395);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(395)=dydt(395)-k(22)*y(395);
dydt(182)=dydt(182)+k(22)*y(395);
dydt(420)=dydt(420)+k(22)*y(395);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(395);
dydt(395)=dydt(395)-k(15)*y(3)*y(395);
dydt(649)=dydt(649)+k(15)*y(3)*y(395);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(395)=dydt(395)-k(10)*y(395);
dydt(3)=dydt(3)+k(10)*y(395);
dydt(580)=dydt(580)+k(10)*y(395);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(395)=dydt(395)-k(8)*y(395);
dydt(219)=dydt(219)+k(8)*y(395);
dydt(184)=dydt(184)+k(8)*y(395);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(395)=dydt(395)-k(6)*y(395);
dydt(218)=dydt(218)+k(6)*y(395);
dydt(391)=dydt(391)+k(6)*y(395);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(396)=dydt(396)-k(34)*y(396);
dydt(388)=dydt(388)+k(34)*y(396);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(396)=dydt(396)-k(32)*y(396);
dydt(213)=dydt(213)+k(32)*y(396);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(396)=dydt(396)-k(23)*y(396);
dydt(399)=dydt(399)+k(23)*y(396);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(396)=dydt(396)-k(22)*y(396);
dydt(182)=dydt(182)+k(22)*y(396);
dydt(404)=dydt(404)+k(22)*y(396);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(396);
dydt(396)=dydt(396)-k(21)*y(197)*y(396);
dydt(435)=dydt(435)+k(21)*y(197)*y(396);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(396);
dydt(396)=dydt(396)-k(21)*y(32)*y(396);
dydt(447)=dydt(447)+k(21)*y(32)*y(396);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(396);
dydt(396)=dydt(396)-k(21)*y(2)*y(396);
dydt(389)=dydt(389)+k(21)*y(2)*y(396);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(396)=dydt(396)-k(11)*y(396);
dydt(430)=dydt(430)+k(11)*y(396);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(396)=dydt(396)-k(10)*y(396);
dydt(3)=dydt(3)+k(10)*y(396);
dydt(451)=dydt(451)+k(10)*y(396);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(396);
dydt(396)=dydt(396)-k(9)*y(3)*y(396);
dydt(457)=dydt(457)+k(9)*y(3)*y(396);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(396)=dydt(396)-k(8)*y(396);
dydt(48)=dydt(48)+k(8)*y(396);
dydt(214)=dydt(214)+k(8)*y(396);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(396)=dydt(396)-k(6)*y(396);
dydt(47)=dydt(47)+k(6)*y(396);
dydt(177)=dydt(177)+k(6)*y(396);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(398)=dydt(398)-k(34)*y(398);
dydt(426)=dydt(426)+k(34)*y(398);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(398)=dydt(398)-k(34)*y(398);
dydt(423)=dydt(423)+k(34)*y(398);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(398)=dydt(398)-k(33)*y(398);
dydt(395)=dydt(395)+k(33)*y(398);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(398)=dydt(398)-k(31)*y(398);
dydt(397)=dydt(397)+k(31)*y(398);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(398)=dydt(398)-k(22)*y(398);
dydt(182)=dydt(182)+k(22)*y(398);
dydt(401)=dydt(401)+k(22)*y(398);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(398)=dydt(398)-k(22)*y(398);
dydt(197)=dydt(197)+k(22)*y(398);
dydt(420)=dydt(420)+k(22)*y(398);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(398);
dydt(398)=dydt(398)-k(15)*y(3)*y(398);
dydt(476)=dydt(476)+k(15)*y(3)*y(398);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(398)=dydt(398)-k(10)*y(398);
dydt(3)=dydt(3)+k(10)*y(398);
dydt(487)=dydt(487)+k(10)*y(398);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(398)=dydt(398)-k(8)*y(398);
dydt(319)=dydt(319)+k(8)*y(398);
dydt(184)=dydt(184)+k(8)*y(398);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(398)=dydt(398)-k(6)*y(398);
dydt(257)=dydt(257)+k(6)*y(398);
dydt(391)=dydt(391)+k(6)*y(398);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(400)=dydt(400)-k(34)*y(400);
dydt(155)=dydt(155)+k(34)*y(400);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(400)=dydt(400)-k(33)*y(400);
dydt(457)=dydt(457)+k(33)*y(400);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(400)=dydt(400)-k(32)*y(400);
dydt(210)=dydt(210)+k(32)*y(400);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(400)=dydt(400)-k(22)*y(400);
dydt(197)=dydt(197)+k(22)*y(400);
dydt(149)=dydt(149)+k(22)*y(400);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(400);
dydt(400)=dydt(400)-k(21)*y(197)*y(400);
dydt(464)=dydt(464)+k(21)*y(197)*y(400);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(400);
dydt(400)=dydt(400)-k(21)*y(32)*y(400);
dydt(459)=dydt(459)+k(21)*y(32)*y(400);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(400);
dydt(400)=dydt(400)-k(21)*y(2)*y(400);
dydt(372)=dydt(372)+k(21)*y(2)*y(400);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(400)=dydt(400)-k(10)*y(400);
dydt(3)=dydt(3)+k(10)*y(400);
dydt(399)=dydt(399)+k(10)*y(400);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(400)=dydt(400)-k(10)*y(400);
dydt(3)=dydt(3)+k(10)*y(400);
dydt(198)=dydt(198)+k(10)*y(400);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(400)=dydt(400)-k(8)*y(400);
dydt(48)=dydt(48)+k(8)*y(400);
dydt(336)=dydt(336)+k(8)*y(400);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(400)=dydt(400)-k(6)*y(400);
dydt(47)=dydt(47)+k(6)*y(400);
dydt(335)=dydt(335)+k(6)*y(400);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(401)=dydt(401)-k(34)*y(401);
dydt(470)=dydt(470)+k(34)*y(401);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(401)=dydt(401)-k(33)*y(401);
dydt(430)=dydt(430)+k(33)*y(401);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(401)=dydt(401)-k(32)*y(401);
dydt(212)=dydt(212)+k(32)*y(401);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(401)=dydt(401)-k(31)*y(401);
dydt(399)=dydt(399)+k(31)*y(401);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(401)=dydt(401)-k(22)*y(401);
dydt(197)=dydt(197)+k(22)*y(401);
dydt(57)=dydt(57)+k(22)*y(401);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(401);
dydt(401)=dydt(401)-k(21)*y(197)*y(401);
dydt(434)=dydt(434)+k(21)*y(197)*y(401);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(401);
dydt(401)=dydt(401)-k(21)*y(32)*y(401);
dydt(471)=dydt(471)+k(21)*y(32)*y(401);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(401);
dydt(401)=dydt(401)-k(21)*y(2)*y(401);
dydt(426)=dydt(426)+k(21)*y(2)*y(401);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(401);
dydt(401)=dydt(401)-k(15)*y(3)*y(401);
dydt(205)=dydt(205)+k(15)*y(3)*y(401);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(401)=dydt(401)-k(10)*y(401);
dydt(3)=dydt(3)+k(10)*y(401);
dydt(618)=dydt(618)+k(10)*y(401);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(401)=dydt(401)-k(8)*y(401);
dydt(48)=dydt(48)+k(8)*y(401);
dydt(319)=dydt(319)+k(8)*y(401);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(401)=dydt(401)-k(6)*y(401);
dydt(47)=dydt(47)+k(6)*y(401);
dydt(257)=dydt(257)+k(6)*y(401);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(402)=dydt(402)-k(34)*y(402);
dydt(446)=dydt(446)+k(34)*y(402);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(402)=dydt(402)-k(33)*y(402);
dydt(374)=dydt(374)+k(33)*y(402);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(402)=dydt(402)-k(33)*y(402);
dydt(447)=dydt(447)+k(33)*y(402);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(402)=dydt(402)-k(30)*y(402);
dydt(403)=dydt(403)+k(30)*y(402);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(402)=dydt(402)-k(22)*y(402);
dydt(32)=dydt(32)+k(22)*y(402);
dydt(399)=dydt(399)+k(22)*y(402);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(402)=dydt(402)-k(22)*y(402);
dydt(197)=dydt(197)+k(22)*y(402);
dydt(405)=dydt(405)+k(22)*y(402);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(402)=dydt(402)-k(11)*y(402);
dydt(471)=dydt(471)+k(11)*y(402);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(402)=dydt(402)-k(10)*y(402);
dydt(3)=dydt(3)+k(10)*y(402);
dydt(454)=dydt(454)+k(10)*y(402);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(402);
dydt(402)=dydt(402)-k(9)*y(3)*y(402);
dydt(459)=dydt(459)+k(9)*y(3)*y(402);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(402)=dydt(402)-k(8)*y(402);
dydt(209)=dydt(209)+k(8)*y(402);
dydt(131)=dydt(131)+k(8)*y(402);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(402)=dydt(402)-k(6)*y(402);
dydt(192)=dydt(192)+k(6)*y(402);
dydt(441)=dydt(441)+k(6)*y(402);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(403)=dydt(403)-k(34)*y(403);
dydt(402)=dydt(402)+k(34)*y(403);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(403)=dydt(403)-k(34)*y(403);
dydt(412)=dydt(412)+k(34)*y(403);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(403)=dydt(403)-k(33)*y(403);
dydt(397)=dydt(397)+k(33)*y(403);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(403)=dydt(403)-k(33)*y(403);
dydt(435)=dydt(435)+k(33)*y(403);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(403)=dydt(403)-k(22)*y(403);
dydt(197)=dydt(197)+k(22)*y(403);
dydt(399)=dydt(399)+k(22)*y(403);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(403)=dydt(403)-k(22)*y(403);
dydt(197)=dydt(197)+k(22)*y(403);
dydt(406)=dydt(406)+k(22)*y(403);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(403)=dydt(403)-k(11)*y(403);
dydt(434)=dydt(434)+k(11)*y(403);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(403)=dydt(403)-k(10)*y(403);
dydt(3)=dydt(3)+k(10)*y(403);
dydt(455)=dydt(455)+k(10)*y(403);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(403);
dydt(403)=dydt(403)-k(9)*y(3)*y(403);
dydt(464)=dydt(464)+k(9)*y(3)*y(403);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(403)=dydt(403)-k(8)*y(403);
dydt(209)=dydt(209)+k(8)*y(403);
dydt(335)=dydt(335)+k(8)*y(403);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(403)=dydt(403)-k(6)*y(403);
dydt(192)=dydt(192)+k(6)*y(403);
dydt(407)=dydt(407)+k(6)*y(403);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(405)=dydt(405)-k(33)*y(405);
dydt(383)=dydt(383)+k(33)*y(405);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(405)=dydt(405)-k(32)*y(405);
dydt(102)=dydt(102)+k(32)*y(405);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(405)=dydt(405)-k(22)*y(405);
dydt(32)=dydt(32)+k(22)*y(405);
dydt(404)=dydt(404)+k(22)*y(405);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(405);
dydt(405)=dydt(405)-k(21)*y(197)*y(405);
dydt(402)=dydt(402)+k(21)*y(197)*y(405);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(405);
dydt(405)=dydt(405)-k(21)*y(32)*y(405);
dydt(446)=dydt(446)+k(21)*y(32)*y(405);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(405);
dydt(405)=dydt(405)-k(21)*y(2)*y(405);
dydt(445)=dydt(445)+k(21)*y(2)*y(405);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(405)=dydt(405)-k(11)*y(405);
dydt(443)=dydt(443)+k(11)*y(405);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(405)=dydt(405)-k(10)*y(405);
dydt(3)=dydt(3)+k(10)*y(405);
dydt(450)=dydt(450)+k(10)*y(405);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(405);
dydt(405)=dydt(405)-k(9)*y(3)*y(405);
dydt(458)=dydt(458)+k(9)*y(3)*y(405);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(405)=dydt(405)-k(8)*y(405);
dydt(16)=dydt(16)+k(8)*y(405);
dydt(131)=dydt(131)+k(8)*y(405);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

dydt(405)=dydt(405)-k(6)*y(405);
dydt(15)=dydt(15)+k(6)*y(405);
dydt(441)=dydt(441)+k(6)*y(405);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(407)=dydt(407)-k(34)*y(407);
dydt(441)=dydt(441)+k(34)*y(407);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(407)=dydt(407)-k(33)*y(407);
dydt(391)=dydt(391)+k(33)*y(407);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(407)=dydt(407)-k(22)*y(407);
dydt(197)=dydt(197)+k(22)*y(407);
dydt(47)=dydt(47)+k(22)*y(407);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(407)=dydt(407)-k(10)*y(407);
dydt(3)=dydt(3)+k(10)*y(407);
dydt(191)=dydt(191)+k(10)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(1)=dydt(1)-k(5)*y(1)*y(407);
dydt(407)=dydt(407)-k(5)*y(1)*y(407);
dydt(413)=dydt(413)+k(5)*y(1)*y(407);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(7)=dydt(7)-k(5)*y(7)*y(407);
dydt(407)=dydt(407)-k(5)*y(7)*y(407);
dydt(414)=dydt(414)+k(5)*y(7)*y(407);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) 

dydt(407)=dydt(407)-k(4)*y(407);
dydt(4)=dydt(4)+k(4)*y(407);
dydt(335)=dydt(335)+k(4)*y(407);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(408)=dydt(408)-k(34)*y(408);
dydt(458)=dydt(458)+k(34)*y(408);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(408)=dydt(408)-k(33)*y(408);
dydt(651)=dydt(651)+k(33)*y(408);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(408)=dydt(408)-k(32)*y(408);
dydt(414)=dydt(414)+k(32)*y(408);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(408)=dydt(408)-k(22)*y(408);
dydt(197)=dydt(197)+k(22)*y(408);
dydt(149)=dydt(149)+k(22)*y(408);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(408);
dydt(408)=dydt(408)-k(21)*y(197)*y(408);
dydt(464)=dydt(464)+k(21)*y(197)*y(408);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(408);
dydt(408)=dydt(408)-k(21)*y(32)*y(408);
dydt(465)=dydt(465)+k(21)*y(32)*y(408);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(408);
dydt(408)=dydt(408)-k(21)*y(2)*y(408);
dydt(466)=dydt(466)+k(21)*y(2)*y(408);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(408)=dydt(408)-k(10)*y(408);
dydt(3)=dydt(3)+k(10)*y(408);
dydt(406)=dydt(406)+k(10)*y(408);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(408)=dydt(408)-k(10)*y(408);
dydt(3)=dydt(3)+k(10)*y(408);
dydt(196)=dydt(196)+k(10)*y(408);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(408)=dydt(408)-k(8)*y(408);
dydt(126)=dydt(126)+k(8)*y(408);
dydt(335)=dydt(335)+k(8)*y(408);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(408)=dydt(408)-k(6)*y(408);
dydt(48)=dydt(48)+k(6)*y(408);
dydt(407)=dydt(407)+k(6)*y(408);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(409)=dydt(409)-k(34)*y(409);
dydt(450)=dydt(450)+k(34)*y(409);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(409)=dydt(409)-k(33)*y(409);
dydt(648)=dydt(648)+k(33)*y(409);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(409)=dydt(409)-k(32)*y(409);
dydt(202)=dydt(202)+k(32)*y(409);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(409)=dydt(409)-k(22)*y(409);
dydt(197)=dydt(197)+k(22)*y(409);
dydt(150)=dydt(150)+k(22)*y(409);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(409);
dydt(409)=dydt(409)-k(21)*y(197)*y(409);
dydt(455)=dydt(455)+k(21)*y(197)*y(409);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(409);
dydt(409)=dydt(409)-k(21)*y(32)*y(409);
dydt(456)=dydt(456)+k(21)*y(32)*y(409);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(409);
dydt(409)=dydt(409)-k(21)*y(2)*y(409);
dydt(453)=dydt(453)+k(21)*y(2)*y(409);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(409);
dydt(409)=dydt(409)-k(9)*y(3)*y(409);
dydt(196)=dydt(196)+k(9)*y(3)*y(409);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(409);
dydt(409)=dydt(409)-k(9)*y(3)*y(409);
dydt(406)=dydt(406)+k(9)*y(3)*y(409);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(409)=dydt(409)-k(8)*y(409);
dydt(16)=dydt(16)+k(8)*y(409);
dydt(192)=dydt(192)+k(8)*y(409);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

dydt(409)=dydt(409)-k(6)*y(409);
dydt(15)=dydt(15)+k(6)*y(409);
dydt(191)=dydt(191)+k(6)*y(409);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(410)=dydt(410)-k(34)*y(410);
dydt(443)=dydt(443)+k(34)*y(410);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(410)=dydt(410)-k(33)*y(410);
dydt(420)=dydt(420)+k(33)*y(410);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(410)=dydt(410)-k(32)*y(410);
dydt(415)=dydt(415)+k(32)*y(410);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(410)=dydt(410)-k(31)*y(410);
dydt(406)=dydt(406)+k(31)*y(410);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(410)=dydt(410)-k(22)*y(410);
dydt(197)=dydt(197)+k(22)*y(410);
dydt(57)=dydt(57)+k(22)*y(410);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(410);
dydt(410)=dydt(410)-k(21)*y(197)*y(410);
dydt(434)=dydt(434)+k(21)*y(197)*y(410);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(410);
dydt(410)=dydt(410)-k(21)*y(32)*y(410);
dydt(439)=dydt(439)+k(21)*y(32)*y(410);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(410);
dydt(410)=dydt(410)-k(21)*y(2)*y(410);
dydt(436)=dydt(436)+k(21)*y(2)*y(410);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(410);
dydt(410)=dydt(410)-k(15)*y(3)*y(410);
dydt(206)=dydt(206)+k(15)*y(3)*y(410);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(410)=dydt(410)-k(10)*y(410);
dydt(3)=dydt(3)+k(10)*y(410);
dydt(292)=dydt(292)+k(10)*y(410);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(410)=dydt(410)-k(8)*y(410);
dydt(315)=dydt(315)+k(8)*y(410);
dydt(335)=dydt(335)+k(8)*y(410);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

dydt(410)=dydt(410)-k(6)*y(410);
dydt(77)=dydt(77)+k(6)*y(410);
dydt(407)=dydt(407)+k(6)*y(410);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(411)=dydt(411)-k(34)*y(411);
dydt(445)=dydt(445)+k(34)*y(411);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(411)=dydt(411)-k(33)*y(411);
dydt(390)=dydt(390)+k(33)*y(411);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(411)=dydt(411)-k(29)*y(411);
dydt(435)=dydt(435)+k(29)*y(411);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(411)=dydt(411)-k(23)*y(411);
dydt(412)=dydt(412)+k(23)*y(411);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(411)=dydt(411)-k(22)*y(411);
dydt(2)=dydt(2)+k(22)*y(411);
dydt(406)=dydt(406)+k(22)*y(411);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(411)=dydt(411)-k(22)*y(411);
dydt(197)=dydt(197)+k(22)*y(411);
dydt(388)=dydt(388)+k(22)*y(411);

% rule    : R4
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(411)=dydt(411)-k(11)*y(411);
dydt(436)=dydt(436)+k(11)*y(411);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(411)=dydt(411)-k(10)*y(411);
dydt(3)=dydt(3)+k(10)*y(411);
dydt(453)=dydt(453)+k(10)*y(411);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(411);
dydt(411)=dydt(411)-k(9)*y(3)*y(411);
dydt(466)=dydt(466)+k(9)*y(3)*y(411);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(411)=dydt(411)-k(8)*y(411);
dydt(22)=dydt(22)+k(8)*y(411);
dydt(335)=dydt(335)+k(8)*y(411);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(411)=dydt(411)-k(6)*y(411);
dydt(21)=dydt(21)+k(6)*y(411);
dydt(407)=dydt(407)+k(6)*y(411);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(412)=dydt(412)-k(34)*y(412);
dydt(446)=dydt(446)+k(34)*y(412);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(412)=dydt(412)-k(33)*y(412);
dydt(411)=dydt(411)+k(33)*y(412);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(412)=dydt(412)-k(33)*y(412);
dydt(385)=dydt(385)+k(33)*y(412);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(412)=dydt(412)-k(29)*y(412);
dydt(403)=dydt(403)+k(29)*y(412);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(412)=dydt(412)-k(22)*y(412);
dydt(32)=dydt(32)+k(22)*y(412);
dydt(406)=dydt(406)+k(22)*y(412);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(412)=dydt(412)-k(22)*y(412);
dydt(197)=dydt(197)+k(22)*y(412);
dydt(384)=dydt(384)+k(22)*y(412);

% rule    : R4
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(412)=dydt(412)-k(11)*y(412);
dydt(439)=dydt(439)+k(11)*y(412);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(412)=dydt(412)-k(10)*y(412);
dydt(3)=dydt(3)+k(10)*y(412);
dydt(456)=dydt(456)+k(10)*y(412);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(412);
dydt(412)=dydt(412)-k(9)*y(3)*y(412);
dydt(465)=dydt(465)+k(9)*y(3)*y(412);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(412)=dydt(412)-k(8)*y(412);
dydt(28)=dydt(28)+k(8)*y(412);
dydt(335)=dydt(335)+k(8)*y(412);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(412)=dydt(412)-k(6)*y(412);
dydt(27)=dydt(27)+k(6)*y(412);
dydt(407)=dydt(407)+k(6)*y(412);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(414)=dydt(414)-k(34)*y(414);
dydt(104)=dydt(104)+k(34)*y(414);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(414)=dydt(414)-k(33)*y(414);
dydt(417)=dydt(417)+k(33)*y(414);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(414)=dydt(414)-k(22)*y(414);
dydt(197)=dydt(197)+k(22)*y(414);
dydt(463)=dydt(463)+k(22)*y(414);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(414)=dydt(414)-k(13)*y(414);
dydt(408)=dydt(408)+k(13)*y(414);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(414)=dydt(414)-k(10)*y(414);
dydt(3)=dydt(3)+k(10)*y(414);
dydt(413)=dydt(413)+k(10)*y(414);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(414)=dydt(414)-k(10)*y(414);
dydt(3)=dydt(3)+k(10)*y(414);
dydt(607)=dydt(607)+k(10)*y(414);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(414)=dydt(414)-k(8)*y(414);
dydt(8)=dydt(8)+k(8)*y(414);
dydt(335)=dydt(335)+k(8)*y(414);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(414)=dydt(414)-k(6)*y(414);
dydt(7)=dydt(7)+k(6)*y(414);
dydt(407)=dydt(407)+k(6)*y(414);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(415)=dydt(415)-k(34)*y(415);
dydt(650)=dydt(650)+k(34)*y(415);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(415)=dydt(415)-k(33)*y(415);
dydt(419)=dydt(419)+k(33)*y(415);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(415)=dydt(415)-k(31)*y(415);
dydt(413)=dydt(413)+k(31)*y(415);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(415)=dydt(415)-k(22)*y(415);
dydt(197)=dydt(197)+k(22)*y(415);
dydt(64)=dydt(64)+k(22)*y(415);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(415);
dydt(415)=dydt(415)-k(15)*y(3)*y(415);
dydt(204)=dydt(204)+k(15)*y(3)*y(415);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(415)=dydt(415)-k(13)*y(415);
dydt(410)=dydt(410)+k(13)*y(415);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(415)=dydt(415)-k(10)*y(415);
dydt(3)=dydt(3)+k(10)*y(415);
dydt(296)=dydt(296)+k(10)*y(415);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(415)=dydt(415)-k(8)*y(415);
dydt(91)=dydt(91)+k(8)*y(415);
dydt(335)=dydt(335)+k(8)*y(415);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

dydt(415)=dydt(415)-k(6)*y(415);
dydt(72)=dydt(72)+k(6)*y(415);
dydt(407)=dydt(407)+k(6)*y(415);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(651)=dydt(651)-k(34)*y(651);
dydt(461)=dydt(461)+k(34)*y(651);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(651)=dydt(651)-k(32)*y(651);
dydt(417)=dydt(417)+k(32)*y(651);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(651)=dydt(651)-k(24)*y(651);
dydt(408)=dydt(408)+k(24)*y(651);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(651)=dydt(651)-k(22)*y(651);
dydt(182)=dydt(182)+k(22)*y(651);
dydt(149)=dydt(149)+k(22)*y(651);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(651);
dydt(651)=dydt(651)-k(21)*y(197)*y(651);
dydt(368)=dydt(368)+k(21)*y(197)*y(651);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(651);
dydt(651)=dydt(651)-k(21)*y(32)*y(651);
dydt(365)=dydt(365)+k(21)*y(32)*y(651);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(651);
dydt(651)=dydt(651)-k(21)*y(2)*y(651);
dydt(178)=dydt(178)+k(21)*y(2)*y(651);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(651)=dydt(651)-k(10)*y(651);
dydt(3)=dydt(3)+k(10)*y(651);
dydt(181)=dydt(181)+k(10)*y(651);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(651)=dydt(651)-k(10)*y(651);
dydt(3)=dydt(3)+k(10)*y(651);
dydt(393)=dydt(393)+k(10)*y(651);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(651)=dydt(651)-k(8)*y(651);
dydt(126)=dydt(126)+k(8)*y(651);
dydt(184)=dydt(184)+k(8)*y(651);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(651)=dydt(651)-k(6)*y(651);
dydt(48)=dydt(48)+k(6)*y(651);
dydt(391)=dydt(391)+k(6)*y(651);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(417)=dydt(417)-k(34)*y(417);
dydt(462)=dydt(462)+k(34)*y(417);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(417)=dydt(417)-k(24)*y(417);
dydt(414)=dydt(414)+k(24)*y(417);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(417)=dydt(417)-k(22)*y(417);
dydt(182)=dydt(182)+k(22)*y(417);
dydt(463)=dydt(463)+k(22)*y(417);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(417)=dydt(417)-k(13)*y(417);
dydt(651)=dydt(651)+k(13)*y(417);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(417)=dydt(417)-k(10)*y(417);
dydt(3)=dydt(3)+k(10)*y(417);
dydt(416)=dydt(416)+k(10)*y(417);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(417)=dydt(417)-k(10)*y(417);
dydt(3)=dydt(3)+k(10)*y(417);
dydt(606)=dydt(606)+k(10)*y(417);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(417)=dydt(417)-k(8)*y(417);
dydt(8)=dydt(8)+k(8)*y(417);
dydt(184)=dydt(184)+k(8)*y(417);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(417)=dydt(417)-k(6)*y(417);
dydt(7)=dydt(7)+k(6)*y(417);
dydt(391)=dydt(391)+k(6)*y(417);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(418)=dydt(418)-k(34)*y(418);
dydt(94)=dydt(94)+k(34)*y(418);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(418)=dydt(418)-k(22)*y(418);
dydt(182)=dydt(182)+k(22)*y(418);
dydt(97)=dydt(97)+k(22)*y(418);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(418);
dydt(418)=dydt(418)-k(9)*y(3)*y(418);
dydt(606)=dydt(606)+k(9)*y(3)*y(418);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(418);
dydt(418)=dydt(418)-k(9)*y(3)*y(418);
dydt(416)=dydt(416)+k(9)*y(3)*y(418);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(418)=dydt(418)-k(8)*y(418);
dydt(5)=dydt(5)+k(8)*y(418);
dydt(177)=dydt(177)+k(8)*y(418);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

dydt(418)=dydt(418)-k(6)*y(418);
dydt(1)=dydt(1)+k(6)*y(418);
dydt(176)=dydt(176)+k(6)*y(418);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(421)=dydt(421)-k(34)*y(421);
dydt(425)=dydt(425)+k(34)*y(421);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(421)=dydt(421)-k(32)*y(421);
dydt(299)=dydt(299)+k(32)*y(421);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(421)=dydt(421)-k(31)*y(421);
dydt(648)=dydt(648)+k(31)*y(421);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(421)=dydt(421)-k(22)*y(421);
dydt(182)=dydt(182)+k(22)*y(421);
dydt(614)=dydt(614)+k(22)*y(421);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(421);
dydt(421)=dydt(421)-k(21)*y(197)*y(421);
dydt(487)=dydt(487)+k(21)*y(197)*y(421);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(421);
dydt(421)=dydt(421)-k(21)*y(32)*y(421);
dydt(498)=dydt(498)+k(21)*y(32)*y(421);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(421);
dydt(421)=dydt(421)-k(21)*y(2)*y(421);
dydt(271)=dydt(271)+k(21)*y(2)*y(421);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(421);
dydt(421)=dydt(421)-k(15)*y(3)*y(421);
dydt(272)=dydt(272)+k(15)*y(3)*y(421);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(421);
dydt(421)=dydt(421)-k(9)*y(3)*y(421);
dydt(420)=dydt(420)+k(9)*y(3)*y(421);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(421)=dydt(421)-k(8)*y(421);
dydt(315)=dydt(315)+k(8)*y(421);
dydt(177)=dydt(177)+k(8)*y(421);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

dydt(421)=dydt(421)-k(6)*y(421);
dydt(77)=dydt(77)+k(6)*y(421);
dydt(176)=dydt(176)+k(6)*y(421);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(422)=dydt(422)-k(34)*y(422);
dydt(58)=dydt(58)+k(34)*y(422);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(422)=dydt(422)-k(32)*y(422);
dydt(298)=dydt(298)+k(32)*y(422);

% rule    : R11_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(422)=dydt(422)-k(26)*y(422);
dydt(206)=dydt(206)+k(26)*y(422);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(422)=dydt(422)-k(22)*y(422);
dydt(182)=dydt(182)+k(22)*y(422);
dydt(53)=dydt(53)+k(22)*y(422);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(422);
dydt(422)=dydt(422)-k(21)*y(197)*y(422);
dydt(476)=dydt(476)+k(21)*y(197)*y(422);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(422);
dydt(422)=dydt(422)-k(21)*y(32)*y(422);
dydt(478)=dydt(478)+k(21)*y(32)*y(422);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(422);
dydt(422)=dydt(422)-k(21)*y(2)*y(422);
dydt(270)=dydt(270)+k(21)*y(2)*y(422);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(422)=dydt(422)-k(16)*y(422);
dydt(3)=dydt(3)+k(16)*y(422);
dydt(420)=dydt(420)+k(16)*y(422);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(422)=dydt(422)-k(10)*y(422);
dydt(3)=dydt(3)+k(10)*y(422);
dydt(272)=dydt(272)+k(10)*y(422);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(422)=dydt(422)-k(8)*y(422);
dydt(55)=dydt(55)+k(8)*y(422);
dydt(184)=dydt(184)+k(8)*y(422);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(422)=dydt(422)-k(6)*y(422);
dydt(54)=dydt(54)+k(6)*y(422);
dydt(391)=dydt(391)+k(6)*y(422);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(423)=dydt(423)-k(34)*y(423);
dydt(382)=dydt(382)+k(34)*y(423);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(423)=dydt(423)-k(33)*y(423);
dydt(392)=dydt(392)+k(33)*y(423);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(423)=dydt(423)-k(31)*y(423);
dydt(385)=dydt(385)+k(31)*y(423);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(423)=dydt(423)-k(29)*y(423);
dydt(398)=dydt(398)+k(29)*y(423);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(423)=dydt(423)-k(22)*y(423);
dydt(32)=dydt(32)+k(22)*y(423);
dydt(420)=dydt(420)+k(22)*y(423);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(423)=dydt(423)-k(22)*y(423);
dydt(182)=dydt(182)+k(22)*y(423);
dydt(470)=dydt(470)+k(22)*y(423);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(423);
dydt(423)=dydt(423)-k(15)*y(3)*y(423);
dydt(478)=dydt(478)+k(15)*y(3)*y(423);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(423)=dydt(423)-k(10)*y(423);
dydt(3)=dydt(3)+k(10)*y(423);
dydt(498)=dydt(498)+k(10)*y(423);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(423)=dydt(423)-k(8)*y(423);
dydt(34)=dydt(34)+k(8)*y(423);
dydt(184)=dydt(184)+k(8)*y(423);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(423)=dydt(423)-k(6)*y(423);
dydt(33)=dydt(33)+k(6)*y(423);
dydt(391)=dydt(391)+k(6)*y(423);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(425)=dydt(425)-k(32)*y(425);
dydt(90)=dydt(90)+k(32)*y(425);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(425)=dydt(425)-k(31)*y(425);
dydt(375)=dydt(375)+k(31)*y(425);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(425)=dydt(425)-k(22)*y(425);
dydt(2)=dydt(2)+k(22)*y(425);
dydt(614)=dydt(614)+k(22)*y(425);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(425);
dydt(425)=dydt(425)-k(21)*y(197)*y(425);
dydt(427)=dydt(427)+k(21)*y(197)*y(425);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(425);
dydt(425)=dydt(425)-k(21)*y(32)*y(425);
dydt(482)=dydt(482)+k(21)*y(32)*y(425);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(425);
dydt(425)=dydt(425)-k(21)*y(2)*y(425);
dydt(275)=dydt(275)+k(21)*y(2)*y(425);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(425);
dydt(425)=dydt(425)-k(15)*y(3)*y(425);
dydt(276)=dydt(276)+k(15)*y(3)*y(425);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(425);
dydt(425)=dydt(425)-k(9)*y(3)*y(425);
dydt(424)=dydt(424)+k(9)*y(3)*y(425);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(425)=dydt(425)-k(8)*y(425);
dydt(315)=dydt(315)+k(8)*y(425);
dydt(21)=dydt(21)+k(8)*y(425);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

dydt(425)=dydt(425)-k(6)*y(425);
dydt(77)=dydt(77)+k(6)*y(425);
dydt(89)=dydt(89)+k(6)*y(425);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(427)=dydt(427)-k(34)*y(427);
dydt(482)=dydt(482)+k(34)*y(427);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(427)=dydt(427)-k(33)*y(427);
dydt(280)=dydt(280)+k(33)*y(427);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(427)=dydt(427)-k(31)*y(427);
dydt(373)=dydt(373)+k(31)*y(427);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(427)=dydt(427)-k(30)*y(427);
dydt(487)=dydt(487)+k(30)*y(427);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(427)=dydt(427)-k(22)*y(427);
dydt(2)=dydt(2)+k(22)*y(427);
dydt(618)=dydt(618)+k(22)*y(427);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(427)=dydt(427)-k(22)*y(427);
dydt(197)=dydt(197)+k(22)*y(427);
dydt(425)=dydt(425)+k(22)*y(427);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(427);
dydt(427)=dydt(427)-k(15)*y(3)*y(427);
dydt(484)=dydt(484)+k(15)*y(3)*y(427);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(427);
dydt(427)=dydt(427)-k(9)*y(3)*y(427);
dydt(426)=dydt(426)+k(9)*y(3)*y(427);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(427)=dydt(427)-k(8)*y(427);
dydt(21)=dydt(21)+k(8)*y(427);
dydt(319)=dydt(319)+k(8)*y(427);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(427)=dydt(427)-k(6)*y(427);
dydt(257)=dydt(257)+k(6)*y(427);
dydt(89)=dydt(89)+k(6)*y(427);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(428)=dydt(428)-k(34)*y(428);
dydt(480)=dydt(480)+k(34)*y(428);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(428)=dydt(428)-k(33)*y(428);
dydt(279)=dydt(279)+k(33)*y(428);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(428)=dydt(428)-k(30)*y(428);
dydt(476)=dydt(476)+k(30)*y(428);

% rule    : R11_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(428)=dydt(428)-k(26)*y(428);
dydt(473)=dydt(473)+k(26)*y(428);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(428)=dydt(428)-k(22)*y(428);
dydt(197)=dydt(197)+k(22)*y(428);
dydt(58)=dydt(58)+k(22)*y(428);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(428)=dydt(428)-k(22)*y(428);
dydt(2)=dydt(2)+k(22)*y(428);
dydt(205)=dydt(205)+k(22)*y(428);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(428)=dydt(428)-k(16)*y(428);
dydt(3)=dydt(3)+k(16)*y(428);
dydt(426)=dydt(426)+k(16)*y(428);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(428)=dydt(428)-k(10)*y(428);
dydt(3)=dydt(3)+k(10)*y(428);
dydt(484)=dydt(484)+k(10)*y(428);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(428)=dydt(428)-k(8)*y(428);
dydt(140)=dydt(140)+k(8)*y(428);
dydt(228)=dydt(228)+k(8)*y(428);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(428)=dydt(428)-k(6)*y(428);
dydt(227)=dydt(227)+k(6)*y(428);
dydt(380)=dydt(380)+k(6)*y(428);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(431)=dydt(431)-k(34)*y(431);
dydt(616)=dydt(616)+k(34)*y(431);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(431)=dydt(431)-k(32)*y(431);
dydt(220)=dydt(220)+k(32)*y(431);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(431)=dydt(431)-k(31)*y(431);
dydt(451)=dydt(451)+k(31)*y(431);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(431)=dydt(431)-k(22)*y(431);
dydt(182)=dydt(182)+k(22)*y(431);
dydt(614)=dydt(614)+k(22)*y(431);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(431);
dydt(431)=dydt(431)-k(21)*y(197)*y(431);
dydt(290)=dydt(290)+k(21)*y(197)*y(431);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(431);
dydt(431)=dydt(431)-k(21)*y(32)*y(431);
dydt(285)=dydt(285)+k(21)*y(32)*y(431);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(431);
dydt(431)=dydt(431)-k(21)*y(2)*y(431);
dydt(280)=dydt(280)+k(21)*y(2)*y(431);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(431);
dydt(431)=dydt(431)-k(15)*y(3)*y(431);
dydt(282)=dydt(282)+k(15)*y(3)*y(431);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(431);
dydt(431)=dydt(431)-k(9)*y(3)*y(431);
dydt(430)=dydt(430)+k(9)*y(3)*y(431);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

dydt(431)=dydt(431)-k(8)*y(431);
dydt(15)=dydt(15)+k(8)*y(431);
dydt(219)=dydt(219)+k(8)*y(431);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(431)=dydt(431)-k(6)*y(431);
dydt(98)=dydt(98)+k(6)*y(431);
dydt(218)=dydt(218)+k(6)*y(431);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(432)=dydt(432)-k(34)*y(432);
dydt(440)=dydt(440)+k(34)*y(432);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(432)=dydt(432)-k(33)*y(432);
dydt(429)=dydt(429)+k(33)*y(432);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(432)=dydt(432)-k(31)*y(432);
dydt(447)=dydt(447)+k(31)*y(432);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(432)=dydt(432)-k(30)*y(432);
dydt(433)=dydt(433)+k(30)*y(432);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(432)=dydt(432)-k(23)*y(432);
dydt(471)=dydt(471)+k(23)*y(432);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(432)=dydt(432)-k(22)*y(432);
dydt(32)=dydt(32)+k(22)*y(432);
dydt(430)=dydt(430)+k(22)*y(432);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(432)=dydt(432)-k(22)*y(432);
dydt(182)=dydt(182)+k(22)*y(432);
dydt(443)=dydt(443)+k(22)*y(432);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(432);
dydt(432)=dydt(432)-k(15)*y(3)*y(432);
dydt(284)=dydt(284)+k(15)*y(3)*y(432);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(432)=dydt(432)-k(10)*y(432);
dydt(3)=dydt(3)+k(10)*y(432);
dydt(285)=dydt(285)+k(10)*y(432);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(432)=dydt(432)-k(8)*y(432);
dydt(219)=dydt(219)+k(8)*y(432);
dydt(131)=dydt(131)+k(8)*y(432);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(432)=dydt(432)-k(6)*y(432);
dydt(218)=dydt(218)+k(6)*y(432);
dydt(441)=dydt(441)+k(6)*y(432);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(434)=dydt(434)-k(34)*y(434);
dydt(471)=dydt(471)+k(34)*y(434);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(434)=dydt(434)-k(34)*y(434);
dydt(439)=dydt(439)+k(34)*y(434);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(434)=dydt(434)-k(33)*y(434);
dydt(398)=dydt(398)+k(33)*y(434);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(434)=dydt(434)-k(33)*y(434);
dydt(433)=dydt(433)+k(33)*y(434);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(434)=dydt(434)-k(31)*y(434);
dydt(403)=dydt(403)+k(31)*y(434);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(434)=dydt(434)-k(22)*y(434);
dydt(197)=dydt(197)+k(22)*y(434);
dydt(401)=dydt(401)+k(22)*y(434);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(434)=dydt(434)-k(22)*y(434);
dydt(197)=dydt(197)+k(22)*y(434);
dydt(410)=dydt(410)+k(22)*y(434);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(434);
dydt(434)=dydt(434)-k(15)*y(3)*y(434);
dydt(475)=dydt(475)+k(15)*y(3)*y(434);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(434)=dydt(434)-k(10)*y(434);
dydt(3)=dydt(3)+k(10)*y(434);
dydt(619)=dydt(619)+k(10)*y(434);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(434)=dydt(434)-k(8)*y(434);
dydt(319)=dydt(319)+k(8)*y(434);
dydt(335)=dydt(335)+k(8)*y(434);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(434)=dydt(434)-k(6)*y(434);
dydt(257)=dydt(257)+k(6)*y(434);
dydt(407)=dydt(407)+k(6)*y(434);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(435)=dydt(435)-k(34)*y(435);
dydt(447)=dydt(447)+k(34)*y(435);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(435)=dydt(435)-k(34)*y(435);
dydt(411)=dydt(411)+k(34)*y(435);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(435)=dydt(435)-k(33)*y(435);
dydt(394)=dydt(394)+k(33)*y(435);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(435)=dydt(435)-k(23)*y(435);
dydt(403)=dydt(403)+k(23)*y(435);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(435)=dydt(435)-k(22)*y(435);
dydt(197)=dydt(197)+k(22)*y(435);
dydt(396)=dydt(396)+k(22)*y(435);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(435)=dydt(435)-k(22)*y(435);
dydt(182)=dydt(182)+k(22)*y(435);
dydt(406)=dydt(406)+k(22)*y(435);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(435)=dydt(435)-k(11)*y(435);
dydt(433)=dydt(433)+k(11)*y(435);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(435)=dydt(435)-k(10)*y(435);
dydt(3)=dydt(3)+k(10)*y(435);
dydt(194)=dydt(194)+k(10)*y(435);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(435);
dydt(435)=dydt(435)-k(9)*y(3)*y(435);
dydt(193)=dydt(193)+k(9)*y(3)*y(435);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(435)=dydt(435)-k(8)*y(435);
dydt(214)=dydt(214)+k(8)*y(435);
dydt(335)=dydt(335)+k(8)*y(435);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(435)=dydt(435)-k(6)*y(435);
dydt(177)=dydt(177)+k(6)*y(435);
dydt(407)=dydt(407)+k(6)*y(435);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(437)=dydt(437)-k(31)*y(437);
dydt(22)=dydt(22)+k(31)*y(437);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~Y) 

dydt(437)=dydt(437)-k(22)*y(437);
dydt(2)=dydt(2)+k(22)*y(437);
dydt(315)=dydt(315)+k(22)*y(437);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(437);
dydt(437)=dydt(437)-k(15)*y(3)*y(437);
dydt(44)=dydt(44)+k(15)*y(3)*y(437);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(1)=dydt(1)-k(7)*y(1)*y(437);
dydt(437)=dydt(437)-k(7)*y(1)*y(437);
dydt(621)=dydt(621)+k(7)*y(1)*y(437);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(7)=dydt(7)-k(7)*y(7)*y(437);
dydt(437)=dydt(437)-k(7)*y(7)*y(437);
dydt(25)=dydt(25)+k(7)*y(7)*y(437);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) 

dydt(437)=dydt(437)-k(2)*y(437);
dydt(4)=dydt(4)+k(2)*y(437);
dydt(111)=dydt(111)+k(2)*y(437);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(438)=dydt(438)-k(34)*y(438);
dydt(442)=dydt(442)+k(34)*y(438);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(438)=dydt(438)-k(33)*y(438);
dydt(271)=dydt(271)+k(33)*y(438);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(438)=dydt(438)-k(31)*y(438);
dydt(453)=dydt(453)+k(31)*y(438);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(438)=dydt(438)-k(29)*y(438);
dydt(290)=dydt(290)+k(29)*y(438);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(438)=dydt(438)-k(22)*y(438);
dydt(197)=dydt(197)+k(22)*y(438);
dydt(616)=dydt(616)+k(22)*y(438);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(438)=dydt(438)-k(22)*y(438);
dydt(2)=dydt(2)+k(22)*y(438);
dydt(292)=dydt(292)+k(22)*y(438);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(438);
dydt(438)=dydt(438)-k(15)*y(3)*y(438);
dydt(262)=dydt(262)+k(15)*y(3)*y(438);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(438);
dydt(438)=dydt(438)-k(9)*y(3)*y(438);
dydt(436)=dydt(436)+k(9)*y(3)*y(438);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(438)=dydt(438)-k(8)*y(438);
dydt(192)=dydt(192)+k(8)*y(438);
dydt(437)=dydt(437)+k(8)*y(438);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(438)=dydt(438)-k(6)*y(438);
dydt(111)=dydt(111)+k(6)*y(438);
dydt(191)=dydt(191)+k(6)*y(438);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(439)=dydt(439)-k(34)*y(439);
dydt(444)=dydt(444)+k(34)*y(439);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(439)=dydt(439)-k(33)*y(439);
dydt(423)=dydt(423)+k(33)*y(439);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(439)=dydt(439)-k(33)*y(439);
dydt(436)=dydt(436)+k(33)*y(439);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(439)=dydt(439)-k(31)*y(439);
dydt(412)=dydt(412)+k(31)*y(439);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(439)=dydt(439)-k(29)*y(439);
dydt(434)=dydt(434)+k(29)*y(439);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(439)=dydt(439)-k(22)*y(439);
dydt(197)=dydt(197)+k(22)*y(439);
dydt(470)=dydt(470)+k(22)*y(439);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(439)=dydt(439)-k(22)*y(439);
dydt(32)=dydt(32)+k(22)*y(439);
dydt(410)=dydt(410)+k(22)*y(439);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(439);
dydt(439)=dydt(439)-k(15)*y(3)*y(439);
dydt(477)=dydt(477)+k(15)*y(3)*y(439);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(439)=dydt(439)-k(10)*y(439);
dydt(3)=dydt(3)+k(10)*y(439);
dydt(620)=dydt(620)+k(10)*y(439);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(439)=dydt(439)-k(8)*y(439);
dydt(34)=dydt(34)+k(8)*y(439);
dydt(335)=dydt(335)+k(8)*y(439);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(439)=dydt(439)-k(6)*y(439);
dydt(33)=dydt(33)+k(6)*y(439);
dydt(407)=dydt(407)+k(6)*y(439);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(441)=dydt(441)-k(33)*y(441);
dydt(380)=dydt(380)+k(33)*y(441);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(441)=dydt(441)-k(22)*y(441);
dydt(32)=dydt(32)+k(22)*y(441);
dydt(47)=dydt(47)+k(22)*y(441);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(441)=dydt(441)-k(10)*y(441);
dydt(3)=dydt(3)+k(10)*y(441);
dydt(86)=dydt(86)+k(10)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(1)=dydt(1)-k(5)*y(1)*y(441);
dydt(441)=dydt(441)-k(5)*y(1)*y(441);
dydt(102)=dydt(102)+k(5)*y(1)*y(441);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(5)*y(7)*y(441);
dydt(441)=dydt(441)-k(5)*y(7)*y(441);
dydt(104)=dydt(104)+k(5)*y(7)*y(441);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) 

dydt(441)=dydt(441)-k(4)*y(441);
dydt(4)=dydt(4)+k(4)*y(441);
dydt(131)=dydt(131)+k(4)*y(441);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(442)=dydt(442)-k(33)*y(442);
dydt(275)=dydt(275)+k(33)*y(442);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(442)=dydt(442)-k(31)*y(442);
dydt(166)=dydt(166)+k(31)*y(442);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(442)=dydt(442)-k(28)*y(442);
dydt(438)=dydt(438)+k(28)*y(442);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(442)=dydt(442)-k(27)*y(442);
dydt(285)=dydt(285)+k(27)*y(442);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(442)=dydt(442)-k(22)*y(442);
dydt(32)=dydt(32)+k(22)*y(442);
dydt(616)=dydt(616)+k(22)*y(442);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(442)=dydt(442)-k(22)*y(442);
dydt(2)=dydt(2)+k(22)*y(442);
dydt(615)=dydt(615)+k(22)*y(442);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(442);
dydt(442)=dydt(442)-k(15)*y(3)*y(442);
dydt(277)=dydt(277)+k(15)*y(3)*y(442);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(442);
dydt(442)=dydt(442)-k(9)*y(3)*y(442);
dydt(440)=dydt(440)+k(9)*y(3)*y(442);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(442)=dydt(442)-k(8)*y(442);
dydt(27)=dydt(27)+k(8)*y(442);
dydt(437)=dydt(437)+k(8)*y(442);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(442)=dydt(442)-k(6)*y(442);
dydt(111)=dydt(111)+k(6)*y(442);
dydt(86)=dydt(86)+k(6)*y(442);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(650)=dydt(650)-k(33)*y(650);
dydt(92)=dydt(92)+k(33)*y(650);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(650)=dydt(650)-k(31)*y(650);
dydt(102)=dydt(102)+k(31)*y(650);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(650)=dydt(650)-k(22)*y(650);
dydt(32)=dydt(32)+k(22)*y(650);
dydt(64)=dydt(64)+k(22)*y(650);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(650);
dydt(650)=dydt(650)-k(15)*y(3)*y(650);
dydt(66)=dydt(66)+k(15)*y(3)*y(650);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(650)=dydt(650)-k(13)*y(650);
dydt(443)=dydt(443)+k(13)*y(650);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(650)=dydt(650)-k(10)*y(650);
dydt(3)=dydt(3)+k(10)*y(650);
dydt(87)=dydt(87)+k(10)*y(650);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(650)=dydt(650)-k(8)*y(650);
dydt(91)=dydt(91)+k(8)*y(650);
dydt(131)=dydt(131)+k(8)*y(650);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

dydt(650)=dydt(650)-k(6)*y(650);
dydt(72)=dydt(72)+k(6)*y(650);
dydt(441)=dydt(441)+k(6)*y(650);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(443)=dydt(443)-k(33)*y(443);
dydt(424)=dydt(424)+k(33)*y(443);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(443)=dydt(443)-k(32)*y(443);
dydt(650)=dydt(650)+k(32)*y(443);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(443)=dydt(443)-k(31)*y(443);
dydt(405)=dydt(405)+k(31)*y(443);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(443)=dydt(443)-k(22)*y(443);
dydt(32)=dydt(32)+k(22)*y(443);
dydt(57)=dydt(57)+k(22)*y(443);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(443);
dydt(443)=dydt(443)-k(21)*y(197)*y(443);
dydt(471)=dydt(471)+k(21)*y(197)*y(443);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(443);
dydt(443)=dydt(443)-k(21)*y(32)*y(443);
dydt(444)=dydt(444)+k(21)*y(32)*y(443);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(443);
dydt(443)=dydt(443)-k(21)*y(2)*y(443);
dydt(440)=dydt(440)+k(21)*y(2)*y(443);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(443);
dydt(443)=dydt(443)-k(15)*y(3)*y(443);
dydt(59)=dydt(59)+k(15)*y(3)*y(443);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(443)=dydt(443)-k(10)*y(443);
dydt(3)=dydt(3)+k(10)*y(443);
dydt(615)=dydt(615)+k(10)*y(443);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(443)=dydt(443)-k(8)*y(443);
dydt(315)=dydt(315)+k(8)*y(443);
dydt(131)=dydt(131)+k(8)*y(443);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

dydt(443)=dydt(443)-k(6)*y(443);
dydt(77)=dydt(77)+k(6)*y(443);
dydt(441)=dydt(441)+k(6)*y(443);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(468)=dydt(468)-k(33)*y(468);
dydt(482)=dydt(482)+k(33)*y(468);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(468)=dydt(468)-k(33)*y(468);
dydt(442)=dydt(442)+k(33)*y(468);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(468)=dydt(468)-k(31)*y(468);
dydt(160)=dydt(160)+k(31)*y(468);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(468)=dydt(468)-k(28)*y(468);
dydt(620)=dydt(620)+k(28)*y(468);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(468)=dydt(468)-k(27)*y(468);
dydt(472)=dydt(472)+k(27)*y(468);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(468)=dydt(468)-k(22)*y(468);
dydt(32)=dydt(32)+k(22)*y(468);
dydt(617)=dydt(617)+k(22)*y(468);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(468)=dydt(468)-k(22)*y(468);
dydt(32)=dydt(32)+k(22)*y(468);
dydt(615)=dydt(615)+k(22)*y(468);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(468);
dydt(468)=dydt(468)-k(15)*y(3)*y(468);
dydt(483)=dydt(483)+k(15)*y(3)*y(468);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(468);
dydt(468)=dydt(468)-k(9)*y(3)*y(468);
dydt(444)=dydt(444)+k(9)*y(3)*y(468);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(468)=dydt(468)-k(8)*y(468);
dydt(27)=dydt(27)+k(8)*y(468);
dydt(34)=dydt(34)+k(8)*y(468);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(468)=dydt(468)-k(6)*y(468);
dydt(33)=dydt(33)+k(6)*y(468);
dydt(86)=dydt(86)+k(6)*y(468);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(469)=dydt(469)-k(33)*y(469);
dydt(52)=dydt(52)+k(33)*y(469);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(469)=dydt(469)-k(33)*y(469);
dydt(480)=dydt(480)+k(33)*y(469);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(469)=dydt(469)-k(28)*y(469);
dydt(477)=dydt(477)+k(28)*y(469);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(469)=dydt(469)-k(27)*y(469);
dydt(473)=dydt(473)+k(27)*y(469);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(469)=dydt(469)-k(22)*y(469);
dydt(32)=dydt(32)+k(22)*y(469);
dydt(59)=dydt(59)+k(22)*y(469);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(469)=dydt(469)-k(22)*y(469);
dydt(32)=dydt(32)+k(22)*y(469);
dydt(40)=dydt(40)+k(22)*y(469);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(469)=dydt(469)-k(16)*y(469);
dydt(3)=dydt(3)+k(16)*y(469);
dydt(444)=dydt(444)+k(16)*y(469);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(469)=dydt(469)-k(10)*y(469);
dydt(3)=dydt(3)+k(10)*y(469);
dydt(483)=dydt(483)+k(10)*y(469);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(469)=dydt(469)-k(8)*y(469);
dydt(131)=dydt(131)+k(8)*y(469);
dydt(38)=dydt(38)+k(8)*y(469);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(469)=dydt(469)-k(6)*y(469);
dydt(37)=dydt(37)+k(6)*y(469);
dydt(441)=dydt(441)+k(6)*y(469);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(470)=dydt(470)-k(33)*y(470);
dydt(50)=dydt(50)+k(33)*y(470);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(470)=dydt(470)-k(32)*y(470);
dydt(31)=dydt(31)+k(32)*y(470);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(470)=dydt(470)-k(31)*y(470);
dydt(384)=dydt(384)+k(31)*y(470);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(470)=dydt(470)-k(22)*y(470);
dydt(32)=dydt(32)+k(22)*y(470);
dydt(57)=dydt(57)+k(22)*y(470);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(470);
dydt(470)=dydt(470)-k(21)*y(197)*y(470);
dydt(439)=dydt(439)+k(21)*y(197)*y(470);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(470);
dydt(470)=dydt(470)-k(21)*y(32)*y(470);
dydt(444)=dydt(444)+k(21)*y(32)*y(470);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(470);
dydt(470)=dydt(470)-k(21)*y(2)*y(470);
dydt(382)=dydt(382)+k(21)*y(2)*y(470);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(470);
dydt(470)=dydt(470)-k(15)*y(3)*y(470);
dydt(40)=dydt(40)+k(15)*y(3)*y(470);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(470)=dydt(470)-k(10)*y(470);
dydt(3)=dydt(3)+k(10)*y(470);
dydt(617)=dydt(617)+k(10)*y(470);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(470)=dydt(470)-k(8)*y(470);
dydt(48)=dydt(48)+k(8)*y(470);
dydt(34)=dydt(34)+k(8)*y(470);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(470)=dydt(470)-k(6)*y(470);
dydt(47)=dydt(47)+k(6)*y(470);
dydt(33)=dydt(33)+k(6)*y(470);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(472)=dydt(472)-k(34)*y(472);
dydt(468)=dydt(468)+k(34)*y(472);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(472)=dydt(472)-k(33)*y(472);
dydt(427)=dydt(427)+k(33)*y(472);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(472)=dydt(472)-k(33)*y(472);
dydt(285)=dydt(285)+k(33)*y(472);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(472)=dydt(472)-k(31)*y(472);
dydt(454)=dydt(454)+k(31)*y(472);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(472)=dydt(472)-k(30)*y(472);
dydt(619)=dydt(619)+k(30)*y(472);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(472)=dydt(472)-k(22)*y(472);
dydt(32)=dydt(32)+k(22)*y(472);
dydt(618)=dydt(618)+k(22)*y(472);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(472)=dydt(472)-k(22)*y(472);
dydt(197)=dydt(197)+k(22)*y(472);
dydt(615)=dydt(615)+k(22)*y(472);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(472);
dydt(472)=dydt(472)-k(15)*y(3)*y(472);
dydt(474)=dydt(474)+k(15)*y(3)*y(472);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(472);
dydt(472)=dydt(472)-k(9)*y(3)*y(472);
dydt(471)=dydt(471)+k(9)*y(3)*y(472);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(472)=dydt(472)-k(8)*y(472);
dydt(27)=dydt(27)+k(8)*y(472);
dydt(319)=dydt(319)+k(8)*y(472);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(472)=dydt(472)-k(6)*y(472);
dydt(257)=dydt(257)+k(6)*y(472);
dydt(86)=dydt(86)+k(6)*y(472);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(474)=dydt(474)-k(34)*y(474);
dydt(483)=dydt(483)+k(34)*y(474);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(474)=dydt(474)-k(33)*y(474);
dydt(484)=dydt(484)+k(33)*y(474);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(474)=dydt(474)-k(33)*y(474);
dydt(283)=dydt(283)+k(33)*y(474);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(474)=dydt(474)-k(30)*y(474);
dydt(294)=dydt(294)+k(30)*y(474);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(474)=dydt(474)-k(22)*y(474);
dydt(32)=dydt(32)+k(22)*y(474);
dydt(232)=dydt(232)+k(22)*y(474);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(474)=dydt(474)-k(22)*y(474);
dydt(197)=dydt(197)+k(22)*y(474);
dydt(287)=dydt(287)+k(22)*y(474);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(474)=dydt(474)-k(18)*y(474);
dydt(635)=dydt(635)+k(18)*y(474);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(474)=dydt(474)-k(16)*y(474);
dydt(3)=dydt(3)+k(16)*y(474);
dydt(472)=dydt(472)+k(16)*y(474);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(474);
dydt(474)=dydt(474)-k(9)*y(3)*y(474);
dydt(473)=dydt(473)+k(9)*y(3)*y(474);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(474)=dydt(474)-k(8)*y(474);
dydt(27)=dydt(27)+k(8)*y(474);
dydt(228)=dydt(228)+k(8)*y(474);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(474)=dydt(474)-k(6)*y(474);
dydt(86)=dydt(86)+k(6)*y(474);
dydt(227)=dydt(227)+k(6)*y(474);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(649)=dydt(649)-k(34)*y(649);
dydt(279)=dydt(279)+k(34)*y(649);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(649)=dydt(649)-k(34)*y(649);
dydt(270)=dydt(270)+k(34)*y(649);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(649)=dydt(649)-k(26)*y(649);
dydt(289)=dydt(289)+k(26)*y(649);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(649)=dydt(649)-k(23)*y(649);
dydt(476)=dydt(476)+k(23)*y(649);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(649)=dydt(649)-k(22)*y(649);
dydt(182)=dydt(182)+k(22)*y(649);
dydt(225)=dydt(225)+k(22)*y(649);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

dydt(649)=dydt(649)-k(22)*y(649);
dydt(182)=dydt(182)+k(22)*y(649);
dydt(422)=dydt(422)+k(22)*y(649);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(649)=dydt(649)-k(16)*y(649);
dydt(3)=dydt(3)+k(16)*y(649);
dydt(395)=dydt(395)+k(16)*y(649);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(649)=dydt(649)-k(10)*y(649);
dydt(3)=dydt(3)+k(10)*y(649);
dydt(273)=dydt(273)+k(10)*y(649);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(649)=dydt(649)-k(8)*y(649);
dydt(184)=dydt(184)+k(8)*y(649);
dydt(223)=dydt(223)+k(8)*y(649);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(649)=dydt(649)-k(6)*y(649);
dydt(222)=dydt(222)+k(6)*y(649);
dydt(391)=dydt(391)+k(6)*y(649);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(476)=dydt(476)-k(34)*y(476);
dydt(428)=dydt(428)+k(34)*y(476);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(476)=dydt(476)-k(34)*y(476);
dydt(478)=dydt(478)+k(34)*y(476);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(476)=dydt(476)-k(33)*y(476);
dydt(649)=dydt(649)+k(33)*y(476);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(476)=dydt(476)-k(26)*y(476);
dydt(475)=dydt(475)+k(26)*y(476);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(476)=dydt(476)-k(22)*y(476);
dydt(182)=dydt(182)+k(22)*y(476);
dydt(205)=dydt(205)+k(22)*y(476);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

dydt(476)=dydt(476)-k(22)*y(476);
dydt(197)=dydt(197)+k(22)*y(476);
dydt(422)=dydt(422)+k(22)*y(476);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(476)=dydt(476)-k(16)*y(476);
dydt(3)=dydt(3)+k(16)*y(476);
dydt(398)=dydt(398)+k(16)*y(476);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(476)=dydt(476)-k(10)*y(476);
dydt(3)=dydt(3)+k(10)*y(476);
dydt(486)=dydt(486)+k(10)*y(476);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(476)=dydt(476)-k(8)*y(476);
dydt(184)=dydt(184)+k(8)*y(476);
dydt(228)=dydt(228)+k(8)*y(476);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(476)=dydt(476)-k(6)*y(476);
dydt(227)=dydt(227)+k(6)*y(476);
dydt(391)=dydt(391)+k(6)*y(476);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(479)=dydt(479)-k(34)*y(479);
dydt(481)=dydt(481)+k(34)*y(479);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(479)=dydt(479)-k(33)*y(479);
dydt(269)=dydt(269)+k(33)*y(479);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(479)=dydt(479)-k(29)*y(479);
dydt(486)=dydt(486)+k(29)*y(479);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(479)=dydt(479)-k(26)*y(479);
dydt(293)=dydt(293)+k(26)*y(479);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(479)=dydt(479)-k(22)*y(479);
dydt(182)=dydt(182)+k(22)*y(479);
dydt(249)=dydt(249)+k(22)*y(479);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(479)=dydt(479)-k(22)*y(479);
dydt(32)=dydt(32)+k(22)*y(479);
dydt(272)=dydt(272)+k(22)*y(479);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(479)=dydt(479)-k(18)*y(479);
dydt(493)=dydt(493)+k(18)*y(479);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(479)=dydt(479)-k(16)*y(479);
dydt(3)=dydt(3)+k(16)*y(479);
dydt(498)=dydt(498)+k(16)*y(479);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(479);
dydt(479)=dydt(479)-k(9)*y(3)*y(479);
dydt(478)=dydt(478)+k(9)*y(3)*y(479);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(479)=dydt(479)-k(8)*y(479);
dydt(177)=dydt(177)+k(8)*y(479);
dydt(38)=dydt(38)+k(8)*y(479);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(479)=dydt(479)-k(6)*y(479);
dydt(176)=dydt(176)+k(6)*y(479);
dydt(37)=dydt(37)+k(6)*y(479);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(482)=dydt(482)-k(33)*y(482);
dydt(275)=dydt(275)+k(33)*y(482);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(482)=dydt(482)-k(31)*y(482);
dydt(378)=dydt(378)+k(31)*y(482);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(482)=dydt(482)-k(28)*y(482);
dydt(498)=dydt(498)+k(28)*y(482);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(482)=dydt(482)-k(27)*y(482);
dydt(427)=dydt(427)+k(27)*y(482);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(482)=dydt(482)-k(22)*y(482);
dydt(2)=dydt(2)+k(22)*y(482);
dydt(617)=dydt(617)+k(22)*y(482);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(482)=dydt(482)-k(22)*y(482);
dydt(32)=dydt(32)+k(22)*y(482);
dydt(425)=dydt(425)+k(22)*y(482);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(482);
dydt(482)=dydt(482)-k(15)*y(3)*y(482);
dydt(481)=dydt(481)+k(15)*y(3)*y(482);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(482);
dydt(482)=dydt(482)-k(9)*y(3)*y(482);
dydt(382)=dydt(382)+k(9)*y(3)*y(482);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(482)=dydt(482)-k(8)*y(482);
dydt(21)=dydt(21)+k(8)*y(482);
dydt(34)=dydt(34)+k(8)*y(482);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(482)=dydt(482)-k(6)*y(482);
dydt(33)=dydt(33)+k(6)*y(482);
dydt(89)=dydt(89)+k(6)*y(482);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(483)=dydt(483)-k(33)*y(483);
dydt(481)=dydt(481)+k(33)*y(483);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(483)=dydt(483)-k(33)*y(483);
dydt(277)=dydt(277)+k(33)*y(483);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(483)=dydt(483)-k(28)*y(483);
dydt(293)=dydt(293)+k(28)*y(483);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(483)=dydt(483)-k(27)*y(483);
dydt(474)=dydt(474)+k(27)*y(483);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(483)=dydt(483)-k(22)*y(483);
dydt(32)=dydt(32)+k(22)*y(483);
dydt(249)=dydt(249)+k(22)*y(483);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(483)=dydt(483)-k(22)*y(483);
dydt(32)=dydt(32)+k(22)*y(483);
dydt(287)=dydt(287)+k(22)*y(483);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(483)=dydt(483)-k(18)*y(483);
dydt(247)=dydt(247)+k(18)*y(483);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(483)=dydt(483)-k(16)*y(483);
dydt(3)=dydt(3)+k(16)*y(483);
dydt(468)=dydt(468)+k(16)*y(483);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(483);
dydt(483)=dydt(483)-k(9)*y(3)*y(483);
dydt(469)=dydt(469)+k(9)*y(3)*y(483);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(483)=dydt(483)-k(8)*y(483);
dydt(27)=dydt(27)+k(8)*y(483);
dydt(38)=dydt(38)+k(8)*y(483);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(483)=dydt(483)-k(6)*y(483);
dydt(86)=dydt(86)+k(6)*y(483);
dydt(37)=dydt(37)+k(6)*y(483);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(485)=dydt(485)-k(34)*y(485);
dydt(246)=dydt(246)+k(34)*y(485);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(485)=dydt(485)-k(33)*y(485);
dydt(281)=dydt(281)+k(33)*y(485);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(485)=dydt(485)-k(31)*y(485);
dydt(484)=dydt(484)+k(31)*y(485);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(485)=dydt(485)-k(30)*y(485);
dydt(488)=dydt(488)+k(30)*y(485);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(485)=dydt(485)-k(26)*y(485);
dydt(635)=dydt(635)+k(26)*y(485);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(485)=dydt(485)-k(22)*y(485);
dydt(2)=dydt(2)+k(22)*y(485);
dydt(491)=dydt(491)+k(22)*y(485);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(485)=dydt(485)-k(22)*y(485);
dydt(197)=dydt(197)+k(22)*y(485);
dydt(629)=dydt(629)+k(22)*y(485);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(485)=dydt(485)-k(16)*y(485);
dydt(3)=dydt(3)+k(16)*y(485);
dydt(502)=dydt(502)+k(16)*y(485);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(485);
dydt(485)=dydt(485)-k(15)*y(3)*y(485);
dydt(632)=dydt(632)+k(15)*y(3)*y(485);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(485)=dydt(485)-k(8)*y(485);
dydt(111)=dydt(111)+k(8)*y(485);
dydt(228)=dydt(228)+k(8)*y(485);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(485)=dydt(485)-k(6)*y(485);
dydt(110)=dydt(110)+k(6)*y(485);
dydt(227)=dydt(227)+k(6)*y(485);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(487)=dydt(487)-k(34)*y(487);
dydt(427)=dydt(427)+k(34)*y(487);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(487)=dydt(487)-k(34)*y(487);
dydt(498)=dydt(498)+k(34)*y(487);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(487)=dydt(487)-k(33)*y(487);
dydt(580)=dydt(580)+k(33)*y(487);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(487)=dydt(487)-k(31)*y(487);
dydt(369)=dydt(369)+k(31)*y(487);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(487)=dydt(487)-k(22)*y(487);
dydt(182)=dydt(182)+k(22)*y(487);
dydt(618)=dydt(618)+k(22)*y(487);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(487)=dydt(487)-k(22)*y(487);
dydt(197)=dydt(197)+k(22)*y(487);
dydt(421)=dydt(421)+k(22)*y(487);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(487);
dydt(487)=dydt(487)-k(15)*y(3)*y(487);
dydt(486)=dydt(486)+k(15)*y(3)*y(487);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(487);
dydt(487)=dydt(487)-k(9)*y(3)*y(487);
dydt(398)=dydt(398)+k(9)*y(3)*y(487);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(487)=dydt(487)-k(8)*y(487);
dydt(177)=dydt(177)+k(8)*y(487);
dydt(319)=dydt(319)+k(8)*y(487);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(487)=dydt(487)-k(6)*y(487);
dydt(257)=dydt(257)+k(6)*y(487);
dydt(176)=dydt(176)+k(6)*y(487);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(489)=dydt(489)-k(34)*y(489);
dydt(632)=dydt(632)+k(34)*y(489);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(489)=dydt(489)-k(34)*y(489);
dydt(494)=dydt(494)+k(34)*y(489);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(489)=dydt(489)-k(33)*y(489);
dydt(640)=dydt(640)+k(33)*y(489);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(489)=dydt(489)-k(26)*y(489);
dydt(642)=dydt(642)+k(26)*y(489);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

dydt(489)=dydt(489)-k(22)*y(489);
dydt(182)=dydt(182)+k(22)*y(489);
dydt(636)=dydt(636)+k(22)*y(489);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

dydt(489)=dydt(489)-k(22)*y(489);
dydt(197)=dydt(197)+k(22)*y(489);
dydt(624)=dydt(624)+k(22)*y(489);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(489)=dydt(489)-k(16)*y(489);
dydt(3)=dydt(3)+k(16)*y(489);
dydt(488)=dydt(488)+k(16)*y(489);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(489)=dydt(489)-k(16)*y(489);
dydt(3)=dydt(3)+k(16)*y(489);
dydt(626)=dydt(626)+k(16)*y(489);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(489)=dydt(489)-k(8)*y(489);
dydt(222)=dydt(222)+k(8)*y(489);
dydt(228)=dydt(228)+k(8)*y(489);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(489)=dydt(489)-k(6)*y(489);
dydt(227)=dydt(227)+k(6)*y(489);
dydt(515)=dydt(515)+k(6)*y(489);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(490)=dydt(490)-k(34)*y(490);
dydt(502)=dydt(502)+k(34)*y(490);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(490)=dydt(490)-k(34)*y(490);
dydt(495)=dydt(495)+k(34)*y(490);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(490)=dydt(490)-k(33)*y(490);
dydt(579)=dydt(579)+k(33)*y(490);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(490)=dydt(490)-k(31)*y(490);
dydt(487)=dydt(487)+k(31)*y(490);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(490)=dydt(490)-k(31)*y(490);
dydt(611)=dydt(611)+k(31)*y(490);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(490)=dydt(490)-k(22)*y(490);
dydt(182)=dydt(182)+k(22)*y(490);
dydt(613)=dydt(613)+k(22)*y(490);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(490)=dydt(490)-k(22)*y(490);
dydt(197)=dydt(197)+k(22)*y(490);
dydt(497)=dydt(497)+k(22)*y(490);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(490);
dydt(490)=dydt(490)-k(15)*y(3)*y(490);
dydt(488)=dydt(488)+k(15)*y(3)*y(490);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(490);
dydt(490)=dydt(490)-k(15)*y(3)*y(490);
dydt(626)=dydt(626)+k(15)*y(3)*y(490);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(490)=dydt(490)-k(8)*y(490);
dydt(218)=dydt(218)+k(8)*y(490);
dydt(319)=dydt(319)+k(8)*y(490);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(490)=dydt(490)-k(6)*y(490);
dydt(257)=dydt(257)+k(6)*y(490);
dydt(264)=dydt(264)+k(6)*y(490);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(491)=dydt(491)-k(34)*y(491);
dydt(243)=dydt(243)+k(34)*y(491);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(491)=dydt(491)-k(33)*y(491);
dydt(645)=dydt(645)+k(33)*y(491);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(491)=dydt(491)-k(32)*y(491);
dydt(231)=dydt(231)+k(32)*y(491);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(491)=dydt(491)-k(31)*y(491);
dydt(232)=dydt(232)+k(31)*y(491);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

dydt(491)=dydt(491)-k(22)*y(491);
dydt(197)=dydt(197)+k(22)*y(491);
dydt(238)=dydt(238)+k(22)*y(491);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(491);
dydt(491)=dydt(491)-k(21)*y(197)*y(491);
dydt(492)=dydt(492)+k(21)*y(197)*y(491);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(491);
dydt(491)=dydt(491)-k(21)*y(32)*y(491);
dydt(635)=dydt(635)+k(21)*y(32)*y(491);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(491);
dydt(491)=dydt(491)-k(21)*y(2)*y(491);
dydt(485)=dydt(485)+k(21)*y(2)*y(491);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(491)=dydt(491)-k(16)*y(491);
dydt(3)=dydt(3)+k(16)*y(491);
dydt(613)=dydt(613)+k(16)*y(491);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(491);
dydt(491)=dydt(491)-k(15)*y(3)*y(491);
dydt(636)=dydt(636)+k(15)*y(3)*y(491);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

dydt(491)=dydt(491)-k(8)*y(491);
dydt(77)=dydt(77)+k(8)*y(491);
dydt(228)=dydt(228)+k(8)*y(491);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(491)=dydt(491)-k(6)*y(491);
dydt(76)=dydt(76)+k(6)*y(491);
dydt(227)=dydt(227)+k(6)*y(491);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(492)=dydt(492)-k(34)*y(492);
dydt(635)=dydt(635)+k(34)*y(492);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(492)=dydt(492)-k(34)*y(492);
dydt(248)=dydt(248)+k(34)*y(492);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(492)=dydt(492)-k(33)*y(492);
dydt(488)=dydt(488)+k(33)*y(492);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(492)=dydt(492)-k(33)*y(492);
dydt(261)=dydt(261)+k(33)*y(492);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(492)=dydt(492)-k(31)*y(492);
dydt(294)=dydt(294)+k(31)*y(492);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(492)=dydt(492)-k(22)*y(492);
dydt(197)=dydt(197)+k(22)*y(492);
dydt(491)=dydt(491)+k(22)*y(492);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(492)=dydt(492)-k(22)*y(492);
dydt(197)=dydt(197)+k(22)*y(492);
dydt(260)=dydt(260)+k(22)*y(492);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(492)=dydt(492)-k(16)*y(492);
dydt(3)=dydt(3)+k(16)*y(492);
dydt(583)=dydt(583)+k(16)*y(492);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(492);
dydt(492)=dydt(492)-k(15)*y(3)*y(492);
dydt(642)=dydt(642)+k(15)*y(3)*y(492);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(492)=dydt(492)-k(8)*y(492);
dydt(257)=dydt(257)+k(8)*y(492);
dydt(228)=dydt(228)+k(8)*y(492);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(492)=dydt(492)-k(6)*y(492);
dydt(256)=dydt(256)+k(6)*y(492);
dydt(227)=dydt(227)+k(6)*y(492);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(494)=dydt(494)-k(34)*y(494);
dydt(631)=dydt(631)+k(34)*y(494);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(494)=dydt(494)-k(33)*y(494);
dydt(265)=dydt(265)+k(33)*y(494);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(494)=dydt(494)-k(29)*y(494);
dydt(489)=dydt(489)+k(29)*y(494);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(494)=dydt(494)-k(26)*y(494);
dydt(643)=dydt(643)+k(26)*y(494);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

dydt(494)=dydt(494)-k(22)*y(494);
dydt(182)=dydt(182)+k(22)*y(494);
dydt(244)=dydt(244)+k(22)*y(494);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

dydt(494)=dydt(494)-k(22)*y(494);
dydt(32)=dydt(32)+k(22)*y(494);
dydt(624)=dydt(624)+k(22)*y(494);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(494)=dydt(494)-k(16)*y(494);
dydt(3)=dydt(3)+k(16)*y(494);
dydt(493)=dydt(493)+k(16)*y(494);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(494)=dydt(494)-k(16)*y(494);
dydt(3)=dydt(3)+k(16)*y(494);
dydt(496)=dydt(496)+k(16)*y(494);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(494)=dydt(494)-k(8)*y(494);
dydt(222)=dydt(222)+k(8)*y(494);
dydt(38)=dydt(38)+k(8)*y(494);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(494)=dydt(494)-k(6)*y(494);
dydt(37)=dydt(37)+k(6)*y(494);
dydt(515)=dydt(515)+k(6)*y(494);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(496)=dydt(496)-k(34)*y(496);
dydt(500)=dydt(500)+k(34)*y(496);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(496)=dydt(496)-k(33)*y(496);
dydt(625)=dydt(625)+k(33)*y(496);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(496)=dydt(496)-k(31)*y(496);
dydt(532)=dydt(532)+k(31)*y(496);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(496)=dydt(496)-k(29)*y(496);
dydt(626)=dydt(626)+k(29)*y(496);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(496)=dydt(496)-k(22)*y(496);
dydt(182)=dydt(182)+k(22)*y(496);
dydt(534)=dydt(534)+k(22)*y(496);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(496)=dydt(496)-k(22)*y(496);
dydt(32)=dydt(32)+k(22)*y(496);
dydt(623)=dydt(623)+k(22)*y(496);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(496)=dydt(496)-k(16)*y(496);
dydt(3)=dydt(3)+k(16)*y(496);
dydt(495)=dydt(495)+k(16)*y(496);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(496);
dydt(496)=dydt(496)-k(15)*y(3)*y(496);
dydt(494)=dydt(494)+k(15)*y(3)*y(496);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(496)=dydt(496)-k(8)*y(496);
dydt(34)=dydt(34)+k(8)*y(496);
dydt(222)=dydt(222)+k(8)*y(496);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(496)=dydt(496)-k(6)*y(496);
dydt(33)=dydt(33)+k(6)*y(496);
dydt(515)=dydt(515)+k(6)*y(496);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(497)=dydt(497)-k(34)*y(497);
dydt(501)=dydt(501)+k(34)*y(497);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(497)=dydt(497)-k(32)*y(497);
dydt(302)=dydt(302)+k(32)*y(497);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(497)=dydt(497)-k(31)*y(497);
dydt(421)=dydt(421)+k(31)*y(497);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(497)=dydt(497)-k(31)*y(497);
dydt(512)=dydt(512)+k(31)*y(497);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(497)=dydt(497)-k(22)*y(497);
dydt(182)=dydt(182)+k(22)*y(497);
dydt(584)=dydt(584)+k(22)*y(497);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(497);
dydt(497)=dydt(497)-k(21)*y(197)*y(497);
dydt(490)=dydt(490)+k(21)*y(197)*y(497);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(497);
dydt(497)=dydt(497)-k(21)*y(32)*y(497);
dydt(495)=dydt(495)+k(21)*y(32)*y(497);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(497);
dydt(497)=dydt(497)-k(21)*y(2)*y(497);
dydt(266)=dydt(266)+k(21)*y(2)*y(497);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(497);
dydt(497)=dydt(497)-k(15)*y(3)*y(497);
dydt(267)=dydt(267)+k(15)*y(3)*y(497);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(497);
dydt(497)=dydt(497)-k(15)*y(3)*y(497);
dydt(623)=dydt(623)+k(15)*y(3)*y(497);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(497)=dydt(497)-k(8)*y(497);
dydt(315)=dydt(315)+k(8)*y(497);
dydt(218)=dydt(218)+k(8)*y(497);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

dydt(497)=dydt(497)-k(6)*y(497);
dydt(77)=dydt(77)+k(6)*y(497);
dydt(264)=dydt(264)+k(6)*y(497);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(498)=dydt(498)-k(34)*y(498);
dydt(482)=dydt(482)+k(34)*y(498);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(498)=dydt(498)-k(33)*y(498);
dydt(271)=dydt(271)+k(33)*y(498);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(498)=dydt(498)-k(31)*y(498);
dydt(366)=dydt(366)+k(31)*y(498);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(498)=dydt(498)-k(29)*y(498);
dydt(487)=dydt(487)+k(29)*y(498);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(498)=dydt(498)-k(22)*y(498);
dydt(182)=dydt(182)+k(22)*y(498);
dydt(617)=dydt(617)+k(22)*y(498);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(498)=dydt(498)-k(22)*y(498);
dydt(32)=dydt(32)+k(22)*y(498);
dydt(421)=dydt(421)+k(22)*y(498);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(498);
dydt(498)=dydt(498)-k(15)*y(3)*y(498);
dydt(479)=dydt(479)+k(15)*y(3)*y(498);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(498);
dydt(498)=dydt(498)-k(9)*y(3)*y(498);
dydt(423)=dydt(423)+k(9)*y(3)*y(498);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(498)=dydt(498)-k(8)*y(498);
dydt(177)=dydt(177)+k(8)*y(498);
dydt(34)=dydt(34)+k(8)*y(498);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(498)=dydt(498)-k(6)*y(498);
dydt(33)=dydt(33)+k(6)*y(498);
dydt(176)=dydt(176)+k(6)*y(498);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(500)=dydt(500)-k(33)*y(500);
dydt(504)=dydt(504)+k(33)*y(500);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(500)=dydt(500)-k(31)*y(500);
dydt(530)=dydt(530)+k(31)*y(500);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(500)=dydt(500)-k(28)*y(500);
dydt(496)=dydt(496)+k(28)*y(500);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(500)=dydt(500)-k(27)*y(500);
dydt(528)=dydt(528)+k(27)*y(500);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(500)=dydt(500)-k(22)*y(500);
dydt(2)=dydt(2)+k(22)*y(500);
dydt(534)=dydt(534)+k(22)*y(500);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(500)=dydt(500)-k(22)*y(500);
dydt(32)=dydt(32)+k(22)*y(500);
dydt(308)=dydt(308)+k(22)*y(500);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(500)=dydt(500)-k(16)*y(500);
dydt(3)=dydt(3)+k(16)*y(500);
dydt(499)=dydt(499)+k(16)*y(500);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(500);
dydt(500)=dydt(500)-k(15)*y(3)*y(500);
dydt(631)=dydt(631)+k(15)*y(3)*y(500);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(500)=dydt(500)-k(8)*y(500);
dydt(34)=dydt(34)+k(8)*y(500);
dydt(43)=dydt(43)+k(8)*y(500);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(500)=dydt(500)-k(6)*y(500);
dydt(33)=dydt(33)+k(6)*y(500);
dydt(114)=dydt(114)+k(6)*y(500);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(501)=dydt(501)-k(32)*y(501);
dydt(305)=dydt(305)+k(32)*y(501);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(501)=dydt(501)-k(31)*y(501);
dydt(425)=dydt(425)+k(31)*y(501);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(501)=dydt(501)-k(31)*y(501);
dydt(354)=dydt(354)+k(31)*y(501);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(501)=dydt(501)-k(22)*y(501);
dydt(2)=dydt(2)+k(22)*y(501);
dydt(584)=dydt(584)+k(22)*y(501);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(501);
dydt(501)=dydt(501)-k(21)*y(197)*y(501);
dydt(502)=dydt(502)+k(21)*y(197)*y(501);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(501);
dydt(501)=dydt(501)-k(21)*y(32)*y(501);
dydt(499)=dydt(499)+k(21)*y(32)*y(501);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(501);
dydt(501)=dydt(501)-k(21)*y(2)*y(501);
dydt(503)=dydt(503)+k(21)*y(2)*y(501);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(501);
dydt(501)=dydt(501)-k(15)*y(3)*y(501);
dydt(629)=dydt(629)+k(15)*y(3)*y(501);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(501);
dydt(501)=dydt(501)-k(15)*y(3)*y(501);
dydt(308)=dydt(308)+k(15)*y(3)*y(501);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(501)=dydt(501)-k(8)*y(501);
dydt(315)=dydt(315)+k(8)*y(501);
dydt(111)=dydt(111)+k(8)*y(501);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

dydt(501)=dydt(501)-k(6)*y(501);
dydt(77)=dydt(77)+k(6)*y(501);
dydt(110)=dydt(110)+k(6)*y(501);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(502)=dydt(502)-k(34)*y(502);
dydt(499)=dydt(499)+k(34)*y(502);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(502)=dydt(502)-k(33)*y(502);
dydt(505)=dydt(505)+k(33)*y(502);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(502)=dydt(502)-k(31)*y(502);
dydt(427)=dydt(427)+k(31)*y(502);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(502)=dydt(502)-k(31)*y(502);
dydt(527)=dydt(527)+k(31)*y(502);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(502)=dydt(502)-k(30)*y(502);
dydt(490)=dydt(490)+k(30)*y(502);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(502)=dydt(502)-k(22)*y(502);
dydt(2)=dydt(2)+k(22)*y(502);
dydt(613)=dydt(613)+k(22)*y(502);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(502)=dydt(502)-k(22)*y(502);
dydt(197)=dydt(197)+k(22)*y(502);
dydt(501)=dydt(501)+k(22)*y(502);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(502);
dydt(502)=dydt(502)-k(15)*y(3)*y(502);
dydt(485)=dydt(485)+k(15)*y(3)*y(502);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(502);
dydt(502)=dydt(502)-k(15)*y(3)*y(502);
dydt(528)=dydt(528)+k(15)*y(3)*y(502);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(502)=dydt(502)-k(8)*y(502);
dydt(111)=dydt(111)+k(8)*y(502);
dydt(319)=dydt(319)+k(8)*y(502);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(502)=dydt(502)-k(6)*y(502);
dydt(257)=dydt(257)+k(6)*y(502);
dydt(110)=dydt(110)+k(6)*y(502);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(504)=dydt(504)-k(31)*y(504);
dydt(507)=dydt(507)+k(31)*y(504);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(504)=dydt(504)-k(28)*y(504);
dydt(625)=dydt(625)+k(28)*y(504);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(504)=dydt(504)-k(27)*y(504);
dydt(523)=dydt(523)+k(27)*y(504);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(504)=dydt(504)-k(25)*y(504);
dydt(500)=dydt(500)+k(25)*y(504);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(504)=dydt(504)-k(22)*y(504);
dydt(2)=dydt(2)+k(22)*y(504);
dydt(540)=dydt(540)+k(22)*y(504);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(504)=dydt(504)-k(22)*y(504);
dydt(2)=dydt(2)+k(22)*y(504);
dydt(308)=dydt(308)+k(22)*y(504);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(504)=dydt(504)-k(16)*y(504);
dydt(3)=dydt(3)+k(16)*y(504);
dydt(503)=dydt(503)+k(16)*y(504);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(504);
dydt(504)=dydt(504)-k(15)*y(3)*y(504);
dydt(630)=dydt(630)+k(15)*y(3)*y(504);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(504)=dydt(504)-k(8)*y(504);
dydt(437)=dydt(437)+k(8)*y(504);
dydt(43)=dydt(43)+k(8)*y(504);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(504)=dydt(504)-k(6)*y(504);
dydt(111)=dydt(111)+k(6)*y(504);
dydt(114)=dydt(114)+k(6)*y(504);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(505)=dydt(505)-k(34)*y(505);
dydt(503)=dydt(503)+k(34)*y(505);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(505)=dydt(505)-k(31)*y(505);
dydt(280)=dydt(280)+k(31)*y(505);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(505)=dydt(505)-k(31)*y(505);
dydt(509)=dydt(509)+k(31)*y(505);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(505)=dydt(505)-k(30)*y(505);
dydt(579)=dydt(579)+k(30)*y(505);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(505)=dydt(505)-k(22)*y(505);
dydt(2)=dydt(2)+k(22)*y(505);
dydt(576)=dydt(576)+k(22)*y(505);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(505)=dydt(505)-k(22)*y(505);
dydt(182)=dydt(182)+k(22)*y(505);
dydt(501)=dydt(501)+k(22)*y(505);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(505);
dydt(505)=dydt(505)-k(15)*y(3)*y(505);
dydt(281)=dydt(281)+k(15)*y(3)*y(505);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(505);
dydt(505)=dydt(505)-k(15)*y(3)*y(505);
dydt(523)=dydt(523)+k(15)*y(3)*y(505);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(505)=dydt(505)-k(8)*y(505);
dydt(111)=dydt(111)+k(8)*y(505);
dydt(219)=dydt(219)+k(8)*y(505);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(505)=dydt(505)-k(6)*y(505);
dydt(218)=dydt(218)+k(6)*y(505);
dydt(110)=dydt(110)+k(6)*y(505);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(507)=dydt(507)-k(28)*y(507);
dydt(511)=dydt(511)+k(28)*y(507);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(507)=dydt(507)-k(27)*y(507);
dydt(521)=dydt(521)+k(27)*y(507);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(507)=dydt(507)-k(25)*y(507);
dydt(530)=dydt(530)+k(25)*y(507);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(507)=dydt(507)-k(22)*y(507);
dydt(2)=dydt(2)+k(22)*y(507);
dydt(538)=dydt(538)+k(22)*y(507);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(507)=dydt(507)-k(22)*y(507);
dydt(2)=dydt(2)+k(22)*y(507);
dydt(524)=dydt(524)+k(22)*y(507);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(507)=dydt(507)-k(17)*y(507);
dydt(504)=dydt(504)+k(17)*y(507);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(507)=dydt(507)-k(16)*y(507);
dydt(3)=dydt(3)+k(16)*y(507);
dydt(506)=dydt(506)+k(16)*y(507);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(507);
dydt(507)=dydt(507)-k(9)*y(3)*y(507);
dydt(604)=dydt(604)+k(9)*y(3)*y(507);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(507)=dydt(507)-k(8)*y(507);
dydt(22)=dydt(22)+k(8)*y(507);
dydt(43)=dydt(43)+k(8)*y(507);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(507)=dydt(507)-k(6)*y(507);
dydt(21)=dydt(21)+k(6)*y(507);
dydt(114)=dydt(114)+k(6)*y(507);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(508)=dydt(508)-k(32)*y(508);
dydt(142)=dydt(142)+k(32)*y(508);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(508)=dydt(508)-k(31)*y(508);
dydt(452)=dydt(452)+k(31)*y(508);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(508)=dydt(508)-k(22)*y(508);
dydt(2)=dydt(2)+k(22)*y(508);
dydt(610)=dydt(610)+k(22)*y(508);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(508);
dydt(508)=dydt(508)-k(21)*y(197)*y(508);
dydt(545)=dydt(545)+k(21)*y(197)*y(508);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(508);
dydt(508)=dydt(508)-k(21)*y(32)*y(508);
dydt(575)=dydt(575)+k(21)*y(32)*y(508);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(508);
dydt(508)=dydt(508)-k(21)*y(2)*y(508);
dydt(506)=dydt(506)+k(21)*y(2)*y(508);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(508);
dydt(508)=dydt(508)-k(15)*y(3)*y(508);
dydt(538)=dydt(538)+k(15)*y(3)*y(508);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(508);
dydt(508)=dydt(508)-k(9)*y(3)*y(508);
dydt(143)=dydt(143)+k(9)*y(3)*y(508);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

dydt(508)=dydt(508)-k(8)*y(508);
dydt(77)=dydt(77)+k(8)*y(508);
dydt(22)=dydt(22)+k(8)*y(508);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(508)=dydt(508)-k(6)*y(508);
dydt(76)=dydt(76)+k(6)*y(508);
dydt(21)=dydt(21)+k(6)*y(508);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(509)=dydt(509)-k(34)*y(509);
dydt(506)=dydt(506)+k(34)*y(509);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(509)=dydt(509)-k(31)*y(509);
dydt(377)=dydt(377)+k(31)*y(509);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(509)=dydt(509)-k(30)*y(509);
dydt(513)=dydt(513)+k(30)*y(509);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(509)=dydt(509)-k(22)*y(509);
dydt(2)=dydt(2)+k(22)*y(509);
dydt(344)=dydt(344)+k(22)*y(509);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(509)=dydt(509)-k(22)*y(509);
dydt(182)=dydt(182)+k(22)*y(509);
dydt(354)=dydt(354)+k(22)*y(509);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(509);
dydt(509)=dydt(509)-k(15)*y(3)*y(509);
dydt(521)=dydt(521)+k(15)*y(3)*y(509);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(509);
dydt(509)=dydt(509)-k(9)*y(3)*y(509);
dydt(346)=dydt(346)+k(9)*y(3)*y(509);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(509)=dydt(509)-k(8)*y(509);
dydt(111)=dydt(111)+k(8)*y(509);
dydt(214)=dydt(214)+k(8)*y(509);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(509)=dydt(509)-k(6)*y(509);
dydt(177)=dydt(177)+k(6)*y(509);
dydt(110)=dydt(110)+k(6)*y(509);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(511)=dydt(511)-k(34)*y(511);
dydt(507)=dydt(507)+k(34)*y(511);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(511)=dydt(511)-k(29)*y(511);
dydt(514)=dydt(514)+k(29)*y(511);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(511)=dydt(511)-k(25)*y(511);
dydt(532)=dydt(532)+k(25)*y(511);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(511)=dydt(511)-k(22)*y(511);
dydt(182)=dydt(182)+k(22)*y(511);
dydt(538)=dydt(538)+k(22)*y(511);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(511)=dydt(511)-k(22)*y(511);
dydt(2)=dydt(2)+k(22)*y(511);
dydt(518)=dydt(518)+k(22)*y(511);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(511)=dydt(511)-k(17)*y(511);
dydt(625)=dydt(625)+k(17)*y(511);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(511)=dydt(511)-k(16)*y(511);
dydt(3)=dydt(3)+k(16)*y(511);
dydt(510)=dydt(510)+k(16)*y(511);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(511);
dydt(511)=dydt(511)-k(9)*y(3)*y(511);
dydt(596)=dydt(596)+k(9)*y(3)*y(511);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(511)=dydt(511)-k(8)*y(511);
dydt(22)=dydt(22)+k(8)*y(511);
dydt(222)=dydt(222)+k(8)*y(511);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(511)=dydt(511)-k(6)*y(511);
dydt(21)=dydt(21)+k(6)*y(511);
dydt(515)=dydt(515)+k(6)*y(511);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(648)=dydt(648)-k(34)*y(648);
dydt(375)=dydt(375)+k(34)*y(648);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(648)=dydt(648)-k(32)*y(648);
dydt(418)=dydt(418)+k(32)*y(648);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(648)=dydt(648)-k(22)*y(648);
dydt(182)=dydt(182)+k(22)*y(648);
dydt(150)=dydt(150)+k(22)*y(648);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(648);
dydt(648)=dydt(648)-k(21)*y(197)*y(648);
dydt(369)=dydt(369)+k(21)*y(197)*y(648);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(648);
dydt(648)=dydt(648)-k(21)*y(32)*y(648);
dydt(366)=dydt(366)+k(21)*y(32)*y(648);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(648);
dydt(648)=dydt(648)-k(21)*y(2)*y(648);
dydt(179)=dydt(179)+k(21)*y(2)*y(648);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(648);
dydt(648)=dydt(648)-k(9)*y(3)*y(648);
dydt(181)=dydt(181)+k(9)*y(3)*y(648);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(648);
dydt(648)=dydt(648)-k(9)*y(3)*y(648);
dydt(393)=dydt(393)+k(9)*y(3)*y(648);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(648)=dydt(648)-k(8)*y(648);
dydt(16)=dydt(16)+k(8)*y(648);
dydt(177)=dydt(177)+k(8)*y(648);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

dydt(648)=dydt(648)-k(6)*y(648);
dydt(15)=dydt(15)+k(6)*y(648);
dydt(176)=dydt(176)+k(6)*y(648);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(512)=dydt(512)-k(34)*y(512);
dydt(354)=dydt(354)+k(34)*y(512);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(512)=dydt(512)-k(32)*y(512);
dydt(304)=dydt(304)+k(32)*y(512);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(512)=dydt(512)-k(31)*y(512);
dydt(648)=dydt(648)+k(31)*y(512);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(512)=dydt(512)-k(22)*y(512);
dydt(182)=dydt(182)+k(22)*y(512);
dydt(610)=dydt(610)+k(22)*y(512);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(512);
dydt(512)=dydt(512)-k(21)*y(197)*y(512);
dydt(611)=dydt(611)+k(21)*y(197)*y(512);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(512);
dydt(512)=dydt(512)-k(21)*y(32)*y(512);
dydt(359)=dydt(359)+k(21)*y(32)*y(512);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(512);
dydt(512)=dydt(512)-k(21)*y(2)*y(512);
dydt(510)=dydt(510)+k(21)*y(2)*y(512);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(512);
dydt(512)=dydt(512)-k(15)*y(3)*y(512);
dydt(518)=dydt(518)+k(15)*y(3)*y(512);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(512);
dydt(512)=dydt(512)-k(9)*y(3)*y(512);
dydt(361)=dydt(361)+k(9)*y(3)*y(512);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(512)=dydt(512)-k(8)*y(512);
dydt(16)=dydt(16)+k(8)*y(512);
dydt(218)=dydt(218)+k(8)*y(512);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

dydt(512)=dydt(512)-k(6)*y(512);
dydt(15)=dydt(15)+k(6)*y(512);
dydt(264)=dydt(264)+k(6)*y(512);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(515)=dydt(515)-k(34)*y(515);
dydt(114)=dydt(114)+k(34)*y(515);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

dydt(515)=dydt(515)-k(22)*y(515);
dydt(182)=dydt(182)+k(22)*y(515);
dydt(118)=dydt(118)+k(22)*y(515);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(515)=dydt(515)-k(16)*y(515);
dydt(3)=dydt(3)+k(16)*y(515);
dydt(264)=dydt(264)+k(16)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(1)=dydt(1)-k(5)*y(1)*y(515);
dydt(515)=dydt(515)-k(5)*y(1)*y(515);
dydt(591)=dydt(591)+k(5)*y(1)*y(515);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(7)=dydt(7)-k(5)*y(7)*y(515);
dydt(515)=dydt(515)-k(5)*y(7)*y(515);
dydt(593)=dydt(593)+k(5)*y(7)*y(515);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) 

dydt(515)=dydt(515)-k(4)*y(515);
dydt(4)=dydt(4)+k(4)*y(515);
dydt(222)=dydt(222)+k(4)*y(515);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(516)=dydt(516)-k(34)*y(516);
dydt(596)=dydt(596)+k(34)*y(516);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(516)=dydt(516)-k(34)*y(516);
dydt(522)=dydt(522)+k(34)*y(516);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(516)=dydt(516)-k(25)*y(516);
dydt(597)=dydt(597)+k(25)*y(516);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(516)=dydt(516)-k(24)*y(516);
dydt(549)=dydt(549)+k(24)*y(516);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(516)=dydt(516)-k(22)*y(516);
dydt(182)=dydt(182)+k(22)*y(516);
dydt(595)=dydt(595)+k(22)*y(516);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

dydt(516)=dydt(516)-k(22)*y(516);
dydt(182)=dydt(182)+k(22)*y(516);
dydt(345)=dydt(345)+k(22)*y(516);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(516)=dydt(516)-k(16)*y(516);
dydt(3)=dydt(3)+k(16)*y(516);
dydt(188)=dydt(188)+k(16)*y(516);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(516)=dydt(516)-k(10)*y(516);
dydt(3)=dydt(3)+k(10)*y(516);
dydt(514)=dydt(514)+k(10)*y(516);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(516)=dydt(516)-k(8)*y(516);
dydt(222)=dydt(222)+k(8)*y(516);
dydt(185)=dydt(185)+k(8)*y(516);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(516)=dydt(516)-k(6)*y(516);
dydt(184)=dydt(184)+k(6)*y(516);
dydt(515)=dydt(515)+k(6)*y(516);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(517)=dydt(517)-k(34)*y(517);
dydt(523)=dydt(523)+k(34)*y(517);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(517)=dydt(517)-k(34)*y(517);
dydt(625)=dydt(625)+k(34)*y(517);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(517)=dydt(517)-k(31)*y(517);
dydt(514)=dydt(514)+k(31)*y(517);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(517)=dydt(517)-k(25)*y(517);
dydt(626)=dydt(626)+k(25)*y(517);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(517)=dydt(517)-k(22)*y(517);
dydt(182)=dydt(182)+k(22)*y(517);
dydt(644)=dydt(644)+k(22)*y(517);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(517)=dydt(517)-k(22)*y(517);
dydt(182)=dydt(182)+k(22)*y(517);
dydt(623)=dydt(623)+k(22)*y(517);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(517)=dydt(517)-k(16)*y(517);
dydt(3)=dydt(3)+k(16)*y(517);
dydt(579)=dydt(579)+k(16)*y(517);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(517);
dydt(517)=dydt(517)-k(15)*y(3)*y(517);
dydt(640)=dydt(640)+k(15)*y(3)*y(517);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(517)=dydt(517)-k(8)*y(517);
dydt(219)=dydt(219)+k(8)*y(517);
dydt(222)=dydt(222)+k(8)*y(517);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(517)=dydt(517)-k(6)*y(517);
dydt(218)=dydt(218)+k(6)*y(517);
dydt(515)=dydt(515)+k(6)*y(517);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(518)=dydt(518)-k(34)*y(518);
dydt(524)=dydt(524)+k(34)*y(518);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(518)=dydt(518)-k(32)*y(518);
dydt(591)=dydt(591)+k(32)*y(518);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

dydt(518)=dydt(518)-k(22)*y(518);
dydt(182)=dydt(182)+k(22)*y(518);
dydt(537)=dydt(537)+k(22)*y(518);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(518);
dydt(518)=dydt(518)-k(21)*y(197)*y(518);
dydt(520)=dydt(520)+k(21)*y(197)*y(518);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(518);
dydt(518)=dydt(518)-k(21)*y(32)*y(518);
dydt(532)=dydt(532)+k(21)*y(32)*y(518);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(518);
dydt(518)=dydt(518)-k(21)*y(2)*y(518);
dydt(511)=dydt(511)+k(21)*y(2)*y(518);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(518)=dydt(518)-k(17)*y(518);
dydt(623)=dydt(623)+k(17)*y(518);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(518)=dydt(518)-k(16)*y(518);
dydt(3)=dydt(3)+k(16)*y(518);
dydt(512)=dydt(512)+k(16)*y(518);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(518);
dydt(518)=dydt(518)-k(9)*y(3)*y(518);
dydt(595)=dydt(595)+k(9)*y(3)*y(518);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(518)=dydt(518)-k(8)*y(518);
dydt(16)=dydt(16)+k(8)*y(518);
dydt(222)=dydt(222)+k(8)*y(518);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

dydt(518)=dydt(518)-k(6)*y(518);
dydt(15)=dydt(15)+k(6)*y(518);
dydt(515)=dydt(515)+k(6)*y(518);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(519)=dydt(519)-k(34)*y(519);
dydt(538)=dydt(538)+k(34)*y(519);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(519)=dydt(519)-k(32)*y(519);
dydt(326)=dydt(326)+k(32)*y(519);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(519)=dydt(519)-k(25)*y(519);
dydt(529)=dydt(529)+k(25)*y(519);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

dydt(519)=dydt(519)-k(22)*y(519);
dydt(182)=dydt(182)+k(22)*y(519);
dydt(537)=dydt(537)+k(22)*y(519);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(519);
dydt(519)=dydt(519)-k(21)*y(197)*y(519);
dydt(548)=dydt(548)+k(21)*y(197)*y(519);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(519);
dydt(519)=dydt(519)-k(21)*y(32)*y(519);
dydt(553)=dydt(553)+k(21)*y(32)*y(519);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(519);
dydt(519)=dydt(519)-k(21)*y(2)*y(519);
dydt(521)=dydt(521)+k(21)*y(2)*y(519);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(519)=dydt(519)-k(17)*y(519);
dydt(644)=dydt(644)+k(17)*y(519);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(519)=dydt(519)-k(16)*y(519);
dydt(3)=dydt(3)+k(16)*y(519);
dydt(344)=dydt(344)+k(16)*y(519);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(519);
dydt(519)=dydt(519)-k(9)*y(3)*y(519);
dydt(345)=dydt(345)+k(9)*y(3)*y(519);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(519)=dydt(519)-k(8)*y(519);
dydt(54)=dydt(54)+k(8)*y(519);
dydt(214)=dydt(214)+k(8)*y(519);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(519)=dydt(519)-k(6)*y(519);
dydt(118)=dydt(118)+k(6)*y(519);
dydt(177)=dydt(177)+k(6)*y(519);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(520)=dydt(520)-k(34)*y(520);
dydt(525)=dydt(525)+k(34)*y(520);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(520)=dydt(520)-k(34)*y(520);
dydt(532)=dydt(532)+k(34)*y(520);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(520)=dydt(520)-k(33)*y(520);
dydt(514)=dydt(514)+k(33)*y(520);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(520)=dydt(520)-k(22)*y(520);
dydt(182)=dydt(182)+k(22)*y(520);
dydt(529)=dydt(529)+k(22)*y(520);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(520)=dydt(520)-k(22)*y(520);
dydt(197)=dydt(197)+k(22)*y(520);
dydt(518)=dydt(518)+k(22)*y(520);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(520)=dydt(520)-k(17)*y(520);
dydt(626)=dydt(626)+k(17)*y(520);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(520)=dydt(520)-k(16)*y(520);
dydt(3)=dydt(3)+k(16)*y(520);
dydt(611)=dydt(611)+k(16)*y(520);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(520);
dydt(520)=dydt(520)-k(9)*y(3)*y(520);
dydt(597)=dydt(597)+k(9)*y(3)*y(520);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(520)=dydt(520)-k(8)*y(520);
dydt(209)=dydt(209)+k(8)*y(520);
dydt(222)=dydt(222)+k(8)*y(520);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(520)=dydt(520)-k(6)*y(520);
dydt(192)=dydt(192)+k(6)*y(520);
dydt(515)=dydt(515)+k(6)*y(520);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(522)=dydt(522)-k(34)*y(522);
dydt(604)=dydt(604)+k(34)*y(522);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(522)=dydt(522)-k(30)*y(522);
dydt(516)=dydt(516)+k(30)*y(522);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(522)=dydt(522)-k(25)*y(522);
dydt(526)=dydt(526)+k(25)*y(522);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(522)=dydt(522)-k(24)*y(522);
dydt(349)=dydt(349)+k(24)*y(522);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(522)=dydt(522)-k(22)*y(522);
dydt(182)=dydt(182)+k(22)*y(522);
dydt(116)=dydt(116)+k(22)*y(522);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

dydt(522)=dydt(522)-k(22)*y(522);
dydt(2)=dydt(2)+k(22)*y(522);
dydt(345)=dydt(345)+k(22)*y(522);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(522)=dydt(522)-k(16)*y(522);
dydt(3)=dydt(3)+k(16)*y(522);
dydt(346)=dydt(346)+k(16)*y(522);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(522)=dydt(522)-k(10)*y(522);
dydt(3)=dydt(3)+k(10)*y(522);
dydt(521)=dydt(521)+k(10)*y(522);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(522)=dydt(522)-k(8)*y(522);
dydt(43)=dydt(43)+k(8)*y(522);
dydt(185)=dydt(185)+k(8)*y(522);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(522)=dydt(522)-k(6)*y(522);
dydt(184)=dydt(184)+k(6)*y(522);
dydt(114)=dydt(114)+k(6)*y(522);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(523)=dydt(523)-k(34)*y(523);
dydt(504)=dydt(504)+k(34)*y(523);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(523)=dydt(523)-k(31)*y(523);
dydt(521)=dydt(521)+k(31)*y(523);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(523)=dydt(523)-k(30)*y(523);
dydt(517)=dydt(517)+k(30)*y(523);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(523)=dydt(523)-k(25)*y(523);
dydt(528)=dydt(528)+k(25)*y(523);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(523)=dydt(523)-k(22)*y(523);
dydt(2)=dydt(2)+k(22)*y(523);
dydt(644)=dydt(644)+k(22)*y(523);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(523)=dydt(523)-k(22)*y(523);
dydt(182)=dydt(182)+k(22)*y(523);
dydt(308)=dydt(308)+k(22)*y(523);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(523)=dydt(523)-k(16)*y(523);
dydt(3)=dydt(3)+k(16)*y(523);
dydt(505)=dydt(505)+k(16)*y(523);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(523);
dydt(523)=dydt(523)-k(15)*y(3)*y(523);
dydt(637)=dydt(637)+k(15)*y(3)*y(523);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(523)=dydt(523)-k(8)*y(523);
dydt(219)=dydt(219)+k(8)*y(523);
dydt(43)=dydt(43)+k(8)*y(523);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(523)=dydt(523)-k(6)*y(523);
dydt(218)=dydt(218)+k(6)*y(523);
dydt(114)=dydt(114)+k(6)*y(523);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(524)=dydt(524)-k(32)*y(524);
dydt(115)=dydt(115)+k(32)*y(524);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

dydt(524)=dydt(524)-k(22)*y(524);
dydt(2)=dydt(2)+k(22)*y(524);
dydt(537)=dydt(537)+k(22)*y(524);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(524);
dydt(524)=dydt(524)-k(21)*y(197)*y(524);
dydt(525)=dydt(525)+k(21)*y(197)*y(524);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(524);
dydt(524)=dydt(524)-k(21)*y(32)*y(524);
dydt(530)=dydt(530)+k(21)*y(32)*y(524);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(524);
dydt(524)=dydt(524)-k(21)*y(2)*y(524);
dydt(507)=dydt(507)+k(21)*y(2)*y(524);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(524)=dydt(524)-k(17)*y(524);
dydt(308)=dydt(308)+k(17)*y(524);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(524)=dydt(524)-k(16)*y(524);
dydt(3)=dydt(3)+k(16)*y(524);
dydt(354)=dydt(354)+k(16)*y(524);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(524);
dydt(524)=dydt(524)-k(9)*y(3)*y(524);
dydt(116)=dydt(116)+k(9)*y(3)*y(524);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(524)=dydt(524)-k(8)*y(524);
dydt(16)=dydt(16)+k(8)*y(524);
dydt(43)=dydt(43)+k(8)*y(524);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

dydt(524)=dydt(524)-k(6)*y(524);
dydt(15)=dydt(15)+k(6)*y(524);
dydt(114)=dydt(114)+k(6)*y(524);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(526)=dydt(526)-k(34)*y(526);
dydt(357)=dydt(357)+k(34)*y(526);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(526)=dydt(526)-k(33)*y(526);
dydt(522)=dydt(522)+k(33)*y(526);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(526)=dydt(526)-k(30)*y(526);
dydt(597)=dydt(597)+k(30)*y(526);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(526)=dydt(526)-k(24)*y(526);
dydt(558)=dydt(558)+k(24)*y(526);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(526)=dydt(526)-k(22)*y(526);
dydt(197)=dydt(197)+k(22)*y(526);
dydt(116)=dydt(116)+k(22)*y(526);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

dydt(526)=dydt(526)-k(22)*y(526);
dydt(2)=dydt(2)+k(22)*y(526);
dydt(339)=dydt(339)+k(22)*y(526);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(526)=dydt(526)-k(16)*y(526);
dydt(3)=dydt(3)+k(16)*y(526);
dydt(340)=dydt(340)+k(16)*y(526);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(526)=dydt(526)-k(10)*y(526);
dydt(3)=dydt(3)+k(10)*y(526);
dydt(525)=dydt(525)+k(10)*y(526);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(526)=dydt(526)-k(8)*y(526);
dydt(43)=dydt(43)+k(8)*y(526);
dydt(336)=dydt(336)+k(8)*y(526);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(526)=dydt(526)-k(6)*y(526);
dydt(335)=dydt(335)+k(6)*y(526);
dydt(114)=dydt(114)+k(6)*y(526);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(527)=dydt(527)-k(34)*y(527);
dydt(356)=dydt(356)+k(34)*y(527);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(527)=dydt(527)-k(33)*y(527);
dydt(509)=dydt(509)+k(33)*y(527);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(527)=dydt(527)-k(31)*y(527);
dydt(373)=dydt(373)+k(31)*y(527);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(527)=dydt(527)-k(30)*y(527);
dydt(611)=dydt(611)+k(30)*y(527);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(527)=dydt(527)-k(22)*y(527);
dydt(2)=dydt(2)+k(22)*y(527);
dydt(338)=dydt(338)+k(22)*y(527);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(527)=dydt(527)-k(22)*y(527);
dydt(197)=dydt(197)+k(22)*y(527);
dydt(354)=dydt(354)+k(22)*y(527);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(527);
dydt(527)=dydt(527)-k(15)*y(3)*y(527);
dydt(525)=dydt(525)+k(15)*y(3)*y(527);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(527);
dydt(527)=dydt(527)-k(9)*y(3)*y(527);
dydt(340)=dydt(340)+k(9)*y(3)*y(527);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(527)=dydt(527)-k(8)*y(527);
dydt(111)=dydt(111)+k(8)*y(527);
dydt(209)=dydt(209)+k(8)*y(527);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(527)=dydt(527)-k(6)*y(527);
dydt(192)=dydt(192)+k(6)*y(527);
dydt(110)=dydt(110)+k(6)*y(527);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(528)=dydt(528)-k(34)*y(528);
dydt(500)=dydt(500)+k(34)*y(528);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(528)=dydt(528)-k(33)*y(528);
dydt(523)=dydt(523)+k(33)*y(528);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(528)=dydt(528)-k(31)*y(528);
dydt(525)=dydt(525)+k(31)*y(528);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(528)=dydt(528)-k(30)*y(528);
dydt(626)=dydt(626)+k(30)*y(528);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(528)=dydt(528)-k(22)*y(528);
dydt(2)=dydt(2)+k(22)*y(528);
dydt(564)=dydt(564)+k(22)*y(528);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(528)=dydt(528)-k(22)*y(528);
dydt(197)=dydt(197)+k(22)*y(528);
dydt(308)=dydt(308)+k(22)*y(528);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(528)=dydt(528)-k(16)*y(528);
dydt(3)=dydt(3)+k(16)*y(528);
dydt(502)=dydt(502)+k(16)*y(528);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(528);
dydt(528)=dydt(528)-k(15)*y(3)*y(528);
dydt(632)=dydt(632)+k(15)*y(3)*y(528);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(528)=dydt(528)-k(8)*y(528);
dydt(319)=dydt(319)+k(8)*y(528);
dydt(43)=dydt(43)+k(8)*y(528);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(528)=dydt(528)-k(6)*y(528);
dydt(257)=dydt(257)+k(6)*y(528);
dydt(114)=dydt(114)+k(6)*y(528);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(529)=dydt(529)-k(34)*y(529);
dydt(531)=dydt(531)+k(34)*y(529);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(529)=dydt(529)-k(33)*y(529);
dydt(519)=dydt(519)+k(33)*y(529);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(529)=dydt(529)-k(32)*y(529);
dydt(322)=dydt(322)+k(32)*y(529);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

dydt(529)=dydt(529)-k(22)*y(529);
dydt(197)=dydt(197)+k(22)*y(529);
dydt(537)=dydt(537)+k(22)*y(529);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(529);
dydt(529)=dydt(529)-k(21)*y(197)*y(529);
dydt(552)=dydt(552)+k(21)*y(197)*y(529);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(529);
dydt(529)=dydt(529)-k(21)*y(32)*y(529);
dydt(557)=dydt(557)+k(21)*y(32)*y(529);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(529);
dydt(529)=dydt(529)-k(21)*y(2)*y(529);
dydt(525)=dydt(525)+k(21)*y(2)*y(529);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(529)=dydt(529)-k(17)*y(529);
dydt(564)=dydt(564)+k(17)*y(529);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(529)=dydt(529)-k(16)*y(529);
dydt(3)=dydt(3)+k(16)*y(529);
dydt(338)=dydt(338)+k(16)*y(529);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(529);
dydt(529)=dydt(529)-k(9)*y(3)*y(529);
dydt(339)=dydt(339)+k(9)*y(3)*y(529);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(529)=dydt(529)-k(8)*y(529);
dydt(54)=dydt(54)+k(8)*y(529);
dydt(209)=dydt(209)+k(8)*y(529);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(529)=dydt(529)-k(6)*y(529);
dydt(118)=dydt(118)+k(6)*y(529);
dydt(192)=dydt(192)+k(6)*y(529);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(533)=dydt(533)-k(33)*y(533);
dydt(508)=dydt(508)+k(33)*y(533);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(533)=dydt(533)-k(32)*y(533);
dydt(136)=dydt(136)+k(32)*y(533);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(533)=dydt(533)-k(31)*y(533);
dydt(156)=dydt(156)+k(31)*y(533);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(533)=dydt(533)-k(22)*y(533);
dydt(32)=dydt(32)+k(22)*y(533);
dydt(610)=dydt(610)+k(22)*y(533);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(533);
dydt(533)=dydt(533)-k(21)*y(197)*y(533);
dydt(608)=dydt(608)+k(21)*y(197)*y(533);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(533);
dydt(533)=dydt(533)-k(21)*y(32)*y(533);
dydt(612)=dydt(612)+k(21)*y(32)*y(533);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(533);
dydt(533)=dydt(533)-k(21)*y(2)*y(533);
dydt(356)=dydt(356)+k(21)*y(2)*y(533);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(533);
dydt(533)=dydt(533)-k(15)*y(3)*y(533);
dydt(531)=dydt(531)+k(15)*y(3)*y(533);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(533);
dydt(533)=dydt(533)-k(9)*y(3)*y(533);
dydt(137)=dydt(137)+k(9)*y(3)*y(533);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

dydt(533)=dydt(533)-k(8)*y(533);
dydt(77)=dydt(77)+k(8)*y(533);
dydt(28)=dydt(28)+k(8)*y(533);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(533)=dydt(533)-k(6)*y(533);
dydt(76)=dydt(76)+k(6)*y(533);
dydt(27)=dydt(27)+k(6)*y(533);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(534)=dydt(534)-k(33)*y(534);
dydt(540)=dydt(540)+k(33)*y(534);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(534)=dydt(534)-k(32)*y(534);
dydt(317)=dydt(317)+k(32)*y(534);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(534)=dydt(534)-k(31)*y(534);
dydt(531)=dydt(531)+k(31)*y(534);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

dydt(534)=dydt(534)-k(22)*y(534);
dydt(32)=dydt(32)+k(22)*y(534);
dydt(310)=dydt(310)+k(22)*y(534);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(534);
dydt(534)=dydt(534)-k(21)*y(197)*y(534);
dydt(569)=dydt(569)+k(21)*y(197)*y(534);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(534);
dydt(534)=dydt(534)-k(21)*y(32)*y(534);
dydt(566)=dydt(566)+k(21)*y(32)*y(534);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(534);
dydt(534)=dydt(534)-k(21)*y(2)*y(534);
dydt(500)=dydt(500)+k(21)*y(2)*y(534);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(534)=dydt(534)-k(16)*y(534);
dydt(3)=dydt(3)+k(16)*y(534);
dydt(245)=dydt(245)+k(16)*y(534);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(534);
dydt(534)=dydt(534)-k(15)*y(3)*y(534);
dydt(244)=dydt(244)+k(15)*y(3)*y(534);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(534)=dydt(534)-k(8)*y(534);
dydt(54)=dydt(54)+k(8)*y(534);
dydt(34)=dydt(34)+k(8)*y(534);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(534)=dydt(534)-k(6)*y(534);
dydt(118)=dydt(118)+k(6)*y(534);
dydt(33)=dydt(33)+k(6)*y(534);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(535)=dydt(535)-k(33)*y(535);
dydt(530)=dydt(530)+k(33)*y(535);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(535)=dydt(535)-k(33)*y(535);
dydt(541)=dydt(541)+k(33)*y(535);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(535)=dydt(535)-k(28)*y(535);
dydt(536)=dydt(536)+k(28)*y(535);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(535)=dydt(535)-k(27)*y(535);
dydt(557)=dydt(557)+k(27)*y(535);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(535)=dydt(535)-k(22)*y(535);
dydt(32)=dydt(32)+k(22)*y(535);
dydt(531)=dydt(531)+k(22)*y(535);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(535)=dydt(535)-k(22)*y(535);
dydt(32)=dydt(32)+k(22)*y(535);
dydt(556)=dydt(556)+k(22)*y(535);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(535)=dydt(535)-k(17)*y(535);
dydt(566)=dydt(566)+k(17)*y(535);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(535)=dydt(535)-k(16)*y(535);
dydt(3)=dydt(3)+k(16)*y(535);
dydt(612)=dydt(612)+k(16)*y(535);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(535);
dydt(535)=dydt(535)-k(9)*y(3)*y(535);
dydt(602)=dydt(602)+k(9)*y(3)*y(535);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(535)=dydt(535)-k(8)*y(535);
dydt(28)=dydt(28)+k(8)*y(535);
dydt(37)=dydt(37)+k(8)*y(535);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(535)=dydt(535)-k(6)*y(535);
dydt(27)=dydt(27)+k(6)*y(535);
dydt(554)=dydt(554)+k(6)*y(535);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(536)=dydt(536)-k(34)*y(536);
dydt(535)=dydt(535)+k(34)*y(536);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(536)=dydt(536)-k(33)*y(536);
dydt(532)=dydt(532)+k(33)*y(536);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(536)=dydt(536)-k(33)*y(536);
dydt(542)=dydt(542)+k(33)*y(536);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(536)=dydt(536)-k(29)*y(536);
dydt(552)=dydt(552)+k(29)*y(536);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(536)=dydt(536)-k(22)*y(536);
dydt(197)=dydt(197)+k(22)*y(536);
dydt(531)=dydt(531)+k(22)*y(536);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(536)=dydt(536)-k(22)*y(536);
dydt(32)=dydt(32)+k(22)*y(536);
dydt(547)=dydt(547)+k(22)*y(536);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(536)=dydt(536)-k(17)*y(536);
dydt(569)=dydt(569)+k(17)*y(536);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(536)=dydt(536)-k(16)*y(536);
dydt(3)=dydt(3)+k(16)*y(536);
dydt(608)=dydt(608)+k(16)*y(536);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(536);
dydt(536)=dydt(536)-k(9)*y(3)*y(536);
dydt(600)=dydt(600)+k(9)*y(3)*y(536);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(536)=dydt(536)-k(8)*y(536);
dydt(28)=dydt(28)+k(8)*y(536);
dydt(227)=dydt(227)+k(8)*y(536);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(536)=dydt(536)-k(6)*y(536);
dydt(27)=dydt(27)+k(6)*y(536);
dydt(543)=dydt(543)+k(6)*y(536);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(537)=dydt(537)-k(32)*y(537);
dydt(127)=dydt(127)+k(32)*y(537);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(537)=dydt(537)-k(32)*y(537);
dydt(119)=dydt(119)+k(32)*y(537);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(537);
dydt(537)=dydt(537)-k(21)*y(197)*y(537);
dydt(529)=dydt(529)+k(21)*y(197)*y(537);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(537);
dydt(537)=dydt(537)-k(21)*y(32)*y(537);
dydt(531)=dydt(531)+k(21)*y(32)*y(537);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(537);
dydt(537)=dydt(537)-k(21)*y(2)*y(537);
dydt(538)=dydt(538)+k(21)*y(2)*y(537);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(537);
dydt(537)=dydt(537)-k(21)*y(197)*y(537);
dydt(547)=dydt(547)+k(21)*y(197)*y(537);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(537);
dydt(537)=dydt(537)-k(21)*y(32)*y(537);
dydt(556)=dydt(556)+k(21)*y(32)*y(537);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(537);
dydt(537)=dydt(537)-k(21)*y(2)*y(537);
dydt(524)=dydt(524)+k(21)*y(2)*y(537);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(537)=dydt(537)-k(17)*y(537);
dydt(310)=dydt(310)+k(17)*y(537);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

dydt(537)=dydt(537)-k(16)*y(537);
dydt(3)=dydt(3)+k(16)*y(537);
dydt(610)=dydt(610)+k(16)*y(537);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(537);
dydt(537)=dydt(537)-k(9)*y(3)*y(537);
dydt(120)=dydt(120)+k(9)*y(3)*y(537);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(537)=dydt(537)-k(8)*y(537);
dydt(16)=dydt(16)+k(8)*y(537);
dydt(54)=dydt(54)+k(8)*y(537);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) 

dydt(537)=dydt(537)-k(6)*y(537);
dydt(15)=dydt(15)+k(6)*y(537);
dydt(118)=dydt(118)+k(6)*y(537);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(539)=dydt(539)-k(32)*y(539);
dydt(129)=dydt(129)+k(32)*y(539);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(539)=dydt(539)-k(25)*y(539);
dydt(134)=dydt(134)+k(25)*y(539);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(539)=dydt(539)-k(22)*y(539);
dydt(2)=dydt(2)+k(22)*y(539);
dydt(120)=dydt(120)+k(22)*y(539);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(539);
dydt(539)=dydt(539)-k(21)*y(197)*y(539);
dydt(544)=dydt(544)+k(21)*y(197)*y(539);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(539);
dydt(539)=dydt(539)-k(21)*y(32)*y(539);
dydt(603)=dydt(603)+k(21)*y(32)*y(539);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(539);
dydt(539)=dydt(539)-k(21)*y(2)*y(539);
dydt(604)=dydt(604)+k(21)*y(2)*y(539);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(539)=dydt(539)-k(16)*y(539);
dydt(3)=dydt(3)+k(16)*y(539);
dydt(143)=dydt(143)+k(16)*y(539);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(539)=dydt(539)-k(10)*y(539);
dydt(3)=dydt(3)+k(10)*y(539);
dydt(538)=dydt(538)+k(10)*y(539);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(539)=dydt(539)-k(8)*y(539);
dydt(54)=dydt(54)+k(8)*y(539);
dydt(141)=dydt(141)+k(8)*y(539);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(539)=dydt(539)-k(6)*y(539);
dydt(118)=dydt(118)+k(6)*y(539);
dydt(140)=dydt(140)+k(6)*y(539);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(540)=dydt(540)-k(32)*y(540);
dydt(316)=dydt(316)+k(32)*y(540);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(540)=dydt(540)-k(31)*y(540);
dydt(538)=dydt(538)+k(31)*y(540);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(540)=dydt(540)-k(25)*y(540);
dydt(534)=dydt(534)+k(25)*y(540);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

dydt(540)=dydt(540)-k(22)*y(540);
dydt(2)=dydt(2)+k(22)*y(540);
dydt(310)=dydt(310)+k(22)*y(540);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(540);
dydt(540)=dydt(540)-k(21)*y(197)*y(540);
dydt(546)=dydt(546)+k(21)*y(197)*y(540);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(540);
dydt(540)=dydt(540)-k(21)*y(32)*y(540);
dydt(570)=dydt(570)+k(21)*y(32)*y(540);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(540);
dydt(540)=dydt(540)-k(21)*y(2)*y(540);
dydt(504)=dydt(504)+k(21)*y(2)*y(540);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(540)=dydt(540)-k(16)*y(540);
dydt(3)=dydt(3)+k(16)*y(540);
dydt(252)=dydt(252)+k(16)*y(540);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(540);
dydt(540)=dydt(540)-k(15)*y(3)*y(540);
dydt(251)=dydt(251)+k(15)*y(3)*y(540);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(540)=dydt(540)-k(8)*y(540);
dydt(54)=dydt(54)+k(8)*y(540);
dydt(437)=dydt(437)+k(8)*y(540);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(540)=dydt(540)-k(6)*y(540);
dydt(118)=dydt(118)+k(6)*y(540);
dydt(111)=dydt(111)+k(6)*y(540);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(541)=dydt(541)-k(33)*y(541);
dydt(507)=dydt(507)+k(33)*y(541);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(541)=dydt(541)-k(28)*y(541);
dydt(542)=dydt(542)+k(28)*y(541);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(541)=dydt(541)-k(27)*y(541);
dydt(553)=dydt(553)+k(27)*y(541);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(541)=dydt(541)-k(25)*y(541);
dydt(535)=dydt(535)+k(25)*y(541);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(541)=dydt(541)-k(22)*y(541);
dydt(32)=dydt(32)+k(22)*y(541);
dydt(538)=dydt(538)+k(22)*y(541);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(541)=dydt(541)-k(22)*y(541);
dydt(2)=dydt(2)+k(22)*y(541);
dydt(556)=dydt(556)+k(22)*y(541);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(541)=dydt(541)-k(17)*y(541);
dydt(570)=dydt(570)+k(17)*y(541);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(541)=dydt(541)-k(16)*y(541);
dydt(3)=dydt(3)+k(16)*y(541);
dydt(575)=dydt(575)+k(16)*y(541);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(541);
dydt(541)=dydt(541)-k(9)*y(3)*y(541);
dydt(603)=dydt(603)+k(9)*y(3)*y(541);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(541)=dydt(541)-k(8)*y(541);
dydt(22)=dydt(22)+k(8)*y(541);
dydt(37)=dydt(37)+k(8)*y(541);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(541)=dydt(541)-k(6)*y(541);
dydt(21)=dydt(21)+k(6)*y(541);
dydt(554)=dydt(554)+k(6)*y(541);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(543)=dydt(543)-k(34)*y(543);
dydt(554)=dydt(554)+k(34)*y(543);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(543)=dydt(543)-k(33)*y(543);
dydt(515)=dydt(515)+k(33)*y(543);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

dydt(543)=dydt(543)-k(22)*y(543);
dydt(197)=dydt(197)+k(22)*y(543);
dydt(118)=dydt(118)+k(22)*y(543);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(543)=dydt(543)-k(16)*y(543);
dydt(3)=dydt(3)+k(16)*y(543);
dydt(256)=dydt(256)+k(16)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(1)=dydt(1)-k(5)*y(1)*y(543);
dydt(543)=dydt(543)-k(5)*y(1)*y(543);
dydt(589)=dydt(589)+k(5)*y(1)*y(543);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(7)=dydt(7)-k(5)*y(7)*y(543);
dydt(543)=dydt(543)-k(5)*y(7)*y(543);
dydt(201)=dydt(201)+k(5)*y(7)*y(543);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) 

dydt(543)=dydt(543)-k(4)*y(543);
dydt(4)=dydt(4)+k(4)*y(543);
dydt(227)=dydt(227)+k(4)*y(543);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(544)=dydt(544)-k(34)*y(544);
dydt(603)=dydt(603)+k(34)*y(544);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(544)=dydt(544)-k(33)*y(544);
dydt(596)=dydt(596)+k(33)*y(544);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(544)=dydt(544)-k(29)*y(544);
dydt(549)=dydt(549)+k(29)*y(544);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(544)=dydt(544)-k(25)*y(544);
dydt(600)=dydt(600)+k(25)*y(544);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(544)=dydt(544)-k(22)*y(544);
dydt(2)=dydt(2)+k(22)*y(544);
dydt(598)=dydt(598)+k(22)*y(544);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(544)=dydt(544)-k(22)*y(544);
dydt(197)=dydt(197)+k(22)*y(544);
dydt(539)=dydt(539)+k(22)*y(544);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(544)=dydt(544)-k(16)*y(544);
dydt(3)=dydt(3)+k(16)*y(544);
dydt(605)=dydt(605)+k(16)*y(544);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(544)=dydt(544)-k(10)*y(544);
dydt(3)=dydt(3)+k(10)*y(544);
dydt(542)=dydt(542)+k(10)*y(544);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(544)=dydt(544)-k(8)*y(544);
dydt(227)=dydt(227)+k(8)*y(544);
dydt(141)=dydt(141)+k(8)*y(544);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(544)=dydt(544)-k(6)*y(544);
dydt(140)=dydt(140)+k(6)*y(544);
dydt(543)=dydt(543)+k(6)*y(544);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(545)=dydt(545)-k(34)*y(545);
dydt(575)=dydt(575)+k(34)*y(545);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(545)=dydt(545)-k(33)*y(545);
dydt(510)=dydt(510)+k(33)*y(545);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(545)=dydt(545)-k(31)*y(545);
dydt(453)=dydt(453)+k(31)*y(545);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(545)=dydt(545)-k(29)*y(545);
dydt(550)=dydt(550)+k(29)*y(545);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(545)=dydt(545)-k(22)*y(545);
dydt(197)=dydt(197)+k(22)*y(545);
dydt(508)=dydt(508)+k(22)*y(545);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(545)=dydt(545)-k(22)*y(545);
dydt(2)=dydt(2)+k(22)*y(545);
dydt(585)=dydt(585)+k(22)*y(545);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(545);
dydt(545)=dydt(545)-k(15)*y(3)*y(545);
dydt(542)=dydt(542)+k(15)*y(3)*y(545);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(545);
dydt(545)=dydt(545)-k(9)*y(3)*y(545);
dydt(605)=dydt(605)+k(9)*y(3)*y(545);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(545)=dydt(545)-k(8)*y(545);
dydt(257)=dydt(257)+k(8)*y(545);
dydt(22)=dydt(22)+k(8)*y(545);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(545)=dydt(545)-k(6)*y(545);
dydt(21)=dydt(21)+k(6)*y(545);
dydt(256)=dydt(256)+k(6)*y(545);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(546)=dydt(546)-k(34)*y(546);
dydt(570)=dydt(570)+k(34)*y(546);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(546)=dydt(546)-k(33)*y(546);
dydt(625)=dydt(625)+k(33)*y(546);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(546)=dydt(546)-k(31)*y(546);
dydt(542)=dydt(542)+k(31)*y(546);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(546)=dydt(546)-k(29)*y(546);
dydt(551)=dydt(551)+k(29)*y(546);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(546)=dydt(546)-k(25)*y(546);
dydt(569)=dydt(569)+k(25)*y(546);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(546)=dydt(546)-k(22)*y(546);
dydt(197)=dydt(197)+k(22)*y(546);
dydt(540)=dydt(540)+k(22)*y(546);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(546)=dydt(546)-k(22)*y(546);
dydt(2)=dydt(2)+k(22)*y(546);
dydt(581)=dydt(581)+k(22)*y(546);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(546)=dydt(546)-k(16)*y(546);
dydt(3)=dydt(3)+k(16)*y(546);
dydt(259)=dydt(259)+k(16)*y(546);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(546);
dydt(546)=dydt(546)-k(15)*y(3)*y(546);
dydt(258)=dydt(258)+k(15)*y(3)*y(546);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(546)=dydt(546)-k(8)*y(546);
dydt(437)=dydt(437)+k(8)*y(546);
dydt(227)=dydt(227)+k(8)*y(546);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(546)=dydt(546)-k(6)*y(546);
dydt(111)=dydt(111)+k(6)*y(546);
dydt(543)=dydt(543)+k(6)*y(546);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(547)=dydt(547)-k(34)*y(547);
dydt(556)=dydt(556)+k(34)*y(547);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(547)=dydt(547)-k(33)*y(547);
dydt(518)=dydt(518)+k(33)*y(547);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(547)=dydt(547)-k(32)*y(547);
dydt(589)=dydt(589)+k(32)*y(547);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

dydt(547)=dydt(547)-k(22)*y(547);
dydt(197)=dydt(197)+k(22)*y(547);
dydt(537)=dydt(537)+k(22)*y(547);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(547);
dydt(547)=dydt(547)-k(21)*y(197)*y(547);
dydt(552)=dydt(552)+k(21)*y(197)*y(547);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(547);
dydt(547)=dydt(547)-k(21)*y(32)*y(547);
dydt(536)=dydt(536)+k(21)*y(32)*y(547);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(547);
dydt(547)=dydt(547)-k(21)*y(2)*y(547);
dydt(542)=dydt(542)+k(21)*y(2)*y(547);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(547)=dydt(547)-k(17)*y(547);
dydt(581)=dydt(581)+k(17)*y(547);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(547)=dydt(547)-k(16)*y(547);
dydt(3)=dydt(3)+k(16)*y(547);
dydt(585)=dydt(585)+k(16)*y(547);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(547);
dydt(547)=dydt(547)-k(9)*y(3)*y(547);
dydt(598)=dydt(598)+k(9)*y(3)*y(547);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(547)=dydt(547)-k(8)*y(547);
dydt(16)=dydt(16)+k(8)*y(547);
dydt(227)=dydt(227)+k(8)*y(547);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

dydt(547)=dydt(547)-k(6)*y(547);
dydt(15)=dydt(15)+k(6)*y(547);
dydt(543)=dydt(543)+k(6)*y(547);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(549)=dydt(549)-k(34)*y(549);
dydt(544)=dydt(544)+k(34)*y(549);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(549)=dydt(549)-k(34)*y(549);
dydt(349)=dydt(349)+k(34)*y(549);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(549)=dydt(549)-k(33)*y(549);
dydt(516)=dydt(516)+k(33)*y(549);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(549)=dydt(549)-k(25)*y(549);
dydt(601)=dydt(601)+k(25)*y(549);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(549)=dydt(549)-k(22)*y(549);
dydt(182)=dydt(182)+k(22)*y(549);
dydt(598)=dydt(598)+k(22)*y(549);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

dydt(549)=dydt(549)-k(22)*y(549);
dydt(197)=dydt(197)+k(22)*y(549);
dydt(345)=dydt(345)+k(22)*y(549);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(549)=dydt(549)-k(16)*y(549);
dydt(3)=dydt(3)+k(16)*y(549);
dydt(195)=dydt(195)+k(16)*y(549);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(549)=dydt(549)-k(10)*y(549);
dydt(3)=dydt(3)+k(10)*y(549);
dydt(548)=dydt(548)+k(10)*y(549);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(549)=dydt(549)-k(8)*y(549);
dydt(227)=dydt(227)+k(8)*y(549);
dydt(185)=dydt(185)+k(8)*y(549);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(549)=dydt(549)-k(6)*y(549);
dydt(184)=dydt(184)+k(6)*y(549);
dydt(543)=dydt(543)+k(6)*y(549);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(550)=dydt(550)-k(34)*y(550);
dydt(348)=dydt(348)+k(34)*y(550);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(550)=dydt(550)-k(34)*y(550);
dydt(545)=dydt(545)+k(34)*y(550);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(550)=dydt(550)-k(33)*y(550);
dydt(513)=dydt(513)+k(33)*y(550);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(550)=dydt(550)-k(31)*y(550);
dydt(194)=dydt(194)+k(31)*y(550);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(550)=dydt(550)-k(22)*y(550);
dydt(197)=dydt(197)+k(22)*y(550);
dydt(344)=dydt(344)+k(22)*y(550);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(550)=dydt(550)-k(22)*y(550);
dydt(182)=dydt(182)+k(22)*y(550);
dydt(585)=dydt(585)+k(22)*y(550);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(550);
dydt(550)=dydt(550)-k(15)*y(3)*y(550);
dydt(548)=dydt(548)+k(15)*y(3)*y(550);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(550);
dydt(550)=dydt(550)-k(9)*y(3)*y(550);
dydt(195)=dydt(195)+k(9)*y(3)*y(550);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(550)=dydt(550)-k(8)*y(550);
dydt(257)=dydt(257)+k(8)*y(550);
dydt(214)=dydt(214)+k(8)*y(550);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(550)=dydt(550)-k(6)*y(550);
dydt(177)=dydt(177)+k(6)*y(550);
dydt(256)=dydt(256)+k(6)*y(550);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(551)=dydt(551)-k(34)*y(551);
dydt(555)=dydt(555)+k(34)*y(551);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(551)=dydt(551)-k(34)*y(551);
dydt(546)=dydt(546)+k(34)*y(551);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(551)=dydt(551)-k(33)*y(551);
dydt(517)=dydt(517)+k(33)*y(551);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(551)=dydt(551)-k(31)*y(551);
dydt(548)=dydt(548)+k(31)*y(551);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(551)=dydt(551)-k(25)*y(551);
dydt(565)=dydt(565)+k(25)*y(551);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(551)=dydt(551)-k(22)*y(551);
dydt(197)=dydt(197)+k(22)*y(551);
dydt(644)=dydt(644)+k(22)*y(551);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(551)=dydt(551)-k(22)*y(551);
dydt(182)=dydt(182)+k(22)*y(551);
dydt(581)=dydt(581)+k(22)*y(551);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(551)=dydt(551)-k(16)*y(551);
dydt(3)=dydt(3)+k(16)*y(551);
dydt(577)=dydt(577)+k(16)*y(551);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(551);
dydt(551)=dydt(551)-k(15)*y(3)*y(551);
dydt(641)=dydt(641)+k(15)*y(3)*y(551);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(551)=dydt(551)-k(8)*y(551);
dydt(219)=dydt(219)+k(8)*y(551);
dydt(227)=dydt(227)+k(8)*y(551);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(551)=dydt(551)-k(6)*y(551);
dydt(218)=dydt(218)+k(6)*y(551);
dydt(543)=dydt(543)+k(6)*y(551);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(552)=dydt(552)-k(34)*y(552);
dydt(557)=dydt(557)+k(34)*y(552);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(552)=dydt(552)-k(34)*y(552);
dydt(536)=dydt(536)+k(34)*y(552);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(552)=dydt(552)-k(33)*y(552);
dydt(520)=dydt(520)+k(33)*y(552);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(552)=dydt(552)-k(33)*y(552);
dydt(548)=dydt(548)+k(33)*y(552);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(552)=dydt(552)-k(22)*y(552);
dydt(197)=dydt(197)+k(22)*y(552);
dydt(529)=dydt(529)+k(22)*y(552);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(552)=dydt(552)-k(22)*y(552);
dydt(197)=dydt(197)+k(22)*y(552);
dydt(547)=dydt(547)+k(22)*y(552);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(552)=dydt(552)-k(17)*y(552);
dydt(565)=dydt(565)+k(17)*y(552);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(552)=dydt(552)-k(16)*y(552);
dydt(3)=dydt(3)+k(16)*y(552);
dydt(609)=dydt(609)+k(16)*y(552);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(552);
dydt(552)=dydt(552)-k(9)*y(3)*y(552);
dydt(601)=dydt(601)+k(9)*y(3)*y(552);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(552)=dydt(552)-k(8)*y(552);
dydt(209)=dydt(209)+k(8)*y(552);
dydt(227)=dydt(227)+k(8)*y(552);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(552)=dydt(552)-k(6)*y(552);
dydt(192)=dydt(192)+k(6)*y(552);
dydt(543)=dydt(543)+k(6)*y(552);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(554)=dydt(554)-k(33)*y(554);
dydt(114)=dydt(114)+k(33)*y(554);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

dydt(554)=dydt(554)-k(22)*y(554);
dydt(32)=dydt(32)+k(22)*y(554);
dydt(118)=dydt(118)+k(22)*y(554);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(554)=dydt(554)-k(16)*y(554);
dydt(3)=dydt(3)+k(16)*y(554);
dydt(82)=dydt(82)+k(16)*y(554);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(1)=dydt(1)-k(5)*y(1)*y(554);
dydt(554)=dydt(554)-k(5)*y(1)*y(554);
dydt(592)=dydt(592)+k(5)*y(1)*y(554);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(7)=dydt(7)-k(5)*y(7)*y(554);
dydt(554)=dydt(554)-k(5)*y(7)*y(554);
dydt(107)=dydt(107)+k(5)*y(7)*y(554);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) 

dydt(554)=dydt(554)-k(4)*y(554);
dydt(4)=dydt(4)+k(4)*y(554);
dydt(37)=dydt(37)+k(4)*y(554);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(555)=dydt(555)-k(34)*y(555);
dydt(570)=dydt(570)+k(34)*y(555);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(555)=dydt(555)-k(33)*y(555);
dydt(523)=dydt(523)+k(33)*y(555);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(555)=dydt(555)-k(31)*y(555);
dydt(553)=dydt(553)+k(31)*y(555);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(555)=dydt(555)-k(30)*y(555);
dydt(551)=dydt(551)+k(30)*y(555);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(555)=dydt(555)-k(25)*y(555);
dydt(560)=dydt(560)+k(25)*y(555);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(555)=dydt(555)-k(22)*y(555);
dydt(32)=dydt(32)+k(22)*y(555);
dydt(644)=dydt(644)+k(22)*y(555);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(555)=dydt(555)-k(22)*y(555);
dydt(182)=dydt(182)+k(22)*y(555);
dydt(563)=dydt(563)+k(22)*y(555);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(555)=dydt(555)-k(16)*y(555);
dydt(3)=dydt(3)+k(16)*y(555);
dydt(574)=dydt(574)+k(16)*y(555);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(555);
dydt(555)=dydt(555)-k(15)*y(3)*y(555);
dydt(639)=dydt(639)+k(15)*y(3)*y(555);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(555)=dydt(555)-k(8)*y(555);
dydt(219)=dydt(219)+k(8)*y(555);
dydt(37)=dydt(37)+k(8)*y(555);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(555)=dydt(555)-k(6)*y(555);
dydt(218)=dydt(218)+k(6)*y(555);
dydt(554)=dydt(554)+k(6)*y(555);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(556)=dydt(556)-k(33)*y(556);
dydt(524)=dydt(524)+k(33)*y(556);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(556)=dydt(556)-k(32)*y(556);
dydt(592)=dydt(592)+k(32)*y(556);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

dydt(556)=dydt(556)-k(22)*y(556);
dydt(32)=dydt(32)+k(22)*y(556);
dydt(537)=dydt(537)+k(22)*y(556);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(556);
dydt(556)=dydt(556)-k(21)*y(197)*y(556);
dydt(557)=dydt(557)+k(21)*y(197)*y(556);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(556);
dydt(556)=dydt(556)-k(21)*y(32)*y(556);
dydt(535)=dydt(535)+k(21)*y(32)*y(556);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(556);
dydt(556)=dydt(556)-k(21)*y(2)*y(556);
dydt(541)=dydt(541)+k(21)*y(2)*y(556);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(556)=dydt(556)-k(17)*y(556);
dydt(563)=dydt(563)+k(17)*y(556);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(556)=dydt(556)-k(16)*y(556);
dydt(3)=dydt(3)+k(16)*y(556);
dydt(351)=dydt(351)+k(16)*y(556);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(556);
dydt(556)=dydt(556)-k(9)*y(3)*y(556);
dydt(352)=dydt(352)+k(9)*y(3)*y(556);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(556)=dydt(556)-k(8)*y(556);
dydt(16)=dydt(16)+k(8)*y(556);
dydt(37)=dydt(37)+k(8)*y(556);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

dydt(556)=dydt(556)-k(6)*y(556);
dydt(15)=dydt(15)+k(6)*y(556);
dydt(554)=dydt(554)+k(6)*y(556);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(558)=dydt(558)-k(34)*y(558);
dydt(602)=dydt(602)+k(34)*y(558);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(558)=dydt(558)-k(33)*y(558);
dydt(349)=dydt(349)+k(33)*y(558);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(558)=dydt(558)-k(33)*y(558);
dydt(526)=dydt(526)+k(33)*y(558);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(558)=dydt(558)-k(30)*y(558);
dydt(601)=dydt(601)+k(30)*y(558);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(558)=dydt(558)-k(22)*y(558);
dydt(197)=dydt(197)+k(22)*y(558);
dydt(352)=dydt(352)+k(22)*y(558);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

dydt(558)=dydt(558)-k(22)*y(558);
dydt(32)=dydt(32)+k(22)*y(558);
dydt(339)=dydt(339)+k(22)*y(558);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(558)=dydt(558)-k(16)*y(558);
dydt(3)=dydt(3)+k(16)*y(558);
dydt(341)=dydt(341)+k(16)*y(558);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(558)=dydt(558)-k(10)*y(558);
dydt(3)=dydt(3)+k(10)*y(558);
dydt(557)=dydt(557)+k(10)*y(558);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(558)=dydt(558)-k(8)*y(558);
dydt(37)=dydt(37)+k(8)*y(558);
dydt(336)=dydt(336)+k(8)*y(558);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(558)=dydt(558)-k(6)*y(558);
dydt(335)=dydt(335)+k(6)*y(558);
dydt(554)=dydt(554)+k(6)*y(558);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(559)=dydt(559)-k(34)*y(559);
dydt(612)=dydt(612)+k(34)*y(559);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(559)=dydt(559)-k(33)*y(559);
dydt(527)=dydt(527)+k(33)*y(559);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(559)=dydt(559)-k(33)*y(559);
dydt(348)=dydt(348)+k(33)*y(559);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(559)=dydt(559)-k(31)*y(559);
dydt(454)=dydt(454)+k(31)*y(559);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(559)=dydt(559)-k(30)*y(559);
dydt(609)=dydt(609)+k(30)*y(559);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(559)=dydt(559)-k(22)*y(559);
dydt(32)=dydt(32)+k(22)*y(559);
dydt(338)=dydt(338)+k(22)*y(559);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(559)=dydt(559)-k(22)*y(559);
dydt(197)=dydt(197)+k(22)*y(559);
dydt(351)=dydt(351)+k(22)*y(559);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(559);
dydt(559)=dydt(559)-k(15)*y(3)*y(559);
dydt(557)=dydt(557)+k(15)*y(3)*y(559);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(559);
dydt(559)=dydt(559)-k(9)*y(3)*y(559);
dydt(341)=dydt(341)+k(9)*y(3)*y(559);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(559)=dydt(559)-k(8)*y(559);
dydt(33)=dydt(33)+k(8)*y(559);
dydt(209)=dydt(209)+k(8)*y(559);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(559)=dydt(559)-k(6)*y(559);
dydt(192)=dydt(192)+k(6)*y(559);
dydt(82)=dydt(82)+k(6)*y(559);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(561)=dydt(561)-k(34)*y(561);
dydt(567)=dydt(567)+k(34)*y(561);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(561)=dydt(561)-k(33)*y(561);
dydt(639)=dydt(639)+k(33)*y(561);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(561)=dydt(561)-k(33)*y(561);
dydt(632)=dydt(632)+k(33)*y(561);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(561)=dydt(561)-k(30)*y(561);
dydt(642)=dydt(642)+k(30)*y(561);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(561)=dydt(561)-k(22)*y(561);
dydt(197)=dydt(197)+k(22)*y(561);
dydt(633)=dydt(633)+k(22)*y(561);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

dydt(561)=dydt(561)-k(22)*y(561);
dydt(32)=dydt(32)+k(22)*y(561);
dydt(636)=dydt(636)+k(22)*y(561);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(561)=dydt(561)-k(16)*y(561);
dydt(3)=dydt(3)+k(16)*y(561);
dydt(560)=dydt(560)+k(16)*y(561);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(561)=dydt(561)-k(16)*y(561);
dydt(3)=dydt(3)+k(16)*y(561);
dydt(635)=dydt(635)+k(16)*y(561);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(561)=dydt(561)-k(8)*y(561);
dydt(37)=dydt(37)+k(8)*y(561);
dydt(228)=dydt(228)+k(8)*y(561);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(561)=dydt(561)-k(6)*y(561);
dydt(227)=dydt(227)+k(6)*y(561);
dydt(554)=dydt(554)+k(6)*y(561);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(562)=dydt(562)-k(34)*y(562);
dydt(568)=dydt(568)+k(34)*y(562);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(562)=dydt(562)-k(33)*y(562);
dydt(502)=dydt(502)+k(33)*y(562);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(562)=dydt(562)-k(33)*y(562);
dydt(574)=dydt(574)+k(33)*y(562);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(562)=dydt(562)-k(31)*y(562);
dydt(472)=dydt(472)+k(31)*y(562);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(562)=dydt(562)-k(31)*y(562);
dydt(559)=dydt(559)+k(31)*y(562);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(562)=dydt(562)-k(30)*y(562);
dydt(583)=dydt(583)+k(30)*y(562);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(562)=dydt(562)-k(22)*y(562);
dydt(32)=dydt(32)+k(22)*y(562);
dydt(613)=dydt(613)+k(22)*y(562);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(562)=dydt(562)-k(22)*y(562);
dydt(197)=dydt(197)+k(22)*y(562);
dydt(573)=dydt(573)+k(22)*y(562);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(562);
dydt(562)=dydt(562)-k(15)*y(3)*y(562);
dydt(635)=dydt(635)+k(15)*y(3)*y(562);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(562);
dydt(562)=dydt(562)-k(15)*y(3)*y(562);
dydt(560)=dydt(560)+k(15)*y(3)*y(562);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(562)=dydt(562)-k(8)*y(562);
dydt(33)=dydt(33)+k(8)*y(562);
dydt(319)=dydt(319)+k(8)*y(562);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(562)=dydt(562)-k(6)*y(562);
dydt(257)=dydt(257)+k(6)*y(562);
dydt(82)=dydt(82)+k(6)*y(562);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(563)=dydt(563)-k(33)*y(563);
dydt(308)=dydt(308)+k(33)*y(563);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(563)=dydt(563)-k(32)*y(563);
dydt(311)=dydt(311)+k(32)*y(563);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(563)=dydt(563)-k(31)*y(563);
dydt(556)=dydt(556)+k(31)*y(563);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

dydt(563)=dydt(563)-k(22)*y(563);
dydt(32)=dydt(32)+k(22)*y(563);
dydt(310)=dydt(310)+k(22)*y(563);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(563);
dydt(563)=dydt(563)-k(21)*y(197)*y(563);
dydt(560)=dydt(560)+k(21)*y(197)*y(563);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(563);
dydt(563)=dydt(563)-k(21)*y(32)*y(563);
dydt(566)=dydt(566)+k(21)*y(32)*y(563);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(563);
dydt(563)=dydt(563)-k(21)*y(2)*y(563);
dydt(570)=dydt(570)+k(21)*y(2)*y(563);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(563)=dydt(563)-k(16)*y(563);
dydt(3)=dydt(3)+k(16)*y(563);
dydt(573)=dydt(573)+k(16)*y(563);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(563);
dydt(563)=dydt(563)-k(15)*y(3)*y(563);
dydt(633)=dydt(633)+k(15)*y(3)*y(563);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(563)=dydt(563)-k(8)*y(563);
dydt(315)=dydt(315)+k(8)*y(563);
dydt(37)=dydt(37)+k(8)*y(563);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

dydt(563)=dydt(563)-k(6)*y(563);
dydt(77)=dydt(77)+k(6)*y(563);
dydt(554)=dydt(554)+k(6)*y(563);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(564)=dydt(564)-k(34)*y(564);
dydt(534)=dydt(534)+k(34)*y(564);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(564)=dydt(564)-k(33)*y(564);
dydt(644)=dydt(644)+k(33)*y(564);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(564)=dydt(564)-k(32)*y(564);
dydt(318)=dydt(318)+k(32)*y(564);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(564)=dydt(564)-k(31)*y(564);
dydt(529)=dydt(529)+k(31)*y(564);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

dydt(564)=dydt(564)-k(22)*y(564);
dydt(197)=dydt(197)+k(22)*y(564);
dydt(310)=dydt(310)+k(22)*y(564);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(564);
dydt(564)=dydt(564)-k(21)*y(197)*y(564);
dydt(565)=dydt(565)+k(21)*y(197)*y(564);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(564);
dydt(564)=dydt(564)-k(21)*y(32)*y(564);
dydt(560)=dydt(560)+k(21)*y(32)*y(564);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(564);
dydt(564)=dydt(564)-k(21)*y(2)*y(564);
dydt(528)=dydt(528)+k(21)*y(2)*y(564);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(564)=dydt(564)-k(16)*y(564);
dydt(3)=dydt(3)+k(16)*y(564);
dydt(613)=dydt(613)+k(16)*y(564);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(564);
dydt(564)=dydt(564)-k(15)*y(3)*y(564);
dydt(636)=dydt(636)+k(15)*y(3)*y(564);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(564)=dydt(564)-k(8)*y(564);
dydt(54)=dydt(54)+k(8)*y(564);
dydt(319)=dydt(319)+k(8)*y(564);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(564)=dydt(564)-k(6)*y(564);
dydt(118)=dydt(118)+k(6)*y(564);
dydt(257)=dydt(257)+k(6)*y(564);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(565)=dydt(565)-k(34)*y(565);
dydt(560)=dydt(560)+k(34)*y(565);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(565)=dydt(565)-k(34)*y(565);
dydt(569)=dydt(569)+k(34)*y(565);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(565)=dydt(565)-k(33)*y(565);
dydt(626)=dydt(626)+k(33)*y(565);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(565)=dydt(565)-k(33)*y(565);
dydt(551)=dydt(551)+k(33)*y(565);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(565)=dydt(565)-k(31)*y(565);
dydt(552)=dydt(552)+k(31)*y(565);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(565)=dydt(565)-k(22)*y(565);
dydt(197)=dydt(197)+k(22)*y(565);
dydt(564)=dydt(564)+k(22)*y(565);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(565)=dydt(565)-k(22)*y(565);
dydt(197)=dydt(197)+k(22)*y(565);
dydt(581)=dydt(581)+k(22)*y(565);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(565)=dydt(565)-k(16)*y(565);
dydt(3)=dydt(3)+k(16)*y(565);
dydt(583)=dydt(583)+k(16)*y(565);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(565);
dydt(565)=dydt(565)-k(15)*y(3)*y(565);
dydt(642)=dydt(642)+k(15)*y(3)*y(565);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(565)=dydt(565)-k(8)*y(565);
dydt(319)=dydt(319)+k(8)*y(565);
dydt(227)=dydt(227)+k(8)*y(565);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(565)=dydt(565)-k(6)*y(565);
dydt(257)=dydt(257)+k(6)*y(565);
dydt(543)=dydt(543)+k(6)*y(565);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(567)=dydt(567)-k(33)*y(567);
dydt(571)=dydt(571)+k(33)*y(567);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(567)=dydt(567)-k(33)*y(567);
dydt(631)=dydt(631)+k(33)*y(567);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(567)=dydt(567)-k(28)*y(567);
dydt(643)=dydt(643)+k(28)*y(567);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(567)=dydt(567)-k(27)*y(567);
dydt(561)=dydt(561)+k(27)*y(567);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(567)=dydt(567)-k(22)*y(567);
dydt(32)=dydt(32)+k(22)*y(567);
dydt(633)=dydt(633)+k(22)*y(567);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

dydt(567)=dydt(567)-k(22)*y(567);
dydt(32)=dydt(32)+k(22)*y(567);
dydt(244)=dydt(244)+k(22)*y(567);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(567)=dydt(567)-k(16)*y(567);
dydt(3)=dydt(3)+k(16)*y(567);
dydt(566)=dydt(566)+k(16)*y(567);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(567)=dydt(567)-k(16)*y(567);
dydt(3)=dydt(3)+k(16)*y(567);
dydt(247)=dydt(247)+k(16)*y(567);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(567)=dydt(567)-k(8)*y(567);
dydt(37)=dydt(37)+k(8)*y(567);
dydt(38)=dydt(38)+k(8)*y(567);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(567)=dydt(567)-k(6)*y(567);
dydt(37)=dydt(37)+k(6)*y(567);
dydt(554)=dydt(554)+k(6)*y(567);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(568)=dydt(568)-k(33)*y(568);
dydt(499)=dydt(499)+k(33)*y(568);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(568)=dydt(568)-k(33)*y(568);
dydt(572)=dydt(572)+k(33)*y(568);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(568)=dydt(568)-k(31)*y(568);
dydt(468)=dydt(468)+k(31)*y(568);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(568)=dydt(568)-k(31)*y(568);
dydt(612)=dydt(612)+k(31)*y(568);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(568)=dydt(568)-k(28)*y(568);
dydt(582)=dydt(582)+k(28)*y(568);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(568)=dydt(568)-k(27)*y(568);
dydt(562)=dydt(562)+k(27)*y(568);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(568)=dydt(568)-k(22)*y(568);
dydt(32)=dydt(32)+k(22)*y(568);
dydt(245)=dydt(245)+k(22)*y(568);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(568)=dydt(568)-k(22)*y(568);
dydt(32)=dydt(32)+k(22)*y(568);
dydt(573)=dydt(573)+k(22)*y(568);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(568);
dydt(568)=dydt(568)-k(15)*y(3)*y(568);
dydt(247)=dydt(247)+k(15)*y(3)*y(568);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(568);
dydt(568)=dydt(568)-k(15)*y(3)*y(568);
dydt(566)=dydt(566)+k(15)*y(3)*y(568);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(568)=dydt(568)-k(8)*y(568);
dydt(33)=dydt(33)+k(8)*y(568);
dydt(34)=dydt(34)+k(8)*y(568);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(568)=dydt(568)-k(6)*y(568);
dydt(33)=dydt(33)+k(6)*y(568);
dydt(82)=dydt(82)+k(6)*y(568);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(569)=dydt(569)-k(34)*y(569);
dydt(566)=dydt(566)+k(34)*y(569);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(569)=dydt(569)-k(33)*y(569);
dydt(496)=dydt(496)+k(33)*y(569);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(569)=dydt(569)-k(33)*y(569);
dydt(546)=dydt(546)+k(33)*y(569);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(569)=dydt(569)-k(31)*y(569);
dydt(536)=dydt(536)+k(31)*y(569);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(569)=dydt(569)-k(29)*y(569);
dydt(565)=dydt(565)+k(29)*y(569);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(569)=dydt(569)-k(22)*y(569);
dydt(197)=dydt(197)+k(22)*y(569);
dydt(534)=dydt(534)+k(22)*y(569);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(569)=dydt(569)-k(22)*y(569);
dydt(32)=dydt(32)+k(22)*y(569);
dydt(581)=dydt(581)+k(22)*y(569);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(569)=dydt(569)-k(16)*y(569);
dydt(3)=dydt(3)+k(16)*y(569);
dydt(582)=dydt(582)+k(16)*y(569);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(569);
dydt(569)=dydt(569)-k(15)*y(3)*y(569);
dydt(643)=dydt(643)+k(15)*y(3)*y(569);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(569)=dydt(569)-k(8)*y(569);
dydt(34)=dydt(34)+k(8)*y(569);
dydt(227)=dydt(227)+k(8)*y(569);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(569)=dydt(569)-k(6)*y(569);
dydt(33)=dydt(33)+k(6)*y(569);
dydt(543)=dydt(543)+k(6)*y(569);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(571)=dydt(571)-k(33)*y(571);
dydt(630)=dydt(630)+k(33)*y(571);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(571)=dydt(571)-k(28)*y(571);
dydt(258)=dydt(258)+k(28)*y(571);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(571)=dydt(571)-k(27)*y(571);
dydt(639)=dydt(639)+k(27)*y(571);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(571)=dydt(571)-k(25)*y(571);
dydt(567)=dydt(567)+k(25)*y(571);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(571)=dydt(571)-k(22)*y(571);
dydt(2)=dydt(2)+k(22)*y(571);
dydt(633)=dydt(633)+k(22)*y(571);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

dydt(571)=dydt(571)-k(22)*y(571);
dydt(32)=dydt(32)+k(22)*y(571);
dydt(251)=dydt(251)+k(22)*y(571);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(571)=dydt(571)-k(16)*y(571);
dydt(3)=dydt(3)+k(16)*y(571);
dydt(570)=dydt(570)+k(16)*y(571);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(571)=dydt(571)-k(16)*y(571);
dydt(3)=dydt(3)+k(16)*y(571);
dydt(254)=dydt(254)+k(16)*y(571);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(571)=dydt(571)-k(8)*y(571);
dydt(37)=dydt(37)+k(8)*y(571);
dydt(44)=dydt(44)+k(8)*y(571);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(571)=dydt(571)-k(6)*y(571);
dydt(43)=dydt(43)+k(6)*y(571);
dydt(554)=dydt(554)+k(6)*y(571);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(573)=dydt(573)-k(33)*y(573);
dydt(501)=dydt(501)+k(33)*y(573);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(573)=dydt(573)-k(32)*y(573);
dydt(84)=dydt(84)+k(32)*y(573);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(573)=dydt(573)-k(31)*y(573);
dydt(615)=dydt(615)+k(31)*y(573);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(573)=dydt(573)-k(31)*y(573);
dydt(351)=dydt(351)+k(31)*y(573);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(573)=dydt(573)-k(22)*y(573);
dydt(32)=dydt(32)+k(22)*y(573);
dydt(584)=dydt(584)+k(22)*y(573);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(573);
dydt(573)=dydt(573)-k(21)*y(197)*y(573);
dydt(562)=dydt(562)+k(21)*y(197)*y(573);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(573);
dydt(573)=dydt(573)-k(21)*y(32)*y(573);
dydt(568)=dydt(568)+k(21)*y(32)*y(573);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(573);
dydt(573)=dydt(573)-k(21)*y(2)*y(573);
dydt(572)=dydt(572)+k(21)*y(2)*y(573);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(573);
dydt(573)=dydt(573)-k(15)*y(3)*y(573);
dydt(634)=dydt(634)+k(15)*y(3)*y(573);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(573);
dydt(573)=dydt(573)-k(15)*y(3)*y(573);
dydt(563)=dydt(563)+k(15)*y(3)*y(573);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(573)=dydt(573)-k(8)*y(573);
dydt(315)=dydt(315)+k(8)*y(573);
dydt(33)=dydt(33)+k(8)*y(573);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

dydt(573)=dydt(573)-k(6)*y(573);
dydt(77)=dydt(77)+k(6)*y(573);
dydt(82)=dydt(82)+k(6)*y(573);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(576)=dydt(576)-k(34)*y(576);
dydt(252)=dydt(252)+k(34)*y(576);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(576)=dydt(576)-k(32)*y(576);
dydt(325)=dydt(325)+k(32)*y(576);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(576)=dydt(576)-k(31)*y(576);
dydt(431)=dydt(431)+k(31)*y(576);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(576)=dydt(576)-k(31)*y(576);
dydt(344)=dydt(344)+k(31)*y(576);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(576)=dydt(576)-k(22)*y(576);
dydt(182)=dydt(182)+k(22)*y(576);
dydt(584)=dydt(584)+k(22)*y(576);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(576);
dydt(576)=dydt(576)-k(21)*y(197)*y(576);
dydt(577)=dydt(577)+k(21)*y(197)*y(576);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(576);
dydt(576)=dydt(576)-k(21)*y(32)*y(576);
dydt(574)=dydt(574)+k(21)*y(32)*y(576);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(576);
dydt(576)=dydt(576)-k(21)*y(2)*y(576);
dydt(505)=dydt(505)+k(21)*y(2)*y(576);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(576);
dydt(576)=dydt(576)-k(15)*y(3)*y(576);
dydt(645)=dydt(645)+k(15)*y(3)*y(576);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(576);
dydt(576)=dydt(576)-k(15)*y(3)*y(576);
dydt(644)=dydt(644)+k(15)*y(3)*y(576);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

dydt(576)=dydt(576)-k(8)*y(576);
dydt(77)=dydt(77)+k(8)*y(576);
dydt(219)=dydt(219)+k(8)*y(576);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(576)=dydt(576)-k(6)*y(576);
dydt(76)=dydt(76)+k(6)*y(576);
dydt(218)=dydt(218)+k(6)*y(576);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(647)=dydt(647)-k(34)*y(647);
dydt(83)=dydt(83)+k(34)*y(647);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(647)=dydt(647)-k(33)*y(647);
dydt(301)=dydt(301)+k(33)*y(647);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(647)=dydt(647)-k(22)*y(647);
dydt(197)=dydt(197)+k(22)*y(647);
dydt(78)=dydt(78)+k(22)*y(647);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(647)=dydt(647)-k(19)*y(647);
dydt(622)=dydt(622)+k(19)*y(647);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(647)=dydt(647)-k(16)*y(647);
dydt(3)=dydt(3)+k(16)*y(647);
dydt(203)=dydt(203)+k(16)*y(647);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(647)=dydt(647)-k(16)*y(647);
dydt(3)=dydt(3)+k(16)*y(647);
dydt(312)=dydt(312)+k(16)*y(647);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(647)=dydt(647)-k(8)*y(647);
dydt(62)=dydt(62)+k(8)*y(647);
dydt(227)=dydt(227)+k(8)*y(647);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(647)=dydt(647)-k(6)*y(647);
dydt(61)=dydt(61)+k(6)*y(647);
dydt(543)=dydt(543)+k(6)*y(647);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(622)=dydt(622)-k(34)*y(622);
dydt(633)=dydt(633)+k(34)*y(622);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(622)=dydt(622)-k(33)*y(622);
dydt(624)=dydt(624)+k(33)*y(622);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(622)=dydt(622)-k(32)*y(622);
dydt(647)=dydt(647)+k(32)*y(622);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(622)=dydt(622)-k(22)*y(622);
dydt(197)=dydt(197)+k(22)*y(622);
dydt(627)=dydt(627)+k(22)*y(622);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(622);
dydt(622)=dydt(622)-k(21)*y(197)*y(622);
dydt(642)=dydt(642)+k(21)*y(197)*y(622);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(622);
dydt(622)=dydt(622)-k(21)*y(32)*y(622);
dydt(643)=dydt(643)+k(21)*y(32)*y(622);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(622);
dydt(622)=dydt(622)-k(21)*y(2)*y(622);
dydt(258)=dydt(258)+k(21)*y(2)*y(622);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(622)=dydt(622)-k(16)*y(622);
dydt(3)=dydt(3)+k(16)*y(622);
dydt(581)=dydt(581)+k(16)*y(622);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(622)=dydt(622)-k(16)*y(622);
dydt(3)=dydt(3)+k(16)*y(622);
dydt(260)=dydt(260)+k(16)*y(622);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(622)=dydt(622)-k(8)*y(622);
dydt(55)=dydt(55)+k(8)*y(622);
dydt(227)=dydt(227)+k(8)*y(622);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(622)=dydt(622)-k(6)*y(622);
dydt(54)=dydt(54)+k(6)*y(622);
dydt(543)=dydt(543)+k(6)*y(622);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(627)=dydt(627)-k(32)*y(627);
dydt(236)=dydt(236)+k(32)*y(627);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(627)=dydt(627)-k(32)*y(627);
dydt(78)=dydt(78)+k(32)*y(627);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(627);
dydt(627)=dydt(627)-k(21)*y(197)*y(627);
dydt(636)=dydt(636)+k(21)*y(197)*y(627);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(627);
dydt(627)=dydt(627)-k(21)*y(32)*y(627);
dydt(244)=dydt(244)+k(21)*y(32)*y(627);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(627);
dydt(627)=dydt(627)-k(21)*y(2)*y(627);
dydt(251)=dydt(251)+k(21)*y(2)*y(627);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(627);
dydt(627)=dydt(627)-k(21)*y(197)*y(627);
dydt(622)=dydt(622)+k(21)*y(197)*y(627);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(32)=dydt(32)-k(21)*y(32)*y(627);
dydt(627)=dydt(627)-k(21)*y(32)*y(627);
dydt(633)=dydt(633)+k(21)*y(32)*y(627);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(2)=dydt(2)-k(21)*y(2)*y(627);
dydt(627)=dydt(627)-k(21)*y(2)*y(627);
dydt(628)=dydt(628)+k(21)*y(2)*y(627);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) 

dydt(627)=dydt(627)-k(16)*y(627);
dydt(3)=dydt(3)+k(16)*y(627);
dydt(238)=dydt(238)+k(16)*y(627);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) 

dydt(627)=dydt(627)-k(16)*y(627);
dydt(3)=dydt(3)+k(16)*y(627);
dydt(310)=dydt(310)+k(16)*y(627);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(627)=dydt(627)-k(8)*y(627);
dydt(54)=dydt(54)+k(8)*y(627);
dydt(55)=dydt(55)+k(8)*y(627);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(627)=dydt(627)-k(6)*y(627);
dydt(54)=dydt(54)+k(6)*y(627);
dydt(118)=dydt(118)+k(6)*y(627);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(629)=dydt(629)-k(32)*y(629);
dydt(80)=dydt(80)+k(32)*y(629);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(629)=dydt(629)-k(31)*y(629);
dydt(276)=dydt(276)+k(31)*y(629);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(629)=dydt(629)-k(26)*y(629);
dydt(634)=dydt(634)+k(26)*y(629);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

dydt(629)=dydt(629)-k(22)*y(629);
dydt(2)=dydt(2)+k(22)*y(629);
dydt(238)=dydt(238)+k(22)*y(629);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(629);
dydt(629)=dydt(629)-k(21)*y(197)*y(629);
dydt(485)=dydt(485)+k(21)*y(197)*y(629);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(629);
dydt(629)=dydt(629)-k(21)*y(32)*y(629);
dydt(246)=dydt(246)+k(21)*y(32)*y(629);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(629);
dydt(629)=dydt(629)-k(21)*y(2)*y(629);
dydt(253)=dydt(253)+k(21)*y(2)*y(629);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(629)=dydt(629)-k(16)*y(629);
dydt(3)=dydt(3)+k(16)*y(629);
dydt(501)=dydt(501)+k(16)*y(629);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(629);
dydt(629)=dydt(629)-k(15)*y(3)*y(629);
dydt(628)=dydt(628)+k(15)*y(3)*y(629);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(629)=dydt(629)-k(8)*y(629);
dydt(55)=dydt(55)+k(8)*y(629);
dydt(111)=dydt(111)+k(8)*y(629);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(629)=dydt(629)-k(6)*y(629);
dydt(54)=dydt(54)+k(6)*y(629);
dydt(110)=dydt(110)+k(6)*y(629);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(630)=dydt(630)-k(28)*y(630);
dydt(265)=dydt(265)+k(28)*y(630);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(630)=dydt(630)-k(27)*y(630);
dydt(637)=dydt(637)+k(27)*y(630);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(630)=dydt(630)-k(26)*y(630);
dydt(571)=dydt(571)+k(26)*y(630);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(630)=dydt(630)-k(25)*y(630);
dydt(631)=dydt(631)+k(25)*y(630);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(630)=dydt(630)-k(22)*y(630);
dydt(2)=dydt(2)+k(22)*y(630);
dydt(628)=dydt(628)+k(22)*y(630);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

dydt(630)=dydt(630)-k(22)*y(630);
dydt(2)=dydt(2)+k(22)*y(630);
dydt(251)=dydt(251)+k(22)*y(630);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(630)=dydt(630)-k(16)*y(630);
dydt(3)=dydt(3)+k(16)*y(630);
dydt(504)=dydt(504)+k(16)*y(630);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(630)=dydt(630)-k(16)*y(630);
dydt(3)=dydt(3)+k(16)*y(630);
dydt(253)=dydt(253)+k(16)*y(630);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(630)=dydt(630)-k(8)*y(630);
dydt(43)=dydt(43)+k(8)*y(630);
dydt(44)=dydt(44)+k(8)*y(630);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(630)=dydt(630)-k(6)*y(630);
dydt(43)=dydt(43)+k(6)*y(630);
dydt(114)=dydt(114)+k(6)*y(630);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(631)=dydt(631)-k(33)*y(631);
dydt(630)=dydt(630)+k(33)*y(631);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(631)=dydt(631)-k(28)*y(631);
dydt(494)=dydt(494)+k(28)*y(631);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(631)=dydt(631)-k(27)*y(631);
dydt(632)=dydt(632)+k(27)*y(631);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(631)=dydt(631)-k(26)*y(631);
dydt(567)=dydt(567)+k(26)*y(631);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(631)=dydt(631)-k(22)*y(631);
dydt(32)=dydt(32)+k(22)*y(631);
dydt(628)=dydt(628)+k(22)*y(631);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

dydt(631)=dydt(631)-k(22)*y(631);
dydt(2)=dydt(2)+k(22)*y(631);
dydt(244)=dydt(244)+k(22)*y(631);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(631)=dydt(631)-k(16)*y(631);
dydt(3)=dydt(3)+k(16)*y(631);
dydt(500)=dydt(500)+k(16)*y(631);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(631)=dydt(631)-k(16)*y(631);
dydt(3)=dydt(3)+k(16)*y(631);
dydt(246)=dydt(246)+k(16)*y(631);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(631)=dydt(631)-k(8)*y(631);
dydt(43)=dydt(43)+k(8)*y(631);
dydt(38)=dydt(38)+k(8)*y(631);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(631)=dydt(631)-k(6)*y(631);
dydt(37)=dydt(37)+k(6)*y(631);
dydt(114)=dydt(114)+k(6)*y(631);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(636)=dydt(636)-k(34)*y(636);
dydt(244)=dydt(244)+k(34)*y(636);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(636)=dydt(636)-k(33)*y(636);
dydt(638)=dydt(638)+k(33)*y(636);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(636)=dydt(636)-k(32)*y(636);
dydt(320)=dydt(320)+k(32)*y(636);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(636)=dydt(636)-k(22)*y(636);
dydt(197)=dydt(197)+k(22)*y(636);
dydt(627)=dydt(627)+k(22)*y(636);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(636);
dydt(636)=dydt(636)-k(21)*y(197)*y(636);
dydt(642)=dydt(642)+k(21)*y(197)*y(636);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(636);
dydt(636)=dydt(636)-k(21)*y(32)*y(636);
dydt(561)=dydt(561)+k(21)*y(32)*y(636);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(636);
dydt(636)=dydt(636)-k(21)*y(2)*y(636);
dydt(632)=dydt(632)+k(21)*y(2)*y(636);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(636)=dydt(636)-k(16)*y(636);
dydt(3)=dydt(3)+k(16)*y(636);
dydt(491)=dydt(491)+k(16)*y(636);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(636)=dydt(636)-k(16)*y(636);
dydt(3)=dydt(3)+k(16)*y(636);
dydt(564)=dydt(564)+k(16)*y(636);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(636)=dydt(636)-k(8)*y(636);
dydt(54)=dydt(54)+k(8)*y(636);
dydt(228)=dydt(228)+k(8)*y(636);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(636)=dydt(636)-k(6)*y(636);
dydt(118)=dydt(118)+k(6)*y(636);
dydt(227)=dydt(227)+k(6)*y(636);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(644)=dydt(644)-k(34)*y(644);
dydt(540)=dydt(540)+k(34)*y(644);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(644)=dydt(644)-k(32)*y(644);
dydt(323)=dydt(323)+k(32)*y(644);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(644)=dydt(644)-k(31)*y(644);
dydt(519)=dydt(519)+k(31)*y(644);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(644)=dydt(644)-k(25)*y(644);
dydt(564)=dydt(564)+k(25)*y(644);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

dydt(644)=dydt(644)-k(22)*y(644);
dydt(182)=dydt(182)+k(22)*y(644);
dydt(310)=dydt(310)+k(22)*y(644);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(644);
dydt(644)=dydt(644)-k(21)*y(197)*y(644);
dydt(551)=dydt(551)+k(21)*y(197)*y(644);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(644);
dydt(644)=dydt(644)-k(21)*y(32)*y(644);
dydt(555)=dydt(555)+k(21)*y(32)*y(644);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(644);
dydt(644)=dydt(644)-k(21)*y(2)*y(644);
dydt(523)=dydt(523)+k(21)*y(2)*y(644);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(644)=dydt(644)-k(16)*y(644);
dydt(3)=dydt(3)+k(16)*y(644);
dydt(576)=dydt(576)+k(16)*y(644);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(644);
dydt(644)=dydt(644)-k(15)*y(3)*y(644);
dydt(638)=dydt(638)+k(15)*y(3)*y(644);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(644)=dydt(644)-k(8)*y(644);
dydt(54)=dydt(54)+k(8)*y(644);
dydt(219)=dydt(219)+k(8)*y(644);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(644)=dydt(644)-k(6)*y(644);
dydt(118)=dydt(118)+k(6)*y(644);
dydt(218)=dydt(218)+k(6)*y(644);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(646)=dydt(646)-k(34)*y(646);
dydt(239)=dydt(239)+k(34)*y(646);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(646)=dydt(646)-k(31)*y(646);
dydt(224)=dydt(224)+k(31)*y(646);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

dydt(646)=dydt(646)-k(22)*y(646);
dydt(182)=dydt(182)+k(22)*y(646);
dydt(235)=dydt(235)+k(22)*y(646);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(646)=dydt(646)-k(20)*y(646);
dydt(645)=dydt(645)+k(20)*y(646);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(646)=dydt(646)-k(16)*y(646);
dydt(3)=dydt(3)+k(16)*y(646);
dydt(325)=dydt(325)+k(16)*y(646);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(646);
dydt(646)=dydt(646)-k(15)*y(3)*y(646);
dydt(324)=dydt(324)+k(15)*y(3)*y(646);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

dydt(646)=dydt(646)-k(8)*y(646);
dydt(72)=dydt(72)+k(8)*y(646);
dydt(223)=dydt(223)+k(8)*y(646);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(646)=dydt(646)-k(6)*y(646);
dydt(71)=dydt(71)+k(6)*y(646);
dydt(222)=dydt(222)+k(6)*y(646);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(645)=dydt(645)-k(34)*y(645);
dydt(250)=dydt(250)+k(34)*y(645);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(645)=dydt(645)-k(32)*y(645);
dydt(646)=dydt(646)+k(32)*y(645);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(645)=dydt(645)-k(31)*y(645);
dydt(282)=dydt(282)+k(31)*y(645);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

dydt(645)=dydt(645)-k(22)*y(645);
dydt(182)=dydt(182)+k(22)*y(645);
dydt(238)=dydt(238)+k(22)*y(645);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(645);
dydt(645)=dydt(645)-k(21)*y(197)*y(645);
dydt(261)=dydt(261)+k(21)*y(197)*y(645);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(645);
dydt(645)=dydt(645)-k(21)*y(32)*y(645);
dydt(286)=dydt(286)+k(21)*y(32)*y(645);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(645);
dydt(645)=dydt(645)-k(21)*y(2)*y(645);
dydt(281)=dydt(281)+k(21)*y(2)*y(645);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(645)=dydt(645)-k(16)*y(645);
dydt(3)=dydt(3)+k(16)*y(645);
dydt(576)=dydt(576)+k(16)*y(645);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(645);
dydt(645)=dydt(645)-k(15)*y(3)*y(645);
dydt(638)=dydt(638)+k(15)*y(3)*y(645);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

dydt(645)=dydt(645)-k(8)*y(645);
dydt(77)=dydt(77)+k(8)*y(645);
dydt(223)=dydt(223)+k(8)*y(645);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(645)=dydt(645)-k(6)*y(645);
dydt(76)=dydt(76)+k(6)*y(645);
dydt(222)=dydt(222)+k(6)*y(645);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(638)=dydt(638)-k(34)*y(638);
dydt(251)=dydt(251)+k(34)*y(638);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(638)=dydt(638)-k(32)*y(638);
dydt(324)=dydt(324)+k(32)*y(638);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(638)=dydt(638)-k(25)*y(638);
dydt(636)=dydt(636)+k(25)*y(638);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(638)=dydt(638)-k(22)*y(638);
dydt(182)=dydt(182)+k(22)*y(638);
dydt(627)=dydt(627)+k(22)*y(638);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(638);
dydt(638)=dydt(638)-k(21)*y(197)*y(638);
dydt(641)=dydt(641)+k(21)*y(197)*y(638);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(638);
dydt(638)=dydt(638)-k(21)*y(32)*y(638);
dydt(639)=dydt(639)+k(21)*y(32)*y(638);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(638);
dydt(638)=dydt(638)-k(21)*y(2)*y(638);
dydt(637)=dydt(637)+k(21)*y(2)*y(638);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(638)=dydt(638)-k(16)*y(638);
dydt(3)=dydt(3)+k(16)*y(638);
dydt(645)=dydt(645)+k(16)*y(638);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(638)=dydt(638)-k(16)*y(638);
dydt(3)=dydt(3)+k(16)*y(638);
dydt(644)=dydt(644)+k(16)*y(638);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(638)=dydt(638)-k(8)*y(638);
dydt(54)=dydt(54)+k(8)*y(638);
dydt(223)=dydt(223)+k(8)*y(638);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(638)=dydt(638)-k(6)*y(638);
dydt(118)=dydt(118)+k(6)*y(638);
dydt(222)=dydt(222)+k(6)*y(638);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(639)=dydt(639)-k(34)*y(639);
dydt(571)=dydt(571)+k(34)*y(639);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(639)=dydt(639)-k(33)*y(639);
dydt(637)=dydt(637)+k(33)*y(639);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(639)=dydt(639)-k(30)*y(639);
dydt(641)=dydt(641)+k(30)*y(639);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(639)=dydt(639)-k(25)*y(639);
dydt(561)=dydt(561)+k(25)*y(639);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

dydt(639)=dydt(639)-k(22)*y(639);
dydt(32)=dydt(32)+k(22)*y(639);
dydt(638)=dydt(638)+k(22)*y(639);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(639)=dydt(639)-k(22)*y(639);
dydt(182)=dydt(182)+k(22)*y(639);
dydt(633)=dydt(633)+k(22)*y(639);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(639)=dydt(639)-k(16)*y(639);
dydt(3)=dydt(3)+k(16)*y(639);
dydt(286)=dydt(286)+k(16)*y(639);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(639)=dydt(639)-k(16)*y(639);
dydt(3)=dydt(3)+k(16)*y(639);
dydt(555)=dydt(555)+k(16)*y(639);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(639)=dydt(639)-k(8)*y(639);
dydt(37)=dydt(37)+k(8)*y(639);
dydt(223)=dydt(223)+k(8)*y(639);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(639)=dydt(639)-k(6)*y(639);
dydt(222)=dydt(222)+k(6)*y(639);
dydt(554)=dydt(554)+k(6)*y(639);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(643)=dydt(643)-k(34)*y(643);
dydt(567)=dydt(567)+k(34)*y(643);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(643)=dydt(643)-k(33)*y(643);
dydt(494)=dydt(494)+k(33)*y(643);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(643)=dydt(643)-k(33)*y(643);
dydt(258)=dydt(258)+k(33)*y(643);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(643)=dydt(643)-k(29)*y(643);
dydt(642)=dydt(642)+k(29)*y(643);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

dydt(643)=dydt(643)-k(22)*y(643);
dydt(197)=dydt(197)+k(22)*y(643);
dydt(244)=dydt(244)+k(22)*y(643);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

dydt(643)=dydt(643)-k(22)*y(643);
dydt(32)=dydt(32)+k(22)*y(643);
dydt(622)=dydt(622)+k(22)*y(643);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(643)=dydt(643)-k(16)*y(643);
dydt(3)=dydt(3)+k(16)*y(643);
dydt(248)=dydt(248)+k(16)*y(643);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(643)=dydt(643)-k(16)*y(643);
dydt(3)=dydt(3)+k(16)*y(643);
dydt(569)=dydt(569)+k(16)*y(643);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(643)=dydt(643)-k(8)*y(643);
dydt(227)=dydt(227)+k(8)*y(643);
dydt(38)=dydt(38)+k(8)*y(643);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(643)=dydt(643)-k(6)*y(643);
dydt(37)=dydt(37)+k(6)*y(643);
dydt(543)=dydt(543)+k(6)*y(643);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(642)=dydt(642)-k(34)*y(642);
dydt(561)=dydt(561)+k(34)*y(642);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(642)=dydt(642)-k(34)*y(642);
dydt(643)=dydt(643)+k(34)*y(642);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(642)=dydt(642)-k(33)*y(642);
dydt(489)=dydt(489)+k(33)*y(642);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(642)=dydt(642)-k(33)*y(642);
dydt(641)=dydt(641)+k(33)*y(642);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

dydt(642)=dydt(642)-k(22)*y(642);
dydt(197)=dydt(197)+k(22)*y(642);
dydt(636)=dydt(636)+k(22)*y(642);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

dydt(642)=dydt(642)-k(22)*y(642);
dydt(197)=dydt(197)+k(22)*y(642);
dydt(622)=dydt(622)+k(22)*y(642);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(642)=dydt(642)-k(16)*y(642);
dydt(3)=dydt(3)+k(16)*y(642);
dydt(492)=dydt(492)+k(16)*y(642);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(642)=dydt(642)-k(16)*y(642);
dydt(3)=dydt(3)+k(16)*y(642);
dydt(565)=dydt(565)+k(16)*y(642);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(642)=dydt(642)-k(8)*y(642);
dydt(227)=dydt(227)+k(8)*y(642);
dydt(228)=dydt(228)+k(8)*y(642);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(642)=dydt(642)-k(6)*y(642);
dydt(227)=dydt(227)+k(6)*y(642);
dydt(543)=dydt(543)+k(6)*y(642);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(641)=dydt(641)-k(34)*y(641);
dydt(639)=dydt(639)+k(34)*y(641);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(641)=dydt(641)-k(34)*y(641);
dydt(258)=dydt(258)+k(34)*y(641);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(641)=dydt(641)-k(33)*y(641);
dydt(640)=dydt(640)+k(33)*y(641);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(641)=dydt(641)-k(25)*y(641);
dydt(642)=dydt(642)+k(25)*y(641);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

dydt(641)=dydt(641)-k(22)*y(641);
dydt(197)=dydt(197)+k(22)*y(641);
dydt(638)=dydt(638)+k(22)*y(641);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

dydt(641)=dydt(641)-k(22)*y(641);
dydt(182)=dydt(182)+k(22)*y(641);
dydt(622)=dydt(622)+k(22)*y(641);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(641)=dydt(641)-k(16)*y(641);
dydt(3)=dydt(3)+k(16)*y(641);
dydt(261)=dydt(261)+k(16)*y(641);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(641)=dydt(641)-k(16)*y(641);
dydt(3)=dydt(3)+k(16)*y(641);
dydt(551)=dydt(551)+k(16)*y(641);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(641)=dydt(641)-k(8)*y(641);
dydt(227)=dydt(227)+k(8)*y(641);
dydt(223)=dydt(223)+k(8)*y(641);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(641)=dydt(641)-k(6)*y(641);
dydt(222)=dydt(222)+k(6)*y(641);
dydt(543)=dydt(543)+k(6)*y(641);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(640)=dydt(640)-k(34)*y(640);
dydt(637)=dydt(637)+k(34)*y(640);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(640)=dydt(640)-k(34)*y(640);
dydt(265)=dydt(265)+k(34)*y(640);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(640)=dydt(640)-k(26)*y(640);
dydt(641)=dydt(641)+k(26)*y(640);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(640)=dydt(640)-k(25)*y(640);
dydt(489)=dydt(489)+k(25)*y(640);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

dydt(640)=dydt(640)-k(22)*y(640);
dydt(182)=dydt(182)+k(22)*y(640);
dydt(638)=dydt(638)+k(22)*y(640);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

dydt(640)=dydt(640)-k(22)*y(640);
dydt(182)=dydt(182)+k(22)*y(640);
dydt(624)=dydt(624)+k(22)*y(640);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(640)=dydt(640)-k(16)*y(640);
dydt(3)=dydt(3)+k(16)*y(640);
dydt(268)=dydt(268)+k(16)*y(640);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(640)=dydt(640)-k(16)*y(640);
dydt(3)=dydt(3)+k(16)*y(640);
dydt(517)=dydt(517)+k(16)*y(640);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(640)=dydt(640)-k(8)*y(640);
dydt(222)=dydt(222)+k(8)*y(640);
dydt(223)=dydt(223)+k(8)*y(640);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(640)=dydt(640)-k(6)*y(640);
dydt(222)=dydt(222)+k(6)*y(640);
dydt(515)=dydt(515)+k(6)*y(640);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(637)=dydt(637)-k(34)*y(637);
dydt(630)=dydt(630)+k(34)*y(637);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(637)=dydt(637)-k(30)*y(637);
dydt(640)=dydt(640)+k(30)*y(637);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(637)=dydt(637)-k(26)*y(637);
dydt(639)=dydt(639)+k(26)*y(637);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(637)=dydt(637)-k(25)*y(637);
dydt(632)=dydt(632)+k(25)*y(637);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

dydt(637)=dydt(637)-k(22)*y(637);
dydt(2)=dydt(2)+k(22)*y(637);
dydt(638)=dydt(638)+k(22)*y(637);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(637)=dydt(637)-k(22)*y(637);
dydt(182)=dydt(182)+k(22)*y(637);
dydt(628)=dydt(628)+k(22)*y(637);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(637)=dydt(637)-k(16)*y(637);
dydt(3)=dydt(3)+k(16)*y(637);
dydt(281)=dydt(281)+k(16)*y(637);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(637)=dydt(637)-k(16)*y(637);
dydt(3)=dydt(3)+k(16)*y(637);
dydt(523)=dydt(523)+k(16)*y(637);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(637)=dydt(637)-k(8)*y(637);
dydt(43)=dydt(43)+k(8)*y(637);
dydt(223)=dydt(223)+k(8)*y(637);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(637)=dydt(637)-k(6)*y(637);
dydt(222)=dydt(222)+k(6)*y(637);
dydt(114)=dydt(114)+k(6)*y(637);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(632)=dydt(632)-k(34)*y(632);
dydt(631)=dydt(631)+k(34)*y(632);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(632)=dydt(632)-k(33)*y(632);
dydt(637)=dydt(637)+k(33)*y(632);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(632)=dydt(632)-k(30)*y(632);
dydt(489)=dydt(489)+k(30)*y(632);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(632)=dydt(632)-k(26)*y(632);
dydt(561)=dydt(561)+k(26)*y(632);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(632)=dydt(632)-k(22)*y(632);
dydt(197)=dydt(197)+k(22)*y(632);
dydt(628)=dydt(628)+k(22)*y(632);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

dydt(632)=dydt(632)-k(22)*y(632);
dydt(2)=dydt(2)+k(22)*y(632);
dydt(636)=dydt(636)+k(22)*y(632);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(632)=dydt(632)-k(16)*y(632);
dydt(3)=dydt(3)+k(16)*y(632);
dydt(528)=dydt(528)+k(16)*y(632);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(632)=dydt(632)-k(16)*y(632);
dydt(3)=dydt(3)+k(16)*y(632);
dydt(485)=dydt(485)+k(16)*y(632);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(632)=dydt(632)-k(8)*y(632);
dydt(43)=dydt(43)+k(8)*y(632);
dydt(228)=dydt(228)+k(8)*y(632);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(632)=dydt(632)-k(6)*y(632);
dydt(227)=dydt(227)+k(6)*y(632);
dydt(114)=dydt(114)+k(6)*y(632);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(635)=dydt(635)-k(34)*y(635);
dydt(247)=dydt(247)+k(34)*y(635);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(635)=dydt(635)-k(33)*y(635);
dydt(286)=dydt(286)+k(33)*y(635);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(635)=dydt(635)-k(33)*y(635);
dydt(485)=dydt(485)+k(33)*y(635);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(635)=dydt(635)-k(31)*y(635);
dydt(474)=dydt(474)+k(31)*y(635);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(635)=dydt(635)-k(30)*y(635);
dydt(492)=dydt(492)+k(30)*y(635);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(635)=dydt(635)-k(22)*y(635);
dydt(197)=dydt(197)+k(22)*y(635);
dydt(634)=dydt(634)+k(22)*y(635);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(635)=dydt(635)-k(22)*y(635);
dydt(32)=dydt(32)+k(22)*y(635);
dydt(491)=dydt(491)+k(22)*y(635);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(635)=dydt(635)-k(16)*y(635);
dydt(3)=dydt(3)+k(16)*y(635);
dydt(562)=dydt(562)+k(16)*y(635);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(635);
dydt(635)=dydt(635)-k(15)*y(3)*y(635);
dydt(561)=dydt(561)+k(15)*y(3)*y(635);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(635)=dydt(635)-k(8)*y(635);
dydt(33)=dydt(33)+k(8)*y(635);
dydt(228)=dydt(228)+k(8)*y(635);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(635)=dydt(635)-k(6)*y(635);
dydt(82)=dydt(82)+k(6)*y(635);
dydt(227)=dydt(227)+k(6)*y(635);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(634)=dydt(634)-k(33)*y(634);
dydt(629)=dydt(629)+k(33)*y(634);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(634)=dydt(634)-k(32)*y(634);
dydt(81)=dydt(81)+k(32)*y(634);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(634)=dydt(634)-k(31)*y(634);
dydt(287)=dydt(287)+k(31)*y(634);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

dydt(634)=dydt(634)-k(22)*y(634);
dydt(32)=dydt(32)+k(22)*y(634);
dydt(238)=dydt(238)+k(22)*y(634);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(634);
dydt(634)=dydt(634)-k(21)*y(197)*y(634);
dydt(635)=dydt(635)+k(21)*y(197)*y(634);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(634);
dydt(634)=dydt(634)-k(21)*y(32)*y(634);
dydt(247)=dydt(247)+k(21)*y(32)*y(634);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(634);
dydt(634)=dydt(634)-k(21)*y(2)*y(634);
dydt(254)=dydt(254)+k(21)*y(2)*y(634);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(634)=dydt(634)-k(16)*y(634);
dydt(3)=dydt(3)+k(16)*y(634);
dydt(573)=dydt(573)+k(16)*y(634);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(634);
dydt(634)=dydt(634)-k(15)*y(3)*y(634);
dydt(633)=dydt(633)+k(15)*y(3)*y(634);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(634)=dydt(634)-k(8)*y(634);
dydt(55)=dydt(55)+k(8)*y(634);
dydt(33)=dydt(33)+k(8)*y(634);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(634)=dydt(634)-k(6)*y(634);
dydt(54)=dydt(54)+k(6)*y(634);
dydt(82)=dydt(82)+k(6)*y(634);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(633)=dydt(633)-k(33)*y(633);
dydt(628)=dydt(628)+k(33)*y(633);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(633)=dydt(633)-k(32)*y(633);
dydt(83)=dydt(83)+k(32)*y(633);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(633)=dydt(633)-k(22)*y(633);
dydt(32)=dydt(32)+k(22)*y(633);
dydt(627)=dydt(627)+k(22)*y(633);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(633);
dydt(633)=dydt(633)-k(21)*y(197)*y(633);
dydt(561)=dydt(561)+k(21)*y(197)*y(633);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(633);
dydt(633)=dydt(633)-k(21)*y(32)*y(633);
dydt(567)=dydt(567)+k(21)*y(32)*y(633);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(633);
dydt(633)=dydt(633)-k(21)*y(2)*y(633);
dydt(571)=dydt(571)+k(21)*y(2)*y(633);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(633)=dydt(633)-k(16)*y(633);
dydt(3)=dydt(3)+k(16)*y(633);
dydt(634)=dydt(634)+k(16)*y(633);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(633)=dydt(633)-k(16)*y(633);
dydt(3)=dydt(3)+k(16)*y(633);
dydt(563)=dydt(563)+k(16)*y(633);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(633)=dydt(633)-k(8)*y(633);
dydt(55)=dydt(55)+k(8)*y(633);
dydt(37)=dydt(37)+k(8)*y(633);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(633)=dydt(633)-k(6)*y(633);
dydt(54)=dydt(54)+k(6)*y(633);
dydt(554)=dydt(554)+k(6)*y(633);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(628)=dydt(628)-k(32)*y(628);
dydt(307)=dydt(307)+k(32)*y(628);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(628)=dydt(628)-k(26)*y(628);
dydt(633)=dydt(633)+k(26)*y(628);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(628)=dydt(628)-k(22)*y(628);
dydt(2)=dydt(2)+k(22)*y(628);
dydt(627)=dydt(627)+k(22)*y(628);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(628);
dydt(628)=dydt(628)-k(21)*y(197)*y(628);
dydt(632)=dydt(632)+k(21)*y(197)*y(628);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(628);
dydt(628)=dydt(628)-k(21)*y(32)*y(628);
dydt(631)=dydt(631)+k(21)*y(32)*y(628);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(628);
dydt(628)=dydt(628)-k(21)*y(2)*y(628);
dydt(630)=dydt(630)+k(21)*y(2)*y(628);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(628)=dydt(628)-k(16)*y(628);
dydt(3)=dydt(3)+k(16)*y(628);
dydt(629)=dydt(629)+k(16)*y(628);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(628)=dydt(628)-k(16)*y(628);
dydt(3)=dydt(3)+k(16)*y(628);
dydt(308)=dydt(308)+k(16)*y(628);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(628)=dydt(628)-k(8)*y(628);
dydt(55)=dydt(55)+k(8)*y(628);
dydt(43)=dydt(43)+k(8)*y(628);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(628)=dydt(628)-k(6)*y(628);
dydt(54)=dydt(54)+k(6)*y(628);
dydt(114)=dydt(114)+k(6)*y(628);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(624)=dydt(624)-k(34)*y(624);
dydt(628)=dydt(628)+k(34)*y(624);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(624)=dydt(624)-k(32)*y(624);
dydt(301)=dydt(301)+k(32)*y(624);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(624)=dydt(624)-k(26)*y(624);
dydt(622)=dydt(622)+k(26)*y(624);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(624)=dydt(624)-k(22)*y(624);
dydt(182)=dydt(182)+k(22)*y(624);
dydt(627)=dydt(627)+k(22)*y(624);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(197)=dydt(197)-k(21)*y(197)*y(624);
dydt(624)=dydt(624)-k(21)*y(197)*y(624);
dydt(489)=dydt(489)+k(21)*y(197)*y(624);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(624);
dydt(624)=dydt(624)-k(21)*y(32)*y(624);
dydt(494)=dydt(494)+k(21)*y(32)*y(624);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(2)=dydt(2)-k(21)*y(2)*y(624);
dydt(624)=dydt(624)-k(21)*y(2)*y(624);
dydt(265)=dydt(265)+k(21)*y(2)*y(624);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(624)=dydt(624)-k(16)*y(624);
dydt(3)=dydt(3)+k(16)*y(624);
dydt(623)=dydt(623)+k(16)*y(624);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(624)=dydt(624)-k(16)*y(624);
dydt(3)=dydt(3)+k(16)*y(624);
dydt(267)=dydt(267)+k(16)*y(624);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(624)=dydt(624)-k(8)*y(624);
dydt(55)=dydt(55)+k(8)*y(624);
dydt(222)=dydt(222)+k(8)*y(624);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(624)=dydt(624)-k(6)*y(624);
dydt(54)=dydt(54)+k(6)*y(624);
dydt(515)=dydt(515)+k(6)*y(624);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(625)=dydt(625)-k(34)*y(625);
dydt(504)=dydt(504)+k(34)*y(625);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(625)=dydt(625)-k(31)*y(625);
dydt(511)=dydt(511)+k(31)*y(625);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(625)=dydt(625)-k(29)*y(625);
dydt(517)=dydt(517)+k(29)*y(625);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(625)=dydt(625)-k(25)*y(625);
dydt(496)=dydt(496)+k(25)*y(625);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(625)=dydt(625)-k(22)*y(625);
dydt(2)=dydt(2)+k(22)*y(625);
dydt(623)=dydt(623)+k(22)*y(625);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(625)=dydt(625)-k(22)*y(625);
dydt(182)=dydt(182)+k(22)*y(625);
dydt(540)=dydt(540)+k(22)*y(625);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(625)=dydt(625)-k(16)*y(625);
dydt(3)=dydt(3)+k(16)*y(625);
dydt(266)=dydt(266)+k(16)*y(625);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(625);
dydt(625)=dydt(625)-k(15)*y(3)*y(625);
dydt(265)=dydt(265)+k(15)*y(3)*y(625);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(625)=dydt(625)-k(8)*y(625);
dydt(437)=dydt(437)+k(8)*y(625);
dydt(222)=dydt(222)+k(8)*y(625);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(625)=dydt(625)-k(6)*y(625);
dydt(111)=dydt(111)+k(6)*y(625);
dydt(515)=dydt(515)+k(6)*y(625);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(626)=dydt(626)-k(34)*y(626);
dydt(496)=dydt(496)+k(34)*y(626);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(626)=dydt(626)-k(34)*y(626);
dydt(528)=dydt(528)+k(34)*y(626);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(626)=dydt(626)-k(33)*y(626);
dydt(517)=dydt(517)+k(33)*y(626);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(626)=dydt(626)-k(31)*y(626);
dydt(520)=dydt(520)+k(31)*y(626);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(626)=dydt(626)-k(22)*y(626);
dydt(197)=dydt(197)+k(22)*y(626);
dydt(623)=dydt(623)+k(22)*y(626);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(626)=dydt(626)-k(22)*y(626);
dydt(182)=dydt(182)+k(22)*y(626);
dydt(564)=dydt(564)+k(22)*y(626);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(626)=dydt(626)-k(16)*y(626);
dydt(3)=dydt(3)+k(16)*y(626);
dydt(490)=dydt(490)+k(16)*y(626);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(626);
dydt(626)=dydt(626)-k(15)*y(3)*y(626);
dydt(489)=dydt(489)+k(15)*y(3)*y(626);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(626)=dydt(626)-k(8)*y(626);
dydt(319)=dydt(319)+k(8)*y(626);
dydt(222)=dydt(222)+k(8)*y(626);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(626)=dydt(626)-k(6)*y(626);
dydt(257)=dydt(257)+k(6)*y(626);
dydt(515)=dydt(515)+k(6)*y(626);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(623)=dydt(623)-k(34)*y(623);
dydt(308)=dydt(308)+k(34)*y(623);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(623)=dydt(623)-k(32)*y(623);
dydt(303)=dydt(303)+k(32)*y(623);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(623)=dydt(623)-k(31)*y(623);
dydt(518)=dydt(518)+k(31)*y(623);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

dydt(623)=dydt(623)-k(22)*y(623);
dydt(182)=dydt(182)+k(22)*y(623);
dydt(310)=dydt(310)+k(22)*y(623);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(623);
dydt(623)=dydt(623)-k(21)*y(197)*y(623);
dydt(626)=dydt(626)+k(21)*y(197)*y(623);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(623);
dydt(623)=dydt(623)-k(21)*y(32)*y(623);
dydt(496)=dydt(496)+k(21)*y(32)*y(623);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(623);
dydt(623)=dydt(623)-k(21)*y(2)*y(623);
dydt(625)=dydt(625)+k(21)*y(2)*y(623);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(623)=dydt(623)-k(16)*y(623);
dydt(3)=dydt(3)+k(16)*y(623);
dydt(497)=dydt(497)+k(16)*y(623);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(623);
dydt(623)=dydt(623)-k(15)*y(3)*y(623);
dydt(624)=dydt(624)+k(15)*y(3)*y(623);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(623)=dydt(623)-k(8)*y(623);
dydt(315)=dydt(315)+k(8)*y(623);
dydt(222)=dydt(222)+k(8)*y(623);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

dydt(623)=dydt(623)-k(6)*y(623);
dydt(77)=dydt(77)+k(6)*y(623);
dydt(515)=dydt(515)+k(6)*y(623);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(581)=dydt(581)-k(34)*y(581);
dydt(563)=dydt(563)+k(34)*y(581);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(581)=dydt(581)-k(33)*y(581);
dydt(623)=dydt(623)+k(33)*y(581);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(581)=dydt(581)-k(32)*y(581);
dydt(312)=dydt(312)+k(32)*y(581);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(581)=dydt(581)-k(31)*y(581);
dydt(547)=dydt(547)+k(31)*y(581);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

dydt(581)=dydt(581)-k(22)*y(581);
dydt(197)=dydt(197)+k(22)*y(581);
dydt(310)=dydt(310)+k(22)*y(581);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(581);
dydt(581)=dydt(581)-k(21)*y(197)*y(581);
dydt(565)=dydt(565)+k(21)*y(197)*y(581);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(581);
dydt(581)=dydt(581)-k(21)*y(32)*y(581);
dydt(569)=dydt(569)+k(21)*y(32)*y(581);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(581);
dydt(581)=dydt(581)-k(21)*y(2)*y(581);
dydt(546)=dydt(546)+k(21)*y(2)*y(581);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(581)=dydt(581)-k(16)*y(581);
dydt(3)=dydt(3)+k(16)*y(581);
dydt(578)=dydt(578)+k(16)*y(581);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(581);
dydt(581)=dydt(581)-k(15)*y(3)*y(581);
dydt(622)=dydt(622)+k(15)*y(3)*y(581);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(581)=dydt(581)-k(8)*y(581);
dydt(315)=dydt(315)+k(8)*y(581);
dydt(227)=dydt(227)+k(8)*y(581);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

dydt(581)=dydt(581)-k(6)*y(581);
dydt(77)=dydt(77)+k(6)*y(581);
dydt(543)=dydt(543)+k(6)*y(581);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(582)=dydt(582)-k(34)*y(582);
dydt(568)=dydt(568)+k(34)*y(582);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(582)=dydt(582)-k(33)*y(582);
dydt(259)=dydt(259)+k(33)*y(582);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(582)=dydt(582)-k(33)*y(582);
dydt(495)=dydt(495)+k(33)*y(582);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(582)=dydt(582)-k(31)*y(582);
dydt(608)=dydt(608)+k(31)*y(582);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(582)=dydt(582)-k(31)*y(582);
dydt(620)=dydt(620)+k(31)*y(582);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(582)=dydt(582)-k(29)*y(582);
dydt(583)=dydt(583)+k(29)*y(582);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(582)=dydt(582)-k(22)*y(582);
dydt(32)=dydt(32)+k(22)*y(582);
dydt(578)=dydt(578)+k(22)*y(582);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(582)=dydt(582)-k(22)*y(582);
dydt(197)=dydt(197)+k(22)*y(582);
dydt(245)=dydt(245)+k(22)*y(582);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(582);
dydt(582)=dydt(582)-k(15)*y(3)*y(582);
dydt(569)=dydt(569)+k(15)*y(3)*y(582);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(582);
dydt(582)=dydt(582)-k(15)*y(3)*y(582);
dydt(248)=dydt(248)+k(15)*y(3)*y(582);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(582)=dydt(582)-k(8)*y(582);
dydt(257)=dydt(257)+k(8)*y(582);
dydt(34)=dydt(34)+k(8)*y(582);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(582)=dydt(582)-k(6)*y(582);
dydt(33)=dydt(33)+k(6)*y(582);
dydt(256)=dydt(256)+k(6)*y(582);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(583)=dydt(583)-k(34)*y(583);
dydt(582)=dydt(582)+k(34)*y(583);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(583)=dydt(583)-k(34)*y(583);
dydt(562)=dydt(562)+k(34)*y(583);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(583)=dydt(583)-k(33)*y(583);
dydt(577)=dydt(577)+k(33)*y(583);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(583)=dydt(583)-k(33)*y(583);
dydt(490)=dydt(490)+k(33)*y(583);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(583)=dydt(583)-k(31)*y(583);
dydt(609)=dydt(609)+k(31)*y(583);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(583)=dydt(583)-k(31)*y(583);
dydt(619)=dydt(619)+k(31)*y(583);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(583)=dydt(583)-k(22)*y(583);
dydt(197)=dydt(197)+k(22)*y(583);
dydt(578)=dydt(578)+k(22)*y(583);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(583)=dydt(583)-k(22)*y(583);
dydt(197)=dydt(197)+k(22)*y(583);
dydt(613)=dydt(613)+k(22)*y(583);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(583);
dydt(583)=dydt(583)-k(15)*y(3)*y(583);
dydt(565)=dydt(565)+k(15)*y(3)*y(583);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(583);
dydt(583)=dydt(583)-k(15)*y(3)*y(583);
dydt(492)=dydt(492)+k(15)*y(3)*y(583);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(583)=dydt(583)-k(8)*y(583);
dydt(257)=dydt(257)+k(8)*y(583);
dydt(319)=dydt(319)+k(8)*y(583);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(583)=dydt(583)-k(6)*y(583);
dydt(257)=dydt(257)+k(6)*y(583);
dydt(256)=dydt(256)+k(6)*y(583);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(613)=dydt(613)-k(34)*y(613);
dydt(245)=dydt(245)+k(34)*y(613);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(613)=dydt(613)-k(33)*y(613);
dydt(576)=dydt(576)+k(33)*y(613);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(613)=dydt(613)-k(32)*y(613);
dydt(321)=dydt(321)+k(32)*y(613);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) 

dydt(613)=dydt(613)-k(31)*y(613);
dydt(338)=dydt(338)+k(31)*y(613);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(613)=dydt(613)-k(31)*y(613);
dydt(618)=dydt(618)+k(31)*y(613);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(613)=dydt(613)-k(22)*y(613);
dydt(197)=dydt(197)+k(22)*y(613);
dydt(584)=dydt(584)+k(22)*y(613);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(613);
dydt(613)=dydt(613)-k(21)*y(197)*y(613);
dydt(583)=dydt(583)+k(21)*y(197)*y(613);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(613);
dydt(613)=dydt(613)-k(21)*y(32)*y(613);
dydt(562)=dydt(562)+k(21)*y(32)*y(613);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(613);
dydt(613)=dydt(613)-k(21)*y(2)*y(613);
dydt(502)=dydt(502)+k(21)*y(2)*y(613);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(613);
dydt(613)=dydt(613)-k(15)*y(3)*y(613);
dydt(564)=dydt(564)+k(15)*y(3)*y(613);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(613);
dydt(613)=dydt(613)-k(15)*y(3)*y(613);
dydt(491)=dydt(491)+k(15)*y(3)*y(613);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

dydt(613)=dydt(613)-k(8)*y(613);
dydt(77)=dydt(77)+k(8)*y(613);
dydt(319)=dydt(319)+k(8)*y(613);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(613)=dydt(613)-k(6)*y(613);
dydt(76)=dydt(76)+k(6)*y(613);
dydt(257)=dydt(257)+k(6)*y(613);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(615)=dydt(615)-k(33)*y(615);
dydt(425)=dydt(425)+k(33)*y(615);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(615)=dydt(615)-k(32)*y(615);
dydt(87)=dydt(87)+k(32)*y(615);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(615)=dydt(615)-k(31)*y(615);
dydt(450)=dydt(450)+k(31)*y(615);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(615)=dydt(615)-k(22)*y(615);
dydt(32)=dydt(32)+k(22)*y(615);
dydt(614)=dydt(614)+k(22)*y(615);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(615);
dydt(615)=dydt(615)-k(21)*y(197)*y(615);
dydt(472)=dydt(472)+k(21)*y(197)*y(615);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(615);
dydt(615)=dydt(615)-k(21)*y(32)*y(615);
dydt(468)=dydt(468)+k(21)*y(32)*y(615);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(615);
dydt(615)=dydt(615)-k(21)*y(2)*y(615);
dydt(442)=dydt(442)+k(21)*y(2)*y(615);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(615);
dydt(615)=dydt(615)-k(15)*y(3)*y(615);
dydt(287)=dydt(287)+k(15)*y(3)*y(615);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(615);
dydt(615)=dydt(615)-k(9)*y(3)*y(615);
dydt(443)=dydt(443)+k(9)*y(3)*y(615);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(615)=dydt(615)-k(8)*y(615);
dydt(315)=dydt(315)+k(8)*y(615);
dydt(27)=dydt(27)+k(8)*y(615);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

dydt(615)=dydt(615)-k(6)*y(615);
dydt(77)=dydt(77)+k(6)*y(615);
dydt(86)=dydt(86)+k(6)*y(615);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(621)=dydt(621)-k(31)*y(621);
dydt(24)=dydt(24)+k(31)*y(621);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(621)=dydt(621)-k(22)*y(621);
dydt(2)=dydt(2)+k(22)*y(621);
dydt(234)=dydt(234)+k(22)*y(621);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(621);
dydt(621)=dydt(621)-k(15)*y(3)*y(621);
dydt(45)=dydt(45)+k(15)*y(3)*y(621);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(621);
dydt(621)=dydt(621)-k(9)*y(3)*y(621);
dydt(25)=dydt(25)+k(9)*y(3)*y(621);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

dydt(621)=dydt(621)-k(8)*y(621);
dydt(1)=dydt(1)+k(8)*y(621);
dydt(437)=dydt(437)+k(8)*y(621);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(621)=dydt(621)-k(6)*y(621);
dydt(6)=dydt(6)+k(6)*y(621);
dydt(111)=dydt(111)+k(6)*y(621);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(616)=dydt(616)-k(32)*y(616);
dydt(621)=dydt(621)+k(32)*y(616);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(616)=dydt(616)-k(31)*y(616);
dydt(452)=dydt(452)+k(31)*y(616);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(616)=dydt(616)-k(22)*y(616);
dydt(2)=dydt(2)+k(22)*y(616);
dydt(614)=dydt(614)+k(22)*y(616);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(616);
dydt(616)=dydt(616)-k(21)*y(197)*y(616);
dydt(438)=dydt(438)+k(21)*y(197)*y(616);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(616);
dydt(616)=dydt(616)-k(21)*y(32)*y(616);
dydt(442)=dydt(442)+k(21)*y(32)*y(616);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(616);
dydt(616)=dydt(616)-k(21)*y(2)*y(616);
dydt(275)=dydt(275)+k(21)*y(2)*y(616);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(616);
dydt(616)=dydt(616)-k(15)*y(3)*y(616);
dydt(49)=dydt(49)+k(15)*y(3)*y(616);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(616);
dydt(616)=dydt(616)-k(9)*y(3)*y(616);
dydt(50)=dydt(50)+k(9)*y(3)*y(616);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

dydt(616)=dydt(616)-k(8)*y(616);
dydt(15)=dydt(15)+k(8)*y(616);
dydt(437)=dydt(437)+k(8)*y(616);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(616)=dydt(616)-k(6)*y(616);
dydt(98)=dydt(98)+k(6)*y(616);
dydt(111)=dydt(111)+k(6)*y(616);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(617)=dydt(617)-k(33)*y(617);
dydt(616)=dydt(616)+k(33)*y(617);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(617)=dydt(617)-k(32)*y(617);
dydt(35)=dydt(35)+k(32)*y(617);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(617)=dydt(617)-k(31)*y(617);
dydt(156)=dydt(156)+k(31)*y(617);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(617)=dydt(617)-k(22)*y(617);
dydt(32)=dydt(32)+k(22)*y(617);
dydt(614)=dydt(614)+k(22)*y(617);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(617);
dydt(617)=dydt(617)-k(21)*y(197)*y(617);
dydt(620)=dydt(620)+k(21)*y(197)*y(617);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(617);
dydt(617)=dydt(617)-k(21)*y(32)*y(617);
dydt(468)=dydt(468)+k(21)*y(32)*y(617);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(617);
dydt(617)=dydt(617)-k(21)*y(2)*y(617);
dydt(482)=dydt(482)+k(21)*y(2)*y(617);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(617);
dydt(617)=dydt(617)-k(15)*y(3)*y(617);
dydt(249)=dydt(249)+k(15)*y(3)*y(617);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(617);
dydt(617)=dydt(617)-k(9)*y(3)*y(617);
dydt(470)=dydt(470)+k(9)*y(3)*y(617);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

dydt(617)=dydt(617)-k(8)*y(617);
dydt(15)=dydt(15)+k(8)*y(617);
dydt(34)=dydt(34)+k(8)*y(617);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(617)=dydt(617)-k(6)*y(617);
dydt(98)=dydt(98)+k(6)*y(617);
dydt(33)=dydt(33)+k(6)*y(617);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(620)=dydt(620)-k(34)*y(620);
dydt(468)=dydt(468)+k(34)*y(620);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(620)=dydt(620)-k(33)*y(620);
dydt(498)=dydt(498)+k(33)*y(620);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(620)=dydt(620)-k(33)*y(620);
dydt(438)=dydt(438)+k(33)*y(620);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(620)=dydt(620)-k(31)*y(620);
dydt(456)=dydt(456)+k(31)*y(620);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(620)=dydt(620)-k(29)*y(620);
dydt(619)=dydt(619)+k(29)*y(620);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(620)=dydt(620)-k(22)*y(620);
dydt(197)=dydt(197)+k(22)*y(620);
dydt(617)=dydt(617)+k(22)*y(620);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(620)=dydt(620)-k(22)*y(620);
dydt(32)=dydt(32)+k(22)*y(620);
dydt(292)=dydt(292)+k(22)*y(620);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(620);
dydt(620)=dydt(620)-k(15)*y(3)*y(620);
dydt(293)=dydt(293)+k(15)*y(3)*y(620);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(620);
dydt(620)=dydt(620)-k(9)*y(3)*y(620);
dydt(439)=dydt(439)+k(9)*y(3)*y(620);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(620)=dydt(620)-k(8)*y(620);
dydt(192)=dydt(192)+k(8)*y(620);
dydt(34)=dydt(34)+k(8)*y(620);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(620)=dydt(620)-k(6)*y(620);
dydt(33)=dydt(33)+k(6)*y(620);
dydt(191)=dydt(191)+k(6)*y(620);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(619)=dydt(619)-k(34)*y(619);
dydt(472)=dydt(472)+k(34)*y(619);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(619)=dydt(619)-k(34)*y(619);
dydt(620)=dydt(620)+k(34)*y(619);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(619)=dydt(619)-k(33)*y(619);
dydt(487)=dydt(487)+k(33)*y(619);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(619)=dydt(619)-k(33)*y(619);
dydt(290)=dydt(290)+k(33)*y(619);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(619)=dydt(619)-k(31)*y(619);
dydt(455)=dydt(455)+k(31)*y(619);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(619)=dydt(619)-k(22)*y(619);
dydt(197)=dydt(197)+k(22)*y(619);
dydt(618)=dydt(618)+k(22)*y(619);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(619)=dydt(619)-k(22)*y(619);
dydt(197)=dydt(197)+k(22)*y(619);
dydt(292)=dydt(292)+k(22)*y(619);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(619);
dydt(619)=dydt(619)-k(15)*y(3)*y(619);
dydt(294)=dydt(294)+k(15)*y(3)*y(619);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(619);
dydt(619)=dydt(619)-k(9)*y(3)*y(619);
dydt(434)=dydt(434)+k(9)*y(3)*y(619);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(619)=dydt(619)-k(8)*y(619);
dydt(192)=dydt(192)+k(8)*y(619);
dydt(319)=dydt(319)+k(8)*y(619);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(619)=dydt(619)-k(6)*y(619);
dydt(257)=dydt(257)+k(6)*y(619);
dydt(191)=dydt(191)+k(6)*y(619);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(618)=dydt(618)-k(34)*y(618);
dydt(617)=dydt(617)+k(34)*y(618);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

dydt(618)=dydt(618)-k(33)*y(618);
dydt(431)=dydt(431)+k(33)*y(618);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(618)=dydt(618)-k(32)*y(618);
dydt(230)=dydt(230)+k(32)*y(618);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) 

dydt(618)=dydt(618)-k(31)*y(618);
dydt(376)=dydt(376)+k(31)*y(618);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(618)=dydt(618)-k(22)*y(618);
dydt(197)=dydt(197)+k(22)*y(618);
dydt(614)=dydt(614)+k(22)*y(618);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(618);
dydt(618)=dydt(618)-k(21)*y(197)*y(618);
dydt(619)=dydt(619)+k(21)*y(197)*y(618);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(618);
dydt(618)=dydt(618)-k(21)*y(32)*y(618);
dydt(472)=dydt(472)+k(21)*y(32)*y(618);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(618);
dydt(618)=dydt(618)-k(21)*y(2)*y(618);
dydt(427)=dydt(427)+k(21)*y(2)*y(618);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(618);
dydt(618)=dydt(618)-k(15)*y(3)*y(618);
dydt(232)=dydt(232)+k(15)*y(3)*y(618);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(618);
dydt(618)=dydt(618)-k(9)*y(3)*y(618);
dydt(401)=dydt(401)+k(9)*y(3)*y(618);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

dydt(618)=dydt(618)-k(8)*y(618);
dydt(15)=dydt(15)+k(8)*y(618);
dydt(319)=dydt(319)+k(8)*y(618);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(618)=dydt(618)-k(6)*y(618);
dydt(98)=dydt(98)+k(6)*y(618);
dydt(257)=dydt(257)+k(6)*y(618);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(614)=dydt(614)-k(32)*y(614);
dydt(234)=dydt(234)+k(32)*y(614);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(614)=dydt(614)-k(32)*y(614);
dydt(93)=dydt(93)+k(32)*y(614);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(614)=dydt(614)-k(31)*y(614);
dydt(150)=dydt(150)+k(31)*y(614);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(614);
dydt(614)=dydt(614)-k(21)*y(197)*y(614);
dydt(618)=dydt(618)+k(21)*y(197)*y(614);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(614);
dydt(614)=dydt(614)-k(21)*y(32)*y(614);
dydt(617)=dydt(617)+k(21)*y(32)*y(614);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(614);
dydt(614)=dydt(614)-k(21)*y(2)*y(614);
dydt(616)=dydt(616)+k(21)*y(2)*y(614);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(614);
dydt(614)=dydt(614)-k(21)*y(197)*y(614);
dydt(292)=dydt(292)+k(21)*y(197)*y(614);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(614);
dydt(614)=dydt(614)-k(21)*y(32)*y(614);
dydt(615)=dydt(615)+k(21)*y(32)*y(614);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(614);
dydt(614)=dydt(614)-k(21)*y(2)*y(614);
dydt(425)=dydt(425)+k(21)*y(2)*y(614);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(614);
dydt(614)=dydt(614)-k(15)*y(3)*y(614);
dydt(56)=dydt(56)+k(15)*y(3)*y(614);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(614);
dydt(614)=dydt(614)-k(9)*y(3)*y(614);
dydt(57)=dydt(57)+k(9)*y(3)*y(614);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) 

dydt(614)=dydt(614)-k(8)*y(614);
dydt(15)=dydt(15)+k(8)*y(614);
dydt(315)=dydt(315)+k(8)*y(614);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) 

dydt(614)=dydt(614)-k(6)*y(614);
dydt(77)=dydt(77)+k(6)*y(614);
dydt(98)=dydt(98)+k(6)*y(614);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(584)=dydt(584)-k(32)*y(584);
dydt(237)=dydt(237)+k(32)*y(584);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(584)=dydt(584)-k(32)*y(584);
dydt(79)=dydt(79)+k(32)*y(584);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(584)=dydt(584)-k(31)*y(584);
dydt(614)=dydt(614)+k(31)*y(584);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(584)=dydt(584)-k(31)*y(584);
dydt(610)=dydt(610)+k(31)*y(584);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(584);
dydt(584)=dydt(584)-k(21)*y(197)*y(584);
dydt(613)=dydt(613)+k(21)*y(197)*y(584);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(584);
dydt(584)=dydt(584)-k(21)*y(32)*y(584);
dydt(245)=dydt(245)+k(21)*y(32)*y(584);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(584);
dydt(584)=dydt(584)-k(21)*y(2)*y(584);
dydt(252)=dydt(252)+k(21)*y(2)*y(584);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(584);
dydt(584)=dydt(584)-k(21)*y(197)*y(584);
dydt(578)=dydt(578)+k(21)*y(197)*y(584);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(584);
dydt(584)=dydt(584)-k(21)*y(32)*y(584);
dydt(573)=dydt(573)+k(21)*y(32)*y(584);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(584);
dydt(584)=dydt(584)-k(21)*y(2)*y(584);
dydt(501)=dydt(501)+k(21)*y(2)*y(584);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(584);
dydt(584)=dydt(584)-k(15)*y(3)*y(584);
dydt(238)=dydt(238)+k(15)*y(3)*y(584);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(3)=dydt(3)-k(15)*y(3)*y(584);
dydt(584)=dydt(584)-k(15)*y(3)*y(584);
dydt(310)=dydt(310)+k(15)*y(3)*y(584);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) 

dydt(584)=dydt(584)-k(8)*y(584);
dydt(77)=dydt(77)+k(8)*y(584);
dydt(315)=dydt(315)+k(8)*y(584);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) 

dydt(584)=dydt(584)-k(6)*y(584);
dydt(77)=dydt(77)+k(6)*y(584);
dydt(76)=dydt(76)+k(6)*y(584);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(612)=dydt(612)-k(33)*y(612);
dydt(356)=dydt(356)+k(33)*y(612);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(612)=dydt(612)-k(33)*y(612);
dydt(575)=dydt(575)+k(33)*y(612);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(612)=dydt(612)-k(31)*y(612);
dydt(160)=dydt(160)+k(31)*y(612);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(612)=dydt(612)-k(28)*y(612);
dydt(608)=dydt(608)+k(28)*y(612);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(612)=dydt(612)-k(27)*y(612);
dydt(559)=dydt(559)+k(27)*y(612);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(612)=dydt(612)-k(22)*y(612);
dydt(32)=dydt(32)+k(22)*y(612);
dydt(533)=dydt(533)+k(22)*y(612);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(612)=dydt(612)-k(22)*y(612);
dydt(32)=dydt(32)+k(22)*y(612);
dydt(351)=dydt(351)+k(22)*y(612);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(612);
dydt(612)=dydt(612)-k(15)*y(3)*y(612);
dydt(535)=dydt(535)+k(15)*y(3)*y(612);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(612);
dydt(612)=dydt(612)-k(9)*y(3)*y(612);
dydt(161)=dydt(161)+k(9)*y(3)*y(612);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(612)=dydt(612)-k(8)*y(612);
dydt(33)=dydt(33)+k(8)*y(612);
dydt(28)=dydt(28)+k(8)*y(612);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(612)=dydt(612)-k(6)*y(612);
dydt(27)=dydt(27)+k(6)*y(612);
dydt(82)=dydt(82)+k(6)*y(612);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(608)=dydt(608)-k(34)*y(608);
dydt(612)=dydt(612)+k(34)*y(608);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(608)=dydt(608)-k(33)*y(608);
dydt(545)=dydt(545)+k(33)*y(608);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(608)=dydt(608)-k(33)*y(608);
dydt(359)=dydt(359)+k(33)*y(608);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(608)=dydt(608)-k(31)*y(608);
dydt(456)=dydt(456)+k(31)*y(608);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(608)=dydt(608)-k(29)*y(608);
dydt(609)=dydt(609)+k(29)*y(608);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(608)=dydt(608)-k(22)*y(608);
dydt(32)=dydt(32)+k(22)*y(608);
dydt(585)=dydt(585)+k(22)*y(608);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(608)=dydt(608)-k(22)*y(608);
dydt(197)=dydt(197)+k(22)*y(608);
dydt(533)=dydt(533)+k(22)*y(608);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(608);
dydt(608)=dydt(608)-k(15)*y(3)*y(608);
dydt(536)=dydt(536)+k(15)*y(3)*y(608);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(608);
dydt(608)=dydt(608)-k(9)*y(3)*y(608);
dydt(362)=dydt(362)+k(9)*y(3)*y(608);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(608)=dydt(608)-k(8)*y(608);
dydt(257)=dydt(257)+k(8)*y(608);
dydt(28)=dydt(28)+k(8)*y(608);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(608)=dydt(608)-k(6)*y(608);
dydt(27)=dydt(27)+k(6)*y(608);
dydt(256)=dydt(256)+k(6)*y(608);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(611)=dydt(611)-k(34)*y(611);
dydt(527)=dydt(527)+k(34)*y(611);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(611)=dydt(611)-k(34)*y(611);
dydt(359)=dydt(359)+k(34)*y(611);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(611)=dydt(611)-k(33)*y(611);
dydt(513)=dydt(513)+k(33)*y(611);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(611)=dydt(611)-k(31)*y(611);
dydt(369)=dydt(369)+k(31)*y(611);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(611)=dydt(611)-k(22)*y(611);
dydt(182)=dydt(182)+k(22)*y(611);
dydt(338)=dydt(338)+k(22)*y(611);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(611)=dydt(611)-k(22)*y(611);
dydt(197)=dydt(197)+k(22)*y(611);
dydt(512)=dydt(512)+k(22)*y(611);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(611);
dydt(611)=dydt(611)-k(15)*y(3)*y(611);
dydt(520)=dydt(520)+k(15)*y(3)*y(611);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(611);
dydt(611)=dydt(611)-k(9)*y(3)*y(611);
dydt(363)=dydt(363)+k(9)*y(3)*y(611);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(611)=dydt(611)-k(8)*y(611);
dydt(218)=dydt(218)+k(8)*y(611);
dydt(209)=dydt(209)+k(8)*y(611);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(611)=dydt(611)-k(6)*y(611);
dydt(192)=dydt(192)+k(6)*y(611);
dydt(264)=dydt(264)+k(6)*y(611);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(609)=dydt(609)-k(34)*y(609);
dydt(608)=dydt(608)+k(34)*y(609);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(609)=dydt(609)-k(34)*y(609);
dydt(559)=dydt(559)+k(34)*y(609);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(609)=dydt(609)-k(33)*y(609);
dydt(550)=dydt(550)+k(33)*y(609);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(609)=dydt(609)-k(33)*y(609);
dydt(611)=dydt(611)+k(33)*y(609);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(609)=dydt(609)-k(31)*y(609);
dydt(455)=dydt(455)+k(31)*y(609);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(609)=dydt(609)-k(22)*y(609);
dydt(197)=dydt(197)+k(22)*y(609);
dydt(585)=dydt(585)+k(22)*y(609);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(609)=dydt(609)-k(22)*y(609);
dydt(197)=dydt(197)+k(22)*y(609);
dydt(338)=dydt(338)+k(22)*y(609);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(609);
dydt(609)=dydt(609)-k(15)*y(3)*y(609);
dydt(552)=dydt(552)+k(15)*y(3)*y(609);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(609);
dydt(609)=dydt(609)-k(9)*y(3)*y(609);
dydt(342)=dydt(342)+k(9)*y(3)*y(609);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(609)=dydt(609)-k(8)*y(609);
dydt(257)=dydt(257)+k(8)*y(609);
dydt(209)=dydt(209)+k(8)*y(609);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(609)=dydt(609)-k(6)*y(609);
dydt(192)=dydt(192)+k(6)*y(609);
dydt(256)=dydt(256)+k(6)*y(609);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(610)=dydt(610)-k(32)*y(610);
dydt(331)=dydt(331)+k(32)*y(610);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

dydt(610)=dydt(610)-k(32)*y(610);
dydt(590)=dydt(590)+k(32)*y(610);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(610)=dydt(610)-k(31)*y(610);
dydt(150)=dydt(150)+k(31)*y(610);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(610);
dydt(610)=dydt(610)-k(21)*y(197)*y(610);
dydt(338)=dydt(338)+k(21)*y(197)*y(610);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(610);
dydt(610)=dydt(610)-k(21)*y(32)*y(610);
dydt(533)=dydt(533)+k(21)*y(32)*y(610);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(610);
dydt(610)=dydt(610)-k(21)*y(2)*y(610);
dydt(508)=dydt(508)+k(21)*y(2)*y(610);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(610);
dydt(610)=dydt(610)-k(21)*y(197)*y(610);
dydt(585)=dydt(585)+k(21)*y(197)*y(610);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(610);
dydt(610)=dydt(610)-k(21)*y(32)*y(610);
dydt(351)=dydt(351)+k(21)*y(32)*y(610);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(610);
dydt(610)=dydt(610)-k(21)*y(2)*y(610);
dydt(354)=dydt(354)+k(21)*y(2)*y(610);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(3)=dydt(3)-k(15)*y(3)*y(610);
dydt(610)=dydt(610)-k(15)*y(3)*y(610);
dydt(537)=dydt(537)+k(15)*y(3)*y(610);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(610);
dydt(610)=dydt(610)-k(9)*y(3)*y(610);
dydt(151)=dydt(151)+k(9)*y(3)*y(610);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) 

dydt(610)=dydt(610)-k(8)*y(610);
dydt(77)=dydt(77)+k(8)*y(610);
dydt(16)=dydt(16)+k(8)*y(610);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) 

dydt(610)=dydt(610)-k(6)*y(610);
dydt(15)=dydt(15)+k(6)*y(610);
dydt(76)=dydt(76)+k(6)*y(610);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(585)=dydt(585)-k(34)*y(585);
dydt(351)=dydt(351)+k(34)*y(585);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(585)=dydt(585)-k(33)*y(585);
dydt(512)=dydt(512)+k(33)*y(585);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(585)=dydt(585)-k(32)*y(585);
dydt(587)=dydt(587)+k(32)*y(585);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(585)=dydt(585)-k(31)*y(585);
dydt(409)=dydt(409)+k(31)*y(585);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(585)=dydt(585)-k(22)*y(585);
dydt(197)=dydt(197)+k(22)*y(585);
dydt(610)=dydt(610)+k(22)*y(585);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(585);
dydt(585)=dydt(585)-k(21)*y(197)*y(585);
dydt(609)=dydt(609)+k(21)*y(197)*y(585);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(585);
dydt(585)=dydt(585)-k(21)*y(32)*y(585);
dydt(608)=dydt(608)+k(21)*y(32)*y(585);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(585);
dydt(585)=dydt(585)-k(21)*y(2)*y(585);
dydt(545)=dydt(545)+k(21)*y(2)*y(585);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(585);
dydt(585)=dydt(585)-k(15)*y(3)*y(585);
dydt(547)=dydt(547)+k(15)*y(3)*y(585);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(585);
dydt(585)=dydt(585)-k(9)*y(3)*y(585);
dydt(599)=dydt(599)+k(9)*y(3)*y(585);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(585)=dydt(585)-k(8)*y(585);
dydt(16)=dydt(16)+k(8)*y(585);
dydt(257)=dydt(257)+k(8)*y(585);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

dydt(585)=dydt(585)-k(6)*y(585);
dydt(15)=dydt(15)+k(6)*y(585);
dydt(256)=dydt(256)+k(6)*y(585);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(588)=dydt(588)-k(34)*y(588);
dydt(105)=dydt(105)+k(34)*y(588);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(588)=dydt(588)-k(33)*y(588);
dydt(594)=dydt(594)+k(33)*y(588);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(588)=dydt(588)-k(31)*y(588);
dydt(607)=dydt(607)+k(31)*y(588);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(588)=dydt(588)-k(22)*y(588);
dydt(197)=dydt(197)+k(22)*y(588);
dydt(108)=dydt(108)+k(22)*y(588);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(588);
dydt(588)=dydt(588)-k(15)*y(3)*y(588);
dydt(201)=dydt(201)+k(15)*y(3)*y(588);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(588)=dydt(588)-k(10)*y(588);
dydt(3)=dydt(3)+k(10)*y(588);
dydt(587)=dydt(587)+k(10)*y(588);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(588)=dydt(588)-k(8)*y(588);
dydt(8)=dydt(8)+k(8)*y(588);
dydt(257)=dydt(257)+k(8)*y(588);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(588)=dydt(588)-k(6)*y(588);
dydt(7)=dydt(7)+k(6)*y(588);
dydt(256)=dydt(256)+k(6)*y(588);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(607)=dydt(607)-k(34)*y(607);
dydt(103)=dydt(103)+k(34)*y(607);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(607)=dydt(607)-k(33)*y(607);
dydt(606)=dydt(606)+k(33)*y(607);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(607)=dydt(607)-k(22)*y(607);
dydt(197)=dydt(197)+k(22)*y(607);
dydt(100)=dydt(100)+k(22)*y(607);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(607)=dydt(607)-k(12)*y(607);
dydt(588)=dydt(588)+k(12)*y(607);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(607)=dydt(607)-k(10)*y(607);
dydt(3)=dydt(3)+k(10)*y(607);
dydt(202)=dydt(202)+k(10)*y(607);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(607);
dydt(607)=dydt(607)-k(9)*y(3)*y(607);
dydt(414)=dydt(414)+k(9)*y(3)*y(607);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(607)=dydt(607)-k(8)*y(607);
dydt(8)=dydt(8)+k(8)*y(607);
dydt(192)=dydt(192)+k(8)*y(607);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(607)=dydt(607)-k(6)*y(607);
dydt(7)=dydt(7)+k(6)*y(607);
dydt(191)=dydt(191)+k(6)*y(607);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(606)=dydt(606)-k(34)*y(606);
dydt(96)=dydt(96)+k(34)*y(606);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(606)=dydt(606)-k(24)*y(606);
dydt(607)=dydt(607)+k(24)*y(606);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(606)=dydt(606)-k(22)*y(606);
dydt(182)=dydt(182)+k(22)*y(606);
dydt(100)=dydt(100)+k(22)*y(606);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(606)=dydt(606)-k(12)*y(606);
dydt(594)=dydt(594)+k(12)*y(606);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(606)=dydt(606)-k(10)*y(606);
dydt(3)=dydt(3)+k(10)*y(606);
dydt(418)=dydt(418)+k(10)*y(606);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(606);
dydt(606)=dydt(606)-k(9)*y(3)*y(606);
dydt(417)=dydt(417)+k(9)*y(3)*y(606);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(606)=dydt(606)-k(8)*y(606);
dydt(8)=dydt(8)+k(8)*y(606);
dydt(177)=dydt(177)+k(8)*y(606);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(606)=dydt(606)-k(6)*y(606);
dydt(7)=dydt(7)+k(6)*y(606);
dydt(176)=dydt(176)+k(6)*y(606);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(594)=dydt(594)-k(34)*y(594);
dydt(109)=dydt(109)+k(34)*y(594);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(594)=dydt(594)-k(31)*y(594);
dydt(606)=dydt(606)+k(31)*y(594);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(594)=dydt(594)-k(24)*y(594);
dydt(588)=dydt(588)+k(24)*y(594);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(594)=dydt(594)-k(22)*y(594);
dydt(182)=dydt(182)+k(22)*y(594);
dydt(108)=dydt(108)+k(22)*y(594);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(594);
dydt(594)=dydt(594)-k(15)*y(3)*y(594);
dydt(593)=dydt(593)+k(15)*y(3)*y(594);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(594)=dydt(594)-k(10)*y(594);
dydt(3)=dydt(3)+k(10)*y(594);
dydt(304)=dydt(304)+k(10)*y(594);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(594)=dydt(594)-k(8)*y(594);
dydt(8)=dydt(8)+k(8)*y(594);
dydt(218)=dydt(218)+k(8)*y(594);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(594)=dydt(594)-k(6)*y(594);
dydt(7)=dydt(7)+k(6)*y(594);
dydt(264)=dydt(264)+k(6)*y(594);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(596)=dydt(596)-k(34)*y(596);
dydt(604)=dydt(604)+k(34)*y(596);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(596)=dydt(596)-k(29)*y(596);
dydt(516)=dydt(516)+k(29)*y(596);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(596)=dydt(596)-k(25)*y(596);
dydt(360)=dydt(360)+k(25)*y(596);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(596)=dydt(596)-k(24)*y(596);
dydt(544)=dydt(544)+k(24)*y(596);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(596)=dydt(596)-k(22)*y(596);
dydt(2)=dydt(2)+k(22)*y(596);
dydt(595)=dydt(595)+k(22)*y(596);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(596)=dydt(596)-k(22)*y(596);
dydt(182)=dydt(182)+k(22)*y(596);
dydt(539)=dydt(539)+k(22)*y(596);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(596)=dydt(596)-k(16)*y(596);
dydt(3)=dydt(3)+k(16)*y(596);
dydt(180)=dydt(180)+k(16)*y(596);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(596)=dydt(596)-k(10)*y(596);
dydt(3)=dydt(3)+k(10)*y(596);
dydt(511)=dydt(511)+k(10)*y(596);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(596)=dydt(596)-k(8)*y(596);
dydt(222)=dydt(222)+k(8)*y(596);
dydt(141)=dydt(141)+k(8)*y(596);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(596)=dydt(596)-k(6)*y(596);
dydt(140)=dydt(140)+k(6)*y(596);
dydt(515)=dydt(515)+k(6)*y(596);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(597)=dydt(597)-k(34)*y(597);
dydt(360)=dydt(360)+k(34)*y(597);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(597)=dydt(597)-k(34)*y(597);
dydt(526)=dydt(526)+k(34)*y(597);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(597)=dydt(597)-k(33)*y(597);
dydt(516)=dydt(516)+k(33)*y(597);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(597)=dydt(597)-k(24)*y(597);
dydt(601)=dydt(601)+k(24)*y(597);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(597)=dydt(597)-k(22)*y(597);
dydt(197)=dydt(197)+k(22)*y(597);
dydt(595)=dydt(595)+k(22)*y(597);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

dydt(597)=dydt(597)-k(22)*y(597);
dydt(182)=dydt(182)+k(22)*y(597);
dydt(339)=dydt(339)+k(22)*y(597);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(597)=dydt(597)-k(16)*y(597);
dydt(3)=dydt(3)+k(16)*y(597);
dydt(363)=dydt(363)+k(16)*y(597);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(597)=dydt(597)-k(10)*y(597);
dydt(3)=dydt(3)+k(10)*y(597);
dydt(520)=dydt(520)+k(10)*y(597);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(597)=dydt(597)-k(8)*y(597);
dydt(222)=dydt(222)+k(8)*y(597);
dydt(336)=dydt(336)+k(8)*y(597);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(597)=dydt(597)-k(6)*y(597);
dydt(335)=dydt(335)+k(6)*y(597);
dydt(515)=dydt(515)+k(6)*y(597);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(605)=dydt(605)-k(34)*y(605);
dydt(167)=dydt(167)+k(34)*y(605);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(605)=dydt(605)-k(33)*y(605);
dydt(180)=dydt(180)+k(33)*y(605);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(605)=dydt(605)-k(31)*y(605);
dydt(169)=dydt(169)+k(31)*y(605);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(605)=dydt(605)-k(29)*y(605);
dydt(195)=dydt(195)+k(29)*y(605);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(605)=dydt(605)-k(22)*y(605);
dydt(2)=dydt(2)+k(22)*y(605);
dydt(599)=dydt(599)+k(22)*y(605);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(605)=dydt(605)-k(22)*y(605);
dydt(197)=dydt(197)+k(22)*y(605);
dydt(143)=dydt(143)+k(22)*y(605);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(605);
dydt(605)=dydt(605)-k(15)*y(3)*y(605);
dydt(544)=dydt(544)+k(15)*y(3)*y(605);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(605)=dydt(605)-k(10)*y(605);
dydt(3)=dydt(3)+k(10)*y(605);
dydt(545)=dydt(545)+k(10)*y(605);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(605)=dydt(605)-k(8)*y(605);
dydt(257)=dydt(257)+k(8)*y(605);
dydt(141)=dydt(141)+k(8)*y(605);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(605)=dydt(605)-k(6)*y(605);
dydt(256)=dydt(256)+k(6)*y(605);
dydt(140)=dydt(140)+k(6)*y(605);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(599)=dydt(599)-k(34)*y(599);
dydt(350)=dydt(350)+k(34)*y(599);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(599)=dydt(599)-k(33)*y(599);
dydt(361)=dydt(361)+k(33)*y(599);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(599)=dydt(599)-k(32)*y(599);
dydt(588)=dydt(588)+k(32)*y(599);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(599)=dydt(599)-k(31)*y(599);
dydt(196)=dydt(196)+k(31)*y(599);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(599)=dydt(599)-k(22)*y(599);
dydt(197)=dydt(197)+k(22)*y(599);
dydt(151)=dydt(151)+k(22)*y(599);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(599);
dydt(599)=dydt(599)-k(21)*y(197)*y(599);
dydt(342)=dydt(342)+k(21)*y(197)*y(599);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(32)=dydt(32)-k(21)*y(32)*y(599);
dydt(599)=dydt(599)-k(21)*y(32)*y(599);
dydt(362)=dydt(362)+k(21)*y(32)*y(599);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(599);
dydt(599)=dydt(599)-k(21)*y(2)*y(599);
dydt(605)=dydt(605)+k(21)*y(2)*y(599);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(599);
dydt(599)=dydt(599)-k(15)*y(3)*y(599);
dydt(598)=dydt(598)+k(15)*y(3)*y(599);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(599)=dydt(599)-k(10)*y(599);
dydt(3)=dydt(3)+k(10)*y(599);
dydt(585)=dydt(585)+k(10)*y(599);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(599)=dydt(599)-k(8)*y(599);
dydt(126)=dydt(126)+k(8)*y(599);
dydt(257)=dydt(257)+k(8)*y(599);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(599)=dydt(599)-k(6)*y(599);
dydt(48)=dydt(48)+k(6)*y(599);
dydt(256)=dydt(256)+k(6)*y(599);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(604)=dydt(604)-k(28)*y(604);
dydt(596)=dydt(596)+k(28)*y(604);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(604)=dydt(604)-k(27)*y(604);
dydt(522)=dydt(522)+k(27)*y(604);

% rule    : R11
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(604)=dydt(604)-k(25)*y(604);
dydt(357)=dydt(357)+k(25)*y(604);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(604)=dydt(604)-k(24)*y(604);
dydt(603)=dydt(603)+k(24)*y(604);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(604)=dydt(604)-k(22)*y(604);
dydt(2)=dydt(2)+k(22)*y(604);
dydt(539)=dydt(539)+k(22)*y(604);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(604)=dydt(604)-k(22)*y(604);
dydt(2)=dydt(2)+k(22)*y(604);
dydt(116)=dydt(116)+k(22)*y(604);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(604)=dydt(604)-k(16)*y(604);
dydt(3)=dydt(3)+k(16)*y(604);
dydt(173)=dydt(173)+k(16)*y(604);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(604)=dydt(604)-k(10)*y(604);
dydt(3)=dydt(3)+k(10)*y(604);
dydt(507)=dydt(507)+k(10)*y(604);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(604)=dydt(604)-k(8)*y(604);
dydt(43)=dydt(43)+k(8)*y(604);
dydt(141)=dydt(141)+k(8)*y(604);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(604)=dydt(604)-k(6)*y(604);
dydt(140)=dydt(140)+k(6)*y(604);
dydt(114)=dydt(114)+k(6)*y(604);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(603)=dydt(603)-k(33)*y(603);
dydt(604)=dydt(604)+k(33)*y(603);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(603)=dydt(603)-k(28)*y(603);
dydt(544)=dydt(544)+k(28)*y(603);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(603)=dydt(603)-k(27)*y(603);
dydt(349)=dydt(349)+k(27)*y(603);

% rule    : R11
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(603)=dydt(603)-k(25)*y(603);
dydt(602)=dydt(602)+k(25)*y(603);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(603)=dydt(603)-k(22)*y(603);
dydt(32)=dydt(32)+k(22)*y(603);
dydt(539)=dydt(539)+k(22)*y(603);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(603)=dydt(603)-k(22)*y(603);
dydt(2)=dydt(2)+k(22)*y(603);
dydt(352)=dydt(352)+k(22)*y(603);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(603)=dydt(603)-k(16)*y(603);
dydt(3)=dydt(3)+k(16)*y(603);
dydt(167)=dydt(167)+k(16)*y(603);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(603)=dydt(603)-k(10)*y(603);
dydt(3)=dydt(3)+k(10)*y(603);
dydt(541)=dydt(541)+k(10)*y(603);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(603)=dydt(603)-k(8)*y(603);
dydt(37)=dydt(37)+k(8)*y(603);
dydt(141)=dydt(141)+k(8)*y(603);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(603)=dydt(603)-k(6)*y(603);
dydt(140)=dydt(140)+k(6)*y(603);
dydt(554)=dydt(554)+k(6)*y(603);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(602)=dydt(602)-k(33)*y(602);
dydt(357)=dydt(357)+k(33)*y(602);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(602)=dydt(602)-k(33)*y(602);
dydt(603)=dydt(603)+k(33)*y(602);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(602)=dydt(602)-k(28)*y(602);
dydt(600)=dydt(600)+k(28)*y(602);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(602)=dydt(602)-k(27)*y(602);
dydt(558)=dydt(558)+k(27)*y(602);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

dydt(602)=dydt(602)-k(22)*y(602);
dydt(32)=dydt(32)+k(22)*y(602);
dydt(134)=dydt(134)+k(22)*y(602);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(602)=dydt(602)-k(22)*y(602);
dydt(32)=dydt(32)+k(22)*y(602);
dydt(352)=dydt(352)+k(22)*y(602);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(602)=dydt(602)-k(16)*y(602);
dydt(3)=dydt(3)+k(16)*y(602);
dydt(161)=dydt(161)+k(16)*y(602);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(602)=dydt(602)-k(10)*y(602);
dydt(3)=dydt(3)+k(10)*y(602);
dydt(535)=dydt(535)+k(10)*y(602);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(602)=dydt(602)-k(8)*y(602);
dydt(37)=dydt(37)+k(8)*y(602);
dydt(132)=dydt(132)+k(8)*y(602);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(602)=dydt(602)-k(6)*y(602);
dydt(131)=dydt(131)+k(6)*y(602);
dydt(554)=dydt(554)+k(6)*y(602);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(600)=dydt(600)-k(34)*y(600);
dydt(602)=dydt(602)+k(34)*y(600);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(600)=dydt(600)-k(33)*y(600);
dydt(544)=dydt(544)+k(33)*y(600);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(600)=dydt(600)-k(33)*y(600);
dydt(360)=dydt(360)+k(33)*y(600);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(600)=dydt(600)-k(29)*y(600);
dydt(601)=dydt(601)+k(29)*y(600);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(600)=dydt(600)-k(22)*y(600);
dydt(32)=dydt(32)+k(22)*y(600);
dydt(598)=dydt(598)+k(22)*y(600);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

dydt(600)=dydt(600)-k(22)*y(600);
dydt(197)=dydt(197)+k(22)*y(600);
dydt(134)=dydt(134)+k(22)*y(600);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(600)=dydt(600)-k(16)*y(600);
dydt(3)=dydt(3)+k(16)*y(600);
dydt(362)=dydt(362)+k(16)*y(600);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(600)=dydt(600)-k(10)*y(600);
dydt(3)=dydt(3)+k(10)*y(600);
dydt(536)=dydt(536)+k(10)*y(600);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(600)=dydt(600)-k(8)*y(600);
dydt(227)=dydt(227)+k(8)*y(600);
dydt(132)=dydt(132)+k(8)*y(600);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(600)=dydt(600)-k(6)*y(600);
dydt(131)=dydt(131)+k(6)*y(600);
dydt(543)=dydt(543)+k(6)*y(600);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(601)=dydt(601)-k(34)*y(601);
dydt(600)=dydt(600)+k(34)*y(601);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(601)=dydt(601)-k(34)*y(601);
dydt(558)=dydt(558)+k(34)*y(601);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(601)=dydt(601)-k(33)*y(601);
dydt(549)=dydt(549)+k(33)*y(601);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(601)=dydt(601)-k(33)*y(601);
dydt(597)=dydt(597)+k(33)*y(601);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(601)=dydt(601)-k(22)*y(601);
dydt(197)=dydt(197)+k(22)*y(601);
dydt(598)=dydt(598)+k(22)*y(601);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

dydt(601)=dydt(601)-k(22)*y(601);
dydt(197)=dydt(197)+k(22)*y(601);
dydt(339)=dydt(339)+k(22)*y(601);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(601)=dydt(601)-k(16)*y(601);
dydt(3)=dydt(3)+k(16)*y(601);
dydt(342)=dydt(342)+k(16)*y(601);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(601)=dydt(601)-k(10)*y(601);
dydt(3)=dydt(3)+k(10)*y(601);
dydt(552)=dydt(552)+k(10)*y(601);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(601)=dydt(601)-k(8)*y(601);
dydt(227)=dydt(227)+k(8)*y(601);
dydt(336)=dydt(336)+k(8)*y(601);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(601)=dydt(601)-k(6)*y(601);
dydt(335)=dydt(335)+k(6)*y(601);
dydt(543)=dydt(543)+k(6)*y(601);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(598)=dydt(598)-k(34)*y(598);
dydt(352)=dydt(352)+k(34)*y(598);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(598)=dydt(598)-k(33)*y(598);
dydt(595)=dydt(595)+k(33)*y(598);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(598)=dydt(598)-k(32)*y(598);
dydt(201)=dydt(201)+k(32)*y(598);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(598)=dydt(598)-k(22)*y(598);
dydt(197)=dydt(197)+k(22)*y(598);
dydt(120)=dydt(120)+k(22)*y(598);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(598);
dydt(598)=dydt(598)-k(21)*y(197)*y(598);
dydt(601)=dydt(601)+k(21)*y(197)*y(598);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(598);
dydt(598)=dydt(598)-k(21)*y(32)*y(598);
dydt(600)=dydt(600)+k(21)*y(32)*y(598);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(598);
dydt(598)=dydt(598)-k(21)*y(2)*y(598);
dydt(544)=dydt(544)+k(21)*y(2)*y(598);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(598)=dydt(598)-k(16)*y(598);
dydt(3)=dydt(3)+k(16)*y(598);
dydt(599)=dydt(599)+k(16)*y(598);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(598)=dydt(598)-k(10)*y(598);
dydt(3)=dydt(3)+k(10)*y(598);
dydt(547)=dydt(547)+k(10)*y(598);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(598)=dydt(598)-k(8)*y(598);
dydt(126)=dydt(126)+k(8)*y(598);
dydt(227)=dydt(227)+k(8)*y(598);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(598)=dydt(598)-k(6)*y(598);
dydt(48)=dydt(48)+k(6)*y(598);
dydt(543)=dydt(543)+k(6)*y(598);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(595)=dydt(595)-k(34)*y(595);
dydt(116)=dydt(116)+k(34)*y(595);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(595)=dydt(595)-k(32)*y(595);
dydt(593)=dydt(593)+k(32)*y(595);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(595)=dydt(595)-k(24)*y(595);
dydt(598)=dydt(598)+k(24)*y(595);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(595)=dydt(595)-k(22)*y(595);
dydt(182)=dydt(182)+k(22)*y(595);
dydt(120)=dydt(120)+k(22)*y(595);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(595);
dydt(595)=dydt(595)-k(21)*y(197)*y(595);
dydt(597)=dydt(597)+k(21)*y(197)*y(595);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(32)=dydt(32)-k(21)*y(32)*y(595);
dydt(595)=dydt(595)-k(21)*y(32)*y(595);
dydt(360)=dydt(360)+k(21)*y(32)*y(595);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(595);
dydt(595)=dydt(595)-k(21)*y(2)*y(595);
dydt(596)=dydt(596)+k(21)*y(2)*y(595);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(595)=dydt(595)-k(16)*y(595);
dydt(3)=dydt(3)+k(16)*y(595);
dydt(361)=dydt(361)+k(16)*y(595);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(595)=dydt(595)-k(10)*y(595);
dydt(3)=dydt(3)+k(10)*y(595);
dydt(518)=dydt(518)+k(10)*y(595);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(595)=dydt(595)-k(8)*y(595);
dydt(126)=dydt(126)+k(8)*y(595);
dydt(222)=dydt(222)+k(8)*y(595);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(595)=dydt(595)-k(6)*y(595);
dydt(48)=dydt(48)+k(6)*y(595);
dydt(515)=dydt(515)+k(6)*y(595);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(593)=dydt(593)-k(34)*y(593);
dydt(113)=dydt(113)+k(34)*y(593);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(593)=dydt(593)-k(24)*y(593);
dydt(201)=dydt(201)+k(24)*y(593);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

dydt(593)=dydt(593)-k(22)*y(593);
dydt(182)=dydt(182)+k(22)*y(593);
dydt(117)=dydt(117)+k(22)*y(593);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(593)=dydt(593)-k(19)*y(593);
dydt(595)=dydt(595)+k(19)*y(593);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(593)=dydt(593)-k(16)*y(593);
dydt(3)=dydt(3)+k(16)*y(593);
dydt(594)=dydt(594)+k(16)*y(593);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(593)=dydt(593)-k(10)*y(593);
dydt(3)=dydt(3)+k(10)*y(593);
dydt(591)=dydt(591)+k(10)*y(593);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(593)=dydt(593)-k(8)*y(593);
dydt(8)=dydt(8)+k(8)*y(593);
dydt(222)=dydt(222)+k(8)*y(593);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(593)=dydt(593)-k(6)*y(593);
dydt(7)=dydt(7)+k(6)*y(593);
dydt(515)=dydt(515)+k(6)*y(593);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(591)=dydt(591)-k(34)*y(591);
dydt(115)=dydt(115)+k(34)*y(591);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

dydt(591)=dydt(591)-k(22)*y(591);
dydt(182)=dydt(182)+k(22)*y(591);
dydt(119)=dydt(119)+k(22)*y(591);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(591)=dydt(591)-k(19)*y(591);
dydt(518)=dydt(518)+k(19)*y(591);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(591)=dydt(591)-k(17)*y(591);
dydt(303)=dydt(303)+k(17)*y(591);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(591)=dydt(591)-k(16)*y(591);
dydt(3)=dydt(3)+k(16)*y(591);
dydt(304)=dydt(304)+k(16)*y(591);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(591);
dydt(591)=dydt(591)-k(9)*y(3)*y(591);
dydt(593)=dydt(593)+k(9)*y(3)*y(591);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(591)=dydt(591)-k(8)*y(591);
dydt(5)=dydt(5)+k(8)*y(591);
dydt(222)=dydt(222)+k(8)*y(591);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

dydt(591)=dydt(591)-k(6)*y(591);
dydt(1)=dydt(1)+k(6)*y(591);
dydt(515)=dydt(515)+k(6)*y(591);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(592)=dydt(592)-k(33)*y(592);
dydt(115)=dydt(115)+k(33)*y(592);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

dydt(592)=dydt(592)-k(22)*y(592);
dydt(32)=dydt(32)+k(22)*y(592);
dydt(119)=dydt(119)+k(22)*y(592);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(592)=dydt(592)-k(19)*y(592);
dydt(556)=dydt(556)+k(19)*y(592);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(592)=dydt(592)-k(17)*y(592);
dydt(311)=dydt(311)+k(17)*y(592);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(592)=dydt(592)-k(16)*y(592);
dydt(3)=dydt(3)+k(16)*y(592);
dydt(106)=dydt(106)+k(16)*y(592);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(592);
dydt(592)=dydt(592)-k(9)*y(3)*y(592);
dydt(107)=dydt(107)+k(9)*y(3)*y(592);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(592)=dydt(592)-k(8)*y(592);
dydt(5)=dydt(5)+k(8)*y(592);
dydt(37)=dydt(37)+k(8)*y(592);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

dydt(592)=dydt(592)-k(6)*y(592);
dydt(1)=dydt(1)+k(6)*y(592);
dydt(554)=dydt(554)+k(6)*y(592);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(589)=dydt(589)-k(34)*y(589);
dydt(592)=dydt(592)+k(34)*y(589);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(589)=dydt(589)-k(33)*y(589);
dydt(591)=dydt(591)+k(33)*y(589);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

dydt(589)=dydt(589)-k(22)*y(589);
dydt(197)=dydt(197)+k(22)*y(589);
dydt(119)=dydt(119)+k(22)*y(589);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(589)=dydt(589)-k(19)*y(589);
dydt(547)=dydt(547)+k(19)*y(589);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(589)=dydt(589)-k(17)*y(589);
dydt(312)=dydt(312)+k(17)*y(589);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(589)=dydt(589)-k(16)*y(589);
dydt(3)=dydt(3)+k(16)*y(589);
dydt(587)=dydt(587)+k(16)*y(589);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(9)*y(3)*y(589);
dydt(589)=dydt(589)-k(9)*y(3)*y(589);
dydt(201)=dydt(201)+k(9)*y(3)*y(589);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(589)=dydt(589)-k(8)*y(589);
dydt(5)=dydt(5)+k(8)*y(589);
dydt(227)=dydt(227)+k(8)*y(589);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

dydt(589)=dydt(589)-k(6)*y(589);
dydt(1)=dydt(1)+k(6)*y(589);
dydt(543)=dydt(543)+k(6)*y(589);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(590)=dydt(590)-k(32)*y(590);
dydt(333)=dydt(333)+k(32)*y(590);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

dydt(590)=dydt(590)-k(31)*y(590);
dydt(97)=dydt(97)+k(31)*y(590);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(590);
dydt(590)=dydt(590)-k(21)*y(197)*y(590);
dydt(587)=dydt(587)+k(21)*y(197)*y(590);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(590);
dydt(590)=dydt(590)-k(21)*y(32)*y(590);
dydt(106)=dydt(106)+k(21)*y(32)*y(590);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(590);
dydt(590)=dydt(590)-k(21)*y(2)*y(590);
dydt(112)=dydt(112)+k(21)*y(2)*y(590);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(3)=dydt(3)-k(15)*y(3)*y(590);
dydt(590)=dydt(590)-k(15)*y(3)*y(590);
dydt(119)=dydt(119)+k(15)*y(3)*y(590);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(590);
dydt(590)=dydt(590)-k(9)*y(3)*y(590);
dydt(108)=dydt(108)+k(9)*y(3)*y(590);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) 

dydt(590)=dydt(590)-k(8)*y(590);
dydt(77)=dydt(77)+k(8)*y(590);
dydt(5)=dydt(5)+k(8)*y(590);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) 

dydt(590)=dydt(590)-k(6)*y(590);
dydt(1)=dydt(1)+k(6)*y(590);
dydt(76)=dydt(76)+k(6)*y(590);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(587)=dydt(587)-k(34)*y(587);
dydt(106)=dydt(106)+k(34)*y(587);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(587)=dydt(587)-k(33)*y(587);
dydt(304)=dydt(304)+k(33)*y(587);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(587)=dydt(587)-k(31)*y(587);
dydt(202)=dydt(202)+k(31)*y(587);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(587)=dydt(587)-k(22)*y(587);
dydt(197)=dydt(197)+k(22)*y(587);
dydt(590)=dydt(590)+k(22)*y(587);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(587);
dydt(587)=dydt(587)-k(15)*y(3)*y(587);
dydt(589)=dydt(589)+k(15)*y(3)*y(587);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(587);
dydt(587)=dydt(587)-k(9)*y(3)*y(587);
dydt(588)=dydt(588)+k(9)*y(3)*y(587);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(587)=dydt(587)-k(8)*y(587);
dydt(5)=dydt(5)+k(8)*y(587);
dydt(257)=dydt(257)+k(8)*y(587);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

dydt(587)=dydt(587)-k(6)*y(587);
dydt(1)=dydt(1)+k(6)*y(587);
dydt(256)=dydt(256)+k(6)*y(587);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(586)=dydt(586)-k(34)*y(586);
dydt(84)=dydt(84)+k(34)*y(586);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(586)=dydt(586)-k(33)*y(586);
dydt(302)=dydt(302)+k(33)*y(586);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(586)=dydt(586)-k(31)*y(586);
dydt(296)=dydt(296)+k(31)*y(586);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(586)=dydt(586)-k(31)*y(586);
dydt(587)=dydt(587)+k(31)*y(586);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(586)=dydt(586)-k(22)*y(586);
dydt(197)=dydt(197)+k(22)*y(586);
dydt(79)=dydt(79)+k(22)*y(586);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(586);
dydt(586)=dydt(586)-k(15)*y(3)*y(586);
dydt(203)=dydt(203)+k(15)*y(3)*y(586);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(586);
dydt(586)=dydt(586)-k(15)*y(3)*y(586);
dydt(312)=dydt(312)+k(15)*y(3)*y(586);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(586)=dydt(586)-k(8)*y(586);
dydt(91)=dydt(91)+k(8)*y(586);
dydt(257)=dydt(257)+k(8)*y(586);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

dydt(586)=dydt(586)-k(6)*y(586);
dydt(72)=dydt(72)+k(6)*y(586);
dydt(256)=dydt(256)+k(6)*y(586);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(578)=dydt(578)-k(34)*y(578);
dydt(573)=dydt(573)+k(34)*y(578);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(578)=dydt(578)-k(33)*y(578);
dydt(497)=dydt(497)+k(33)*y(578);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(578)=dydt(578)-k(32)*y(578);
dydt(586)=dydt(586)+k(32)*y(578);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(578)=dydt(578)-k(31)*y(578);
dydt(292)=dydt(292)+k(31)*y(578);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(578)=dydt(578)-k(31)*y(578);
dydt(585)=dydt(585)+k(31)*y(578);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(578)=dydt(578)-k(22)*y(578);
dydt(197)=dydt(197)+k(22)*y(578);
dydt(584)=dydt(584)+k(22)*y(578);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(578);
dydt(578)=dydt(578)-k(21)*y(197)*y(578);
dydt(583)=dydt(583)+k(21)*y(197)*y(578);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(578);
dydt(578)=dydt(578)-k(21)*y(32)*y(578);
dydt(582)=dydt(582)+k(21)*y(32)*y(578);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(578);
dydt(578)=dydt(578)-k(21)*y(2)*y(578);
dydt(259)=dydt(259)+k(21)*y(2)*y(578);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(578);
dydt(578)=dydt(578)-k(15)*y(3)*y(578);
dydt(260)=dydt(260)+k(15)*y(3)*y(578);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(578);
dydt(578)=dydt(578)-k(15)*y(3)*y(578);
dydt(581)=dydt(581)+k(15)*y(3)*y(578);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(578)=dydt(578)-k(8)*y(578);
dydt(315)=dydt(315)+k(8)*y(578);
dydt(257)=dydt(257)+k(8)*y(578);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

dydt(578)=dydt(578)-k(6)*y(578);
dydt(77)=dydt(77)+k(6)*y(578);
dydt(256)=dydt(256)+k(6)*y(578);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(580)=dydt(580)-k(34)*y(580);
dydt(280)=dydt(280)+k(34)*y(580);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(580)=dydt(580)-k(34)*y(580);
dydt(271)=dydt(271)+k(34)*y(580);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(580)=dydt(580)-k(31)*y(580);
dydt(187)=dydt(187)+k(31)*y(580);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(580)=dydt(580)-k(22)*y(580);
dydt(182)=dydt(182)+k(22)*y(580);
dydt(431)=dydt(431)+k(22)*y(580);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(580)=dydt(580)-k(22)*y(580);
dydt(182)=dydt(182)+k(22)*y(580);
dydt(421)=dydt(421)+k(22)*y(580);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(580);
dydt(580)=dydt(580)-k(15)*y(3)*y(580);
dydt(273)=dydt(273)+k(15)*y(3)*y(580);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(580);
dydt(580)=dydt(580)-k(9)*y(3)*y(580);
dydt(395)=dydt(395)+k(9)*y(3)*y(580);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(580)=dydt(580)-k(8)*y(580);
dydt(177)=dydt(177)+k(8)*y(580);
dydt(219)=dydt(219)+k(8)*y(580);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(580)=dydt(580)-k(6)*y(580);
dydt(218)=dydt(218)+k(6)*y(580);
dydt(176)=dydt(176)+k(6)*y(580);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(579)=dydt(579)-k(34)*y(579);
dydt(505)=dydt(505)+k(34)*y(579);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(579)=dydt(579)-k(34)*y(579);
dydt(266)=dydt(266)+k(34)*y(579);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(579)=dydt(579)-k(31)*y(579);
dydt(580)=dydt(580)+k(31)*y(579);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(579)=dydt(579)-k(31)*y(579);
dydt(513)=dydt(513)+k(31)*y(579);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(579)=dydt(579)-k(22)*y(579);
dydt(182)=dydt(182)+k(22)*y(579);
dydt(576)=dydt(576)+k(22)*y(579);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(579)=dydt(579)-k(22)*y(579);
dydt(182)=dydt(182)+k(22)*y(579);
dydt(497)=dydt(497)+k(22)*y(579);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(579);
dydt(579)=dydt(579)-k(15)*y(3)*y(579);
dydt(268)=dydt(268)+k(15)*y(3)*y(579);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(579);
dydt(579)=dydt(579)-k(15)*y(3)*y(579);
dydt(517)=dydt(517)+k(15)*y(3)*y(579);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(579)=dydt(579)-k(8)*y(579);
dydt(218)=dydt(218)+k(8)*y(579);
dydt(219)=dydt(219)+k(8)*y(579);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(579)=dydt(579)-k(6)*y(579);
dydt(218)=dydt(218)+k(6)*y(579);
dydt(264)=dydt(264)+k(6)*y(579);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(577)=dydt(577)-k(34)*y(577);
dydt(574)=dydt(574)+k(34)*y(577);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(577)=dydt(577)-k(34)*y(577);
dydt(259)=dydt(259)+k(34)*y(577);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(577)=dydt(577)-k(33)*y(577);
dydt(579)=dydt(579)+k(33)*y(577);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(577)=dydt(577)-k(31)*y(577);
dydt(290)=dydt(290)+k(31)*y(577);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(577)=dydt(577)-k(31)*y(577);
dydt(550)=dydt(550)+k(31)*y(577);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(577)=dydt(577)-k(22)*y(577);
dydt(197)=dydt(197)+k(22)*y(577);
dydt(576)=dydt(576)+k(22)*y(577);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(577)=dydt(577)-k(22)*y(577);
dydt(182)=dydt(182)+k(22)*y(577);
dydt(578)=dydt(578)+k(22)*y(577);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(577);
dydt(577)=dydt(577)-k(15)*y(3)*y(577);
dydt(261)=dydt(261)+k(15)*y(3)*y(577);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(577);
dydt(577)=dydt(577)-k(15)*y(3)*y(577);
dydt(551)=dydt(551)+k(15)*y(3)*y(577);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(577)=dydt(577)-k(8)*y(577);
dydt(257)=dydt(257)+k(8)*y(577);
dydt(219)=dydt(219)+k(8)*y(577);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(577)=dydt(577)-k(6)*y(577);
dydt(218)=dydt(218)+k(6)*y(577);
dydt(256)=dydt(256)+k(6)*y(577);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(574)=dydt(574)-k(34)*y(574);
dydt(572)=dydt(572)+k(34)*y(574);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(574)=dydt(574)-k(33)*y(574);
dydt(505)=dydt(505)+k(33)*y(574);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(574)=dydt(574)-k(31)*y(574);
dydt(285)=dydt(285)+k(31)*y(574);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(574)=dydt(574)-k(31)*y(574);
dydt(348)=dydt(348)+k(31)*y(574);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(574)=dydt(574)-k(30)*y(574);
dydt(577)=dydt(577)+k(30)*y(574);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(574)=dydt(574)-k(22)*y(574);
dydt(32)=dydt(32)+k(22)*y(574);
dydt(576)=dydt(576)+k(22)*y(574);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(574)=dydt(574)-k(22)*y(574);
dydt(182)=dydt(182)+k(22)*y(574);
dydt(573)=dydt(573)+k(22)*y(574);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(574);
dydt(574)=dydt(574)-k(15)*y(3)*y(574);
dydt(286)=dydt(286)+k(15)*y(3)*y(574);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(574);
dydt(574)=dydt(574)-k(15)*y(3)*y(574);
dydt(555)=dydt(555)+k(15)*y(3)*y(574);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(574)=dydt(574)-k(8)*y(574);
dydt(33)=dydt(33)+k(8)*y(574);
dydt(219)=dydt(219)+k(8)*y(574);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(574)=dydt(574)-k(6)*y(574);
dydt(218)=dydt(218)+k(6)*y(574);
dydt(82)=dydt(82)+k(6)*y(574);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(575)=dydt(575)-k(33)*y(575);
dydt(506)=dydt(506)+k(33)*y(575);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(575)=dydt(575)-k(31)*y(575);
dydt(166)=dydt(166)+k(31)*y(575);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(575)=dydt(575)-k(28)*y(575);
dydt(545)=dydt(545)+k(28)*y(575);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(575)=dydt(575)-k(27)*y(575);
dydt(348)=dydt(348)+k(27)*y(575);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(575)=dydt(575)-k(22)*y(575);
dydt(32)=dydt(32)+k(22)*y(575);
dydt(508)=dydt(508)+k(22)*y(575);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(575)=dydt(575)-k(22)*y(575);
dydt(2)=dydt(2)+k(22)*y(575);
dydt(351)=dydt(351)+k(22)*y(575);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(575);
dydt(575)=dydt(575)-k(15)*y(3)*y(575);
dydt(541)=dydt(541)+k(15)*y(3)*y(575);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(575);
dydt(575)=dydt(575)-k(9)*y(3)*y(575);
dydt(167)=dydt(167)+k(9)*y(3)*y(575);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(575)=dydt(575)-k(8)*y(575);
dydt(33)=dydt(33)+k(8)*y(575);
dydt(22)=dydt(22)+k(8)*y(575);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(575)=dydt(575)-k(6)*y(575);
dydt(21)=dydt(21)+k(6)*y(575);
dydt(82)=dydt(82)+k(6)*y(575);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(572)=dydt(572)-k(33)*y(572);
dydt(503)=dydt(503)+k(33)*y(572);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(572)=dydt(572)-k(31)*y(572);
dydt(442)=dydt(442)+k(31)*y(572);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(572)=dydt(572)-k(31)*y(572);
dydt(575)=dydt(575)+k(31)*y(572);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(572)=dydt(572)-k(28)*y(572);
dydt(259)=dydt(259)+k(28)*y(572);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(572)=dydt(572)-k(27)*y(572);
dydt(574)=dydt(574)+k(27)*y(572);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(572)=dydt(572)-k(22)*y(572);
dydt(32)=dydt(32)+k(22)*y(572);
dydt(252)=dydt(252)+k(22)*y(572);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(572)=dydt(572)-k(22)*y(572);
dydt(2)=dydt(2)+k(22)*y(572);
dydt(573)=dydt(573)+k(22)*y(572);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(572);
dydt(572)=dydt(572)-k(15)*y(3)*y(572);
dydt(254)=dydt(254)+k(15)*y(3)*y(572);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(572);
dydt(572)=dydt(572)-k(15)*y(3)*y(572);
dydt(570)=dydt(570)+k(15)*y(3)*y(572);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(572)=dydt(572)-k(8)*y(572);
dydt(33)=dydt(33)+k(8)*y(572);
dydt(437)=dydt(437)+k(8)*y(572);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(572)=dydt(572)-k(6)*y(572);
dydt(111)=dydt(111)+k(6)*y(572);
dydt(82)=dydt(82)+k(6)*y(572);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(570)=dydt(570)-k(33)*y(570);
dydt(504)=dydt(504)+k(33)*y(570);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(570)=dydt(570)-k(31)*y(570);
dydt(541)=dydt(541)+k(31)*y(570);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(570)=dydt(570)-k(28)*y(570);
dydt(546)=dydt(546)+k(28)*y(570);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(570)=dydt(570)-k(27)*y(570);
dydt(555)=dydt(555)+k(27)*y(570);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(570)=dydt(570)-k(25)*y(570);
dydt(566)=dydt(566)+k(25)*y(570);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(570)=dydt(570)-k(22)*y(570);
dydt(32)=dydt(32)+k(22)*y(570);
dydt(540)=dydt(540)+k(22)*y(570);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(570)=dydt(570)-k(22)*y(570);
dydt(2)=dydt(2)+k(22)*y(570);
dydt(563)=dydt(563)+k(22)*y(570);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(570)=dydt(570)-k(16)*y(570);
dydt(3)=dydt(3)+k(16)*y(570);
dydt(572)=dydt(572)+k(16)*y(570);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(570);
dydt(570)=dydt(570)-k(15)*y(3)*y(570);
dydt(571)=dydt(571)+k(15)*y(3)*y(570);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(570)=dydt(570)-k(8)*y(570);
dydt(437)=dydt(437)+k(8)*y(570);
dydt(37)=dydt(37)+k(8)*y(570);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(570)=dydt(570)-k(6)*y(570);
dydt(111)=dydt(111)+k(6)*y(570);
dydt(554)=dydt(554)+k(6)*y(570);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(566)=dydt(566)-k(33)*y(566);
dydt(500)=dydt(500)+k(33)*y(566);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(566)=dydt(566)-k(33)*y(566);
dydt(570)=dydt(570)+k(33)*y(566);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(566)=dydt(566)-k(31)*y(566);
dydt(535)=dydt(535)+k(31)*y(566);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(566)=dydt(566)-k(28)*y(566);
dydt(569)=dydt(569)+k(28)*y(566);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(566)=dydt(566)-k(27)*y(566);
dydt(560)=dydt(560)+k(27)*y(566);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(566)=dydt(566)-k(22)*y(566);
dydt(32)=dydt(32)+k(22)*y(566);
dydt(534)=dydt(534)+k(22)*y(566);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(566)=dydt(566)-k(22)*y(566);
dydt(32)=dydt(32)+k(22)*y(566);
dydt(563)=dydt(563)+k(22)*y(566);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(566)=dydt(566)-k(16)*y(566);
dydt(3)=dydt(3)+k(16)*y(566);
dydt(568)=dydt(568)+k(16)*y(566);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(566);
dydt(566)=dydt(566)-k(15)*y(3)*y(566);
dydt(567)=dydt(567)+k(15)*y(3)*y(566);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(566)=dydt(566)-k(8)*y(566);
dydt(34)=dydt(34)+k(8)*y(566);
dydt(37)=dydt(37)+k(8)*y(566);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(566)=dydt(566)-k(6)*y(566);
dydt(33)=dydt(33)+k(6)*y(566);
dydt(554)=dydt(554)+k(6)*y(566);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(560)=dydt(560)-k(34)*y(560);
dydt(566)=dydt(566)+k(34)*y(560);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(560)=dydt(560)-k(33)*y(560);
dydt(528)=dydt(528)+k(33)*y(560);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(560)=dydt(560)-k(33)*y(560);
dydt(555)=dydt(555)+k(33)*y(560);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(560)=dydt(560)-k(31)*y(560);
dydt(557)=dydt(557)+k(31)*y(560);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(560)=dydt(560)-k(30)*y(560);
dydt(565)=dydt(565)+k(30)*y(560);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(560)=dydt(560)-k(22)*y(560);
dydt(32)=dydt(32)+k(22)*y(560);
dydt(564)=dydt(564)+k(22)*y(560);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(560)=dydt(560)-k(22)*y(560);
dydt(197)=dydt(197)+k(22)*y(560);
dydt(563)=dydt(563)+k(22)*y(560);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(560)=dydt(560)-k(16)*y(560);
dydt(3)=dydt(3)+k(16)*y(560);
dydt(562)=dydt(562)+k(16)*y(560);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(560);
dydt(560)=dydt(560)-k(15)*y(3)*y(560);
dydt(561)=dydt(561)+k(15)*y(3)*y(560);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(560)=dydt(560)-k(8)*y(560);
dydt(319)=dydt(319)+k(8)*y(560);
dydt(37)=dydt(37)+k(8)*y(560);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(560)=dydt(560)-k(6)*y(560);
dydt(257)=dydt(257)+k(6)*y(560);
dydt(554)=dydt(554)+k(6)*y(560);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(557)=dydt(557)-k(34)*y(557);
dydt(535)=dydt(535)+k(34)*y(557);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(557)=dydt(557)-k(33)*y(557);
dydt(525)=dydt(525)+k(33)*y(557);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(557)=dydt(557)-k(33)*y(557);
dydt(553)=dydt(553)+k(33)*y(557);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(557)=dydt(557)-k(30)*y(557);
dydt(552)=dydt(552)+k(30)*y(557);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(557)=dydt(557)-k(22)*y(557);
dydt(32)=dydt(32)+k(22)*y(557);
dydt(529)=dydt(529)+k(22)*y(557);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(557)=dydt(557)-k(22)*y(557);
dydt(197)=dydt(197)+k(22)*y(557);
dydt(556)=dydt(556)+k(22)*y(557);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(557)=dydt(557)-k(17)*y(557);
dydt(560)=dydt(560)+k(17)*y(557);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(557)=dydt(557)-k(16)*y(557);
dydt(3)=dydt(3)+k(16)*y(557);
dydt(559)=dydt(559)+k(16)*y(557);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(557);
dydt(557)=dydt(557)-k(9)*y(3)*y(557);
dydt(558)=dydt(558)+k(9)*y(3)*y(557);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(557)=dydt(557)-k(8)*y(557);
dydt(209)=dydt(209)+k(8)*y(557);
dydt(37)=dydt(37)+k(8)*y(557);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(557)=dydt(557)-k(6)*y(557);
dydt(192)=dydt(192)+k(6)*y(557);
dydt(554)=dydt(554)+k(6)*y(557);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(553)=dydt(553)-k(34)*y(553);
dydt(541)=dydt(541)+k(34)*y(553);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(553)=dydt(553)-k(33)*y(553);
dydt(521)=dydt(521)+k(33)*y(553);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(553)=dydt(553)-k(30)*y(553);
dydt(548)=dydt(548)+k(30)*y(553);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(553)=dydt(553)-k(25)*y(553);
dydt(557)=dydt(557)+k(25)*y(553);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(553)=dydt(553)-k(22)*y(553);
dydt(32)=dydt(32)+k(22)*y(553);
dydt(519)=dydt(519)+k(22)*y(553);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(553)=dydt(553)-k(22)*y(553);
dydt(182)=dydt(182)+k(22)*y(553);
dydt(556)=dydt(556)+k(22)*y(553);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(553)=dydt(553)-k(17)*y(553);
dydt(555)=dydt(555)+k(17)*y(553);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(553)=dydt(553)-k(16)*y(553);
dydt(3)=dydt(3)+k(16)*y(553);
dydt(348)=dydt(348)+k(16)*y(553);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(553);
dydt(553)=dydt(553)-k(9)*y(3)*y(553);
dydt(349)=dydt(349)+k(9)*y(3)*y(553);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(553)=dydt(553)-k(8)*y(553);
dydt(214)=dydt(214)+k(8)*y(553);
dydt(37)=dydt(37)+k(8)*y(553);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(553)=dydt(553)-k(6)*y(553);
dydt(177)=dydt(177)+k(6)*y(553);
dydt(554)=dydt(554)+k(6)*y(553);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(548)=dydt(548)-k(34)*y(548);
dydt(553)=dydt(553)+k(34)*y(548);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(548)=dydt(548)-k(34)*y(548);
dydt(542)=dydt(542)+k(34)*y(548);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(548)=dydt(548)-k(33)*y(548);
dydt(514)=dydt(514)+k(33)*y(548);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(548)=dydt(548)-k(25)*y(548);
dydt(552)=dydt(552)+k(25)*y(548);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(548)=dydt(548)-k(22)*y(548);
dydt(197)=dydt(197)+k(22)*y(548);
dydt(519)=dydt(519)+k(22)*y(548);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(548)=dydt(548)-k(22)*y(548);
dydt(182)=dydt(182)+k(22)*y(548);
dydt(547)=dydt(547)+k(22)*y(548);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(548)=dydt(548)-k(17)*y(548);
dydt(551)=dydt(551)+k(17)*y(548);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(548)=dydt(548)-k(16)*y(548);
dydt(3)=dydt(3)+k(16)*y(548);
dydt(550)=dydt(550)+k(16)*y(548);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(548);
dydt(548)=dydt(548)-k(9)*y(3)*y(548);
dydt(549)=dydt(549)+k(9)*y(3)*y(548);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(548)=dydt(548)-k(8)*y(548);
dydt(214)=dydt(214)+k(8)*y(548);
dydt(227)=dydt(227)+k(8)*y(548);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(548)=dydt(548)-k(6)*y(548);
dydt(177)=dydt(177)+k(6)*y(548);
dydt(543)=dydt(543)+k(6)*y(548);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(542)=dydt(542)-k(34)*y(542);
dydt(541)=dydt(541)+k(34)*y(542);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(542)=dydt(542)-k(33)*y(542);
dydt(511)=dydt(511)+k(33)*y(542);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(542)=dydt(542)-k(29)*y(542);
dydt(548)=dydt(548)+k(29)*y(542);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(542)=dydt(542)-k(25)*y(542);
dydt(536)=dydt(536)+k(25)*y(542);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(542)=dydt(542)-k(22)*y(542);
dydt(197)=dydt(197)+k(22)*y(542);
dydt(538)=dydt(538)+k(22)*y(542);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(542)=dydt(542)-k(22)*y(542);
dydt(2)=dydt(2)+k(22)*y(542);
dydt(547)=dydt(547)+k(22)*y(542);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(542)=dydt(542)-k(17)*y(542);
dydt(546)=dydt(546)+k(17)*y(542);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(542)=dydt(542)-k(16)*y(542);
dydt(3)=dydt(3)+k(16)*y(542);
dydt(545)=dydt(545)+k(16)*y(542);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(542);
dydt(542)=dydt(542)-k(9)*y(3)*y(542);
dydt(544)=dydt(544)+k(9)*y(3)*y(542);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(542)=dydt(542)-k(8)*y(542);
dydt(22)=dydt(22)+k(8)*y(542);
dydt(227)=dydt(227)+k(8)*y(542);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(542)=dydt(542)-k(6)*y(542);
dydt(21)=dydt(21)+k(6)*y(542);
dydt(543)=dydt(543)+k(6)*y(542);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(538)=dydt(538)-k(32)*y(538);
dydt(329)=dydt(329)+k(32)*y(538);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(538)=dydt(538)-k(25)*y(538);
dydt(531)=dydt(531)+k(25)*y(538);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

dydt(538)=dydt(538)-k(22)*y(538);
dydt(2)=dydt(2)+k(22)*y(538);
dydt(537)=dydt(537)+k(22)*y(538);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(538);
dydt(538)=dydt(538)-k(21)*y(197)*y(538);
dydt(542)=dydt(542)+k(21)*y(197)*y(538);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(538);
dydt(538)=dydt(538)-k(21)*y(32)*y(538);
dydt(541)=dydt(541)+k(21)*y(32)*y(538);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(538);
dydt(538)=dydt(538)-k(21)*y(2)*y(538);
dydt(507)=dydt(507)+k(21)*y(2)*y(538);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(538)=dydt(538)-k(17)*y(538);
dydt(540)=dydt(540)+k(17)*y(538);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(538)=dydt(538)-k(16)*y(538);
dydt(3)=dydt(3)+k(16)*y(538);
dydt(508)=dydt(508)+k(16)*y(538);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(538);
dydt(538)=dydt(538)-k(9)*y(3)*y(538);
dydt(539)=dydt(539)+k(9)*y(3)*y(538);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(538)=dydt(538)-k(8)*y(538);
dydt(54)=dydt(54)+k(8)*y(538);
dydt(22)=dydt(22)+k(8)*y(538);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(538)=dydt(538)-k(6)*y(538);
dydt(118)=dydt(118)+k(6)*y(538);
dydt(21)=dydt(21)+k(6)*y(538);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(531)=dydt(531)-k(33)*y(531);
dydt(538)=dydt(538)+k(33)*y(531);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(531)=dydt(531)-k(32)*y(531);
dydt(133)=dydt(133)+k(32)*y(531);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

dydt(531)=dydt(531)-k(22)*y(531);
dydt(32)=dydt(32)+k(22)*y(531);
dydt(537)=dydt(537)+k(22)*y(531);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(531);
dydt(531)=dydt(531)-k(21)*y(197)*y(531);
dydt(536)=dydt(536)+k(21)*y(197)*y(531);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(531);
dydt(531)=dydt(531)-k(21)*y(32)*y(531);
dydt(535)=dydt(535)+k(21)*y(32)*y(531);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(531);
dydt(531)=dydt(531)-k(21)*y(2)*y(531);
dydt(530)=dydt(530)+k(21)*y(2)*y(531);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(531)=dydt(531)-k(17)*y(531);
dydt(534)=dydt(534)+k(17)*y(531);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(531)=dydt(531)-k(16)*y(531);
dydt(3)=dydt(3)+k(16)*y(531);
dydt(533)=dydt(533)+k(16)*y(531);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(531);
dydt(531)=dydt(531)-k(9)*y(3)*y(531);
dydt(134)=dydt(134)+k(9)*y(3)*y(531);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(531)=dydt(531)-k(8)*y(531);
dydt(54)=dydt(54)+k(8)*y(531);
dydt(28)=dydt(28)+k(8)*y(531);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(531)=dydt(531)-k(6)*y(531);
dydt(118)=dydt(118)+k(6)*y(531);
dydt(27)=dydt(27)+k(6)*y(531);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(532)=dydt(532)-k(34)*y(532);
dydt(530)=dydt(530)+k(34)*y(532);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(532)=dydt(532)-k(33)*y(532);
dydt(511)=dydt(511)+k(33)*y(532);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(532)=dydt(532)-k(29)*y(532);
dydt(520)=dydt(520)+k(29)*y(532);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(532)=dydt(532)-k(22)*y(532);
dydt(182)=dydt(182)+k(22)*y(532);
dydt(531)=dydt(531)+k(22)*y(532);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(532)=dydt(532)-k(22)*y(532);
dydt(32)=dydt(32)+k(22)*y(532);
dydt(518)=dydt(518)+k(22)*y(532);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(532)=dydt(532)-k(17)*y(532);
dydt(496)=dydt(496)+k(17)*y(532);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(532)=dydt(532)-k(16)*y(532);
dydt(3)=dydt(3)+k(16)*y(532);
dydt(359)=dydt(359)+k(16)*y(532);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(532);
dydt(532)=dydt(532)-k(9)*y(3)*y(532);
dydt(360)=dydt(360)+k(9)*y(3)*y(532);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(532)=dydt(532)-k(8)*y(532);
dydt(28)=dydt(28)+k(8)*y(532);
dydt(222)=dydt(222)+k(8)*y(532);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(532)=dydt(532)-k(6)*y(532);
dydt(27)=dydt(27)+k(6)*y(532);
dydt(515)=dydt(515)+k(6)*y(532);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(530)=dydt(530)-k(33)*y(530);
dydt(507)=dydt(507)+k(33)*y(530);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(530)=dydt(530)-k(28)*y(530);
dydt(532)=dydt(532)+k(28)*y(530);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(530)=dydt(530)-k(27)*y(530);
dydt(525)=dydt(525)+k(27)*y(530);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(530)=dydt(530)-k(22)*y(530);
dydt(2)=dydt(2)+k(22)*y(530);
dydt(531)=dydt(531)+k(22)*y(530);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(530)=dydt(530)-k(22)*y(530);
dydt(32)=dydt(32)+k(22)*y(530);
dydt(524)=dydt(524)+k(22)*y(530);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(530)=dydt(530)-k(17)*y(530);
dydt(500)=dydt(500)+k(17)*y(530);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(530)=dydt(530)-k(16)*y(530);
dydt(3)=dydt(3)+k(16)*y(530);
dydt(356)=dydt(356)+k(16)*y(530);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(530);
dydt(530)=dydt(530)-k(9)*y(3)*y(530);
dydt(357)=dydt(357)+k(9)*y(3)*y(530);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(530)=dydt(530)-k(8)*y(530);
dydt(28)=dydt(28)+k(8)*y(530);
dydt(43)=dydt(43)+k(8)*y(530);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(530)=dydt(530)-k(6)*y(530);
dydt(27)=dydt(27)+k(6)*y(530);
dydt(114)=dydt(114)+k(6)*y(530);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(525)=dydt(525)-k(34)*y(525);
dydt(530)=dydt(530)+k(34)*y(525);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(525)=dydt(525)-k(33)*y(525);
dydt(521)=dydt(521)+k(33)*y(525);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(525)=dydt(525)-k(30)*y(525);
dydt(520)=dydt(520)+k(30)*y(525);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(525)=dydt(525)-k(22)*y(525);
dydt(2)=dydt(2)+k(22)*y(525);
dydt(529)=dydt(529)+k(22)*y(525);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(525)=dydt(525)-k(22)*y(525);
dydt(197)=dydt(197)+k(22)*y(525);
dydt(524)=dydt(524)+k(22)*y(525);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(525)=dydt(525)-k(17)*y(525);
dydt(528)=dydt(528)+k(17)*y(525);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(525)=dydt(525)-k(16)*y(525);
dydt(3)=dydt(3)+k(16)*y(525);
dydt(527)=dydt(527)+k(16)*y(525);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(525);
dydt(525)=dydt(525)-k(9)*y(3)*y(525);
dydt(526)=dydt(526)+k(9)*y(3)*y(525);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(525)=dydt(525)-k(8)*y(525);
dydt(209)=dydt(209)+k(8)*y(525);
dydt(43)=dydt(43)+k(8)*y(525);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(525)=dydt(525)-k(6)*y(525);
dydt(192)=dydt(192)+k(6)*y(525);
dydt(114)=dydt(114)+k(6)*y(525);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(521)=dydt(521)-k(34)*y(521);
dydt(507)=dydt(507)+k(34)*y(521);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(521)=dydt(521)-k(30)*y(521);
dydt(514)=dydt(514)+k(30)*y(521);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(521)=dydt(521)-k(25)*y(521);
dydt(525)=dydt(525)+k(25)*y(521);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(521)=dydt(521)-k(22)*y(521);
dydt(2)=dydt(2)+k(22)*y(521);
dydt(519)=dydt(519)+k(22)*y(521);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(521)=dydt(521)-k(22)*y(521);
dydt(182)=dydt(182)+k(22)*y(521);
dydt(524)=dydt(524)+k(22)*y(521);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(521)=dydt(521)-k(17)*y(521);
dydt(523)=dydt(523)+k(17)*y(521);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(521)=dydt(521)-k(16)*y(521);
dydt(3)=dydt(3)+k(16)*y(521);
dydt(509)=dydt(509)+k(16)*y(521);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(521);
dydt(521)=dydt(521)-k(9)*y(3)*y(521);
dydt(522)=dydt(522)+k(9)*y(3)*y(521);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(521)=dydt(521)-k(8)*y(521);
dydt(214)=dydt(214)+k(8)*y(521);
dydt(43)=dydt(43)+k(8)*y(521);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(521)=dydt(521)-k(6)*y(521);
dydt(177)=dydt(177)+k(6)*y(521);
dydt(114)=dydt(114)+k(6)*y(521);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(514)=dydt(514)-k(34)*y(514);
dydt(521)=dydt(521)+k(34)*y(514);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(514)=dydt(514)-k(34)*y(514);
dydt(511)=dydt(511)+k(34)*y(514);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(514)=dydt(514)-k(25)*y(514);
dydt(520)=dydt(520)+k(25)*y(514);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(514)=dydt(514)-k(22)*y(514);
dydt(182)=dydt(182)+k(22)*y(514);
dydt(519)=dydt(519)+k(22)*y(514);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(514)=dydt(514)-k(22)*y(514);
dydt(182)=dydt(182)+k(22)*y(514);
dydt(518)=dydt(518)+k(22)*y(514);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(514)=dydt(514)-k(17)*y(514);
dydt(517)=dydt(517)+k(17)*y(514);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(514)=dydt(514)-k(16)*y(514);
dydt(3)=dydt(3)+k(16)*y(514);
dydt(513)=dydt(513)+k(16)*y(514);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(514);
dydt(514)=dydt(514)-k(9)*y(3)*y(514);
dydt(516)=dydt(516)+k(9)*y(3)*y(514);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(514)=dydt(514)-k(8)*y(514);
dydt(214)=dydt(214)+k(8)*y(514);
dydt(222)=dydt(222)+k(8)*y(514);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(514)=dydt(514)-k(6)*y(514);
dydt(177)=dydt(177)+k(6)*y(514);
dydt(515)=dydt(515)+k(6)*y(514);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(513)=dydt(513)-k(34)*y(513);
dydt(509)=dydt(509)+k(34)*y(513);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(513)=dydt(513)-k(34)*y(513);
dydt(510)=dydt(510)+k(34)*y(513);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(513)=dydt(513)-k(31)*y(513);
dydt(187)=dydt(187)+k(31)*y(513);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(513)=dydt(513)-k(22)*y(513);
dydt(182)=dydt(182)+k(22)*y(513);
dydt(344)=dydt(344)+k(22)*y(513);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(513)=dydt(513)-k(22)*y(513);
dydt(182)=dydt(182)+k(22)*y(513);
dydt(512)=dydt(512)+k(22)*y(513);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(513);
dydt(513)=dydt(513)-k(15)*y(3)*y(513);
dydt(514)=dydt(514)+k(15)*y(3)*y(513);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(513);
dydt(513)=dydt(513)-k(9)*y(3)*y(513);
dydt(188)=dydt(188)+k(9)*y(3)*y(513);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(513)=dydt(513)-k(8)*y(513);
dydt(218)=dydt(218)+k(8)*y(513);
dydt(214)=dydt(214)+k(8)*y(513);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(513)=dydt(513)-k(6)*y(513);
dydt(177)=dydt(177)+k(6)*y(513);
dydt(264)=dydt(264)+k(6)*y(513);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(510)=dydt(510)-k(34)*y(510);
dydt(506)=dydt(506)+k(34)*y(510);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(510)=dydt(510)-k(31)*y(510);
dydt(179)=dydt(179)+k(31)*y(510);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(510)=dydt(510)-k(29)*y(510);
dydt(513)=dydt(513)+k(29)*y(510);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(510)=dydt(510)-k(22)*y(510);
dydt(182)=dydt(182)+k(22)*y(510);
dydt(508)=dydt(508)+k(22)*y(510);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(510)=dydt(510)-k(22)*y(510);
dydt(2)=dydt(2)+k(22)*y(510);
dydt(512)=dydt(512)+k(22)*y(510);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(510);
dydt(510)=dydt(510)-k(15)*y(3)*y(510);
dydt(511)=dydt(511)+k(15)*y(3)*y(510);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(510);
dydt(510)=dydt(510)-k(9)*y(3)*y(510);
dydt(180)=dydt(180)+k(9)*y(3)*y(510);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(510)=dydt(510)-k(8)*y(510);
dydt(218)=dydt(218)+k(8)*y(510);
dydt(22)=dydt(22)+k(8)*y(510);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(510)=dydt(510)-k(6)*y(510);
dydt(21)=dydt(21)+k(6)*y(510);
dydt(264)=dydt(264)+k(6)*y(510);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(506)=dydt(506)-k(31)*y(506);
dydt(172)=dydt(172)+k(31)*y(506);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(506)=dydt(506)-k(28)*y(506);
dydt(510)=dydt(510)+k(28)*y(506);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(506)=dydt(506)-k(27)*y(506);
dydt(509)=dydt(509)+k(27)*y(506);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(506)=dydt(506)-k(22)*y(506);
dydt(2)=dydt(2)+k(22)*y(506);
dydt(508)=dydt(508)+k(22)*y(506);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(506)=dydt(506)-k(22)*y(506);
dydt(2)=dydt(2)+k(22)*y(506);
dydt(354)=dydt(354)+k(22)*y(506);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(506);
dydt(506)=dydt(506)-k(15)*y(3)*y(506);
dydt(507)=dydt(507)+k(15)*y(3)*y(506);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(506);
dydt(506)=dydt(506)-k(9)*y(3)*y(506);
dydt(173)=dydt(173)+k(9)*y(3)*y(506);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(506)=dydt(506)-k(8)*y(506);
dydt(111)=dydt(111)+k(8)*y(506);
dydt(22)=dydt(22)+k(8)*y(506);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(506)=dydt(506)-k(6)*y(506);
dydt(21)=dydt(21)+k(6)*y(506);
dydt(110)=dydt(110)+k(6)*y(506);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(503)=dydt(503)-k(31)*y(503);
dydt(275)=dydt(275)+k(31)*y(503);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(503)=dydt(503)-k(31)*y(503);
dydt(506)=dydt(506)+k(31)*y(503);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(503)=dydt(503)-k(28)*y(503);
dydt(266)=dydt(266)+k(28)*y(503);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(503)=dydt(503)-k(27)*y(503);
dydt(505)=dydt(505)+k(27)*y(503);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(503)=dydt(503)-k(22)*y(503);
dydt(2)=dydt(2)+k(22)*y(503);
dydt(252)=dydt(252)+k(22)*y(503);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(503)=dydt(503)-k(22)*y(503);
dydt(2)=dydt(2)+k(22)*y(503);
dydt(501)=dydt(501)+k(22)*y(503);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(503);
dydt(503)=dydt(503)-k(15)*y(3)*y(503);
dydt(253)=dydt(253)+k(15)*y(3)*y(503);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(503);
dydt(503)=dydt(503)-k(15)*y(3)*y(503);
dydt(504)=dydt(504)+k(15)*y(3)*y(503);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(503)=dydt(503)-k(8)*y(503);
dydt(111)=dydt(111)+k(8)*y(503);
dydt(437)=dydt(437)+k(8)*y(503);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(503)=dydt(503)-k(6)*y(503);
dydt(111)=dydt(111)+k(6)*y(503);
dydt(110)=dydt(110)+k(6)*y(503);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(499)=dydt(499)-k(33)*y(499);
dydt(503)=dydt(503)+k(33)*y(499);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(499)=dydt(499)-k(31)*y(499);
dydt(482)=dydt(482)+k(31)*y(499);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(499)=dydt(499)-k(31)*y(499);
dydt(356)=dydt(356)+k(31)*y(499);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(499)=dydt(499)-k(28)*y(499);
dydt(495)=dydt(495)+k(28)*y(499);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(499)=dydt(499)-k(27)*y(499);
dydt(502)=dydt(502)+k(27)*y(499);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(499)=dydt(499)-k(22)*y(499);
dydt(2)=dydt(2)+k(22)*y(499);
dydt(245)=dydt(245)+k(22)*y(499);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(499)=dydt(499)-k(22)*y(499);
dydt(32)=dydt(32)+k(22)*y(499);
dydt(501)=dydt(501)+k(22)*y(499);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(499);
dydt(499)=dydt(499)-k(15)*y(3)*y(499);
dydt(246)=dydt(246)+k(15)*y(3)*y(499);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(499);
dydt(499)=dydt(499)-k(15)*y(3)*y(499);
dydt(500)=dydt(500)+k(15)*y(3)*y(499);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(499)=dydt(499)-k(8)*y(499);
dydt(111)=dydt(111)+k(8)*y(499);
dydt(34)=dydt(34)+k(8)*y(499);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(499)=dydt(499)-k(6)*y(499);
dydt(33)=dydt(33)+k(6)*y(499);
dydt(110)=dydt(110)+k(6)*y(499);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(495)=dydt(495)-k(34)*y(495);
dydt(499)=dydt(499)+k(34)*y(495);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(495)=dydt(495)-k(33)*y(495);
dydt(266)=dydt(266)+k(33)*y(495);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(495)=dydt(495)-k(31)*y(495);
dydt(498)=dydt(498)+k(31)*y(495);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(495)=dydt(495)-k(31)*y(495);
dydt(359)=dydt(359)+k(31)*y(495);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(495)=dydt(495)-k(29)*y(495);
dydt(490)=dydt(490)+k(29)*y(495);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(495)=dydt(495)-k(22)*y(495);
dydt(182)=dydt(182)+k(22)*y(495);
dydt(245)=dydt(245)+k(22)*y(495);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(495)=dydt(495)-k(22)*y(495);
dydt(32)=dydt(32)+k(22)*y(495);
dydt(497)=dydt(497)+k(22)*y(495);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(495);
dydt(495)=dydt(495)-k(15)*y(3)*y(495);
dydt(493)=dydt(493)+k(15)*y(3)*y(495);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(495);
dydt(495)=dydt(495)-k(15)*y(3)*y(495);
dydt(496)=dydt(496)+k(15)*y(3)*y(495);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(495)=dydt(495)-k(8)*y(495);
dydt(218)=dydt(218)+k(8)*y(495);
dydt(34)=dydt(34)+k(8)*y(495);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(495)=dydt(495)-k(6)*y(495);
dydt(33)=dydt(33)+k(6)*y(495);
dydt(264)=dydt(264)+k(6)*y(495);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(493)=dydt(493)-k(34)*y(493);
dydt(246)=dydt(246)+k(34)*y(493);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(493)=dydt(493)-k(33)*y(493);
dydt(263)=dydt(263)+k(33)*y(493);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(493)=dydt(493)-k(31)*y(493);
dydt(479)=dydt(479)+k(31)*y(493);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(493)=dydt(493)-k(29)*y(493);
dydt(488)=dydt(488)+k(29)*y(493);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(493)=dydt(493)-k(26)*y(493);
dydt(248)=dydt(248)+k(26)*y(493);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(493)=dydt(493)-k(22)*y(493);
dydt(182)=dydt(182)+k(22)*y(493);
dydt(243)=dydt(243)+k(22)*y(493);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(493)=dydt(493)-k(22)*y(493);
dydt(32)=dydt(32)+k(22)*y(493);
dydt(267)=dydt(267)+k(22)*y(493);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(493)=dydt(493)-k(16)*y(493);
dydt(3)=dydt(3)+k(16)*y(493);
dydt(495)=dydt(495)+k(16)*y(493);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(493);
dydt(493)=dydt(493)-k(15)*y(3)*y(493);
dydt(494)=dydt(494)+k(15)*y(3)*y(493);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(493)=dydt(493)-k(8)*y(493);
dydt(218)=dydt(218)+k(8)*y(493);
dydt(38)=dydt(38)+k(8)*y(493);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(493)=dydt(493)-k(6)*y(493);
dydt(264)=dydt(264)+k(6)*y(493);
dydt(37)=dydt(37)+k(6)*y(493);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(488)=dydt(488)-k(34)*y(488);
dydt(485)=dydt(485)+k(34)*y(488);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(488)=dydt(488)-k(34)*y(488);
dydt(493)=dydt(493)+k(34)*y(488);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(488)=dydt(488)-k(33)*y(488);
dydt(268)=dydt(268)+k(33)*y(488);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(488)=dydt(488)-k(31)*y(488);
dydt(486)=dydt(486)+k(31)*y(488);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(488)=dydt(488)-k(26)*y(488);
dydt(492)=dydt(492)+k(26)*y(488);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(488)=dydt(488)-k(22)*y(488);
dydt(182)=dydt(182)+k(22)*y(488);
dydt(491)=dydt(491)+k(22)*y(488);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(488)=dydt(488)-k(22)*y(488);
dydt(197)=dydt(197)+k(22)*y(488);
dydt(267)=dydt(267)+k(22)*y(488);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(488)=dydt(488)-k(16)*y(488);
dydt(3)=dydt(3)+k(16)*y(488);
dydt(490)=dydt(490)+k(16)*y(488);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(488);
dydt(488)=dydt(488)-k(15)*y(3)*y(488);
dydt(489)=dydt(489)+k(15)*y(3)*y(488);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(488)=dydt(488)-k(8)*y(488);
dydt(218)=dydt(218)+k(8)*y(488);
dydt(228)=dydt(228)+k(8)*y(488);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(488)=dydt(488)-k(6)*y(488);
dydt(264)=dydt(264)+k(6)*y(488);
dydt(227)=dydt(227)+k(6)*y(488);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(486)=dydt(486)-k(34)*y(486);
dydt(484)=dydt(484)+k(34)*y(486);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(486)=dydt(486)-k(34)*y(486);
dydt(479)=dydt(479)+k(34)*y(486);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(486)=dydt(486)-k(33)*y(486);
dydt(273)=dydt(273)+k(33)*y(486);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(486)=dydt(486)-k(26)*y(486);
dydt(294)=dydt(294)+k(26)*y(486);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(486)=dydt(486)-k(22)*y(486);
dydt(182)=dydt(182)+k(22)*y(486);
dydt(232)=dydt(232)+k(22)*y(486);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(486)=dydt(486)-k(22)*y(486);
dydt(197)=dydt(197)+k(22)*y(486);
dydt(272)=dydt(272)+k(22)*y(486);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(486)=dydt(486)-k(18)*y(486);
dydt(488)=dydt(488)+k(18)*y(486);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(486)=dydt(486)-k(16)*y(486);
dydt(3)=dydt(3)+k(16)*y(486);
dydt(487)=dydt(487)+k(16)*y(486);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(486);
dydt(486)=dydt(486)-k(9)*y(3)*y(486);
dydt(476)=dydt(476)+k(9)*y(3)*y(486);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(486)=dydt(486)-k(8)*y(486);
dydt(177)=dydt(177)+k(8)*y(486);
dydt(228)=dydt(228)+k(8)*y(486);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(486)=dydt(486)-k(6)*y(486);
dydt(176)=dydt(176)+k(6)*y(486);
dydt(227)=dydt(227)+k(6)*y(486);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(484)=dydt(484)-k(34)*y(484);
dydt(481)=dydt(481)+k(34)*y(484);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(484)=dydt(484)-k(33)*y(484);
dydt(278)=dydt(278)+k(33)*y(484);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(484)=dydt(484)-k(30)*y(484);
dydt(486)=dydt(486)+k(30)*y(484);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(484)=dydt(484)-k(26)*y(484);
dydt(474)=dydt(474)+k(26)*y(484);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(484)=dydt(484)-k(22)*y(484);
dydt(2)=dydt(2)+k(22)*y(484);
dydt(232)=dydt(232)+k(22)*y(484);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(484)=dydt(484)-k(22)*y(484);
dydt(197)=dydt(197)+k(22)*y(484);
dydt(276)=dydt(276)+k(22)*y(484);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(484)=dydt(484)-k(18)*y(484);
dydt(485)=dydt(485)+k(18)*y(484);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(484)=dydt(484)-k(16)*y(484);
dydt(3)=dydt(3)+k(16)*y(484);
dydt(427)=dydt(427)+k(16)*y(484);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(9)*y(3)*y(484);
dydt(484)=dydt(484)-k(9)*y(3)*y(484);
dydt(428)=dydt(428)+k(9)*y(3)*y(484);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(484)=dydt(484)-k(8)*y(484);
dydt(21)=dydt(21)+k(8)*y(484);
dydt(228)=dydt(228)+k(8)*y(484);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(484)=dydt(484)-k(6)*y(484);
dydt(89)=dydt(89)+k(6)*y(484);
dydt(227)=dydt(227)+k(6)*y(484);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(481)=dydt(481)-k(33)*y(481);
dydt(274)=dydt(274)+k(33)*y(481);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(481)=dydt(481)-k(28)*y(481);
dydt(479)=dydt(479)+k(28)*y(481);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(481)=dydt(481)-k(27)*y(481);
dydt(484)=dydt(484)+k(27)*y(481);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(481)=dydt(481)-k(26)*y(481);
dydt(483)=dydt(483)+k(26)*y(481);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(481)=dydt(481)-k(22)*y(481);
dydt(2)=dydt(2)+k(22)*y(481);
dydt(249)=dydt(249)+k(22)*y(481);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(481)=dydt(481)-k(22)*y(481);
dydt(32)=dydt(32)+k(22)*y(481);
dydt(276)=dydt(276)+k(22)*y(481);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(481)=dydt(481)-k(18)*y(481);
dydt(246)=dydt(246)+k(18)*y(481);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(481)=dydt(481)-k(16)*y(481);
dydt(3)=dydt(3)+k(16)*y(481);
dydt(482)=dydt(482)+k(16)*y(481);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(481);
dydt(481)=dydt(481)-k(9)*y(3)*y(481);
dydt(480)=dydt(480)+k(9)*y(3)*y(481);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(481)=dydt(481)-k(8)*y(481);
dydt(21)=dydt(21)+k(8)*y(481);
dydt(38)=dydt(38)+k(8)*y(481);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(481)=dydt(481)-k(6)*y(481);
dydt(89)=dydt(89)+k(6)*y(481);
dydt(37)=dydt(37)+k(6)*y(481);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(480)=dydt(480)-k(33)*y(480);
dydt(51)=dydt(51)+k(33)*y(480);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(480)=dydt(480)-k(28)*y(480);
dydt(478)=dydt(478)+k(28)*y(480);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(480)=dydt(480)-k(27)*y(480);
dydt(428)=dydt(428)+k(27)*y(480);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(480)=dydt(480)-k(26)*y(480);
dydt(469)=dydt(469)+k(26)*y(480);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(480)=dydt(480)-k(22)*y(480);
dydt(2)=dydt(2)+k(22)*y(480);
dydt(40)=dydt(40)+k(22)*y(480);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(480)=dydt(480)-k(22)*y(480);
dydt(32)=dydt(32)+k(22)*y(480);
dydt(58)=dydt(58)+k(22)*y(480);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(480)=dydt(480)-k(16)*y(480);
dydt(3)=dydt(3)+k(16)*y(480);
dydt(382)=dydt(382)+k(16)*y(480);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(480)=dydt(480)-k(10)*y(480);
dydt(3)=dydt(3)+k(10)*y(480);
dydt(481)=dydt(481)+k(10)*y(480);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(480)=dydt(480)-k(8)*y(480);
dydt(140)=dydt(140)+k(8)*y(480);
dydt(38)=dydt(38)+k(8)*y(480);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(480)=dydt(480)-k(6)*y(480);
dydt(37)=dydt(37)+k(6)*y(480);
dydt(380)=dydt(380)+k(6)*y(480);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(478)=dydt(478)-k(34)*y(478);
dydt(480)=dydt(480)+k(34)*y(478);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(478)=dydt(478)-k(33)*y(478);
dydt(270)=dydt(270)+k(33)*y(478);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(478)=dydt(478)-k(29)*y(478);
dydt(476)=dydt(476)+k(29)*y(478);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(478)=dydt(478)-k(26)*y(478);
dydt(477)=dydt(477)+k(26)*y(478);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(478)=dydt(478)-k(22)*y(478);
dydt(182)=dydt(182)+k(22)*y(478);
dydt(40)=dydt(40)+k(22)*y(478);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

dydt(478)=dydt(478)-k(22)*y(478);
dydt(32)=dydt(32)+k(22)*y(478);
dydt(422)=dydt(422)+k(22)*y(478);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(478)=dydt(478)-k(16)*y(478);
dydt(3)=dydt(3)+k(16)*y(478);
dydt(423)=dydt(423)+k(16)*y(478);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

dydt(478)=dydt(478)-k(10)*y(478);
dydt(3)=dydt(3)+k(10)*y(478);
dydt(479)=dydt(479)+k(10)*y(478);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(478)=dydt(478)-k(8)*y(478);
dydt(184)=dydt(184)+k(8)*y(478);
dydt(38)=dydt(38)+k(8)*y(478);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(478)=dydt(478)-k(6)*y(478);
dydt(37)=dydt(37)+k(6)*y(478);
dydt(391)=dydt(391)+k(6)*y(478);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(477)=dydt(477)-k(34)*y(477);
dydt(469)=dydt(469)+k(34)*y(477);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(477)=dydt(477)-k(33)*y(477);
dydt(478)=dydt(478)+k(33)*y(477);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(477)=dydt(477)-k(33)*y(477);
dydt(207)=dydt(207)+k(33)*y(477);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(477)=dydt(477)-k(29)*y(477);
dydt(475)=dydt(475)+k(29)*y(477);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(477)=dydt(477)-k(22)*y(477);
dydt(197)=dydt(197)+k(22)*y(477);
dydt(40)=dydt(40)+k(22)*y(477);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(477)=dydt(477)-k(22)*y(477);
dydt(32)=dydt(32)+k(22)*y(477);
dydt(206)=dydt(206)+k(22)*y(477);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(477)=dydt(477)-k(16)*y(477);
dydt(3)=dydt(3)+k(16)*y(477);
dydt(439)=dydt(439)+k(16)*y(477);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(477)=dydt(477)-k(10)*y(477);
dydt(3)=dydt(3)+k(10)*y(477);
dydt(293)=dydt(293)+k(10)*y(477);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(477)=dydt(477)-k(8)*y(477);
dydt(335)=dydt(335)+k(8)*y(477);
dydt(38)=dydt(38)+k(8)*y(477);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(477)=dydt(477)-k(6)*y(477);
dydt(37)=dydt(37)+k(6)*y(477);
dydt(407)=dydt(407)+k(6)*y(477);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(475)=dydt(475)-k(34)*y(475);
dydt(473)=dydt(473)+k(34)*y(475);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(475)=dydt(475)-k(34)*y(475);
dydt(477)=dydt(477)+k(34)*y(475);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(475)=dydt(475)-k(33)*y(475);
dydt(476)=dydt(476)+k(33)*y(475);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(475)=dydt(475)-k(33)*y(475);
dydt(289)=dydt(289)+k(33)*y(475);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

dydt(475)=dydt(475)-k(22)*y(475);
dydt(197)=dydt(197)+k(22)*y(475);
dydt(205)=dydt(205)+k(22)*y(475);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(475)=dydt(475)-k(22)*y(475);
dydt(197)=dydt(197)+k(22)*y(475);
dydt(206)=dydt(206)+k(22)*y(475);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(475)=dydt(475)-k(16)*y(475);
dydt(3)=dydt(3)+k(16)*y(475);
dydt(434)=dydt(434)+k(16)*y(475);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(475)=dydt(475)-k(10)*y(475);
dydt(3)=dydt(3)+k(10)*y(475);
dydt(294)=dydt(294)+k(10)*y(475);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(475)=dydt(475)-k(8)*y(475);
dydt(335)=dydt(335)+k(8)*y(475);
dydt(228)=dydt(228)+k(8)*y(475);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(475)=dydt(475)-k(6)*y(475);
dydt(227)=dydt(227)+k(6)*y(475);
dydt(407)=dydt(407)+k(6)*y(475);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(473)=dydt(473)-k(34)*y(473);
dydt(469)=dydt(469)+k(34)*y(473);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(473)=dydt(473)-k(33)*y(473);
dydt(284)=dydt(284)+k(33)*y(473);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(473)=dydt(473)-k(33)*y(473);
dydt(428)=dydt(428)+k(33)*y(473);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(473)=dydt(473)-k(30)*y(473);
dydt(475)=dydt(475)+k(30)*y(473);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(473)=dydt(473)-k(22)*y(473);
dydt(197)=dydt(197)+k(22)*y(473);
dydt(59)=dydt(59)+k(22)*y(473);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(473)=dydt(473)-k(22)*y(473);
dydt(32)=dydt(32)+k(22)*y(473);
dydt(205)=dydt(205)+k(22)*y(473);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(473)=dydt(473)-k(16)*y(473);
dydt(3)=dydt(3)+k(16)*y(473);
dydt(471)=dydt(471)+k(16)*y(473);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(473)=dydt(473)-k(10)*y(473);
dydt(3)=dydt(3)+k(10)*y(473);
dydt(474)=dydt(474)+k(10)*y(473);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(473)=dydt(473)-k(8)*y(473);
dydt(131)=dydt(131)+k(8)*y(473);
dydt(228)=dydt(228)+k(8)*y(473);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(473)=dydt(473)-k(6)*y(473);
dydt(227)=dydt(227)+k(6)*y(473);
dydt(441)=dydt(441)+k(6)*y(473);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(471)=dydt(471)-k(34)*y(471);
dydt(444)=dydt(444)+k(34)*y(471);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(471)=dydt(471)-k(33)*y(471);
dydt(426)=dydt(426)+k(33)*y(471);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(471)=dydt(471)-k(33)*y(471);
dydt(432)=dydt(432)+k(33)*y(471);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(471)=dydt(471)-k(31)*y(471);
dydt(402)=dydt(402)+k(31)*y(471);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(471)=dydt(471)-k(30)*y(471);
dydt(434)=dydt(434)+k(30)*y(471);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(471)=dydt(471)-k(22)*y(471);
dydt(32)=dydt(32)+k(22)*y(471);
dydt(401)=dydt(401)+k(22)*y(471);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(471)=dydt(471)-k(22)*y(471);
dydt(197)=dydt(197)+k(22)*y(471);
dydt(443)=dydt(443)+k(22)*y(471);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(471);
dydt(471)=dydt(471)-k(15)*y(3)*y(471);
dydt(473)=dydt(473)+k(15)*y(3)*y(471);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(471)=dydt(471)-k(10)*y(471);
dydt(3)=dydt(3)+k(10)*y(471);
dydt(472)=dydt(472)+k(10)*y(471);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(471)=dydt(471)-k(8)*y(471);
dydt(319)=dydt(319)+k(8)*y(471);
dydt(131)=dydt(131)+k(8)*y(471);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(471)=dydt(471)-k(6)*y(471);
dydt(257)=dydt(257)+k(6)*y(471);
dydt(441)=dydt(441)+k(6)*y(471);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(444)=dydt(444)-k(33)*y(444);
dydt(382)=dydt(382)+k(33)*y(444);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(444)=dydt(444)-k(33)*y(444);
dydt(440)=dydt(440)+k(33)*y(444);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(444)=dydt(444)-k(31)*y(444);
dydt(446)=dydt(446)+k(31)*y(444);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(444)=dydt(444)-k(28)*y(444);
dydt(439)=dydt(439)+k(28)*y(444);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(444)=dydt(444)-k(27)*y(444);
dydt(471)=dydt(471)+k(27)*y(444);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(444)=dydt(444)-k(22)*y(444);
dydt(32)=dydt(32)+k(22)*y(444);
dydt(470)=dydt(470)+k(22)*y(444);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(444)=dydt(444)-k(22)*y(444);
dydt(32)=dydt(32)+k(22)*y(444);
dydt(443)=dydt(443)+k(22)*y(444);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(444);
dydt(444)=dydt(444)-k(15)*y(3)*y(444);
dydt(469)=dydt(469)+k(15)*y(3)*y(444);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(444)=dydt(444)-k(10)*y(444);
dydt(3)=dydt(3)+k(10)*y(444);
dydt(468)=dydt(468)+k(10)*y(444);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(444)=dydt(444)-k(8)*y(444);
dydt(34)=dydt(34)+k(8)*y(444);
dydt(131)=dydt(131)+k(8)*y(444);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(444)=dydt(444)-k(6)*y(444);
dydt(33)=dydt(33)+k(6)*y(444);
dydt(441)=dydt(441)+k(6)*y(444);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(446)=dydt(446)-k(33)*y(446);
dydt(379)=dydt(379)+k(33)*y(446);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(446)=dydt(446)-k(33)*y(446);
dydt(445)=dydt(445)+k(33)*y(446);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(446)=dydt(446)-k(28)*y(446);
dydt(412)=dydt(412)+k(28)*y(446);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(446)=dydt(446)-k(27)*y(446);
dydt(402)=dydt(402)+k(27)*y(446);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(446)=dydt(446)-k(22)*y(446);
dydt(32)=dydt(32)+k(22)*y(446);
dydt(384)=dydt(384)+k(22)*y(446);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(446)=dydt(446)-k(22)*y(446);
dydt(32)=dydt(32)+k(22)*y(446);
dydt(405)=dydt(405)+k(22)*y(446);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(446)=dydt(446)-k(11)*y(446);
dydt(444)=dydt(444)+k(11)*y(446);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(446)=dydt(446)-k(10)*y(446);
dydt(3)=dydt(3)+k(10)*y(446);
dydt(160)=dydt(160)+k(10)*y(446);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(446);
dydt(446)=dydt(446)-k(9)*y(3)*y(446);
dydt(159)=dydt(159)+k(9)*y(3)*y(446);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(446)=dydt(446)-k(8)*y(446);
dydt(28)=dydt(28)+k(8)*y(446);
dydt(131)=dydt(131)+k(8)*y(446);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(446)=dydt(446)-k(6)*y(446);
dydt(27)=dydt(27)+k(6)*y(446);
dydt(441)=dydt(441)+k(6)*y(446);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(457)=dydt(457)-k(34)*y(457);
dydt(467)=dydt(467)+k(34)*y(457);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(457)=dydt(457)-k(32)*y(457);
dydt(215)=dydt(215)+k(32)*y(457);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(457)=dydt(457)-k(23)*y(457);
dydt(400)=dydt(400)+k(23)*y(457);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(457)=dydt(457)-k(22)*y(457);
dydt(182)=dydt(182)+k(22)*y(457);
dydt(149)=dydt(149)+k(22)*y(457);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(457);
dydt(457)=dydt(457)-k(21)*y(197)*y(457);
dydt(193)=dydt(193)+k(21)*y(197)*y(457);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(457);
dydt(457)=dydt(457)-k(21)*y(32)*y(457);
dydt(448)=dydt(448)+k(21)*y(32)*y(457);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(457);
dydt(457)=dydt(457)-k(21)*y(2)*y(457);
dydt(460)=dydt(460)+k(21)*y(2)*y(457);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(457)=dydt(457)-k(10)*y(457);
dydt(3)=dydt(3)+k(10)*y(457);
dydt(189)=dydt(189)+k(10)*y(457);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(457)=dydt(457)-k(10)*y(457);
dydt(3)=dydt(3)+k(10)*y(457);
dydt(396)=dydt(396)+k(10)*y(457);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(457)=dydt(457)-k(8)*y(457);
dydt(48)=dydt(48)+k(8)*y(457);
dydt(185)=dydt(185)+k(8)*y(457);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(457)=dydt(457)-k(6)*y(457);
dydt(47)=dydt(47)+k(6)*y(457);
dydt(184)=dydt(184)+k(6)*y(457);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(458)=dydt(458)-k(33)*y(458);
dydt(461)=dydt(461)+k(33)*y(458);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(458)=dydt(458)-k(32)*y(458);
dydt(104)=dydt(104)+k(32)*y(458);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(458)=dydt(458)-k(22)*y(458);
dydt(32)=dydt(32)+k(22)*y(458);
dydt(149)=dydt(149)+k(22)*y(458);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(458);
dydt(458)=dydt(458)-k(21)*y(197)*y(458);
dydt(459)=dydt(459)+k(21)*y(197)*y(458);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(458);
dydt(458)=dydt(458)-k(21)*y(32)*y(458);
dydt(159)=dydt(159)+k(21)*y(32)*y(458);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(458);
dydt(458)=dydt(458)-k(21)*y(2)*y(458);
dydt(165)=dydt(165)+k(21)*y(2)*y(458);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(458)=dydt(458)-k(10)*y(458);
dydt(3)=dydt(3)+k(10)*y(458);
dydt(153)=dydt(153)+k(10)*y(458);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(458)=dydt(458)-k(10)*y(458);
dydt(3)=dydt(3)+k(10)*y(458);
dydt(405)=dydt(405)+k(10)*y(458);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(458)=dydt(458)-k(8)*y(458);
dydt(126)=dydt(126)+k(8)*y(458);
dydt(131)=dydt(131)+k(8)*y(458);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(458)=dydt(458)-k(6)*y(458);
dydt(48)=dydt(48)+k(6)*y(458);
dydt(441)=dydt(441)+k(6)*y(458);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(467)=dydt(467)-k(32)*y(467);
dydt(23)=dydt(23)+k(32)*y(467);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(467)=dydt(467)-k(23)*y(467);
dydt(155)=dydt(155)+k(23)*y(467);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(467)=dydt(467)-k(22)*y(467);
dydt(2)=dydt(2)+k(22)*y(467);
dydt(149)=dydt(149)+k(22)*y(467);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(467);
dydt(467)=dydt(467)-k(21)*y(197)*y(467);
dydt(466)=dydt(466)+k(21)*y(197)*y(467);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(467);
dydt(467)=dydt(467)-k(21)*y(32)*y(467);
dydt(165)=dydt(165)+k(21)*y(32)*y(467);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(467);
dydt(467)=dydt(467)-k(21)*y(2)*y(467);
dydt(171)=dydt(171)+k(21)*y(2)*y(467);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(467)=dydt(467)-k(10)*y(467);
dydt(3)=dydt(3)+k(10)*y(467);
dydt(145)=dydt(145)+k(10)*y(467);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(467)=dydt(467)-k(10)*y(467);
dydt(3)=dydt(3)+k(10)*y(467);
dydt(388)=dydt(388)+k(10)*y(467);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(467)=dydt(467)-k(8)*y(467);
dydt(48)=dydt(48)+k(8)*y(467);
dydt(141)=dydt(141)+k(8)*y(467);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(467)=dydt(467)-k(6)*y(467);
dydt(47)=dydt(47)+k(6)*y(467);
dydt(140)=dydt(140)+k(6)*y(467);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(466)=dydt(466)-k(34)*y(466);
dydt(165)=dydt(165)+k(34)*y(466);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(466)=dydt(466)-k(33)*y(466);
dydt(178)=dydt(178)+k(33)*y(466);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(466)=dydt(466)-k(29)*y(466);
dydt(193)=dydt(193)+k(29)*y(466);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(466)=dydt(466)-k(23)*y(466);
dydt(465)=dydt(465)+k(23)*y(466);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(466)=dydt(466)-k(22)*y(466);
dydt(197)=dydt(197)+k(22)*y(466);
dydt(467)=dydt(467)+k(22)*y(466);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(466)=dydt(466)-k(22)*y(466);
dydt(2)=dydt(2)+k(22)*y(466);
dydt(408)=dydt(408)+k(22)*y(466);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(466)=dydt(466)-k(10)*y(466);
dydt(3)=dydt(3)+k(10)*y(466);
dydt(169)=dydt(169)+k(10)*y(466);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(466)=dydt(466)-k(10)*y(466);
dydt(3)=dydt(3)+k(10)*y(466);
dydt(411)=dydt(411)+k(10)*y(466);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(466)=dydt(466)-k(8)*y(466);
dydt(335)=dydt(335)+k(8)*y(466);
dydt(141)=dydt(141)+k(8)*y(466);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(466)=dydt(466)-k(6)*y(466);
dydt(140)=dydt(140)+k(6)*y(466);
dydt(407)=dydt(407)+k(6)*y(466);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(465)=dydt(465)-k(34)*y(465);
dydt(159)=dydt(159)+k(34)*y(465);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(465)=dydt(465)-k(33)*y(465);
dydt(365)=dydt(365)+k(33)*y(465);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(465)=dydt(465)-k(33)*y(465);
dydt(466)=dydt(466)+k(33)*y(465);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(465)=dydt(465)-k(29)*y(465);
dydt(464)=dydt(464)+k(29)*y(465);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(465)=dydt(465)-k(22)*y(465);
dydt(197)=dydt(197)+k(22)*y(465);
dydt(155)=dydt(155)+k(22)*y(465);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(465)=dydt(465)-k(22)*y(465);
dydt(32)=dydt(32)+k(22)*y(465);
dydt(408)=dydt(408)+k(22)*y(465);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(465)=dydt(465)-k(10)*y(465);
dydt(3)=dydt(3)+k(10)*y(465);
dydt(163)=dydt(163)+k(10)*y(465);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(465)=dydt(465)-k(10)*y(465);
dydt(3)=dydt(3)+k(10)*y(465);
dydt(412)=dydt(412)+k(10)*y(465);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(465)=dydt(465)-k(8)*y(465);
dydt(335)=dydt(335)+k(8)*y(465);
dydt(132)=dydt(132)+k(8)*y(465);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(465)=dydt(465)-k(6)*y(465);
dydt(131)=dydt(131)+k(6)*y(465);
dydt(407)=dydt(407)+k(6)*y(465);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(464)=dydt(464)-k(34)*y(464);
dydt(459)=dydt(459)+k(34)*y(464);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(464)=dydt(464)-k(34)*y(464);
dydt(465)=dydt(465)+k(34)*y(464);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(464)=dydt(464)-k(33)*y(464);
dydt(368)=dydt(368)+k(33)*y(464);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(464)=dydt(464)-k(33)*y(464);
dydt(193)=dydt(193)+k(33)*y(464);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(464)=dydt(464)-k(22)*y(464);
dydt(197)=dydt(197)+k(22)*y(464);
dydt(400)=dydt(400)+k(22)*y(464);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(464)=dydt(464)-k(22)*y(464);
dydt(197)=dydt(197)+k(22)*y(464);
dydt(408)=dydt(408)+k(22)*y(464);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(464)=dydt(464)-k(10)*y(464);
dydt(3)=dydt(3)+k(10)*y(464);
dydt(370)=dydt(370)+k(10)*y(464);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

dydt(464)=dydt(464)-k(10)*y(464);
dydt(3)=dydt(3)+k(10)*y(464);
dydt(403)=dydt(403)+k(10)*y(464);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(464)=dydt(464)-k(8)*y(464);
dydt(335)=dydt(335)+k(8)*y(464);
dydt(336)=dydt(336)+k(8)*y(464);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(464)=dydt(464)-k(6)*y(464);
dydt(335)=dydt(335)+k(6)*y(464);
dydt(407)=dydt(407)+k(6)*y(464);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(459)=dydt(459)-k(34)*y(459);
dydt(159)=dydt(159)+k(34)*y(459);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(459)=dydt(459)-k(33)*y(459);
dydt(372)=dydt(372)+k(33)*y(459);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(459)=dydt(459)-k(33)*y(459);
dydt(448)=dydt(448)+k(33)*y(459);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(459)=dydt(459)-k(30)*y(459);
dydt(464)=dydt(464)+k(30)*y(459);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(459)=dydt(459)-k(22)*y(459);
dydt(32)=dydt(32)+k(22)*y(459);
dydt(400)=dydt(400)+k(22)*y(459);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(459)=dydt(459)-k(22)*y(459);
dydt(197)=dydt(197)+k(22)*y(459);
dydt(458)=dydt(458)+k(22)*y(459);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(459)=dydt(459)-k(10)*y(459);
dydt(3)=dydt(3)+k(10)*y(459);
dydt(162)=dydt(162)+k(10)*y(459);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(459)=dydt(459)-k(10)*y(459);
dydt(3)=dydt(3)+k(10)*y(459);
dydt(402)=dydt(402)+k(10)*y(459);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(459)=dydt(459)-k(8)*y(459);
dydt(131)=dydt(131)+k(8)*y(459);
dydt(336)=dydt(336)+k(8)*y(459);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(459)=dydt(459)-k(6)*y(459);
dydt(335)=dydt(335)+k(6)*y(459);
dydt(441)=dydt(441)+k(6)*y(459);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(463)=dydt(463)-k(32)*y(463);
dydt(10)=dydt(10)+k(32)*y(463);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(463);
dydt(463)=dydt(463)-k(21)*y(197)*y(463);
dydt(414)=dydt(414)+k(21)*y(197)*y(463);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(463);
dydt(463)=dydt(463)-k(21)*y(32)*y(463);
dydt(104)=dydt(104)+k(21)*y(32)*y(463);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(463);
dydt(463)=dydt(463)-k(21)*y(2)*y(463);
dydt(462)=dydt(462)+k(21)*y(2)*y(463);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(463)=dydt(463)-k(13)*y(463);
dydt(149)=dydt(149)+k(13)*y(463);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) 

dydt(463)=dydt(463)-k(10)*y(463);
dydt(3)=dydt(3)+k(10)*y(463);
dydt(100)=dydt(100)+k(10)*y(463);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) 

dydt(463)=dydt(463)-k(10)*y(463);
dydt(3)=dydt(3)+k(10)*y(463);
dydt(99)=dydt(99)+k(10)*y(463);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(463)=dydt(463)-k(8)*y(463);
dydt(48)=dydt(48)+k(8)*y(463);
dydt(8)=dydt(8)+k(8)*y(463);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(463)=dydt(463)-k(6)*y(463);
dydt(7)=dydt(7)+k(6)*y(463);
dydt(47)=dydt(47)+k(6)*y(463);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(462)=dydt(462)-k(24)*y(462);
dydt(104)=dydt(104)+k(24)*y(462);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(462)=dydt(462)-k(22)*y(462);
dydt(2)=dydt(2)+k(22)*y(462);
dydt(463)=dydt(463)+k(22)*y(462);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(462)=dydt(462)-k(13)*y(462);
dydt(461)=dydt(461)+k(13)*y(462);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(462)=dydt(462)-k(10)*y(462);
dydt(3)=dydt(3)+k(10)*y(462);
dydt(96)=dydt(96)+k(10)*y(462);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(462)=dydt(462)-k(10)*y(462);
dydt(3)=dydt(3)+k(10)*y(462);
dydt(95)=dydt(95)+k(10)*y(462);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(462)=dydt(462)-k(8)*y(462);
dydt(8)=dydt(8)+k(8)*y(462);
dydt(140)=dydt(140)+k(8)*y(462);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(462)=dydt(462)-k(6)*y(462);
dydt(7)=dydt(7)+k(6)*y(462);
dydt(380)=dydt(380)+k(6)*y(462);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(461)=dydt(461)-k(32)*y(461);
dydt(462)=dydt(462)+k(32)*y(461);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(461)=dydt(461)-k(24)*y(461);
dydt(458)=dydt(458)+k(24)*y(461);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(461)=dydt(461)-k(22)*y(461);
dydt(2)=dydt(2)+k(22)*y(461);
dydt(149)=dydt(149)+k(22)*y(461);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(461);
dydt(461)=dydt(461)-k(21)*y(197)*y(461);
dydt(372)=dydt(372)+k(21)*y(197)*y(461);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(461);
dydt(461)=dydt(461)-k(21)*y(32)*y(461);
dydt(381)=dydt(381)+k(21)*y(32)*y(461);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(461);
dydt(461)=dydt(461)-k(21)*y(2)*y(461);
dydt(171)=dydt(171)+k(21)*y(2)*y(461);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(461)=dydt(461)-k(10)*y(461);
dydt(3)=dydt(3)+k(10)*y(461);
dydt(152)=dydt(152)+k(10)*y(461);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(461)=dydt(461)-k(10)*y(461);
dydt(3)=dydt(3)+k(10)*y(461);
dydt(383)=dydt(383)+k(10)*y(461);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(461)=dydt(461)-k(8)*y(461);
dydt(126)=dydt(126)+k(8)*y(461);
dydt(140)=dydt(140)+k(8)*y(461);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(461)=dydt(461)-k(6)*y(461);
dydt(48)=dydt(48)+k(6)*y(461);
dydt(380)=dydt(380)+k(6)*y(461);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(460)=dydt(460)-k(34)*y(460);
dydt(171)=dydt(171)+k(34)*y(460);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(460)=dydt(460)-k(30)*y(460);
dydt(186)=dydt(186)+k(30)*y(460);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(460)=dydt(460)-k(24)*y(460);
dydt(448)=dydt(448)+k(24)*y(460);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(460)=dydt(460)-k(23)*y(460);
dydt(372)=dydt(372)+k(23)*y(460);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(460)=dydt(460)-k(22)*y(460);
dydt(2)=dydt(2)+k(22)*y(460);
dydt(457)=dydt(457)+k(22)*y(460);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(460)=dydt(460)-k(22)*y(460);
dydt(182)=dydt(182)+k(22)*y(460);
dydt(461)=dydt(461)+k(22)*y(460);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(460)=dydt(460)-k(10)*y(460);
dydt(3)=dydt(3)+k(10)*y(460);
dydt(174)=dydt(174)+k(10)*y(460);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

dydt(460)=dydt(460)-k(10)*y(460);
dydt(3)=dydt(3)+k(10)*y(460);
dydt(389)=dydt(389)+k(10)*y(460);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(460)=dydt(460)-k(8)*y(460);
dydt(140)=dydt(140)+k(8)*y(460);
dydt(185)=dydt(185)+k(8)*y(460);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(460)=dydt(460)-k(6)*y(460);
dydt(184)=dydt(184)+k(6)*y(460);
dydt(380)=dydt(380)+k(6)*y(460);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(448)=dydt(448)-k(34)*y(448);
dydt(165)=dydt(165)+k(34)*y(448);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(448)=dydt(448)-k(33)*y(448);
dydt(460)=dydt(460)+k(33)*y(448);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(448)=dydt(448)-k(30)*y(448);
dydt(193)=dydt(193)+k(30)*y(448);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(448)=dydt(448)-k(23)*y(448);
dydt(459)=dydt(459)+k(23)*y(448);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(448)=dydt(448)-k(22)*y(448);
dydt(182)=dydt(182)+k(22)*y(448);
dydt(458)=dydt(458)+k(22)*y(448);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(448)=dydt(448)-k(22)*y(448);
dydt(32)=dydt(32)+k(22)*y(448);
dydt(457)=dydt(457)+k(22)*y(448);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(448)=dydt(448)-k(10)*y(448);
dydt(3)=dydt(3)+k(10)*y(448);
dydt(447)=dydt(447)+k(10)*y(448);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

dydt(448)=dydt(448)-k(10)*y(448);
dydt(3)=dydt(3)+k(10)*y(448);
dydt(168)=dydt(168)+k(10)*y(448);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(448)=dydt(448)-k(8)*y(448);
dydt(131)=dydt(131)+k(8)*y(448);
dydt(185)=dydt(185)+k(8)*y(448);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(448)=dydt(448)-k(6)*y(448);
dydt(184)=dydt(184)+k(6)*y(448);
dydt(441)=dydt(441)+k(6)*y(448);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(456)=dydt(456)-k(34)*y(456);
dydt(160)=dydt(160)+k(34)*y(456);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(456)=dydt(456)-k(33)*y(456);
dydt(366)=dydt(366)+k(33)*y(456);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(456)=dydt(456)-k(33)*y(456);
dydt(453)=dydt(453)+k(33)*y(456);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(456)=dydt(456)-k(29)*y(456);
dydt(455)=dydt(455)+k(29)*y(456);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(456)=dydt(456)-k(22)*y(456);
dydt(197)=dydt(197)+k(22)*y(456);
dydt(156)=dydt(156)+k(22)*y(456);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(456)=dydt(456)-k(22)*y(456);
dydt(32)=dydt(32)+k(22)*y(456);
dydt(409)=dydt(409)+k(22)*y(456);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(456);
dydt(456)=dydt(456)-k(9)*y(3)*y(456);
dydt(163)=dydt(163)+k(9)*y(3)*y(456);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(456);
dydt(456)=dydt(456)-k(9)*y(3)*y(456);
dydt(412)=dydt(412)+k(9)*y(3)*y(456);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(456)=dydt(456)-k(8)*y(456);
dydt(192)=dydt(192)+k(8)*y(456);
dydt(28)=dydt(28)+k(8)*y(456);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(456)=dydt(456)-k(6)*y(456);
dydt(27)=dydt(27)+k(6)*y(456);
dydt(191)=dydt(191)+k(6)*y(456);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(455)=dydt(455)-k(34)*y(455);
dydt(454)=dydt(454)+k(34)*y(455);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(455)=dydt(455)-k(34)*y(455);
dydt(456)=dydt(456)+k(34)*y(455);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(455)=dydt(455)-k(33)*y(455);
dydt(369)=dydt(369)+k(33)*y(455);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(455)=dydt(455)-k(33)*y(455);
dydt(194)=dydt(194)+k(33)*y(455);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(455)=dydt(455)-k(22)*y(455);
dydt(197)=dydt(197)+k(22)*y(455);
dydt(376)=dydt(376)+k(22)*y(455);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(455)=dydt(455)-k(22)*y(455);
dydt(197)=dydt(197)+k(22)*y(455);
dydt(409)=dydt(409)+k(22)*y(455);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(455);
dydt(455)=dydt(455)-k(9)*y(3)*y(455);
dydt(370)=dydt(370)+k(9)*y(3)*y(455);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(455);
dydt(455)=dydt(455)-k(9)*y(3)*y(455);
dydt(403)=dydt(403)+k(9)*y(3)*y(455);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(455)=dydt(455)-k(8)*y(455);
dydt(192)=dydt(192)+k(8)*y(455);
dydt(209)=dydt(209)+k(8)*y(455);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(455)=dydt(455)-k(6)*y(455);
dydt(192)=dydt(192)+k(6)*y(455);
dydt(191)=dydt(191)+k(6)*y(455);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(454)=dydt(454)-k(34)*y(454);
dydt(160)=dydt(160)+k(34)*y(454);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(454)=dydt(454)-k(33)*y(454);
dydt(449)=dydt(449)+k(33)*y(454);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(454)=dydt(454)-k(33)*y(454);
dydt(373)=dydt(373)+k(33)*y(454);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(454)=dydt(454)-k(30)*y(454);
dydt(455)=dydt(455)+k(30)*y(454);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(454)=dydt(454)-k(22)*y(454);
dydt(197)=dydt(197)+k(22)*y(454);
dydt(450)=dydt(450)+k(22)*y(454);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(454)=dydt(454)-k(22)*y(454);
dydt(32)=dydt(32)+k(22)*y(454);
dydt(376)=dydt(376)+k(22)*y(454);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(454);
dydt(454)=dydt(454)-k(9)*y(3)*y(454);
dydt(402)=dydt(402)+k(9)*y(3)*y(454);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(454);
dydt(454)=dydt(454)-k(9)*y(3)*y(454);
dydt(162)=dydt(162)+k(9)*y(3)*y(454);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(454)=dydt(454)-k(8)*y(454);
dydt(27)=dydt(27)+k(8)*y(454);
dydt(209)=dydt(209)+k(8)*y(454);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(454)=dydt(454)-k(6)*y(454);
dydt(192)=dydt(192)+k(6)*y(454);
dydt(86)=dydt(86)+k(6)*y(454);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(450)=dydt(450)-k(33)*y(450);
dydt(375)=dydt(375)+k(33)*y(450);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(450)=dydt(450)-k(32)*y(450);
dydt(101)=dydt(101)+k(32)*y(450);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(450)=dydt(450)-k(22)*y(450);
dydt(32)=dydt(32)+k(22)*y(450);
dydt(150)=dydt(150)+k(22)*y(450);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(450);
dydt(450)=dydt(450)-k(21)*y(197)*y(450);
dydt(454)=dydt(454)+k(21)*y(197)*y(450);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(450);
dydt(450)=dydt(450)-k(21)*y(32)*y(450);
dydt(160)=dydt(160)+k(21)*y(32)*y(450);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(450);
dydt(450)=dydt(450)-k(21)*y(2)*y(450);
dydt(166)=dydt(166)+k(21)*y(2)*y(450);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(450);
dydt(450)=dydt(450)-k(9)*y(3)*y(450);
dydt(153)=dydt(153)+k(9)*y(3)*y(450);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(450);
dydt(450)=dydt(450)-k(9)*y(3)*y(450);
dydt(405)=dydt(405)+k(9)*y(3)*y(450);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(450)=dydt(450)-k(8)*y(450);
dydt(16)=dydt(16)+k(8)*y(450);
dydt(27)=dydt(27)+k(8)*y(450);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

dydt(450)=dydt(450)-k(6)*y(450);
dydt(15)=dydt(15)+k(6)*y(450);
dydt(86)=dydt(86)+k(6)*y(450);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(453)=dydt(453)-k(34)*y(453);
dydt(166)=dydt(166)+k(34)*y(453);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(453)=dydt(453)-k(33)*y(453);
dydt(179)=dydt(179)+k(33)*y(453);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(453)=dydt(453)-k(29)*y(453);
dydt(194)=dydt(194)+k(29)*y(453);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(453)=dydt(453)-k(22)*y(453);
dydt(197)=dydt(197)+k(22)*y(453);
dydt(452)=dydt(452)+k(22)*y(453);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(453)=dydt(453)-k(22)*y(453);
dydt(2)=dydt(2)+k(22)*y(453);
dydt(409)=dydt(409)+k(22)*y(453);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(453);
dydt(453)=dydt(453)-k(9)*y(3)*y(453);
dydt(169)=dydt(169)+k(9)*y(3)*y(453);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(453);
dydt(453)=dydt(453)-k(9)*y(3)*y(453);
dydt(411)=dydt(411)+k(9)*y(3)*y(453);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(453)=dydt(453)-k(8)*y(453);
dydt(192)=dydt(192)+k(8)*y(453);
dydt(22)=dydt(22)+k(8)*y(453);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(453)=dydt(453)-k(6)*y(453);
dydt(21)=dydt(21)+k(6)*y(453);
dydt(191)=dydt(191)+k(6)*y(453);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(452)=dydt(452)-k(32)*y(452);
dydt(24)=dydt(24)+k(32)*y(452);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(452)=dydt(452)-k(22)*y(452);
dydt(2)=dydt(2)+k(22)*y(452);
dydt(150)=dydt(150)+k(22)*y(452);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(452);
dydt(452)=dydt(452)-k(21)*y(197)*y(452);
dydt(453)=dydt(453)+k(21)*y(197)*y(452);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(452);
dydt(452)=dydt(452)-k(21)*y(32)*y(452);
dydt(166)=dydt(166)+k(21)*y(32)*y(452);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(452);
dydt(452)=dydt(452)-k(21)*y(2)*y(452);
dydt(172)=dydt(172)+k(21)*y(2)*y(452);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(452);
dydt(452)=dydt(452)-k(9)*y(3)*y(452);
dydt(145)=dydt(145)+k(9)*y(3)*y(452);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(452);
dydt(452)=dydt(452)-k(9)*y(3)*y(452);
dydt(388)=dydt(388)+k(9)*y(3)*y(452);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

dydt(452)=dydt(452)-k(8)*y(452);
dydt(15)=dydt(15)+k(8)*y(452);
dydt(22)=dydt(22)+k(8)*y(452);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(452)=dydt(452)-k(6)*y(452);
dydt(98)=dydt(98)+k(6)*y(452);
dydt(21)=dydt(21)+k(6)*y(452);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

dydt(451)=dydt(451)-k(34)*y(451);
dydt(452)=dydt(452)+k(34)*y(451);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(451)=dydt(451)-k(32)*y(451);
dydt(216)=dydt(216)+k(32)*y(451);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(451)=dydt(451)-k(22)*y(451);
dydt(182)=dydt(182)+k(22)*y(451);
dydt(150)=dydt(150)+k(22)*y(451);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(451);
dydt(451)=dydt(451)-k(21)*y(197)*y(451);
dydt(194)=dydt(194)+k(21)*y(197)*y(451);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(451);
dydt(451)=dydt(451)-k(21)*y(32)*y(451);
dydt(449)=dydt(449)+k(21)*y(32)*y(451);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(451);
dydt(451)=dydt(451)-k(21)*y(2)*y(451);
dydt(377)=dydt(377)+k(21)*y(2)*y(451);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(451);
dydt(451)=dydt(451)-k(9)*y(3)*y(451);
dydt(189)=dydt(189)+k(9)*y(3)*y(451);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(451);
dydt(451)=dydt(451)-k(9)*y(3)*y(451);
dydt(396)=dydt(396)+k(9)*y(3)*y(451);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

dydt(451)=dydt(451)-k(8)*y(451);
dydt(15)=dydt(15)+k(8)*y(451);
dydt(214)=dydt(214)+k(8)*y(451);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(451)=dydt(451)-k(6)*y(451);
dydt(98)=dydt(98)+k(6)*y(451);
dydt(177)=dydt(177)+k(6)*y(451);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

dydt(449)=dydt(449)-k(34)*y(449);
dydt(166)=dydt(166)+k(34)*y(449);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(449)=dydt(449)-k(33)*y(449);
dydt(377)=dydt(377)+k(33)*y(449);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

dydt(449)=dydt(449)-k(30)*y(449);
dydt(194)=dydt(194)+k(30)*y(449);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(449)=dydt(449)-k(22)*y(449);
dydt(32)=dydt(32)+k(22)*y(449);
dydt(451)=dydt(451)+k(22)*y(449);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(449)=dydt(449)-k(22)*y(449);
dydt(182)=dydt(182)+k(22)*y(449);
dydt(450)=dydt(450)+k(22)*y(449);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(449);
dydt(449)=dydt(449)-k(9)*y(3)*y(449);
dydt(168)=dydt(168)+k(9)*y(3)*y(449);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(449);
dydt(449)=dydt(449)-k(9)*y(3)*y(449);
dydt(447)=dydt(447)+k(9)*y(3)*y(449);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(449)=dydt(449)-k(8)*y(449);
dydt(27)=dydt(27)+k(8)*y(449);
dydt(214)=dydt(214)+k(8)*y(449);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(449)=dydt(449)-k(6)*y(449);
dydt(177)=dydt(177)+k(6)*y(449);
dydt(86)=dydt(86)+k(6)*y(449);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(447)=dydt(447)-k(34)*y(447);
dydt(445)=dydt(445)+k(34)*y(447);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(447)=dydt(447)-k(33)*y(447);
dydt(389)=dydt(389)+k(33)*y(447);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(447)=dydt(447)-k(30)*y(447);
dydt(435)=dydt(435)+k(30)*y(447);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(447)=dydt(447)-k(23)*y(447);
dydt(402)=dydt(402)+k(23)*y(447);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(447)=dydt(447)-k(22)*y(447);
dydt(32)=dydt(32)+k(22)*y(447);
dydt(396)=dydt(396)+k(22)*y(447);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(447)=dydt(447)-k(22)*y(447);
dydt(182)=dydt(182)+k(22)*y(447);
dydt(405)=dydt(405)+k(22)*y(447);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(447)=dydt(447)-k(11)*y(447);
dydt(432)=dydt(432)+k(11)*y(447);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(447)=dydt(447)-k(10)*y(447);
dydt(3)=dydt(3)+k(10)*y(447);
dydt(449)=dydt(449)+k(10)*y(447);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(447);
dydt(447)=dydt(447)-k(9)*y(3)*y(447);
dydt(448)=dydt(448)+k(9)*y(3)*y(447);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(447)=dydt(447)-k(8)*y(447);
dydt(214)=dydt(214)+k(8)*y(447);
dydt(131)=dydt(131)+k(8)*y(447);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(447)=dydt(447)-k(6)*y(447);
dydt(177)=dydt(177)+k(6)*y(447);
dydt(441)=dydt(441)+k(6)*y(447);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(445)=dydt(445)-k(33)*y(445);
dydt(386)=dydt(386)+k(33)*y(445);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(445)=dydt(445)-k(28)*y(445);
dydt(411)=dydt(411)+k(28)*y(445);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(445)=dydt(445)-k(27)*y(445);
dydt(447)=dydt(447)+k(27)*y(445);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(445)=dydt(445)-k(23)*y(445);
dydt(446)=dydt(446)+k(23)*y(445);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(445)=dydt(445)-k(22)*y(445);
dydt(32)=dydt(32)+k(22)*y(445);
dydt(388)=dydt(388)+k(22)*y(445);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(445)=dydt(445)-k(22)*y(445);
dydt(2)=dydt(2)+k(22)*y(445);
dydt(405)=dydt(405)+k(22)*y(445);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(445)=dydt(445)-k(11)*y(445);
dydt(440)=dydt(440)+k(11)*y(445);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(445)=dydt(445)-k(10)*y(445);
dydt(3)=dydt(3)+k(10)*y(445);
dydt(166)=dydt(166)+k(10)*y(445);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(445);
dydt(445)=dydt(445)-k(9)*y(3)*y(445);
dydt(165)=dydt(165)+k(9)*y(3)*y(445);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(445)=dydt(445)-k(8)*y(445);
dydt(22)=dydt(22)+k(8)*y(445);
dydt(131)=dydt(131)+k(8)*y(445);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(445)=dydt(445)-k(6)*y(445);
dydt(21)=dydt(21)+k(6)*y(445);
dydt(441)=dydt(441)+k(6)*y(445);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(440)=dydt(440)-k(33)*y(440);
dydt(387)=dydt(387)+k(33)*y(440);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(440)=dydt(440)-k(31)*y(440);
dydt(445)=dydt(445)+k(31)*y(440);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(440)=dydt(440)-k(28)*y(440);
dydt(436)=dydt(436)+k(28)*y(440);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(440)=dydt(440)-k(27)*y(440);
dydt(432)=dydt(432)+k(27)*y(440);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(440)=dydt(440)-k(23)*y(440);
dydt(444)=dydt(444)+k(23)*y(440);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(440)=dydt(440)-k(22)*y(440);
dydt(32)=dydt(32)+k(22)*y(440);
dydt(50)=dydt(50)+k(22)*y(440);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(440)=dydt(440)-k(22)*y(440);
dydt(2)=dydt(2)+k(22)*y(440);
dydt(443)=dydt(443)+k(22)*y(440);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(440);
dydt(440)=dydt(440)-k(15)*y(3)*y(440);
dydt(52)=dydt(52)+k(15)*y(3)*y(440);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(440)=dydt(440)-k(10)*y(440);
dydt(3)=dydt(3)+k(10)*y(440);
dydt(442)=dydt(442)+k(10)*y(440);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(440)=dydt(440)-k(8)*y(440);
dydt(437)=dydt(437)+k(8)*y(440);
dydt(131)=dydt(131)+k(8)*y(440);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(440)=dydt(440)-k(6)*y(440);
dydt(111)=dydt(111)+k(6)*y(440);
dydt(441)=dydt(441)+k(6)*y(440);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(436)=dydt(436)-k(34)*y(436);
dydt(440)=dydt(440)+k(34)*y(436);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(436)=dydt(436)-k(33)*y(436);
dydt(392)=dydt(392)+k(33)*y(436);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(436)=dydt(436)-k(31)*y(436);
dydt(411)=dydt(411)+k(31)*y(436);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(436)=dydt(436)-k(29)*y(436);
dydt(433)=dydt(433)+k(29)*y(436);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(436)=dydt(436)-k(23)*y(436);
dydt(439)=dydt(439)+k(23)*y(436);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(436)=dydt(436)-k(22)*y(436);
dydt(197)=dydt(197)+k(22)*y(436);
dydt(50)=dydt(50)+k(22)*y(436);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(436)=dydt(436)-k(22)*y(436);
dydt(2)=dydt(2)+k(22)*y(436);
dydt(410)=dydt(410)+k(22)*y(436);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(436);
dydt(436)=dydt(436)-k(15)*y(3)*y(436);
dydt(207)=dydt(207)+k(15)*y(3)*y(436);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(436)=dydt(436)-k(10)*y(436);
dydt(3)=dydt(3)+k(10)*y(436);
dydt(438)=dydt(438)+k(10)*y(436);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(436)=dydt(436)-k(8)*y(436);
dydt(437)=dydt(437)+k(8)*y(436);
dydt(335)=dydt(335)+k(8)*y(436);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(436)=dydt(436)-k(6)*y(436);
dydt(111)=dydt(111)+k(6)*y(436);
dydt(407)=dydt(407)+k(6)*y(436);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(433)=dydt(433)-k(34)*y(433);
dydt(432)=dydt(432)+k(34)*y(433);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(433)=dydt(433)-k(34)*y(433);
dydt(436)=dydt(436)+k(34)*y(433);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(433)=dydt(433)-k(33)*y(433);
dydt(395)=dydt(395)+k(33)*y(433);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(433)=dydt(433)-k(31)*y(433);
dydt(435)=dydt(435)+k(31)*y(433);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(433)=dydt(433)-k(23)*y(433);
dydt(434)=dydt(434)+k(23)*y(433);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(433)=dydt(433)-k(22)*y(433);
dydt(197)=dydt(197)+k(22)*y(433);
dydt(430)=dydt(430)+k(22)*y(433);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(433)=dydt(433)-k(22)*y(433);
dydt(182)=dydt(182)+k(22)*y(433);
dydt(410)=dydt(410)+k(22)*y(433);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(433);
dydt(433)=dydt(433)-k(15)*y(3)*y(433);
dydt(289)=dydt(289)+k(15)*y(3)*y(433);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(433)=dydt(433)-k(10)*y(433);
dydt(3)=dydt(3)+k(10)*y(433);
dydt(290)=dydt(290)+k(10)*y(433);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(433)=dydt(433)-k(8)*y(433);
dydt(219)=dydt(219)+k(8)*y(433);
dydt(335)=dydt(335)+k(8)*y(433);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(433)=dydt(433)-k(6)*y(433);
dydt(218)=dydt(218)+k(6)*y(433);
dydt(407)=dydt(407)+k(6)*y(433);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(430)=dydt(430)-k(34)*y(430);
dydt(50)=dydt(50)+k(34)*y(430);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(430)=dydt(430)-k(32)*y(430);
dydt(217)=dydt(217)+k(32)*y(430);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(430)=dydt(430)-k(31)*y(430);
dydt(396)=dydt(396)+k(31)*y(430);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(430)=dydt(430)-k(23)*y(430);
dydt(401)=dydt(401)+k(23)*y(430);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(430)=dydt(430)-k(22)*y(430);
dydt(182)=dydt(182)+k(22)*y(430);
dydt(57)=dydt(57)+k(22)*y(430);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(430);
dydt(430)=dydt(430)-k(21)*y(197)*y(430);
dydt(433)=dydt(433)+k(21)*y(197)*y(430);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(430);
dydt(430)=dydt(430)-k(21)*y(32)*y(430);
dydt(432)=dydt(432)+k(21)*y(32)*y(430);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(430);
dydt(430)=dydt(430)-k(21)*y(2)*y(430);
dydt(429)=dydt(429)+k(21)*y(2)*y(430);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(430);
dydt(430)=dydt(430)-k(15)*y(3)*y(430);
dydt(225)=dydt(225)+k(15)*y(3)*y(430);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(430)=dydt(430)-k(10)*y(430);
dydt(3)=dydt(3)+k(10)*y(430);
dydt(431)=dydt(431)+k(10)*y(430);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(430)=dydt(430)-k(8)*y(430);
dydt(48)=dydt(48)+k(8)*y(430);
dydt(219)=dydt(219)+k(8)*y(430);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(430)=dydt(430)-k(6)*y(430);
dydt(47)=dydt(47)+k(6)*y(430);
dydt(218)=dydt(218)+k(6)*y(430);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(429)=dydt(429)-k(34)*y(429);
dydt(387)=dydt(387)+k(34)*y(429);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(429)=dydt(429)-k(31)*y(429);
dydt(389)=dydt(389)+k(31)*y(429);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(429)=dydt(429)-k(30)*y(429);
dydt(395)=dydt(395)+k(30)*y(429);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(429)=dydt(429)-k(23)*y(429);
dydt(426)=dydt(426)+k(23)*y(429);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(429)=dydt(429)-k(22)*y(429);
dydt(2)=dydt(2)+k(22)*y(429);
dydt(430)=dydt(430)+k(22)*y(429);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(429)=dydt(429)-k(22)*y(429);
dydt(182)=dydt(182)+k(22)*y(429);
dydt(424)=dydt(424)+k(22)*y(429);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(429);
dydt(429)=dydt(429)-k(15)*y(3)*y(429);
dydt(279)=dydt(279)+k(15)*y(3)*y(429);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(429)=dydt(429)-k(10)*y(429);
dydt(3)=dydt(3)+k(10)*y(429);
dydt(280)=dydt(280)+k(10)*y(429);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(429)=dydt(429)-k(8)*y(429);
dydt(219)=dydt(219)+k(8)*y(429);
dydt(140)=dydt(140)+k(8)*y(429);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(429)=dydt(429)-k(6)*y(429);
dydt(218)=dydt(218)+k(6)*y(429);
dydt(380)=dydt(380)+k(6)*y(429);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(426)=dydt(426)-k(34)*y(426);
dydt(382)=dydt(382)+k(34)*y(426);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(426)=dydt(426)-k(33)*y(426);
dydt(429)=dydt(429)+k(33)*y(426);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(426)=dydt(426)-k(31)*y(426);
dydt(374)=dydt(374)+k(31)*y(426);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(426)=dydt(426)-k(30)*y(426);
dydt(398)=dydt(398)+k(30)*y(426);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(426)=dydt(426)-k(22)*y(426);
dydt(197)=dydt(197)+k(22)*y(426);
dydt(424)=dydt(424)+k(22)*y(426);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(426)=dydt(426)-k(22)*y(426);
dydt(2)=dydt(2)+k(22)*y(426);
dydt(401)=dydt(401)+k(22)*y(426);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(426);
dydt(426)=dydt(426)-k(15)*y(3)*y(426);
dydt(428)=dydt(428)+k(15)*y(3)*y(426);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(426)=dydt(426)-k(10)*y(426);
dydt(3)=dydt(3)+k(10)*y(426);
dydt(427)=dydt(427)+k(10)*y(426);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(426)=dydt(426)-k(8)*y(426);
dydt(319)=dydt(319)+k(8)*y(426);
dydt(140)=dydt(140)+k(8)*y(426);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(426)=dydt(426)-k(6)*y(426);
dydt(257)=dydt(257)+k(6)*y(426);
dydt(380)=dydt(380)+k(6)*y(426);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(424)=dydt(424)-k(32)*y(424);
dydt(92)=dydt(92)+k(32)*y(424);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(424)=dydt(424)-k(31)*y(424);
dydt(383)=dydt(383)+k(31)*y(424);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(424)=dydt(424)-k(22)*y(424);
dydt(2)=dydt(2)+k(22)*y(424);
dydt(57)=dydt(57)+k(22)*y(424);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(424);
dydt(424)=dydt(424)-k(21)*y(197)*y(424);
dydt(426)=dydt(426)+k(21)*y(197)*y(424);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(424);
dydt(424)=dydt(424)-k(21)*y(32)*y(424);
dydt(382)=dydt(382)+k(21)*y(32)*y(424);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(424);
dydt(424)=dydt(424)-k(21)*y(2)*y(424);
dydt(387)=dydt(387)+k(21)*y(2)*y(424);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(424);
dydt(424)=dydt(424)-k(15)*y(3)*y(424);
dydt(58)=dydt(58)+k(15)*y(3)*y(424);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(424)=dydt(424)-k(10)*y(424);
dydt(3)=dydt(3)+k(10)*y(424);
dydt(425)=dydt(425)+k(10)*y(424);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(424)=dydt(424)-k(8)*y(424);
dydt(315)=dydt(315)+k(8)*y(424);
dydt(140)=dydt(140)+k(8)*y(424);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

dydt(424)=dydt(424)-k(6)*y(424);
dydt(77)=dydt(77)+k(6)*y(424);
dydt(380)=dydt(380)+k(6)*y(424);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(420)=dydt(420)-k(34)*y(420);
dydt(424)=dydt(424)+k(34)*y(420);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(420)=dydt(420)-k(32)*y(420);
dydt(419)=dydt(419)+k(32)*y(420);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(420)=dydt(420)-k(31)*y(420);
dydt(393)=dydt(393)+k(31)*y(420);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(420)=dydt(420)-k(22)*y(420);
dydt(182)=dydt(182)+k(22)*y(420);
dydt(57)=dydt(57)+k(22)*y(420);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(420);
dydt(420)=dydt(420)-k(21)*y(197)*y(420);
dydt(398)=dydt(398)+k(21)*y(197)*y(420);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(420);
dydt(420)=dydt(420)-k(21)*y(32)*y(420);
dydt(423)=dydt(423)+k(21)*y(32)*y(420);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(420);
dydt(420)=dydt(420)-k(21)*y(2)*y(420);
dydt(392)=dydt(392)+k(21)*y(2)*y(420);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(420);
dydt(420)=dydt(420)-k(15)*y(3)*y(420);
dydt(422)=dydt(422)+k(15)*y(3)*y(420);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(420)=dydt(420)-k(10)*y(420);
dydt(3)=dydt(3)+k(10)*y(420);
dydt(421)=dydt(421)+k(10)*y(420);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(420)=dydt(420)-k(8)*y(420);
dydt(315)=dydt(315)+k(8)*y(420);
dydt(184)=dydt(184)+k(8)*y(420);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

dydt(420)=dydt(420)-k(6)*y(420);
dydt(77)=dydt(77)+k(6)*y(420);
dydt(391)=dydt(391)+k(6)*y(420);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(419)=dydt(419)-k(34)*y(419);
dydt(92)=dydt(92)+k(34)*y(419);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(419)=dydt(419)-k(31)*y(419);
dydt(416)=dydt(416)+k(31)*y(419);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(419)=dydt(419)-k(22)*y(419);
dydt(182)=dydt(182)+k(22)*y(419);
dydt(64)=dydt(64)+k(22)*y(419);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(419);
dydt(419)=dydt(419)-k(15)*y(3)*y(419);
dydt(298)=dydt(298)+k(15)*y(3)*y(419);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(419)=dydt(419)-k(13)*y(419);
dydt(420)=dydt(420)+k(13)*y(419);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(419)=dydt(419)-k(10)*y(419);
dydt(3)=dydt(3)+k(10)*y(419);
dydt(299)=dydt(299)+k(10)*y(419);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(419)=dydt(419)-k(8)*y(419);
dydt(91)=dydt(91)+k(8)*y(419);
dydt(184)=dydt(184)+k(8)*y(419);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

dydt(419)=dydt(419)-k(6)*y(419);
dydt(72)=dydt(72)+k(6)*y(419);
dydt(391)=dydt(391)+k(6)*y(419);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(416)=dydt(416)-k(34)*y(416);
dydt(95)=dydt(95)+k(34)*y(416);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(416)=dydt(416)-k(22)*y(416);
dydt(182)=dydt(182)+k(22)*y(416);
dydt(99)=dydt(99)+k(22)*y(416);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(416)=dydt(416)-k(13)*y(416);
dydt(393)=dydt(393)+k(13)*y(416);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(416)=dydt(416)-k(11)*y(416);
dydt(419)=dydt(419)+k(11)*y(416);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(416)=dydt(416)-k(10)*y(416);
dydt(3)=dydt(3)+k(10)*y(416);
dydt(418)=dydt(418)+k(10)*y(416);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(416);
dydt(416)=dydt(416)-k(9)*y(3)*y(416);
dydt(417)=dydt(417)+k(9)*y(3)*y(416);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(416)=dydt(416)-k(8)*y(416);
dydt(5)=dydt(5)+k(8)*y(416);
dydt(184)=dydt(184)+k(8)*y(416);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

dydt(416)=dydt(416)-k(6)*y(416);
dydt(1)=dydt(1)+k(6)*y(416);
dydt(391)=dydt(391)+k(6)*y(416);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(413)=dydt(413)-k(34)*y(413);
dydt(102)=dydt(102)+k(34)*y(413);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(413)=dydt(413)-k(33)*y(413);
dydt(416)=dydt(416)+k(33)*y(413);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(413)=dydt(413)-k(22)*y(413);
dydt(197)=dydt(197)+k(22)*y(413);
dydt(99)=dydt(99)+k(22)*y(413);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(413)=dydt(413)-k(13)*y(413);
dydt(406)=dydt(406)+k(13)*y(413);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(413)=dydt(413)-k(11)*y(413);
dydt(415)=dydt(415)+k(11)*y(413);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(413)=dydt(413)-k(10)*y(413);
dydt(3)=dydt(3)+k(10)*y(413);
dydt(202)=dydt(202)+k(10)*y(413);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(413);
dydt(413)=dydt(413)-k(9)*y(3)*y(413);
dydt(414)=dydt(414)+k(9)*y(3)*y(413);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(413)=dydt(413)-k(8)*y(413);
dydt(5)=dydt(5)+k(8)*y(413);
dydt(335)=dydt(335)+k(8)*y(413);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

dydt(413)=dydt(413)-k(6)*y(413);
dydt(1)=dydt(1)+k(6)*y(413);
dydt(407)=dydt(407)+k(6)*y(413);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(406)=dydt(406)-k(34)*y(406);
dydt(405)=dydt(405)+k(34)*y(406);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(406)=dydt(406)-k(33)*y(406);
dydt(393)=dydt(393)+k(33)*y(406);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(406)=dydt(406)-k(32)*y(406);
dydt(413)=dydt(413)+k(32)*y(406);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(406)=dydt(406)-k(22)*y(406);
dydt(197)=dydt(197)+k(22)*y(406);
dydt(404)=dydt(404)+k(22)*y(406);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(406);
dydt(406)=dydt(406)-k(21)*y(197)*y(406);
dydt(403)=dydt(403)+k(21)*y(197)*y(406);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(406);
dydt(406)=dydt(406)-k(21)*y(32)*y(406);
dydt(412)=dydt(412)+k(21)*y(32)*y(406);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(406);
dydt(406)=dydt(406)-k(21)*y(2)*y(406);
dydt(411)=dydt(411)+k(21)*y(2)*y(406);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(406)=dydt(406)-k(11)*y(406);
dydt(410)=dydt(410)+k(11)*y(406);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(406)=dydt(406)-k(10)*y(406);
dydt(3)=dydt(3)+k(10)*y(406);
dydt(409)=dydt(409)+k(10)*y(406);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(406);
dydt(406)=dydt(406)-k(9)*y(3)*y(406);
dydt(408)=dydt(408)+k(9)*y(3)*y(406);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(406)=dydt(406)-k(8)*y(406);
dydt(16)=dydt(16)+k(8)*y(406);
dydt(335)=dydt(335)+k(8)*y(406);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

dydt(406)=dydt(406)-k(6)*y(406);
dydt(15)=dydt(15)+k(6)*y(406);
dydt(407)=dydt(407)+k(6)*y(406);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(404)=dydt(404)-k(32)*y(404);
dydt(14)=dydt(14)+k(32)*y(404);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(404)=dydt(404)-k(32)*y(404);
dydt(99)=dydt(99)+k(32)*y(404);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(404);
dydt(404)=dydt(404)-k(21)*y(197)*y(404);
dydt(399)=dydt(399)+k(21)*y(197)*y(404);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(404);
dydt(404)=dydt(404)-k(21)*y(32)*y(404);
dydt(384)=dydt(384)+k(21)*y(32)*y(404);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(404);
dydt(404)=dydt(404)-k(21)*y(2)*y(404);
dydt(388)=dydt(388)+k(21)*y(2)*y(404);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(404);
dydt(404)=dydt(404)-k(21)*y(197)*y(404);
dydt(406)=dydt(406)+k(21)*y(197)*y(404);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(404);
dydt(404)=dydt(404)-k(21)*y(32)*y(404);
dydt(405)=dydt(405)+k(21)*y(32)*y(404);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(404);
dydt(404)=dydt(404)-k(21)*y(2)*y(404);
dydt(383)=dydt(383)+k(21)*y(2)*y(404);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(404)=dydt(404)-k(11)*y(404);
dydt(57)=dydt(57)+k(11)*y(404);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

dydt(404)=dydt(404)-k(10)*y(404);
dydt(3)=dydt(3)+k(10)*y(404);
dydt(150)=dydt(150)+k(10)*y(404);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(404);
dydt(404)=dydt(404)-k(9)*y(3)*y(404);
dydt(149)=dydt(149)+k(9)*y(3)*y(404);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(404)=dydt(404)-k(8)*y(404);
dydt(16)=dydt(16)+k(8)*y(404);
dydt(48)=dydt(48)+k(8)*y(404);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) 

dydt(404)=dydt(404)-k(6)*y(404);
dydt(15)=dydt(15)+k(6)*y(404);
dydt(47)=dydt(47)+k(6)*y(404);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(399)=dydt(399)-k(34)*y(399);
dydt(384)=dydt(384)+k(34)*y(399);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(399)=dydt(399)-k(33)*y(399);
dydt(396)=dydt(396)+k(33)*y(399);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(399)=dydt(399)-k(32)*y(399);
dydt(208)=dydt(208)+k(32)*y(399);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(399)=dydt(399)-k(22)*y(399);
dydt(197)=dydt(197)+k(22)*y(399);
dydt(404)=dydt(404)+k(22)*y(399);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(399);
dydt(399)=dydt(399)-k(21)*y(197)*y(399);
dydt(403)=dydt(403)+k(21)*y(197)*y(399);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(399);
dydt(399)=dydt(399)-k(21)*y(32)*y(399);
dydt(402)=dydt(402)+k(21)*y(32)*y(399);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(399);
dydt(399)=dydt(399)-k(21)*y(2)*y(399);
dydt(374)=dydt(374)+k(21)*y(2)*y(399);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(399)=dydt(399)-k(11)*y(399);
dydt(401)=dydt(401)+k(11)*y(399);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(399)=dydt(399)-k(10)*y(399);
dydt(3)=dydt(3)+k(10)*y(399);
dydt(376)=dydt(376)+k(10)*y(399);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(399);
dydt(399)=dydt(399)-k(9)*y(3)*y(399);
dydt(400)=dydt(400)+k(9)*y(3)*y(399);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(399)=dydt(399)-k(8)*y(399);
dydt(48)=dydt(48)+k(8)*y(399);
dydt(209)=dydt(209)+k(8)*y(399);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(399)=dydt(399)-k(6)*y(399);
dydt(47)=dydt(47)+k(6)*y(399);
dydt(192)=dydt(192)+k(6)*y(399);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(397)=dydt(397)-k(34)*y(397);
dydt(374)=dydt(374)+k(34)*y(397);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(397)=dydt(397)-k(34)*y(397);
dydt(385)=dydt(385)+k(34)*y(397);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(397)=dydt(397)-k(33)*y(397);
dydt(394)=dydt(394)+k(33)*y(397);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(397)=dydt(397)-k(22)*y(397);
dydt(182)=dydt(182)+k(22)*y(397);
dydt(399)=dydt(399)+k(22)*y(397);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(397)=dydt(397)-k(22)*y(397);
dydt(197)=dydt(197)+k(22)*y(397);
dydt(393)=dydt(393)+k(22)*y(397);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(397)=dydt(397)-k(11)*y(397);
dydt(398)=dydt(398)+k(11)*y(397);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(397)=dydt(397)-k(10)*y(397);
dydt(3)=dydt(3)+k(10)*y(397);
dydt(369)=dydt(369)+k(10)*y(397);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(397);
dydt(397)=dydt(397)-k(9)*y(3)*y(397);
dydt(368)=dydt(368)+k(9)*y(3)*y(397);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

dydt(397)=dydt(397)-k(8)*y(397);
dydt(209)=dydt(209)+k(8)*y(397);
dydt(184)=dydt(184)+k(8)*y(397);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(397)=dydt(397)-k(6)*y(397);
dydt(192)=dydt(192)+k(6)*y(397);
dydt(391)=dydt(391)+k(6)*y(397);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(394)=dydt(394)-k(34)*y(394);
dydt(389)=dydt(389)+k(34)*y(394);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(394)=dydt(394)-k(34)*y(394);
dydt(390)=dydt(390)+k(34)*y(394);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(394)=dydt(394)-k(23)*y(394);
dydt(397)=dydt(397)+k(23)*y(394);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(394)=dydt(394)-k(22)*y(394);
dydt(182)=dydt(182)+k(22)*y(394);
dydt(396)=dydt(396)+k(22)*y(394);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(394)=dydt(394)-k(22)*y(394);
dydt(182)=dydt(182)+k(22)*y(394);
dydt(393)=dydt(393)+k(22)*y(394);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(394)=dydt(394)-k(11)*y(394);
dydt(395)=dydt(395)+k(11)*y(394);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(394)=dydt(394)-k(10)*y(394);
dydt(3)=dydt(3)+k(10)*y(394);
dydt(187)=dydt(187)+k(10)*y(394);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(394);
dydt(394)=dydt(394)-k(9)*y(3)*y(394);
dydt(186)=dydt(186)+k(9)*y(3)*y(394);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

dydt(394)=dydt(394)-k(8)*y(394);
dydt(214)=dydt(214)+k(8)*y(394);
dydt(184)=dydt(184)+k(8)*y(394);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(394)=dydt(394)-k(6)*y(394);
dydt(177)=dydt(177)+k(6)*y(394);
dydt(391)=dydt(391)+k(6)*y(394);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(390)=dydt(390)-k(34)*y(390);
dydt(386)=dydt(386)+k(34)*y(390);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(390)=dydt(390)-k(29)*y(390);
dydt(394)=dydt(394)+k(29)*y(390);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(390)=dydt(390)-k(23)*y(390);
dydt(385)=dydt(385)+k(23)*y(390);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(390)=dydt(390)-k(22)*y(390);
dydt(182)=dydt(182)+k(22)*y(390);
dydt(388)=dydt(388)+k(22)*y(390);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(390)=dydt(390)-k(22)*y(390);
dydt(2)=dydt(2)+k(22)*y(390);
dydt(393)=dydt(393)+k(22)*y(390);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(390)=dydt(390)-k(11)*y(390);
dydt(392)=dydt(392)+k(11)*y(390);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(390)=dydt(390)-k(10)*y(390);
dydt(3)=dydt(3)+k(10)*y(390);
dydt(179)=dydt(179)+k(10)*y(390);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(390);
dydt(390)=dydt(390)-k(9)*y(3)*y(390);
dydt(178)=dydt(178)+k(9)*y(3)*y(390);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(390)=dydt(390)-k(8)*y(390);
dydt(22)=dydt(22)+k(8)*y(390);
dydt(184)=dydt(184)+k(8)*y(390);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(390)=dydt(390)-k(6)*y(390);
dydt(21)=dydt(21)+k(6)*y(390);
dydt(391)=dydt(391)+k(6)*y(390);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(386)=dydt(386)-k(28)*y(386);
dydt(390)=dydt(390)+k(28)*y(386);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(386)=dydt(386)-k(27)*y(386);
dydt(389)=dydt(389)+k(27)*y(386);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(386)=dydt(386)-k(23)*y(386);
dydt(379)=dydt(379)+k(23)*y(386);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(386)=dydt(386)-k(22)*y(386);
dydt(2)=dydt(2)+k(22)*y(386);
dydt(388)=dydt(388)+k(22)*y(386);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(386)=dydt(386)-k(22)*y(386);
dydt(2)=dydt(2)+k(22)*y(386);
dydt(383)=dydt(383)+k(22)*y(386);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(386)=dydt(386)-k(11)*y(386);
dydt(387)=dydt(387)+k(11)*y(386);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(386)=dydt(386)-k(10)*y(386);
dydt(3)=dydt(3)+k(10)*y(386);
dydt(172)=dydt(172)+k(10)*y(386);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(386);
dydt(386)=dydt(386)-k(9)*y(3)*y(386);
dydt(171)=dydt(171)+k(9)*y(3)*y(386);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

dydt(386)=dydt(386)-k(8)*y(386);
dydt(22)=dydt(22)+k(8)*y(386);
dydt(140)=dydt(140)+k(8)*y(386);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(386)=dydt(386)-k(6)*y(386);
dydt(21)=dydt(21)+k(6)*y(386);
dydt(380)=dydt(380)+k(6)*y(386);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(379)=dydt(379)-k(33)*y(379);
dydt(386)=dydt(386)+k(33)*y(379);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(379)=dydt(379)-k(28)*y(379);
dydt(385)=dydt(385)+k(28)*y(379);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(379)=dydt(379)-k(27)*y(379);
dydt(374)=dydt(374)+k(27)*y(379);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(379)=dydt(379)-k(22)*y(379);
dydt(2)=dydt(2)+k(22)*y(379);
dydt(384)=dydt(384)+k(22)*y(379);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(379)=dydt(379)-k(22)*y(379);
dydt(32)=dydt(32)+k(22)*y(379);
dydt(383)=dydt(383)+k(22)*y(379);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(379)=dydt(379)-k(11)*y(379);
dydt(382)=dydt(382)+k(11)*y(379);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(379)=dydt(379)-k(10)*y(379);
dydt(3)=dydt(3)+k(10)*y(379);
dydt(378)=dydt(378)+k(10)*y(379);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(379);
dydt(379)=dydt(379)-k(9)*y(3)*y(379);
dydt(381)=dydt(381)+k(9)*y(3)*y(379);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

dydt(379)=dydt(379)-k(8)*y(379);
dydt(28)=dydt(28)+k(8)*y(379);
dydt(140)=dydt(140)+k(8)*y(379);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(379)=dydt(379)-k(6)*y(379);
dydt(27)=dydt(27)+k(6)*y(379);
dydt(380)=dydt(380)+k(6)*y(379);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(378)=dydt(378)-k(33)*y(378);
dydt(172)=dydt(172)+k(33)*y(378);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

dydt(378)=dydt(378)-k(28)*y(378);
dydt(366)=dydt(366)+k(28)*y(378);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(378)=dydt(378)-k(27)*y(378);
dydt(373)=dydt(373)+k(27)*y(378);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(378)=dydt(378)-k(22)*y(378);
dydt(2)=dydt(2)+k(22)*y(378);
dydt(156)=dydt(156)+k(22)*y(378);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(378)=dydt(378)-k(22)*y(378);
dydt(32)=dydt(32)+k(22)*y(378);
dydt(375)=dydt(375)+k(22)*y(378);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(378);
dydt(378)=dydt(378)-k(9)*y(3)*y(378);
dydt(157)=dydt(157)+k(9)*y(3)*y(378);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(378);
dydt(378)=dydt(378)-k(9)*y(3)*y(378);
dydt(379)=dydt(379)+k(9)*y(3)*y(378);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(378)=dydt(378)-k(8)*y(378);
dydt(21)=dydt(21)+k(8)*y(378);
dydt(28)=dydt(28)+k(8)*y(378);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(378)=dydt(378)-k(6)*y(378);
dydt(27)=dydt(27)+k(6)*y(378);
dydt(89)=dydt(89)+k(6)*y(378);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

dydt(373)=dydt(373)-k(34)*y(373);
dydt(378)=dydt(378)+k(34)*y(373);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

dydt(373)=dydt(373)-k(33)*y(373);
dydt(377)=dydt(377)+k(33)*y(373);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

dydt(373)=dydt(373)-k(30)*y(373);
dydt(369)=dydt(369)+k(30)*y(373);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(373)=dydt(373)-k(22)*y(373);
dydt(2)=dydt(2)+k(22)*y(373);
dydt(376)=dydt(376)+k(22)*y(373);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(373)=dydt(373)-k(22)*y(373);
dydt(197)=dydt(197)+k(22)*y(373);
dydt(375)=dydt(375)+k(22)*y(373);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(373);
dydt(373)=dydt(373)-k(9)*y(3)*y(373);
dydt(371)=dydt(371)+k(9)*y(3)*y(373);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(373);
dydt(373)=dydt(373)-k(9)*y(3)*y(373);
dydt(374)=dydt(374)+k(9)*y(3)*y(373);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(373)=dydt(373)-k(8)*y(373);
dydt(21)=dydt(21)+k(8)*y(373);
dydt(209)=dydt(209)+k(8)*y(373);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(373)=dydt(373)-k(6)*y(373);
dydt(192)=dydt(192)+k(6)*y(373);
dydt(89)=dydt(89)+k(6)*y(373);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(371)=dydt(371)-k(34)*y(371);
dydt(157)=dydt(157)+k(34)*y(371);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(371)=dydt(371)-k(33)*y(371);
dydt(174)=dydt(174)+k(33)*y(371);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(371)=dydt(371)-k(30)*y(371);
dydt(367)=dydt(367)+k(30)*y(371);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(371)=dydt(371)-k(24)*y(371);
dydt(162)=dydt(162)+k(24)*y(371);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(371)=dydt(371)-k(22)*y(371);
dydt(2)=dydt(2)+k(22)*y(371);
dydt(198)=dydt(198)+k(22)*y(371);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(371)=dydt(371)-k(22)*y(371);
dydt(197)=dydt(197)+k(22)*y(371);
dydt(152)=dydt(152)+k(22)*y(371);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(371)=dydt(371)-k(12)*y(371);
dydt(340)=dydt(340)+k(12)*y(371);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(371)=dydt(371)-k(10)*y(371);
dydt(3)=dydt(3)+k(10)*y(371);
dydt(373)=dydt(373)+k(10)*y(371);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(371);
dydt(371)=dydt(371)-k(9)*y(3)*y(371);
dydt(372)=dydt(372)+k(9)*y(3)*y(371);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(371)=dydt(371)-k(8)*y(371);
dydt(21)=dydt(21)+k(8)*y(371);
dydt(336)=dydt(336)+k(8)*y(371);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(371)=dydt(371)-k(6)*y(371);
dydt(89)=dydt(89)+k(6)*y(371);
dydt(335)=dydt(335)+k(6)*y(371);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(367)=dydt(367)-k(34)*y(367);
dydt(371)=dydt(371)+k(34)*y(367);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(367)=dydt(367)-k(34)*y(367);
dydt(364)=dydt(364)+k(34)*y(367);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(367)=dydt(367)-k(33)*y(367);
dydt(183)=dydt(183)+k(33)*y(367);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(367)=dydt(367)-k(24)*y(367);
dydt(370)=dydt(370)+k(24)*y(367);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(367)=dydt(367)-k(22)*y(367);
dydt(182)=dydt(182)+k(22)*y(367);
dydt(198)=dydt(198)+k(22)*y(367);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(367)=dydt(367)-k(22)*y(367);
dydt(197)=dydt(197)+k(22)*y(367);
dydt(181)=dydt(181)+k(22)*y(367);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(367)=dydt(367)-k(12)*y(367);
dydt(363)=dydt(363)+k(12)*y(367);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(367)=dydt(367)-k(10)*y(367);
dydt(3)=dydt(3)+k(10)*y(367);
dydt(369)=dydt(369)+k(10)*y(367);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(367);
dydt(367)=dydt(367)-k(9)*y(3)*y(367);
dydt(368)=dydt(368)+k(9)*y(3)*y(367);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(367)=dydt(367)-k(8)*y(367);
dydt(177)=dydt(177)+k(8)*y(367);
dydt(336)=dydt(336)+k(8)*y(367);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(367)=dydt(367)-k(6)*y(367);
dydt(176)=dydt(176)+k(6)*y(367);
dydt(335)=dydt(335)+k(6)*y(367);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(364)=dydt(364)-k(34)*y(364);
dydt(157)=dydt(157)+k(34)*y(364);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(364)=dydt(364)-k(33)*y(364);
dydt(175)=dydt(175)+k(33)*y(364);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(364)=dydt(364)-k(29)*y(364);
dydt(367)=dydt(367)+k(29)*y(364);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(364)=dydt(364)-k(24)*y(364);
dydt(163)=dydt(163)+k(24)*y(364);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(364)=dydt(364)-k(22)*y(364);
dydt(182)=dydt(182)+k(22)*y(364);
dydt(154)=dydt(154)+k(22)*y(364);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(364)=dydt(364)-k(22)*y(364);
dydt(32)=dydt(32)+k(22)*y(364);
dydt(181)=dydt(181)+k(22)*y(364);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(364)=dydt(364)-k(12)*y(364);
dydt(358)=dydt(358)+k(12)*y(364);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(364)=dydt(364)-k(10)*y(364);
dydt(3)=dydt(3)+k(10)*y(364);
dydt(366)=dydt(366)+k(10)*y(364);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(364);
dydt(364)=dydt(364)-k(9)*y(3)*y(364);
dydt(365)=dydt(365)+k(9)*y(3)*y(364);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(364)=dydt(364)-k(8)*y(364);
dydt(177)=dydt(177)+k(8)*y(364);
dydt(132)=dydt(132)+k(8)*y(364);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(364)=dydt(364)-k(6)*y(364);
dydt(176)=dydt(176)+k(6)*y(364);
dydt(131)=dydt(131)+k(6)*y(364);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(358)=dydt(358)-k(34)*y(358);
dydt(355)=dydt(355)+k(34)*y(358);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(358)=dydt(358)-k(33)*y(358);
dydt(180)=dydt(180)+k(33)*y(358);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(358)=dydt(358)-k(31)*y(358);
dydt(364)=dydt(364)+k(31)*y(358);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(358)=dydt(358)-k(29)*y(358);
dydt(363)=dydt(363)+k(29)*y(358);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(358)=dydt(358)-k(24)*y(358);
dydt(362)=dydt(362)+k(24)*y(358);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(358)=dydt(358)-k(22)*y(358);
dydt(182)=dydt(182)+k(22)*y(358);
dydt(137)=dydt(137)+k(22)*y(358);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(358)=dydt(358)-k(22)*y(358);
dydt(32)=dydt(32)+k(22)*y(358);
dydt(361)=dydt(361)+k(22)*y(358);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(358);
dydt(358)=dydt(358)-k(15)*y(3)*y(358);
dydt(360)=dydt(360)+k(15)*y(3)*y(358);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(358)=dydt(358)-k(10)*y(358);
dydt(3)=dydt(3)+k(10)*y(358);
dydt(359)=dydt(359)+k(10)*y(358);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(358)=dydt(358)-k(8)*y(358);
dydt(218)=dydt(218)+k(8)*y(358);
dydt(132)=dydt(132)+k(8)*y(358);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(358)=dydt(358)-k(6)*y(358);
dydt(264)=dydt(264)+k(6)*y(358);
dydt(131)=dydt(131)+k(6)*y(358);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(355)=dydt(355)-k(33)*y(355);
dydt(173)=dydt(173)+k(33)*y(355);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(355)=dydt(355)-k(31)*y(355);
dydt(157)=dydt(157)+k(31)*y(355);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(355)=dydt(355)-k(28)*y(355);
dydt(358)=dydt(358)+k(28)*y(355);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(355)=dydt(355)-k(27)*y(355);
dydt(340)=dydt(340)+k(27)*y(355);

% rule    : R10_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(355)=dydt(355)-k(24)*y(355);
dydt(161)=dydt(161)+k(24)*y(355);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(355)=dydt(355)-k(22)*y(355);
dydt(32)=dydt(32)+k(22)*y(355);
dydt(353)=dydt(353)+k(22)*y(355);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(355)=dydt(355)-k(22)*y(355);
dydt(2)=dydt(2)+k(22)*y(355);
dydt(137)=dydt(137)+k(22)*y(355);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(355);
dydt(355)=dydt(355)-k(15)*y(3)*y(355);
dydt(357)=dydt(357)+k(15)*y(3)*y(355);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(355)=dydt(355)-k(10)*y(355);
dydt(3)=dydt(3)+k(10)*y(355);
dydt(356)=dydt(356)+k(10)*y(355);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(355)=dydt(355)-k(8)*y(355);
dydt(111)=dydt(111)+k(8)*y(355);
dydt(132)=dydt(132)+k(8)*y(355);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(355)=dydt(355)-k(6)*y(355);
dydt(110)=dydt(110)+k(6)*y(355);
dydt(131)=dydt(131)+k(6)*y(355);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(353)=dydt(353)-k(32)*y(353);
dydt(109)=dydt(109)+k(32)*y(353);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(353)=dydt(353)-k(31)*y(353);
dydt(152)=dydt(152)+k(31)*y(353);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(353)=dydt(353)-k(24)*y(353);
dydt(350)=dydt(350)+k(24)*y(353);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(353)=dydt(353)-k(22)*y(353);
dydt(2)=dydt(2)+k(22)*y(353);
dydt(151)=dydt(151)+k(22)*y(353);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(353);
dydt(353)=dydt(353)-k(21)*y(197)*y(353);
dydt(340)=dydt(340)+k(21)*y(197)*y(353);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(353);
dydt(353)=dydt(353)-k(21)*y(32)*y(353);
dydt(355)=dydt(355)+k(21)*y(32)*y(353);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(353);
dydt(353)=dydt(353)-k(21)*y(2)*y(353);
dydt(173)=dydt(173)+k(21)*y(2)*y(353);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(353);
dydt(353)=dydt(353)-k(15)*y(3)*y(353);
dydt(116)=dydt(116)+k(15)*y(3)*y(353);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(353)=dydt(353)-k(10)*y(353);
dydt(3)=dydt(3)+k(10)*y(353);
dydt(354)=dydt(354)+k(10)*y(353);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(353)=dydt(353)-k(8)*y(353);
dydt(126)=dydt(126)+k(8)*y(353);
dydt(111)=dydt(111)+k(8)*y(353);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(353)=dydt(353)-k(6)*y(353);
dydt(48)=dydt(48)+k(6)*y(353);
dydt(110)=dydt(110)+k(6)*y(353);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(350)=dydt(350)-k(33)*y(350);
dydt(353)=dydt(353)+k(33)*y(350);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(350)=dydt(350)-k(32)*y(350);
dydt(105)=dydt(105)+k(32)*y(350);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(350)=dydt(350)-k(31)*y(350);
dydt(153)=dydt(153)+k(31)*y(350);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(350)=dydt(350)-k(22)*y(350);
dydt(32)=dydt(32)+k(22)*y(350);
dydt(151)=dydt(151)+k(22)*y(350);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(197)=dydt(197)-k(21)*y(197)*y(350);
dydt(350)=dydt(350)-k(21)*y(197)*y(350);
dydt(341)=dydt(341)+k(21)*y(197)*y(350);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(350);
dydt(350)=dydt(350)-k(21)*y(32)*y(350);
dydt(161)=dydt(161)+k(21)*y(32)*y(350);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(350);
dydt(350)=dydt(350)-k(21)*y(2)*y(350);
dydt(167)=dydt(167)+k(21)*y(2)*y(350);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(350);
dydt(350)=dydt(350)-k(15)*y(3)*y(350);
dydt(352)=dydt(352)+k(15)*y(3)*y(350);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(350)=dydt(350)-k(10)*y(350);
dydt(3)=dydt(3)+k(10)*y(350);
dydt(351)=dydt(351)+k(10)*y(350);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(350)=dydt(350)-k(8)*y(350);
dydt(126)=dydt(126)+k(8)*y(350);
dydt(33)=dydt(33)+k(8)*y(350);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(350)=dydt(350)-k(6)*y(350);
dydt(48)=dydt(48)+k(6)*y(350);
dydt(82)=dydt(82)+k(6)*y(350);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(347)=dydt(347)-k(34)*y(347);
dydt(167)=dydt(167)+k(34)*y(347);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(347)=dydt(347)-k(33)*y(347);
dydt(346)=dydt(346)+k(33)*y(347);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(347)=dydt(347)-k(31)*y(347);
dydt(168)=dydt(168)+k(31)*y(347);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(347)=dydt(347)-k(30)*y(347);
dydt(195)=dydt(195)+k(30)*y(347);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(347)=dydt(347)-k(22)*y(347);
dydt(32)=dydt(32)+k(22)*y(347);
dydt(343)=dydt(343)+k(22)*y(347);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(347)=dydt(347)-k(22)*y(347);
dydt(182)=dydt(182)+k(22)*y(347);
dydt(350)=dydt(350)+k(22)*y(347);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(347);
dydt(347)=dydt(347)-k(15)*y(3)*y(347);
dydt(349)=dydt(349)+k(15)*y(3)*y(347);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(347)=dydt(347)-k(10)*y(347);
dydt(3)=dydt(3)+k(10)*y(347);
dydt(348)=dydt(348)+k(10)*y(347);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(347)=dydt(347)-k(8)*y(347);
dydt(33)=dydt(33)+k(8)*y(347);
dydt(185)=dydt(185)+k(8)*y(347);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(347)=dydt(347)-k(6)*y(347);
dydt(82)=dydt(82)+k(6)*y(347);
dydt(184)=dydt(184)+k(6)*y(347);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(343)=dydt(343)-k(34)*y(343);
dydt(143)=dydt(143)+k(34)*y(343);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(343)=dydt(343)-k(32)*y(343);
dydt(330)=dydt(330)+k(32)*y(343);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(343)=dydt(343)-k(31)*y(343);
dydt(189)=dydt(189)+k(31)*y(343);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(343)=dydt(343)-k(22)*y(343);
dydt(182)=dydt(182)+k(22)*y(343);
dydt(151)=dydt(151)+k(22)*y(343);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(343);
dydt(343)=dydt(343)-k(21)*y(197)*y(343);
dydt(195)=dydt(195)+k(21)*y(197)*y(343);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(343);
dydt(343)=dydt(343)-k(21)*y(32)*y(343);
dydt(347)=dydt(347)+k(21)*y(32)*y(343);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(343);
dydt(343)=dydt(343)-k(21)*y(2)*y(343);
dydt(346)=dydt(346)+k(21)*y(2)*y(343);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(343);
dydt(343)=dydt(343)-k(15)*y(3)*y(343);
dydt(345)=dydt(345)+k(15)*y(3)*y(343);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(343)=dydt(343)-k(10)*y(343);
dydt(3)=dydt(3)+k(10)*y(343);
dydt(344)=dydt(344)+k(10)*y(343);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

dydt(343)=dydt(343)-k(8)*y(343);
dydt(77)=dydt(77)+k(8)*y(343);
dydt(185)=dydt(185)+k(8)*y(343);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(343)=dydt(343)-k(6)*y(343);
dydt(76)=dydt(76)+k(6)*y(343);
dydt(184)=dydt(184)+k(6)*y(343);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(337)=dydt(337)-k(34)*y(337);
dydt(137)=dydt(137)+k(34)*y(337);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

dydt(337)=dydt(337)-k(33)*y(337);
dydt(343)=dydt(343)+k(33)*y(337);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(337)=dydt(337)-k(32)*y(337);
dydt(334)=dydt(334)+k(32)*y(337);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(337)=dydt(337)-k(31)*y(337);
dydt(198)=dydt(198)+k(31)*y(337);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(337)=dydt(337)-k(22)*y(337);
dydt(197)=dydt(197)+k(22)*y(337);
dydt(151)=dydt(151)+k(22)*y(337);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(337);
dydt(337)=dydt(337)-k(21)*y(197)*y(337);
dydt(342)=dydt(342)+k(21)*y(197)*y(337);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(337);
dydt(337)=dydt(337)-k(21)*y(32)*y(337);
dydt(341)=dydt(341)+k(21)*y(32)*y(337);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(337);
dydt(337)=dydt(337)-k(21)*y(2)*y(337);
dydt(340)=dydt(340)+k(21)*y(2)*y(337);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(337);
dydt(337)=dydt(337)-k(15)*y(3)*y(337);
dydt(339)=dydt(339)+k(15)*y(3)*y(337);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(337)=dydt(337)-k(10)*y(337);
dydt(3)=dydt(3)+k(10)*y(337);
dydt(338)=dydt(338)+k(10)*y(337);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

dydt(337)=dydt(337)-k(8)*y(337);
dydt(77)=dydt(77)+k(8)*y(337);
dydt(336)=dydt(336)+k(8)*y(337);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(337)=dydt(337)-k(6)*y(337);
dydt(76)=dydt(76)+k(6)*y(337);
dydt(335)=dydt(335)+k(6)*y(337);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(334)=dydt(334)-k(34)*y(334);
dydt(135)=dydt(135)+k(34)*y(334);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(334)=dydt(334)-k(33)*y(334);
dydt(330)=dydt(330)+k(33)*y(334);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(334)=dydt(334)-k(31)*y(334);
dydt(199)=dydt(199)+k(31)*y(334);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(334)=dydt(334)-k(22)*y(334);
dydt(197)=dydt(197)+k(22)*y(334);
dydt(128)=dydt(128)+k(22)*y(334);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(334);
dydt(334)=dydt(334)-k(15)*y(3)*y(334);
dydt(200)=dydt(200)+k(15)*y(3)*y(334);

% rule    : R5_2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

dydt(334)=dydt(334)-k(14)*y(334);
dydt(337)=dydt(337)+k(14)*y(334);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(334)=dydt(334)-k(10)*y(334);
dydt(3)=dydt(3)+k(10)*y(334);
dydt(332)=dydt(332)+k(10)*y(334);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

dydt(334)=dydt(334)-k(8)*y(334);
dydt(72)=dydt(72)+k(8)*y(334);
dydt(336)=dydt(336)+k(8)*y(334);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(334)=dydt(334)-k(6)*y(334);
dydt(71)=dydt(71)+k(6)*y(334);
dydt(335)=dydt(335)+k(6)*y(334);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(332)=dydt(332)-k(34)*y(332);
dydt(136)=dydt(136)+k(34)*y(332);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(332)=dydt(332)-k(33)*y(332);
dydt(328)=dydt(328)+k(33)*y(332);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

dydt(332)=dydt(332)-k(31)*y(332);
dydt(211)=dydt(211)+k(31)*y(332);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(332)=dydt(332)-k(22)*y(332);
dydt(197)=dydt(197)+k(22)*y(332);
dydt(331)=dydt(331)+k(22)*y(332);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(332);
dydt(332)=dydt(332)-k(15)*y(3)*y(332);
dydt(322)=dydt(322)+k(15)*y(3)*y(332);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(332);
dydt(332)=dydt(332)-k(9)*y(3)*y(332);
dydt(334)=dydt(334)+k(9)*y(3)*y(332);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

dydt(332)=dydt(332)-k(8)*y(332);
dydt(72)=dydt(72)+k(8)*y(332);
dydt(209)=dydt(209)+k(8)*y(332);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(332)=dydt(332)-k(6)*y(332);
dydt(71)=dydt(71)+k(6)*y(332);
dydt(192)=dydt(192)+k(6)*y(332);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(333)=dydt(333)-k(31)*y(333);
dydt(12)=dydt(12)+k(31)*y(333);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(3)=dydt(3)-k(15)*y(3)*y(333);
dydt(333)=dydt(333)-k(15)*y(3)*y(333);
dydt(123)=dydt(123)+k(15)*y(3)*y(333);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(333);
dydt(333)=dydt(333)-k(9)*y(3)*y(333);
dydt(124)=dydt(124)+k(9)*y(3)*y(333);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) 

dydt(333)=dydt(333)-k(8)*y(333);
dydt(72)=dydt(72)+k(8)*y(333);
dydt(5)=dydt(5)+k(8)*y(333);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) 

dydt(333)=dydt(333)-k(6)*y(333);
dydt(1)=dydt(1)+k(6)*y(333);
dydt(71)=dydt(71)+k(6)*y(333);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

dydt(331)=dydt(331)-k(32)*y(331);
dydt(333)=dydt(333)+k(32)*y(331);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(331)=dydt(331)-k(31)*y(331);
dydt(18)=dydt(18)+k(31)*y(331);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(331);
dydt(331)=dydt(331)-k(21)*y(197)*y(331);
dydt(332)=dydt(332)+k(21)*y(197)*y(331);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(331);
dydt(331)=dydt(331)-k(21)*y(32)*y(331);
dydt(136)=dydt(136)+k(21)*y(32)*y(331);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(331);
dydt(331)=dydt(331)-k(21)*y(2)*y(331);
dydt(142)=dydt(142)+k(21)*y(2)*y(331);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(3)=dydt(3)-k(15)*y(3)*y(331);
dydt(331)=dydt(331)-k(15)*y(3)*y(331);
dydt(127)=dydt(127)+k(15)*y(3)*y(331);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(331);
dydt(331)=dydt(331)-k(9)*y(3)*y(331);
dydt(128)=dydt(128)+k(9)*y(3)*y(331);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) 

dydt(331)=dydt(331)-k(8)*y(331);
dydt(72)=dydt(72)+k(8)*y(331);
dydt(16)=dydt(16)+k(8)*y(331);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) 

dydt(331)=dydt(331)-k(6)*y(331);
dydt(15)=dydt(15)+k(6)*y(331);
dydt(71)=dydt(71)+k(6)*y(331);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

dydt(328)=dydt(328)-k(34)*y(328);
dydt(142)=dydt(142)+k(34)*y(328);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

dydt(328)=dydt(328)-k(31)*y(328);
dydt(216)=dydt(216)+k(31)*y(328);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(328)=dydt(328)-k(22)*y(328);
dydt(182)=dydt(182)+k(22)*y(328);
dydt(331)=dydt(331)+k(22)*y(328);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(328);
dydt(328)=dydt(328)-k(15)*y(3)*y(328);
dydt(326)=dydt(326)+k(15)*y(3)*y(328);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(9)*y(3)*y(328);
dydt(328)=dydt(328)-k(9)*y(3)*y(328);
dydt(330)=dydt(330)+k(9)*y(3)*y(328);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

dydt(328)=dydt(328)-k(8)*y(328);
dydt(72)=dydt(72)+k(8)*y(328);
dydt(214)=dydt(214)+k(8)*y(328);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(328)=dydt(328)-k(6)*y(328);
dydt(71)=dydt(71)+k(6)*y(328);
dydt(177)=dydt(177)+k(6)*y(328);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

dydt(329)=dydt(329)-k(25)*y(329);
dydt(133)=dydt(133)+k(25)*y(329);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

dydt(329)=dydt(329)-k(22)*y(329);
dydt(2)=dydt(2)+k(22)*y(329);
dydt(127)=dydt(127)+k(22)*y(329);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(329)=dydt(329)-k(17)*y(329);
dydt(316)=dydt(316)+k(17)*y(329);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(329)=dydt(329)-k(16)*y(329);
dydt(3)=dydt(3)+k(16)*y(329);
dydt(142)=dydt(142)+k(16)*y(329);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(329);
dydt(329)=dydt(329)-k(9)*y(3)*y(329);
dydt(129)=dydt(129)+k(9)*y(3)*y(329);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(329)=dydt(329)-k(8)*y(329);
dydt(61)=dydt(61)+k(8)*y(329);
dydt(22)=dydt(22)+k(8)*y(329);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(329)=dydt(329)-k(6)*y(329);
dydt(122)=dydt(122)+k(6)*y(329);
dydt(21)=dydt(21)+k(6)*y(329);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

dydt(326)=dydt(326)-k(34)*y(326);
dydt(329)=dydt(329)+k(34)*y(326);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(326)=dydt(326)-k(25)*y(326);
dydt(322)=dydt(322)+k(25)*y(326);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

dydt(326)=dydt(326)-k(22)*y(326);
dydt(182)=dydt(182)+k(22)*y(326);
dydt(127)=dydt(127)+k(22)*y(326);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(326)=dydt(326)-k(17)*y(326);
dydt(323)=dydt(323)+k(17)*y(326);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(326)=dydt(326)-k(16)*y(326);
dydt(3)=dydt(3)+k(16)*y(326);
dydt(328)=dydt(328)+k(16)*y(326);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(326);
dydt(326)=dydt(326)-k(9)*y(3)*y(326);
dydt(327)=dydt(327)+k(9)*y(3)*y(326);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(326)=dydt(326)-k(8)*y(326);
dydt(61)=dydt(61)+k(8)*y(326);
dydt(214)=dydt(214)+k(8)*y(326);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(326)=dydt(326)-k(6)*y(326);
dydt(122)=dydt(122)+k(6)*y(326);
dydt(177)=dydt(177)+k(6)*y(326);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(323)=dydt(323)-k(34)*y(323);
dydt(316)=dydt(316)+k(34)*y(323);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(323)=dydt(323)-k(31)*y(323);
dydt(326)=dydt(326)+k(31)*y(323);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(323)=dydt(323)-k(25)*y(323);
dydt(318)=dydt(318)+k(25)*y(323);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

dydt(323)=dydt(323)-k(22)*y(323);
dydt(182)=dydt(182)+k(22)*y(323);
dydt(314)=dydt(314)+k(22)*y(323);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(323)=dydt(323)-k(16)*y(323);
dydt(3)=dydt(3)+k(16)*y(323);
dydt(325)=dydt(325)+k(16)*y(323);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(323);
dydt(323)=dydt(323)-k(15)*y(3)*y(323);
dydt(324)=dydt(324)+k(15)*y(3)*y(323);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(323)=dydt(323)-k(8)*y(323);
dydt(61)=dydt(61)+k(8)*y(323);
dydt(219)=dydt(219)+k(8)*y(323);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(323)=dydt(323)-k(6)*y(323);
dydt(122)=dydt(122)+k(6)*y(323);
dydt(218)=dydt(218)+k(6)*y(323);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(318)=dydt(318)-k(34)*y(318);
dydt(317)=dydt(317)+k(34)*y(318);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

dydt(318)=dydt(318)-k(33)*y(318);
dydt(323)=dydt(323)+k(33)*y(318);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

dydt(318)=dydt(318)-k(31)*y(318);
dydt(322)=dydt(322)+k(31)*y(318);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

dydt(318)=dydt(318)-k(22)*y(318);
dydt(197)=dydt(197)+k(22)*y(318);
dydt(314)=dydt(314)+k(22)*y(318);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(318)=dydt(318)-k(16)*y(318);
dydt(3)=dydt(3)+k(16)*y(318);
dydt(321)=dydt(321)+k(16)*y(318);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(318);
dydt(318)=dydt(318)-k(15)*y(3)*y(318);
dydt(320)=dydt(320)+k(15)*y(3)*y(318);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(318)=dydt(318)-k(8)*y(318);
dydt(61)=dydt(61)+k(8)*y(318);
dydt(319)=dydt(319)+k(8)*y(318);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(318)=dydt(318)-k(6)*y(318);
dydt(122)=dydt(122)+k(6)*y(318);
dydt(257)=dydt(257)+k(6)*y(318);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(314)=dydt(314)-k(32)*y(314);
dydt(313)=dydt(313)+k(32)*y(314);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(314)=dydt(314)-k(31)*y(314);
dydt(127)=dydt(127)+k(31)*y(314);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(314);
dydt(314)=dydt(314)-k(21)*y(197)*y(314);
dydt(318)=dydt(318)+k(21)*y(197)*y(314);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(314);
dydt(314)=dydt(314)-k(21)*y(32)*y(314);
dydt(317)=dydt(317)+k(21)*y(32)*y(314);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(314);
dydt(314)=dydt(314)-k(21)*y(2)*y(314);
dydt(316)=dydt(316)+k(21)*y(2)*y(314);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

dydt(314)=dydt(314)-k(16)*y(314);
dydt(3)=dydt(3)+k(16)*y(314);
dydt(237)=dydt(237)+k(16)*y(314);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(314);
dydt(314)=dydt(314)-k(15)*y(3)*y(314);
dydt(236)=dydt(236)+k(15)*y(3)*y(314);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

dydt(314)=dydt(314)-k(8)*y(314);
dydt(315)=dydt(315)+k(8)*y(314);
dydt(61)=dydt(61)+k(8)*y(314);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) 

dydt(314)=dydt(314)-k(6)*y(314);
dydt(77)=dydt(77)+k(6)*y(314);
dydt(122)=dydt(122)+k(6)*y(314);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(313)=dydt(313)-k(31)*y(313);
dydt(123)=dydt(123)+k(31)*y(313);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(313)=dydt(313)-k(19)*y(313);
dydt(314)=dydt(314)+k(19)*y(313);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

dydt(313)=dydt(313)-k(16)*y(313);
dydt(3)=dydt(3)+k(16)*y(313);
dydt(74)=dydt(74)+k(16)*y(313);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(313);
dydt(313)=dydt(313)-k(15)*y(3)*y(313);
dydt(73)=dydt(73)+k(15)*y(3)*y(313);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(313)=dydt(313)-k(8)*y(313);
dydt(91)=dydt(91)+k(8)*y(313);
dydt(61)=dydt(61)+k(8)*y(313);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) 

dydt(313)=dydt(313)-k(6)*y(313);
dydt(72)=dydt(72)+k(6)*y(313);
dydt(122)=dydt(122)+k(6)*y(313);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

dydt(309)=dydt(309)-k(32)*y(309);
dydt(313)=dydt(313)+k(32)*y(309);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(309)=dydt(309)-k(31)*y(309);
dydt(119)=dydt(119)+k(31)*y(309);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(309);
dydt(309)=dydt(309)-k(21)*y(197)*y(309);
dydt(312)=dydt(312)+k(21)*y(197)*y(309);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(309);
dydt(309)=dydt(309)-k(21)*y(32)*y(309);
dydt(311)=dydt(311)+k(21)*y(32)*y(309);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(309);
dydt(309)=dydt(309)-k(21)*y(2)*y(309);
dydt(306)=dydt(306)+k(21)*y(2)*y(309);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

dydt(309)=dydt(309)-k(19)*y(309);
dydt(310)=dydt(310)+k(19)*y(309);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

dydt(309)=dydt(309)-k(16)*y(309);
dydt(3)=dydt(3)+k(16)*y(309);
dydt(79)=dydt(79)+k(16)*y(309);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(309);
dydt(309)=dydt(309)-k(15)*y(3)*y(309);
dydt(78)=dydt(78)+k(15)*y(3)*y(309);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(309)=dydt(309)-k(8)*y(309);
dydt(91)=dydt(91)+k(8)*y(309);
dydt(54)=dydt(54)+k(8)*y(309);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) 

dydt(309)=dydt(309)-k(6)*y(309);
dydt(72)=dydt(72)+k(6)*y(309);
dydt(118)=dydt(118)+k(6)*y(309);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(306)=dydt(306)-k(31)*y(306);
dydt(115)=dydt(115)+k(31)*y(306);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

dydt(306)=dydt(306)-k(22)*y(306);
dydt(2)=dydt(2)+k(22)*y(306);
dydt(309)=dydt(309)+k(22)*y(306);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(306)=dydt(306)-k(19)*y(306);
dydt(308)=dydt(308)+k(19)*y(306);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(306)=dydt(306)-k(16)*y(306);
dydt(3)=dydt(3)+k(16)*y(306);
dydt(305)=dydt(305)+k(16)*y(306);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(306);
dydt(306)=dydt(306)-k(15)*y(3)*y(306);
dydt(307)=dydt(307)+k(15)*y(3)*y(306);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(306)=dydt(306)-k(8)*y(306);
dydt(91)=dydt(91)+k(8)*y(306);
dydt(43)=dydt(43)+k(8)*y(306);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

dydt(306)=dydt(306)-k(6)*y(306);
dydt(72)=dydt(72)+k(6)*y(306);
dydt(114)=dydt(114)+k(6)*y(306);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(305)=dydt(305)-k(31)*y(305);
dydt(90)=dydt(90)+k(31)*y(305);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(305)=dydt(305)-k(31)*y(305);
dydt(112)=dydt(112)+k(31)*y(305);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(305)=dydt(305)-k(22)*y(305);
dydt(2)=dydt(2)+k(22)*y(305);
dydt(79)=dydt(79)+k(22)*y(305);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(305);
dydt(305)=dydt(305)-k(15)*y(3)*y(305);
dydt(80)=dydt(80)+k(15)*y(3)*y(305);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(305);
dydt(305)=dydt(305)-k(15)*y(3)*y(305);
dydt(306)=dydt(306)+k(15)*y(3)*y(305);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(305)=dydt(305)-k(8)*y(305);
dydt(91)=dydt(91)+k(8)*y(305);
dydt(111)=dydt(111)+k(8)*y(305);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

dydt(305)=dydt(305)-k(6)*y(305);
dydt(72)=dydt(72)+k(6)*y(305);
dydt(110)=dydt(110)+k(6)*y(305);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(302)=dydt(302)-k(34)*y(302);
dydt(305)=dydt(305)+k(34)*y(302);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(302)=dydt(302)-k(31)*y(302);
dydt(299)=dydt(299)+k(31)*y(302);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

dydt(302)=dydt(302)-k(31)*y(302);
dydt(304)=dydt(304)+k(31)*y(302);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(302)=dydt(302)-k(22)*y(302);
dydt(182)=dydt(182)+k(22)*y(302);
dydt(79)=dydt(79)+k(22)*y(302);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(302);
dydt(302)=dydt(302)-k(15)*y(3)*y(302);
dydt(300)=dydt(300)+k(15)*y(3)*y(302);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

dydt(3)=dydt(3)-k(15)*y(3)*y(302);
dydt(302)=dydt(302)-k(15)*y(3)*y(302);
dydt(303)=dydt(303)+k(15)*y(3)*y(302);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(302)=dydt(302)-k(8)*y(302);
dydt(91)=dydt(91)+k(8)*y(302);
dydt(218)=dydt(218)+k(8)*y(302);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

dydt(302)=dydt(302)-k(6)*y(302);
dydt(72)=dydt(72)+k(6)*y(302);
dydt(264)=dydt(264)+k(6)*y(302);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(300)=dydt(300)-k(34)*y(300);
dydt(80)=dydt(80)+k(34)*y(300);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(300)=dydt(300)-k(31)*y(300);
dydt(297)=dydt(297)+k(31)*y(300);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(300)=dydt(300)-k(26)*y(300);
dydt(203)=dydt(203)+k(26)*y(300);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

dydt(300)=dydt(300)-k(22)*y(300);
dydt(182)=dydt(182)+k(22)*y(300);
dydt(75)=dydt(75)+k(22)*y(300);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(300)=dydt(300)-k(16)*y(300);
dydt(3)=dydt(3)+k(16)*y(300);
dydt(302)=dydt(302)+k(16)*y(300);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(300);
dydt(300)=dydt(300)-k(15)*y(3)*y(300);
dydt(301)=dydt(301)+k(15)*y(3)*y(300);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(300)=dydt(300)-k(8)*y(300);
dydt(62)=dydt(62)+k(8)*y(300);
dydt(218)=dydt(218)+k(8)*y(300);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(300)=dydt(300)-k(6)*y(300);
dydt(61)=dydt(61)+k(6)*y(300);
dydt(264)=dydt(264)+k(6)*y(300);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(297)=dydt(297)-k(34)*y(297);
dydt(88)=dydt(88)+k(34)*y(297);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(297)=dydt(297)-k(26)*y(297);
dydt(295)=dydt(295)+k(26)*y(297);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

dydt(297)=dydt(297)-k(22)*y(297);
dydt(182)=dydt(182)+k(22)*y(297);
dydt(63)=dydt(63)+k(22)*y(297);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(297)=dydt(297)-k(18)*y(297);
dydt(300)=dydt(300)+k(18)*y(297);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(297)=dydt(297)-k(16)*y(297);
dydt(3)=dydt(3)+k(16)*y(297);
dydt(299)=dydt(299)+k(16)*y(297);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(297);
dydt(297)=dydt(297)-k(9)*y(3)*y(297);
dydt(298)=dydt(298)+k(9)*y(3)*y(297);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(297)=dydt(297)-k(8)*y(297);
dydt(62)=dydt(62)+k(8)*y(297);
dydt(177)=dydt(177)+k(8)*y(297);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(297)=dydt(297)-k(6)*y(297);
dydt(61)=dydt(61)+k(6)*y(297);
dydt(176)=dydt(176)+k(6)*y(297);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(295)=dydt(295)-k(34)*y(295);
dydt(85)=dydt(85)+k(34)*y(295);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(295)=dydt(295)-k(33)*y(295);
dydt(297)=dydt(297)+k(33)*y(295);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

dydt(295)=dydt(295)-k(22)*y(295);
dydt(197)=dydt(197)+k(22)*y(295);
dydt(63)=dydt(63)+k(22)*y(295);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(295)=dydt(295)-k(18)*y(295);
dydt(203)=dydt(203)+k(18)*y(295);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(295)=dydt(295)-k(16)*y(295);
dydt(3)=dydt(3)+k(16)*y(295);
dydt(296)=dydt(296)+k(16)*y(295);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(295);
dydt(295)=dydt(295)-k(9)*y(3)*y(295);
dydt(204)=dydt(204)+k(9)*y(3)*y(295);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(295)=dydt(295)-k(8)*y(295);
dydt(62)=dydt(62)+k(8)*y(295);
dydt(192)=dydt(192)+k(8)*y(295);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(295)=dydt(295)-k(6)*y(295);
dydt(61)=dydt(61)+k(6)*y(295);
dydt(191)=dydt(191)+k(6)*y(295);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(291)=dydt(291)-k(34)*y(291);
dydt(287)=dydt(287)+k(34)*y(291);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

dydt(291)=dydt(291)-k(33)*y(291);
dydt(272)=dydt(272)+k(33)*y(291);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(291)=dydt(291)-k(32)*y(291);
dydt(295)=dydt(295)+k(32)*y(291);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

dydt(291)=dydt(291)-k(22)*y(291);
dydt(197)=dydt(197)+k(22)*y(291);
dydt(56)=dydt(56)+k(22)*y(291);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(291);
dydt(291)=dydt(291)-k(21)*y(197)*y(291);
dydt(294)=dydt(294)+k(21)*y(197)*y(291);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(291);
dydt(291)=dydt(291)-k(21)*y(32)*y(291);
dydt(293)=dydt(293)+k(21)*y(32)*y(291);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(2)=dydt(2)-k(21)*y(2)*y(291);
dydt(291)=dydt(291)-k(21)*y(2)*y(291);
dydt(262)=dydt(262)+k(21)*y(2)*y(291);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(291)=dydt(291)-k(18)*y(291);
dydt(260)=dydt(260)+k(18)*y(291);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(291)=dydt(291)-k(16)*y(291);
dydt(3)=dydt(3)+k(16)*y(291);
dydt(292)=dydt(292)+k(16)*y(291);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(291);
dydt(291)=dydt(291)-k(9)*y(3)*y(291);
dydt(206)=dydt(206)+k(9)*y(3)*y(291);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(291)=dydt(291)-k(8)*y(291);
dydt(55)=dydt(55)+k(8)*y(291);
dydt(192)=dydt(192)+k(8)*y(291);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(291)=dydt(291)-k(6)*y(291);
dydt(54)=dydt(54)+k(6)*y(291);
dydt(191)=dydt(191)+k(6)*y(291);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(288)=dydt(288)-k(34)*y(288);
dydt(283)=dydt(283)+k(34)*y(288);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(288)=dydt(288)-k(34)*y(288);
dydt(262)=dydt(262)+k(34)*y(288);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(288)=dydt(288)-k(33)*y(288);
dydt(273)=dydt(273)+k(33)*y(288);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(288)=dydt(288)-k(22)*y(288);
dydt(197)=dydt(197)+k(22)*y(288);
dydt(282)=dydt(282)+k(22)*y(288);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(288)=dydt(288)-k(22)*y(288);
dydt(182)=dydt(182)+k(22)*y(288);
dydt(291)=dydt(291)+k(22)*y(288);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(288)=dydt(288)-k(18)*y(288);
dydt(261)=dydt(261)+k(18)*y(288);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(288)=dydt(288)-k(16)*y(288);
dydt(3)=dydt(3)+k(16)*y(288);
dydt(290)=dydt(290)+k(16)*y(288);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(288);
dydt(288)=dydt(288)-k(9)*y(3)*y(288);
dydt(289)=dydt(289)+k(9)*y(3)*y(288);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(288)=dydt(288)-k(8)*y(288);
dydt(192)=dydt(192)+k(8)*y(288);
dydt(223)=dydt(223)+k(8)*y(288);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(288)=dydt(288)-k(6)*y(288);
dydt(191)=dydt(191)+k(6)*y(288);
dydt(222)=dydt(222)+k(6)*y(288);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(283)=dydt(283)-k(34)*y(283);
dydt(277)=dydt(277)+k(34)*y(283);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(283)=dydt(283)-k(33)*y(283);
dydt(278)=dydt(278)+k(33)*y(283);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(283)=dydt(283)-k(30)*y(283);
dydt(288)=dydt(288)+k(30)*y(283);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(283)=dydt(283)-k(22)*y(283);
dydt(32)=dydt(32)+k(22)*y(283);
dydt(282)=dydt(282)+k(22)*y(283);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(283)=dydt(283)-k(22)*y(283);
dydt(182)=dydt(182)+k(22)*y(283);
dydt(287)=dydt(287)+k(22)*y(283);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(283)=dydt(283)-k(18)*y(283);
dydt(286)=dydt(286)+k(18)*y(283);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(283)=dydt(283)-k(16)*y(283);
dydt(3)=dydt(3)+k(16)*y(283);
dydt(285)=dydt(285)+k(16)*y(283);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(283);
dydt(283)=dydt(283)-k(9)*y(3)*y(283);
dydt(284)=dydt(284)+k(9)*y(3)*y(283);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(283)=dydt(283)-k(8)*y(283);
dydt(27)=dydt(27)+k(8)*y(283);
dydt(223)=dydt(223)+k(8)*y(283);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(283)=dydt(283)-k(6)*y(283);
dydt(86)=dydt(86)+k(6)*y(283);
dydt(222)=dydt(222)+k(6)*y(283);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(278)=dydt(278)-k(34)*y(278);
dydt(274)=dydt(274)+k(34)*y(278);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(278)=dydt(278)-k(30)*y(278);
dydt(273)=dydt(273)+k(30)*y(278);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(278)=dydt(278)-k(26)*y(278);
dydt(283)=dydt(283)+k(26)*y(278);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(278)=dydt(278)-k(22)*y(278);
dydt(2)=dydt(2)+k(22)*y(278);
dydt(282)=dydt(282)+k(22)*y(278);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(278)=dydt(278)-k(22)*y(278);
dydt(182)=dydt(182)+k(22)*y(278);
dydt(276)=dydt(276)+k(22)*y(278);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(278)=dydt(278)-k(18)*y(278);
dydt(281)=dydt(281)+k(18)*y(278);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(278)=dydt(278)-k(16)*y(278);
dydt(3)=dydt(3)+k(16)*y(278);
dydt(280)=dydt(280)+k(16)*y(278);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(278);
dydt(278)=dydt(278)-k(9)*y(3)*y(278);
dydt(279)=dydt(279)+k(9)*y(3)*y(278);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(278)=dydt(278)-k(8)*y(278);
dydt(21)=dydt(21)+k(8)*y(278);
dydt(223)=dydt(223)+k(8)*y(278);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(278)=dydt(278)-k(6)*y(278);
dydt(89)=dydt(89)+k(6)*y(278);
dydt(222)=dydt(222)+k(6)*y(278);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(274)=dydt(274)-k(28)*y(274);
dydt(269)=dydt(269)+k(28)*y(274);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(274)=dydt(274)-k(27)*y(274);
dydt(278)=dydt(278)+k(27)*y(274);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(274)=dydt(274)-k(26)*y(274);
dydt(277)=dydt(277)+k(26)*y(274);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(274)=dydt(274)-k(22)*y(274);
dydt(2)=dydt(2)+k(22)*y(274);
dydt(49)=dydt(49)+k(22)*y(274);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(274)=dydt(274)-k(22)*y(274);
dydt(2)=dydt(2)+k(22)*y(274);
dydt(276)=dydt(276)+k(22)*y(274);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(274)=dydt(274)-k(18)*y(274);
dydt(253)=dydt(253)+k(18)*y(274);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(274)=dydt(274)-k(16)*y(274);
dydt(3)=dydt(3)+k(16)*y(274);
dydt(275)=dydt(275)+k(16)*y(274);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(274);
dydt(274)=dydt(274)-k(9)*y(3)*y(274);
dydt(51)=dydt(51)+k(9)*y(3)*y(274);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(274)=dydt(274)-k(8)*y(274);
dydt(21)=dydt(21)+k(8)*y(274);
dydt(44)=dydt(44)+k(8)*y(274);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(274)=dydt(274)-k(6)*y(274);
dydt(89)=dydt(89)+k(6)*y(274);
dydt(43)=dydt(43)+k(6)*y(274);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(269)=dydt(269)-k(34)*y(269);
dydt(274)=dydt(274)+k(34)*y(269);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(269)=dydt(269)-k(29)*y(269);
dydt(273)=dydt(273)+k(29)*y(269);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(269)=dydt(269)-k(26)*y(269);
dydt(262)=dydt(262)+k(26)*y(269);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(269)=dydt(269)-k(22)*y(269);
dydt(182)=dydt(182)+k(22)*y(269);
dydt(49)=dydt(49)+k(22)*y(269);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(269)=dydt(269)-k(22)*y(269);
dydt(2)=dydt(2)+k(22)*y(269);
dydt(272)=dydt(272)+k(22)*y(269);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(269)=dydt(269)-k(18)*y(269);
dydt(263)=dydt(263)+k(18)*y(269);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(269)=dydt(269)-k(16)*y(269);
dydt(3)=dydt(3)+k(16)*y(269);
dydt(271)=dydt(271)+k(16)*y(269);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(269);
dydt(269)=dydt(269)-k(9)*y(3)*y(269);
dydt(270)=dydt(270)+k(9)*y(3)*y(269);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(269)=dydt(269)-k(8)*y(269);
dydt(177)=dydt(177)+k(8)*y(269);
dydt(44)=dydt(44)+k(8)*y(269);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(269)=dydt(269)-k(6)*y(269);
dydt(176)=dydt(176)+k(6)*y(269);
dydt(43)=dydt(43)+k(6)*y(269);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(263)=dydt(263)-k(34)*y(263);
dydt(253)=dydt(253)+k(34)*y(263);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(263)=dydt(263)-k(31)*y(263);
dydt(269)=dydt(269)+k(31)*y(263);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(263)=dydt(263)-k(29)*y(263);
dydt(268)=dydt(268)+k(29)*y(263);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(263)=dydt(263)-k(26)*y(263);
dydt(255)=dydt(255)+k(26)*y(263);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(263)=dydt(263)-k(22)*y(263);
dydt(182)=dydt(182)+k(22)*y(263);
dydt(250)=dydt(250)+k(22)*y(263);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(263)=dydt(263)-k(22)*y(263);
dydt(2)=dydt(2)+k(22)*y(263);
dydt(267)=dydt(267)+k(22)*y(263);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(263)=dydt(263)-k(16)*y(263);
dydt(3)=dydt(3)+k(16)*y(263);
dydt(266)=dydt(266)+k(16)*y(263);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(263);
dydt(263)=dydt(263)-k(15)*y(3)*y(263);
dydt(265)=dydt(265)+k(15)*y(3)*y(263);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(263)=dydt(263)-k(8)*y(263);
dydt(218)=dydt(218)+k(8)*y(263);
dydt(44)=dydt(44)+k(8)*y(263);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(263)=dydt(263)-k(6)*y(263);
dydt(264)=dydt(264)+k(6)*y(263);
dydt(43)=dydt(43)+k(6)*y(263);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(255)=dydt(255)-k(34)*y(255);
dydt(254)=dydt(254)+k(34)*y(255);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(255)=dydt(255)-k(33)*y(255);
dydt(263)=dydt(263)+k(33)*y(255);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(255)=dydt(255)-k(31)*y(255);
dydt(262)=dydt(262)+k(31)*y(255);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(255)=dydt(255)-k(29)*y(255);
dydt(261)=dydt(261)+k(29)*y(255);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

dydt(255)=dydt(255)-k(22)*y(255);
dydt(197)=dydt(197)+k(22)*y(255);
dydt(250)=dydt(250)+k(22)*y(255);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(255)=dydt(255)-k(22)*y(255);
dydt(2)=dydt(2)+k(22)*y(255);
dydt(260)=dydt(260)+k(22)*y(255);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(255)=dydt(255)-k(16)*y(255);
dydt(3)=dydt(3)+k(16)*y(255);
dydt(259)=dydt(259)+k(16)*y(255);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(255);
dydt(255)=dydt(255)-k(15)*y(3)*y(255);
dydt(258)=dydt(258)+k(15)*y(3)*y(255);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(255)=dydt(255)-k(8)*y(255);
dydt(257)=dydt(257)+k(8)*y(255);
dydt(44)=dydt(44)+k(8)*y(255);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(255)=dydt(255)-k(6)*y(255);
dydt(256)=dydt(256)+k(6)*y(255);
dydt(43)=dydt(43)+k(6)*y(255);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(250)=dydt(250)-k(32)*y(250);
dydt(239)=dydt(239)+k(32)*y(250);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(250)=dydt(250)-k(31)*y(250);
dydt(49)=dydt(49)+k(31)*y(250);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

dydt(250)=dydt(250)-k(22)*y(250);
dydt(2)=dydt(2)+k(22)*y(250);
dydt(238)=dydt(238)+k(22)*y(250);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(250);
dydt(250)=dydt(250)-k(21)*y(197)*y(250);
dydt(255)=dydt(255)+k(21)*y(197)*y(250);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(250);
dydt(250)=dydt(250)-k(21)*y(32)*y(250);
dydt(254)=dydt(254)+k(21)*y(32)*y(250);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(250);
dydt(250)=dydt(250)-k(21)*y(2)*y(250);
dydt(253)=dydt(253)+k(21)*y(2)*y(250);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(250)=dydt(250)-k(16)*y(250);
dydt(3)=dydt(3)+k(16)*y(250);
dydt(252)=dydt(252)+k(16)*y(250);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(250);
dydt(250)=dydt(250)-k(15)*y(3)*y(250);
dydt(251)=dydt(251)+k(15)*y(3)*y(250);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

dydt(250)=dydt(250)-k(8)*y(250);
dydt(77)=dydt(77)+k(8)*y(250);
dydt(44)=dydt(44)+k(8)*y(250);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(250)=dydt(250)-k(6)*y(250);
dydt(76)=dydt(76)+k(6)*y(250);
dydt(43)=dydt(43)+k(6)*y(250);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(243)=dydt(243)-k(33)*y(243);
dydt(250)=dydt(250)+k(33)*y(243);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(243)=dydt(243)-k(32)*y(243);
dydt(240)=dydt(240)+k(32)*y(243);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(243)=dydt(243)-k(31)*y(243);
dydt(249)=dydt(249)+k(31)*y(243);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

dydt(243)=dydt(243)-k(22)*y(243);
dydt(32)=dydt(32)+k(22)*y(243);
dydt(238)=dydt(238)+k(22)*y(243);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(243);
dydt(243)=dydt(243)-k(21)*y(197)*y(243);
dydt(248)=dydt(248)+k(21)*y(197)*y(243);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(243);
dydt(243)=dydt(243)-k(21)*y(32)*y(243);
dydt(247)=dydt(247)+k(21)*y(32)*y(243);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(243);
dydt(243)=dydt(243)-k(21)*y(2)*y(243);
dydt(246)=dydt(246)+k(21)*y(2)*y(243);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(243)=dydt(243)-k(16)*y(243);
dydt(3)=dydt(3)+k(16)*y(243);
dydt(245)=dydt(245)+k(16)*y(243);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(243);
dydt(243)=dydt(243)-k(15)*y(3)*y(243);
dydt(244)=dydt(244)+k(15)*y(3)*y(243);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

dydt(243)=dydt(243)-k(8)*y(243);
dydt(77)=dydt(77)+k(8)*y(243);
dydt(38)=dydt(38)+k(8)*y(243);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(243)=dydt(243)-k(6)*y(243);
dydt(76)=dydt(76)+k(6)*y(243);
dydt(37)=dydt(37)+k(6)*y(243);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(240)=dydt(240)-k(33)*y(240);
dydt(239)=dydt(239)+k(33)*y(240);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(240)=dydt(240)-k(31)*y(240);
dydt(39)=dydt(39)+k(31)*y(240);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

dydt(240)=dydt(240)-k(22)*y(240);
dydt(32)=dydt(32)+k(22)*y(240);
dydt(235)=dydt(235)+k(22)*y(240);

% rule    : R8_2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(240)=dydt(240)-k(20)*y(240);
dydt(243)=dydt(243)+k(20)*y(240);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(240)=dydt(240)-k(16)*y(240);
dydt(3)=dydt(3)+k(16)*y(240);
dydt(242)=dydt(242)+k(16)*y(240);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(3)=dydt(3)-k(15)*y(3)*y(240);
dydt(240)=dydt(240)-k(15)*y(3)*y(240);
dydt(241)=dydt(241)+k(15)*y(3)*y(240);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

dydt(240)=dydt(240)-k(8)*y(240);
dydt(72)=dydt(72)+k(8)*y(240);
dydt(38)=dydt(38)+k(8)*y(240);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(240)=dydt(240)-k(6)*y(240);
dydt(71)=dydt(71)+k(6)*y(240);
dydt(37)=dydt(37)+k(6)*y(240);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(235)=dydt(235)-k(32)*y(235);
dydt(70)=dydt(70)+k(32)*y(235);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(235)=dydt(235)-k(31)*y(235);
dydt(233)=dydt(233)+k(31)*y(235);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(235);
dydt(235)=dydt(235)-k(21)*y(197)*y(235);
dydt(231)=dydt(231)+k(21)*y(197)*y(235);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(235);
dydt(235)=dydt(235)-k(21)*y(32)*y(235);
dydt(240)=dydt(240)+k(21)*y(32)*y(235);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(235);
dydt(235)=dydt(235)-k(21)*y(2)*y(235);
dydt(239)=dydt(239)+k(21)*y(2)*y(235);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(235)=dydt(235)-k(20)*y(235);
dydt(238)=dydt(238)+k(20)*y(235);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

dydt(235)=dydt(235)-k(16)*y(235);
dydt(3)=dydt(3)+k(16)*y(235);
dydt(237)=dydt(237)+k(16)*y(235);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(235);
dydt(235)=dydt(235)-k(15)*y(3)*y(235);
dydt(236)=dydt(236)+k(15)*y(3)*y(235);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) 

dydt(235)=dydt(235)-k(8)*y(235);
dydt(72)=dydt(72)+k(8)*y(235);
dydt(55)=dydt(55)+k(8)*y(235);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(235)=dydt(235)-k(6)*y(235);
dydt(71)=dydt(71)+k(6)*y(235);
dydt(54)=dydt(54)+k(6)*y(235);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(233)=dydt(233)-k(32)*y(233);
dydt(68)=dydt(68)+k(32)*y(233);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(233);
dydt(233)=dydt(233)-k(21)*y(197)*y(233);
dydt(229)=dydt(229)+k(21)*y(197)*y(233);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(233);
dydt(233)=dydt(233)-k(21)*y(32)*y(233);
dydt(39)=dydt(39)+k(21)*y(32)*y(233);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(233);
dydt(233)=dydt(233)-k(21)*y(2)*y(233);
dydt(45)=dydt(45)+k(21)*y(2)*y(233);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(233)=dydt(233)-k(20)*y(233);
dydt(56)=dydt(56)+k(20)*y(233);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(233)=dydt(233)-k(18)*y(233);
dydt(235)=dydt(235)+k(18)*y(233);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

dydt(233)=dydt(233)-k(16)*y(233);
dydt(3)=dydt(3)+k(16)*y(233);
dydt(234)=dydt(234)+k(16)*y(233);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(233);
dydt(233)=dydt(233)-k(9)*y(3)*y(233);
dydt(41)=dydt(41)+k(9)*y(3)*y(233);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) 

dydt(233)=dydt(233)-k(8)*y(233);
dydt(1)=dydt(1)+k(8)*y(233);
dydt(55)=dydt(55)+k(8)*y(233);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(233)=dydt(233)-k(6)*y(233);
dydt(6)=dydt(6)+k(6)*y(233);
dydt(54)=dydt(54)+k(6)*y(233);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

dydt(229)=dydt(229)-k(34)*y(229);
dydt(39)=dydt(39)+k(34)*y(229);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

dydt(229)=dydt(229)-k(33)*y(229);
dydt(224)=dydt(224)+k(33)*y(229);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

dydt(229)=dydt(229)-k(22)*y(229);
dydt(197)=dydt(197)+k(22)*y(229);
dydt(233)=dydt(233)+k(22)*y(229);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(229)=dydt(229)-k(20)*y(229);
dydt(232)=dydt(232)+k(20)*y(229);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

dydt(229)=dydt(229)-k(18)*y(229);
dydt(231)=dydt(231)+k(18)*y(229);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(229)=dydt(229)-k(16)*y(229);
dydt(3)=dydt(3)+k(16)*y(229);
dydt(230)=dydt(230)+k(16)*y(229);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(229);
dydt(229)=dydt(229)-k(9)*y(3)*y(229);
dydt(226)=dydt(226)+k(9)*y(3)*y(229);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

dydt(229)=dydt(229)-k(8)*y(229);
dydt(1)=dydt(1)+k(8)*y(229);
dydt(228)=dydt(228)+k(8)*y(229);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(229)=dydt(229)-k(6)*y(229);
dydt(6)=dydt(6)+k(6)*y(229);
dydt(227)=dydt(227)+k(6)*y(229);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(226)=dydt(226)-k(34)*y(226);
dydt(36)=dydt(36)+k(34)*y(226);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(226)=dydt(226)-k(33)*y(226);
dydt(221)=dydt(221)+k(33)*y(226);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(226)=dydt(226)-k(22)*y(226);
dydt(197)=dydt(197)+k(22)*y(226);
dydt(41)=dydt(41)+k(22)*y(226);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(226)=dydt(226)-k(20)*y(226);
dydt(205)=dydt(205)+k(20)*y(226);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(226)=dydt(226)-k(16)*y(226);
dydt(3)=dydt(3)+k(16)*y(226);
dydt(212)=dydt(212)+k(16)*y(226);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

dydt(226)=dydt(226)-k(10)*y(226);
dydt(3)=dydt(3)+k(10)*y(226);
dydt(229)=dydt(229)+k(10)*y(226);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(226)=dydt(226)-k(8)*y(226);
dydt(7)=dydt(7)+k(8)*y(226);
dydt(228)=dydt(228)+k(8)*y(226);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(226)=dydt(226)-k(6)*y(226);
dydt(9)=dydt(9)+k(6)*y(226);
dydt(227)=dydt(227)+k(6)*y(226);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(221)=dydt(221)-k(34)*y(221);
dydt(42)=dydt(42)+k(34)*y(221);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(221)=dydt(221)-k(23)*y(221);
dydt(226)=dydt(226)+k(23)*y(221);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(221)=dydt(221)-k(22)*y(221);
dydt(182)=dydt(182)+k(22)*y(221);
dydt(41)=dydt(41)+k(22)*y(221);

% rule    : R8_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(221)=dydt(221)-k(20)*y(221);
dydt(225)=dydt(225)+k(20)*y(221);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(221)=dydt(221)-k(16)*y(221);
dydt(3)=dydt(3)+k(16)*y(221);
dydt(217)=dydt(217)+k(16)*y(221);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

dydt(221)=dydt(221)-k(10)*y(221);
dydt(3)=dydt(3)+k(10)*y(221);
dydt(224)=dydt(224)+k(10)*y(221);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(221)=dydt(221)-k(8)*y(221);
dydt(7)=dydt(7)+k(8)*y(221);
dydt(223)=dydt(223)+k(8)*y(221);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(221)=dydt(221)-k(6)*y(221);
dydt(9)=dydt(9)+k(6)*y(221);
dydt(222)=dydt(222)+k(6)*y(221);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(217)=dydt(217)-k(34)*y(217);
dydt(25)=dydt(25)+k(34)*y(217);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(217)=dydt(217)-k(31)*y(217);
dydt(213)=dydt(213)+k(31)*y(217);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(217)=dydt(217)-k(23)*y(217);
dydt(212)=dydt(212)+k(23)*y(217);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(217)=dydt(217)-k(22)*y(217);
dydt(182)=dydt(182)+k(22)*y(217);
dydt(19)=dydt(19)+k(22)*y(217);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(217);
dydt(217)=dydt(217)-k(15)*y(3)*y(217);
dydt(221)=dydt(221)+k(15)*y(3)*y(217);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(217)=dydt(217)-k(10)*y(217);
dydt(3)=dydt(3)+k(10)*y(217);
dydt(220)=dydt(220)+k(10)*y(217);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(217)=dydt(217)-k(8)*y(217);
dydt(7)=dydt(7)+k(8)*y(217);
dydt(219)=dydt(219)+k(8)*y(217);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(217)=dydt(217)-k(6)*y(217);
dydt(9)=dydt(9)+k(6)*y(217);
dydt(218)=dydt(218)+k(6)*y(217);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(213)=dydt(213)-k(34)*y(213);
dydt(20)=dydt(20)+k(34)*y(213);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(213)=dydt(213)-k(23)*y(213);
dydt(208)=dydt(208)+k(23)*y(213);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

dydt(213)=dydt(213)-k(22)*y(213);
dydt(182)=dydt(182)+k(22)*y(213);
dydt(14)=dydt(14)+k(22)*y(213);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(213)=dydt(213)-k(11)*y(213);
dydt(217)=dydt(217)+k(11)*y(213);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

dydt(213)=dydt(213)-k(10)*y(213);
dydt(3)=dydt(3)+k(10)*y(213);
dydt(216)=dydt(216)+k(10)*y(213);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(213);
dydt(213)=dydt(213)-k(9)*y(3)*y(213);
dydt(215)=dydt(215)+k(9)*y(3)*y(213);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(213)=dydt(213)-k(8)*y(213);
dydt(7)=dydt(7)+k(8)*y(213);
dydt(214)=dydt(214)+k(8)*y(213);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(213)=dydt(213)-k(6)*y(213);
dydt(9)=dydt(9)+k(6)*y(213);
dydt(177)=dydt(177)+k(6)*y(213);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(208)=dydt(208)-k(34)*y(208);
dydt(26)=dydt(26)+k(34)*y(208);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

dydt(208)=dydt(208)-k(33)*y(208);
dydt(213)=dydt(213)+k(33)*y(208);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

dydt(208)=dydt(208)-k(22)*y(208);
dydt(197)=dydt(197)+k(22)*y(208);
dydt(14)=dydt(14)+k(22)*y(208);

% rule    : R4
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

dydt(208)=dydt(208)-k(11)*y(208);
dydt(212)=dydt(212)+k(11)*y(208);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

dydt(208)=dydt(208)-k(10)*y(208);
dydt(3)=dydt(3)+k(10)*y(208);
dydt(211)=dydt(211)+k(10)*y(208);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(208);
dydt(208)=dydt(208)-k(9)*y(3)*y(208);
dydt(210)=dydt(210)+k(9)*y(3)*y(208);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(208)=dydt(208)-k(8)*y(208);
dydt(7)=dydt(7)+k(8)*y(208);
dydt(209)=dydt(209)+k(8)*y(208);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(208)=dydt(208)-k(6)*y(208);
dydt(9)=dydt(9)+k(6)*y(208);
dydt(192)=dydt(192)+k(6)*y(208);

% rule    : R19
% reaction: Syk(tSH2, l~pY, a~pY) -> Syk(tSH2, l~pY, a~Y) 

dydt(197)=dydt(197)-k(36)*y(197);
dydt(32)=dydt(32)+k(36)*y(197);

% rule    : R18
% reaction: Syk(tSH2, l~pY, a~pY) -> Syk(tSH2, l~Y, a~pY) 

dydt(197)=dydt(197)-k(35)*y(197);
dydt(182)=dydt(182)+k(35)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(14);
dydt(14)=dydt(14)-k(21)*y(197)*y(14);
dydt(208)=dydt(208)+k(21)*y(197)*y(14);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(46);
dydt(46)=dydt(46)-k(21)*y(197)*y(46);
dydt(207)=dydt(207)+k(21)*y(197)*y(46);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(53);
dydt(53)=dydt(53)-k(21)*y(197)*y(53);
dydt(206)=dydt(206)+k(21)*y(197)*y(53);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(197)=dydt(197)-k(21)*y(197)*y(53);
dydt(53)=dydt(53)-k(21)*y(197)*y(53);
dydt(205)=dydt(205)+k(21)*y(197)*y(53);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(60);
dydt(60)=dydt(60)-k(21)*y(197)*y(60);
dydt(204)=dydt(204)+k(21)*y(197)*y(60);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(75);
dydt(75)=dydt(75)-k(21)*y(197)*y(75);
dydt(203)=dydt(203)+k(21)*y(197)*y(75);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(97);
dydt(97)=dydt(97)-k(21)*y(197)*y(97);
dydt(202)=dydt(202)+k(21)*y(197)*y(97);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

dydt(197)=dydt(197)-k(21)*y(197)*y(117);
dydt(117)=dydt(117)-k(21)*y(197)*y(117);
dydt(201)=dydt(201)+k(21)*y(197)*y(117);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

dydt(197)=dydt(197)-k(21)*y(197)*y(125);
dydt(125)=dydt(125)-k(21)*y(197)*y(125);
dydt(200)=dydt(200)+k(21)*y(197)*y(125);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(146);
dydt(146)=dydt(146)-k(21)*y(197)*y(146);
dydt(199)=dydt(199)+k(21)*y(197)*y(146);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(147);
dydt(147)=dydt(147)-k(21)*y(197)*y(147);
dydt(196)=dydt(196)+k(21)*y(197)*y(147);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(147);
dydt(147)=dydt(147)-k(21)*y(197)*y(147);
dydt(198)=dydt(198)+k(21)*y(197)*y(147);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(197)=dydt(197)-k(21)*y(197)*y(154);
dydt(154)=dydt(154)-k(21)*y(197)*y(154);
dydt(163)=dydt(163)+k(21)*y(197)*y(154);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(190)=dydt(190)-k(34)*y(190);
dydt(168)=dydt(168)+k(34)*y(190);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(190)=dydt(190)-k(34)*y(190);
dydt(169)=dydt(169)+k(34)*y(190);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(190)=dydt(190)-k(33)*y(190);
dydt(183)=dydt(183)+k(33)*y(190);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

dydt(190)=dydt(190)-k(22)*y(190);
dydt(197)=dydt(197)+k(22)*y(190);
dydt(189)=dydt(189)+k(22)*y(190);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(190)=dydt(190)-k(22)*y(190);
dydt(182)=dydt(182)+k(22)*y(190);
dydt(196)=dydt(196)+k(22)*y(190);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(190)=dydt(190)-k(12)*y(190);
dydt(195)=dydt(195)+k(12)*y(190);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

dydt(190)=dydt(190)-k(10)*y(190);
dydt(3)=dydt(3)+k(10)*y(190);
dydt(194)=dydt(194)+k(10)*y(190);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(190);
dydt(190)=dydt(190)-k(9)*y(3)*y(190);
dydt(193)=dydt(193)+k(9)*y(3)*y(190);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

dydt(190)=dydt(190)-k(8)*y(190);
dydt(192)=dydt(192)+k(8)*y(190);
dydt(185)=dydt(185)+k(8)*y(190);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

dydt(190)=dydt(190)-k(6)*y(190);
dydt(191)=dydt(191)+k(6)*y(190);
dydt(184)=dydt(184)+k(6)*y(190);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(183)=dydt(183)-k(34)*y(183);
dydt(174)=dydt(174)+k(34)*y(183);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(183)=dydt(183)-k(34)*y(183);
dydt(175)=dydt(175)+k(34)*y(183);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

dydt(183)=dydt(183)-k(24)*y(183);
dydt(190)=dydt(190)+k(24)*y(183);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(183)=dydt(183)-k(22)*y(183);
dydt(182)=dydt(182)+k(22)*y(183);
dydt(189)=dydt(189)+k(22)*y(183);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

dydt(183)=dydt(183)-k(22)*y(183);
dydt(182)=dydt(182)+k(22)*y(183);
dydt(181)=dydt(181)+k(22)*y(183);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(183)=dydt(183)-k(12)*y(183);
dydt(188)=dydt(188)+k(12)*y(183);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(183)=dydt(183)-k(10)*y(183);
dydt(3)=dydt(3)+k(10)*y(183);
dydt(187)=dydt(187)+k(10)*y(183);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(183);
dydt(183)=dydt(183)-k(9)*y(3)*y(183);
dydt(186)=dydt(186)+k(9)*y(3)*y(183);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(183)=dydt(183)-k(8)*y(183);
dydt(177)=dydt(177)+k(8)*y(183);
dydt(185)=dydt(185)+k(8)*y(183);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(183)=dydt(183)-k(6)*y(183);
dydt(176)=dydt(176)+k(6)*y(183);
dydt(184)=dydt(184)+k(6)*y(183);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(175)=dydt(175)-k(34)*y(175);
dydt(170)=dydt(170)+k(34)*y(175);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

dydt(175)=dydt(175)-k(29)*y(175);
dydt(183)=dydt(183)+k(29)*y(175);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(175)=dydt(175)-k(24)*y(175);
dydt(169)=dydt(169)+k(24)*y(175);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

dydt(175)=dydt(175)-k(22)*y(175);
dydt(182)=dydt(182)+k(22)*y(175);
dydt(145)=dydt(145)+k(22)*y(175);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(175)=dydt(175)-k(22)*y(175);
dydt(2)=dydt(2)+k(22)*y(175);
dydt(181)=dydt(181)+k(22)*y(175);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(175)=dydt(175)-k(12)*y(175);
dydt(180)=dydt(180)+k(12)*y(175);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

dydt(175)=dydt(175)-k(10)*y(175);
dydt(3)=dydt(3)+k(10)*y(175);
dydt(179)=dydt(179)+k(10)*y(175);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(175);
dydt(175)=dydt(175)-k(9)*y(3)*y(175);
dydt(178)=dydt(178)+k(9)*y(3)*y(175);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

dydt(175)=dydt(175)-k(8)*y(175);
dydt(177)=dydt(177)+k(8)*y(175);
dydt(141)=dydt(141)+k(8)*y(175);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

dydt(175)=dydt(175)-k(6)*y(175);
dydt(176)=dydt(176)+k(6)*y(175);
dydt(140)=dydt(140)+k(6)*y(175);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

dydt(170)=dydt(170)-k(28)*y(170);
dydt(175)=dydt(175)+k(28)*y(170);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

dydt(170)=dydt(170)-k(27)*y(170);
dydt(174)=dydt(174)+k(27)*y(170);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(170)=dydt(170)-k(24)*y(170);
dydt(164)=dydt(164)+k(24)*y(170);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(170)=dydt(170)-k(22)*y(170);
dydt(2)=dydt(2)+k(22)*y(170);
dydt(145)=dydt(145)+k(22)*y(170);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(170)=dydt(170)-k(22)*y(170);
dydt(2)=dydt(2)+k(22)*y(170);
dydt(152)=dydt(152)+k(22)*y(170);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(170)=dydt(170)-k(12)*y(170);
dydt(173)=dydt(173)+k(12)*y(170);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(170)=dydt(170)-k(10)*y(170);
dydt(3)=dydt(3)+k(10)*y(170);
dydt(172)=dydt(172)+k(10)*y(170);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(170);
dydt(170)=dydt(170)-k(9)*y(3)*y(170);
dydt(171)=dydt(171)+k(9)*y(3)*y(170);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

dydt(170)=dydt(170)-k(8)*y(170);
dydt(21)=dydt(21)+k(8)*y(170);
dydt(141)=dydt(141)+k(8)*y(170);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

dydt(170)=dydt(170)-k(6)*y(170);
dydt(89)=dydt(89)+k(6)*y(170);
dydt(140)=dydt(140)+k(6)*y(170);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(164)=dydt(164)-k(33)*y(164);
dydt(170)=dydt(170)+k(33)*y(164);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(164)=dydt(164)-k(28)*y(164);
dydt(169)=dydt(169)+k(28)*y(164);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(164)=dydt(164)-k(27)*y(164);
dydt(168)=dydt(168)+k(27)*y(164);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(164)=dydt(164)-k(22)*y(164);
dydt(32)=dydt(32)+k(22)*y(164);
dydt(145)=dydt(145)+k(22)*y(164);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

dydt(164)=dydt(164)-k(22)*y(164);
dydt(2)=dydt(2)+k(22)*y(164);
dydt(153)=dydt(153)+k(22)*y(164);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(164)=dydt(164)-k(12)*y(164);
dydt(167)=dydt(167)+k(12)*y(164);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(164)=dydt(164)-k(10)*y(164);
dydt(3)=dydt(3)+k(10)*y(164);
dydt(166)=dydt(166)+k(10)*y(164);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(164);
dydt(164)=dydt(164)-k(9)*y(3)*y(164);
dydt(165)=dydt(165)+k(9)*y(3)*y(164);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(164)=dydt(164)-k(8)*y(164);
dydt(27)=dydt(27)+k(8)*y(164);
dydt(141)=dydt(141)+k(8)*y(164);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(164)=dydt(164)-k(6)*y(164);
dydt(86)=dydt(86)+k(6)*y(164);
dydt(140)=dydt(140)+k(6)*y(164);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(158)=dydt(158)-k(33)*y(158);
dydt(157)=dydt(157)+k(33)*y(158);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(158)=dydt(158)-k(33)*y(158);
dydt(164)=dydt(164)+k(33)*y(158);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

dydt(158)=dydt(158)-k(28)*y(158);
dydt(163)=dydt(163)+k(28)*y(158);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

dydt(158)=dydt(158)-k(27)*y(158);
dydt(162)=dydt(162)+k(27)*y(158);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(158)=dydt(158)-k(22)*y(158);
dydt(32)=dydt(32)+k(22)*y(158);
dydt(154)=dydt(154)+k(22)*y(158);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

dydt(158)=dydt(158)-k(22)*y(158);
dydt(32)=dydt(32)+k(22)*y(158);
dydt(153)=dydt(153)+k(22)*y(158);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(158)=dydt(158)-k(12)*y(158);
dydt(161)=dydt(161)+k(12)*y(158);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

dydt(158)=dydt(158)-k(10)*y(158);
dydt(3)=dydt(3)+k(10)*y(158);
dydt(160)=dydt(160)+k(10)*y(158);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(158);
dydt(158)=dydt(158)-k(9)*y(3)*y(158);
dydt(159)=dydt(159)+k(9)*y(3)*y(158);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

dydt(158)=dydt(158)-k(8)*y(158);
dydt(27)=dydt(27)+k(8)*y(158);
dydt(132)=dydt(132)+k(8)*y(158);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

dydt(158)=dydt(158)-k(6)*y(158);
dydt(86)=dydt(86)+k(6)*y(158);
dydt(131)=dydt(131)+k(6)*y(158);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(154)=dydt(154)-k(33)*y(154);
dydt(145)=dydt(145)+k(33)*y(154);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(154)=dydt(154)-k(32)*y(154);
dydt(138)=dydt(138)+k(32)*y(154);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(154)=dydt(154)-k(22)*y(154);
dydt(32)=dydt(32)+k(22)*y(154);
dydt(147)=dydt(147)+k(22)*y(154);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(154);
dydt(154)=dydt(154)-k(21)*y(32)*y(154);
dydt(158)=dydt(158)+k(21)*y(32)*y(154);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(154);
dydt(154)=dydt(154)-k(21)*y(2)*y(154);
dydt(157)=dydt(157)+k(21)*y(2)*y(154);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(154)=dydt(154)-k(12)*y(154);
dydt(137)=dydt(137)+k(12)*y(154);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(154)=dydt(154)-k(10)*y(154);
dydt(3)=dydt(3)+k(10)*y(154);
dydt(156)=dydt(156)+k(10)*y(154);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(154);
dydt(154)=dydt(154)-k(9)*y(3)*y(154);
dydt(155)=dydt(155)+k(9)*y(3)*y(154);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

dydt(154)=dydt(154)-k(8)*y(154);
dydt(15)=dydt(15)+k(8)*y(154);
dydt(132)=dydt(132)+k(8)*y(154);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(154)=dydt(154)-k(6)*y(154);
dydt(98)=dydt(98)+k(6)*y(154);
dydt(131)=dydt(131)+k(6)*y(154);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(147)=dydt(147)-k(32)*y(147);
dydt(146)=dydt(146)+k(32)*y(147);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(147)=dydt(147)-k(32)*y(147);
dydt(100)=dydt(100)+k(32)*y(147);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(147);
dydt(147)=dydt(147)-k(21)*y(32)*y(147);
dydt(154)=dydt(154)+k(21)*y(32)*y(147);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(147);
dydt(147)=dydt(147)-k(21)*y(2)*y(147);
dydt(145)=dydt(145)+k(21)*y(2)*y(147);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(147);
dydt(147)=dydt(147)-k(21)*y(32)*y(147);
dydt(153)=dydt(153)+k(21)*y(32)*y(147);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(147);
dydt(147)=dydt(147)-k(21)*y(2)*y(147);
dydt(152)=dydt(152)+k(21)*y(2)*y(147);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

dydt(147)=dydt(147)-k(12)*y(147);
dydt(151)=dydt(151)+k(12)*y(147);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

dydt(147)=dydt(147)-k(10)*y(147);
dydt(3)=dydt(3)+k(10)*y(147);
dydt(150)=dydt(150)+k(10)*y(147);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(147);
dydt(147)=dydt(147)-k(9)*y(3)*y(147);
dydt(149)=dydt(149)+k(9)*y(3)*y(147);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) 

dydt(147)=dydt(147)-k(8)*y(147);
dydt(15)=dydt(15)+k(8)*y(147);
dydt(126)=dydt(126)+k(8)*y(147);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

dydt(147)=dydt(147)-k(6)*y(147);
dydt(98)=dydt(98)+k(6)*y(147);
dydt(48)=dydt(48)+k(6)*y(147);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(148)=dydt(148)-k(14)*y(148);
dydt(100)=dydt(100)+k(14)*y(148);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(148)=dydt(148)-k(12)*y(148);
dydt(124)=dydt(124)+k(12)*y(148);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

dydt(148)=dydt(148)-k(10)*y(148);
dydt(3)=dydt(3)+k(10)*y(148);
dydt(12)=dydt(12)+k(10)*y(148);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(148);
dydt(148)=dydt(148)-k(9)*y(3)*y(148);
dydt(10)=dydt(10)+k(9)*y(3)*y(148);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) 

dydt(148)=dydt(148)-k(8)*y(148);
dydt(1)=dydt(1)+k(8)*y(148);
dydt(8)=dydt(8)+k(8)*y(148);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(148)=dydt(148)-k(6)*y(148);
dydt(6)=dydt(6)+k(6)*y(148);
dydt(7)=dydt(7)+k(6)*y(148);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

dydt(146)=dydt(146)-k(32)*y(146);
dydt(148)=dydt(148)+k(32)*y(146);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(146);
dydt(146)=dydt(146)-k(21)*y(32)*y(146);
dydt(138)=dydt(138)+k(21)*y(32)*y(146);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(146);
dydt(146)=dydt(146)-k(21)*y(2)*y(146);
dydt(144)=dydt(144)+k(21)*y(2)*y(146);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) 

dydt(146)=dydt(146)-k(14)*y(146);
dydt(147)=dydt(147)+k(14)*y(146);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

dydt(146)=dydt(146)-k(12)*y(146);
dydt(128)=dydt(128)+k(12)*y(146);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

dydt(146)=dydt(146)-k(10)*y(146);
dydt(3)=dydt(3)+k(10)*y(146);
dydt(18)=dydt(18)+k(10)*y(146);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(146);
dydt(146)=dydt(146)-k(9)*y(3)*y(146);
dydt(17)=dydt(17)+k(9)*y(3)*y(146);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) 

dydt(146)=dydt(146)-k(8)*y(146);
dydt(1)=dydt(1)+k(8)*y(146);
dydt(126)=dydt(126)+k(8)*y(146);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(146)=dydt(146)-k(6)*y(146);
dydt(6)=dydt(6)+k(6)*y(146);
dydt(48)=dydt(48)+k(6)*y(146);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(144)=dydt(144)-k(22)*y(144);
dydt(2)=dydt(2)+k(22)*y(144);
dydt(146)=dydt(146)+k(22)*y(144);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(144)=dydt(144)-k(14)*y(144);
dydt(145)=dydt(145)+k(14)*y(144);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(144)=dydt(144)-k(12)*y(144);
dydt(139)=dydt(139)+k(12)*y(144);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(144)=dydt(144)-k(10)*y(144);
dydt(3)=dydt(3)+k(10)*y(144);
dydt(24)=dydt(24)+k(10)*y(144);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(144);
dydt(144)=dydt(144)-k(9)*y(3)*y(144);
dydt(23)=dydt(23)+k(9)*y(3)*y(144);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

dydt(144)=dydt(144)-k(8)*y(144);
dydt(1)=dydt(1)+k(8)*y(144);
dydt(141)=dydt(141)+k(8)*y(144);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(144)=dydt(144)-k(6)*y(144);
dydt(6)=dydt(6)+k(6)*y(144);
dydt(140)=dydt(140)+k(6)*y(144);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(139)=dydt(139)-k(31)*y(139);
dydt(144)=dydt(144)+k(31)*y(139);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(139)=dydt(139)-k(22)*y(139);
dydt(2)=dydt(2)+k(22)*y(139);
dydt(128)=dydt(128)+k(22)*y(139);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(139);
dydt(139)=dydt(139)-k(15)*y(3)*y(139);
dydt(129)=dydt(129)+k(15)*y(3)*y(139);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(139)=dydt(139)-k(14)*y(139);
dydt(143)=dydt(143)+k(14)*y(139);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(139)=dydt(139)-k(10)*y(139);
dydt(3)=dydt(3)+k(10)*y(139);
dydt(142)=dydt(142)+k(10)*y(139);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

dydt(139)=dydt(139)-k(8)*y(139);
dydt(72)=dydt(72)+k(8)*y(139);
dydt(141)=dydt(141)+k(8)*y(139);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(139)=dydt(139)-k(6)*y(139);
dydt(71)=dydt(71)+k(6)*y(139);
dydt(140)=dydt(140)+k(6)*y(139);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

dydt(135)=dydt(135)-k(33)*y(135);
dydt(139)=dydt(139)+k(33)*y(135);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(135)=dydt(135)-k(31)*y(135);
dydt(138)=dydt(138)+k(31)*y(135);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(135)=dydt(135)-k(22)*y(135);
dydt(32)=dydt(32)+k(22)*y(135);
dydt(128)=dydt(128)+k(22)*y(135);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(15)*y(3)*y(135);
dydt(135)=dydt(135)-k(15)*y(3)*y(135);
dydt(130)=dydt(130)+k(15)*y(3)*y(135);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

dydt(135)=dydt(135)-k(14)*y(135);
dydt(137)=dydt(137)+k(14)*y(135);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(135)=dydt(135)-k(10)*y(135);
dydt(3)=dydt(3)+k(10)*y(135);
dydt(136)=dydt(136)+k(10)*y(135);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

dydt(135)=dydt(135)-k(8)*y(135);
dydt(72)=dydt(72)+k(8)*y(135);
dydt(132)=dydt(132)+k(8)*y(135);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(135)=dydt(135)-k(6)*y(135);
dydt(71)=dydt(71)+k(6)*y(135);
dydt(131)=dydt(131)+k(6)*y(135);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(130)=dydt(130)-k(33)*y(130);
dydt(129)=dydt(129)+k(33)*y(130);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

dydt(130)=dydt(130)-k(22)*y(130);
dydt(32)=dydt(32)+k(22)*y(130);
dydt(125)=dydt(125)+k(22)*y(130);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(130)=dydt(130)-k(16)*y(130);
dydt(3)=dydt(3)+k(16)*y(130);
dydt(135)=dydt(135)+k(16)*y(130);

% rule    : R5_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

dydt(130)=dydt(130)-k(14)*y(130);
dydt(134)=dydt(134)+k(14)*y(130);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(130)=dydt(130)-k(10)*y(130);
dydt(3)=dydt(3)+k(10)*y(130);
dydt(133)=dydt(133)+k(10)*y(130);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(130)=dydt(130)-k(8)*y(130);
dydt(61)=dydt(61)+k(8)*y(130);
dydt(132)=dydt(132)+k(8)*y(130);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

dydt(130)=dydt(130)-k(6)*y(130);
dydt(122)=dydt(122)+k(6)*y(130);
dydt(131)=dydt(131)+k(6)*y(130);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(125)=dydt(125)-k(32)*y(125);
dydt(121)=dydt(121)+k(32)*y(125);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(125);
dydt(125)=dydt(125)-k(21)*y(32)*y(125);
dydt(130)=dydt(130)+k(21)*y(32)*y(125);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(125);
dydt(125)=dydt(125)-k(21)*y(2)*y(125);
dydt(129)=dydt(129)+k(21)*y(2)*y(125);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) 

dydt(125)=dydt(125)-k(16)*y(125);
dydt(3)=dydt(3)+k(16)*y(125);
dydt(128)=dydt(128)+k(16)*y(125);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(125)=dydt(125)-k(14)*y(125);
dydt(120)=dydt(120)+k(14)*y(125);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) 

dydt(125)=dydt(125)-k(10)*y(125);
dydt(3)=dydt(3)+k(10)*y(125);
dydt(127)=dydt(127)+k(10)*y(125);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(125)=dydt(125)-k(8)*y(125);
dydt(61)=dydt(61)+k(8)*y(125);
dydt(126)=dydt(126)+k(8)*y(125);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(125)=dydt(125)-k(6)*y(125);
dydt(48)=dydt(48)+k(6)*y(125);
dydt(122)=dydt(122)+k(6)*y(125);

% rule    : R8
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(121)=dydt(121)-k(19)*y(121);
dydt(125)=dydt(125)+k(19)*y(121);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) 

dydt(121)=dydt(121)-k(16)*y(121);
dydt(3)=dydt(3)+k(16)*y(121);
dydt(124)=dydt(124)+k(16)*y(121);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(121)=dydt(121)-k(14)*y(121);
dydt(117)=dydt(117)+k(14)*y(121);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) 

dydt(121)=dydt(121)-k(10)*y(121);
dydt(3)=dydt(3)+k(10)*y(121);
dydt(123)=dydt(123)+k(10)*y(121);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(121)=dydt(121)-k(8)*y(121);
dydt(61)=dydt(61)+k(8)*y(121);
dydt(8)=dydt(8)+k(8)*y(121);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(121)=dydt(121)-k(6)*y(121);
dydt(7)=dydt(7)+k(6)*y(121);
dydt(122)=dydt(122)+k(6)*y(121);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(117)=dydt(117)-k(32)*y(117);
dydt(121)=dydt(121)+k(32)*y(117);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(32)=dydt(32)-k(21)*y(32)*y(117);
dydt(117)=dydt(117)-k(21)*y(32)*y(117);
dydt(107)=dydt(107)+k(21)*y(32)*y(117);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(2)=dydt(2)-k(21)*y(2)*y(117);
dydt(117)=dydt(117)-k(21)*y(2)*y(117);
dydt(113)=dydt(113)+k(21)*y(2)*y(117);

% rule    : R8
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

dydt(117)=dydt(117)-k(19)*y(117);
dydt(120)=dydt(120)+k(19)*y(117);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) 

dydt(117)=dydt(117)-k(16)*y(117);
dydt(3)=dydt(3)+k(16)*y(117);
dydt(108)=dydt(108)+k(16)*y(117);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) 

dydt(117)=dydt(117)-k(10)*y(117);
dydt(3)=dydt(3)+k(10)*y(117);
dydt(119)=dydt(119)+k(10)*y(117);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(117)=dydt(117)-k(8)*y(117);
dydt(54)=dydt(54)+k(8)*y(117);
dydt(8)=dydt(8)+k(8)*y(117);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(117)=dydt(117)-k(6)*y(117);
dydt(7)=dydt(7)+k(6)*y(117);
dydt(118)=dydt(118)+k(6)*y(117);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(113)=dydt(113)-k(24)*y(113);
dydt(107)=dydt(107)+k(24)*y(113);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

dydt(113)=dydt(113)-k(22)*y(113);
dydt(2)=dydt(2)+k(22)*y(113);
dydt(117)=dydt(117)+k(22)*y(113);

% rule    : R8
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(113)=dydt(113)-k(19)*y(113);
dydt(116)=dydt(116)+k(19)*y(113);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(113)=dydt(113)-k(16)*y(113);
dydt(3)=dydt(3)+k(16)*y(113);
dydt(109)=dydt(109)+k(16)*y(113);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

dydt(113)=dydt(113)-k(10)*y(113);
dydt(3)=dydt(3)+k(10)*y(113);
dydt(115)=dydt(115)+k(10)*y(113);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(113)=dydt(113)-k(8)*y(113);
dydt(8)=dydt(8)+k(8)*y(113);
dydt(43)=dydt(43)+k(8)*y(113);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(113)=dydt(113)-k(6)*y(113);
dydt(7)=dydt(7)+k(6)*y(113);
dydt(114)=dydt(114)+k(6)*y(113);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(109)=dydt(109)-k(31)*y(109);
dydt(96)=dydt(96)+k(31)*y(109);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(109)=dydt(109)-k(24)*y(109);
dydt(105)=dydt(105)+k(24)*y(109);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(109)=dydt(109)-k(22)*y(109);
dydt(2)=dydt(2)+k(22)*y(109);
dydt(108)=dydt(108)+k(22)*y(109);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(109);
dydt(109)=dydt(109)-k(15)*y(3)*y(109);
dydt(113)=dydt(113)+k(15)*y(3)*y(109);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(109)=dydt(109)-k(10)*y(109);
dydt(3)=dydt(3)+k(10)*y(109);
dydt(112)=dydt(112)+k(10)*y(109);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(109)=dydt(109)-k(8)*y(109);
dydt(8)=dydt(8)+k(8)*y(109);
dydt(111)=dydt(111)+k(8)*y(109);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(109)=dydt(109)-k(6)*y(109);
dydt(7)=dydt(7)+k(6)*y(109);
dydt(110)=dydt(110)+k(6)*y(109);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(105)=dydt(105)-k(33)*y(105);
dydt(109)=dydt(109)+k(33)*y(105);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(105)=dydt(105)-k(31)*y(105);
dydt(103)=dydt(103)+k(31)*y(105);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(105)=dydt(105)-k(22)*y(105);
dydt(32)=dydt(32)+k(22)*y(105);
dydt(108)=dydt(108)+k(22)*y(105);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(105);
dydt(105)=dydt(105)-k(15)*y(3)*y(105);
dydt(107)=dydt(107)+k(15)*y(3)*y(105);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(105)=dydt(105)-k(10)*y(105);
dydt(3)=dydt(3)+k(10)*y(105);
dydt(106)=dydt(106)+k(10)*y(105);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(105)=dydt(105)-k(8)*y(105);
dydt(8)=dydt(8)+k(8)*y(105);
dydt(33)=dydt(33)+k(8)*y(105);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(105)=dydt(105)-k(6)*y(105);
dydt(7)=dydt(7)+k(6)*y(105);
dydt(82)=dydt(82)+k(6)*y(105);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(103)=dydt(103)-k(33)*y(103);
dydt(96)=dydt(96)+k(33)*y(103);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(103)=dydt(103)-k(22)*y(103);
dydt(32)=dydt(32)+k(22)*y(103);
dydt(100)=dydt(100)+k(22)*y(103);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(103)=dydt(103)-k(12)*y(103);
dydt(105)=dydt(105)+k(12)*y(103);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(103)=dydt(103)-k(10)*y(103);
dydt(3)=dydt(3)+k(10)*y(103);
dydt(101)=dydt(101)+k(10)*y(103);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(103);
dydt(103)=dydt(103)-k(9)*y(3)*y(103);
dydt(104)=dydt(104)+k(9)*y(3)*y(103);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(103)=dydt(103)-k(8)*y(103);
dydt(8)=dydt(8)+k(8)*y(103);
dydt(27)=dydt(27)+k(8)*y(103);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(103)=dydt(103)-k(6)*y(103);
dydt(7)=dydt(7)+k(6)*y(103);
dydt(86)=dydt(86)+k(6)*y(103);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(101)=dydt(101)-k(33)*y(101);
dydt(94)=dydt(94)+k(33)*y(101);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

dydt(101)=dydt(101)-k(22)*y(101);
dydt(32)=dydt(32)+k(22)*y(101);
dydt(97)=dydt(97)+k(22)*y(101);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(101);
dydt(101)=dydt(101)-k(9)*y(3)*y(101);
dydt(103)=dydt(103)+k(9)*y(3)*y(101);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(101);
dydt(101)=dydt(101)-k(9)*y(3)*y(101);
dydt(102)=dydt(102)+k(9)*y(3)*y(101);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(101)=dydt(101)-k(8)*y(101);
dydt(5)=dydt(5)+k(8)*y(101);
dydt(27)=dydt(27)+k(8)*y(101);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

dydt(101)=dydt(101)-k(6)*y(101);
dydt(1)=dydt(1)+k(6)*y(101);
dydt(86)=dydt(86)+k(6)*y(101);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

dydt(97)=dydt(97)-k(32)*y(97);
dydt(12)=dydt(12)+k(32)*y(97);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(97);
dydt(97)=dydt(97)-k(21)*y(32)*y(97);
dydt(101)=dydt(101)+k(21)*y(32)*y(97);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(97);
dydt(97)=dydt(97)-k(21)*y(2)*y(97);
dydt(94)=dydt(94)+k(21)*y(2)*y(97);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(97);
dydt(97)=dydt(97)-k(9)*y(3)*y(97);
dydt(100)=dydt(100)+k(9)*y(3)*y(97);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(97);
dydt(97)=dydt(97)-k(9)*y(3)*y(97);
dydt(99)=dydt(99)+k(9)*y(3)*y(97);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) 

dydt(97)=dydt(97)-k(8)*y(97);
dydt(15)=dydt(15)+k(8)*y(97);
dydt(5)=dydt(5)+k(8)*y(97);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) 

dydt(97)=dydt(97)-k(6)*y(97);
dydt(1)=dydt(1)+k(6)*y(97);
dydt(98)=dydt(98)+k(6)*y(97);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(94)=dydt(94)-k(22)*y(94);
dydt(2)=dydt(2)+k(22)*y(94);
dydt(97)=dydt(97)+k(22)*y(94);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(94);
dydt(94)=dydt(94)-k(9)*y(3)*y(94);
dydt(96)=dydt(96)+k(9)*y(3)*y(94);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(94);
dydt(94)=dydt(94)-k(9)*y(3)*y(94);
dydt(95)=dydt(95)+k(9)*y(3)*y(94);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(94)=dydt(94)-k(8)*y(94);
dydt(5)=dydt(5)+k(8)*y(94);
dydt(21)=dydt(21)+k(8)*y(94);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

dydt(94)=dydt(94)-k(6)*y(94);
dydt(1)=dydt(1)+k(6)*y(94);
dydt(89)=dydt(89)+k(6)*y(94);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

dydt(90)=dydt(90)-k(31)*y(90);
dydt(94)=dydt(94)+k(31)*y(90);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

dydt(90)=dydt(90)-k(22)*y(90);
dydt(2)=dydt(2)+k(22)*y(90);
dydt(93)=dydt(93)+k(22)*y(90);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(15)*y(3)*y(90);
dydt(90)=dydt(90)-k(15)*y(3)*y(90);
dydt(88)=dydt(88)+k(15)*y(3)*y(90);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(3)=dydt(3)-k(9)*y(3)*y(90);
dydt(90)=dydt(90)-k(9)*y(3)*y(90);
dydt(92)=dydt(92)+k(9)*y(3)*y(90);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

dydt(90)=dydt(90)-k(8)*y(90);
dydt(91)=dydt(91)+k(8)*y(90);
dydt(21)=dydt(21)+k(8)*y(90);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

dydt(90)=dydt(90)-k(6)*y(90);
dydt(72)=dydt(72)+k(6)*y(90);
dydt(89)=dydt(89)+k(6)*y(90);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(88)=dydt(88)-k(26)*y(88);
dydt(85)=dydt(85)+k(26)*y(88);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

dydt(88)=dydt(88)-k(22)*y(88);
dydt(2)=dydt(2)+k(22)*y(88);
dydt(63)=dydt(63)+k(22)*y(88);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(88)=dydt(88)-k(18)*y(88);
dydt(80)=dydt(80)+k(18)*y(88);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(88)=dydt(88)-k(16)*y(88);
dydt(3)=dydt(3)+k(16)*y(88);
dydt(90)=dydt(90)+k(16)*y(88);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(88);
dydt(88)=dydt(88)-k(9)*y(3)*y(88);
dydt(65)=dydt(65)+k(9)*y(3)*y(88);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(88)=dydt(88)-k(8)*y(88);
dydt(62)=dydt(62)+k(8)*y(88);
dydt(21)=dydt(21)+k(8)*y(88);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(88)=dydt(88)-k(6)*y(88);
dydt(61)=dydt(61)+k(6)*y(88);
dydt(89)=dydt(89)+k(6)*y(88);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(85)=dydt(85)-k(33)*y(85);
dydt(88)=dydt(88)+k(33)*y(85);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

dydt(85)=dydt(85)-k(22)*y(85);
dydt(32)=dydt(32)+k(22)*y(85);
dydt(63)=dydt(63)+k(22)*y(85);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(85)=dydt(85)-k(18)*y(85);
dydt(81)=dydt(81)+k(18)*y(85);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(85)=dydt(85)-k(16)*y(85);
dydt(3)=dydt(3)+k(16)*y(85);
dydt(87)=dydt(87)+k(16)*y(85);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(85);
dydt(85)=dydt(85)-k(9)*y(3)*y(85);
dydt(66)=dydt(66)+k(9)*y(3)*y(85);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(85)=dydt(85)-k(8)*y(85);
dydt(62)=dydt(62)+k(8)*y(85);
dydt(27)=dydt(27)+k(8)*y(85);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(85)=dydt(85)-k(6)*y(85);
dydt(61)=dydt(61)+k(6)*y(85);
dydt(86)=dydt(86)+k(6)*y(85);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(81)=dydt(81)-k(33)*y(81);
dydt(80)=dydt(80)+k(33)*y(81);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(81)=dydt(81)-k(31)*y(81);
dydt(85)=dydt(85)+k(31)*y(81);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

dydt(81)=dydt(81)-k(22)*y(81);
dydt(32)=dydt(32)+k(22)*y(81);
dydt(75)=dydt(75)+k(22)*y(81);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(81)=dydt(81)-k(16)*y(81);
dydt(3)=dydt(3)+k(16)*y(81);
dydt(84)=dydt(84)+k(16)*y(81);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(81);
dydt(81)=dydt(81)-k(15)*y(3)*y(81);
dydt(83)=dydt(83)+k(15)*y(3)*y(81);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

dydt(81)=dydt(81)-k(8)*y(81);
dydt(62)=dydt(62)+k(8)*y(81);
dydt(33)=dydt(33)+k(8)*y(81);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(81)=dydt(81)-k(6)*y(81);
dydt(61)=dydt(61)+k(6)*y(81);
dydt(82)=dydt(82)+k(6)*y(81);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(75)=dydt(75)-k(32)*y(75);
dydt(70)=dydt(70)+k(32)*y(75);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(75)=dydt(75)-k(31)*y(75);
dydt(63)=dydt(63)+k(31)*y(75);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(75);
dydt(75)=dydt(75)-k(21)*y(32)*y(75);
dydt(81)=dydt(81)+k(21)*y(32)*y(75);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(75);
dydt(75)=dydt(75)-k(21)*y(2)*y(75);
dydt(80)=dydt(80)+k(21)*y(2)*y(75);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

dydt(75)=dydt(75)-k(16)*y(75);
dydt(3)=dydt(3)+k(16)*y(75);
dydt(79)=dydt(79)+k(16)*y(75);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(75);
dydt(75)=dydt(75)-k(15)*y(3)*y(75);
dydt(78)=dydt(78)+k(15)*y(3)*y(75);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) 

dydt(75)=dydt(75)-k(8)*y(75);
dydt(77)=dydt(77)+k(8)*y(75);
dydt(62)=dydt(62)+k(8)*y(75);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

dydt(75)=dydt(75)-k(6)*y(75);
dydt(76)=dydt(76)+k(6)*y(75);
dydt(61)=dydt(61)+k(6)*y(75);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

dydt(70)=dydt(70)-k(31)*y(70);
dydt(68)=dydt(68)+k(31)*y(70);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

dydt(70)=dydt(70)-k(20)*y(70);
dydt(75)=dydt(75)+k(20)*y(70);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

dydt(70)=dydt(70)-k(16)*y(70);
dydt(3)=dydt(3)+k(16)*y(70);
dydt(74)=dydt(74)+k(16)*y(70);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(70);
dydt(70)=dydt(70)-k(15)*y(3)*y(70);
dydt(73)=dydt(73)+k(15)*y(3)*y(70);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) 

dydt(70)=dydt(70)-k(8)*y(70);
dydt(72)=dydt(72)+k(8)*y(70);
dydt(62)=dydt(62)+k(8)*y(70);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

dydt(70)=dydt(70)-k(6)*y(70);
dydt(71)=dydt(71)+k(6)*y(70);
dydt(61)=dydt(61)+k(6)*y(70);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

dydt(68)=dydt(68)-k(20)*y(68);
dydt(63)=dydt(63)+k(20)*y(68);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

dydt(68)=dydt(68)-k(18)*y(68);
dydt(70)=dydt(70)+k(18)*y(68);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

dydt(68)=dydt(68)-k(16)*y(68);
dydt(3)=dydt(3)+k(16)*y(68);
dydt(69)=dydt(69)+k(16)*y(68);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(68);
dydt(68)=dydt(68)-k(9)*y(3)*y(68);
dydt(67)=dydt(67)+k(9)*y(3)*y(68);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) 

dydt(68)=dydt(68)-k(8)*y(68);
dydt(1)=dydt(1)+k(8)*y(68);
dydt(62)=dydt(62)+k(8)*y(68);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(68)=dydt(68)-k(6)*y(68);
dydt(6)=dydt(6)+k(6)*y(68);
dydt(61)=dydt(61)+k(6)*y(68);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(67)=dydt(67)-k(20)*y(67);
dydt(60)=dydt(60)+k(20)*y(67);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) 

dydt(67)=dydt(67)-k(16)*y(67);
dydt(3)=dydt(3)+k(16)*y(67);
dydt(13)=dydt(13)+k(16)*y(67);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(67)=dydt(67)-k(13)*y(67);
dydt(41)=dydt(41)+k(13)*y(67);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) 

dydt(67)=dydt(67)-k(10)*y(67);
dydt(3)=dydt(3)+k(10)*y(67);
dydt(68)=dydt(68)+k(10)*y(67);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(67)=dydt(67)-k(8)*y(67);
dydt(7)=dydt(7)+k(8)*y(67);
dydt(62)=dydt(62)+k(8)*y(67);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(67)=dydt(67)-k(6)*y(67);
dydt(61)=dydt(61)+k(6)*y(67);
dydt(9)=dydt(9)+k(6)*y(67);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(60)=dydt(60)-k(32)*y(60);
dydt(67)=dydt(67)+k(32)*y(60);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(60);
dydt(60)=dydt(60)-k(21)*y(32)*y(60);
dydt(66)=dydt(66)+k(21)*y(32)*y(60);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(60);
dydt(60)=dydt(60)-k(21)*y(2)*y(60);
dydt(65)=dydt(65)+k(21)*y(2)*y(60);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) 

dydt(60)=dydt(60)-k(16)*y(60);
dydt(3)=dydt(3)+k(16)*y(60);
dydt(64)=dydt(64)+k(16)*y(60);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(60)=dydt(60)-k(13)*y(60);
dydt(53)=dydt(53)+k(13)*y(60);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) 

dydt(60)=dydt(60)-k(10)*y(60);
dydt(3)=dydt(3)+k(10)*y(60);
dydt(63)=dydt(63)+k(10)*y(60);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(60)=dydt(60)-k(8)*y(60);
dydt(48)=dydt(48)+k(8)*y(60);
dydt(62)=dydt(62)+k(8)*y(60);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

dydt(60)=dydt(60)-k(6)*y(60);
dydt(61)=dydt(61)+k(6)*y(60);
dydt(47)=dydt(47)+k(6)*y(60);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(53)=dydt(53)-k(32)*y(53);
dydt(41)=dydt(41)+k(32)*y(53);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

dydt(53)=dydt(53)-k(32)*y(53);
dydt(60)=dydt(60)+k(32)*y(53);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(53);
dydt(53)=dydt(53)-k(21)*y(32)*y(53);
dydt(40)=dydt(40)+k(21)*y(32)*y(53);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(53);
dydt(53)=dydt(53)-k(21)*y(2)*y(53);
dydt(46)=dydt(46)+k(21)*y(2)*y(53);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(53);
dydt(53)=dydt(53)-k(21)*y(32)*y(53);
dydt(59)=dydt(59)+k(21)*y(32)*y(53);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(53);
dydt(53)=dydt(53)-k(21)*y(2)*y(53);
dydt(58)=dydt(58)+k(21)*y(2)*y(53);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) 

dydt(53)=dydt(53)-k(16)*y(53);
dydt(3)=dydt(3)+k(16)*y(53);
dydt(57)=dydt(57)+k(16)*y(53);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) 

dydt(53)=dydt(53)-k(10)*y(53);
dydt(3)=dydt(3)+k(10)*y(53);
dydt(56)=dydt(56)+k(10)*y(53);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(53)=dydt(53)-k(8)*y(53);
dydt(48)=dydt(48)+k(8)*y(53);
dydt(55)=dydt(55)+k(8)*y(53);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

dydt(53)=dydt(53)-k(6)*y(53);
dydt(54)=dydt(54)+k(6)*y(53);
dydt(47)=dydt(47)+k(6)*y(53);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(46)=dydt(46)-k(32)*y(46);
dydt(42)=dydt(42)+k(32)*y(46);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(46)=dydt(46)-k(23)*y(46);
dydt(40)=dydt(40)+k(23)*y(46);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(46)=dydt(46)-k(22)*y(46);
dydt(2)=dydt(2)+k(22)*y(46);
dydt(53)=dydt(53)+k(22)*y(46);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(32)=dydt(32)-k(21)*y(32)*y(46);
dydt(46)=dydt(46)-k(21)*y(32)*y(46);
dydt(52)=dydt(52)+k(21)*y(32)*y(46);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(2)=dydt(2)-k(21)*y(2)*y(46);
dydt(46)=dydt(46)-k(21)*y(2)*y(46);
dydt(51)=dydt(51)+k(21)*y(2)*y(46);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(46)=dydt(46)-k(16)*y(46);
dydt(3)=dydt(3)+k(16)*y(46);
dydt(50)=dydt(50)+k(16)*y(46);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(46)=dydt(46)-k(10)*y(46);
dydt(3)=dydt(3)+k(10)*y(46);
dydt(49)=dydt(49)+k(10)*y(46);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

dydt(46)=dydt(46)-k(8)*y(46);
dydt(48)=dydt(48)+k(8)*y(46);
dydt(44)=dydt(44)+k(8)*y(46);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(46)=dydt(46)-k(6)*y(46);
dydt(47)=dydt(47)+k(6)*y(46);
dydt(43)=dydt(43)+k(6)*y(46);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(42)=dydt(42)-k(23)*y(42);
dydt(36)=dydt(36)+k(23)*y(42);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(42)=dydt(42)-k(22)*y(42);
dydt(2)=dydt(2)+k(22)*y(42);
dydt(41)=dydt(41)+k(22)*y(42);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(42)=dydt(42)-k(20)*y(42);
dydt(46)=dydt(46)+k(20)*y(42);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(42)=dydt(42)-k(16)*y(42);
dydt(3)=dydt(3)+k(16)*y(42);
dydt(25)=dydt(25)+k(16)*y(42);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

dydt(42)=dydt(42)-k(10)*y(42);
dydt(3)=dydt(3)+k(10)*y(42);
dydt(45)=dydt(45)+k(10)*y(42);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(42)=dydt(42)-k(8)*y(42);
dydt(7)=dydt(7)+k(8)*y(42);
dydt(44)=dydt(44)+k(8)*y(42);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(42)=dydt(42)-k(6)*y(42);
dydt(9)=dydt(9)+k(6)*y(42);
dydt(43)=dydt(43)+k(6)*y(42);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

dydt(36)=dydt(36)-k(33)*y(36);
dydt(42)=dydt(42)+k(33)*y(36);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(36)=dydt(36)-k(22)*y(36);
dydt(32)=dydt(32)+k(22)*y(36);
dydt(41)=dydt(41)+k(22)*y(36);

% rule    : R8_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

dydt(36)=dydt(36)-k(20)*y(36);
dydt(40)=dydt(40)+k(20)*y(36);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(36)=dydt(36)-k(16)*y(36);
dydt(3)=dydt(3)+k(16)*y(36);
dydt(31)=dydt(31)+k(16)*y(36);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

dydt(36)=dydt(36)-k(10)*y(36);
dydt(3)=dydt(3)+k(10)*y(36);
dydt(39)=dydt(39)+k(10)*y(36);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(36)=dydt(36)-k(8)*y(36);
dydt(7)=dydt(7)+k(8)*y(36);
dydt(38)=dydt(38)+k(8)*y(36);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(36)=dydt(36)-k(6)*y(36);
dydt(9)=dydt(9)+k(6)*y(36);
dydt(37)=dydt(37)+k(6)*y(36);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(31)=dydt(31)-k(33)*y(31);
dydt(25)=dydt(25)+k(33)*y(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(31)=dydt(31)-k(31)*y(31);
dydt(26)=dydt(26)+k(31)*y(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(31)=dydt(31)-k(22)*y(31);
dydt(32)=dydt(32)+k(22)*y(31);
dydt(19)=dydt(19)+k(22)*y(31);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

dydt(3)=dydt(3)-k(15)*y(3)*y(31);
dydt(31)=dydt(31)-k(15)*y(3)*y(31);
dydt(36)=dydt(36)+k(15)*y(3)*y(31);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(31)=dydt(31)-k(10)*y(31);
dydt(3)=dydt(3)+k(10)*y(31);
dydt(35)=dydt(35)+k(10)*y(31);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(31)=dydt(31)-k(8)*y(31);
dydt(7)=dydt(7)+k(8)*y(31);
dydt(34)=dydt(34)+k(8)*y(31);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(31)=dydt(31)-k(6)*y(31);
dydt(9)=dydt(9)+k(6)*y(31);
dydt(33)=dydt(33)+k(6)*y(31);

% rule    : R18
% reaction: Syk(tSH2, l~pY, a~Y) -> Syk(tSH2, l~Y, a~Y) 

dydt(32)=dydt(32)-k(35)*y(32);
dydt(2)=dydt(2)+k(35)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(32)=dydt(32)-k(21)*y(32)*y(14);
dydt(14)=dydt(14)-k(21)*y(32)*y(14);
dydt(26)=dydt(26)+k(21)*y(32)*y(14);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(26)=dydt(26)-k(33)*y(26);
dydt(20)=dydt(20)+k(33)*y(26);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

dydt(26)=dydt(26)-k(22)*y(26);
dydt(32)=dydt(32)+k(22)*y(26);
dydt(14)=dydt(14)+k(22)*y(26);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(26)=dydt(26)-k(11)*y(26);
dydt(31)=dydt(31)+k(11)*y(26);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

dydt(26)=dydt(26)-k(10)*y(26);
dydt(3)=dydt(3)+k(10)*y(26);
dydt(30)=dydt(30)+k(10)*y(26);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(26);
dydt(26)=dydt(26)-k(9)*y(3)*y(26);
dydt(29)=dydt(29)+k(9)*y(3)*y(26);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(26)=dydt(26)-k(8)*y(26);
dydt(7)=dydt(7)+k(8)*y(26);
dydt(28)=dydt(28)+k(8)*y(26);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(26)=dydt(26)-k(6)*y(26);
dydt(9)=dydt(9)+k(6)*y(26);
dydt(27)=dydt(27)+k(6)*y(26);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

dydt(20)=dydt(20)-k(23)*y(20);
dydt(26)=dydt(26)+k(23)*y(20);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

dydt(20)=dydt(20)-k(22)*y(20);
dydt(2)=dydt(2)+k(22)*y(20);
dydt(14)=dydt(14)+k(22)*y(20);

% rule    : R4
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

dydt(20)=dydt(20)-k(11)*y(20);
dydt(25)=dydt(25)+k(11)*y(20);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

dydt(20)=dydt(20)-k(10)*y(20);
dydt(3)=dydt(3)+k(10)*y(20);
dydt(24)=dydt(24)+k(10)*y(20);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(20);
dydt(20)=dydt(20)-k(9)*y(3)*y(20);
dydt(23)=dydt(23)+k(9)*y(3)*y(20);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(20)=dydt(20)-k(8)*y(20);
dydt(7)=dydt(7)+k(8)*y(20);
dydt(22)=dydt(22)+k(8)*y(20);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(20)=dydt(20)-k(6)*y(20);
dydt(9)=dydt(9)+k(6)*y(20);
dydt(21)=dydt(21)+k(6)*y(20);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(14)=dydt(14)-k(32)*y(14);
dydt(11)=dydt(11)+k(32)*y(14);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

dydt(2)=dydt(2)-k(21)*y(2)*y(14);
dydt(14)=dydt(14)-k(21)*y(2)*y(14);
dydt(20)=dydt(20)+k(21)*y(2)*y(14);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(14)=dydt(14)-k(11)*y(14);
dydt(19)=dydt(19)+k(11)*y(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

dydt(14)=dydt(14)-k(10)*y(14);
dydt(3)=dydt(3)+k(10)*y(14);
dydt(18)=dydt(18)+k(10)*y(14);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(14);
dydt(14)=dydt(14)-k(9)*y(3)*y(14);
dydt(17)=dydt(17)+k(9)*y(3)*y(14);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

dydt(14)=dydt(14)-k(8)*y(14);
dydt(16)=dydt(16)+k(8)*y(14);
dydt(7)=dydt(7)+k(8)*y(14);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) 

dydt(14)=dydt(14)-k(6)*y(14);
dydt(15)=dydt(15)+k(6)*y(14);
dydt(9)=dydt(9)+k(6)*y(14);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(11)=dydt(11)-k(13)*y(11);
dydt(14)=dydt(14)+k(13)*y(11);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(11)=dydt(11)-k(11)*y(11);
dydt(13)=dydt(13)+k(11)*y(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

dydt(11)=dydt(11)-k(10)*y(11);
dydt(3)=dydt(3)+k(10)*y(11);
dydt(12)=dydt(12)+k(10)*y(11);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(11);
dydt(11)=dydt(11)-k(9)*y(3)*y(11);
dydt(10)=dydt(10)+k(9)*y(3)*y(11);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(11)=dydt(11)-k(8)*y(11);
dydt(5)=dydt(5)+k(8)*y(11);
dydt(7)=dydt(7)+k(8)*y(11);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) 

dydt(11)=dydt(11)-k(6)*y(11);
dydt(1)=dydt(1)+k(6)*y(11);
dydt(9)=dydt(9)+k(6)*y(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Lyn(U, SH2) 

dydt(9)=dydt(9)-k(10)*y(9);
dydt(3)=dydt(3)+k(10)*y(9);
dydt(6)=dydt(6)+k(10)*y(9);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

dydt(1)=dydt(1)-k(5)*y(1)*y(9);
dydt(9)=dydt(9)-k(5)*y(1)*y(9);
dydt(11)=dydt(11)+k(5)*y(1)*y(9);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

dydt(7)=dydt(7)-k(5)*y(7)*y(9);
dydt(9)=dydt(9)-k(5)*y(7)*y(9);
dydt(10)=dydt(10)+k(5)*y(7)*y(9);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Lig(l1, l2) 

dydt(9)=dydt(9)-k(4)*y(9);
dydt(4)=dydt(4)+k(4)*y(9);
dydt(7)=dydt(7)+k(4)*y(9);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a, b~Y, g~Y) + Lyn(U, SH2) 

dydt(7)=dydt(7)-k(10)*y(7);
dydt(3)=dydt(3)+k(10)*y(7);
dydt(1)=dydt(1)+k(10)*y(7);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

dydt(4)=dydt(4)-k(3)*y(4)*y(7);
dydt(7)=dydt(7)-k(3)*y(4)*y(7);
dydt(9)=dydt(9)+k(3)*y(4)*y(7);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

dydt(4)=dydt(4)-k(1)*y(4)*y(7);
dydt(7)=dydt(7)-k(1)*y(4)*y(7);
dydt(8)=dydt(8)+k(1)*y(4)*y(7);

% rule    : R3
% reaction: Rec(a, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

dydt(3)=dydt(3)-k(9)*y(3)*y(1);
dydt(1)=dydt(1)-k(9)*y(3)*y(1);
dydt(7)=dydt(7)+k(9)*y(3)*y(1);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

dydt(4)=dydt(4)-k(3)*y(4)*y(1);
dydt(1)=dydt(1)-k(3)*y(4)*y(1);
dydt(6)=dydt(6)+k(3)*y(4)*y(1);

% rule    : R1
% reaction: Rec(a, b~Y, g~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

dydt(4)=dydt(4)-k(1)*y(4)*y(1);
dydt(1)=dydt(1)-k(1)*y(4)*y(1);
dydt(5)=dydt(5)+k(1)*y(4)*y(1);
dydt(655)=1;

end


function jac=ode_jacobian(t,y)

global nodevar
global max_stoc_coef
global jacvar
global var
global k
global kd
global kun
global kdun
global stoc

global jack
global jackd
global jackun
global jackund
global jacstoc


k(1)=var(5);
k(2)=var(6);
k(3)=var(5);
k(4)=var(6);
k(5)=var(7);
k(6)=var(8);
k(7)=var(7);
k(8)=var(8);
k(9)=var(9);
k(10)=var(10);
k(11)=var(17);
k(12)=var(17);
k(13)=var(19);
k(14)=var(19);
k(15)=var(11);
k(16)=var(12);
k(17)=var(18);
k(18)=var(18);
k(19)=var(20);
k(20)=var(20);
k(21)=var(13);
k(22)=var(14);
k(23)=var(21);
k(24)=var(21);
k(25)=var(22);
k(26)=var(22);
k(27)=var(23);
k(28)=var(23);
k(29)=var(24);
k(30)=var(24);
k(31)=var(25);
k(32)=var(25);
k(33)=var(25);
k(34)=var(25);
k(35)=var(26);
k(36)=var(26);


jac=sparse(nodevar,nodevar);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(5);
jac(3,5)=jac(3,5)-k(9)*y(3);
jac(5,3)=jac(5,3)-k(9)*y(5);
jac(5,5)=jac(5,5)-k(9)*y(3);
jac(8,3)=jac(8,3)+k(9)*y(5);
jac(8,5)=jac(8,5)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(1,1)=jac(1,1)-k(7)*y(5);
jac(1,5)=jac(1,5)-k(7)*y(1);
jac(5,1)=jac(5,1)-k(7)*y(5);
jac(5,5)=jac(5,5)-k(7)*y(1);
jac(12,1)=jac(12,1)+k(7)*y(5);
jac(12,5)=jac(12,5)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(5,5)=jac(5,5)-k(7)*y(7);
jac(5,7)=jac(5,7)-k(7)*y(5);
jac(7,5)=jac(7,5)-k(7)*y(7);
jac(7,7)=jac(7,7)-k(7)*y(5);
jac(11,5)=jac(11,5)+k(7)*y(7);
jac(11,7)=jac(11,7)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(5,5)=jac(5,5)-k(7)*y(335);
jac(5,335)=jac(5,335)-k(7)*y(5);
jac(335,5)=jac(335,5)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(5);
jac(413,5)=jac(413,5)+k(7)*y(335);
jac(413,335)=jac(413,335)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(5,5)=jac(5,5)-k(7)*y(257);
jac(5,257)=jac(5,257)-k(7)*y(5);
jac(257,5)=jac(257,5)-k(7)*y(257);
jac(257,257)=jac(257,257)-k(7)*y(5);
jac(587,5)=jac(587,5)+k(7)*y(257);
jac(587,257)=jac(587,257)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(5,5)=jac(5,5)-k(7)*y(227);
jac(5,227)=jac(5,227)-k(7)*y(5);
jac(227,5)=jac(227,5)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(5);
jac(589,5)=jac(589,5)+k(7)*y(227);
jac(589,227)=jac(589,227)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(5,5)=jac(5,5)-k(7)*y(222);
jac(5,222)=jac(5,222)-k(7)*y(5);
jac(222,5)=jac(222,5)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(5);
jac(591,5)=jac(591,5)+k(7)*y(222);
jac(591,222)=jac(591,222)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(5,5)=jac(5,5)-k(7)*y(218);
jac(5,218)=jac(5,218)-k(7)*y(5);
jac(218,5)=jac(218,5)-k(7)*y(218);
jac(218,218)=jac(218,218)-k(7)*y(5);
jac(304,5)=jac(304,5)+k(7)*y(218);
jac(304,218)=jac(304,218)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(5,5)=jac(5,5)-k(7)*y(192);
jac(5,192)=jac(5,192)-k(7)*y(5);
jac(192,5)=jac(192,5)-k(7)*y(192);
jac(192,192)=jac(192,192)-k(7)*y(5);
jac(202,5)=jac(202,5)+k(7)*y(192);
jac(202,192)=jac(202,192)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(5,5)=jac(5,5)-k(7)*y(184);
jac(5,184)=jac(5,184)-k(7)*y(5);
jac(184,5)=jac(184,5)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(5);
jac(416,5)=jac(416,5)+k(7)*y(184);
jac(416,184)=jac(416,184)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(5,5)=jac(5,5)-k(7)*y(177);
jac(5,177)=jac(5,177)-k(7)*y(5);
jac(177,5)=jac(177,5)-k(7)*y(177);
jac(177,177)=jac(177,177)-k(7)*y(5);
jac(418,5)=jac(418,5)+k(7)*y(177);
jac(418,177)=jac(418,177)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(5,5)=jac(5,5)-k(7)*y(140);
jac(5,140)=jac(5,140)-k(7)*y(5);
jac(140,5)=jac(140,5)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(5);
jac(95,5)=jac(95,5)+k(7)*y(140);
jac(95,140)=jac(95,140)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(5,5)=jac(5,5)-k(7)*y(131);
jac(5,131)=jac(5,131)-k(7)*y(5);
jac(131,5)=jac(131,5)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(5);
jac(102,5)=jac(102,5)+k(7)*y(131);
jac(102,131)=jac(102,131)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(5,5)=jac(5,5)-k(7)*y(111);
jac(5,111)=jac(5,111)-k(7)*y(5);
jac(111,5)=jac(111,5)-k(7)*y(111);
jac(111,111)=jac(111,111)-k(7)*y(5);
jac(112,5)=jac(112,5)+k(7)*y(111);
jac(112,111)=jac(112,111)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(77,77)=jac(77,77)-k(7)*y(5);
jac(77,5)=jac(77,5)-k(7)*y(77);
jac(5,77)=jac(5,77)-k(7)*y(5);
jac(5,5)=jac(5,5)-k(7)*y(77);
jac(590,77)=jac(590,77)+k(7)*y(5);
jac(590,5)=jac(590,5)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(72,72)=jac(72,72)-k(7)*y(5);
jac(72,5)=jac(72,5)-k(7)*y(72);
jac(5,72)=jac(5,72)-k(7)*y(5);
jac(5,5)=jac(5,5)-k(7)*y(72);
jac(333,72)=jac(333,72)+k(7)*y(5);
jac(333,5)=jac(333,5)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(5,5)=jac(5,5)-k(7)*y(61);
jac(5,61)=jac(5,61)-k(7)*y(5);
jac(61,5)=jac(61,5)-k(7)*y(61);
jac(61,61)=jac(61,61)-k(7)*y(5);
jac(123,5)=jac(123,5)+k(7)*y(61);
jac(123,61)=jac(123,61)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(5,5)=jac(5,5)-k(7)*y(54);
jac(5,54)=jac(5,54)-k(7)*y(5);
jac(54,5)=jac(54,5)-k(7)*y(54);
jac(54,54)=jac(54,54)-k(7)*y(5);
jac(119,5)=jac(119,5)+k(7)*y(54);
jac(119,54)=jac(119,54)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(5,5)=jac(5,5)-k(7)*y(48);
jac(5,48)=jac(5,48)-k(7)*y(5);
jac(48,5)=jac(48,5)-k(7)*y(48);
jac(48,48)=jac(48,48)-k(7)*y(5);
jac(99,5)=jac(99,5)+k(7)*y(48);
jac(99,48)=jac(99,48)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(5,5)=jac(5,5)-k(7)*y(43);
jac(5,43)=jac(5,43)-k(7)*y(5);
jac(43,5)=jac(43,5)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(5);
jac(115,5)=jac(115,5)+k(7)*y(43);
jac(115,43)=jac(115,43)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(5,5)=jac(5,5)-k(7)*y(37);
jac(5,37)=jac(5,37)-k(7)*y(5);
jac(37,5)=jac(37,5)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(5);
jac(592,5)=jac(592,5)+k(7)*y(37);
jac(592,37)=jac(592,37)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(5,5)=jac(5,5)-k(7)*y(33);
jac(5,33)=jac(5,33)-k(7)*y(5);
jac(33,5)=jac(33,5)-k(7)*y(33);
jac(33,33)=jac(33,33)-k(7)*y(5);
jac(106,5)=jac(106,5)+k(7)*y(33);
jac(106,33)=jac(106,33)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(5,5)=jac(5,5)-k(7)*y(27);
jac(5,27)=jac(5,27)-k(7)*y(5);
jac(27,5)=jac(27,5)-k(7)*y(27);
jac(27,27)=jac(27,27)-k(7)*y(5);
jac(101,5)=jac(101,5)+k(7)*y(27);
jac(101,27)=jac(101,27)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(5,5)=jac(5,5)-k(7)*y(21);
jac(5,21)=jac(5,21)-k(7)*y(5);
jac(21,5)=jac(21,5)-k(7)*y(21);
jac(21,21)=jac(21,21)-k(7)*y(5);
jac(94,5)=jac(94,5)+k(7)*y(21);
jac(94,21)=jac(94,21)+k(7)*y(5);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(15,15)=jac(15,15)-k(7)*y(5);
jac(15,5)=jac(15,5)-k(7)*y(15);
jac(5,15)=jac(5,15)-k(7)*y(5);
jac(5,5)=jac(5,5)-k(7)*y(15);
jac(97,15)=jac(97,15)+k(7)*y(5);
jac(97,5)=jac(97,5)+k(7)*y(15);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) -> Rec(a, b~Y, g~Y) + Lig(l1, l2) 

jac(5,5)=jac(5,5)-k(2);
jac(4,5)=jac(4,5)+k(2);
jac(1,5)=jac(1,5)+k(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(6);
jac(3,6)=jac(3,6)-k(9)*y(3);
jac(6,3)=jac(6,3)-k(9)*y(6);
jac(6,6)=jac(6,6)-k(9)*y(3);
jac(9,3)=jac(9,3)+k(9)*y(6);
jac(9,6)=jac(9,6)+k(9)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(1,1)=jac(1,1)-k(5)*y(6);
jac(1,6)=jac(1,6)-k(5)*y(1);
jac(6,1)=jac(6,1)-k(5)*y(6);
jac(6,6)=jac(6,6)-k(5)*y(1);
jac(12,1)=jac(12,1)+k(5)*y(6);
jac(12,6)=jac(12,6)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(6,6)=jac(6,6)-k(5)*y(7);
jac(6,7)=jac(6,7)-k(5)*y(6);
jac(7,6)=jac(7,6)-k(5)*y(7);
jac(7,7)=jac(7,7)-k(5)*y(6);
jac(148,6)=jac(148,6)+k(5)*y(7);
jac(148,7)=jac(148,7)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(6,6)=jac(6,6)-k(5)*y(335);
jac(6,335)=jac(6,335)-k(5)*y(6);
jac(335,6)=jac(335,6)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(6);
jac(199,6)=jac(199,6)+k(5)*y(335);
jac(199,335)=jac(199,335)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(6,6)=jac(6,6)-k(5)*y(257);
jac(6,257)=jac(6,257)-k(5)*y(6);
jac(257,6)=jac(257,6)-k(5)*y(257);
jac(257,257)=jac(257,257)-k(5)*y(6);
jac(230,6)=jac(230,6)+k(5)*y(257);
jac(230,257)=jac(230,257)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(6,6)=jac(6,6)-k(5)*y(227);
jac(6,227)=jac(6,227)-k(5)*y(6);
jac(227,6)=jac(227,6)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(6);
jac(229,6)=jac(229,6)+k(5)*y(227);
jac(229,227)=jac(229,227)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(6,6)=jac(6,6)-k(5)*y(222);
jac(6,222)=jac(6,222)-k(5)*y(6);
jac(222,6)=jac(222,6)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(6);
jac(224,6)=jac(224,6)+k(5)*y(222);
jac(224,222)=jac(224,222)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(6,6)=jac(6,6)-k(5)*y(218);
jac(6,218)=jac(6,218)-k(5)*y(6);
jac(218,6)=jac(218,6)-k(5)*y(218);
jac(218,218)=jac(218,218)-k(5)*y(6);
jac(220,6)=jac(220,6)+k(5)*y(218);
jac(220,218)=jac(220,218)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(6,6)=jac(6,6)-k(5)*y(192);
jac(6,192)=jac(6,192)-k(5)*y(6);
jac(192,6)=jac(192,6)-k(5)*y(192);
jac(192,192)=jac(192,192)-k(5)*y(6);
jac(211,6)=jac(211,6)+k(5)*y(192);
jac(211,192)=jac(211,192)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(6,6)=jac(6,6)-k(5)*y(184);
jac(6,184)=jac(6,184)-k(5)*y(6);
jac(184,6)=jac(184,6)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(6);
jac(652,6)=jac(652,6)+k(5)*y(184);
jac(652,184)=jac(652,184)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(6,6)=jac(6,6)-k(5)*y(177);
jac(6,177)=jac(6,177)-k(5)*y(6);
jac(177,6)=jac(177,6)-k(5)*y(177);
jac(177,177)=jac(177,177)-k(5)*y(6);
jac(216,6)=jac(216,6)+k(5)*y(177);
jac(216,177)=jac(216,177)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(6,6)=jac(6,6)-k(5)*y(140);
jac(6,140)=jac(6,140)-k(5)*y(6);
jac(140,6)=jac(140,6)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(6);
jac(144,6)=jac(144,6)+k(5)*y(140);
jac(144,140)=jac(144,140)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(6,6)=jac(6,6)-k(5)*y(131);
jac(6,131)=jac(6,131)-k(5)*y(6);
jac(131,6)=jac(131,6)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(6);
jac(138,6)=jac(138,6)+k(5)*y(131);
jac(138,131)=jac(138,131)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(6,6)=jac(6,6)-k(5)*y(111);
jac(6,111)=jac(6,111)-k(5)*y(6);
jac(111,6)=jac(111,6)-k(5)*y(111);
jac(111,111)=jac(111,111)-k(5)*y(6);
jac(621,6)=jac(621,6)+k(5)*y(111);
jac(621,111)=jac(621,111)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(77,77)=jac(77,77)-k(5)*y(6);
jac(77,6)=jac(77,6)-k(5)*y(77);
jac(6,77)=jac(6,77)-k(5)*y(6);
jac(6,6)=jac(6,6)-k(5)*y(77);
jac(234,77)=jac(234,77)+k(5)*y(6);
jac(234,6)=jac(234,6)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(72,72)=jac(72,72)-k(5)*y(6);
jac(72,6)=jac(72,6)-k(5)*y(72);
jac(6,72)=jac(6,72)-k(5)*y(6);
jac(6,6)=jac(6,6)-k(5)*y(72);
jac(69,72)=jac(69,72)+k(5)*y(6);
jac(69,6)=jac(69,6)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(6,6)=jac(6,6)-k(5)*y(61);
jac(6,61)=jac(6,61)-k(5)*y(6);
jac(61,6)=jac(61,6)-k(5)*y(61);
jac(61,61)=jac(61,61)-k(5)*y(6);
jac(68,6)=jac(68,6)+k(5)*y(61);
jac(68,61)=jac(68,61)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(6,6)=jac(6,6)-k(5)*y(54);
jac(6,54)=jac(6,54)-k(5)*y(6);
jac(54,6)=jac(54,6)-k(5)*y(54);
jac(54,54)=jac(54,54)-k(5)*y(6);
jac(233,6)=jac(233,6)+k(5)*y(54);
jac(233,54)=jac(233,54)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(6,6)=jac(6,6)-k(5)*y(48);
jac(6,48)=jac(6,48)-k(5)*y(6);
jac(48,6)=jac(48,6)-k(5)*y(48);
jac(48,48)=jac(48,48)-k(5)*y(6);
jac(146,6)=jac(146,6)+k(5)*y(48);
jac(146,48)=jac(146,48)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(6,6)=jac(6,6)-k(5)*y(43);
jac(6,43)=jac(6,43)-k(5)*y(6);
jac(43,6)=jac(43,6)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(6);
jac(45,6)=jac(45,6)+k(5)*y(43);
jac(45,43)=jac(45,43)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(6,6)=jac(6,6)-k(5)*y(37);
jac(6,37)=jac(6,37)-k(5)*y(6);
jac(37,6)=jac(37,6)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(6);
jac(39,6)=jac(39,6)+k(5)*y(37);
jac(39,37)=jac(39,37)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(6,6)=jac(6,6)-k(5)*y(33);
jac(6,33)=jac(6,33)-k(5)*y(6);
jac(33,6)=jac(33,6)-k(5)*y(33);
jac(33,33)=jac(33,33)-k(5)*y(6);
jac(35,6)=jac(35,6)+k(5)*y(33);
jac(35,33)=jac(35,33)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(6,6)=jac(6,6)-k(5)*y(27);
jac(6,27)=jac(6,27)-k(5)*y(6);
jac(27,6)=jac(27,6)-k(5)*y(27);
jac(27,27)=jac(27,27)-k(5)*y(6);
jac(30,6)=jac(30,6)+k(5)*y(27);
jac(30,27)=jac(30,27)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(6,6)=jac(6,6)-k(5)*y(21);
jac(6,21)=jac(6,21)-k(5)*y(6);
jac(21,6)=jac(21,6)-k(5)*y(21);
jac(21,21)=jac(21,21)-k(5)*y(6);
jac(24,6)=jac(24,6)+k(5)*y(21);
jac(24,21)=jac(24,21)+k(5)*y(6);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(15,15)=jac(15,15)-k(5)*y(6);
jac(15,6)=jac(15,6)-k(5)*y(15);
jac(6,15)=jac(6,15)-k(5)*y(6);
jac(6,6)=jac(6,6)-k(5)*y(15);
jac(18,15)=jac(18,15)+k(5)*y(6);
jac(18,6)=jac(18,6)+k(5)*y(15);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) -> Rec(a, b~Y, g~Y) + Lig(l1, l2) 

jac(6,6)=jac(6,6)-k(4);
jac(4,6)=jac(4,6)+k(4);
jac(1,6)=jac(1,6)+k(4);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Lyn(U, SH2) 

jac(8,8)=jac(8,8)-k(10);
jac(3,8)=jac(3,8)+k(10);
jac(5,8)=jac(5,8)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(1,1)=jac(1,1)-k(7)*y(8);
jac(1,8)=jac(1,8)-k(7)*y(1);
jac(8,1)=jac(8,1)-k(7)*y(8);
jac(8,8)=jac(8,8)-k(7)*y(1);
jac(148,1)=jac(148,1)+k(7)*y(8);
jac(148,8)=jac(148,8)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(7)*y(8);
jac(7,8)=jac(7,8)-k(7)*y(7);
jac(8,7)=jac(8,7)-k(7)*y(8);
jac(8,8)=jac(8,8)-k(7)*y(7);
jac(10,7)=jac(10,7)+k(7)*y(8);
jac(10,8)=jac(10,8)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(8,8)=jac(8,8)-k(7)*y(335);
jac(8,335)=jac(8,335)-k(7)*y(8);
jac(335,8)=jac(335,8)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(8);
jac(414,8)=jac(414,8)+k(7)*y(335);
jac(414,335)=jac(414,335)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(8,8)=jac(8,8)-k(7)*y(257);
jac(8,257)=jac(8,257)-k(7)*y(8);
jac(257,8)=jac(257,8)-k(7)*y(257);
jac(257,257)=jac(257,257)-k(7)*y(8);
jac(588,8)=jac(588,8)+k(7)*y(257);
jac(588,257)=jac(588,257)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(8,8)=jac(8,8)-k(7)*y(227);
jac(8,227)=jac(8,227)-k(7)*y(8);
jac(227,8)=jac(227,8)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(8);
jac(201,8)=jac(201,8)+k(7)*y(227);
jac(201,227)=jac(201,227)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(8,8)=jac(8,8)-k(7)*y(222);
jac(8,222)=jac(8,222)-k(7)*y(8);
jac(222,8)=jac(222,8)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(8);
jac(593,8)=jac(593,8)+k(7)*y(222);
jac(593,222)=jac(593,222)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(8,8)=jac(8,8)-k(7)*y(218);
jac(8,218)=jac(8,218)-k(7)*y(8);
jac(218,8)=jac(218,8)-k(7)*y(218);
jac(218,218)=jac(218,218)-k(7)*y(8);
jac(594,8)=jac(594,8)+k(7)*y(218);
jac(594,218)=jac(594,218)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(8,8)=jac(8,8)-k(7)*y(192);
jac(8,192)=jac(8,192)-k(7)*y(8);
jac(192,8)=jac(192,8)-k(7)*y(192);
jac(192,192)=jac(192,192)-k(7)*y(8);
jac(607,8)=jac(607,8)+k(7)*y(192);
jac(607,192)=jac(607,192)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(8,8)=jac(8,8)-k(7)*y(184);
jac(8,184)=jac(8,184)-k(7)*y(8);
jac(184,8)=jac(184,8)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(8);
jac(417,8)=jac(417,8)+k(7)*y(184);
jac(417,184)=jac(417,184)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(8,8)=jac(8,8)-k(7)*y(177);
jac(8,177)=jac(8,177)-k(7)*y(8);
jac(177,8)=jac(177,8)-k(7)*y(177);
jac(177,177)=jac(177,177)-k(7)*y(8);
jac(606,8)=jac(606,8)+k(7)*y(177);
jac(606,177)=jac(606,177)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(8,8)=jac(8,8)-k(7)*y(140);
jac(8,140)=jac(8,140)-k(7)*y(8);
jac(140,8)=jac(140,8)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(8);
jac(462,8)=jac(462,8)+k(7)*y(140);
jac(462,140)=jac(462,140)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(8,8)=jac(8,8)-k(7)*y(131);
jac(8,131)=jac(8,131)-k(7)*y(8);
jac(131,8)=jac(131,8)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(8);
jac(104,8)=jac(104,8)+k(7)*y(131);
jac(104,131)=jac(104,131)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(8,8)=jac(8,8)-k(7)*y(111);
jac(8,111)=jac(8,111)-k(7)*y(8);
jac(111,8)=jac(111,8)-k(7)*y(111);
jac(111,111)=jac(111,111)-k(7)*y(8);
jac(109,8)=jac(109,8)+k(7)*y(111);
jac(109,111)=jac(109,111)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(77,77)=jac(77,77)-k(7)*y(8);
jac(77,8)=jac(77,8)-k(7)*y(77);
jac(8,77)=jac(8,77)-k(7)*y(8);
jac(8,8)=jac(8,8)-k(7)*y(77);
jac(108,77)=jac(108,77)+k(7)*y(8);
jac(108,8)=jac(108,8)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(72,72)=jac(72,72)-k(7)*y(8);
jac(72,8)=jac(72,8)-k(7)*y(72);
jac(8,72)=jac(8,72)-k(7)*y(8);
jac(8,8)=jac(8,8)-k(7)*y(72);
jac(124,72)=jac(124,72)+k(7)*y(8);
jac(124,8)=jac(124,8)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(7)*y(8);
jac(61,8)=jac(61,8)-k(7)*y(61);
jac(8,61)=jac(8,61)-k(7)*y(8);
jac(8,8)=jac(8,8)-k(7)*y(61);
jac(121,61)=jac(121,61)+k(7)*y(8);
jac(121,8)=jac(121,8)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(54,54)=jac(54,54)-k(7)*y(8);
jac(54,8)=jac(54,8)-k(7)*y(54);
jac(8,54)=jac(8,54)-k(7)*y(8);
jac(8,8)=jac(8,8)-k(7)*y(54);
jac(117,54)=jac(117,54)+k(7)*y(8);
jac(117,8)=jac(117,8)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(8);
jac(48,8)=jac(48,8)-k(7)*y(48);
jac(8,48)=jac(8,48)-k(7)*y(8);
jac(8,8)=jac(8,8)-k(7)*y(48);
jac(463,48)=jac(463,48)+k(7)*y(8);
jac(463,8)=jac(463,8)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(8,8)=jac(8,8)-k(7)*y(43);
jac(8,43)=jac(8,43)-k(7)*y(8);
jac(43,8)=jac(43,8)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(8);
jac(113,8)=jac(113,8)+k(7)*y(43);
jac(113,43)=jac(113,43)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(8,8)=jac(8,8)-k(7)*y(37);
jac(8,37)=jac(8,37)-k(7)*y(8);
jac(37,8)=jac(37,8)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(8);
jac(107,8)=jac(107,8)+k(7)*y(37);
jac(107,37)=jac(107,37)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(8,8)=jac(8,8)-k(7)*y(33);
jac(8,33)=jac(8,33)-k(7)*y(8);
jac(33,8)=jac(33,8)-k(7)*y(33);
jac(33,33)=jac(33,33)-k(7)*y(8);
jac(105,8)=jac(105,8)+k(7)*y(33);
jac(105,33)=jac(105,33)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(8,8)=jac(8,8)-k(7)*y(27);
jac(8,27)=jac(8,27)-k(7)*y(8);
jac(27,8)=jac(27,8)-k(7)*y(27);
jac(27,27)=jac(27,27)-k(7)*y(8);
jac(103,8)=jac(103,8)+k(7)*y(27);
jac(103,27)=jac(103,27)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(8,8)=jac(8,8)-k(7)*y(21);
jac(8,21)=jac(8,21)-k(7)*y(8);
jac(21,8)=jac(21,8)-k(7)*y(21);
jac(21,21)=jac(21,21)-k(7)*y(8);
jac(96,8)=jac(96,8)+k(7)*y(21);
jac(96,21)=jac(96,21)+k(7)*y(8);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(15,15)=jac(15,15)-k(7)*y(8);
jac(15,8)=jac(15,8)-k(7)*y(15);
jac(8,15)=jac(8,15)-k(7)*y(8);
jac(8,8)=jac(8,8)-k(7)*y(15);
jac(100,15)=jac(100,15)+k(7)*y(8);
jac(100,8)=jac(100,8)+k(7)*y(15);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Lig(l1, l2) 

jac(8,8)=jac(8,8)-k(2);
jac(4,8)=jac(4,8)+k(2);
jac(7,8)=jac(7,8)+k(2);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(10,10)=jac(10,10)-k(14);
jac(463,10)=jac(463,10)+k(14);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(10,10)=jac(10,10)-k(13);
jac(17,10)=jac(17,10)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) 

jac(10,10)=jac(10,10)-k(10);
jac(3,10)=jac(3,10)+k(10);
jac(148,10)=jac(148,10)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) 

jac(10,10)=jac(10,10)-k(10);
jac(3,10)=jac(3,10)+k(10);
jac(11,10)=jac(11,10)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(10,10)=jac(10,10)-k(8);
jac(7,10)=jac(7,10)+k(8);
jac(8,10)=jac(8,10)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(10,10)=jac(10,10)-k(6);
jac(7,10)=jac(7,10)+k(6);
jac(9,10)=jac(9,10)+k(6);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(12);
jac(3,12)=jac(3,12)-k(9)*y(3);
jac(12,3)=jac(12,3)-k(9)*y(12);
jac(12,12)=jac(12,12)-k(9)*y(3);
jac(148,3)=jac(148,3)+k(9)*y(12);
jac(148,12)=jac(148,12)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(12);
jac(3,12)=jac(3,12)-k(9)*y(3);
jac(12,3)=jac(12,3)-k(9)*y(12);
jac(12,12)=jac(12,12)-k(9)*y(3);
jac(11,3)=jac(11,3)+k(9)*y(12);
jac(11,12)=jac(11,12)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) 

jac(12,12)=jac(12,12)-k(8);
jac(1,12)=jac(1,12)+k(8);
jac(5,12)=jac(5,12)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) 

jac(12,12)=jac(12,12)-k(6);
jac(1,12)=jac(1,12)+k(6);
jac(6,12)=jac(6,12)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(13,13)=jac(13,13)-k(31);
jac(11,13)=jac(11,13)+k(31);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(13);
jac(3,13)=jac(3,13)-k(15)*y(3);
jac(13,3)=jac(13,3)-k(15)*y(13);
jac(13,13)=jac(13,13)-k(15)*y(3);
jac(67,3)=jac(67,3)+k(15)*y(13);
jac(67,13)=jac(67,13)+k(15)*y(3);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(13,13)=jac(13,13)-k(13);
jac(19,13)=jac(19,13)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

jac(13,13)=jac(13,13)-k(10);
jac(3,13)=jac(3,13)+k(10);
jac(69,13)=jac(69,13)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(13,13)=jac(13,13)-k(8);
jac(91,13)=jac(91,13)+k(8);
jac(7,13)=jac(7,13)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) 

jac(13,13)=jac(13,13)-k(6);
jac(72,13)=jac(72,13)+k(6);
jac(9,13)=jac(9,13)+k(6);

% rule    : R15
% reaction: Rec(a, b~Y, g~pY) -> Rec(a, b~Y, g~Y) 

jac(15,15)=jac(15,15)-k(32);
jac(1,15)=jac(1,15)+k(32);

% rule    : R9
% reaction: Rec(a, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(15);
jac(182,15)=jac(182,15)-k(21)*y(182);
jac(15,182)=jac(15,182)-k(21)*y(15);
jac(15,15)=jac(15,15)-k(21)*y(182);
jac(177,182)=jac(177,182)+k(21)*y(15);
jac(177,15)=jac(177,15)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(15);
jac(197,15)=jac(197,15)-k(21)*y(197);
jac(15,197)=jac(15,197)-k(21)*y(15);
jac(15,15)=jac(15,15)-k(21)*y(197);
jac(192,197)=jac(192,197)+k(21)*y(15);
jac(192,15)=jac(192,15)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(15);
jac(32,15)=jac(32,15)-k(21)*y(32);
jac(15,32)=jac(15,32)-k(21)*y(15);
jac(15,15)=jac(15,15)-k(21)*y(32);
jac(27,32)=jac(27,32)+k(21)*y(15);
jac(27,15)=jac(27,15)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(15);
jac(2,15)=jac(2,15)-k(21)*y(2);
jac(15,2)=jac(15,2)-k(21)*y(15);
jac(15,15)=jac(15,15)-k(21)*y(2);
jac(21,2)=jac(21,2)+k(21)*y(15);
jac(21,15)=jac(21,15)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(15);
jac(3,15)=jac(3,15)-k(9)*y(3);
jac(15,3)=jac(15,3)-k(9)*y(15);
jac(15,15)=jac(15,15)-k(9)*y(3);
jac(48,3)=jac(48,3)+k(9)*y(15);
jac(48,15)=jac(48,15)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(15,15)=jac(15,15)-k(7)*y(16);
jac(15,16)=jac(15,16)-k(7)*y(15);
jac(16,15)=jac(16,15)-k(7)*y(16);
jac(16,16)=jac(16,16)-k(7)*y(15);
jac(150,15)=jac(150,15)+k(7)*y(16);
jac(150,16)=jac(150,16)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(15,15)=jac(15,15)-k(7)*y(22);
jac(15,22)=jac(15,22)-k(7)*y(15);
jac(22,15)=jac(22,15)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(15);
jac(452,15)=jac(452,15)+k(7)*y(22);
jac(452,22)=jac(452,22)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(15,15)=jac(15,15)-k(7)*y(28);
jac(15,28)=jac(15,28)-k(7)*y(15);
jac(28,15)=jac(28,15)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(15);
jac(156,15)=jac(156,15)+k(7)*y(28);
jac(156,28)=jac(156,28)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(15,15)=jac(15,15)-k(7)*y(34);
jac(15,34)=jac(15,34)-k(7)*y(15);
jac(34,15)=jac(34,15)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(15);
jac(617,15)=jac(617,15)+k(7)*y(34);
jac(617,34)=jac(617,34)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(15,15)=jac(15,15)-k(7)*y(38);
jac(15,38)=jac(15,38)-k(7)*y(15);
jac(38,15)=jac(38,15)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(15);
jac(249,15)=jac(249,15)+k(7)*y(38);
jac(249,38)=jac(249,38)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(15,15)=jac(15,15)-k(7)*y(44);
jac(15,44)=jac(15,44)-k(7)*y(15);
jac(44,15)=jac(44,15)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(15);
jac(49,15)=jac(49,15)+k(7)*y(44);
jac(49,44)=jac(49,44)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(15,15)=jac(15,15)-k(7)*y(55);
jac(15,55)=jac(15,55)-k(7)*y(15);
jac(55,15)=jac(55,15)-k(7)*y(55);
jac(55,55)=jac(55,55)-k(7)*y(15);
jac(56,15)=jac(56,15)+k(7)*y(55);
jac(56,55)=jac(56,55)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(15,15)=jac(15,15)-k(7)*y(62);
jac(15,62)=jac(15,62)-k(7)*y(15);
jac(62,15)=jac(62,15)-k(7)*y(62);
jac(62,62)=jac(62,62)-k(7)*y(15);
jac(63,15)=jac(63,15)+k(7)*y(62);
jac(63,62)=jac(63,62)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(15,15)=jac(15,15)-k(7)*y(91);
jac(15,91)=jac(15,91)-k(7)*y(15);
jac(91,15)=jac(91,15)-k(7)*y(91);
jac(91,91)=jac(91,91)-k(7)*y(15);
jac(93,15)=jac(93,15)+k(7)*y(91);
jac(93,91)=jac(93,91)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) 

jac(15,15)=jac(15,15)-k(7)*y(126);
jac(15,126)=jac(15,126)-k(7)*y(15);
jac(126,15)=jac(126,15)-k(7)*y(126);
jac(126,126)=jac(126,126)-k(7)*y(15);
jac(147,15)=jac(147,15)+k(7)*y(126);
jac(147,126)=jac(147,126)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(15,15)=jac(15,15)-k(7)*y(132);
jac(15,132)=jac(15,132)-k(7)*y(15);
jac(132,15)=jac(132,15)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(15);
jac(154,15)=jac(154,15)+k(7)*y(132);
jac(154,132)=jac(154,132)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(15,15)=jac(15,15)-k(7)*y(141);
jac(15,141)=jac(15,141)-k(7)*y(15);
jac(141,15)=jac(141,15)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(15);
jac(145,15)=jac(145,15)+k(7)*y(141);
jac(145,141)=jac(145,141)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(15,15)=jac(15,15)-k(7)*y(185);
jac(15,185)=jac(15,185)-k(7)*y(15);
jac(185,15)=jac(185,15)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(15);
jac(189,15)=jac(189,15)+k(7)*y(185);
jac(189,185)=jac(189,185)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) 

jac(15,15)=jac(15,15)-k(7)*y(209);
jac(15,209)=jac(15,209)-k(7)*y(15);
jac(209,15)=jac(209,15)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(15);
jac(376,15)=jac(376,15)+k(7)*y(209);
jac(376,209)=jac(376,209)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(15,15)=jac(15,15)-k(7)*y(214);
jac(15,214)=jac(15,214)-k(7)*y(15);
jac(214,15)=jac(214,15)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(15);
jac(451,15)=jac(451,15)+k(7)*y(214);
jac(451,214)=jac(451,214)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(15,15)=jac(15,15)-k(7)*y(219);
jac(15,219)=jac(15,219)-k(7)*y(15);
jac(219,15)=jac(219,15)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(15);
jac(431,15)=jac(431,15)+k(7)*y(219);
jac(431,219)=jac(431,219)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(15,15)=jac(15,15)-k(7)*y(223);
jac(15,223)=jac(15,223)-k(7)*y(15);
jac(223,15)=jac(223,15)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(15);
jac(282,15)=jac(282,15)+k(7)*y(223);
jac(282,223)=jac(282,223)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(15,15)=jac(15,15)-k(7)*y(228);
jac(15,228)=jac(15,228)-k(7)*y(15);
jac(228,15)=jac(228,15)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(15);
jac(232,15)=jac(232,15)+k(7)*y(228);
jac(232,228)=jac(232,228)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(15,15)=jac(15,15)-k(7)*y(315);
jac(15,315)=jac(15,315)-k(7)*y(15);
jac(315,15)=jac(315,15)-k(7)*y(315);
jac(315,315)=jac(315,315)-k(7)*y(15);
jac(614,15)=jac(614,15)+k(7)*y(315);
jac(614,315)=jac(614,315)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(15,15)=jac(15,15)-k(7)*y(319);
jac(15,319)=jac(15,319)-k(7)*y(15);
jac(319,15)=jac(319,15)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(15);
jac(618,15)=jac(618,15)+k(7)*y(319);
jac(618,319)=jac(618,319)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(15,15)=jac(15,15)-k(7)*y(336);
jac(15,336)=jac(15,336)-k(7)*y(15);
jac(336,15)=jac(336,15)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(15);
jac(198,15)=jac(198,15)+k(7)*y(336);
jac(198,336)=jac(198,336)+k(7)*y(15);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(15,15)=jac(15,15)-k(7)*y(437);
jac(15,437)=jac(15,437)-k(7)*y(15);
jac(437,15)=jac(437,15)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(15);
jac(616,15)=jac(616,15)+k(7)*y(437);
jac(616,437)=jac(616,437)+k(7)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(15,15)=jac(15,15)-k(5)*y(47);
jac(15,47)=jac(15,47)-k(5)*y(15);
jac(47,15)=jac(47,15)-k(5)*y(47);
jac(47,47)=jac(47,47)-k(5)*y(15);
jac(404,15)=jac(404,15)+k(5)*y(47);
jac(404,47)=jac(404,47)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(15,15)=jac(15,15)-k(5)*y(71);
jac(15,71)=jac(15,71)-k(5)*y(15);
jac(71,15)=jac(71,15)-k(5)*y(71);
jac(71,71)=jac(71,71)-k(5)*y(15);
jac(331,15)=jac(331,15)+k(5)*y(71);
jac(331,71)=jac(331,71)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(15,15)=jac(15,15)-k(5)*y(76);
jac(15,76)=jac(15,76)-k(5)*y(15);
jac(76,15)=jac(76,15)-k(5)*y(76);
jac(76,76)=jac(76,76)-k(5)*y(15);
jac(610,15)=jac(610,15)+k(5)*y(76);
jac(610,76)=jac(610,76)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(15,15)=jac(15,15)-k(5)*y(82);
jac(15,82)=jac(15,82)-k(5)*y(15);
jac(82,15)=jac(82,15)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(15);
jac(351,15)=jac(351,15)+k(5)*y(82);
jac(351,82)=jac(351,82)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(15,15)=jac(15,15)-k(5)*y(86);
jac(15,86)=jac(15,86)-k(5)*y(15);
jac(86,15)=jac(86,15)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(15);
jac(450,15)=jac(450,15)+k(5)*y(86);
jac(450,86)=jac(450,86)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(15,15)=jac(15,15)-k(5)*y(89);
jac(15,89)=jac(15,89)-k(5)*y(15);
jac(89,15)=jac(89,15)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(15);
jac(375,15)=jac(375,15)+k(5)*y(89);
jac(375,89)=jac(375,89)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(15,15)=jac(15,15)-k(5)*y(98);
jac(15,98)=jac(15,98)-k(5)*y(15);
jac(98,15)=jac(98,15)-k(5)*y(98);
jac(98,98)=jac(98,98)-k(5)*y(15);
jac(150,15)=jac(150,15)+k(5)*y(98);
jac(150,98)=jac(150,98)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(15,15)=jac(15,15)-k(5)*y(110);
jac(15,110)=jac(15,110)-k(5)*y(15);
jac(110,15)=jac(110,15)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(15);
jac(354,15)=jac(354,15)+k(5)*y(110);
jac(354,110)=jac(354,110)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(15,15)=jac(15,15)-k(5)*y(114);
jac(15,114)=jac(15,114)-k(5)*y(15);
jac(114,15)=jac(114,15)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(15);
jac(524,15)=jac(524,15)+k(5)*y(114);
jac(524,114)=jac(524,114)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(15,15)=jac(15,15)-k(5)*y(118);
jac(15,118)=jac(15,118)-k(5)*y(15);
jac(118,15)=jac(118,15)-k(5)*y(118);
jac(118,118)=jac(118,118)-k(5)*y(15);
jac(537,15)=jac(537,15)+k(5)*y(118);
jac(537,118)=jac(537,118)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(15,15)=jac(15,15)-k(5)*y(122);
jac(15,122)=jac(15,122)-k(5)*y(15);
jac(122,15)=jac(122,15)-k(5)*y(122);
jac(122,122)=jac(122,122)-k(5)*y(15);
jac(127,15)=jac(127,15)+k(5)*y(122);
jac(127,122)=jac(127,122)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(15,15)=jac(15,15)-k(5)*y(176);
jac(15,176)=jac(15,176)-k(5)*y(15);
jac(176,15)=jac(176,15)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(15);
jac(648,15)=jac(648,15)+k(5)*y(176);
jac(648,176)=jac(648,176)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(15,15)=jac(15,15)-k(5)*y(191);
jac(15,191)=jac(15,191)-k(5)*y(15);
jac(191,15)=jac(191,15)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(15);
jac(409,15)=jac(409,15)+k(5)*y(191);
jac(409,191)=jac(409,191)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(15,15)=jac(15,15)-k(5)*y(256);
jac(15,256)=jac(15,256)-k(5)*y(15);
jac(256,15)=jac(256,15)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(15);
jac(585,15)=jac(585,15)+k(5)*y(256);
jac(585,256)=jac(585,256)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(15,15)=jac(15,15)-k(5)*y(264);
jac(15,264)=jac(15,264)-k(5)*y(15);
jac(264,15)=jac(264,15)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(15);
jac(512,15)=jac(512,15)+k(5)*y(264);
jac(512,264)=jac(512,264)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(15,15)=jac(15,15)-k(5)*y(380);
jac(15,380)=jac(15,380)-k(5)*y(15);
jac(380,15)=jac(380,15)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(15);
jac(383,15)=jac(383,15)+k(5)*y(380);
jac(383,380)=jac(383,380)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(15,15)=jac(15,15)-k(5)*y(391);
jac(15,391)=jac(15,391)-k(5)*y(15);
jac(391,15)=jac(391,15)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(15);
jac(393,15)=jac(393,15)+k(5)*y(391);
jac(393,391)=jac(393,391)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(15,15)=jac(15,15)-k(5)*y(407);
jac(15,407)=jac(15,407)-k(5)*y(15);
jac(407,15)=jac(407,15)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(15);
jac(406,15)=jac(406,15)+k(5)*y(407);
jac(406,407)=jac(406,407)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(15,15)=jac(15,15)-k(5)*y(441);
jac(15,441)=jac(15,441)-k(5)*y(15);
jac(441,15)=jac(441,15)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(15);
jac(405,15)=jac(405,15)+k(5)*y(441);
jac(405,441)=jac(405,441)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(15,15)=jac(15,15)-k(5)*y(515);
jac(15,515)=jac(15,515)-k(5)*y(15);
jac(515,15)=jac(515,15)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(15);
jac(518,15)=jac(518,15)+k(5)*y(515);
jac(518,515)=jac(518,515)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(15,15)=jac(15,15)-k(5)*y(543);
jac(15,543)=jac(15,543)-k(5)*y(15);
jac(543,15)=jac(543,15)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(15);
jac(547,15)=jac(547,15)+k(5)*y(543);
jac(547,543)=jac(547,543)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(15,15)=jac(15,15)-k(5)*y(554);
jac(15,554)=jac(15,554)-k(5)*y(15);
jac(554,15)=jac(554,15)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(15);
jac(556,15)=jac(556,15)+k(5)*y(554);
jac(556,554)=jac(556,554)+k(5)*y(15);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(15,15)=jac(15,15)-k(5)*y(9);
jac(15,9)=jac(15,9)-k(5)*y(15);
jac(9,15)=jac(9,15)-k(5)*y(9);
jac(9,9)=jac(9,9)-k(5)*y(15);
jac(14,15)=jac(14,15)+k(5)*y(9);
jac(14,9)=jac(14,9)+k(5)*y(15);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(4,4)=jac(4,4)-k(3)*y(15);
jac(4,15)=jac(4,15)-k(3)*y(4);
jac(15,4)=jac(15,4)-k(3)*y(15);
jac(15,15)=jac(15,15)-k(3)*y(4);
jac(98,4)=jac(98,4)+k(3)*y(15);
jac(98,15)=jac(98,15)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y, g~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(4,4)=jac(4,4)-k(1)*y(15);
jac(4,15)=jac(4,15)-k(1)*y(4);
jac(15,4)=jac(15,4)-k(1)*y(15);
jac(15,15)=jac(15,15)-k(1)*y(4);
jac(16,4)=jac(16,4)+k(1)*y(15);
jac(16,15)=jac(16,15)+k(1)*y(4);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(16,16)=jac(16,16)-k(32);
jac(5,16)=jac(5,16)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(16);
jac(182,16)=jac(182,16)-k(21)*y(182);
jac(16,182)=jac(16,182)-k(21)*y(16);
jac(16,16)=jac(16,16)-k(21)*y(182);
jac(214,182)=jac(214,182)+k(21)*y(16);
jac(214,16)=jac(214,16)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(16);
jac(197,16)=jac(197,16)-k(21)*y(197);
jac(16,197)=jac(16,197)-k(21)*y(16);
jac(16,16)=jac(16,16)-k(21)*y(197);
jac(209,197)=jac(209,197)+k(21)*y(16);
jac(209,16)=jac(209,16)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(16);
jac(32,16)=jac(32,16)-k(21)*y(32);
jac(16,32)=jac(16,32)-k(21)*y(16);
jac(16,16)=jac(16,16)-k(21)*y(32);
jac(28,32)=jac(28,32)+k(21)*y(16);
jac(28,16)=jac(28,16)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(16);
jac(2,16)=jac(2,16)-k(21)*y(2);
jac(16,2)=jac(16,2)-k(21)*y(16);
jac(16,16)=jac(16,16)-k(21)*y(2);
jac(22,2)=jac(22,2)+k(21)*y(16);
jac(22,16)=jac(22,16)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(16);
jac(3,16)=jac(3,16)-k(9)*y(3);
jac(16,3)=jac(16,3)-k(9)*y(16);
jac(16,16)=jac(16,16)-k(9)*y(3);
jac(126,3)=jac(126,3)+k(9)*y(16);
jac(126,16)=jac(126,16)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(1,1)=jac(1,1)-k(7)*y(16);
jac(1,16)=jac(1,16)-k(7)*y(1);
jac(16,1)=jac(16,1)-k(7)*y(16);
jac(16,16)=jac(16,16)-k(7)*y(1);
jac(18,1)=jac(18,1)+k(7)*y(16);
jac(18,16)=jac(18,16)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(16,16)=jac(16,16)-k(7)*y(7);
jac(16,7)=jac(16,7)-k(7)*y(16);
jac(7,16)=jac(7,16)-k(7)*y(7);
jac(7,7)=jac(7,7)-k(7)*y(16);
jac(14,16)=jac(14,16)+k(7)*y(7);
jac(14,7)=jac(14,7)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(16,16)=jac(16,16)-k(7)*y(335);
jac(16,335)=jac(16,335)-k(7)*y(16);
jac(335,16)=jac(335,16)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(16);
jac(406,16)=jac(406,16)+k(7)*y(335);
jac(406,335)=jac(406,335)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(16,16)=jac(16,16)-k(7)*y(257);
jac(16,257)=jac(16,257)-k(7)*y(16);
jac(257,16)=jac(257,16)-k(7)*y(257);
jac(257,257)=jac(257,257)-k(7)*y(16);
jac(585,16)=jac(585,16)+k(7)*y(257);
jac(585,257)=jac(585,257)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(16,16)=jac(16,16)-k(7)*y(227);
jac(16,227)=jac(16,227)-k(7)*y(16);
jac(227,16)=jac(227,16)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(16);
jac(547,16)=jac(547,16)+k(7)*y(227);
jac(547,227)=jac(547,227)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(16,16)=jac(16,16)-k(7)*y(222);
jac(16,222)=jac(16,222)-k(7)*y(16);
jac(222,16)=jac(222,16)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(16);
jac(518,16)=jac(518,16)+k(7)*y(222);
jac(518,222)=jac(518,222)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(16,16)=jac(16,16)-k(7)*y(218);
jac(16,218)=jac(16,218)-k(7)*y(16);
jac(218,16)=jac(218,16)-k(7)*y(218);
jac(218,218)=jac(218,218)-k(7)*y(16);
jac(512,16)=jac(512,16)+k(7)*y(218);
jac(512,218)=jac(512,218)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(16,16)=jac(16,16)-k(7)*y(192);
jac(16,192)=jac(16,192)-k(7)*y(16);
jac(192,16)=jac(192,16)-k(7)*y(192);
jac(192,192)=jac(192,192)-k(7)*y(16);
jac(409,16)=jac(409,16)+k(7)*y(192);
jac(409,192)=jac(409,192)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(16,16)=jac(16,16)-k(7)*y(184);
jac(16,184)=jac(16,184)-k(7)*y(16);
jac(184,16)=jac(184,16)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(16);
jac(393,16)=jac(393,16)+k(7)*y(184);
jac(393,184)=jac(393,184)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(16,16)=jac(16,16)-k(7)*y(177);
jac(16,177)=jac(16,177)-k(7)*y(16);
jac(177,16)=jac(177,16)-k(7)*y(177);
jac(177,177)=jac(177,177)-k(7)*y(16);
jac(648,16)=jac(648,16)+k(7)*y(177);
jac(648,177)=jac(648,177)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(16,16)=jac(16,16)-k(7)*y(140);
jac(16,140)=jac(16,140)-k(7)*y(16);
jac(140,16)=jac(140,16)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(16);
jac(383,16)=jac(383,16)+k(7)*y(140);
jac(383,140)=jac(383,140)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(16,16)=jac(16,16)-k(7)*y(131);
jac(16,131)=jac(16,131)-k(7)*y(16);
jac(131,16)=jac(131,16)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(16);
jac(405,16)=jac(405,16)+k(7)*y(131);
jac(405,131)=jac(405,131)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(16,16)=jac(16,16)-k(7)*y(111);
jac(16,111)=jac(16,111)-k(7)*y(16);
jac(111,16)=jac(111,16)-k(7)*y(111);
jac(111,111)=jac(111,111)-k(7)*y(16);
jac(354,16)=jac(354,16)+k(7)*y(111);
jac(354,111)=jac(354,111)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(77,77)=jac(77,77)-k(7)*y(16);
jac(77,16)=jac(77,16)-k(7)*y(77);
jac(16,77)=jac(16,77)-k(7)*y(16);
jac(16,16)=jac(16,16)-k(7)*y(77);
jac(610,77)=jac(610,77)+k(7)*y(16);
jac(610,16)=jac(610,16)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(72,72)=jac(72,72)-k(7)*y(16);
jac(72,16)=jac(72,16)-k(7)*y(72);
jac(16,72)=jac(16,72)-k(7)*y(16);
jac(16,16)=jac(16,16)-k(7)*y(72);
jac(331,72)=jac(331,72)+k(7)*y(16);
jac(331,16)=jac(331,16)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(16,16)=jac(16,16)-k(7)*y(61);
jac(16,61)=jac(16,61)-k(7)*y(16);
jac(61,16)=jac(61,16)-k(7)*y(61);
jac(61,61)=jac(61,61)-k(7)*y(16);
jac(127,16)=jac(127,16)+k(7)*y(61);
jac(127,61)=jac(127,61)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(16,16)=jac(16,16)-k(7)*y(54);
jac(16,54)=jac(16,54)-k(7)*y(16);
jac(54,16)=jac(54,16)-k(7)*y(54);
jac(54,54)=jac(54,54)-k(7)*y(16);
jac(537,16)=jac(537,16)+k(7)*y(54);
jac(537,54)=jac(537,54)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(16,16)=jac(16,16)-k(7)*y(48);
jac(16,48)=jac(16,48)-k(7)*y(16);
jac(48,16)=jac(48,16)-k(7)*y(48);
jac(48,48)=jac(48,48)-k(7)*y(16);
jac(404,16)=jac(404,16)+k(7)*y(48);
jac(404,48)=jac(404,48)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(16,16)=jac(16,16)-k(7)*y(43);
jac(16,43)=jac(16,43)-k(7)*y(16);
jac(43,16)=jac(43,16)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(16);
jac(524,16)=jac(524,16)+k(7)*y(43);
jac(524,43)=jac(524,43)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(16,16)=jac(16,16)-k(7)*y(37);
jac(16,37)=jac(16,37)-k(7)*y(16);
jac(37,16)=jac(37,16)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(16);
jac(556,16)=jac(556,16)+k(7)*y(37);
jac(556,37)=jac(556,37)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(16,16)=jac(16,16)-k(7)*y(33);
jac(16,33)=jac(16,33)-k(7)*y(16);
jac(33,16)=jac(33,16)-k(7)*y(33);
jac(33,33)=jac(33,33)-k(7)*y(16);
jac(351,16)=jac(351,16)+k(7)*y(33);
jac(351,33)=jac(351,33)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(16,16)=jac(16,16)-k(7)*y(27);
jac(16,27)=jac(16,27)-k(7)*y(16);
jac(27,16)=jac(27,16)-k(7)*y(27);
jac(27,27)=jac(27,27)-k(7)*y(16);
jac(450,16)=jac(450,16)+k(7)*y(27);
jac(450,27)=jac(450,27)+k(7)*y(16);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(16,16)=jac(16,16)-k(7)*y(21);
jac(16,21)=jac(16,21)-k(7)*y(16);
jac(21,16)=jac(21,16)-k(7)*y(21);
jac(21,21)=jac(21,21)-k(7)*y(16);
jac(375,16)=jac(375,16)+k(7)*y(21);
jac(375,21)=jac(375,21)+k(7)*y(16);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) -> Rec(a, b~Y, g~pY) + Lig(l1, l2) 

jac(16,16)=jac(16,16)-k(2);
jac(4,16)=jac(4,16)+k(2);
jac(15,16)=jac(15,16)+k(2);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(17,17)=jac(17,17)-k(32);
jac(10,17)=jac(10,17)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(17);
jac(182,17)=jac(182,17)-k(21)*y(182);
jac(17,182)=jac(17,182)-k(21)*y(17);
jac(17,17)=jac(17,17)-k(21)*y(182);
jac(215,182)=jac(215,182)+k(21)*y(17);
jac(215,17)=jac(215,17)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(17);
jac(197,17)=jac(197,17)-k(21)*y(197);
jac(17,197)=jac(17,197)-k(21)*y(17);
jac(17,17)=jac(17,17)-k(21)*y(197);
jac(210,197)=jac(210,197)+k(21)*y(17);
jac(210,17)=jac(210,17)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(17);
jac(32,17)=jac(32,17)-k(21)*y(32);
jac(17,32)=jac(17,32)-k(21)*y(17);
jac(17,17)=jac(17,17)-k(21)*y(32);
jac(29,32)=jac(29,32)+k(21)*y(17);
jac(29,17)=jac(29,17)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(17);
jac(2,17)=jac(2,17)-k(21)*y(2);
jac(17,2)=jac(17,2)-k(21)*y(17);
jac(17,17)=jac(17,17)-k(21)*y(2);
jac(23,2)=jac(23,2)+k(21)*y(17);
jac(23,17)=jac(23,17)+k(21)*y(2);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(17,17)=jac(17,17)-k(14);
jac(149,17)=jac(149,17)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) 

jac(17,17)=jac(17,17)-k(10);
jac(3,17)=jac(3,17)+k(10);
jac(14,17)=jac(14,17)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) 

jac(17,17)=jac(17,17)-k(10);
jac(3,17)=jac(3,17)+k(10);
jac(146,17)=jac(146,17)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(17,17)=jac(17,17)-k(8);
jac(7,17)=jac(7,17)+k(8);
jac(126,17)=jac(126,17)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(17,17)=jac(17,17)-k(6);
jac(48,17)=jac(48,17)+k(6);
jac(9,17)=jac(9,17)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(18,18)=jac(18,18)-k(32);
jac(12,18)=jac(12,18)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(18);
jac(182,18)=jac(182,18)-k(21)*y(182);
jac(18,182)=jac(18,182)-k(21)*y(18);
jac(18,18)=jac(18,18)-k(21)*y(182);
jac(216,182)=jac(216,182)+k(21)*y(18);
jac(216,18)=jac(216,18)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(18);
jac(197,18)=jac(197,18)-k(21)*y(197);
jac(18,197)=jac(18,197)-k(21)*y(18);
jac(18,18)=jac(18,18)-k(21)*y(197);
jac(211,197)=jac(211,197)+k(21)*y(18);
jac(211,18)=jac(211,18)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(18);
jac(32,18)=jac(32,18)-k(21)*y(32);
jac(18,32)=jac(18,32)-k(21)*y(18);
jac(18,18)=jac(18,18)-k(21)*y(32);
jac(30,32)=jac(30,32)+k(21)*y(18);
jac(30,18)=jac(30,18)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(18);
jac(2,18)=jac(2,18)-k(21)*y(2);
jac(18,2)=jac(18,2)-k(21)*y(18);
jac(18,18)=jac(18,18)-k(21)*y(2);
jac(24,2)=jac(24,2)+k(21)*y(18);
jac(24,18)=jac(24,18)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(18);
jac(3,18)=jac(3,18)-k(9)*y(3);
jac(18,3)=jac(18,3)-k(9)*y(18);
jac(18,18)=jac(18,18)-k(9)*y(3);
jac(146,3)=jac(146,3)+k(9)*y(18);
jac(146,18)=jac(146,18)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(18);
jac(3,18)=jac(3,18)-k(9)*y(3);
jac(18,3)=jac(18,3)-k(9)*y(18);
jac(18,18)=jac(18,18)-k(9)*y(3);
jac(14,3)=jac(14,3)+k(9)*y(18);
jac(14,18)=jac(14,18)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) 

jac(18,18)=jac(18,18)-k(8);
jac(1,18)=jac(1,18)+k(8);
jac(16,18)=jac(16,18)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) 

jac(18,18)=jac(18,18)-k(6);
jac(15,18)=jac(15,18)+k(6);
jac(6,18)=jac(6,18)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(19,19)=jac(19,19)-k(32);
jac(13,19)=jac(13,19)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(19,19)=jac(19,19)-k(31);
jac(14,19)=jac(14,19)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(19);
jac(182,19)=jac(182,19)-k(21)*y(182);
jac(19,182)=jac(19,182)-k(21)*y(19);
jac(19,19)=jac(19,19)-k(21)*y(182);
jac(217,182)=jac(217,182)+k(21)*y(19);
jac(217,19)=jac(217,19)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(19);
jac(197,19)=jac(197,19)-k(21)*y(197);
jac(19,197)=jac(19,197)-k(21)*y(19);
jac(19,19)=jac(19,19)-k(21)*y(197);
jac(212,197)=jac(212,197)+k(21)*y(19);
jac(212,19)=jac(212,19)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(19);
jac(32,19)=jac(32,19)-k(21)*y(32);
jac(19,32)=jac(19,32)-k(21)*y(19);
jac(19,19)=jac(19,19)-k(21)*y(32);
jac(31,32)=jac(31,32)+k(21)*y(19);
jac(31,19)=jac(31,19)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(19);
jac(2,19)=jac(2,19)-k(21)*y(2);
jac(19,2)=jac(19,2)-k(21)*y(19);
jac(19,19)=jac(19,19)-k(21)*y(2);
jac(25,2)=jac(25,2)+k(21)*y(19);
jac(25,19)=jac(25,19)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(19);
jac(3,19)=jac(3,19)-k(15)*y(3);
jac(19,3)=jac(19,3)-k(15)*y(19);
jac(19,19)=jac(19,19)-k(15)*y(3);
jac(41,3)=jac(41,3)+k(15)*y(19);
jac(41,19)=jac(41,19)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

jac(19,19)=jac(19,19)-k(10);
jac(3,19)=jac(3,19)+k(10);
jac(234,19)=jac(234,19)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(19,19)=jac(19,19)-k(8);
jac(315,19)=jac(315,19)+k(8);
jac(7,19)=jac(7,19)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) 

jac(19,19)=jac(19,19)-k(6);
jac(77,19)=jac(77,19)+k(6);
jac(9,19)=jac(9,19)+k(6);

% rule    : R9_op
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(22);
jac(2,21)=jac(2,21)+k(22);
jac(15,21)=jac(15,21)+k(22);

% rule    : R3
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(21);
jac(3,21)=jac(3,21)-k(9)*y(3);
jac(21,3)=jac(21,3)-k(9)*y(21);
jac(21,21)=jac(21,21)-k(9)*y(3);
jac(140,3)=jac(140,3)+k(9)*y(21);
jac(140,21)=jac(140,21)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(22);
jac(21,22)=jac(21,22)-k(7)*y(21);
jac(22,21)=jac(22,21)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(21);
jac(172,21)=jac(172,21)+k(7)*y(22);
jac(172,22)=jac(172,22)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(28);
jac(21,28)=jac(21,28)-k(7)*y(21);
jac(28,21)=jac(28,21)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(21);
jac(378,21)=jac(378,21)+k(7)*y(28);
jac(378,28)=jac(378,28)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(34);
jac(21,34)=jac(21,34)-k(7)*y(21);
jac(34,21)=jac(34,21)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(21);
jac(482,21)=jac(482,21)+k(7)*y(34);
jac(482,34)=jac(482,34)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(38);
jac(21,38)=jac(21,38)-k(7)*y(21);
jac(38,21)=jac(38,21)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(21);
jac(481,21)=jac(481,21)+k(7)*y(38);
jac(481,38)=jac(481,38)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(44);
jac(21,44)=jac(21,44)-k(7)*y(21);
jac(44,21)=jac(44,21)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(21);
jac(274,21)=jac(274,21)+k(7)*y(44);
jac(274,44)=jac(274,44)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(55,55)=jac(55,55)-k(7)*y(21);
jac(55,21)=jac(55,21)-k(7)*y(55);
jac(21,55)=jac(21,55)-k(7)*y(21);
jac(21,21)=jac(21,21)-k(7)*y(55);
jac(276,55)=jac(276,55)+k(7)*y(21);
jac(276,21)=jac(276,21)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(62,62)=jac(62,62)-k(7)*y(21);
jac(62,21)=jac(62,21)-k(7)*y(62);
jac(21,62)=jac(21,62)-k(7)*y(21);
jac(21,21)=jac(21,21)-k(7)*y(62);
jac(88,62)=jac(88,62)+k(7)*y(21);
jac(88,21)=jac(88,21)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(91,91)=jac(91,91)-k(7)*y(21);
jac(91,21)=jac(91,21)-k(7)*y(91);
jac(21,91)=jac(21,91)-k(7)*y(21);
jac(21,21)=jac(21,21)-k(7)*y(91);
jac(90,91)=jac(90,91)+k(7)*y(21);
jac(90,21)=jac(90,21)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(126,126)=jac(126,126)-k(7)*y(21);
jac(126,21)=jac(126,21)-k(7)*y(126);
jac(21,126)=jac(21,126)-k(7)*y(21);
jac(21,21)=jac(21,21)-k(7)*y(126);
jac(152,126)=jac(152,126)+k(7)*y(21);
jac(152,21)=jac(152,21)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(132);
jac(21,132)=jac(21,132)-k(7)*y(21);
jac(132,21)=jac(132,21)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(21);
jac(157,21)=jac(157,21)+k(7)*y(132);
jac(157,132)=jac(157,132)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(141);
jac(21,141)=jac(21,141)-k(7)*y(21);
jac(141,21)=jac(141,21)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(21);
jac(170,21)=jac(170,21)+k(7)*y(141);
jac(170,141)=jac(170,141)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(185);
jac(21,185)=jac(21,185)-k(7)*y(21);
jac(185,21)=jac(185,21)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(21);
jac(174,21)=jac(174,21)+k(7)*y(185);
jac(174,185)=jac(174,185)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(209);
jac(21,209)=jac(21,209)-k(7)*y(21);
jac(209,21)=jac(209,21)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(21);
jac(373,21)=jac(373,21)+k(7)*y(209);
jac(373,209)=jac(373,209)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(214);
jac(21,214)=jac(21,214)-k(7)*y(21);
jac(214,21)=jac(214,21)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(21);
jac(377,21)=jac(377,21)+k(7)*y(214);
jac(377,214)=jac(377,214)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(219);
jac(21,219)=jac(21,219)-k(7)*y(21);
jac(219,21)=jac(219,21)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(21);
jac(280,21)=jac(280,21)+k(7)*y(219);
jac(280,219)=jac(280,219)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(223);
jac(21,223)=jac(21,223)-k(7)*y(21);
jac(223,21)=jac(223,21)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(21);
jac(278,21)=jac(278,21)+k(7)*y(223);
jac(278,223)=jac(278,223)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(228);
jac(21,228)=jac(21,228)-k(7)*y(21);
jac(228,21)=jac(228,21)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(21);
jac(484,21)=jac(484,21)+k(7)*y(228);
jac(484,228)=jac(484,228)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(315,315)=jac(315,315)-k(7)*y(21);
jac(315,21)=jac(315,21)-k(7)*y(315);
jac(21,315)=jac(21,315)-k(7)*y(21);
jac(21,21)=jac(21,21)-k(7)*y(315);
jac(425,315)=jac(425,315)+k(7)*y(21);
jac(425,21)=jac(425,21)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(319);
jac(21,319)=jac(21,319)-k(7)*y(21);
jac(319,21)=jac(319,21)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(21);
jac(427,21)=jac(427,21)+k(7)*y(319);
jac(427,319)=jac(427,319)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(336);
jac(21,336)=jac(21,336)-k(7)*y(21);
jac(336,21)=jac(336,21)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(21);
jac(371,21)=jac(371,21)+k(7)*y(336);
jac(371,336)=jac(371,336)+k(7)*y(21);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(7)*y(437);
jac(21,437)=jac(21,437)-k(7)*y(21);
jac(437,21)=jac(437,21)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(21);
jac(275,21)=jac(275,21)+k(7)*y(437);
jac(275,437)=jac(275,437)+k(7)*y(21);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(47,47)=jac(47,47)-k(5)*y(21);
jac(47,21)=jac(47,21)-k(5)*y(47);
jac(21,47)=jac(21,47)-k(5)*y(21);
jac(21,21)=jac(21,21)-k(5)*y(47);
jac(388,47)=jac(388,47)+k(5)*y(21);
jac(388,21)=jac(388,21)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(71,71)=jac(71,71)-k(5)*y(21);
jac(71,21)=jac(71,21)-k(5)*y(71);
jac(21,71)=jac(21,71)-k(5)*y(21);
jac(21,21)=jac(21,21)-k(5)*y(71);
jac(142,71)=jac(142,71)+k(5)*y(21);
jac(142,21)=jac(142,21)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(76,76)=jac(76,76)-k(5)*y(21);
jac(76,21)=jac(76,21)-k(5)*y(76);
jac(21,76)=jac(21,76)-k(5)*y(21);
jac(21,21)=jac(21,21)-k(5)*y(76);
jac(508,76)=jac(508,76)+k(5)*y(21);
jac(508,21)=jac(508,21)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(21,21)=jac(21,21)-k(5)*y(82);
jac(21,82)=jac(21,82)-k(5)*y(21);
jac(82,21)=jac(82,21)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(21);
jac(575,21)=jac(575,21)+k(5)*y(82);
jac(575,82)=jac(575,82)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(21,21)=jac(21,21)-k(5)*y(86);
jac(21,86)=jac(21,86)-k(5)*y(21);
jac(86,21)=jac(86,21)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(21);
jac(166,21)=jac(166,21)+k(5)*y(86);
jac(166,86)=jac(166,86)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(5)*y(89);
jac(21,89)=jac(21,89)-k(5)*y(21);
jac(89,21)=jac(89,21)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(21);
jac(172,21)=jac(172,21)+k(5)*y(89);
jac(172,89)=jac(172,89)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(98,98)=jac(98,98)-k(5)*y(21);
jac(98,21)=jac(98,21)-k(5)*y(98);
jac(21,98)=jac(21,98)-k(5)*y(21);
jac(21,21)=jac(21,21)-k(5)*y(98);
jac(452,98)=jac(452,98)+k(5)*y(21);
jac(452,21)=jac(452,21)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(5)*y(110);
jac(21,110)=jac(21,110)-k(5)*y(21);
jac(110,21)=jac(110,21)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(21);
jac(506,21)=jac(506,21)+k(5)*y(110);
jac(506,110)=jac(506,110)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(5)*y(114);
jac(21,114)=jac(21,114)-k(5)*y(21);
jac(114,21)=jac(114,21)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(21);
jac(507,21)=jac(507,21)+k(5)*y(114);
jac(507,114)=jac(507,114)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(118,118)=jac(118,118)-k(5)*y(21);
jac(118,21)=jac(118,21)-k(5)*y(118);
jac(21,118)=jac(21,118)-k(5)*y(21);
jac(21,21)=jac(21,21)-k(5)*y(118);
jac(538,118)=jac(538,118)+k(5)*y(21);
jac(538,21)=jac(538,21)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(122,122)=jac(122,122)-k(5)*y(21);
jac(122,21)=jac(122,21)-k(5)*y(122);
jac(21,122)=jac(21,122)-k(5)*y(21);
jac(21,21)=jac(21,21)-k(5)*y(122);
jac(329,122)=jac(329,122)+k(5)*y(21);
jac(329,21)=jac(329,21)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(21,21)=jac(21,21)-k(5)*y(176);
jac(21,176)=jac(21,176)-k(5)*y(21);
jac(176,21)=jac(176,21)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(21);
jac(179,21)=jac(179,21)+k(5)*y(176);
jac(179,176)=jac(179,176)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(21,21)=jac(21,21)-k(5)*y(191);
jac(21,191)=jac(21,191)-k(5)*y(21);
jac(191,21)=jac(191,21)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(21);
jac(453,21)=jac(453,21)+k(5)*y(191);
jac(453,191)=jac(453,191)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(21,21)=jac(21,21)-k(5)*y(256);
jac(21,256)=jac(21,256)-k(5)*y(21);
jac(256,21)=jac(256,21)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(21);
jac(545,21)=jac(545,21)+k(5)*y(256);
jac(545,256)=jac(545,256)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(21,21)=jac(21,21)-k(5)*y(264);
jac(21,264)=jac(21,264)-k(5)*y(21);
jac(264,21)=jac(264,21)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(21);
jac(510,21)=jac(510,21)+k(5)*y(264);
jac(510,264)=jac(510,264)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(5)*y(380);
jac(21,380)=jac(21,380)-k(5)*y(21);
jac(380,21)=jac(380,21)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(21);
jac(386,21)=jac(386,21)+k(5)*y(380);
jac(386,380)=jac(386,380)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(21,21)=jac(21,21)-k(5)*y(391);
jac(21,391)=jac(21,391)-k(5)*y(21);
jac(391,21)=jac(391,21)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(21);
jac(390,21)=jac(390,21)+k(5)*y(391);
jac(390,391)=jac(390,391)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(21,21)=jac(21,21)-k(5)*y(407);
jac(21,407)=jac(21,407)-k(5)*y(21);
jac(407,21)=jac(407,21)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(21);
jac(411,21)=jac(411,21)+k(5)*y(407);
jac(411,407)=jac(411,407)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(21,21)=jac(21,21)-k(5)*y(441);
jac(21,441)=jac(21,441)-k(5)*y(21);
jac(441,21)=jac(441,21)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(21);
jac(445,21)=jac(445,21)+k(5)*y(441);
jac(445,441)=jac(445,441)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(21,21)=jac(21,21)-k(5)*y(515);
jac(21,515)=jac(21,515)-k(5)*y(21);
jac(515,21)=jac(515,21)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(21);
jac(511,21)=jac(511,21)+k(5)*y(515);
jac(511,515)=jac(511,515)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(21,21)=jac(21,21)-k(5)*y(543);
jac(21,543)=jac(21,543)-k(5)*y(21);
jac(543,21)=jac(543,21)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(21);
jac(542,21)=jac(542,21)+k(5)*y(543);
jac(542,543)=jac(542,543)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(21,21)=jac(21,21)-k(5)*y(554);
jac(21,554)=jac(21,554)-k(5)*y(21);
jac(554,21)=jac(554,21)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(21);
jac(541,21)=jac(541,21)+k(5)*y(554);
jac(541,554)=jac(541,554)+k(5)*y(21);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(9,9)=jac(9,9)-k(5)*y(21);
jac(9,21)=jac(9,21)-k(5)*y(9);
jac(21,9)=jac(21,9)-k(5)*y(21);
jac(21,21)=jac(21,21)-k(5)*y(9);
jac(20,9)=jac(20,9)+k(5)*y(21);
jac(20,21)=jac(20,21)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(4,4)=jac(4,4)-k(3)*y(21);
jac(4,21)=jac(4,21)-k(3)*y(4);
jac(21,4)=jac(21,4)-k(3)*y(21);
jac(21,21)=jac(21,21)-k(3)*y(4);
jac(89,4)=jac(89,4)+k(3)*y(21);
jac(89,21)=jac(89,21)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(4,4)=jac(4,4)-k(1)*y(21);
jac(4,21)=jac(4,21)-k(1)*y(4);
jac(21,4)=jac(21,4)-k(1)*y(21);
jac(21,21)=jac(21,21)-k(1)*y(4);
jac(22,4)=jac(22,4)+k(1)*y(21);
jac(22,21)=jac(22,21)+k(1)*y(4);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~Y) 

jac(22,22)=jac(22,22)-k(22);
jac(2,22)=jac(2,22)+k(22);
jac(16,22)=jac(16,22)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(22);
jac(3,22)=jac(3,22)-k(9)*y(3);
jac(22,3)=jac(22,3)-k(9)*y(22);
jac(22,22)=jac(22,22)-k(9)*y(3);
jac(141,3)=jac(141,3)+k(9)*y(22);
jac(141,22)=jac(141,22)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(1,1)=jac(1,1)-k(7)*y(22);
jac(1,22)=jac(1,22)-k(7)*y(1);
jac(22,1)=jac(22,1)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(1);
jac(24,1)=jac(24,1)+k(7)*y(22);
jac(24,22)=jac(24,22)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(7,7)=jac(7,7)-k(7)*y(22);
jac(7,22)=jac(7,22)-k(7)*y(7);
jac(22,7)=jac(22,7)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(7);
jac(20,7)=jac(20,7)+k(7)*y(22);
jac(20,22)=jac(20,22)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(22,22)=jac(22,22)-k(7)*y(335);
jac(22,335)=jac(22,335)-k(7)*y(22);
jac(335,22)=jac(335,22)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(22);
jac(411,22)=jac(411,22)+k(7)*y(335);
jac(411,335)=jac(411,335)+k(7)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(22);
jac(257,22)=jac(257,22)-k(7)*y(257);
jac(22,257)=jac(22,257)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(257);
jac(545,257)=jac(545,257)+k(7)*y(22);
jac(545,22)=jac(545,22)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(22,22)=jac(22,22)-k(7)*y(227);
jac(22,227)=jac(22,227)-k(7)*y(22);
jac(227,22)=jac(227,22)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(22);
jac(542,22)=jac(542,22)+k(7)*y(227);
jac(542,227)=jac(542,227)+k(7)*y(22);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(22,22)=jac(22,22)-k(7)*y(222);
jac(22,222)=jac(22,222)-k(7)*y(22);
jac(222,22)=jac(222,22)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(22);
jac(511,22)=jac(511,22)+k(7)*y(222);
jac(511,222)=jac(511,222)+k(7)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(22);
jac(218,22)=jac(218,22)-k(7)*y(218);
jac(22,218)=jac(22,218)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(218);
jac(510,218)=jac(510,218)+k(7)*y(22);
jac(510,22)=jac(510,22)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(22);
jac(192,22)=jac(192,22)-k(7)*y(192);
jac(22,192)=jac(22,192)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(192);
jac(453,192)=jac(453,192)+k(7)*y(22);
jac(453,22)=jac(453,22)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(22,22)=jac(22,22)-k(7)*y(184);
jac(22,184)=jac(22,184)-k(7)*y(22);
jac(184,22)=jac(184,22)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(22);
jac(390,22)=jac(390,22)+k(7)*y(184);
jac(390,184)=jac(390,184)+k(7)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(22);
jac(177,22)=jac(177,22)-k(7)*y(177);
jac(22,177)=jac(22,177)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(177);
jac(179,177)=jac(179,177)+k(7)*y(22);
jac(179,22)=jac(179,22)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(22,22)=jac(22,22)-k(7)*y(140);
jac(22,140)=jac(22,140)-k(7)*y(22);
jac(140,22)=jac(140,22)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(22);
jac(386,22)=jac(386,22)+k(7)*y(140);
jac(386,140)=jac(386,140)+k(7)*y(22);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(22,22)=jac(22,22)-k(7)*y(131);
jac(22,131)=jac(22,131)-k(7)*y(22);
jac(131,22)=jac(131,22)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(22);
jac(445,22)=jac(445,22)+k(7)*y(131);
jac(445,131)=jac(445,131)+k(7)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(22);
jac(111,22)=jac(111,22)-k(7)*y(111);
jac(22,111)=jac(22,111)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(111);
jac(506,111)=jac(506,111)+k(7)*y(22);
jac(506,22)=jac(506,22)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(77,77)=jac(77,77)-k(7)*y(22);
jac(77,22)=jac(77,22)-k(7)*y(77);
jac(22,77)=jac(22,77)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(77);
jac(508,77)=jac(508,77)+k(7)*y(22);
jac(508,22)=jac(508,22)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(72,72)=jac(72,72)-k(7)*y(22);
jac(72,22)=jac(72,22)-k(7)*y(72);
jac(22,72)=jac(22,72)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(72);
jac(142,72)=jac(142,72)+k(7)*y(22);
jac(142,22)=jac(142,22)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(61,61)=jac(61,61)-k(7)*y(22);
jac(61,22)=jac(61,22)-k(7)*y(61);
jac(22,61)=jac(22,61)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(61);
jac(329,61)=jac(329,61)+k(7)*y(22);
jac(329,22)=jac(329,22)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(54,54)=jac(54,54)-k(7)*y(22);
jac(54,22)=jac(54,22)-k(7)*y(54);
jac(22,54)=jac(22,54)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(54);
jac(538,54)=jac(538,54)+k(7)*y(22);
jac(538,22)=jac(538,22)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(48,48)=jac(48,48)-k(7)*y(22);
jac(48,22)=jac(48,22)-k(7)*y(48);
jac(22,48)=jac(22,48)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(48);
jac(388,48)=jac(388,48)+k(7)*y(22);
jac(388,22)=jac(388,22)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(22,22)=jac(22,22)-k(7)*y(43);
jac(22,43)=jac(22,43)-k(7)*y(22);
jac(43,22)=jac(43,22)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(22);
jac(507,22)=jac(507,22)+k(7)*y(43);
jac(507,43)=jac(507,43)+k(7)*y(22);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(22,22)=jac(22,22)-k(7)*y(37);
jac(22,37)=jac(22,37)-k(7)*y(22);
jac(37,22)=jac(37,22)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(22);
jac(541,22)=jac(541,22)+k(7)*y(37);
jac(541,37)=jac(541,37)+k(7)*y(22);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(22);
jac(33,22)=jac(33,22)-k(7)*y(33);
jac(22,33)=jac(22,33)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(33);
jac(575,33)=jac(575,33)+k(7)*y(22);
jac(575,22)=jac(575,22)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(22);
jac(27,22)=jac(27,22)-k(7)*y(27);
jac(22,27)=jac(22,27)-k(7)*y(22);
jac(22,22)=jac(22,22)-k(7)*y(27);
jac(166,27)=jac(166,27)+k(7)*y(22);
jac(166,22)=jac(166,22)+k(7)*y(27);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) 

jac(22,22)=jac(22,22)-k(2);
jac(4,22)=jac(4,22)+k(2);
jac(21,22)=jac(21,22)+k(2);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(23,23)=jac(23,23)-k(23);
jac(29,23)=jac(29,23)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(23,23)=jac(23,23)-k(22);
jac(2,23)=jac(2,23)+k(22);
jac(17,23)=jac(17,23)+k(22);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(23,23)=jac(23,23)-k(14);
jac(467,23)=jac(467,23)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(23,23)=jac(23,23)-k(10);
jac(3,23)=jac(3,23)+k(10);
jac(20,23)=jac(20,23)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(23,23)=jac(23,23)-k(10);
jac(3,23)=jac(3,23)+k(10);
jac(144,23)=jac(144,23)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(23,23)=jac(23,23)-k(8);
jac(7,23)=jac(7,23)+k(8);
jac(141,23)=jac(141,23)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(23,23)=jac(23,23)-k(6);
jac(9,23)=jac(9,23)+k(6);
jac(140,23)=jac(140,23)+k(6);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~Y) 

jac(24,24)=jac(24,24)-k(22);
jac(2,24)=jac(2,24)+k(22);
jac(18,24)=jac(18,24)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(24);
jac(3,24)=jac(3,24)-k(9)*y(3);
jac(24,3)=jac(24,3)-k(9)*y(24);
jac(24,24)=jac(24,24)-k(9)*y(3);
jac(144,3)=jac(144,3)+k(9)*y(24);
jac(144,24)=jac(144,24)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(24);
jac(3,24)=jac(3,24)-k(9)*y(3);
jac(24,3)=jac(24,3)-k(9)*y(24);
jac(24,24)=jac(24,24)-k(9)*y(3);
jac(20,3)=jac(20,3)+k(9)*y(24);
jac(20,24)=jac(20,24)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

jac(24,24)=jac(24,24)-k(8);
jac(1,24)=jac(1,24)+k(8);
jac(22,24)=jac(22,24)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(24,24)=jac(24,24)-k(6);
jac(6,24)=jac(6,24)+k(6);
jac(21,24)=jac(21,24)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(25,25)=jac(25,25)-k(31);
jac(20,25)=jac(20,25)+k(31);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(25,25)=jac(25,25)-k(23);
jac(31,25)=jac(31,25)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(25,25)=jac(25,25)-k(22);
jac(2,25)=jac(2,25)+k(22);
jac(19,25)=jac(19,25)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(25);
jac(3,25)=jac(3,25)-k(15)*y(3);
jac(25,3)=jac(25,3)-k(15)*y(25);
jac(25,25)=jac(25,25)-k(15)*y(3);
jac(42,3)=jac(42,3)+k(15)*y(25);
jac(42,25)=jac(42,25)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(25,25)=jac(25,25)-k(10);
jac(3,25)=jac(3,25)+k(10);
jac(621,25)=jac(621,25)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(25,25)=jac(25,25)-k(8);
jac(7,25)=jac(7,25)+k(8);
jac(437,25)=jac(437,25)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(25,25)=jac(25,25)-k(6);
jac(9,25)=jac(9,25)+k(6);
jac(111,25)=jac(111,25)+k(6);

% rule    : R16
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(27,27)=jac(27,27)-k(33);
jac(21,27)=jac(21,27)+k(33);

% rule    : R9_op
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(22);
jac(32,27)=jac(32,27)+k(22);
jac(15,27)=jac(15,27)+k(22);

% rule    : R3
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(27);
jac(3,27)=jac(3,27)-k(9)*y(3);
jac(27,3)=jac(27,3)-k(9)*y(27);
jac(27,27)=jac(27,27)-k(9)*y(3);
jac(131,3)=jac(131,3)+k(9)*y(27);
jac(131,27)=jac(131,27)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(28);
jac(27,28)=jac(27,28)-k(7)*y(27);
jac(28,27)=jac(28,27)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(27);
jac(160,27)=jac(160,27)+k(7)*y(28);
jac(160,28)=jac(160,28)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(34);
jac(27,34)=jac(27,34)-k(7)*y(27);
jac(34,27)=jac(34,27)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(27);
jac(468,27)=jac(468,27)+k(7)*y(34);
jac(468,34)=jac(468,34)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(38);
jac(27,38)=jac(27,38)-k(7)*y(27);
jac(38,27)=jac(38,27)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(27);
jac(483,27)=jac(483,27)+k(7)*y(38);
jac(483,38)=jac(483,38)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(44);
jac(27,44)=jac(27,44)-k(7)*y(27);
jac(44,27)=jac(44,27)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(27);
jac(277,27)=jac(277,27)+k(7)*y(44);
jac(277,44)=jac(277,44)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(55,55)=jac(55,55)-k(7)*y(27);
jac(55,27)=jac(55,27)-k(7)*y(55);
jac(27,55)=jac(27,55)-k(7)*y(27);
jac(27,27)=jac(27,27)-k(7)*y(55);
jac(287,55)=jac(287,55)+k(7)*y(27);
jac(287,27)=jac(287,27)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(62,62)=jac(62,62)-k(7)*y(27);
jac(62,27)=jac(62,27)-k(7)*y(62);
jac(27,62)=jac(27,62)-k(7)*y(27);
jac(27,27)=jac(27,27)-k(7)*y(62);
jac(85,62)=jac(85,62)+k(7)*y(27);
jac(85,27)=jac(85,27)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(91,91)=jac(91,91)-k(7)*y(27);
jac(91,27)=jac(91,27)-k(7)*y(91);
jac(27,91)=jac(27,91)-k(7)*y(27);
jac(27,27)=jac(27,27)-k(7)*y(91);
jac(87,91)=jac(87,91)+k(7)*y(27);
jac(87,27)=jac(87,27)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(126,126)=jac(126,126)-k(7)*y(27);
jac(126,27)=jac(126,27)-k(7)*y(126);
jac(27,126)=jac(27,126)-k(7)*y(27);
jac(27,27)=jac(27,27)-k(7)*y(126);
jac(153,126)=jac(153,126)+k(7)*y(27);
jac(153,27)=jac(153,27)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(132);
jac(27,132)=jac(27,132)-k(7)*y(27);
jac(132,27)=jac(132,27)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(27);
jac(158,27)=jac(158,27)+k(7)*y(132);
jac(158,132)=jac(158,132)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(141);
jac(27,141)=jac(27,141)-k(7)*y(27);
jac(141,27)=jac(141,27)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(27);
jac(164,27)=jac(164,27)+k(7)*y(141);
jac(164,141)=jac(164,141)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(185);
jac(27,185)=jac(27,185)-k(7)*y(27);
jac(185,27)=jac(185,27)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(27);
jac(168,27)=jac(168,27)+k(7)*y(185);
jac(168,185)=jac(168,185)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(27,27)=jac(27,27)-k(7)*y(209);
jac(27,209)=jac(27,209)-k(7)*y(27);
jac(209,27)=jac(209,27)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(27);
jac(454,27)=jac(454,27)+k(7)*y(209);
jac(454,209)=jac(454,209)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(214);
jac(27,214)=jac(27,214)-k(7)*y(27);
jac(214,27)=jac(214,27)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(27);
jac(449,27)=jac(449,27)+k(7)*y(214);
jac(449,214)=jac(449,214)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(219);
jac(27,219)=jac(27,219)-k(7)*y(27);
jac(219,27)=jac(219,27)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(27);
jac(285,27)=jac(285,27)+k(7)*y(219);
jac(285,219)=jac(285,219)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(223);
jac(27,223)=jac(27,223)-k(7)*y(27);
jac(223,27)=jac(223,27)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(27);
jac(283,27)=jac(283,27)+k(7)*y(223);
jac(283,223)=jac(283,223)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(228);
jac(27,228)=jac(27,228)-k(7)*y(27);
jac(228,27)=jac(228,27)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(27);
jac(474,27)=jac(474,27)+k(7)*y(228);
jac(474,228)=jac(474,228)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(315,315)=jac(315,315)-k(7)*y(27);
jac(315,27)=jac(315,27)-k(7)*y(315);
jac(27,315)=jac(27,315)-k(7)*y(27);
jac(27,27)=jac(27,27)-k(7)*y(315);
jac(615,315)=jac(615,315)+k(7)*y(27);
jac(615,27)=jac(615,27)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(319);
jac(27,319)=jac(27,319)-k(7)*y(27);
jac(319,27)=jac(319,27)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(27);
jac(472,27)=jac(472,27)+k(7)*y(319);
jac(472,319)=jac(472,319)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(336);
jac(27,336)=jac(27,336)-k(7)*y(27);
jac(336,27)=jac(336,27)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(27);
jac(162,27)=jac(162,27)+k(7)*y(336);
jac(162,336)=jac(162,336)+k(7)*y(27);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(7)*y(437);
jac(27,437)=jac(27,437)-k(7)*y(27);
jac(437,27)=jac(437,27)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(27);
jac(442,27)=jac(442,27)+k(7)*y(437);
jac(442,437)=jac(442,437)+k(7)*y(27);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(47,47)=jac(47,47)-k(5)*y(27);
jac(47,27)=jac(47,27)-k(5)*y(47);
jac(27,47)=jac(27,47)-k(5)*y(27);
jac(27,27)=jac(27,27)-k(5)*y(47);
jac(384,47)=jac(384,47)+k(5)*y(27);
jac(384,27)=jac(384,27)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(71,71)=jac(71,71)-k(5)*y(27);
jac(71,27)=jac(71,27)-k(5)*y(71);
jac(27,71)=jac(27,71)-k(5)*y(27);
jac(27,27)=jac(27,27)-k(5)*y(71);
jac(136,71)=jac(136,71)+k(5)*y(27);
jac(136,27)=jac(136,27)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(76,76)=jac(76,76)-k(5)*y(27);
jac(76,27)=jac(76,27)-k(5)*y(76);
jac(27,76)=jac(27,76)-k(5)*y(27);
jac(27,27)=jac(27,27)-k(5)*y(76);
jac(533,76)=jac(533,76)+k(5)*y(27);
jac(533,27)=jac(533,27)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(5)*y(82);
jac(27,82)=jac(27,82)-k(5)*y(27);
jac(82,27)=jac(82,27)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(27);
jac(612,27)=jac(612,27)+k(5)*y(82);
jac(612,82)=jac(612,82)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(5)*y(86);
jac(27,86)=jac(27,86)-k(5)*y(27);
jac(86,27)=jac(86,27)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(27);
jac(160,27)=jac(160,27)+k(5)*y(86);
jac(160,86)=jac(160,86)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(27,27)=jac(27,27)-k(5)*y(89);
jac(27,89)=jac(27,89)-k(5)*y(27);
jac(89,27)=jac(89,27)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(27);
jac(378,27)=jac(378,27)+k(5)*y(89);
jac(378,89)=jac(378,89)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(98,98)=jac(98,98)-k(5)*y(27);
jac(98,27)=jac(98,27)-k(5)*y(98);
jac(27,98)=jac(27,98)-k(5)*y(27);
jac(27,27)=jac(27,27)-k(5)*y(98);
jac(156,98)=jac(156,98)+k(5)*y(27);
jac(156,27)=jac(156,27)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(27,27)=jac(27,27)-k(5)*y(110);
jac(27,110)=jac(27,110)-k(5)*y(27);
jac(110,27)=jac(110,27)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(27);
jac(356,27)=jac(356,27)+k(5)*y(110);
jac(356,110)=jac(356,110)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(27,27)=jac(27,27)-k(5)*y(114);
jac(27,114)=jac(27,114)-k(5)*y(27);
jac(114,27)=jac(114,27)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(27);
jac(530,27)=jac(530,27)+k(5)*y(114);
jac(530,114)=jac(530,114)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(118,118)=jac(118,118)-k(5)*y(27);
jac(118,27)=jac(118,27)-k(5)*y(118);
jac(27,118)=jac(27,118)-k(5)*y(27);
jac(27,27)=jac(27,27)-k(5)*y(118);
jac(531,118)=jac(531,118)+k(5)*y(27);
jac(531,27)=jac(531,27)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(122,122)=jac(122,122)-k(5)*y(27);
jac(122,27)=jac(122,27)-k(5)*y(122);
jac(27,122)=jac(27,122)-k(5)*y(27);
jac(27,27)=jac(27,27)-k(5)*y(122);
jac(133,122)=jac(133,122)+k(5)*y(27);
jac(133,27)=jac(133,27)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(27,27)=jac(27,27)-k(5)*y(176);
jac(27,176)=jac(27,176)-k(5)*y(27);
jac(176,27)=jac(176,27)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(27);
jac(366,27)=jac(366,27)+k(5)*y(176);
jac(366,176)=jac(366,176)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(27,27)=jac(27,27)-k(5)*y(191);
jac(27,191)=jac(27,191)-k(5)*y(27);
jac(191,27)=jac(191,27)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(27);
jac(456,27)=jac(456,27)+k(5)*y(191);
jac(456,191)=jac(456,191)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(5)*y(256);
jac(27,256)=jac(27,256)-k(5)*y(27);
jac(256,27)=jac(256,27)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(27);
jac(608,27)=jac(608,27)+k(5)*y(256);
jac(608,256)=jac(608,256)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(27,27)=jac(27,27)-k(5)*y(264);
jac(27,264)=jac(27,264)-k(5)*y(27);
jac(264,27)=jac(264,27)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(27);
jac(359,27)=jac(359,27)+k(5)*y(264);
jac(359,264)=jac(359,264)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(27,27)=jac(27,27)-k(5)*y(380);
jac(27,380)=jac(27,380)-k(5)*y(27);
jac(380,27)=jac(380,27)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(27);
jac(379,27)=jac(379,27)+k(5)*y(380);
jac(379,380)=jac(379,380)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(27,27)=jac(27,27)-k(5)*y(391);
jac(27,391)=jac(27,391)-k(5)*y(27);
jac(391,27)=jac(391,27)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(27);
jac(385,27)=jac(385,27)+k(5)*y(391);
jac(385,391)=jac(385,391)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(5)*y(407);
jac(27,407)=jac(27,407)-k(5)*y(27);
jac(407,27)=jac(407,27)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(27);
jac(412,27)=jac(412,27)+k(5)*y(407);
jac(412,407)=jac(412,407)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(5)*y(441);
jac(27,441)=jac(27,441)-k(5)*y(27);
jac(441,27)=jac(441,27)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(27);
jac(446,27)=jac(446,27)+k(5)*y(441);
jac(446,441)=jac(446,441)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(27,27)=jac(27,27)-k(5)*y(515);
jac(27,515)=jac(27,515)-k(5)*y(27);
jac(515,27)=jac(515,27)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(27);
jac(532,27)=jac(532,27)+k(5)*y(515);
jac(532,515)=jac(532,515)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(27,27)=jac(27,27)-k(5)*y(543);
jac(27,543)=jac(27,543)-k(5)*y(27);
jac(543,27)=jac(543,27)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(27);
jac(536,27)=jac(536,27)+k(5)*y(543);
jac(536,543)=jac(536,543)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(27,27)=jac(27,27)-k(5)*y(554);
jac(27,554)=jac(27,554)-k(5)*y(27);
jac(554,27)=jac(554,27)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(27);
jac(535,27)=jac(535,27)+k(5)*y(554);
jac(535,554)=jac(535,554)+k(5)*y(27);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(9,9)=jac(9,9)-k(5)*y(27);
jac(9,27)=jac(9,27)-k(5)*y(9);
jac(27,9)=jac(27,9)-k(5)*y(27);
jac(27,27)=jac(27,27)-k(5)*y(9);
jac(26,9)=jac(26,9)+k(5)*y(27);
jac(26,27)=jac(26,27)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(4,4)=jac(4,4)-k(3)*y(27);
jac(4,27)=jac(4,27)-k(3)*y(4);
jac(27,4)=jac(27,4)-k(3)*y(27);
jac(27,27)=jac(27,27)-k(3)*y(4);
jac(86,4)=jac(86,4)+k(3)*y(27);
jac(86,27)=jac(86,27)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(4,4)=jac(4,4)-k(1)*y(27);
jac(4,27)=jac(4,27)-k(1)*y(4);
jac(27,4)=jac(27,4)-k(1)*y(27);
jac(27,27)=jac(27,27)-k(1)*y(4);
jac(28,4)=jac(28,4)+k(1)*y(27);
jac(28,27)=jac(28,27)+k(1)*y(4);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(28,28)=jac(28,28)-k(33);
jac(22,28)=jac(22,28)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~Y) 

jac(28,28)=jac(28,28)-k(22);
jac(32,28)=jac(32,28)+k(22);
jac(16,28)=jac(16,28)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(28);
jac(3,28)=jac(3,28)-k(9)*y(3);
jac(28,3)=jac(28,3)-k(9)*y(28);
jac(28,28)=jac(28,28)-k(9)*y(3);
jac(132,3)=jac(132,3)+k(9)*y(28);
jac(132,28)=jac(132,28)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(1,1)=jac(1,1)-k(7)*y(28);
jac(1,28)=jac(1,28)-k(7)*y(1);
jac(28,1)=jac(28,1)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(1);
jac(30,1)=jac(30,1)+k(7)*y(28);
jac(30,28)=jac(30,28)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(7,7)=jac(7,7)-k(7)*y(28);
jac(7,28)=jac(7,28)-k(7)*y(7);
jac(28,7)=jac(28,7)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(7);
jac(26,7)=jac(26,7)+k(7)*y(28);
jac(26,28)=jac(26,28)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(28,28)=jac(28,28)-k(7)*y(335);
jac(28,335)=jac(28,335)-k(7)*y(28);
jac(335,28)=jac(335,28)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(28);
jac(412,28)=jac(412,28)+k(7)*y(335);
jac(412,335)=jac(412,335)+k(7)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(257,257)=jac(257,257)-k(7)*y(28);
jac(257,28)=jac(257,28)-k(7)*y(257);
jac(28,257)=jac(28,257)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(257);
jac(608,257)=jac(608,257)+k(7)*y(28);
jac(608,28)=jac(608,28)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(28,28)=jac(28,28)-k(7)*y(227);
jac(28,227)=jac(28,227)-k(7)*y(28);
jac(227,28)=jac(227,28)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(28);
jac(536,28)=jac(536,28)+k(7)*y(227);
jac(536,227)=jac(536,227)+k(7)*y(28);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(28,28)=jac(28,28)-k(7)*y(222);
jac(28,222)=jac(28,222)-k(7)*y(28);
jac(222,28)=jac(222,28)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(28);
jac(532,28)=jac(532,28)+k(7)*y(222);
jac(532,222)=jac(532,222)+k(7)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(28);
jac(218,28)=jac(218,28)-k(7)*y(218);
jac(28,218)=jac(28,218)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(218);
jac(359,218)=jac(359,218)+k(7)*y(28);
jac(359,28)=jac(359,28)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(28);
jac(192,28)=jac(192,28)-k(7)*y(192);
jac(28,192)=jac(28,192)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(192);
jac(456,192)=jac(456,192)+k(7)*y(28);
jac(456,28)=jac(456,28)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(28,28)=jac(28,28)-k(7)*y(184);
jac(28,184)=jac(28,184)-k(7)*y(28);
jac(184,28)=jac(184,28)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(28);
jac(385,28)=jac(385,28)+k(7)*y(184);
jac(385,184)=jac(385,184)+k(7)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(28);
jac(177,28)=jac(177,28)-k(7)*y(177);
jac(28,177)=jac(28,177)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(177);
jac(366,177)=jac(366,177)+k(7)*y(28);
jac(366,28)=jac(366,28)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(28,28)=jac(28,28)-k(7)*y(140);
jac(28,140)=jac(28,140)-k(7)*y(28);
jac(140,28)=jac(140,28)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(28);
jac(379,28)=jac(379,28)+k(7)*y(140);
jac(379,140)=jac(379,140)+k(7)*y(28);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(28,28)=jac(28,28)-k(7)*y(131);
jac(28,131)=jac(28,131)-k(7)*y(28);
jac(131,28)=jac(131,28)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(28);
jac(446,28)=jac(446,28)+k(7)*y(131);
jac(446,131)=jac(446,131)+k(7)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(28);
jac(111,28)=jac(111,28)-k(7)*y(111);
jac(28,111)=jac(28,111)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(111);
jac(356,111)=jac(356,111)+k(7)*y(28);
jac(356,28)=jac(356,28)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(77,77)=jac(77,77)-k(7)*y(28);
jac(77,28)=jac(77,28)-k(7)*y(77);
jac(28,77)=jac(28,77)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(77);
jac(533,77)=jac(533,77)+k(7)*y(28);
jac(533,28)=jac(533,28)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(72,72)=jac(72,72)-k(7)*y(28);
jac(72,28)=jac(72,28)-k(7)*y(72);
jac(28,72)=jac(28,72)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(72);
jac(136,72)=jac(136,72)+k(7)*y(28);
jac(136,28)=jac(136,28)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(61,61)=jac(61,61)-k(7)*y(28);
jac(61,28)=jac(61,28)-k(7)*y(61);
jac(28,61)=jac(28,61)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(61);
jac(133,61)=jac(133,61)+k(7)*y(28);
jac(133,28)=jac(133,28)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(54,54)=jac(54,54)-k(7)*y(28);
jac(54,28)=jac(54,28)-k(7)*y(54);
jac(28,54)=jac(28,54)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(54);
jac(531,54)=jac(531,54)+k(7)*y(28);
jac(531,28)=jac(531,28)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(48,48)=jac(48,48)-k(7)*y(28);
jac(48,28)=jac(48,28)-k(7)*y(48);
jac(28,48)=jac(28,48)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(48);
jac(384,48)=jac(384,48)+k(7)*y(28);
jac(384,28)=jac(384,28)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(28,28)=jac(28,28)-k(7)*y(43);
jac(28,43)=jac(28,43)-k(7)*y(28);
jac(43,28)=jac(43,28)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(28);
jac(530,28)=jac(530,28)+k(7)*y(43);
jac(530,43)=jac(530,43)+k(7)*y(28);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(28,28)=jac(28,28)-k(7)*y(37);
jac(28,37)=jac(28,37)-k(7)*y(28);
jac(37,28)=jac(37,28)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(28);
jac(535,28)=jac(535,28)+k(7)*y(37);
jac(535,37)=jac(535,37)+k(7)*y(28);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(28);
jac(33,28)=jac(33,28)-k(7)*y(33);
jac(28,33)=jac(28,33)-k(7)*y(28);
jac(28,28)=jac(28,28)-k(7)*y(33);
jac(612,33)=jac(612,33)+k(7)*y(28);
jac(612,28)=jac(612,28)+k(7)*y(33);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) 

jac(28,28)=jac(28,28)-k(2);
jac(4,28)=jac(4,28)+k(2);
jac(27,28)=jac(27,28)+k(2);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(29,29)=jac(29,29)-k(33);
jac(23,29)=jac(23,29)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(29,29)=jac(29,29)-k(22);
jac(32,29)=jac(32,29)+k(22);
jac(17,29)=jac(17,29)+k(22);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(29,29)=jac(29,29)-k(14);
jac(155,29)=jac(155,29)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(29,29)=jac(29,29)-k(10);
jac(3,29)=jac(3,29)+k(10);
jac(26,29)=jac(26,29)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(29,29)=jac(29,29)-k(10);
jac(3,29)=jac(3,29)+k(10);
jac(138,29)=jac(138,29)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(29,29)=jac(29,29)-k(8);
jac(7,29)=jac(7,29)+k(8);
jac(132,29)=jac(132,29)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(29,29)=jac(29,29)-k(6);
jac(9,29)=jac(9,29)+k(6);
jac(131,29)=jac(131,29)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(30,30)=jac(30,30)-k(33);
jac(24,30)=jac(24,30)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~Y) 

jac(30,30)=jac(30,30)-k(22);
jac(32,30)=jac(32,30)+k(22);
jac(18,30)=jac(18,30)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(30);
jac(3,30)=jac(3,30)-k(9)*y(3);
jac(30,3)=jac(30,3)-k(9)*y(30);
jac(30,30)=jac(30,30)-k(9)*y(3);
jac(138,3)=jac(138,3)+k(9)*y(30);
jac(138,30)=jac(138,30)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(30);
jac(3,30)=jac(3,30)-k(9)*y(3);
jac(30,3)=jac(30,3)-k(9)*y(30);
jac(30,30)=jac(30,30)-k(9)*y(3);
jac(26,3)=jac(26,3)+k(9)*y(30);
jac(26,30)=jac(26,30)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

jac(30,30)=jac(30,30)-k(8);
jac(1,30)=jac(1,30)+k(8);
jac(28,30)=jac(28,30)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(30,30)=jac(30,30)-k(6);
jac(6,30)=jac(6,30)+k(6);
jac(27,30)=jac(27,30)+k(6);

% rule    : R16
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(33,33)=jac(33,33)-k(33);
jac(111,33)=jac(111,33)+k(33);

% rule    : R14
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(31);
jac(27,33)=jac(27,33)+k(31);

% rule    : R9_op
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(22);
jac(32,33)=jac(32,33)+k(22);
jac(77,33)=jac(77,33)+k(22);

% rule    : R6
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(33);
jac(3,33)=jac(3,33)-k(15)*y(3);
jac(33,3)=jac(33,3)-k(15)*y(33);
jac(33,33)=jac(33,33)-k(15)*y(3);
jac(37,3)=jac(37,3)+k(15)*y(33);
jac(37,33)=jac(37,33)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(34);
jac(33,34)=jac(33,34)-k(7)*y(33);
jac(34,33)=jac(34,33)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(33);
jac(568,33)=jac(568,33)+k(7)*y(34);
jac(568,34)=jac(568,34)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(38);
jac(33,38)=jac(33,38)-k(7)*y(33);
jac(38,33)=jac(38,33)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(33);
jac(247,33)=jac(247,33)+k(7)*y(38);
jac(247,38)=jac(247,38)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(44);
jac(33,44)=jac(33,44)-k(7)*y(33);
jac(44,33)=jac(44,33)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(33);
jac(254,33)=jac(254,33)+k(7)*y(44);
jac(254,44)=jac(254,44)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(55,55)=jac(55,55)-k(7)*y(33);
jac(55,33)=jac(55,33)-k(7)*y(55);
jac(33,55)=jac(33,55)-k(7)*y(33);
jac(33,33)=jac(33,33)-k(7)*y(55);
jac(634,55)=jac(634,55)+k(7)*y(33);
jac(634,33)=jac(634,33)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(62,62)=jac(62,62)-k(7)*y(33);
jac(62,33)=jac(62,33)-k(7)*y(62);
jac(33,62)=jac(33,62)-k(7)*y(33);
jac(33,33)=jac(33,33)-k(7)*y(62);
jac(81,62)=jac(81,62)+k(7)*y(33);
jac(81,33)=jac(81,33)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(91,91)=jac(91,91)-k(7)*y(33);
jac(91,33)=jac(91,33)-k(7)*y(91);
jac(33,91)=jac(33,91)-k(7)*y(33);
jac(33,33)=jac(33,33)-k(7)*y(91);
jac(84,91)=jac(84,91)+k(7)*y(33);
jac(84,33)=jac(84,33)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(126,126)=jac(126,126)-k(7)*y(33);
jac(126,33)=jac(126,33)-k(7)*y(126);
jac(33,126)=jac(33,126)-k(7)*y(33);
jac(33,33)=jac(33,33)-k(7)*y(126);
jac(350,126)=jac(350,126)+k(7)*y(33);
jac(350,33)=jac(350,33)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(132);
jac(33,132)=jac(33,132)-k(7)*y(33);
jac(132,33)=jac(132,33)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(33);
jac(161,33)=jac(161,33)+k(7)*y(132);
jac(161,132)=jac(161,132)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(141);
jac(33,141)=jac(33,141)-k(7)*y(33);
jac(141,33)=jac(141,33)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(33);
jac(167,33)=jac(167,33)+k(7)*y(141);
jac(167,141)=jac(167,141)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(185);
jac(33,185)=jac(33,185)-k(7)*y(33);
jac(185,33)=jac(185,33)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(33);
jac(347,33)=jac(347,33)+k(7)*y(185);
jac(347,185)=jac(347,185)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(209);
jac(33,209)=jac(33,209)-k(7)*y(33);
jac(209,33)=jac(209,33)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(33);
jac(559,33)=jac(559,33)+k(7)*y(209);
jac(559,209)=jac(559,209)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(214);
jac(33,214)=jac(33,214)-k(7)*y(33);
jac(214,33)=jac(214,33)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(33);
jac(348,33)=jac(348,33)+k(7)*y(214);
jac(348,214)=jac(348,214)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(219);
jac(33,219)=jac(33,219)-k(7)*y(33);
jac(219,33)=jac(219,33)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(33);
jac(574,33)=jac(574,33)+k(7)*y(219);
jac(574,219)=jac(574,219)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(223);
jac(33,223)=jac(33,223)-k(7)*y(33);
jac(223,33)=jac(223,33)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(33);
jac(286,33)=jac(286,33)+k(7)*y(223);
jac(286,223)=jac(286,223)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(33,33)=jac(33,33)-k(7)*y(228);
jac(33,228)=jac(33,228)-k(7)*y(33);
jac(228,33)=jac(228,33)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(33);
jac(635,33)=jac(635,33)+k(7)*y(228);
jac(635,228)=jac(635,228)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(315,315)=jac(315,315)-k(7)*y(33);
jac(315,33)=jac(315,33)-k(7)*y(315);
jac(33,315)=jac(33,315)-k(7)*y(33);
jac(33,33)=jac(33,33)-k(7)*y(315);
jac(573,315)=jac(573,315)+k(7)*y(33);
jac(573,33)=jac(573,33)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(319);
jac(33,319)=jac(33,319)-k(7)*y(33);
jac(319,33)=jac(319,33)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(33);
jac(562,33)=jac(562,33)+k(7)*y(319);
jac(562,319)=jac(562,319)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(336);
jac(33,336)=jac(33,336)-k(7)*y(33);
jac(336,33)=jac(336,33)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(33);
jac(341,33)=jac(341,33)+k(7)*y(336);
jac(341,336)=jac(341,336)+k(7)*y(33);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(7)*y(437);
jac(33,437)=jac(33,437)-k(7)*y(33);
jac(437,33)=jac(437,33)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(33);
jac(572,33)=jac(572,33)+k(7)*y(437);
jac(572,437)=jac(572,437)+k(7)*y(33);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(47,47)=jac(47,47)-k(5)*y(33);
jac(47,33)=jac(47,33)-k(5)*y(47);
jac(33,47)=jac(33,47)-k(5)*y(33);
jac(33,33)=jac(33,33)-k(5)*y(47);
jac(470,47)=jac(470,47)+k(5)*y(33);
jac(470,33)=jac(470,33)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(71,71)=jac(71,71)-k(5)*y(33);
jac(71,33)=jac(71,33)-k(5)*y(71);
jac(33,71)=jac(33,71)-k(5)*y(33);
jac(33,33)=jac(33,33)-k(5)*y(71);
jac(242,71)=jac(242,71)+k(5)*y(33);
jac(242,33)=jac(242,33)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(76,76)=jac(76,76)-k(5)*y(33);
jac(76,33)=jac(76,33)-k(5)*y(76);
jac(33,76)=jac(33,76)-k(5)*y(33);
jac(33,33)=jac(33,33)-k(5)*y(76);
jac(245,76)=jac(245,76)+k(5)*y(33);
jac(245,33)=jac(245,33)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(5)*y(82);
jac(33,82)=jac(33,82)-k(5)*y(33);
jac(82,33)=jac(82,33)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(33);
jac(568,33)=jac(568,33)+k(5)*y(82);
jac(568,82)=jac(568,82)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(5)*y(86);
jac(33,86)=jac(33,86)-k(5)*y(33);
jac(86,33)=jac(86,33)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(33);
jac(468,33)=jac(468,33)+k(5)*y(86);
jac(468,86)=jac(468,86)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(33,33)=jac(33,33)-k(5)*y(89);
jac(33,89)=jac(33,89)-k(5)*y(33);
jac(89,33)=jac(89,33)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(33);
jac(482,33)=jac(482,33)+k(5)*y(89);
jac(482,89)=jac(482,89)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(98,98)=jac(98,98)-k(5)*y(33);
jac(98,33)=jac(98,33)-k(5)*y(98);
jac(33,98)=jac(33,98)-k(5)*y(33);
jac(33,33)=jac(33,33)-k(5)*y(98);
jac(617,98)=jac(617,98)+k(5)*y(33);
jac(617,33)=jac(617,33)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(33,33)=jac(33,33)-k(5)*y(110);
jac(33,110)=jac(33,110)-k(5)*y(33);
jac(110,33)=jac(110,33)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(33);
jac(499,33)=jac(499,33)+k(5)*y(110);
jac(499,110)=jac(499,110)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(33,33)=jac(33,33)-k(5)*y(114);
jac(33,114)=jac(33,114)-k(5)*y(33);
jac(114,33)=jac(114,33)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(33);
jac(500,33)=jac(500,33)+k(5)*y(114);
jac(500,114)=jac(500,114)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(118,118)=jac(118,118)-k(5)*y(33);
jac(118,33)=jac(118,33)-k(5)*y(118);
jac(33,118)=jac(33,118)-k(5)*y(33);
jac(33,33)=jac(33,33)-k(5)*y(118);
jac(534,118)=jac(534,118)+k(5)*y(33);
jac(534,33)=jac(534,33)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(122,122)=jac(122,122)-k(5)*y(33);
jac(122,33)=jac(122,33)-k(5)*y(122);
jac(33,122)=jac(33,122)-k(5)*y(33);
jac(33,33)=jac(33,33)-k(5)*y(122);
jac(317,122)=jac(317,122)+k(5)*y(33);
jac(317,33)=jac(317,33)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(33,33)=jac(33,33)-k(5)*y(176);
jac(33,176)=jac(33,176)-k(5)*y(33);
jac(176,33)=jac(176,33)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(33);
jac(498,33)=jac(498,33)+k(5)*y(176);
jac(498,176)=jac(498,176)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(33,33)=jac(33,33)-k(5)*y(191);
jac(33,191)=jac(33,191)-k(5)*y(33);
jac(191,33)=jac(191,33)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(33);
jac(620,33)=jac(620,33)+k(5)*y(191);
jac(620,191)=jac(620,191)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(5)*y(256);
jac(33,256)=jac(33,256)-k(5)*y(33);
jac(256,33)=jac(256,33)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(33);
jac(582,33)=jac(582,33)+k(5)*y(256);
jac(582,256)=jac(582,256)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(33,33)=jac(33,33)-k(5)*y(264);
jac(33,264)=jac(33,264)-k(5)*y(33);
jac(264,33)=jac(264,33)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(33);
jac(495,33)=jac(495,33)+k(5)*y(264);
jac(495,264)=jac(495,264)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(33,33)=jac(33,33)-k(5)*y(380);
jac(33,380)=jac(33,380)-k(5)*y(33);
jac(380,33)=jac(380,33)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(33);
jac(382,33)=jac(382,33)+k(5)*y(380);
jac(382,380)=jac(382,380)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(5)*y(391);
jac(33,391)=jac(33,391)-k(5)*y(33);
jac(391,33)=jac(391,33)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(33);
jac(423,33)=jac(423,33)+k(5)*y(391);
jac(423,391)=jac(423,391)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(33,33)=jac(33,33)-k(5)*y(407);
jac(33,407)=jac(33,407)-k(5)*y(33);
jac(407,33)=jac(407,33)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(33);
jac(439,33)=jac(439,33)+k(5)*y(407);
jac(439,407)=jac(439,407)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(5)*y(441);
jac(33,441)=jac(33,441)-k(5)*y(33);
jac(441,33)=jac(441,33)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(33);
jac(444,33)=jac(444,33)+k(5)*y(441);
jac(444,441)=jac(444,441)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(33,33)=jac(33,33)-k(5)*y(515);
jac(33,515)=jac(33,515)-k(5)*y(33);
jac(515,33)=jac(515,33)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(33);
jac(496,33)=jac(496,33)+k(5)*y(515);
jac(496,515)=jac(496,515)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(33,33)=jac(33,33)-k(5)*y(543);
jac(33,543)=jac(33,543)-k(5)*y(33);
jac(543,33)=jac(543,33)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(33);
jac(569,33)=jac(569,33)+k(5)*y(543);
jac(569,543)=jac(569,543)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(33,33)=jac(33,33)-k(5)*y(554);
jac(33,554)=jac(33,554)-k(5)*y(33);
jac(554,33)=jac(554,33)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(33);
jac(566,33)=jac(566,33)+k(5)*y(554);
jac(566,554)=jac(566,554)+k(5)*y(33);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(9,9)=jac(9,9)-k(5)*y(33);
jac(9,33)=jac(9,33)-k(5)*y(9);
jac(33,9)=jac(33,9)-k(5)*y(33);
jac(33,33)=jac(33,33)-k(5)*y(9);
jac(31,9)=jac(31,9)+k(5)*y(33);
jac(31,33)=jac(31,33)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(4,4)=jac(4,4)-k(3)*y(33);
jac(4,33)=jac(4,33)-k(3)*y(4);
jac(33,4)=jac(33,4)-k(3)*y(33);
jac(33,33)=jac(33,33)-k(3)*y(4);
jac(82,4)=jac(82,4)+k(3)*y(33);
jac(82,33)=jac(82,33)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(4,4)=jac(4,4)-k(1)*y(33);
jac(4,33)=jac(4,33)-k(1)*y(4);
jac(33,4)=jac(33,4)-k(1)*y(33);
jac(33,33)=jac(33,33)-k(1)*y(4);
jac(34,4)=jac(34,4)+k(1)*y(33);
jac(34,33)=jac(34,33)+k(1)*y(4);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(34,34)=jac(34,34)-k(33);
jac(437,34)=jac(437,34)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(34,34)=jac(34,34)-k(31);
jac(28,34)=jac(28,34)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~Y) 

jac(34,34)=jac(34,34)-k(22);
jac(32,34)=jac(32,34)+k(22);
jac(315,34)=jac(315,34)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(34);
jac(3,34)=jac(3,34)-k(15)*y(3);
jac(34,3)=jac(34,3)-k(15)*y(34);
jac(34,34)=jac(34,34)-k(15)*y(3);
jac(38,3)=jac(38,3)+k(15)*y(34);
jac(38,34)=jac(38,34)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(1,1)=jac(1,1)-k(7)*y(34);
jac(1,34)=jac(1,34)-k(7)*y(1);
jac(34,1)=jac(34,1)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(1);
jac(35,1)=jac(35,1)+k(7)*y(34);
jac(35,34)=jac(35,34)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(7,7)=jac(7,7)-k(7)*y(34);
jac(7,34)=jac(7,34)-k(7)*y(7);
jac(34,7)=jac(34,7)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(7);
jac(31,7)=jac(31,7)+k(7)*y(34);
jac(31,34)=jac(31,34)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(34,34)=jac(34,34)-k(7)*y(335);
jac(34,335)=jac(34,335)-k(7)*y(34);
jac(335,34)=jac(335,34)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(34);
jac(439,34)=jac(439,34)+k(7)*y(335);
jac(439,335)=jac(439,335)+k(7)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(257,257)=jac(257,257)-k(7)*y(34);
jac(257,34)=jac(257,34)-k(7)*y(257);
jac(34,257)=jac(34,257)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(257);
jac(582,257)=jac(582,257)+k(7)*y(34);
jac(582,34)=jac(582,34)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(34,34)=jac(34,34)-k(7)*y(227);
jac(34,227)=jac(34,227)-k(7)*y(34);
jac(227,34)=jac(227,34)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(34);
jac(569,34)=jac(569,34)+k(7)*y(227);
jac(569,227)=jac(569,227)+k(7)*y(34);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(34,34)=jac(34,34)-k(7)*y(222);
jac(34,222)=jac(34,222)-k(7)*y(34);
jac(222,34)=jac(222,34)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(34);
jac(496,34)=jac(496,34)+k(7)*y(222);
jac(496,222)=jac(496,222)+k(7)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(34);
jac(218,34)=jac(218,34)-k(7)*y(218);
jac(34,218)=jac(34,218)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(218);
jac(495,218)=jac(495,218)+k(7)*y(34);
jac(495,34)=jac(495,34)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(34);
jac(192,34)=jac(192,34)-k(7)*y(192);
jac(34,192)=jac(34,192)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(192);
jac(620,192)=jac(620,192)+k(7)*y(34);
jac(620,34)=jac(620,34)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(34,34)=jac(34,34)-k(7)*y(184);
jac(34,184)=jac(34,184)-k(7)*y(34);
jac(184,34)=jac(184,34)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(34);
jac(423,34)=jac(423,34)+k(7)*y(184);
jac(423,184)=jac(423,184)+k(7)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(34);
jac(177,34)=jac(177,34)-k(7)*y(177);
jac(34,177)=jac(34,177)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(177);
jac(498,177)=jac(498,177)+k(7)*y(34);
jac(498,34)=jac(498,34)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(34,34)=jac(34,34)-k(7)*y(140);
jac(34,140)=jac(34,140)-k(7)*y(34);
jac(140,34)=jac(140,34)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(34);
jac(382,34)=jac(382,34)+k(7)*y(140);
jac(382,140)=jac(382,140)+k(7)*y(34);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(34,34)=jac(34,34)-k(7)*y(131);
jac(34,131)=jac(34,131)-k(7)*y(34);
jac(131,34)=jac(131,34)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(34);
jac(444,34)=jac(444,34)+k(7)*y(131);
jac(444,131)=jac(444,131)+k(7)*y(34);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(34);
jac(111,34)=jac(111,34)-k(7)*y(111);
jac(34,111)=jac(34,111)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(111);
jac(499,111)=jac(499,111)+k(7)*y(34);
jac(499,34)=jac(499,34)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(77,77)=jac(77,77)-k(7)*y(34);
jac(77,34)=jac(77,34)-k(7)*y(77);
jac(34,77)=jac(34,77)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(77);
jac(245,77)=jac(245,77)+k(7)*y(34);
jac(245,34)=jac(245,34)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(72,72)=jac(72,72)-k(7)*y(34);
jac(72,34)=jac(72,34)-k(7)*y(72);
jac(34,72)=jac(34,72)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(72);
jac(242,72)=jac(242,72)+k(7)*y(34);
jac(242,34)=jac(242,34)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(61,61)=jac(61,61)-k(7)*y(34);
jac(61,34)=jac(61,34)-k(7)*y(61);
jac(34,61)=jac(34,61)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(61);
jac(317,61)=jac(317,61)+k(7)*y(34);
jac(317,34)=jac(317,34)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(54,54)=jac(54,54)-k(7)*y(34);
jac(54,34)=jac(54,34)-k(7)*y(54);
jac(34,54)=jac(34,54)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(54);
jac(534,54)=jac(534,54)+k(7)*y(34);
jac(534,34)=jac(534,34)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(48,48)=jac(48,48)-k(7)*y(34);
jac(48,34)=jac(48,34)-k(7)*y(48);
jac(34,48)=jac(34,48)-k(7)*y(34);
jac(34,34)=jac(34,34)-k(7)*y(48);
jac(470,48)=jac(470,48)+k(7)*y(34);
jac(470,34)=jac(470,34)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(34,34)=jac(34,34)-k(7)*y(43);
jac(34,43)=jac(34,43)-k(7)*y(34);
jac(43,34)=jac(43,34)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(34);
jac(500,34)=jac(500,34)+k(7)*y(43);
jac(500,43)=jac(500,43)+k(7)*y(34);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(34,34)=jac(34,34)-k(7)*y(37);
jac(34,37)=jac(34,37)-k(7)*y(34);
jac(37,34)=jac(37,34)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(34);
jac(566,34)=jac(566,34)+k(7)*y(37);
jac(566,37)=jac(566,37)+k(7)*y(34);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) 

jac(34,34)=jac(34,34)-k(2);
jac(4,34)=jac(4,34)+k(2);
jac(33,34)=jac(33,34)+k(2);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(35,35)=jac(35,35)-k(33);
jac(621,35)=jac(621,35)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(35,35)=jac(35,35)-k(31);
jac(30,35)=jac(30,35)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~Y) 

jac(35,35)=jac(35,35)-k(22);
jac(32,35)=jac(32,35)+k(22);
jac(234,35)=jac(234,35)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(35);
jac(3,35)=jac(3,35)-k(15)*y(3);
jac(35,3)=jac(35,3)-k(15)*y(35);
jac(35,35)=jac(35,35)-k(15)*y(3);
jac(39,3)=jac(39,3)+k(15)*y(35);
jac(39,35)=jac(39,35)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(35);
jac(3,35)=jac(3,35)-k(9)*y(3);
jac(35,3)=jac(35,3)-k(9)*y(35);
jac(35,35)=jac(35,35)-k(9)*y(3);
jac(31,3)=jac(31,3)+k(9)*y(35);
jac(31,35)=jac(31,35)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

jac(35,35)=jac(35,35)-k(8);
jac(1,35)=jac(1,35)+k(8);
jac(34,35)=jac(34,35)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(35,35)=jac(35,35)-k(6);
jac(6,35)=jac(6,35)+k(6);
jac(33,35)=jac(33,35)+k(6);

% rule    : R16
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(37,37)=jac(37,37)-k(33);
jac(43,37)=jac(43,37)+k(33);

% rule    : R9_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~pY, a~Y) 

jac(37,37)=jac(37,37)-k(22);
jac(32,37)=jac(32,37)+k(22);
jac(54,37)=jac(54,37)+k(22);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lyn(U, SH2) 

jac(37,37)=jac(37,37)-k(16);
jac(3,37)=jac(3,37)+k(16);
jac(33,37)=jac(33,37)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(7)*y(38);
jac(37,38)=jac(37,38)-k(7)*y(37);
jac(38,37)=jac(38,37)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(37);
jac(567,37)=jac(567,37)+k(7)*y(38);
jac(567,38)=jac(567,38)+k(7)*y(37);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(7)*y(44);
jac(37,44)=jac(37,44)-k(7)*y(37);
jac(44,37)=jac(44,37)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(37);
jac(571,37)=jac(571,37)+k(7)*y(44);
jac(571,44)=jac(571,44)+k(7)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(55,55)=jac(55,55)-k(7)*y(37);
jac(55,37)=jac(55,37)-k(7)*y(55);
jac(37,55)=jac(37,55)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(55);
jac(633,55)=jac(633,55)+k(7)*y(37);
jac(633,37)=jac(633,37)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(62,62)=jac(62,62)-k(7)*y(37);
jac(62,37)=jac(62,37)-k(7)*y(62);
jac(37,62)=jac(37,62)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(62);
jac(83,62)=jac(83,62)+k(7)*y(37);
jac(83,37)=jac(83,37)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(91,91)=jac(91,91)-k(7)*y(37);
jac(91,37)=jac(91,37)-k(7)*y(91);
jac(37,91)=jac(37,91)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(91);
jac(311,91)=jac(311,91)+k(7)*y(37);
jac(311,37)=jac(311,37)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(126,126)=jac(126,126)-k(7)*y(37);
jac(126,37)=jac(126,37)-k(7)*y(126);
jac(37,126)=jac(37,126)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(126);
jac(352,126)=jac(352,126)+k(7)*y(37);
jac(352,37)=jac(352,37)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(7)*y(132);
jac(37,132)=jac(37,132)-k(7)*y(37);
jac(132,37)=jac(132,37)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(37);
jac(602,37)=jac(602,37)+k(7)*y(132);
jac(602,132)=jac(602,132)+k(7)*y(37);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(7)*y(141);
jac(37,141)=jac(37,141)-k(7)*y(37);
jac(141,37)=jac(141,37)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(37);
jac(603,37)=jac(603,37)+k(7)*y(141);
jac(603,141)=jac(603,141)+k(7)*y(37);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(7)*y(185);
jac(37,185)=jac(37,185)-k(7)*y(37);
jac(185,37)=jac(185,37)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(37);
jac(349,37)=jac(349,37)+k(7)*y(185);
jac(349,185)=jac(349,185)+k(7)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(209,209)=jac(209,209)-k(7)*y(37);
jac(209,37)=jac(209,37)-k(7)*y(209);
jac(37,209)=jac(37,209)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(209);
jac(557,209)=jac(557,209)+k(7)*y(37);
jac(557,37)=jac(557,37)+k(7)*y(209);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(214,214)=jac(214,214)-k(7)*y(37);
jac(214,37)=jac(214,37)-k(7)*y(214);
jac(37,214)=jac(37,214)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(214);
jac(553,214)=jac(553,214)+k(7)*y(37);
jac(553,37)=jac(553,37)+k(7)*y(214);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(219,219)=jac(219,219)-k(7)*y(37);
jac(219,37)=jac(219,37)-k(7)*y(219);
jac(37,219)=jac(37,219)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(219);
jac(555,219)=jac(555,219)+k(7)*y(37);
jac(555,37)=jac(555,37)+k(7)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(7)*y(223);
jac(37,223)=jac(37,223)-k(7)*y(37);
jac(223,37)=jac(223,37)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(37);
jac(639,37)=jac(639,37)+k(7)*y(223);
jac(639,223)=jac(639,223)+k(7)*y(37);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(7)*y(228);
jac(37,228)=jac(37,228)-k(7)*y(37);
jac(228,37)=jac(228,37)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(37);
jac(561,37)=jac(561,37)+k(7)*y(228);
jac(561,228)=jac(561,228)+k(7)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(315,315)=jac(315,315)-k(7)*y(37);
jac(315,37)=jac(315,37)-k(7)*y(315);
jac(37,315)=jac(37,315)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(315);
jac(563,315)=jac(563,315)+k(7)*y(37);
jac(563,37)=jac(563,37)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(319,319)=jac(319,319)-k(7)*y(37);
jac(319,37)=jac(319,37)-k(7)*y(319);
jac(37,319)=jac(37,319)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(319);
jac(560,319)=jac(560,319)+k(7)*y(37);
jac(560,37)=jac(560,37)+k(7)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(37,37)=jac(37,37)-k(7)*y(336);
jac(37,336)=jac(37,336)-k(7)*y(37);
jac(336,37)=jac(336,37)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(37);
jac(558,37)=jac(558,37)+k(7)*y(336);
jac(558,336)=jac(558,336)+k(7)*y(37);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(437,437)=jac(437,437)-k(7)*y(37);
jac(437,37)=jac(437,37)-k(7)*y(437);
jac(37,437)=jac(37,437)-k(7)*y(37);
jac(37,37)=jac(37,37)-k(7)*y(437);
jac(570,437)=jac(570,437)+k(7)*y(37);
jac(570,37)=jac(570,37)+k(7)*y(437);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(47,47)=jac(47,47)-k(5)*y(37);
jac(47,37)=jac(47,37)-k(5)*y(47);
jac(37,47)=jac(37,47)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(47);
jac(40,47)=jac(40,47)+k(5)*y(37);
jac(40,37)=jac(40,37)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(71,71)=jac(71,71)-k(5)*y(37);
jac(71,37)=jac(71,37)-k(5)*y(71);
jac(37,71)=jac(37,71)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(71);
jac(240,71)=jac(240,71)+k(5)*y(37);
jac(240,37)=jac(240,37)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(76,76)=jac(76,76)-k(5)*y(37);
jac(76,37)=jac(76,37)-k(5)*y(76);
jac(37,76)=jac(37,76)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(76);
jac(243,76)=jac(243,76)+k(5)*y(37);
jac(243,37)=jac(243,37)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(82,82)=jac(82,82)-k(5)*y(37);
jac(82,37)=jac(82,37)-k(5)*y(82);
jac(37,82)=jac(37,82)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(82);
jac(247,82)=jac(247,82)+k(5)*y(37);
jac(247,37)=jac(247,37)+k(5)*y(82);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(86,86)=jac(86,86)-k(5)*y(37);
jac(86,37)=jac(86,37)-k(5)*y(86);
jac(37,86)=jac(37,86)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(86);
jac(483,86)=jac(483,86)+k(5)*y(37);
jac(483,37)=jac(483,37)+k(5)*y(86);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(89,89)=jac(89,89)-k(5)*y(37);
jac(89,37)=jac(89,37)-k(5)*y(89);
jac(37,89)=jac(37,89)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(89);
jac(481,89)=jac(481,89)+k(5)*y(37);
jac(481,37)=jac(481,37)+k(5)*y(89);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(98,98)=jac(98,98)-k(5)*y(37);
jac(98,37)=jac(98,37)-k(5)*y(98);
jac(37,98)=jac(37,98)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(98);
jac(249,98)=jac(249,98)+k(5)*y(37);
jac(249,37)=jac(249,37)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(110,110)=jac(110,110)-k(5)*y(37);
jac(110,37)=jac(110,37)-k(5)*y(110);
jac(37,110)=jac(37,110)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(110);
jac(246,110)=jac(246,110)+k(5)*y(37);
jac(246,37)=jac(246,37)+k(5)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(5)*y(114);
jac(37,114)=jac(37,114)-k(5)*y(37);
jac(114,37)=jac(114,37)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(37);
jac(631,37)=jac(631,37)+k(5)*y(114);
jac(631,114)=jac(631,114)+k(5)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(118,118)=jac(118,118)-k(5)*y(37);
jac(118,37)=jac(118,37)-k(5)*y(118);
jac(37,118)=jac(37,118)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(118);
jac(244,118)=jac(244,118)+k(5)*y(37);
jac(244,37)=jac(244,37)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(122,122)=jac(122,122)-k(5)*y(37);
jac(122,37)=jac(122,37)-k(5)*y(122);
jac(37,122)=jac(37,122)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(122);
jac(241,122)=jac(241,122)+k(5)*y(37);
jac(241,37)=jac(241,37)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(176,176)=jac(176,176)-k(5)*y(37);
jac(176,37)=jac(176,37)-k(5)*y(176);
jac(37,176)=jac(37,176)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(176);
jac(479,176)=jac(479,176)+k(5)*y(37);
jac(479,37)=jac(479,37)+k(5)*y(176);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(191,191)=jac(191,191)-k(5)*y(37);
jac(191,37)=jac(191,37)-k(5)*y(191);
jac(37,191)=jac(37,191)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(191);
jac(293,191)=jac(293,191)+k(5)*y(37);
jac(293,37)=jac(293,37)+k(5)*y(191);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(256,256)=jac(256,256)-k(5)*y(37);
jac(256,37)=jac(256,37)-k(5)*y(256);
jac(37,256)=jac(37,256)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(256);
jac(248,256)=jac(248,256)+k(5)*y(37);
jac(248,37)=jac(248,37)+k(5)*y(256);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(264,264)=jac(264,264)-k(5)*y(37);
jac(264,37)=jac(264,37)-k(5)*y(264);
jac(37,264)=jac(37,264)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(264);
jac(493,264)=jac(493,264)+k(5)*y(37);
jac(493,37)=jac(493,37)+k(5)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(37,37)=jac(37,37)-k(5)*y(380);
jac(37,380)=jac(37,380)-k(5)*y(37);
jac(380,37)=jac(380,37)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(37);
jac(480,37)=jac(480,37)+k(5)*y(380);
jac(480,380)=jac(480,380)+k(5)*y(37);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(37,37)=jac(37,37)-k(5)*y(391);
jac(37,391)=jac(37,391)-k(5)*y(37);
jac(391,37)=jac(391,37)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(37);
jac(478,37)=jac(478,37)+k(5)*y(391);
jac(478,391)=jac(478,391)+k(5)*y(37);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(37,37)=jac(37,37)-k(5)*y(407);
jac(37,407)=jac(37,407)-k(5)*y(37);
jac(407,37)=jac(407,37)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(37);
jac(477,37)=jac(477,37)+k(5)*y(407);
jac(477,407)=jac(477,407)+k(5)*y(37);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(5)*y(441);
jac(37,441)=jac(37,441)-k(5)*y(37);
jac(441,37)=jac(441,37)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(37);
jac(469,37)=jac(469,37)+k(5)*y(441);
jac(469,441)=jac(469,441)+k(5)*y(37);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(5)*y(515);
jac(37,515)=jac(37,515)-k(5)*y(37);
jac(515,37)=jac(515,37)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(37);
jac(494,37)=jac(494,37)+k(5)*y(515);
jac(494,515)=jac(494,515)+k(5)*y(37);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(5)*y(543);
jac(37,543)=jac(37,543)-k(5)*y(37);
jac(543,37)=jac(543,37)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(37);
jac(643,37)=jac(643,37)+k(5)*y(543);
jac(643,543)=jac(643,543)+k(5)*y(37);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(37,37)=jac(37,37)-k(5)*y(554);
jac(37,554)=jac(37,554)-k(5)*y(37);
jac(554,37)=jac(554,37)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(37);
jac(567,37)=jac(567,37)+k(5)*y(554);
jac(567,554)=jac(567,554)+k(5)*y(37);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(9,9)=jac(9,9)-k(5)*y(37);
jac(9,37)=jac(9,37)-k(5)*y(9);
jac(37,9)=jac(37,9)-k(5)*y(37);
jac(37,37)=jac(37,37)-k(5)*y(9);
jac(36,9)=jac(36,9)+k(5)*y(37);
jac(36,37)=jac(36,37)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(4,4)=jac(4,4)-k(3)*y(37);
jac(4,37)=jac(4,37)-k(3)*y(4);
jac(37,4)=jac(37,4)-k(3)*y(37);
jac(37,37)=jac(37,37)-k(3)*y(4);
jac(554,4)=jac(554,4)+k(3)*y(37);
jac(554,37)=jac(554,37)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(4,4)=jac(4,4)-k(1)*y(37);
jac(4,37)=jac(4,37)-k(1)*y(4);
jac(37,4)=jac(37,4)-k(1)*y(37);
jac(37,37)=jac(37,37)-k(1)*y(4);
jac(38,4)=jac(38,4)+k(1)*y(37);
jac(38,37)=jac(38,37)+k(1)*y(4);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(38,38)=jac(38,38)-k(33);
jac(44,38)=jac(44,38)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

jac(38,38)=jac(38,38)-k(22);
jac(32,38)=jac(32,38)+k(22);
jac(55,38)=jac(55,38)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(38,38)=jac(38,38)-k(16);
jac(3,38)=jac(3,38)+k(16);
jac(34,38)=jac(34,38)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(1,1)=jac(1,1)-k(7)*y(38);
jac(1,38)=jac(1,38)-k(7)*y(1);
jac(38,1)=jac(38,1)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(1);
jac(39,1)=jac(39,1)+k(7)*y(38);
jac(39,38)=jac(39,38)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(7,7)=jac(7,7)-k(7)*y(38);
jac(7,38)=jac(7,38)-k(7)*y(7);
jac(38,7)=jac(38,7)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(7);
jac(36,7)=jac(36,7)+k(7)*y(38);
jac(36,38)=jac(36,38)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(7)*y(38);
jac(335,38)=jac(335,38)-k(7)*y(335);
jac(38,335)=jac(38,335)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(335);
jac(477,335)=jac(477,335)+k(7)*y(38);
jac(477,38)=jac(477,38)+k(7)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(38);
jac(257,38)=jac(257,38)-k(7)*y(257);
jac(38,257)=jac(38,257)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(257);
jac(248,257)=jac(248,257)+k(7)*y(38);
jac(248,38)=jac(248,38)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(227,227)=jac(227,227)-k(7)*y(38);
jac(227,38)=jac(227,38)-k(7)*y(227);
jac(38,227)=jac(38,227)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(227);
jac(643,227)=jac(643,227)+k(7)*y(38);
jac(643,38)=jac(643,38)+k(7)*y(227);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(222,222)=jac(222,222)-k(7)*y(38);
jac(222,38)=jac(222,38)-k(7)*y(222);
jac(38,222)=jac(38,222)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(222);
jac(494,222)=jac(494,222)+k(7)*y(38);
jac(494,38)=jac(494,38)+k(7)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(38);
jac(218,38)=jac(218,38)-k(7)*y(218);
jac(38,218)=jac(38,218)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(218);
jac(493,218)=jac(493,218)+k(7)*y(38);
jac(493,38)=jac(493,38)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(192,192)=jac(192,192)-k(7)*y(38);
jac(192,38)=jac(192,38)-k(7)*y(192);
jac(38,192)=jac(38,192)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(192);
jac(293,192)=jac(293,192)+k(7)*y(38);
jac(293,38)=jac(293,38)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(7)*y(38);
jac(184,38)=jac(184,38)-k(7)*y(184);
jac(38,184)=jac(38,184)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(184);
jac(478,184)=jac(478,184)+k(7)*y(38);
jac(478,38)=jac(478,38)+k(7)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(38);
jac(177,38)=jac(177,38)-k(7)*y(177);
jac(38,177)=jac(38,177)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(177);
jac(479,177)=jac(479,177)+k(7)*y(38);
jac(479,38)=jac(479,38)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(7)*y(38);
jac(140,38)=jac(140,38)-k(7)*y(140);
jac(38,140)=jac(38,140)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(140);
jac(480,140)=jac(480,140)+k(7)*y(38);
jac(480,38)=jac(480,38)+k(7)*y(140);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(131,131)=jac(131,131)-k(7)*y(38);
jac(131,38)=jac(131,38)-k(7)*y(131);
jac(38,131)=jac(38,131)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(131);
jac(469,131)=jac(469,131)+k(7)*y(38);
jac(469,38)=jac(469,38)+k(7)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(38);
jac(111,38)=jac(111,38)-k(7)*y(111);
jac(38,111)=jac(38,111)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(111);
jac(246,111)=jac(246,111)+k(7)*y(38);
jac(246,38)=jac(246,38)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(77,77)=jac(77,77)-k(7)*y(38);
jac(77,38)=jac(77,38)-k(7)*y(77);
jac(38,77)=jac(38,77)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(77);
jac(243,77)=jac(243,77)+k(7)*y(38);
jac(243,38)=jac(243,38)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(72,72)=jac(72,72)-k(7)*y(38);
jac(72,38)=jac(72,38)-k(7)*y(72);
jac(38,72)=jac(38,72)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(72);
jac(240,72)=jac(240,72)+k(7)*y(38);
jac(240,38)=jac(240,38)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(61,61)=jac(61,61)-k(7)*y(38);
jac(61,38)=jac(61,38)-k(7)*y(61);
jac(38,61)=jac(38,61)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(61);
jac(241,61)=jac(241,61)+k(7)*y(38);
jac(241,38)=jac(241,38)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(54,54)=jac(54,54)-k(7)*y(38);
jac(54,38)=jac(54,38)-k(7)*y(54);
jac(38,54)=jac(38,54)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(54);
jac(244,54)=jac(244,54)+k(7)*y(38);
jac(244,38)=jac(244,38)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(38);
jac(48,38)=jac(48,38)-k(7)*y(48);
jac(38,48)=jac(38,48)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(48);
jac(40,48)=jac(40,48)+k(7)*y(38);
jac(40,38)=jac(40,38)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(43,43)=jac(43,43)-k(7)*y(38);
jac(43,38)=jac(43,38)-k(7)*y(43);
jac(38,43)=jac(38,43)-k(7)*y(38);
jac(38,38)=jac(38,38)-k(7)*y(43);
jac(631,43)=jac(631,43)+k(7)*y(38);
jac(631,38)=jac(631,38)+k(7)*y(43);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) 

jac(38,38)=jac(38,38)-k(2);
jac(4,38)=jac(4,38)+k(2);
jac(37,38)=jac(37,38)+k(2);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(39,39)=jac(39,39)-k(33);
jac(45,39)=jac(45,39)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

jac(39,39)=jac(39,39)-k(22);
jac(32,39)=jac(32,39)+k(22);
jac(233,39)=jac(233,39)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(39,39)=jac(39,39)-k(20);
jac(249,39)=jac(249,39)+k(20);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(39,39)=jac(39,39)-k(18);
jac(240,39)=jac(240,39)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(39,39)=jac(39,39)-k(16);
jac(3,39)=jac(3,39)+k(16);
jac(35,39)=jac(35,39)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(39);
jac(3,39)=jac(3,39)-k(9)*y(3);
jac(39,3)=jac(39,3)-k(9)*y(39);
jac(39,39)=jac(39,39)-k(9)*y(3);
jac(36,3)=jac(36,3)+k(9)*y(39);
jac(36,39)=jac(36,39)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

jac(39,39)=jac(39,39)-k(8);
jac(1,39)=jac(1,39)+k(8);
jac(38,39)=jac(38,39)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(39,39)=jac(39,39)-k(6);
jac(6,39)=jac(6,39)+k(6);
jac(37,39)=jac(37,39)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(40,40)=jac(40,40)-k(33);
jac(46,40)=jac(46,40)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(40,40)=jac(40,40)-k(32);
jac(36,40)=jac(36,40)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(40,40)=jac(40,40)-k(22);
jac(32,40)=jac(32,40)+k(22);
jac(53,40)=jac(53,40)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(40);
jac(182,40)=jac(182,40)-k(21)*y(182);
jac(40,182)=jac(40,182)-k(21)*y(40);
jac(40,40)=jac(40,40)-k(21)*y(182);
jac(478,182)=jac(478,182)+k(21)*y(40);
jac(478,40)=jac(478,40)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(40);
jac(197,40)=jac(197,40)-k(21)*y(197);
jac(40,197)=jac(40,197)-k(21)*y(40);
jac(40,40)=jac(40,40)-k(21)*y(197);
jac(477,197)=jac(477,197)+k(21)*y(40);
jac(477,40)=jac(477,40)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(40);
jac(32,40)=jac(32,40)-k(21)*y(32);
jac(40,32)=jac(40,32)-k(21)*y(40);
jac(40,40)=jac(40,40)-k(21)*y(32);
jac(469,32)=jac(469,32)+k(21)*y(40);
jac(469,40)=jac(469,40)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(40);
jac(2,40)=jac(2,40)-k(21)*y(2);
jac(40,2)=jac(40,2)-k(21)*y(40);
jac(40,40)=jac(40,40)-k(21)*y(2);
jac(480,2)=jac(480,2)+k(21)*y(40);
jac(480,40)=jac(480,40)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(40,40)=jac(40,40)-k(16);
jac(3,40)=jac(3,40)+k(16);
jac(470,40)=jac(470,40)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(40,40)=jac(40,40)-k(10);
jac(3,40)=jac(3,40)+k(10);
jac(249,40)=jac(249,40)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(40,40)=jac(40,40)-k(8);
jac(48,40)=jac(48,40)+k(8);
jac(38,40)=jac(38,40)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(40,40)=jac(40,40)-k(6);
jac(47,40)=jac(47,40)+k(6);
jac(37,40)=jac(37,40)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(41,41)=jac(41,41)-k(32);
jac(67,41)=jac(67,41)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(182,182)=jac(182,182)-k(21)*y(41);
jac(182,41)=jac(182,41)-k(21)*y(182);
jac(41,182)=jac(41,182)-k(21)*y(41);
jac(41,41)=jac(41,41)-k(21)*y(182);
jac(221,182)=jac(221,182)+k(21)*y(41);
jac(221,41)=jac(221,41)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(41);
jac(197,41)=jac(197,41)-k(21)*y(197);
jac(41,197)=jac(41,197)-k(21)*y(41);
jac(41,41)=jac(41,41)-k(21)*y(197);
jac(226,197)=jac(226,197)+k(21)*y(41);
jac(226,41)=jac(226,41)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(32,32)=jac(32,32)-k(21)*y(41);
jac(32,41)=jac(32,41)-k(21)*y(32);
jac(41,32)=jac(41,32)-k(21)*y(41);
jac(41,41)=jac(41,41)-k(21)*y(32);
jac(36,32)=jac(36,32)+k(21)*y(41);
jac(36,41)=jac(36,41)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(41);
jac(2,41)=jac(2,41)-k(21)*y(2);
jac(41,2)=jac(41,2)-k(21)*y(41);
jac(41,41)=jac(41,41)-k(21)*y(2);
jac(42,2)=jac(42,2)+k(21)*y(41);
jac(42,41)=jac(42,41)+k(21)*y(2);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(41,41)=jac(41,41)-k(20);
jac(53,41)=jac(53,41)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) 

jac(41,41)=jac(41,41)-k(16);
jac(3,41)=jac(3,41)+k(16);
jac(19,41)=jac(19,41)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) 

jac(41,41)=jac(41,41)-k(10);
jac(3,41)=jac(3,41)+k(10);
jac(233,41)=jac(233,41)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(41,41)=jac(41,41)-k(8);
jac(7,41)=jac(7,41)+k(8);
jac(55,41)=jac(55,41)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(41,41)=jac(41,41)-k(6);
jac(54,41)=jac(54,41)+k(6);
jac(9,41)=jac(9,41)+k(6);

% rule    : R9_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~Y, a~Y) 

jac(43,43)=jac(43,43)-k(22);
jac(2,43)=jac(2,43)+k(22);
jac(54,43)=jac(54,43)+k(22);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lyn(U, SH2) 

jac(43,43)=jac(43,43)-k(16);
jac(3,43)=jac(3,43)+k(16);
jac(111,43)=jac(111,43)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(43,43)=jac(43,43)-k(7)*y(44);
jac(43,44)=jac(43,44)-k(7)*y(43);
jac(44,43)=jac(44,43)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(43);
jac(630,43)=jac(630,43)+k(7)*y(44);
jac(630,44)=jac(630,44)+k(7)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(55,55)=jac(55,55)-k(7)*y(43);
jac(55,43)=jac(55,43)-k(7)*y(55);
jac(43,55)=jac(43,55)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(55);
jac(628,55)=jac(628,55)+k(7)*y(43);
jac(628,43)=jac(628,43)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(62,62)=jac(62,62)-k(7)*y(43);
jac(62,43)=jac(62,43)-k(7)*y(62);
jac(43,62)=jac(43,62)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(62);
jac(307,62)=jac(307,62)+k(7)*y(43);
jac(307,43)=jac(307,43)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(91,91)=jac(91,91)-k(7)*y(43);
jac(91,43)=jac(91,43)-k(7)*y(91);
jac(43,91)=jac(43,91)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(91);
jac(306,91)=jac(306,91)+k(7)*y(43);
jac(306,43)=jac(306,43)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(126,126)=jac(126,126)-k(7)*y(43);
jac(126,43)=jac(126,43)-k(7)*y(126);
jac(43,126)=jac(43,126)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(126);
jac(116,126)=jac(116,126)+k(7)*y(43);
jac(116,43)=jac(116,43)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(43,43)=jac(43,43)-k(7)*y(132);
jac(43,132)=jac(43,132)-k(7)*y(43);
jac(132,43)=jac(132,43)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(43);
jac(357,43)=jac(357,43)+k(7)*y(132);
jac(357,132)=jac(357,132)+k(7)*y(43);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(43,43)=jac(43,43)-k(7)*y(141);
jac(43,141)=jac(43,141)-k(7)*y(43);
jac(141,43)=jac(141,43)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(43);
jac(604,43)=jac(604,43)+k(7)*y(141);
jac(604,141)=jac(604,141)+k(7)*y(43);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(43,43)=jac(43,43)-k(7)*y(185);
jac(43,185)=jac(43,185)-k(7)*y(43);
jac(185,43)=jac(185,43)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(43);
jac(522,43)=jac(522,43)+k(7)*y(185);
jac(522,185)=jac(522,185)+k(7)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(209,209)=jac(209,209)-k(7)*y(43);
jac(209,43)=jac(209,43)-k(7)*y(209);
jac(43,209)=jac(43,209)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(209);
jac(525,209)=jac(525,209)+k(7)*y(43);
jac(525,43)=jac(525,43)+k(7)*y(209);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(214,214)=jac(214,214)-k(7)*y(43);
jac(214,43)=jac(214,43)-k(7)*y(214);
jac(43,214)=jac(43,214)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(214);
jac(521,214)=jac(521,214)+k(7)*y(43);
jac(521,43)=jac(521,43)+k(7)*y(214);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(219,219)=jac(219,219)-k(7)*y(43);
jac(219,43)=jac(219,43)-k(7)*y(219);
jac(43,219)=jac(43,219)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(219);
jac(523,219)=jac(523,219)+k(7)*y(43);
jac(523,43)=jac(523,43)+k(7)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(43,43)=jac(43,43)-k(7)*y(223);
jac(43,223)=jac(43,223)-k(7)*y(43);
jac(223,43)=jac(223,43)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(43);
jac(637,43)=jac(637,43)+k(7)*y(223);
jac(637,223)=jac(637,223)+k(7)*y(43);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(43,43)=jac(43,43)-k(7)*y(228);
jac(43,228)=jac(43,228)-k(7)*y(43);
jac(228,43)=jac(228,43)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(43);
jac(632,43)=jac(632,43)+k(7)*y(228);
jac(632,228)=jac(632,228)+k(7)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(315,315)=jac(315,315)-k(7)*y(43);
jac(315,43)=jac(315,43)-k(7)*y(315);
jac(43,315)=jac(43,315)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(315);
jac(308,315)=jac(308,315)+k(7)*y(43);
jac(308,43)=jac(308,43)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(319,319)=jac(319,319)-k(7)*y(43);
jac(319,43)=jac(319,43)-k(7)*y(319);
jac(43,319)=jac(43,319)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(319);
jac(528,319)=jac(528,319)+k(7)*y(43);
jac(528,43)=jac(528,43)+k(7)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(43,43)=jac(43,43)-k(7)*y(336);
jac(43,336)=jac(43,336)-k(7)*y(43);
jac(336,43)=jac(336,43)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(43);
jac(526,43)=jac(526,43)+k(7)*y(336);
jac(526,336)=jac(526,336)+k(7)*y(43);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(437,437)=jac(437,437)-k(7)*y(43);
jac(437,43)=jac(437,43)-k(7)*y(437);
jac(43,437)=jac(43,437)-k(7)*y(43);
jac(43,43)=jac(43,43)-k(7)*y(437);
jac(504,437)=jac(504,437)+k(7)*y(43);
jac(504,43)=jac(504,43)+k(7)*y(437);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(47,47)=jac(47,47)-k(5)*y(43);
jac(47,43)=jac(47,43)-k(5)*y(47);
jac(43,47)=jac(43,47)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(47);
jac(46,47)=jac(46,47)+k(5)*y(43);
jac(46,43)=jac(46,43)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(71,71)=jac(71,71)-k(5)*y(43);
jac(71,43)=jac(71,43)-k(5)*y(71);
jac(43,71)=jac(43,71)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(71);
jac(239,71)=jac(239,71)+k(5)*y(43);
jac(239,43)=jac(239,43)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(76,76)=jac(76,76)-k(5)*y(43);
jac(76,43)=jac(76,43)-k(5)*y(76);
jac(43,76)=jac(43,76)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(76);
jac(250,76)=jac(250,76)+k(5)*y(43);
jac(250,43)=jac(250,43)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(82,82)=jac(82,82)-k(5)*y(43);
jac(82,43)=jac(82,43)-k(5)*y(82);
jac(43,82)=jac(43,82)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(82);
jac(254,82)=jac(254,82)+k(5)*y(43);
jac(254,43)=jac(254,43)+k(5)*y(82);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(86,86)=jac(86,86)-k(5)*y(43);
jac(86,43)=jac(86,43)-k(5)*y(86);
jac(43,86)=jac(43,86)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(86);
jac(277,86)=jac(277,86)+k(5)*y(43);
jac(277,43)=jac(277,43)+k(5)*y(86);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(89,89)=jac(89,89)-k(5)*y(43);
jac(89,43)=jac(89,43)-k(5)*y(89);
jac(43,89)=jac(43,89)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(89);
jac(274,89)=jac(274,89)+k(5)*y(43);
jac(274,43)=jac(274,43)+k(5)*y(89);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(98,98)=jac(98,98)-k(5)*y(43);
jac(98,43)=jac(98,43)-k(5)*y(98);
jac(43,98)=jac(43,98)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(98);
jac(49,98)=jac(49,98)+k(5)*y(43);
jac(49,43)=jac(49,43)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(110,110)=jac(110,110)-k(5)*y(43);
jac(110,43)=jac(110,43)-k(5)*y(110);
jac(43,110)=jac(43,110)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(110);
jac(253,110)=jac(253,110)+k(5)*y(43);
jac(253,43)=jac(253,43)+k(5)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(43,43)=jac(43,43)-k(5)*y(114);
jac(43,114)=jac(43,114)-k(5)*y(43);
jac(114,43)=jac(114,43)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(43);
jac(630,43)=jac(630,43)+k(5)*y(114);
jac(630,114)=jac(630,114)+k(5)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(118,118)=jac(118,118)-k(5)*y(43);
jac(118,43)=jac(118,43)-k(5)*y(118);
jac(43,118)=jac(43,118)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(118);
jac(251,118)=jac(251,118)+k(5)*y(43);
jac(251,43)=jac(251,43)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(122,122)=jac(122,122)-k(5)*y(43);
jac(122,43)=jac(122,43)-k(5)*y(122);
jac(43,122)=jac(43,122)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(122);
jac(654,122)=jac(654,122)+k(5)*y(43);
jac(654,43)=jac(654,43)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(176,176)=jac(176,176)-k(5)*y(43);
jac(176,43)=jac(176,43)-k(5)*y(176);
jac(43,176)=jac(43,176)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(176);
jac(269,176)=jac(269,176)+k(5)*y(43);
jac(269,43)=jac(269,43)+k(5)*y(176);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(191,191)=jac(191,191)-k(5)*y(43);
jac(191,43)=jac(191,43)-k(5)*y(191);
jac(43,191)=jac(43,191)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(191);
jac(262,191)=jac(262,191)+k(5)*y(43);
jac(262,43)=jac(262,43)+k(5)*y(191);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(256,256)=jac(256,256)-k(5)*y(43);
jac(256,43)=jac(256,43)-k(5)*y(256);
jac(43,256)=jac(43,256)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(256);
jac(255,256)=jac(255,256)+k(5)*y(43);
jac(255,43)=jac(255,43)+k(5)*y(256);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(264,264)=jac(264,264)-k(5)*y(43);
jac(264,43)=jac(264,43)-k(5)*y(264);
jac(43,264)=jac(43,264)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(264);
jac(263,264)=jac(263,264)+k(5)*y(43);
jac(263,43)=jac(263,43)+k(5)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(43,43)=jac(43,43)-k(5)*y(380);
jac(43,380)=jac(43,380)-k(5)*y(43);
jac(380,43)=jac(380,43)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(43);
jac(51,43)=jac(51,43)+k(5)*y(380);
jac(51,380)=jac(51,380)+k(5)*y(43);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(43,43)=jac(43,43)-k(5)*y(391);
jac(43,391)=jac(43,391)-k(5)*y(43);
jac(391,43)=jac(391,43)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(43);
jac(270,43)=jac(270,43)+k(5)*y(391);
jac(270,391)=jac(270,391)+k(5)*y(43);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(43,43)=jac(43,43)-k(5)*y(407);
jac(43,407)=jac(43,407)-k(5)*y(43);
jac(407,43)=jac(407,43)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(43);
jac(207,43)=jac(207,43)+k(5)*y(407);
jac(207,407)=jac(207,407)+k(5)*y(43);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(43,43)=jac(43,43)-k(5)*y(441);
jac(43,441)=jac(43,441)-k(5)*y(43);
jac(441,43)=jac(441,43)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(43);
jac(52,43)=jac(52,43)+k(5)*y(441);
jac(52,441)=jac(52,441)+k(5)*y(43);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(43,43)=jac(43,43)-k(5)*y(515);
jac(43,515)=jac(43,515)-k(5)*y(43);
jac(515,43)=jac(515,43)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(43);
jac(265,43)=jac(265,43)+k(5)*y(515);
jac(265,515)=jac(265,515)+k(5)*y(43);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(43,43)=jac(43,43)-k(5)*y(543);
jac(43,543)=jac(43,543)-k(5)*y(43);
jac(543,43)=jac(543,43)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(43);
jac(258,43)=jac(258,43)+k(5)*y(543);
jac(258,543)=jac(258,543)+k(5)*y(43);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(43,43)=jac(43,43)-k(5)*y(554);
jac(43,554)=jac(43,554)-k(5)*y(43);
jac(554,43)=jac(554,43)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(43);
jac(571,43)=jac(571,43)+k(5)*y(554);
jac(571,554)=jac(571,554)+k(5)*y(43);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(9,9)=jac(9,9)-k(5)*y(43);
jac(9,43)=jac(9,43)-k(5)*y(9);
jac(43,9)=jac(43,9)-k(5)*y(43);
jac(43,43)=jac(43,43)-k(5)*y(9);
jac(42,9)=jac(42,9)+k(5)*y(43);
jac(42,43)=jac(42,43)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(4,4)=jac(4,4)-k(3)*y(43);
jac(4,43)=jac(4,43)-k(3)*y(4);
jac(43,4)=jac(43,4)-k(3)*y(43);
jac(43,43)=jac(43,43)-k(3)*y(4);
jac(114,4)=jac(114,4)+k(3)*y(43);
jac(114,43)=jac(114,43)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(4,4)=jac(4,4)-k(1)*y(43);
jac(4,43)=jac(4,43)-k(1)*y(4);
jac(43,4)=jac(43,4)-k(1)*y(43);
jac(43,43)=jac(43,43)-k(1)*y(4);
jac(44,4)=jac(44,4)+k(1)*y(43);
jac(44,43)=jac(44,43)+k(1)*y(4);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

jac(44,44)=jac(44,44)-k(22);
jac(2,44)=jac(2,44)+k(22);
jac(55,44)=jac(55,44)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(44,44)=jac(44,44)-k(16);
jac(3,44)=jac(3,44)+k(16);
jac(437,44)=jac(437,44)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(1,1)=jac(1,1)-k(7)*y(44);
jac(1,44)=jac(1,44)-k(7)*y(1);
jac(44,1)=jac(44,1)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(1);
jac(45,1)=jac(45,1)+k(7)*y(44);
jac(45,44)=jac(45,44)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(7,7)=jac(7,7)-k(7)*y(44);
jac(7,44)=jac(7,44)-k(7)*y(7);
jac(44,7)=jac(44,7)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(7);
jac(42,7)=jac(42,7)+k(7)*y(44);
jac(42,44)=jac(42,44)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(7)*y(44);
jac(335,44)=jac(335,44)-k(7)*y(335);
jac(44,335)=jac(44,335)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(335);
jac(207,335)=jac(207,335)+k(7)*y(44);
jac(207,44)=jac(207,44)+k(7)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(44);
jac(257,44)=jac(257,44)-k(7)*y(257);
jac(44,257)=jac(44,257)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(257);
jac(255,257)=jac(255,257)+k(7)*y(44);
jac(255,44)=jac(255,44)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(227,227)=jac(227,227)-k(7)*y(44);
jac(227,44)=jac(227,44)-k(7)*y(227);
jac(44,227)=jac(44,227)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(227);
jac(258,227)=jac(258,227)+k(7)*y(44);
jac(258,44)=jac(258,44)+k(7)*y(227);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(222,222)=jac(222,222)-k(7)*y(44);
jac(222,44)=jac(222,44)-k(7)*y(222);
jac(44,222)=jac(44,222)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(222);
jac(265,222)=jac(265,222)+k(7)*y(44);
jac(265,44)=jac(265,44)+k(7)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(44);
jac(218,44)=jac(218,44)-k(7)*y(218);
jac(44,218)=jac(44,218)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(218);
jac(263,218)=jac(263,218)+k(7)*y(44);
jac(263,44)=jac(263,44)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(44);
jac(192,44)=jac(192,44)-k(7)*y(192);
jac(44,192)=jac(44,192)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(192);
jac(262,192)=jac(262,192)+k(7)*y(44);
jac(262,44)=jac(262,44)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(7)*y(44);
jac(184,44)=jac(184,44)-k(7)*y(184);
jac(44,184)=jac(44,184)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(184);
jac(270,184)=jac(270,184)+k(7)*y(44);
jac(270,44)=jac(270,44)+k(7)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(44);
jac(177,44)=jac(177,44)-k(7)*y(177);
jac(44,177)=jac(44,177)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(177);
jac(269,177)=jac(269,177)+k(7)*y(44);
jac(269,44)=jac(269,44)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(7)*y(44);
jac(140,44)=jac(140,44)-k(7)*y(140);
jac(44,140)=jac(44,140)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(140);
jac(51,140)=jac(51,140)+k(7)*y(44);
jac(51,44)=jac(51,44)+k(7)*y(140);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(7)*y(44);
jac(131,44)=jac(131,44)-k(7)*y(131);
jac(44,131)=jac(44,131)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(131);
jac(52,131)=jac(52,131)+k(7)*y(44);
jac(52,44)=jac(52,44)+k(7)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(44);
jac(111,44)=jac(111,44)-k(7)*y(111);
jac(44,111)=jac(44,111)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(111);
jac(253,111)=jac(253,111)+k(7)*y(44);
jac(253,44)=jac(253,44)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(77,77)=jac(77,77)-k(7)*y(44);
jac(77,44)=jac(77,44)-k(7)*y(77);
jac(44,77)=jac(44,77)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(77);
jac(250,77)=jac(250,77)+k(7)*y(44);
jac(250,44)=jac(250,44)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(72,72)=jac(72,72)-k(7)*y(44);
jac(72,44)=jac(72,44)-k(7)*y(72);
jac(44,72)=jac(44,72)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(72);
jac(239,72)=jac(239,72)+k(7)*y(44);
jac(239,44)=jac(239,44)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(61,61)=jac(61,61)-k(7)*y(44);
jac(61,44)=jac(61,44)-k(7)*y(61);
jac(44,61)=jac(44,61)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(61);
jac(654,61)=jac(654,61)+k(7)*y(44);
jac(654,44)=jac(654,44)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(54,54)=jac(54,54)-k(7)*y(44);
jac(54,44)=jac(54,44)-k(7)*y(54);
jac(44,54)=jac(44,54)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(54);
jac(251,54)=jac(251,54)+k(7)*y(44);
jac(251,44)=jac(251,44)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(44);
jac(48,44)=jac(48,44)-k(7)*y(48);
jac(44,48)=jac(44,48)-k(7)*y(44);
jac(44,44)=jac(44,44)-k(7)*y(48);
jac(46,48)=jac(46,48)+k(7)*y(44);
jac(46,44)=jac(46,44)+k(7)*y(48);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) 

jac(44,44)=jac(44,44)-k(2);
jac(4,44)=jac(4,44)+k(2);
jac(43,44)=jac(43,44)+k(2);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

jac(45,45)=jac(45,45)-k(22);
jac(2,45)=jac(2,45)+k(22);
jac(233,45)=jac(233,45)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(45,45)=jac(45,45)-k(20);
jac(49,45)=jac(49,45)+k(20);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(45,45)=jac(45,45)-k(18);
jac(239,45)=jac(239,45)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(45,45)=jac(45,45)-k(16);
jac(3,45)=jac(3,45)+k(16);
jac(621,45)=jac(621,45)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(45);
jac(3,45)=jac(3,45)-k(9)*y(3);
jac(45,3)=jac(45,3)-k(9)*y(45);
jac(45,45)=jac(45,45)-k(9)*y(3);
jac(42,3)=jac(42,3)+k(9)*y(45);
jac(42,45)=jac(42,45)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

jac(45,45)=jac(45,45)-k(8);
jac(1,45)=jac(1,45)+k(8);
jac(44,45)=jac(44,45)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(45,45)=jac(45,45)-k(6);
jac(6,45)=jac(6,45)+k(6);
jac(43,45)=jac(43,45)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(47,47)=jac(47,47)-k(32);
jac(9,47)=jac(9,47)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(47);
jac(182,47)=jac(182,47)-k(21)*y(182);
jac(47,182)=jac(47,182)-k(21)*y(47);
jac(47,47)=jac(47,47)-k(21)*y(182);
jac(391,182)=jac(391,182)+k(21)*y(47);
jac(391,47)=jac(391,47)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(47);
jac(197,47)=jac(197,47)-k(21)*y(197);
jac(47,197)=jac(47,197)-k(21)*y(47);
jac(47,47)=jac(47,47)-k(21)*y(197);
jac(407,197)=jac(407,197)+k(21)*y(47);
jac(407,47)=jac(407,47)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(47);
jac(32,47)=jac(32,47)-k(21)*y(32);
jac(47,32)=jac(47,32)-k(21)*y(47);
jac(47,47)=jac(47,47)-k(21)*y(32);
jac(441,32)=jac(441,32)+k(21)*y(47);
jac(441,47)=jac(441,47)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(47);
jac(2,47)=jac(2,47)-k(21)*y(2);
jac(47,2)=jac(47,2)-k(21)*y(47);
jac(47,47)=jac(47,47)-k(21)*y(2);
jac(380,2)=jac(380,2)+k(21)*y(47);
jac(380,47)=jac(380,47)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Lyn(U, SH2) 

jac(47,47)=jac(47,47)-k(10);
jac(3,47)=jac(3,47)+k(10);
jac(98,47)=jac(98,47)+k(10);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(1,1)=jac(1,1)-k(5)*y(47);
jac(1,47)=jac(1,47)-k(5)*y(1);
jac(47,1)=jac(47,1)-k(5)*y(47);
jac(47,47)=jac(47,47)-k(5)*y(1);
jac(99,1)=jac(99,1)+k(5)*y(47);
jac(99,47)=jac(99,47)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(5)*y(47);
jac(7,47)=jac(7,47)-k(5)*y(7);
jac(47,7)=jac(47,7)-k(5)*y(47);
jac(47,47)=jac(47,47)-k(5)*y(7);
jac(463,7)=jac(463,7)+k(5)*y(47);
jac(463,47)=jac(463,47)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(47,47)=jac(47,47)-k(5)*y(335);
jac(47,335)=jac(47,335)-k(5)*y(47);
jac(335,47)=jac(335,47)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(47);
jac(400,47)=jac(400,47)+k(5)*y(335);
jac(400,335)=jac(400,335)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(47,47)=jac(47,47)-k(5)*y(257);
jac(47,257)=jac(47,257)-k(5)*y(47);
jac(257,47)=jac(257,47)-k(5)*y(257);
jac(257,257)=jac(257,257)-k(5)*y(47);
jac(401,47)=jac(401,47)+k(5)*y(257);
jac(401,257)=jac(401,257)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(47,47)=jac(47,47)-k(5)*y(227);
jac(47,227)=jac(47,227)-k(5)*y(47);
jac(227,47)=jac(227,47)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(47);
jac(205,47)=jac(205,47)+k(5)*y(227);
jac(205,227)=jac(205,227)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(47,47)=jac(47,47)-k(5)*y(222);
jac(47,222)=jac(47,222)-k(5)*y(47);
jac(222,47)=jac(222,47)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(47);
jac(225,47)=jac(225,47)+k(5)*y(222);
jac(225,222)=jac(225,222)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(47,47)=jac(47,47)-k(5)*y(218);
jac(47,218)=jac(47,218)-k(5)*y(47);
jac(218,47)=jac(218,47)-k(5)*y(218);
jac(218,218)=jac(218,218)-k(5)*y(47);
jac(430,47)=jac(430,47)+k(5)*y(218);
jac(430,218)=jac(430,218)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(47,47)=jac(47,47)-k(5)*y(192);
jac(47,192)=jac(47,192)-k(5)*y(47);
jac(192,47)=jac(192,47)-k(5)*y(192);
jac(192,192)=jac(192,192)-k(5)*y(47);
jac(399,47)=jac(399,47)+k(5)*y(192);
jac(399,192)=jac(399,192)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(47,47)=jac(47,47)-k(5)*y(184);
jac(47,184)=jac(47,184)-k(5)*y(47);
jac(184,47)=jac(184,47)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(47);
jac(457,47)=jac(457,47)+k(5)*y(184);
jac(457,184)=jac(457,184)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(47,47)=jac(47,47)-k(5)*y(177);
jac(47,177)=jac(47,177)-k(5)*y(47);
jac(177,47)=jac(177,47)-k(5)*y(177);
jac(177,177)=jac(177,177)-k(5)*y(47);
jac(396,47)=jac(396,47)+k(5)*y(177);
jac(396,177)=jac(396,177)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(47,47)=jac(47,47)-k(5)*y(140);
jac(47,140)=jac(47,140)-k(5)*y(47);
jac(140,47)=jac(140,47)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(47);
jac(467,47)=jac(467,47)+k(5)*y(140);
jac(467,140)=jac(467,140)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(47,47)=jac(47,47)-k(5)*y(131);
jac(47,131)=jac(47,131)-k(5)*y(47);
jac(131,47)=jac(131,47)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(47);
jac(155,47)=jac(155,47)+k(5)*y(131);
jac(155,131)=jac(155,131)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(47,47)=jac(47,47)-k(5)*y(111);
jac(47,111)=jac(47,111)-k(5)*y(47);
jac(111,47)=jac(111,47)-k(5)*y(111);
jac(111,111)=jac(111,111)-k(5)*y(47);
jac(50,47)=jac(50,47)+k(5)*y(111);
jac(50,111)=jac(50,111)+k(5)*y(47);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(77,77)=jac(77,77)-k(5)*y(47);
jac(77,47)=jac(77,47)-k(5)*y(77);
jac(47,77)=jac(47,77)-k(5)*y(47);
jac(47,47)=jac(47,47)-k(5)*y(77);
jac(57,77)=jac(57,77)+k(5)*y(47);
jac(57,47)=jac(57,47)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(72,72)=jac(72,72)-k(5)*y(47);
jac(72,47)=jac(72,47)-k(5)*y(72);
jac(47,72)=jac(47,72)-k(5)*y(47);
jac(47,47)=jac(47,47)-k(5)*y(72);
jac(64,72)=jac(64,72)+k(5)*y(47);
jac(64,47)=jac(64,47)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(61,61)=jac(61,61)-k(5)*y(47);
jac(61,47)=jac(61,47)-k(5)*y(61);
jac(47,61)=jac(47,61)-k(5)*y(47);
jac(47,47)=jac(47,47)-k(5)*y(61);
jac(60,61)=jac(60,61)+k(5)*y(47);
jac(60,47)=jac(60,47)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(54,54)=jac(54,54)-k(5)*y(47);
jac(54,47)=jac(54,47)-k(5)*y(54);
jac(47,54)=jac(47,54)-k(5)*y(47);
jac(47,47)=jac(47,47)-k(5)*y(54);
jac(53,54)=jac(53,54)+k(5)*y(47);
jac(53,47)=jac(53,47)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(48,48)=jac(48,48)-k(5)*y(47);
jac(48,47)=jac(48,47)-k(5)*y(48);
jac(47,48)=jac(47,48)-k(5)*y(47);
jac(47,47)=jac(47,47)-k(5)*y(48);
jac(149,48)=jac(149,48)+k(5)*y(47);
jac(149,47)=jac(149,47)+k(5)*y(48);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Lig(l1, l2) 

jac(47,47)=jac(47,47)-k(4);
jac(4,47)=jac(4,47)+k(4);
jac(48,47)=jac(48,47)+k(4);

% rule    : R15
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(48,48)=jac(48,48)-k(32);
jac(7,48)=jac(7,48)+k(32);

% rule    : R9
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(48);
jac(182,48)=jac(182,48)-k(21)*y(182);
jac(48,182)=jac(48,182)-k(21)*y(48);
jac(48,48)=jac(48,48)-k(21)*y(182);
jac(184,182)=jac(184,182)+k(21)*y(48);
jac(184,48)=jac(184,48)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(48);
jac(197,48)=jac(197,48)-k(21)*y(197);
jac(48,197)=jac(48,197)-k(21)*y(48);
jac(48,48)=jac(48,48)-k(21)*y(197);
jac(335,197)=jac(335,197)+k(21)*y(48);
jac(335,48)=jac(335,48)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(48);
jac(32,48)=jac(32,48)-k(21)*y(32);
jac(48,32)=jac(48,32)-k(21)*y(48);
jac(48,48)=jac(48,48)-k(21)*y(32);
jac(131,32)=jac(131,32)+k(21)*y(48);
jac(131,48)=jac(131,48)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(48);
jac(2,48)=jac(2,48)-k(21)*y(2);
jac(48,2)=jac(48,2)-k(21)*y(48);
jac(48,48)=jac(48,48)-k(21)*y(2);
jac(140,2)=jac(140,2)+k(21)*y(48);
jac(140,48)=jac(140,48)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a, b~Y, g~pY) + Lyn(U, SH2) 

jac(48,48)=jac(48,48)-k(10);
jac(3,48)=jac(3,48)+k(10);
jac(15,48)=jac(15,48)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(55);
jac(48,55)=jac(48,55)-k(7)*y(48);
jac(55,48)=jac(55,48)-k(7)*y(55);
jac(55,55)=jac(55,55)-k(7)*y(48);
jac(53,48)=jac(53,48)+k(7)*y(55);
jac(53,55)=jac(53,55)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(62);
jac(48,62)=jac(48,62)-k(7)*y(48);
jac(62,48)=jac(62,48)-k(7)*y(62);
jac(62,62)=jac(62,62)-k(7)*y(48);
jac(60,48)=jac(60,48)+k(7)*y(62);
jac(60,62)=jac(60,62)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(91,91)=jac(91,91)-k(7)*y(48);
jac(91,48)=jac(91,48)-k(7)*y(91);
jac(48,91)=jac(48,91)-k(7)*y(48);
jac(48,48)=jac(48,48)-k(7)*y(91);
jac(64,91)=jac(64,91)+k(7)*y(48);
jac(64,48)=jac(64,48)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(126);
jac(48,126)=jac(48,126)-k(7)*y(48);
jac(126,48)=jac(126,48)-k(7)*y(126);
jac(126,126)=jac(126,126)-k(7)*y(48);
jac(149,48)=jac(149,48)+k(7)*y(126);
jac(149,126)=jac(149,126)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(132);
jac(48,132)=jac(48,132)-k(7)*y(48);
jac(132,48)=jac(132,48)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(48);
jac(155,48)=jac(155,48)+k(7)*y(132);
jac(155,132)=jac(155,132)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(141);
jac(48,141)=jac(48,141)-k(7)*y(48);
jac(141,48)=jac(141,48)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(48);
jac(467,48)=jac(467,48)+k(7)*y(141);
jac(467,141)=jac(467,141)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(185);
jac(48,185)=jac(48,185)-k(7)*y(48);
jac(185,48)=jac(185,48)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(48);
jac(457,48)=jac(457,48)+k(7)*y(185);
jac(457,185)=jac(457,185)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(48,48)=jac(48,48)-k(7)*y(209);
jac(48,209)=jac(48,209)-k(7)*y(48);
jac(209,48)=jac(209,48)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(48);
jac(399,48)=jac(399,48)+k(7)*y(209);
jac(399,209)=jac(399,209)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(48,48)=jac(48,48)-k(7)*y(214);
jac(48,214)=jac(48,214)-k(7)*y(48);
jac(214,48)=jac(214,48)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(48);
jac(396,48)=jac(396,48)+k(7)*y(214);
jac(396,214)=jac(396,214)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(48,48)=jac(48,48)-k(7)*y(219);
jac(48,219)=jac(48,219)-k(7)*y(48);
jac(219,48)=jac(219,48)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(48);
jac(430,48)=jac(430,48)+k(7)*y(219);
jac(430,219)=jac(430,219)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(223);
jac(48,223)=jac(48,223)-k(7)*y(48);
jac(223,48)=jac(223,48)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(48);
jac(225,48)=jac(225,48)+k(7)*y(223);
jac(225,223)=jac(225,223)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(48,48)=jac(48,48)-k(7)*y(228);
jac(48,228)=jac(48,228)-k(7)*y(48);
jac(228,48)=jac(228,48)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(48);
jac(205,48)=jac(205,48)+k(7)*y(228);
jac(205,228)=jac(205,228)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(315,315)=jac(315,315)-k(7)*y(48);
jac(315,48)=jac(315,48)-k(7)*y(315);
jac(48,315)=jac(48,315)-k(7)*y(48);
jac(48,48)=jac(48,48)-k(7)*y(315);
jac(57,315)=jac(57,315)+k(7)*y(48);
jac(57,48)=jac(57,48)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(48,48)=jac(48,48)-k(7)*y(319);
jac(48,319)=jac(48,319)-k(7)*y(48);
jac(319,48)=jac(319,48)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(48);
jac(401,48)=jac(401,48)+k(7)*y(319);
jac(401,319)=jac(401,319)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(48,48)=jac(48,48)-k(7)*y(336);
jac(48,336)=jac(48,336)-k(7)*y(48);
jac(336,48)=jac(336,48)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(48);
jac(400,48)=jac(400,48)+k(7)*y(336);
jac(400,336)=jac(400,336)+k(7)*y(48);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(48,48)=jac(48,48)-k(7)*y(437);
jac(48,437)=jac(48,437)-k(7)*y(48);
jac(437,48)=jac(437,48)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(48);
jac(50,48)=jac(50,48)+k(7)*y(437);
jac(50,437)=jac(50,437)+k(7)*y(48);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(71,71)=jac(71,71)-k(5)*y(48);
jac(71,48)=jac(71,48)-k(5)*y(71);
jac(48,71)=jac(48,71)-k(5)*y(48);
jac(48,48)=jac(48,48)-k(5)*y(71);
jac(128,71)=jac(128,71)+k(5)*y(48);
jac(128,48)=jac(128,48)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(76,76)=jac(76,76)-k(5)*y(48);
jac(76,48)=jac(76,48)-k(5)*y(76);
jac(48,76)=jac(48,76)-k(5)*y(48);
jac(48,48)=jac(48,48)-k(5)*y(76);
jac(151,76)=jac(151,76)+k(5)*y(48);
jac(151,48)=jac(151,48)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(48,48)=jac(48,48)-k(5)*y(82);
jac(48,82)=jac(48,82)-k(5)*y(48);
jac(82,48)=jac(82,48)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(48);
jac(350,48)=jac(350,48)+k(5)*y(82);
jac(350,82)=jac(350,82)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(48,48)=jac(48,48)-k(5)*y(86);
jac(48,86)=jac(48,86)-k(5)*y(48);
jac(86,48)=jac(86,48)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(48);
jac(153,48)=jac(153,48)+k(5)*y(86);
jac(153,86)=jac(153,86)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(48,48)=jac(48,48)-k(5)*y(89);
jac(48,89)=jac(48,89)-k(5)*y(48);
jac(89,48)=jac(89,48)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(48);
jac(152,48)=jac(152,48)+k(5)*y(89);
jac(152,89)=jac(152,89)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) 

jac(98,98)=jac(98,98)-k(5)*y(48);
jac(98,48)=jac(98,48)-k(5)*y(98);
jac(48,98)=jac(48,98)-k(5)*y(48);
jac(48,48)=jac(48,48)-k(5)*y(98);
jac(147,98)=jac(147,98)+k(5)*y(48);
jac(147,48)=jac(147,48)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(48,48)=jac(48,48)-k(5)*y(110);
jac(48,110)=jac(48,110)-k(5)*y(48);
jac(110,48)=jac(110,48)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(48);
jac(353,48)=jac(353,48)+k(5)*y(110);
jac(353,110)=jac(353,110)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(48,48)=jac(48,48)-k(5)*y(114);
jac(48,114)=jac(48,114)-k(5)*y(48);
jac(114,48)=jac(114,48)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(48);
jac(116,48)=jac(116,48)+k(5)*y(114);
jac(116,114)=jac(116,114)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(48,48)=jac(48,48)-k(5)*y(118);
jac(48,118)=jac(48,118)-k(5)*y(48);
jac(118,48)=jac(118,48)-k(5)*y(118);
jac(118,118)=jac(118,118)-k(5)*y(48);
jac(120,48)=jac(120,48)+k(5)*y(118);
jac(120,118)=jac(120,118)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(48,48)=jac(48,48)-k(5)*y(122);
jac(48,122)=jac(48,122)-k(5)*y(48);
jac(122,48)=jac(122,48)-k(5)*y(122);
jac(122,122)=jac(122,122)-k(5)*y(48);
jac(125,48)=jac(125,48)+k(5)*y(122);
jac(125,122)=jac(125,122)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(48,48)=jac(48,48)-k(5)*y(176);
jac(48,176)=jac(48,176)-k(5)*y(48);
jac(176,48)=jac(176,48)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(48);
jac(181,48)=jac(181,48)+k(5)*y(176);
jac(181,176)=jac(181,176)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(48,48)=jac(48,48)-k(5)*y(191);
jac(48,191)=jac(48,191)-k(5)*y(48);
jac(191,48)=jac(191,48)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(48);
jac(196,48)=jac(196,48)+k(5)*y(191);
jac(196,191)=jac(196,191)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(48,48)=jac(48,48)-k(5)*y(256);
jac(48,256)=jac(48,256)-k(5)*y(48);
jac(256,48)=jac(256,48)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(48);
jac(599,48)=jac(599,48)+k(5)*y(256);
jac(599,256)=jac(599,256)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(48,48)=jac(48,48)-k(5)*y(264);
jac(48,264)=jac(48,264)-k(5)*y(48);
jac(264,48)=jac(264,48)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(48);
jac(361,48)=jac(361,48)+k(5)*y(264);
jac(361,264)=jac(361,264)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(48,48)=jac(48,48)-k(5)*y(380);
jac(48,380)=jac(48,380)-k(5)*y(48);
jac(380,48)=jac(380,48)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(48);
jac(461,48)=jac(461,48)+k(5)*y(380);
jac(461,380)=jac(461,380)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(48,48)=jac(48,48)-k(5)*y(391);
jac(48,391)=jac(48,391)-k(5)*y(48);
jac(391,48)=jac(391,48)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(48);
jac(651,48)=jac(651,48)+k(5)*y(391);
jac(651,391)=jac(651,391)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(48,48)=jac(48,48)-k(5)*y(407);
jac(48,407)=jac(48,407)-k(5)*y(48);
jac(407,48)=jac(407,48)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(48);
jac(408,48)=jac(408,48)+k(5)*y(407);
jac(408,407)=jac(408,407)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(48,48)=jac(48,48)-k(5)*y(441);
jac(48,441)=jac(48,441)-k(5)*y(48);
jac(441,48)=jac(441,48)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(48);
jac(458,48)=jac(458,48)+k(5)*y(441);
jac(458,441)=jac(458,441)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(48,48)=jac(48,48)-k(5)*y(515);
jac(48,515)=jac(48,515)-k(5)*y(48);
jac(515,48)=jac(515,48)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(48);
jac(595,48)=jac(595,48)+k(5)*y(515);
jac(595,515)=jac(595,515)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(48,48)=jac(48,48)-k(5)*y(543);
jac(48,543)=jac(48,543)-k(5)*y(48);
jac(543,48)=jac(543,48)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(48);
jac(598,48)=jac(598,48)+k(5)*y(543);
jac(598,543)=jac(598,543)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(48,48)=jac(48,48)-k(5)*y(554);
jac(48,554)=jac(48,554)-k(5)*y(48);
jac(554,48)=jac(554,48)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(48);
jac(352,48)=jac(352,48)+k(5)*y(554);
jac(352,554)=jac(352,554)+k(5)*y(48);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(48,48)=jac(48,48)-k(5)*y(9);
jac(48,9)=jac(48,9)-k(5)*y(48);
jac(9,48)=jac(9,48)-k(5)*y(9);
jac(9,9)=jac(9,9)-k(5)*y(48);
jac(17,48)=jac(17,48)+k(5)*y(9);
jac(17,9)=jac(17,9)+k(5)*y(48);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(4,4)=jac(4,4)-k(3)*y(48);
jac(4,48)=jac(4,48)-k(3)*y(4);
jac(48,4)=jac(48,4)-k(3)*y(48);
jac(48,48)=jac(48,48)-k(3)*y(4);
jac(47,4)=jac(47,4)+k(3)*y(48);
jac(47,48)=jac(47,48)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(4,4)=jac(4,4)-k(1)*y(48);
jac(4,48)=jac(4,48)-k(1)*y(4);
jac(48,4)=jac(48,4)-k(1)*y(48);
jac(48,48)=jac(48,48)-k(1)*y(4);
jac(126,4)=jac(126,4)+k(1)*y(48);
jac(126,48)=jac(126,48)+k(1)*y(4);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(49,49)=jac(49,49)-k(32);
jac(45,49)=jac(45,49)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

jac(49,49)=jac(49,49)-k(22);
jac(2,49)=jac(2,49)+k(22);
jac(56,49)=jac(56,49)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(49);
jac(182,49)=jac(182,49)-k(21)*y(182);
jac(49,182)=jac(49,182)-k(21)*y(49);
jac(49,49)=jac(49,49)-k(21)*y(182);
jac(269,182)=jac(269,182)+k(21)*y(49);
jac(269,49)=jac(269,49)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(49);
jac(197,49)=jac(197,49)-k(21)*y(197);
jac(49,197)=jac(49,197)-k(21)*y(49);
jac(49,49)=jac(49,49)-k(21)*y(197);
jac(262,197)=jac(262,197)+k(21)*y(49);
jac(262,49)=jac(262,49)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(49);
jac(32,49)=jac(32,49)-k(21)*y(32);
jac(49,32)=jac(49,32)-k(21)*y(49);
jac(49,49)=jac(49,49)-k(21)*y(32);
jac(277,32)=jac(277,32)+k(21)*y(49);
jac(277,49)=jac(277,49)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(49);
jac(2,49)=jac(2,49)-k(21)*y(2);
jac(49,2)=jac(49,2)-k(21)*y(49);
jac(49,49)=jac(49,49)-k(21)*y(2);
jac(274,2)=jac(274,2)+k(21)*y(49);
jac(274,49)=jac(274,49)+k(21)*y(2);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(49,49)=jac(49,49)-k(18);
jac(250,49)=jac(250,49)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(49,49)=jac(49,49)-k(16);
jac(3,49)=jac(3,49)+k(16);
jac(616,49)=jac(616,49)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(49);
jac(3,49)=jac(3,49)-k(9)*y(3);
jac(49,3)=jac(49,3)-k(9)*y(49);
jac(49,49)=jac(49,49)-k(9)*y(3);
jac(46,3)=jac(46,3)+k(9)*y(49);
jac(46,49)=jac(46,49)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

jac(49,49)=jac(49,49)-k(8);
jac(15,49)=jac(15,49)+k(8);
jac(44,49)=jac(44,49)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(49,49)=jac(49,49)-k(6);
jac(98,49)=jac(98,49)+k(6);
jac(43,49)=jac(43,49)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(50,50)=jac(50,50)-k(32);
jac(25,50)=jac(25,50)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(50,50)=jac(50,50)-k(31);
jac(388,50)=jac(388,50)+k(31);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(50,50)=jac(50,50)-k(23);
jac(470,50)=jac(470,50)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(50,50)=jac(50,50)-k(22);
jac(2,50)=jac(2,50)+k(22);
jac(57,50)=jac(57,50)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(50);
jac(182,50)=jac(182,50)-k(21)*y(182);
jac(50,182)=jac(50,182)-k(21)*y(50);
jac(50,50)=jac(50,50)-k(21)*y(182);
jac(392,182)=jac(392,182)+k(21)*y(50);
jac(392,50)=jac(392,50)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(50);
jac(197,50)=jac(197,50)-k(21)*y(197);
jac(50,197)=jac(50,197)-k(21)*y(50);
jac(50,50)=jac(50,50)-k(21)*y(197);
jac(436,197)=jac(436,197)+k(21)*y(50);
jac(436,50)=jac(436,50)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(50);
jac(32,50)=jac(32,50)-k(21)*y(32);
jac(50,32)=jac(50,32)-k(21)*y(50);
jac(50,50)=jac(50,50)-k(21)*y(32);
jac(440,32)=jac(440,32)+k(21)*y(50);
jac(440,50)=jac(440,50)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(50);
jac(2,50)=jac(2,50)-k(21)*y(2);
jac(50,2)=jac(50,2)-k(21)*y(50);
jac(50,50)=jac(50,50)-k(21)*y(2);
jac(387,2)=jac(387,2)+k(21)*y(50);
jac(387,50)=jac(387,50)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(50);
jac(3,50)=jac(3,50)-k(15)*y(3);
jac(50,3)=jac(50,3)-k(15)*y(50);
jac(50,50)=jac(50,50)-k(15)*y(3);
jac(46,3)=jac(46,3)+k(15)*y(50);
jac(46,50)=jac(46,50)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(50,50)=jac(50,50)-k(10);
jac(3,50)=jac(3,50)+k(10);
jac(616,50)=jac(616,50)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(50,50)=jac(50,50)-k(8);
jac(48,50)=jac(48,50)+k(8);
jac(437,50)=jac(437,50)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(50,50)=jac(50,50)-k(6);
jac(47,50)=jac(47,50)+k(6);
jac(111,50)=jac(111,50)+k(6);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(51,51)=jac(51,51)-k(28);
jac(270,51)=jac(270,51)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(51,51)=jac(51,51)-k(27);
jac(279,51)=jac(279,51)+k(27);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(51,51)=jac(51,51)-k(26);
jac(52,51)=jac(52,51)+k(26);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(51,51)=jac(51,51)-k(23);
jac(480,51)=jac(480,51)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(51,51)=jac(51,51)-k(22);
jac(2,51)=jac(2,51)+k(22);
jac(46,51)=jac(46,51)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(51,51)=jac(51,51)-k(22);
jac(2,51)=jac(2,51)+k(22);
jac(58,51)=jac(58,51)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(51,51)=jac(51,51)-k(16);
jac(3,51)=jac(3,51)+k(16);
jac(387,51)=jac(387,51)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(51,51)=jac(51,51)-k(10);
jac(3,51)=jac(3,51)+k(10);
jac(274,51)=jac(274,51)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(51,51)=jac(51,51)-k(8);
jac(140,51)=jac(140,51)+k(8);
jac(44,51)=jac(44,51)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(51,51)=jac(51,51)-k(6);
jac(43,51)=jac(43,51)+k(6);
jac(380,51)=jac(380,51)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(52,52)=jac(52,52)-k(33);
jac(51,52)=jac(51,52)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(52,52)=jac(52,52)-k(28);
jac(207,52)=jac(207,52)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(52,52)=jac(52,52)-k(27);
jac(284,52)=jac(284,52)+k(27);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(52,52)=jac(52,52)-k(23);
jac(469,52)=jac(469,52)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(52,52)=jac(52,52)-k(22);
jac(32,52)=jac(32,52)+k(22);
jac(46,52)=jac(46,52)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(52,52)=jac(52,52)-k(22);
jac(2,52)=jac(2,52)+k(22);
jac(59,52)=jac(59,52)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(52,52)=jac(52,52)-k(16);
jac(3,52)=jac(3,52)+k(16);
jac(440,52)=jac(440,52)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(52,52)=jac(52,52)-k(10);
jac(3,52)=jac(3,52)+k(10);
jac(277,52)=jac(277,52)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(52,52)=jac(52,52)-k(8);
jac(131,52)=jac(131,52)+k(8);
jac(44,52)=jac(44,52)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(52,52)=jac(52,52)-k(6);
jac(43,52)=jac(43,52)+k(6);
jac(441,52)=jac(441,52)+k(6);

% rule    : R15
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(54,54)=jac(54,54)-k(32);
jac(61,54)=jac(61,54)+k(32);

% rule    : R9
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(54);
jac(182,54)=jac(182,54)-k(21)*y(182);
jac(54,182)=jac(54,182)-k(21)*y(54);
jac(54,54)=jac(54,54)-k(21)*y(182);
jac(222,182)=jac(222,182)+k(21)*y(54);
jac(222,54)=jac(222,54)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(54);
jac(197,54)=jac(197,54)-k(21)*y(197);
jac(54,197)=jac(54,197)-k(21)*y(54);
jac(54,54)=jac(54,54)-k(21)*y(197);
jac(227,197)=jac(227,197)+k(21)*y(54);
jac(227,54)=jac(227,54)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(54);
jac(32,54)=jac(32,54)-k(21)*y(32);
jac(54,32)=jac(54,32)-k(21)*y(54);
jac(54,54)=jac(54,54)-k(21)*y(32);
jac(37,32)=jac(37,32)+k(21)*y(54);
jac(37,54)=jac(37,54)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(54);
jac(2,54)=jac(2,54)-k(21)*y(2);
jac(54,2)=jac(54,2)-k(21)*y(54);
jac(54,54)=jac(54,54)-k(21)*y(2);
jac(43,2)=jac(43,2)+k(21)*y(54);
jac(43,54)=jac(43,54)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a, b~pY, g~pY) + Lyn(U, SH2) 

jac(54,54)=jac(54,54)-k(16);
jac(3,54)=jac(3,54)+k(16);
jac(77,54)=jac(77,54)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(54,54)=jac(54,54)-k(7)*y(55);
jac(54,55)=jac(54,55)-k(7)*y(54);
jac(55,54)=jac(55,54)-k(7)*y(55);
jac(55,55)=jac(55,55)-k(7)*y(54);
jac(627,54)=jac(627,54)+k(7)*y(55);
jac(627,55)=jac(627,55)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(54,54)=jac(54,54)-k(7)*y(62);
jac(54,62)=jac(54,62)-k(7)*y(54);
jac(62,54)=jac(62,54)-k(7)*y(62);
jac(62,62)=jac(62,62)-k(7)*y(54);
jac(78,54)=jac(78,54)+k(7)*y(62);
jac(78,62)=jac(78,62)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(91,91)=jac(91,91)-k(7)*y(54);
jac(91,54)=jac(91,54)-k(7)*y(91);
jac(54,91)=jac(54,91)-k(7)*y(54);
jac(54,54)=jac(54,54)-k(7)*y(91);
jac(309,91)=jac(309,91)+k(7)*y(54);
jac(309,54)=jac(309,54)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(54,54)=jac(54,54)-k(7)*y(126);
jac(54,126)=jac(54,126)-k(7)*y(54);
jac(126,54)=jac(126,54)-k(7)*y(126);
jac(126,126)=jac(126,126)-k(7)*y(54);
jac(120,54)=jac(120,54)+k(7)*y(126);
jac(120,126)=jac(120,126)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(54,54)=jac(54,54)-k(7)*y(132);
jac(54,132)=jac(54,132)-k(7)*y(54);
jac(132,54)=jac(132,54)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(54);
jac(134,54)=jac(134,54)+k(7)*y(132);
jac(134,132)=jac(134,132)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(54,54)=jac(54,54)-k(7)*y(141);
jac(54,141)=jac(54,141)-k(7)*y(54);
jac(141,54)=jac(141,54)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(54);
jac(539,54)=jac(539,54)+k(7)*y(141);
jac(539,141)=jac(539,141)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(54,54)=jac(54,54)-k(7)*y(185);
jac(54,185)=jac(54,185)-k(7)*y(54);
jac(185,54)=jac(185,54)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(54);
jac(345,54)=jac(345,54)+k(7)*y(185);
jac(345,185)=jac(345,185)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(54,54)=jac(54,54)-k(7)*y(209);
jac(54,209)=jac(54,209)-k(7)*y(54);
jac(209,54)=jac(209,54)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(54);
jac(529,54)=jac(529,54)+k(7)*y(209);
jac(529,209)=jac(529,209)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(54,54)=jac(54,54)-k(7)*y(214);
jac(54,214)=jac(54,214)-k(7)*y(54);
jac(214,54)=jac(214,54)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(54);
jac(519,54)=jac(519,54)+k(7)*y(214);
jac(519,214)=jac(519,214)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(54,54)=jac(54,54)-k(7)*y(219);
jac(54,219)=jac(54,219)-k(7)*y(54);
jac(219,54)=jac(219,54)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(54);
jac(644,54)=jac(644,54)+k(7)*y(219);
jac(644,219)=jac(644,219)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(54,54)=jac(54,54)-k(7)*y(223);
jac(54,223)=jac(54,223)-k(7)*y(54);
jac(223,54)=jac(223,54)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(54);
jac(638,54)=jac(638,54)+k(7)*y(223);
jac(638,223)=jac(638,223)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(54,54)=jac(54,54)-k(7)*y(228);
jac(54,228)=jac(54,228)-k(7)*y(54);
jac(228,54)=jac(228,54)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(54);
jac(636,54)=jac(636,54)+k(7)*y(228);
jac(636,228)=jac(636,228)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(315,315)=jac(315,315)-k(7)*y(54);
jac(315,54)=jac(315,54)-k(7)*y(315);
jac(54,315)=jac(54,315)-k(7)*y(54);
jac(54,54)=jac(54,54)-k(7)*y(315);
jac(310,315)=jac(310,315)+k(7)*y(54);
jac(310,54)=jac(310,54)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(54,54)=jac(54,54)-k(7)*y(319);
jac(54,319)=jac(54,319)-k(7)*y(54);
jac(319,54)=jac(319,54)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(54);
jac(564,54)=jac(564,54)+k(7)*y(319);
jac(564,319)=jac(564,319)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(54,54)=jac(54,54)-k(7)*y(336);
jac(54,336)=jac(54,336)-k(7)*y(54);
jac(336,54)=jac(336,54)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(54);
jac(339,54)=jac(339,54)+k(7)*y(336);
jac(339,336)=jac(339,336)+k(7)*y(54);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(54,54)=jac(54,54)-k(7)*y(437);
jac(54,437)=jac(54,437)-k(7)*y(54);
jac(437,54)=jac(437,54)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(54);
jac(540,54)=jac(540,54)+k(7)*y(437);
jac(540,437)=jac(540,437)+k(7)*y(54);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(71,71)=jac(71,71)-k(5)*y(54);
jac(71,54)=jac(71,54)-k(5)*y(71);
jac(54,71)=jac(54,71)-k(5)*y(54);
jac(54,54)=jac(54,54)-k(5)*y(71);
jac(235,71)=jac(235,71)+k(5)*y(54);
jac(235,54)=jac(235,54)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(76,76)=jac(76,76)-k(5)*y(54);
jac(76,54)=jac(76,54)-k(5)*y(76);
jac(54,76)=jac(54,76)-k(5)*y(54);
jac(54,54)=jac(54,54)-k(5)*y(76);
jac(238,76)=jac(238,76)+k(5)*y(54);
jac(238,54)=jac(238,54)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(54,54)=jac(54,54)-k(5)*y(82);
jac(54,82)=jac(54,82)-k(5)*y(54);
jac(82,54)=jac(82,54)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(54);
jac(634,54)=jac(634,54)+k(5)*y(82);
jac(634,82)=jac(634,82)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(54,54)=jac(54,54)-k(5)*y(86);
jac(54,86)=jac(54,86)-k(5)*y(54);
jac(86,54)=jac(86,54)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(54);
jac(287,54)=jac(287,54)+k(5)*y(86);
jac(287,86)=jac(287,86)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(54,54)=jac(54,54)-k(5)*y(89);
jac(54,89)=jac(54,89)-k(5)*y(54);
jac(89,54)=jac(89,54)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(54);
jac(276,54)=jac(276,54)+k(5)*y(89);
jac(276,89)=jac(276,89)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(98,98)=jac(98,98)-k(5)*y(54);
jac(98,54)=jac(98,54)-k(5)*y(98);
jac(54,98)=jac(54,98)-k(5)*y(54);
jac(54,54)=jac(54,54)-k(5)*y(98);
jac(56,98)=jac(56,98)+k(5)*y(54);
jac(56,54)=jac(56,54)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(54,54)=jac(54,54)-k(5)*y(110);
jac(54,110)=jac(54,110)-k(5)*y(54);
jac(110,54)=jac(110,54)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(54);
jac(629,54)=jac(629,54)+k(5)*y(110);
jac(629,110)=jac(629,110)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(54,54)=jac(54,54)-k(5)*y(114);
jac(54,114)=jac(54,114)-k(5)*y(54);
jac(114,54)=jac(114,54)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(54);
jac(628,54)=jac(628,54)+k(5)*y(114);
jac(628,114)=jac(628,114)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(54,54)=jac(54,54)-k(5)*y(118);
jac(54,118)=jac(54,118)-k(5)*y(54);
jac(118,54)=jac(118,54)-k(5)*y(118);
jac(118,118)=jac(118,118)-k(5)*y(54);
jac(627,54)=jac(627,54)+k(5)*y(118);
jac(627,118)=jac(627,118)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(54,54)=jac(54,54)-k(5)*y(122);
jac(54,122)=jac(54,122)-k(5)*y(54);
jac(122,54)=jac(122,54)-k(5)*y(122);
jac(122,122)=jac(122,122)-k(5)*y(54);
jac(236,54)=jac(236,54)+k(5)*y(122);
jac(236,122)=jac(236,122)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(54,54)=jac(54,54)-k(5)*y(176);
jac(54,176)=jac(54,176)-k(5)*y(54);
jac(176,54)=jac(176,54)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(54);
jac(272,54)=jac(272,54)+k(5)*y(176);
jac(272,176)=jac(272,176)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(54,54)=jac(54,54)-k(5)*y(191);
jac(54,191)=jac(54,191)-k(5)*y(54);
jac(191,54)=jac(191,54)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(54);
jac(291,54)=jac(291,54)+k(5)*y(191);
jac(291,191)=jac(291,191)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(54,54)=jac(54,54)-k(5)*y(256);
jac(54,256)=jac(54,256)-k(5)*y(54);
jac(256,54)=jac(256,54)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(54);
jac(260,54)=jac(260,54)+k(5)*y(256);
jac(260,256)=jac(260,256)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(54,54)=jac(54,54)-k(5)*y(264);
jac(54,264)=jac(54,264)-k(5)*y(54);
jac(264,54)=jac(264,54)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(54);
jac(267,54)=jac(267,54)+k(5)*y(264);
jac(267,264)=jac(267,264)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(54,54)=jac(54,54)-k(5)*y(380);
jac(54,380)=jac(54,380)-k(5)*y(54);
jac(380,54)=jac(380,54)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(54);
jac(58,54)=jac(58,54)+k(5)*y(380);
jac(58,380)=jac(58,380)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(54,54)=jac(54,54)-k(5)*y(391);
jac(54,391)=jac(54,391)-k(5)*y(54);
jac(391,54)=jac(391,54)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(54);
jac(422,54)=jac(422,54)+k(5)*y(391);
jac(422,391)=jac(422,391)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(54,54)=jac(54,54)-k(5)*y(407);
jac(54,407)=jac(54,407)-k(5)*y(54);
jac(407,54)=jac(407,54)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(54);
jac(206,54)=jac(206,54)+k(5)*y(407);
jac(206,407)=jac(206,407)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(54,54)=jac(54,54)-k(5)*y(441);
jac(54,441)=jac(54,441)-k(5)*y(54);
jac(441,54)=jac(441,54)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(54);
jac(59,54)=jac(59,54)+k(5)*y(441);
jac(59,441)=jac(59,441)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(54,54)=jac(54,54)-k(5)*y(515);
jac(54,515)=jac(54,515)-k(5)*y(54);
jac(515,54)=jac(515,54)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(54);
jac(624,54)=jac(624,54)+k(5)*y(515);
jac(624,515)=jac(624,515)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(54,54)=jac(54,54)-k(5)*y(543);
jac(54,543)=jac(54,543)-k(5)*y(54);
jac(543,54)=jac(543,54)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(54);
jac(622,54)=jac(622,54)+k(5)*y(543);
jac(622,543)=jac(622,543)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(54,54)=jac(54,54)-k(5)*y(554);
jac(54,554)=jac(54,554)-k(5)*y(54);
jac(554,54)=jac(554,54)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(54);
jac(633,54)=jac(633,54)+k(5)*y(554);
jac(633,554)=jac(633,554)+k(5)*y(54);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(54,54)=jac(54,54)-k(5)*y(9);
jac(54,9)=jac(54,9)-k(5)*y(54);
jac(9,54)=jac(9,54)-k(5)*y(9);
jac(9,9)=jac(9,9)-k(5)*y(54);
jac(41,54)=jac(41,54)+k(5)*y(9);
jac(41,9)=jac(41,9)+k(5)*y(54);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(4,4)=jac(4,4)-k(3)*y(54);
jac(4,54)=jac(4,54)-k(3)*y(4);
jac(54,4)=jac(54,4)-k(3)*y(54);
jac(54,54)=jac(54,54)-k(3)*y(4);
jac(118,4)=jac(118,4)+k(3)*y(54);
jac(118,54)=jac(118,54)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(4,4)=jac(4,4)-k(1)*y(54);
jac(4,54)=jac(4,54)-k(1)*y(4);
jac(54,4)=jac(54,4)-k(1)*y(54);
jac(54,54)=jac(54,54)-k(1)*y(4);
jac(55,4)=jac(55,4)+k(1)*y(54);
jac(55,54)=jac(55,54)+k(1)*y(4);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(55,55)=jac(55,55)-k(32);
jac(62,55)=jac(62,55)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(55);
jac(182,55)=jac(182,55)-k(21)*y(182);
jac(55,182)=jac(55,182)-k(21)*y(55);
jac(55,55)=jac(55,55)-k(21)*y(182);
jac(223,182)=jac(223,182)+k(21)*y(55);
jac(223,55)=jac(223,55)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(55);
jac(197,55)=jac(197,55)-k(21)*y(197);
jac(55,197)=jac(55,197)-k(21)*y(55);
jac(55,55)=jac(55,55)-k(21)*y(197);
jac(228,197)=jac(228,197)+k(21)*y(55);
jac(228,55)=jac(228,55)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(55);
jac(32,55)=jac(32,55)-k(21)*y(32);
jac(55,32)=jac(55,32)-k(21)*y(55);
jac(55,55)=jac(55,55)-k(21)*y(32);
jac(38,32)=jac(38,32)+k(21)*y(55);
jac(38,55)=jac(38,55)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(55);
jac(2,55)=jac(2,55)-k(21)*y(2);
jac(55,2)=jac(55,2)-k(21)*y(55);
jac(55,55)=jac(55,55)-k(21)*y(2);
jac(44,2)=jac(44,2)+k(21)*y(55);
jac(44,55)=jac(44,55)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Lyn(U, SH2) 

jac(55,55)=jac(55,55)-k(16);
jac(3,55)=jac(3,55)+k(16);
jac(315,55)=jac(315,55)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(1,1)=jac(1,1)-k(7)*y(55);
jac(1,55)=jac(1,55)-k(7)*y(1);
jac(55,1)=jac(55,1)-k(7)*y(55);
jac(55,55)=jac(55,55)-k(7)*y(1);
jac(233,1)=jac(233,1)+k(7)*y(55);
jac(233,55)=jac(233,55)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(7)*y(55);
jac(7,55)=jac(7,55)-k(7)*y(7);
jac(55,7)=jac(55,7)-k(7)*y(55);
jac(55,55)=jac(55,55)-k(7)*y(7);
jac(41,7)=jac(41,7)+k(7)*y(55);
jac(41,55)=jac(41,55)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(55,55)=jac(55,55)-k(7)*y(335);
jac(55,335)=jac(55,335)-k(7)*y(55);
jac(335,55)=jac(335,55)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(55);
jac(206,55)=jac(206,55)+k(7)*y(335);
jac(206,335)=jac(206,335)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(55,55)=jac(55,55)-k(7)*y(257);
jac(55,257)=jac(55,257)-k(7)*y(55);
jac(257,55)=jac(257,55)-k(7)*y(257);
jac(257,257)=jac(257,257)-k(7)*y(55);
jac(260,55)=jac(260,55)+k(7)*y(257);
jac(260,257)=jac(260,257)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(55,55)=jac(55,55)-k(7)*y(227);
jac(55,227)=jac(55,227)-k(7)*y(55);
jac(227,55)=jac(227,55)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(55);
jac(622,55)=jac(622,55)+k(7)*y(227);
jac(622,227)=jac(622,227)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(55,55)=jac(55,55)-k(7)*y(222);
jac(55,222)=jac(55,222)-k(7)*y(55);
jac(222,55)=jac(222,55)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(55);
jac(624,55)=jac(624,55)+k(7)*y(222);
jac(624,222)=jac(624,222)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(55,55)=jac(55,55)-k(7)*y(218);
jac(55,218)=jac(55,218)-k(7)*y(55);
jac(218,55)=jac(218,55)-k(7)*y(218);
jac(218,218)=jac(218,218)-k(7)*y(55);
jac(267,55)=jac(267,55)+k(7)*y(218);
jac(267,218)=jac(267,218)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(55,55)=jac(55,55)-k(7)*y(192);
jac(55,192)=jac(55,192)-k(7)*y(55);
jac(192,55)=jac(192,55)-k(7)*y(192);
jac(192,192)=jac(192,192)-k(7)*y(55);
jac(291,55)=jac(291,55)+k(7)*y(192);
jac(291,192)=jac(291,192)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(55,55)=jac(55,55)-k(7)*y(184);
jac(55,184)=jac(55,184)-k(7)*y(55);
jac(184,55)=jac(184,55)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(55);
jac(422,55)=jac(422,55)+k(7)*y(184);
jac(422,184)=jac(422,184)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(55,55)=jac(55,55)-k(7)*y(177);
jac(55,177)=jac(55,177)-k(7)*y(55);
jac(177,55)=jac(177,55)-k(7)*y(177);
jac(177,177)=jac(177,177)-k(7)*y(55);
jac(272,55)=jac(272,55)+k(7)*y(177);
jac(272,177)=jac(272,177)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(55,55)=jac(55,55)-k(7)*y(140);
jac(55,140)=jac(55,140)-k(7)*y(55);
jac(140,55)=jac(140,55)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(55);
jac(58,55)=jac(58,55)+k(7)*y(140);
jac(58,140)=jac(58,140)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(55,55)=jac(55,55)-k(7)*y(131);
jac(55,131)=jac(55,131)-k(7)*y(55);
jac(131,55)=jac(131,55)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(55);
jac(59,55)=jac(59,55)+k(7)*y(131);
jac(59,131)=jac(59,131)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(55,55)=jac(55,55)-k(7)*y(111);
jac(55,111)=jac(55,111)-k(7)*y(55);
jac(111,55)=jac(111,55)-k(7)*y(111);
jac(111,111)=jac(111,111)-k(7)*y(55);
jac(629,55)=jac(629,55)+k(7)*y(111);
jac(629,111)=jac(629,111)+k(7)*y(55);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(77,77)=jac(77,77)-k(7)*y(55);
jac(77,55)=jac(77,55)-k(7)*y(77);
jac(55,77)=jac(55,77)-k(7)*y(55);
jac(55,55)=jac(55,55)-k(7)*y(77);
jac(238,77)=jac(238,77)+k(7)*y(55);
jac(238,55)=jac(238,55)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(72,72)=jac(72,72)-k(7)*y(55);
jac(72,55)=jac(72,55)-k(7)*y(72);
jac(55,72)=jac(55,72)-k(7)*y(55);
jac(55,55)=jac(55,55)-k(7)*y(72);
jac(235,72)=jac(235,72)+k(7)*y(55);
jac(235,55)=jac(235,55)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(7)*y(55);
jac(61,55)=jac(61,55)-k(7)*y(61);
jac(55,61)=jac(55,61)-k(7)*y(55);
jac(55,55)=jac(55,55)-k(7)*y(61);
jac(236,61)=jac(236,61)+k(7)*y(55);
jac(236,55)=jac(236,55)+k(7)*y(61);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Lig(l1, l2) 

jac(55,55)=jac(55,55)-k(2);
jac(4,55)=jac(4,55)+k(2);
jac(54,55)=jac(54,55)+k(2);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(56,56)=jac(56,56)-k(32);
jac(233,56)=jac(233,56)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(56,56)=jac(56,56)-k(32);
jac(63,56)=jac(63,56)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(56);
jac(182,56)=jac(182,56)-k(21)*y(182);
jac(56,182)=jac(56,182)-k(21)*y(56);
jac(56,56)=jac(56,56)-k(21)*y(182);
jac(282,182)=jac(282,182)+k(21)*y(56);
jac(282,56)=jac(282,56)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(56);
jac(197,56)=jac(197,56)-k(21)*y(197);
jac(56,197)=jac(56,197)-k(21)*y(56);
jac(56,56)=jac(56,56)-k(21)*y(197);
jac(232,197)=jac(232,197)+k(21)*y(56);
jac(232,56)=jac(232,56)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(56);
jac(32,56)=jac(32,56)-k(21)*y(32);
jac(56,32)=jac(56,32)-k(21)*y(56);
jac(56,56)=jac(56,56)-k(21)*y(32);
jac(249,32)=jac(249,32)+k(21)*y(56);
jac(249,56)=jac(249,56)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(56);
jac(2,56)=jac(2,56)-k(21)*y(2);
jac(56,2)=jac(56,2)-k(21)*y(56);
jac(56,56)=jac(56,56)-k(21)*y(2);
jac(49,2)=jac(49,2)+k(21)*y(56);
jac(49,56)=jac(49,56)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(56);
jac(182,56)=jac(182,56)-k(21)*y(182);
jac(56,182)=jac(56,182)-k(21)*y(56);
jac(56,56)=jac(56,56)-k(21)*y(182);
jac(272,182)=jac(272,182)+k(21)*y(56);
jac(272,56)=jac(272,56)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(56);
jac(197,56)=jac(197,56)-k(21)*y(197);
jac(56,197)=jac(56,197)-k(21)*y(56);
jac(56,56)=jac(56,56)-k(21)*y(197);
jac(291,197)=jac(291,197)+k(21)*y(56);
jac(291,56)=jac(291,56)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(56);
jac(32,56)=jac(32,56)-k(21)*y(32);
jac(56,32)=jac(56,32)-k(21)*y(56);
jac(56,56)=jac(56,56)-k(21)*y(32);
jac(287,32)=jac(287,32)+k(21)*y(56);
jac(287,56)=jac(287,56)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(56);
jac(2,56)=jac(2,56)-k(21)*y(2);
jac(56,2)=jac(56,2)-k(21)*y(56);
jac(56,56)=jac(56,56)-k(21)*y(2);
jac(276,2)=jac(276,2)+k(21)*y(56);
jac(276,56)=jac(276,56)+k(21)*y(2);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(56,56)=jac(56,56)-k(18);
jac(238,56)=jac(238,56)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

jac(56,56)=jac(56,56)-k(16);
jac(3,56)=jac(3,56)+k(16);
jac(614,56)=jac(614,56)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(56);
jac(3,56)=jac(3,56)-k(9)*y(3);
jac(56,3)=jac(56,3)-k(9)*y(56);
jac(56,56)=jac(56,56)-k(9)*y(3);
jac(53,3)=jac(53,3)+k(9)*y(56);
jac(53,56)=jac(53,56)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) 

jac(56,56)=jac(56,56)-k(8);
jac(15,56)=jac(15,56)+k(8);
jac(55,56)=jac(55,56)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(56,56)=jac(56,56)-k(6);
jac(98,56)=jac(98,56)+k(6);
jac(54,56)=jac(54,56)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(57,57)=jac(57,57)-k(32);
jac(19,57)=jac(19,57)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(57,57)=jac(57,57)-k(32);
jac(64,57)=jac(64,57)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(57,57)=jac(57,57)-k(31);
jac(404,57)=jac(404,57)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(57);
jac(182,57)=jac(182,57)-k(21)*y(182);
jac(57,182)=jac(57,182)-k(21)*y(57);
jac(57,57)=jac(57,57)-k(21)*y(182);
jac(430,182)=jac(430,182)+k(21)*y(57);
jac(430,57)=jac(430,57)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(57);
jac(197,57)=jac(197,57)-k(21)*y(197);
jac(57,197)=jac(57,197)-k(21)*y(57);
jac(57,57)=jac(57,57)-k(21)*y(197);
jac(401,197)=jac(401,197)+k(21)*y(57);
jac(401,57)=jac(401,57)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(57);
jac(32,57)=jac(32,57)-k(21)*y(32);
jac(57,32)=jac(57,32)-k(21)*y(57);
jac(57,57)=jac(57,57)-k(21)*y(32);
jac(470,32)=jac(470,32)+k(21)*y(57);
jac(470,57)=jac(470,57)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(57);
jac(2,57)=jac(2,57)-k(21)*y(2);
jac(57,2)=jac(57,2)-k(21)*y(57);
jac(57,57)=jac(57,57)-k(21)*y(2);
jac(50,2)=jac(50,2)+k(21)*y(57);
jac(50,57)=jac(50,57)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(57);
jac(182,57)=jac(182,57)-k(21)*y(182);
jac(57,182)=jac(57,182)-k(21)*y(57);
jac(57,57)=jac(57,57)-k(21)*y(182);
jac(420,182)=jac(420,182)+k(21)*y(57);
jac(420,57)=jac(420,57)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(57);
jac(197,57)=jac(197,57)-k(21)*y(197);
jac(57,197)=jac(57,197)-k(21)*y(57);
jac(57,57)=jac(57,57)-k(21)*y(197);
jac(410,197)=jac(410,197)+k(21)*y(57);
jac(410,57)=jac(410,57)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(57);
jac(32,57)=jac(32,57)-k(21)*y(32);
jac(57,32)=jac(57,32)-k(21)*y(57);
jac(57,57)=jac(57,57)-k(21)*y(32);
jac(443,32)=jac(443,32)+k(21)*y(57);
jac(443,57)=jac(443,57)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(57);
jac(2,57)=jac(2,57)-k(21)*y(2);
jac(57,2)=jac(57,2)-k(21)*y(57);
jac(57,57)=jac(57,57)-k(21)*y(2);
jac(424,2)=jac(424,2)+k(21)*y(57);
jac(424,57)=jac(424,57)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(57);
jac(3,57)=jac(3,57)-k(15)*y(3);
jac(57,3)=jac(57,3)-k(15)*y(57);
jac(57,57)=jac(57,57)-k(15)*y(3);
jac(53,3)=jac(53,3)+k(15)*y(57);
jac(53,57)=jac(53,57)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

jac(57,57)=jac(57,57)-k(10);
jac(3,57)=jac(3,57)+k(10);
jac(614,57)=jac(614,57)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(57,57)=jac(57,57)-k(8);
jac(315,57)=jac(315,57)+k(8);
jac(48,57)=jac(48,57)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) 

jac(57,57)=jac(57,57)-k(6);
jac(77,57)=jac(77,57)+k(6);
jac(47,57)=jac(47,57)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(58,58)=jac(58,58)-k(32);
jac(65,58)=jac(65,58)+k(32);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(58,58)=jac(58,58)-k(26);
jac(59,58)=jac(59,58)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(58,58)=jac(58,58)-k(22);
jac(2,58)=jac(2,58)+k(22);
jac(53,58)=jac(53,58)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(58);
jac(182,58)=jac(182,58)-k(21)*y(182);
jac(58,182)=jac(58,182)-k(21)*y(58);
jac(58,58)=jac(58,58)-k(21)*y(182);
jac(279,182)=jac(279,182)+k(21)*y(58);
jac(279,58)=jac(279,58)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(58);
jac(197,58)=jac(197,58)-k(21)*y(197);
jac(58,197)=jac(58,197)-k(21)*y(58);
jac(58,58)=jac(58,58)-k(21)*y(197);
jac(428,197)=jac(428,197)+k(21)*y(58);
jac(428,58)=jac(428,58)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(58);
jac(32,58)=jac(32,58)-k(21)*y(32);
jac(58,32)=jac(58,32)-k(21)*y(58);
jac(58,58)=jac(58,58)-k(21)*y(32);
jac(480,32)=jac(480,32)+k(21)*y(58);
jac(480,58)=jac(480,58)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(58);
jac(2,58)=jac(2,58)-k(21)*y(2);
jac(58,2)=jac(58,2)-k(21)*y(58);
jac(58,58)=jac(58,58)-k(21)*y(2);
jac(51,2)=jac(51,2)+k(21)*y(58);
jac(51,58)=jac(51,58)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(58,58)=jac(58,58)-k(16);
jac(3,58)=jac(3,58)+k(16);
jac(424,58)=jac(424,58)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(58,58)=jac(58,58)-k(10);
jac(3,58)=jac(3,58)+k(10);
jac(276,58)=jac(276,58)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(58,58)=jac(58,58)-k(8);
jac(55,58)=jac(55,58)+k(8);
jac(140,58)=jac(140,58)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(58,58)=jac(58,58)-k(6);
jac(54,58)=jac(54,58)+k(6);
jac(380,58)=jac(380,58)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(59,59)=jac(59,59)-k(33);
jac(58,59)=jac(58,59)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(59,59)=jac(59,59)-k(32);
jac(66,59)=jac(66,59)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(59,59)=jac(59,59)-k(22);
jac(32,59)=jac(32,59)+k(22);
jac(53,59)=jac(53,59)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(59);
jac(182,59)=jac(182,59)-k(21)*y(182);
jac(59,182)=jac(59,182)-k(21)*y(59);
jac(59,59)=jac(59,59)-k(21)*y(182);
jac(284,182)=jac(284,182)+k(21)*y(59);
jac(284,59)=jac(284,59)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(59);
jac(197,59)=jac(197,59)-k(21)*y(197);
jac(59,197)=jac(59,197)-k(21)*y(59);
jac(59,59)=jac(59,59)-k(21)*y(197);
jac(473,197)=jac(473,197)+k(21)*y(59);
jac(473,59)=jac(473,59)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(59);
jac(32,59)=jac(32,59)-k(21)*y(32);
jac(59,32)=jac(59,32)-k(21)*y(59);
jac(59,59)=jac(59,59)-k(21)*y(32);
jac(469,32)=jac(469,32)+k(21)*y(59);
jac(469,59)=jac(469,59)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(59);
jac(2,59)=jac(2,59)-k(21)*y(2);
jac(59,2)=jac(59,2)-k(21)*y(59);
jac(59,59)=jac(59,59)-k(21)*y(2);
jac(52,2)=jac(52,2)+k(21)*y(59);
jac(52,59)=jac(52,59)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(59,59)=jac(59,59)-k(16);
jac(3,59)=jac(3,59)+k(16);
jac(443,59)=jac(443,59)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(59,59)=jac(59,59)-k(10);
jac(3,59)=jac(3,59)+k(10);
jac(287,59)=jac(287,59)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(59,59)=jac(59,59)-k(8);
jac(55,59)=jac(55,59)+k(8);
jac(131,59)=jac(131,59)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(59,59)=jac(59,59)-k(6);
jac(54,59)=jac(54,59)+k(6);
jac(441,59)=jac(441,59)+k(6);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a, b~pY, g~Y) + Lyn(U, SH2) 

jac(61,61)=jac(61,61)-k(16);
jac(3,61)=jac(3,61)+k(16);
jac(72,61)=jac(72,61)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(7)*y(62);
jac(61,62)=jac(61,62)-k(7)*y(61);
jac(62,61)=jac(62,61)-k(7)*y(62);
jac(62,62)=jac(62,62)-k(7)*y(61);
jac(73,61)=jac(73,61)+k(7)*y(62);
jac(73,62)=jac(73,62)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(91,91)=jac(91,91)-k(7)*y(61);
jac(91,61)=jac(91,61)-k(7)*y(91);
jac(61,91)=jac(61,91)-k(7)*y(61);
jac(61,61)=jac(61,61)-k(7)*y(91);
jac(313,91)=jac(313,91)+k(7)*y(61);
jac(313,61)=jac(313,61)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(7)*y(126);
jac(61,126)=jac(61,126)-k(7)*y(61);
jac(126,61)=jac(126,61)-k(7)*y(126);
jac(126,126)=jac(126,126)-k(7)*y(61);
jac(125,61)=jac(125,61)+k(7)*y(126);
jac(125,126)=jac(125,126)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(61,61)=jac(61,61)-k(7)*y(132);
jac(61,132)=jac(61,132)-k(7)*y(61);
jac(132,61)=jac(132,61)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(61);
jac(130,61)=jac(130,61)+k(7)*y(132);
jac(130,132)=jac(130,132)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(61,61)=jac(61,61)-k(7)*y(141);
jac(61,141)=jac(61,141)-k(7)*y(61);
jac(141,61)=jac(141,61)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(61);
jac(129,61)=jac(129,61)+k(7)*y(141);
jac(129,141)=jac(129,141)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(61,61)=jac(61,61)-k(7)*y(185);
jac(61,185)=jac(61,185)-k(7)*y(61);
jac(185,61)=jac(185,61)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(61);
jac(327,61)=jac(327,61)+k(7)*y(185);
jac(327,185)=jac(327,185)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(61,61)=jac(61,61)-k(7)*y(209);
jac(61,209)=jac(61,209)-k(7)*y(61);
jac(209,61)=jac(209,61)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(61);
jac(322,61)=jac(322,61)+k(7)*y(209);
jac(322,209)=jac(322,209)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(61,61)=jac(61,61)-k(7)*y(214);
jac(61,214)=jac(61,214)-k(7)*y(61);
jac(214,61)=jac(214,61)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(61);
jac(326,61)=jac(326,61)+k(7)*y(214);
jac(326,214)=jac(326,214)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(61,61)=jac(61,61)-k(7)*y(219);
jac(61,219)=jac(61,219)-k(7)*y(61);
jac(219,61)=jac(219,61)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(61);
jac(323,61)=jac(323,61)+k(7)*y(219);
jac(323,219)=jac(323,219)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(7)*y(223);
jac(61,223)=jac(61,223)-k(7)*y(61);
jac(223,61)=jac(223,61)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(61);
jac(324,61)=jac(324,61)+k(7)*y(223);
jac(324,223)=jac(324,223)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(7)*y(228);
jac(61,228)=jac(61,228)-k(7)*y(61);
jac(228,61)=jac(228,61)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(61);
jac(320,61)=jac(320,61)+k(7)*y(228);
jac(320,228)=jac(320,228)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(315,315)=jac(315,315)-k(7)*y(61);
jac(315,61)=jac(315,61)-k(7)*y(315);
jac(61,315)=jac(61,315)-k(7)*y(61);
jac(61,61)=jac(61,61)-k(7)*y(315);
jac(314,315)=jac(314,315)+k(7)*y(61);
jac(314,61)=jac(314,61)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(61,61)=jac(61,61)-k(7)*y(319);
jac(61,319)=jac(61,319)-k(7)*y(61);
jac(319,61)=jac(319,61)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(61);
jac(318,61)=jac(318,61)+k(7)*y(319);
jac(318,319)=jac(318,319)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(61,61)=jac(61,61)-k(7)*y(336);
jac(61,336)=jac(61,336)-k(7)*y(61);
jac(336,61)=jac(336,61)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(61);
jac(200,61)=jac(200,61)+k(7)*y(336);
jac(200,336)=jac(200,336)+k(7)*y(61);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(61,61)=jac(61,61)-k(7)*y(437);
jac(61,437)=jac(61,437)-k(7)*y(61);
jac(437,61)=jac(437,61)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(61);
jac(316,61)=jac(316,61)+k(7)*y(437);
jac(316,437)=jac(316,437)+k(7)*y(61);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(71,71)=jac(71,71)-k(5)*y(61);
jac(71,61)=jac(71,61)-k(5)*y(71);
jac(61,71)=jac(61,71)-k(5)*y(61);
jac(61,61)=jac(61,61)-k(5)*y(71);
jac(70,71)=jac(70,71)+k(5)*y(61);
jac(70,61)=jac(70,61)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(76,76)=jac(76,76)-k(5)*y(61);
jac(76,61)=jac(76,61)-k(5)*y(76);
jac(61,76)=jac(61,76)-k(5)*y(61);
jac(61,61)=jac(61,61)-k(5)*y(76);
jac(75,76)=jac(75,76)+k(5)*y(61);
jac(75,61)=jac(75,61)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(61,61)=jac(61,61)-k(5)*y(82);
jac(61,82)=jac(61,82)-k(5)*y(61);
jac(82,61)=jac(82,61)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(61);
jac(81,61)=jac(81,61)+k(5)*y(82);
jac(81,82)=jac(81,82)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(61,61)=jac(61,61)-k(5)*y(86);
jac(61,86)=jac(61,86)-k(5)*y(61);
jac(86,61)=jac(86,61)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(61);
jac(85,61)=jac(85,61)+k(5)*y(86);
jac(85,86)=jac(85,86)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(61,61)=jac(61,61)-k(5)*y(89);
jac(61,89)=jac(61,89)-k(5)*y(61);
jac(89,61)=jac(89,61)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(61);
jac(88,61)=jac(88,61)+k(5)*y(89);
jac(88,89)=jac(88,89)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(98,98)=jac(98,98)-k(5)*y(61);
jac(98,61)=jac(98,61)-k(5)*y(98);
jac(61,98)=jac(61,98)-k(5)*y(61);
jac(61,61)=jac(61,61)-k(5)*y(98);
jac(63,98)=jac(63,98)+k(5)*y(61);
jac(63,61)=jac(63,61)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(61,61)=jac(61,61)-k(5)*y(110);
jac(61,110)=jac(61,110)-k(5)*y(61);
jac(110,61)=jac(110,61)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(61);
jac(80,61)=jac(80,61)+k(5)*y(110);
jac(80,110)=jac(80,110)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(61,61)=jac(61,61)-k(5)*y(114);
jac(61,114)=jac(61,114)-k(5)*y(61);
jac(114,61)=jac(114,61)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(61);
jac(307,61)=jac(307,61)+k(5)*y(114);
jac(307,114)=jac(307,114)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(5)*y(118);
jac(61,118)=jac(61,118)-k(5)*y(61);
jac(118,61)=jac(118,61)-k(5)*y(118);
jac(118,118)=jac(118,118)-k(5)*y(61);
jac(78,61)=jac(78,61)+k(5)*y(118);
jac(78,118)=jac(78,118)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(5)*y(122);
jac(61,122)=jac(61,122)-k(5)*y(61);
jac(122,61)=jac(122,61)-k(5)*y(122);
jac(122,122)=jac(122,122)-k(5)*y(61);
jac(73,61)=jac(73,61)+k(5)*y(122);
jac(73,122)=jac(73,122)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(61,61)=jac(61,61)-k(5)*y(176);
jac(61,176)=jac(61,176)-k(5)*y(61);
jac(176,61)=jac(176,61)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(61);
jac(297,61)=jac(297,61)+k(5)*y(176);
jac(297,176)=jac(297,176)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(61,61)=jac(61,61)-k(5)*y(191);
jac(61,191)=jac(61,191)-k(5)*y(61);
jac(191,61)=jac(191,61)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(61);
jac(295,61)=jac(295,61)+k(5)*y(191);
jac(295,191)=jac(295,191)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(61,61)=jac(61,61)-k(5)*y(256);
jac(61,256)=jac(61,256)-k(5)*y(61);
jac(256,61)=jac(256,61)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(61);
jac(203,61)=jac(203,61)+k(5)*y(256);
jac(203,256)=jac(203,256)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(61,61)=jac(61,61)-k(5)*y(264);
jac(61,264)=jac(61,264)-k(5)*y(61);
jac(264,61)=jac(264,61)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(61);
jac(300,61)=jac(300,61)+k(5)*y(264);
jac(300,264)=jac(300,264)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(61,61)=jac(61,61)-k(5)*y(380);
jac(61,380)=jac(61,380)-k(5)*y(61);
jac(380,61)=jac(380,61)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(61);
jac(65,61)=jac(65,61)+k(5)*y(380);
jac(65,380)=jac(65,380)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(61,61)=jac(61,61)-k(5)*y(391);
jac(61,391)=jac(61,391)-k(5)*y(61);
jac(391,61)=jac(391,61)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(61);
jac(298,61)=jac(298,61)+k(5)*y(391);
jac(298,391)=jac(298,391)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(61,61)=jac(61,61)-k(5)*y(407);
jac(61,407)=jac(61,407)-k(5)*y(61);
jac(407,61)=jac(407,61)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(61);
jac(204,61)=jac(204,61)+k(5)*y(407);
jac(204,407)=jac(204,407)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(61,61)=jac(61,61)-k(5)*y(441);
jac(61,441)=jac(61,441)-k(5)*y(61);
jac(441,61)=jac(441,61)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(61);
jac(66,61)=jac(66,61)+k(5)*y(441);
jac(66,441)=jac(66,441)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(5)*y(515);
jac(61,515)=jac(61,515)-k(5)*y(61);
jac(515,61)=jac(515,61)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(61);
jac(301,61)=jac(301,61)+k(5)*y(515);
jac(301,515)=jac(301,515)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(5)*y(543);
jac(61,543)=jac(61,543)-k(5)*y(61);
jac(543,61)=jac(543,61)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(61);
jac(647,61)=jac(647,61)+k(5)*y(543);
jac(647,543)=jac(647,543)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(61,61)=jac(61,61)-k(5)*y(554);
jac(61,554)=jac(61,554)-k(5)*y(61);
jac(554,61)=jac(554,61)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(61);
jac(83,61)=jac(83,61)+k(5)*y(554);
jac(83,554)=jac(83,554)+k(5)*y(61);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(61,61)=jac(61,61)-k(5)*y(9);
jac(61,9)=jac(61,9)-k(5)*y(61);
jac(9,61)=jac(9,61)-k(5)*y(9);
jac(9,9)=jac(9,9)-k(5)*y(61);
jac(67,61)=jac(67,61)+k(5)*y(9);
jac(67,9)=jac(67,9)+k(5)*y(61);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(4,4)=jac(4,4)-k(3)*y(61);
jac(4,61)=jac(4,61)-k(3)*y(4);
jac(61,4)=jac(61,4)-k(3)*y(61);
jac(61,61)=jac(61,61)-k(3)*y(4);
jac(122,4)=jac(122,4)+k(3)*y(61);
jac(122,61)=jac(122,61)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(4,4)=jac(4,4)-k(1)*y(61);
jac(4,61)=jac(4,61)-k(1)*y(4);
jac(61,4)=jac(61,4)-k(1)*y(61);
jac(61,61)=jac(61,61)-k(1)*y(4);
jac(62,4)=jac(62,4)+k(1)*y(61);
jac(62,61)=jac(62,61)+k(1)*y(4);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Lyn(U, SH2) 

jac(62,62)=jac(62,62)-k(16);
jac(3,62)=jac(3,62)+k(16);
jac(91,62)=jac(91,62)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(1,1)=jac(1,1)-k(7)*y(62);
jac(1,62)=jac(1,62)-k(7)*y(1);
jac(62,1)=jac(62,1)-k(7)*y(62);
jac(62,62)=jac(62,62)-k(7)*y(1);
jac(68,1)=jac(68,1)+k(7)*y(62);
jac(68,62)=jac(68,62)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(7)*y(62);
jac(7,62)=jac(7,62)-k(7)*y(7);
jac(62,7)=jac(62,7)-k(7)*y(62);
jac(62,62)=jac(62,62)-k(7)*y(7);
jac(67,7)=jac(67,7)+k(7)*y(62);
jac(67,62)=jac(67,62)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(62,62)=jac(62,62)-k(7)*y(335);
jac(62,335)=jac(62,335)-k(7)*y(62);
jac(335,62)=jac(335,62)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(62);
jac(204,62)=jac(204,62)+k(7)*y(335);
jac(204,335)=jac(204,335)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(62,62)=jac(62,62)-k(7)*y(257);
jac(62,257)=jac(62,257)-k(7)*y(62);
jac(257,62)=jac(257,62)-k(7)*y(257);
jac(257,257)=jac(257,257)-k(7)*y(62);
jac(203,62)=jac(203,62)+k(7)*y(257);
jac(203,257)=jac(203,257)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(62,62)=jac(62,62)-k(7)*y(227);
jac(62,227)=jac(62,227)-k(7)*y(62);
jac(227,62)=jac(227,62)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(62);
jac(647,62)=jac(647,62)+k(7)*y(227);
jac(647,227)=jac(647,227)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(62,62)=jac(62,62)-k(7)*y(222);
jac(62,222)=jac(62,222)-k(7)*y(62);
jac(222,62)=jac(222,62)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(62);
jac(301,62)=jac(301,62)+k(7)*y(222);
jac(301,222)=jac(301,222)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(62,62)=jac(62,62)-k(7)*y(218);
jac(62,218)=jac(62,218)-k(7)*y(62);
jac(218,62)=jac(218,62)-k(7)*y(218);
jac(218,218)=jac(218,218)-k(7)*y(62);
jac(300,62)=jac(300,62)+k(7)*y(218);
jac(300,218)=jac(300,218)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(62,62)=jac(62,62)-k(7)*y(192);
jac(62,192)=jac(62,192)-k(7)*y(62);
jac(192,62)=jac(192,62)-k(7)*y(192);
jac(192,192)=jac(192,192)-k(7)*y(62);
jac(295,62)=jac(295,62)+k(7)*y(192);
jac(295,192)=jac(295,192)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(62,62)=jac(62,62)-k(7)*y(184);
jac(62,184)=jac(62,184)-k(7)*y(62);
jac(184,62)=jac(184,62)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(62);
jac(298,62)=jac(298,62)+k(7)*y(184);
jac(298,184)=jac(298,184)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(62,62)=jac(62,62)-k(7)*y(177);
jac(62,177)=jac(62,177)-k(7)*y(62);
jac(177,62)=jac(177,62)-k(7)*y(177);
jac(177,177)=jac(177,177)-k(7)*y(62);
jac(297,62)=jac(297,62)+k(7)*y(177);
jac(297,177)=jac(297,177)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(62,62)=jac(62,62)-k(7)*y(140);
jac(62,140)=jac(62,140)-k(7)*y(62);
jac(140,62)=jac(140,62)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(62);
jac(65,62)=jac(65,62)+k(7)*y(140);
jac(65,140)=jac(65,140)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(62,62)=jac(62,62)-k(7)*y(131);
jac(62,131)=jac(62,131)-k(7)*y(62);
jac(131,62)=jac(131,62)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(62);
jac(66,62)=jac(66,62)+k(7)*y(131);
jac(66,131)=jac(66,131)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(62,62)=jac(62,62)-k(7)*y(111);
jac(62,111)=jac(62,111)-k(7)*y(62);
jac(111,62)=jac(111,62)-k(7)*y(111);
jac(111,111)=jac(111,111)-k(7)*y(62);
jac(80,62)=jac(80,62)+k(7)*y(111);
jac(80,111)=jac(80,111)+k(7)*y(62);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(77,77)=jac(77,77)-k(7)*y(62);
jac(77,62)=jac(77,62)-k(7)*y(77);
jac(62,77)=jac(62,77)-k(7)*y(62);
jac(62,62)=jac(62,62)-k(7)*y(77);
jac(75,77)=jac(75,77)+k(7)*y(62);
jac(75,62)=jac(75,62)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(72,72)=jac(72,72)-k(7)*y(62);
jac(72,62)=jac(72,62)-k(7)*y(72);
jac(62,72)=jac(62,72)-k(7)*y(62);
jac(62,62)=jac(62,62)-k(7)*y(72);
jac(70,72)=jac(70,72)+k(7)*y(62);
jac(70,62)=jac(70,62)+k(7)*y(72);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Lig(l1, l2) 

jac(62,62)=jac(62,62)-k(2);
jac(4,62)=jac(4,62)+k(2);
jac(61,62)=jac(61,62)+k(2);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(63,63)=jac(63,63)-k(32);
jac(68,63)=jac(68,63)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(63);
jac(182,63)=jac(182,63)-k(21)*y(182);
jac(63,182)=jac(63,182)-k(21)*y(63);
jac(63,63)=jac(63,63)-k(21)*y(182);
jac(297,182)=jac(297,182)+k(21)*y(63);
jac(297,63)=jac(297,63)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(63);
jac(197,63)=jac(197,63)-k(21)*y(197);
jac(63,197)=jac(63,197)-k(21)*y(63);
jac(63,63)=jac(63,63)-k(21)*y(197);
jac(295,197)=jac(295,197)+k(21)*y(63);
jac(295,63)=jac(295,63)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(63);
jac(32,63)=jac(32,63)-k(21)*y(32);
jac(63,32)=jac(63,32)-k(21)*y(63);
jac(63,63)=jac(63,63)-k(21)*y(32);
jac(85,32)=jac(85,32)+k(21)*y(63);
jac(85,63)=jac(85,63)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(63);
jac(2,63)=jac(2,63)-k(21)*y(2);
jac(63,2)=jac(63,2)-k(21)*y(63);
jac(63,63)=jac(63,63)-k(21)*y(2);
jac(88,2)=jac(88,2)+k(21)*y(63);
jac(88,63)=jac(88,63)+k(21)*y(2);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(63,63)=jac(63,63)-k(18);
jac(75,63)=jac(75,63)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

jac(63,63)=jac(63,63)-k(16);
jac(3,63)=jac(3,63)+k(16);
jac(93,63)=jac(93,63)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(63);
jac(3,63)=jac(3,63)-k(9)*y(3);
jac(63,3)=jac(63,3)-k(9)*y(63);
jac(63,63)=jac(63,63)-k(9)*y(3);
jac(60,3)=jac(60,3)+k(9)*y(63);
jac(60,63)=jac(60,63)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) 

jac(63,63)=jac(63,63)-k(8);
jac(15,63)=jac(15,63)+k(8);
jac(62,63)=jac(62,63)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(63,63)=jac(63,63)-k(6);
jac(98,63)=jac(98,63)+k(6);
jac(61,63)=jac(61,63)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(64,64)=jac(64,64)-k(32);
jac(13,64)=jac(13,64)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(64,64)=jac(64,64)-k(31);
jac(99,64)=jac(99,64)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(64);
jac(182,64)=jac(182,64)-k(21)*y(182);
jac(64,182)=jac(64,182)-k(21)*y(64);
jac(64,64)=jac(64,64)-k(21)*y(182);
jac(419,182)=jac(419,182)+k(21)*y(64);
jac(419,64)=jac(419,64)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(64);
jac(197,64)=jac(197,64)-k(21)*y(197);
jac(64,197)=jac(64,197)-k(21)*y(64);
jac(64,64)=jac(64,64)-k(21)*y(197);
jac(415,197)=jac(415,197)+k(21)*y(64);
jac(415,64)=jac(415,64)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(64);
jac(32,64)=jac(32,64)-k(21)*y(32);
jac(64,32)=jac(64,32)-k(21)*y(64);
jac(64,64)=jac(64,64)-k(21)*y(32);
jac(650,32)=jac(650,32)+k(21)*y(64);
jac(650,64)=jac(650,64)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(64);
jac(2,64)=jac(2,64)-k(21)*y(2);
jac(64,2)=jac(64,2)-k(21)*y(64);
jac(64,64)=jac(64,64)-k(21)*y(2);
jac(92,2)=jac(92,2)+k(21)*y(64);
jac(92,64)=jac(92,64)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(64);
jac(3,64)=jac(3,64)-k(15)*y(3);
jac(64,3)=jac(64,3)-k(15)*y(64);
jac(64,64)=jac(64,64)-k(15)*y(3);
jac(60,3)=jac(60,3)+k(15)*y(64);
jac(60,64)=jac(60,64)+k(15)*y(3);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(64,64)=jac(64,64)-k(13);
jac(57,64)=jac(57,64)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

jac(64,64)=jac(64,64)-k(10);
jac(3,64)=jac(3,64)+k(10);
jac(93,64)=jac(93,64)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(64,64)=jac(64,64)-k(8);
jac(91,64)=jac(91,64)+k(8);
jac(48,64)=jac(48,64)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) 

jac(64,64)=jac(64,64)-k(6);
jac(72,64)=jac(72,64)+k(6);
jac(47,64)=jac(47,64)+k(6);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(65,65)=jac(65,65)-k(26);
jac(66,65)=jac(66,65)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(65,65)=jac(65,65)-k(22);
jac(2,65)=jac(2,65)+k(22);
jac(60,65)=jac(60,65)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(65,65)=jac(65,65)-k(16);
jac(3,65)=jac(3,65)+k(16);
jac(92,65)=jac(92,65)+k(16);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(65,65)=jac(65,65)-k(13);
jac(58,65)=jac(58,65)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(65,65)=jac(65,65)-k(10);
jac(3,65)=jac(3,65)+k(10);
jac(88,65)=jac(88,65)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(65,65)=jac(65,65)-k(8);
jac(62,65)=jac(62,65)+k(8);
jac(140,65)=jac(140,65)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(65,65)=jac(65,65)-k(6);
jac(61,65)=jac(61,65)+k(6);
jac(380,65)=jac(380,65)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(66,66)=jac(66,66)-k(33);
jac(65,66)=jac(65,66)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(66,66)=jac(66,66)-k(22);
jac(32,66)=jac(32,66)+k(22);
jac(60,66)=jac(60,66)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(66,66)=jac(66,66)-k(16);
jac(3,66)=jac(3,66)+k(16);
jac(650,66)=jac(650,66)+k(16);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(66,66)=jac(66,66)-k(13);
jac(59,66)=jac(59,66)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(66,66)=jac(66,66)-k(10);
jac(3,66)=jac(3,66)+k(10);
jac(85,66)=jac(85,66)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(66,66)=jac(66,66)-k(8);
jac(62,66)=jac(62,66)+k(8);
jac(131,66)=jac(131,66)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(66,66)=jac(66,66)-k(6);
jac(61,66)=jac(61,66)+k(6);
jac(441,66)=jac(441,66)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(69,69)=jac(69,69)-k(31);
jac(12,69)=jac(12,69)+k(31);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(69);
jac(3,69)=jac(3,69)-k(15)*y(3);
jac(69,3)=jac(69,3)-k(15)*y(69);
jac(69,69)=jac(69,69)-k(15)*y(3);
jac(68,3)=jac(68,3)+k(15)*y(69);
jac(68,69)=jac(68,69)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(69);
jac(3,69)=jac(3,69)-k(9)*y(3);
jac(69,3)=jac(69,3)-k(9)*y(69);
jac(69,69)=jac(69,69)-k(9)*y(3);
jac(13,3)=jac(13,3)+k(9)*y(69);
jac(13,69)=jac(13,69)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) 

jac(69,69)=jac(69,69)-k(8);
jac(1,69)=jac(1,69)+k(8);
jac(91,69)=jac(91,69)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) 

jac(69,69)=jac(69,69)-k(6);
jac(72,69)=jac(72,69)+k(6);
jac(6,69)=jac(6,69)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(71,71)=jac(71,71)-k(31);
jac(6,71)=jac(6,71)+k(31);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(71);
jac(3,71)=jac(3,71)-k(15)*y(3);
jac(71,3)=jac(71,3)-k(15)*y(71);
jac(71,71)=jac(71,71)-k(15)*y(3);
jac(122,3)=jac(122,3)+k(15)*y(71);
jac(122,71)=jac(122,71)+k(15)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(1,1)=jac(1,1)-k(5)*y(71);
jac(1,71)=jac(1,71)-k(5)*y(1);
jac(71,1)=jac(71,1)-k(5)*y(71);
jac(71,71)=jac(71,71)-k(5)*y(1);
jac(333,1)=jac(333,1)+k(5)*y(71);
jac(333,71)=jac(333,71)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(71,71)=jac(71,71)-k(5)*y(7);
jac(71,7)=jac(71,7)-k(5)*y(71);
jac(7,71)=jac(7,71)-k(5)*y(7);
jac(7,7)=jac(7,7)-k(5)*y(71);
jac(124,71)=jac(124,71)+k(5)*y(7);
jac(124,7)=jac(124,7)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(71,71)=jac(71,71)-k(5)*y(335);
jac(71,335)=jac(71,335)-k(5)*y(71);
jac(335,71)=jac(335,71)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(71);
jac(334,71)=jac(334,71)+k(5)*y(335);
jac(334,335)=jac(334,335)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(71,71)=jac(71,71)-k(5)*y(257);
jac(71,257)=jac(71,257)-k(5)*y(71);
jac(257,71)=jac(257,71)-k(5)*y(257);
jac(257,257)=jac(257,257)-k(5)*y(71);
jac(321,71)=jac(321,71)+k(5)*y(257);
jac(321,257)=jac(321,257)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(71,71)=jac(71,71)-k(5)*y(227);
jac(71,227)=jac(71,227)-k(5)*y(71);
jac(227,71)=jac(227,71)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(71);
jac(231,71)=jac(231,71)+k(5)*y(227);
jac(231,227)=jac(231,227)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(71,71)=jac(71,71)-k(5)*y(222);
jac(71,222)=jac(71,222)-k(5)*y(71);
jac(222,71)=jac(222,71)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(71);
jac(646,71)=jac(646,71)+k(5)*y(222);
jac(646,222)=jac(646,222)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(71,71)=jac(71,71)-k(5)*y(218);
jac(71,218)=jac(71,218)-k(5)*y(71);
jac(218,71)=jac(218,71)-k(5)*y(218);
jac(218,218)=jac(218,218)-k(5)*y(71);
jac(325,71)=jac(325,71)+k(5)*y(218);
jac(325,218)=jac(325,218)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(71,71)=jac(71,71)-k(5)*y(192);
jac(71,192)=jac(71,192)-k(5)*y(71);
jac(192,71)=jac(192,71)-k(5)*y(192);
jac(192,192)=jac(192,192)-k(5)*y(71);
jac(332,71)=jac(332,71)+k(5)*y(192);
jac(332,192)=jac(332,192)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(71,71)=jac(71,71)-k(5)*y(184);
jac(71,184)=jac(71,184)-k(5)*y(71);
jac(184,71)=jac(184,71)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(71);
jac(330,71)=jac(330,71)+k(5)*y(184);
jac(330,184)=jac(330,184)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(71,71)=jac(71,71)-k(5)*y(177);
jac(71,177)=jac(71,177)-k(5)*y(71);
jac(177,71)=jac(177,71)-k(5)*y(177);
jac(177,177)=jac(177,177)-k(5)*y(71);
jac(328,71)=jac(328,71)+k(5)*y(177);
jac(328,177)=jac(328,177)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(71,71)=jac(71,71)-k(5)*y(140);
jac(71,140)=jac(71,140)-k(5)*y(71);
jac(140,71)=jac(140,71)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(71);
jac(139,71)=jac(139,71)+k(5)*y(140);
jac(139,140)=jac(139,140)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(71,71)=jac(71,71)-k(5)*y(131);
jac(71,131)=jac(71,131)-k(5)*y(71);
jac(131,71)=jac(131,71)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(71);
jac(135,71)=jac(135,71)+k(5)*y(131);
jac(135,131)=jac(135,131)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(71,71)=jac(71,71)-k(5)*y(111);
jac(71,111)=jac(71,111)-k(5)*y(71);
jac(111,71)=jac(111,71)-k(5)*y(111);
jac(111,111)=jac(111,111)-k(5)*y(71);
jac(653,71)=jac(653,71)+k(5)*y(111);
jac(653,111)=jac(653,111)+k(5)*y(71);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(77,77)=jac(77,77)-k(5)*y(71);
jac(77,71)=jac(77,71)-k(5)*y(77);
jac(71,77)=jac(71,77)-k(5)*y(71);
jac(71,71)=jac(71,71)-k(5)*y(77);
jac(237,77)=jac(237,77)+k(5)*y(71);
jac(237,71)=jac(237,71)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(72,72)=jac(72,72)-k(5)*y(71);
jac(72,71)=jac(72,71)-k(5)*y(72);
jac(71,72)=jac(71,72)-k(5)*y(71);
jac(71,71)=jac(71,71)-k(5)*y(72);
jac(74,72)=jac(74,72)+k(5)*y(71);
jac(74,71)=jac(74,71)+k(5)*y(72);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) -> Rec(a, b~pY, g~Y) + Lig(l1, l2) 

jac(71,71)=jac(71,71)-k(4);
jac(4,71)=jac(4,71)+k(4);
jac(72,71)=jac(72,71)+k(4);

% rule    : R14
% reaction: Rec(a, b~pY, g~Y) -> Rec(a, b~Y, g~Y) 

jac(72,72)=jac(72,72)-k(31);
jac(1,72)=jac(1,72)+k(31);

% rule    : R6
% reaction: Rec(a, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(3,3)=jac(3,3)-k(15)*y(72);
jac(3,72)=jac(3,72)-k(15)*y(3);
jac(72,3)=jac(72,3)-k(15)*y(72);
jac(72,72)=jac(72,72)-k(15)*y(3);
jac(61,3)=jac(61,3)+k(15)*y(72);
jac(61,72)=jac(61,72)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(72,72)=jac(72,72)-k(7)*y(91);
jac(72,91)=jac(72,91)-k(7)*y(72);
jac(91,72)=jac(91,72)-k(7)*y(91);
jac(91,91)=jac(91,91)-k(7)*y(72);
jac(74,72)=jac(74,72)+k(7)*y(91);
jac(74,91)=jac(74,91)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(72,72)=jac(72,72)-k(7)*y(126);
jac(72,126)=jac(72,126)-k(7)*y(72);
jac(126,72)=jac(126,72)-k(7)*y(126);
jac(126,126)=jac(126,126)-k(7)*y(72);
jac(128,72)=jac(128,72)+k(7)*y(126);
jac(128,126)=jac(128,126)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(72,72)=jac(72,72)-k(7)*y(132);
jac(72,132)=jac(72,132)-k(7)*y(72);
jac(132,72)=jac(132,72)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(72);
jac(135,72)=jac(135,72)+k(7)*y(132);
jac(135,132)=jac(135,132)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(72,72)=jac(72,72)-k(7)*y(141);
jac(72,141)=jac(72,141)-k(7)*y(72);
jac(141,72)=jac(141,72)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(72);
jac(139,72)=jac(139,72)+k(7)*y(141);
jac(139,141)=jac(139,141)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(72,72)=jac(72,72)-k(7)*y(185);
jac(72,185)=jac(72,185)-k(7)*y(72);
jac(185,72)=jac(185,72)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(72);
jac(330,72)=jac(330,72)+k(7)*y(185);
jac(330,185)=jac(330,185)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(72,72)=jac(72,72)-k(7)*y(209);
jac(72,209)=jac(72,209)-k(7)*y(72);
jac(209,72)=jac(209,72)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(72);
jac(332,72)=jac(332,72)+k(7)*y(209);
jac(332,209)=jac(332,209)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(72,72)=jac(72,72)-k(7)*y(214);
jac(72,214)=jac(72,214)-k(7)*y(72);
jac(214,72)=jac(214,72)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(72);
jac(328,72)=jac(328,72)+k(7)*y(214);
jac(328,214)=jac(328,214)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(72,72)=jac(72,72)-k(7)*y(219);
jac(72,219)=jac(72,219)-k(7)*y(72);
jac(219,72)=jac(219,72)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(72);
jac(325,72)=jac(325,72)+k(7)*y(219);
jac(325,219)=jac(325,219)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(72,72)=jac(72,72)-k(7)*y(223);
jac(72,223)=jac(72,223)-k(7)*y(72);
jac(223,72)=jac(223,72)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(72);
jac(646,72)=jac(646,72)+k(7)*y(223);
jac(646,223)=jac(646,223)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(72,72)=jac(72,72)-k(7)*y(228);
jac(72,228)=jac(72,228)-k(7)*y(72);
jac(228,72)=jac(228,72)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(72);
jac(231,72)=jac(231,72)+k(7)*y(228);
jac(231,228)=jac(231,228)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(72,72)=jac(72,72)-k(7)*y(315);
jac(72,315)=jac(72,315)-k(7)*y(72);
jac(315,72)=jac(315,72)-k(7)*y(315);
jac(315,315)=jac(315,315)-k(7)*y(72);
jac(237,72)=jac(237,72)+k(7)*y(315);
jac(237,315)=jac(237,315)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(72,72)=jac(72,72)-k(7)*y(319);
jac(72,319)=jac(72,319)-k(7)*y(72);
jac(319,72)=jac(319,72)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(72);
jac(321,72)=jac(321,72)+k(7)*y(319);
jac(321,319)=jac(321,319)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(72,72)=jac(72,72)-k(7)*y(336);
jac(72,336)=jac(72,336)-k(7)*y(72);
jac(336,72)=jac(336,72)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(72);
jac(334,72)=jac(334,72)+k(7)*y(336);
jac(334,336)=jac(334,336)+k(7)*y(72);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(72,72)=jac(72,72)-k(7)*y(437);
jac(72,437)=jac(72,437)-k(7)*y(72);
jac(437,72)=jac(437,72)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(72);
jac(653,72)=jac(653,72)+k(7)*y(437);
jac(653,437)=jac(653,437)+k(7)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(72,72)=jac(72,72)-k(5)*y(76);
jac(72,76)=jac(72,76)-k(5)*y(72);
jac(76,72)=jac(76,72)-k(5)*y(76);
jac(76,76)=jac(76,76)-k(5)*y(72);
jac(79,72)=jac(79,72)+k(5)*y(76);
jac(79,76)=jac(79,76)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(72,72)=jac(72,72)-k(5)*y(82);
jac(72,82)=jac(72,82)-k(5)*y(72);
jac(82,72)=jac(82,72)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(72);
jac(84,72)=jac(84,72)+k(5)*y(82);
jac(84,82)=jac(84,82)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(72,72)=jac(72,72)-k(5)*y(86);
jac(72,86)=jac(72,86)-k(5)*y(72);
jac(86,72)=jac(86,72)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(72);
jac(87,72)=jac(87,72)+k(5)*y(86);
jac(87,86)=jac(87,86)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(72,72)=jac(72,72)-k(5)*y(89);
jac(72,89)=jac(72,89)-k(5)*y(72);
jac(89,72)=jac(89,72)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(72);
jac(90,72)=jac(90,72)+k(5)*y(89);
jac(90,89)=jac(90,89)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(72,72)=jac(72,72)-k(5)*y(98);
jac(72,98)=jac(72,98)-k(5)*y(72);
jac(98,72)=jac(98,72)-k(5)*y(98);
jac(98,98)=jac(98,98)-k(5)*y(72);
jac(93,72)=jac(93,72)+k(5)*y(98);
jac(93,98)=jac(93,98)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(72,72)=jac(72,72)-k(5)*y(110);
jac(72,110)=jac(72,110)-k(5)*y(72);
jac(110,72)=jac(110,72)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(72);
jac(305,72)=jac(305,72)+k(5)*y(110);
jac(305,110)=jac(305,110)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(72,72)=jac(72,72)-k(5)*y(114);
jac(72,114)=jac(72,114)-k(5)*y(72);
jac(114,72)=jac(114,72)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(72);
jac(306,72)=jac(306,72)+k(5)*y(114);
jac(306,114)=jac(306,114)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(72,72)=jac(72,72)-k(5)*y(118);
jac(72,118)=jac(72,118)-k(5)*y(72);
jac(118,72)=jac(118,72)-k(5)*y(118);
jac(118,118)=jac(118,118)-k(5)*y(72);
jac(309,72)=jac(309,72)+k(5)*y(118);
jac(309,118)=jac(309,118)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(72,72)=jac(72,72)-k(5)*y(122);
jac(72,122)=jac(72,122)-k(5)*y(72);
jac(122,72)=jac(122,72)-k(5)*y(122);
jac(122,122)=jac(122,122)-k(5)*y(72);
jac(313,72)=jac(313,72)+k(5)*y(122);
jac(313,122)=jac(313,122)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(72,72)=jac(72,72)-k(5)*y(176);
jac(72,176)=jac(72,176)-k(5)*y(72);
jac(176,72)=jac(176,72)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(72);
jac(299,72)=jac(299,72)+k(5)*y(176);
jac(299,176)=jac(299,176)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(72,72)=jac(72,72)-k(5)*y(191);
jac(72,191)=jac(72,191)-k(5)*y(72);
jac(191,72)=jac(191,72)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(72);
jac(296,72)=jac(296,72)+k(5)*y(191);
jac(296,191)=jac(296,191)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(72,72)=jac(72,72)-k(5)*y(256);
jac(72,256)=jac(72,256)-k(5)*y(72);
jac(256,72)=jac(256,72)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(72);
jac(586,72)=jac(586,72)+k(5)*y(256);
jac(586,256)=jac(586,256)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(72,72)=jac(72,72)-k(5)*y(264);
jac(72,264)=jac(72,264)-k(5)*y(72);
jac(264,72)=jac(264,72)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(72);
jac(302,72)=jac(302,72)+k(5)*y(264);
jac(302,264)=jac(302,264)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(72,72)=jac(72,72)-k(5)*y(380);
jac(72,380)=jac(72,380)-k(5)*y(72);
jac(380,72)=jac(380,72)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(72);
jac(92,72)=jac(92,72)+k(5)*y(380);
jac(92,380)=jac(92,380)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(72,72)=jac(72,72)-k(5)*y(391);
jac(72,391)=jac(72,391)-k(5)*y(72);
jac(391,72)=jac(391,72)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(72);
jac(419,72)=jac(419,72)+k(5)*y(391);
jac(419,391)=jac(419,391)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(72,72)=jac(72,72)-k(5)*y(407);
jac(72,407)=jac(72,407)-k(5)*y(72);
jac(407,72)=jac(407,72)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(72);
jac(415,72)=jac(415,72)+k(5)*y(407);
jac(415,407)=jac(415,407)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(72,72)=jac(72,72)-k(5)*y(441);
jac(72,441)=jac(72,441)-k(5)*y(72);
jac(441,72)=jac(441,72)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(72);
jac(650,72)=jac(650,72)+k(5)*y(441);
jac(650,441)=jac(650,441)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(72,72)=jac(72,72)-k(5)*y(515);
jac(72,515)=jac(72,515)-k(5)*y(72);
jac(515,72)=jac(515,72)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(72);
jac(303,72)=jac(303,72)+k(5)*y(515);
jac(303,515)=jac(303,515)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(72,72)=jac(72,72)-k(5)*y(543);
jac(72,543)=jac(72,543)-k(5)*y(72);
jac(543,72)=jac(543,72)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(72);
jac(312,72)=jac(312,72)+k(5)*y(543);
jac(312,543)=jac(312,543)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(72,72)=jac(72,72)-k(5)*y(554);
jac(72,554)=jac(72,554)-k(5)*y(72);
jac(554,72)=jac(554,72)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(72);
jac(311,72)=jac(311,72)+k(5)*y(554);
jac(311,554)=jac(311,554)+k(5)*y(72);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(72,72)=jac(72,72)-k(5)*y(9);
jac(72,9)=jac(72,9)-k(5)*y(72);
jac(9,72)=jac(9,72)-k(5)*y(9);
jac(9,9)=jac(9,9)-k(5)*y(72);
jac(13,72)=jac(13,72)+k(5)*y(9);
jac(13,9)=jac(13,9)+k(5)*y(72);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(4,4)=jac(4,4)-k(3)*y(72);
jac(4,72)=jac(4,72)-k(3)*y(4);
jac(72,4)=jac(72,4)-k(3)*y(72);
jac(72,72)=jac(72,72)-k(3)*y(4);
jac(71,4)=jac(71,4)+k(3)*y(72);
jac(71,72)=jac(71,72)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY, g~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(4,4)=jac(4,4)-k(1)*y(72);
jac(4,72)=jac(4,72)-k(1)*y(4);
jac(72,4)=jac(72,4)-k(1)*y(72);
jac(72,72)=jac(72,72)-k(1)*y(4);
jac(91,4)=jac(91,4)+k(1)*y(72);
jac(91,72)=jac(91,72)+k(1)*y(4);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(73,73)=jac(73,73)-k(20);
jac(78,73)=jac(78,73)+k(20);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(73,73)=jac(73,73)-k(19);
jac(236,73)=jac(236,73)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) 

jac(73,73)=jac(73,73)-k(16);
jac(3,73)=jac(3,73)+k(16);
jac(70,73)=jac(70,73)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) 

jac(73,73)=jac(73,73)-k(16);
jac(3,73)=jac(3,73)+k(16);
jac(313,73)=jac(313,73)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(73,73)=jac(73,73)-k(8);
jac(61,73)=jac(61,73)+k(8);
jac(62,73)=jac(62,73)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(73,73)=jac(73,73)-k(6);
jac(61,73)=jac(61,73)+k(6);
jac(122,73)=jac(122,73)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(74,74)=jac(74,74)-k(31);
jac(69,74)=jac(69,74)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(74,74)=jac(74,74)-k(31);
jac(333,74)=jac(333,74)+k(31);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(74);
jac(3,74)=jac(3,74)-k(15)*y(3);
jac(74,3)=jac(74,3)-k(15)*y(74);
jac(74,74)=jac(74,74)-k(15)*y(3);
jac(70,3)=jac(70,3)+k(15)*y(74);
jac(70,74)=jac(70,74)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(3,3)=jac(3,3)-k(15)*y(74);
jac(3,74)=jac(3,74)-k(15)*y(3);
jac(74,3)=jac(74,3)-k(15)*y(74);
jac(74,74)=jac(74,74)-k(15)*y(3);
jac(313,3)=jac(313,3)+k(15)*y(74);
jac(313,74)=jac(313,74)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) 

jac(74,74)=jac(74,74)-k(8);
jac(72,74)=jac(72,74)+k(8);
jac(91,74)=jac(91,74)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) 

jac(74,74)=jac(74,74)-k(6);
jac(72,74)=jac(72,74)+k(6);
jac(71,74)=jac(71,74)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(76,76)=jac(76,76)-k(32);
jac(71,76)=jac(71,76)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(76,76)=jac(76,76)-k(31);
jac(98,76)=jac(98,76)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(76);
jac(182,76)=jac(182,76)-k(21)*y(182);
jac(76,182)=jac(76,182)-k(21)*y(76);
jac(76,76)=jac(76,76)-k(21)*y(182);
jac(264,182)=jac(264,182)+k(21)*y(76);
jac(264,76)=jac(264,76)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(76);
jac(197,76)=jac(197,76)-k(21)*y(197);
jac(76,197)=jac(76,197)-k(21)*y(76);
jac(76,76)=jac(76,76)-k(21)*y(197);
jac(256,197)=jac(256,197)+k(21)*y(76);
jac(256,76)=jac(256,76)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(76);
jac(32,76)=jac(32,76)-k(21)*y(32);
jac(76,32)=jac(76,32)-k(21)*y(76);
jac(76,76)=jac(76,76)-k(21)*y(32);
jac(82,32)=jac(82,32)+k(21)*y(76);
jac(82,76)=jac(82,76)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(76);
jac(2,76)=jac(2,76)-k(21)*y(2);
jac(76,2)=jac(76,2)-k(21)*y(76);
jac(76,76)=jac(76,76)-k(21)*y(2);
jac(110,2)=jac(110,2)+k(21)*y(76);
jac(110,76)=jac(110,76)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(76);
jac(3,76)=jac(3,76)-k(15)*y(3);
jac(76,3)=jac(76,3)-k(15)*y(76);
jac(76,76)=jac(76,76)-k(15)*y(3);
jac(118,3)=jac(118,3)+k(15)*y(76);
jac(118,76)=jac(118,76)+k(15)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(1,1)=jac(1,1)-k(5)*y(76);
jac(1,76)=jac(1,76)-k(5)*y(1);
jac(76,1)=jac(76,1)-k(5)*y(76);
jac(76,76)=jac(76,76)-k(5)*y(1);
jac(590,1)=jac(590,1)+k(5)*y(76);
jac(590,76)=jac(590,76)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(76,76)=jac(76,76)-k(5)*y(7);
jac(76,7)=jac(76,7)-k(5)*y(76);
jac(7,76)=jac(7,76)-k(5)*y(7);
jac(7,7)=jac(7,7)-k(5)*y(76);
jac(108,76)=jac(108,76)+k(5)*y(7);
jac(108,7)=jac(108,7)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(76,76)=jac(76,76)-k(5)*y(335);
jac(76,335)=jac(76,335)-k(5)*y(76);
jac(335,76)=jac(335,76)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(76);
jac(337,76)=jac(337,76)+k(5)*y(335);
jac(337,335)=jac(337,335)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(76,76)=jac(76,76)-k(5)*y(257);
jac(76,257)=jac(76,257)-k(5)*y(76);
jac(257,76)=jac(257,76)-k(5)*y(257);
jac(257,257)=jac(257,257)-k(5)*y(76);
jac(613,76)=jac(613,76)+k(5)*y(257);
jac(613,257)=jac(613,257)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(76,76)=jac(76,76)-k(5)*y(227);
jac(76,227)=jac(76,227)-k(5)*y(76);
jac(227,76)=jac(227,76)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(76);
jac(491,76)=jac(491,76)+k(5)*y(227);
jac(491,227)=jac(491,227)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(76,76)=jac(76,76)-k(5)*y(222);
jac(76,222)=jac(76,222)-k(5)*y(76);
jac(222,76)=jac(222,76)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(76);
jac(645,76)=jac(645,76)+k(5)*y(222);
jac(645,222)=jac(645,222)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(76,76)=jac(76,76)-k(5)*y(218);
jac(76,218)=jac(76,218)-k(5)*y(76);
jac(218,76)=jac(218,76)-k(5)*y(218);
jac(218,218)=jac(218,218)-k(5)*y(76);
jac(576,76)=jac(576,76)+k(5)*y(218);
jac(576,218)=jac(576,218)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) 

jac(76,76)=jac(76,76)-k(5)*y(192);
jac(76,192)=jac(76,192)-k(5)*y(76);
jac(192,76)=jac(192,76)-k(5)*y(192);
jac(192,192)=jac(192,192)-k(5)*y(76);
jac(338,76)=jac(338,76)+k(5)*y(192);
jac(338,192)=jac(338,192)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(76,76)=jac(76,76)-k(5)*y(184);
jac(76,184)=jac(76,184)-k(5)*y(76);
jac(184,76)=jac(184,76)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(76);
jac(343,76)=jac(343,76)+k(5)*y(184);
jac(343,184)=jac(343,184)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(76,76)=jac(76,76)-k(5)*y(177);
jac(76,177)=jac(76,177)-k(5)*y(76);
jac(177,76)=jac(177,76)-k(5)*y(177);
jac(177,177)=jac(177,177)-k(5)*y(76);
jac(344,76)=jac(344,76)+k(5)*y(177);
jac(344,177)=jac(344,177)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(76,76)=jac(76,76)-k(5)*y(140);
jac(76,140)=jac(76,140)-k(5)*y(76);
jac(140,76)=jac(140,76)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(76);
jac(143,76)=jac(143,76)+k(5)*y(140);
jac(143,140)=jac(143,140)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(76,76)=jac(76,76)-k(5)*y(131);
jac(76,131)=jac(76,131)-k(5)*y(76);
jac(131,76)=jac(131,76)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(76);
jac(137,76)=jac(137,76)+k(5)*y(131);
jac(137,131)=jac(137,131)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(76,76)=jac(76,76)-k(5)*y(111);
jac(76,111)=jac(76,111)-k(5)*y(76);
jac(111,76)=jac(111,76)-k(5)*y(111);
jac(111,111)=jac(111,111)-k(5)*y(76);
jac(252,76)=jac(252,76)+k(5)*y(111);
jac(252,111)=jac(252,111)+k(5)*y(76);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(77,77)=jac(77,77)-k(5)*y(76);
jac(77,76)=jac(77,76)-k(5)*y(77);
jac(76,77)=jac(76,77)-k(5)*y(76);
jac(76,76)=jac(76,76)-k(5)*y(77);
jac(584,77)=jac(584,77)+k(5)*y(76);
jac(584,76)=jac(584,76)+k(5)*y(77);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) -> Rec(a, b~pY, g~pY) + Lig(l1, l2) 

jac(76,76)=jac(76,76)-k(4);
jac(4,76)=jac(4,76)+k(4);
jac(77,76)=jac(77,76)+k(4);

% rule    : R15
% reaction: Rec(a, b~pY, g~pY) -> Rec(a, b~pY, g~Y) 

jac(77,77)=jac(77,77)-k(32);
jac(72,77)=jac(72,77)+k(32);

% rule    : R14
% reaction: Rec(a, b~pY, g~pY) -> Rec(a, b~Y, g~pY) 

jac(77,77)=jac(77,77)-k(31);
jac(15,77)=jac(15,77)+k(31);

% rule    : R9
% reaction: Rec(a, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(77);
jac(182,77)=jac(182,77)-k(21)*y(182);
jac(77,182)=jac(77,182)-k(21)*y(77);
jac(77,77)=jac(77,77)-k(21)*y(182);
jac(218,182)=jac(218,182)+k(21)*y(77);
jac(218,77)=jac(218,77)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(77);
jac(197,77)=jac(197,77)-k(21)*y(197);
jac(77,197)=jac(77,197)-k(21)*y(77);
jac(77,77)=jac(77,77)-k(21)*y(197);
jac(257,197)=jac(257,197)+k(21)*y(77);
jac(257,77)=jac(257,77)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(77);
jac(32,77)=jac(32,77)-k(21)*y(32);
jac(77,32)=jac(77,32)-k(21)*y(77);
jac(77,77)=jac(77,77)-k(21)*y(32);
jac(33,32)=jac(33,32)+k(21)*y(77);
jac(33,77)=jac(33,77)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(77);
jac(2,77)=jac(2,77)-k(21)*y(2);
jac(77,2)=jac(77,2)-k(21)*y(77);
jac(77,77)=jac(77,77)-k(21)*y(2);
jac(111,2)=jac(111,2)+k(21)*y(77);
jac(111,77)=jac(111,77)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(3,3)=jac(3,3)-k(15)*y(77);
jac(3,77)=jac(3,77)-k(15)*y(3);
jac(77,3)=jac(77,3)-k(15)*y(77);
jac(77,77)=jac(77,77)-k(15)*y(3);
jac(54,3)=jac(54,3)+k(15)*y(77);
jac(54,77)=jac(54,77)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(77,77)=jac(77,77)-k(7)*y(91);
jac(77,91)=jac(77,91)-k(7)*y(77);
jac(91,77)=jac(91,77)-k(7)*y(91);
jac(91,91)=jac(91,91)-k(7)*y(77);
jac(79,77)=jac(79,77)+k(7)*y(91);
jac(79,91)=jac(79,91)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(77,77)=jac(77,77)-k(7)*y(126);
jac(77,126)=jac(77,126)-k(7)*y(77);
jac(126,77)=jac(126,77)-k(7)*y(126);
jac(126,126)=jac(126,126)-k(7)*y(77);
jac(151,77)=jac(151,77)+k(7)*y(126);
jac(151,126)=jac(151,126)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(77,77)=jac(77,77)-k(7)*y(132);
jac(77,132)=jac(77,132)-k(7)*y(77);
jac(132,77)=jac(132,77)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(77);
jac(137,77)=jac(137,77)+k(7)*y(132);
jac(137,132)=jac(137,132)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(77,77)=jac(77,77)-k(7)*y(141);
jac(77,141)=jac(77,141)-k(7)*y(77);
jac(141,77)=jac(141,77)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(77);
jac(143,77)=jac(143,77)+k(7)*y(141);
jac(143,141)=jac(143,141)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(77,77)=jac(77,77)-k(7)*y(185);
jac(77,185)=jac(77,185)-k(7)*y(77);
jac(185,77)=jac(185,77)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(77);
jac(343,77)=jac(343,77)+k(7)*y(185);
jac(343,185)=jac(343,185)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) 

jac(77,77)=jac(77,77)-k(7)*y(209);
jac(77,209)=jac(77,209)-k(7)*y(77);
jac(209,77)=jac(209,77)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(77);
jac(338,77)=jac(338,77)+k(7)*y(209);
jac(338,209)=jac(338,209)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(77,77)=jac(77,77)-k(7)*y(214);
jac(77,214)=jac(77,214)-k(7)*y(77);
jac(214,77)=jac(214,77)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(77);
jac(344,77)=jac(344,77)+k(7)*y(214);
jac(344,214)=jac(344,214)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(77,77)=jac(77,77)-k(7)*y(219);
jac(77,219)=jac(77,219)-k(7)*y(77);
jac(219,77)=jac(219,77)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(77);
jac(576,77)=jac(576,77)+k(7)*y(219);
jac(576,219)=jac(576,219)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(77,77)=jac(77,77)-k(7)*y(223);
jac(77,223)=jac(77,223)-k(7)*y(77);
jac(223,77)=jac(223,77)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(77);
jac(645,77)=jac(645,77)+k(7)*y(223);
jac(645,223)=jac(645,223)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(77,77)=jac(77,77)-k(7)*y(228);
jac(77,228)=jac(77,228)-k(7)*y(77);
jac(228,77)=jac(228,77)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(77);
jac(491,77)=jac(491,77)+k(7)*y(228);
jac(491,228)=jac(491,228)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(77,77)=jac(77,77)-k(7)*y(315);
jac(77,315)=jac(77,315)-k(7)*y(77);
jac(315,77)=jac(315,77)-k(7)*y(315);
jac(315,315)=jac(315,315)-k(7)*y(77);
jac(584,77)=jac(584,77)+k(7)*y(315);
jac(584,315)=jac(584,315)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(77,77)=jac(77,77)-k(7)*y(319);
jac(77,319)=jac(77,319)-k(7)*y(77);
jac(319,77)=jac(319,77)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(77);
jac(613,77)=jac(613,77)+k(7)*y(319);
jac(613,319)=jac(613,319)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(77,77)=jac(77,77)-k(7)*y(336);
jac(77,336)=jac(77,336)-k(7)*y(77);
jac(336,77)=jac(336,77)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(77);
jac(337,77)=jac(337,77)+k(7)*y(336);
jac(337,336)=jac(337,336)+k(7)*y(77);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(77,77)=jac(77,77)-k(7)*y(437);
jac(77,437)=jac(77,437)-k(7)*y(77);
jac(437,77)=jac(437,77)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(77);
jac(252,77)=jac(252,77)+k(7)*y(437);
jac(252,437)=jac(252,437)+k(7)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(77,77)=jac(77,77)-k(5)*y(82);
jac(77,82)=jac(77,82)-k(5)*y(77);
jac(82,77)=jac(82,77)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(77);
jac(573,77)=jac(573,77)+k(5)*y(82);
jac(573,82)=jac(573,82)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(77,77)=jac(77,77)-k(5)*y(86);
jac(77,86)=jac(77,86)-k(5)*y(77);
jac(86,77)=jac(86,77)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(77);
jac(615,77)=jac(615,77)+k(5)*y(86);
jac(615,86)=jac(615,86)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(77,77)=jac(77,77)-k(5)*y(89);
jac(77,89)=jac(77,89)-k(5)*y(77);
jac(89,77)=jac(89,77)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(77);
jac(425,77)=jac(425,77)+k(5)*y(89);
jac(425,89)=jac(425,89)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(77,77)=jac(77,77)-k(5)*y(98);
jac(77,98)=jac(77,98)-k(5)*y(77);
jac(98,77)=jac(98,77)-k(5)*y(98);
jac(98,98)=jac(98,98)-k(5)*y(77);
jac(614,77)=jac(614,77)+k(5)*y(98);
jac(614,98)=jac(614,98)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(77,77)=jac(77,77)-k(5)*y(110);
jac(77,110)=jac(77,110)-k(5)*y(77);
jac(110,77)=jac(110,77)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(77);
jac(501,77)=jac(501,77)+k(5)*y(110);
jac(501,110)=jac(501,110)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(77,77)=jac(77,77)-k(5)*y(114);
jac(77,114)=jac(77,114)-k(5)*y(77);
jac(114,77)=jac(114,77)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(77);
jac(308,77)=jac(308,77)+k(5)*y(114);
jac(308,114)=jac(308,114)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(77,77)=jac(77,77)-k(5)*y(118);
jac(77,118)=jac(77,118)-k(5)*y(77);
jac(118,77)=jac(118,77)-k(5)*y(118);
jac(118,118)=jac(118,118)-k(5)*y(77);
jac(310,77)=jac(310,77)+k(5)*y(118);
jac(310,118)=jac(310,118)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(77,77)=jac(77,77)-k(5)*y(122);
jac(77,122)=jac(77,122)-k(5)*y(77);
jac(122,77)=jac(122,77)-k(5)*y(122);
jac(122,122)=jac(122,122)-k(5)*y(77);
jac(314,77)=jac(314,77)+k(5)*y(122);
jac(314,122)=jac(314,122)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(77,77)=jac(77,77)-k(5)*y(176);
jac(77,176)=jac(77,176)-k(5)*y(77);
jac(176,77)=jac(176,77)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(77);
jac(421,77)=jac(421,77)+k(5)*y(176);
jac(421,176)=jac(421,176)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(77,77)=jac(77,77)-k(5)*y(191);
jac(77,191)=jac(77,191)-k(5)*y(77);
jac(191,77)=jac(191,77)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(77);
jac(292,77)=jac(292,77)+k(5)*y(191);
jac(292,191)=jac(292,191)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(77,77)=jac(77,77)-k(5)*y(256);
jac(77,256)=jac(77,256)-k(5)*y(77);
jac(256,77)=jac(256,77)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(77);
jac(578,77)=jac(578,77)+k(5)*y(256);
jac(578,256)=jac(578,256)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(77,77)=jac(77,77)-k(5)*y(264);
jac(77,264)=jac(77,264)-k(5)*y(77);
jac(264,77)=jac(264,77)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(77);
jac(497,77)=jac(497,77)+k(5)*y(264);
jac(497,264)=jac(497,264)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(77,77)=jac(77,77)-k(5)*y(380);
jac(77,380)=jac(77,380)-k(5)*y(77);
jac(380,77)=jac(380,77)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(77);
jac(424,77)=jac(424,77)+k(5)*y(380);
jac(424,380)=jac(424,380)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(77,77)=jac(77,77)-k(5)*y(391);
jac(77,391)=jac(77,391)-k(5)*y(77);
jac(391,77)=jac(391,77)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(77);
jac(420,77)=jac(420,77)+k(5)*y(391);
jac(420,391)=jac(420,391)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(77,77)=jac(77,77)-k(5)*y(407);
jac(77,407)=jac(77,407)-k(5)*y(77);
jac(407,77)=jac(407,77)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(77);
jac(410,77)=jac(410,77)+k(5)*y(407);
jac(410,407)=jac(410,407)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(77,77)=jac(77,77)-k(5)*y(441);
jac(77,441)=jac(77,441)-k(5)*y(77);
jac(441,77)=jac(441,77)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(77);
jac(443,77)=jac(443,77)+k(5)*y(441);
jac(443,441)=jac(443,441)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(77,77)=jac(77,77)-k(5)*y(515);
jac(77,515)=jac(77,515)-k(5)*y(77);
jac(515,77)=jac(515,77)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(77);
jac(623,77)=jac(623,77)+k(5)*y(515);
jac(623,515)=jac(623,515)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(77,77)=jac(77,77)-k(5)*y(543);
jac(77,543)=jac(77,543)-k(5)*y(77);
jac(543,77)=jac(543,77)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(77);
jac(581,77)=jac(581,77)+k(5)*y(543);
jac(581,543)=jac(581,543)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(77,77)=jac(77,77)-k(5)*y(554);
jac(77,554)=jac(77,554)-k(5)*y(77);
jac(554,77)=jac(554,77)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(77);
jac(563,77)=jac(563,77)+k(5)*y(554);
jac(563,554)=jac(563,554)+k(5)*y(77);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(77,77)=jac(77,77)-k(5)*y(9);
jac(77,9)=jac(77,9)-k(5)*y(77);
jac(9,77)=jac(9,77)-k(5)*y(9);
jac(9,9)=jac(9,9)-k(5)*y(77);
jac(19,77)=jac(19,77)+k(5)*y(9);
jac(19,9)=jac(19,9)+k(5)*y(77);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(4,4)=jac(4,4)-k(3)*y(77);
jac(4,77)=jac(4,77)-k(3)*y(4);
jac(77,4)=jac(77,4)-k(3)*y(77);
jac(77,77)=jac(77,77)-k(3)*y(4);
jac(76,4)=jac(76,4)+k(3)*y(77);
jac(76,77)=jac(76,77)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY, g~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(4,4)=jac(4,4)-k(1)*y(77);
jac(4,77)=jac(4,77)-k(1)*y(4);
jac(77,4)=jac(77,4)-k(1)*y(77);
jac(77,77)=jac(77,77)-k(1)*y(4);
jac(315,4)=jac(315,4)+k(1)*y(77);
jac(315,77)=jac(315,77)+k(1)*y(4);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(78,78)=jac(78,78)-k(32);
jac(73,78)=jac(73,78)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(182,182)=jac(182,182)-k(21)*y(78);
jac(182,78)=jac(182,78)-k(21)*y(182);
jac(78,182)=jac(78,182)-k(21)*y(78);
jac(78,78)=jac(78,78)-k(21)*y(182);
jac(301,182)=jac(301,182)+k(21)*y(78);
jac(301,78)=jac(301,78)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(197,197)=jac(197,197)-k(21)*y(78);
jac(197,78)=jac(197,78)-k(21)*y(197);
jac(78,197)=jac(78,197)-k(21)*y(78);
jac(78,78)=jac(78,78)-k(21)*y(197);
jac(647,197)=jac(647,197)+k(21)*y(78);
jac(647,78)=jac(647,78)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(32,32)=jac(32,32)-k(21)*y(78);
jac(32,78)=jac(32,78)-k(21)*y(32);
jac(78,32)=jac(78,32)-k(21)*y(78);
jac(78,78)=jac(78,78)-k(21)*y(32);
jac(83,32)=jac(83,32)+k(21)*y(78);
jac(83,78)=jac(83,78)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(78);
jac(2,78)=jac(2,78)-k(21)*y(2);
jac(78,2)=jac(78,2)-k(21)*y(78);
jac(78,78)=jac(78,78)-k(21)*y(2);
jac(307,2)=jac(307,2)+k(21)*y(78);
jac(307,78)=jac(307,78)+k(21)*y(2);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(78,78)=jac(78,78)-k(19);
jac(627,78)=jac(627,78)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) 

jac(78,78)=jac(78,78)-k(16);
jac(3,78)=jac(3,78)+k(16);
jac(75,78)=jac(75,78)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) 

jac(78,78)=jac(78,78)-k(16);
jac(3,78)=jac(3,78)+k(16);
jac(309,78)=jac(309,78)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(78,78)=jac(78,78)-k(8);
jac(54,78)=jac(54,78)+k(8);
jac(62,78)=jac(62,78)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(78,78)=jac(78,78)-k(6);
jac(61,78)=jac(61,78)+k(6);
jac(118,78)=jac(118,78)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(79,79)=jac(79,79)-k(32);
jac(74,79)=jac(74,79)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(79,79)=jac(79,79)-k(31);
jac(93,79)=jac(93,79)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(79,79)=jac(79,79)-k(31);
jac(590,79)=jac(590,79)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(79);
jac(182,79)=jac(182,79)-k(21)*y(182);
jac(79,182)=jac(79,182)-k(21)*y(79);
jac(79,79)=jac(79,79)-k(21)*y(182);
jac(302,182)=jac(302,182)+k(21)*y(79);
jac(302,79)=jac(302,79)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(79);
jac(197,79)=jac(197,79)-k(21)*y(197);
jac(79,197)=jac(79,197)-k(21)*y(79);
jac(79,79)=jac(79,79)-k(21)*y(197);
jac(586,197)=jac(586,197)+k(21)*y(79);
jac(586,79)=jac(586,79)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(79);
jac(32,79)=jac(32,79)-k(21)*y(32);
jac(79,32)=jac(79,32)-k(21)*y(79);
jac(79,79)=jac(79,79)-k(21)*y(32);
jac(84,32)=jac(84,32)+k(21)*y(79);
jac(84,79)=jac(84,79)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(79);
jac(2,79)=jac(2,79)-k(21)*y(2);
jac(79,2)=jac(79,2)-k(21)*y(79);
jac(79,79)=jac(79,79)-k(21)*y(2);
jac(305,2)=jac(305,2)+k(21)*y(79);
jac(305,79)=jac(305,79)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(79);
jac(3,79)=jac(3,79)-k(15)*y(3);
jac(79,3)=jac(79,3)-k(15)*y(79);
jac(79,79)=jac(79,79)-k(15)*y(3);
jac(75,3)=jac(75,3)+k(15)*y(79);
jac(75,79)=jac(75,79)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(3,3)=jac(3,3)-k(15)*y(79);
jac(3,79)=jac(3,79)-k(15)*y(3);
jac(79,3)=jac(79,3)-k(15)*y(79);
jac(79,79)=jac(79,79)-k(15)*y(3);
jac(309,3)=jac(309,3)+k(15)*y(79);
jac(309,79)=jac(309,79)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) 

jac(79,79)=jac(79,79)-k(8);
jac(77,79)=jac(77,79)+k(8);
jac(91,79)=jac(91,79)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) 

jac(79,79)=jac(79,79)-k(6);
jac(72,79)=jac(72,79)+k(6);
jac(76,79)=jac(76,79)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(80,80)=jac(80,80)-k(31);
jac(88,80)=jac(88,80)+k(31);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(80,80)=jac(80,80)-k(26);
jac(81,80)=jac(81,80)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

jac(80,80)=jac(80,80)-k(22);
jac(2,80)=jac(2,80)+k(22);
jac(75,80)=jac(75,80)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(80,80)=jac(80,80)-k(16);
jac(3,80)=jac(3,80)+k(16);
jac(305,80)=jac(305,80)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(80);
jac(3,80)=jac(3,80)-k(15)*y(3);
jac(80,3)=jac(80,3)-k(15)*y(80);
jac(80,80)=jac(80,80)-k(15)*y(3);
jac(307,3)=jac(307,3)+k(15)*y(80);
jac(307,80)=jac(307,80)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(80,80)=jac(80,80)-k(8);
jac(62,80)=jac(62,80)+k(8);
jac(111,80)=jac(111,80)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(80,80)=jac(80,80)-k(6);
jac(61,80)=jac(61,80)+k(6);
jac(110,80)=jac(110,80)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(82,82)=jac(82,82)-k(33);
jac(110,82)=jac(110,82)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(82,82)=jac(82,82)-k(31);
jac(86,82)=jac(86,82)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~Y) 

jac(82,82)=jac(82,82)-k(22);
jac(32,82)=jac(32,82)+k(22);
jac(76,82)=jac(76,82)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(82);
jac(3,82)=jac(3,82)-k(15)*y(3);
jac(82,3)=jac(82,3)-k(15)*y(82);
jac(82,82)=jac(82,82)-k(15)*y(3);
jac(554,3)=jac(554,3)+k(15)*y(82);
jac(554,82)=jac(554,82)+k(15)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(1,1)=jac(1,1)-k(5)*y(82);
jac(1,82)=jac(1,82)-k(5)*y(1);
jac(82,1)=jac(82,1)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(1);
jac(106,1)=jac(106,1)+k(5)*y(82);
jac(106,82)=jac(106,82)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(7,7)=jac(7,7)-k(5)*y(82);
jac(7,82)=jac(7,82)-k(5)*y(7);
jac(82,7)=jac(82,7)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(7);
jac(105,7)=jac(105,7)+k(5)*y(82);
jac(105,82)=jac(105,82)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(82,82)=jac(82,82)-k(5)*y(335);
jac(82,335)=jac(82,335)-k(5)*y(82);
jac(335,82)=jac(335,82)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(82);
jac(341,82)=jac(341,82)+k(5)*y(335);
jac(341,335)=jac(341,335)+k(5)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(257,257)=jac(257,257)-k(5)*y(82);
jac(257,82)=jac(257,82)-k(5)*y(257);
jac(82,257)=jac(82,257)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(257);
jac(562,257)=jac(562,257)+k(5)*y(82);
jac(562,82)=jac(562,82)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(82,82)=jac(82,82)-k(5)*y(227);
jac(82,227)=jac(82,227)-k(5)*y(82);
jac(227,82)=jac(227,82)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(82);
jac(635,82)=jac(635,82)+k(5)*y(227);
jac(635,227)=jac(635,227)+k(5)*y(82);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(82,82)=jac(82,82)-k(5)*y(222);
jac(82,222)=jac(82,222)-k(5)*y(82);
jac(222,82)=jac(222,82)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(82);
jac(286,82)=jac(286,82)+k(5)*y(222);
jac(286,222)=jac(286,222)+k(5)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(218,218)=jac(218,218)-k(5)*y(82);
jac(218,82)=jac(218,82)-k(5)*y(218);
jac(82,218)=jac(82,218)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(218);
jac(574,218)=jac(574,218)+k(5)*y(82);
jac(574,82)=jac(574,82)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(192,192)=jac(192,192)-k(5)*y(82);
jac(192,82)=jac(192,82)-k(5)*y(192);
jac(82,192)=jac(82,192)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(192);
jac(559,192)=jac(559,192)+k(5)*y(82);
jac(559,82)=jac(559,82)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(82,82)=jac(82,82)-k(5)*y(184);
jac(82,184)=jac(82,184)-k(5)*y(82);
jac(184,82)=jac(184,82)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(82);
jac(347,82)=jac(347,82)+k(5)*y(184);
jac(347,184)=jac(347,184)+k(5)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(177,177)=jac(177,177)-k(5)*y(82);
jac(177,82)=jac(177,82)-k(5)*y(177);
jac(82,177)=jac(82,177)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(177);
jac(348,177)=jac(348,177)+k(5)*y(82);
jac(348,82)=jac(348,82)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(82,82)=jac(82,82)-k(5)*y(140);
jac(82,140)=jac(82,140)-k(5)*y(82);
jac(140,82)=jac(140,82)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(82);
jac(167,82)=jac(167,82)+k(5)*y(140);
jac(167,140)=jac(167,140)+k(5)*y(82);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(82,82)=jac(82,82)-k(5)*y(131);
jac(82,131)=jac(82,131)-k(5)*y(82);
jac(131,82)=jac(131,82)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(82);
jac(161,82)=jac(161,82)+k(5)*y(131);
jac(161,131)=jac(161,131)+k(5)*y(82);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(111,111)=jac(111,111)-k(5)*y(82);
jac(111,82)=jac(111,82)-k(5)*y(111);
jac(82,111)=jac(82,111)-k(5)*y(82);
jac(82,82)=jac(82,82)-k(5)*y(111);
jac(572,111)=jac(572,111)+k(5)*y(82);
jac(572,82)=jac(572,82)+k(5)*y(111);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) 

jac(82,82)=jac(82,82)-k(4);
jac(4,82)=jac(4,82)+k(4);
jac(33,82)=jac(33,82)+k(4);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(83,83)=jac(83,83)-k(33);
jac(307,83)=jac(307,83)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(83,83)=jac(83,83)-k(22);
jac(32,83)=jac(32,83)+k(22);
jac(78,83)=jac(78,83)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(83,83)=jac(83,83)-k(19);
jac(633,83)=jac(633,83)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(83,83)=jac(83,83)-k(16);
jac(3,83)=jac(3,83)+k(16);
jac(81,83)=jac(81,83)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(83,83)=jac(83,83)-k(16);
jac(3,83)=jac(3,83)+k(16);
jac(311,83)=jac(311,83)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(83,83)=jac(83,83)-k(8);
jac(62,83)=jac(62,83)+k(8);
jac(37,83)=jac(37,83)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(83,83)=jac(83,83)-k(6);
jac(61,83)=jac(61,83)+k(6);
jac(554,83)=jac(554,83)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(84,84)=jac(84,84)-k(33);
jac(305,84)=jac(305,84)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(84,84)=jac(84,84)-k(31);
jac(87,84)=jac(87,84)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(84,84)=jac(84,84)-k(31);
jac(106,84)=jac(106,84)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(84,84)=jac(84,84)-k(22);
jac(32,84)=jac(32,84)+k(22);
jac(79,84)=jac(79,84)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(84);
jac(3,84)=jac(3,84)-k(15)*y(3);
jac(84,3)=jac(84,3)-k(15)*y(84);
jac(84,84)=jac(84,84)-k(15)*y(3);
jac(81,3)=jac(81,3)+k(15)*y(84);
jac(81,84)=jac(81,84)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(84);
jac(3,84)=jac(3,84)-k(15)*y(3);
jac(84,3)=jac(84,3)-k(15)*y(84);
jac(84,84)=jac(84,84)-k(15)*y(3);
jac(311,3)=jac(311,3)+k(15)*y(84);
jac(311,84)=jac(311,84)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(84,84)=jac(84,84)-k(8);
jac(91,84)=jac(91,84)+k(8);
jac(33,84)=jac(33,84)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

jac(84,84)=jac(84,84)-k(6);
jac(72,84)=jac(72,84)+k(6);
jac(82,84)=jac(82,84)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(86,86)=jac(86,86)-k(33);
jac(89,86)=jac(89,86)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~Y) 

jac(86,86)=jac(86,86)-k(22);
jac(32,86)=jac(32,86)+k(22);
jac(98,86)=jac(98,86)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(86);
jac(3,86)=jac(3,86)-k(9)*y(3);
jac(86,3)=jac(86,3)-k(9)*y(86);
jac(86,86)=jac(86,86)-k(9)*y(3);
jac(441,3)=jac(441,3)+k(9)*y(86);
jac(441,86)=jac(441,86)+k(9)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(1,1)=jac(1,1)-k(5)*y(86);
jac(1,86)=jac(1,86)-k(5)*y(1);
jac(86,1)=jac(86,1)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(1);
jac(101,1)=jac(101,1)+k(5)*y(86);
jac(101,86)=jac(101,86)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(7,7)=jac(7,7)-k(5)*y(86);
jac(7,86)=jac(7,86)-k(5)*y(7);
jac(86,7)=jac(86,7)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(7);
jac(103,7)=jac(103,7)+k(5)*y(86);
jac(103,86)=jac(103,86)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(86,86)=jac(86,86)-k(5)*y(335);
jac(86,335)=jac(86,335)-k(5)*y(86);
jac(335,86)=jac(335,86)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(86);
jac(162,86)=jac(162,86)+k(5)*y(335);
jac(162,335)=jac(162,335)+k(5)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(257,257)=jac(257,257)-k(5)*y(86);
jac(257,86)=jac(257,86)-k(5)*y(257);
jac(86,257)=jac(86,257)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(257);
jac(472,257)=jac(472,257)+k(5)*y(86);
jac(472,86)=jac(472,86)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(86,86)=jac(86,86)-k(5)*y(227);
jac(86,227)=jac(86,227)-k(5)*y(86);
jac(227,86)=jac(227,86)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(86);
jac(474,86)=jac(474,86)+k(5)*y(227);
jac(474,227)=jac(474,227)+k(5)*y(86);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(86,86)=jac(86,86)-k(5)*y(222);
jac(86,222)=jac(86,222)-k(5)*y(86);
jac(222,86)=jac(222,86)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(86);
jac(283,86)=jac(283,86)+k(5)*y(222);
jac(283,222)=jac(283,222)+k(5)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(218,218)=jac(218,218)-k(5)*y(86);
jac(218,86)=jac(218,86)-k(5)*y(218);
jac(86,218)=jac(86,218)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(218);
jac(285,218)=jac(285,218)+k(5)*y(86);
jac(285,86)=jac(285,86)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(5)*y(86);
jac(192,86)=jac(192,86)-k(5)*y(192);
jac(86,192)=jac(86,192)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(192);
jac(454,192)=jac(454,192)+k(5)*y(86);
jac(454,86)=jac(454,86)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(86,86)=jac(86,86)-k(5)*y(184);
jac(86,184)=jac(86,184)-k(5)*y(86);
jac(184,86)=jac(184,86)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(86);
jac(168,86)=jac(168,86)+k(5)*y(184);
jac(168,184)=jac(168,184)+k(5)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(177,177)=jac(177,177)-k(5)*y(86);
jac(177,86)=jac(177,86)-k(5)*y(177);
jac(86,177)=jac(86,177)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(177);
jac(449,177)=jac(449,177)+k(5)*y(86);
jac(449,86)=jac(449,86)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(86,86)=jac(86,86)-k(5)*y(140);
jac(86,140)=jac(86,140)-k(5)*y(86);
jac(140,86)=jac(140,86)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(86);
jac(164,86)=jac(164,86)+k(5)*y(140);
jac(164,140)=jac(164,140)+k(5)*y(86);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(86,86)=jac(86,86)-k(5)*y(131);
jac(86,131)=jac(86,131)-k(5)*y(86);
jac(131,86)=jac(131,86)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(86);
jac(158,86)=jac(158,86)+k(5)*y(131);
jac(158,131)=jac(158,131)+k(5)*y(86);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(111,111)=jac(111,111)-k(5)*y(86);
jac(111,86)=jac(111,86)-k(5)*y(111);
jac(86,111)=jac(86,111)-k(5)*y(86);
jac(86,86)=jac(86,86)-k(5)*y(111);
jac(442,111)=jac(442,111)+k(5)*y(86);
jac(442,86)=jac(442,86)+k(5)*y(111);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lig(l1, l2) 

jac(86,86)=jac(86,86)-k(4);
jac(4,86)=jac(4,86)+k(4);
jac(27,86)=jac(27,86)+k(4);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(87,87)=jac(87,87)-k(33);
jac(90,87)=jac(90,87)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(87,87)=jac(87,87)-k(31);
jac(101,87)=jac(101,87)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(87,87)=jac(87,87)-k(22);
jac(32,87)=jac(32,87)+k(22);
jac(93,87)=jac(93,87)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(87);
jac(3,87)=jac(3,87)-k(15)*y(3);
jac(87,3)=jac(87,3)-k(15)*y(87);
jac(87,87)=jac(87,87)-k(15)*y(3);
jac(85,3)=jac(85,3)+k(15)*y(87);
jac(85,87)=jac(85,87)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(87);
jac(3,87)=jac(3,87)-k(9)*y(3);
jac(87,3)=jac(87,3)-k(9)*y(87);
jac(87,87)=jac(87,87)-k(9)*y(3);
jac(650,3)=jac(650,3)+k(9)*y(87);
jac(650,87)=jac(650,87)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(87,87)=jac(87,87)-k(8);
jac(91,87)=jac(91,87)+k(8);
jac(27,87)=jac(27,87)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

jac(87,87)=jac(87,87)-k(6);
jac(72,87)=jac(72,87)+k(6);
jac(86,87)=jac(86,87)+k(6);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~Y) 

jac(89,89)=jac(89,89)-k(22);
jac(2,89)=jac(2,89)+k(22);
jac(98,89)=jac(98,89)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(89);
jac(3,89)=jac(3,89)-k(9)*y(3);
jac(89,3)=jac(89,3)-k(9)*y(89);
jac(89,89)=jac(89,89)-k(9)*y(3);
jac(380,3)=jac(380,3)+k(9)*y(89);
jac(380,89)=jac(380,89)+k(9)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(1,1)=jac(1,1)-k(5)*y(89);
jac(1,89)=jac(1,89)-k(5)*y(1);
jac(89,1)=jac(89,1)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(1);
jac(94,1)=jac(94,1)+k(5)*y(89);
jac(94,89)=jac(94,89)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(7,7)=jac(7,7)-k(5)*y(89);
jac(7,89)=jac(7,89)-k(5)*y(7);
jac(89,7)=jac(89,7)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(7);
jac(96,7)=jac(96,7)+k(5)*y(89);
jac(96,89)=jac(96,89)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(89,89)=jac(89,89)-k(5)*y(335);
jac(89,335)=jac(89,335)-k(5)*y(89);
jac(335,89)=jac(335,89)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(89);
jac(371,89)=jac(371,89)+k(5)*y(335);
jac(371,335)=jac(371,335)+k(5)*y(89);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(257,257)=jac(257,257)-k(5)*y(89);
jac(257,89)=jac(257,89)-k(5)*y(257);
jac(89,257)=jac(89,257)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(257);
jac(427,257)=jac(427,257)+k(5)*y(89);
jac(427,89)=jac(427,89)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(89,89)=jac(89,89)-k(5)*y(227);
jac(89,227)=jac(89,227)-k(5)*y(89);
jac(227,89)=jac(227,89)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(89);
jac(484,89)=jac(484,89)+k(5)*y(227);
jac(484,227)=jac(484,227)+k(5)*y(89);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(89,89)=jac(89,89)-k(5)*y(222);
jac(89,222)=jac(89,222)-k(5)*y(89);
jac(222,89)=jac(222,89)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(89);
jac(278,89)=jac(278,89)+k(5)*y(222);
jac(278,222)=jac(278,222)+k(5)*y(89);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(218,218)=jac(218,218)-k(5)*y(89);
jac(218,89)=jac(218,89)-k(5)*y(218);
jac(89,218)=jac(89,218)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(218);
jac(280,218)=jac(280,218)+k(5)*y(89);
jac(280,89)=jac(280,89)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(192,192)=jac(192,192)-k(5)*y(89);
jac(192,89)=jac(192,89)-k(5)*y(192);
jac(89,192)=jac(89,192)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(192);
jac(373,192)=jac(373,192)+k(5)*y(89);
jac(373,89)=jac(373,89)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(89,89)=jac(89,89)-k(5)*y(184);
jac(89,184)=jac(89,184)-k(5)*y(89);
jac(184,89)=jac(184,89)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(89);
jac(174,89)=jac(174,89)+k(5)*y(184);
jac(174,184)=jac(174,184)+k(5)*y(89);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(177,177)=jac(177,177)-k(5)*y(89);
jac(177,89)=jac(177,89)-k(5)*y(177);
jac(89,177)=jac(89,177)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(177);
jac(377,177)=jac(377,177)+k(5)*y(89);
jac(377,89)=jac(377,89)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(89,89)=jac(89,89)-k(5)*y(140);
jac(89,140)=jac(89,140)-k(5)*y(89);
jac(140,89)=jac(140,89)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(89);
jac(170,89)=jac(170,89)+k(5)*y(140);
jac(170,140)=jac(170,140)+k(5)*y(89);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(89,89)=jac(89,89)-k(5)*y(131);
jac(89,131)=jac(89,131)-k(5)*y(89);
jac(131,89)=jac(131,89)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(89);
jac(157,89)=jac(157,89)+k(5)*y(131);
jac(157,131)=jac(157,131)+k(5)*y(89);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(5)*y(89);
jac(111,89)=jac(111,89)-k(5)*y(111);
jac(89,111)=jac(89,111)-k(5)*y(89);
jac(89,89)=jac(89,89)-k(5)*y(111);
jac(275,111)=jac(275,111)+k(5)*y(89);
jac(275,89)=jac(275,89)+k(5)*y(111);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) 

jac(89,89)=jac(89,89)-k(4);
jac(4,89)=jac(4,89)+k(4);
jac(21,89)=jac(21,89)+k(4);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(91,91)=jac(91,91)-k(31);
jac(5,91)=jac(5,91)+k(31);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(91);
jac(3,91)=jac(3,91)-k(15)*y(3);
jac(91,3)=jac(91,3)-k(15)*y(91);
jac(91,91)=jac(91,91)-k(15)*y(3);
jac(62,3)=jac(62,3)+k(15)*y(91);
jac(62,91)=jac(62,91)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(1,1)=jac(1,1)-k(7)*y(91);
jac(1,91)=jac(1,91)-k(7)*y(1);
jac(91,1)=jac(91,1)-k(7)*y(91);
jac(91,91)=jac(91,91)-k(7)*y(1);
jac(69,1)=jac(69,1)+k(7)*y(91);
jac(69,91)=jac(69,91)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(91,91)=jac(91,91)-k(7)*y(7);
jac(91,7)=jac(91,7)-k(7)*y(91);
jac(7,91)=jac(7,91)-k(7)*y(7);
jac(7,7)=jac(7,7)-k(7)*y(91);
jac(13,91)=jac(13,91)+k(7)*y(7);
jac(13,7)=jac(13,7)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(91,91)=jac(91,91)-k(7)*y(335);
jac(91,335)=jac(91,335)-k(7)*y(91);
jac(335,91)=jac(335,91)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(91);
jac(415,91)=jac(415,91)+k(7)*y(335);
jac(415,335)=jac(415,335)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(91,91)=jac(91,91)-k(7)*y(257);
jac(91,257)=jac(91,257)-k(7)*y(91);
jac(257,91)=jac(257,91)-k(7)*y(257);
jac(257,257)=jac(257,257)-k(7)*y(91);
jac(586,91)=jac(586,91)+k(7)*y(257);
jac(586,257)=jac(586,257)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(91,91)=jac(91,91)-k(7)*y(227);
jac(91,227)=jac(91,227)-k(7)*y(91);
jac(227,91)=jac(227,91)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(91);
jac(312,91)=jac(312,91)+k(7)*y(227);
jac(312,227)=jac(312,227)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(91,91)=jac(91,91)-k(7)*y(222);
jac(91,222)=jac(91,222)-k(7)*y(91);
jac(222,91)=jac(222,91)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(91);
jac(303,91)=jac(303,91)+k(7)*y(222);
jac(303,222)=jac(303,222)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(91,91)=jac(91,91)-k(7)*y(218);
jac(91,218)=jac(91,218)-k(7)*y(91);
jac(218,91)=jac(218,91)-k(7)*y(218);
jac(218,218)=jac(218,218)-k(7)*y(91);
jac(302,91)=jac(302,91)+k(7)*y(218);
jac(302,218)=jac(302,218)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(91,91)=jac(91,91)-k(7)*y(192);
jac(91,192)=jac(91,192)-k(7)*y(91);
jac(192,91)=jac(192,91)-k(7)*y(192);
jac(192,192)=jac(192,192)-k(7)*y(91);
jac(296,91)=jac(296,91)+k(7)*y(192);
jac(296,192)=jac(296,192)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(91,91)=jac(91,91)-k(7)*y(184);
jac(91,184)=jac(91,184)-k(7)*y(91);
jac(184,91)=jac(184,91)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(91);
jac(419,91)=jac(419,91)+k(7)*y(184);
jac(419,184)=jac(419,184)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(91,91)=jac(91,91)-k(7)*y(177);
jac(91,177)=jac(91,177)-k(7)*y(91);
jac(177,91)=jac(177,91)-k(7)*y(177);
jac(177,177)=jac(177,177)-k(7)*y(91);
jac(299,91)=jac(299,91)+k(7)*y(177);
jac(299,177)=jac(299,177)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(91,91)=jac(91,91)-k(7)*y(140);
jac(91,140)=jac(91,140)-k(7)*y(91);
jac(140,91)=jac(140,91)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(91);
jac(92,91)=jac(92,91)+k(7)*y(140);
jac(92,140)=jac(92,140)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(91,91)=jac(91,91)-k(7)*y(131);
jac(91,131)=jac(91,131)-k(7)*y(91);
jac(131,91)=jac(131,91)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(91);
jac(650,91)=jac(650,91)+k(7)*y(131);
jac(650,131)=jac(650,131)+k(7)*y(91);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(91,91)=jac(91,91)-k(7)*y(111);
jac(91,111)=jac(91,111)-k(7)*y(91);
jac(111,91)=jac(111,91)-k(7)*y(111);
jac(111,111)=jac(111,111)-k(7)*y(91);
jac(305,91)=jac(305,91)+k(7)*y(111);
jac(305,111)=jac(305,111)+k(7)*y(91);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) -> Rec(a, b~pY, g~Y) + Lig(l1, l2) 

jac(91,91)=jac(91,91)-k(2);
jac(4,91)=jac(4,91)+k(2);
jac(72,91)=jac(72,91)+k(2);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(92,92)=jac(92,92)-k(31);
jac(95,92)=jac(95,92)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(92,92)=jac(92,92)-k(22);
jac(2,92)=jac(2,92)+k(22);
jac(64,92)=jac(64,92)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(92);
jac(3,92)=jac(3,92)-k(15)*y(3);
jac(92,3)=jac(92,3)-k(15)*y(92);
jac(92,92)=jac(92,92)-k(15)*y(3);
jac(65,3)=jac(65,3)+k(15)*y(92);
jac(65,92)=jac(65,92)+k(15)*y(3);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(92,92)=jac(92,92)-k(13);
jac(424,92)=jac(424,92)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(92,92)=jac(92,92)-k(10);
jac(3,92)=jac(3,92)+k(10);
jac(90,92)=jac(90,92)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(92,92)=jac(92,92)-k(8);
jac(91,92)=jac(91,92)+k(8);
jac(140,92)=jac(140,92)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

jac(92,92)=jac(92,92)-k(6);
jac(72,92)=jac(72,92)+k(6);
jac(380,92)=jac(380,92)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(93,93)=jac(93,93)-k(32);
jac(69,93)=jac(69,93)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(93,93)=jac(93,93)-k(31);
jac(97,93)=jac(97,93)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(93);
jac(182,93)=jac(182,93)-k(21)*y(182);
jac(93,182)=jac(93,182)-k(21)*y(93);
jac(93,93)=jac(93,93)-k(21)*y(182);
jac(299,182)=jac(299,182)+k(21)*y(93);
jac(299,93)=jac(299,93)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(93);
jac(197,93)=jac(197,93)-k(21)*y(197);
jac(93,197)=jac(93,197)-k(21)*y(93);
jac(93,93)=jac(93,93)-k(21)*y(197);
jac(296,197)=jac(296,197)+k(21)*y(93);
jac(296,93)=jac(296,93)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(93);
jac(32,93)=jac(32,93)-k(21)*y(32);
jac(93,32)=jac(93,32)-k(21)*y(93);
jac(93,93)=jac(93,93)-k(21)*y(32);
jac(87,32)=jac(87,32)+k(21)*y(93);
jac(87,93)=jac(87,93)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(93);
jac(2,93)=jac(2,93)-k(21)*y(2);
jac(93,2)=jac(93,2)-k(21)*y(93);
jac(93,93)=jac(93,93)-k(21)*y(2);
jac(90,2)=jac(90,2)+k(21)*y(93);
jac(90,93)=jac(90,93)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(93);
jac(3,93)=jac(3,93)-k(15)*y(3);
jac(93,3)=jac(93,3)-k(15)*y(93);
jac(93,93)=jac(93,93)-k(15)*y(3);
jac(63,3)=jac(63,3)+k(15)*y(93);
jac(63,93)=jac(63,93)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(93);
jac(3,93)=jac(3,93)-k(9)*y(3);
jac(93,3)=jac(93,3)-k(9)*y(93);
jac(93,93)=jac(93,93)-k(9)*y(3);
jac(64,3)=jac(64,3)+k(9)*y(93);
jac(64,93)=jac(64,93)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) 

jac(93,93)=jac(93,93)-k(8);
jac(15,93)=jac(15,93)+k(8);
jac(91,93)=jac(91,93)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~Y) 

jac(93,93)=jac(93,93)-k(6);
jac(72,93)=jac(72,93)+k(6);
jac(98,93)=jac(98,93)+k(6);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(95,95)=jac(95,95)-k(22);
jac(2,95)=jac(2,95)+k(22);
jac(99,95)=jac(99,95)+k(22);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(95,95)=jac(95,95)-k(13);
jac(383,95)=jac(383,95)+k(13);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(95,95)=jac(95,95)-k(11);
jac(92,95)=jac(92,95)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(95,95)=jac(95,95)-k(10);
jac(3,95)=jac(3,95)+k(10);
jac(94,95)=jac(94,95)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(95);
jac(3,95)=jac(3,95)-k(9)*y(3);
jac(95,3)=jac(95,3)-k(9)*y(95);
jac(95,95)=jac(95,95)-k(9)*y(3);
jac(462,3)=jac(462,3)+k(9)*y(95);
jac(462,95)=jac(462,95)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(95,95)=jac(95,95)-k(8);
jac(5,95)=jac(5,95)+k(8);
jac(140,95)=jac(140,95)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

jac(95,95)=jac(95,95)-k(6);
jac(1,95)=jac(1,95)+k(6);
jac(380,95)=jac(380,95)+k(6);

% rule    : R10_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(96,96)=jac(96,96)-k(24);
jac(103,96)=jac(103,96)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(96,96)=jac(96,96)-k(22);
jac(2,96)=jac(2,96)+k(22);
jac(100,96)=jac(100,96)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(96,96)=jac(96,96)-k(12);
jac(109,96)=jac(109,96)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(96,96)=jac(96,96)-k(10);
jac(3,96)=jac(3,96)+k(10);
jac(94,96)=jac(94,96)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(96);
jac(3,96)=jac(3,96)-k(9)*y(3);
jac(96,3)=jac(96,3)-k(9)*y(96);
jac(96,96)=jac(96,96)-k(9)*y(3);
jac(462,3)=jac(462,3)+k(9)*y(96);
jac(462,96)=jac(462,96)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(96,96)=jac(96,96)-k(8);
jac(8,96)=jac(8,96)+k(8);
jac(21,96)=jac(21,96)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(96,96)=jac(96,96)-k(6);
jac(7,96)=jac(7,96)+k(6);
jac(89,96)=jac(89,96)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(98,98)=jac(98,98)-k(32);
jac(6,98)=jac(6,98)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(98);
jac(182,98)=jac(182,98)-k(21)*y(182);
jac(98,182)=jac(98,182)-k(21)*y(98);
jac(98,98)=jac(98,98)-k(21)*y(182);
jac(176,182)=jac(176,182)+k(21)*y(98);
jac(176,98)=jac(176,98)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(98);
jac(197,98)=jac(197,98)-k(21)*y(197);
jac(98,197)=jac(98,197)-k(21)*y(98);
jac(98,98)=jac(98,98)-k(21)*y(197);
jac(191,197)=jac(191,197)+k(21)*y(98);
jac(191,98)=jac(191,98)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(98);
jac(32,98)=jac(32,98)-k(21)*y(32);
jac(98,32)=jac(98,32)-k(21)*y(98);
jac(98,98)=jac(98,98)-k(21)*y(32);
jac(86,32)=jac(86,32)+k(21)*y(98);
jac(86,98)=jac(86,98)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(98);
jac(2,98)=jac(2,98)-k(21)*y(2);
jac(98,2)=jac(98,2)-k(21)*y(98);
jac(98,98)=jac(98,98)-k(21)*y(2);
jac(89,2)=jac(89,2)+k(21)*y(98);
jac(89,98)=jac(89,98)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(98);
jac(3,98)=jac(3,98)-k(9)*y(3);
jac(98,3)=jac(98,3)-k(9)*y(98);
jac(98,98)=jac(98,98)-k(9)*y(3);
jac(47,3)=jac(47,3)+k(9)*y(98);
jac(47,98)=jac(47,98)+k(9)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(1,1)=jac(1,1)-k(5)*y(98);
jac(1,98)=jac(1,98)-k(5)*y(1);
jac(98,1)=jac(98,1)-k(5)*y(98);
jac(98,98)=jac(98,98)-k(5)*y(1);
jac(97,1)=jac(97,1)+k(5)*y(98);
jac(97,98)=jac(97,98)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(98,98)=jac(98,98)-k(5)*y(7);
jac(98,7)=jac(98,7)-k(5)*y(98);
jac(7,98)=jac(7,98)-k(5)*y(7);
jac(7,7)=jac(7,7)-k(5)*y(98);
jac(100,98)=jac(100,98)+k(5)*y(7);
jac(100,7)=jac(100,7)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(98,98)=jac(98,98)-k(5)*y(335);
jac(98,335)=jac(98,335)-k(5)*y(98);
jac(335,98)=jac(335,98)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(98);
jac(198,98)=jac(198,98)+k(5)*y(335);
jac(198,335)=jac(198,335)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(98,98)=jac(98,98)-k(5)*y(257);
jac(98,257)=jac(98,257)-k(5)*y(98);
jac(257,98)=jac(257,98)-k(5)*y(257);
jac(257,257)=jac(257,257)-k(5)*y(98);
jac(618,98)=jac(618,98)+k(5)*y(257);
jac(618,257)=jac(618,257)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(98,98)=jac(98,98)-k(5)*y(227);
jac(98,227)=jac(98,227)-k(5)*y(98);
jac(227,98)=jac(227,98)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(98);
jac(232,98)=jac(232,98)+k(5)*y(227);
jac(232,227)=jac(232,227)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(98,98)=jac(98,98)-k(5)*y(222);
jac(98,222)=jac(98,222)-k(5)*y(98);
jac(222,98)=jac(222,98)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(98);
jac(282,98)=jac(282,98)+k(5)*y(222);
jac(282,222)=jac(282,222)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(98,98)=jac(98,98)-k(5)*y(218);
jac(98,218)=jac(98,218)-k(5)*y(98);
jac(218,98)=jac(218,98)-k(5)*y(218);
jac(218,218)=jac(218,218)-k(5)*y(98);
jac(431,98)=jac(431,98)+k(5)*y(218);
jac(431,218)=jac(431,218)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) 

jac(98,98)=jac(98,98)-k(5)*y(192);
jac(98,192)=jac(98,192)-k(5)*y(98);
jac(192,98)=jac(192,98)-k(5)*y(192);
jac(192,192)=jac(192,192)-k(5)*y(98);
jac(376,98)=jac(376,98)+k(5)*y(192);
jac(376,192)=jac(376,192)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(98,98)=jac(98,98)-k(5)*y(184);
jac(98,184)=jac(98,184)-k(5)*y(98);
jac(184,98)=jac(184,98)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(98);
jac(189,98)=jac(189,98)+k(5)*y(184);
jac(189,184)=jac(189,184)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(98,98)=jac(98,98)-k(5)*y(177);
jac(98,177)=jac(98,177)-k(5)*y(98);
jac(177,98)=jac(177,98)-k(5)*y(177);
jac(177,177)=jac(177,177)-k(5)*y(98);
jac(451,98)=jac(451,98)+k(5)*y(177);
jac(451,177)=jac(451,177)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(98,98)=jac(98,98)-k(5)*y(140);
jac(98,140)=jac(98,140)-k(5)*y(98);
jac(140,98)=jac(140,98)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(98);
jac(145,98)=jac(145,98)+k(5)*y(140);
jac(145,140)=jac(145,140)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(98,98)=jac(98,98)-k(5)*y(131);
jac(98,131)=jac(98,131)-k(5)*y(98);
jac(131,98)=jac(131,98)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(98);
jac(154,98)=jac(154,98)+k(5)*y(131);
jac(154,131)=jac(154,131)+k(5)*y(98);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(98,98)=jac(98,98)-k(5)*y(111);
jac(98,111)=jac(98,111)-k(5)*y(98);
jac(111,98)=jac(111,98)-k(5)*y(111);
jac(111,111)=jac(111,111)-k(5)*y(98);
jac(616,98)=jac(616,98)+k(5)*y(111);
jac(616,111)=jac(616,111)+k(5)*y(98);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) -> Rec(a, b~Y, g~pY) + Lig(l1, l2) 

jac(98,98)=jac(98,98)-k(4);
jac(4,98)=jac(4,98)+k(4);
jac(15,98)=jac(15,98)+k(4);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(99,99)=jac(99,99)-k(32);
jac(11,99)=jac(11,99)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(99);
jac(182,99)=jac(182,99)-k(21)*y(182);
jac(99,182)=jac(99,182)-k(21)*y(99);
jac(99,99)=jac(99,99)-k(21)*y(182);
jac(416,182)=jac(416,182)+k(21)*y(99);
jac(416,99)=jac(416,99)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(99);
jac(197,99)=jac(197,99)-k(21)*y(197);
jac(99,197)=jac(99,197)-k(21)*y(99);
jac(99,99)=jac(99,99)-k(21)*y(197);
jac(413,197)=jac(413,197)+k(21)*y(99);
jac(413,99)=jac(413,99)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(99);
jac(32,99)=jac(32,99)-k(21)*y(32);
jac(99,32)=jac(99,32)-k(21)*y(99);
jac(99,99)=jac(99,99)-k(21)*y(32);
jac(102,32)=jac(102,32)+k(21)*y(99);
jac(102,99)=jac(102,99)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(99);
jac(2,99)=jac(2,99)-k(21)*y(2);
jac(99,2)=jac(99,2)-k(21)*y(99);
jac(99,99)=jac(99,99)-k(21)*y(2);
jac(95,2)=jac(95,2)+k(21)*y(99);
jac(95,99)=jac(95,99)+k(21)*y(2);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(99,99)=jac(99,99)-k(13);
jac(404,99)=jac(404,99)+k(13);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(99,99)=jac(99,99)-k(11);
jac(64,99)=jac(64,99)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

jac(99,99)=jac(99,99)-k(10);
jac(3,99)=jac(3,99)+k(10);
jac(97,99)=jac(97,99)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(99);
jac(3,99)=jac(3,99)-k(9)*y(3);
jac(99,3)=jac(99,3)-k(9)*y(99);
jac(99,99)=jac(99,99)-k(9)*y(3);
jac(463,3)=jac(463,3)+k(9)*y(99);
jac(463,99)=jac(463,99)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(99,99)=jac(99,99)-k(8);
jac(5,99)=jac(5,99)+k(8);
jac(48,99)=jac(48,99)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) 

jac(99,99)=jac(99,99)-k(6);
jac(1,99)=jac(1,99)+k(6);
jac(47,99)=jac(47,99)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(100,100)=jac(100,100)-k(32);
jac(148,100)=jac(148,100)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(100);
jac(182,100)=jac(182,100)-k(21)*y(182);
jac(100,182)=jac(100,182)-k(21)*y(100);
jac(100,100)=jac(100,100)-k(21)*y(182);
jac(606,182)=jac(606,182)+k(21)*y(100);
jac(606,100)=jac(606,100)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(100);
jac(197,100)=jac(197,100)-k(21)*y(197);
jac(100,197)=jac(100,197)-k(21)*y(100);
jac(100,100)=jac(100,100)-k(21)*y(197);
jac(607,197)=jac(607,197)+k(21)*y(100);
jac(607,100)=jac(607,100)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(100);
jac(32,100)=jac(32,100)-k(21)*y(32);
jac(100,32)=jac(100,32)-k(21)*y(100);
jac(100,100)=jac(100,100)-k(21)*y(32);
jac(103,32)=jac(103,32)+k(21)*y(100);
jac(103,100)=jac(103,100)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(100);
jac(2,100)=jac(2,100)-k(21)*y(2);
jac(100,2)=jac(100,2)-k(21)*y(100);
jac(100,100)=jac(100,100)-k(21)*y(2);
jac(96,2)=jac(96,2)+k(21)*y(100);
jac(96,100)=jac(96,100)+k(21)*y(2);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(100,100)=jac(100,100)-k(12);
jac(108,100)=jac(108,100)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

jac(100,100)=jac(100,100)-k(10);
jac(3,100)=jac(3,100)+k(10);
jac(97,100)=jac(97,100)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(100);
jac(3,100)=jac(3,100)-k(9)*y(3);
jac(100,3)=jac(100,3)-k(9)*y(100);
jac(100,100)=jac(100,100)-k(9)*y(3);
jac(463,3)=jac(463,3)+k(9)*y(100);
jac(463,100)=jac(463,100)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) 

jac(100,100)=jac(100,100)-k(8);
jac(15,100)=jac(15,100)+k(8);
jac(8,100)=jac(8,100)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(100,100)=jac(100,100)-k(6);
jac(98,100)=jac(98,100)+k(6);
jac(7,100)=jac(7,100)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(102,102)=jac(102,102)-k(33);
jac(95,102)=jac(95,102)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(102,102)=jac(102,102)-k(22);
jac(32,102)=jac(32,102)+k(22);
jac(99,102)=jac(99,102)+k(22);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(102,102)=jac(102,102)-k(13);
jac(405,102)=jac(405,102)+k(13);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(102,102)=jac(102,102)-k(11);
jac(650,102)=jac(650,102)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(102,102)=jac(102,102)-k(10);
jac(3,102)=jac(3,102)+k(10);
jac(101,102)=jac(101,102)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(102);
jac(3,102)=jac(3,102)-k(9)*y(3);
jac(102,3)=jac(102,3)-k(9)*y(102);
jac(102,102)=jac(102,102)-k(9)*y(3);
jac(104,3)=jac(104,3)+k(9)*y(102);
jac(104,102)=jac(104,102)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(102,102)=jac(102,102)-k(8);
jac(5,102)=jac(5,102)+k(8);
jac(131,102)=jac(131,102)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

jac(102,102)=jac(102,102)-k(6);
jac(1,102)=jac(1,102)+k(6);
jac(441,102)=jac(441,102)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(104,104)=jac(104,104)-k(33);
jac(462,104)=jac(462,104)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(104,104)=jac(104,104)-k(22);
jac(32,104)=jac(32,104)+k(22);
jac(463,104)=jac(463,104)+k(22);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(104,104)=jac(104,104)-k(13);
jac(458,104)=jac(458,104)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(104,104)=jac(104,104)-k(10);
jac(3,104)=jac(3,104)+k(10);
jac(103,104)=jac(103,104)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(104,104)=jac(104,104)-k(10);
jac(3,104)=jac(3,104)+k(10);
jac(102,104)=jac(102,104)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(104,104)=jac(104,104)-k(8);
jac(8,104)=jac(8,104)+k(8);
jac(131,104)=jac(131,104)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(104,104)=jac(104,104)-k(6);
jac(7,104)=jac(7,104)+k(6);
jac(441,104)=jac(441,104)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(106,106)=jac(106,106)-k(33);
jac(112,106)=jac(112,106)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(106,106)=jac(106,106)-k(31);
jac(101,106)=jac(101,106)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(106,106)=jac(106,106)-k(22);
jac(32,106)=jac(32,106)+k(22);
jac(590,106)=jac(590,106)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(106);
jac(3,106)=jac(3,106)-k(15)*y(3);
jac(106,3)=jac(106,3)-k(15)*y(106);
jac(106,106)=jac(106,106)-k(15)*y(3);
jac(592,3)=jac(592,3)+k(15)*y(106);
jac(592,106)=jac(592,106)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(106);
jac(3,106)=jac(3,106)-k(9)*y(3);
jac(106,3)=jac(106,3)-k(9)*y(106);
jac(106,106)=jac(106,106)-k(9)*y(3);
jac(105,3)=jac(105,3)+k(9)*y(106);
jac(105,106)=jac(105,106)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(106,106)=jac(106,106)-k(8);
jac(5,106)=jac(5,106)+k(8);
jac(33,106)=jac(33,106)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

jac(106,106)=jac(106,106)-k(6);
jac(1,106)=jac(1,106)+k(6);
jac(82,106)=jac(82,106)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(107,107)=jac(107,107)-k(33);
jac(113,107)=jac(113,107)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(107,107)=jac(107,107)-k(22);
jac(32,107)=jac(32,107)+k(22);
jac(117,107)=jac(117,107)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(107,107)=jac(107,107)-k(19);
jac(352,107)=jac(352,107)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(107,107)=jac(107,107)-k(16);
jac(3,107)=jac(3,107)+k(16);
jac(105,107)=jac(105,107)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(107,107)=jac(107,107)-k(10);
jac(3,107)=jac(3,107)+k(10);
jac(592,107)=jac(592,107)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(107,107)=jac(107,107)-k(8);
jac(8,107)=jac(8,107)+k(8);
jac(37,107)=jac(37,107)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(107,107)=jac(107,107)-k(6);
jac(7,107)=jac(7,107)+k(6);
jac(554,107)=jac(554,107)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(108,108)=jac(108,108)-k(32);
jac(124,108)=jac(124,108)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(108,108)=jac(108,108)-k(31);
jac(100,108)=jac(100,108)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(108);
jac(182,108)=jac(182,108)-k(21)*y(182);
jac(108,182)=jac(108,182)-k(21)*y(108);
jac(108,108)=jac(108,108)-k(21)*y(182);
jac(594,182)=jac(594,182)+k(21)*y(108);
jac(594,108)=jac(594,108)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(108);
jac(197,108)=jac(197,108)-k(21)*y(197);
jac(108,197)=jac(108,197)-k(21)*y(108);
jac(108,108)=jac(108,108)-k(21)*y(197);
jac(588,197)=jac(588,197)+k(21)*y(108);
jac(588,108)=jac(588,108)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(108);
jac(32,108)=jac(32,108)-k(21)*y(32);
jac(108,32)=jac(108,32)-k(21)*y(108);
jac(108,108)=jac(108,108)-k(21)*y(32);
jac(105,32)=jac(105,32)+k(21)*y(108);
jac(105,108)=jac(105,108)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(108);
jac(2,108)=jac(2,108)-k(21)*y(2);
jac(108,2)=jac(108,2)-k(21)*y(108);
jac(108,108)=jac(108,108)-k(21)*y(2);
jac(109,2)=jac(109,2)+k(21)*y(108);
jac(109,108)=jac(109,108)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(108);
jac(3,108)=jac(3,108)-k(15)*y(3);
jac(108,3)=jac(108,3)-k(15)*y(108);
jac(108,108)=jac(108,108)-k(15)*y(3);
jac(117,3)=jac(117,3)+k(15)*y(108);
jac(117,108)=jac(117,108)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

jac(108,108)=jac(108,108)-k(10);
jac(3,108)=jac(3,108)+k(10);
jac(590,108)=jac(590,108)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) 

jac(108,108)=jac(108,108)-k(8);
jac(77,108)=jac(77,108)+k(8);
jac(8,108)=jac(8,108)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(108,108)=jac(108,108)-k(6);
jac(76,108)=jac(76,108)+k(6);
jac(7,108)=jac(7,108)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(110,110)=jac(110,110)-k(31);
jac(89,110)=jac(89,110)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~Y) 

jac(110,110)=jac(110,110)-k(22);
jac(2,110)=jac(2,110)+k(22);
jac(76,110)=jac(76,110)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(110);
jac(3,110)=jac(3,110)-k(15)*y(3);
jac(110,3)=jac(110,3)-k(15)*y(110);
jac(110,110)=jac(110,110)-k(15)*y(3);
jac(114,3)=jac(114,3)+k(15)*y(110);
jac(114,110)=jac(114,110)+k(15)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(1,1)=jac(1,1)-k(5)*y(110);
jac(1,110)=jac(1,110)-k(5)*y(1);
jac(110,1)=jac(110,1)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(1);
jac(112,1)=jac(112,1)+k(5)*y(110);
jac(112,110)=jac(112,110)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(7,7)=jac(7,7)-k(5)*y(110);
jac(7,110)=jac(7,110)-k(5)*y(7);
jac(110,7)=jac(110,7)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(7);
jac(109,7)=jac(109,7)+k(5)*y(110);
jac(109,110)=jac(109,110)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(110,110)=jac(110,110)-k(5)*y(335);
jac(110,335)=jac(110,335)-k(5)*y(110);
jac(335,110)=jac(335,110)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(110);
jac(340,110)=jac(340,110)+k(5)*y(335);
jac(340,335)=jac(340,335)+k(5)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(257,257)=jac(257,257)-k(5)*y(110);
jac(257,110)=jac(257,110)-k(5)*y(257);
jac(110,257)=jac(110,257)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(257);
jac(502,257)=jac(502,257)+k(5)*y(110);
jac(502,110)=jac(502,110)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(110,110)=jac(110,110)-k(5)*y(227);
jac(110,227)=jac(110,227)-k(5)*y(110);
jac(227,110)=jac(227,110)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(110);
jac(485,110)=jac(485,110)+k(5)*y(227);
jac(485,227)=jac(485,227)+k(5)*y(110);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(110,110)=jac(110,110)-k(5)*y(222);
jac(110,222)=jac(110,222)-k(5)*y(110);
jac(222,110)=jac(222,110)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(110);
jac(281,110)=jac(281,110)+k(5)*y(222);
jac(281,222)=jac(281,222)+k(5)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(218,218)=jac(218,218)-k(5)*y(110);
jac(218,110)=jac(218,110)-k(5)*y(218);
jac(110,218)=jac(110,218)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(218);
jac(505,218)=jac(505,218)+k(5)*y(110);
jac(505,110)=jac(505,110)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(192,192)=jac(192,192)-k(5)*y(110);
jac(192,110)=jac(192,110)-k(5)*y(192);
jac(110,192)=jac(110,192)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(192);
jac(527,192)=jac(527,192)+k(5)*y(110);
jac(527,110)=jac(527,110)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(110,110)=jac(110,110)-k(5)*y(184);
jac(110,184)=jac(110,184)-k(5)*y(110);
jac(184,110)=jac(184,110)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(110);
jac(346,110)=jac(346,110)+k(5)*y(184);
jac(346,184)=jac(346,184)+k(5)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(177,177)=jac(177,177)-k(5)*y(110);
jac(177,110)=jac(177,110)-k(5)*y(177);
jac(110,177)=jac(110,177)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(177);
jac(509,177)=jac(509,177)+k(5)*y(110);
jac(509,110)=jac(509,110)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(110,110)=jac(110,110)-k(5)*y(140);
jac(110,140)=jac(110,140)-k(5)*y(110);
jac(140,110)=jac(140,110)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(110);
jac(173,110)=jac(173,110)+k(5)*y(140);
jac(173,140)=jac(173,140)+k(5)*y(110);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(110,110)=jac(110,110)-k(5)*y(131);
jac(110,131)=jac(110,131)-k(5)*y(110);
jac(131,110)=jac(131,110)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(110);
jac(355,110)=jac(355,110)+k(5)*y(131);
jac(355,131)=jac(355,131)+k(5)*y(110);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(5)*y(110);
jac(111,110)=jac(111,110)-k(5)*y(111);
jac(110,111)=jac(110,111)-k(5)*y(110);
jac(110,110)=jac(110,110)-k(5)*y(111);
jac(503,111)=jac(503,111)+k(5)*y(110);
jac(503,110)=jac(503,110)+k(5)*y(111);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) 

jac(110,110)=jac(110,110)-k(4);
jac(4,110)=jac(4,110)+k(4);
jac(111,110)=jac(111,110)+k(4);

% rule    : R14
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(31);
jac(21,111)=jac(21,111)+k(31);

% rule    : R9_op
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(22);
jac(2,111)=jac(2,111)+k(22);
jac(77,111)=jac(77,111)+k(22);

% rule    : R6
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(111);
jac(3,111)=jac(3,111)-k(15)*y(3);
jac(111,3)=jac(111,3)-k(15)*y(111);
jac(111,111)=jac(111,111)-k(15)*y(3);
jac(43,3)=jac(43,3)+k(15)*y(111);
jac(43,111)=jac(43,111)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(126,126)=jac(126,126)-k(7)*y(111);
jac(126,111)=jac(126,111)-k(7)*y(126);
jac(111,126)=jac(111,126)-k(7)*y(111);
jac(111,111)=jac(111,111)-k(7)*y(126);
jac(353,126)=jac(353,126)+k(7)*y(111);
jac(353,111)=jac(353,111)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(132);
jac(111,132)=jac(111,132)-k(7)*y(111);
jac(132,111)=jac(132,111)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(111);
jac(355,111)=jac(355,111)+k(7)*y(132);
jac(355,132)=jac(355,132)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(141);
jac(111,141)=jac(111,141)-k(7)*y(111);
jac(141,111)=jac(141,111)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(111);
jac(173,111)=jac(173,111)+k(7)*y(141);
jac(173,141)=jac(173,141)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(185);
jac(111,185)=jac(111,185)-k(7)*y(111);
jac(185,111)=jac(185,111)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(111);
jac(346,111)=jac(346,111)+k(7)*y(185);
jac(346,185)=jac(346,185)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(209);
jac(111,209)=jac(111,209)-k(7)*y(111);
jac(209,111)=jac(209,111)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(111);
jac(527,111)=jac(527,111)+k(7)*y(209);
jac(527,209)=jac(527,209)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(214);
jac(111,214)=jac(111,214)-k(7)*y(111);
jac(214,111)=jac(214,111)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(111);
jac(509,111)=jac(509,111)+k(7)*y(214);
jac(509,214)=jac(509,214)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(219);
jac(111,219)=jac(111,219)-k(7)*y(111);
jac(219,111)=jac(219,111)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(111);
jac(505,111)=jac(505,111)+k(7)*y(219);
jac(505,219)=jac(505,219)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(223);
jac(111,223)=jac(111,223)-k(7)*y(111);
jac(223,111)=jac(223,111)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(111);
jac(281,111)=jac(281,111)+k(7)*y(223);
jac(281,223)=jac(281,223)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(228);
jac(111,228)=jac(111,228)-k(7)*y(111);
jac(228,111)=jac(228,111)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(111);
jac(485,111)=jac(485,111)+k(7)*y(228);
jac(485,228)=jac(485,228)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(315,315)=jac(315,315)-k(7)*y(111);
jac(315,111)=jac(315,111)-k(7)*y(315);
jac(111,315)=jac(111,315)-k(7)*y(111);
jac(111,111)=jac(111,111)-k(7)*y(315);
jac(501,315)=jac(501,315)+k(7)*y(111);
jac(501,111)=jac(501,111)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(319);
jac(111,319)=jac(111,319)-k(7)*y(111);
jac(319,111)=jac(319,111)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(111);
jac(502,111)=jac(502,111)+k(7)*y(319);
jac(502,319)=jac(502,319)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(336);
jac(111,336)=jac(111,336)-k(7)*y(111);
jac(336,111)=jac(336,111)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(111);
jac(340,111)=jac(340,111)+k(7)*y(336);
jac(340,336)=jac(340,336)+k(7)*y(111);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(7)*y(437);
jac(111,437)=jac(111,437)-k(7)*y(111);
jac(437,111)=jac(437,111)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(111);
jac(503,111)=jac(503,111)+k(7)*y(437);
jac(503,437)=jac(503,437)+k(7)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(5)*y(114);
jac(111,114)=jac(111,114)-k(5)*y(111);
jac(114,111)=jac(114,111)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(111);
jac(504,111)=jac(504,111)+k(5)*y(114);
jac(504,114)=jac(504,114)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(118,118)=jac(118,118)-k(5)*y(111);
jac(118,111)=jac(118,111)-k(5)*y(118);
jac(111,118)=jac(111,118)-k(5)*y(111);
jac(111,111)=jac(111,111)-k(5)*y(118);
jac(540,118)=jac(540,118)+k(5)*y(111);
jac(540,111)=jac(540,111)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(122,122)=jac(122,122)-k(5)*y(111);
jac(122,111)=jac(122,111)-k(5)*y(122);
jac(111,122)=jac(111,122)-k(5)*y(111);
jac(111,111)=jac(111,111)-k(5)*y(122);
jac(316,122)=jac(316,122)+k(5)*y(111);
jac(316,111)=jac(316,111)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(111,111)=jac(111,111)-k(5)*y(176);
jac(111,176)=jac(111,176)-k(5)*y(111);
jac(176,111)=jac(176,111)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(111);
jac(271,111)=jac(271,111)+k(5)*y(176);
jac(271,176)=jac(271,176)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(111,111)=jac(111,111)-k(5)*y(191);
jac(111,191)=jac(111,191)-k(5)*y(111);
jac(191,111)=jac(191,111)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(111);
jac(438,111)=jac(438,111)+k(5)*y(191);
jac(438,191)=jac(438,191)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(111,111)=jac(111,111)-k(5)*y(256);
jac(111,256)=jac(111,256)-k(5)*y(111);
jac(256,111)=jac(256,111)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(111);
jac(259,111)=jac(259,111)+k(5)*y(256);
jac(259,256)=jac(259,256)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(111,111)=jac(111,111)-k(5)*y(264);
jac(111,264)=jac(111,264)-k(5)*y(111);
jac(264,111)=jac(264,111)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(111);
jac(266,111)=jac(266,111)+k(5)*y(264);
jac(266,264)=jac(266,264)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(5)*y(380);
jac(111,380)=jac(111,380)-k(5)*y(111);
jac(380,111)=jac(380,111)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(111);
jac(387,111)=jac(387,111)+k(5)*y(380);
jac(387,380)=jac(387,380)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(111,111)=jac(111,111)-k(5)*y(391);
jac(111,391)=jac(111,391)-k(5)*y(111);
jac(391,111)=jac(391,111)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(111);
jac(392,111)=jac(392,111)+k(5)*y(391);
jac(392,391)=jac(392,391)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(111,111)=jac(111,111)-k(5)*y(407);
jac(111,407)=jac(111,407)-k(5)*y(111);
jac(407,111)=jac(407,111)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(111);
jac(436,111)=jac(436,111)+k(5)*y(407);
jac(436,407)=jac(436,407)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(111,111)=jac(111,111)-k(5)*y(441);
jac(111,441)=jac(111,441)-k(5)*y(111);
jac(441,111)=jac(441,111)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(111);
jac(440,111)=jac(440,111)+k(5)*y(441);
jac(440,441)=jac(440,441)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(111,111)=jac(111,111)-k(5)*y(515);
jac(111,515)=jac(111,515)-k(5)*y(111);
jac(515,111)=jac(515,111)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(111);
jac(625,111)=jac(625,111)+k(5)*y(515);
jac(625,515)=jac(625,515)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(111,111)=jac(111,111)-k(5)*y(543);
jac(111,543)=jac(111,543)-k(5)*y(111);
jac(543,111)=jac(543,111)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(111);
jac(546,111)=jac(546,111)+k(5)*y(543);
jac(546,543)=jac(546,543)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(111,111)=jac(111,111)-k(5)*y(554);
jac(111,554)=jac(111,554)-k(5)*y(111);
jac(554,111)=jac(554,111)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(111);
jac(570,111)=jac(570,111)+k(5)*y(554);
jac(570,554)=jac(570,554)+k(5)*y(111);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(9,9)=jac(9,9)-k(5)*y(111);
jac(9,111)=jac(9,111)-k(5)*y(9);
jac(111,9)=jac(111,9)-k(5)*y(111);
jac(111,111)=jac(111,111)-k(5)*y(9);
jac(25,9)=jac(25,9)+k(5)*y(111);
jac(25,111)=jac(25,111)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(4,4)=jac(4,4)-k(3)*y(111);
jac(4,111)=jac(4,111)-k(3)*y(4);
jac(111,4)=jac(111,4)-k(3)*y(111);
jac(111,111)=jac(111,111)-k(3)*y(4);
jac(110,4)=jac(110,4)+k(3)*y(111);
jac(110,111)=jac(110,111)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(4,4)=jac(4,4)-k(1)*y(111);
jac(4,111)=jac(4,111)-k(1)*y(4);
jac(111,4)=jac(111,4)-k(1)*y(111);
jac(111,111)=jac(111,111)-k(1)*y(4);
jac(437,4)=jac(437,4)+k(1)*y(111);
jac(437,111)=jac(437,111)+k(1)*y(4);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(112,112)=jac(112,112)-k(31);
jac(94,112)=jac(94,112)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(112,112)=jac(112,112)-k(22);
jac(2,112)=jac(2,112)+k(22);
jac(590,112)=jac(590,112)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(112);
jac(3,112)=jac(3,112)-k(15)*y(3);
jac(112,3)=jac(112,3)-k(15)*y(112);
jac(112,112)=jac(112,112)-k(15)*y(3);
jac(115,3)=jac(115,3)+k(15)*y(112);
jac(115,112)=jac(115,112)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(112);
jac(3,112)=jac(3,112)-k(9)*y(3);
jac(112,3)=jac(112,3)-k(9)*y(112);
jac(112,112)=jac(112,112)-k(9)*y(3);
jac(109,3)=jac(109,3)+k(9)*y(112);
jac(109,112)=jac(109,112)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(112,112)=jac(112,112)-k(8);
jac(5,112)=jac(5,112)+k(8);
jac(111,112)=jac(111,112)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

jac(112,112)=jac(112,112)-k(6);
jac(1,112)=jac(1,112)+k(6);
jac(110,112)=jac(110,112)+k(6);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

jac(114,114)=jac(114,114)-k(22);
jac(2,114)=jac(2,114)+k(22);
jac(118,114)=jac(118,114)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(114,114)=jac(114,114)-k(16);
jac(3,114)=jac(3,114)+k(16);
jac(110,114)=jac(110,114)+k(16);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(1,1)=jac(1,1)-k(5)*y(114);
jac(1,114)=jac(1,114)-k(5)*y(1);
jac(114,1)=jac(114,1)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(1);
jac(115,1)=jac(115,1)+k(5)*y(114);
jac(115,114)=jac(115,114)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(7,7)=jac(7,7)-k(5)*y(114);
jac(7,114)=jac(7,114)-k(5)*y(7);
jac(114,7)=jac(114,7)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(7);
jac(113,7)=jac(113,7)+k(5)*y(114);
jac(113,114)=jac(113,114)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(5)*y(114);
jac(335,114)=jac(335,114)-k(5)*y(335);
jac(114,335)=jac(114,335)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(335);
jac(526,335)=jac(526,335)+k(5)*y(114);
jac(526,114)=jac(526,114)+k(5)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(257,257)=jac(257,257)-k(5)*y(114);
jac(257,114)=jac(257,114)-k(5)*y(257);
jac(114,257)=jac(114,257)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(257);
jac(528,257)=jac(528,257)+k(5)*y(114);
jac(528,114)=jac(528,114)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(227,227)=jac(227,227)-k(5)*y(114);
jac(227,114)=jac(227,114)-k(5)*y(227);
jac(114,227)=jac(114,227)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(227);
jac(632,227)=jac(632,227)+k(5)*y(114);
jac(632,114)=jac(632,114)+k(5)*y(227);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(222,222)=jac(222,222)-k(5)*y(114);
jac(222,114)=jac(222,114)-k(5)*y(222);
jac(114,222)=jac(114,222)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(222);
jac(637,222)=jac(637,222)+k(5)*y(114);
jac(637,114)=jac(637,114)+k(5)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(218,218)=jac(218,218)-k(5)*y(114);
jac(218,114)=jac(218,114)-k(5)*y(218);
jac(114,218)=jac(114,218)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(218);
jac(523,218)=jac(523,218)+k(5)*y(114);
jac(523,114)=jac(523,114)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(192,192)=jac(192,192)-k(5)*y(114);
jac(192,114)=jac(192,114)-k(5)*y(192);
jac(114,192)=jac(114,192)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(192);
jac(525,192)=jac(525,192)+k(5)*y(114);
jac(525,114)=jac(525,114)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(5)*y(114);
jac(184,114)=jac(184,114)-k(5)*y(184);
jac(114,184)=jac(114,184)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(184);
jac(522,184)=jac(522,184)+k(5)*y(114);
jac(522,114)=jac(522,114)+k(5)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(177,177)=jac(177,177)-k(5)*y(114);
jac(177,114)=jac(177,114)-k(5)*y(177);
jac(114,177)=jac(114,177)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(177);
jac(521,177)=jac(521,177)+k(5)*y(114);
jac(521,114)=jac(521,114)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(140,140)=jac(140,140)-k(5)*y(114);
jac(140,114)=jac(140,114)-k(5)*y(140);
jac(114,140)=jac(114,140)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(140);
jac(604,140)=jac(604,140)+k(5)*y(114);
jac(604,114)=jac(604,114)+k(5)*y(140);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(131,131)=jac(131,131)-k(5)*y(114);
jac(131,114)=jac(131,114)-k(5)*y(131);
jac(114,131)=jac(114,131)-k(5)*y(114);
jac(114,114)=jac(114,114)-k(5)*y(131);
jac(357,131)=jac(357,131)+k(5)*y(114);
jac(357,114)=jac(357,114)+k(5)*y(131);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) 

jac(114,114)=jac(114,114)-k(4);
jac(4,114)=jac(4,114)+k(4);
jac(43,114)=jac(43,114)+k(4);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

jac(115,115)=jac(115,115)-k(22);
jac(2,115)=jac(2,115)+k(22);
jac(119,115)=jac(119,115)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(115,115)=jac(115,115)-k(19);
jac(524,115)=jac(524,115)+k(19);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(115,115)=jac(115,115)-k(17);
jac(306,115)=jac(306,115)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(115,115)=jac(115,115)-k(16);
jac(3,115)=jac(3,115)+k(16);
jac(112,115)=jac(112,115)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(115);
jac(3,115)=jac(3,115)-k(9)*y(3);
jac(115,3)=jac(115,3)-k(9)*y(115);
jac(115,115)=jac(115,115)-k(9)*y(3);
jac(113,3)=jac(113,3)+k(9)*y(115);
jac(113,115)=jac(113,115)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(115,115)=jac(115,115)-k(8);
jac(5,115)=jac(5,115)+k(8);
jac(43,115)=jac(43,115)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

jac(115,115)=jac(115,115)-k(6);
jac(1,115)=jac(1,115)+k(6);
jac(114,115)=jac(114,115)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(116,116)=jac(116,116)-k(32);
jac(113,116)=jac(113,116)+k(32);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(116,116)=jac(116,116)-k(24);
jac(352,116)=jac(352,116)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(116,116)=jac(116,116)-k(22);
jac(2,116)=jac(2,116)+k(22);
jac(120,116)=jac(120,116)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(116);
jac(182,116)=jac(182,116)-k(21)*y(182);
jac(116,182)=jac(116,182)-k(21)*y(116);
jac(116,116)=jac(116,116)-k(21)*y(182);
jac(522,182)=jac(522,182)+k(21)*y(116);
jac(522,116)=jac(522,116)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(116);
jac(197,116)=jac(197,116)-k(21)*y(197);
jac(116,197)=jac(116,197)-k(21)*y(116);
jac(116,116)=jac(116,116)-k(21)*y(197);
jac(526,197)=jac(526,197)+k(21)*y(116);
jac(526,116)=jac(526,116)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(116);
jac(32,116)=jac(32,116)-k(21)*y(32);
jac(116,32)=jac(116,32)-k(21)*y(116);
jac(116,116)=jac(116,116)-k(21)*y(32);
jac(357,32)=jac(357,32)+k(21)*y(116);
jac(357,116)=jac(357,116)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(116);
jac(2,116)=jac(2,116)-k(21)*y(2);
jac(116,2)=jac(116,2)-k(21)*y(116);
jac(116,116)=jac(116,116)-k(21)*y(2);
jac(604,2)=jac(604,2)+k(21)*y(116);
jac(604,116)=jac(604,116)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(116,116)=jac(116,116)-k(16);
jac(3,116)=jac(3,116)+k(16);
jac(353,116)=jac(353,116)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(116,116)=jac(116,116)-k(10);
jac(3,116)=jac(3,116)+k(10);
jac(524,116)=jac(524,116)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(116,116)=jac(116,116)-k(8);
jac(126,116)=jac(126,116)+k(8);
jac(43,116)=jac(43,116)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(116,116)=jac(116,116)-k(6);
jac(48,116)=jac(48,116)+k(6);
jac(114,116)=jac(114,116)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(118,118)=jac(118,118)-k(32);
jac(122,118)=jac(122,118)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(118);
jac(182,118)=jac(182,118)-k(21)*y(182);
jac(118,182)=jac(118,182)-k(21)*y(118);
jac(118,118)=jac(118,118)-k(21)*y(182);
jac(515,182)=jac(515,182)+k(21)*y(118);
jac(515,118)=jac(515,118)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(118);
jac(197,118)=jac(197,118)-k(21)*y(197);
jac(118,197)=jac(118,197)-k(21)*y(118);
jac(118,118)=jac(118,118)-k(21)*y(197);
jac(543,197)=jac(543,197)+k(21)*y(118);
jac(543,118)=jac(543,118)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(118);
jac(32,118)=jac(32,118)-k(21)*y(32);
jac(118,32)=jac(118,32)-k(21)*y(118);
jac(118,118)=jac(118,118)-k(21)*y(32);
jac(554,32)=jac(554,32)+k(21)*y(118);
jac(554,118)=jac(554,118)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(118);
jac(2,118)=jac(2,118)-k(21)*y(2);
jac(118,2)=jac(118,2)-k(21)*y(118);
jac(118,118)=jac(118,118)-k(21)*y(2);
jac(114,2)=jac(114,2)+k(21)*y(118);
jac(114,118)=jac(114,118)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Lyn(U, SH2) 

jac(118,118)=jac(118,118)-k(16);
jac(3,118)=jac(3,118)+k(16);
jac(76,118)=jac(76,118)+k(16);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(1,1)=jac(1,1)-k(5)*y(118);
jac(1,118)=jac(1,118)-k(5)*y(1);
jac(118,1)=jac(118,1)-k(5)*y(118);
jac(118,118)=jac(118,118)-k(5)*y(1);
jac(119,1)=jac(119,1)+k(5)*y(118);
jac(119,118)=jac(119,118)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(7,7)=jac(7,7)-k(5)*y(118);
jac(7,118)=jac(7,118)-k(5)*y(7);
jac(118,7)=jac(118,7)-k(5)*y(118);
jac(118,118)=jac(118,118)-k(5)*y(7);
jac(117,7)=jac(117,7)+k(5)*y(118);
jac(117,118)=jac(117,118)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(118,118)=jac(118,118)-k(5)*y(335);
jac(118,335)=jac(118,335)-k(5)*y(118);
jac(335,118)=jac(335,118)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(118);
jac(339,118)=jac(339,118)+k(5)*y(335);
jac(339,335)=jac(339,335)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(118,118)=jac(118,118)-k(5)*y(257);
jac(118,257)=jac(118,257)-k(5)*y(118);
jac(257,118)=jac(257,118)-k(5)*y(257);
jac(257,257)=jac(257,257)-k(5)*y(118);
jac(564,118)=jac(564,118)+k(5)*y(257);
jac(564,257)=jac(564,257)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(118,118)=jac(118,118)-k(5)*y(227);
jac(118,227)=jac(118,227)-k(5)*y(118);
jac(227,118)=jac(227,118)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(118);
jac(636,118)=jac(636,118)+k(5)*y(227);
jac(636,227)=jac(636,227)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(118,118)=jac(118,118)-k(5)*y(222);
jac(118,222)=jac(118,222)-k(5)*y(118);
jac(222,118)=jac(222,118)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(118);
jac(638,118)=jac(638,118)+k(5)*y(222);
jac(638,222)=jac(638,222)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(118,118)=jac(118,118)-k(5)*y(218);
jac(118,218)=jac(118,218)-k(5)*y(118);
jac(218,118)=jac(218,118)-k(5)*y(218);
jac(218,218)=jac(218,218)-k(5)*y(118);
jac(644,118)=jac(644,118)+k(5)*y(218);
jac(644,218)=jac(644,218)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(118,118)=jac(118,118)-k(5)*y(192);
jac(118,192)=jac(118,192)-k(5)*y(118);
jac(192,118)=jac(192,118)-k(5)*y(192);
jac(192,192)=jac(192,192)-k(5)*y(118);
jac(529,118)=jac(529,118)+k(5)*y(192);
jac(529,192)=jac(529,192)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(118,118)=jac(118,118)-k(5)*y(184);
jac(118,184)=jac(118,184)-k(5)*y(118);
jac(184,118)=jac(184,118)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(118);
jac(345,118)=jac(345,118)+k(5)*y(184);
jac(345,184)=jac(345,184)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(118,118)=jac(118,118)-k(5)*y(177);
jac(118,177)=jac(118,177)-k(5)*y(118);
jac(177,118)=jac(177,118)-k(5)*y(177);
jac(177,177)=jac(177,177)-k(5)*y(118);
jac(519,118)=jac(519,118)+k(5)*y(177);
jac(519,177)=jac(519,177)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(118,118)=jac(118,118)-k(5)*y(140);
jac(118,140)=jac(118,140)-k(5)*y(118);
jac(140,118)=jac(140,118)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(118);
jac(539,118)=jac(539,118)+k(5)*y(140);
jac(539,140)=jac(539,140)+k(5)*y(118);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(118,118)=jac(118,118)-k(5)*y(131);
jac(118,131)=jac(118,131)-k(5)*y(118);
jac(131,118)=jac(131,118)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(118);
jac(134,118)=jac(134,118)+k(5)*y(131);
jac(134,131)=jac(134,131)+k(5)*y(118);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Lig(l1, l2) 

jac(118,118)=jac(118,118)-k(4);
jac(4,118)=jac(4,118)+k(4);
jac(54,118)=jac(54,118)+k(4);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(119,119)=jac(119,119)-k(32);
jac(123,119)=jac(123,119)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(119);
jac(182,119)=jac(182,119)-k(21)*y(182);
jac(119,182)=jac(119,182)-k(21)*y(119);
jac(119,119)=jac(119,119)-k(21)*y(182);
jac(591,182)=jac(591,182)+k(21)*y(119);
jac(591,119)=jac(591,119)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(119);
jac(197,119)=jac(197,119)-k(21)*y(197);
jac(119,197)=jac(119,197)-k(21)*y(119);
jac(119,119)=jac(119,119)-k(21)*y(197);
jac(589,197)=jac(589,197)+k(21)*y(119);
jac(589,119)=jac(589,119)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(119);
jac(32,119)=jac(32,119)-k(21)*y(32);
jac(119,32)=jac(119,32)-k(21)*y(119);
jac(119,119)=jac(119,119)-k(21)*y(32);
jac(592,32)=jac(592,32)+k(21)*y(119);
jac(592,119)=jac(592,119)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(119);
jac(2,119)=jac(2,119)-k(21)*y(2);
jac(119,2)=jac(119,2)-k(21)*y(119);
jac(119,119)=jac(119,119)-k(21)*y(2);
jac(115,2)=jac(115,2)+k(21)*y(119);
jac(115,119)=jac(115,119)+k(21)*y(2);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(119,119)=jac(119,119)-k(19);
jac(537,119)=jac(537,119)+k(19);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(119,119)=jac(119,119)-k(17);
jac(309,119)=jac(309,119)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

jac(119,119)=jac(119,119)-k(16);
jac(3,119)=jac(3,119)+k(16);
jac(590,119)=jac(590,119)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(119);
jac(3,119)=jac(3,119)-k(9)*y(3);
jac(119,3)=jac(119,3)-k(9)*y(119);
jac(119,119)=jac(119,119)-k(9)*y(3);
jac(117,3)=jac(117,3)+k(9)*y(119);
jac(117,119)=jac(117,119)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(119,119)=jac(119,119)-k(8);
jac(5,119)=jac(5,119)+k(8);
jac(54,119)=jac(54,119)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) 

jac(119,119)=jac(119,119)-k(6);
jac(1,119)=jac(1,119)+k(6);
jac(118,119)=jac(118,119)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(120,120)=jac(120,120)-k(32);
jac(125,120)=jac(125,120)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(120,120)=jac(120,120)-k(32);
jac(117,120)=jac(117,120)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(120);
jac(182,120)=jac(182,120)-k(21)*y(182);
jac(120,182)=jac(120,182)-k(21)*y(120);
jac(120,120)=jac(120,120)-k(21)*y(182);
jac(345,182)=jac(345,182)+k(21)*y(120);
jac(345,120)=jac(345,120)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(120);
jac(197,120)=jac(197,120)-k(21)*y(197);
jac(120,197)=jac(120,197)-k(21)*y(120);
jac(120,120)=jac(120,120)-k(21)*y(197);
jac(339,197)=jac(339,197)+k(21)*y(120);
jac(339,120)=jac(339,120)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(120);
jac(32,120)=jac(32,120)-k(21)*y(32);
jac(120,32)=jac(120,32)-k(21)*y(120);
jac(120,120)=jac(120,120)-k(21)*y(32);
jac(134,32)=jac(134,32)+k(21)*y(120);
jac(134,120)=jac(134,120)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(120);
jac(2,120)=jac(2,120)-k(21)*y(2);
jac(120,2)=jac(120,2)-k(21)*y(120);
jac(120,120)=jac(120,120)-k(21)*y(2);
jac(539,2)=jac(539,2)+k(21)*y(120);
jac(539,120)=jac(539,120)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(182,182)=jac(182,182)-k(21)*y(120);
jac(182,120)=jac(182,120)-k(21)*y(182);
jac(120,182)=jac(120,182)-k(21)*y(120);
jac(120,120)=jac(120,120)-k(21)*y(182);
jac(595,182)=jac(595,182)+k(21)*y(120);
jac(595,120)=jac(595,120)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(197,197)=jac(197,197)-k(21)*y(120);
jac(197,120)=jac(197,120)-k(21)*y(197);
jac(120,197)=jac(120,197)-k(21)*y(120);
jac(120,120)=jac(120,120)-k(21)*y(197);
jac(598,197)=jac(598,197)+k(21)*y(120);
jac(598,120)=jac(598,120)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(32,32)=jac(32,32)-k(21)*y(120);
jac(32,120)=jac(32,120)-k(21)*y(32);
jac(120,32)=jac(120,32)-k(21)*y(120);
jac(120,120)=jac(120,120)-k(21)*y(32);
jac(352,32)=jac(352,32)+k(21)*y(120);
jac(352,120)=jac(352,120)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(2,2)=jac(2,2)-k(21)*y(120);
jac(2,120)=jac(2,120)-k(21)*y(2);
jac(120,2)=jac(120,2)-k(21)*y(120);
jac(120,120)=jac(120,120)-k(21)*y(2);
jac(116,2)=jac(116,2)+k(21)*y(120);
jac(116,120)=jac(116,120)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) 

jac(120,120)=jac(120,120)-k(16);
jac(3,120)=jac(3,120)+k(16);
jac(151,120)=jac(151,120)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) 

jac(120,120)=jac(120,120)-k(10);
jac(3,120)=jac(3,120)+k(10);
jac(537,120)=jac(537,120)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(120,120)=jac(120,120)-k(8);
jac(54,120)=jac(54,120)+k(8);
jac(126,120)=jac(126,120)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(120,120)=jac(120,120)-k(6);
jac(48,120)=jac(48,120)+k(6);
jac(118,120)=jac(118,120)+k(6);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Lyn(U, SH2) 

jac(122,122)=jac(122,122)-k(16);
jac(3,122)=jac(3,122)+k(16);
jac(71,122)=jac(71,122)+k(16);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(1,1)=jac(1,1)-k(5)*y(122);
jac(1,122)=jac(1,122)-k(5)*y(1);
jac(122,1)=jac(122,1)-k(5)*y(122);
jac(122,122)=jac(122,122)-k(5)*y(1);
jac(123,1)=jac(123,1)+k(5)*y(122);
jac(123,122)=jac(123,122)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(7,7)=jac(7,7)-k(5)*y(122);
jac(7,122)=jac(7,122)-k(5)*y(7);
jac(122,7)=jac(122,7)-k(5)*y(122);
jac(122,122)=jac(122,122)-k(5)*y(7);
jac(121,7)=jac(121,7)+k(5)*y(122);
jac(121,122)=jac(121,122)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(122,122)=jac(122,122)-k(5)*y(335);
jac(122,335)=jac(122,335)-k(5)*y(122);
jac(335,122)=jac(335,122)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(122);
jac(200,122)=jac(200,122)+k(5)*y(335);
jac(200,335)=jac(200,335)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(122,122)=jac(122,122)-k(5)*y(257);
jac(122,257)=jac(122,257)-k(5)*y(122);
jac(257,122)=jac(257,122)-k(5)*y(257);
jac(257,257)=jac(257,257)-k(5)*y(122);
jac(318,122)=jac(318,122)+k(5)*y(257);
jac(318,257)=jac(318,257)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(122,122)=jac(122,122)-k(5)*y(227);
jac(122,227)=jac(122,227)-k(5)*y(122);
jac(227,122)=jac(227,122)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(122);
jac(320,122)=jac(320,122)+k(5)*y(227);
jac(320,227)=jac(320,227)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(122,122)=jac(122,122)-k(5)*y(222);
jac(122,222)=jac(122,222)-k(5)*y(122);
jac(222,122)=jac(222,122)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(122);
jac(324,122)=jac(324,122)+k(5)*y(222);
jac(324,222)=jac(324,222)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(122,122)=jac(122,122)-k(5)*y(218);
jac(122,218)=jac(122,218)-k(5)*y(122);
jac(218,122)=jac(218,122)-k(5)*y(218);
jac(218,218)=jac(218,218)-k(5)*y(122);
jac(323,122)=jac(323,122)+k(5)*y(218);
jac(323,218)=jac(323,218)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(122,122)=jac(122,122)-k(5)*y(192);
jac(122,192)=jac(122,192)-k(5)*y(122);
jac(192,122)=jac(192,122)-k(5)*y(192);
jac(192,192)=jac(192,192)-k(5)*y(122);
jac(322,122)=jac(322,122)+k(5)*y(192);
jac(322,192)=jac(322,192)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(122,122)=jac(122,122)-k(5)*y(184);
jac(122,184)=jac(122,184)-k(5)*y(122);
jac(184,122)=jac(184,122)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(122);
jac(327,122)=jac(327,122)+k(5)*y(184);
jac(327,184)=jac(327,184)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(122,122)=jac(122,122)-k(5)*y(177);
jac(122,177)=jac(122,177)-k(5)*y(122);
jac(177,122)=jac(177,122)-k(5)*y(177);
jac(177,177)=jac(177,177)-k(5)*y(122);
jac(326,122)=jac(326,122)+k(5)*y(177);
jac(326,177)=jac(326,177)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(122,122)=jac(122,122)-k(5)*y(140);
jac(122,140)=jac(122,140)-k(5)*y(122);
jac(140,122)=jac(140,122)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(122);
jac(129,122)=jac(129,122)+k(5)*y(140);
jac(129,140)=jac(129,140)+k(5)*y(122);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(122,122)=jac(122,122)-k(5)*y(131);
jac(122,131)=jac(122,131)-k(5)*y(122);
jac(131,122)=jac(131,122)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(122);
jac(130,122)=jac(130,122)+k(5)*y(131);
jac(130,131)=jac(130,131)+k(5)*y(122);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Lig(l1, l2) 

jac(122,122)=jac(122,122)-k(4);
jac(4,122)=jac(4,122)+k(4);
jac(61,122)=jac(61,122)+k(4);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(123,123)=jac(123,123)-k(19);
jac(127,123)=jac(127,123)+k(19);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(123,123)=jac(123,123)-k(17);
jac(313,123)=jac(313,123)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

jac(123,123)=jac(123,123)-k(16);
jac(3,123)=jac(3,123)+k(16);
jac(333,123)=jac(333,123)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(123);
jac(3,123)=jac(3,123)-k(9)*y(3);
jac(123,3)=jac(123,3)-k(9)*y(123);
jac(123,123)=jac(123,123)-k(9)*y(3);
jac(121,3)=jac(121,3)+k(9)*y(123);
jac(121,123)=jac(121,123)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(123,123)=jac(123,123)-k(8);
jac(5,123)=jac(5,123)+k(8);
jac(61,123)=jac(61,123)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) 

jac(123,123)=jac(123,123)-k(6);
jac(1,123)=jac(1,123)+k(6);
jac(122,123)=jac(122,123)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(124,124)=jac(124,124)-k(31);
jac(148,124)=jac(148,124)+k(31);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(124);
jac(3,124)=jac(3,124)-k(15)*y(3);
jac(124,3)=jac(124,3)-k(15)*y(124);
jac(124,124)=jac(124,124)-k(15)*y(3);
jac(121,3)=jac(121,3)+k(15)*y(124);
jac(121,124)=jac(121,124)+k(15)*y(3);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(124,124)=jac(124,124)-k(14);
jac(108,124)=jac(108,124)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

jac(124,124)=jac(124,124)-k(10);
jac(3,124)=jac(3,124)+k(10);
jac(333,124)=jac(333,124)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) 

jac(124,124)=jac(124,124)-k(8);
jac(72,124)=jac(72,124)+k(8);
jac(8,124)=jac(8,124)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(124,124)=jac(124,124)-k(6);
jac(71,124)=jac(71,124)+k(6);
jac(7,124)=jac(7,124)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(126,126)=jac(126,126)-k(32);
jac(8,126)=jac(8,126)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(126);
jac(182,126)=jac(182,126)-k(21)*y(182);
jac(126,182)=jac(126,182)-k(21)*y(126);
jac(126,126)=jac(126,126)-k(21)*y(182);
jac(185,182)=jac(185,182)+k(21)*y(126);
jac(185,126)=jac(185,126)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(126);
jac(197,126)=jac(197,126)-k(21)*y(197);
jac(126,197)=jac(126,197)-k(21)*y(126);
jac(126,126)=jac(126,126)-k(21)*y(197);
jac(336,197)=jac(336,197)+k(21)*y(126);
jac(336,126)=jac(336,126)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(126);
jac(32,126)=jac(32,126)-k(21)*y(32);
jac(126,32)=jac(126,32)-k(21)*y(126);
jac(126,126)=jac(126,126)-k(21)*y(32);
jac(132,32)=jac(132,32)+k(21)*y(126);
jac(132,126)=jac(132,126)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(126);
jac(2,126)=jac(2,126)-k(21)*y(2);
jac(126,2)=jac(126,2)-k(21)*y(126);
jac(126,126)=jac(126,126)-k(21)*y(2);
jac(141,2)=jac(141,2)+k(21)*y(126);
jac(141,126)=jac(141,126)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Lyn(U, SH2) 

jac(126,126)=jac(126,126)-k(10);
jac(3,126)=jac(3,126)+k(10);
jac(16,126)=jac(16,126)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(1,1)=jac(1,1)-k(7)*y(126);
jac(1,126)=jac(1,126)-k(7)*y(1);
jac(126,1)=jac(126,1)-k(7)*y(126);
jac(126,126)=jac(126,126)-k(7)*y(1);
jac(146,1)=jac(146,1)+k(7)*y(126);
jac(146,126)=jac(146,126)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(7)*y(126);
jac(7,126)=jac(7,126)-k(7)*y(7);
jac(126,7)=jac(126,7)-k(7)*y(126);
jac(126,126)=jac(126,126)-k(7)*y(7);
jac(17,7)=jac(17,7)+k(7)*y(126);
jac(17,126)=jac(17,126)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(126,126)=jac(126,126)-k(7)*y(335);
jac(126,335)=jac(126,335)-k(7)*y(126);
jac(335,126)=jac(335,126)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(126);
jac(408,126)=jac(408,126)+k(7)*y(335);
jac(408,335)=jac(408,335)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(126,126)=jac(126,126)-k(7)*y(257);
jac(126,257)=jac(126,257)-k(7)*y(126);
jac(257,126)=jac(257,126)-k(7)*y(257);
jac(257,257)=jac(257,257)-k(7)*y(126);
jac(599,126)=jac(599,126)+k(7)*y(257);
jac(599,257)=jac(599,257)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(126,126)=jac(126,126)-k(7)*y(227);
jac(126,227)=jac(126,227)-k(7)*y(126);
jac(227,126)=jac(227,126)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(126);
jac(598,126)=jac(598,126)+k(7)*y(227);
jac(598,227)=jac(598,227)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(126,126)=jac(126,126)-k(7)*y(222);
jac(126,222)=jac(126,222)-k(7)*y(126);
jac(222,126)=jac(222,126)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(126);
jac(595,126)=jac(595,126)+k(7)*y(222);
jac(595,222)=jac(595,222)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(126,126)=jac(126,126)-k(7)*y(218);
jac(126,218)=jac(126,218)-k(7)*y(126);
jac(218,126)=jac(218,126)-k(7)*y(218);
jac(218,218)=jac(218,218)-k(7)*y(126);
jac(361,126)=jac(361,126)+k(7)*y(218);
jac(361,218)=jac(361,218)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(126,126)=jac(126,126)-k(7)*y(192);
jac(126,192)=jac(126,192)-k(7)*y(126);
jac(192,126)=jac(192,126)-k(7)*y(192);
jac(192,192)=jac(192,192)-k(7)*y(126);
jac(196,126)=jac(196,126)+k(7)*y(192);
jac(196,192)=jac(196,192)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(126,126)=jac(126,126)-k(7)*y(184);
jac(126,184)=jac(126,184)-k(7)*y(126);
jac(184,126)=jac(184,126)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(126);
jac(651,126)=jac(651,126)+k(7)*y(184);
jac(651,184)=jac(651,184)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(126,126)=jac(126,126)-k(7)*y(177);
jac(126,177)=jac(126,177)-k(7)*y(126);
jac(177,126)=jac(177,126)-k(7)*y(177);
jac(177,177)=jac(177,177)-k(7)*y(126);
jac(181,126)=jac(181,126)+k(7)*y(177);
jac(181,177)=jac(181,177)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(126,126)=jac(126,126)-k(7)*y(140);
jac(126,140)=jac(126,140)-k(7)*y(126);
jac(140,126)=jac(140,126)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(126);
jac(461,126)=jac(461,126)+k(7)*y(140);
jac(461,140)=jac(461,140)+k(7)*y(126);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(126,126)=jac(126,126)-k(7)*y(131);
jac(126,131)=jac(126,131)-k(7)*y(126);
jac(131,126)=jac(131,126)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(126);
jac(458,126)=jac(458,126)+k(7)*y(131);
jac(458,131)=jac(458,131)+k(7)*y(126);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Lig(l1, l2) 

jac(126,126)=jac(126,126)-k(2);
jac(4,126)=jac(4,126)+k(2);
jac(48,126)=jac(48,126)+k(2);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(127,127)=jac(127,127)-k(32);
jac(123,127)=jac(123,127)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(127);
jac(182,127)=jac(182,127)-k(21)*y(182);
jac(127,182)=jac(127,182)-k(21)*y(127);
jac(127,127)=jac(127,127)-k(21)*y(182);
jac(326,182)=jac(326,182)+k(21)*y(127);
jac(326,127)=jac(326,127)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(127);
jac(197,127)=jac(197,127)-k(21)*y(197);
jac(127,197)=jac(127,197)-k(21)*y(127);
jac(127,127)=jac(127,127)-k(21)*y(197);
jac(322,197)=jac(322,197)+k(21)*y(127);
jac(322,127)=jac(322,127)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(127);
jac(32,127)=jac(32,127)-k(21)*y(32);
jac(127,32)=jac(127,32)-k(21)*y(127);
jac(127,127)=jac(127,127)-k(21)*y(32);
jac(133,32)=jac(133,32)+k(21)*y(127);
jac(133,127)=jac(133,127)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(127);
jac(2,127)=jac(2,127)-k(21)*y(2);
jac(127,2)=jac(127,2)-k(21)*y(127);
jac(127,127)=jac(127,127)-k(21)*y(2);
jac(329,2)=jac(329,2)+k(21)*y(127);
jac(329,127)=jac(329,127)+k(21)*y(2);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(127,127)=jac(127,127)-k(17);
jac(314,127)=jac(314,127)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

jac(127,127)=jac(127,127)-k(16);
jac(3,127)=jac(3,127)+k(16);
jac(331,127)=jac(331,127)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(127);
jac(3,127)=jac(3,127)-k(9)*y(3);
jac(127,3)=jac(127,3)-k(9)*y(127);
jac(127,127)=jac(127,127)-k(9)*y(3);
jac(125,3)=jac(125,3)+k(9)*y(127);
jac(125,127)=jac(125,127)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(127,127)=jac(127,127)-k(8);
jac(16,127)=jac(16,127)+k(8);
jac(61,127)=jac(61,127)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) 

jac(127,127)=jac(127,127)-k(6);
jac(15,127)=jac(15,127)+k(6);
jac(122,127)=jac(122,127)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(128,128)=jac(128,128)-k(32);
jac(124,128)=jac(124,128)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(128,128)=jac(128,128)-k(31);
jac(146,128)=jac(146,128)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(128);
jac(182,128)=jac(182,128)-k(21)*y(182);
jac(128,182)=jac(128,182)-k(21)*y(128);
jac(128,128)=jac(128,128)-k(21)*y(182);
jac(330,182)=jac(330,182)+k(21)*y(128);
jac(330,128)=jac(330,128)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(128);
jac(197,128)=jac(197,128)-k(21)*y(197);
jac(128,197)=jac(128,197)-k(21)*y(128);
jac(128,128)=jac(128,128)-k(21)*y(197);
jac(334,197)=jac(334,197)+k(21)*y(128);
jac(334,128)=jac(334,128)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(128);
jac(32,128)=jac(32,128)-k(21)*y(32);
jac(128,32)=jac(128,32)-k(21)*y(128);
jac(128,128)=jac(128,128)-k(21)*y(32);
jac(135,32)=jac(135,32)+k(21)*y(128);
jac(135,128)=jac(135,128)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(128);
jac(2,128)=jac(2,128)-k(21)*y(2);
jac(128,2)=jac(128,2)-k(21)*y(128);
jac(128,128)=jac(128,128)-k(21)*y(2);
jac(139,2)=jac(139,2)+k(21)*y(128);
jac(139,128)=jac(139,128)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(128);
jac(3,128)=jac(3,128)-k(15)*y(3);
jac(128,3)=jac(128,3)-k(15)*y(128);
jac(128,128)=jac(128,128)-k(15)*y(3);
jac(125,3)=jac(125,3)+k(15)*y(128);
jac(125,128)=jac(125,128)+k(15)*y(3);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(128,128)=jac(128,128)-k(14);
jac(151,128)=jac(151,128)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

jac(128,128)=jac(128,128)-k(10);
jac(3,128)=jac(3,128)+k(10);
jac(331,128)=jac(331,128)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~Y) 

jac(128,128)=jac(128,128)-k(8);
jac(72,128)=jac(72,128)+k(8);
jac(126,128)=jac(126,128)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(128,128)=jac(128,128)-k(6);
jac(71,128)=jac(71,128)+k(6);
jac(48,128)=jac(48,128)+k(6);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(129,129)=jac(129,129)-k(25);
jac(130,129)=jac(130,129)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(129,129)=jac(129,129)-k(22);
jac(2,129)=jac(2,129)+k(22);
jac(125,129)=jac(125,129)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(129,129)=jac(129,129)-k(16);
jac(3,129)=jac(3,129)+k(16);
jac(139,129)=jac(139,129)+k(16);

% rule    : R5_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(129,129)=jac(129,129)-k(14);
jac(539,129)=jac(539,129)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(129,129)=jac(129,129)-k(10);
jac(3,129)=jac(3,129)+k(10);
jac(329,129)=jac(329,129)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(129,129)=jac(129,129)-k(8);
jac(61,129)=jac(61,129)+k(8);
jac(141,129)=jac(141,129)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(129,129)=jac(129,129)-k(6);
jac(122,129)=jac(122,129)+k(6);
jac(140,129)=jac(140,129)+k(6);

% rule    : R16
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(131,131)=jac(131,131)-k(33);
jac(140,131)=jac(140,131)+k(33);

% rule    : R9_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(131,131)=jac(131,131)-k(22);
jac(32,131)=jac(32,131)+k(22);
jac(48,131)=jac(48,131)+k(22);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Lyn(U, SH2) 

jac(131,131)=jac(131,131)-k(10);
jac(3,131)=jac(3,131)+k(10);
jac(27,131)=jac(27,131)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(7)*y(132);
jac(131,132)=jac(131,132)-k(7)*y(131);
jac(132,131)=jac(132,131)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(131);
jac(159,131)=jac(159,131)+k(7)*y(132);
jac(159,132)=jac(159,132)+k(7)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(7)*y(141);
jac(131,141)=jac(131,141)-k(7)*y(131);
jac(141,131)=jac(141,131)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(131);
jac(165,131)=jac(165,131)+k(7)*y(141);
jac(165,141)=jac(165,141)+k(7)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(7)*y(185);
jac(131,185)=jac(131,185)-k(7)*y(131);
jac(185,131)=jac(185,131)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(131);
jac(448,131)=jac(448,131)+k(7)*y(185);
jac(448,185)=jac(448,185)+k(7)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(209,209)=jac(209,209)-k(7)*y(131);
jac(209,131)=jac(209,131)-k(7)*y(209);
jac(131,209)=jac(131,209)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(209);
jac(402,209)=jac(402,209)+k(7)*y(131);
jac(402,131)=jac(402,131)+k(7)*y(209);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(214,214)=jac(214,214)-k(7)*y(131);
jac(214,131)=jac(214,131)-k(7)*y(214);
jac(131,214)=jac(131,214)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(214);
jac(447,214)=jac(447,214)+k(7)*y(131);
jac(447,131)=jac(447,131)+k(7)*y(214);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(219,219)=jac(219,219)-k(7)*y(131);
jac(219,131)=jac(219,131)-k(7)*y(219);
jac(131,219)=jac(131,219)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(219);
jac(432,219)=jac(432,219)+k(7)*y(131);
jac(432,131)=jac(432,131)+k(7)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(7)*y(223);
jac(131,223)=jac(131,223)-k(7)*y(131);
jac(223,131)=jac(223,131)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(131);
jac(284,131)=jac(284,131)+k(7)*y(223);
jac(284,223)=jac(284,223)+k(7)*y(131);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(131,131)=jac(131,131)-k(7)*y(228);
jac(131,228)=jac(131,228)-k(7)*y(131);
jac(228,131)=jac(228,131)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(131);
jac(473,131)=jac(473,131)+k(7)*y(228);
jac(473,228)=jac(473,228)+k(7)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(315,315)=jac(315,315)-k(7)*y(131);
jac(315,131)=jac(315,131)-k(7)*y(315);
jac(131,315)=jac(131,315)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(315);
jac(443,315)=jac(443,315)+k(7)*y(131);
jac(443,131)=jac(443,131)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(319,319)=jac(319,319)-k(7)*y(131);
jac(319,131)=jac(319,131)-k(7)*y(319);
jac(131,319)=jac(131,319)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(319);
jac(471,319)=jac(471,319)+k(7)*y(131);
jac(471,131)=jac(471,131)+k(7)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(7)*y(336);
jac(131,336)=jac(131,336)-k(7)*y(131);
jac(336,131)=jac(336,131)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(131);
jac(459,131)=jac(459,131)+k(7)*y(336);
jac(459,336)=jac(459,336)+k(7)*y(131);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(437,437)=jac(437,437)-k(7)*y(131);
jac(437,131)=jac(437,131)-k(7)*y(437);
jac(131,437)=jac(131,437)-k(7)*y(131);
jac(131,131)=jac(131,131)-k(7)*y(437);
jac(440,437)=jac(440,437)+k(7)*y(131);
jac(440,131)=jac(440,131)+k(7)*y(437);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(176,176)=jac(176,176)-k(5)*y(131);
jac(176,131)=jac(176,131)-k(5)*y(176);
jac(131,176)=jac(131,176)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(176);
jac(364,176)=jac(364,176)+k(5)*y(131);
jac(364,131)=jac(364,131)+k(5)*y(176);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(191,191)=jac(191,191)-k(5)*y(131);
jac(191,131)=jac(191,131)-k(5)*y(191);
jac(131,191)=jac(131,191)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(191);
jac(163,191)=jac(163,191)+k(5)*y(131);
jac(163,131)=jac(163,131)+k(5)*y(191);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(256,256)=jac(256,256)-k(5)*y(131);
jac(256,131)=jac(256,131)-k(5)*y(256);
jac(131,256)=jac(131,256)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(256);
jac(362,256)=jac(362,256)+k(5)*y(131);
jac(362,131)=jac(362,131)+k(5)*y(256);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(264,264)=jac(264,264)-k(5)*y(131);
jac(264,131)=jac(264,131)-k(5)*y(264);
jac(131,264)=jac(131,264)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(264);
jac(358,264)=jac(358,264)+k(5)*y(131);
jac(358,131)=jac(358,131)+k(5)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(5)*y(380);
jac(131,380)=jac(131,380)-k(5)*y(131);
jac(380,131)=jac(380,131)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(131);
jac(381,131)=jac(381,131)+k(5)*y(380);
jac(381,380)=jac(381,380)+k(5)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(5)*y(391);
jac(131,391)=jac(131,391)-k(5)*y(131);
jac(391,131)=jac(391,131)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(131);
jac(365,131)=jac(365,131)+k(5)*y(391);
jac(365,391)=jac(365,391)+k(5)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(5)*y(407);
jac(131,407)=jac(131,407)-k(5)*y(131);
jac(407,131)=jac(407,131)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(131);
jac(465,131)=jac(465,131)+k(5)*y(407);
jac(465,407)=jac(465,407)+k(5)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(5)*y(441);
jac(131,441)=jac(131,441)-k(5)*y(131);
jac(441,131)=jac(441,131)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(131);
jac(159,131)=jac(159,131)+k(5)*y(441);
jac(159,441)=jac(159,441)+k(5)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(131,131)=jac(131,131)-k(5)*y(515);
jac(131,515)=jac(131,515)-k(5)*y(131);
jac(515,131)=jac(515,131)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(131);
jac(360,131)=jac(360,131)+k(5)*y(515);
jac(360,515)=jac(360,515)+k(5)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(131,131)=jac(131,131)-k(5)*y(543);
jac(131,543)=jac(131,543)-k(5)*y(131);
jac(543,131)=jac(543,131)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(131);
jac(600,131)=jac(600,131)+k(5)*y(543);
jac(600,543)=jac(600,543)+k(5)*y(131);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(131,131)=jac(131,131)-k(5)*y(554);
jac(131,554)=jac(131,554)-k(5)*y(131);
jac(554,131)=jac(554,131)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(131);
jac(602,131)=jac(602,131)+k(5)*y(554);
jac(602,554)=jac(602,554)+k(5)*y(131);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(9,9)=jac(9,9)-k(5)*y(131);
jac(9,131)=jac(9,131)-k(5)*y(9);
jac(131,9)=jac(131,9)-k(5)*y(131);
jac(131,131)=jac(131,131)-k(5)*y(9);
jac(29,9)=jac(29,9)+k(5)*y(131);
jac(29,131)=jac(29,131)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(4,4)=jac(4,4)-k(3)*y(131);
jac(4,131)=jac(4,131)-k(3)*y(4);
jac(131,4)=jac(131,4)-k(3)*y(131);
jac(131,131)=jac(131,131)-k(3)*y(4);
jac(441,4)=jac(441,4)+k(3)*y(131);
jac(441,131)=jac(441,131)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(4,4)=jac(4,4)-k(1)*y(131);
jac(4,131)=jac(4,131)-k(1)*y(4);
jac(131,4)=jac(131,4)-k(1)*y(131);
jac(131,131)=jac(131,131)-k(1)*y(4);
jac(132,4)=jac(132,4)+k(1)*y(131);
jac(132,131)=jac(132,131)+k(1)*y(4);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(132,132)=jac(132,132)-k(33);
jac(141,132)=jac(141,132)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(132,132)=jac(132,132)-k(22);
jac(32,132)=jac(32,132)+k(22);
jac(126,132)=jac(126,132)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(132,132)=jac(132,132)-k(10);
jac(3,132)=jac(3,132)+k(10);
jac(28,132)=jac(28,132)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(1,1)=jac(1,1)-k(7)*y(132);
jac(1,132)=jac(1,132)-k(7)*y(1);
jac(132,1)=jac(132,1)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(1);
jac(138,1)=jac(138,1)+k(7)*y(132);
jac(138,132)=jac(138,132)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(7)*y(132);
jac(7,132)=jac(7,132)-k(7)*y(7);
jac(132,7)=jac(132,7)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(7);
jac(29,7)=jac(29,7)+k(7)*y(132);
jac(29,132)=jac(29,132)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(7)*y(132);
jac(335,132)=jac(335,132)-k(7)*y(335);
jac(132,335)=jac(132,335)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(335);
jac(465,335)=jac(465,335)+k(7)*y(132);
jac(465,132)=jac(465,132)+k(7)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(132);
jac(257,132)=jac(257,132)-k(7)*y(257);
jac(132,257)=jac(132,257)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(257);
jac(362,257)=jac(362,257)+k(7)*y(132);
jac(362,132)=jac(362,132)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(227,227)=jac(227,227)-k(7)*y(132);
jac(227,132)=jac(227,132)-k(7)*y(227);
jac(132,227)=jac(132,227)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(227);
jac(600,227)=jac(600,227)+k(7)*y(132);
jac(600,132)=jac(600,132)+k(7)*y(227);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(222,222)=jac(222,222)-k(7)*y(132);
jac(222,132)=jac(222,132)-k(7)*y(222);
jac(132,222)=jac(132,222)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(222);
jac(360,222)=jac(360,222)+k(7)*y(132);
jac(360,132)=jac(360,132)+k(7)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(132);
jac(218,132)=jac(218,132)-k(7)*y(218);
jac(132,218)=jac(132,218)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(218);
jac(358,218)=jac(358,218)+k(7)*y(132);
jac(358,132)=jac(358,132)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(132);
jac(192,132)=jac(192,132)-k(7)*y(192);
jac(132,192)=jac(132,192)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(192);
jac(163,192)=jac(163,192)+k(7)*y(132);
jac(163,132)=jac(163,132)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(7)*y(132);
jac(184,132)=jac(184,132)-k(7)*y(184);
jac(132,184)=jac(132,184)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(184);
jac(365,184)=jac(365,184)+k(7)*y(132);
jac(365,132)=jac(365,132)+k(7)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(132);
jac(177,132)=jac(177,132)-k(7)*y(177);
jac(132,177)=jac(132,177)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(177);
jac(364,177)=jac(364,177)+k(7)*y(132);
jac(364,132)=jac(364,132)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(7)*y(132);
jac(140,132)=jac(140,132)-k(7)*y(140);
jac(132,140)=jac(132,140)-k(7)*y(132);
jac(132,132)=jac(132,132)-k(7)*y(140);
jac(381,140)=jac(381,140)+k(7)*y(132);
jac(381,132)=jac(381,132)+k(7)*y(140);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) 

jac(132,132)=jac(132,132)-k(2);
jac(4,132)=jac(4,132)+k(2);
jac(131,132)=jac(131,132)+k(2);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(133,133)=jac(133,133)-k(33);
jac(329,133)=jac(329,133)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

jac(133,133)=jac(133,133)-k(22);
jac(32,133)=jac(32,133)+k(22);
jac(127,133)=jac(127,133)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(133,133)=jac(133,133)-k(17);
jac(317,133)=jac(317,133)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(133,133)=jac(133,133)-k(16);
jac(3,133)=jac(3,133)+k(16);
jac(136,133)=jac(136,133)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(133);
jac(3,133)=jac(3,133)-k(9)*y(3);
jac(133,3)=jac(133,3)-k(9)*y(133);
jac(133,133)=jac(133,133)-k(9)*y(3);
jac(130,3)=jac(130,3)+k(9)*y(133);
jac(130,133)=jac(130,133)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(133,133)=jac(133,133)-k(8);
jac(61,133)=jac(61,133)+k(8);
jac(28,133)=jac(28,133)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(133,133)=jac(133,133)-k(6);
jac(122,133)=jac(122,133)+k(6);
jac(27,133)=jac(27,133)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(134,134)=jac(134,134)-k(33);
jac(539,134)=jac(539,134)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(134,134)=jac(134,134)-k(32);
jac(130,134)=jac(130,134)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(134,134)=jac(134,134)-k(22);
jac(32,134)=jac(32,134)+k(22);
jac(120,134)=jac(120,134)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(134);
jac(182,134)=jac(182,134)-k(21)*y(182);
jac(134,182)=jac(134,182)-k(21)*y(134);
jac(134,134)=jac(134,134)-k(21)*y(182);
jac(360,182)=jac(360,182)+k(21)*y(134);
jac(360,134)=jac(360,134)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(134);
jac(197,134)=jac(197,134)-k(21)*y(197);
jac(134,197)=jac(134,197)-k(21)*y(134);
jac(134,134)=jac(134,134)-k(21)*y(197);
jac(600,197)=jac(600,197)+k(21)*y(134);
jac(600,134)=jac(600,134)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(134);
jac(32,134)=jac(32,134)-k(21)*y(32);
jac(134,32)=jac(134,32)-k(21)*y(134);
jac(134,134)=jac(134,134)-k(21)*y(32);
jac(602,32)=jac(602,32)+k(21)*y(134);
jac(602,134)=jac(602,134)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(134);
jac(2,134)=jac(2,134)-k(21)*y(2);
jac(134,2)=jac(134,2)-k(21)*y(134);
jac(134,134)=jac(134,134)-k(21)*y(2);
jac(357,2)=jac(357,2)+k(21)*y(134);
jac(357,134)=jac(357,134)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(134,134)=jac(134,134)-k(16);
jac(3,134)=jac(3,134)+k(16);
jac(137,134)=jac(137,134)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(134,134)=jac(134,134)-k(10);
jac(3,134)=jac(3,134)+k(10);
jac(531,134)=jac(531,134)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(134,134)=jac(134,134)-k(8);
jac(54,134)=jac(54,134)+k(8);
jac(132,134)=jac(132,134)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(134,134)=jac(134,134)-k(6);
jac(118,134)=jac(118,134)+k(6);
jac(131,134)=jac(131,134)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(136,136)=jac(136,136)-k(33);
jac(142,136)=jac(142,136)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(136,136)=jac(136,136)-k(31);
jac(30,136)=jac(30,136)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~Y) 

jac(136,136)=jac(136,136)-k(22);
jac(32,136)=jac(32,136)+k(22);
jac(331,136)=jac(331,136)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(136);
jac(3,136)=jac(3,136)-k(15)*y(3);
jac(136,3)=jac(136,3)-k(15)*y(136);
jac(136,136)=jac(136,136)-k(15)*y(3);
jac(133,3)=jac(133,3)+k(15)*y(136);
jac(133,136)=jac(133,136)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(136);
jac(3,136)=jac(3,136)-k(9)*y(3);
jac(136,3)=jac(136,3)-k(9)*y(136);
jac(136,136)=jac(136,136)-k(9)*y(3);
jac(135,3)=jac(135,3)+k(9)*y(136);
jac(135,136)=jac(135,136)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

jac(136,136)=jac(136,136)-k(8);
jac(72,136)=jac(72,136)+k(8);
jac(28,136)=jac(28,136)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(136,136)=jac(136,136)-k(6);
jac(71,136)=jac(71,136)+k(6);
jac(27,136)=jac(27,136)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(137,137)=jac(137,137)-k(33);
jac(143,137)=jac(143,137)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(137,137)=jac(137,137)-k(32);
jac(135,137)=jac(135,137)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(137,137)=jac(137,137)-k(31);
jac(154,137)=jac(154,137)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(137,137)=jac(137,137)-k(22);
jac(32,137)=jac(32,137)+k(22);
jac(151,137)=jac(151,137)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(137);
jac(182,137)=jac(182,137)-k(21)*y(182);
jac(137,182)=jac(137,182)-k(21)*y(137);
jac(137,137)=jac(137,137)-k(21)*y(182);
jac(358,182)=jac(358,182)+k(21)*y(137);
jac(358,137)=jac(358,137)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(137);
jac(197,137)=jac(197,137)-k(21)*y(197);
jac(137,197)=jac(137,197)-k(21)*y(137);
jac(137,137)=jac(137,137)-k(21)*y(197);
jac(362,197)=jac(362,197)+k(21)*y(137);
jac(362,137)=jac(362,137)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(137);
jac(32,137)=jac(32,137)-k(21)*y(32);
jac(137,32)=jac(137,32)-k(21)*y(137);
jac(137,137)=jac(137,137)-k(21)*y(32);
jac(161,32)=jac(161,32)+k(21)*y(137);
jac(161,137)=jac(161,137)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(137);
jac(2,137)=jac(2,137)-k(21)*y(2);
jac(137,2)=jac(137,2)-k(21)*y(137);
jac(137,137)=jac(137,137)-k(21)*y(2);
jac(355,2)=jac(355,2)+k(21)*y(137);
jac(355,137)=jac(355,137)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(137);
jac(3,137)=jac(3,137)-k(15)*y(3);
jac(137,3)=jac(137,3)-k(15)*y(137);
jac(137,137)=jac(137,137)-k(15)*y(3);
jac(134,3)=jac(134,3)+k(15)*y(137);
jac(134,137)=jac(134,137)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(137,137)=jac(137,137)-k(10);
jac(3,137)=jac(3,137)+k(10);
jac(533,137)=jac(533,137)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

jac(137,137)=jac(137,137)-k(8);
jac(77,137)=jac(77,137)+k(8);
jac(132,137)=jac(132,137)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(137,137)=jac(137,137)-k(6);
jac(76,137)=jac(76,137)+k(6);
jac(131,137)=jac(131,137)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(138,138)=jac(138,138)-k(33);
jac(144,138)=jac(144,138)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(138,138)=jac(138,138)-k(22);
jac(32,138)=jac(32,138)+k(22);
jac(146,138)=jac(146,138)+k(22);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(138,138)=jac(138,138)-k(14);
jac(154,138)=jac(154,138)+k(14);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(138,138)=jac(138,138)-k(12);
jac(135,138)=jac(135,138)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(138,138)=jac(138,138)-k(10);
jac(3,138)=jac(3,138)+k(10);
jac(30,138)=jac(30,138)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(138);
jac(3,138)=jac(3,138)-k(9)*y(3);
jac(138,3)=jac(138,3)-k(9)*y(138);
jac(138,138)=jac(138,138)-k(9)*y(3);
jac(29,3)=jac(29,3)+k(9)*y(138);
jac(29,138)=jac(29,138)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

jac(138,138)=jac(138,138)-k(8);
jac(1,138)=jac(1,138)+k(8);
jac(132,138)=jac(132,138)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(138,138)=jac(138,138)-k(6);
jac(6,138)=jac(6,138)+k(6);
jac(131,138)=jac(131,138)+k(6);

% rule    : R9_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(140,140)=jac(140,140)-k(22);
jac(2,140)=jac(2,140)+k(22);
jac(48,140)=jac(48,140)+k(22);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lyn(U, SH2) 

jac(140,140)=jac(140,140)-k(10);
jac(3,140)=jac(3,140)+k(10);
jac(21,140)=jac(21,140)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(7)*y(141);
jac(140,141)=jac(140,141)-k(7)*y(140);
jac(141,140)=jac(141,140)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(140);
jac(171,140)=jac(171,140)+k(7)*y(141);
jac(171,141)=jac(171,141)+k(7)*y(140);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(7)*y(185);
jac(140,185)=jac(140,185)-k(7)*y(140);
jac(185,140)=jac(185,140)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(140);
jac(460,140)=jac(460,140)+k(7)*y(185);
jac(460,185)=jac(460,185)+k(7)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(209,209)=jac(209,209)-k(7)*y(140);
jac(209,140)=jac(209,140)-k(7)*y(209);
jac(140,209)=jac(140,209)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(209);
jac(374,209)=jac(374,209)+k(7)*y(140);
jac(374,140)=jac(374,140)+k(7)*y(209);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(214,214)=jac(214,214)-k(7)*y(140);
jac(214,140)=jac(214,140)-k(7)*y(214);
jac(140,214)=jac(140,214)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(214);
jac(389,214)=jac(389,214)+k(7)*y(140);
jac(389,140)=jac(389,140)+k(7)*y(214);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(219,219)=jac(219,219)-k(7)*y(140);
jac(219,140)=jac(219,140)-k(7)*y(219);
jac(140,219)=jac(140,219)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(219);
jac(429,219)=jac(429,219)+k(7)*y(140);
jac(429,140)=jac(429,140)+k(7)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(7)*y(223);
jac(140,223)=jac(140,223)-k(7)*y(140);
jac(223,140)=jac(223,140)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(140);
jac(279,140)=jac(279,140)+k(7)*y(223);
jac(279,223)=jac(279,223)+k(7)*y(140);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(140,140)=jac(140,140)-k(7)*y(228);
jac(140,228)=jac(140,228)-k(7)*y(140);
jac(228,140)=jac(228,140)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(140);
jac(428,140)=jac(428,140)+k(7)*y(228);
jac(428,228)=jac(428,228)+k(7)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(315,315)=jac(315,315)-k(7)*y(140);
jac(315,140)=jac(315,140)-k(7)*y(315);
jac(140,315)=jac(140,315)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(315);
jac(424,315)=jac(424,315)+k(7)*y(140);
jac(424,140)=jac(424,140)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(319,319)=jac(319,319)-k(7)*y(140);
jac(319,140)=jac(319,140)-k(7)*y(319);
jac(140,319)=jac(140,319)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(319);
jac(426,319)=jac(426,319)+k(7)*y(140);
jac(426,140)=jac(426,140)+k(7)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(7)*y(336);
jac(140,336)=jac(140,336)-k(7)*y(140);
jac(336,140)=jac(336,140)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(140);
jac(372,140)=jac(372,140)+k(7)*y(336);
jac(372,336)=jac(372,336)+k(7)*y(140);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(437,437)=jac(437,437)-k(7)*y(140);
jac(437,140)=jac(437,140)-k(7)*y(437);
jac(140,437)=jac(140,437)-k(7)*y(140);
jac(140,140)=jac(140,140)-k(7)*y(437);
jac(387,437)=jac(387,437)+k(7)*y(140);
jac(387,140)=jac(387,140)+k(7)*y(437);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(176,176)=jac(176,176)-k(5)*y(140);
jac(176,140)=jac(176,140)-k(5)*y(176);
jac(140,176)=jac(140,176)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(176);
jac(175,176)=jac(175,176)+k(5)*y(140);
jac(175,140)=jac(175,140)+k(5)*y(176);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(191,191)=jac(191,191)-k(5)*y(140);
jac(191,140)=jac(191,140)-k(5)*y(191);
jac(140,191)=jac(140,191)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(191);
jac(169,191)=jac(169,191)+k(5)*y(140);
jac(169,140)=jac(169,140)+k(5)*y(191);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(256,256)=jac(256,256)-k(5)*y(140);
jac(256,140)=jac(256,140)-k(5)*y(256);
jac(140,256)=jac(140,256)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(256);
jac(605,256)=jac(605,256)+k(5)*y(140);
jac(605,140)=jac(605,140)+k(5)*y(256);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(264,264)=jac(264,264)-k(5)*y(140);
jac(264,140)=jac(264,140)-k(5)*y(264);
jac(140,264)=jac(140,264)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(264);
jac(180,264)=jac(180,264)+k(5)*y(140);
jac(180,140)=jac(180,140)+k(5)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(5)*y(380);
jac(140,380)=jac(140,380)-k(5)*y(140);
jac(380,140)=jac(380,140)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(140);
jac(171,140)=jac(171,140)+k(5)*y(380);
jac(171,380)=jac(171,380)+k(5)*y(140);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(5)*y(391);
jac(140,391)=jac(140,391)-k(5)*y(140);
jac(391,140)=jac(391,140)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(140);
jac(178,140)=jac(178,140)+k(5)*y(391);
jac(178,391)=jac(178,391)+k(5)*y(140);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(5)*y(407);
jac(140,407)=jac(140,407)-k(5)*y(140);
jac(407,140)=jac(407,140)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(140);
jac(466,140)=jac(466,140)+k(5)*y(407);
jac(466,407)=jac(466,407)+k(5)*y(140);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(5)*y(441);
jac(140,441)=jac(140,441)-k(5)*y(140);
jac(441,140)=jac(441,140)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(140);
jac(165,140)=jac(165,140)+k(5)*y(441);
jac(165,441)=jac(165,441)+k(5)*y(140);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(5)*y(515);
jac(140,515)=jac(140,515)-k(5)*y(140);
jac(515,140)=jac(515,140)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(140);
jac(596,140)=jac(596,140)+k(5)*y(515);
jac(596,515)=jac(596,515)+k(5)*y(140);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(140,140)=jac(140,140)-k(5)*y(543);
jac(140,543)=jac(140,543)-k(5)*y(140);
jac(543,140)=jac(543,140)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(140);
jac(544,140)=jac(544,140)+k(5)*y(543);
jac(544,543)=jac(544,543)+k(5)*y(140);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(140,140)=jac(140,140)-k(5)*y(554);
jac(140,554)=jac(140,554)-k(5)*y(140);
jac(554,140)=jac(554,140)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(140);
jac(603,140)=jac(603,140)+k(5)*y(554);
jac(603,554)=jac(603,554)+k(5)*y(140);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(9,9)=jac(9,9)-k(5)*y(140);
jac(9,140)=jac(9,140)-k(5)*y(9);
jac(140,9)=jac(140,9)-k(5)*y(140);
jac(140,140)=jac(140,140)-k(5)*y(9);
jac(23,9)=jac(23,9)+k(5)*y(140);
jac(23,140)=jac(23,140)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(4,4)=jac(4,4)-k(3)*y(140);
jac(4,140)=jac(4,140)-k(3)*y(4);
jac(140,4)=jac(140,4)-k(3)*y(140);
jac(140,140)=jac(140,140)-k(3)*y(4);
jac(380,4)=jac(380,4)+k(3)*y(140);
jac(380,140)=jac(380,140)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(4,4)=jac(4,4)-k(1)*y(140);
jac(4,140)=jac(4,140)-k(1)*y(4);
jac(140,4)=jac(140,4)-k(1)*y(140);
jac(140,140)=jac(140,140)-k(1)*y(4);
jac(141,4)=jac(141,4)+k(1)*y(140);
jac(141,140)=jac(141,140)+k(1)*y(4);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(141,141)=jac(141,141)-k(22);
jac(2,141)=jac(2,141)+k(22);
jac(126,141)=jac(126,141)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(141,141)=jac(141,141)-k(10);
jac(3,141)=jac(3,141)+k(10);
jac(22,141)=jac(22,141)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(1,1)=jac(1,1)-k(7)*y(141);
jac(1,141)=jac(1,141)-k(7)*y(1);
jac(141,1)=jac(141,1)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(1);
jac(144,1)=jac(144,1)+k(7)*y(141);
jac(144,141)=jac(144,141)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(7)*y(141);
jac(7,141)=jac(7,141)-k(7)*y(7);
jac(141,7)=jac(141,7)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(7);
jac(23,7)=jac(23,7)+k(7)*y(141);
jac(23,141)=jac(23,141)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(7)*y(141);
jac(335,141)=jac(335,141)-k(7)*y(335);
jac(141,335)=jac(141,335)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(335);
jac(466,335)=jac(466,335)+k(7)*y(141);
jac(466,141)=jac(466,141)+k(7)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(257,257)=jac(257,257)-k(7)*y(141);
jac(257,141)=jac(257,141)-k(7)*y(257);
jac(141,257)=jac(141,257)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(257);
jac(605,257)=jac(605,257)+k(7)*y(141);
jac(605,141)=jac(605,141)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(227,227)=jac(227,227)-k(7)*y(141);
jac(227,141)=jac(227,141)-k(7)*y(227);
jac(141,227)=jac(141,227)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(227);
jac(544,227)=jac(544,227)+k(7)*y(141);
jac(544,141)=jac(544,141)+k(7)*y(227);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(222,222)=jac(222,222)-k(7)*y(141);
jac(222,141)=jac(222,141)-k(7)*y(222);
jac(141,222)=jac(141,222)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(222);
jac(596,222)=jac(596,222)+k(7)*y(141);
jac(596,141)=jac(596,141)+k(7)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(141);
jac(218,141)=jac(218,141)-k(7)*y(218);
jac(141,218)=jac(141,218)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(218);
jac(180,218)=jac(180,218)+k(7)*y(141);
jac(180,141)=jac(180,141)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(141);
jac(192,141)=jac(192,141)-k(7)*y(192);
jac(141,192)=jac(141,192)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(192);
jac(169,192)=jac(169,192)+k(7)*y(141);
jac(169,141)=jac(169,141)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(7)*y(141);
jac(184,141)=jac(184,141)-k(7)*y(184);
jac(141,184)=jac(141,184)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(184);
jac(178,184)=jac(178,184)+k(7)*y(141);
jac(178,141)=jac(178,141)+k(7)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(141);
jac(177,141)=jac(177,141)-k(7)*y(177);
jac(141,177)=jac(141,177)-k(7)*y(141);
jac(141,141)=jac(141,141)-k(7)*y(177);
jac(175,177)=jac(175,177)+k(7)*y(141);
jac(175,141)=jac(175,141)+k(7)*y(177);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) 

jac(141,141)=jac(141,141)-k(2);
jac(4,141)=jac(4,141)+k(2);
jac(140,141)=jac(140,141)+k(2);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(142,142)=jac(142,142)-k(31);
jac(24,142)=jac(24,142)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~Y) 

jac(142,142)=jac(142,142)-k(22);
jac(2,142)=jac(2,142)+k(22);
jac(331,142)=jac(331,142)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(142);
jac(3,142)=jac(3,142)-k(15)*y(3);
jac(142,3)=jac(142,3)-k(15)*y(142);
jac(142,142)=jac(142,142)-k(15)*y(3);
jac(329,3)=jac(329,3)+k(15)*y(142);
jac(329,142)=jac(329,142)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(142);
jac(3,142)=jac(3,142)-k(9)*y(3);
jac(142,3)=jac(142,3)-k(9)*y(142);
jac(142,142)=jac(142,142)-k(9)*y(3);
jac(139,3)=jac(139,3)+k(9)*y(142);
jac(139,142)=jac(139,142)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

jac(142,142)=jac(142,142)-k(8);
jac(72,142)=jac(72,142)+k(8);
jac(22,142)=jac(22,142)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(142,142)=jac(142,142)-k(6);
jac(71,142)=jac(71,142)+k(6);
jac(21,142)=jac(21,142)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(143,143)=jac(143,143)-k(32);
jac(139,143)=jac(139,143)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(143,143)=jac(143,143)-k(31);
jac(145,143)=jac(145,143)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(143,143)=jac(143,143)-k(22);
jac(2,143)=jac(2,143)+k(22);
jac(151,143)=jac(151,143)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(143);
jac(182,143)=jac(182,143)-k(21)*y(182);
jac(143,182)=jac(143,182)-k(21)*y(143);
jac(143,143)=jac(143,143)-k(21)*y(182);
jac(180,182)=jac(180,182)+k(21)*y(143);
jac(180,143)=jac(180,143)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(143);
jac(197,143)=jac(197,143)-k(21)*y(197);
jac(143,197)=jac(143,197)-k(21)*y(143);
jac(143,143)=jac(143,143)-k(21)*y(197);
jac(605,197)=jac(605,197)+k(21)*y(143);
jac(605,143)=jac(605,143)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(143);
jac(32,143)=jac(32,143)-k(21)*y(32);
jac(143,32)=jac(143,32)-k(21)*y(143);
jac(143,143)=jac(143,143)-k(21)*y(32);
jac(167,32)=jac(167,32)+k(21)*y(143);
jac(167,143)=jac(167,143)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(143);
jac(2,143)=jac(2,143)-k(21)*y(2);
jac(143,2)=jac(143,2)-k(21)*y(143);
jac(143,143)=jac(143,143)-k(21)*y(2);
jac(173,2)=jac(173,2)+k(21)*y(143);
jac(173,143)=jac(173,143)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(143);
jac(3,143)=jac(3,143)-k(15)*y(3);
jac(143,3)=jac(143,3)-k(15)*y(143);
jac(143,143)=jac(143,143)-k(15)*y(3);
jac(539,3)=jac(539,3)+k(15)*y(143);
jac(539,143)=jac(539,143)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(143,143)=jac(143,143)-k(10);
jac(3,143)=jac(3,143)+k(10);
jac(508,143)=jac(508,143)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

jac(143,143)=jac(143,143)-k(8);
jac(77,143)=jac(77,143)+k(8);
jac(141,143)=jac(141,143)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(143,143)=jac(143,143)-k(6);
jac(76,143)=jac(76,143)+k(6);
jac(140,143)=jac(140,143)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(145,145)=jac(145,145)-k(32);
jac(144,145)=jac(144,145)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(145,145)=jac(145,145)-k(22);
jac(2,145)=jac(2,145)+k(22);
jac(147,145)=jac(147,145)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(145);
jac(182,145)=jac(182,145)-k(21)*y(182);
jac(145,182)=jac(145,182)-k(21)*y(145);
jac(145,145)=jac(145,145)-k(21)*y(182);
jac(175,182)=jac(175,182)+k(21)*y(145);
jac(175,145)=jac(175,145)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(145);
jac(197,145)=jac(197,145)-k(21)*y(197);
jac(145,197)=jac(145,197)-k(21)*y(145);
jac(145,145)=jac(145,145)-k(21)*y(197);
jac(169,197)=jac(169,197)+k(21)*y(145);
jac(169,145)=jac(169,145)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(145);
jac(32,145)=jac(32,145)-k(21)*y(32);
jac(145,32)=jac(145,32)-k(21)*y(145);
jac(145,145)=jac(145,145)-k(21)*y(32);
jac(164,32)=jac(164,32)+k(21)*y(145);
jac(164,145)=jac(164,145)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(145);
jac(2,145)=jac(2,145)-k(21)*y(2);
jac(145,2)=jac(145,2)-k(21)*y(145);
jac(145,145)=jac(145,145)-k(21)*y(2);
jac(170,2)=jac(170,2)+k(21)*y(145);
jac(170,145)=jac(170,145)+k(21)*y(2);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(145,145)=jac(145,145)-k(12);
jac(143,145)=jac(143,145)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(145,145)=jac(145,145)-k(10);
jac(3,145)=jac(3,145)+k(10);
jac(452,145)=jac(452,145)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(145);
jac(3,145)=jac(3,145)-k(9)*y(3);
jac(145,3)=jac(145,3)-k(9)*y(145);
jac(145,145)=jac(145,145)-k(9)*y(3);
jac(467,3)=jac(467,3)+k(9)*y(145);
jac(467,145)=jac(467,145)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

jac(145,145)=jac(145,145)-k(8);
jac(15,145)=jac(15,145)+k(8);
jac(141,145)=jac(141,145)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(145,145)=jac(145,145)-k(6);
jac(98,145)=jac(98,145)+k(6);
jac(140,145)=jac(140,145)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(149,149)=jac(149,149)-k(32);
jac(17,149)=jac(17,149)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(149,149)=jac(149,149)-k(32);
jac(463,149)=jac(463,149)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(149);
jac(182,149)=jac(182,149)-k(21)*y(182);
jac(149,182)=jac(149,182)-k(21)*y(149);
jac(149,149)=jac(149,149)-k(21)*y(182);
jac(457,182)=jac(457,182)+k(21)*y(149);
jac(457,149)=jac(457,149)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(149);
jac(197,149)=jac(197,149)-k(21)*y(197);
jac(149,197)=jac(149,197)-k(21)*y(149);
jac(149,149)=jac(149,149)-k(21)*y(197);
jac(400,197)=jac(400,197)+k(21)*y(149);
jac(400,149)=jac(400,149)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(149);
jac(32,149)=jac(32,149)-k(21)*y(32);
jac(149,32)=jac(149,32)-k(21)*y(149);
jac(149,149)=jac(149,149)-k(21)*y(32);
jac(155,32)=jac(155,32)+k(21)*y(149);
jac(155,149)=jac(155,149)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(149);
jac(2,149)=jac(2,149)-k(21)*y(2);
jac(149,2)=jac(149,2)-k(21)*y(149);
jac(149,149)=jac(149,149)-k(21)*y(2);
jac(467,2)=jac(467,2)+k(21)*y(149);
jac(467,149)=jac(467,149)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(149);
jac(182,149)=jac(182,149)-k(21)*y(182);
jac(149,182)=jac(149,182)-k(21)*y(149);
jac(149,149)=jac(149,149)-k(21)*y(182);
jac(651,182)=jac(651,182)+k(21)*y(149);
jac(651,149)=jac(651,149)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(149);
jac(197,149)=jac(197,149)-k(21)*y(197);
jac(149,197)=jac(149,197)-k(21)*y(149);
jac(149,149)=jac(149,149)-k(21)*y(197);
jac(408,197)=jac(408,197)+k(21)*y(149);
jac(408,149)=jac(408,149)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(149);
jac(32,149)=jac(32,149)-k(21)*y(32);
jac(149,32)=jac(149,32)-k(21)*y(149);
jac(149,149)=jac(149,149)-k(21)*y(32);
jac(458,32)=jac(458,32)+k(21)*y(149);
jac(458,149)=jac(458,149)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(149);
jac(2,149)=jac(2,149)-k(21)*y(2);
jac(149,2)=jac(149,2)-k(21)*y(149);
jac(149,149)=jac(149,149)-k(21)*y(2);
jac(461,2)=jac(461,2)+k(21)*y(149);
jac(461,149)=jac(461,149)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) 

jac(149,149)=jac(149,149)-k(10);
jac(3,149)=jac(3,149)+k(10);
jac(147,149)=jac(147,149)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) 

jac(149,149)=jac(149,149)-k(10);
jac(3,149)=jac(3,149)+k(10);
jac(404,149)=jac(404,149)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(149,149)=jac(149,149)-k(8);
jac(48,149)=jac(48,149)+k(8);
jac(126,149)=jac(126,149)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(149,149)=jac(149,149)-k(6);
jac(48,149)=jac(48,149)+k(6);
jac(47,149)=jac(47,149)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(150,150)=jac(150,150)-k(32);
jac(18,150)=jac(18,150)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(150,150)=jac(150,150)-k(32);
jac(97,150)=jac(97,150)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(150);
jac(182,150)=jac(182,150)-k(21)*y(182);
jac(150,182)=jac(150,182)-k(21)*y(150);
jac(150,150)=jac(150,150)-k(21)*y(182);
jac(451,182)=jac(451,182)+k(21)*y(150);
jac(451,150)=jac(451,150)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(150);
jac(197,150)=jac(197,150)-k(21)*y(197);
jac(150,197)=jac(150,197)-k(21)*y(150);
jac(150,150)=jac(150,150)-k(21)*y(197);
jac(376,197)=jac(376,197)+k(21)*y(150);
jac(376,150)=jac(376,150)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(150);
jac(32,150)=jac(32,150)-k(21)*y(32);
jac(150,32)=jac(150,32)-k(21)*y(150);
jac(150,150)=jac(150,150)-k(21)*y(32);
jac(156,32)=jac(156,32)+k(21)*y(150);
jac(156,150)=jac(156,150)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(150);
jac(2,150)=jac(2,150)-k(21)*y(2);
jac(150,2)=jac(150,2)-k(21)*y(150);
jac(150,150)=jac(150,150)-k(21)*y(2);
jac(452,2)=jac(452,2)+k(21)*y(150);
jac(452,150)=jac(452,150)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(150);
jac(182,150)=jac(182,150)-k(21)*y(182);
jac(150,182)=jac(150,182)-k(21)*y(150);
jac(150,150)=jac(150,150)-k(21)*y(182);
jac(648,182)=jac(648,182)+k(21)*y(150);
jac(648,150)=jac(648,150)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(150);
jac(197,150)=jac(197,150)-k(21)*y(197);
jac(150,197)=jac(150,197)-k(21)*y(150);
jac(150,150)=jac(150,150)-k(21)*y(197);
jac(409,197)=jac(409,197)+k(21)*y(150);
jac(409,150)=jac(409,150)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(150);
jac(32,150)=jac(32,150)-k(21)*y(32);
jac(150,32)=jac(150,32)-k(21)*y(150);
jac(150,150)=jac(150,150)-k(21)*y(32);
jac(450,32)=jac(450,32)+k(21)*y(150);
jac(450,150)=jac(450,150)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(150);
jac(2,150)=jac(2,150)-k(21)*y(2);
jac(150,2)=jac(150,2)-k(21)*y(150);
jac(150,150)=jac(150,150)-k(21)*y(2);
jac(375,2)=jac(375,2)+k(21)*y(150);
jac(375,150)=jac(375,150)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(150);
jac(3,150)=jac(3,150)-k(9)*y(3);
jac(150,3)=jac(150,3)-k(9)*y(150);
jac(150,150)=jac(150,150)-k(9)*y(3);
jac(147,3)=jac(147,3)+k(9)*y(150);
jac(147,150)=jac(147,150)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(150);
jac(3,150)=jac(3,150)-k(9)*y(3);
jac(150,3)=jac(150,3)-k(9)*y(150);
jac(150,150)=jac(150,150)-k(9)*y(3);
jac(404,3)=jac(404,3)+k(9)*y(150);
jac(404,150)=jac(404,150)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) 

jac(150,150)=jac(150,150)-k(8);
jac(15,150)=jac(15,150)+k(8);
jac(16,150)=jac(16,150)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) 

jac(150,150)=jac(150,150)-k(6);
jac(15,150)=jac(15,150)+k(6);
jac(98,150)=jac(98,150)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(151,151)=jac(151,151)-k(32);
jac(128,151)=jac(128,151)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(151,151)=jac(151,151)-k(32);
jac(108,151)=jac(108,151)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) 

jac(151,151)=jac(151,151)-k(31);
jac(147,151)=jac(147,151)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(151);
jac(182,151)=jac(182,151)-k(21)*y(182);
jac(151,182)=jac(151,182)-k(21)*y(151);
jac(151,151)=jac(151,151)-k(21)*y(182);
jac(343,182)=jac(343,182)+k(21)*y(151);
jac(343,151)=jac(343,151)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(151);
jac(197,151)=jac(197,151)-k(21)*y(197);
jac(151,197)=jac(151,197)-k(21)*y(151);
jac(151,151)=jac(151,151)-k(21)*y(197);
jac(337,197)=jac(337,197)+k(21)*y(151);
jac(337,151)=jac(337,151)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(151);
jac(32,151)=jac(32,151)-k(21)*y(32);
jac(151,32)=jac(151,32)-k(21)*y(151);
jac(151,151)=jac(151,151)-k(21)*y(32);
jac(137,32)=jac(137,32)+k(21)*y(151);
jac(137,151)=jac(137,151)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(151);
jac(2,151)=jac(2,151)-k(21)*y(2);
jac(151,2)=jac(151,2)-k(21)*y(151);
jac(151,151)=jac(151,151)-k(21)*y(2);
jac(143,2)=jac(143,2)+k(21)*y(151);
jac(143,151)=jac(143,151)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(151);
jac(182,151)=jac(182,151)-k(21)*y(182);
jac(151,182)=jac(151,182)-k(21)*y(151);
jac(151,151)=jac(151,151)-k(21)*y(182);
jac(361,182)=jac(361,182)+k(21)*y(151);
jac(361,151)=jac(361,151)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(151);
jac(197,151)=jac(197,151)-k(21)*y(197);
jac(151,197)=jac(151,197)-k(21)*y(151);
jac(151,151)=jac(151,151)-k(21)*y(197);
jac(599,197)=jac(599,197)+k(21)*y(151);
jac(599,151)=jac(599,151)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(151);
jac(32,151)=jac(32,151)-k(21)*y(32);
jac(151,32)=jac(151,32)-k(21)*y(151);
jac(151,151)=jac(151,151)-k(21)*y(32);
jac(350,32)=jac(350,32)+k(21)*y(151);
jac(350,151)=jac(350,151)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(151);
jac(2,151)=jac(2,151)-k(21)*y(2);
jac(151,2)=jac(151,2)-k(21)*y(151);
jac(151,151)=jac(151,151)-k(21)*y(2);
jac(353,2)=jac(353,2)+k(21)*y(151);
jac(353,151)=jac(353,151)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(151);
jac(3,151)=jac(3,151)-k(15)*y(3);
jac(151,3)=jac(151,3)-k(15)*y(151);
jac(151,151)=jac(151,151)-k(15)*y(3);
jac(120,3)=jac(120,3)+k(15)*y(151);
jac(120,151)=jac(120,151)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

jac(151,151)=jac(151,151)-k(10);
jac(3,151)=jac(3,151)+k(10);
jac(610,151)=jac(610,151)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY, g~pY) 

jac(151,151)=jac(151,151)-k(8);
jac(77,151)=jac(77,151)+k(8);
jac(126,151)=jac(126,151)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(151,151)=jac(151,151)-k(6);
jac(76,151)=jac(76,151)+k(6);
jac(48,151)=jac(48,151)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(152,152)=jac(152,152)-k(32);
jac(96,152)=jac(96,152)+k(32);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(152,152)=jac(152,152)-k(24);
jac(153,152)=jac(153,152)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(152,152)=jac(152,152)-k(22);
jac(2,152)=jac(2,152)+k(22);
jac(147,152)=jac(147,152)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(152);
jac(182,152)=jac(182,152)-k(21)*y(182);
jac(152,182)=jac(152,182)-k(21)*y(152);
jac(152,152)=jac(152,152)-k(21)*y(182);
jac(174,182)=jac(174,182)+k(21)*y(152);
jac(174,152)=jac(174,152)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(152);
jac(197,152)=jac(197,152)-k(21)*y(197);
jac(152,197)=jac(152,197)-k(21)*y(152);
jac(152,152)=jac(152,152)-k(21)*y(197);
jac(371,197)=jac(371,197)+k(21)*y(152);
jac(371,152)=jac(371,152)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(152);
jac(32,152)=jac(32,152)-k(21)*y(32);
jac(152,32)=jac(152,32)-k(21)*y(152);
jac(152,152)=jac(152,152)-k(21)*y(32);
jac(157,32)=jac(157,32)+k(21)*y(152);
jac(157,152)=jac(157,152)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(152);
jac(2,152)=jac(2,152)-k(21)*y(2);
jac(152,2)=jac(152,2)-k(21)*y(152);
jac(152,152)=jac(152,152)-k(21)*y(2);
jac(170,2)=jac(170,2)+k(21)*y(152);
jac(170,152)=jac(170,152)+k(21)*y(2);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(152,152)=jac(152,152)-k(12);
jac(353,152)=jac(353,152)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(152,152)=jac(152,152)-k(10);
jac(3,152)=jac(3,152)+k(10);
jac(375,152)=jac(375,152)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(152);
jac(3,152)=jac(3,152)-k(9)*y(3);
jac(152,3)=jac(152,3)-k(9)*y(152);
jac(152,152)=jac(152,152)-k(9)*y(3);
jac(461,3)=jac(461,3)+k(9)*y(152);
jac(461,152)=jac(461,152)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(152,152)=jac(152,152)-k(8);
jac(126,152)=jac(126,152)+k(8);
jac(21,152)=jac(21,152)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(152,152)=jac(152,152)-k(6);
jac(48,152)=jac(48,152)+k(6);
jac(89,152)=jac(89,152)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(153,153)=jac(153,153)-k(33);
jac(152,153)=jac(152,153)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(153,153)=jac(153,153)-k(32);
jac(103,153)=jac(103,153)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(153,153)=jac(153,153)-k(22);
jac(32,153)=jac(32,153)+k(22);
jac(147,153)=jac(147,153)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(153);
jac(182,153)=jac(182,153)-k(21)*y(182);
jac(153,182)=jac(153,182)-k(21)*y(153);
jac(153,153)=jac(153,153)-k(21)*y(182);
jac(168,182)=jac(168,182)+k(21)*y(153);
jac(168,153)=jac(168,153)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(153);
jac(197,153)=jac(197,153)-k(21)*y(197);
jac(153,197)=jac(153,197)-k(21)*y(153);
jac(153,153)=jac(153,153)-k(21)*y(197);
jac(162,197)=jac(162,197)+k(21)*y(153);
jac(162,153)=jac(162,153)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(153);
jac(32,153)=jac(32,153)-k(21)*y(32);
jac(153,32)=jac(153,32)-k(21)*y(153);
jac(153,153)=jac(153,153)-k(21)*y(32);
jac(158,32)=jac(158,32)+k(21)*y(153);
jac(158,153)=jac(158,153)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(153);
jac(2,153)=jac(2,153)-k(21)*y(2);
jac(153,2)=jac(153,2)-k(21)*y(153);
jac(153,153)=jac(153,153)-k(21)*y(2);
jac(164,2)=jac(164,2)+k(21)*y(153);
jac(164,153)=jac(164,153)+k(21)*y(2);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(153,153)=jac(153,153)-k(12);
jac(350,153)=jac(350,153)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(153,153)=jac(153,153)-k(10);
jac(3,153)=jac(3,153)+k(10);
jac(450,153)=jac(450,153)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(153);
jac(3,153)=jac(3,153)-k(9)*y(3);
jac(153,3)=jac(153,3)-k(9)*y(153);
jac(153,153)=jac(153,153)-k(9)*y(3);
jac(458,3)=jac(458,3)+k(9)*y(153);
jac(458,153)=jac(458,153)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(153,153)=jac(153,153)-k(8);
jac(126,153)=jac(126,153)+k(8);
jac(27,153)=jac(27,153)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(153,153)=jac(153,153)-k(6);
jac(48,153)=jac(48,153)+k(6);
jac(86,153)=jac(86,153)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(155,155)=jac(155,155)-k(33);
jac(467,155)=jac(467,155)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(155,155)=jac(155,155)-k(32);
jac(29,155)=jac(29,155)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(155,155)=jac(155,155)-k(22);
jac(32,155)=jac(32,155)+k(22);
jac(149,155)=jac(149,155)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(155);
jac(182,155)=jac(182,155)-k(21)*y(182);
jac(155,182)=jac(155,182)-k(21)*y(155);
jac(155,155)=jac(155,155)-k(21)*y(182);
jac(365,182)=jac(365,182)+k(21)*y(155);
jac(365,155)=jac(365,155)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(155);
jac(197,155)=jac(197,155)-k(21)*y(197);
jac(155,197)=jac(155,197)-k(21)*y(155);
jac(155,155)=jac(155,155)-k(21)*y(197);
jac(465,197)=jac(465,197)+k(21)*y(155);
jac(465,155)=jac(465,155)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(155);
jac(32,155)=jac(32,155)-k(21)*y(32);
jac(155,32)=jac(155,32)-k(21)*y(155);
jac(155,155)=jac(155,155)-k(21)*y(32);
jac(159,32)=jac(159,32)+k(21)*y(155);
jac(159,155)=jac(159,155)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(155);
jac(2,155)=jac(2,155)-k(21)*y(2);
jac(155,2)=jac(155,2)-k(21)*y(155);
jac(155,155)=jac(155,155)-k(21)*y(2);
jac(381,2)=jac(381,2)+k(21)*y(155);
jac(381,155)=jac(381,155)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(155,155)=jac(155,155)-k(10);
jac(3,155)=jac(3,155)+k(10);
jac(154,155)=jac(154,155)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(155,155)=jac(155,155)-k(10);
jac(3,155)=jac(3,155)+k(10);
jac(384,155)=jac(384,155)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(155,155)=jac(155,155)-k(8);
jac(48,155)=jac(48,155)+k(8);
jac(132,155)=jac(132,155)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(155,155)=jac(155,155)-k(6);
jac(47,155)=jac(47,155)+k(6);
jac(131,155)=jac(131,155)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(156,156)=jac(156,156)-k(33);
jac(452,156)=jac(452,156)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(156,156)=jac(156,156)-k(32);
jac(30,156)=jac(30,156)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(156,156)=jac(156,156)-k(22);
jac(32,156)=jac(32,156)+k(22);
jac(150,156)=jac(150,156)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(156);
jac(182,156)=jac(182,156)-k(21)*y(182);
jac(156,182)=jac(156,182)-k(21)*y(156);
jac(156,156)=jac(156,156)-k(21)*y(182);
jac(366,182)=jac(366,182)+k(21)*y(156);
jac(366,156)=jac(366,156)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(156);
jac(197,156)=jac(197,156)-k(21)*y(197);
jac(156,197)=jac(156,197)-k(21)*y(156);
jac(156,156)=jac(156,156)-k(21)*y(197);
jac(456,197)=jac(456,197)+k(21)*y(156);
jac(456,156)=jac(456,156)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(156);
jac(32,156)=jac(32,156)-k(21)*y(32);
jac(156,32)=jac(156,32)-k(21)*y(156);
jac(156,156)=jac(156,156)-k(21)*y(32);
jac(160,32)=jac(160,32)+k(21)*y(156);
jac(160,156)=jac(160,156)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(156);
jac(2,156)=jac(2,156)-k(21)*y(2);
jac(156,2)=jac(156,2)-k(21)*y(156);
jac(156,156)=jac(156,156)-k(21)*y(2);
jac(378,2)=jac(378,2)+k(21)*y(156);
jac(378,156)=jac(378,156)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(156);
jac(3,156)=jac(3,156)-k(9)*y(3);
jac(156,3)=jac(156,3)-k(9)*y(156);
jac(156,156)=jac(156,156)-k(9)*y(3);
jac(154,3)=jac(154,3)+k(9)*y(156);
jac(154,156)=jac(154,156)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(156);
jac(3,156)=jac(3,156)-k(9)*y(3);
jac(156,3)=jac(156,3)-k(9)*y(156);
jac(156,156)=jac(156,156)-k(9)*y(3);
jac(384,3)=jac(384,3)+k(9)*y(156);
jac(384,156)=jac(384,156)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

jac(156,156)=jac(156,156)-k(8);
jac(15,156)=jac(15,156)+k(8);
jac(28,156)=jac(28,156)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(156,156)=jac(156,156)-k(6);
jac(98,156)=jac(98,156)+k(6);
jac(27,156)=jac(27,156)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(157,157)=jac(157,157)-k(33);
jac(170,157)=jac(170,157)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(157,157)=jac(157,157)-k(28);
jac(364,157)=jac(364,157)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(157,157)=jac(157,157)-k(27);
jac(371,157)=jac(371,157)+k(27);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(157,157)=jac(157,157)-k(24);
jac(158,157)=jac(158,157)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(157,157)=jac(157,157)-k(22);
jac(2,157)=jac(2,157)+k(22);
jac(154,157)=jac(154,157)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(157,157)=jac(157,157)-k(22);
jac(32,157)=jac(32,157)+k(22);
jac(152,157)=jac(152,157)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(157,157)=jac(157,157)-k(12);
jac(355,157)=jac(355,157)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(157,157)=jac(157,157)-k(10);
jac(3,157)=jac(3,157)+k(10);
jac(378,157)=jac(378,157)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(157);
jac(3,157)=jac(3,157)-k(9)*y(3);
jac(157,3)=jac(157,3)-k(9)*y(157);
jac(157,157)=jac(157,157)-k(9)*y(3);
jac(381,3)=jac(381,3)+k(9)*y(157);
jac(381,157)=jac(381,157)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(157,157)=jac(157,157)-k(8);
jac(21,157)=jac(21,157)+k(8);
jac(132,157)=jac(132,157)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(157,157)=jac(157,157)-k(6);
jac(89,157)=jac(89,157)+k(6);
jac(131,157)=jac(131,157)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(159,159)=jac(159,159)-k(33);
jac(381,159)=jac(381,159)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(159,159)=jac(159,159)-k(33);
jac(165,159)=jac(165,159)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(159,159)=jac(159,159)-k(28);
jac(465,159)=jac(465,159)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(159,159)=jac(159,159)-k(27);
jac(459,159)=jac(459,159)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(159,159)=jac(159,159)-k(22);
jac(32,159)=jac(32,159)+k(22);
jac(155,159)=jac(155,159)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(159,159)=jac(159,159)-k(22);
jac(32,159)=jac(32,159)+k(22);
jac(458,159)=jac(458,159)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(159,159)=jac(159,159)-k(10);
jac(3,159)=jac(3,159)+k(10);
jac(158,159)=jac(158,159)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(159,159)=jac(159,159)-k(10);
jac(3,159)=jac(3,159)+k(10);
jac(446,159)=jac(446,159)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(159,159)=jac(159,159)-k(8);
jac(131,159)=jac(131,159)+k(8);
jac(132,159)=jac(132,159)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(159,159)=jac(159,159)-k(6);
jac(131,159)=jac(131,159)+k(6);
jac(441,159)=jac(441,159)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(160,160)=jac(160,160)-k(33);
jac(378,160)=jac(378,160)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(160,160)=jac(160,160)-k(33);
jac(166,160)=jac(166,160)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(160,160)=jac(160,160)-k(28);
jac(456,160)=jac(456,160)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(160,160)=jac(160,160)-k(27);
jac(454,160)=jac(454,160)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(160,160)=jac(160,160)-k(22);
jac(32,160)=jac(32,160)+k(22);
jac(156,160)=jac(156,160)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(160,160)=jac(160,160)-k(22);
jac(32,160)=jac(32,160)+k(22);
jac(450,160)=jac(450,160)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(160);
jac(3,160)=jac(3,160)-k(9)*y(3);
jac(160,3)=jac(160,3)-k(9)*y(160);
jac(160,160)=jac(160,160)-k(9)*y(3);
jac(158,3)=jac(158,3)+k(9)*y(160);
jac(158,160)=jac(158,160)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(160);
jac(3,160)=jac(3,160)-k(9)*y(3);
jac(160,3)=jac(160,3)-k(9)*y(160);
jac(160,160)=jac(160,160)-k(9)*y(3);
jac(446,3)=jac(446,3)+k(9)*y(160);
jac(446,160)=jac(446,160)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(160,160)=jac(160,160)-k(8);
jac(27,160)=jac(27,160)+k(8);
jac(28,160)=jac(28,160)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(160,160)=jac(160,160)-k(6);
jac(27,160)=jac(27,160)+k(6);
jac(86,160)=jac(86,160)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(161,161)=jac(161,161)-k(33);
jac(355,161)=jac(355,161)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(161,161)=jac(161,161)-k(33);
jac(167,161)=jac(167,161)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(161,161)=jac(161,161)-k(31);
jac(158,161)=jac(158,161)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(161,161)=jac(161,161)-k(28);
jac(362,161)=jac(362,161)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(161,161)=jac(161,161)-k(27);
jac(341,161)=jac(341,161)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(161,161)=jac(161,161)-k(22);
jac(32,161)=jac(32,161)+k(22);
jac(137,161)=jac(137,161)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(161,161)=jac(161,161)-k(22);
jac(32,161)=jac(32,161)+k(22);
jac(350,161)=jac(350,161)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(161);
jac(3,161)=jac(3,161)-k(15)*y(3);
jac(161,3)=jac(161,3)-k(15)*y(161);
jac(161,161)=jac(161,161)-k(15)*y(3);
jac(602,3)=jac(602,3)+k(15)*y(161);
jac(602,161)=jac(602,161)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(161,161)=jac(161,161)-k(10);
jac(3,161)=jac(3,161)+k(10);
jac(612,161)=jac(612,161)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(161,161)=jac(161,161)-k(8);
jac(33,161)=jac(33,161)+k(8);
jac(132,161)=jac(132,161)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(161,161)=jac(161,161)-k(6);
jac(82,161)=jac(82,161)+k(6);
jac(131,161)=jac(131,161)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(162,162)=jac(162,162)-k(34);
jac(158,162)=jac(158,162)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(162,162)=jac(162,162)-k(33);
jac(371,162)=jac(371,162)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(162,162)=jac(162,162)-k(33);
jac(168,162)=jac(168,162)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(162,162)=jac(162,162)-k(30);
jac(370,162)=jac(370,162)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(162,162)=jac(162,162)-k(22);
jac(32,162)=jac(32,162)+k(22);
jac(198,162)=jac(198,162)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(162,162)=jac(162,162)-k(22);
jac(197,162)=jac(197,162)+k(22);
jac(153,162)=jac(153,162)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(162,162)=jac(162,162)-k(12);
jac(341,162)=jac(341,162)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(162,162)=jac(162,162)-k(10);
jac(3,162)=jac(3,162)+k(10);
jac(454,162)=jac(454,162)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(162);
jac(3,162)=jac(3,162)-k(9)*y(3);
jac(162,3)=jac(162,3)-k(9)*y(162);
jac(162,162)=jac(162,162)-k(9)*y(3);
jac(459,3)=jac(459,3)+k(9)*y(162);
jac(459,162)=jac(459,162)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(162,162)=jac(162,162)-k(8);
jac(27,162)=jac(27,162)+k(8);
jac(336,162)=jac(336,162)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(162,162)=jac(162,162)-k(6);
jac(86,162)=jac(86,162)+k(6);
jac(335,162)=jac(335,162)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(163,163)=jac(163,163)-k(34);
jac(158,163)=jac(158,163)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(163,163)=jac(163,163)-k(33);
jac(364,163)=jac(364,163)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(163,163)=jac(163,163)-k(33);
jac(169,163)=jac(169,163)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(163,163)=jac(163,163)-k(29);
jac(370,163)=jac(370,163)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(163,163)=jac(163,163)-k(22);
jac(197,163)=jac(197,163)+k(22);
jac(154,163)=jac(154,163)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(163,163)=jac(163,163)-k(22);
jac(32,163)=jac(32,163)+k(22);
jac(196,163)=jac(196,163)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(163,163)=jac(163,163)-k(12);
jac(362,163)=jac(362,163)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(163,163)=jac(163,163)-k(10);
jac(3,163)=jac(3,163)+k(10);
jac(456,163)=jac(456,163)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(163);
jac(3,163)=jac(3,163)-k(9)*y(3);
jac(163,3)=jac(163,3)-k(9)*y(163);
jac(163,163)=jac(163,163)-k(9)*y(3);
jac(465,3)=jac(465,3)+k(9)*y(163);
jac(465,163)=jac(465,163)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(163,163)=jac(163,163)-k(8);
jac(192,163)=jac(192,163)+k(8);
jac(132,163)=jac(132,163)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(163,163)=jac(163,163)-k(6);
jac(191,163)=jac(191,163)+k(6);
jac(131,163)=jac(131,163)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(165,165)=jac(165,165)-k(33);
jac(171,165)=jac(171,165)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(165,165)=jac(165,165)-k(28);
jac(466,165)=jac(466,165)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(165,165)=jac(165,165)-k(27);
jac(448,165)=jac(448,165)+k(27);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(165,165)=jac(165,165)-k(23);
jac(159,165)=jac(159,165)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(165,165)=jac(165,165)-k(22);
jac(32,165)=jac(32,165)+k(22);
jac(467,165)=jac(467,165)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(165,165)=jac(165,165)-k(22);
jac(2,165)=jac(2,165)+k(22);
jac(458,165)=jac(458,165)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(165,165)=jac(165,165)-k(10);
jac(3,165)=jac(3,165)+k(10);
jac(164,165)=jac(164,165)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(165,165)=jac(165,165)-k(10);
jac(3,165)=jac(3,165)+k(10);
jac(445,165)=jac(445,165)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(165,165)=jac(165,165)-k(8);
jac(131,165)=jac(131,165)+k(8);
jac(141,165)=jac(141,165)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(165,165)=jac(165,165)-k(6);
jac(140,165)=jac(140,165)+k(6);
jac(441,165)=jac(441,165)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(166,166)=jac(166,166)-k(33);
jac(172,166)=jac(172,166)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(166,166)=jac(166,166)-k(28);
jac(453,166)=jac(453,166)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(166,166)=jac(166,166)-k(27);
jac(449,166)=jac(449,166)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(166,166)=jac(166,166)-k(22);
jac(32,166)=jac(32,166)+k(22);
jac(452,166)=jac(452,166)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(166,166)=jac(166,166)-k(22);
jac(2,166)=jac(2,166)+k(22);
jac(450,166)=jac(450,166)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(166);
jac(3,166)=jac(3,166)-k(9)*y(3);
jac(166,3)=jac(166,3)-k(9)*y(166);
jac(166,166)=jac(166,166)-k(9)*y(3);
jac(164,3)=jac(164,3)+k(9)*y(166);
jac(164,166)=jac(164,166)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(166);
jac(3,166)=jac(3,166)-k(9)*y(3);
jac(166,3)=jac(166,3)-k(9)*y(166);
jac(166,166)=jac(166,166)-k(9)*y(3);
jac(445,3)=jac(445,3)+k(9)*y(166);
jac(445,166)=jac(445,166)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(166,166)=jac(166,166)-k(8);
jac(27,166)=jac(27,166)+k(8);
jac(22,166)=jac(22,166)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(166,166)=jac(166,166)-k(6);
jac(21,166)=jac(21,166)+k(6);
jac(86,166)=jac(86,166)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(167,167)=jac(167,167)-k(33);
jac(173,167)=jac(173,167)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(167,167)=jac(167,167)-k(31);
jac(164,167)=jac(164,167)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(167,167)=jac(167,167)-k(28);
jac(605,167)=jac(605,167)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(167,167)=jac(167,167)-k(27);
jac(347,167)=jac(347,167)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(167,167)=jac(167,167)-k(22);
jac(32,167)=jac(32,167)+k(22);
jac(143,167)=jac(143,167)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(167,167)=jac(167,167)-k(22);
jac(2,167)=jac(2,167)+k(22);
jac(350,167)=jac(350,167)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(167);
jac(3,167)=jac(3,167)-k(15)*y(3);
jac(167,3)=jac(167,3)-k(15)*y(167);
jac(167,167)=jac(167,167)-k(15)*y(3);
jac(603,3)=jac(603,3)+k(15)*y(167);
jac(603,167)=jac(603,167)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(167,167)=jac(167,167)-k(10);
jac(3,167)=jac(3,167)+k(10);
jac(575,167)=jac(575,167)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(167,167)=jac(167,167)-k(8);
jac(33,167)=jac(33,167)+k(8);
jac(141,167)=jac(141,167)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(167,167)=jac(167,167)-k(6);
jac(82,167)=jac(82,167)+k(6);
jac(140,167)=jac(140,167)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(168,168)=jac(168,168)-k(34);
jac(164,168)=jac(164,168)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(168,168)=jac(168,168)-k(33);
jac(174,168)=jac(174,168)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(168,168)=jac(168,168)-k(30);
jac(190,168)=jac(190,168)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(168,168)=jac(168,168)-k(22);
jac(32,168)=jac(32,168)+k(22);
jac(189,168)=jac(189,168)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(168,168)=jac(168,168)-k(22);
jac(182,168)=jac(182,168)+k(22);
jac(153,168)=jac(153,168)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(168,168)=jac(168,168)-k(12);
jac(347,168)=jac(347,168)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(168,168)=jac(168,168)-k(10);
jac(3,168)=jac(3,168)+k(10);
jac(449,168)=jac(449,168)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(168);
jac(3,168)=jac(3,168)-k(9)*y(3);
jac(168,3)=jac(168,3)-k(9)*y(168);
jac(168,168)=jac(168,168)-k(9)*y(3);
jac(448,3)=jac(448,3)+k(9)*y(168);
jac(448,168)=jac(448,168)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(168,168)=jac(168,168)-k(8);
jac(27,168)=jac(27,168)+k(8);
jac(185,168)=jac(185,168)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(168,168)=jac(168,168)-k(6);
jac(86,168)=jac(86,168)+k(6);
jac(184,168)=jac(184,168)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(169,169)=jac(169,169)-k(34);
jac(164,169)=jac(164,169)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(169,169)=jac(169,169)-k(33);
jac(175,169)=jac(175,169)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(169,169)=jac(169,169)-k(29);
jac(190,169)=jac(190,169)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(169,169)=jac(169,169)-k(22);
jac(197,169)=jac(197,169)+k(22);
jac(145,169)=jac(145,169)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(169,169)=jac(169,169)-k(22);
jac(2,169)=jac(2,169)+k(22);
jac(196,169)=jac(196,169)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(169,169)=jac(169,169)-k(12);
jac(605,169)=jac(605,169)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(169,169)=jac(169,169)-k(10);
jac(3,169)=jac(3,169)+k(10);
jac(453,169)=jac(453,169)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(169);
jac(3,169)=jac(3,169)-k(9)*y(3);
jac(169,3)=jac(169,3)-k(9)*y(169);
jac(169,169)=jac(169,169)-k(9)*y(3);
jac(466,3)=jac(466,3)+k(9)*y(169);
jac(466,169)=jac(466,169)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(169,169)=jac(169,169)-k(8);
jac(192,169)=jac(192,169)+k(8);
jac(141,169)=jac(141,169)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(169,169)=jac(169,169)-k(6);
jac(191,169)=jac(191,169)+k(6);
jac(140,169)=jac(140,169)+k(6);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(171,171)=jac(171,171)-k(28);
jac(178,171)=jac(178,171)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(171,171)=jac(171,171)-k(27);
jac(460,171)=jac(460,171)+k(27);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(171,171)=jac(171,171)-k(24);
jac(165,171)=jac(165,171)+k(24);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(171,171)=jac(171,171)-k(23);
jac(381,171)=jac(381,171)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(171,171)=jac(171,171)-k(22);
jac(2,171)=jac(2,171)+k(22);
jac(467,171)=jac(467,171)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(171,171)=jac(171,171)-k(22);
jac(2,171)=jac(2,171)+k(22);
jac(461,171)=jac(461,171)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(171,171)=jac(171,171)-k(10);
jac(3,171)=jac(3,171)+k(10);
jac(170,171)=jac(170,171)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(171,171)=jac(171,171)-k(10);
jac(3,171)=jac(3,171)+k(10);
jac(386,171)=jac(386,171)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(171,171)=jac(171,171)-k(8);
jac(140,171)=jac(140,171)+k(8);
jac(141,171)=jac(141,171)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(171,171)=jac(171,171)-k(6);
jac(140,171)=jac(140,171)+k(6);
jac(380,171)=jac(380,171)+k(6);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(172,172)=jac(172,172)-k(28);
jac(179,172)=jac(179,172)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(172,172)=jac(172,172)-k(27);
jac(377,172)=jac(377,172)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(172,172)=jac(172,172)-k(22);
jac(2,172)=jac(2,172)+k(22);
jac(452,172)=jac(452,172)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(172,172)=jac(172,172)-k(22);
jac(2,172)=jac(2,172)+k(22);
jac(375,172)=jac(375,172)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(172);
jac(3,172)=jac(3,172)-k(9)*y(3);
jac(172,3)=jac(172,3)-k(9)*y(172);
jac(172,172)=jac(172,172)-k(9)*y(3);
jac(170,3)=jac(170,3)+k(9)*y(172);
jac(170,172)=jac(170,172)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(172);
jac(3,172)=jac(3,172)-k(9)*y(3);
jac(172,3)=jac(172,3)-k(9)*y(172);
jac(172,172)=jac(172,172)-k(9)*y(3);
jac(386,3)=jac(386,3)+k(9)*y(172);
jac(386,172)=jac(386,172)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(172,172)=jac(172,172)-k(8);
jac(21,172)=jac(21,172)+k(8);
jac(22,172)=jac(22,172)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(172,172)=jac(172,172)-k(6);
jac(21,172)=jac(21,172)+k(6);
jac(89,172)=jac(89,172)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(173,173)=jac(173,173)-k(31);
jac(170,173)=jac(170,173)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(173,173)=jac(173,173)-k(28);
jac(180,173)=jac(180,173)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(173,173)=jac(173,173)-k(27);
jac(346,173)=jac(346,173)+k(27);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(173,173)=jac(173,173)-k(24);
jac(167,173)=jac(167,173)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(173,173)=jac(173,173)-k(22);
jac(2,173)=jac(2,173)+k(22);
jac(143,173)=jac(143,173)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(173,173)=jac(173,173)-k(22);
jac(2,173)=jac(2,173)+k(22);
jac(353,173)=jac(353,173)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(173);
jac(3,173)=jac(3,173)-k(15)*y(3);
jac(173,3)=jac(173,3)-k(15)*y(173);
jac(173,173)=jac(173,173)-k(15)*y(3);
jac(604,3)=jac(604,3)+k(15)*y(173);
jac(604,173)=jac(604,173)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(173,173)=jac(173,173)-k(10);
jac(3,173)=jac(3,173)+k(10);
jac(506,173)=jac(506,173)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(173,173)=jac(173,173)-k(8);
jac(111,173)=jac(111,173)+k(8);
jac(141,173)=jac(141,173)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(173,173)=jac(173,173)-k(6);
jac(110,173)=jac(110,173)+k(6);
jac(140,173)=jac(140,173)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(174,174)=jac(174,174)-k(34);
jac(170,174)=jac(170,174)+k(34);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(174,174)=jac(174,174)-k(30);
jac(183,174)=jac(183,174)+k(30);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(174,174)=jac(174,174)-k(24);
jac(168,174)=jac(168,174)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(174,174)=jac(174,174)-k(22);
jac(2,174)=jac(2,174)+k(22);
jac(189,174)=jac(189,174)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(174,174)=jac(174,174)-k(22);
jac(182,174)=jac(182,174)+k(22);
jac(152,174)=jac(152,174)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(174,174)=jac(174,174)-k(12);
jac(346,174)=jac(346,174)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(174,174)=jac(174,174)-k(10);
jac(3,174)=jac(3,174)+k(10);
jac(377,174)=jac(377,174)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(174);
jac(3,174)=jac(3,174)-k(9)*y(3);
jac(174,3)=jac(174,3)-k(9)*y(174);
jac(174,174)=jac(174,174)-k(9)*y(3);
jac(460,3)=jac(460,3)+k(9)*y(174);
jac(460,174)=jac(460,174)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(174,174)=jac(174,174)-k(8);
jac(21,174)=jac(21,174)+k(8);
jac(185,174)=jac(185,174)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(174,174)=jac(174,174)-k(6);
jac(89,174)=jac(89,174)+k(6);
jac(184,174)=jac(184,174)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(176,176)=jac(176,176)-k(34);
jac(89,176)=jac(89,176)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~pY) 

jac(176,176)=jac(176,176)-k(22);
jac(182,176)=jac(182,176)+k(22);
jac(98,176)=jac(98,176)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(176);
jac(3,176)=jac(3,176)-k(9)*y(3);
jac(176,3)=jac(176,3)-k(9)*y(176);
jac(176,176)=jac(176,176)-k(9)*y(3);
jac(391,3)=jac(391,3)+k(9)*y(176);
jac(391,176)=jac(391,176)+k(9)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(1,1)=jac(1,1)-k(5)*y(176);
jac(1,176)=jac(1,176)-k(5)*y(1);
jac(176,1)=jac(176,1)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(1);
jac(418,1)=jac(418,1)+k(5)*y(176);
jac(418,176)=jac(418,176)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(7,7)=jac(7,7)-k(5)*y(176);
jac(7,176)=jac(7,176)-k(5)*y(7);
jac(176,7)=jac(176,7)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(7);
jac(606,7)=jac(606,7)+k(5)*y(176);
jac(606,176)=jac(606,176)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(176,176)=jac(176,176)-k(5)*y(335);
jac(176,335)=jac(176,335)-k(5)*y(176);
jac(335,176)=jac(335,176)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(176);
jac(367,176)=jac(367,176)+k(5)*y(335);
jac(367,335)=jac(367,335)+k(5)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(257,257)=jac(257,257)-k(5)*y(176);
jac(257,176)=jac(257,176)-k(5)*y(257);
jac(176,257)=jac(176,257)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(257);
jac(487,257)=jac(487,257)+k(5)*y(176);
jac(487,176)=jac(487,176)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(176,176)=jac(176,176)-k(5)*y(227);
jac(176,227)=jac(176,227)-k(5)*y(176);
jac(227,176)=jac(227,176)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(176);
jac(486,176)=jac(486,176)+k(5)*y(227);
jac(486,227)=jac(486,227)+k(5)*y(176);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(176,176)=jac(176,176)-k(5)*y(222);
jac(176,222)=jac(176,222)-k(5)*y(176);
jac(222,176)=jac(222,176)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(176);
jac(273,176)=jac(273,176)+k(5)*y(222);
jac(273,222)=jac(273,222)+k(5)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(5)*y(176);
jac(218,176)=jac(218,176)-k(5)*y(218);
jac(176,218)=jac(176,218)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(218);
jac(580,218)=jac(580,218)+k(5)*y(176);
jac(580,176)=jac(580,176)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(192,192)=jac(192,192)-k(5)*y(176);
jac(192,176)=jac(192,176)-k(5)*y(192);
jac(176,192)=jac(176,192)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(192);
jac(369,192)=jac(369,192)+k(5)*y(176);
jac(369,176)=jac(369,176)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(176,176)=jac(176,176)-k(5)*y(184);
jac(176,184)=jac(176,184)-k(5)*y(176);
jac(184,176)=jac(184,176)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(176);
jac(183,176)=jac(183,176)+k(5)*y(184);
jac(183,184)=jac(183,184)+k(5)*y(176);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(5)*y(176);
jac(177,176)=jac(177,176)-k(5)*y(177);
jac(176,177)=jac(176,177)-k(5)*y(176);
jac(176,176)=jac(176,176)-k(5)*y(177);
jac(187,177)=jac(187,177)+k(5)*y(176);
jac(187,176)=jac(187,176)+k(5)*y(177);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) 

jac(176,176)=jac(176,176)-k(4);
jac(4,176)=jac(4,176)+k(4);
jac(177,176)=jac(177,176)+k(4);

% rule    : R17
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(177,177)=jac(177,177)-k(34);
jac(21,177)=jac(21,177)+k(34);

% rule    : R9_op
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(22);
jac(182,177)=jac(182,177)+k(22);
jac(15,177)=jac(15,177)+k(22);

% rule    : R3
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(177);
jac(3,177)=jac(3,177)-k(9)*y(3);
jac(177,3)=jac(177,3)-k(9)*y(177);
jac(177,177)=jac(177,177)-k(9)*y(3);
jac(184,3)=jac(184,3)+k(9)*y(177);
jac(184,177)=jac(184,177)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(185);
jac(177,185)=jac(177,185)-k(7)*y(177);
jac(185,177)=jac(185,177)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(177);
jac(183,177)=jac(183,177)+k(7)*y(185);
jac(183,185)=jac(183,185)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(209);
jac(177,209)=jac(177,209)-k(7)*y(177);
jac(209,177)=jac(209,177)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(177);
jac(369,177)=jac(369,177)+k(7)*y(209);
jac(369,209)=jac(369,209)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(214);
jac(177,214)=jac(177,214)-k(7)*y(177);
jac(214,177)=jac(214,177)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(177);
jac(187,177)=jac(187,177)+k(7)*y(214);
jac(187,214)=jac(187,214)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(219);
jac(177,219)=jac(177,219)-k(7)*y(177);
jac(219,177)=jac(219,177)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(177);
jac(580,177)=jac(580,177)+k(7)*y(219);
jac(580,219)=jac(580,219)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(223);
jac(177,223)=jac(177,223)-k(7)*y(177);
jac(223,177)=jac(223,177)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(177);
jac(273,177)=jac(273,177)+k(7)*y(223);
jac(273,223)=jac(273,223)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(228);
jac(177,228)=jac(177,228)-k(7)*y(177);
jac(228,177)=jac(228,177)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(177);
jac(486,177)=jac(486,177)+k(7)*y(228);
jac(486,228)=jac(486,228)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(315,315)=jac(315,315)-k(7)*y(177);
jac(315,177)=jac(315,177)-k(7)*y(315);
jac(177,315)=jac(177,315)-k(7)*y(177);
jac(177,177)=jac(177,177)-k(7)*y(315);
jac(421,315)=jac(421,315)+k(7)*y(177);
jac(421,177)=jac(421,177)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(319);
jac(177,319)=jac(177,319)-k(7)*y(177);
jac(319,177)=jac(319,177)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(177);
jac(487,177)=jac(487,177)+k(7)*y(319);
jac(487,319)=jac(487,319)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(336);
jac(177,336)=jac(177,336)-k(7)*y(177);
jac(336,177)=jac(336,177)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(177);
jac(367,177)=jac(367,177)+k(7)*y(336);
jac(367,336)=jac(367,336)+k(7)*y(177);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(7)*y(437);
jac(177,437)=jac(177,437)-k(7)*y(177);
jac(437,177)=jac(437,177)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(177);
jac(271,177)=jac(271,177)+k(7)*y(437);
jac(271,437)=jac(271,437)+k(7)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(177,177)=jac(177,177)-k(5)*y(191);
jac(177,191)=jac(177,191)-k(5)*y(177);
jac(191,177)=jac(191,177)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(177);
jac(194,177)=jac(194,177)+k(5)*y(191);
jac(194,191)=jac(194,191)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(177,177)=jac(177,177)-k(5)*y(256);
jac(177,256)=jac(177,256)-k(5)*y(177);
jac(256,177)=jac(256,177)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(177);
jac(550,177)=jac(550,177)+k(5)*y(256);
jac(550,256)=jac(550,256)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(5)*y(264);
jac(177,264)=jac(177,264)-k(5)*y(177);
jac(264,177)=jac(264,177)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(177);
jac(513,177)=jac(513,177)+k(5)*y(264);
jac(513,264)=jac(513,264)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(177,177)=jac(177,177)-k(5)*y(380);
jac(177,380)=jac(177,380)-k(5)*y(177);
jac(380,177)=jac(380,177)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(177);
jac(389,177)=jac(389,177)+k(5)*y(380);
jac(389,380)=jac(389,380)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(5)*y(391);
jac(177,391)=jac(177,391)-k(5)*y(177);
jac(391,177)=jac(391,177)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(177);
jac(394,177)=jac(394,177)+k(5)*y(391);
jac(394,391)=jac(394,391)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(177,177)=jac(177,177)-k(5)*y(407);
jac(177,407)=jac(177,407)-k(5)*y(177);
jac(407,177)=jac(407,177)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(177);
jac(435,177)=jac(435,177)+k(5)*y(407);
jac(435,407)=jac(435,407)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(177,177)=jac(177,177)-k(5)*y(441);
jac(177,441)=jac(177,441)-k(5)*y(177);
jac(441,177)=jac(441,177)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(177);
jac(447,177)=jac(447,177)+k(5)*y(441);
jac(447,441)=jac(447,441)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(177,177)=jac(177,177)-k(5)*y(515);
jac(177,515)=jac(177,515)-k(5)*y(177);
jac(515,177)=jac(515,177)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(177);
jac(514,177)=jac(514,177)+k(5)*y(515);
jac(514,515)=jac(514,515)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(177,177)=jac(177,177)-k(5)*y(543);
jac(177,543)=jac(177,543)-k(5)*y(177);
jac(543,177)=jac(543,177)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(177);
jac(548,177)=jac(548,177)+k(5)*y(543);
jac(548,543)=jac(548,543)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(177,177)=jac(177,177)-k(5)*y(554);
jac(177,554)=jac(177,554)-k(5)*y(177);
jac(554,177)=jac(554,177)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(177);
jac(553,177)=jac(553,177)+k(5)*y(554);
jac(553,554)=jac(553,554)+k(5)*y(177);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(9,9)=jac(9,9)-k(5)*y(177);
jac(9,177)=jac(9,177)-k(5)*y(9);
jac(177,9)=jac(177,9)-k(5)*y(177);
jac(177,177)=jac(177,177)-k(5)*y(9);
jac(213,9)=jac(213,9)+k(5)*y(177);
jac(213,177)=jac(213,177)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(4,4)=jac(4,4)-k(3)*y(177);
jac(4,177)=jac(4,177)-k(3)*y(4);
jac(177,4)=jac(177,4)-k(3)*y(177);
jac(177,177)=jac(177,177)-k(3)*y(4);
jac(176,4)=jac(176,4)+k(3)*y(177);
jac(176,177)=jac(176,177)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(4,4)=jac(4,4)-k(1)*y(177);
jac(4,177)=jac(4,177)-k(1)*y(4);
jac(177,4)=jac(177,4)-k(1)*y(177);
jac(177,177)=jac(177,177)-k(1)*y(4);
jac(214,4)=jac(214,4)+k(1)*y(177);
jac(214,177)=jac(214,177)+k(1)*y(4);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(178,178)=jac(178,178)-k(34);
jac(171,178)=jac(171,178)+k(34);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(178,178)=jac(178,178)-k(29);
jac(186,178)=jac(186,178)+k(29);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(178,178)=jac(178,178)-k(24);
jac(466,178)=jac(466,178)+k(24);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(178,178)=jac(178,178)-k(23);
jac(365,178)=jac(365,178)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(178,178)=jac(178,178)-k(22);
jac(182,178)=jac(182,178)+k(22);
jac(467,178)=jac(467,178)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(178,178)=jac(178,178)-k(22);
jac(2,178)=jac(2,178)+k(22);
jac(651,178)=jac(651,178)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(178,178)=jac(178,178)-k(10);
jac(3,178)=jac(3,178)+k(10);
jac(175,178)=jac(175,178)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(178,178)=jac(178,178)-k(10);
jac(3,178)=jac(3,178)+k(10);
jac(390,178)=jac(390,178)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(178,178)=jac(178,178)-k(8);
jac(184,178)=jac(184,178)+k(8);
jac(141,178)=jac(141,178)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(178,178)=jac(178,178)-k(6);
jac(140,178)=jac(140,178)+k(6);
jac(391,178)=jac(391,178)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(179,179)=jac(179,179)-k(34);
jac(172,179)=jac(172,179)+k(34);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(179,179)=jac(179,179)-k(29);
jac(187,179)=jac(187,179)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(179,179)=jac(179,179)-k(22);
jac(182,179)=jac(182,179)+k(22);
jac(452,179)=jac(452,179)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(179,179)=jac(179,179)-k(22);
jac(2,179)=jac(2,179)+k(22);
jac(648,179)=jac(648,179)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(179);
jac(3,179)=jac(3,179)-k(9)*y(3);
jac(179,3)=jac(179,3)-k(9)*y(179);
jac(179,179)=jac(179,179)-k(9)*y(3);
jac(175,3)=jac(175,3)+k(9)*y(179);
jac(175,179)=jac(175,179)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(179);
jac(3,179)=jac(3,179)-k(9)*y(3);
jac(179,3)=jac(179,3)-k(9)*y(179);
jac(179,179)=jac(179,179)-k(9)*y(3);
jac(390,3)=jac(390,3)+k(9)*y(179);
jac(390,179)=jac(390,179)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(179,179)=jac(179,179)-k(8);
jac(177,179)=jac(177,179)+k(8);
jac(22,179)=jac(22,179)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(179,179)=jac(179,179)-k(6);
jac(21,179)=jac(21,179)+k(6);
jac(176,179)=jac(176,179)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(180,180)=jac(180,180)-k(34);
jac(173,180)=jac(173,180)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(180,180)=jac(180,180)-k(31);
jac(175,180)=jac(175,180)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(180,180)=jac(180,180)-k(29);
jac(188,180)=jac(188,180)+k(29);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(180,180)=jac(180,180)-k(24);
jac(605,180)=jac(605,180)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(180,180)=jac(180,180)-k(22);
jac(182,180)=jac(182,180)+k(22);
jac(143,180)=jac(143,180)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(180,180)=jac(180,180)-k(22);
jac(2,180)=jac(2,180)+k(22);
jac(361,180)=jac(361,180)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(180);
jac(3,180)=jac(3,180)-k(15)*y(3);
jac(180,3)=jac(180,3)-k(15)*y(180);
jac(180,180)=jac(180,180)-k(15)*y(3);
jac(596,3)=jac(596,3)+k(15)*y(180);
jac(596,180)=jac(596,180)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(180,180)=jac(180,180)-k(10);
jac(3,180)=jac(3,180)+k(10);
jac(510,180)=jac(510,180)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(180,180)=jac(180,180)-k(8);
jac(218,180)=jac(218,180)+k(8);
jac(141,180)=jac(141,180)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(180,180)=jac(180,180)-k(6);
jac(264,180)=jac(264,180)+k(6);
jac(140,180)=jac(140,180)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(181,181)=jac(181,181)-k(34);
jac(152,181)=jac(152,181)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(181,181)=jac(181,181)-k(32);
jac(606,181)=jac(606,181)+k(32);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(181,181)=jac(181,181)-k(24);
jac(196,181)=jac(196,181)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(181,181)=jac(181,181)-k(22);
jac(182,181)=jac(182,181)+k(22);
jac(147,181)=jac(147,181)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(181);
jac(182,181)=jac(182,181)-k(21)*y(182);
jac(181,182)=jac(181,182)-k(21)*y(181);
jac(181,181)=jac(181,181)-k(21)*y(182);
jac(183,182)=jac(183,182)+k(21)*y(181);
jac(183,181)=jac(183,181)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(181);
jac(197,181)=jac(197,181)-k(21)*y(197);
jac(181,197)=jac(181,197)-k(21)*y(181);
jac(181,181)=jac(181,181)-k(21)*y(197);
jac(367,197)=jac(367,197)+k(21)*y(181);
jac(367,181)=jac(367,181)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(181);
jac(32,181)=jac(32,181)-k(21)*y(32);
jac(181,32)=jac(181,32)-k(21)*y(181);
jac(181,181)=jac(181,181)-k(21)*y(32);
jac(364,32)=jac(364,32)+k(21)*y(181);
jac(364,181)=jac(364,181)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(181);
jac(2,181)=jac(2,181)-k(21)*y(2);
jac(181,2)=jac(181,2)-k(21)*y(181);
jac(181,181)=jac(181,181)-k(21)*y(2);
jac(175,2)=jac(175,2)+k(21)*y(181);
jac(175,181)=jac(175,181)+k(21)*y(2);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(181,181)=jac(181,181)-k(12);
jac(361,181)=jac(361,181)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(181,181)=jac(181,181)-k(10);
jac(3,181)=jac(3,181)+k(10);
jac(648,181)=jac(648,181)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(181);
jac(3,181)=jac(3,181)-k(9)*y(3);
jac(181,3)=jac(181,3)-k(9)*y(181);
jac(181,181)=jac(181,181)-k(9)*y(3);
jac(651,3)=jac(651,3)+k(9)*y(181);
jac(651,181)=jac(651,181)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(181,181)=jac(181,181)-k(8);
jac(126,181)=jac(126,181)+k(8);
jac(177,181)=jac(177,181)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(181,181)=jac(181,181)-k(6);
jac(48,181)=jac(48,181)+k(6);
jac(176,181)=jac(176,181)+k(6);

% rule    : R19
% reaction: Syk(tSH2, l~Y, a~pY) -> Syk(tSH2, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(36);
jac(2,182)=jac(2,182)+k(36);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(14);
jac(182,14)=jac(182,14)-k(21)*y(182);
jac(14,182)=jac(14,182)-k(21)*y(14);
jac(14,14)=jac(14,14)-k(21)*y(182);
jac(213,182)=jac(213,182)+k(21)*y(14);
jac(213,14)=jac(213,14)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(46);
jac(182,46)=jac(182,46)-k(21)*y(182);
jac(46,182)=jac(46,182)-k(21)*y(46);
jac(46,46)=jac(46,46)-k(21)*y(182);
jac(270,182)=jac(270,182)+k(21)*y(46);
jac(270,46)=jac(270,46)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(53);
jac(182,53)=jac(182,53)-k(21)*y(182);
jac(53,182)=jac(53,182)-k(21)*y(53);
jac(53,53)=jac(53,53)-k(21)*y(182);
jac(422,182)=jac(422,182)+k(21)*y(53);
jac(422,53)=jac(422,53)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(53);
jac(182,53)=jac(182,53)-k(21)*y(182);
jac(53,182)=jac(53,182)-k(21)*y(53);
jac(53,53)=jac(53,53)-k(21)*y(182);
jac(225,182)=jac(225,182)+k(21)*y(53);
jac(225,53)=jac(225,53)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(60);
jac(182,60)=jac(182,60)-k(21)*y(182);
jac(60,182)=jac(60,182)-k(21)*y(60);
jac(60,60)=jac(60,60)-k(21)*y(182);
jac(298,182)=jac(298,182)+k(21)*y(60);
jac(298,60)=jac(298,60)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(75);
jac(182,75)=jac(182,75)-k(21)*y(182);
jac(75,182)=jac(75,182)-k(21)*y(75);
jac(75,75)=jac(75,75)-k(21)*y(182);
jac(300,182)=jac(300,182)+k(21)*y(75);
jac(300,75)=jac(300,75)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(97);
jac(182,97)=jac(182,97)-k(21)*y(182);
jac(97,182)=jac(97,182)-k(21)*y(97);
jac(97,97)=jac(97,97)-k(21)*y(182);
jac(418,182)=jac(418,182)+k(21)*y(97);
jac(418,97)=jac(418,97)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(117);
jac(182,117)=jac(182,117)-k(21)*y(182);
jac(117,182)=jac(117,182)-k(21)*y(117);
jac(117,117)=jac(117,117)-k(21)*y(182);
jac(593,182)=jac(593,182)+k(21)*y(117);
jac(593,117)=jac(593,117)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(125);
jac(182,125)=jac(182,125)-k(21)*y(182);
jac(125,182)=jac(125,182)-k(21)*y(125);
jac(125,125)=jac(125,125)-k(21)*y(182);
jac(327,182)=jac(327,182)+k(21)*y(125);
jac(327,125)=jac(327,125)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(146);
jac(182,146)=jac(182,146)-k(21)*y(182);
jac(146,182)=jac(146,182)-k(21)*y(146);
jac(146,146)=jac(146,146)-k(21)*y(182);
jac(652,182)=jac(652,182)+k(21)*y(146);
jac(652,146)=jac(652,146)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(147);
jac(182,147)=jac(182,147)-k(21)*y(182);
jac(147,182)=jac(147,182)-k(21)*y(147);
jac(147,147)=jac(147,147)-k(21)*y(182);
jac(181,182)=jac(181,182)+k(21)*y(147);
jac(181,147)=jac(181,147)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(147);
jac(182,147)=jac(182,147)-k(21)*y(182);
jac(147,182)=jac(147,182)-k(21)*y(147);
jac(147,147)=jac(147,147)-k(21)*y(182);
jac(189,182)=jac(189,182)+k(21)*y(147);
jac(189,147)=jac(189,147)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(154);
jac(182,154)=jac(182,154)-k(21)*y(182);
jac(154,182)=jac(154,182)-k(21)*y(154);
jac(154,154)=jac(154,154)-k(21)*y(182);
jac(364,182)=jac(364,182)+k(21)*y(154);
jac(364,154)=jac(364,154)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(233);
jac(182,233)=jac(182,233)-k(21)*y(182);
jac(233,182)=jac(233,182)-k(21)*y(233);
jac(233,233)=jac(233,233)-k(21)*y(182);
jac(224,182)=jac(224,182)+k(21)*y(233);
jac(224,233)=jac(224,233)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(235);
jac(182,235)=jac(182,235)-k(21)*y(182);
jac(235,182)=jac(235,182)-k(21)*y(235);
jac(235,235)=jac(235,235)-k(21)*y(182);
jac(646,182)=jac(646,182)+k(21)*y(235);
jac(646,235)=jac(646,235)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(243);
jac(182,243)=jac(182,243)-k(21)*y(182);
jac(243,182)=jac(243,182)-k(21)*y(243);
jac(243,243)=jac(243,243)-k(21)*y(182);
jac(493,182)=jac(493,182)+k(21)*y(243);
jac(493,243)=jac(493,243)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(250);
jac(182,250)=jac(182,250)-k(21)*y(182);
jac(250,182)=jac(250,182)-k(21)*y(250);
jac(250,250)=jac(250,250)-k(21)*y(182);
jac(263,182)=jac(263,182)+k(21)*y(250);
jac(263,250)=jac(263,250)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(291);
jac(182,291)=jac(182,291)-k(21)*y(182);
jac(291,182)=jac(291,182)-k(21)*y(291);
jac(291,291)=jac(291,291)-k(21)*y(182);
jac(288,182)=jac(288,182)+k(21)*y(291);
jac(288,291)=jac(288,291)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(309);
jac(182,309)=jac(182,309)-k(21)*y(182);
jac(309,182)=jac(309,182)-k(21)*y(309);
jac(309,309)=jac(309,309)-k(21)*y(182);
jac(303,182)=jac(303,182)+k(21)*y(309);
jac(303,309)=jac(303,309)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(314);
jac(182,314)=jac(182,314)-k(21)*y(182);
jac(314,182)=jac(314,182)-k(21)*y(314);
jac(314,314)=jac(314,314)-k(21)*y(182);
jac(323,182)=jac(323,182)+k(21)*y(314);
jac(323,314)=jac(323,314)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(331);
jac(182,331)=jac(182,331)-k(21)*y(182);
jac(331,182)=jac(331,182)-k(21)*y(331);
jac(331,331)=jac(331,331)-k(21)*y(182);
jac(328,182)=jac(328,182)+k(21)*y(331);
jac(328,331)=jac(328,331)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(337);
jac(182,337)=jac(182,337)-k(21)*y(182);
jac(337,182)=jac(337,182)-k(21)*y(337);
jac(337,337)=jac(337,337)-k(21)*y(182);
jac(363,182)=jac(363,182)+k(21)*y(337);
jac(363,337)=jac(363,337)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(343);
jac(182,343)=jac(182,343)-k(21)*y(182);
jac(343,182)=jac(343,182)-k(21)*y(343);
jac(343,343)=jac(343,343)-k(21)*y(182);
jac(188,182)=jac(188,182)+k(21)*y(343);
jac(188,343)=jac(188,343)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(350);
jac(182,350)=jac(182,350)-k(21)*y(182);
jac(350,182)=jac(350,182)-k(21)*y(350);
jac(350,350)=jac(350,350)-k(21)*y(182);
jac(347,182)=jac(347,182)+k(21)*y(350);
jac(347,350)=jac(347,350)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(353);
jac(182,353)=jac(182,353)-k(21)*y(182);
jac(353,182)=jac(353,182)-k(21)*y(353);
jac(353,353)=jac(353,353)-k(21)*y(182);
jac(346,182)=jac(346,182)+k(21)*y(353);
jac(346,353)=jac(346,353)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(399);
jac(182,399)=jac(182,399)-k(21)*y(182);
jac(399,182)=jac(399,182)-k(21)*y(399);
jac(399,399)=jac(399,399)-k(21)*y(182);
jac(397,182)=jac(397,182)+k(21)*y(399);
jac(397,399)=jac(397,399)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(404);
jac(182,404)=jac(182,404)-k(21)*y(182);
jac(404,182)=jac(404,182)-k(21)*y(404);
jac(404,404)=jac(404,404)-k(21)*y(182);
jac(393,182)=jac(393,182)+k(21)*y(404);
jac(393,404)=jac(393,404)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(404);
jac(182,404)=jac(182,404)-k(21)*y(182);
jac(404,182)=jac(404,182)-k(21)*y(404);
jac(404,404)=jac(404,404)-k(21)*y(182);
jac(396,182)=jac(396,182)+k(21)*y(404);
jac(396,404)=jac(396,404)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(406);
jac(182,406)=jac(182,406)-k(21)*y(182);
jac(406,182)=jac(406,182)-k(21)*y(406);
jac(406,406)=jac(406,406)-k(21)*y(182);
jac(435,182)=jac(435,182)+k(21)*y(406);
jac(435,406)=jac(435,406)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(420);
jac(182,420)=jac(182,420)-k(21)*y(182);
jac(420,182)=jac(420,182)-k(21)*y(420);
jac(420,420)=jac(420,420)-k(21)*y(182);
jac(395,182)=jac(395,182)+k(21)*y(420);
jac(395,420)=jac(395,420)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(424);
jac(182,424)=jac(182,424)-k(21)*y(182);
jac(424,182)=jac(424,182)-k(21)*y(424);
jac(424,424)=jac(424,424)-k(21)*y(182);
jac(429,182)=jac(429,182)+k(21)*y(424);
jac(429,424)=jac(429,424)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(430);
jac(182,430)=jac(182,430)-k(21)*y(182);
jac(430,182)=jac(430,182)-k(21)*y(430);
jac(430,430)=jac(430,430)-k(21)*y(182);
jac(395,182)=jac(395,182)+k(21)*y(430);
jac(395,430)=jac(395,430)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(451);
jac(182,451)=jac(182,451)-k(21)*y(182);
jac(451,182)=jac(451,182)-k(21)*y(451);
jac(451,451)=jac(451,451)-k(21)*y(182);
jac(187,182)=jac(187,182)+k(21)*y(451);
jac(187,451)=jac(187,451)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(452);
jac(182,452)=jac(182,452)-k(21)*y(182);
jac(452,182)=jac(452,182)-k(21)*y(452);
jac(452,452)=jac(452,452)-k(21)*y(182);
jac(179,182)=jac(179,182)+k(21)*y(452);
jac(179,452)=jac(179,452)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(450);
jac(182,450)=jac(182,450)-k(21)*y(182);
jac(450,182)=jac(450,182)-k(21)*y(450);
jac(450,450)=jac(450,450)-k(21)*y(182);
jac(449,182)=jac(449,182)+k(21)*y(450);
jac(449,450)=jac(449,450)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(461);
jac(182,461)=jac(182,461)-k(21)*y(182);
jac(461,182)=jac(461,182)-k(21)*y(461);
jac(461,461)=jac(461,461)-k(21)*y(182);
jac(460,182)=jac(460,182)+k(21)*y(461);
jac(460,461)=jac(460,461)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(463);
jac(182,463)=jac(182,463)-k(21)*y(182);
jac(463,182)=jac(463,182)-k(21)*y(463);
jac(463,463)=jac(463,463)-k(21)*y(182);
jac(417,182)=jac(417,182)+k(21)*y(463);
jac(417,463)=jac(417,463)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(467);
jac(182,467)=jac(182,467)-k(21)*y(182);
jac(467,182)=jac(467,182)-k(21)*y(467);
jac(467,467)=jac(467,467)-k(21)*y(182);
jac(178,182)=jac(178,182)+k(21)*y(467);
jac(178,467)=jac(178,467)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(458);
jac(182,458)=jac(182,458)-k(21)*y(182);
jac(458,182)=jac(458,182)-k(21)*y(458);
jac(458,458)=jac(458,458)-k(21)*y(182);
jac(448,182)=jac(448,182)+k(21)*y(458);
jac(448,458)=jac(448,458)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(457);
jac(182,457)=jac(182,457)-k(21)*y(182);
jac(457,182)=jac(457,182)-k(21)*y(457);
jac(457,457)=jac(457,457)-k(21)*y(182);
jac(186,182)=jac(186,182)+k(21)*y(457);
jac(186,457)=jac(186,457)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(531);
jac(182,531)=jac(182,531)-k(21)*y(182);
jac(531,182)=jac(531,182)-k(21)*y(531);
jac(531,531)=jac(531,531)-k(21)*y(182);
jac(532,182)=jac(532,182)+k(21)*y(531);
jac(532,531)=jac(532,531)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(538);
jac(182,538)=jac(182,538)-k(21)*y(182);
jac(538,182)=jac(538,182)-k(21)*y(538);
jac(538,538)=jac(538,538)-k(21)*y(182);
jac(511,182)=jac(511,182)+k(21)*y(538);
jac(511,538)=jac(511,538)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(578);
jac(182,578)=jac(182,578)-k(21)*y(182);
jac(578,182)=jac(578,182)-k(21)*y(578);
jac(578,578)=jac(578,578)-k(21)*y(182);
jac(577,182)=jac(577,182)+k(21)*y(578);
jac(577,578)=jac(577,578)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(590);
jac(182,590)=jac(182,590)-k(21)*y(182);
jac(590,182)=jac(590,182)-k(21)*y(590);
jac(590,590)=jac(590,590)-k(21)*y(182);
jac(304,182)=jac(304,182)+k(21)*y(590);
jac(304,590)=jac(304,590)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(595);
jac(182,595)=jac(182,595)-k(21)*y(182);
jac(595,182)=jac(595,182)-k(21)*y(595);
jac(595,595)=jac(595,595)-k(21)*y(182);
jac(516,182)=jac(516,182)+k(21)*y(595);
jac(516,595)=jac(516,595)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(598);
jac(182,598)=jac(182,598)-k(21)*y(182);
jac(598,182)=jac(598,182)-k(21)*y(598);
jac(598,598)=jac(598,598)-k(21)*y(182);
jac(549,182)=jac(549,182)+k(21)*y(598);
jac(549,598)=jac(549,598)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(599);
jac(182,599)=jac(182,599)-k(21)*y(182);
jac(599,182)=jac(599,182)-k(21)*y(599);
jac(599,599)=jac(599,599)-k(21)*y(182);
jac(195,182)=jac(195,182)+k(21)*y(599);
jac(195,599)=jac(195,599)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(585);
jac(182,585)=jac(182,585)-k(21)*y(182);
jac(585,182)=jac(585,182)-k(21)*y(585);
jac(585,585)=jac(585,585)-k(21)*y(182);
jac(550,182)=jac(550,182)+k(21)*y(585);
jac(550,585)=jac(550,585)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(610);
jac(182,610)=jac(182,610)-k(21)*y(182);
jac(610,182)=jac(610,182)-k(21)*y(610);
jac(610,610)=jac(610,610)-k(21)*y(182);
jac(512,182)=jac(512,182)+k(21)*y(610);
jac(512,610)=jac(512,610)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(610);
jac(182,610)=jac(182,610)-k(21)*y(182);
jac(610,182)=jac(610,182)-k(21)*y(610);
jac(610,610)=jac(610,610)-k(21)*y(182);
jac(344,182)=jac(344,182)+k(21)*y(610);
jac(344,610)=jac(344,610)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(584);
jac(182,584)=jac(182,584)-k(21)*y(182);
jac(584,182)=jac(584,182)-k(21)*y(584);
jac(584,584)=jac(584,584)-k(21)*y(182);
jac(497,182)=jac(497,182)+k(21)*y(584);
jac(497,584)=jac(497,584)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(584);
jac(182,584)=jac(182,584)-k(21)*y(182);
jac(584,182)=jac(584,182)-k(21)*y(584);
jac(584,584)=jac(584,584)-k(21)*y(182);
jac(576,182)=jac(576,182)+k(21)*y(584);
jac(576,584)=jac(576,584)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(614);
jac(182,614)=jac(182,614)-k(21)*y(182);
jac(614,182)=jac(614,182)-k(21)*y(614);
jac(614,614)=jac(614,614)-k(21)*y(182);
jac(421,182)=jac(421,182)+k(21)*y(614);
jac(421,614)=jac(421,614)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(614);
jac(182,614)=jac(182,614)-k(21)*y(182);
jac(614,182)=jac(614,182)-k(21)*y(614);
jac(614,614)=jac(614,614)-k(21)*y(182);
jac(431,182)=jac(431,182)+k(21)*y(614);
jac(431,614)=jac(431,614)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(618);
jac(182,618)=jac(182,618)-k(21)*y(182);
jac(618,182)=jac(618,182)-k(21)*y(618);
jac(618,618)=jac(618,618)-k(21)*y(182);
jac(487,182)=jac(487,182)+k(21)*y(618);
jac(487,618)=jac(487,618)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(617);
jac(182,617)=jac(182,617)-k(21)*y(182);
jac(617,182)=jac(617,182)-k(21)*y(617);
jac(617,617)=jac(617,617)-k(21)*y(182);
jac(498,182)=jac(498,182)+k(21)*y(617);
jac(498,617)=jac(498,617)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(616);
jac(182,616)=jac(182,616)-k(21)*y(182);
jac(616,182)=jac(616,182)-k(21)*y(616);
jac(616,616)=jac(616,616)-k(21)*y(182);
jac(271,182)=jac(271,182)+k(21)*y(616);
jac(271,616)=jac(271,616)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(615);
jac(182,615)=jac(182,615)-k(21)*y(182);
jac(615,182)=jac(615,182)-k(21)*y(615);
jac(615,615)=jac(615,615)-k(21)*y(182);
jac(285,182)=jac(285,182)+k(21)*y(615);
jac(285,615)=jac(285,615)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(613);
jac(182,613)=jac(182,613)-k(21)*y(182);
jac(613,182)=jac(613,182)-k(21)*y(613);
jac(613,613)=jac(613,613)-k(21)*y(182);
jac(490,182)=jac(490,182)+k(21)*y(613);
jac(490,613)=jac(490,613)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(581);
jac(182,581)=jac(182,581)-k(21)*y(182);
jac(581,182)=jac(581,182)-k(21)*y(581);
jac(581,581)=jac(581,581)-k(21)*y(182);
jac(551,182)=jac(551,182)+k(21)*y(581);
jac(551,581)=jac(551,581)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(623);
jac(182,623)=jac(182,623)-k(21)*y(182);
jac(623,182)=jac(623,182)-k(21)*y(623);
jac(623,623)=jac(623,623)-k(21)*y(182);
jac(517,182)=jac(517,182)+k(21)*y(623);
jac(517,623)=jac(517,623)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(624);
jac(182,624)=jac(182,624)-k(21)*y(182);
jac(624,182)=jac(624,182)-k(21)*y(624);
jac(624,624)=jac(624,624)-k(21)*y(182);
jac(640,182)=jac(640,182)+k(21)*y(624);
jac(640,624)=jac(640,624)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(628);
jac(182,628)=jac(182,628)-k(21)*y(182);
jac(628,182)=jac(628,182)-k(21)*y(628);
jac(628,628)=jac(628,628)-k(21)*y(182);
jac(637,182)=jac(637,182)+k(21)*y(628);
jac(637,628)=jac(637,628)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(633);
jac(182,633)=jac(182,633)-k(21)*y(182);
jac(633,182)=jac(633,182)-k(21)*y(633);
jac(633,633)=jac(633,633)-k(21)*y(182);
jac(639,182)=jac(639,182)+k(21)*y(633);
jac(639,633)=jac(639,633)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(634);
jac(182,634)=jac(182,634)-k(21)*y(182);
jac(634,182)=jac(634,182)-k(21)*y(634);
jac(634,634)=jac(634,634)-k(21)*y(182);
jac(286,182)=jac(286,182)+k(21)*y(634);
jac(286,634)=jac(286,634)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(638);
jac(182,638)=jac(182,638)-k(21)*y(182);
jac(638,182)=jac(638,182)-k(21)*y(638);
jac(638,638)=jac(638,638)-k(21)*y(182);
jac(640,182)=jac(640,182)+k(21)*y(638);
jac(640,638)=jac(640,638)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(645);
jac(182,645)=jac(182,645)-k(21)*y(182);
jac(645,182)=jac(645,182)-k(21)*y(645);
jac(645,645)=jac(645,645)-k(21)*y(182);
jac(268,182)=jac(268,182)+k(21)*y(645);
jac(268,645)=jac(268,645)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(644);
jac(182,644)=jac(182,644)-k(21)*y(182);
jac(644,182)=jac(644,182)-k(21)*y(644);
jac(644,644)=jac(644,644)-k(21)*y(182);
jac(517,182)=jac(517,182)+k(21)*y(644);
jac(517,644)=jac(517,644)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(636);
jac(182,636)=jac(182,636)-k(21)*y(182);
jac(636,182)=jac(636,182)-k(21)*y(636);
jac(636,636)=jac(636,636)-k(21)*y(182);
jac(489,182)=jac(489,182)+k(21)*y(636);
jac(489,636)=jac(489,636)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(629);
jac(182,629)=jac(182,629)-k(21)*y(182);
jac(629,182)=jac(629,182)-k(21)*y(629);
jac(629,629)=jac(629,629)-k(21)*y(182);
jac(281,182)=jac(281,182)+k(21)*y(629);
jac(281,629)=jac(281,629)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(627);
jac(182,627)=jac(182,627)-k(21)*y(182);
jac(627,182)=jac(627,182)-k(21)*y(627);
jac(627,627)=jac(627,627)-k(21)*y(182);
jac(624,182)=jac(624,182)+k(21)*y(627);
jac(624,627)=jac(624,627)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(627);
jac(182,627)=jac(182,627)-k(21)*y(182);
jac(627,182)=jac(627,182)-k(21)*y(627);
jac(627,627)=jac(627,627)-k(21)*y(182);
jac(638,182)=jac(638,182)+k(21)*y(627);
jac(638,627)=jac(638,627)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(622);
jac(182,622)=jac(182,622)-k(21)*y(182);
jac(622,182)=jac(622,182)-k(21)*y(622);
jac(622,622)=jac(622,622)-k(21)*y(182);
jac(641,182)=jac(641,182)+k(21)*y(622);
jac(641,622)=jac(641,622)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(576);
jac(182,576)=jac(182,576)-k(21)*y(182);
jac(576,182)=jac(576,182)-k(21)*y(576);
jac(576,576)=jac(576,576)-k(21)*y(182);
jac(579,182)=jac(579,182)+k(21)*y(576);
jac(579,576)=jac(579,576)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(573);
jac(182,573)=jac(182,573)-k(21)*y(182);
jac(573,182)=jac(573,182)-k(21)*y(573);
jac(573,573)=jac(573,573)-k(21)*y(182);
jac(574,182)=jac(574,182)+k(21)*y(573);
jac(574,573)=jac(574,573)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(564);
jac(182,564)=jac(182,564)-k(21)*y(182);
jac(564,182)=jac(564,182)-k(21)*y(564);
jac(564,564)=jac(564,564)-k(21)*y(182);
jac(626,182)=jac(626,182)+k(21)*y(564);
jac(626,564)=jac(626,564)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(563);
jac(182,563)=jac(182,563)-k(21)*y(182);
jac(563,182)=jac(563,182)-k(21)*y(563);
jac(563,563)=jac(563,563)-k(21)*y(182);
jac(555,182)=jac(555,182)+k(21)*y(563);
jac(555,563)=jac(555,563)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(556);
jac(182,556)=jac(182,556)-k(21)*y(182);
jac(556,182)=jac(556,182)-k(21)*y(556);
jac(556,556)=jac(556,556)-k(21)*y(182);
jac(553,182)=jac(553,182)+k(21)*y(556);
jac(553,556)=jac(553,556)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(547);
jac(182,547)=jac(182,547)-k(21)*y(182);
jac(547,182)=jac(547,182)-k(21)*y(547);
jac(547,547)=jac(547,547)-k(21)*y(182);
jac(548,182)=jac(548,182)+k(21)*y(547);
jac(548,547)=jac(548,547)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(540);
jac(182,540)=jac(182,540)-k(21)*y(182);
jac(540,182)=jac(540,182)-k(21)*y(540);
jac(540,540)=jac(540,540)-k(21)*y(182);
jac(625,182)=jac(625,182)+k(21)*y(540);
jac(625,540)=jac(625,540)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(539);
jac(182,539)=jac(182,539)-k(21)*y(182);
jac(539,182)=jac(539,182)-k(21)*y(539);
jac(539,539)=jac(539,539)-k(21)*y(182);
jac(596,182)=jac(596,182)+k(21)*y(539);
jac(596,539)=jac(596,539)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(537);
jac(182,537)=jac(182,537)-k(21)*y(182);
jac(537,182)=jac(537,182)-k(21)*y(537);
jac(537,537)=jac(537,537)-k(21)*y(182);
jac(518,182)=jac(518,182)+k(21)*y(537);
jac(518,537)=jac(518,537)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(537);
jac(182,537)=jac(182,537)-k(21)*y(182);
jac(537,182)=jac(537,182)-k(21)*y(537);
jac(537,537)=jac(537,537)-k(21)*y(182);
jac(519,182)=jac(519,182)+k(21)*y(537);
jac(519,537)=jac(519,537)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(534);
jac(182,534)=jac(182,534)-k(21)*y(182);
jac(534,182)=jac(534,182)-k(21)*y(534);
jac(534,534)=jac(534,534)-k(21)*y(182);
jac(496,182)=jac(496,182)+k(21)*y(534);
jac(496,534)=jac(496,534)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(533);
jac(182,533)=jac(182,533)-k(21)*y(182);
jac(533,182)=jac(533,182)-k(21)*y(533);
jac(533,533)=jac(533,533)-k(21)*y(182);
jac(359,182)=jac(359,182)+k(21)*y(533);
jac(359,533)=jac(359,533)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(529);
jac(182,529)=jac(182,529)-k(21)*y(182);
jac(529,182)=jac(529,182)-k(21)*y(529);
jac(529,529)=jac(529,529)-k(21)*y(182);
jac(520,182)=jac(520,182)+k(21)*y(529);
jac(520,529)=jac(520,529)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(524);
jac(182,524)=jac(182,524)-k(21)*y(182);
jac(524,182)=jac(524,182)-k(21)*y(524);
jac(524,524)=jac(524,524)-k(21)*y(182);
jac(521,182)=jac(521,182)+k(21)*y(524);
jac(521,524)=jac(521,524)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(519);
jac(182,519)=jac(182,519)-k(21)*y(182);
jac(519,182)=jac(519,182)-k(21)*y(519);
jac(519,519)=jac(519,519)-k(21)*y(182);
jac(514,182)=jac(514,182)+k(21)*y(519);
jac(514,519)=jac(514,519)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(518);
jac(182,518)=jac(182,518)-k(21)*y(182);
jac(518,182)=jac(518,182)-k(21)*y(518);
jac(518,518)=jac(518,518)-k(21)*y(182);
jac(514,182)=jac(514,182)+k(21)*y(518);
jac(514,518)=jac(514,518)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(512);
jac(182,512)=jac(182,512)-k(21)*y(182);
jac(512,182)=jac(512,182)-k(21)*y(512);
jac(512,512)=jac(512,512)-k(21)*y(182);
jac(513,182)=jac(513,182)+k(21)*y(512);
jac(513,512)=jac(513,512)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(648);
jac(182,648)=jac(182,648)-k(21)*y(182);
jac(648,182)=jac(648,182)-k(21)*y(648);
jac(648,648)=jac(648,648)-k(21)*y(182);
jac(187,182)=jac(187,182)+k(21)*y(648);
jac(187,648)=jac(187,648)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(508);
jac(182,508)=jac(182,508)-k(21)*y(182);
jac(508,182)=jac(508,182)-k(21)*y(508);
jac(508,508)=jac(508,508)-k(21)*y(182);
jac(510,182)=jac(510,182)+k(21)*y(508);
jac(510,508)=jac(510,508)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(501);
jac(182,501)=jac(182,501)-k(21)*y(182);
jac(501,182)=jac(501,182)-k(21)*y(501);
jac(501,501)=jac(501,501)-k(21)*y(182);
jac(505,182)=jac(505,182)+k(21)*y(501);
jac(505,501)=jac(505,501)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(497);
jac(182,497)=jac(182,497)-k(21)*y(182);
jac(497,182)=jac(497,182)-k(21)*y(497);
jac(497,497)=jac(497,497)-k(21)*y(182);
jac(579,182)=jac(579,182)+k(21)*y(497);
jac(579,497)=jac(579,497)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(491);
jac(182,491)=jac(182,491)-k(21)*y(182);
jac(491,182)=jac(491,182)-k(21)*y(491);
jac(491,491)=jac(491,491)-k(21)*y(182);
jac(488,182)=jac(488,182)+k(21)*y(491);
jac(488,491)=jac(488,491)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(470);
jac(182,470)=jac(182,470)-k(21)*y(182);
jac(470,182)=jac(470,182)-k(21)*y(470);
jac(470,470)=jac(470,470)-k(21)*y(182);
jac(423,182)=jac(423,182)+k(21)*y(470);
jac(423,470)=jac(423,470)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(443);
jac(182,443)=jac(182,443)-k(21)*y(182);
jac(443,182)=jac(443,182)-k(21)*y(443);
jac(443,443)=jac(443,443)-k(21)*y(182);
jac(432,182)=jac(432,182)+k(21)*y(443);
jac(432,443)=jac(432,443)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(431);
jac(182,431)=jac(182,431)-k(21)*y(182);
jac(431,182)=jac(431,182)-k(21)*y(431);
jac(431,431)=jac(431,431)-k(21)*y(182);
jac(580,182)=jac(580,182)+k(21)*y(431);
jac(580,431)=jac(580,431)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(425);
jac(182,425)=jac(182,425)-k(21)*y(182);
jac(425,182)=jac(425,182)-k(21)*y(425);
jac(425,425)=jac(425,425)-k(21)*y(182);
jac(280,182)=jac(280,182)+k(21)*y(425);
jac(280,425)=jac(280,425)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(422);
jac(182,422)=jac(182,422)-k(21)*y(182);
jac(422,182)=jac(422,182)-k(21)*y(422);
jac(422,422)=jac(422,422)-k(21)*y(182);
jac(649,182)=jac(649,182)+k(21)*y(422);
jac(649,422)=jac(649,422)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(421);
jac(182,421)=jac(182,421)-k(21)*y(182);
jac(421,182)=jac(421,182)-k(21)*y(421);
jac(421,421)=jac(421,421)-k(21)*y(182);
jac(580,182)=jac(580,182)+k(21)*y(421);
jac(580,421)=jac(580,421)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(651);
jac(182,651)=jac(182,651)-k(21)*y(182);
jac(651,182)=jac(651,182)-k(21)*y(651);
jac(651,651)=jac(651,651)-k(21)*y(182);
jac(186,182)=jac(186,182)+k(21)*y(651);
jac(186,651)=jac(186,651)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(410);
jac(182,410)=jac(182,410)-k(21)*y(182);
jac(410,182)=jac(410,182)-k(21)*y(410);
jac(410,410)=jac(410,410)-k(21)*y(182);
jac(433,182)=jac(433,182)+k(21)*y(410);
jac(433,410)=jac(433,410)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(409);
jac(182,409)=jac(182,409)-k(21)*y(182);
jac(409,182)=jac(409,182)-k(21)*y(409);
jac(409,409)=jac(409,409)-k(21)*y(182);
jac(194,182)=jac(194,182)+k(21)*y(409);
jac(194,409)=jac(194,409)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(408);
jac(182,408)=jac(182,408)-k(21)*y(182);
jac(408,182)=jac(408,182)-k(21)*y(408);
jac(408,408)=jac(408,408)-k(21)*y(182);
jac(193,182)=jac(193,182)+k(21)*y(408);
jac(193,408)=jac(193,408)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(405);
jac(182,405)=jac(182,405)-k(21)*y(182);
jac(405,182)=jac(405,182)-k(21)*y(405);
jac(405,405)=jac(405,405)-k(21)*y(182);
jac(447,182)=jac(447,182)+k(21)*y(405);
jac(447,405)=jac(447,405)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(401);
jac(182,401)=jac(182,401)-k(21)*y(182);
jac(401,182)=jac(401,182)-k(21)*y(401);
jac(401,401)=jac(401,401)-k(21)*y(182);
jac(398,182)=jac(398,182)+k(21)*y(401);
jac(398,401)=jac(398,401)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(400);
jac(182,400)=jac(182,400)-k(21)*y(182);
jac(400,182)=jac(400,182)-k(21)*y(400);
jac(400,400)=jac(400,400)-k(21)*y(182);
jac(368,182)=jac(368,182)+k(21)*y(400);
jac(368,400)=jac(368,400)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(396);
jac(182,396)=jac(182,396)-k(21)*y(182);
jac(396,182)=jac(396,182)-k(21)*y(396);
jac(396,396)=jac(396,396)-k(21)*y(182);
jac(394,182)=jac(394,182)+k(21)*y(396);
jac(394,396)=jac(394,396)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(393);
jac(182,393)=jac(182,393)-k(21)*y(182);
jac(393,182)=jac(393,182)-k(21)*y(393);
jac(393,393)=jac(393,393)-k(21)*y(182);
jac(394,182)=jac(394,182)+k(21)*y(393);
jac(394,393)=jac(394,393)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(388);
jac(182,388)=jac(182,388)-k(21)*y(182);
jac(388,182)=jac(388,182)-k(21)*y(388);
jac(388,388)=jac(388,388)-k(21)*y(182);
jac(390,182)=jac(390,182)+k(21)*y(388);
jac(390,388)=jac(390,388)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(384);
jac(182,384)=jac(182,384)-k(21)*y(182);
jac(384,182)=jac(384,182)-k(21)*y(384);
jac(384,384)=jac(384,384)-k(21)*y(182);
jac(385,182)=jac(385,182)+k(21)*y(384);
jac(385,384)=jac(385,384)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(383);
jac(182,383)=jac(182,383)-k(21)*y(182);
jac(383,182)=jac(383,182)-k(21)*y(383);
jac(383,383)=jac(383,383)-k(21)*y(182);
jac(389,182)=jac(389,182)+k(21)*y(383);
jac(389,383)=jac(389,383)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(376);
jac(182,376)=jac(182,376)-k(21)*y(182);
jac(376,182)=jac(376,182)-k(21)*y(376);
jac(376,376)=jac(376,376)-k(21)*y(182);
jac(369,182)=jac(369,182)+k(21)*y(376);
jac(369,376)=jac(369,376)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(375);
jac(182,375)=jac(182,375)-k(21)*y(182);
jac(375,182)=jac(375,182)-k(21)*y(375);
jac(375,375)=jac(375,375)-k(21)*y(182);
jac(377,182)=jac(377,182)+k(21)*y(375);
jac(377,375)=jac(377,375)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(361);
jac(182,361)=jac(182,361)-k(21)*y(182);
jac(361,182)=jac(361,182)-k(21)*y(361);
jac(361,361)=jac(361,361)-k(21)*y(182);
jac(188,182)=jac(188,182)+k(21)*y(361);
jac(188,361)=jac(188,361)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(354);
jac(182,354)=jac(182,354)-k(21)*y(182);
jac(354,182)=jac(354,182)-k(21)*y(354);
jac(354,354)=jac(354,354)-k(21)*y(182);
jac(509,182)=jac(509,182)+k(21)*y(354);
jac(509,354)=jac(509,354)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(352);
jac(182,352)=jac(182,352)-k(21)*y(182);
jac(352,182)=jac(352,182)-k(21)*y(352);
jac(352,352)=jac(352,352)-k(21)*y(182);
jac(349,182)=jac(349,182)+k(21)*y(352);
jac(349,352)=jac(349,352)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(351);
jac(182,351)=jac(182,351)-k(21)*y(182);
jac(351,182)=jac(351,182)-k(21)*y(351);
jac(351,351)=jac(351,351)-k(21)*y(182);
jac(348,182)=jac(348,182)+k(21)*y(351);
jac(348,351)=jac(348,351)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(345);
jac(182,345)=jac(182,345)-k(21)*y(182);
jac(345,182)=jac(345,182)-k(21)*y(345);
jac(345,345)=jac(345,345)-k(21)*y(182);
jac(516,182)=jac(516,182)+k(21)*y(345);
jac(516,345)=jac(516,345)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(344);
jac(182,344)=jac(182,344)-k(21)*y(182);
jac(344,182)=jac(344,182)-k(21)*y(344);
jac(344,344)=jac(344,344)-k(21)*y(182);
jac(513,182)=jac(513,182)+k(21)*y(344);
jac(513,344)=jac(513,344)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(339);
jac(182,339)=jac(182,339)-k(21)*y(182);
jac(339,182)=jac(339,182)-k(21)*y(339);
jac(339,339)=jac(339,339)-k(21)*y(182);
jac(597,182)=jac(597,182)+k(21)*y(339);
jac(597,339)=jac(597,339)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(338);
jac(182,338)=jac(182,338)-k(21)*y(182);
jac(338,182)=jac(338,182)-k(21)*y(338);
jac(338,338)=jac(338,338)-k(21)*y(182);
jac(611,182)=jac(611,182)+k(21)*y(338);
jac(611,338)=jac(611,338)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(315);
jac(182,315)=jac(182,315)-k(21)*y(182);
jac(315,182)=jac(315,182)-k(21)*y(315);
jac(315,315)=jac(315,315)-k(21)*y(182);
jac(219,182)=jac(219,182)+k(21)*y(315);
jac(219,315)=jac(219,315)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(310);
jac(182,310)=jac(182,310)-k(21)*y(182);
jac(310,182)=jac(310,182)-k(21)*y(310);
jac(310,310)=jac(310,310)-k(21)*y(182);
jac(623,182)=jac(623,182)+k(21)*y(310);
jac(623,310)=jac(623,310)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(310);
jac(182,310)=jac(182,310)-k(21)*y(182);
jac(310,182)=jac(310,182)-k(21)*y(310);
jac(310,310)=jac(310,310)-k(21)*y(182);
jac(644,182)=jac(644,182)+k(21)*y(310);
jac(644,310)=jac(644,310)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(308);
jac(182,308)=jac(182,308)-k(21)*y(182);
jac(308,182)=jac(308,182)-k(21)*y(308);
jac(308,308)=jac(308,308)-k(21)*y(182);
jac(523,182)=jac(523,182)+k(21)*y(308);
jac(523,308)=jac(523,308)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(292);
jac(182,292)=jac(182,292)-k(21)*y(182);
jac(292,182)=jac(292,182)-k(21)*y(292);
jac(292,292)=jac(292,292)-k(21)*y(182);
jac(290,182)=jac(290,182)+k(21)*y(292);
jac(290,292)=jac(290,292)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(287);
jac(182,287)=jac(182,287)-k(21)*y(182);
jac(287,182)=jac(287,182)-k(21)*y(287);
jac(287,287)=jac(287,287)-k(21)*y(182);
jac(283,182)=jac(283,182)+k(21)*y(287);
jac(283,287)=jac(283,287)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(282);
jac(182,282)=jac(182,282)-k(21)*y(182);
jac(282,182)=jac(282,182)-k(21)*y(282);
jac(282,282)=jac(282,282)-k(21)*y(182);
jac(273,182)=jac(273,182)+k(21)*y(282);
jac(273,282)=jac(273,282)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(182,182)=jac(182,182)-k(21)*y(276);
jac(182,276)=jac(182,276)-k(21)*y(182);
jac(276,182)=jac(276,182)-k(21)*y(276);
jac(276,276)=jac(276,276)-k(21)*y(182);
jac(278,182)=jac(278,182)+k(21)*y(276);
jac(278,276)=jac(278,276)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(272);
jac(182,272)=jac(182,272)-k(21)*y(182);
jac(272,182)=jac(272,182)-k(21)*y(272);
jac(272,272)=jac(272,272)-k(21)*y(182);
jac(273,182)=jac(273,182)+k(21)*y(272);
jac(273,272)=jac(273,272)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(267);
jac(182,267)=jac(182,267)-k(21)*y(182);
jac(267,182)=jac(267,182)-k(21)*y(267);
jac(267,267)=jac(267,267)-k(21)*y(182);
jac(268,182)=jac(268,182)+k(21)*y(267);
jac(268,267)=jac(268,267)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(260);
jac(182,260)=jac(182,260)-k(21)*y(182);
jac(260,182)=jac(260,182)-k(21)*y(260);
jac(260,260)=jac(260,260)-k(21)*y(182);
jac(261,182)=jac(261,182)+k(21)*y(260);
jac(261,260)=jac(261,260)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(252);
jac(182,252)=jac(182,252)-k(21)*y(182);
jac(252,182)=jac(252,182)-k(21)*y(252);
jac(252,252)=jac(252,252)-k(21)*y(182);
jac(266,182)=jac(266,182)+k(21)*y(252);
jac(266,252)=jac(266,252)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(251);
jac(182,251)=jac(182,251)-k(21)*y(182);
jac(251,182)=jac(251,182)-k(21)*y(251);
jac(251,251)=jac(251,251)-k(21)*y(182);
jac(265,182)=jac(265,182)+k(21)*y(251);
jac(265,251)=jac(265,251)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(249);
jac(182,249)=jac(182,249)-k(21)*y(182);
jac(249,182)=jac(249,182)-k(21)*y(249);
jac(249,249)=jac(249,249)-k(21)*y(182);
jac(479,182)=jac(479,182)+k(21)*y(249);
jac(479,249)=jac(479,249)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(245);
jac(182,245)=jac(182,245)-k(21)*y(182);
jac(245,182)=jac(245,182)-k(21)*y(245);
jac(245,245)=jac(245,245)-k(21)*y(182);
jac(495,182)=jac(495,182)+k(21)*y(245);
jac(495,245)=jac(495,245)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(182,182)=jac(182,182)-k(21)*y(244);
jac(182,244)=jac(182,244)-k(21)*y(182);
jac(244,182)=jac(244,182)-k(21)*y(244);
jac(244,244)=jac(244,244)-k(21)*y(182);
jac(494,182)=jac(494,182)+k(21)*y(244);
jac(494,244)=jac(494,244)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(238);
jac(182,238)=jac(182,238)-k(21)*y(182);
jac(238,182)=jac(238,182)-k(21)*y(238);
jac(238,238)=jac(238,238)-k(21)*y(182);
jac(267,182)=jac(267,182)+k(21)*y(238);
jac(267,238)=jac(267,238)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(238);
jac(182,238)=jac(182,238)-k(21)*y(182);
jac(238,182)=jac(238,182)-k(21)*y(238);
jac(238,238)=jac(238,238)-k(21)*y(182);
jac(645,182)=jac(645,182)+k(21)*y(238);
jac(645,238)=jac(645,238)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(237);
jac(182,237)=jac(182,237)-k(21)*y(182);
jac(237,182)=jac(237,182)-k(21)*y(237);
jac(237,237)=jac(237,237)-k(21)*y(182);
jac(325,182)=jac(325,182)+k(21)*y(237);
jac(325,237)=jac(325,237)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(182,182)=jac(182,182)-k(21)*y(236);
jac(182,236)=jac(182,236)-k(21)*y(182);
jac(236,182)=jac(236,182)-k(21)*y(236);
jac(236,236)=jac(236,236)-k(21)*y(182);
jac(324,182)=jac(324,182)+k(21)*y(236);
jac(324,236)=jac(324,236)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(234);
jac(182,234)=jac(182,234)-k(21)*y(182);
jac(234,182)=jac(234,182)-k(21)*y(234);
jac(234,234)=jac(234,234)-k(21)*y(182);
jac(220,182)=jac(220,182)+k(21)*y(234);
jac(220,234)=jac(220,234)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(232);
jac(182,232)=jac(182,232)-k(21)*y(182);
jac(232,182)=jac(232,182)-k(21)*y(232);
jac(232,232)=jac(232,232)-k(21)*y(182);
jac(486,182)=jac(486,182)+k(21)*y(232);
jac(486,232)=jac(486,232)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(225);
jac(182,225)=jac(182,225)-k(21)*y(182);
jac(225,182)=jac(225,182)-k(21)*y(225);
jac(225,225)=jac(225,225)-k(21)*y(182);
jac(649,182)=jac(649,182)+k(21)*y(225);
jac(649,225)=jac(649,225)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(206);
jac(182,206)=jac(182,206)-k(21)*y(182);
jac(206,182)=jac(206,182)-k(21)*y(206);
jac(206,206)=jac(206,206)-k(21)*y(182);
jac(289,182)=jac(289,182)+k(21)*y(206);
jac(289,206)=jac(289,206)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(182,182)=jac(182,182)-k(21)*y(205);
jac(182,205)=jac(182,205)-k(21)*y(182);
jac(205,182)=jac(205,182)-k(21)*y(205);
jac(205,205)=jac(205,205)-k(21)*y(182);
jac(476,182)=jac(476,182)+k(21)*y(205);
jac(476,205)=jac(476,205)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(198);
jac(182,198)=jac(182,198)-k(21)*y(182);
jac(198,182)=jac(198,182)-k(21)*y(198);
jac(198,198)=jac(198,198)-k(21)*y(182);
jac(367,182)=jac(367,182)+k(21)*y(198);
jac(367,198)=jac(367,198)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(196);
jac(182,196)=jac(182,196)-k(21)*y(182);
jac(196,182)=jac(196,182)-k(21)*y(196);
jac(196,196)=jac(196,196)-k(21)*y(182);
jac(190,182)=jac(190,182)+k(21)*y(196);
jac(190,196)=jac(190,196)+k(21)*y(182);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(182,182)=jac(182,182)-k(21)*y(189);
jac(182,189)=jac(182,189)-k(21)*y(182);
jac(189,182)=jac(189,182)-k(21)*y(189);
jac(189,189)=jac(189,189)-k(21)*y(182);
jac(183,182)=jac(183,182)+k(21)*y(189);
jac(183,189)=jac(183,189)+k(21)*y(182);

% rule    : R17
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(184,184)=jac(184,184)-k(34);
jac(140,184)=jac(140,184)+k(34);

% rule    : R9_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(184,184)=jac(184,184)-k(22);
jac(182,184)=jac(182,184)+k(22);
jac(48,184)=jac(48,184)+k(22);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lyn(U, SH2) 

jac(184,184)=jac(184,184)-k(10);
jac(3,184)=jac(3,184)+k(10);
jac(177,184)=jac(177,184)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(7)*y(185);
jac(184,185)=jac(184,185)-k(7)*y(184);
jac(185,184)=jac(185,184)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(184);
jac(186,184)=jac(186,184)+k(7)*y(185);
jac(186,185)=jac(186,185)+k(7)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(209,209)=jac(209,209)-k(7)*y(184);
jac(209,184)=jac(209,184)-k(7)*y(209);
jac(184,209)=jac(184,209)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(209);
jac(397,209)=jac(397,209)+k(7)*y(184);
jac(397,184)=jac(397,184)+k(7)*y(209);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(214,214)=jac(214,214)-k(7)*y(184);
jac(214,184)=jac(214,184)-k(7)*y(214);
jac(184,214)=jac(184,214)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(214);
jac(394,214)=jac(394,214)+k(7)*y(184);
jac(394,184)=jac(394,184)+k(7)*y(214);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(219,219)=jac(219,219)-k(7)*y(184);
jac(219,184)=jac(219,184)-k(7)*y(219);
jac(184,219)=jac(184,219)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(219);
jac(395,219)=jac(395,219)+k(7)*y(184);
jac(395,184)=jac(395,184)+k(7)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(7)*y(223);
jac(184,223)=jac(184,223)-k(7)*y(184);
jac(223,184)=jac(223,184)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(184);
jac(649,184)=jac(649,184)+k(7)*y(223);
jac(649,223)=jac(649,223)+k(7)*y(184);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(7)*y(228);
jac(184,228)=jac(184,228)-k(7)*y(184);
jac(228,184)=jac(228,184)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(184);
jac(476,184)=jac(476,184)+k(7)*y(228);
jac(476,228)=jac(476,228)+k(7)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(315,315)=jac(315,315)-k(7)*y(184);
jac(315,184)=jac(315,184)-k(7)*y(315);
jac(184,315)=jac(184,315)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(315);
jac(420,315)=jac(420,315)+k(7)*y(184);
jac(420,184)=jac(420,184)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(319,319)=jac(319,319)-k(7)*y(184);
jac(319,184)=jac(319,184)-k(7)*y(319);
jac(184,319)=jac(184,319)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(319);
jac(398,319)=jac(398,319)+k(7)*y(184);
jac(398,184)=jac(398,184)+k(7)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(7)*y(336);
jac(184,336)=jac(184,336)-k(7)*y(184);
jac(336,184)=jac(336,184)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(184);
jac(368,184)=jac(368,184)+k(7)*y(336);
jac(368,336)=jac(368,336)+k(7)*y(184);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(437,437)=jac(437,437)-k(7)*y(184);
jac(437,184)=jac(437,184)-k(7)*y(437);
jac(184,437)=jac(184,437)-k(7)*y(184);
jac(184,184)=jac(184,184)-k(7)*y(437);
jac(392,437)=jac(392,437)+k(7)*y(184);
jac(392,184)=jac(392,184)+k(7)*y(437);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(191,191)=jac(191,191)-k(5)*y(184);
jac(191,184)=jac(191,184)-k(5)*y(191);
jac(184,191)=jac(184,191)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(191);
jac(190,191)=jac(190,191)+k(5)*y(184);
jac(190,184)=jac(190,184)+k(5)*y(191);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(256,256)=jac(256,256)-k(5)*y(184);
jac(256,184)=jac(256,184)-k(5)*y(256);
jac(184,256)=jac(184,256)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(256);
jac(195,256)=jac(195,256)+k(5)*y(184);
jac(195,184)=jac(195,184)+k(5)*y(256);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(264,264)=jac(264,264)-k(5)*y(184);
jac(264,184)=jac(264,184)-k(5)*y(264);
jac(184,264)=jac(184,264)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(264);
jac(188,264)=jac(188,264)+k(5)*y(184);
jac(188,184)=jac(188,184)+k(5)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(5)*y(380);
jac(184,380)=jac(184,380)-k(5)*y(184);
jac(380,184)=jac(380,184)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(184);
jac(460,184)=jac(460,184)+k(5)*y(380);
jac(460,380)=jac(460,380)+k(5)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(5)*y(391);
jac(184,391)=jac(184,391)-k(5)*y(184);
jac(391,184)=jac(391,184)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(184);
jac(186,184)=jac(186,184)+k(5)*y(391);
jac(186,391)=jac(186,391)+k(5)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(5)*y(407);
jac(184,407)=jac(184,407)-k(5)*y(184);
jac(407,184)=jac(407,184)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(184);
jac(193,184)=jac(193,184)+k(5)*y(407);
jac(193,407)=jac(193,407)+k(5)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(5)*y(441);
jac(184,441)=jac(184,441)-k(5)*y(184);
jac(441,184)=jac(441,184)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(184);
jac(448,184)=jac(448,184)+k(5)*y(441);
jac(448,441)=jac(448,441)+k(5)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(5)*y(515);
jac(184,515)=jac(184,515)-k(5)*y(184);
jac(515,184)=jac(515,184)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(184);
jac(516,184)=jac(516,184)+k(5)*y(515);
jac(516,515)=jac(516,515)+k(5)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(184,184)=jac(184,184)-k(5)*y(543);
jac(184,543)=jac(184,543)-k(5)*y(184);
jac(543,184)=jac(543,184)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(184);
jac(549,184)=jac(549,184)+k(5)*y(543);
jac(549,543)=jac(549,543)+k(5)*y(184);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(184,184)=jac(184,184)-k(5)*y(554);
jac(184,554)=jac(184,554)-k(5)*y(184);
jac(554,184)=jac(554,184)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(184);
jac(349,184)=jac(349,184)+k(5)*y(554);
jac(349,554)=jac(349,554)+k(5)*y(184);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(9,9)=jac(9,9)-k(5)*y(184);
jac(9,184)=jac(9,184)-k(5)*y(9);
jac(184,9)=jac(184,9)-k(5)*y(184);
jac(184,184)=jac(184,184)-k(5)*y(9);
jac(215,9)=jac(215,9)+k(5)*y(184);
jac(215,184)=jac(215,184)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(4,4)=jac(4,4)-k(3)*y(184);
jac(4,184)=jac(4,184)-k(3)*y(4);
jac(184,4)=jac(184,4)-k(3)*y(184);
jac(184,184)=jac(184,184)-k(3)*y(4);
jac(391,4)=jac(391,4)+k(3)*y(184);
jac(391,184)=jac(391,184)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(4,4)=jac(4,4)-k(1)*y(184);
jac(4,184)=jac(4,184)-k(1)*y(4);
jac(184,4)=jac(184,4)-k(1)*y(184);
jac(184,184)=jac(184,184)-k(1)*y(4);
jac(185,4)=jac(185,4)+k(1)*y(184);
jac(185,184)=jac(185,184)+k(1)*y(4);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(185,185)=jac(185,185)-k(34);
jac(141,185)=jac(141,185)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(185,185)=jac(185,185)-k(22);
jac(182,185)=jac(182,185)+k(22);
jac(126,185)=jac(126,185)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(185,185)=jac(185,185)-k(10);
jac(3,185)=jac(3,185)+k(10);
jac(214,185)=jac(214,185)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(1,1)=jac(1,1)-k(7)*y(185);
jac(1,185)=jac(1,185)-k(7)*y(1);
jac(185,1)=jac(185,1)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(1);
jac(652,1)=jac(652,1)+k(7)*y(185);
jac(652,185)=jac(652,185)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(7)*y(185);
jac(7,185)=jac(7,185)-k(7)*y(7);
jac(185,7)=jac(185,7)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(7);
jac(215,7)=jac(215,7)+k(7)*y(185);
jac(215,185)=jac(215,185)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(7)*y(185);
jac(335,185)=jac(335,185)-k(7)*y(335);
jac(185,335)=jac(185,335)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(335);
jac(193,335)=jac(193,335)+k(7)*y(185);
jac(193,185)=jac(193,185)+k(7)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(185);
jac(257,185)=jac(257,185)-k(7)*y(257);
jac(185,257)=jac(185,257)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(257);
jac(195,257)=jac(195,257)+k(7)*y(185);
jac(195,185)=jac(195,185)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(227,227)=jac(227,227)-k(7)*y(185);
jac(227,185)=jac(227,185)-k(7)*y(227);
jac(185,227)=jac(185,227)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(227);
jac(549,227)=jac(549,227)+k(7)*y(185);
jac(549,185)=jac(549,185)+k(7)*y(227);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(222,222)=jac(222,222)-k(7)*y(185);
jac(222,185)=jac(222,185)-k(7)*y(222);
jac(185,222)=jac(185,222)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(222);
jac(516,222)=jac(516,222)+k(7)*y(185);
jac(516,185)=jac(516,185)+k(7)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(185);
jac(218,185)=jac(218,185)-k(7)*y(218);
jac(185,218)=jac(185,218)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(218);
jac(188,218)=jac(188,218)+k(7)*y(185);
jac(188,185)=jac(188,185)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(185);
jac(192,185)=jac(192,185)-k(7)*y(192);
jac(185,192)=jac(185,192)-k(7)*y(185);
jac(185,185)=jac(185,185)-k(7)*y(192);
jac(190,192)=jac(190,192)+k(7)*y(185);
jac(190,185)=jac(190,185)+k(7)*y(192);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) 

jac(185,185)=jac(185,185)-k(2);
jac(4,185)=jac(4,185)+k(2);
jac(184,185)=jac(184,185)+k(2);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(186,186)=jac(186,186)-k(34);
jac(460,186)=jac(460,186)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(186,186)=jac(186,186)-k(34);
jac(178,186)=jac(178,186)+k(34);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(186,186)=jac(186,186)-k(24);
jac(193,186)=jac(193,186)+k(24);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(186,186)=jac(186,186)-k(23);
jac(368,186)=jac(368,186)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(186,186)=jac(186,186)-k(22);
jac(182,186)=jac(182,186)+k(22);
jac(457,186)=jac(457,186)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(186,186)=jac(186,186)-k(22);
jac(182,186)=jac(182,186)+k(22);
jac(651,186)=jac(651,186)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(186,186)=jac(186,186)-k(10);
jac(3,186)=jac(3,186)+k(10);
jac(183,186)=jac(183,186)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(186,186)=jac(186,186)-k(10);
jac(3,186)=jac(3,186)+k(10);
jac(394,186)=jac(394,186)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(186,186)=jac(186,186)-k(8);
jac(184,186)=jac(184,186)+k(8);
jac(185,186)=jac(185,186)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(186,186)=jac(186,186)-k(6);
jac(184,186)=jac(184,186)+k(6);
jac(391,186)=jac(391,186)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(187,187)=jac(187,187)-k(34);
jac(377,187)=jac(377,187)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(187,187)=jac(187,187)-k(34);
jac(179,187)=jac(179,187)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(187,187)=jac(187,187)-k(22);
jac(182,187)=jac(182,187)+k(22);
jac(451,187)=jac(451,187)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(187,187)=jac(187,187)-k(22);
jac(182,187)=jac(182,187)+k(22);
jac(648,187)=jac(648,187)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(187);
jac(3,187)=jac(3,187)-k(9)*y(3);
jac(187,3)=jac(187,3)-k(9)*y(187);
jac(187,187)=jac(187,187)-k(9)*y(3);
jac(183,3)=jac(183,3)+k(9)*y(187);
jac(183,187)=jac(183,187)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(187);
jac(3,187)=jac(3,187)-k(9)*y(3);
jac(187,3)=jac(187,3)-k(9)*y(187);
jac(187,187)=jac(187,187)-k(9)*y(3);
jac(394,3)=jac(394,3)+k(9)*y(187);
jac(394,187)=jac(394,187)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(187,187)=jac(187,187)-k(8);
jac(177,187)=jac(177,187)+k(8);
jac(214,187)=jac(214,187)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(187,187)=jac(187,187)-k(6);
jac(177,187)=jac(177,187)+k(6);
jac(176,187)=jac(176,187)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(188,188)=jac(188,188)-k(34);
jac(346,188)=jac(346,188)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(188,188)=jac(188,188)-k(34);
jac(180,188)=jac(180,188)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(188,188)=jac(188,188)-k(31);
jac(183,188)=jac(183,188)+k(31);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(188,188)=jac(188,188)-k(24);
jac(195,188)=jac(195,188)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(188,188)=jac(188,188)-k(22);
jac(182,188)=jac(182,188)+k(22);
jac(343,188)=jac(343,188)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(188,188)=jac(188,188)-k(22);
jac(182,188)=jac(182,188)+k(22);
jac(361,188)=jac(361,188)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(188);
jac(3,188)=jac(3,188)-k(15)*y(3);
jac(188,3)=jac(188,3)-k(15)*y(188);
jac(188,188)=jac(188,188)-k(15)*y(3);
jac(516,3)=jac(516,3)+k(15)*y(188);
jac(516,188)=jac(516,188)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(188,188)=jac(188,188)-k(10);
jac(3,188)=jac(3,188)+k(10);
jac(513,188)=jac(513,188)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(188,188)=jac(188,188)-k(8);
jac(218,188)=jac(218,188)+k(8);
jac(185,188)=jac(185,188)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(188,188)=jac(188,188)-k(6);
jac(264,188)=jac(264,188)+k(6);
jac(184,188)=jac(184,188)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(189,189)=jac(189,189)-k(34);
jac(145,189)=jac(145,189)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(189,189)=jac(189,189)-k(32);
jac(652,189)=jac(652,189)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(189,189)=jac(189,189)-k(22);
jac(182,189)=jac(182,189)+k(22);
jac(147,189)=jac(147,189)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(189);
jac(197,189)=jac(197,189)-k(21)*y(197);
jac(189,197)=jac(189,197)-k(21)*y(189);
jac(189,189)=jac(189,189)-k(21)*y(197);
jac(190,197)=jac(190,197)+k(21)*y(189);
jac(190,189)=jac(190,189)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(189);
jac(32,189)=jac(32,189)-k(21)*y(32);
jac(189,32)=jac(189,32)-k(21)*y(189);
jac(189,189)=jac(189,189)-k(21)*y(32);
jac(168,32)=jac(168,32)+k(21)*y(189);
jac(168,189)=jac(168,189)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(189);
jac(2,189)=jac(2,189)-k(21)*y(2);
jac(189,2)=jac(189,2)-k(21)*y(189);
jac(189,189)=jac(189,189)-k(21)*y(2);
jac(174,2)=jac(174,2)+k(21)*y(189);
jac(174,189)=jac(174,189)+k(21)*y(2);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(189,189)=jac(189,189)-k(12);
jac(343,189)=jac(343,189)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(189,189)=jac(189,189)-k(10);
jac(3,189)=jac(3,189)+k(10);
jac(451,189)=jac(451,189)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(189);
jac(3,189)=jac(3,189)-k(9)*y(3);
jac(189,3)=jac(189,3)-k(9)*y(189);
jac(189,189)=jac(189,189)-k(9)*y(3);
jac(457,3)=jac(457,3)+k(9)*y(189);
jac(457,189)=jac(457,189)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

jac(189,189)=jac(189,189)-k(8);
jac(15,189)=jac(15,189)+k(8);
jac(185,189)=jac(185,189)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(189,189)=jac(189,189)-k(6);
jac(98,189)=jac(98,189)+k(6);
jac(184,189)=jac(184,189)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(191,191)=jac(191,191)-k(34);
jac(86,191)=jac(86,191)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(191,191)=jac(191,191)-k(33);
jac(176,191)=jac(176,191)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~pY) 

jac(191,191)=jac(191,191)-k(22);
jac(197,191)=jac(197,191)+k(22);
jac(98,191)=jac(98,191)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(191);
jac(3,191)=jac(3,191)-k(9)*y(3);
jac(191,3)=jac(191,3)-k(9)*y(191);
jac(191,191)=jac(191,191)-k(9)*y(3);
jac(407,3)=jac(407,3)+k(9)*y(191);
jac(407,191)=jac(407,191)+k(9)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(1,1)=jac(1,1)-k(5)*y(191);
jac(1,191)=jac(1,191)-k(5)*y(1);
jac(191,1)=jac(191,1)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(1);
jac(202,1)=jac(202,1)+k(5)*y(191);
jac(202,191)=jac(202,191)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(7,7)=jac(7,7)-k(5)*y(191);
jac(7,191)=jac(7,191)-k(5)*y(7);
jac(191,7)=jac(191,7)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(7);
jac(607,7)=jac(607,7)+k(5)*y(191);
jac(607,191)=jac(607,191)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(191,191)=jac(191,191)-k(5)*y(335);
jac(191,335)=jac(191,335)-k(5)*y(191);
jac(335,191)=jac(335,191)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(191);
jac(370,191)=jac(370,191)+k(5)*y(335);
jac(370,335)=jac(370,335)+k(5)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(5)*y(191);
jac(257,191)=jac(257,191)-k(5)*y(257);
jac(191,257)=jac(191,257)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(257);
jac(619,257)=jac(619,257)+k(5)*y(191);
jac(619,191)=jac(619,191)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(191,191)=jac(191,191)-k(5)*y(227);
jac(191,227)=jac(191,227)-k(5)*y(191);
jac(227,191)=jac(227,191)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(191);
jac(294,191)=jac(294,191)+k(5)*y(227);
jac(294,227)=jac(294,227)+k(5)*y(191);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(191,191)=jac(191,191)-k(5)*y(222);
jac(191,222)=jac(191,222)-k(5)*y(191);
jac(222,191)=jac(222,191)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(191);
jac(288,191)=jac(288,191)+k(5)*y(222);
jac(288,222)=jac(288,222)+k(5)*y(191);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(218,218)=jac(218,218)-k(5)*y(191);
jac(218,191)=jac(218,191)-k(5)*y(218);
jac(191,218)=jac(191,218)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(218);
jac(290,218)=jac(290,218)+k(5)*y(191);
jac(290,191)=jac(290,191)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(5)*y(191);
jac(192,191)=jac(192,191)-k(5)*y(192);
jac(191,192)=jac(191,192)-k(5)*y(191);
jac(191,191)=jac(191,191)-k(5)*y(192);
jac(455,192)=jac(455,192)+k(5)*y(191);
jac(455,191)=jac(455,191)+k(5)*y(192);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) 

jac(191,191)=jac(191,191)-k(4);
jac(4,191)=jac(4,191)+k(4);
jac(192,191)=jac(192,191)+k(4);

% rule    : R17
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(192,192)=jac(192,192)-k(34);
jac(27,192)=jac(27,192)+k(34);

% rule    : R16
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(192,192)=jac(192,192)-k(33);
jac(177,192)=jac(177,192)+k(33);

% rule    : R9_op
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(22);
jac(197,192)=jac(197,192)+k(22);
jac(15,192)=jac(15,192)+k(22);

% rule    : R3
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(192);
jac(3,192)=jac(3,192)-k(9)*y(3);
jac(192,3)=jac(192,3)-k(9)*y(192);
jac(192,192)=jac(192,192)-k(9)*y(3);
jac(335,3)=jac(335,3)+k(9)*y(192);
jac(335,192)=jac(335,192)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(209);
jac(192,209)=jac(192,209)-k(7)*y(192);
jac(209,192)=jac(209,192)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(192);
jac(455,192)=jac(455,192)+k(7)*y(209);
jac(455,209)=jac(455,209)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(214);
jac(192,214)=jac(192,214)-k(7)*y(192);
jac(214,192)=jac(214,192)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(192);
jac(194,192)=jac(194,192)+k(7)*y(214);
jac(194,214)=jac(194,214)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(219);
jac(192,219)=jac(192,219)-k(7)*y(192);
jac(219,192)=jac(219,192)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(192);
jac(290,192)=jac(290,192)+k(7)*y(219);
jac(290,219)=jac(290,219)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(223);
jac(192,223)=jac(192,223)-k(7)*y(192);
jac(223,192)=jac(223,192)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(192);
jac(288,192)=jac(288,192)+k(7)*y(223);
jac(288,223)=jac(288,223)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(228);
jac(192,228)=jac(192,228)-k(7)*y(192);
jac(228,192)=jac(228,192)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(192);
jac(294,192)=jac(294,192)+k(7)*y(228);
jac(294,228)=jac(294,228)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(315,315)=jac(315,315)-k(7)*y(192);
jac(315,192)=jac(315,192)-k(7)*y(315);
jac(192,315)=jac(192,315)-k(7)*y(192);
jac(192,192)=jac(192,192)-k(7)*y(315);
jac(292,315)=jac(292,315)+k(7)*y(192);
jac(292,192)=jac(292,192)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(319);
jac(192,319)=jac(192,319)-k(7)*y(192);
jac(319,192)=jac(319,192)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(192);
jac(619,192)=jac(619,192)+k(7)*y(319);
jac(619,319)=jac(619,319)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(336);
jac(192,336)=jac(192,336)-k(7)*y(192);
jac(336,192)=jac(336,192)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(192);
jac(370,192)=jac(370,192)+k(7)*y(336);
jac(370,336)=jac(370,336)+k(7)*y(192);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(7)*y(437);
jac(192,437)=jac(192,437)-k(7)*y(192);
jac(437,192)=jac(437,192)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(192);
jac(438,192)=jac(438,192)+k(7)*y(437);
jac(438,437)=jac(438,437)+k(7)*y(192);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(5)*y(256);
jac(192,256)=jac(192,256)-k(5)*y(192);
jac(256,192)=jac(256,192)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(192);
jac(609,192)=jac(609,192)+k(5)*y(256);
jac(609,256)=jac(609,256)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(192,192)=jac(192,192)-k(5)*y(264);
jac(192,264)=jac(192,264)-k(5)*y(192);
jac(264,192)=jac(264,192)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(192);
jac(611,192)=jac(611,192)+k(5)*y(264);
jac(611,264)=jac(611,264)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(192,192)=jac(192,192)-k(5)*y(380);
jac(192,380)=jac(192,380)-k(5)*y(192);
jac(380,192)=jac(380,192)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(192);
jac(374,192)=jac(374,192)+k(5)*y(380);
jac(374,380)=jac(374,380)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(192,192)=jac(192,192)-k(5)*y(391);
jac(192,391)=jac(192,391)-k(5)*y(192);
jac(391,192)=jac(391,192)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(192);
jac(397,192)=jac(397,192)+k(5)*y(391);
jac(397,391)=jac(397,391)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(5)*y(407);
jac(192,407)=jac(192,407)-k(5)*y(192);
jac(407,192)=jac(407,192)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(192);
jac(403,192)=jac(403,192)+k(5)*y(407);
jac(403,407)=jac(403,407)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(192,192)=jac(192,192)-k(5)*y(441);
jac(192,441)=jac(192,441)-k(5)*y(192);
jac(441,192)=jac(441,192)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(192);
jac(402,192)=jac(402,192)+k(5)*y(441);
jac(402,441)=jac(402,441)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(192,192)=jac(192,192)-k(5)*y(515);
jac(192,515)=jac(192,515)-k(5)*y(192);
jac(515,192)=jac(515,192)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(192);
jac(520,192)=jac(520,192)+k(5)*y(515);
jac(520,515)=jac(520,515)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(192,192)=jac(192,192)-k(5)*y(543);
jac(192,543)=jac(192,543)-k(5)*y(192);
jac(543,192)=jac(543,192)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(192);
jac(552,192)=jac(552,192)+k(5)*y(543);
jac(552,543)=jac(552,543)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(192,192)=jac(192,192)-k(5)*y(554);
jac(192,554)=jac(192,554)-k(5)*y(192);
jac(554,192)=jac(554,192)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(192);
jac(557,192)=jac(557,192)+k(5)*y(554);
jac(557,554)=jac(557,554)+k(5)*y(192);

% rule    : R2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(9,9)=jac(9,9)-k(5)*y(192);
jac(9,192)=jac(9,192)-k(5)*y(9);
jac(192,9)=jac(192,9)-k(5)*y(192);
jac(192,192)=jac(192,192)-k(5)*y(9);
jac(208,9)=jac(208,9)+k(5)*y(192);
jac(208,192)=jac(208,192)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(4,4)=jac(4,4)-k(3)*y(192);
jac(4,192)=jac(4,192)-k(3)*y(4);
jac(192,4)=jac(192,4)-k(3)*y(192);
jac(192,192)=jac(192,192)-k(3)*y(4);
jac(191,4)=jac(191,4)+k(3)*y(192);
jac(191,192)=jac(191,192)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(4,4)=jac(4,4)-k(1)*y(192);
jac(4,192)=jac(4,192)-k(1)*y(4);
jac(192,4)=jac(192,4)-k(1)*y(192);
jac(192,192)=jac(192,192)-k(1)*y(4);
jac(209,4)=jac(209,4)+k(1)*y(192);
jac(209,192)=jac(209,192)+k(1)*y(4);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(193,193)=jac(193,193)-k(34);
jac(448,193)=jac(448,193)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(193,193)=jac(193,193)-k(34);
jac(466,193)=jac(466,193)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(193,193)=jac(193,193)-k(33);
jac(186,193)=jac(186,193)+k(33);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(193,193)=jac(193,193)-k(23);
jac(464,193)=jac(464,193)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(193,193)=jac(193,193)-k(22);
jac(197,193)=jac(197,193)+k(22);
jac(457,193)=jac(457,193)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(193,193)=jac(193,193)-k(22);
jac(182,193)=jac(182,193)+k(22);
jac(408,193)=jac(408,193)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(193,193)=jac(193,193)-k(10);
jac(3,193)=jac(3,193)+k(10);
jac(190,193)=jac(190,193)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(193,193)=jac(193,193)-k(10);
jac(3,193)=jac(3,193)+k(10);
jac(435,193)=jac(435,193)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(193,193)=jac(193,193)-k(8);
jac(335,193)=jac(335,193)+k(8);
jac(185,193)=jac(185,193)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(193,193)=jac(193,193)-k(6);
jac(184,193)=jac(184,193)+k(6);
jac(407,193)=jac(407,193)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(194,194)=jac(194,194)-k(34);
jac(449,194)=jac(449,194)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(194,194)=jac(194,194)-k(34);
jac(453,194)=jac(453,194)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(194,194)=jac(194,194)-k(33);
jac(187,194)=jac(187,194)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(194,194)=jac(194,194)-k(22);
jac(197,194)=jac(197,194)+k(22);
jac(451,194)=jac(451,194)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(194,194)=jac(194,194)-k(22);
jac(182,194)=jac(182,194)+k(22);
jac(409,194)=jac(409,194)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(194);
jac(3,194)=jac(3,194)-k(9)*y(3);
jac(194,3)=jac(194,3)-k(9)*y(194);
jac(194,194)=jac(194,194)-k(9)*y(3);
jac(190,3)=jac(190,3)+k(9)*y(194);
jac(190,194)=jac(190,194)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(194);
jac(3,194)=jac(3,194)-k(9)*y(3);
jac(194,3)=jac(194,3)-k(9)*y(194);
jac(194,194)=jac(194,194)-k(9)*y(3);
jac(435,3)=jac(435,3)+k(9)*y(194);
jac(435,194)=jac(435,194)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(194,194)=jac(194,194)-k(8);
jac(192,194)=jac(192,194)+k(8);
jac(214,194)=jac(214,194)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(194,194)=jac(194,194)-k(6);
jac(177,194)=jac(177,194)+k(6);
jac(191,194)=jac(191,194)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(195,195)=jac(195,195)-k(34);
jac(347,195)=jac(347,195)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(195,195)=jac(195,195)-k(34);
jac(605,195)=jac(605,195)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(195,195)=jac(195,195)-k(33);
jac(188,195)=jac(188,195)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(195,195)=jac(195,195)-k(31);
jac(190,195)=jac(190,195)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(195,195)=jac(195,195)-k(22);
jac(197,195)=jac(197,195)+k(22);
jac(343,195)=jac(343,195)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(195,195)=jac(195,195)-k(22);
jac(182,195)=jac(182,195)+k(22);
jac(599,195)=jac(599,195)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(195);
jac(3,195)=jac(3,195)-k(15)*y(3);
jac(195,3)=jac(195,3)-k(15)*y(195);
jac(195,195)=jac(195,195)-k(15)*y(3);
jac(549,3)=jac(549,3)+k(15)*y(195);
jac(549,195)=jac(549,195)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(195,195)=jac(195,195)-k(10);
jac(3,195)=jac(3,195)+k(10);
jac(550,195)=jac(550,195)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(195,195)=jac(195,195)-k(8);
jac(257,195)=jac(257,195)+k(8);
jac(185,195)=jac(185,195)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(195,195)=jac(195,195)-k(6);
jac(256,195)=jac(256,195)+k(6);
jac(184,195)=jac(184,195)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(196,196)=jac(196,196)-k(34);
jac(153,196)=jac(153,196)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(196,196)=jac(196,196)-k(33);
jac(181,196)=jac(181,196)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(196,196)=jac(196,196)-k(32);
jac(607,196)=jac(607,196)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(196,196)=jac(196,196)-k(22);
jac(197,196)=jac(197,196)+k(22);
jac(147,196)=jac(147,196)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(196);
jac(197,196)=jac(197,196)-k(21)*y(197);
jac(196,197)=jac(196,197)-k(21)*y(196);
jac(196,196)=jac(196,196)-k(21)*y(197);
jac(370,197)=jac(370,197)+k(21)*y(196);
jac(370,196)=jac(370,196)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(196);
jac(32,196)=jac(32,196)-k(21)*y(32);
jac(196,32)=jac(196,32)-k(21)*y(196);
jac(196,196)=jac(196,196)-k(21)*y(32);
jac(163,32)=jac(163,32)+k(21)*y(196);
jac(163,196)=jac(163,196)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(196);
jac(2,196)=jac(2,196)-k(21)*y(2);
jac(196,2)=jac(196,2)-k(21)*y(196);
jac(196,196)=jac(196,196)-k(21)*y(2);
jac(169,2)=jac(169,2)+k(21)*y(196);
jac(169,196)=jac(169,196)+k(21)*y(2);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(196,196)=jac(196,196)-k(12);
jac(599,196)=jac(599,196)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(196,196)=jac(196,196)-k(10);
jac(3,196)=jac(3,196)+k(10);
jac(409,196)=jac(409,196)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(196);
jac(3,196)=jac(3,196)-k(9)*y(3);
jac(196,3)=jac(196,3)-k(9)*y(196);
jac(196,196)=jac(196,196)-k(9)*y(3);
jac(408,3)=jac(408,3)+k(9)*y(196);
jac(408,196)=jac(408,196)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(196,196)=jac(196,196)-k(8);
jac(126,196)=jac(126,196)+k(8);
jac(192,196)=jac(192,196)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(196,196)=jac(196,196)-k(6);
jac(48,196)=jac(48,196)+k(6);
jac(191,196)=jac(191,196)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(198,198)=jac(198,198)-k(34);
jac(154,198)=jac(154,198)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(198,198)=jac(198,198)-k(33);
jac(189,198)=jac(189,198)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(198,198)=jac(198,198)-k(32);
jac(199,198)=jac(199,198)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(198,198)=jac(198,198)-k(22);
jac(197,198)=jac(197,198)+k(22);
jac(147,198)=jac(147,198)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(198);
jac(197,198)=jac(197,198)-k(21)*y(197);
jac(198,197)=jac(198,197)-k(21)*y(198);
jac(198,198)=jac(198,198)-k(21)*y(197);
jac(370,197)=jac(370,197)+k(21)*y(198);
jac(370,198)=jac(370,198)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(198);
jac(32,198)=jac(32,198)-k(21)*y(32);
jac(198,32)=jac(198,32)-k(21)*y(198);
jac(198,198)=jac(198,198)-k(21)*y(32);
jac(162,32)=jac(162,32)+k(21)*y(198);
jac(162,198)=jac(162,198)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(198);
jac(2,198)=jac(2,198)-k(21)*y(2);
jac(198,2)=jac(198,2)-k(21)*y(198);
jac(198,198)=jac(198,198)-k(21)*y(2);
jac(371,2)=jac(371,2)+k(21)*y(198);
jac(371,198)=jac(371,198)+k(21)*y(2);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(198,198)=jac(198,198)-k(12);
jac(337,198)=jac(337,198)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(198,198)=jac(198,198)-k(10);
jac(3,198)=jac(3,198)+k(10);
jac(376,198)=jac(376,198)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(198);
jac(3,198)=jac(3,198)-k(9)*y(3);
jac(198,3)=jac(198,3)-k(9)*y(198);
jac(198,198)=jac(198,198)-k(9)*y(3);
jac(400,3)=jac(400,3)+k(9)*y(198);
jac(400,198)=jac(400,198)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

jac(198,198)=jac(198,198)-k(8);
jac(15,198)=jac(15,198)+k(8);
jac(336,198)=jac(336,198)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(198,198)=jac(198,198)-k(6);
jac(98,198)=jac(98,198)+k(6);
jac(335,198)=jac(335,198)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(199,199)=jac(199,199)-k(34);
jac(138,199)=jac(138,199)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(199,199)=jac(199,199)-k(33);
jac(652,199)=jac(652,199)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(199,199)=jac(199,199)-k(22);
jac(197,199)=jac(197,199)+k(22);
jac(146,199)=jac(146,199)+k(22);

% rule    : R5_2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(199,199)=jac(199,199)-k(14);
jac(198,199)=jac(198,199)+k(14);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(199,199)=jac(199,199)-k(12);
jac(334,199)=jac(334,199)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(199,199)=jac(199,199)-k(10);
jac(3,199)=jac(3,199)+k(10);
jac(211,199)=jac(211,199)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(199);
jac(3,199)=jac(3,199)-k(9)*y(3);
jac(199,3)=jac(199,3)-k(9)*y(199);
jac(199,199)=jac(199,199)-k(9)*y(3);
jac(210,3)=jac(210,3)+k(9)*y(199);
jac(210,199)=jac(210,199)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

jac(199,199)=jac(199,199)-k(8);
jac(1,199)=jac(1,199)+k(8);
jac(336,199)=jac(336,199)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(199,199)=jac(199,199)-k(6);
jac(6,199)=jac(6,199)+k(6);
jac(335,199)=jac(335,199)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(200,200)=jac(200,200)-k(34);
jac(130,200)=jac(130,200)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(200,200)=jac(200,200)-k(33);
jac(327,200)=jac(327,200)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(200,200)=jac(200,200)-k(22);
jac(197,200)=jac(197,200)+k(22);
jac(125,200)=jac(125,200)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(200,200)=jac(200,200)-k(16);
jac(3,200)=jac(3,200)+k(16);
jac(334,200)=jac(334,200)+k(16);

% rule    : R5_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(200,200)=jac(200,200)-k(14);
jac(339,200)=jac(339,200)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(200,200)=jac(200,200)-k(10);
jac(3,200)=jac(3,200)+k(10);
jac(322,200)=jac(322,200)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(200,200)=jac(200,200)-k(8);
jac(61,200)=jac(61,200)+k(8);
jac(336,200)=jac(336,200)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(200,200)=jac(200,200)-k(6);
jac(122,200)=jac(122,200)+k(6);
jac(335,200)=jac(335,200)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(201,201)=jac(201,201)-k(34);
jac(107,201)=jac(107,201)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(201,201)=jac(201,201)-k(33);
jac(593,201)=jac(593,201)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(201,201)=jac(201,201)-k(22);
jac(197,201)=jac(197,201)+k(22);
jac(117,201)=jac(117,201)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(201,201)=jac(201,201)-k(19);
jac(598,201)=jac(598,201)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(201,201)=jac(201,201)-k(16);
jac(3,201)=jac(3,201)+k(16);
jac(588,201)=jac(588,201)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(201,201)=jac(201,201)-k(10);
jac(3,201)=jac(3,201)+k(10);
jac(589,201)=jac(589,201)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(201,201)=jac(201,201)-k(8);
jac(8,201)=jac(8,201)+k(8);
jac(227,201)=jac(227,201)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(201,201)=jac(201,201)-k(6);
jac(7,201)=jac(7,201)+k(6);
jac(543,201)=jac(543,201)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(202,202)=jac(202,202)-k(34);
jac(101,202)=jac(101,202)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(202,202)=jac(202,202)-k(33);
jac(418,202)=jac(418,202)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(202,202)=jac(202,202)-k(22);
jac(197,202)=jac(197,202)+k(22);
jac(97,202)=jac(97,202)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(202);
jac(3,202)=jac(3,202)-k(9)*y(3);
jac(202,3)=jac(202,3)-k(9)*y(202);
jac(202,202)=jac(202,202)-k(9)*y(3);
jac(607,3)=jac(607,3)+k(9)*y(202);
jac(607,202)=jac(607,202)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(202);
jac(3,202)=jac(3,202)-k(9)*y(3);
jac(202,3)=jac(202,3)-k(9)*y(202);
jac(202,202)=jac(202,202)-k(9)*y(3);
jac(413,3)=jac(413,3)+k(9)*y(202);
jac(413,202)=jac(413,202)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(202,202)=jac(202,202)-k(8);
jac(5,202)=jac(5,202)+k(8);
jac(192,202)=jac(192,202)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

jac(202,202)=jac(202,202)-k(6);
jac(1,202)=jac(1,202)+k(6);
jac(191,202)=jac(191,202)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(203,203)=jac(203,203)-k(34);
jac(81,203)=jac(81,203)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(203,203)=jac(203,203)-k(33);
jac(300,203)=jac(300,203)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(203,203)=jac(203,203)-k(31);
jac(295,203)=jac(295,203)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

jac(203,203)=jac(203,203)-k(22);
jac(197,203)=jac(197,203)+k(22);
jac(75,203)=jac(75,203)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(203,203)=jac(203,203)-k(16);
jac(3,203)=jac(3,203)+k(16);
jac(586,203)=jac(586,203)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(203);
jac(3,203)=jac(3,203)-k(15)*y(3);
jac(203,3)=jac(203,3)-k(15)*y(203);
jac(203,203)=jac(203,203)-k(15)*y(3);
jac(647,3)=jac(647,3)+k(15)*y(203);
jac(647,203)=jac(647,203)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(203,203)=jac(203,203)-k(8);
jac(62,203)=jac(62,203)+k(8);
jac(257,203)=jac(257,203)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(203,203)=jac(203,203)-k(6);
jac(61,203)=jac(61,203)+k(6);
jac(256,203)=jac(256,203)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(204,204)=jac(204,204)-k(34);
jac(66,204)=jac(66,204)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(204,204)=jac(204,204)-k(33);
jac(298,204)=jac(298,204)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(204,204)=jac(204,204)-k(22);
jac(197,204)=jac(197,204)+k(22);
jac(60,204)=jac(60,204)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(204,204)=jac(204,204)-k(16);
jac(3,204)=jac(3,204)+k(16);
jac(415,204)=jac(415,204)+k(16);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(204,204)=jac(204,204)-k(13);
jac(206,204)=jac(206,204)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(204,204)=jac(204,204)-k(10);
jac(3,204)=jac(3,204)+k(10);
jac(295,204)=jac(295,204)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(204,204)=jac(204,204)-k(8);
jac(62,204)=jac(62,204)+k(8);
jac(335,204)=jac(335,204)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(204,204)=jac(204,204)-k(6);
jac(61,204)=jac(61,204)+k(6);
jac(407,204)=jac(407,204)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(205,205)=jac(205,205)-k(34);
jac(40,205)=jac(40,205)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(205,205)=jac(205,205)-k(33);
jac(225,205)=jac(225,205)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(205,205)=jac(205,205)-k(32);
jac(226,205)=jac(226,205)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(205,205)=jac(205,205)-k(22);
jac(197,205)=jac(197,205)+k(22);
jac(53,205)=jac(53,205)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(205);
jac(197,205)=jac(197,205)-k(21)*y(197);
jac(205,197)=jac(205,197)-k(21)*y(205);
jac(205,205)=jac(205,205)-k(21)*y(197);
jac(475,197)=jac(475,197)+k(21)*y(205);
jac(475,205)=jac(475,205)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(205);
jac(32,205)=jac(32,205)-k(21)*y(32);
jac(205,32)=jac(205,32)-k(21)*y(205);
jac(205,205)=jac(205,205)-k(21)*y(32);
jac(473,32)=jac(473,32)+k(21)*y(205);
jac(473,205)=jac(473,205)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(205);
jac(2,205)=jac(2,205)-k(21)*y(2);
jac(205,2)=jac(205,2)-k(21)*y(205);
jac(205,205)=jac(205,205)-k(21)*y(2);
jac(428,2)=jac(428,2)+k(21)*y(205);
jac(428,205)=jac(428,205)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(205,205)=jac(205,205)-k(16);
jac(3,205)=jac(3,205)+k(16);
jac(401,205)=jac(401,205)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(205,205)=jac(205,205)-k(10);
jac(3,205)=jac(3,205)+k(10);
jac(232,205)=jac(232,205)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(205,205)=jac(205,205)-k(8);
jac(48,205)=jac(48,205)+k(8);
jac(228,205)=jac(228,205)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(205,205)=jac(205,205)-k(6);
jac(47,205)=jac(47,205)+k(6);
jac(227,205)=jac(227,205)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(206,206)=jac(206,206)-k(34);
jac(59,206)=jac(59,206)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(206,206)=jac(206,206)-k(33);
jac(422,206)=jac(422,206)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(206,206)=jac(206,206)-k(32);
jac(204,206)=jac(204,206)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(206,206)=jac(206,206)-k(22);
jac(197,206)=jac(197,206)+k(22);
jac(53,206)=jac(53,206)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(206);
jac(197,206)=jac(197,206)-k(21)*y(197);
jac(206,197)=jac(206,197)-k(21)*y(206);
jac(206,206)=jac(206,206)-k(21)*y(197);
jac(475,197)=jac(475,197)+k(21)*y(206);
jac(475,206)=jac(475,206)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(206);
jac(32,206)=jac(32,206)-k(21)*y(32);
jac(206,32)=jac(206,32)-k(21)*y(206);
jac(206,206)=jac(206,206)-k(21)*y(32);
jac(477,32)=jac(477,32)+k(21)*y(206);
jac(477,206)=jac(477,206)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(206);
jac(2,206)=jac(2,206)-k(21)*y(2);
jac(206,2)=jac(206,2)-k(21)*y(206);
jac(206,206)=jac(206,206)-k(21)*y(2);
jac(207,2)=jac(207,2)+k(21)*y(206);
jac(207,206)=jac(207,206)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(206,206)=jac(206,206)-k(16);
jac(3,206)=jac(3,206)+k(16);
jac(410,206)=jac(410,206)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(206,206)=jac(206,206)-k(10);
jac(3,206)=jac(3,206)+k(10);
jac(291,206)=jac(291,206)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(206,206)=jac(206,206)-k(8);
jac(55,206)=jac(55,206)+k(8);
jac(335,206)=jac(335,206)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(206,206)=jac(206,206)-k(6);
jac(54,206)=jac(54,206)+k(6);
jac(407,206)=jac(407,206)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(207,207)=jac(207,207)-k(34);
jac(52,207)=jac(52,207)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(207,207)=jac(207,207)-k(33);
jac(270,207)=jac(270,207)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(207,207)=jac(207,207)-k(29);
jac(289,207)=jac(289,207)+k(29);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(207,207)=jac(207,207)-k(23);
jac(477,207)=jac(477,207)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(207,207)=jac(207,207)-k(22);
jac(197,207)=jac(197,207)+k(22);
jac(46,207)=jac(46,207)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(207,207)=jac(207,207)-k(22);
jac(2,207)=jac(2,207)+k(22);
jac(206,207)=jac(206,207)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(207,207)=jac(207,207)-k(16);
jac(3,207)=jac(3,207)+k(16);
jac(436,207)=jac(436,207)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(207,207)=jac(207,207)-k(10);
jac(3,207)=jac(3,207)+k(10);
jac(262,207)=jac(262,207)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(207,207)=jac(207,207)-k(8);
jac(335,207)=jac(335,207)+k(8);
jac(44,207)=jac(44,207)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(207,207)=jac(207,207)-k(6);
jac(43,207)=jac(43,207)+k(6);
jac(407,207)=jac(407,207)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(209,209)=jac(209,209)-k(34);
jac(28,209)=jac(28,209)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(209,209)=jac(209,209)-k(33);
jac(214,209)=jac(214,209)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~pY) 

jac(209,209)=jac(209,209)-k(22);
jac(197,209)=jac(197,209)+k(22);
jac(16,209)=jac(16,209)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(209);
jac(3,209)=jac(3,209)-k(9)*y(3);
jac(209,3)=jac(209,3)-k(9)*y(209);
jac(209,209)=jac(209,209)-k(9)*y(3);
jac(336,3)=jac(336,3)+k(9)*y(209);
jac(336,209)=jac(336,209)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(1,1)=jac(1,1)-k(7)*y(209);
jac(1,209)=jac(1,209)-k(7)*y(1);
jac(209,1)=jac(209,1)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(1);
jac(211,1)=jac(211,1)+k(7)*y(209);
jac(211,209)=jac(211,209)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(7,7)=jac(7,7)-k(7)*y(209);
jac(7,209)=jac(7,209)-k(7)*y(7);
jac(209,7)=jac(209,7)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(7);
jac(208,7)=jac(208,7)+k(7)*y(209);
jac(208,209)=jac(208,209)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(209,209)=jac(209,209)-k(7)*y(335);
jac(209,335)=jac(209,335)-k(7)*y(209);
jac(335,209)=jac(335,209)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(209);
jac(403,209)=jac(403,209)+k(7)*y(335);
jac(403,335)=jac(403,335)+k(7)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(209);
jac(257,209)=jac(257,209)-k(7)*y(257);
jac(209,257)=jac(209,257)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(257);
jac(609,257)=jac(609,257)+k(7)*y(209);
jac(609,209)=jac(609,209)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(209,209)=jac(209,209)-k(7)*y(227);
jac(209,227)=jac(209,227)-k(7)*y(209);
jac(227,209)=jac(227,209)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(209);
jac(552,209)=jac(552,209)+k(7)*y(227);
jac(552,227)=jac(552,227)+k(7)*y(209);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(209,209)=jac(209,209)-k(7)*y(222);
jac(209,222)=jac(209,222)-k(7)*y(209);
jac(222,209)=jac(222,209)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(209);
jac(520,209)=jac(520,209)+k(7)*y(222);
jac(520,222)=jac(520,222)+k(7)*y(209);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(209);
jac(218,209)=jac(218,209)-k(7)*y(218);
jac(209,218)=jac(209,218)-k(7)*y(209);
jac(209,209)=jac(209,209)-k(7)*y(218);
jac(611,218)=jac(611,218)+k(7)*y(209);
jac(611,209)=jac(611,209)+k(7)*y(218);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) 

jac(209,209)=jac(209,209)-k(2);
jac(4,209)=jac(4,209)+k(2);
jac(192,209)=jac(192,209)+k(2);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(210,210)=jac(210,210)-k(34);
jac(29,210)=jac(29,210)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(210,210)=jac(210,210)-k(33);
jac(215,210)=jac(215,210)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(210,210)=jac(210,210)-k(22);
jac(197,210)=jac(197,210)+k(22);
jac(17,210)=jac(17,210)+k(22);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(210,210)=jac(210,210)-k(14);
jac(400,210)=jac(400,210)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(210,210)=jac(210,210)-k(10);
jac(3,210)=jac(3,210)+k(10);
jac(208,210)=jac(208,210)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(210,210)=jac(210,210)-k(10);
jac(3,210)=jac(3,210)+k(10);
jac(199,210)=jac(199,210)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(210,210)=jac(210,210)-k(8);
jac(7,210)=jac(7,210)+k(8);
jac(336,210)=jac(336,210)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(210,210)=jac(210,210)-k(6);
jac(9,210)=jac(9,210)+k(6);
jac(335,210)=jac(335,210)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(211,211)=jac(211,211)-k(34);
jac(30,211)=jac(30,211)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(211,211)=jac(211,211)-k(33);
jac(216,211)=jac(216,211)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~pY) 

jac(211,211)=jac(211,211)-k(22);
jac(197,211)=jac(197,211)+k(22);
jac(18,211)=jac(18,211)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(211);
jac(3,211)=jac(3,211)-k(9)*y(3);
jac(211,3)=jac(211,3)-k(9)*y(211);
jac(211,211)=jac(211,211)-k(9)*y(3);
jac(199,3)=jac(199,3)+k(9)*y(211);
jac(199,211)=jac(199,211)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(211);
jac(3,211)=jac(3,211)-k(9)*y(3);
jac(211,3)=jac(211,3)-k(9)*y(211);
jac(211,211)=jac(211,211)-k(9)*y(3);
jac(208,3)=jac(208,3)+k(9)*y(211);
jac(208,211)=jac(208,211)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

jac(211,211)=jac(211,211)-k(8);
jac(1,211)=jac(1,211)+k(8);
jac(209,211)=jac(209,211)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(211,211)=jac(211,211)-k(6);
jac(6,211)=jac(6,211)+k(6);
jac(192,211)=jac(192,211)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(212,212)=jac(212,212)-k(34);
jac(31,212)=jac(31,212)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(212,212)=jac(212,212)-k(33);
jac(217,212)=jac(217,212)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(212,212)=jac(212,212)-k(31);
jac(208,212)=jac(208,212)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(212,212)=jac(212,212)-k(22);
jac(197,212)=jac(197,212)+k(22);
jac(19,212)=jac(19,212)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(212);
jac(3,212)=jac(3,212)-k(15)*y(3);
jac(212,3)=jac(212,3)-k(15)*y(212);
jac(212,212)=jac(212,212)-k(15)*y(3);
jac(226,3)=jac(226,3)+k(15)*y(212);
jac(226,212)=jac(226,212)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(212,212)=jac(212,212)-k(10);
jac(3,212)=jac(3,212)+k(10);
jac(230,212)=jac(230,212)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(212,212)=jac(212,212)-k(8);
jac(7,212)=jac(7,212)+k(8);
jac(319,212)=jac(319,212)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(212,212)=jac(212,212)-k(6);
jac(9,212)=jac(9,212)+k(6);
jac(257,212)=jac(257,212)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(214,214)=jac(214,214)-k(34);
jac(22,214)=jac(22,214)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~pY) 

jac(214,214)=jac(214,214)-k(22);
jac(182,214)=jac(182,214)+k(22);
jac(16,214)=jac(16,214)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(214);
jac(3,214)=jac(3,214)-k(9)*y(3);
jac(214,3)=jac(214,3)-k(9)*y(214);
jac(214,214)=jac(214,214)-k(9)*y(3);
jac(185,3)=jac(185,3)+k(9)*y(214);
jac(185,214)=jac(185,214)+k(9)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(1,1)=jac(1,1)-k(7)*y(214);
jac(1,214)=jac(1,214)-k(7)*y(1);
jac(214,1)=jac(214,1)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(1);
jac(216,1)=jac(216,1)+k(7)*y(214);
jac(216,214)=jac(216,214)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(7,7)=jac(7,7)-k(7)*y(214);
jac(7,214)=jac(7,214)-k(7)*y(7);
jac(214,7)=jac(214,7)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(7);
jac(213,7)=jac(213,7)+k(7)*y(214);
jac(213,214)=jac(213,214)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(214,214)=jac(214,214)-k(7)*y(335);
jac(214,335)=jac(214,335)-k(7)*y(214);
jac(335,214)=jac(335,214)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(214);
jac(435,214)=jac(435,214)+k(7)*y(335);
jac(435,335)=jac(435,335)+k(7)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(214);
jac(257,214)=jac(257,214)-k(7)*y(257);
jac(214,257)=jac(214,257)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(257);
jac(550,257)=jac(550,257)+k(7)*y(214);
jac(550,214)=jac(550,214)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(214,214)=jac(214,214)-k(7)*y(227);
jac(214,227)=jac(214,227)-k(7)*y(214);
jac(227,214)=jac(227,214)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(214);
jac(548,214)=jac(548,214)+k(7)*y(227);
jac(548,227)=jac(548,227)+k(7)*y(214);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(214,214)=jac(214,214)-k(7)*y(222);
jac(214,222)=jac(214,222)-k(7)*y(214);
jac(222,214)=jac(222,214)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(214);
jac(514,214)=jac(514,214)+k(7)*y(222);
jac(514,222)=jac(514,222)+k(7)*y(214);

% rule    : R2_1
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(214);
jac(218,214)=jac(218,214)-k(7)*y(218);
jac(214,218)=jac(214,218)-k(7)*y(214);
jac(214,214)=jac(214,214)-k(7)*y(218);
jac(513,218)=jac(513,218)+k(7)*y(214);
jac(513,214)=jac(513,214)+k(7)*y(218);

% rule    : R1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) 

jac(214,214)=jac(214,214)-k(2);
jac(4,214)=jac(4,214)+k(2);
jac(177,214)=jac(177,214)+k(2);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(215,215)=jac(215,215)-k(34);
jac(23,215)=jac(23,215)+k(34);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(215,215)=jac(215,215)-k(23);
jac(210,215)=jac(210,215)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(215,215)=jac(215,215)-k(22);
jac(182,215)=jac(182,215)+k(22);
jac(17,215)=jac(17,215)+k(22);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(215,215)=jac(215,215)-k(14);
jac(457,215)=jac(457,215)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(215,215)=jac(215,215)-k(10);
jac(3,215)=jac(3,215)+k(10);
jac(213,215)=jac(213,215)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(215,215)=jac(215,215)-k(10);
jac(3,215)=jac(3,215)+k(10);
jac(652,215)=jac(652,215)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(215,215)=jac(215,215)-k(8);
jac(7,215)=jac(7,215)+k(8);
jac(185,215)=jac(185,215)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(215,215)=jac(215,215)-k(6);
jac(9,215)=jac(9,215)+k(6);
jac(184,215)=jac(184,215)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(216,216)=jac(216,216)-k(34);
jac(24,216)=jac(24,216)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~pY) 

jac(216,216)=jac(216,216)-k(22);
jac(182,216)=jac(182,216)+k(22);
jac(18,216)=jac(18,216)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(216);
jac(3,216)=jac(3,216)-k(9)*y(3);
jac(216,3)=jac(216,3)-k(9)*y(216);
jac(216,216)=jac(216,216)-k(9)*y(3);
jac(652,3)=jac(652,3)+k(9)*y(216);
jac(652,216)=jac(652,216)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(216);
jac(3,216)=jac(3,216)-k(9)*y(3);
jac(216,3)=jac(216,3)-k(9)*y(216);
jac(216,216)=jac(216,216)-k(9)*y(3);
jac(213,3)=jac(213,3)+k(9)*y(216);
jac(213,216)=jac(213,216)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

jac(216,216)=jac(216,216)-k(8);
jac(1,216)=jac(1,216)+k(8);
jac(214,216)=jac(214,216)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(216,216)=jac(216,216)-k(6);
jac(6,216)=jac(6,216)+k(6);
jac(177,216)=jac(177,216)+k(6);

% rule    : R17
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(218,218)=jac(218,218)-k(34);
jac(111,218)=jac(111,218)+k(34);

% rule    : R14
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(31);
jac(177,218)=jac(177,218)+k(31);

% rule    : R9_op
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(22);
jac(182,218)=jac(182,218)+k(22);
jac(77,218)=jac(77,218)+k(22);

% rule    : R6
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(218);
jac(3,218)=jac(3,218)-k(15)*y(3);
jac(218,3)=jac(218,3)-k(15)*y(218);
jac(218,218)=jac(218,218)-k(15)*y(3);
jac(222,3)=jac(222,3)+k(15)*y(218);
jac(222,218)=jac(222,218)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(219);
jac(218,219)=jac(218,219)-k(7)*y(218);
jac(219,218)=jac(219,218)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(218);
jac(579,218)=jac(579,218)+k(7)*y(219);
jac(579,219)=jac(579,219)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(223);
jac(218,223)=jac(218,223)-k(7)*y(218);
jac(223,218)=jac(223,218)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(218);
jac(268,218)=jac(268,218)+k(7)*y(223);
jac(268,223)=jac(268,223)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(228);
jac(218,228)=jac(218,228)-k(7)*y(218);
jac(228,218)=jac(228,218)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(218);
jac(488,218)=jac(488,218)+k(7)*y(228);
jac(488,228)=jac(488,228)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(315,315)=jac(315,315)-k(7)*y(218);
jac(315,218)=jac(315,218)-k(7)*y(315);
jac(218,315)=jac(218,315)-k(7)*y(218);
jac(218,218)=jac(218,218)-k(7)*y(315);
jac(497,315)=jac(497,315)+k(7)*y(218);
jac(497,218)=jac(497,218)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(319);
jac(218,319)=jac(218,319)-k(7)*y(218);
jac(319,218)=jac(319,218)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(218);
jac(490,218)=jac(490,218)+k(7)*y(319);
jac(490,319)=jac(490,319)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(336);
jac(218,336)=jac(218,336)-k(7)*y(218);
jac(336,218)=jac(336,218)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(218);
jac(363,218)=jac(363,218)+k(7)*y(336);
jac(363,336)=jac(363,336)+k(7)*y(218);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(7)*y(437);
jac(218,437)=jac(218,437)-k(7)*y(218);
jac(437,218)=jac(437,218)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(218);
jac(266,218)=jac(266,218)+k(7)*y(437);
jac(266,437)=jac(266,437)+k(7)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(218,218)=jac(218,218)-k(5)*y(256);
jac(218,256)=jac(218,256)-k(5)*y(218);
jac(256,218)=jac(256,218)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(218);
jac(577,218)=jac(577,218)+k(5)*y(256);
jac(577,256)=jac(577,256)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(5)*y(264);
jac(218,264)=jac(218,264)-k(5)*y(218);
jac(264,218)=jac(264,218)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(218);
jac(579,218)=jac(579,218)+k(5)*y(264);
jac(579,264)=jac(579,264)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(218,218)=jac(218,218)-k(5)*y(380);
jac(218,380)=jac(218,380)-k(5)*y(218);
jac(380,218)=jac(380,218)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(218);
jac(429,218)=jac(429,218)+k(5)*y(380);
jac(429,380)=jac(429,380)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(5)*y(391);
jac(218,391)=jac(218,391)-k(5)*y(218);
jac(391,218)=jac(391,218)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(218);
jac(395,218)=jac(395,218)+k(5)*y(391);
jac(395,391)=jac(395,391)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(218,218)=jac(218,218)-k(5)*y(407);
jac(218,407)=jac(218,407)-k(5)*y(218);
jac(407,218)=jac(407,218)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(218);
jac(433,218)=jac(433,218)+k(5)*y(407);
jac(433,407)=jac(433,407)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(218,218)=jac(218,218)-k(5)*y(441);
jac(218,441)=jac(218,441)-k(5)*y(218);
jac(441,218)=jac(441,218)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(218);
jac(432,218)=jac(432,218)+k(5)*y(441);
jac(432,441)=jac(432,441)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(218,218)=jac(218,218)-k(5)*y(515);
jac(218,515)=jac(218,515)-k(5)*y(218);
jac(515,218)=jac(515,218)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(218);
jac(517,218)=jac(517,218)+k(5)*y(515);
jac(517,515)=jac(517,515)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(218,218)=jac(218,218)-k(5)*y(543);
jac(218,543)=jac(218,543)-k(5)*y(218);
jac(543,218)=jac(543,218)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(218);
jac(551,218)=jac(551,218)+k(5)*y(543);
jac(551,543)=jac(551,543)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(218,218)=jac(218,218)-k(5)*y(554);
jac(218,554)=jac(218,554)-k(5)*y(218);
jac(554,218)=jac(554,218)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(218);
jac(555,218)=jac(555,218)+k(5)*y(554);
jac(555,554)=jac(555,554)+k(5)*y(218);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(9,9)=jac(9,9)-k(5)*y(218);
jac(9,218)=jac(9,218)-k(5)*y(9);
jac(218,9)=jac(218,9)-k(5)*y(218);
jac(218,218)=jac(218,218)-k(5)*y(9);
jac(217,9)=jac(217,9)+k(5)*y(218);
jac(217,218)=jac(217,218)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(4,4)=jac(4,4)-k(3)*y(218);
jac(4,218)=jac(4,218)-k(3)*y(4);
jac(218,4)=jac(218,4)-k(3)*y(218);
jac(218,218)=jac(218,218)-k(3)*y(4);
jac(264,4)=jac(264,4)+k(3)*y(218);
jac(264,218)=jac(264,218)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(4,4)=jac(4,4)-k(1)*y(218);
jac(4,218)=jac(4,218)-k(1)*y(4);
jac(218,4)=jac(218,4)-k(1)*y(218);
jac(218,218)=jac(218,218)-k(1)*y(4);
jac(219,4)=jac(219,4)+k(1)*y(218);
jac(219,218)=jac(219,218)+k(1)*y(4);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(219,219)=jac(219,219)-k(34);
jac(437,219)=jac(437,219)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(219,219)=jac(219,219)-k(31);
jac(214,219)=jac(214,219)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~pY) 

jac(219,219)=jac(219,219)-k(22);
jac(182,219)=jac(182,219)+k(22);
jac(315,219)=jac(315,219)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(219);
jac(3,219)=jac(3,219)-k(15)*y(3);
jac(219,3)=jac(219,3)-k(15)*y(219);
jac(219,219)=jac(219,219)-k(15)*y(3);
jac(223,3)=jac(223,3)+k(15)*y(219);
jac(223,219)=jac(223,219)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(1,1)=jac(1,1)-k(7)*y(219);
jac(1,219)=jac(1,219)-k(7)*y(1);
jac(219,1)=jac(219,1)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(1);
jac(220,1)=jac(220,1)+k(7)*y(219);
jac(220,219)=jac(220,219)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(7,7)=jac(7,7)-k(7)*y(219);
jac(7,219)=jac(7,219)-k(7)*y(7);
jac(219,7)=jac(219,7)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(7);
jac(217,7)=jac(217,7)+k(7)*y(219);
jac(217,219)=jac(217,219)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(219,219)=jac(219,219)-k(7)*y(335);
jac(219,335)=jac(219,335)-k(7)*y(219);
jac(335,219)=jac(335,219)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(219);
jac(433,219)=jac(433,219)+k(7)*y(335);
jac(433,335)=jac(433,335)+k(7)*y(219);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(219);
jac(257,219)=jac(257,219)-k(7)*y(257);
jac(219,257)=jac(219,257)-k(7)*y(219);
jac(219,219)=jac(219,219)-k(7)*y(257);
jac(577,257)=jac(577,257)+k(7)*y(219);
jac(577,219)=jac(577,219)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(219,219)=jac(219,219)-k(7)*y(227);
jac(219,227)=jac(219,227)-k(7)*y(219);
jac(227,219)=jac(227,219)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(219);
jac(551,219)=jac(551,219)+k(7)*y(227);
jac(551,227)=jac(551,227)+k(7)*y(219);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(219,219)=jac(219,219)-k(7)*y(222);
jac(219,222)=jac(219,222)-k(7)*y(219);
jac(222,219)=jac(222,219)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(219);
jac(517,219)=jac(517,219)+k(7)*y(222);
jac(517,222)=jac(517,222)+k(7)*y(219);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) 

jac(219,219)=jac(219,219)-k(2);
jac(4,219)=jac(4,219)+k(2);
jac(218,219)=jac(218,219)+k(2);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(220,220)=jac(220,220)-k(34);
jac(621,220)=jac(621,220)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(220,220)=jac(220,220)-k(31);
jac(216,220)=jac(216,220)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~pY) 

jac(220,220)=jac(220,220)-k(22);
jac(182,220)=jac(182,220)+k(22);
jac(234,220)=jac(234,220)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(220);
jac(3,220)=jac(3,220)-k(15)*y(3);
jac(220,3)=jac(220,3)-k(15)*y(220);
jac(220,220)=jac(220,220)-k(15)*y(3);
jac(224,3)=jac(224,3)+k(15)*y(220);
jac(224,220)=jac(224,220)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(220);
jac(3,220)=jac(3,220)-k(9)*y(3);
jac(220,3)=jac(220,3)-k(9)*y(220);
jac(220,220)=jac(220,220)-k(9)*y(3);
jac(217,3)=jac(217,3)+k(9)*y(220);
jac(217,220)=jac(217,220)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

jac(220,220)=jac(220,220)-k(8);
jac(1,220)=jac(1,220)+k(8);
jac(219,220)=jac(219,220)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(220,220)=jac(220,220)-k(6);
jac(6,220)=jac(6,220)+k(6);
jac(218,220)=jac(218,220)+k(6);

% rule    : R17
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(222,222)=jac(222,222)-k(34);
jac(43,222)=jac(43,222)+k(34);

% rule    : R9_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~Y, a~pY) 

jac(222,222)=jac(222,222)-k(22);
jac(182,222)=jac(182,222)+k(22);
jac(54,222)=jac(54,222)+k(22);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lyn(U, SH2) 

jac(222,222)=jac(222,222)-k(16);
jac(3,222)=jac(3,222)+k(16);
jac(218,222)=jac(218,222)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(222,222)=jac(222,222)-k(7)*y(223);
jac(222,223)=jac(222,223)-k(7)*y(222);
jac(223,222)=jac(223,222)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(222);
jac(640,222)=jac(640,222)+k(7)*y(223);
jac(640,223)=jac(640,223)+k(7)*y(222);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(222,222)=jac(222,222)-k(7)*y(228);
jac(222,228)=jac(222,228)-k(7)*y(222);
jac(228,222)=jac(228,222)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(222);
jac(489,222)=jac(489,222)+k(7)*y(228);
jac(489,228)=jac(489,228)+k(7)*y(222);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(315,315)=jac(315,315)-k(7)*y(222);
jac(315,222)=jac(315,222)-k(7)*y(315);
jac(222,315)=jac(222,315)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(315);
jac(623,315)=jac(623,315)+k(7)*y(222);
jac(623,222)=jac(623,222)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(319,319)=jac(319,319)-k(7)*y(222);
jac(319,222)=jac(319,222)-k(7)*y(319);
jac(222,319)=jac(222,319)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(319);
jac(626,319)=jac(626,319)+k(7)*y(222);
jac(626,222)=jac(626,222)+k(7)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(222,222)=jac(222,222)-k(7)*y(336);
jac(222,336)=jac(222,336)-k(7)*y(222);
jac(336,222)=jac(336,222)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(222);
jac(597,222)=jac(597,222)+k(7)*y(336);
jac(597,336)=jac(597,336)+k(7)*y(222);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(437,437)=jac(437,437)-k(7)*y(222);
jac(437,222)=jac(437,222)-k(7)*y(437);
jac(222,437)=jac(222,437)-k(7)*y(222);
jac(222,222)=jac(222,222)-k(7)*y(437);
jac(625,437)=jac(625,437)+k(7)*y(222);
jac(625,222)=jac(625,222)+k(7)*y(437);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(256,256)=jac(256,256)-k(5)*y(222);
jac(256,222)=jac(256,222)-k(5)*y(256);
jac(222,256)=jac(222,256)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(256);
jac(261,256)=jac(261,256)+k(5)*y(222);
jac(261,222)=jac(261,222)+k(5)*y(256);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(264,264)=jac(264,264)-k(5)*y(222);
jac(264,222)=jac(264,222)-k(5)*y(264);
jac(222,264)=jac(222,264)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(264);
jac(268,264)=jac(268,264)+k(5)*y(222);
jac(268,222)=jac(268,222)+k(5)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(222,222)=jac(222,222)-k(5)*y(380);
jac(222,380)=jac(222,380)-k(5)*y(222);
jac(380,222)=jac(380,222)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(222);
jac(279,222)=jac(279,222)+k(5)*y(380);
jac(279,380)=jac(279,380)+k(5)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(222,222)=jac(222,222)-k(5)*y(391);
jac(222,391)=jac(222,391)-k(5)*y(222);
jac(391,222)=jac(391,222)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(222);
jac(649,222)=jac(649,222)+k(5)*y(391);
jac(649,391)=jac(649,391)+k(5)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(222,222)=jac(222,222)-k(5)*y(407);
jac(222,407)=jac(222,407)-k(5)*y(222);
jac(407,222)=jac(407,222)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(222);
jac(289,222)=jac(289,222)+k(5)*y(407);
jac(289,407)=jac(289,407)+k(5)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(222,222)=jac(222,222)-k(5)*y(441);
jac(222,441)=jac(222,441)-k(5)*y(222);
jac(441,222)=jac(441,222)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(222);
jac(284,222)=jac(284,222)+k(5)*y(441);
jac(284,441)=jac(284,441)+k(5)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(222,222)=jac(222,222)-k(5)*y(515);
jac(222,515)=jac(222,515)-k(5)*y(222);
jac(515,222)=jac(515,222)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(222);
jac(640,222)=jac(640,222)+k(5)*y(515);
jac(640,515)=jac(640,515)+k(5)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(222,222)=jac(222,222)-k(5)*y(543);
jac(222,543)=jac(222,543)-k(5)*y(222);
jac(543,222)=jac(543,222)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(222);
jac(641,222)=jac(641,222)+k(5)*y(543);
jac(641,543)=jac(641,543)+k(5)*y(222);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(222,222)=jac(222,222)-k(5)*y(554);
jac(222,554)=jac(222,554)-k(5)*y(222);
jac(554,222)=jac(554,222)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(222);
jac(639,222)=jac(639,222)+k(5)*y(554);
jac(639,554)=jac(639,554)+k(5)*y(222);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(9,9)=jac(9,9)-k(5)*y(222);
jac(9,222)=jac(9,222)-k(5)*y(9);
jac(222,9)=jac(222,9)-k(5)*y(222);
jac(222,222)=jac(222,222)-k(5)*y(9);
jac(221,9)=jac(221,9)+k(5)*y(222);
jac(221,222)=jac(221,222)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(4,4)=jac(4,4)-k(3)*y(222);
jac(4,222)=jac(4,222)-k(3)*y(4);
jac(222,4)=jac(222,4)-k(3)*y(222);
jac(222,222)=jac(222,222)-k(3)*y(4);
jac(515,4)=jac(515,4)+k(3)*y(222);
jac(515,222)=jac(515,222)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(4,4)=jac(4,4)-k(1)*y(222);
jac(4,222)=jac(4,222)-k(1)*y(4);
jac(222,4)=jac(222,4)-k(1)*y(222);
jac(222,222)=jac(222,222)-k(1)*y(4);
jac(223,4)=jac(223,4)+k(1)*y(222);
jac(223,222)=jac(223,222)+k(1)*y(4);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(223,223)=jac(223,223)-k(34);
jac(44,223)=jac(44,223)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

jac(223,223)=jac(223,223)-k(22);
jac(182,223)=jac(182,223)+k(22);
jac(55,223)=jac(55,223)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(223,223)=jac(223,223)-k(16);
jac(3,223)=jac(3,223)+k(16);
jac(219,223)=jac(219,223)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(1,1)=jac(1,1)-k(7)*y(223);
jac(1,223)=jac(1,223)-k(7)*y(1);
jac(223,1)=jac(223,1)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(1);
jac(224,1)=jac(224,1)+k(7)*y(223);
jac(224,223)=jac(224,223)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(7,7)=jac(7,7)-k(7)*y(223);
jac(7,223)=jac(7,223)-k(7)*y(7);
jac(223,7)=jac(223,7)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(7);
jac(221,7)=jac(221,7)+k(7)*y(223);
jac(221,223)=jac(221,223)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(7)*y(223);
jac(335,223)=jac(335,223)-k(7)*y(335);
jac(223,335)=jac(223,335)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(335);
jac(289,335)=jac(289,335)+k(7)*y(223);
jac(289,223)=jac(289,223)+k(7)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(223);
jac(257,223)=jac(257,223)-k(7)*y(257);
jac(223,257)=jac(223,257)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(257);
jac(261,257)=jac(261,257)+k(7)*y(223);
jac(261,223)=jac(261,223)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(227,227)=jac(227,227)-k(7)*y(223);
jac(227,223)=jac(227,223)-k(7)*y(227);
jac(223,227)=jac(223,227)-k(7)*y(223);
jac(223,223)=jac(223,223)-k(7)*y(227);
jac(641,227)=jac(641,227)+k(7)*y(223);
jac(641,223)=jac(641,223)+k(7)*y(227);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) 

jac(223,223)=jac(223,223)-k(2);
jac(4,223)=jac(4,223)+k(2);
jac(222,223)=jac(222,223)+k(2);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(224,224)=jac(224,224)-k(34);
jac(45,224)=jac(45,224)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

jac(224,224)=jac(224,224)-k(22);
jac(182,224)=jac(182,224)+k(22);
jac(233,224)=jac(233,224)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(224,224)=jac(224,224)-k(20);
jac(282,224)=jac(282,224)+k(20);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(224,224)=jac(224,224)-k(18);
jac(646,224)=jac(646,224)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(224,224)=jac(224,224)-k(16);
jac(3,224)=jac(3,224)+k(16);
jac(220,224)=jac(220,224)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(224);
jac(3,224)=jac(3,224)-k(9)*y(3);
jac(224,3)=jac(224,3)-k(9)*y(224);
jac(224,224)=jac(224,224)-k(9)*y(3);
jac(221,3)=jac(221,3)+k(9)*y(224);
jac(221,224)=jac(221,224)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

jac(224,224)=jac(224,224)-k(8);
jac(1,224)=jac(1,224)+k(8);
jac(223,224)=jac(223,224)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(224,224)=jac(224,224)-k(6);
jac(6,224)=jac(6,224)+k(6);
jac(222,224)=jac(222,224)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(225,225)=jac(225,225)-k(34);
jac(46,225)=jac(46,225)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(225,225)=jac(225,225)-k(32);
jac(221,225)=jac(221,225)+k(32);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(225,225)=jac(225,225)-k(23);
jac(205,225)=jac(205,225)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(225,225)=jac(225,225)-k(22);
jac(182,225)=jac(182,225)+k(22);
jac(53,225)=jac(53,225)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(225);
jac(197,225)=jac(197,225)-k(21)*y(197);
jac(225,197)=jac(225,197)-k(21)*y(225);
jac(225,225)=jac(225,225)-k(21)*y(197);
jac(289,197)=jac(289,197)+k(21)*y(225);
jac(289,225)=jac(289,225)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(225);
jac(32,225)=jac(32,225)-k(21)*y(32);
jac(225,32)=jac(225,32)-k(21)*y(225);
jac(225,225)=jac(225,225)-k(21)*y(32);
jac(284,32)=jac(284,32)+k(21)*y(225);
jac(284,225)=jac(284,225)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(225);
jac(2,225)=jac(2,225)-k(21)*y(2);
jac(225,2)=jac(225,2)-k(21)*y(225);
jac(225,225)=jac(225,225)-k(21)*y(2);
jac(279,2)=jac(279,2)+k(21)*y(225);
jac(279,225)=jac(279,225)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(225,225)=jac(225,225)-k(16);
jac(3,225)=jac(3,225)+k(16);
jac(430,225)=jac(430,225)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(225,225)=jac(225,225)-k(10);
jac(3,225)=jac(3,225)+k(10);
jac(282,225)=jac(282,225)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(225,225)=jac(225,225)-k(8);
jac(48,225)=jac(48,225)+k(8);
jac(223,225)=jac(223,225)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(225,225)=jac(225,225)-k(6);
jac(47,225)=jac(47,225)+k(6);
jac(222,225)=jac(222,225)+k(6);

% rule    : R17
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(227,227)=jac(227,227)-k(34);
jac(37,227)=jac(37,227)+k(34);

% rule    : R16
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(227,227)=jac(227,227)-k(33);
jac(222,227)=jac(222,227)+k(33);

% rule    : R9_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Syk(tSH2, l~pY, a~pY) 

jac(227,227)=jac(227,227)-k(22);
jac(197,227)=jac(197,227)+k(22);
jac(54,227)=jac(54,227)+k(22);

% rule    : R6_op
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lyn(U, SH2) 

jac(227,227)=jac(227,227)-k(16);
jac(3,227)=jac(3,227)+k(16);
jac(257,227)=jac(257,227)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(227,227)=jac(227,227)-k(7)*y(228);
jac(227,228)=jac(227,228)-k(7)*y(227);
jac(228,227)=jac(228,227)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(227);
jac(642,227)=jac(642,227)+k(7)*y(228);
jac(642,228)=jac(642,228)+k(7)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(315,315)=jac(315,315)-k(7)*y(227);
jac(315,227)=jac(315,227)-k(7)*y(315);
jac(227,315)=jac(227,315)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(315);
jac(581,315)=jac(581,315)+k(7)*y(227);
jac(581,227)=jac(581,227)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(319,319)=jac(319,319)-k(7)*y(227);
jac(319,227)=jac(319,227)-k(7)*y(319);
jac(227,319)=jac(227,319)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(319);
jac(565,319)=jac(565,319)+k(7)*y(227);
jac(565,227)=jac(565,227)+k(7)*y(319);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(227,227)=jac(227,227)-k(7)*y(336);
jac(227,336)=jac(227,336)-k(7)*y(227);
jac(336,227)=jac(336,227)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(227);
jac(601,227)=jac(601,227)+k(7)*y(336);
jac(601,336)=jac(601,336)+k(7)*y(227);

% rule    : R2_1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(437,437)=jac(437,437)-k(7)*y(227);
jac(437,227)=jac(437,227)-k(7)*y(437);
jac(227,437)=jac(227,437)-k(7)*y(227);
jac(227,227)=jac(227,227)-k(7)*y(437);
jac(546,437)=jac(546,437)+k(7)*y(227);
jac(546,227)=jac(546,227)+k(7)*y(437);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(256,256)=jac(256,256)-k(5)*y(227);
jac(256,227)=jac(256,227)-k(5)*y(256);
jac(227,256)=jac(227,256)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(256);
jac(492,256)=jac(492,256)+k(5)*y(227);
jac(492,227)=jac(492,227)+k(5)*y(256);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(264,264)=jac(264,264)-k(5)*y(227);
jac(264,227)=jac(264,227)-k(5)*y(264);
jac(227,264)=jac(227,264)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(264);
jac(488,264)=jac(488,264)+k(5)*y(227);
jac(488,227)=jac(488,227)+k(5)*y(264);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(227,227)=jac(227,227)-k(5)*y(380);
jac(227,380)=jac(227,380)-k(5)*y(227);
jac(380,227)=jac(380,227)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(227);
jac(428,227)=jac(428,227)+k(5)*y(380);
jac(428,380)=jac(428,380)+k(5)*y(227);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(227,227)=jac(227,227)-k(5)*y(391);
jac(227,391)=jac(227,391)-k(5)*y(227);
jac(391,227)=jac(391,227)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(227);
jac(476,227)=jac(476,227)+k(5)*y(391);
jac(476,391)=jac(476,391)+k(5)*y(227);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(227,227)=jac(227,227)-k(5)*y(407);
jac(227,407)=jac(227,407)-k(5)*y(227);
jac(407,227)=jac(407,227)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(227);
jac(475,227)=jac(475,227)+k(5)*y(407);
jac(475,407)=jac(475,407)+k(5)*y(227);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(227,227)=jac(227,227)-k(5)*y(441);
jac(227,441)=jac(227,441)-k(5)*y(227);
jac(441,227)=jac(441,227)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(227);
jac(473,227)=jac(473,227)+k(5)*y(441);
jac(473,441)=jac(473,441)+k(5)*y(227);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(227,227)=jac(227,227)-k(5)*y(515);
jac(227,515)=jac(227,515)-k(5)*y(227);
jac(515,227)=jac(515,227)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(227);
jac(489,227)=jac(489,227)+k(5)*y(515);
jac(489,515)=jac(489,515)+k(5)*y(227);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(227,227)=jac(227,227)-k(5)*y(543);
jac(227,543)=jac(227,543)-k(5)*y(227);
jac(543,227)=jac(543,227)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(227);
jac(642,227)=jac(642,227)+k(5)*y(543);
jac(642,543)=jac(642,543)+k(5)*y(227);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(227,227)=jac(227,227)-k(5)*y(554);
jac(227,554)=jac(227,554)-k(5)*y(227);
jac(554,227)=jac(554,227)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(227);
jac(561,227)=jac(561,227)+k(5)*y(554);
jac(561,554)=jac(561,554)+k(5)*y(227);

% rule    : R2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(9,9)=jac(9,9)-k(5)*y(227);
jac(9,227)=jac(9,227)-k(5)*y(9);
jac(227,9)=jac(227,9)-k(5)*y(227);
jac(227,227)=jac(227,227)-k(5)*y(9);
jac(226,9)=jac(226,9)+k(5)*y(227);
jac(226,227)=jac(226,227)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(4,4)=jac(4,4)-k(3)*y(227);
jac(4,227)=jac(4,227)-k(3)*y(4);
jac(227,4)=jac(227,4)-k(3)*y(227);
jac(227,227)=jac(227,227)-k(3)*y(4);
jac(543,4)=jac(543,4)+k(3)*y(227);
jac(543,227)=jac(543,227)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(4,4)=jac(4,4)-k(1)*y(227);
jac(4,227)=jac(4,227)-k(1)*y(4);
jac(227,4)=jac(227,4)-k(1)*y(227);
jac(227,227)=jac(227,227)-k(1)*y(4);
jac(228,4)=jac(228,4)+k(1)*y(227);
jac(228,227)=jac(228,227)+k(1)*y(4);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(228,228)=jac(228,228)-k(34);
jac(38,228)=jac(38,228)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(228,228)=jac(228,228)-k(33);
jac(223,228)=jac(223,228)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

jac(228,228)=jac(228,228)-k(22);
jac(197,228)=jac(197,228)+k(22);
jac(55,228)=jac(55,228)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(228,228)=jac(228,228)-k(16);
jac(3,228)=jac(3,228)+k(16);
jac(319,228)=jac(319,228)+k(16);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(1,1)=jac(1,1)-k(7)*y(228);
jac(1,228)=jac(1,228)-k(7)*y(1);
jac(228,1)=jac(228,1)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(1);
jac(229,1)=jac(229,1)+k(7)*y(228);
jac(229,228)=jac(229,228)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(7,7)=jac(7,7)-k(7)*y(228);
jac(7,228)=jac(7,228)-k(7)*y(7);
jac(228,7)=jac(228,7)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(7);
jac(226,7)=jac(226,7)+k(7)*y(228);
jac(226,228)=jac(226,228)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(7)*y(228);
jac(335,228)=jac(335,228)-k(7)*y(335);
jac(228,335)=jac(228,335)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(335);
jac(475,335)=jac(475,335)+k(7)*y(228);
jac(475,228)=jac(475,228)+k(7)*y(335);

% rule    : R2_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(228);
jac(257,228)=jac(257,228)-k(7)*y(257);
jac(228,257)=jac(228,257)-k(7)*y(228);
jac(228,228)=jac(228,228)-k(7)*y(257);
jac(492,257)=jac(492,257)+k(7)*y(228);
jac(492,228)=jac(492,228)+k(7)*y(257);

% rule    : R1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) 

jac(228,228)=jac(228,228)-k(2);
jac(4,228)=jac(4,228)+k(2);
jac(227,228)=jac(227,228)+k(2);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(230,230)=jac(230,230)-k(34);
jac(35,230)=jac(35,230)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(230,230)=jac(230,230)-k(33);
jac(220,230)=jac(220,230)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(230,230)=jac(230,230)-k(31);
jac(211,230)=jac(211,230)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~pY) 

jac(230,230)=jac(230,230)-k(22);
jac(197,230)=jac(197,230)+k(22);
jac(234,230)=jac(234,230)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(230);
jac(3,230)=jac(3,230)-k(15)*y(3);
jac(230,3)=jac(230,3)-k(15)*y(230);
jac(230,230)=jac(230,230)-k(15)*y(3);
jac(229,3)=jac(229,3)+k(15)*y(230);
jac(229,230)=jac(229,230)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(230);
jac(3,230)=jac(3,230)-k(9)*y(3);
jac(230,3)=jac(230,3)-k(9)*y(230);
jac(230,230)=jac(230,230)-k(9)*y(3);
jac(212,3)=jac(212,3)+k(9)*y(230);
jac(212,230)=jac(212,230)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

jac(230,230)=jac(230,230)-k(8);
jac(1,230)=jac(1,230)+k(8);
jac(319,230)=jac(319,230)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(230,230)=jac(230,230)-k(6);
jac(6,230)=jac(6,230)+k(6);
jac(257,230)=jac(257,230)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(231,231)=jac(231,231)-k(34);
jac(240,231)=jac(240,231)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(231,231)=jac(231,231)-k(33);
jac(646,231)=jac(646,231)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(231,231)=jac(231,231)-k(31);
jac(229,231)=jac(229,231)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

jac(231,231)=jac(231,231)-k(22);
jac(197,231)=jac(197,231)+k(22);
jac(235,231)=jac(235,231)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(231,231)=jac(231,231)-k(20);
jac(491,231)=jac(491,231)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(231,231)=jac(231,231)-k(16);
jac(3,231)=jac(3,231)+k(16);
jac(321,231)=jac(321,231)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(231);
jac(3,231)=jac(3,231)-k(15)*y(3);
jac(231,3)=jac(231,3)-k(15)*y(231);
jac(231,231)=jac(231,231)-k(15)*y(3);
jac(320,3)=jac(320,3)+k(15)*y(231);
jac(320,231)=jac(320,231)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

jac(231,231)=jac(231,231)-k(8);
jac(72,231)=jac(72,231)+k(8);
jac(228,231)=jac(228,231)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(231,231)=jac(231,231)-k(6);
jac(71,231)=jac(71,231)+k(6);
jac(227,231)=jac(227,231)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(232,232)=jac(232,232)-k(34);
jac(249,232)=jac(249,232)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(232,232)=jac(232,232)-k(33);
jac(282,232)=jac(282,232)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(232,232)=jac(232,232)-k(32);
jac(229,232)=jac(229,232)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

jac(232,232)=jac(232,232)-k(22);
jac(197,232)=jac(197,232)+k(22);
jac(56,232)=jac(56,232)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(232);
jac(197,232)=jac(197,232)-k(21)*y(197);
jac(232,197)=jac(232,197)-k(21)*y(232);
jac(232,232)=jac(232,232)-k(21)*y(197);
jac(294,197)=jac(294,197)+k(21)*y(232);
jac(294,232)=jac(294,232)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(232);
jac(32,232)=jac(32,232)-k(21)*y(32);
jac(232,32)=jac(232,32)-k(21)*y(232);
jac(232,232)=jac(232,232)-k(21)*y(32);
jac(474,32)=jac(474,32)+k(21)*y(232);
jac(474,232)=jac(474,232)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(232);
jac(2,232)=jac(2,232)-k(21)*y(2);
jac(232,2)=jac(232,2)-k(21)*y(232);
jac(232,232)=jac(232,232)-k(21)*y(2);
jac(484,2)=jac(484,2)+k(21)*y(232);
jac(484,232)=jac(484,232)+k(21)*y(2);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(232,232)=jac(232,232)-k(18);
jac(491,232)=jac(491,232)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(232,232)=jac(232,232)-k(16);
jac(3,232)=jac(3,232)+k(16);
jac(618,232)=jac(618,232)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(232);
jac(3,232)=jac(3,232)-k(9)*y(3);
jac(232,3)=jac(232,3)-k(9)*y(232);
jac(232,232)=jac(232,232)-k(9)*y(3);
jac(205,3)=jac(205,3)+k(9)*y(232);
jac(205,232)=jac(205,232)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

jac(232,232)=jac(232,232)-k(8);
jac(15,232)=jac(15,232)+k(8);
jac(228,232)=jac(228,232)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(232,232)=jac(232,232)-k(6);
jac(98,232)=jac(98,232)+k(6);
jac(227,232)=jac(227,232)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(234,234)=jac(234,234)-k(32);
jac(69,234)=jac(69,234)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(234,234)=jac(234,234)-k(31);
jac(18,234)=jac(18,234)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(234);
jac(197,234)=jac(197,234)-k(21)*y(197);
jac(234,197)=jac(234,197)-k(21)*y(234);
jac(234,234)=jac(234,234)-k(21)*y(197);
jac(230,197)=jac(230,197)+k(21)*y(234);
jac(230,234)=jac(230,234)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(234);
jac(32,234)=jac(32,234)-k(21)*y(32);
jac(234,32)=jac(234,32)-k(21)*y(234);
jac(234,234)=jac(234,234)-k(21)*y(32);
jac(35,32)=jac(35,32)+k(21)*y(234);
jac(35,234)=jac(35,234)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(234);
jac(2,234)=jac(2,234)-k(21)*y(2);
jac(234,2)=jac(234,2)-k(21)*y(234);
jac(234,234)=jac(234,234)-k(21)*y(2);
jac(621,2)=jac(621,2)+k(21)*y(234);
jac(621,234)=jac(621,234)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(234);
jac(3,234)=jac(3,234)-k(15)*y(3);
jac(234,3)=jac(234,3)-k(15)*y(234);
jac(234,234)=jac(234,234)-k(15)*y(3);
jac(233,3)=jac(233,3)+k(15)*y(234);
jac(233,234)=jac(233,234)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(234);
jac(3,234)=jac(3,234)-k(9)*y(3);
jac(234,3)=jac(234,3)-k(9)*y(234);
jac(234,234)=jac(234,234)-k(9)*y(3);
jac(19,3)=jac(19,3)+k(9)*y(234);
jac(19,234)=jac(19,234)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) 

jac(234,234)=jac(234,234)-k(8);
jac(1,234)=jac(1,234)+k(8);
jac(315,234)=jac(315,234)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) 

jac(234,234)=jac(234,234)-k(6);
jac(77,234)=jac(77,234)+k(6);
jac(6,234)=jac(6,234)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(236,236)=jac(236,236)-k(32);
jac(73,236)=jac(73,236)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(197,197)=jac(197,197)-k(21)*y(236);
jac(197,236)=jac(197,236)-k(21)*y(197);
jac(236,197)=jac(236,197)-k(21)*y(236);
jac(236,236)=jac(236,236)-k(21)*y(197);
jac(320,197)=jac(320,197)+k(21)*y(236);
jac(320,236)=jac(320,236)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(236);
jac(32,236)=jac(32,236)-k(21)*y(32);
jac(236,32)=jac(236,32)-k(21)*y(236);
jac(236,236)=jac(236,236)-k(21)*y(32);
jac(241,32)=jac(241,32)+k(21)*y(236);
jac(241,236)=jac(241,236)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(236);
jac(2,236)=jac(2,236)-k(21)*y(2);
jac(236,2)=jac(236,2)-k(21)*y(236);
jac(236,236)=jac(236,236)-k(21)*y(2);
jac(654,2)=jac(654,2)+k(21)*y(236);
jac(654,236)=jac(654,236)+k(21)*y(2);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(236,236)=jac(236,236)-k(20);
jac(627,236)=jac(627,236)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) 

jac(236,236)=jac(236,236)-k(16);
jac(3,236)=jac(3,236)+k(16);
jac(235,236)=jac(235,236)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) 

jac(236,236)=jac(236,236)-k(16);
jac(3,236)=jac(3,236)+k(16);
jac(314,236)=jac(314,236)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(236,236)=jac(236,236)-k(8);
jac(61,236)=jac(61,236)+k(8);
jac(55,236)=jac(55,236)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(236,236)=jac(236,236)-k(6);
jac(54,236)=jac(54,236)+k(6);
jac(122,236)=jac(122,236)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(237,237)=jac(237,237)-k(32);
jac(74,237)=jac(74,237)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(237,237)=jac(237,237)-k(31);
jac(234,237)=jac(234,237)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(237,237)=jac(237,237)-k(31);
jac(331,237)=jac(331,237)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(237);
jac(197,237)=jac(197,237)-k(21)*y(197);
jac(237,197)=jac(237,197)-k(21)*y(237);
jac(237,237)=jac(237,237)-k(21)*y(197);
jac(321,197)=jac(321,197)+k(21)*y(237);
jac(321,237)=jac(321,237)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(237);
jac(32,237)=jac(32,237)-k(21)*y(32);
jac(237,32)=jac(237,32)-k(21)*y(237);
jac(237,237)=jac(237,237)-k(21)*y(32);
jac(242,32)=jac(242,32)+k(21)*y(237);
jac(242,237)=jac(242,237)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(237);
jac(2,237)=jac(2,237)-k(21)*y(2);
jac(237,2)=jac(237,2)-k(21)*y(237);
jac(237,237)=jac(237,237)-k(21)*y(2);
jac(653,2)=jac(653,2)+k(21)*y(237);
jac(653,237)=jac(653,237)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(237);
jac(3,237)=jac(3,237)-k(15)*y(3);
jac(237,3)=jac(237,3)-k(15)*y(237);
jac(237,237)=jac(237,237)-k(15)*y(3);
jac(235,3)=jac(235,3)+k(15)*y(237);
jac(235,237)=jac(235,237)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(3,3)=jac(3,3)-k(15)*y(237);
jac(3,237)=jac(3,237)-k(15)*y(3);
jac(237,3)=jac(237,3)-k(15)*y(237);
jac(237,237)=jac(237,237)-k(15)*y(3);
jac(314,3)=jac(314,3)+k(15)*y(237);
jac(314,237)=jac(314,237)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) 

jac(237,237)=jac(237,237)-k(8);
jac(72,237)=jac(72,237)+k(8);
jac(315,237)=jac(315,237)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) 

jac(237,237)=jac(237,237)-k(6);
jac(77,237)=jac(77,237)+k(6);
jac(71,237)=jac(71,237)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(238,238)=jac(238,238)-k(32);
jac(235,238)=jac(235,238)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(238,238)=jac(238,238)-k(32);
jac(75,238)=jac(75,238)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(238,238)=jac(238,238)-k(31);
jac(56,238)=jac(56,238)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(238);
jac(197,238)=jac(197,238)-k(21)*y(197);
jac(238,197)=jac(238,197)-k(21)*y(238);
jac(238,238)=jac(238,238)-k(21)*y(197);
jac(491,197)=jac(491,197)+k(21)*y(238);
jac(491,238)=jac(491,238)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(238);
jac(32,238)=jac(32,238)-k(21)*y(32);
jac(238,32)=jac(238,32)-k(21)*y(238);
jac(238,238)=jac(238,238)-k(21)*y(32);
jac(243,32)=jac(243,32)+k(21)*y(238);
jac(243,238)=jac(243,238)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(238);
jac(2,238)=jac(2,238)-k(21)*y(2);
jac(238,2)=jac(238,2)-k(21)*y(238);
jac(238,238)=jac(238,238)-k(21)*y(2);
jac(250,2)=jac(250,2)+k(21)*y(238);
jac(250,238)=jac(250,238)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(238);
jac(197,238)=jac(197,238)-k(21)*y(197);
jac(238,197)=jac(238,197)-k(21)*y(238);
jac(238,238)=jac(238,238)-k(21)*y(197);
jac(260,197)=jac(260,197)+k(21)*y(238);
jac(260,238)=jac(260,238)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(238);
jac(32,238)=jac(32,238)-k(21)*y(32);
jac(238,32)=jac(238,32)-k(21)*y(238);
jac(238,238)=jac(238,238)-k(21)*y(32);
jac(634,32)=jac(634,32)+k(21)*y(238);
jac(634,238)=jac(634,238)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(238);
jac(2,238)=jac(2,238)-k(21)*y(2);
jac(238,2)=jac(238,2)-k(21)*y(238);
jac(238,238)=jac(238,238)-k(21)*y(2);
jac(629,2)=jac(629,2)+k(21)*y(238);
jac(629,238)=jac(629,238)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

jac(238,238)=jac(238,238)-k(16);
jac(3,238)=jac(3,238)+k(16);
jac(584,238)=jac(584,238)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(238);
jac(3,238)=jac(3,238)-k(15)*y(3);
jac(238,3)=jac(238,3)-k(15)*y(238);
jac(238,238)=jac(238,238)-k(15)*y(3);
jac(627,3)=jac(627,3)+k(15)*y(238);
jac(627,238)=jac(627,238)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) 

jac(238,238)=jac(238,238)-k(8);
jac(77,238)=jac(77,238)+k(8);
jac(55,238)=jac(55,238)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(238,238)=jac(238,238)-k(6);
jac(76,238)=jac(76,238)+k(6);
jac(54,238)=jac(54,238)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(239,239)=jac(239,239)-k(31);
jac(45,239)=jac(45,239)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

jac(239,239)=jac(239,239)-k(22);
jac(2,239)=jac(2,239)+k(22);
jac(235,239)=jac(235,239)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(239,239)=jac(239,239)-k(20);
jac(250,239)=jac(250,239)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(239,239)=jac(239,239)-k(16);
jac(3,239)=jac(3,239)+k(16);
jac(653,239)=jac(653,239)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(239);
jac(3,239)=jac(3,239)-k(15)*y(3);
jac(239,3)=jac(239,3)-k(15)*y(239);
jac(239,239)=jac(239,239)-k(15)*y(3);
jac(654,3)=jac(654,3)+k(15)*y(239);
jac(654,239)=jac(654,239)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

jac(239,239)=jac(239,239)-k(8);
jac(72,239)=jac(72,239)+k(8);
jac(44,239)=jac(44,239)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(239,239)=jac(239,239)-k(6);
jac(71,239)=jac(71,239)+k(6);
jac(43,239)=jac(43,239)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(241,241)=jac(241,241)-k(33);
jac(654,241)=jac(654,241)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(241,241)=jac(241,241)-k(22);
jac(32,241)=jac(32,241)+k(22);
jac(236,241)=jac(236,241)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(241,241)=jac(241,241)-k(20);
jac(244,241)=jac(244,241)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(241,241)=jac(241,241)-k(16);
jac(3,241)=jac(3,241)+k(16);
jac(240,241)=jac(240,241)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(241,241)=jac(241,241)-k(16);
jac(3,241)=jac(3,241)+k(16);
jac(317,241)=jac(317,241)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(241,241)=jac(241,241)-k(8);
jac(61,241)=jac(61,241)+k(8);
jac(38,241)=jac(38,241)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(241,241)=jac(241,241)-k(6);
jac(122,241)=jac(122,241)+k(6);
jac(37,241)=jac(37,241)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(242,242)=jac(242,242)-k(33);
jac(653,242)=jac(653,242)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(242,242)=jac(242,242)-k(31);
jac(35,242)=jac(35,242)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(242,242)=jac(242,242)-k(31);
jac(136,242)=jac(136,242)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~Y) 

jac(242,242)=jac(242,242)-k(22);
jac(32,242)=jac(32,242)+k(22);
jac(237,242)=jac(237,242)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(242);
jac(3,242)=jac(3,242)-k(15)*y(3);
jac(242,3)=jac(242,3)-k(15)*y(242);
jac(242,242)=jac(242,242)-k(15)*y(3);
jac(240,3)=jac(240,3)+k(15)*y(242);
jac(240,242)=jac(240,242)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(242);
jac(3,242)=jac(3,242)-k(15)*y(3);
jac(242,3)=jac(242,3)-k(15)*y(242);
jac(242,242)=jac(242,242)-k(15)*y(3);
jac(317,3)=jac(317,3)+k(15)*y(242);
jac(317,242)=jac(317,242)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

jac(242,242)=jac(242,242)-k(8);
jac(72,242)=jac(72,242)+k(8);
jac(34,242)=jac(34,242)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(242,242)=jac(242,242)-k(6);
jac(71,242)=jac(71,242)+k(6);
jac(33,242)=jac(33,242)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(244,244)=jac(244,244)-k(33);
jac(251,244)=jac(251,244)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(244,244)=jac(244,244)-k(32);
jac(241,244)=jac(241,244)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(244,244)=jac(244,244)-k(22);
jac(32,244)=jac(32,244)+k(22);
jac(627,244)=jac(627,244)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(244);
jac(197,244)=jac(197,244)-k(21)*y(197);
jac(244,197)=jac(244,197)-k(21)*y(244);
jac(244,244)=jac(244,244)-k(21)*y(197);
jac(643,197)=jac(643,197)+k(21)*y(244);
jac(643,244)=jac(643,244)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(244);
jac(32,244)=jac(32,244)-k(21)*y(32);
jac(244,32)=jac(244,32)-k(21)*y(244);
jac(244,244)=jac(244,244)-k(21)*y(32);
jac(567,32)=jac(567,32)+k(21)*y(244);
jac(567,244)=jac(567,244)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(244);
jac(2,244)=jac(2,244)-k(21)*y(2);
jac(244,2)=jac(244,2)-k(21)*y(244);
jac(244,244)=jac(244,244)-k(21)*y(2);
jac(631,2)=jac(631,2)+k(21)*y(244);
jac(631,244)=jac(631,244)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(244,244)=jac(244,244)-k(16);
jac(3,244)=jac(3,244)+k(16);
jac(243,244)=jac(243,244)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(244,244)=jac(244,244)-k(16);
jac(3,244)=jac(3,244)+k(16);
jac(534,244)=jac(534,244)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(244,244)=jac(244,244)-k(8);
jac(54,244)=jac(54,244)+k(8);
jac(38,244)=jac(38,244)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(244,244)=jac(244,244)-k(6);
jac(118,244)=jac(118,244)+k(6);
jac(37,244)=jac(37,244)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(245,245)=jac(245,245)-k(33);
jac(252,245)=jac(252,245)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(245,245)=jac(245,245)-k(32);
jac(242,245)=jac(242,245)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(245,245)=jac(245,245)-k(31);
jac(617,245)=jac(617,245)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(245,245)=jac(245,245)-k(31);
jac(533,245)=jac(533,245)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(245,245)=jac(245,245)-k(22);
jac(32,245)=jac(32,245)+k(22);
jac(584,245)=jac(584,245)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(245);
jac(197,245)=jac(197,245)-k(21)*y(197);
jac(245,197)=jac(245,197)-k(21)*y(245);
jac(245,245)=jac(245,245)-k(21)*y(197);
jac(582,197)=jac(582,197)+k(21)*y(245);
jac(582,245)=jac(582,245)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(245);
jac(32,245)=jac(32,245)-k(21)*y(32);
jac(245,32)=jac(245,32)-k(21)*y(245);
jac(245,245)=jac(245,245)-k(21)*y(32);
jac(568,32)=jac(568,32)+k(21)*y(245);
jac(568,245)=jac(568,245)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(245);
jac(2,245)=jac(2,245)-k(21)*y(2);
jac(245,2)=jac(245,2)-k(21)*y(245);
jac(245,245)=jac(245,245)-k(21)*y(2);
jac(499,2)=jac(499,2)+k(21)*y(245);
jac(499,245)=jac(499,245)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(245);
jac(3,245)=jac(3,245)-k(15)*y(3);
jac(245,3)=jac(245,3)-k(15)*y(245);
jac(245,245)=jac(245,245)-k(15)*y(3);
jac(243,3)=jac(243,3)+k(15)*y(245);
jac(243,245)=jac(243,245)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(245);
jac(3,245)=jac(3,245)-k(15)*y(3);
jac(245,3)=jac(245,3)-k(15)*y(245);
jac(245,245)=jac(245,245)-k(15)*y(3);
jac(534,3)=jac(534,3)+k(15)*y(245);
jac(534,245)=jac(534,245)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

jac(245,245)=jac(245,245)-k(8);
jac(77,245)=jac(77,245)+k(8);
jac(34,245)=jac(34,245)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(245,245)=jac(245,245)-k(6);
jac(76,245)=jac(76,245)+k(6);
jac(33,245)=jac(33,245)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(246,246)=jac(246,246)-k(33);
jac(253,246)=jac(253,246)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(246,246)=jac(246,246)-k(31);
jac(481,246)=jac(481,246)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(246,246)=jac(246,246)-k(28);
jac(493,246)=jac(493,246)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(246,246)=jac(246,246)-k(27);
jac(485,246)=jac(485,246)+k(27);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(246,246)=jac(246,246)-k(26);
jac(247,246)=jac(247,246)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(246,246)=jac(246,246)-k(22);
jac(2,246)=jac(2,246)+k(22);
jac(243,246)=jac(243,246)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(246,246)=jac(246,246)-k(22);
jac(32,246)=jac(32,246)+k(22);
jac(629,246)=jac(629,246)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(246,246)=jac(246,246)-k(16);
jac(3,246)=jac(3,246)+k(16);
jac(499,246)=jac(499,246)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(246);
jac(3,246)=jac(3,246)-k(15)*y(3);
jac(246,3)=jac(246,3)-k(15)*y(246);
jac(246,246)=jac(246,246)-k(15)*y(3);
jac(631,3)=jac(631,3)+k(15)*y(246);
jac(631,246)=jac(631,246)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(246,246)=jac(246,246)-k(8);
jac(111,246)=jac(111,246)+k(8);
jac(38,246)=jac(38,246)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(246,246)=jac(246,246)-k(6);
jac(110,246)=jac(110,246)+k(6);
jac(37,246)=jac(37,246)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(247,247)=jac(247,247)-k(33);
jac(246,247)=jac(246,247)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(247,247)=jac(247,247)-k(33);
jac(254,247)=jac(254,247)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(247,247)=jac(247,247)-k(31);
jac(483,247)=jac(483,247)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(247,247)=jac(247,247)-k(28);
jac(248,247)=jac(248,247)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(247,247)=jac(247,247)-k(27);
jac(635,247)=jac(635,247)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(247,247)=jac(247,247)-k(22);
jac(32,247)=jac(32,247)+k(22);
jac(243,247)=jac(243,247)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(247,247)=jac(247,247)-k(22);
jac(32,247)=jac(32,247)+k(22);
jac(634,247)=jac(634,247)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(247,247)=jac(247,247)-k(16);
jac(3,247)=jac(3,247)+k(16);
jac(568,247)=jac(568,247)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(247);
jac(3,247)=jac(3,247)-k(15)*y(3);
jac(247,3)=jac(247,3)-k(15)*y(247);
jac(247,247)=jac(247,247)-k(15)*y(3);
jac(567,3)=jac(567,3)+k(15)*y(247);
jac(567,247)=jac(567,247)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(247,247)=jac(247,247)-k(8);
jac(33,247)=jac(33,247)+k(8);
jac(38,247)=jac(38,247)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(247,247)=jac(247,247)-k(6);
jac(82,247)=jac(82,247)+k(6);
jac(37,247)=jac(37,247)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(248,248)=jac(248,248)-k(34);
jac(247,248)=jac(247,248)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(248,248)=jac(248,248)-k(33);
jac(493,248)=jac(493,248)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(248,248)=jac(248,248)-k(33);
jac(255,248)=jac(255,248)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(248,248)=jac(248,248)-k(31);
jac(293,248)=jac(293,248)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(248,248)=jac(248,248)-k(29);
jac(492,248)=jac(492,248)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(248,248)=jac(248,248)-k(22);
jac(197,248)=jac(197,248)+k(22);
jac(243,248)=jac(243,248)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(248,248)=jac(248,248)-k(22);
jac(32,248)=jac(32,248)+k(22);
jac(260,248)=jac(260,248)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(248,248)=jac(248,248)-k(16);
jac(3,248)=jac(3,248)+k(16);
jac(582,248)=jac(582,248)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(248);
jac(3,248)=jac(3,248)-k(15)*y(3);
jac(248,3)=jac(248,3)-k(15)*y(248);
jac(248,248)=jac(248,248)-k(15)*y(3);
jac(643,3)=jac(643,3)+k(15)*y(248);
jac(643,248)=jac(643,248)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(248,248)=jac(248,248)-k(8);
jac(257,248)=jac(257,248)+k(8);
jac(38,248)=jac(38,248)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(248,248)=jac(248,248)-k(6);
jac(256,248)=jac(256,248)+k(6);
jac(37,248)=jac(37,248)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(249,249)=jac(249,249)-k(33);
jac(49,249)=jac(49,249)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(249,249)=jac(249,249)-k(32);
jac(39,249)=jac(39,249)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

jac(249,249)=jac(249,249)-k(22);
jac(32,249)=jac(32,249)+k(22);
jac(56,249)=jac(56,249)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(249);
jac(197,249)=jac(197,249)-k(21)*y(197);
jac(249,197)=jac(249,197)-k(21)*y(249);
jac(249,249)=jac(249,249)-k(21)*y(197);
jac(293,197)=jac(293,197)+k(21)*y(249);
jac(293,249)=jac(293,249)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(249);
jac(32,249)=jac(32,249)-k(21)*y(32);
jac(249,32)=jac(249,32)-k(21)*y(249);
jac(249,249)=jac(249,249)-k(21)*y(32);
jac(483,32)=jac(483,32)+k(21)*y(249);
jac(483,249)=jac(483,249)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(249);
jac(2,249)=jac(2,249)-k(21)*y(2);
jac(249,2)=jac(249,2)-k(21)*y(249);
jac(249,249)=jac(249,249)-k(21)*y(2);
jac(481,2)=jac(481,2)+k(21)*y(249);
jac(481,249)=jac(481,249)+k(21)*y(2);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(249,249)=jac(249,249)-k(18);
jac(243,249)=jac(243,249)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(249,249)=jac(249,249)-k(16);
jac(3,249)=jac(3,249)+k(16);
jac(617,249)=jac(617,249)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(249);
jac(3,249)=jac(3,249)-k(9)*y(3);
jac(249,3)=jac(249,3)-k(9)*y(249);
jac(249,249)=jac(249,249)-k(9)*y(3);
jac(40,3)=jac(40,3)+k(9)*y(249);
jac(40,249)=jac(40,249)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

jac(249,249)=jac(249,249)-k(8);
jac(15,249)=jac(15,249)+k(8);
jac(38,249)=jac(38,249)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(249,249)=jac(249,249)-k(6);
jac(98,249)=jac(98,249)+k(6);
jac(37,249)=jac(37,249)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(251,251)=jac(251,251)-k(32);
jac(654,251)=jac(654,251)+k(32);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(251,251)=jac(251,251)-k(25);
jac(244,251)=jac(244,251)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(251,251)=jac(251,251)-k(22);
jac(2,251)=jac(2,251)+k(22);
jac(627,251)=jac(627,251)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(251);
jac(197,251)=jac(197,251)-k(21)*y(197);
jac(251,197)=jac(251,197)-k(21)*y(251);
jac(251,251)=jac(251,251)-k(21)*y(197);
jac(258,197)=jac(258,197)+k(21)*y(251);
jac(258,251)=jac(258,251)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(251);
jac(32,251)=jac(32,251)-k(21)*y(32);
jac(251,32)=jac(251,32)-k(21)*y(251);
jac(251,251)=jac(251,251)-k(21)*y(32);
jac(571,32)=jac(571,32)+k(21)*y(251);
jac(571,251)=jac(571,251)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(251);
jac(2,251)=jac(2,251)-k(21)*y(2);
jac(251,2)=jac(251,2)-k(21)*y(251);
jac(251,251)=jac(251,251)-k(21)*y(2);
jac(630,2)=jac(630,2)+k(21)*y(251);
jac(630,251)=jac(630,251)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(251,251)=jac(251,251)-k(16);
jac(3,251)=jac(3,251)+k(16);
jac(250,251)=jac(250,251)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(251,251)=jac(251,251)-k(16);
jac(3,251)=jac(3,251)+k(16);
jac(540,251)=jac(540,251)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(251,251)=jac(251,251)-k(8);
jac(54,251)=jac(54,251)+k(8);
jac(44,251)=jac(44,251)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(251,251)=jac(251,251)-k(6);
jac(118,251)=jac(118,251)+k(6);
jac(43,251)=jac(43,251)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(252,252)=jac(252,252)-k(32);
jac(653,252)=jac(653,252)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(252,252)=jac(252,252)-k(31);
jac(616,252)=jac(616,252)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(252,252)=jac(252,252)-k(31);
jac(508,252)=jac(508,252)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(252,252)=jac(252,252)-k(22);
jac(2,252)=jac(2,252)+k(22);
jac(584,252)=jac(584,252)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(252);
jac(197,252)=jac(197,252)-k(21)*y(197);
jac(252,197)=jac(252,197)-k(21)*y(252);
jac(252,252)=jac(252,252)-k(21)*y(197);
jac(259,197)=jac(259,197)+k(21)*y(252);
jac(259,252)=jac(259,252)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(252);
jac(32,252)=jac(32,252)-k(21)*y(32);
jac(252,32)=jac(252,32)-k(21)*y(252);
jac(252,252)=jac(252,252)-k(21)*y(32);
jac(572,32)=jac(572,32)+k(21)*y(252);
jac(572,252)=jac(572,252)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(252);
jac(2,252)=jac(2,252)-k(21)*y(2);
jac(252,2)=jac(252,2)-k(21)*y(252);
jac(252,252)=jac(252,252)-k(21)*y(2);
jac(503,2)=jac(503,2)+k(21)*y(252);
jac(503,252)=jac(503,252)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(252);
jac(3,252)=jac(3,252)-k(15)*y(3);
jac(252,3)=jac(252,3)-k(15)*y(252);
jac(252,252)=jac(252,252)-k(15)*y(3);
jac(250,3)=jac(250,3)+k(15)*y(252);
jac(250,252)=jac(250,252)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(252);
jac(3,252)=jac(3,252)-k(15)*y(3);
jac(252,3)=jac(252,3)-k(15)*y(252);
jac(252,252)=jac(252,252)-k(15)*y(3);
jac(540,3)=jac(540,3)+k(15)*y(252);
jac(540,252)=jac(540,252)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

jac(252,252)=jac(252,252)-k(8);
jac(77,252)=jac(77,252)+k(8);
jac(437,252)=jac(437,252)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(252,252)=jac(252,252)-k(6);
jac(76,252)=jac(76,252)+k(6);
jac(111,252)=jac(111,252)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(253,253)=jac(253,253)-k(31);
jac(274,253)=jac(274,253)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(253,253)=jac(253,253)-k(28);
jac(263,253)=jac(263,253)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(253,253)=jac(253,253)-k(27);
jac(281,253)=jac(281,253)+k(27);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(253,253)=jac(253,253)-k(26);
jac(254,253)=jac(254,253)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(253,253)=jac(253,253)-k(22);
jac(2,253)=jac(2,253)+k(22);
jac(250,253)=jac(250,253)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(253,253)=jac(253,253)-k(22);
jac(2,253)=jac(2,253)+k(22);
jac(629,253)=jac(629,253)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(253,253)=jac(253,253)-k(16);
jac(3,253)=jac(3,253)+k(16);
jac(503,253)=jac(503,253)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(253);
jac(3,253)=jac(3,253)-k(15)*y(3);
jac(253,3)=jac(253,3)-k(15)*y(253);
jac(253,253)=jac(253,253)-k(15)*y(3);
jac(630,3)=jac(630,3)+k(15)*y(253);
jac(630,253)=jac(630,253)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(253,253)=jac(253,253)-k(8);
jac(111,253)=jac(111,253)+k(8);
jac(44,253)=jac(44,253)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(253,253)=jac(253,253)-k(6);
jac(110,253)=jac(110,253)+k(6);
jac(43,253)=jac(43,253)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(254,254)=jac(254,254)-k(33);
jac(253,254)=jac(253,254)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(254,254)=jac(254,254)-k(31);
jac(277,254)=jac(277,254)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(254,254)=jac(254,254)-k(28);
jac(255,254)=jac(255,254)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(254,254)=jac(254,254)-k(27);
jac(286,254)=jac(286,254)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(254,254)=jac(254,254)-k(22);
jac(32,254)=jac(32,254)+k(22);
jac(250,254)=jac(250,254)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(254,254)=jac(254,254)-k(22);
jac(2,254)=jac(2,254)+k(22);
jac(634,254)=jac(634,254)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(254,254)=jac(254,254)-k(16);
jac(3,254)=jac(3,254)+k(16);
jac(572,254)=jac(572,254)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(254);
jac(3,254)=jac(3,254)-k(15)*y(3);
jac(254,3)=jac(254,3)-k(15)*y(254);
jac(254,254)=jac(254,254)-k(15)*y(3);
jac(571,3)=jac(571,3)+k(15)*y(254);
jac(571,254)=jac(571,254)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(254,254)=jac(254,254)-k(8);
jac(33,254)=jac(33,254)+k(8);
jac(44,254)=jac(44,254)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(254,254)=jac(254,254)-k(6);
jac(82,254)=jac(82,254)+k(6);
jac(43,254)=jac(43,254)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(256,256)=jac(256,256)-k(34);
jac(82,256)=jac(82,256)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(256,256)=jac(256,256)-k(33);
jac(264,256)=jac(264,256)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(256,256)=jac(256,256)-k(31);
jac(191,256)=jac(191,256)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~pY, a~pY) 

jac(256,256)=jac(256,256)-k(22);
jac(197,256)=jac(197,256)+k(22);
jac(76,256)=jac(76,256)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(256);
jac(3,256)=jac(3,256)-k(15)*y(3);
jac(256,3)=jac(256,3)-k(15)*y(256);
jac(256,256)=jac(256,256)-k(15)*y(3);
jac(543,3)=jac(543,3)+k(15)*y(256);
jac(543,256)=jac(543,256)+k(15)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(1,1)=jac(1,1)-k(5)*y(256);
jac(1,256)=jac(1,256)-k(5)*y(1);
jac(256,1)=jac(256,1)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(1);
jac(587,1)=jac(587,1)+k(5)*y(256);
jac(587,256)=jac(587,256)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(7,7)=jac(7,7)-k(5)*y(256);
jac(7,256)=jac(7,256)-k(5)*y(7);
jac(256,7)=jac(256,7)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(7);
jac(588,7)=jac(588,7)+k(5)*y(256);
jac(588,256)=jac(588,256)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(256,256)=jac(256,256)-k(5)*y(335);
jac(256,335)=jac(256,335)-k(5)*y(256);
jac(335,256)=jac(335,256)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(256);
jac(342,256)=jac(342,256)+k(5)*y(335);
jac(342,335)=jac(342,335)+k(5)*y(256);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(5)*y(256);
jac(257,256)=jac(257,256)-k(5)*y(257);
jac(256,257)=jac(256,257)-k(5)*y(256);
jac(256,256)=jac(256,256)-k(5)*y(257);
jac(583,257)=jac(583,257)+k(5)*y(256);
jac(583,256)=jac(583,256)+k(5)*y(257);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) 

jac(256,256)=jac(256,256)-k(4);
jac(4,256)=jac(4,256)+k(4);
jac(257,256)=jac(257,256)+k(4);

% rule    : R17
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(257,257)=jac(257,257)-k(34);
jac(33,257)=jac(33,257)+k(34);

% rule    : R16
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(257,257)=jac(257,257)-k(33);
jac(218,257)=jac(218,257)+k(33);

% rule    : R14
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(31);
jac(192,257)=jac(192,257)+k(31);

% rule    : R9_op
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(22);
jac(197,257)=jac(197,257)+k(22);
jac(77,257)=jac(77,257)+k(22);

% rule    : R6
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(257);
jac(3,257)=jac(3,257)-k(15)*y(3);
jac(257,3)=jac(257,3)-k(15)*y(257);
jac(257,257)=jac(257,257)-k(15)*y(3);
jac(227,3)=jac(227,3)+k(15)*y(257);
jac(227,257)=jac(227,257)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(315,315)=jac(315,315)-k(7)*y(257);
jac(315,257)=jac(315,257)-k(7)*y(315);
jac(257,315)=jac(257,315)-k(7)*y(257);
jac(257,257)=jac(257,257)-k(7)*y(315);
jac(578,315)=jac(578,315)+k(7)*y(257);
jac(578,257)=jac(578,257)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(319);
jac(257,319)=jac(257,319)-k(7)*y(257);
jac(319,257)=jac(319,257)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(257);
jac(583,257)=jac(583,257)+k(7)*y(319);
jac(583,319)=jac(583,319)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(336);
jac(257,336)=jac(257,336)-k(7)*y(257);
jac(336,257)=jac(336,257)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(257);
jac(342,257)=jac(342,257)+k(7)*y(336);
jac(342,336)=jac(342,336)+k(7)*y(257);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(7)*y(437);
jac(257,437)=jac(257,437)-k(7)*y(257);
jac(437,257)=jac(437,257)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(257);
jac(259,257)=jac(259,257)+k(7)*y(437);
jac(259,437)=jac(259,437)+k(7)*y(257);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(257,257)=jac(257,257)-k(5)*y(264);
jac(257,264)=jac(257,264)-k(5)*y(257);
jac(264,257)=jac(264,257)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(257);
jac(490,257)=jac(490,257)+k(5)*y(264);
jac(490,264)=jac(490,264)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(5)*y(380);
jac(257,380)=jac(257,380)-k(5)*y(257);
jac(380,257)=jac(380,257)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(257);
jac(426,257)=jac(426,257)+k(5)*y(380);
jac(426,380)=jac(426,380)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(257,257)=jac(257,257)-k(5)*y(391);
jac(257,391)=jac(257,391)-k(5)*y(257);
jac(391,257)=jac(391,257)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(257);
jac(398,257)=jac(398,257)+k(5)*y(391);
jac(398,391)=jac(398,391)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(5)*y(407);
jac(257,407)=jac(257,407)-k(5)*y(257);
jac(407,257)=jac(407,257)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(257);
jac(434,257)=jac(434,257)+k(5)*y(407);
jac(434,407)=jac(434,407)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(257,257)=jac(257,257)-k(5)*y(441);
jac(257,441)=jac(257,441)-k(5)*y(257);
jac(441,257)=jac(441,257)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(257);
jac(471,257)=jac(471,257)+k(5)*y(441);
jac(471,441)=jac(471,441)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(5)*y(515);
jac(257,515)=jac(257,515)-k(5)*y(257);
jac(515,257)=jac(515,257)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(257);
jac(626,257)=jac(626,257)+k(5)*y(515);
jac(626,515)=jac(626,515)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(257,257)=jac(257,257)-k(5)*y(543);
jac(257,543)=jac(257,543)-k(5)*y(257);
jac(543,257)=jac(543,257)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(257);
jac(565,257)=jac(565,257)+k(5)*y(543);
jac(565,543)=jac(565,543)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(257,257)=jac(257,257)-k(5)*y(554);
jac(257,554)=jac(257,554)-k(5)*y(257);
jac(554,257)=jac(554,257)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(257);
jac(560,257)=jac(560,257)+k(5)*y(554);
jac(560,554)=jac(560,554)+k(5)*y(257);

% rule    : R2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(9,9)=jac(9,9)-k(5)*y(257);
jac(9,257)=jac(9,257)-k(5)*y(9);
jac(257,9)=jac(257,9)-k(5)*y(257);
jac(257,257)=jac(257,257)-k(5)*y(9);
jac(212,9)=jac(212,9)+k(5)*y(257);
jac(212,257)=jac(212,257)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(4,4)=jac(4,4)-k(3)*y(257);
jac(4,257)=jac(4,257)-k(3)*y(4);
jac(257,4)=jac(257,4)-k(3)*y(257);
jac(257,257)=jac(257,257)-k(3)*y(4);
jac(256,4)=jac(256,4)+k(3)*y(257);
jac(256,257)=jac(256,257)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(4,4)=jac(4,4)-k(1)*y(257);
jac(4,257)=jac(4,257)-k(1)*y(4);
jac(257,4)=jac(257,4)-k(1)*y(257);
jac(257,257)=jac(257,257)-k(1)*y(4);
jac(319,4)=jac(319,4)+k(1)*y(257);
jac(319,257)=jac(319,257)+k(1)*y(4);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(258,258)=jac(258,258)-k(34);
jac(571,258)=jac(571,258)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(258,258)=jac(258,258)-k(33);
jac(265,258)=jac(265,258)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(258,258)=jac(258,258)-k(29);
jac(641,258)=jac(641,258)+k(29);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(258,258)=jac(258,258)-k(25);
jac(643,258)=jac(643,258)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

jac(258,258)=jac(258,258)-k(22);
jac(197,258)=jac(197,258)+k(22);
jac(251,258)=jac(251,258)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

jac(258,258)=jac(258,258)-k(22);
jac(2,258)=jac(2,258)+k(22);
jac(622,258)=jac(622,258)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(258,258)=jac(258,258)-k(16);
jac(3,258)=jac(3,258)+k(16);
jac(255,258)=jac(255,258)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(258,258)=jac(258,258)-k(16);
jac(3,258)=jac(3,258)+k(16);
jac(546,258)=jac(546,258)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(258,258)=jac(258,258)-k(8);
jac(227,258)=jac(227,258)+k(8);
jac(44,258)=jac(44,258)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(258,258)=jac(258,258)-k(6);
jac(43,258)=jac(43,258)+k(6);
jac(543,258)=jac(543,258)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(259,259)=jac(259,259)-k(34);
jac(572,259)=jac(572,259)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(259,259)=jac(259,259)-k(33);
jac(266,259)=jac(266,259)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(259,259)=jac(259,259)-k(31);
jac(438,259)=jac(438,259)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(259,259)=jac(259,259)-k(31);
jac(545,259)=jac(545,259)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(259,259)=jac(259,259)-k(29);
jac(577,259)=jac(577,259)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(259,259)=jac(259,259)-k(22);
jac(197,259)=jac(197,259)+k(22);
jac(252,259)=jac(252,259)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(259,259)=jac(259,259)-k(22);
jac(2,259)=jac(2,259)+k(22);
jac(578,259)=jac(578,259)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(259);
jac(3,259)=jac(3,259)-k(15)*y(3);
jac(259,3)=jac(259,3)-k(15)*y(259);
jac(259,259)=jac(259,259)-k(15)*y(3);
jac(255,3)=jac(255,3)+k(15)*y(259);
jac(255,259)=jac(255,259)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(259);
jac(3,259)=jac(3,259)-k(15)*y(3);
jac(259,3)=jac(259,3)-k(15)*y(259);
jac(259,259)=jac(259,259)-k(15)*y(3);
jac(546,3)=jac(546,3)+k(15)*y(259);
jac(546,259)=jac(546,259)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(259,259)=jac(259,259)-k(8);
jac(257,259)=jac(257,259)+k(8);
jac(437,259)=jac(437,259)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(259,259)=jac(259,259)-k(6);
jac(111,259)=jac(111,259)+k(6);
jac(256,259)=jac(256,259)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(260,260)=jac(260,260)-k(34);
jac(634,260)=jac(634,260)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(260,260)=jac(260,260)-k(33);
jac(267,260)=jac(267,260)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(260,260)=jac(260,260)-k(32);
jac(203,260)=jac(203,260)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(260,260)=jac(260,260)-k(31);
jac(291,260)=jac(291,260)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

jac(260,260)=jac(260,260)-k(22);
jac(197,260)=jac(197,260)+k(22);
jac(238,260)=jac(238,260)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(260);
jac(197,260)=jac(197,260)-k(21)*y(197);
jac(260,197)=jac(260,197)-k(21)*y(260);
jac(260,260)=jac(260,260)-k(21)*y(197);
jac(492,197)=jac(492,197)+k(21)*y(260);
jac(492,260)=jac(492,260)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(260);
jac(32,260)=jac(32,260)-k(21)*y(32);
jac(260,32)=jac(260,32)-k(21)*y(260);
jac(260,260)=jac(260,260)-k(21)*y(32);
jac(248,32)=jac(248,32)+k(21)*y(260);
jac(248,260)=jac(248,260)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(260);
jac(2,260)=jac(2,260)-k(21)*y(2);
jac(260,2)=jac(260,2)-k(21)*y(260);
jac(260,260)=jac(260,260)-k(21)*y(2);
jac(255,2)=jac(255,2)+k(21)*y(260);
jac(255,260)=jac(255,260)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(260,260)=jac(260,260)-k(16);
jac(3,260)=jac(3,260)+k(16);
jac(578,260)=jac(578,260)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(260);
jac(3,260)=jac(3,260)-k(15)*y(3);
jac(260,3)=jac(260,3)-k(15)*y(260);
jac(260,260)=jac(260,260)-k(15)*y(3);
jac(622,3)=jac(622,3)+k(15)*y(260);
jac(622,260)=jac(622,260)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(260,260)=jac(260,260)-k(8);
jac(55,260)=jac(55,260)+k(8);
jac(257,260)=jac(257,260)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(260,260)=jac(260,260)-k(6);
jac(54,260)=jac(54,260)+k(6);
jac(256,260)=jac(256,260)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(261,261)=jac(261,261)-k(34);
jac(286,261)=jac(286,261)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(261,261)=jac(261,261)-k(34);
jac(255,261)=jac(255,261)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(261,261)=jac(261,261)-k(33);
jac(268,261)=jac(268,261)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(261,261)=jac(261,261)-k(31);
jac(288,261)=jac(288,261)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(261,261)=jac(261,261)-k(22);
jac(197,261)=jac(197,261)+k(22);
jac(645,261)=jac(645,261)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(261,261)=jac(261,261)-k(22);
jac(182,261)=jac(182,261)+k(22);
jac(260,261)=jac(260,261)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(261,261)=jac(261,261)-k(16);
jac(3,261)=jac(3,261)+k(16);
jac(577,261)=jac(577,261)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(261);
jac(3,261)=jac(3,261)-k(15)*y(3);
jac(261,3)=jac(261,3)-k(15)*y(261);
jac(261,261)=jac(261,261)-k(15)*y(3);
jac(641,3)=jac(641,3)+k(15)*y(261);
jac(641,261)=jac(641,261)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(261,261)=jac(261,261)-k(8);
jac(257,261)=jac(257,261)+k(8);
jac(223,261)=jac(223,261)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(261,261)=jac(261,261)-k(6);
jac(256,261)=jac(256,261)+k(6);
jac(222,261)=jac(222,261)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(262,262)=jac(262,262)-k(34);
jac(277,262)=jac(277,262)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(262,262)=jac(262,262)-k(33);
jac(269,262)=jac(269,262)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(262,262)=jac(262,262)-k(29);
jac(288,262)=jac(288,262)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(262,262)=jac(262,262)-k(22);
jac(197,262)=jac(197,262)+k(22);
jac(49,262)=jac(49,262)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(262,262)=jac(262,262)-k(22);
jac(2,262)=jac(2,262)+k(22);
jac(291,262)=jac(291,262)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(262,262)=jac(262,262)-k(18);
jac(255,262)=jac(255,262)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(262,262)=jac(262,262)-k(16);
jac(3,262)=jac(3,262)+k(16);
jac(438,262)=jac(438,262)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(262);
jac(3,262)=jac(3,262)-k(9)*y(3);
jac(262,3)=jac(262,3)-k(9)*y(262);
jac(262,262)=jac(262,262)-k(9)*y(3);
jac(207,3)=jac(207,3)+k(9)*y(262);
jac(207,262)=jac(207,262)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(262,262)=jac(262,262)-k(8);
jac(192,262)=jac(192,262)+k(8);
jac(44,262)=jac(44,262)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(262,262)=jac(262,262)-k(6);
jac(191,262)=jac(191,262)+k(6);
jac(43,262)=jac(43,262)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(264,264)=jac(264,264)-k(34);
jac(110,264)=jac(110,264)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(264,264)=jac(264,264)-k(31);
jac(176,264)=jac(176,264)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Syk(tSH2, l~Y, a~pY) 

jac(264,264)=jac(264,264)-k(22);
jac(182,264)=jac(182,264)+k(22);
jac(76,264)=jac(76,264)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(264);
jac(3,264)=jac(3,264)-k(15)*y(3);
jac(264,3)=jac(264,3)-k(15)*y(264);
jac(264,264)=jac(264,264)-k(15)*y(3);
jac(515,3)=jac(515,3)+k(15)*y(264);
jac(515,264)=jac(515,264)+k(15)*y(3);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(1,1)=jac(1,1)-k(5)*y(264);
jac(1,264)=jac(1,264)-k(5)*y(1);
jac(264,1)=jac(264,1)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(1);
jac(304,1)=jac(304,1)+k(5)*y(264);
jac(304,264)=jac(304,264)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(7,7)=jac(7,7)-k(5)*y(264);
jac(7,264)=jac(7,264)-k(5)*y(7);
jac(264,7)=jac(264,7)-k(5)*y(264);
jac(264,264)=jac(264,264)-k(5)*y(7);
jac(594,7)=jac(594,7)+k(5)*y(264);
jac(594,264)=jac(594,264)+k(5)*y(7);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(264,264)=jac(264,264)-k(5)*y(335);
jac(264,335)=jac(264,335)-k(5)*y(264);
jac(335,264)=jac(335,264)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(264);
jac(363,264)=jac(363,264)+k(5)*y(335);
jac(363,335)=jac(363,335)+k(5)*y(264);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Lig(l1, l2) 

jac(264,264)=jac(264,264)-k(4);
jac(4,264)=jac(4,264)+k(4);
jac(218,264)=jac(218,264)+k(4);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(265,265)=jac(265,265)-k(34);
jac(630,265)=jac(630,265)+k(34);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(265,265)=jac(265,265)-k(29);
jac(640,265)=jac(640,265)+k(29);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(265,265)=jac(265,265)-k(26);
jac(258,265)=jac(258,265)+k(26);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(265,265)=jac(265,265)-k(25);
jac(494,265)=jac(494,265)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

jac(265,265)=jac(265,265)-k(22);
jac(182,265)=jac(182,265)+k(22);
jac(251,265)=jac(251,265)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

jac(265,265)=jac(265,265)-k(22);
jac(2,265)=jac(2,265)+k(22);
jac(624,265)=jac(624,265)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(265,265)=jac(265,265)-k(16);
jac(3,265)=jac(3,265)+k(16);
jac(263,265)=jac(263,265)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(265,265)=jac(265,265)-k(16);
jac(3,265)=jac(3,265)+k(16);
jac(625,265)=jac(625,265)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(265,265)=jac(265,265)-k(8);
jac(222,265)=jac(222,265)+k(8);
jac(44,265)=jac(44,265)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(265,265)=jac(265,265)-k(6);
jac(43,265)=jac(43,265)+k(6);
jac(515,265)=jac(515,265)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(266,266)=jac(266,266)-k(34);
jac(503,266)=jac(503,266)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(266,266)=jac(266,266)-k(31);
jac(271,266)=jac(271,266)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(266,266)=jac(266,266)-k(31);
jac(510,266)=jac(510,266)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(266,266)=jac(266,266)-k(29);
jac(579,266)=jac(579,266)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(266,266)=jac(266,266)-k(22);
jac(182,266)=jac(182,266)+k(22);
jac(252,266)=jac(252,266)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(266,266)=jac(266,266)-k(22);
jac(2,266)=jac(2,266)+k(22);
jac(497,266)=jac(497,266)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(266);
jac(3,266)=jac(3,266)-k(15)*y(3);
jac(266,3)=jac(266,3)-k(15)*y(266);
jac(266,266)=jac(266,266)-k(15)*y(3);
jac(263,3)=jac(263,3)+k(15)*y(266);
jac(263,266)=jac(263,266)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(266);
jac(3,266)=jac(3,266)-k(15)*y(3);
jac(266,3)=jac(266,3)-k(15)*y(266);
jac(266,266)=jac(266,266)-k(15)*y(3);
jac(625,3)=jac(625,3)+k(15)*y(266);
jac(625,266)=jac(625,266)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(266,266)=jac(266,266)-k(8);
jac(218,266)=jac(218,266)+k(8);
jac(437,266)=jac(437,266)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(266,266)=jac(266,266)-k(6);
jac(111,266)=jac(111,266)+k(6);
jac(264,266)=jac(264,266)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(267,267)=jac(267,267)-k(34);
jac(629,267)=jac(629,267)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(267,267)=jac(267,267)-k(32);
jac(300,267)=jac(300,267)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(267,267)=jac(267,267)-k(31);
jac(272,267)=jac(272,267)+k(31);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(267,267)=jac(267,267)-k(26);
jac(260,267)=jac(260,267)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

jac(267,267)=jac(267,267)-k(22);
jac(182,267)=jac(182,267)+k(22);
jac(238,267)=jac(238,267)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(267);
jac(197,267)=jac(197,267)-k(21)*y(197);
jac(267,197)=jac(267,197)-k(21)*y(267);
jac(267,267)=jac(267,267)-k(21)*y(197);
jac(488,197)=jac(488,197)+k(21)*y(267);
jac(488,267)=jac(488,267)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(267);
jac(32,267)=jac(32,267)-k(21)*y(32);
jac(267,32)=jac(267,32)-k(21)*y(267);
jac(267,267)=jac(267,267)-k(21)*y(32);
jac(493,32)=jac(493,32)+k(21)*y(267);
jac(493,267)=jac(493,267)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(267);
jac(2,267)=jac(2,267)-k(21)*y(2);
jac(267,2)=jac(267,2)-k(21)*y(267);
jac(267,267)=jac(267,267)-k(21)*y(2);
jac(263,2)=jac(263,2)+k(21)*y(267);
jac(263,267)=jac(263,267)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(267,267)=jac(267,267)-k(16);
jac(3,267)=jac(3,267)+k(16);
jac(497,267)=jac(497,267)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(267);
jac(3,267)=jac(3,267)-k(15)*y(3);
jac(267,3)=jac(267,3)-k(15)*y(267);
jac(267,267)=jac(267,267)-k(15)*y(3);
jac(624,3)=jac(624,3)+k(15)*y(267);
jac(624,267)=jac(624,267)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(267,267)=jac(267,267)-k(8);
jac(55,267)=jac(55,267)+k(8);
jac(218,267)=jac(218,267)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(267,267)=jac(267,267)-k(6);
jac(54,267)=jac(54,267)+k(6);
jac(264,267)=jac(264,267)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(268,268)=jac(268,268)-k(34);
jac(281,268)=jac(281,268)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(268,268)=jac(268,268)-k(34);
jac(263,268)=jac(263,268)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(268,268)=jac(268,268)-k(31);
jac(273,268)=jac(273,268)+k(31);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(268,268)=jac(268,268)-k(26);
jac(261,268)=jac(261,268)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(268,268)=jac(268,268)-k(22);
jac(182,268)=jac(182,268)+k(22);
jac(645,268)=jac(645,268)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(268,268)=jac(268,268)-k(22);
jac(182,268)=jac(182,268)+k(22);
jac(267,268)=jac(267,268)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(268,268)=jac(268,268)-k(16);
jac(3,268)=jac(3,268)+k(16);
jac(579,268)=jac(579,268)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(268);
jac(3,268)=jac(3,268)-k(15)*y(3);
jac(268,3)=jac(268,3)-k(15)*y(268);
jac(268,268)=jac(268,268)-k(15)*y(3);
jac(640,3)=jac(640,3)+k(15)*y(268);
jac(640,268)=jac(640,268)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(268,268)=jac(268,268)-k(8);
jac(218,268)=jac(218,268)+k(8);
jac(223,268)=jac(223,268)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(268,268)=jac(268,268)-k(6);
jac(264,268)=jac(264,268)+k(6);
jac(222,268)=jac(222,268)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(270,270)=jac(270,270)-k(34);
jac(51,270)=jac(51,270)+k(34);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(270,270)=jac(270,270)-k(29);
jac(649,270)=jac(649,270)+k(29);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(270,270)=jac(270,270)-k(26);
jac(207,270)=jac(207,270)+k(26);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(270,270)=jac(270,270)-k(23);
jac(478,270)=jac(478,270)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(270,270)=jac(270,270)-k(22);
jac(182,270)=jac(182,270)+k(22);
jac(46,270)=jac(46,270)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

jac(270,270)=jac(270,270)-k(22);
jac(2,270)=jac(2,270)+k(22);
jac(422,270)=jac(422,270)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(270,270)=jac(270,270)-k(16);
jac(3,270)=jac(3,270)+k(16);
jac(392,270)=jac(392,270)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(270,270)=jac(270,270)-k(10);
jac(3,270)=jac(3,270)+k(10);
jac(269,270)=jac(269,270)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(270,270)=jac(270,270)-k(8);
jac(184,270)=jac(184,270)+k(8);
jac(44,270)=jac(44,270)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(270,270)=jac(270,270)-k(6);
jac(43,270)=jac(43,270)+k(6);
jac(391,270)=jac(391,270)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(271,271)=jac(271,271)-k(34);
jac(275,271)=jac(275,271)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(271,271)=jac(271,271)-k(31);
jac(179,271)=jac(179,271)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(271,271)=jac(271,271)-k(29);
jac(580,271)=jac(580,271)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(271,271)=jac(271,271)-k(22);
jac(182,271)=jac(182,271)+k(22);
jac(616,271)=jac(616,271)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(271,271)=jac(271,271)-k(22);
jac(2,271)=jac(2,271)+k(22);
jac(421,271)=jac(421,271)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(271);
jac(3,271)=jac(3,271)-k(15)*y(3);
jac(271,3)=jac(271,3)-k(15)*y(271);
jac(271,271)=jac(271,271)-k(15)*y(3);
jac(269,3)=jac(269,3)+k(15)*y(271);
jac(269,271)=jac(269,271)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(271);
jac(3,271)=jac(3,271)-k(9)*y(3);
jac(271,3)=jac(271,3)-k(9)*y(271);
jac(271,271)=jac(271,271)-k(9)*y(3);
jac(392,3)=jac(392,3)+k(9)*y(271);
jac(392,271)=jac(392,271)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(271,271)=jac(271,271)-k(8);
jac(177,271)=jac(177,271)+k(8);
jac(437,271)=jac(437,271)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(271,271)=jac(271,271)-k(6);
jac(111,271)=jac(111,271)+k(6);
jac(176,271)=jac(176,271)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(272,272)=jac(272,272)-k(34);
jac(276,272)=jac(276,272)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(272,272)=jac(272,272)-k(32);
jac(297,272)=jac(297,272)+k(32);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(272,272)=jac(272,272)-k(26);
jac(291,272)=jac(291,272)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

jac(272,272)=jac(272,272)-k(22);
jac(182,272)=jac(182,272)+k(22);
jac(56,272)=jac(56,272)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(272);
jac(197,272)=jac(197,272)-k(21)*y(197);
jac(272,197)=jac(272,197)-k(21)*y(272);
jac(272,272)=jac(272,272)-k(21)*y(197);
jac(486,197)=jac(486,197)+k(21)*y(272);
jac(486,272)=jac(486,272)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(272);
jac(32,272)=jac(32,272)-k(21)*y(32);
jac(272,32)=jac(272,32)-k(21)*y(272);
jac(272,272)=jac(272,272)-k(21)*y(32);
jac(479,32)=jac(479,32)+k(21)*y(272);
jac(479,272)=jac(479,272)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(272);
jac(2,272)=jac(2,272)-k(21)*y(2);
jac(272,2)=jac(272,2)-k(21)*y(272);
jac(272,272)=jac(272,272)-k(21)*y(2);
jac(269,2)=jac(269,2)+k(21)*y(272);
jac(269,272)=jac(269,272)+k(21)*y(2);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(272,272)=jac(272,272)-k(18);
jac(267,272)=jac(267,272)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(272,272)=jac(272,272)-k(16);
jac(3,272)=jac(3,272)+k(16);
jac(421,272)=jac(421,272)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(272);
jac(3,272)=jac(3,272)-k(9)*y(3);
jac(272,3)=jac(272,3)-k(9)*y(272);
jac(272,272)=jac(272,272)-k(9)*y(3);
jac(422,3)=jac(422,3)+k(9)*y(272);
jac(422,272)=jac(422,272)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(272,272)=jac(272,272)-k(8);
jac(55,272)=jac(55,272)+k(8);
jac(177,272)=jac(177,272)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(272,272)=jac(272,272)-k(6);
jac(54,272)=jac(54,272)+k(6);
jac(176,272)=jac(176,272)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(273,273)=jac(273,273)-k(34);
jac(278,273)=jac(278,273)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(273,273)=jac(273,273)-k(34);
jac(269,273)=jac(269,273)+k(34);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(273,273)=jac(273,273)-k(26);
jac(288,273)=jac(288,273)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(273,273)=jac(273,273)-k(22);
jac(182,273)=jac(182,273)+k(22);
jac(282,273)=jac(282,273)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(273,273)=jac(273,273)-k(22);
jac(182,273)=jac(182,273)+k(22);
jac(272,273)=jac(272,273)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(273,273)=jac(273,273)-k(18);
jac(268,273)=jac(268,273)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(273,273)=jac(273,273)-k(16);
jac(3,273)=jac(3,273)+k(16);
jac(580,273)=jac(580,273)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(273);
jac(3,273)=jac(3,273)-k(9)*y(3);
jac(273,3)=jac(273,3)-k(9)*y(273);
jac(273,273)=jac(273,273)-k(9)*y(3);
jac(649,3)=jac(649,3)+k(9)*y(273);
jac(649,273)=jac(649,273)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(273,273)=jac(273,273)-k(8);
jac(177,273)=jac(177,273)+k(8);
jac(223,273)=jac(223,273)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(273,273)=jac(273,273)-k(6);
jac(176,273)=jac(176,273)+k(6);
jac(222,273)=jac(222,273)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(275,275)=jac(275,275)-k(31);
jac(172,275)=jac(172,275)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(275,275)=jac(275,275)-k(28);
jac(271,275)=jac(271,275)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(275,275)=jac(275,275)-k(27);
jac(280,275)=jac(280,275)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(275,275)=jac(275,275)-k(22);
jac(2,275)=jac(2,275)+k(22);
jac(616,275)=jac(616,275)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(275,275)=jac(275,275)-k(22);
jac(2,275)=jac(2,275)+k(22);
jac(425,275)=jac(425,275)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(275);
jac(3,275)=jac(3,275)-k(15)*y(3);
jac(275,3)=jac(275,3)-k(15)*y(275);
jac(275,275)=jac(275,275)-k(15)*y(3);
jac(274,3)=jac(274,3)+k(15)*y(275);
jac(274,275)=jac(274,275)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(275);
jac(3,275)=jac(3,275)-k(9)*y(3);
jac(275,3)=jac(275,3)-k(9)*y(275);
jac(275,275)=jac(275,275)-k(9)*y(3);
jac(387,3)=jac(387,3)+k(9)*y(275);
jac(387,275)=jac(387,275)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(275,275)=jac(275,275)-k(8);
jac(21,275)=jac(21,275)+k(8);
jac(437,275)=jac(437,275)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(275,275)=jac(275,275)-k(6);
jac(111,275)=jac(111,275)+k(6);
jac(89,275)=jac(89,275)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(276,276)=jac(276,276)-k(32);
jac(88,276)=jac(88,276)+k(32);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(276,276)=jac(276,276)-k(26);
jac(287,276)=jac(287,276)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

jac(276,276)=jac(276,276)-k(22);
jac(2,276)=jac(2,276)+k(22);
jac(56,276)=jac(56,276)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(276);
jac(197,276)=jac(197,276)-k(21)*y(197);
jac(276,197)=jac(276,197)-k(21)*y(276);
jac(276,276)=jac(276,276)-k(21)*y(197);
jac(484,197)=jac(484,197)+k(21)*y(276);
jac(484,276)=jac(484,276)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(276);
jac(32,276)=jac(32,276)-k(21)*y(32);
jac(276,32)=jac(276,32)-k(21)*y(276);
jac(276,276)=jac(276,276)-k(21)*y(32);
jac(481,32)=jac(481,32)+k(21)*y(276);
jac(481,276)=jac(481,276)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(276);
jac(2,276)=jac(2,276)-k(21)*y(2);
jac(276,2)=jac(276,2)-k(21)*y(276);
jac(276,276)=jac(276,276)-k(21)*y(2);
jac(274,2)=jac(274,2)+k(21)*y(276);
jac(274,276)=jac(274,276)+k(21)*y(2);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(276,276)=jac(276,276)-k(18);
jac(629,276)=jac(629,276)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(276,276)=jac(276,276)-k(16);
jac(3,276)=jac(3,276)+k(16);
jac(425,276)=jac(425,276)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(276);
jac(3,276)=jac(3,276)-k(9)*y(3);
jac(276,3)=jac(276,3)-k(9)*y(276);
jac(276,276)=jac(276,276)-k(9)*y(3);
jac(58,3)=jac(58,3)+k(9)*y(276);
jac(58,276)=jac(58,276)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(276,276)=jac(276,276)-k(8);
jac(55,276)=jac(55,276)+k(8);
jac(21,276)=jac(21,276)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(276,276)=jac(276,276)-k(6);
jac(54,276)=jac(54,276)+k(6);
jac(89,276)=jac(89,276)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(277,277)=jac(277,277)-k(33);
jac(274,277)=jac(274,277)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(277,277)=jac(277,277)-k(28);
jac(262,277)=jac(262,277)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(277,277)=jac(277,277)-k(27);
jac(283,277)=jac(283,277)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(277,277)=jac(277,277)-k(22);
jac(32,277)=jac(32,277)+k(22);
jac(49,277)=jac(49,277)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(277,277)=jac(277,277)-k(22);
jac(2,277)=jac(2,277)+k(22);
jac(287,277)=jac(287,277)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(277,277)=jac(277,277)-k(18);
jac(254,277)=jac(254,277)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(277,277)=jac(277,277)-k(16);
jac(3,277)=jac(3,277)+k(16);
jac(442,277)=jac(442,277)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(277);
jac(3,277)=jac(3,277)-k(9)*y(3);
jac(277,3)=jac(277,3)-k(9)*y(277);
jac(277,277)=jac(277,277)-k(9)*y(3);
jac(52,3)=jac(52,3)+k(9)*y(277);
jac(52,277)=jac(52,277)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(277,277)=jac(277,277)-k(8);
jac(27,277)=jac(27,277)+k(8);
jac(44,277)=jac(44,277)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(277,277)=jac(277,277)-k(6);
jac(86,277)=jac(86,277)+k(6);
jac(43,277)=jac(43,277)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(279,279)=jac(279,279)-k(34);
jac(51,279)=jac(51,279)+k(34);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(279,279)=jac(279,279)-k(30);
jac(649,279)=jac(649,279)+k(30);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(279,279)=jac(279,279)-k(26);
jac(284,279)=jac(284,279)+k(26);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(279,279)=jac(279,279)-k(23);
jac(428,279)=jac(428,279)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(279,279)=jac(279,279)-k(22);
jac(2,279)=jac(2,279)+k(22);
jac(225,279)=jac(225,279)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(279,279)=jac(279,279)-k(22);
jac(182,279)=jac(182,279)+k(22);
jac(58,279)=jac(58,279)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(279,279)=jac(279,279)-k(16);
jac(3,279)=jac(3,279)+k(16);
jac(429,279)=jac(429,279)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(279,279)=jac(279,279)-k(10);
jac(3,279)=jac(3,279)+k(10);
jac(278,279)=jac(278,279)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(279,279)=jac(279,279)-k(8);
jac(140,279)=jac(140,279)+k(8);
jac(223,279)=jac(223,279)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(279,279)=jac(279,279)-k(6);
jac(222,279)=jac(222,279)+k(6);
jac(380,279)=jac(380,279)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(280,280)=jac(280,280)-k(34);
jac(275,280)=jac(275,280)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(280,280)=jac(280,280)-k(31);
jac(377,280)=jac(377,280)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(280,280)=jac(280,280)-k(30);
jac(580,280)=jac(580,280)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(280,280)=jac(280,280)-k(22);
jac(2,280)=jac(2,280)+k(22);
jac(431,280)=jac(431,280)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(280,280)=jac(280,280)-k(22);
jac(182,280)=jac(182,280)+k(22);
jac(425,280)=jac(425,280)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(280);
jac(3,280)=jac(3,280)-k(15)*y(3);
jac(280,3)=jac(280,3)-k(15)*y(280);
jac(280,280)=jac(280,280)-k(15)*y(3);
jac(278,3)=jac(278,3)+k(15)*y(280);
jac(278,280)=jac(278,280)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(280);
jac(3,280)=jac(3,280)-k(9)*y(3);
jac(280,3)=jac(280,3)-k(9)*y(280);
jac(280,280)=jac(280,280)-k(9)*y(3);
jac(429,3)=jac(429,3)+k(9)*y(280);
jac(429,280)=jac(429,280)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(280,280)=jac(280,280)-k(8);
jac(21,280)=jac(21,280)+k(8);
jac(219,280)=jac(219,280)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(280,280)=jac(280,280)-k(6);
jac(218,280)=jac(218,280)+k(6);
jac(89,280)=jac(89,280)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(281,281)=jac(281,281)-k(34);
jac(253,281)=jac(253,281)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(281,281)=jac(281,281)-k(31);
jac(278,281)=jac(278,281)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(281,281)=jac(281,281)-k(30);
jac(268,281)=jac(268,281)+k(30);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(281,281)=jac(281,281)-k(26);
jac(286,281)=jac(286,281)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(281,281)=jac(281,281)-k(22);
jac(2,281)=jac(2,281)+k(22);
jac(645,281)=jac(645,281)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(281,281)=jac(281,281)-k(22);
jac(182,281)=jac(182,281)+k(22);
jac(629,281)=jac(629,281)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(281,281)=jac(281,281)-k(16);
jac(3,281)=jac(3,281)+k(16);
jac(505,281)=jac(505,281)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(281);
jac(3,281)=jac(3,281)-k(15)*y(3);
jac(281,3)=jac(281,3)-k(15)*y(281);
jac(281,281)=jac(281,281)-k(15)*y(3);
jac(637,3)=jac(637,3)+k(15)*y(281);
jac(637,281)=jac(637,281)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(281,281)=jac(281,281)-k(8);
jac(111,281)=jac(111,281)+k(8);
jac(223,281)=jac(223,281)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(281,281)=jac(281,281)-k(6);
jac(110,281)=jac(110,281)+k(6);
jac(222,281)=jac(222,281)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(282,282)=jac(282,282)-k(34);
jac(49,282)=jac(49,282)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(282,282)=jac(282,282)-k(32);
jac(224,282)=jac(224,282)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

jac(282,282)=jac(282,282)-k(22);
jac(182,282)=jac(182,282)+k(22);
jac(56,282)=jac(56,282)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(282);
jac(197,282)=jac(197,282)-k(21)*y(197);
jac(282,197)=jac(282,197)-k(21)*y(282);
jac(282,282)=jac(282,282)-k(21)*y(197);
jac(288,197)=jac(288,197)+k(21)*y(282);
jac(288,282)=jac(288,282)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(282);
jac(32,282)=jac(32,282)-k(21)*y(32);
jac(282,32)=jac(282,32)-k(21)*y(282);
jac(282,282)=jac(282,282)-k(21)*y(32);
jac(283,32)=jac(283,32)+k(21)*y(282);
jac(283,282)=jac(283,282)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(282);
jac(2,282)=jac(2,282)-k(21)*y(2);
jac(282,2)=jac(282,2)-k(21)*y(282);
jac(282,282)=jac(282,282)-k(21)*y(2);
jac(278,2)=jac(278,2)+k(21)*y(282);
jac(278,282)=jac(278,282)+k(21)*y(2);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(282,282)=jac(282,282)-k(18);
jac(645,282)=jac(645,282)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(282,282)=jac(282,282)-k(16);
jac(3,282)=jac(3,282)+k(16);
jac(431,282)=jac(431,282)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(282);
jac(3,282)=jac(3,282)-k(9)*y(3);
jac(282,3)=jac(282,3)-k(9)*y(282);
jac(282,282)=jac(282,282)-k(9)*y(3);
jac(225,3)=jac(225,3)+k(9)*y(282);
jac(225,282)=jac(225,282)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

jac(282,282)=jac(282,282)-k(8);
jac(15,282)=jac(15,282)+k(8);
jac(223,282)=jac(223,282)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(282,282)=jac(282,282)-k(6);
jac(98,282)=jac(98,282)+k(6);
jac(222,282)=jac(222,282)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(284,284)=jac(284,284)-k(34);
jac(52,284)=jac(52,284)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(284,284)=jac(284,284)-k(33);
jac(279,284)=jac(279,284)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(284,284)=jac(284,284)-k(30);
jac(289,284)=jac(289,284)+k(30);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(284,284)=jac(284,284)-k(23);
jac(473,284)=jac(473,284)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(284,284)=jac(284,284)-k(22);
jac(32,284)=jac(32,284)+k(22);
jac(225,284)=jac(225,284)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(284,284)=jac(284,284)-k(22);
jac(182,284)=jac(182,284)+k(22);
jac(59,284)=jac(59,284)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(284,284)=jac(284,284)-k(16);
jac(3,284)=jac(3,284)+k(16);
jac(432,284)=jac(432,284)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(284,284)=jac(284,284)-k(10);
jac(3,284)=jac(3,284)+k(10);
jac(283,284)=jac(283,284)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(284,284)=jac(284,284)-k(8);
jac(131,284)=jac(131,284)+k(8);
jac(223,284)=jac(223,284)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(284,284)=jac(284,284)-k(6);
jac(222,284)=jac(222,284)+k(6);
jac(441,284)=jac(441,284)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(285,285)=jac(285,285)-k(34);
jac(442,285)=jac(442,285)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(285,285)=jac(285,285)-k(33);
jac(280,285)=jac(280,285)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(285,285)=jac(285,285)-k(31);
jac(449,285)=jac(449,285)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(285,285)=jac(285,285)-k(30);
jac(290,285)=jac(290,285)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(285,285)=jac(285,285)-k(22);
jac(32,285)=jac(32,285)+k(22);
jac(431,285)=jac(431,285)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(285,285)=jac(285,285)-k(22);
jac(182,285)=jac(182,285)+k(22);
jac(615,285)=jac(615,285)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(285);
jac(3,285)=jac(3,285)-k(15)*y(3);
jac(285,3)=jac(285,3)-k(15)*y(285);
jac(285,285)=jac(285,285)-k(15)*y(3);
jac(283,3)=jac(283,3)+k(15)*y(285);
jac(283,285)=jac(283,285)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(285);
jac(3,285)=jac(3,285)-k(9)*y(3);
jac(285,3)=jac(285,3)-k(9)*y(285);
jac(285,285)=jac(285,285)-k(9)*y(3);
jac(432,3)=jac(432,3)+k(9)*y(285);
jac(432,285)=jac(432,285)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(285,285)=jac(285,285)-k(8);
jac(27,285)=jac(27,285)+k(8);
jac(219,285)=jac(219,285)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(285,285)=jac(285,285)-k(6);
jac(218,285)=jac(218,285)+k(6);
jac(86,285)=jac(86,285)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(286,286)=jac(286,286)-k(34);
jac(254,286)=jac(254,286)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(286,286)=jac(286,286)-k(33);
jac(281,286)=jac(281,286)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(286,286)=jac(286,286)-k(31);
jac(283,286)=jac(283,286)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(286,286)=jac(286,286)-k(30);
jac(261,286)=jac(261,286)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(286,286)=jac(286,286)-k(22);
jac(32,286)=jac(32,286)+k(22);
jac(645,286)=jac(645,286)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(286,286)=jac(286,286)-k(22);
jac(182,286)=jac(182,286)+k(22);
jac(634,286)=jac(634,286)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(286,286)=jac(286,286)-k(16);
jac(3,286)=jac(3,286)+k(16);
jac(574,286)=jac(574,286)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(286);
jac(3,286)=jac(3,286)-k(15)*y(3);
jac(286,3)=jac(286,3)-k(15)*y(286);
jac(286,286)=jac(286,286)-k(15)*y(3);
jac(639,3)=jac(639,3)+k(15)*y(286);
jac(639,286)=jac(639,286)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(286,286)=jac(286,286)-k(8);
jac(33,286)=jac(33,286)+k(8);
jac(223,286)=jac(223,286)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(286,286)=jac(286,286)-k(6);
jac(82,286)=jac(82,286)+k(6);
jac(222,286)=jac(222,286)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(287,287)=jac(287,287)-k(33);
jac(276,287)=jac(276,287)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(287,287)=jac(287,287)-k(32);
jac(85,287)=jac(85,287)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

jac(287,287)=jac(287,287)-k(22);
jac(32,287)=jac(32,287)+k(22);
jac(56,287)=jac(56,287)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(287);
jac(197,287)=jac(197,287)-k(21)*y(197);
jac(287,197)=jac(287,197)-k(21)*y(287);
jac(287,287)=jac(287,287)-k(21)*y(197);
jac(474,197)=jac(474,197)+k(21)*y(287);
jac(474,287)=jac(474,287)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(287);
jac(32,287)=jac(32,287)-k(21)*y(32);
jac(287,32)=jac(287,32)-k(21)*y(287);
jac(287,287)=jac(287,287)-k(21)*y(32);
jac(483,32)=jac(483,32)+k(21)*y(287);
jac(483,287)=jac(483,287)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(287);
jac(2,287)=jac(2,287)-k(21)*y(2);
jac(287,2)=jac(287,2)-k(21)*y(287);
jac(287,287)=jac(287,287)-k(21)*y(2);
jac(277,2)=jac(277,2)+k(21)*y(287);
jac(277,287)=jac(277,287)+k(21)*y(2);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(287,287)=jac(287,287)-k(18);
jac(634,287)=jac(634,287)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(287,287)=jac(287,287)-k(16);
jac(3,287)=jac(3,287)+k(16);
jac(615,287)=jac(615,287)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(287);
jac(3,287)=jac(3,287)-k(9)*y(3);
jac(287,3)=jac(287,3)-k(9)*y(287);
jac(287,287)=jac(287,287)-k(9)*y(3);
jac(59,3)=jac(59,3)+k(9)*y(287);
jac(59,287)=jac(59,287)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(287,287)=jac(287,287)-k(8);
jac(55,287)=jac(55,287)+k(8);
jac(27,287)=jac(27,287)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(287,287)=jac(287,287)-k(6);
jac(54,287)=jac(54,287)+k(6);
jac(86,287)=jac(86,287)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(289,289)=jac(289,289)-k(34);
jac(284,289)=jac(284,289)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(289,289)=jac(289,289)-k(34);
jac(207,289)=jac(207,289)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(289,289)=jac(289,289)-k(33);
jac(649,289)=jac(649,289)+k(33);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(289,289)=jac(289,289)-k(23);
jac(475,289)=jac(475,289)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(289,289)=jac(289,289)-k(22);
jac(197,289)=jac(197,289)+k(22);
jac(225,289)=jac(225,289)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(289,289)=jac(289,289)-k(22);
jac(182,289)=jac(182,289)+k(22);
jac(206,289)=jac(206,289)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(289,289)=jac(289,289)-k(16);
jac(3,289)=jac(3,289)+k(16);
jac(433,289)=jac(433,289)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(289,289)=jac(289,289)-k(10);
jac(3,289)=jac(3,289)+k(10);
jac(288,289)=jac(288,289)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(289,289)=jac(289,289)-k(8);
jac(335,289)=jac(335,289)+k(8);
jac(223,289)=jac(223,289)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(289,289)=jac(289,289)-k(6);
jac(222,289)=jac(222,289)+k(6);
jac(407,289)=jac(407,289)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(290,290)=jac(290,290)-k(34);
jac(285,290)=jac(285,290)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(290,290)=jac(290,290)-k(34);
jac(438,290)=jac(438,290)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(290,290)=jac(290,290)-k(33);
jac(580,290)=jac(580,290)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(290,290)=jac(290,290)-k(31);
jac(194,290)=jac(194,290)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(290,290)=jac(290,290)-k(22);
jac(197,290)=jac(197,290)+k(22);
jac(431,290)=jac(431,290)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(290,290)=jac(290,290)-k(22);
jac(182,290)=jac(182,290)+k(22);
jac(292,290)=jac(292,290)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(290);
jac(3,290)=jac(3,290)-k(15)*y(3);
jac(290,3)=jac(290,3)-k(15)*y(290);
jac(290,290)=jac(290,290)-k(15)*y(3);
jac(288,3)=jac(288,3)+k(15)*y(290);
jac(288,290)=jac(288,290)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(290);
jac(3,290)=jac(3,290)-k(9)*y(3);
jac(290,3)=jac(290,3)-k(9)*y(290);
jac(290,290)=jac(290,290)-k(9)*y(3);
jac(433,3)=jac(433,3)+k(9)*y(290);
jac(433,290)=jac(433,290)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(290,290)=jac(290,290)-k(8);
jac(192,290)=jac(192,290)+k(8);
jac(219,290)=jac(219,290)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(290,290)=jac(290,290)-k(6);
jac(218,290)=jac(218,290)+k(6);
jac(191,290)=jac(191,290)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(292,292)=jac(292,292)-k(34);
jac(615,292)=jac(615,292)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(292,292)=jac(292,292)-k(33);
jac(421,292)=jac(421,292)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(292,292)=jac(292,292)-k(32);
jac(296,292)=jac(296,292)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(292,292)=jac(292,292)-k(31);
jac(409,292)=jac(409,292)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(292,292)=jac(292,292)-k(22);
jac(197,292)=jac(197,292)+k(22);
jac(614,292)=jac(614,292)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(292);
jac(197,292)=jac(197,292)-k(21)*y(197);
jac(292,197)=jac(292,197)-k(21)*y(292);
jac(292,292)=jac(292,292)-k(21)*y(197);
jac(619,197)=jac(619,197)+k(21)*y(292);
jac(619,292)=jac(619,292)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(292);
jac(32,292)=jac(32,292)-k(21)*y(32);
jac(292,32)=jac(292,32)-k(21)*y(292);
jac(292,292)=jac(292,292)-k(21)*y(32);
jac(620,32)=jac(620,32)+k(21)*y(292);
jac(620,292)=jac(620,292)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(292);
jac(2,292)=jac(2,292)-k(21)*y(2);
jac(292,2)=jac(292,2)-k(21)*y(292);
jac(292,292)=jac(292,292)-k(21)*y(2);
jac(438,2)=jac(438,2)+k(21)*y(292);
jac(438,292)=jac(438,292)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(292);
jac(3,292)=jac(3,292)-k(15)*y(3);
jac(292,3)=jac(292,3)-k(15)*y(292);
jac(292,292)=jac(292,292)-k(15)*y(3);
jac(291,3)=jac(291,3)+k(15)*y(292);
jac(291,292)=jac(291,292)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(292);
jac(3,292)=jac(3,292)-k(9)*y(3);
jac(292,3)=jac(292,3)-k(9)*y(292);
jac(292,292)=jac(292,292)-k(9)*y(3);
jac(410,3)=jac(410,3)+k(9)*y(292);
jac(410,292)=jac(410,292)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(292,292)=jac(292,292)-k(8);
jac(315,292)=jac(315,292)+k(8);
jac(192,292)=jac(192,292)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

jac(292,292)=jac(292,292)-k(6);
jac(77,292)=jac(77,292)+k(6);
jac(191,292)=jac(191,292)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(293,293)=jac(293,293)-k(34);
jac(483,293)=jac(483,293)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(293,293)=jac(293,293)-k(33);
jac(262,293)=jac(262,293)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(293,293)=jac(293,293)-k(33);
jac(479,293)=jac(479,293)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(293,293)=jac(293,293)-k(29);
jac(294,293)=jac(294,293)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(293,293)=jac(293,293)-k(22);
jac(32,293)=jac(32,293)+k(22);
jac(291,293)=jac(291,293)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(293,293)=jac(293,293)-k(22);
jac(197,293)=jac(197,293)+k(22);
jac(249,293)=jac(249,293)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(293,293)=jac(293,293)-k(18);
jac(248,293)=jac(248,293)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(293,293)=jac(293,293)-k(16);
jac(3,293)=jac(3,293)+k(16);
jac(620,293)=jac(620,293)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(293);
jac(3,293)=jac(3,293)-k(9)*y(3);
jac(293,3)=jac(293,3)-k(9)*y(293);
jac(293,293)=jac(293,293)-k(9)*y(3);
jac(477,3)=jac(477,3)+k(9)*y(293);
jac(477,293)=jac(477,293)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(293,293)=jac(293,293)-k(8);
jac(192,293)=jac(192,293)+k(8);
jac(38,293)=jac(38,293)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(293,293)=jac(293,293)-k(6);
jac(191,293)=jac(191,293)+k(6);
jac(37,293)=jac(37,293)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(294,294)=jac(294,294)-k(34);
jac(293,294)=jac(293,294)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(294,294)=jac(294,294)-k(34);
jac(474,294)=jac(474,294)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(294,294)=jac(294,294)-k(33);
jac(288,294)=jac(288,294)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(294,294)=jac(294,294)-k(33);
jac(486,294)=jac(486,294)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(294,294)=jac(294,294)-k(22);
jac(197,294)=jac(197,294)+k(22);
jac(291,294)=jac(291,294)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(294,294)=jac(294,294)-k(22);
jac(197,294)=jac(197,294)+k(22);
jac(232,294)=jac(232,294)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(294,294)=jac(294,294)-k(18);
jac(492,294)=jac(492,294)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(294,294)=jac(294,294)-k(16);
jac(3,294)=jac(3,294)+k(16);
jac(619,294)=jac(619,294)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(294);
jac(3,294)=jac(3,294)-k(9)*y(3);
jac(294,3)=jac(294,3)-k(9)*y(294);
jac(294,294)=jac(294,294)-k(9)*y(3);
jac(475,3)=jac(475,3)+k(9)*y(294);
jac(475,294)=jac(475,294)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(294,294)=jac(294,294)-k(8);
jac(192,294)=jac(192,294)+k(8);
jac(228,294)=jac(228,294)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(294,294)=jac(294,294)-k(6);
jac(191,294)=jac(191,294)+k(6);
jac(227,294)=jac(227,294)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(296,296)=jac(296,296)-k(34);
jac(87,296)=jac(87,296)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(296,296)=jac(296,296)-k(33);
jac(299,296)=jac(299,296)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(296,296)=jac(296,296)-k(31);
jac(202,296)=jac(202,296)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(296,296)=jac(296,296)-k(22);
jac(197,296)=jac(197,296)+k(22);
jac(93,296)=jac(93,296)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(296);
jac(3,296)=jac(3,296)-k(15)*y(3);
jac(296,3)=jac(296,3)-k(15)*y(296);
jac(296,296)=jac(296,296)-k(15)*y(3);
jac(295,3)=jac(295,3)+k(15)*y(296);
jac(295,296)=jac(295,296)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(296);
jac(3,296)=jac(3,296)-k(9)*y(3);
jac(296,3)=jac(296,3)-k(9)*y(296);
jac(296,296)=jac(296,296)-k(9)*y(3);
jac(415,3)=jac(415,3)+k(9)*y(296);
jac(415,296)=jac(415,296)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(296,296)=jac(296,296)-k(8);
jac(91,296)=jac(91,296)+k(8);
jac(192,296)=jac(192,296)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

jac(296,296)=jac(296,296)-k(6);
jac(72,296)=jac(72,296)+k(6);
jac(191,296)=jac(191,296)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(298,298)=jac(298,298)-k(34);
jac(65,298)=jac(65,298)+k(34);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(298,298)=jac(298,298)-k(26);
jac(204,298)=jac(204,298)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(298,298)=jac(298,298)-k(22);
jac(182,298)=jac(182,298)+k(22);
jac(60,298)=jac(60,298)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(298,298)=jac(298,298)-k(16);
jac(3,298)=jac(3,298)+k(16);
jac(419,298)=jac(419,298)+k(16);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(298,298)=jac(298,298)-k(13);
jac(422,298)=jac(422,298)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(298,298)=jac(298,298)-k(10);
jac(3,298)=jac(3,298)+k(10);
jac(297,298)=jac(297,298)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(298,298)=jac(298,298)-k(8);
jac(62,298)=jac(62,298)+k(8);
jac(184,298)=jac(184,298)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(298,298)=jac(298,298)-k(6);
jac(61,298)=jac(61,298)+k(6);
jac(391,298)=jac(391,298)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(299,299)=jac(299,299)-k(34);
jac(90,299)=jac(90,299)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(299,299)=jac(299,299)-k(31);
jac(418,299)=jac(418,299)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(299,299)=jac(299,299)-k(22);
jac(182,299)=jac(182,299)+k(22);
jac(93,299)=jac(93,299)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(299);
jac(3,299)=jac(3,299)-k(15)*y(3);
jac(299,3)=jac(299,3)-k(15)*y(299);
jac(299,299)=jac(299,299)-k(15)*y(3);
jac(297,3)=jac(297,3)+k(15)*y(299);
jac(297,299)=jac(297,299)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(299);
jac(3,299)=jac(3,299)-k(9)*y(3);
jac(299,3)=jac(299,3)-k(9)*y(299);
jac(299,299)=jac(299,299)-k(9)*y(3);
jac(419,3)=jac(419,3)+k(9)*y(299);
jac(419,299)=jac(419,299)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(299,299)=jac(299,299)-k(8);
jac(91,299)=jac(91,299)+k(8);
jac(177,299)=jac(177,299)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

jac(299,299)=jac(299,299)-k(6);
jac(72,299)=jac(72,299)+k(6);
jac(176,299)=jac(176,299)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(301,301)=jac(301,301)-k(34);
jac(307,301)=jac(307,301)+k(34);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(301,301)=jac(301,301)-k(26);
jac(647,301)=jac(647,301)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(301,301)=jac(301,301)-k(22);
jac(182,301)=jac(182,301)+k(22);
jac(78,301)=jac(78,301)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(301,301)=jac(301,301)-k(19);
jac(624,301)=jac(624,301)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(301,301)=jac(301,301)-k(16);
jac(3,301)=jac(3,301)+k(16);
jac(300,301)=jac(300,301)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(301,301)=jac(301,301)-k(16);
jac(3,301)=jac(3,301)+k(16);
jac(303,301)=jac(303,301)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(301,301)=jac(301,301)-k(8);
jac(62,301)=jac(62,301)+k(8);
jac(222,301)=jac(222,301)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(301,301)=jac(301,301)-k(6);
jac(61,301)=jac(61,301)+k(6);
jac(515,301)=jac(515,301)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(303,303)=jac(303,303)-k(34);
jac(306,303)=jac(306,303)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(303,303)=jac(303,303)-k(31);
jac(591,303)=jac(591,303)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

jac(303,303)=jac(303,303)-k(22);
jac(182,303)=jac(182,303)+k(22);
jac(309,303)=jac(309,303)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(303,303)=jac(303,303)-k(19);
jac(623,303)=jac(623,303)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(303,303)=jac(303,303)-k(16);
jac(3,303)=jac(3,303)+k(16);
jac(302,303)=jac(302,303)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(303);
jac(3,303)=jac(3,303)-k(15)*y(3);
jac(303,3)=jac(303,3)-k(15)*y(303);
jac(303,303)=jac(303,303)-k(15)*y(3);
jac(301,3)=jac(301,3)+k(15)*y(303);
jac(301,303)=jac(301,303)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(303,303)=jac(303,303)-k(8);
jac(91,303)=jac(91,303)+k(8);
jac(222,303)=jac(222,303)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

jac(303,303)=jac(303,303)-k(6);
jac(72,303)=jac(72,303)+k(6);
jac(515,303)=jac(515,303)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(304,304)=jac(304,304)-k(34);
jac(112,304)=jac(112,304)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(304,304)=jac(304,304)-k(31);
jac(418,304)=jac(418,304)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(304,304)=jac(304,304)-k(22);
jac(182,304)=jac(182,304)+k(22);
jac(590,304)=jac(590,304)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(304);
jac(3,304)=jac(3,304)-k(15)*y(3);
jac(304,3)=jac(304,3)-k(15)*y(304);
jac(304,304)=jac(304,304)-k(15)*y(3);
jac(591,3)=jac(591,3)+k(15)*y(304);
jac(591,304)=jac(591,304)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(304);
jac(3,304)=jac(3,304)-k(9)*y(3);
jac(304,3)=jac(304,3)-k(9)*y(304);
jac(304,304)=jac(304,304)-k(9)*y(3);
jac(594,3)=jac(594,3)+k(9)*y(304);
jac(594,304)=jac(594,304)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(304,304)=jac(304,304)-k(8);
jac(5,304)=jac(5,304)+k(8);
jac(218,304)=jac(218,304)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

jac(304,304)=jac(304,304)-k(6);
jac(1,304)=jac(1,304)+k(6);
jac(264,304)=jac(264,304)+k(6);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(307,307)=jac(307,307)-k(26);
jac(83,307)=jac(83,307)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(307,307)=jac(307,307)-k(22);
jac(2,307)=jac(2,307)+k(22);
jac(78,307)=jac(78,307)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(307,307)=jac(307,307)-k(19);
jac(628,307)=jac(628,307)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(307,307)=jac(307,307)-k(16);
jac(3,307)=jac(3,307)+k(16);
jac(306,307)=jac(306,307)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(307,307)=jac(307,307)-k(16);
jac(3,307)=jac(3,307)+k(16);
jac(80,307)=jac(80,307)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(307,307)=jac(307,307)-k(8);
jac(62,307)=jac(62,307)+k(8);
jac(43,307)=jac(43,307)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(307,307)=jac(307,307)-k(6);
jac(61,307)=jac(61,307)+k(6);
jac(114,307)=jac(114,307)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(308,308)=jac(308,308)-k(32);
jac(306,308)=jac(306,308)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(308,308)=jac(308,308)-k(31);
jac(524,308)=jac(524,308)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

jac(308,308)=jac(308,308)-k(22);
jac(2,308)=jac(2,308)+k(22);
jac(310,308)=jac(310,308)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(308);
jac(197,308)=jac(197,308)-k(21)*y(197);
jac(308,197)=jac(308,197)-k(21)*y(308);
jac(308,308)=jac(308,308)-k(21)*y(197);
jac(528,197)=jac(528,197)+k(21)*y(308);
jac(528,308)=jac(528,308)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(308);
jac(32,308)=jac(32,308)-k(21)*y(32);
jac(308,32)=jac(308,32)-k(21)*y(308);
jac(308,308)=jac(308,308)-k(21)*y(32);
jac(500,32)=jac(500,32)+k(21)*y(308);
jac(500,308)=jac(500,308)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(308);
jac(2,308)=jac(2,308)-k(21)*y(2);
jac(308,2)=jac(308,2)-k(21)*y(308);
jac(308,308)=jac(308,308)-k(21)*y(2);
jac(504,2)=jac(504,2)+k(21)*y(308);
jac(504,308)=jac(504,308)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(308,308)=jac(308,308)-k(16);
jac(3,308)=jac(3,308)+k(16);
jac(501,308)=jac(501,308)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(308);
jac(3,308)=jac(3,308)-k(15)*y(3);
jac(308,3)=jac(308,3)-k(15)*y(308);
jac(308,308)=jac(308,308)-k(15)*y(3);
jac(628,3)=jac(628,3)+k(15)*y(308);
jac(628,308)=jac(628,308)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(308,308)=jac(308,308)-k(8);
jac(315,308)=jac(315,308)+k(8);
jac(43,308)=jac(43,308)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

jac(308,308)=jac(308,308)-k(6);
jac(77,308)=jac(77,308)+k(6);
jac(114,308)=jac(114,308)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(310,310)=jac(310,310)-k(32);
jac(314,310)=jac(314,310)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(310,310)=jac(310,310)-k(32);
jac(309,310)=jac(309,310)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(310,310)=jac(310,310)-k(31);
jac(537,310)=jac(537,310)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(310);
jac(197,310)=jac(197,310)-k(21)*y(197);
jac(310,197)=jac(310,197)-k(21)*y(310);
jac(310,310)=jac(310,310)-k(21)*y(197);
jac(564,197)=jac(564,197)+k(21)*y(310);
jac(564,310)=jac(564,310)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(310);
jac(32,310)=jac(32,310)-k(21)*y(32);
jac(310,32)=jac(310,32)-k(21)*y(310);
jac(310,310)=jac(310,310)-k(21)*y(32);
jac(534,32)=jac(534,32)+k(21)*y(310);
jac(534,310)=jac(534,310)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(310);
jac(2,310)=jac(2,310)-k(21)*y(2);
jac(310,2)=jac(310,2)-k(21)*y(310);
jac(310,310)=jac(310,310)-k(21)*y(2);
jac(540,2)=jac(540,2)+k(21)*y(310);
jac(540,310)=jac(540,310)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(310);
jac(197,310)=jac(197,310)-k(21)*y(197);
jac(310,197)=jac(310,197)-k(21)*y(310);
jac(310,310)=jac(310,310)-k(21)*y(197);
jac(581,197)=jac(581,197)+k(21)*y(310);
jac(581,310)=jac(581,310)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(310);
jac(32,310)=jac(32,310)-k(21)*y(32);
jac(310,32)=jac(310,32)-k(21)*y(310);
jac(310,310)=jac(310,310)-k(21)*y(32);
jac(563,32)=jac(563,32)+k(21)*y(310);
jac(563,310)=jac(563,310)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(310);
jac(2,310)=jac(2,310)-k(21)*y(2);
jac(310,2)=jac(310,2)-k(21)*y(310);
jac(310,310)=jac(310,310)-k(21)*y(2);
jac(308,2)=jac(308,2)+k(21)*y(310);
jac(308,310)=jac(308,310)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

jac(310,310)=jac(310,310)-k(16);
jac(3,310)=jac(3,310)+k(16);
jac(584,310)=jac(584,310)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(310);
jac(3,310)=jac(3,310)-k(15)*y(3);
jac(310,3)=jac(310,3)-k(15)*y(310);
jac(310,310)=jac(310,310)-k(15)*y(3);
jac(627,3)=jac(627,3)+k(15)*y(310);
jac(627,310)=jac(627,310)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(310,310)=jac(310,310)-k(8);
jac(315,310)=jac(315,310)+k(8);
jac(54,310)=jac(54,310)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) 

jac(310,310)=jac(310,310)-k(6);
jac(77,310)=jac(77,310)+k(6);
jac(118,310)=jac(118,310)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(311,311)=jac(311,311)-k(33);
jac(306,311)=jac(306,311)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(311,311)=jac(311,311)-k(31);
jac(592,311)=jac(592,311)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

jac(311,311)=jac(311,311)-k(22);
jac(32,311)=jac(32,311)+k(22);
jac(309,311)=jac(309,311)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(311,311)=jac(311,311)-k(19);
jac(563,311)=jac(563,311)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(311,311)=jac(311,311)-k(16);
jac(3,311)=jac(3,311)+k(16);
jac(84,311)=jac(84,311)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(311);
jac(3,311)=jac(3,311)-k(15)*y(3);
jac(311,3)=jac(311,3)-k(15)*y(311);
jac(311,311)=jac(311,311)-k(15)*y(3);
jac(83,3)=jac(83,3)+k(15)*y(311);
jac(83,311)=jac(83,311)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(311,311)=jac(311,311)-k(8);
jac(91,311)=jac(91,311)+k(8);
jac(37,311)=jac(37,311)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

jac(311,311)=jac(311,311)-k(6);
jac(72,311)=jac(72,311)+k(6);
jac(554,311)=jac(554,311)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(312,312)=jac(312,312)-k(34);
jac(311,312)=jac(311,312)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(312,312)=jac(312,312)-k(33);
jac(303,312)=jac(303,312)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(312,312)=jac(312,312)-k(31);
jac(589,312)=jac(589,312)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

jac(312,312)=jac(312,312)-k(22);
jac(197,312)=jac(197,312)+k(22);
jac(309,312)=jac(309,312)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(312,312)=jac(312,312)-k(19);
jac(581,312)=jac(581,312)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(312,312)=jac(312,312)-k(16);
jac(3,312)=jac(3,312)+k(16);
jac(586,312)=jac(586,312)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(312);
jac(3,312)=jac(3,312)-k(15)*y(3);
jac(312,3)=jac(312,3)-k(15)*y(312);
jac(312,312)=jac(312,312)-k(15)*y(3);
jac(647,3)=jac(647,3)+k(15)*y(312);
jac(647,312)=jac(647,312)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(312,312)=jac(312,312)-k(8);
jac(91,312)=jac(91,312)+k(8);
jac(227,312)=jac(227,312)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

jac(312,312)=jac(312,312)-k(6);
jac(72,312)=jac(72,312)+k(6);
jac(543,312)=jac(543,312)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(315,315)=jac(315,315)-k(32);
jac(91,315)=jac(91,315)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(315,315)=jac(315,315)-k(31);
jac(16,315)=jac(16,315)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(315);
jac(197,315)=jac(197,315)-k(21)*y(197);
jac(315,197)=jac(315,197)-k(21)*y(315);
jac(315,315)=jac(315,315)-k(21)*y(197);
jac(319,197)=jac(319,197)+k(21)*y(315);
jac(319,315)=jac(319,315)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(315);
jac(32,315)=jac(32,315)-k(21)*y(32);
jac(315,32)=jac(315,32)-k(21)*y(315);
jac(315,315)=jac(315,315)-k(21)*y(32);
jac(34,32)=jac(34,32)+k(21)*y(315);
jac(34,315)=jac(34,315)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(315);
jac(2,315)=jac(2,315)-k(21)*y(2);
jac(315,2)=jac(315,2)-k(21)*y(315);
jac(315,315)=jac(315,315)-k(21)*y(2);
jac(437,2)=jac(437,2)+k(21)*y(315);
jac(437,315)=jac(437,315)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(315);
jac(3,315)=jac(3,315)-k(15)*y(3);
jac(315,3)=jac(315,3)-k(15)*y(315);
jac(315,315)=jac(315,315)-k(15)*y(3);
jac(55,3)=jac(55,3)+k(15)*y(315);
jac(55,315)=jac(55,315)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(1,1)=jac(1,1)-k(7)*y(315);
jac(1,315)=jac(1,315)-k(7)*y(1);
jac(315,1)=jac(315,1)-k(7)*y(315);
jac(315,315)=jac(315,315)-k(7)*y(1);
jac(234,1)=jac(234,1)+k(7)*y(315);
jac(234,315)=jac(234,315)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(315,315)=jac(315,315)-k(7)*y(7);
jac(315,7)=jac(315,7)-k(7)*y(315);
jac(7,315)=jac(7,315)-k(7)*y(7);
jac(7,7)=jac(7,7)-k(7)*y(315);
jac(19,315)=jac(19,315)+k(7)*y(7);
jac(19,7)=jac(19,7)+k(7)*y(315);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(315,315)=jac(315,315)-k(7)*y(335);
jac(315,335)=jac(315,335)-k(7)*y(315);
jac(335,315)=jac(335,315)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(315);
jac(410,315)=jac(410,315)+k(7)*y(335);
jac(410,335)=jac(410,335)+k(7)*y(315);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) -> Rec(a, b~pY, g~pY) + Lig(l1, l2) 

jac(315,315)=jac(315,315)-k(2);
jac(4,315)=jac(4,315)+k(2);
jac(77,315)=jac(77,315)+k(2);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(316,316)=jac(316,316)-k(31);
jac(329,316)=jac(329,316)+k(31);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(316,316)=jac(316,316)-k(25);
jac(317,316)=jac(317,316)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

jac(316,316)=jac(316,316)-k(22);
jac(2,316)=jac(2,316)+k(22);
jac(314,316)=jac(314,316)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(316,316)=jac(316,316)-k(16);
jac(3,316)=jac(3,316)+k(16);
jac(653,316)=jac(653,316)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(316);
jac(3,316)=jac(3,316)-k(15)*y(3);
jac(316,3)=jac(316,3)-k(15)*y(316);
jac(316,316)=jac(316,316)-k(15)*y(3);
jac(654,3)=jac(654,3)+k(15)*y(316);
jac(654,316)=jac(654,316)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(316,316)=jac(316,316)-k(8);
jac(61,316)=jac(61,316)+k(8);
jac(437,316)=jac(437,316)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(316,316)=jac(316,316)-k(6);
jac(122,316)=jac(122,316)+k(6);
jac(111,316)=jac(111,316)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(317,317)=jac(317,317)-k(33);
jac(316,317)=jac(316,317)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(317,317)=jac(317,317)-k(31);
jac(133,317)=jac(133,317)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

jac(317,317)=jac(317,317)-k(22);
jac(32,317)=jac(32,317)+k(22);
jac(314,317)=jac(314,317)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(317,317)=jac(317,317)-k(16);
jac(3,317)=jac(3,317)+k(16);
jac(242,317)=jac(242,317)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(317);
jac(3,317)=jac(3,317)-k(15)*y(3);
jac(317,3)=jac(317,3)-k(15)*y(317);
jac(317,317)=jac(317,317)-k(15)*y(3);
jac(241,3)=jac(241,3)+k(15)*y(317);
jac(241,317)=jac(241,317)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(317,317)=jac(317,317)-k(8);
jac(61,317)=jac(61,317)+k(8);
jac(34,317)=jac(34,317)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(317,317)=jac(317,317)-k(6);
jac(122,317)=jac(122,317)+k(6);
jac(33,317)=jac(33,317)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(319,319)=jac(319,319)-k(34);
jac(34,319)=jac(34,319)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(319,319)=jac(319,319)-k(33);
jac(219,319)=jac(219,319)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(319,319)=jac(319,319)-k(31);
jac(209,319)=jac(209,319)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~pY, a~pY) 

jac(319,319)=jac(319,319)-k(22);
jac(197,319)=jac(197,319)+k(22);
jac(315,319)=jac(315,319)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(319);
jac(3,319)=jac(3,319)-k(15)*y(3);
jac(319,3)=jac(319,3)-k(15)*y(319);
jac(319,319)=jac(319,319)-k(15)*y(3);
jac(228,3)=jac(228,3)+k(15)*y(319);
jac(228,319)=jac(228,319)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(1,1)=jac(1,1)-k(7)*y(319);
jac(1,319)=jac(1,319)-k(7)*y(1);
jac(319,1)=jac(319,1)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(1);
jac(230,1)=jac(230,1)+k(7)*y(319);
jac(230,319)=jac(230,319)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(7,7)=jac(7,7)-k(7)*y(319);
jac(7,319)=jac(7,319)-k(7)*y(7);
jac(319,7)=jac(319,7)-k(7)*y(319);
jac(319,319)=jac(319,319)-k(7)*y(7);
jac(212,7)=jac(212,7)+k(7)*y(319);
jac(212,319)=jac(212,319)+k(7)*y(7);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(319,319)=jac(319,319)-k(7)*y(335);
jac(319,335)=jac(319,335)-k(7)*y(319);
jac(335,319)=jac(335,319)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(319);
jac(434,319)=jac(434,319)+k(7)*y(335);
jac(434,335)=jac(434,335)+k(7)*y(319);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lig(l1, l2) 

jac(319,319)=jac(319,319)-k(2);
jac(4,319)=jac(4,319)+k(2);
jac(257,319)=jac(257,319)+k(2);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(320,320)=jac(320,320)-k(34);
jac(241,320)=jac(241,320)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(320,320)=jac(320,320)-k(33);
jac(324,320)=jac(324,320)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(320,320)=jac(320,320)-k(22);
jac(197,320)=jac(197,320)+k(22);
jac(236,320)=jac(236,320)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(320,320)=jac(320,320)-k(20);
jac(636,320)=jac(636,320)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(320,320)=jac(320,320)-k(16);
jac(3,320)=jac(3,320)+k(16);
jac(318,320)=jac(318,320)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(320,320)=jac(320,320)-k(16);
jac(3,320)=jac(3,320)+k(16);
jac(231,320)=jac(231,320)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(320,320)=jac(320,320)-k(8);
jac(61,320)=jac(61,320)+k(8);
jac(228,320)=jac(228,320)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(320,320)=jac(320,320)-k(6);
jac(122,320)=jac(122,320)+k(6);
jac(227,320)=jac(227,320)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(321,321)=jac(321,321)-k(34);
jac(242,321)=jac(242,321)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(321,321)=jac(321,321)-k(33);
jac(325,321)=jac(325,321)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(321,321)=jac(321,321)-k(31);
jac(230,321)=jac(230,321)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(321,321)=jac(321,321)-k(31);
jac(332,321)=jac(332,321)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~pY) 

jac(321,321)=jac(321,321)-k(22);
jac(197,321)=jac(197,321)+k(22);
jac(237,321)=jac(237,321)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(321);
jac(3,321)=jac(3,321)-k(15)*y(3);
jac(321,3)=jac(321,3)-k(15)*y(321);
jac(321,321)=jac(321,321)-k(15)*y(3);
jac(231,3)=jac(231,3)+k(15)*y(321);
jac(231,321)=jac(231,321)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(321);
jac(3,321)=jac(3,321)-k(15)*y(3);
jac(321,3)=jac(321,3)-k(15)*y(321);
jac(321,321)=jac(321,321)-k(15)*y(3);
jac(318,3)=jac(318,3)+k(15)*y(321);
jac(318,321)=jac(318,321)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

jac(321,321)=jac(321,321)-k(8);
jac(72,321)=jac(72,321)+k(8);
jac(319,321)=jac(319,321)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(321,321)=jac(321,321)-k(6);
jac(71,321)=jac(71,321)+k(6);
jac(257,321)=jac(257,321)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(322,322)=jac(322,322)-k(34);
jac(133,322)=jac(133,322)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(322,322)=jac(322,322)-k(33);
jac(326,322)=jac(326,322)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

jac(322,322)=jac(322,322)-k(22);
jac(197,322)=jac(197,322)+k(22);
jac(127,322)=jac(127,322)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(322,322)=jac(322,322)-k(17);
jac(318,322)=jac(318,322)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(322,322)=jac(322,322)-k(16);
jac(3,322)=jac(3,322)+k(16);
jac(332,322)=jac(332,322)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(322);
jac(3,322)=jac(3,322)-k(9)*y(3);
jac(322,3)=jac(322,3)-k(9)*y(322);
jac(322,322)=jac(322,322)-k(9)*y(3);
jac(200,3)=jac(200,3)+k(9)*y(322);
jac(200,322)=jac(200,322)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(322,322)=jac(322,322)-k(8);
jac(61,322)=jac(61,322)+k(8);
jac(209,322)=jac(209,322)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(322,322)=jac(322,322)-k(6);
jac(122,322)=jac(122,322)+k(6);
jac(192,322)=jac(192,322)+k(6);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(654,654)=jac(654,654)-k(25);
jac(241,654)=jac(241,654)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(654,654)=jac(654,654)-k(22);
jac(2,654)=jac(2,654)+k(22);
jac(236,654)=jac(236,654)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(654,654)=jac(654,654)-k(20);
jac(251,654)=jac(251,654)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(654,654)=jac(654,654)-k(16);
jac(3,654)=jac(3,654)+k(16);
jac(239,654)=jac(239,654)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(654,654)=jac(654,654)-k(16);
jac(3,654)=jac(3,654)+k(16);
jac(316,654)=jac(316,654)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(654,654)=jac(654,654)-k(8);
jac(61,654)=jac(61,654)+k(8);
jac(44,654)=jac(44,654)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(654,654)=jac(654,654)-k(6);
jac(122,654)=jac(122,654)+k(6);
jac(43,654)=jac(43,654)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(324,324)=jac(324,324)-k(34);
jac(654,324)=jac(654,324)+k(34);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(324,324)=jac(324,324)-k(25);
jac(320,324)=jac(320,324)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(324,324)=jac(324,324)-k(22);
jac(182,324)=jac(182,324)+k(22);
jac(236,324)=jac(236,324)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(324,324)=jac(324,324)-k(20);
jac(638,324)=jac(638,324)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(324,324)=jac(324,324)-k(16);
jac(3,324)=jac(3,324)+k(16);
jac(323,324)=jac(323,324)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(324,324)=jac(324,324)-k(16);
jac(3,324)=jac(3,324)+k(16);
jac(646,324)=jac(646,324)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(324,324)=jac(324,324)-k(8);
jac(61,324)=jac(61,324)+k(8);
jac(223,324)=jac(223,324)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(324,324)=jac(324,324)-k(6);
jac(122,324)=jac(122,324)+k(6);
jac(222,324)=jac(222,324)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(653,653)=jac(653,653)-k(31);
jac(621,653)=jac(621,653)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(653,653)=jac(653,653)-k(31);
jac(142,653)=jac(142,653)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~Y) 

jac(653,653)=jac(653,653)-k(22);
jac(2,653)=jac(2,653)+k(22);
jac(237,653)=jac(237,653)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(653);
jac(3,653)=jac(3,653)-k(15)*y(3);
jac(653,3)=jac(653,3)-k(15)*y(653);
jac(653,653)=jac(653,653)-k(15)*y(3);
jac(239,3)=jac(239,3)+k(15)*y(653);
jac(239,653)=jac(239,653)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(653);
jac(3,653)=jac(3,653)-k(15)*y(3);
jac(653,3)=jac(653,3)-k(15)*y(653);
jac(653,653)=jac(653,653)-k(15)*y(3);
jac(316,3)=jac(316,3)+k(15)*y(653);
jac(316,653)=jac(316,653)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

jac(653,653)=jac(653,653)-k(8);
jac(72,653)=jac(72,653)+k(8);
jac(437,653)=jac(437,653)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(653,653)=jac(653,653)-k(6);
jac(71,653)=jac(71,653)+k(6);
jac(111,653)=jac(111,653)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(325,325)=jac(325,325)-k(34);
jac(653,325)=jac(653,325)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(325,325)=jac(325,325)-k(31);
jac(220,325)=jac(220,325)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(325,325)=jac(325,325)-k(31);
jac(328,325)=jac(328,325)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~pY) 

jac(325,325)=jac(325,325)-k(22);
jac(182,325)=jac(182,325)+k(22);
jac(237,325)=jac(237,325)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(325);
jac(3,325)=jac(3,325)-k(15)*y(3);
jac(325,3)=jac(325,3)-k(15)*y(325);
jac(325,325)=jac(325,325)-k(15)*y(3);
jac(646,3)=jac(646,3)+k(15)*y(325);
jac(646,325)=jac(646,325)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(325);
jac(3,325)=jac(3,325)-k(15)*y(3);
jac(325,3)=jac(325,3)-k(15)*y(325);
jac(325,325)=jac(325,325)-k(15)*y(3);
jac(323,3)=jac(323,3)+k(15)*y(325);
jac(323,325)=jac(323,325)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

jac(325,325)=jac(325,325)-k(8);
jac(72,325)=jac(72,325)+k(8);
jac(219,325)=jac(219,325)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(325,325)=jac(325,325)-k(6);
jac(71,325)=jac(71,325)+k(6);
jac(218,325)=jac(218,325)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(327,327)=jac(327,327)-k(34);
jac(129,327)=jac(129,327)+k(34);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(327,327)=jac(327,327)-k(25);
jac(200,327)=jac(200,327)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(327,327)=jac(327,327)-k(22);
jac(182,327)=jac(182,327)+k(22);
jac(125,327)=jac(125,327)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(327,327)=jac(327,327)-k(16);
jac(3,327)=jac(3,327)+k(16);
jac(330,327)=jac(330,327)+k(16);

% rule    : R5_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(327,327)=jac(327,327)-k(14);
jac(345,327)=jac(345,327)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(327,327)=jac(327,327)-k(10);
jac(3,327)=jac(3,327)+k(10);
jac(326,327)=jac(326,327)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(327,327)=jac(327,327)-k(8);
jac(61,327)=jac(61,327)+k(8);
jac(185,327)=jac(185,327)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(327,327)=jac(327,327)-k(6);
jac(122,327)=jac(122,327)+k(6);
jac(184,327)=jac(184,327)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(652,652)=jac(652,652)-k(34);
jac(144,652)=jac(144,652)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(652,652)=jac(652,652)-k(22);
jac(182,652)=jac(182,652)+k(22);
jac(146,652)=jac(146,652)+k(22);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(652,652)=jac(652,652)-k(14);
jac(189,652)=jac(189,652)+k(14);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(652,652)=jac(652,652)-k(12);
jac(330,652)=jac(330,652)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(652,652)=jac(652,652)-k(10);
jac(3,652)=jac(3,652)+k(10);
jac(216,652)=jac(216,652)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(652);
jac(3,652)=jac(3,652)-k(9)*y(3);
jac(652,3)=jac(652,3)-k(9)*y(652);
jac(652,652)=jac(652,652)-k(9)*y(3);
jac(215,3)=jac(215,3)+k(9)*y(652);
jac(215,652)=jac(215,652)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

jac(652,652)=jac(652,652)-k(8);
jac(1,652)=jac(1,652)+k(8);
jac(185,652)=jac(185,652)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(652,652)=jac(652,652)-k(6);
jac(6,652)=jac(6,652)+k(6);
jac(184,652)=jac(184,652)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(330,330)=jac(330,330)-k(34);
jac(139,330)=jac(139,330)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(330,330)=jac(330,330)-k(31);
jac(652,330)=jac(652,330)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(330,330)=jac(330,330)-k(22);
jac(182,330)=jac(182,330)+k(22);
jac(128,330)=jac(128,330)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(330);
jac(3,330)=jac(3,330)-k(15)*y(3);
jac(330,3)=jac(330,3)-k(15)*y(330);
jac(330,330)=jac(330,330)-k(15)*y(3);
jac(327,3)=jac(327,3)+k(15)*y(330);
jac(327,330)=jac(327,330)+k(15)*y(3);

% rule    : R5_2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(330,330)=jac(330,330)-k(14);
jac(343,330)=jac(343,330)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(330,330)=jac(330,330)-k(10);
jac(3,330)=jac(3,330)+k(10);
jac(328,330)=jac(328,330)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

jac(330,330)=jac(330,330)-k(8);
jac(72,330)=jac(72,330)+k(8);
jac(185,330)=jac(185,330)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(330,330)=jac(330,330)-k(6);
jac(71,330)=jac(71,330)+k(6);
jac(184,330)=jac(184,330)+k(6);

% rule    : R17
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(335,335)=jac(335,335)-k(34);
jac(131,335)=jac(131,335)+k(34);

% rule    : R16
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(335,335)=jac(335,335)-k(33);
jac(184,335)=jac(184,335)+k(33);

% rule    : R9_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(335,335)=jac(335,335)-k(22);
jac(197,335)=jac(197,335)+k(22);
jac(48,335)=jac(48,335)+k(22);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Lyn(U, SH2) 

jac(335,335)=jac(335,335)-k(10);
jac(3,335)=jac(3,335)+k(10);
jac(192,335)=jac(192,335)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(7)*y(336);
jac(335,336)=jac(335,336)-k(7)*y(335);
jac(336,335)=jac(336,335)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(335);
jac(464,335)=jac(464,335)+k(7)*y(336);
jac(464,336)=jac(464,336)+k(7)*y(335);

% rule    : R2_1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(437,437)=jac(437,437)-k(7)*y(335);
jac(437,335)=jac(437,335)-k(7)*y(437);
jac(335,437)=jac(335,437)-k(7)*y(335);
jac(335,335)=jac(335,335)-k(7)*y(437);
jac(436,437)=jac(436,437)+k(7)*y(335);
jac(436,335)=jac(436,335)+k(7)*y(437);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(5)*y(380);
jac(335,380)=jac(335,380)-k(5)*y(335);
jac(380,335)=jac(380,335)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(335);
jac(372,335)=jac(372,335)+k(5)*y(380);
jac(372,380)=jac(372,380)+k(5)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(5)*y(391);
jac(335,391)=jac(335,391)-k(5)*y(335);
jac(391,335)=jac(391,335)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(335);
jac(368,335)=jac(368,335)+k(5)*y(391);
jac(368,391)=jac(368,391)+k(5)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(5)*y(407);
jac(335,407)=jac(335,407)-k(5)*y(335);
jac(407,335)=jac(407,335)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(335);
jac(464,335)=jac(464,335)+k(5)*y(407);
jac(464,407)=jac(464,407)+k(5)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(5)*y(441);
jac(335,441)=jac(335,441)-k(5)*y(335);
jac(441,335)=jac(441,335)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(335);
jac(459,335)=jac(459,335)+k(5)*y(441);
jac(459,441)=jac(459,441)+k(5)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(5)*y(515);
jac(335,515)=jac(335,515)-k(5)*y(335);
jac(515,335)=jac(515,335)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(335);
jac(597,335)=jac(597,335)+k(5)*y(515);
jac(597,515)=jac(597,515)+k(5)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(5)*y(543);
jac(335,543)=jac(335,543)-k(5)*y(335);
jac(543,335)=jac(543,335)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(335);
jac(601,335)=jac(601,335)+k(5)*y(543);
jac(601,543)=jac(601,543)+k(5)*y(335);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(335,335)=jac(335,335)-k(5)*y(554);
jac(335,554)=jac(335,554)-k(5)*y(335);
jac(554,335)=jac(554,335)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(335);
jac(558,335)=jac(558,335)+k(5)*y(554);
jac(558,554)=jac(558,554)+k(5)*y(335);

% rule    : R2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(9,9)=jac(9,9)-k(5)*y(335);
jac(9,335)=jac(9,335)-k(5)*y(9);
jac(335,9)=jac(335,9)-k(5)*y(335);
jac(335,335)=jac(335,335)-k(5)*y(9);
jac(210,9)=jac(210,9)+k(5)*y(335);
jac(210,335)=jac(210,335)+k(5)*y(9);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(4,4)=jac(4,4)-k(3)*y(335);
jac(4,335)=jac(4,335)-k(3)*y(4);
jac(335,4)=jac(335,4)-k(3)*y(335);
jac(335,335)=jac(335,335)-k(3)*y(4);
jac(407,4)=jac(407,4)+k(3)*y(335);
jac(407,335)=jac(407,335)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(4,4)=jac(4,4)-k(1)*y(335);
jac(4,335)=jac(4,335)-k(1)*y(4);
jac(335,4)=jac(335,4)-k(1)*y(335);
jac(335,335)=jac(335,335)-k(1)*y(4);
jac(336,4)=jac(336,4)+k(1)*y(335);
jac(336,335)=jac(336,335)+k(1)*y(4);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(336,336)=jac(336,336)-k(34);
jac(132,336)=jac(132,336)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(336,336)=jac(336,336)-k(33);
jac(185,336)=jac(185,336)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(336,336)=jac(336,336)-k(22);
jac(197,336)=jac(197,336)+k(22);
jac(126,336)=jac(126,336)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(336,336)=jac(336,336)-k(10);
jac(3,336)=jac(3,336)+k(10);
jac(209,336)=jac(209,336)+k(10);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(1,1)=jac(1,1)-k(7)*y(336);
jac(1,336)=jac(1,336)-k(7)*y(1);
jac(336,1)=jac(336,1)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(1);
jac(199,1)=jac(199,1)+k(7)*y(336);
jac(199,336)=jac(199,336)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(7)*y(336);
jac(7,336)=jac(7,336)-k(7)*y(7);
jac(336,7)=jac(336,7)-k(7)*y(336);
jac(336,336)=jac(336,336)-k(7)*y(7);
jac(210,7)=jac(210,7)+k(7)*y(336);
jac(210,336)=jac(210,336)+k(7)*y(7);

% rule    : R1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) 

jac(336,336)=jac(336,336)-k(2);
jac(4,336)=jac(4,336)+k(2);
jac(335,336)=jac(335,336)+k(2);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(338,338)=jac(338,338)-k(34);
jac(533,338)=jac(533,338)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(338,338)=jac(338,338)-k(33);
jac(344,338)=jac(344,338)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(338,338)=jac(338,338)-k(32);
jac(332,338)=jac(332,338)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) 

jac(338,338)=jac(338,338)-k(31);
jac(376,338)=jac(376,338)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(338,338)=jac(338,338)-k(22);
jac(197,338)=jac(197,338)+k(22);
jac(610,338)=jac(610,338)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(338);
jac(197,338)=jac(197,338)-k(21)*y(197);
jac(338,197)=jac(338,197)-k(21)*y(338);
jac(338,338)=jac(338,338)-k(21)*y(197);
jac(609,197)=jac(609,197)+k(21)*y(338);
jac(609,338)=jac(609,338)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(338);
jac(32,338)=jac(32,338)-k(21)*y(32);
jac(338,32)=jac(338,32)-k(21)*y(338);
jac(338,338)=jac(338,338)-k(21)*y(32);
jac(559,32)=jac(559,32)+k(21)*y(338);
jac(559,338)=jac(559,338)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(338);
jac(2,338)=jac(2,338)-k(21)*y(2);
jac(338,2)=jac(338,2)-k(21)*y(338);
jac(338,338)=jac(338,338)-k(21)*y(2);
jac(527,2)=jac(527,2)+k(21)*y(338);
jac(527,338)=jac(527,338)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(338);
jac(3,338)=jac(3,338)-k(15)*y(3);
jac(338,3)=jac(338,3)-k(15)*y(338);
jac(338,338)=jac(338,338)-k(15)*y(3);
jac(529,3)=jac(529,3)+k(15)*y(338);
jac(529,338)=jac(529,338)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(338);
jac(3,338)=jac(3,338)-k(9)*y(3);
jac(338,3)=jac(338,3)-k(9)*y(338);
jac(338,338)=jac(338,338)-k(9)*y(3);
jac(337,3)=jac(337,3)+k(9)*y(338);
jac(337,338)=jac(337,338)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

jac(338,338)=jac(338,338)-k(8);
jac(77,338)=jac(77,338)+k(8);
jac(209,338)=jac(209,338)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(338,338)=jac(338,338)-k(6);
jac(76,338)=jac(76,338)+k(6);
jac(192,338)=jac(192,338)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(339,339)=jac(339,339)-k(34);
jac(134,339)=jac(134,339)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(339,339)=jac(339,339)-k(33);
jac(345,339)=jac(345,339)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(339,339)=jac(339,339)-k(32);
jac(200,339)=jac(200,339)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(339,339)=jac(339,339)-k(22);
jac(197,339)=jac(197,339)+k(22);
jac(120,339)=jac(120,339)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(339);
jac(197,339)=jac(197,339)-k(21)*y(197);
jac(339,197)=jac(339,197)-k(21)*y(339);
jac(339,339)=jac(339,339)-k(21)*y(197);
jac(601,197)=jac(601,197)+k(21)*y(339);
jac(601,339)=jac(601,339)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(339);
jac(32,339)=jac(32,339)-k(21)*y(32);
jac(339,32)=jac(339,32)-k(21)*y(339);
jac(339,339)=jac(339,339)-k(21)*y(32);
jac(558,32)=jac(558,32)+k(21)*y(339);
jac(558,339)=jac(558,339)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(339);
jac(2,339)=jac(2,339)-k(21)*y(2);
jac(339,2)=jac(339,2)-k(21)*y(339);
jac(339,339)=jac(339,339)-k(21)*y(2);
jac(526,2)=jac(526,2)+k(21)*y(339);
jac(526,339)=jac(526,339)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(339,339)=jac(339,339)-k(16);
jac(3,339)=jac(3,339)+k(16);
jac(337,339)=jac(337,339)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(339,339)=jac(339,339)-k(10);
jac(3,339)=jac(3,339)+k(10);
jac(529,339)=jac(529,339)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(339,339)=jac(339,339)-k(8);
jac(54,339)=jac(54,339)+k(8);
jac(336,339)=jac(336,339)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(339,339)=jac(339,339)-k(6);
jac(118,339)=jac(118,339)+k(6);
jac(335,339)=jac(335,339)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(340,340)=jac(340,340)-k(34);
jac(355,340)=jac(355,340)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(340,340)=jac(340,340)-k(33);
jac(346,340)=jac(346,340)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(340,340)=jac(340,340)-k(31);
jac(371,340)=jac(371,340)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(340,340)=jac(340,340)-k(30);
jac(363,340)=jac(363,340)+k(30);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(340,340)=jac(340,340)-k(24);
jac(341,340)=jac(341,340)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(340,340)=jac(340,340)-k(22);
jac(2,340)=jac(2,340)+k(22);
jac(337,340)=jac(337,340)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(340,340)=jac(340,340)-k(22);
jac(197,340)=jac(197,340)+k(22);
jac(353,340)=jac(353,340)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(340);
jac(3,340)=jac(3,340)-k(15)*y(3);
jac(340,3)=jac(340,3)-k(15)*y(340);
jac(340,340)=jac(340,340)-k(15)*y(3);
jac(526,3)=jac(526,3)+k(15)*y(340);
jac(526,340)=jac(526,340)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(340,340)=jac(340,340)-k(10);
jac(3,340)=jac(3,340)+k(10);
jac(527,340)=jac(527,340)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(340,340)=jac(340,340)-k(8);
jac(111,340)=jac(111,340)+k(8);
jac(336,340)=jac(336,340)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(340,340)=jac(340,340)-k(6);
jac(110,340)=jac(110,340)+k(6);
jac(335,340)=jac(335,340)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(341,341)=jac(341,341)-k(34);
jac(161,341)=jac(161,341)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(341,341)=jac(341,341)-k(33);
jac(340,341)=jac(340,341)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(341,341)=jac(341,341)-k(33);
jac(347,341)=jac(347,341)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(341,341)=jac(341,341)-k(31);
jac(162,341)=jac(162,341)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(341,341)=jac(341,341)-k(30);
jac(342,341)=jac(342,341)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(341,341)=jac(341,341)-k(22);
jac(32,341)=jac(32,341)+k(22);
jac(337,341)=jac(337,341)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(341,341)=jac(341,341)-k(22);
jac(197,341)=jac(197,341)+k(22);
jac(350,341)=jac(350,341)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(341);
jac(3,341)=jac(3,341)-k(15)*y(3);
jac(341,3)=jac(341,3)-k(15)*y(341);
jac(341,341)=jac(341,341)-k(15)*y(3);
jac(558,3)=jac(558,3)+k(15)*y(341);
jac(558,341)=jac(558,341)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(341,341)=jac(341,341)-k(10);
jac(3,341)=jac(3,341)+k(10);
jac(559,341)=jac(559,341)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(341,341)=jac(341,341)-k(8);
jac(33,341)=jac(33,341)+k(8);
jac(336,341)=jac(336,341)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(341,341)=jac(341,341)-k(6);
jac(82,341)=jac(82,341)+k(6);
jac(335,341)=jac(335,341)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(342,342)=jac(342,342)-k(34);
jac(341,342)=jac(341,342)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(342,342)=jac(342,342)-k(34);
jac(362,342)=jac(362,342)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(342,342)=jac(342,342)-k(33);
jac(363,342)=jac(363,342)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(342,342)=jac(342,342)-k(33);
jac(195,342)=jac(195,342)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(342,342)=jac(342,342)-k(31);
jac(370,342)=jac(370,342)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(342,342)=jac(342,342)-k(22);
jac(197,342)=jac(197,342)+k(22);
jac(337,342)=jac(337,342)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(342,342)=jac(342,342)-k(22);
jac(197,342)=jac(197,342)+k(22);
jac(599,342)=jac(599,342)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(342);
jac(3,342)=jac(3,342)-k(15)*y(3);
jac(342,3)=jac(342,3)-k(15)*y(342);
jac(342,342)=jac(342,342)-k(15)*y(3);
jac(601,3)=jac(601,3)+k(15)*y(342);
jac(601,342)=jac(601,342)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(342,342)=jac(342,342)-k(10);
jac(3,342)=jac(3,342)+k(10);
jac(609,342)=jac(609,342)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(342,342)=jac(342,342)-k(8);
jac(257,342)=jac(257,342)+k(8);
jac(336,342)=jac(336,342)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(342,342)=jac(342,342)-k(6);
jac(256,342)=jac(256,342)+k(6);
jac(335,342)=jac(335,342)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(344,344)=jac(344,344)-k(34);
jac(508,344)=jac(508,344)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(344,344)=jac(344,344)-k(32);
jac(328,344)=jac(328,344)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(344,344)=jac(344,344)-k(31);
jac(451,344)=jac(451,344)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(344,344)=jac(344,344)-k(22);
jac(182,344)=jac(182,344)+k(22);
jac(610,344)=jac(610,344)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(344);
jac(197,344)=jac(197,344)-k(21)*y(197);
jac(344,197)=jac(344,197)-k(21)*y(344);
jac(344,344)=jac(344,344)-k(21)*y(197);
jac(550,197)=jac(550,197)+k(21)*y(344);
jac(550,344)=jac(550,344)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(344);
jac(32,344)=jac(32,344)-k(21)*y(32);
jac(344,32)=jac(344,32)-k(21)*y(344);
jac(344,344)=jac(344,344)-k(21)*y(32);
jac(348,32)=jac(348,32)+k(21)*y(344);
jac(348,344)=jac(348,344)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(344);
jac(2,344)=jac(2,344)-k(21)*y(2);
jac(344,2)=jac(344,2)-k(21)*y(344);
jac(344,344)=jac(344,344)-k(21)*y(2);
jac(509,2)=jac(509,2)+k(21)*y(344);
jac(509,344)=jac(509,344)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(344);
jac(3,344)=jac(3,344)-k(15)*y(3);
jac(344,3)=jac(344,3)-k(15)*y(344);
jac(344,344)=jac(344,344)-k(15)*y(3);
jac(519,3)=jac(519,3)+k(15)*y(344);
jac(519,344)=jac(519,344)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(344);
jac(3,344)=jac(3,344)-k(9)*y(3);
jac(344,3)=jac(344,3)-k(9)*y(344);
jac(344,344)=jac(344,344)-k(9)*y(3);
jac(343,3)=jac(343,3)+k(9)*y(344);
jac(343,344)=jac(343,344)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

jac(344,344)=jac(344,344)-k(8);
jac(77,344)=jac(77,344)+k(8);
jac(214,344)=jac(214,344)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(344,344)=jac(344,344)-k(6);
jac(76,344)=jac(76,344)+k(6);
jac(177,344)=jac(177,344)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(345,345)=jac(345,345)-k(34);
jac(539,345)=jac(539,345)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(345,345)=jac(345,345)-k(32);
jac(327,345)=jac(327,345)+k(32);

% rule    : R11
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(345,345)=jac(345,345)-k(25);
jac(339,345)=jac(339,345)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(345,345)=jac(345,345)-k(22);
jac(182,345)=jac(182,345)+k(22);
jac(120,345)=jac(120,345)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(345);
jac(197,345)=jac(197,345)-k(21)*y(197);
jac(345,197)=jac(345,197)-k(21)*y(345);
jac(345,345)=jac(345,345)-k(21)*y(197);
jac(549,197)=jac(549,197)+k(21)*y(345);
jac(549,345)=jac(549,345)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(345);
jac(32,345)=jac(32,345)-k(21)*y(32);
jac(345,32)=jac(345,32)-k(21)*y(345);
jac(345,345)=jac(345,345)-k(21)*y(32);
jac(349,32)=jac(349,32)+k(21)*y(345);
jac(349,345)=jac(349,345)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(345);
jac(2,345)=jac(2,345)-k(21)*y(2);
jac(345,2)=jac(345,2)-k(21)*y(345);
jac(345,345)=jac(345,345)-k(21)*y(2);
jac(522,2)=jac(522,2)+k(21)*y(345);
jac(522,345)=jac(522,345)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(345,345)=jac(345,345)-k(16);
jac(3,345)=jac(3,345)+k(16);
jac(343,345)=jac(343,345)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(345,345)=jac(345,345)-k(10);
jac(3,345)=jac(3,345)+k(10);
jac(519,345)=jac(519,345)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(345,345)=jac(345,345)-k(8);
jac(54,345)=jac(54,345)+k(8);
jac(185,345)=jac(185,345)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(345,345)=jac(345,345)-k(6);
jac(118,345)=jac(118,345)+k(6);
jac(184,345)=jac(184,345)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(346,346)=jac(346,346)-k(34);
jac(173,346)=jac(173,346)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(346,346)=jac(346,346)-k(31);
jac(174,346)=jac(174,346)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(346,346)=jac(346,346)-k(30);
jac(188,346)=jac(188,346)+k(30);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(346,346)=jac(346,346)-k(24);
jac(347,346)=jac(347,346)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(346,346)=jac(346,346)-k(22);
jac(2,346)=jac(2,346)+k(22);
jac(343,346)=jac(343,346)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(346,346)=jac(346,346)-k(22);
jac(182,346)=jac(182,346)+k(22);
jac(353,346)=jac(353,346)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(346);
jac(3,346)=jac(3,346)-k(15)*y(3);
jac(346,3)=jac(346,3)-k(15)*y(346);
jac(346,346)=jac(346,346)-k(15)*y(3);
jac(522,3)=jac(522,3)+k(15)*y(346);
jac(522,346)=jac(522,346)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(346,346)=jac(346,346)-k(10);
jac(3,346)=jac(3,346)+k(10);
jac(509,346)=jac(509,346)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(346,346)=jac(346,346)-k(8);
jac(111,346)=jac(111,346)+k(8);
jac(185,346)=jac(185,346)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(346,346)=jac(346,346)-k(6);
jac(110,346)=jac(110,346)+k(6);
jac(184,346)=jac(184,346)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(348,348)=jac(348,348)-k(34);
jac(575,348)=jac(575,348)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(348,348)=jac(348,348)-k(33);
jac(509,348)=jac(509,348)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(348,348)=jac(348,348)-k(31);
jac(449,348)=jac(449,348)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(348,348)=jac(348,348)-k(30);
jac(550,348)=jac(550,348)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(348,348)=jac(348,348)-k(22);
jac(32,348)=jac(32,348)+k(22);
jac(344,348)=jac(344,348)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(348,348)=jac(348,348)-k(22);
jac(182,348)=jac(182,348)+k(22);
jac(351,348)=jac(351,348)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(348);
jac(3,348)=jac(3,348)-k(15)*y(3);
jac(348,3)=jac(348,3)-k(15)*y(348);
jac(348,348)=jac(348,348)-k(15)*y(3);
jac(553,3)=jac(553,3)+k(15)*y(348);
jac(553,348)=jac(553,348)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(348);
jac(3,348)=jac(3,348)-k(9)*y(3);
jac(348,3)=jac(348,3)-k(9)*y(348);
jac(348,348)=jac(348,348)-k(9)*y(3);
jac(347,3)=jac(347,3)+k(9)*y(348);
jac(347,348)=jac(347,348)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(348,348)=jac(348,348)-k(8);
jac(33,348)=jac(33,348)+k(8);
jac(214,348)=jac(214,348)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(348,348)=jac(348,348)-k(6);
jac(177,348)=jac(177,348)+k(6);
jac(82,348)=jac(82,348)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(349,349)=jac(349,349)-k(34);
jac(603,349)=jac(603,349)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(349,349)=jac(349,349)-k(33);
jac(522,349)=jac(522,349)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(349,349)=jac(349,349)-k(30);
jac(549,349)=jac(549,349)+k(30);

% rule    : R11
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(349,349)=jac(349,349)-k(25);
jac(558,349)=jac(558,349)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

jac(349,349)=jac(349,349)-k(22);
jac(32,349)=jac(32,349)+k(22);
jac(345,349)=jac(345,349)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(349,349)=jac(349,349)-k(22);
jac(182,349)=jac(182,349)+k(22);
jac(352,349)=jac(352,349)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(349,349)=jac(349,349)-k(16);
jac(3,349)=jac(3,349)+k(16);
jac(347,349)=jac(347,349)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(349,349)=jac(349,349)-k(10);
jac(3,349)=jac(3,349)+k(10);
jac(553,349)=jac(553,349)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(349,349)=jac(349,349)-k(8);
jac(37,349)=jac(37,349)+k(8);
jac(185,349)=jac(185,349)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(349,349)=jac(349,349)-k(6);
jac(184,349)=jac(184,349)+k(6);
jac(554,349)=jac(554,349)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(351,351)=jac(351,351)-k(33);
jac(354,351)=jac(354,351)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(351,351)=jac(351,351)-k(32);
jac(106,351)=jac(106,351)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(351,351)=jac(351,351)-k(31);
jac(450,351)=jac(450,351)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(351,351)=jac(351,351)-k(22);
jac(32,351)=jac(32,351)+k(22);
jac(610,351)=jac(610,351)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(351);
jac(197,351)=jac(197,351)-k(21)*y(197);
jac(351,197)=jac(351,197)-k(21)*y(351);
jac(351,351)=jac(351,351)-k(21)*y(197);
jac(559,197)=jac(559,197)+k(21)*y(351);
jac(559,351)=jac(559,351)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(351);
jac(32,351)=jac(32,351)-k(21)*y(32);
jac(351,32)=jac(351,32)-k(21)*y(351);
jac(351,351)=jac(351,351)-k(21)*y(32);
jac(612,32)=jac(612,32)+k(21)*y(351);
jac(612,351)=jac(612,351)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(351);
jac(2,351)=jac(2,351)-k(21)*y(2);
jac(351,2)=jac(351,2)-k(21)*y(351);
jac(351,351)=jac(351,351)-k(21)*y(2);
jac(575,2)=jac(575,2)+k(21)*y(351);
jac(575,351)=jac(575,351)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(351);
jac(3,351)=jac(3,351)-k(15)*y(3);
jac(351,3)=jac(351,3)-k(15)*y(351);
jac(351,351)=jac(351,351)-k(15)*y(3);
jac(556,3)=jac(556,3)+k(15)*y(351);
jac(556,351)=jac(556,351)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(351);
jac(3,351)=jac(3,351)-k(9)*y(3);
jac(351,3)=jac(351,3)-k(9)*y(351);
jac(351,351)=jac(351,351)-k(9)*y(3);
jac(350,3)=jac(350,3)+k(9)*y(351);
jac(350,351)=jac(350,351)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(351,351)=jac(351,351)-k(8);
jac(16,351)=jac(16,351)+k(8);
jac(33,351)=jac(33,351)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

jac(351,351)=jac(351,351)-k(6);
jac(15,351)=jac(15,351)+k(6);
jac(82,351)=jac(82,351)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(352,352)=jac(352,352)-k(33);
jac(116,352)=jac(116,352)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(352,352)=jac(352,352)-k(32);
jac(107,352)=jac(107,352)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(352,352)=jac(352,352)-k(22);
jac(32,352)=jac(32,352)+k(22);
jac(120,352)=jac(120,352)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(352);
jac(197,352)=jac(197,352)-k(21)*y(197);
jac(352,197)=jac(352,197)-k(21)*y(352);
jac(352,352)=jac(352,352)-k(21)*y(197);
jac(558,197)=jac(558,197)+k(21)*y(352);
jac(558,352)=jac(558,352)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(352);
jac(32,352)=jac(32,352)-k(21)*y(32);
jac(352,32)=jac(352,32)-k(21)*y(352);
jac(352,352)=jac(352,352)-k(21)*y(32);
jac(602,32)=jac(602,32)+k(21)*y(352);
jac(602,352)=jac(602,352)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(352);
jac(2,352)=jac(2,352)-k(21)*y(2);
jac(352,2)=jac(352,2)-k(21)*y(352);
jac(352,352)=jac(352,352)-k(21)*y(2);
jac(603,2)=jac(603,2)+k(21)*y(352);
jac(603,352)=jac(603,352)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(352,352)=jac(352,352)-k(16);
jac(3,352)=jac(3,352)+k(16);
jac(350,352)=jac(350,352)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(352,352)=jac(352,352)-k(10);
jac(3,352)=jac(3,352)+k(10);
jac(556,352)=jac(556,352)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(352,352)=jac(352,352)-k(8);
jac(126,352)=jac(126,352)+k(8);
jac(37,352)=jac(37,352)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(352,352)=jac(352,352)-k(6);
jac(48,352)=jac(48,352)+k(6);
jac(554,352)=jac(554,352)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(354,354)=jac(354,354)-k(32);
jac(112,354)=jac(112,354)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(354,354)=jac(354,354)-k(31);
jac(375,354)=jac(375,354)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(354,354)=jac(354,354)-k(22);
jac(2,354)=jac(2,354)+k(22);
jac(610,354)=jac(610,354)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(354);
jac(197,354)=jac(197,354)-k(21)*y(197);
jac(354,197)=jac(354,197)-k(21)*y(354);
jac(354,354)=jac(354,354)-k(21)*y(197);
jac(527,197)=jac(527,197)+k(21)*y(354);
jac(527,354)=jac(527,354)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(354);
jac(32,354)=jac(32,354)-k(21)*y(32);
jac(354,32)=jac(354,32)-k(21)*y(354);
jac(354,354)=jac(354,354)-k(21)*y(32);
jac(356,32)=jac(356,32)+k(21)*y(354);
jac(356,354)=jac(356,354)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(354);
jac(2,354)=jac(2,354)-k(21)*y(2);
jac(354,2)=jac(354,2)-k(21)*y(354);
jac(354,354)=jac(354,354)-k(21)*y(2);
jac(506,2)=jac(506,2)+k(21)*y(354);
jac(506,354)=jac(506,354)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(354);
jac(3,354)=jac(3,354)-k(15)*y(3);
jac(354,3)=jac(354,3)-k(15)*y(354);
jac(354,354)=jac(354,354)-k(15)*y(3);
jac(524,3)=jac(524,3)+k(15)*y(354);
jac(524,354)=jac(524,354)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(354);
jac(3,354)=jac(3,354)-k(9)*y(3);
jac(354,3)=jac(354,3)-k(9)*y(354);
jac(354,354)=jac(354,354)-k(9)*y(3);
jac(353,3)=jac(353,3)+k(9)*y(354);
jac(353,354)=jac(353,354)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(354,354)=jac(354,354)-k(8);
jac(16,354)=jac(16,354)+k(8);
jac(111,354)=jac(111,354)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

jac(354,354)=jac(354,354)-k(6);
jac(15,354)=jac(15,354)+k(6);
jac(110,354)=jac(110,354)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(356,356)=jac(356,356)-k(33);
jac(506,356)=jac(506,356)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(356,356)=jac(356,356)-k(31);
jac(378,356)=jac(378,356)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(356,356)=jac(356,356)-k(28);
jac(359,356)=jac(359,356)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(356,356)=jac(356,356)-k(27);
jac(527,356)=jac(527,356)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(356,356)=jac(356,356)-k(22);
jac(2,356)=jac(2,356)+k(22);
jac(533,356)=jac(533,356)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(356,356)=jac(356,356)-k(22);
jac(32,356)=jac(32,356)+k(22);
jac(354,356)=jac(354,356)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(356);
jac(3,356)=jac(3,356)-k(15)*y(3);
jac(356,3)=jac(356,3)-k(15)*y(356);
jac(356,356)=jac(356,356)-k(15)*y(3);
jac(530,3)=jac(530,3)+k(15)*y(356);
jac(530,356)=jac(530,356)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(356);
jac(3,356)=jac(3,356)-k(9)*y(3);
jac(356,3)=jac(356,3)-k(9)*y(356);
jac(356,356)=jac(356,356)-k(9)*y(3);
jac(355,3)=jac(355,3)+k(9)*y(356);
jac(355,356)=jac(355,356)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(356,356)=jac(356,356)-k(8);
jac(111,356)=jac(111,356)+k(8);
jac(28,356)=jac(28,356)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(356,356)=jac(356,356)-k(6);
jac(27,356)=jac(27,356)+k(6);
jac(110,356)=jac(110,356)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(357,357)=jac(357,357)-k(33);
jac(604,357)=jac(604,357)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(357,357)=jac(357,357)-k(28);
jac(360,357)=jac(360,357)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(357,357)=jac(357,357)-k(27);
jac(526,357)=jac(526,357)+k(27);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(357,357)=jac(357,357)-k(24);
jac(602,357)=jac(602,357)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

jac(357,357)=jac(357,357)-k(22);
jac(2,357)=jac(2,357)+k(22);
jac(134,357)=jac(134,357)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(357,357)=jac(357,357)-k(22);
jac(32,357)=jac(32,357)+k(22);
jac(116,357)=jac(116,357)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(357,357)=jac(357,357)-k(16);
jac(3,357)=jac(3,357)+k(16);
jac(355,357)=jac(355,357)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(357,357)=jac(357,357)-k(10);
jac(3,357)=jac(3,357)+k(10);
jac(530,357)=jac(530,357)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(357,357)=jac(357,357)-k(8);
jac(43,357)=jac(43,357)+k(8);
jac(132,357)=jac(132,357)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(357,357)=jac(357,357)-k(6);
jac(131,357)=jac(131,357)+k(6);
jac(114,357)=jac(114,357)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(359,359)=jac(359,359)-k(34);
jac(356,359)=jac(356,359)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(359,359)=jac(359,359)-k(33);
jac(510,359)=jac(510,359)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(359,359)=jac(359,359)-k(31);
jac(366,359)=jac(366,359)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(359,359)=jac(359,359)-k(29);
jac(611,359)=jac(611,359)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(359,359)=jac(359,359)-k(22);
jac(182,359)=jac(182,359)+k(22);
jac(533,359)=jac(533,359)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(359,359)=jac(359,359)-k(22);
jac(32,359)=jac(32,359)+k(22);
jac(512,359)=jac(512,359)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(359);
jac(3,359)=jac(3,359)-k(15)*y(3);
jac(359,3)=jac(359,3)-k(15)*y(359);
jac(359,359)=jac(359,359)-k(15)*y(3);
jac(532,3)=jac(532,3)+k(15)*y(359);
jac(532,359)=jac(532,359)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(359);
jac(3,359)=jac(3,359)-k(9)*y(3);
jac(359,3)=jac(359,3)-k(9)*y(359);
jac(359,359)=jac(359,359)-k(9)*y(3);
jac(358,3)=jac(358,3)+k(9)*y(359);
jac(358,359)=jac(358,359)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(359,359)=jac(359,359)-k(8);
jac(218,359)=jac(218,359)+k(8);
jac(28,359)=jac(28,359)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(359,359)=jac(359,359)-k(6);
jac(27,359)=jac(27,359)+k(6);
jac(264,359)=jac(264,359)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(360,360)=jac(360,360)-k(34);
jac(357,360)=jac(357,360)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(360,360)=jac(360,360)-k(33);
jac(596,360)=jac(596,360)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(360,360)=jac(360,360)-k(29);
jac(597,360)=jac(597,360)+k(29);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(360,360)=jac(360,360)-k(24);
jac(600,360)=jac(600,360)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

jac(360,360)=jac(360,360)-k(22);
jac(182,360)=jac(182,360)+k(22);
jac(134,360)=jac(134,360)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(360,360)=jac(360,360)-k(22);
jac(32,360)=jac(32,360)+k(22);
jac(595,360)=jac(595,360)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(360,360)=jac(360,360)-k(16);
jac(3,360)=jac(3,360)+k(16);
jac(358,360)=jac(358,360)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(360,360)=jac(360,360)-k(10);
jac(3,360)=jac(3,360)+k(10);
jac(532,360)=jac(532,360)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(360,360)=jac(360,360)-k(8);
jac(222,360)=jac(222,360)+k(8);
jac(132,360)=jac(132,360)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(360,360)=jac(360,360)-k(6);
jac(131,360)=jac(131,360)+k(6);
jac(515,360)=jac(515,360)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(361,361)=jac(361,361)-k(34);
jac(353,361)=jac(353,361)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(361,361)=jac(361,361)-k(32);
jac(594,361)=jac(594,361)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(361,361)=jac(361,361)-k(31);
jac(181,361)=jac(181,361)+k(31);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(361,361)=jac(361,361)-k(24);
jac(599,361)=jac(599,361)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(361,361)=jac(361,361)-k(22);
jac(182,361)=jac(182,361)+k(22);
jac(151,361)=jac(151,361)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(361);
jac(197,361)=jac(197,361)-k(21)*y(197);
jac(361,197)=jac(361,197)-k(21)*y(361);
jac(361,361)=jac(361,361)-k(21)*y(197);
jac(363,197)=jac(363,197)+k(21)*y(361);
jac(363,361)=jac(363,361)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(361);
jac(32,361)=jac(32,361)-k(21)*y(32);
jac(361,32)=jac(361,32)-k(21)*y(361);
jac(361,361)=jac(361,361)-k(21)*y(32);
jac(358,32)=jac(358,32)+k(21)*y(361);
jac(358,361)=jac(358,361)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(361);
jac(2,361)=jac(2,361)-k(21)*y(2);
jac(361,2)=jac(361,2)-k(21)*y(361);
jac(361,361)=jac(361,361)-k(21)*y(2);
jac(180,2)=jac(180,2)+k(21)*y(361);
jac(180,361)=jac(180,361)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(361);
jac(3,361)=jac(3,361)-k(15)*y(3);
jac(361,3)=jac(361,3)-k(15)*y(361);
jac(361,361)=jac(361,361)-k(15)*y(3);
jac(595,3)=jac(595,3)+k(15)*y(361);
jac(595,361)=jac(595,361)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(361,361)=jac(361,361)-k(10);
jac(3,361)=jac(3,361)+k(10);
jac(512,361)=jac(512,361)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(361,361)=jac(361,361)-k(8);
jac(126,361)=jac(126,361)+k(8);
jac(218,361)=jac(218,361)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(361,361)=jac(361,361)-k(6);
jac(48,361)=jac(48,361)+k(6);
jac(264,361)=jac(264,361)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(362,362)=jac(362,362)-k(34);
jac(161,362)=jac(161,362)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(362,362)=jac(362,362)-k(33);
jac(358,362)=jac(358,362)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(362,362)=jac(362,362)-k(33);
jac(605,362)=jac(605,362)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(362,362)=jac(362,362)-k(31);
jac(163,362)=jac(163,362)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(362,362)=jac(362,362)-k(29);
jac(342,362)=jac(342,362)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(362,362)=jac(362,362)-k(22);
jac(197,362)=jac(197,362)+k(22);
jac(137,362)=jac(137,362)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(362,362)=jac(362,362)-k(22);
jac(32,362)=jac(32,362)+k(22);
jac(599,362)=jac(599,362)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(362);
jac(3,362)=jac(3,362)-k(15)*y(3);
jac(362,3)=jac(362,3)-k(15)*y(362);
jac(362,362)=jac(362,362)-k(15)*y(3);
jac(600,3)=jac(600,3)+k(15)*y(362);
jac(600,362)=jac(600,362)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(362,362)=jac(362,362)-k(10);
jac(3,362)=jac(3,362)+k(10);
jac(608,362)=jac(608,362)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(362,362)=jac(362,362)-k(8);
jac(257,362)=jac(257,362)+k(8);
jac(132,362)=jac(132,362)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(362,362)=jac(362,362)-k(6);
jac(256,362)=jac(256,362)+k(6);
jac(131,362)=jac(131,362)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(363,363)=jac(363,363)-k(34);
jac(340,363)=jac(340,363)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(363,363)=jac(363,363)-k(34);
jac(358,363)=jac(358,363)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(363,363)=jac(363,363)-k(33);
jac(188,363)=jac(188,363)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(363,363)=jac(363,363)-k(31);
jac(367,363)=jac(367,363)+k(31);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(363,363)=jac(363,363)-k(24);
jac(342,363)=jac(342,363)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(363,363)=jac(363,363)-k(22);
jac(182,363)=jac(182,363)+k(22);
jac(337,363)=jac(337,363)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(363,363)=jac(363,363)-k(22);
jac(197,363)=jac(197,363)+k(22);
jac(361,363)=jac(361,363)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(363);
jac(3,363)=jac(3,363)-k(15)*y(3);
jac(363,3)=jac(363,3)-k(15)*y(363);
jac(363,363)=jac(363,363)-k(15)*y(3);
jac(597,3)=jac(597,3)+k(15)*y(363);
jac(597,363)=jac(597,363)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(363,363)=jac(363,363)-k(10);
jac(3,363)=jac(3,363)+k(10);
jac(611,363)=jac(611,363)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(363,363)=jac(363,363)-k(8);
jac(218,363)=jac(218,363)+k(8);
jac(336,363)=jac(336,363)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(363,363)=jac(363,363)-k(6);
jac(264,363)=jac(264,363)+k(6);
jac(335,363)=jac(335,363)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(365,365)=jac(365,365)-k(34);
jac(381,365)=jac(381,365)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(365,365)=jac(365,365)-k(33);
jac(178,365)=jac(178,365)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(365,365)=jac(365,365)-k(29);
jac(368,365)=jac(368,365)+k(29);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(365,365)=jac(365,365)-k(24);
jac(465,365)=jac(465,365)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(365,365)=jac(365,365)-k(22);
jac(182,365)=jac(182,365)+k(22);
jac(155,365)=jac(155,365)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(365,365)=jac(365,365)-k(22);
jac(32,365)=jac(32,365)+k(22);
jac(651,365)=jac(651,365)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(365,365)=jac(365,365)-k(10);
jac(3,365)=jac(3,365)+k(10);
jac(364,365)=jac(364,365)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(365,365)=jac(365,365)-k(10);
jac(3,365)=jac(3,365)+k(10);
jac(385,365)=jac(385,365)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(365,365)=jac(365,365)-k(8);
jac(184,365)=jac(184,365)+k(8);
jac(132,365)=jac(132,365)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(365,365)=jac(365,365)-k(6);
jac(131,365)=jac(131,365)+k(6);
jac(391,365)=jac(391,365)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(366,366)=jac(366,366)-k(34);
jac(378,366)=jac(378,366)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(366,366)=jac(366,366)-k(33);
jac(179,366)=jac(179,366)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(366,366)=jac(366,366)-k(29);
jac(369,366)=jac(369,366)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(366,366)=jac(366,366)-k(22);
jac(182,366)=jac(182,366)+k(22);
jac(156,366)=jac(156,366)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(366,366)=jac(366,366)-k(22);
jac(32,366)=jac(32,366)+k(22);
jac(648,366)=jac(648,366)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(366);
jac(3,366)=jac(3,366)-k(9)*y(3);
jac(366,3)=jac(366,3)-k(9)*y(366);
jac(366,366)=jac(366,366)-k(9)*y(3);
jac(364,3)=jac(364,3)+k(9)*y(366);
jac(364,366)=jac(364,366)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(366);
jac(3,366)=jac(3,366)-k(9)*y(3);
jac(366,3)=jac(366,3)-k(9)*y(366);
jac(366,366)=jac(366,366)-k(9)*y(3);
jac(385,3)=jac(385,3)+k(9)*y(366);
jac(385,366)=jac(385,366)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(366,366)=jac(366,366)-k(8);
jac(177,366)=jac(177,366)+k(8);
jac(28,366)=jac(28,366)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(366,366)=jac(366,366)-k(6);
jac(27,366)=jac(27,366)+k(6);
jac(176,366)=jac(176,366)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(368,368)=jac(368,368)-k(34);
jac(372,368)=jac(372,368)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(368,368)=jac(368,368)-k(34);
jac(365,368)=jac(365,368)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(368,368)=jac(368,368)-k(33);
jac(186,368)=jac(186,368)+k(33);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(368,368)=jac(368,368)-k(24);
jac(464,368)=jac(464,368)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(368,368)=jac(368,368)-k(22);
jac(182,368)=jac(182,368)+k(22);
jac(400,368)=jac(400,368)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(368,368)=jac(368,368)-k(22);
jac(197,368)=jac(197,368)+k(22);
jac(651,368)=jac(651,368)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(368,368)=jac(368,368)-k(10);
jac(3,368)=jac(3,368)+k(10);
jac(367,368)=jac(367,368)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(368,368)=jac(368,368)-k(10);
jac(3,368)=jac(3,368)+k(10);
jac(397,368)=jac(397,368)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(368,368)=jac(368,368)-k(8);
jac(184,368)=jac(184,368)+k(8);
jac(336,368)=jac(336,368)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(368,368)=jac(368,368)-k(6);
jac(335,368)=jac(335,368)+k(6);
jac(391,368)=jac(391,368)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(369,369)=jac(369,369)-k(34);
jac(373,369)=jac(373,369)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(369,369)=jac(369,369)-k(34);
jac(366,369)=jac(366,369)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(369,369)=jac(369,369)-k(33);
jac(187,369)=jac(187,369)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(369,369)=jac(369,369)-k(22);
jac(182,369)=jac(182,369)+k(22);
jac(376,369)=jac(376,369)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(369,369)=jac(369,369)-k(22);
jac(197,369)=jac(197,369)+k(22);
jac(648,369)=jac(648,369)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(369);
jac(3,369)=jac(3,369)-k(9)*y(3);
jac(369,3)=jac(369,3)-k(9)*y(369);
jac(369,369)=jac(369,369)-k(9)*y(3);
jac(367,3)=jac(367,3)+k(9)*y(369);
jac(367,369)=jac(367,369)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(369);
jac(3,369)=jac(3,369)-k(9)*y(3);
jac(369,3)=jac(369,3)-k(9)*y(369);
jac(369,369)=jac(369,369)-k(9)*y(3);
jac(397,3)=jac(397,3)+k(9)*y(369);
jac(397,369)=jac(397,369)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(369,369)=jac(369,369)-k(8);
jac(177,369)=jac(177,369)+k(8);
jac(209,369)=jac(209,369)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(369,369)=jac(369,369)-k(6);
jac(192,369)=jac(192,369)+k(6);
jac(176,369)=jac(176,369)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(370,370)=jac(370,370)-k(34);
jac(162,370)=jac(162,370)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(370,370)=jac(370,370)-k(34);
jac(163,370)=jac(163,370)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(370,370)=jac(370,370)-k(33);
jac(367,370)=jac(367,370)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(370,370)=jac(370,370)-k(33);
jac(190,370)=jac(190,370)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(370,370)=jac(370,370)-k(22);
jac(197,370)=jac(197,370)+k(22);
jac(198,370)=jac(198,370)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(370,370)=jac(370,370)-k(22);
jac(197,370)=jac(197,370)+k(22);
jac(196,370)=jac(196,370)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(370,370)=jac(370,370)-k(12);
jac(342,370)=jac(342,370)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(370,370)=jac(370,370)-k(10);
jac(3,370)=jac(3,370)+k(10);
jac(455,370)=jac(455,370)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(370);
jac(3,370)=jac(3,370)-k(9)*y(3);
jac(370,3)=jac(370,3)-k(9)*y(370);
jac(370,370)=jac(370,370)-k(9)*y(3);
jac(464,3)=jac(464,3)+k(9)*y(370);
jac(464,370)=jac(464,370)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(370,370)=jac(370,370)-k(8);
jac(192,370)=jac(192,370)+k(8);
jac(336,370)=jac(336,370)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(370,370)=jac(370,370)-k(6);
jac(191,370)=jac(191,370)+k(6);
jac(335,370)=jac(335,370)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(372,372)=jac(372,372)-k(34);
jac(381,372)=jac(381,372)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(372,372)=jac(372,372)-k(33);
jac(460,372)=jac(460,372)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(372,372)=jac(372,372)-k(30);
jac(368,372)=jac(368,372)+k(30);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(372,372)=jac(372,372)-k(24);
jac(459,372)=jac(459,372)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(372,372)=jac(372,372)-k(22);
jac(2,372)=jac(2,372)+k(22);
jac(400,372)=jac(400,372)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(372,372)=jac(372,372)-k(22);
jac(197,372)=jac(197,372)+k(22);
jac(461,372)=jac(461,372)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(372,372)=jac(372,372)-k(10);
jac(3,372)=jac(3,372)+k(10);
jac(371,372)=jac(371,372)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(372,372)=jac(372,372)-k(10);
jac(3,372)=jac(3,372)+k(10);
jac(374,372)=jac(374,372)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(372,372)=jac(372,372)-k(8);
jac(140,372)=jac(140,372)+k(8);
jac(336,372)=jac(336,372)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(372,372)=jac(372,372)-k(6);
jac(335,372)=jac(335,372)+k(6);
jac(380,372)=jac(380,372)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(374,374)=jac(374,374)-k(34);
jac(379,374)=jac(379,374)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(374,374)=jac(374,374)-k(33);
jac(389,374)=jac(389,374)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(374,374)=jac(374,374)-k(30);
jac(397,374)=jac(397,374)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(374,374)=jac(374,374)-k(22);
jac(2,374)=jac(2,374)+k(22);
jac(399,374)=jac(399,374)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(374,374)=jac(374,374)-k(22);
jac(197,374)=jac(197,374)+k(22);
jac(383,374)=jac(383,374)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(374,374)=jac(374,374)-k(11);
jac(426,374)=jac(426,374)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(374,374)=jac(374,374)-k(10);
jac(3,374)=jac(3,374)+k(10);
jac(373,374)=jac(373,374)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(374);
jac(3,374)=jac(3,374)-k(9)*y(3);
jac(374,3)=jac(374,3)-k(9)*y(374);
jac(374,374)=jac(374,374)-k(9)*y(3);
jac(372,3)=jac(372,3)+k(9)*y(374);
jac(372,374)=jac(372,374)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(374,374)=jac(374,374)-k(8);
jac(209,374)=jac(209,374)+k(8);
jac(140,374)=jac(140,374)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(374,374)=jac(374,374)-k(6);
jac(192,374)=jac(192,374)+k(6);
jac(380,374)=jac(380,374)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(375,375)=jac(375,375)-k(32);
jac(94,375)=jac(94,375)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(375,375)=jac(375,375)-k(22);
jac(2,375)=jac(2,375)+k(22);
jac(150,375)=jac(150,375)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(375);
jac(197,375)=jac(197,375)-k(21)*y(197);
jac(375,197)=jac(375,197)-k(21)*y(375);
jac(375,375)=jac(375,375)-k(21)*y(197);
jac(373,197)=jac(373,197)+k(21)*y(375);
jac(373,375)=jac(373,375)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(375);
jac(32,375)=jac(32,375)-k(21)*y(32);
jac(375,32)=jac(375,32)-k(21)*y(375);
jac(375,375)=jac(375,375)-k(21)*y(32);
jac(378,32)=jac(378,32)+k(21)*y(375);
jac(378,375)=jac(378,375)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(375);
jac(2,375)=jac(2,375)-k(21)*y(2);
jac(375,2)=jac(375,2)-k(21)*y(375);
jac(375,375)=jac(375,375)-k(21)*y(2);
jac(172,2)=jac(172,2)+k(21)*y(375);
jac(172,375)=jac(172,375)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(375);
jac(3,375)=jac(3,375)-k(9)*y(3);
jac(375,3)=jac(375,3)-k(9)*y(375);
jac(375,375)=jac(375,375)-k(9)*y(3);
jac(152,3)=jac(152,3)+k(9)*y(375);
jac(152,375)=jac(152,375)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(375);
jac(3,375)=jac(3,375)-k(9)*y(3);
jac(375,3)=jac(375,3)-k(9)*y(375);
jac(375,375)=jac(375,375)-k(9)*y(3);
jac(383,3)=jac(383,3)+k(9)*y(375);
jac(383,375)=jac(383,375)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(375,375)=jac(375,375)-k(8);
jac(16,375)=jac(16,375)+k(8);
jac(21,375)=jac(21,375)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

jac(375,375)=jac(375,375)-k(6);
jac(15,375)=jac(15,375)+k(6);
jac(89,375)=jac(89,375)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(376,376)=jac(376,376)-k(34);
jac(156,376)=jac(156,376)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(376,376)=jac(376,376)-k(33);
jac(451,376)=jac(451,376)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(376,376)=jac(376,376)-k(32);
jac(211,376)=jac(211,376)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(376,376)=jac(376,376)-k(22);
jac(197,376)=jac(197,376)+k(22);
jac(150,376)=jac(150,376)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(376);
jac(197,376)=jac(197,376)-k(21)*y(197);
jac(376,197)=jac(376,197)-k(21)*y(376);
jac(376,376)=jac(376,376)-k(21)*y(197);
jac(455,197)=jac(455,197)+k(21)*y(376);
jac(455,376)=jac(455,376)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(376);
jac(32,376)=jac(32,376)-k(21)*y(32);
jac(376,32)=jac(376,32)-k(21)*y(376);
jac(376,376)=jac(376,376)-k(21)*y(32);
jac(454,32)=jac(454,32)+k(21)*y(376);
jac(454,376)=jac(454,376)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(376);
jac(2,376)=jac(2,376)-k(21)*y(2);
jac(376,2)=jac(376,2)-k(21)*y(376);
jac(376,376)=jac(376,376)-k(21)*y(2);
jac(373,2)=jac(373,2)+k(21)*y(376);
jac(373,376)=jac(373,376)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(376);
jac(3,376)=jac(3,376)-k(9)*y(3);
jac(376,3)=jac(376,3)-k(9)*y(376);
jac(376,376)=jac(376,376)-k(9)*y(3);
jac(198,3)=jac(198,3)+k(9)*y(376);
jac(198,376)=jac(198,376)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(376);
jac(3,376)=jac(3,376)-k(9)*y(3);
jac(376,3)=jac(376,3)-k(9)*y(376);
jac(376,376)=jac(376,376)-k(9)*y(3);
jac(399,3)=jac(399,3)+k(9)*y(376);
jac(399,376)=jac(399,376)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

jac(376,376)=jac(376,376)-k(8);
jac(15,376)=jac(15,376)+k(8);
jac(209,376)=jac(209,376)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(376,376)=jac(376,376)-k(6);
jac(98,376)=jac(98,376)+k(6);
jac(192,376)=jac(192,376)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(377,377)=jac(377,377)-k(34);
jac(172,377)=jac(172,377)+k(34);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(377,377)=jac(377,377)-k(30);
jac(187,377)=jac(187,377)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(377,377)=jac(377,377)-k(22);
jac(2,377)=jac(2,377)+k(22);
jac(451,377)=jac(451,377)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(377,377)=jac(377,377)-k(22);
jac(182,377)=jac(182,377)+k(22);
jac(375,377)=jac(375,377)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(377);
jac(3,377)=jac(3,377)-k(9)*y(3);
jac(377,3)=jac(377,3)-k(9)*y(377);
jac(377,377)=jac(377,377)-k(9)*y(3);
jac(174,3)=jac(174,3)+k(9)*y(377);
jac(174,377)=jac(174,377)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(377);
jac(3,377)=jac(3,377)-k(9)*y(3);
jac(377,3)=jac(377,3)-k(9)*y(377);
jac(377,377)=jac(377,377)-k(9)*y(3);
jac(389,3)=jac(389,3)+k(9)*y(377);
jac(389,377)=jac(389,377)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(377,377)=jac(377,377)-k(8);
jac(21,377)=jac(21,377)+k(8);
jac(214,377)=jac(214,377)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(377,377)=jac(377,377)-k(6);
jac(177,377)=jac(177,377)+k(6);
jac(89,377)=jac(89,377)+k(6);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(380,380)=jac(380,380)-k(22);
jac(2,380)=jac(2,380)+k(22);
jac(47,380)=jac(47,380)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(380,380)=jac(380,380)-k(10);
jac(3,380)=jac(3,380)+k(10);
jac(89,380)=jac(89,380)+k(10);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(1,1)=jac(1,1)-k(5)*y(380);
jac(1,380)=jac(1,380)-k(5)*y(1);
jac(380,1)=jac(380,1)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(1);
jac(95,1)=jac(95,1)+k(5)*y(380);
jac(95,380)=jac(95,380)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(5)*y(380);
jac(7,380)=jac(7,380)-k(5)*y(7);
jac(380,7)=jac(380,7)-k(5)*y(380);
jac(380,380)=jac(380,380)-k(5)*y(7);
jac(462,7)=jac(462,7)+k(5)*y(380);
jac(462,380)=jac(462,380)+k(5)*y(7);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Lig(l1, l2) 

jac(380,380)=jac(380,380)-k(4);
jac(4,380)=jac(4,380)+k(4);
jac(140,380)=jac(140,380)+k(4);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(381,381)=jac(381,381)-k(33);
jac(171,381)=jac(171,381)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(381,381)=jac(381,381)-k(28);
jac(365,381)=jac(365,381)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(381,381)=jac(381,381)-k(27);
jac(372,381)=jac(372,381)+k(27);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(381,381)=jac(381,381)-k(24);
jac(159,381)=jac(159,381)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(381,381)=jac(381,381)-k(22);
jac(32,381)=jac(32,381)+k(22);
jac(461,381)=jac(461,381)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(381,381)=jac(381,381)-k(22);
jac(2,381)=jac(2,381)+k(22);
jac(155,381)=jac(155,381)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(381,381)=jac(381,381)-k(10);
jac(3,381)=jac(3,381)+k(10);
jac(379,381)=jac(379,381)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(381,381)=jac(381,381)-k(10);
jac(3,381)=jac(3,381)+k(10);
jac(157,381)=jac(157,381)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(381,381)=jac(381,381)-k(8);
jac(140,381)=jac(140,381)+k(8);
jac(132,381)=jac(132,381)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(381,381)=jac(381,381)-k(6);
jac(131,381)=jac(131,381)+k(6);
jac(380,381)=jac(380,381)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(382,382)=jac(382,382)-k(33);
jac(387,382)=jac(387,382)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(382,382)=jac(382,382)-k(31);
jac(379,382)=jac(379,382)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(382,382)=jac(382,382)-k(28);
jac(423,382)=jac(423,382)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(382,382)=jac(382,382)-k(27);
jac(426,382)=jac(426,382)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(382,382)=jac(382,382)-k(22);
jac(2,382)=jac(2,382)+k(22);
jac(470,382)=jac(470,382)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(382,382)=jac(382,382)-k(22);
jac(32,382)=jac(32,382)+k(22);
jac(424,382)=jac(424,382)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(382);
jac(3,382)=jac(3,382)-k(15)*y(3);
jac(382,3)=jac(382,3)-k(15)*y(382);
jac(382,382)=jac(382,382)-k(15)*y(3);
jac(480,3)=jac(480,3)+k(15)*y(382);
jac(480,382)=jac(480,382)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(382,382)=jac(382,382)-k(10);
jac(3,382)=jac(3,382)+k(10);
jac(482,382)=jac(482,382)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(382,382)=jac(382,382)-k(8);
jac(34,382)=jac(34,382)+k(8);
jac(140,382)=jac(140,382)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(382,382)=jac(382,382)-k(6);
jac(33,382)=jac(33,382)+k(6);
jac(380,382)=jac(380,382)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(383,383)=jac(383,383)-k(32);
jac(95,383)=jac(95,383)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(383,383)=jac(383,383)-k(22);
jac(2,383)=jac(2,383)+k(22);
jac(404,383)=jac(404,383)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(383);
jac(197,383)=jac(197,383)-k(21)*y(197);
jac(383,197)=jac(383,197)-k(21)*y(383);
jac(383,383)=jac(383,383)-k(21)*y(197);
jac(374,197)=jac(374,197)+k(21)*y(383);
jac(374,383)=jac(374,383)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(383);
jac(32,383)=jac(32,383)-k(21)*y(32);
jac(383,32)=jac(383,32)-k(21)*y(383);
jac(383,383)=jac(383,383)-k(21)*y(32);
jac(379,32)=jac(379,32)+k(21)*y(383);
jac(379,383)=jac(379,383)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(383);
jac(2,383)=jac(2,383)-k(21)*y(2);
jac(383,2)=jac(383,2)-k(21)*y(383);
jac(383,383)=jac(383,383)-k(21)*y(2);
jac(386,2)=jac(386,2)+k(21)*y(383);
jac(386,383)=jac(386,383)+k(21)*y(2);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(383,383)=jac(383,383)-k(11);
jac(424,383)=jac(424,383)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(383,383)=jac(383,383)-k(10);
jac(3,383)=jac(3,383)+k(10);
jac(375,383)=jac(375,383)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(383);
jac(3,383)=jac(3,383)-k(9)*y(3);
jac(383,3)=jac(383,3)-k(9)*y(383);
jac(383,383)=jac(383,383)-k(9)*y(3);
jac(461,3)=jac(461,3)+k(9)*y(383);
jac(461,383)=jac(461,383)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(383,383)=jac(383,383)-k(8);
jac(16,383)=jac(16,383)+k(8);
jac(140,383)=jac(140,383)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

jac(383,383)=jac(383,383)-k(6);
jac(15,383)=jac(15,383)+k(6);
jac(380,383)=jac(380,383)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(384,384)=jac(384,384)-k(33);
jac(388,384)=jac(388,384)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(384,384)=jac(384,384)-k(32);
jac(26,384)=jac(26,384)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(384,384)=jac(384,384)-k(22);
jac(32,384)=jac(32,384)+k(22);
jac(404,384)=jac(404,384)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(384);
jac(197,384)=jac(197,384)-k(21)*y(197);
jac(384,197)=jac(384,197)-k(21)*y(384);
jac(384,384)=jac(384,384)-k(21)*y(197);
jac(412,197)=jac(412,197)+k(21)*y(384);
jac(412,384)=jac(412,384)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(384);
jac(32,384)=jac(32,384)-k(21)*y(32);
jac(384,32)=jac(384,32)-k(21)*y(384);
jac(384,384)=jac(384,384)-k(21)*y(32);
jac(446,32)=jac(446,32)+k(21)*y(384);
jac(446,384)=jac(446,384)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(384);
jac(2,384)=jac(2,384)-k(21)*y(2);
jac(384,2)=jac(384,2)-k(21)*y(384);
jac(384,384)=jac(384,384)-k(21)*y(2);
jac(379,2)=jac(379,2)+k(21)*y(384);
jac(379,384)=jac(379,384)+k(21)*y(2);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(384,384)=jac(384,384)-k(11);
jac(470,384)=jac(470,384)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(384,384)=jac(384,384)-k(10);
jac(3,384)=jac(3,384)+k(10);
jac(156,384)=jac(156,384)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(384);
jac(3,384)=jac(3,384)-k(9)*y(3);
jac(384,3)=jac(384,3)-k(9)*y(384);
jac(384,384)=jac(384,384)-k(9)*y(3);
jac(155,3)=jac(155,3)+k(9)*y(384);
jac(155,384)=jac(155,384)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(384,384)=jac(384,384)-k(8);
jac(48,384)=jac(48,384)+k(8);
jac(28,384)=jac(28,384)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(384,384)=jac(384,384)-k(6);
jac(47,384)=jac(47,384)+k(6);
jac(27,384)=jac(27,384)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(385,385)=jac(385,385)-k(34);
jac(379,385)=jac(379,385)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(385,385)=jac(385,385)-k(33);
jac(390,385)=jac(390,385)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(385,385)=jac(385,385)-k(29);
jac(397,385)=jac(397,385)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(385,385)=jac(385,385)-k(22);
jac(182,385)=jac(182,385)+k(22);
jac(384,385)=jac(384,385)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(385,385)=jac(385,385)-k(22);
jac(32,385)=jac(32,385)+k(22);
jac(393,385)=jac(393,385)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(385,385)=jac(385,385)-k(11);
jac(423,385)=jac(423,385)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(385,385)=jac(385,385)-k(10);
jac(3,385)=jac(3,385)+k(10);
jac(366,385)=jac(366,385)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(385);
jac(3,385)=jac(3,385)-k(9)*y(3);
jac(385,3)=jac(385,3)-k(9)*y(385);
jac(385,385)=jac(385,385)-k(9)*y(3);
jac(365,3)=jac(365,3)+k(9)*y(385);
jac(365,385)=jac(365,385)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(385,385)=jac(385,385)-k(8);
jac(28,385)=jac(28,385)+k(8);
jac(184,385)=jac(184,385)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(385,385)=jac(385,385)-k(6);
jac(27,385)=jac(27,385)+k(6);
jac(391,385)=jac(391,385)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(387,387)=jac(387,387)-k(31);
jac(386,387)=jac(386,387)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(387,387)=jac(387,387)-k(28);
jac(392,387)=jac(392,387)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(387,387)=jac(387,387)-k(27);
jac(429,387)=jac(429,387)+k(27);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(387,387)=jac(387,387)-k(23);
jac(382,387)=jac(382,387)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(387,387)=jac(387,387)-k(22);
jac(2,387)=jac(2,387)+k(22);
jac(50,387)=jac(50,387)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(387,387)=jac(387,387)-k(22);
jac(2,387)=jac(2,387)+k(22);
jac(424,387)=jac(424,387)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(387);
jac(3,387)=jac(3,387)-k(15)*y(3);
jac(387,3)=jac(387,3)-k(15)*y(387);
jac(387,387)=jac(387,387)-k(15)*y(3);
jac(51,3)=jac(51,3)+k(15)*y(387);
jac(51,387)=jac(51,387)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(387,387)=jac(387,387)-k(10);
jac(3,387)=jac(3,387)+k(10);
jac(275,387)=jac(275,387)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(387,387)=jac(387,387)-k(8);
jac(437,387)=jac(437,387)+k(8);
jac(140,387)=jac(140,387)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(387,387)=jac(387,387)-k(6);
jac(111,387)=jac(111,387)+k(6);
jac(380,387)=jac(380,387)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(388,388)=jac(388,388)-k(32);
jac(20,388)=jac(20,388)+k(32);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(388,388)=jac(388,388)-k(23);
jac(384,388)=jac(384,388)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(388,388)=jac(388,388)-k(22);
jac(2,388)=jac(2,388)+k(22);
jac(404,388)=jac(404,388)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(388);
jac(197,388)=jac(197,388)-k(21)*y(197);
jac(388,197)=jac(388,197)-k(21)*y(388);
jac(388,388)=jac(388,388)-k(21)*y(197);
jac(411,197)=jac(411,197)+k(21)*y(388);
jac(411,388)=jac(411,388)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(388);
jac(32,388)=jac(32,388)-k(21)*y(32);
jac(388,32)=jac(388,32)-k(21)*y(388);
jac(388,388)=jac(388,388)-k(21)*y(32);
jac(445,32)=jac(445,32)+k(21)*y(388);
jac(445,388)=jac(445,388)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(388);
jac(2,388)=jac(2,388)-k(21)*y(2);
jac(388,2)=jac(388,2)-k(21)*y(388);
jac(388,388)=jac(388,388)-k(21)*y(2);
jac(386,2)=jac(386,2)+k(21)*y(388);
jac(386,388)=jac(386,388)+k(21)*y(2);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(388,388)=jac(388,388)-k(11);
jac(50,388)=jac(50,388)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(388,388)=jac(388,388)-k(10);
jac(3,388)=jac(3,388)+k(10);
jac(452,388)=jac(452,388)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(388);
jac(3,388)=jac(3,388)-k(9)*y(3);
jac(388,3)=jac(388,3)-k(9)*y(388);
jac(388,388)=jac(388,388)-k(9)*y(3);
jac(467,3)=jac(467,3)+k(9)*y(388);
jac(467,388)=jac(467,388)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(388,388)=jac(388,388)-k(8);
jac(48,388)=jac(48,388)+k(8);
jac(22,388)=jac(22,388)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(388,388)=jac(388,388)-k(6);
jac(47,388)=jac(47,388)+k(6);
jac(21,388)=jac(21,388)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(389,389)=jac(389,389)-k(34);
jac(386,389)=jac(386,389)+k(34);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(389,389)=jac(389,389)-k(30);
jac(394,389)=jac(394,389)+k(30);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(389,389)=jac(389,389)-k(23);
jac(374,389)=jac(374,389)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(389,389)=jac(389,389)-k(22);
jac(2,389)=jac(2,389)+k(22);
jac(396,389)=jac(396,389)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(389,389)=jac(389,389)-k(22);
jac(182,389)=jac(182,389)+k(22);
jac(383,389)=jac(383,389)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(389,389)=jac(389,389)-k(11);
jac(429,389)=jac(429,389)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(389,389)=jac(389,389)-k(10);
jac(3,389)=jac(3,389)+k(10);
jac(377,389)=jac(377,389)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(389);
jac(3,389)=jac(3,389)-k(9)*y(3);
jac(389,3)=jac(389,3)-k(9)*y(389);
jac(389,389)=jac(389,389)-k(9)*y(3);
jac(460,3)=jac(460,3)+k(9)*y(389);
jac(460,389)=jac(460,389)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(389,389)=jac(389,389)-k(8);
jac(214,389)=jac(214,389)+k(8);
jac(140,389)=jac(140,389)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(389,389)=jac(389,389)-k(6);
jac(177,389)=jac(177,389)+k(6);
jac(380,389)=jac(380,389)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(391,391)=jac(391,391)-k(34);
jac(380,391)=jac(380,391)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(391,391)=jac(391,391)-k(22);
jac(182,391)=jac(182,391)+k(22);
jac(47,391)=jac(47,391)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(391,391)=jac(391,391)-k(10);
jac(3,391)=jac(3,391)+k(10);
jac(176,391)=jac(176,391)+k(10);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(1,1)=jac(1,1)-k(5)*y(391);
jac(1,391)=jac(1,391)-k(5)*y(1);
jac(391,1)=jac(391,1)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(1);
jac(416,1)=jac(416,1)+k(5)*y(391);
jac(416,391)=jac(416,391)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(7,7)=jac(7,7)-k(5)*y(391);
jac(7,391)=jac(7,391)-k(5)*y(7);
jac(391,7)=jac(391,7)-k(5)*y(391);
jac(391,391)=jac(391,391)-k(5)*y(7);
jac(417,7)=jac(417,7)+k(5)*y(391);
jac(417,391)=jac(417,391)+k(5)*y(7);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) 

jac(391,391)=jac(391,391)-k(4);
jac(4,391)=jac(4,391)+k(4);
jac(184,391)=jac(184,391)+k(4);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(392,392)=jac(392,392)-k(34);
jac(387,392)=jac(387,392)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(392,392)=jac(392,392)-k(31);
jac(390,392)=jac(390,392)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(392,392)=jac(392,392)-k(29);
jac(395,392)=jac(395,392)+k(29);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(392,392)=jac(392,392)-k(23);
jac(423,392)=jac(423,392)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(392,392)=jac(392,392)-k(22);
jac(182,392)=jac(182,392)+k(22);
jac(50,392)=jac(50,392)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(392,392)=jac(392,392)-k(22);
jac(2,392)=jac(2,392)+k(22);
jac(420,392)=jac(420,392)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(392);
jac(3,392)=jac(3,392)-k(15)*y(3);
jac(392,3)=jac(392,3)-k(15)*y(392);
jac(392,392)=jac(392,392)-k(15)*y(3);
jac(270,3)=jac(270,3)+k(15)*y(392);
jac(270,392)=jac(270,392)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(392,392)=jac(392,392)-k(10);
jac(3,392)=jac(3,392)+k(10);
jac(271,392)=jac(271,392)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(392,392)=jac(392,392)-k(8);
jac(437,392)=jac(437,392)+k(8);
jac(184,392)=jac(184,392)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(392,392)=jac(392,392)-k(6);
jac(111,392)=jac(111,392)+k(6);
jac(391,392)=jac(391,392)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(393,393)=jac(393,393)-k(34);
jac(383,393)=jac(383,393)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(393,393)=jac(393,393)-k(32);
jac(416,393)=jac(416,393)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(393,393)=jac(393,393)-k(22);
jac(182,393)=jac(182,393)+k(22);
jac(404,393)=jac(404,393)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(393);
jac(197,393)=jac(197,393)-k(21)*y(197);
jac(393,197)=jac(393,197)-k(21)*y(393);
jac(393,393)=jac(393,393)-k(21)*y(197);
jac(397,197)=jac(397,197)+k(21)*y(393);
jac(397,393)=jac(397,393)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(393);
jac(32,393)=jac(32,393)-k(21)*y(32);
jac(393,32)=jac(393,32)-k(21)*y(393);
jac(393,393)=jac(393,393)-k(21)*y(32);
jac(385,32)=jac(385,32)+k(21)*y(393);
jac(385,393)=jac(385,393)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(393);
jac(2,393)=jac(2,393)-k(21)*y(2);
jac(393,2)=jac(393,2)-k(21)*y(393);
jac(393,393)=jac(393,393)-k(21)*y(2);
jac(390,2)=jac(390,2)+k(21)*y(393);
jac(390,393)=jac(390,393)+k(21)*y(2);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(393,393)=jac(393,393)-k(11);
jac(420,393)=jac(420,393)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(393,393)=jac(393,393)-k(10);
jac(3,393)=jac(3,393)+k(10);
jac(648,393)=jac(648,393)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(393);
jac(3,393)=jac(3,393)-k(9)*y(3);
jac(393,3)=jac(393,3)-k(9)*y(393);
jac(393,393)=jac(393,393)-k(9)*y(3);
jac(651,3)=jac(651,3)+k(9)*y(393);
jac(651,393)=jac(651,393)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(393,393)=jac(393,393)-k(8);
jac(16,393)=jac(16,393)+k(8);
jac(184,393)=jac(184,393)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

jac(393,393)=jac(393,393)-k(6);
jac(15,393)=jac(15,393)+k(6);
jac(391,393)=jac(391,393)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(395,395)=jac(395,395)-k(34);
jac(429,395)=jac(429,395)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(395,395)=jac(395,395)-k(34);
jac(392,395)=jac(392,395)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(395,395)=jac(395,395)-k(31);
jac(394,395)=jac(394,395)+k(31);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(395,395)=jac(395,395)-k(23);
jac(398,395)=jac(398,395)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(395,395)=jac(395,395)-k(22);
jac(182,395)=jac(182,395)+k(22);
jac(430,395)=jac(430,395)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(395,395)=jac(395,395)-k(22);
jac(182,395)=jac(182,395)+k(22);
jac(420,395)=jac(420,395)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(395);
jac(3,395)=jac(3,395)-k(15)*y(3);
jac(395,3)=jac(395,3)-k(15)*y(395);
jac(395,395)=jac(395,395)-k(15)*y(3);
jac(649,3)=jac(649,3)+k(15)*y(395);
jac(649,395)=jac(649,395)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(395,395)=jac(395,395)-k(10);
jac(3,395)=jac(3,395)+k(10);
jac(580,395)=jac(580,395)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(395,395)=jac(395,395)-k(8);
jac(219,395)=jac(219,395)+k(8);
jac(184,395)=jac(184,395)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(395,395)=jac(395,395)-k(6);
jac(218,395)=jac(218,395)+k(6);
jac(391,395)=jac(391,395)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(396,396)=jac(396,396)-k(34);
jac(388,396)=jac(388,396)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(396,396)=jac(396,396)-k(32);
jac(213,396)=jac(213,396)+k(32);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(396,396)=jac(396,396)-k(23);
jac(399,396)=jac(399,396)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(396,396)=jac(396,396)-k(22);
jac(182,396)=jac(182,396)+k(22);
jac(404,396)=jac(404,396)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(396);
jac(197,396)=jac(197,396)-k(21)*y(197);
jac(396,197)=jac(396,197)-k(21)*y(396);
jac(396,396)=jac(396,396)-k(21)*y(197);
jac(435,197)=jac(435,197)+k(21)*y(396);
jac(435,396)=jac(435,396)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(396);
jac(32,396)=jac(32,396)-k(21)*y(32);
jac(396,32)=jac(396,32)-k(21)*y(396);
jac(396,396)=jac(396,396)-k(21)*y(32);
jac(447,32)=jac(447,32)+k(21)*y(396);
jac(447,396)=jac(447,396)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(396);
jac(2,396)=jac(2,396)-k(21)*y(2);
jac(396,2)=jac(396,2)-k(21)*y(396);
jac(396,396)=jac(396,396)-k(21)*y(2);
jac(389,2)=jac(389,2)+k(21)*y(396);
jac(389,396)=jac(389,396)+k(21)*y(2);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(396,396)=jac(396,396)-k(11);
jac(430,396)=jac(430,396)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(396,396)=jac(396,396)-k(10);
jac(3,396)=jac(3,396)+k(10);
jac(451,396)=jac(451,396)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(396);
jac(3,396)=jac(3,396)-k(9)*y(3);
jac(396,3)=jac(396,3)-k(9)*y(396);
jac(396,396)=jac(396,396)-k(9)*y(3);
jac(457,3)=jac(457,3)+k(9)*y(396);
jac(457,396)=jac(457,396)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(396,396)=jac(396,396)-k(8);
jac(48,396)=jac(48,396)+k(8);
jac(214,396)=jac(214,396)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(396,396)=jac(396,396)-k(6);
jac(47,396)=jac(47,396)+k(6);
jac(177,396)=jac(177,396)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(398,398)=jac(398,398)-k(34);
jac(426,398)=jac(426,398)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(398,398)=jac(398,398)-k(34);
jac(423,398)=jac(423,398)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(398,398)=jac(398,398)-k(33);
jac(395,398)=jac(395,398)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(398,398)=jac(398,398)-k(31);
jac(397,398)=jac(397,398)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(398,398)=jac(398,398)-k(22);
jac(182,398)=jac(182,398)+k(22);
jac(401,398)=jac(401,398)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(398,398)=jac(398,398)-k(22);
jac(197,398)=jac(197,398)+k(22);
jac(420,398)=jac(420,398)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(398);
jac(3,398)=jac(3,398)-k(15)*y(3);
jac(398,3)=jac(398,3)-k(15)*y(398);
jac(398,398)=jac(398,398)-k(15)*y(3);
jac(476,3)=jac(476,3)+k(15)*y(398);
jac(476,398)=jac(476,398)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(398,398)=jac(398,398)-k(10);
jac(3,398)=jac(3,398)+k(10);
jac(487,398)=jac(487,398)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(398,398)=jac(398,398)-k(8);
jac(319,398)=jac(319,398)+k(8);
jac(184,398)=jac(184,398)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(398,398)=jac(398,398)-k(6);
jac(257,398)=jac(257,398)+k(6);
jac(391,398)=jac(391,398)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(400,400)=jac(400,400)-k(34);
jac(155,400)=jac(155,400)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(400,400)=jac(400,400)-k(33);
jac(457,400)=jac(457,400)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(400,400)=jac(400,400)-k(32);
jac(210,400)=jac(210,400)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(400,400)=jac(400,400)-k(22);
jac(197,400)=jac(197,400)+k(22);
jac(149,400)=jac(149,400)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(400);
jac(197,400)=jac(197,400)-k(21)*y(197);
jac(400,197)=jac(400,197)-k(21)*y(400);
jac(400,400)=jac(400,400)-k(21)*y(197);
jac(464,197)=jac(464,197)+k(21)*y(400);
jac(464,400)=jac(464,400)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(400);
jac(32,400)=jac(32,400)-k(21)*y(32);
jac(400,32)=jac(400,32)-k(21)*y(400);
jac(400,400)=jac(400,400)-k(21)*y(32);
jac(459,32)=jac(459,32)+k(21)*y(400);
jac(459,400)=jac(459,400)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(400);
jac(2,400)=jac(2,400)-k(21)*y(2);
jac(400,2)=jac(400,2)-k(21)*y(400);
jac(400,400)=jac(400,400)-k(21)*y(2);
jac(372,2)=jac(372,2)+k(21)*y(400);
jac(372,400)=jac(372,400)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(400,400)=jac(400,400)-k(10);
jac(3,400)=jac(3,400)+k(10);
jac(399,400)=jac(399,400)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(400,400)=jac(400,400)-k(10);
jac(3,400)=jac(3,400)+k(10);
jac(198,400)=jac(198,400)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(400,400)=jac(400,400)-k(8);
jac(48,400)=jac(48,400)+k(8);
jac(336,400)=jac(336,400)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(400,400)=jac(400,400)-k(6);
jac(47,400)=jac(47,400)+k(6);
jac(335,400)=jac(335,400)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(401,401)=jac(401,401)-k(34);
jac(470,401)=jac(470,401)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(401,401)=jac(401,401)-k(33);
jac(430,401)=jac(430,401)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(401,401)=jac(401,401)-k(32);
jac(212,401)=jac(212,401)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(401,401)=jac(401,401)-k(31);
jac(399,401)=jac(399,401)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(401,401)=jac(401,401)-k(22);
jac(197,401)=jac(197,401)+k(22);
jac(57,401)=jac(57,401)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(401);
jac(197,401)=jac(197,401)-k(21)*y(197);
jac(401,197)=jac(401,197)-k(21)*y(401);
jac(401,401)=jac(401,401)-k(21)*y(197);
jac(434,197)=jac(434,197)+k(21)*y(401);
jac(434,401)=jac(434,401)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(401);
jac(32,401)=jac(32,401)-k(21)*y(32);
jac(401,32)=jac(401,32)-k(21)*y(401);
jac(401,401)=jac(401,401)-k(21)*y(32);
jac(471,32)=jac(471,32)+k(21)*y(401);
jac(471,401)=jac(471,401)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(401);
jac(2,401)=jac(2,401)-k(21)*y(2);
jac(401,2)=jac(401,2)-k(21)*y(401);
jac(401,401)=jac(401,401)-k(21)*y(2);
jac(426,2)=jac(426,2)+k(21)*y(401);
jac(426,401)=jac(426,401)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(401);
jac(3,401)=jac(3,401)-k(15)*y(3);
jac(401,3)=jac(401,3)-k(15)*y(401);
jac(401,401)=jac(401,401)-k(15)*y(3);
jac(205,3)=jac(205,3)+k(15)*y(401);
jac(205,401)=jac(205,401)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(401,401)=jac(401,401)-k(10);
jac(3,401)=jac(3,401)+k(10);
jac(618,401)=jac(618,401)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(401,401)=jac(401,401)-k(8);
jac(48,401)=jac(48,401)+k(8);
jac(319,401)=jac(319,401)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(401,401)=jac(401,401)-k(6);
jac(47,401)=jac(47,401)+k(6);
jac(257,401)=jac(257,401)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(402,402)=jac(402,402)-k(34);
jac(446,402)=jac(446,402)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(402,402)=jac(402,402)-k(33);
jac(374,402)=jac(374,402)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(402,402)=jac(402,402)-k(33);
jac(447,402)=jac(447,402)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(402,402)=jac(402,402)-k(30);
jac(403,402)=jac(403,402)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(402,402)=jac(402,402)-k(22);
jac(32,402)=jac(32,402)+k(22);
jac(399,402)=jac(399,402)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(402,402)=jac(402,402)-k(22);
jac(197,402)=jac(197,402)+k(22);
jac(405,402)=jac(405,402)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(402,402)=jac(402,402)-k(11);
jac(471,402)=jac(471,402)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(402,402)=jac(402,402)-k(10);
jac(3,402)=jac(3,402)+k(10);
jac(454,402)=jac(454,402)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(402);
jac(3,402)=jac(3,402)-k(9)*y(3);
jac(402,3)=jac(402,3)-k(9)*y(402);
jac(402,402)=jac(402,402)-k(9)*y(3);
jac(459,3)=jac(459,3)+k(9)*y(402);
jac(459,402)=jac(459,402)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(402,402)=jac(402,402)-k(8);
jac(209,402)=jac(209,402)+k(8);
jac(131,402)=jac(131,402)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(402,402)=jac(402,402)-k(6);
jac(192,402)=jac(192,402)+k(6);
jac(441,402)=jac(441,402)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(403,403)=jac(403,403)-k(34);
jac(402,403)=jac(402,403)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(403,403)=jac(403,403)-k(34);
jac(412,403)=jac(412,403)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(403,403)=jac(403,403)-k(33);
jac(397,403)=jac(397,403)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(403,403)=jac(403,403)-k(33);
jac(435,403)=jac(435,403)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(403,403)=jac(403,403)-k(22);
jac(197,403)=jac(197,403)+k(22);
jac(399,403)=jac(399,403)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(403,403)=jac(403,403)-k(22);
jac(197,403)=jac(197,403)+k(22);
jac(406,403)=jac(406,403)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(403,403)=jac(403,403)-k(11);
jac(434,403)=jac(434,403)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(403,403)=jac(403,403)-k(10);
jac(3,403)=jac(3,403)+k(10);
jac(455,403)=jac(455,403)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(403);
jac(3,403)=jac(3,403)-k(9)*y(3);
jac(403,3)=jac(403,3)-k(9)*y(403);
jac(403,403)=jac(403,403)-k(9)*y(3);
jac(464,3)=jac(464,3)+k(9)*y(403);
jac(464,403)=jac(464,403)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(403,403)=jac(403,403)-k(8);
jac(209,403)=jac(209,403)+k(8);
jac(335,403)=jac(335,403)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(403,403)=jac(403,403)-k(6);
jac(192,403)=jac(192,403)+k(6);
jac(407,403)=jac(407,403)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(405,405)=jac(405,405)-k(33);
jac(383,405)=jac(383,405)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(405,405)=jac(405,405)-k(32);
jac(102,405)=jac(102,405)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(405,405)=jac(405,405)-k(22);
jac(32,405)=jac(32,405)+k(22);
jac(404,405)=jac(404,405)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(405);
jac(197,405)=jac(197,405)-k(21)*y(197);
jac(405,197)=jac(405,197)-k(21)*y(405);
jac(405,405)=jac(405,405)-k(21)*y(197);
jac(402,197)=jac(402,197)+k(21)*y(405);
jac(402,405)=jac(402,405)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(405);
jac(32,405)=jac(32,405)-k(21)*y(32);
jac(405,32)=jac(405,32)-k(21)*y(405);
jac(405,405)=jac(405,405)-k(21)*y(32);
jac(446,32)=jac(446,32)+k(21)*y(405);
jac(446,405)=jac(446,405)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(405);
jac(2,405)=jac(2,405)-k(21)*y(2);
jac(405,2)=jac(405,2)-k(21)*y(405);
jac(405,405)=jac(405,405)-k(21)*y(2);
jac(445,2)=jac(445,2)+k(21)*y(405);
jac(445,405)=jac(445,405)+k(21)*y(2);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(405,405)=jac(405,405)-k(11);
jac(443,405)=jac(443,405)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(405,405)=jac(405,405)-k(10);
jac(3,405)=jac(3,405)+k(10);
jac(450,405)=jac(450,405)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(405);
jac(3,405)=jac(3,405)-k(9)*y(3);
jac(405,3)=jac(405,3)-k(9)*y(405);
jac(405,405)=jac(405,405)-k(9)*y(3);
jac(458,3)=jac(458,3)+k(9)*y(405);
jac(458,405)=jac(458,405)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(405,405)=jac(405,405)-k(8);
jac(16,405)=jac(16,405)+k(8);
jac(131,405)=jac(131,405)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

jac(405,405)=jac(405,405)-k(6);
jac(15,405)=jac(15,405)+k(6);
jac(441,405)=jac(441,405)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(407,407)=jac(407,407)-k(34);
jac(441,407)=jac(441,407)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(407,407)=jac(407,407)-k(33);
jac(391,407)=jac(391,407)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(407,407)=jac(407,407)-k(22);
jac(197,407)=jac(197,407)+k(22);
jac(47,407)=jac(47,407)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(407,407)=jac(407,407)-k(10);
jac(3,407)=jac(3,407)+k(10);
jac(191,407)=jac(191,407)+k(10);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(1,1)=jac(1,1)-k(5)*y(407);
jac(1,407)=jac(1,407)-k(5)*y(1);
jac(407,1)=jac(407,1)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(1);
jac(413,1)=jac(413,1)+k(5)*y(407);
jac(413,407)=jac(413,407)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(7,7)=jac(7,7)-k(5)*y(407);
jac(7,407)=jac(7,407)-k(5)*y(7);
jac(407,7)=jac(407,7)-k(5)*y(407);
jac(407,407)=jac(407,407)-k(5)*y(7);
jac(414,7)=jac(414,7)+k(5)*y(407);
jac(414,407)=jac(414,407)+k(5)*y(7);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) 

jac(407,407)=jac(407,407)-k(4);
jac(4,407)=jac(4,407)+k(4);
jac(335,407)=jac(335,407)+k(4);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(408,408)=jac(408,408)-k(34);
jac(458,408)=jac(458,408)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(408,408)=jac(408,408)-k(33);
jac(651,408)=jac(651,408)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(408,408)=jac(408,408)-k(32);
jac(414,408)=jac(414,408)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(408,408)=jac(408,408)-k(22);
jac(197,408)=jac(197,408)+k(22);
jac(149,408)=jac(149,408)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(408);
jac(197,408)=jac(197,408)-k(21)*y(197);
jac(408,197)=jac(408,197)-k(21)*y(408);
jac(408,408)=jac(408,408)-k(21)*y(197);
jac(464,197)=jac(464,197)+k(21)*y(408);
jac(464,408)=jac(464,408)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(408);
jac(32,408)=jac(32,408)-k(21)*y(32);
jac(408,32)=jac(408,32)-k(21)*y(408);
jac(408,408)=jac(408,408)-k(21)*y(32);
jac(465,32)=jac(465,32)+k(21)*y(408);
jac(465,408)=jac(465,408)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(408);
jac(2,408)=jac(2,408)-k(21)*y(2);
jac(408,2)=jac(408,2)-k(21)*y(408);
jac(408,408)=jac(408,408)-k(21)*y(2);
jac(466,2)=jac(466,2)+k(21)*y(408);
jac(466,408)=jac(466,408)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(408,408)=jac(408,408)-k(10);
jac(3,408)=jac(3,408)+k(10);
jac(406,408)=jac(406,408)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(408,408)=jac(408,408)-k(10);
jac(3,408)=jac(3,408)+k(10);
jac(196,408)=jac(196,408)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(408,408)=jac(408,408)-k(8);
jac(126,408)=jac(126,408)+k(8);
jac(335,408)=jac(335,408)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(408,408)=jac(408,408)-k(6);
jac(48,408)=jac(48,408)+k(6);
jac(407,408)=jac(407,408)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(409,409)=jac(409,409)-k(34);
jac(450,409)=jac(450,409)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(409,409)=jac(409,409)-k(33);
jac(648,409)=jac(648,409)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(409,409)=jac(409,409)-k(32);
jac(202,409)=jac(202,409)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(409,409)=jac(409,409)-k(22);
jac(197,409)=jac(197,409)+k(22);
jac(150,409)=jac(150,409)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(409);
jac(197,409)=jac(197,409)-k(21)*y(197);
jac(409,197)=jac(409,197)-k(21)*y(409);
jac(409,409)=jac(409,409)-k(21)*y(197);
jac(455,197)=jac(455,197)+k(21)*y(409);
jac(455,409)=jac(455,409)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(409);
jac(32,409)=jac(32,409)-k(21)*y(32);
jac(409,32)=jac(409,32)-k(21)*y(409);
jac(409,409)=jac(409,409)-k(21)*y(32);
jac(456,32)=jac(456,32)+k(21)*y(409);
jac(456,409)=jac(456,409)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(409);
jac(2,409)=jac(2,409)-k(21)*y(2);
jac(409,2)=jac(409,2)-k(21)*y(409);
jac(409,409)=jac(409,409)-k(21)*y(2);
jac(453,2)=jac(453,2)+k(21)*y(409);
jac(453,409)=jac(453,409)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(409);
jac(3,409)=jac(3,409)-k(9)*y(3);
jac(409,3)=jac(409,3)-k(9)*y(409);
jac(409,409)=jac(409,409)-k(9)*y(3);
jac(196,3)=jac(196,3)+k(9)*y(409);
jac(196,409)=jac(196,409)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(409);
jac(3,409)=jac(3,409)-k(9)*y(3);
jac(409,3)=jac(409,3)-k(9)*y(409);
jac(409,409)=jac(409,409)-k(9)*y(3);
jac(406,3)=jac(406,3)+k(9)*y(409);
jac(406,409)=jac(406,409)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(409,409)=jac(409,409)-k(8);
jac(16,409)=jac(16,409)+k(8);
jac(192,409)=jac(192,409)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

jac(409,409)=jac(409,409)-k(6);
jac(15,409)=jac(15,409)+k(6);
jac(191,409)=jac(191,409)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(410,410)=jac(410,410)-k(34);
jac(443,410)=jac(443,410)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(410,410)=jac(410,410)-k(33);
jac(420,410)=jac(420,410)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(410,410)=jac(410,410)-k(32);
jac(415,410)=jac(415,410)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(410,410)=jac(410,410)-k(31);
jac(406,410)=jac(406,410)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(410,410)=jac(410,410)-k(22);
jac(197,410)=jac(197,410)+k(22);
jac(57,410)=jac(57,410)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(410);
jac(197,410)=jac(197,410)-k(21)*y(197);
jac(410,197)=jac(410,197)-k(21)*y(410);
jac(410,410)=jac(410,410)-k(21)*y(197);
jac(434,197)=jac(434,197)+k(21)*y(410);
jac(434,410)=jac(434,410)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(410);
jac(32,410)=jac(32,410)-k(21)*y(32);
jac(410,32)=jac(410,32)-k(21)*y(410);
jac(410,410)=jac(410,410)-k(21)*y(32);
jac(439,32)=jac(439,32)+k(21)*y(410);
jac(439,410)=jac(439,410)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(410);
jac(2,410)=jac(2,410)-k(21)*y(2);
jac(410,2)=jac(410,2)-k(21)*y(410);
jac(410,410)=jac(410,410)-k(21)*y(2);
jac(436,2)=jac(436,2)+k(21)*y(410);
jac(436,410)=jac(436,410)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(410);
jac(3,410)=jac(3,410)-k(15)*y(3);
jac(410,3)=jac(410,3)-k(15)*y(410);
jac(410,410)=jac(410,410)-k(15)*y(3);
jac(206,3)=jac(206,3)+k(15)*y(410);
jac(206,410)=jac(206,410)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(410,410)=jac(410,410)-k(10);
jac(3,410)=jac(3,410)+k(10);
jac(292,410)=jac(292,410)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(410,410)=jac(410,410)-k(8);
jac(315,410)=jac(315,410)+k(8);
jac(335,410)=jac(335,410)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

jac(410,410)=jac(410,410)-k(6);
jac(77,410)=jac(77,410)+k(6);
jac(407,410)=jac(407,410)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(411,411)=jac(411,411)-k(34);
jac(445,411)=jac(445,411)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(411,411)=jac(411,411)-k(33);
jac(390,411)=jac(390,411)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(411,411)=jac(411,411)-k(29);
jac(435,411)=jac(435,411)+k(29);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(411,411)=jac(411,411)-k(23);
jac(412,411)=jac(412,411)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(411,411)=jac(411,411)-k(22);
jac(2,411)=jac(2,411)+k(22);
jac(406,411)=jac(406,411)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(411,411)=jac(411,411)-k(22);
jac(197,411)=jac(197,411)+k(22);
jac(388,411)=jac(388,411)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(411,411)=jac(411,411)-k(11);
jac(436,411)=jac(436,411)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(411,411)=jac(411,411)-k(10);
jac(3,411)=jac(3,411)+k(10);
jac(453,411)=jac(453,411)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(411);
jac(3,411)=jac(3,411)-k(9)*y(3);
jac(411,3)=jac(411,3)-k(9)*y(411);
jac(411,411)=jac(411,411)-k(9)*y(3);
jac(466,3)=jac(466,3)+k(9)*y(411);
jac(466,411)=jac(466,411)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(411,411)=jac(411,411)-k(8);
jac(22,411)=jac(22,411)+k(8);
jac(335,411)=jac(335,411)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(411,411)=jac(411,411)-k(6);
jac(21,411)=jac(21,411)+k(6);
jac(407,411)=jac(407,411)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(412,412)=jac(412,412)-k(34);
jac(446,412)=jac(446,412)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(412,412)=jac(412,412)-k(33);
jac(411,412)=jac(411,412)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(412,412)=jac(412,412)-k(33);
jac(385,412)=jac(385,412)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(412,412)=jac(412,412)-k(29);
jac(403,412)=jac(403,412)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(412,412)=jac(412,412)-k(22);
jac(32,412)=jac(32,412)+k(22);
jac(406,412)=jac(406,412)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(412,412)=jac(412,412)-k(22);
jac(197,412)=jac(197,412)+k(22);
jac(384,412)=jac(384,412)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(412,412)=jac(412,412)-k(11);
jac(439,412)=jac(439,412)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(412,412)=jac(412,412)-k(10);
jac(3,412)=jac(3,412)+k(10);
jac(456,412)=jac(456,412)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(412);
jac(3,412)=jac(3,412)-k(9)*y(3);
jac(412,3)=jac(412,3)-k(9)*y(412);
jac(412,412)=jac(412,412)-k(9)*y(3);
jac(465,3)=jac(465,3)+k(9)*y(412);
jac(465,412)=jac(465,412)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(412,412)=jac(412,412)-k(8);
jac(28,412)=jac(28,412)+k(8);
jac(335,412)=jac(335,412)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(412,412)=jac(412,412)-k(6);
jac(27,412)=jac(27,412)+k(6);
jac(407,412)=jac(407,412)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(414,414)=jac(414,414)-k(34);
jac(104,414)=jac(104,414)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(414,414)=jac(414,414)-k(33);
jac(417,414)=jac(417,414)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(414,414)=jac(414,414)-k(22);
jac(197,414)=jac(197,414)+k(22);
jac(463,414)=jac(463,414)+k(22);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(414,414)=jac(414,414)-k(13);
jac(408,414)=jac(408,414)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(414,414)=jac(414,414)-k(10);
jac(3,414)=jac(3,414)+k(10);
jac(413,414)=jac(413,414)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(414,414)=jac(414,414)-k(10);
jac(3,414)=jac(3,414)+k(10);
jac(607,414)=jac(607,414)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(414,414)=jac(414,414)-k(8);
jac(8,414)=jac(8,414)+k(8);
jac(335,414)=jac(335,414)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(414,414)=jac(414,414)-k(6);
jac(7,414)=jac(7,414)+k(6);
jac(407,414)=jac(407,414)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(415,415)=jac(415,415)-k(34);
jac(650,415)=jac(650,415)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(415,415)=jac(415,415)-k(33);
jac(419,415)=jac(419,415)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(415,415)=jac(415,415)-k(31);
jac(413,415)=jac(413,415)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(415,415)=jac(415,415)-k(22);
jac(197,415)=jac(197,415)+k(22);
jac(64,415)=jac(64,415)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(415);
jac(3,415)=jac(3,415)-k(15)*y(3);
jac(415,3)=jac(415,3)-k(15)*y(415);
jac(415,415)=jac(415,415)-k(15)*y(3);
jac(204,3)=jac(204,3)+k(15)*y(415);
jac(204,415)=jac(204,415)+k(15)*y(3);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(415,415)=jac(415,415)-k(13);
jac(410,415)=jac(410,415)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(415,415)=jac(415,415)-k(10);
jac(3,415)=jac(3,415)+k(10);
jac(296,415)=jac(296,415)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(415,415)=jac(415,415)-k(8);
jac(91,415)=jac(91,415)+k(8);
jac(335,415)=jac(335,415)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

jac(415,415)=jac(415,415)-k(6);
jac(72,415)=jac(72,415)+k(6);
jac(407,415)=jac(407,415)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(651,651)=jac(651,651)-k(34);
jac(461,651)=jac(461,651)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(651,651)=jac(651,651)-k(32);
jac(417,651)=jac(417,651)+k(32);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(651,651)=jac(651,651)-k(24);
jac(408,651)=jac(408,651)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(651,651)=jac(651,651)-k(22);
jac(182,651)=jac(182,651)+k(22);
jac(149,651)=jac(149,651)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(651);
jac(197,651)=jac(197,651)-k(21)*y(197);
jac(651,197)=jac(651,197)-k(21)*y(651);
jac(651,651)=jac(651,651)-k(21)*y(197);
jac(368,197)=jac(368,197)+k(21)*y(651);
jac(368,651)=jac(368,651)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(651);
jac(32,651)=jac(32,651)-k(21)*y(32);
jac(651,32)=jac(651,32)-k(21)*y(651);
jac(651,651)=jac(651,651)-k(21)*y(32);
jac(365,32)=jac(365,32)+k(21)*y(651);
jac(365,651)=jac(365,651)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(651);
jac(2,651)=jac(2,651)-k(21)*y(2);
jac(651,2)=jac(651,2)-k(21)*y(651);
jac(651,651)=jac(651,651)-k(21)*y(2);
jac(178,2)=jac(178,2)+k(21)*y(651);
jac(178,651)=jac(178,651)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(651,651)=jac(651,651)-k(10);
jac(3,651)=jac(3,651)+k(10);
jac(181,651)=jac(181,651)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(651,651)=jac(651,651)-k(10);
jac(3,651)=jac(3,651)+k(10);
jac(393,651)=jac(393,651)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(651,651)=jac(651,651)-k(8);
jac(126,651)=jac(126,651)+k(8);
jac(184,651)=jac(184,651)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(651,651)=jac(651,651)-k(6);
jac(48,651)=jac(48,651)+k(6);
jac(391,651)=jac(391,651)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(417,417)=jac(417,417)-k(34);
jac(462,417)=jac(462,417)+k(34);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(417,417)=jac(417,417)-k(24);
jac(414,417)=jac(414,417)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(417,417)=jac(417,417)-k(22);
jac(182,417)=jac(182,417)+k(22);
jac(463,417)=jac(463,417)+k(22);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(417,417)=jac(417,417)-k(13);
jac(651,417)=jac(651,417)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(417,417)=jac(417,417)-k(10);
jac(3,417)=jac(3,417)+k(10);
jac(416,417)=jac(416,417)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(417,417)=jac(417,417)-k(10);
jac(3,417)=jac(3,417)+k(10);
jac(606,417)=jac(606,417)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(417,417)=jac(417,417)-k(8);
jac(8,417)=jac(8,417)+k(8);
jac(184,417)=jac(184,417)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(417,417)=jac(417,417)-k(6);
jac(7,417)=jac(7,417)+k(6);
jac(391,417)=jac(391,417)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(418,418)=jac(418,418)-k(34);
jac(94,418)=jac(94,418)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(418,418)=jac(418,418)-k(22);
jac(182,418)=jac(182,418)+k(22);
jac(97,418)=jac(97,418)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(418);
jac(3,418)=jac(3,418)-k(9)*y(3);
jac(418,3)=jac(418,3)-k(9)*y(418);
jac(418,418)=jac(418,418)-k(9)*y(3);
jac(606,3)=jac(606,3)+k(9)*y(418);
jac(606,418)=jac(606,418)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(418);
jac(3,418)=jac(3,418)-k(9)*y(3);
jac(418,3)=jac(418,3)-k(9)*y(418);
jac(418,418)=jac(418,418)-k(9)*y(3);
jac(416,3)=jac(416,3)+k(9)*y(418);
jac(416,418)=jac(416,418)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(418,418)=jac(418,418)-k(8);
jac(5,418)=jac(5,418)+k(8);
jac(177,418)=jac(177,418)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

jac(418,418)=jac(418,418)-k(6);
jac(1,418)=jac(1,418)+k(6);
jac(176,418)=jac(176,418)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(421,421)=jac(421,421)-k(34);
jac(425,421)=jac(425,421)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(421,421)=jac(421,421)-k(32);
jac(299,421)=jac(299,421)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(421,421)=jac(421,421)-k(31);
jac(648,421)=jac(648,421)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(421,421)=jac(421,421)-k(22);
jac(182,421)=jac(182,421)+k(22);
jac(614,421)=jac(614,421)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(421);
jac(197,421)=jac(197,421)-k(21)*y(197);
jac(421,197)=jac(421,197)-k(21)*y(421);
jac(421,421)=jac(421,421)-k(21)*y(197);
jac(487,197)=jac(487,197)+k(21)*y(421);
jac(487,421)=jac(487,421)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(421);
jac(32,421)=jac(32,421)-k(21)*y(32);
jac(421,32)=jac(421,32)-k(21)*y(421);
jac(421,421)=jac(421,421)-k(21)*y(32);
jac(498,32)=jac(498,32)+k(21)*y(421);
jac(498,421)=jac(498,421)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(421);
jac(2,421)=jac(2,421)-k(21)*y(2);
jac(421,2)=jac(421,2)-k(21)*y(421);
jac(421,421)=jac(421,421)-k(21)*y(2);
jac(271,2)=jac(271,2)+k(21)*y(421);
jac(271,421)=jac(271,421)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(421);
jac(3,421)=jac(3,421)-k(15)*y(3);
jac(421,3)=jac(421,3)-k(15)*y(421);
jac(421,421)=jac(421,421)-k(15)*y(3);
jac(272,3)=jac(272,3)+k(15)*y(421);
jac(272,421)=jac(272,421)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(421);
jac(3,421)=jac(3,421)-k(9)*y(3);
jac(421,3)=jac(421,3)-k(9)*y(421);
jac(421,421)=jac(421,421)-k(9)*y(3);
jac(420,3)=jac(420,3)+k(9)*y(421);
jac(420,421)=jac(420,421)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(421,421)=jac(421,421)-k(8);
jac(315,421)=jac(315,421)+k(8);
jac(177,421)=jac(177,421)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

jac(421,421)=jac(421,421)-k(6);
jac(77,421)=jac(77,421)+k(6);
jac(176,421)=jac(176,421)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(422,422)=jac(422,422)-k(34);
jac(58,422)=jac(58,422)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(422,422)=jac(422,422)-k(32);
jac(298,422)=jac(298,422)+k(32);

% rule    : R11_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(422,422)=jac(422,422)-k(26);
jac(206,422)=jac(206,422)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(422,422)=jac(422,422)-k(22);
jac(182,422)=jac(182,422)+k(22);
jac(53,422)=jac(53,422)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(422);
jac(197,422)=jac(197,422)-k(21)*y(197);
jac(422,197)=jac(422,197)-k(21)*y(422);
jac(422,422)=jac(422,422)-k(21)*y(197);
jac(476,197)=jac(476,197)+k(21)*y(422);
jac(476,422)=jac(476,422)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(422);
jac(32,422)=jac(32,422)-k(21)*y(32);
jac(422,32)=jac(422,32)-k(21)*y(422);
jac(422,422)=jac(422,422)-k(21)*y(32);
jac(478,32)=jac(478,32)+k(21)*y(422);
jac(478,422)=jac(478,422)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(422);
jac(2,422)=jac(2,422)-k(21)*y(2);
jac(422,2)=jac(422,2)-k(21)*y(422);
jac(422,422)=jac(422,422)-k(21)*y(2);
jac(270,2)=jac(270,2)+k(21)*y(422);
jac(270,422)=jac(270,422)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(422,422)=jac(422,422)-k(16);
jac(3,422)=jac(3,422)+k(16);
jac(420,422)=jac(420,422)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(422,422)=jac(422,422)-k(10);
jac(3,422)=jac(3,422)+k(10);
jac(272,422)=jac(272,422)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(422,422)=jac(422,422)-k(8);
jac(55,422)=jac(55,422)+k(8);
jac(184,422)=jac(184,422)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(422,422)=jac(422,422)-k(6);
jac(54,422)=jac(54,422)+k(6);
jac(391,422)=jac(391,422)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(423,423)=jac(423,423)-k(34);
jac(382,423)=jac(382,423)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(423,423)=jac(423,423)-k(33);
jac(392,423)=jac(392,423)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(423,423)=jac(423,423)-k(31);
jac(385,423)=jac(385,423)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(423,423)=jac(423,423)-k(29);
jac(398,423)=jac(398,423)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(423,423)=jac(423,423)-k(22);
jac(32,423)=jac(32,423)+k(22);
jac(420,423)=jac(420,423)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(423,423)=jac(423,423)-k(22);
jac(182,423)=jac(182,423)+k(22);
jac(470,423)=jac(470,423)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(423);
jac(3,423)=jac(3,423)-k(15)*y(3);
jac(423,3)=jac(423,3)-k(15)*y(423);
jac(423,423)=jac(423,423)-k(15)*y(3);
jac(478,3)=jac(478,3)+k(15)*y(423);
jac(478,423)=jac(478,423)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(423,423)=jac(423,423)-k(10);
jac(3,423)=jac(3,423)+k(10);
jac(498,423)=jac(498,423)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(423,423)=jac(423,423)-k(8);
jac(34,423)=jac(34,423)+k(8);
jac(184,423)=jac(184,423)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(423,423)=jac(423,423)-k(6);
jac(33,423)=jac(33,423)+k(6);
jac(391,423)=jac(391,423)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(425,425)=jac(425,425)-k(32);
jac(90,425)=jac(90,425)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(425,425)=jac(425,425)-k(31);
jac(375,425)=jac(375,425)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(425,425)=jac(425,425)-k(22);
jac(2,425)=jac(2,425)+k(22);
jac(614,425)=jac(614,425)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(425);
jac(197,425)=jac(197,425)-k(21)*y(197);
jac(425,197)=jac(425,197)-k(21)*y(425);
jac(425,425)=jac(425,425)-k(21)*y(197);
jac(427,197)=jac(427,197)+k(21)*y(425);
jac(427,425)=jac(427,425)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(425);
jac(32,425)=jac(32,425)-k(21)*y(32);
jac(425,32)=jac(425,32)-k(21)*y(425);
jac(425,425)=jac(425,425)-k(21)*y(32);
jac(482,32)=jac(482,32)+k(21)*y(425);
jac(482,425)=jac(482,425)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(425);
jac(2,425)=jac(2,425)-k(21)*y(2);
jac(425,2)=jac(425,2)-k(21)*y(425);
jac(425,425)=jac(425,425)-k(21)*y(2);
jac(275,2)=jac(275,2)+k(21)*y(425);
jac(275,425)=jac(275,425)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(425);
jac(3,425)=jac(3,425)-k(15)*y(3);
jac(425,3)=jac(425,3)-k(15)*y(425);
jac(425,425)=jac(425,425)-k(15)*y(3);
jac(276,3)=jac(276,3)+k(15)*y(425);
jac(276,425)=jac(276,425)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(425);
jac(3,425)=jac(3,425)-k(9)*y(3);
jac(425,3)=jac(425,3)-k(9)*y(425);
jac(425,425)=jac(425,425)-k(9)*y(3);
jac(424,3)=jac(424,3)+k(9)*y(425);
jac(424,425)=jac(424,425)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(425,425)=jac(425,425)-k(8);
jac(315,425)=jac(315,425)+k(8);
jac(21,425)=jac(21,425)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

jac(425,425)=jac(425,425)-k(6);
jac(77,425)=jac(77,425)+k(6);
jac(89,425)=jac(89,425)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(427,427)=jac(427,427)-k(34);
jac(482,427)=jac(482,427)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(427,427)=jac(427,427)-k(33);
jac(280,427)=jac(280,427)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(427,427)=jac(427,427)-k(31);
jac(373,427)=jac(373,427)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(427,427)=jac(427,427)-k(30);
jac(487,427)=jac(487,427)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(427,427)=jac(427,427)-k(22);
jac(2,427)=jac(2,427)+k(22);
jac(618,427)=jac(618,427)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(427,427)=jac(427,427)-k(22);
jac(197,427)=jac(197,427)+k(22);
jac(425,427)=jac(425,427)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(427);
jac(3,427)=jac(3,427)-k(15)*y(3);
jac(427,3)=jac(427,3)-k(15)*y(427);
jac(427,427)=jac(427,427)-k(15)*y(3);
jac(484,3)=jac(484,3)+k(15)*y(427);
jac(484,427)=jac(484,427)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(427);
jac(3,427)=jac(3,427)-k(9)*y(3);
jac(427,3)=jac(427,3)-k(9)*y(427);
jac(427,427)=jac(427,427)-k(9)*y(3);
jac(426,3)=jac(426,3)+k(9)*y(427);
jac(426,427)=jac(426,427)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(427,427)=jac(427,427)-k(8);
jac(21,427)=jac(21,427)+k(8);
jac(319,427)=jac(319,427)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(427,427)=jac(427,427)-k(6);
jac(257,427)=jac(257,427)+k(6);
jac(89,427)=jac(89,427)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(428,428)=jac(428,428)-k(34);
jac(480,428)=jac(480,428)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(428,428)=jac(428,428)-k(33);
jac(279,428)=jac(279,428)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(428,428)=jac(428,428)-k(30);
jac(476,428)=jac(476,428)+k(30);

% rule    : R11_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(428,428)=jac(428,428)-k(26);
jac(473,428)=jac(473,428)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(428,428)=jac(428,428)-k(22);
jac(197,428)=jac(197,428)+k(22);
jac(58,428)=jac(58,428)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(428,428)=jac(428,428)-k(22);
jac(2,428)=jac(2,428)+k(22);
jac(205,428)=jac(205,428)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(428,428)=jac(428,428)-k(16);
jac(3,428)=jac(3,428)+k(16);
jac(426,428)=jac(426,428)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(428,428)=jac(428,428)-k(10);
jac(3,428)=jac(3,428)+k(10);
jac(484,428)=jac(484,428)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(428,428)=jac(428,428)-k(8);
jac(140,428)=jac(140,428)+k(8);
jac(228,428)=jac(228,428)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(428,428)=jac(428,428)-k(6);
jac(227,428)=jac(227,428)+k(6);
jac(380,428)=jac(380,428)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(431,431)=jac(431,431)-k(34);
jac(616,431)=jac(616,431)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(431,431)=jac(431,431)-k(32);
jac(220,431)=jac(220,431)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(431,431)=jac(431,431)-k(31);
jac(451,431)=jac(451,431)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(431,431)=jac(431,431)-k(22);
jac(182,431)=jac(182,431)+k(22);
jac(614,431)=jac(614,431)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(431);
jac(197,431)=jac(197,431)-k(21)*y(197);
jac(431,197)=jac(431,197)-k(21)*y(431);
jac(431,431)=jac(431,431)-k(21)*y(197);
jac(290,197)=jac(290,197)+k(21)*y(431);
jac(290,431)=jac(290,431)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(431);
jac(32,431)=jac(32,431)-k(21)*y(32);
jac(431,32)=jac(431,32)-k(21)*y(431);
jac(431,431)=jac(431,431)-k(21)*y(32);
jac(285,32)=jac(285,32)+k(21)*y(431);
jac(285,431)=jac(285,431)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(431);
jac(2,431)=jac(2,431)-k(21)*y(2);
jac(431,2)=jac(431,2)-k(21)*y(431);
jac(431,431)=jac(431,431)-k(21)*y(2);
jac(280,2)=jac(280,2)+k(21)*y(431);
jac(280,431)=jac(280,431)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(431);
jac(3,431)=jac(3,431)-k(15)*y(3);
jac(431,3)=jac(431,3)-k(15)*y(431);
jac(431,431)=jac(431,431)-k(15)*y(3);
jac(282,3)=jac(282,3)+k(15)*y(431);
jac(282,431)=jac(282,431)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(431);
jac(3,431)=jac(3,431)-k(9)*y(3);
jac(431,3)=jac(431,3)-k(9)*y(431);
jac(431,431)=jac(431,431)-k(9)*y(3);
jac(430,3)=jac(430,3)+k(9)*y(431);
jac(430,431)=jac(430,431)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

jac(431,431)=jac(431,431)-k(8);
jac(15,431)=jac(15,431)+k(8);
jac(219,431)=jac(219,431)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(431,431)=jac(431,431)-k(6);
jac(98,431)=jac(98,431)+k(6);
jac(218,431)=jac(218,431)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(432,432)=jac(432,432)-k(34);
jac(440,432)=jac(440,432)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(432,432)=jac(432,432)-k(33);
jac(429,432)=jac(429,432)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(432,432)=jac(432,432)-k(31);
jac(447,432)=jac(447,432)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(432,432)=jac(432,432)-k(30);
jac(433,432)=jac(433,432)+k(30);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(432,432)=jac(432,432)-k(23);
jac(471,432)=jac(471,432)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(432,432)=jac(432,432)-k(22);
jac(32,432)=jac(32,432)+k(22);
jac(430,432)=jac(430,432)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(432,432)=jac(432,432)-k(22);
jac(182,432)=jac(182,432)+k(22);
jac(443,432)=jac(443,432)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(432);
jac(3,432)=jac(3,432)-k(15)*y(3);
jac(432,3)=jac(432,3)-k(15)*y(432);
jac(432,432)=jac(432,432)-k(15)*y(3);
jac(284,3)=jac(284,3)+k(15)*y(432);
jac(284,432)=jac(284,432)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(432,432)=jac(432,432)-k(10);
jac(3,432)=jac(3,432)+k(10);
jac(285,432)=jac(285,432)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(432,432)=jac(432,432)-k(8);
jac(219,432)=jac(219,432)+k(8);
jac(131,432)=jac(131,432)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(432,432)=jac(432,432)-k(6);
jac(218,432)=jac(218,432)+k(6);
jac(441,432)=jac(441,432)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(434,434)=jac(434,434)-k(34);
jac(471,434)=jac(471,434)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(434,434)=jac(434,434)-k(34);
jac(439,434)=jac(439,434)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(434,434)=jac(434,434)-k(33);
jac(398,434)=jac(398,434)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(434,434)=jac(434,434)-k(33);
jac(433,434)=jac(433,434)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(434,434)=jac(434,434)-k(31);
jac(403,434)=jac(403,434)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(434,434)=jac(434,434)-k(22);
jac(197,434)=jac(197,434)+k(22);
jac(401,434)=jac(401,434)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(434,434)=jac(434,434)-k(22);
jac(197,434)=jac(197,434)+k(22);
jac(410,434)=jac(410,434)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(434);
jac(3,434)=jac(3,434)-k(15)*y(3);
jac(434,3)=jac(434,3)-k(15)*y(434);
jac(434,434)=jac(434,434)-k(15)*y(3);
jac(475,3)=jac(475,3)+k(15)*y(434);
jac(475,434)=jac(475,434)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(434,434)=jac(434,434)-k(10);
jac(3,434)=jac(3,434)+k(10);
jac(619,434)=jac(619,434)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(434,434)=jac(434,434)-k(8);
jac(319,434)=jac(319,434)+k(8);
jac(335,434)=jac(335,434)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(434,434)=jac(434,434)-k(6);
jac(257,434)=jac(257,434)+k(6);
jac(407,434)=jac(407,434)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(435,435)=jac(435,435)-k(34);
jac(447,435)=jac(447,435)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(435,435)=jac(435,435)-k(34);
jac(411,435)=jac(411,435)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(435,435)=jac(435,435)-k(33);
jac(394,435)=jac(394,435)+k(33);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(435,435)=jac(435,435)-k(23);
jac(403,435)=jac(403,435)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(435,435)=jac(435,435)-k(22);
jac(197,435)=jac(197,435)+k(22);
jac(396,435)=jac(396,435)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(435,435)=jac(435,435)-k(22);
jac(182,435)=jac(182,435)+k(22);
jac(406,435)=jac(406,435)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(435,435)=jac(435,435)-k(11);
jac(433,435)=jac(433,435)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(435,435)=jac(435,435)-k(10);
jac(3,435)=jac(3,435)+k(10);
jac(194,435)=jac(194,435)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(435);
jac(3,435)=jac(3,435)-k(9)*y(3);
jac(435,3)=jac(435,3)-k(9)*y(435);
jac(435,435)=jac(435,435)-k(9)*y(3);
jac(193,3)=jac(193,3)+k(9)*y(435);
jac(193,435)=jac(193,435)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(435,435)=jac(435,435)-k(8);
jac(214,435)=jac(214,435)+k(8);
jac(335,435)=jac(335,435)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(435,435)=jac(435,435)-k(6);
jac(177,435)=jac(177,435)+k(6);
jac(407,435)=jac(407,435)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(437,437)=jac(437,437)-k(31);
jac(22,437)=jac(22,437)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Syk(tSH2, l~Y, a~Y) 

jac(437,437)=jac(437,437)-k(22);
jac(2,437)=jac(2,437)+k(22);
jac(315,437)=jac(315,437)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(437);
jac(3,437)=jac(3,437)-k(15)*y(3);
jac(437,3)=jac(437,3)-k(15)*y(437);
jac(437,437)=jac(437,437)-k(15)*y(3);
jac(44,3)=jac(44,3)+k(15)*y(437);
jac(44,437)=jac(44,437)+k(15)*y(3);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(1,1)=jac(1,1)-k(7)*y(437);
jac(1,437)=jac(1,437)-k(7)*y(1);
jac(437,1)=jac(437,1)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(1);
jac(621,1)=jac(621,1)+k(7)*y(437);
jac(621,437)=jac(621,437)+k(7)*y(1);

% rule    : R2_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(7,7)=jac(7,7)-k(7)*y(437);
jac(7,437)=jac(7,437)-k(7)*y(7);
jac(437,7)=jac(437,7)-k(7)*y(437);
jac(437,437)=jac(437,437)-k(7)*y(7);
jac(25,7)=jac(25,7)+k(7)*y(437);
jac(25,437)=jac(25,437)+k(7)*y(7);

% rule    : R1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Lig(l1, l2) 

jac(437,437)=jac(437,437)-k(2);
jac(4,437)=jac(4,437)+k(2);
jac(111,437)=jac(111,437)+k(2);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(438,438)=jac(438,438)-k(34);
jac(442,438)=jac(442,438)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(438,438)=jac(438,438)-k(33);
jac(271,438)=jac(271,438)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(438,438)=jac(438,438)-k(31);
jac(453,438)=jac(453,438)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(438,438)=jac(438,438)-k(29);
jac(290,438)=jac(290,438)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(438,438)=jac(438,438)-k(22);
jac(197,438)=jac(197,438)+k(22);
jac(616,438)=jac(616,438)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(438,438)=jac(438,438)-k(22);
jac(2,438)=jac(2,438)+k(22);
jac(292,438)=jac(292,438)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(438);
jac(3,438)=jac(3,438)-k(15)*y(3);
jac(438,3)=jac(438,3)-k(15)*y(438);
jac(438,438)=jac(438,438)-k(15)*y(3);
jac(262,3)=jac(262,3)+k(15)*y(438);
jac(262,438)=jac(262,438)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(438);
jac(3,438)=jac(3,438)-k(9)*y(3);
jac(438,3)=jac(438,3)-k(9)*y(438);
jac(438,438)=jac(438,438)-k(9)*y(3);
jac(436,3)=jac(436,3)+k(9)*y(438);
jac(436,438)=jac(436,438)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(438,438)=jac(438,438)-k(8);
jac(192,438)=jac(192,438)+k(8);
jac(437,438)=jac(437,438)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(438,438)=jac(438,438)-k(6);
jac(111,438)=jac(111,438)+k(6);
jac(191,438)=jac(191,438)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(439,439)=jac(439,439)-k(34);
jac(444,439)=jac(444,439)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(439,439)=jac(439,439)-k(33);
jac(423,439)=jac(423,439)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(439,439)=jac(439,439)-k(33);
jac(436,439)=jac(436,439)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(439,439)=jac(439,439)-k(31);
jac(412,439)=jac(412,439)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(439,439)=jac(439,439)-k(29);
jac(434,439)=jac(434,439)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(439,439)=jac(439,439)-k(22);
jac(197,439)=jac(197,439)+k(22);
jac(470,439)=jac(470,439)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(439,439)=jac(439,439)-k(22);
jac(32,439)=jac(32,439)+k(22);
jac(410,439)=jac(410,439)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(439);
jac(3,439)=jac(3,439)-k(15)*y(3);
jac(439,3)=jac(439,3)-k(15)*y(439);
jac(439,439)=jac(439,439)-k(15)*y(3);
jac(477,3)=jac(477,3)+k(15)*y(439);
jac(477,439)=jac(477,439)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(439,439)=jac(439,439)-k(10);
jac(3,439)=jac(3,439)+k(10);
jac(620,439)=jac(620,439)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(439,439)=jac(439,439)-k(8);
jac(34,439)=jac(34,439)+k(8);
jac(335,439)=jac(335,439)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(439,439)=jac(439,439)-k(6);
jac(33,439)=jac(33,439)+k(6);
jac(407,439)=jac(407,439)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(441,441)=jac(441,441)-k(33);
jac(380,441)=jac(380,441)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(441,441)=jac(441,441)-k(22);
jac(32,441)=jac(32,441)+k(22);
jac(47,441)=jac(47,441)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(441,441)=jac(441,441)-k(10);
jac(3,441)=jac(3,441)+k(10);
jac(86,441)=jac(86,441)+k(10);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(1,1)=jac(1,1)-k(5)*y(441);
jac(1,441)=jac(1,441)-k(5)*y(1);
jac(441,1)=jac(441,1)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(1);
jac(102,1)=jac(102,1)+k(5)*y(441);
jac(102,441)=jac(102,441)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(5)*y(441);
jac(7,441)=jac(7,441)-k(5)*y(7);
jac(441,7)=jac(441,7)-k(5)*y(441);
jac(441,441)=jac(441,441)-k(5)*y(7);
jac(104,7)=jac(104,7)+k(5)*y(441);
jac(104,441)=jac(104,441)+k(5)*y(7);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) 

jac(441,441)=jac(441,441)-k(4);
jac(4,441)=jac(4,441)+k(4);
jac(131,441)=jac(131,441)+k(4);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(442,442)=jac(442,442)-k(33);
jac(275,442)=jac(275,442)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(442,442)=jac(442,442)-k(31);
jac(166,442)=jac(166,442)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(442,442)=jac(442,442)-k(28);
jac(438,442)=jac(438,442)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(442,442)=jac(442,442)-k(27);
jac(285,442)=jac(285,442)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(442,442)=jac(442,442)-k(22);
jac(32,442)=jac(32,442)+k(22);
jac(616,442)=jac(616,442)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(442,442)=jac(442,442)-k(22);
jac(2,442)=jac(2,442)+k(22);
jac(615,442)=jac(615,442)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(442);
jac(3,442)=jac(3,442)-k(15)*y(3);
jac(442,3)=jac(442,3)-k(15)*y(442);
jac(442,442)=jac(442,442)-k(15)*y(3);
jac(277,3)=jac(277,3)+k(15)*y(442);
jac(277,442)=jac(277,442)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(442);
jac(3,442)=jac(3,442)-k(9)*y(3);
jac(442,3)=jac(442,3)-k(9)*y(442);
jac(442,442)=jac(442,442)-k(9)*y(3);
jac(440,3)=jac(440,3)+k(9)*y(442);
jac(440,442)=jac(440,442)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(442,442)=jac(442,442)-k(8);
jac(27,442)=jac(27,442)+k(8);
jac(437,442)=jac(437,442)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(442,442)=jac(442,442)-k(6);
jac(111,442)=jac(111,442)+k(6);
jac(86,442)=jac(86,442)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(650,650)=jac(650,650)-k(33);
jac(92,650)=jac(92,650)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(650,650)=jac(650,650)-k(31);
jac(102,650)=jac(102,650)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(650,650)=jac(650,650)-k(22);
jac(32,650)=jac(32,650)+k(22);
jac(64,650)=jac(64,650)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(650);
jac(3,650)=jac(3,650)-k(15)*y(3);
jac(650,3)=jac(650,3)-k(15)*y(650);
jac(650,650)=jac(650,650)-k(15)*y(3);
jac(66,3)=jac(66,3)+k(15)*y(650);
jac(66,650)=jac(66,650)+k(15)*y(3);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(650,650)=jac(650,650)-k(13);
jac(443,650)=jac(443,650)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(650,650)=jac(650,650)-k(10);
jac(3,650)=jac(3,650)+k(10);
jac(87,650)=jac(87,650)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(650,650)=jac(650,650)-k(8);
jac(91,650)=jac(91,650)+k(8);
jac(131,650)=jac(131,650)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

jac(650,650)=jac(650,650)-k(6);
jac(72,650)=jac(72,650)+k(6);
jac(441,650)=jac(441,650)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(443,443)=jac(443,443)-k(33);
jac(424,443)=jac(424,443)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(443,443)=jac(443,443)-k(32);
jac(650,443)=jac(650,443)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(443,443)=jac(443,443)-k(31);
jac(405,443)=jac(405,443)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(443,443)=jac(443,443)-k(22);
jac(32,443)=jac(32,443)+k(22);
jac(57,443)=jac(57,443)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(443);
jac(197,443)=jac(197,443)-k(21)*y(197);
jac(443,197)=jac(443,197)-k(21)*y(443);
jac(443,443)=jac(443,443)-k(21)*y(197);
jac(471,197)=jac(471,197)+k(21)*y(443);
jac(471,443)=jac(471,443)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(443);
jac(32,443)=jac(32,443)-k(21)*y(32);
jac(443,32)=jac(443,32)-k(21)*y(443);
jac(443,443)=jac(443,443)-k(21)*y(32);
jac(444,32)=jac(444,32)+k(21)*y(443);
jac(444,443)=jac(444,443)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(443);
jac(2,443)=jac(2,443)-k(21)*y(2);
jac(443,2)=jac(443,2)-k(21)*y(443);
jac(443,443)=jac(443,443)-k(21)*y(2);
jac(440,2)=jac(440,2)+k(21)*y(443);
jac(440,443)=jac(440,443)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(443);
jac(3,443)=jac(3,443)-k(15)*y(3);
jac(443,3)=jac(443,3)-k(15)*y(443);
jac(443,443)=jac(443,443)-k(15)*y(3);
jac(59,3)=jac(59,3)+k(15)*y(443);
jac(59,443)=jac(59,443)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(443,443)=jac(443,443)-k(10);
jac(3,443)=jac(3,443)+k(10);
jac(615,443)=jac(615,443)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(443,443)=jac(443,443)-k(8);
jac(315,443)=jac(315,443)+k(8);
jac(131,443)=jac(131,443)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

jac(443,443)=jac(443,443)-k(6);
jac(77,443)=jac(77,443)+k(6);
jac(441,443)=jac(441,443)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(468,468)=jac(468,468)-k(33);
jac(482,468)=jac(482,468)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(468,468)=jac(468,468)-k(33);
jac(442,468)=jac(442,468)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(468,468)=jac(468,468)-k(31);
jac(160,468)=jac(160,468)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(468,468)=jac(468,468)-k(28);
jac(620,468)=jac(620,468)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(468,468)=jac(468,468)-k(27);
jac(472,468)=jac(472,468)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(468,468)=jac(468,468)-k(22);
jac(32,468)=jac(32,468)+k(22);
jac(617,468)=jac(617,468)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(468,468)=jac(468,468)-k(22);
jac(32,468)=jac(32,468)+k(22);
jac(615,468)=jac(615,468)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(468);
jac(3,468)=jac(3,468)-k(15)*y(3);
jac(468,3)=jac(468,3)-k(15)*y(468);
jac(468,468)=jac(468,468)-k(15)*y(3);
jac(483,3)=jac(483,3)+k(15)*y(468);
jac(483,468)=jac(483,468)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(468);
jac(3,468)=jac(3,468)-k(9)*y(3);
jac(468,3)=jac(468,3)-k(9)*y(468);
jac(468,468)=jac(468,468)-k(9)*y(3);
jac(444,3)=jac(444,3)+k(9)*y(468);
jac(444,468)=jac(444,468)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(468,468)=jac(468,468)-k(8);
jac(27,468)=jac(27,468)+k(8);
jac(34,468)=jac(34,468)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(468,468)=jac(468,468)-k(6);
jac(33,468)=jac(33,468)+k(6);
jac(86,468)=jac(86,468)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(469,469)=jac(469,469)-k(33);
jac(52,469)=jac(52,469)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(469,469)=jac(469,469)-k(33);
jac(480,469)=jac(480,469)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(469,469)=jac(469,469)-k(28);
jac(477,469)=jac(477,469)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(469,469)=jac(469,469)-k(27);
jac(473,469)=jac(473,469)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(469,469)=jac(469,469)-k(22);
jac(32,469)=jac(32,469)+k(22);
jac(59,469)=jac(59,469)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(469,469)=jac(469,469)-k(22);
jac(32,469)=jac(32,469)+k(22);
jac(40,469)=jac(40,469)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(469,469)=jac(469,469)-k(16);
jac(3,469)=jac(3,469)+k(16);
jac(444,469)=jac(444,469)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(469,469)=jac(469,469)-k(10);
jac(3,469)=jac(3,469)+k(10);
jac(483,469)=jac(483,469)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(469,469)=jac(469,469)-k(8);
jac(131,469)=jac(131,469)+k(8);
jac(38,469)=jac(38,469)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(469,469)=jac(469,469)-k(6);
jac(37,469)=jac(37,469)+k(6);
jac(441,469)=jac(441,469)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(470,470)=jac(470,470)-k(33);
jac(50,470)=jac(50,470)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(470,470)=jac(470,470)-k(32);
jac(31,470)=jac(31,470)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(470,470)=jac(470,470)-k(31);
jac(384,470)=jac(384,470)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(470,470)=jac(470,470)-k(22);
jac(32,470)=jac(32,470)+k(22);
jac(57,470)=jac(57,470)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(470);
jac(197,470)=jac(197,470)-k(21)*y(197);
jac(470,197)=jac(470,197)-k(21)*y(470);
jac(470,470)=jac(470,470)-k(21)*y(197);
jac(439,197)=jac(439,197)+k(21)*y(470);
jac(439,470)=jac(439,470)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(470);
jac(32,470)=jac(32,470)-k(21)*y(32);
jac(470,32)=jac(470,32)-k(21)*y(470);
jac(470,470)=jac(470,470)-k(21)*y(32);
jac(444,32)=jac(444,32)+k(21)*y(470);
jac(444,470)=jac(444,470)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(470);
jac(2,470)=jac(2,470)-k(21)*y(2);
jac(470,2)=jac(470,2)-k(21)*y(470);
jac(470,470)=jac(470,470)-k(21)*y(2);
jac(382,2)=jac(382,2)+k(21)*y(470);
jac(382,470)=jac(382,470)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(470);
jac(3,470)=jac(3,470)-k(15)*y(3);
jac(470,3)=jac(470,3)-k(15)*y(470);
jac(470,470)=jac(470,470)-k(15)*y(3);
jac(40,3)=jac(40,3)+k(15)*y(470);
jac(40,470)=jac(40,470)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(470,470)=jac(470,470)-k(10);
jac(3,470)=jac(3,470)+k(10);
jac(617,470)=jac(617,470)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(470,470)=jac(470,470)-k(8);
jac(48,470)=jac(48,470)+k(8);
jac(34,470)=jac(34,470)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(470,470)=jac(470,470)-k(6);
jac(47,470)=jac(47,470)+k(6);
jac(33,470)=jac(33,470)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(472,472)=jac(472,472)-k(34);
jac(468,472)=jac(468,472)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(472,472)=jac(472,472)-k(33);
jac(427,472)=jac(427,472)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(472,472)=jac(472,472)-k(33);
jac(285,472)=jac(285,472)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(472,472)=jac(472,472)-k(31);
jac(454,472)=jac(454,472)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(472,472)=jac(472,472)-k(30);
jac(619,472)=jac(619,472)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(472,472)=jac(472,472)-k(22);
jac(32,472)=jac(32,472)+k(22);
jac(618,472)=jac(618,472)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(472,472)=jac(472,472)-k(22);
jac(197,472)=jac(197,472)+k(22);
jac(615,472)=jac(615,472)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(472);
jac(3,472)=jac(3,472)-k(15)*y(3);
jac(472,3)=jac(472,3)-k(15)*y(472);
jac(472,472)=jac(472,472)-k(15)*y(3);
jac(474,3)=jac(474,3)+k(15)*y(472);
jac(474,472)=jac(474,472)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(472);
jac(3,472)=jac(3,472)-k(9)*y(3);
jac(472,3)=jac(472,3)-k(9)*y(472);
jac(472,472)=jac(472,472)-k(9)*y(3);
jac(471,3)=jac(471,3)+k(9)*y(472);
jac(471,472)=jac(471,472)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(472,472)=jac(472,472)-k(8);
jac(27,472)=jac(27,472)+k(8);
jac(319,472)=jac(319,472)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(472,472)=jac(472,472)-k(6);
jac(257,472)=jac(257,472)+k(6);
jac(86,472)=jac(86,472)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(474,474)=jac(474,474)-k(34);
jac(483,474)=jac(483,474)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(474,474)=jac(474,474)-k(33);
jac(484,474)=jac(484,474)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(474,474)=jac(474,474)-k(33);
jac(283,474)=jac(283,474)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(474,474)=jac(474,474)-k(30);
jac(294,474)=jac(294,474)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(474,474)=jac(474,474)-k(22);
jac(32,474)=jac(32,474)+k(22);
jac(232,474)=jac(232,474)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(474,474)=jac(474,474)-k(22);
jac(197,474)=jac(197,474)+k(22);
jac(287,474)=jac(287,474)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(474,474)=jac(474,474)-k(18);
jac(635,474)=jac(635,474)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(474,474)=jac(474,474)-k(16);
jac(3,474)=jac(3,474)+k(16);
jac(472,474)=jac(472,474)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(474);
jac(3,474)=jac(3,474)-k(9)*y(3);
jac(474,3)=jac(474,3)-k(9)*y(474);
jac(474,474)=jac(474,474)-k(9)*y(3);
jac(473,3)=jac(473,3)+k(9)*y(474);
jac(473,474)=jac(473,474)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(474,474)=jac(474,474)-k(8);
jac(27,474)=jac(27,474)+k(8);
jac(228,474)=jac(228,474)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(474,474)=jac(474,474)-k(6);
jac(86,474)=jac(86,474)+k(6);
jac(227,474)=jac(227,474)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(649,649)=jac(649,649)-k(34);
jac(279,649)=jac(279,649)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(649,649)=jac(649,649)-k(34);
jac(270,649)=jac(270,649)+k(34);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(649,649)=jac(649,649)-k(26);
jac(289,649)=jac(289,649)+k(26);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(649,649)=jac(649,649)-k(23);
jac(476,649)=jac(476,649)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(649,649)=jac(649,649)-k(22);
jac(182,649)=jac(182,649)+k(22);
jac(225,649)=jac(225,649)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

jac(649,649)=jac(649,649)-k(22);
jac(182,649)=jac(182,649)+k(22);
jac(422,649)=jac(422,649)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(649,649)=jac(649,649)-k(16);
jac(3,649)=jac(3,649)+k(16);
jac(395,649)=jac(395,649)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(649,649)=jac(649,649)-k(10);
jac(3,649)=jac(3,649)+k(10);
jac(273,649)=jac(273,649)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(649,649)=jac(649,649)-k(8);
jac(184,649)=jac(184,649)+k(8);
jac(223,649)=jac(223,649)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(649,649)=jac(649,649)-k(6);
jac(222,649)=jac(222,649)+k(6);
jac(391,649)=jac(391,649)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(476,476)=jac(476,476)-k(34);
jac(428,476)=jac(428,476)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(476,476)=jac(476,476)-k(34);
jac(478,476)=jac(478,476)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(476,476)=jac(476,476)-k(33);
jac(649,476)=jac(649,476)+k(33);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(476,476)=jac(476,476)-k(26);
jac(475,476)=jac(475,476)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(476,476)=jac(476,476)-k(22);
jac(182,476)=jac(182,476)+k(22);
jac(205,476)=jac(205,476)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

jac(476,476)=jac(476,476)-k(22);
jac(197,476)=jac(197,476)+k(22);
jac(422,476)=jac(422,476)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(476,476)=jac(476,476)-k(16);
jac(3,476)=jac(3,476)+k(16);
jac(398,476)=jac(398,476)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(476,476)=jac(476,476)-k(10);
jac(3,476)=jac(3,476)+k(10);
jac(486,476)=jac(486,476)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(476,476)=jac(476,476)-k(8);
jac(184,476)=jac(184,476)+k(8);
jac(228,476)=jac(228,476)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(476,476)=jac(476,476)-k(6);
jac(227,476)=jac(227,476)+k(6);
jac(391,476)=jac(391,476)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(479,479)=jac(479,479)-k(34);
jac(481,479)=jac(481,479)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(479,479)=jac(479,479)-k(33);
jac(269,479)=jac(269,479)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(479,479)=jac(479,479)-k(29);
jac(486,479)=jac(486,479)+k(29);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(479,479)=jac(479,479)-k(26);
jac(293,479)=jac(293,479)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(479,479)=jac(479,479)-k(22);
jac(182,479)=jac(182,479)+k(22);
jac(249,479)=jac(249,479)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(479,479)=jac(479,479)-k(22);
jac(32,479)=jac(32,479)+k(22);
jac(272,479)=jac(272,479)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(479,479)=jac(479,479)-k(18);
jac(493,479)=jac(493,479)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(479,479)=jac(479,479)-k(16);
jac(3,479)=jac(3,479)+k(16);
jac(498,479)=jac(498,479)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(479);
jac(3,479)=jac(3,479)-k(9)*y(3);
jac(479,3)=jac(479,3)-k(9)*y(479);
jac(479,479)=jac(479,479)-k(9)*y(3);
jac(478,3)=jac(478,3)+k(9)*y(479);
jac(478,479)=jac(478,479)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(479,479)=jac(479,479)-k(8);
jac(177,479)=jac(177,479)+k(8);
jac(38,479)=jac(38,479)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(479,479)=jac(479,479)-k(6);
jac(176,479)=jac(176,479)+k(6);
jac(37,479)=jac(37,479)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(482,482)=jac(482,482)-k(33);
jac(275,482)=jac(275,482)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(482,482)=jac(482,482)-k(31);
jac(378,482)=jac(378,482)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(482,482)=jac(482,482)-k(28);
jac(498,482)=jac(498,482)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(482,482)=jac(482,482)-k(27);
jac(427,482)=jac(427,482)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(482,482)=jac(482,482)-k(22);
jac(2,482)=jac(2,482)+k(22);
jac(617,482)=jac(617,482)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(482,482)=jac(482,482)-k(22);
jac(32,482)=jac(32,482)+k(22);
jac(425,482)=jac(425,482)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(482);
jac(3,482)=jac(3,482)-k(15)*y(3);
jac(482,3)=jac(482,3)-k(15)*y(482);
jac(482,482)=jac(482,482)-k(15)*y(3);
jac(481,3)=jac(481,3)+k(15)*y(482);
jac(481,482)=jac(481,482)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(482);
jac(3,482)=jac(3,482)-k(9)*y(3);
jac(482,3)=jac(482,3)-k(9)*y(482);
jac(482,482)=jac(482,482)-k(9)*y(3);
jac(382,3)=jac(382,3)+k(9)*y(482);
jac(382,482)=jac(382,482)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(482,482)=jac(482,482)-k(8);
jac(21,482)=jac(21,482)+k(8);
jac(34,482)=jac(34,482)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(482,482)=jac(482,482)-k(6);
jac(33,482)=jac(33,482)+k(6);
jac(89,482)=jac(89,482)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(483,483)=jac(483,483)-k(33);
jac(481,483)=jac(481,483)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(483,483)=jac(483,483)-k(33);
jac(277,483)=jac(277,483)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(483,483)=jac(483,483)-k(28);
jac(293,483)=jac(293,483)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(483,483)=jac(483,483)-k(27);
jac(474,483)=jac(474,483)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(483,483)=jac(483,483)-k(22);
jac(32,483)=jac(32,483)+k(22);
jac(249,483)=jac(249,483)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(483,483)=jac(483,483)-k(22);
jac(32,483)=jac(32,483)+k(22);
jac(287,483)=jac(287,483)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(483,483)=jac(483,483)-k(18);
jac(247,483)=jac(247,483)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(483,483)=jac(483,483)-k(16);
jac(3,483)=jac(3,483)+k(16);
jac(468,483)=jac(468,483)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(483);
jac(3,483)=jac(3,483)-k(9)*y(3);
jac(483,3)=jac(483,3)-k(9)*y(483);
jac(483,483)=jac(483,483)-k(9)*y(3);
jac(469,3)=jac(469,3)+k(9)*y(483);
jac(469,483)=jac(469,483)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(483,483)=jac(483,483)-k(8);
jac(27,483)=jac(27,483)+k(8);
jac(38,483)=jac(38,483)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(483,483)=jac(483,483)-k(6);
jac(86,483)=jac(86,483)+k(6);
jac(37,483)=jac(37,483)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(485,485)=jac(485,485)-k(34);
jac(246,485)=jac(246,485)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(485,485)=jac(485,485)-k(33);
jac(281,485)=jac(281,485)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(485,485)=jac(485,485)-k(31);
jac(484,485)=jac(484,485)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(485,485)=jac(485,485)-k(30);
jac(488,485)=jac(488,485)+k(30);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(485,485)=jac(485,485)-k(26);
jac(635,485)=jac(635,485)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(485,485)=jac(485,485)-k(22);
jac(2,485)=jac(2,485)+k(22);
jac(491,485)=jac(491,485)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(485,485)=jac(485,485)-k(22);
jac(197,485)=jac(197,485)+k(22);
jac(629,485)=jac(629,485)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(485,485)=jac(485,485)-k(16);
jac(3,485)=jac(3,485)+k(16);
jac(502,485)=jac(502,485)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(485);
jac(3,485)=jac(3,485)-k(15)*y(3);
jac(485,3)=jac(485,3)-k(15)*y(485);
jac(485,485)=jac(485,485)-k(15)*y(3);
jac(632,3)=jac(632,3)+k(15)*y(485);
jac(632,485)=jac(632,485)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(485,485)=jac(485,485)-k(8);
jac(111,485)=jac(111,485)+k(8);
jac(228,485)=jac(228,485)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(485,485)=jac(485,485)-k(6);
jac(110,485)=jac(110,485)+k(6);
jac(227,485)=jac(227,485)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(487,487)=jac(487,487)-k(34);
jac(427,487)=jac(427,487)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(487,487)=jac(487,487)-k(34);
jac(498,487)=jac(498,487)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(487,487)=jac(487,487)-k(33);
jac(580,487)=jac(580,487)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(487,487)=jac(487,487)-k(31);
jac(369,487)=jac(369,487)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(487,487)=jac(487,487)-k(22);
jac(182,487)=jac(182,487)+k(22);
jac(618,487)=jac(618,487)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(487,487)=jac(487,487)-k(22);
jac(197,487)=jac(197,487)+k(22);
jac(421,487)=jac(421,487)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(487);
jac(3,487)=jac(3,487)-k(15)*y(3);
jac(487,3)=jac(487,3)-k(15)*y(487);
jac(487,487)=jac(487,487)-k(15)*y(3);
jac(486,3)=jac(486,3)+k(15)*y(487);
jac(486,487)=jac(486,487)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(487);
jac(3,487)=jac(3,487)-k(9)*y(3);
jac(487,3)=jac(487,3)-k(9)*y(487);
jac(487,487)=jac(487,487)-k(9)*y(3);
jac(398,3)=jac(398,3)+k(9)*y(487);
jac(398,487)=jac(398,487)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(487,487)=jac(487,487)-k(8);
jac(177,487)=jac(177,487)+k(8);
jac(319,487)=jac(319,487)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(487,487)=jac(487,487)-k(6);
jac(257,487)=jac(257,487)+k(6);
jac(176,487)=jac(176,487)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(489,489)=jac(489,489)-k(34);
jac(632,489)=jac(632,489)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(489,489)=jac(489,489)-k(34);
jac(494,489)=jac(494,489)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(489,489)=jac(489,489)-k(33);
jac(640,489)=jac(640,489)+k(33);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(489,489)=jac(489,489)-k(26);
jac(642,489)=jac(642,489)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

jac(489,489)=jac(489,489)-k(22);
jac(182,489)=jac(182,489)+k(22);
jac(636,489)=jac(636,489)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

jac(489,489)=jac(489,489)-k(22);
jac(197,489)=jac(197,489)+k(22);
jac(624,489)=jac(624,489)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(489,489)=jac(489,489)-k(16);
jac(3,489)=jac(3,489)+k(16);
jac(488,489)=jac(488,489)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(489,489)=jac(489,489)-k(16);
jac(3,489)=jac(3,489)+k(16);
jac(626,489)=jac(626,489)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(489,489)=jac(489,489)-k(8);
jac(222,489)=jac(222,489)+k(8);
jac(228,489)=jac(228,489)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(489,489)=jac(489,489)-k(6);
jac(227,489)=jac(227,489)+k(6);
jac(515,489)=jac(515,489)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(490,490)=jac(490,490)-k(34);
jac(502,490)=jac(502,490)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(490,490)=jac(490,490)-k(34);
jac(495,490)=jac(495,490)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(490,490)=jac(490,490)-k(33);
jac(579,490)=jac(579,490)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(490,490)=jac(490,490)-k(31);
jac(487,490)=jac(487,490)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(490,490)=jac(490,490)-k(31);
jac(611,490)=jac(611,490)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(490,490)=jac(490,490)-k(22);
jac(182,490)=jac(182,490)+k(22);
jac(613,490)=jac(613,490)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(490,490)=jac(490,490)-k(22);
jac(197,490)=jac(197,490)+k(22);
jac(497,490)=jac(497,490)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(490);
jac(3,490)=jac(3,490)-k(15)*y(3);
jac(490,3)=jac(490,3)-k(15)*y(490);
jac(490,490)=jac(490,490)-k(15)*y(3);
jac(488,3)=jac(488,3)+k(15)*y(490);
jac(488,490)=jac(488,490)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(490);
jac(3,490)=jac(3,490)-k(15)*y(3);
jac(490,3)=jac(490,3)-k(15)*y(490);
jac(490,490)=jac(490,490)-k(15)*y(3);
jac(626,3)=jac(626,3)+k(15)*y(490);
jac(626,490)=jac(626,490)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(490,490)=jac(490,490)-k(8);
jac(218,490)=jac(218,490)+k(8);
jac(319,490)=jac(319,490)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(490,490)=jac(490,490)-k(6);
jac(257,490)=jac(257,490)+k(6);
jac(264,490)=jac(264,490)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(491,491)=jac(491,491)-k(34);
jac(243,491)=jac(243,491)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(491,491)=jac(491,491)-k(33);
jac(645,491)=jac(645,491)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(491,491)=jac(491,491)-k(32);
jac(231,491)=jac(231,491)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(491,491)=jac(491,491)-k(31);
jac(232,491)=jac(232,491)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

jac(491,491)=jac(491,491)-k(22);
jac(197,491)=jac(197,491)+k(22);
jac(238,491)=jac(238,491)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(491);
jac(197,491)=jac(197,491)-k(21)*y(197);
jac(491,197)=jac(491,197)-k(21)*y(491);
jac(491,491)=jac(491,491)-k(21)*y(197);
jac(492,197)=jac(492,197)+k(21)*y(491);
jac(492,491)=jac(492,491)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(491);
jac(32,491)=jac(32,491)-k(21)*y(32);
jac(491,32)=jac(491,32)-k(21)*y(491);
jac(491,491)=jac(491,491)-k(21)*y(32);
jac(635,32)=jac(635,32)+k(21)*y(491);
jac(635,491)=jac(635,491)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(491);
jac(2,491)=jac(2,491)-k(21)*y(2);
jac(491,2)=jac(491,2)-k(21)*y(491);
jac(491,491)=jac(491,491)-k(21)*y(2);
jac(485,2)=jac(485,2)+k(21)*y(491);
jac(485,491)=jac(485,491)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(491,491)=jac(491,491)-k(16);
jac(3,491)=jac(3,491)+k(16);
jac(613,491)=jac(613,491)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(491);
jac(3,491)=jac(3,491)-k(15)*y(3);
jac(491,3)=jac(491,3)-k(15)*y(491);
jac(491,491)=jac(491,491)-k(15)*y(3);
jac(636,3)=jac(636,3)+k(15)*y(491);
jac(636,491)=jac(636,491)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

jac(491,491)=jac(491,491)-k(8);
jac(77,491)=jac(77,491)+k(8);
jac(228,491)=jac(228,491)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(491,491)=jac(491,491)-k(6);
jac(76,491)=jac(76,491)+k(6);
jac(227,491)=jac(227,491)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(492,492)=jac(492,492)-k(34);
jac(635,492)=jac(635,492)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(492,492)=jac(492,492)-k(34);
jac(248,492)=jac(248,492)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(492,492)=jac(492,492)-k(33);
jac(488,492)=jac(488,492)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(492,492)=jac(492,492)-k(33);
jac(261,492)=jac(261,492)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(492,492)=jac(492,492)-k(31);
jac(294,492)=jac(294,492)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(492,492)=jac(492,492)-k(22);
jac(197,492)=jac(197,492)+k(22);
jac(491,492)=jac(491,492)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(492,492)=jac(492,492)-k(22);
jac(197,492)=jac(197,492)+k(22);
jac(260,492)=jac(260,492)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(492,492)=jac(492,492)-k(16);
jac(3,492)=jac(3,492)+k(16);
jac(583,492)=jac(583,492)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(492);
jac(3,492)=jac(3,492)-k(15)*y(3);
jac(492,3)=jac(492,3)-k(15)*y(492);
jac(492,492)=jac(492,492)-k(15)*y(3);
jac(642,3)=jac(642,3)+k(15)*y(492);
jac(642,492)=jac(642,492)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(492,492)=jac(492,492)-k(8);
jac(257,492)=jac(257,492)+k(8);
jac(228,492)=jac(228,492)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(492,492)=jac(492,492)-k(6);
jac(256,492)=jac(256,492)+k(6);
jac(227,492)=jac(227,492)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(494,494)=jac(494,494)-k(34);
jac(631,494)=jac(631,494)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(494,494)=jac(494,494)-k(33);
jac(265,494)=jac(265,494)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(494,494)=jac(494,494)-k(29);
jac(489,494)=jac(489,494)+k(29);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(494,494)=jac(494,494)-k(26);
jac(643,494)=jac(643,494)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

jac(494,494)=jac(494,494)-k(22);
jac(182,494)=jac(182,494)+k(22);
jac(244,494)=jac(244,494)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

jac(494,494)=jac(494,494)-k(22);
jac(32,494)=jac(32,494)+k(22);
jac(624,494)=jac(624,494)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(494,494)=jac(494,494)-k(16);
jac(3,494)=jac(3,494)+k(16);
jac(493,494)=jac(493,494)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(494,494)=jac(494,494)-k(16);
jac(3,494)=jac(3,494)+k(16);
jac(496,494)=jac(496,494)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(494,494)=jac(494,494)-k(8);
jac(222,494)=jac(222,494)+k(8);
jac(38,494)=jac(38,494)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(494,494)=jac(494,494)-k(6);
jac(37,494)=jac(37,494)+k(6);
jac(515,494)=jac(515,494)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(496,496)=jac(496,496)-k(34);
jac(500,496)=jac(500,496)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(496,496)=jac(496,496)-k(33);
jac(625,496)=jac(625,496)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(496,496)=jac(496,496)-k(31);
jac(532,496)=jac(532,496)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(496,496)=jac(496,496)-k(29);
jac(626,496)=jac(626,496)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(496,496)=jac(496,496)-k(22);
jac(182,496)=jac(182,496)+k(22);
jac(534,496)=jac(534,496)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(496,496)=jac(496,496)-k(22);
jac(32,496)=jac(32,496)+k(22);
jac(623,496)=jac(623,496)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(496,496)=jac(496,496)-k(16);
jac(3,496)=jac(3,496)+k(16);
jac(495,496)=jac(495,496)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(496);
jac(3,496)=jac(3,496)-k(15)*y(3);
jac(496,3)=jac(496,3)-k(15)*y(496);
jac(496,496)=jac(496,496)-k(15)*y(3);
jac(494,3)=jac(494,3)+k(15)*y(496);
jac(494,496)=jac(494,496)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(496,496)=jac(496,496)-k(8);
jac(34,496)=jac(34,496)+k(8);
jac(222,496)=jac(222,496)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(496,496)=jac(496,496)-k(6);
jac(33,496)=jac(33,496)+k(6);
jac(515,496)=jac(515,496)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(497,497)=jac(497,497)-k(34);
jac(501,497)=jac(501,497)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(497,497)=jac(497,497)-k(32);
jac(302,497)=jac(302,497)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(497,497)=jac(497,497)-k(31);
jac(421,497)=jac(421,497)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(497,497)=jac(497,497)-k(31);
jac(512,497)=jac(512,497)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(497,497)=jac(497,497)-k(22);
jac(182,497)=jac(182,497)+k(22);
jac(584,497)=jac(584,497)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(497);
jac(197,497)=jac(197,497)-k(21)*y(197);
jac(497,197)=jac(497,197)-k(21)*y(497);
jac(497,497)=jac(497,497)-k(21)*y(197);
jac(490,197)=jac(490,197)+k(21)*y(497);
jac(490,497)=jac(490,497)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(497);
jac(32,497)=jac(32,497)-k(21)*y(32);
jac(497,32)=jac(497,32)-k(21)*y(497);
jac(497,497)=jac(497,497)-k(21)*y(32);
jac(495,32)=jac(495,32)+k(21)*y(497);
jac(495,497)=jac(495,497)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(497);
jac(2,497)=jac(2,497)-k(21)*y(2);
jac(497,2)=jac(497,2)-k(21)*y(497);
jac(497,497)=jac(497,497)-k(21)*y(2);
jac(266,2)=jac(266,2)+k(21)*y(497);
jac(266,497)=jac(266,497)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(497);
jac(3,497)=jac(3,497)-k(15)*y(3);
jac(497,3)=jac(497,3)-k(15)*y(497);
jac(497,497)=jac(497,497)-k(15)*y(3);
jac(267,3)=jac(267,3)+k(15)*y(497);
jac(267,497)=jac(267,497)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(497);
jac(3,497)=jac(3,497)-k(15)*y(3);
jac(497,3)=jac(497,3)-k(15)*y(497);
jac(497,497)=jac(497,497)-k(15)*y(3);
jac(623,3)=jac(623,3)+k(15)*y(497);
jac(623,497)=jac(623,497)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(497,497)=jac(497,497)-k(8);
jac(315,497)=jac(315,497)+k(8);
jac(218,497)=jac(218,497)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

jac(497,497)=jac(497,497)-k(6);
jac(77,497)=jac(77,497)+k(6);
jac(264,497)=jac(264,497)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(498,498)=jac(498,498)-k(34);
jac(482,498)=jac(482,498)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(498,498)=jac(498,498)-k(33);
jac(271,498)=jac(271,498)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(498,498)=jac(498,498)-k(31);
jac(366,498)=jac(366,498)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(498,498)=jac(498,498)-k(29);
jac(487,498)=jac(487,498)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(498,498)=jac(498,498)-k(22);
jac(182,498)=jac(182,498)+k(22);
jac(617,498)=jac(617,498)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(498,498)=jac(498,498)-k(22);
jac(32,498)=jac(32,498)+k(22);
jac(421,498)=jac(421,498)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(498);
jac(3,498)=jac(3,498)-k(15)*y(3);
jac(498,3)=jac(498,3)-k(15)*y(498);
jac(498,498)=jac(498,498)-k(15)*y(3);
jac(479,3)=jac(479,3)+k(15)*y(498);
jac(479,498)=jac(479,498)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(498);
jac(3,498)=jac(3,498)-k(9)*y(3);
jac(498,3)=jac(498,3)-k(9)*y(498);
jac(498,498)=jac(498,498)-k(9)*y(3);
jac(423,3)=jac(423,3)+k(9)*y(498);
jac(423,498)=jac(423,498)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(498,498)=jac(498,498)-k(8);
jac(177,498)=jac(177,498)+k(8);
jac(34,498)=jac(34,498)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(498,498)=jac(498,498)-k(6);
jac(33,498)=jac(33,498)+k(6);
jac(176,498)=jac(176,498)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(500,500)=jac(500,500)-k(33);
jac(504,500)=jac(504,500)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(500,500)=jac(500,500)-k(31);
jac(530,500)=jac(530,500)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(500,500)=jac(500,500)-k(28);
jac(496,500)=jac(496,500)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(500,500)=jac(500,500)-k(27);
jac(528,500)=jac(528,500)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(500,500)=jac(500,500)-k(22);
jac(2,500)=jac(2,500)+k(22);
jac(534,500)=jac(534,500)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(500,500)=jac(500,500)-k(22);
jac(32,500)=jac(32,500)+k(22);
jac(308,500)=jac(308,500)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(500,500)=jac(500,500)-k(16);
jac(3,500)=jac(3,500)+k(16);
jac(499,500)=jac(499,500)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(500);
jac(3,500)=jac(3,500)-k(15)*y(3);
jac(500,3)=jac(500,3)-k(15)*y(500);
jac(500,500)=jac(500,500)-k(15)*y(3);
jac(631,3)=jac(631,3)+k(15)*y(500);
jac(631,500)=jac(631,500)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(500,500)=jac(500,500)-k(8);
jac(34,500)=jac(34,500)+k(8);
jac(43,500)=jac(43,500)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(500,500)=jac(500,500)-k(6);
jac(33,500)=jac(33,500)+k(6);
jac(114,500)=jac(114,500)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(501,501)=jac(501,501)-k(32);
jac(305,501)=jac(305,501)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(501,501)=jac(501,501)-k(31);
jac(425,501)=jac(425,501)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(501,501)=jac(501,501)-k(31);
jac(354,501)=jac(354,501)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(501,501)=jac(501,501)-k(22);
jac(2,501)=jac(2,501)+k(22);
jac(584,501)=jac(584,501)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(501);
jac(197,501)=jac(197,501)-k(21)*y(197);
jac(501,197)=jac(501,197)-k(21)*y(501);
jac(501,501)=jac(501,501)-k(21)*y(197);
jac(502,197)=jac(502,197)+k(21)*y(501);
jac(502,501)=jac(502,501)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(501);
jac(32,501)=jac(32,501)-k(21)*y(32);
jac(501,32)=jac(501,32)-k(21)*y(501);
jac(501,501)=jac(501,501)-k(21)*y(32);
jac(499,32)=jac(499,32)+k(21)*y(501);
jac(499,501)=jac(499,501)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(501);
jac(2,501)=jac(2,501)-k(21)*y(2);
jac(501,2)=jac(501,2)-k(21)*y(501);
jac(501,501)=jac(501,501)-k(21)*y(2);
jac(503,2)=jac(503,2)+k(21)*y(501);
jac(503,501)=jac(503,501)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(501);
jac(3,501)=jac(3,501)-k(15)*y(3);
jac(501,3)=jac(501,3)-k(15)*y(501);
jac(501,501)=jac(501,501)-k(15)*y(3);
jac(629,3)=jac(629,3)+k(15)*y(501);
jac(629,501)=jac(629,501)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(501);
jac(3,501)=jac(3,501)-k(15)*y(3);
jac(501,3)=jac(501,3)-k(15)*y(501);
jac(501,501)=jac(501,501)-k(15)*y(3);
jac(308,3)=jac(308,3)+k(15)*y(501);
jac(308,501)=jac(308,501)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(501,501)=jac(501,501)-k(8);
jac(315,501)=jac(315,501)+k(8);
jac(111,501)=jac(111,501)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

jac(501,501)=jac(501,501)-k(6);
jac(77,501)=jac(77,501)+k(6);
jac(110,501)=jac(110,501)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(502,502)=jac(502,502)-k(34);
jac(499,502)=jac(499,502)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(502,502)=jac(502,502)-k(33);
jac(505,502)=jac(505,502)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(502,502)=jac(502,502)-k(31);
jac(427,502)=jac(427,502)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(502,502)=jac(502,502)-k(31);
jac(527,502)=jac(527,502)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(502,502)=jac(502,502)-k(30);
jac(490,502)=jac(490,502)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(502,502)=jac(502,502)-k(22);
jac(2,502)=jac(2,502)+k(22);
jac(613,502)=jac(613,502)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(502,502)=jac(502,502)-k(22);
jac(197,502)=jac(197,502)+k(22);
jac(501,502)=jac(501,502)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(502);
jac(3,502)=jac(3,502)-k(15)*y(3);
jac(502,3)=jac(502,3)-k(15)*y(502);
jac(502,502)=jac(502,502)-k(15)*y(3);
jac(485,3)=jac(485,3)+k(15)*y(502);
jac(485,502)=jac(485,502)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(502);
jac(3,502)=jac(3,502)-k(15)*y(3);
jac(502,3)=jac(502,3)-k(15)*y(502);
jac(502,502)=jac(502,502)-k(15)*y(3);
jac(528,3)=jac(528,3)+k(15)*y(502);
jac(528,502)=jac(528,502)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(502,502)=jac(502,502)-k(8);
jac(111,502)=jac(111,502)+k(8);
jac(319,502)=jac(319,502)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(502,502)=jac(502,502)-k(6);
jac(257,502)=jac(257,502)+k(6);
jac(110,502)=jac(110,502)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(504,504)=jac(504,504)-k(31);
jac(507,504)=jac(507,504)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(504,504)=jac(504,504)-k(28);
jac(625,504)=jac(625,504)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(504,504)=jac(504,504)-k(27);
jac(523,504)=jac(523,504)+k(27);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(504,504)=jac(504,504)-k(25);
jac(500,504)=jac(500,504)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(504,504)=jac(504,504)-k(22);
jac(2,504)=jac(2,504)+k(22);
jac(540,504)=jac(540,504)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(504,504)=jac(504,504)-k(22);
jac(2,504)=jac(2,504)+k(22);
jac(308,504)=jac(308,504)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(504,504)=jac(504,504)-k(16);
jac(3,504)=jac(3,504)+k(16);
jac(503,504)=jac(503,504)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(504);
jac(3,504)=jac(3,504)-k(15)*y(3);
jac(504,3)=jac(504,3)-k(15)*y(504);
jac(504,504)=jac(504,504)-k(15)*y(3);
jac(630,3)=jac(630,3)+k(15)*y(504);
jac(630,504)=jac(630,504)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(504,504)=jac(504,504)-k(8);
jac(437,504)=jac(437,504)+k(8);
jac(43,504)=jac(43,504)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(504,504)=jac(504,504)-k(6);
jac(111,504)=jac(111,504)+k(6);
jac(114,504)=jac(114,504)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(505,505)=jac(505,505)-k(34);
jac(503,505)=jac(503,505)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(505,505)=jac(505,505)-k(31);
jac(280,505)=jac(280,505)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(505,505)=jac(505,505)-k(31);
jac(509,505)=jac(509,505)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(505,505)=jac(505,505)-k(30);
jac(579,505)=jac(579,505)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(505,505)=jac(505,505)-k(22);
jac(2,505)=jac(2,505)+k(22);
jac(576,505)=jac(576,505)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(505,505)=jac(505,505)-k(22);
jac(182,505)=jac(182,505)+k(22);
jac(501,505)=jac(501,505)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(505);
jac(3,505)=jac(3,505)-k(15)*y(3);
jac(505,3)=jac(505,3)-k(15)*y(505);
jac(505,505)=jac(505,505)-k(15)*y(3);
jac(281,3)=jac(281,3)+k(15)*y(505);
jac(281,505)=jac(281,505)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(505);
jac(3,505)=jac(3,505)-k(15)*y(3);
jac(505,3)=jac(505,3)-k(15)*y(505);
jac(505,505)=jac(505,505)-k(15)*y(3);
jac(523,3)=jac(523,3)+k(15)*y(505);
jac(523,505)=jac(523,505)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(505,505)=jac(505,505)-k(8);
jac(111,505)=jac(111,505)+k(8);
jac(219,505)=jac(219,505)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(505,505)=jac(505,505)-k(6);
jac(218,505)=jac(218,505)+k(6);
jac(110,505)=jac(110,505)+k(6);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(507,507)=jac(507,507)-k(28);
jac(511,507)=jac(511,507)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(507,507)=jac(507,507)-k(27);
jac(521,507)=jac(521,507)+k(27);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(507,507)=jac(507,507)-k(25);
jac(530,507)=jac(530,507)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(507,507)=jac(507,507)-k(22);
jac(2,507)=jac(2,507)+k(22);
jac(538,507)=jac(538,507)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(507,507)=jac(507,507)-k(22);
jac(2,507)=jac(2,507)+k(22);
jac(524,507)=jac(524,507)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(507,507)=jac(507,507)-k(17);
jac(504,507)=jac(504,507)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(507,507)=jac(507,507)-k(16);
jac(3,507)=jac(3,507)+k(16);
jac(506,507)=jac(506,507)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(507);
jac(3,507)=jac(3,507)-k(9)*y(3);
jac(507,3)=jac(507,3)-k(9)*y(507);
jac(507,507)=jac(507,507)-k(9)*y(3);
jac(604,3)=jac(604,3)+k(9)*y(507);
jac(604,507)=jac(604,507)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(507,507)=jac(507,507)-k(8);
jac(22,507)=jac(22,507)+k(8);
jac(43,507)=jac(43,507)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(507,507)=jac(507,507)-k(6);
jac(21,507)=jac(21,507)+k(6);
jac(114,507)=jac(114,507)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(508,508)=jac(508,508)-k(32);
jac(142,508)=jac(142,508)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(508,508)=jac(508,508)-k(31);
jac(452,508)=jac(452,508)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(508,508)=jac(508,508)-k(22);
jac(2,508)=jac(2,508)+k(22);
jac(610,508)=jac(610,508)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(508);
jac(197,508)=jac(197,508)-k(21)*y(197);
jac(508,197)=jac(508,197)-k(21)*y(508);
jac(508,508)=jac(508,508)-k(21)*y(197);
jac(545,197)=jac(545,197)+k(21)*y(508);
jac(545,508)=jac(545,508)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(508);
jac(32,508)=jac(32,508)-k(21)*y(32);
jac(508,32)=jac(508,32)-k(21)*y(508);
jac(508,508)=jac(508,508)-k(21)*y(32);
jac(575,32)=jac(575,32)+k(21)*y(508);
jac(575,508)=jac(575,508)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(508);
jac(2,508)=jac(2,508)-k(21)*y(2);
jac(508,2)=jac(508,2)-k(21)*y(508);
jac(508,508)=jac(508,508)-k(21)*y(2);
jac(506,2)=jac(506,2)+k(21)*y(508);
jac(506,508)=jac(506,508)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(508);
jac(3,508)=jac(3,508)-k(15)*y(3);
jac(508,3)=jac(508,3)-k(15)*y(508);
jac(508,508)=jac(508,508)-k(15)*y(3);
jac(538,3)=jac(538,3)+k(15)*y(508);
jac(538,508)=jac(538,508)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(508);
jac(3,508)=jac(3,508)-k(9)*y(3);
jac(508,3)=jac(508,3)-k(9)*y(508);
jac(508,508)=jac(508,508)-k(9)*y(3);
jac(143,3)=jac(143,3)+k(9)*y(508);
jac(143,508)=jac(143,508)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

jac(508,508)=jac(508,508)-k(8);
jac(77,508)=jac(77,508)+k(8);
jac(22,508)=jac(22,508)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(508,508)=jac(508,508)-k(6);
jac(76,508)=jac(76,508)+k(6);
jac(21,508)=jac(21,508)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(509,509)=jac(509,509)-k(34);
jac(506,509)=jac(506,509)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(509,509)=jac(509,509)-k(31);
jac(377,509)=jac(377,509)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(509,509)=jac(509,509)-k(30);
jac(513,509)=jac(513,509)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(509,509)=jac(509,509)-k(22);
jac(2,509)=jac(2,509)+k(22);
jac(344,509)=jac(344,509)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(509,509)=jac(509,509)-k(22);
jac(182,509)=jac(182,509)+k(22);
jac(354,509)=jac(354,509)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(509);
jac(3,509)=jac(3,509)-k(15)*y(3);
jac(509,3)=jac(509,3)-k(15)*y(509);
jac(509,509)=jac(509,509)-k(15)*y(3);
jac(521,3)=jac(521,3)+k(15)*y(509);
jac(521,509)=jac(521,509)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(509);
jac(3,509)=jac(3,509)-k(9)*y(3);
jac(509,3)=jac(509,3)-k(9)*y(509);
jac(509,509)=jac(509,509)-k(9)*y(3);
jac(346,3)=jac(346,3)+k(9)*y(509);
jac(346,509)=jac(346,509)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(509,509)=jac(509,509)-k(8);
jac(111,509)=jac(111,509)+k(8);
jac(214,509)=jac(214,509)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(509,509)=jac(509,509)-k(6);
jac(177,509)=jac(177,509)+k(6);
jac(110,509)=jac(110,509)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(511,511)=jac(511,511)-k(34);
jac(507,511)=jac(507,511)+k(34);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(511,511)=jac(511,511)-k(29);
jac(514,511)=jac(514,511)+k(29);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(511,511)=jac(511,511)-k(25);
jac(532,511)=jac(532,511)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(511,511)=jac(511,511)-k(22);
jac(182,511)=jac(182,511)+k(22);
jac(538,511)=jac(538,511)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(511,511)=jac(511,511)-k(22);
jac(2,511)=jac(2,511)+k(22);
jac(518,511)=jac(518,511)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(511,511)=jac(511,511)-k(17);
jac(625,511)=jac(625,511)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(511,511)=jac(511,511)-k(16);
jac(3,511)=jac(3,511)+k(16);
jac(510,511)=jac(510,511)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(511);
jac(3,511)=jac(3,511)-k(9)*y(3);
jac(511,3)=jac(511,3)-k(9)*y(511);
jac(511,511)=jac(511,511)-k(9)*y(3);
jac(596,3)=jac(596,3)+k(9)*y(511);
jac(596,511)=jac(596,511)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(511,511)=jac(511,511)-k(8);
jac(22,511)=jac(22,511)+k(8);
jac(222,511)=jac(222,511)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(511,511)=jac(511,511)-k(6);
jac(21,511)=jac(21,511)+k(6);
jac(515,511)=jac(515,511)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(648,648)=jac(648,648)-k(34);
jac(375,648)=jac(375,648)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(648,648)=jac(648,648)-k(32);
jac(418,648)=jac(418,648)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(648,648)=jac(648,648)-k(22);
jac(182,648)=jac(182,648)+k(22);
jac(150,648)=jac(150,648)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(648);
jac(197,648)=jac(197,648)-k(21)*y(197);
jac(648,197)=jac(648,197)-k(21)*y(648);
jac(648,648)=jac(648,648)-k(21)*y(197);
jac(369,197)=jac(369,197)+k(21)*y(648);
jac(369,648)=jac(369,648)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(648);
jac(32,648)=jac(32,648)-k(21)*y(32);
jac(648,32)=jac(648,32)-k(21)*y(648);
jac(648,648)=jac(648,648)-k(21)*y(32);
jac(366,32)=jac(366,32)+k(21)*y(648);
jac(366,648)=jac(366,648)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(648);
jac(2,648)=jac(2,648)-k(21)*y(2);
jac(648,2)=jac(648,2)-k(21)*y(648);
jac(648,648)=jac(648,648)-k(21)*y(2);
jac(179,2)=jac(179,2)+k(21)*y(648);
jac(179,648)=jac(179,648)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(648);
jac(3,648)=jac(3,648)-k(9)*y(3);
jac(648,3)=jac(648,3)-k(9)*y(648);
jac(648,648)=jac(648,648)-k(9)*y(3);
jac(181,3)=jac(181,3)+k(9)*y(648);
jac(181,648)=jac(181,648)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(648);
jac(3,648)=jac(3,648)-k(9)*y(3);
jac(648,3)=jac(648,3)-k(9)*y(648);
jac(648,648)=jac(648,648)-k(9)*y(3);
jac(393,3)=jac(393,3)+k(9)*y(648);
jac(393,648)=jac(393,648)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(648,648)=jac(648,648)-k(8);
jac(16,648)=jac(16,648)+k(8);
jac(177,648)=jac(177,648)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

jac(648,648)=jac(648,648)-k(6);
jac(15,648)=jac(15,648)+k(6);
jac(176,648)=jac(176,648)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(512,512)=jac(512,512)-k(34);
jac(354,512)=jac(354,512)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(512,512)=jac(512,512)-k(32);
jac(304,512)=jac(304,512)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(512,512)=jac(512,512)-k(31);
jac(648,512)=jac(648,512)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(512,512)=jac(512,512)-k(22);
jac(182,512)=jac(182,512)+k(22);
jac(610,512)=jac(610,512)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(512);
jac(197,512)=jac(197,512)-k(21)*y(197);
jac(512,197)=jac(512,197)-k(21)*y(512);
jac(512,512)=jac(512,512)-k(21)*y(197);
jac(611,197)=jac(611,197)+k(21)*y(512);
jac(611,512)=jac(611,512)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(512);
jac(32,512)=jac(32,512)-k(21)*y(32);
jac(512,32)=jac(512,32)-k(21)*y(512);
jac(512,512)=jac(512,512)-k(21)*y(32);
jac(359,32)=jac(359,32)+k(21)*y(512);
jac(359,512)=jac(359,512)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(512);
jac(2,512)=jac(2,512)-k(21)*y(2);
jac(512,2)=jac(512,2)-k(21)*y(512);
jac(512,512)=jac(512,512)-k(21)*y(2);
jac(510,2)=jac(510,2)+k(21)*y(512);
jac(510,512)=jac(510,512)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(512);
jac(3,512)=jac(3,512)-k(15)*y(3);
jac(512,3)=jac(512,3)-k(15)*y(512);
jac(512,512)=jac(512,512)-k(15)*y(3);
jac(518,3)=jac(518,3)+k(15)*y(512);
jac(518,512)=jac(518,512)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(512);
jac(3,512)=jac(3,512)-k(9)*y(3);
jac(512,3)=jac(512,3)-k(9)*y(512);
jac(512,512)=jac(512,512)-k(9)*y(3);
jac(361,3)=jac(361,3)+k(9)*y(512);
jac(361,512)=jac(361,512)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(512,512)=jac(512,512)-k(8);
jac(16,512)=jac(16,512)+k(8);
jac(218,512)=jac(218,512)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

jac(512,512)=jac(512,512)-k(6);
jac(15,512)=jac(15,512)+k(6);
jac(264,512)=jac(264,512)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(515,515)=jac(515,515)-k(34);
jac(114,515)=jac(114,515)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

jac(515,515)=jac(515,515)-k(22);
jac(182,515)=jac(182,515)+k(22);
jac(118,515)=jac(118,515)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(515,515)=jac(515,515)-k(16);
jac(3,515)=jac(3,515)+k(16);
jac(264,515)=jac(264,515)+k(16);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(1,1)=jac(1,1)-k(5)*y(515);
jac(1,515)=jac(1,515)-k(5)*y(1);
jac(515,1)=jac(515,1)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(1);
jac(591,1)=jac(591,1)+k(5)*y(515);
jac(591,515)=jac(591,515)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(7,7)=jac(7,7)-k(5)*y(515);
jac(7,515)=jac(7,515)-k(5)*y(7);
jac(515,7)=jac(515,7)-k(5)*y(515);
jac(515,515)=jac(515,515)-k(5)*y(7);
jac(593,7)=jac(593,7)+k(5)*y(515);
jac(593,515)=jac(593,515)+k(5)*y(7);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Lig(l1, l2) 

jac(515,515)=jac(515,515)-k(4);
jac(4,515)=jac(4,515)+k(4);
jac(222,515)=jac(222,515)+k(4);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(516,516)=jac(516,516)-k(34);
jac(596,516)=jac(596,516)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(516,516)=jac(516,516)-k(34);
jac(522,516)=jac(522,516)+k(34);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(516,516)=jac(516,516)-k(25);
jac(597,516)=jac(597,516)+k(25);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(516,516)=jac(516,516)-k(24);
jac(549,516)=jac(549,516)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(516,516)=jac(516,516)-k(22);
jac(182,516)=jac(182,516)+k(22);
jac(595,516)=jac(595,516)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

jac(516,516)=jac(516,516)-k(22);
jac(182,516)=jac(182,516)+k(22);
jac(345,516)=jac(345,516)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(516,516)=jac(516,516)-k(16);
jac(3,516)=jac(3,516)+k(16);
jac(188,516)=jac(188,516)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(516,516)=jac(516,516)-k(10);
jac(3,516)=jac(3,516)+k(10);
jac(514,516)=jac(514,516)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(516,516)=jac(516,516)-k(8);
jac(222,516)=jac(222,516)+k(8);
jac(185,516)=jac(185,516)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(516,516)=jac(516,516)-k(6);
jac(184,516)=jac(184,516)+k(6);
jac(515,516)=jac(515,516)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(517,517)=jac(517,517)-k(34);
jac(523,517)=jac(523,517)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(517,517)=jac(517,517)-k(34);
jac(625,517)=jac(625,517)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(517,517)=jac(517,517)-k(31);
jac(514,517)=jac(514,517)+k(31);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(517,517)=jac(517,517)-k(25);
jac(626,517)=jac(626,517)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(517,517)=jac(517,517)-k(22);
jac(182,517)=jac(182,517)+k(22);
jac(644,517)=jac(644,517)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(517,517)=jac(517,517)-k(22);
jac(182,517)=jac(182,517)+k(22);
jac(623,517)=jac(623,517)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(517,517)=jac(517,517)-k(16);
jac(3,517)=jac(3,517)+k(16);
jac(579,517)=jac(579,517)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(517);
jac(3,517)=jac(3,517)-k(15)*y(3);
jac(517,3)=jac(517,3)-k(15)*y(517);
jac(517,517)=jac(517,517)-k(15)*y(3);
jac(640,3)=jac(640,3)+k(15)*y(517);
jac(640,517)=jac(640,517)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(517,517)=jac(517,517)-k(8);
jac(219,517)=jac(219,517)+k(8);
jac(222,517)=jac(222,517)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(517,517)=jac(517,517)-k(6);
jac(218,517)=jac(218,517)+k(6);
jac(515,517)=jac(515,517)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(518,518)=jac(518,518)-k(34);
jac(524,518)=jac(524,518)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(518,518)=jac(518,518)-k(32);
jac(591,518)=jac(591,518)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

jac(518,518)=jac(518,518)-k(22);
jac(182,518)=jac(182,518)+k(22);
jac(537,518)=jac(537,518)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(518);
jac(197,518)=jac(197,518)-k(21)*y(197);
jac(518,197)=jac(518,197)-k(21)*y(518);
jac(518,518)=jac(518,518)-k(21)*y(197);
jac(520,197)=jac(520,197)+k(21)*y(518);
jac(520,518)=jac(520,518)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(518);
jac(32,518)=jac(32,518)-k(21)*y(32);
jac(518,32)=jac(518,32)-k(21)*y(518);
jac(518,518)=jac(518,518)-k(21)*y(32);
jac(532,32)=jac(532,32)+k(21)*y(518);
jac(532,518)=jac(532,518)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(518);
jac(2,518)=jac(2,518)-k(21)*y(2);
jac(518,2)=jac(518,2)-k(21)*y(518);
jac(518,518)=jac(518,518)-k(21)*y(2);
jac(511,2)=jac(511,2)+k(21)*y(518);
jac(511,518)=jac(511,518)+k(21)*y(2);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(518,518)=jac(518,518)-k(17);
jac(623,518)=jac(623,518)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(518,518)=jac(518,518)-k(16);
jac(3,518)=jac(3,518)+k(16);
jac(512,518)=jac(512,518)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(518);
jac(3,518)=jac(3,518)-k(9)*y(3);
jac(518,3)=jac(518,3)-k(9)*y(518);
jac(518,518)=jac(518,518)-k(9)*y(3);
jac(595,3)=jac(595,3)+k(9)*y(518);
jac(595,518)=jac(595,518)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(518,518)=jac(518,518)-k(8);
jac(16,518)=jac(16,518)+k(8);
jac(222,518)=jac(222,518)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

jac(518,518)=jac(518,518)-k(6);
jac(15,518)=jac(15,518)+k(6);
jac(515,518)=jac(515,518)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(519,519)=jac(519,519)-k(34);
jac(538,519)=jac(538,519)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(519,519)=jac(519,519)-k(32);
jac(326,519)=jac(326,519)+k(32);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(519,519)=jac(519,519)-k(25);
jac(529,519)=jac(529,519)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

jac(519,519)=jac(519,519)-k(22);
jac(182,519)=jac(182,519)+k(22);
jac(537,519)=jac(537,519)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(519);
jac(197,519)=jac(197,519)-k(21)*y(197);
jac(519,197)=jac(519,197)-k(21)*y(519);
jac(519,519)=jac(519,519)-k(21)*y(197);
jac(548,197)=jac(548,197)+k(21)*y(519);
jac(548,519)=jac(548,519)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(519);
jac(32,519)=jac(32,519)-k(21)*y(32);
jac(519,32)=jac(519,32)-k(21)*y(519);
jac(519,519)=jac(519,519)-k(21)*y(32);
jac(553,32)=jac(553,32)+k(21)*y(519);
jac(553,519)=jac(553,519)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(519);
jac(2,519)=jac(2,519)-k(21)*y(2);
jac(519,2)=jac(519,2)-k(21)*y(519);
jac(519,519)=jac(519,519)-k(21)*y(2);
jac(521,2)=jac(521,2)+k(21)*y(519);
jac(521,519)=jac(521,519)+k(21)*y(2);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(519,519)=jac(519,519)-k(17);
jac(644,519)=jac(644,519)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(519,519)=jac(519,519)-k(16);
jac(3,519)=jac(3,519)+k(16);
jac(344,519)=jac(344,519)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(519);
jac(3,519)=jac(3,519)-k(9)*y(3);
jac(519,3)=jac(519,3)-k(9)*y(519);
jac(519,519)=jac(519,519)-k(9)*y(3);
jac(345,3)=jac(345,3)+k(9)*y(519);
jac(345,519)=jac(345,519)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(519,519)=jac(519,519)-k(8);
jac(54,519)=jac(54,519)+k(8);
jac(214,519)=jac(214,519)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(519,519)=jac(519,519)-k(6);
jac(118,519)=jac(118,519)+k(6);
jac(177,519)=jac(177,519)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(520,520)=jac(520,520)-k(34);
jac(525,520)=jac(525,520)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(520,520)=jac(520,520)-k(34);
jac(532,520)=jac(532,520)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(520,520)=jac(520,520)-k(33);
jac(514,520)=jac(514,520)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(520,520)=jac(520,520)-k(22);
jac(182,520)=jac(182,520)+k(22);
jac(529,520)=jac(529,520)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(520,520)=jac(520,520)-k(22);
jac(197,520)=jac(197,520)+k(22);
jac(518,520)=jac(518,520)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(520,520)=jac(520,520)-k(17);
jac(626,520)=jac(626,520)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(520,520)=jac(520,520)-k(16);
jac(3,520)=jac(3,520)+k(16);
jac(611,520)=jac(611,520)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(520);
jac(3,520)=jac(3,520)-k(9)*y(3);
jac(520,3)=jac(520,3)-k(9)*y(520);
jac(520,520)=jac(520,520)-k(9)*y(3);
jac(597,3)=jac(597,3)+k(9)*y(520);
jac(597,520)=jac(597,520)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(520,520)=jac(520,520)-k(8);
jac(209,520)=jac(209,520)+k(8);
jac(222,520)=jac(222,520)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(520,520)=jac(520,520)-k(6);
jac(192,520)=jac(192,520)+k(6);
jac(515,520)=jac(515,520)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(522,522)=jac(522,522)-k(34);
jac(604,522)=jac(604,522)+k(34);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(522,522)=jac(522,522)-k(30);
jac(516,522)=jac(516,522)+k(30);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(522,522)=jac(522,522)-k(25);
jac(526,522)=jac(526,522)+k(25);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(522,522)=jac(522,522)-k(24);
jac(349,522)=jac(349,522)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(522,522)=jac(522,522)-k(22);
jac(182,522)=jac(182,522)+k(22);
jac(116,522)=jac(116,522)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

jac(522,522)=jac(522,522)-k(22);
jac(2,522)=jac(2,522)+k(22);
jac(345,522)=jac(345,522)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(522,522)=jac(522,522)-k(16);
jac(3,522)=jac(3,522)+k(16);
jac(346,522)=jac(346,522)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(522,522)=jac(522,522)-k(10);
jac(3,522)=jac(3,522)+k(10);
jac(521,522)=jac(521,522)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(522,522)=jac(522,522)-k(8);
jac(43,522)=jac(43,522)+k(8);
jac(185,522)=jac(185,522)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(522,522)=jac(522,522)-k(6);
jac(184,522)=jac(184,522)+k(6);
jac(114,522)=jac(114,522)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(523,523)=jac(523,523)-k(34);
jac(504,523)=jac(504,523)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(523,523)=jac(523,523)-k(31);
jac(521,523)=jac(521,523)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(523,523)=jac(523,523)-k(30);
jac(517,523)=jac(517,523)+k(30);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(523,523)=jac(523,523)-k(25);
jac(528,523)=jac(528,523)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(523,523)=jac(523,523)-k(22);
jac(2,523)=jac(2,523)+k(22);
jac(644,523)=jac(644,523)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(523,523)=jac(523,523)-k(22);
jac(182,523)=jac(182,523)+k(22);
jac(308,523)=jac(308,523)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(523,523)=jac(523,523)-k(16);
jac(3,523)=jac(3,523)+k(16);
jac(505,523)=jac(505,523)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(523);
jac(3,523)=jac(3,523)-k(15)*y(3);
jac(523,3)=jac(523,3)-k(15)*y(523);
jac(523,523)=jac(523,523)-k(15)*y(3);
jac(637,3)=jac(637,3)+k(15)*y(523);
jac(637,523)=jac(637,523)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(523,523)=jac(523,523)-k(8);
jac(219,523)=jac(219,523)+k(8);
jac(43,523)=jac(43,523)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(523,523)=jac(523,523)-k(6);
jac(218,523)=jac(218,523)+k(6);
jac(114,523)=jac(114,523)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(524,524)=jac(524,524)-k(32);
jac(115,524)=jac(115,524)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

jac(524,524)=jac(524,524)-k(22);
jac(2,524)=jac(2,524)+k(22);
jac(537,524)=jac(537,524)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(524);
jac(197,524)=jac(197,524)-k(21)*y(197);
jac(524,197)=jac(524,197)-k(21)*y(524);
jac(524,524)=jac(524,524)-k(21)*y(197);
jac(525,197)=jac(525,197)+k(21)*y(524);
jac(525,524)=jac(525,524)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(524);
jac(32,524)=jac(32,524)-k(21)*y(32);
jac(524,32)=jac(524,32)-k(21)*y(524);
jac(524,524)=jac(524,524)-k(21)*y(32);
jac(530,32)=jac(530,32)+k(21)*y(524);
jac(530,524)=jac(530,524)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(524);
jac(2,524)=jac(2,524)-k(21)*y(2);
jac(524,2)=jac(524,2)-k(21)*y(524);
jac(524,524)=jac(524,524)-k(21)*y(2);
jac(507,2)=jac(507,2)+k(21)*y(524);
jac(507,524)=jac(507,524)+k(21)*y(2);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(524,524)=jac(524,524)-k(17);
jac(308,524)=jac(308,524)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(524,524)=jac(524,524)-k(16);
jac(3,524)=jac(3,524)+k(16);
jac(354,524)=jac(354,524)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(524);
jac(3,524)=jac(3,524)-k(9)*y(3);
jac(524,3)=jac(524,3)-k(9)*y(524);
jac(524,524)=jac(524,524)-k(9)*y(3);
jac(116,3)=jac(116,3)+k(9)*y(524);
jac(116,524)=jac(116,524)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(524,524)=jac(524,524)-k(8);
jac(16,524)=jac(16,524)+k(8);
jac(43,524)=jac(43,524)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

jac(524,524)=jac(524,524)-k(6);
jac(15,524)=jac(15,524)+k(6);
jac(114,524)=jac(114,524)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(526,526)=jac(526,526)-k(34);
jac(357,526)=jac(357,526)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(526,526)=jac(526,526)-k(33);
jac(522,526)=jac(522,526)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(526,526)=jac(526,526)-k(30);
jac(597,526)=jac(597,526)+k(30);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(526,526)=jac(526,526)-k(24);
jac(558,526)=jac(558,526)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(526,526)=jac(526,526)-k(22);
jac(197,526)=jac(197,526)+k(22);
jac(116,526)=jac(116,526)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

jac(526,526)=jac(526,526)-k(22);
jac(2,526)=jac(2,526)+k(22);
jac(339,526)=jac(339,526)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(526,526)=jac(526,526)-k(16);
jac(3,526)=jac(3,526)+k(16);
jac(340,526)=jac(340,526)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(526,526)=jac(526,526)-k(10);
jac(3,526)=jac(3,526)+k(10);
jac(525,526)=jac(525,526)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(526,526)=jac(526,526)-k(8);
jac(43,526)=jac(43,526)+k(8);
jac(336,526)=jac(336,526)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(526,526)=jac(526,526)-k(6);
jac(335,526)=jac(335,526)+k(6);
jac(114,526)=jac(114,526)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(527,527)=jac(527,527)-k(34);
jac(356,527)=jac(356,527)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(527,527)=jac(527,527)-k(33);
jac(509,527)=jac(509,527)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(527,527)=jac(527,527)-k(31);
jac(373,527)=jac(373,527)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(527,527)=jac(527,527)-k(30);
jac(611,527)=jac(611,527)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(527,527)=jac(527,527)-k(22);
jac(2,527)=jac(2,527)+k(22);
jac(338,527)=jac(338,527)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(527,527)=jac(527,527)-k(22);
jac(197,527)=jac(197,527)+k(22);
jac(354,527)=jac(354,527)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(527);
jac(3,527)=jac(3,527)-k(15)*y(3);
jac(527,3)=jac(527,3)-k(15)*y(527);
jac(527,527)=jac(527,527)-k(15)*y(3);
jac(525,3)=jac(525,3)+k(15)*y(527);
jac(525,527)=jac(525,527)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(527);
jac(3,527)=jac(3,527)-k(9)*y(3);
jac(527,3)=jac(527,3)-k(9)*y(527);
jac(527,527)=jac(527,527)-k(9)*y(3);
jac(340,3)=jac(340,3)+k(9)*y(527);
jac(340,527)=jac(340,527)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(527,527)=jac(527,527)-k(8);
jac(111,527)=jac(111,527)+k(8);
jac(209,527)=jac(209,527)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(527,527)=jac(527,527)-k(6);
jac(192,527)=jac(192,527)+k(6);
jac(110,527)=jac(110,527)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(528,528)=jac(528,528)-k(34);
jac(500,528)=jac(500,528)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(528,528)=jac(528,528)-k(33);
jac(523,528)=jac(523,528)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(528,528)=jac(528,528)-k(31);
jac(525,528)=jac(525,528)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(528,528)=jac(528,528)-k(30);
jac(626,528)=jac(626,528)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(528,528)=jac(528,528)-k(22);
jac(2,528)=jac(2,528)+k(22);
jac(564,528)=jac(564,528)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(528,528)=jac(528,528)-k(22);
jac(197,528)=jac(197,528)+k(22);
jac(308,528)=jac(308,528)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(528,528)=jac(528,528)-k(16);
jac(3,528)=jac(3,528)+k(16);
jac(502,528)=jac(502,528)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(528);
jac(3,528)=jac(3,528)-k(15)*y(3);
jac(528,3)=jac(528,3)-k(15)*y(528);
jac(528,528)=jac(528,528)-k(15)*y(3);
jac(632,3)=jac(632,3)+k(15)*y(528);
jac(632,528)=jac(632,528)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(528,528)=jac(528,528)-k(8);
jac(319,528)=jac(319,528)+k(8);
jac(43,528)=jac(43,528)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(528,528)=jac(528,528)-k(6);
jac(257,528)=jac(257,528)+k(6);
jac(114,528)=jac(114,528)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(529,529)=jac(529,529)-k(34);
jac(531,529)=jac(531,529)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(529,529)=jac(529,529)-k(33);
jac(519,529)=jac(519,529)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(529,529)=jac(529,529)-k(32);
jac(322,529)=jac(322,529)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

jac(529,529)=jac(529,529)-k(22);
jac(197,529)=jac(197,529)+k(22);
jac(537,529)=jac(537,529)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(529);
jac(197,529)=jac(197,529)-k(21)*y(197);
jac(529,197)=jac(529,197)-k(21)*y(529);
jac(529,529)=jac(529,529)-k(21)*y(197);
jac(552,197)=jac(552,197)+k(21)*y(529);
jac(552,529)=jac(552,529)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(529);
jac(32,529)=jac(32,529)-k(21)*y(32);
jac(529,32)=jac(529,32)-k(21)*y(529);
jac(529,529)=jac(529,529)-k(21)*y(32);
jac(557,32)=jac(557,32)+k(21)*y(529);
jac(557,529)=jac(557,529)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(529);
jac(2,529)=jac(2,529)-k(21)*y(2);
jac(529,2)=jac(529,2)-k(21)*y(529);
jac(529,529)=jac(529,529)-k(21)*y(2);
jac(525,2)=jac(525,2)+k(21)*y(529);
jac(525,529)=jac(525,529)+k(21)*y(2);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(529,529)=jac(529,529)-k(17);
jac(564,529)=jac(564,529)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(529,529)=jac(529,529)-k(16);
jac(3,529)=jac(3,529)+k(16);
jac(338,529)=jac(338,529)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(529);
jac(3,529)=jac(3,529)-k(9)*y(3);
jac(529,3)=jac(529,3)-k(9)*y(529);
jac(529,529)=jac(529,529)-k(9)*y(3);
jac(339,3)=jac(339,3)+k(9)*y(529);
jac(339,529)=jac(339,529)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(529,529)=jac(529,529)-k(8);
jac(54,529)=jac(54,529)+k(8);
jac(209,529)=jac(209,529)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(529,529)=jac(529,529)-k(6);
jac(118,529)=jac(118,529)+k(6);
jac(192,529)=jac(192,529)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(533,533)=jac(533,533)-k(33);
jac(508,533)=jac(508,533)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(533,533)=jac(533,533)-k(32);
jac(136,533)=jac(136,533)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(533,533)=jac(533,533)-k(31);
jac(156,533)=jac(156,533)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(533,533)=jac(533,533)-k(22);
jac(32,533)=jac(32,533)+k(22);
jac(610,533)=jac(610,533)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(533);
jac(197,533)=jac(197,533)-k(21)*y(197);
jac(533,197)=jac(533,197)-k(21)*y(533);
jac(533,533)=jac(533,533)-k(21)*y(197);
jac(608,197)=jac(608,197)+k(21)*y(533);
jac(608,533)=jac(608,533)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(533);
jac(32,533)=jac(32,533)-k(21)*y(32);
jac(533,32)=jac(533,32)-k(21)*y(533);
jac(533,533)=jac(533,533)-k(21)*y(32);
jac(612,32)=jac(612,32)+k(21)*y(533);
jac(612,533)=jac(612,533)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(533);
jac(2,533)=jac(2,533)-k(21)*y(2);
jac(533,2)=jac(533,2)-k(21)*y(533);
jac(533,533)=jac(533,533)-k(21)*y(2);
jac(356,2)=jac(356,2)+k(21)*y(533);
jac(356,533)=jac(356,533)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(533);
jac(3,533)=jac(3,533)-k(15)*y(3);
jac(533,3)=jac(533,3)-k(15)*y(533);
jac(533,533)=jac(533,533)-k(15)*y(3);
jac(531,3)=jac(531,3)+k(15)*y(533);
jac(531,533)=jac(531,533)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(533);
jac(3,533)=jac(3,533)-k(9)*y(3);
jac(533,3)=jac(533,3)-k(9)*y(533);
jac(533,533)=jac(533,533)-k(9)*y(3);
jac(137,3)=jac(137,3)+k(9)*y(533);
jac(137,533)=jac(137,533)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

jac(533,533)=jac(533,533)-k(8);
jac(77,533)=jac(77,533)+k(8);
jac(28,533)=jac(28,533)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(533,533)=jac(533,533)-k(6);
jac(76,533)=jac(76,533)+k(6);
jac(27,533)=jac(27,533)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(534,534)=jac(534,534)-k(33);
jac(540,534)=jac(540,534)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(534,534)=jac(534,534)-k(32);
jac(317,534)=jac(317,534)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(534,534)=jac(534,534)-k(31);
jac(531,534)=jac(531,534)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

jac(534,534)=jac(534,534)-k(22);
jac(32,534)=jac(32,534)+k(22);
jac(310,534)=jac(310,534)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(534);
jac(197,534)=jac(197,534)-k(21)*y(197);
jac(534,197)=jac(534,197)-k(21)*y(534);
jac(534,534)=jac(534,534)-k(21)*y(197);
jac(569,197)=jac(569,197)+k(21)*y(534);
jac(569,534)=jac(569,534)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(534);
jac(32,534)=jac(32,534)-k(21)*y(32);
jac(534,32)=jac(534,32)-k(21)*y(534);
jac(534,534)=jac(534,534)-k(21)*y(32);
jac(566,32)=jac(566,32)+k(21)*y(534);
jac(566,534)=jac(566,534)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(534);
jac(2,534)=jac(2,534)-k(21)*y(2);
jac(534,2)=jac(534,2)-k(21)*y(534);
jac(534,534)=jac(534,534)-k(21)*y(2);
jac(500,2)=jac(500,2)+k(21)*y(534);
jac(500,534)=jac(500,534)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(534,534)=jac(534,534)-k(16);
jac(3,534)=jac(3,534)+k(16);
jac(245,534)=jac(245,534)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(534);
jac(3,534)=jac(3,534)-k(15)*y(3);
jac(534,3)=jac(534,3)-k(15)*y(534);
jac(534,534)=jac(534,534)-k(15)*y(3);
jac(244,3)=jac(244,3)+k(15)*y(534);
jac(244,534)=jac(244,534)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(534,534)=jac(534,534)-k(8);
jac(54,534)=jac(54,534)+k(8);
jac(34,534)=jac(34,534)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(534,534)=jac(534,534)-k(6);
jac(118,534)=jac(118,534)+k(6);
jac(33,534)=jac(33,534)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(535,535)=jac(535,535)-k(33);
jac(530,535)=jac(530,535)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(535,535)=jac(535,535)-k(33);
jac(541,535)=jac(541,535)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(535,535)=jac(535,535)-k(28);
jac(536,535)=jac(536,535)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(535,535)=jac(535,535)-k(27);
jac(557,535)=jac(557,535)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(535,535)=jac(535,535)-k(22);
jac(32,535)=jac(32,535)+k(22);
jac(531,535)=jac(531,535)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(535,535)=jac(535,535)-k(22);
jac(32,535)=jac(32,535)+k(22);
jac(556,535)=jac(556,535)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(535,535)=jac(535,535)-k(17);
jac(566,535)=jac(566,535)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(535,535)=jac(535,535)-k(16);
jac(3,535)=jac(3,535)+k(16);
jac(612,535)=jac(612,535)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(535);
jac(3,535)=jac(3,535)-k(9)*y(3);
jac(535,3)=jac(535,3)-k(9)*y(535);
jac(535,535)=jac(535,535)-k(9)*y(3);
jac(602,3)=jac(602,3)+k(9)*y(535);
jac(602,535)=jac(602,535)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(535,535)=jac(535,535)-k(8);
jac(28,535)=jac(28,535)+k(8);
jac(37,535)=jac(37,535)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(535,535)=jac(535,535)-k(6);
jac(27,535)=jac(27,535)+k(6);
jac(554,535)=jac(554,535)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(536,536)=jac(536,536)-k(34);
jac(535,536)=jac(535,536)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(536,536)=jac(536,536)-k(33);
jac(532,536)=jac(532,536)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(536,536)=jac(536,536)-k(33);
jac(542,536)=jac(542,536)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(536,536)=jac(536,536)-k(29);
jac(552,536)=jac(552,536)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(536,536)=jac(536,536)-k(22);
jac(197,536)=jac(197,536)+k(22);
jac(531,536)=jac(531,536)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(536,536)=jac(536,536)-k(22);
jac(32,536)=jac(32,536)+k(22);
jac(547,536)=jac(547,536)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(536,536)=jac(536,536)-k(17);
jac(569,536)=jac(569,536)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(536,536)=jac(536,536)-k(16);
jac(3,536)=jac(3,536)+k(16);
jac(608,536)=jac(608,536)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(536);
jac(3,536)=jac(3,536)-k(9)*y(3);
jac(536,3)=jac(536,3)-k(9)*y(536);
jac(536,536)=jac(536,536)-k(9)*y(3);
jac(600,3)=jac(600,3)+k(9)*y(536);
jac(600,536)=jac(600,536)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(536,536)=jac(536,536)-k(8);
jac(28,536)=jac(28,536)+k(8);
jac(227,536)=jac(227,536)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(536,536)=jac(536,536)-k(6);
jac(27,536)=jac(27,536)+k(6);
jac(543,536)=jac(543,536)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(537,537)=jac(537,537)-k(32);
jac(127,537)=jac(127,537)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(537,537)=jac(537,537)-k(32);
jac(119,537)=jac(119,537)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(537);
jac(197,537)=jac(197,537)-k(21)*y(197);
jac(537,197)=jac(537,197)-k(21)*y(537);
jac(537,537)=jac(537,537)-k(21)*y(197);
jac(529,197)=jac(529,197)+k(21)*y(537);
jac(529,537)=jac(529,537)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(537);
jac(32,537)=jac(32,537)-k(21)*y(32);
jac(537,32)=jac(537,32)-k(21)*y(537);
jac(537,537)=jac(537,537)-k(21)*y(32);
jac(531,32)=jac(531,32)+k(21)*y(537);
jac(531,537)=jac(531,537)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(537);
jac(2,537)=jac(2,537)-k(21)*y(2);
jac(537,2)=jac(537,2)-k(21)*y(537);
jac(537,537)=jac(537,537)-k(21)*y(2);
jac(538,2)=jac(538,2)+k(21)*y(537);
jac(538,537)=jac(538,537)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(537);
jac(197,537)=jac(197,537)-k(21)*y(197);
jac(537,197)=jac(537,197)-k(21)*y(537);
jac(537,537)=jac(537,537)-k(21)*y(197);
jac(547,197)=jac(547,197)+k(21)*y(537);
jac(547,537)=jac(547,537)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(537);
jac(32,537)=jac(32,537)-k(21)*y(32);
jac(537,32)=jac(537,32)-k(21)*y(537);
jac(537,537)=jac(537,537)-k(21)*y(32);
jac(556,32)=jac(556,32)+k(21)*y(537);
jac(556,537)=jac(556,537)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(537);
jac(2,537)=jac(2,537)-k(21)*y(2);
jac(537,2)=jac(537,2)-k(21)*y(537);
jac(537,537)=jac(537,537)-k(21)*y(2);
jac(524,2)=jac(524,2)+k(21)*y(537);
jac(524,537)=jac(524,537)+k(21)*y(2);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(537,537)=jac(537,537)-k(17);
jac(310,537)=jac(310,537)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

jac(537,537)=jac(537,537)-k(16);
jac(3,537)=jac(3,537)+k(16);
jac(610,537)=jac(610,537)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(537);
jac(3,537)=jac(3,537)-k(9)*y(3);
jac(537,3)=jac(537,3)-k(9)*y(537);
jac(537,537)=jac(537,537)-k(9)*y(3);
jac(120,3)=jac(120,3)+k(9)*y(537);
jac(120,537)=jac(120,537)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(537,537)=jac(537,537)-k(8);
jac(16,537)=jac(16,537)+k(8);
jac(54,537)=jac(54,537)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y, g~pY) 

jac(537,537)=jac(537,537)-k(6);
jac(15,537)=jac(15,537)+k(6);
jac(118,537)=jac(118,537)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(539,539)=jac(539,539)-k(32);
jac(129,539)=jac(129,539)+k(32);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(539,539)=jac(539,539)-k(25);
jac(134,539)=jac(134,539)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(539,539)=jac(539,539)-k(22);
jac(2,539)=jac(2,539)+k(22);
jac(120,539)=jac(120,539)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(539);
jac(197,539)=jac(197,539)-k(21)*y(197);
jac(539,197)=jac(539,197)-k(21)*y(539);
jac(539,539)=jac(539,539)-k(21)*y(197);
jac(544,197)=jac(544,197)+k(21)*y(539);
jac(544,539)=jac(544,539)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(539);
jac(32,539)=jac(32,539)-k(21)*y(32);
jac(539,32)=jac(539,32)-k(21)*y(539);
jac(539,539)=jac(539,539)-k(21)*y(32);
jac(603,32)=jac(603,32)+k(21)*y(539);
jac(603,539)=jac(603,539)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(539);
jac(2,539)=jac(2,539)-k(21)*y(2);
jac(539,2)=jac(539,2)-k(21)*y(539);
jac(539,539)=jac(539,539)-k(21)*y(2);
jac(604,2)=jac(604,2)+k(21)*y(539);
jac(604,539)=jac(604,539)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(539,539)=jac(539,539)-k(16);
jac(3,539)=jac(3,539)+k(16);
jac(143,539)=jac(143,539)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(539,539)=jac(539,539)-k(10);
jac(3,539)=jac(3,539)+k(10);
jac(538,539)=jac(538,539)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(539,539)=jac(539,539)-k(8);
jac(54,539)=jac(54,539)+k(8);
jac(141,539)=jac(141,539)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(539,539)=jac(539,539)-k(6);
jac(118,539)=jac(118,539)+k(6);
jac(140,539)=jac(140,539)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(540,540)=jac(540,540)-k(32);
jac(316,540)=jac(316,540)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(540,540)=jac(540,540)-k(31);
jac(538,540)=jac(538,540)+k(31);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(540,540)=jac(540,540)-k(25);
jac(534,540)=jac(534,540)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

jac(540,540)=jac(540,540)-k(22);
jac(2,540)=jac(2,540)+k(22);
jac(310,540)=jac(310,540)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(540);
jac(197,540)=jac(197,540)-k(21)*y(197);
jac(540,197)=jac(540,197)-k(21)*y(540);
jac(540,540)=jac(540,540)-k(21)*y(197);
jac(546,197)=jac(546,197)+k(21)*y(540);
jac(546,540)=jac(546,540)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(540);
jac(32,540)=jac(32,540)-k(21)*y(32);
jac(540,32)=jac(540,32)-k(21)*y(540);
jac(540,540)=jac(540,540)-k(21)*y(32);
jac(570,32)=jac(570,32)+k(21)*y(540);
jac(570,540)=jac(570,540)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(540);
jac(2,540)=jac(2,540)-k(21)*y(2);
jac(540,2)=jac(540,2)-k(21)*y(540);
jac(540,540)=jac(540,540)-k(21)*y(2);
jac(504,2)=jac(504,2)+k(21)*y(540);
jac(504,540)=jac(504,540)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(540,540)=jac(540,540)-k(16);
jac(3,540)=jac(3,540)+k(16);
jac(252,540)=jac(252,540)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(540);
jac(3,540)=jac(3,540)-k(15)*y(3);
jac(540,3)=jac(540,3)-k(15)*y(540);
jac(540,540)=jac(540,540)-k(15)*y(3);
jac(251,3)=jac(251,3)+k(15)*y(540);
jac(251,540)=jac(251,540)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(540,540)=jac(540,540)-k(8);
jac(54,540)=jac(54,540)+k(8);
jac(437,540)=jac(437,540)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(540,540)=jac(540,540)-k(6);
jac(118,540)=jac(118,540)+k(6);
jac(111,540)=jac(111,540)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(541,541)=jac(541,541)-k(33);
jac(507,541)=jac(507,541)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(541,541)=jac(541,541)-k(28);
jac(542,541)=jac(542,541)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(541,541)=jac(541,541)-k(27);
jac(553,541)=jac(553,541)+k(27);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(541,541)=jac(541,541)-k(25);
jac(535,541)=jac(535,541)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(541,541)=jac(541,541)-k(22);
jac(32,541)=jac(32,541)+k(22);
jac(538,541)=jac(538,541)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(541,541)=jac(541,541)-k(22);
jac(2,541)=jac(2,541)+k(22);
jac(556,541)=jac(556,541)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(541,541)=jac(541,541)-k(17);
jac(570,541)=jac(570,541)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(541,541)=jac(541,541)-k(16);
jac(3,541)=jac(3,541)+k(16);
jac(575,541)=jac(575,541)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(541);
jac(3,541)=jac(3,541)-k(9)*y(3);
jac(541,3)=jac(541,3)-k(9)*y(541);
jac(541,541)=jac(541,541)-k(9)*y(3);
jac(603,3)=jac(603,3)+k(9)*y(541);
jac(603,541)=jac(603,541)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(541,541)=jac(541,541)-k(8);
jac(22,541)=jac(22,541)+k(8);
jac(37,541)=jac(37,541)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(541,541)=jac(541,541)-k(6);
jac(21,541)=jac(21,541)+k(6);
jac(554,541)=jac(554,541)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(543,543)=jac(543,543)-k(34);
jac(554,543)=jac(554,543)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(543,543)=jac(543,543)-k(33);
jac(515,543)=jac(515,543)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

jac(543,543)=jac(543,543)-k(22);
jac(197,543)=jac(197,543)+k(22);
jac(118,543)=jac(118,543)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(543,543)=jac(543,543)-k(16);
jac(3,543)=jac(3,543)+k(16);
jac(256,543)=jac(256,543)+k(16);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(1,1)=jac(1,1)-k(5)*y(543);
jac(1,543)=jac(1,543)-k(5)*y(1);
jac(543,1)=jac(543,1)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(1);
jac(589,1)=jac(589,1)+k(5)*y(543);
jac(589,543)=jac(589,543)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(7,7)=jac(7,7)-k(5)*y(543);
jac(7,543)=jac(7,543)-k(5)*y(7);
jac(543,7)=jac(543,7)-k(5)*y(543);
jac(543,543)=jac(543,543)-k(5)*y(7);
jac(201,7)=jac(201,7)+k(5)*y(543);
jac(201,543)=jac(201,543)+k(5)*y(7);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Lig(l1, l2) 

jac(543,543)=jac(543,543)-k(4);
jac(4,543)=jac(4,543)+k(4);
jac(227,543)=jac(227,543)+k(4);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(544,544)=jac(544,544)-k(34);
jac(603,544)=jac(603,544)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(544,544)=jac(544,544)-k(33);
jac(596,544)=jac(596,544)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(544,544)=jac(544,544)-k(29);
jac(549,544)=jac(549,544)+k(29);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(544,544)=jac(544,544)-k(25);
jac(600,544)=jac(600,544)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(544,544)=jac(544,544)-k(22);
jac(2,544)=jac(2,544)+k(22);
jac(598,544)=jac(598,544)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(544,544)=jac(544,544)-k(22);
jac(197,544)=jac(197,544)+k(22);
jac(539,544)=jac(539,544)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(544,544)=jac(544,544)-k(16);
jac(3,544)=jac(3,544)+k(16);
jac(605,544)=jac(605,544)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(544,544)=jac(544,544)-k(10);
jac(3,544)=jac(3,544)+k(10);
jac(542,544)=jac(542,544)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(544,544)=jac(544,544)-k(8);
jac(227,544)=jac(227,544)+k(8);
jac(141,544)=jac(141,544)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(544,544)=jac(544,544)-k(6);
jac(140,544)=jac(140,544)+k(6);
jac(543,544)=jac(543,544)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(545,545)=jac(545,545)-k(34);
jac(575,545)=jac(575,545)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(545,545)=jac(545,545)-k(33);
jac(510,545)=jac(510,545)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(545,545)=jac(545,545)-k(31);
jac(453,545)=jac(453,545)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(545,545)=jac(545,545)-k(29);
jac(550,545)=jac(550,545)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(545,545)=jac(545,545)-k(22);
jac(197,545)=jac(197,545)+k(22);
jac(508,545)=jac(508,545)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(545,545)=jac(545,545)-k(22);
jac(2,545)=jac(2,545)+k(22);
jac(585,545)=jac(585,545)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(545);
jac(3,545)=jac(3,545)-k(15)*y(3);
jac(545,3)=jac(545,3)-k(15)*y(545);
jac(545,545)=jac(545,545)-k(15)*y(3);
jac(542,3)=jac(542,3)+k(15)*y(545);
jac(542,545)=jac(542,545)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(545);
jac(3,545)=jac(3,545)-k(9)*y(3);
jac(545,3)=jac(545,3)-k(9)*y(545);
jac(545,545)=jac(545,545)-k(9)*y(3);
jac(605,3)=jac(605,3)+k(9)*y(545);
jac(605,545)=jac(605,545)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(545,545)=jac(545,545)-k(8);
jac(257,545)=jac(257,545)+k(8);
jac(22,545)=jac(22,545)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(545,545)=jac(545,545)-k(6);
jac(21,545)=jac(21,545)+k(6);
jac(256,545)=jac(256,545)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(546,546)=jac(546,546)-k(34);
jac(570,546)=jac(570,546)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(546,546)=jac(546,546)-k(33);
jac(625,546)=jac(625,546)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(546,546)=jac(546,546)-k(31);
jac(542,546)=jac(542,546)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(546,546)=jac(546,546)-k(29);
jac(551,546)=jac(551,546)+k(29);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(546,546)=jac(546,546)-k(25);
jac(569,546)=jac(569,546)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(546,546)=jac(546,546)-k(22);
jac(197,546)=jac(197,546)+k(22);
jac(540,546)=jac(540,546)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(546,546)=jac(546,546)-k(22);
jac(2,546)=jac(2,546)+k(22);
jac(581,546)=jac(581,546)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(546,546)=jac(546,546)-k(16);
jac(3,546)=jac(3,546)+k(16);
jac(259,546)=jac(259,546)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(546);
jac(3,546)=jac(3,546)-k(15)*y(3);
jac(546,3)=jac(546,3)-k(15)*y(546);
jac(546,546)=jac(546,546)-k(15)*y(3);
jac(258,3)=jac(258,3)+k(15)*y(546);
jac(258,546)=jac(258,546)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(546,546)=jac(546,546)-k(8);
jac(437,546)=jac(437,546)+k(8);
jac(227,546)=jac(227,546)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(546,546)=jac(546,546)-k(6);
jac(111,546)=jac(111,546)+k(6);
jac(543,546)=jac(543,546)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(547,547)=jac(547,547)-k(34);
jac(556,547)=jac(556,547)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(547,547)=jac(547,547)-k(33);
jac(518,547)=jac(518,547)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(547,547)=jac(547,547)-k(32);
jac(589,547)=jac(589,547)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

jac(547,547)=jac(547,547)-k(22);
jac(197,547)=jac(197,547)+k(22);
jac(537,547)=jac(537,547)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(547);
jac(197,547)=jac(197,547)-k(21)*y(197);
jac(547,197)=jac(547,197)-k(21)*y(547);
jac(547,547)=jac(547,547)-k(21)*y(197);
jac(552,197)=jac(552,197)+k(21)*y(547);
jac(552,547)=jac(552,547)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(547);
jac(32,547)=jac(32,547)-k(21)*y(32);
jac(547,32)=jac(547,32)-k(21)*y(547);
jac(547,547)=jac(547,547)-k(21)*y(32);
jac(536,32)=jac(536,32)+k(21)*y(547);
jac(536,547)=jac(536,547)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(547);
jac(2,547)=jac(2,547)-k(21)*y(2);
jac(547,2)=jac(547,2)-k(21)*y(547);
jac(547,547)=jac(547,547)-k(21)*y(2);
jac(542,2)=jac(542,2)+k(21)*y(547);
jac(542,547)=jac(542,547)+k(21)*y(2);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(547,547)=jac(547,547)-k(17);
jac(581,547)=jac(581,547)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(547,547)=jac(547,547)-k(16);
jac(3,547)=jac(3,547)+k(16);
jac(585,547)=jac(585,547)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(547);
jac(3,547)=jac(3,547)-k(9)*y(3);
jac(547,3)=jac(547,3)-k(9)*y(547);
jac(547,547)=jac(547,547)-k(9)*y(3);
jac(598,3)=jac(598,3)+k(9)*y(547);
jac(598,547)=jac(598,547)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(547,547)=jac(547,547)-k(8);
jac(16,547)=jac(16,547)+k(8);
jac(227,547)=jac(227,547)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

jac(547,547)=jac(547,547)-k(6);
jac(15,547)=jac(15,547)+k(6);
jac(543,547)=jac(543,547)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(549,549)=jac(549,549)-k(34);
jac(544,549)=jac(544,549)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(549,549)=jac(549,549)-k(34);
jac(349,549)=jac(349,549)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(549,549)=jac(549,549)-k(33);
jac(516,549)=jac(516,549)+k(33);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(549,549)=jac(549,549)-k(25);
jac(601,549)=jac(601,549)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(549,549)=jac(549,549)-k(22);
jac(182,549)=jac(182,549)+k(22);
jac(598,549)=jac(598,549)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

jac(549,549)=jac(549,549)-k(22);
jac(197,549)=jac(197,549)+k(22);
jac(345,549)=jac(345,549)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(549,549)=jac(549,549)-k(16);
jac(3,549)=jac(3,549)+k(16);
jac(195,549)=jac(195,549)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(549,549)=jac(549,549)-k(10);
jac(3,549)=jac(3,549)+k(10);
jac(548,549)=jac(548,549)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(549,549)=jac(549,549)-k(8);
jac(227,549)=jac(227,549)+k(8);
jac(185,549)=jac(185,549)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(549,549)=jac(549,549)-k(6);
jac(184,549)=jac(184,549)+k(6);
jac(543,549)=jac(543,549)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(550,550)=jac(550,550)-k(34);
jac(348,550)=jac(348,550)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(550,550)=jac(550,550)-k(34);
jac(545,550)=jac(545,550)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(550,550)=jac(550,550)-k(33);
jac(513,550)=jac(513,550)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(550,550)=jac(550,550)-k(31);
jac(194,550)=jac(194,550)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(550,550)=jac(550,550)-k(22);
jac(197,550)=jac(197,550)+k(22);
jac(344,550)=jac(344,550)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(550,550)=jac(550,550)-k(22);
jac(182,550)=jac(182,550)+k(22);
jac(585,550)=jac(585,550)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(550);
jac(3,550)=jac(3,550)-k(15)*y(3);
jac(550,3)=jac(550,3)-k(15)*y(550);
jac(550,550)=jac(550,550)-k(15)*y(3);
jac(548,3)=jac(548,3)+k(15)*y(550);
jac(548,550)=jac(548,550)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(550);
jac(3,550)=jac(3,550)-k(9)*y(3);
jac(550,3)=jac(550,3)-k(9)*y(550);
jac(550,550)=jac(550,550)-k(9)*y(3);
jac(195,3)=jac(195,3)+k(9)*y(550);
jac(195,550)=jac(195,550)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(550,550)=jac(550,550)-k(8);
jac(257,550)=jac(257,550)+k(8);
jac(214,550)=jac(214,550)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(550,550)=jac(550,550)-k(6);
jac(177,550)=jac(177,550)+k(6);
jac(256,550)=jac(256,550)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(551,551)=jac(551,551)-k(34);
jac(555,551)=jac(555,551)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(551,551)=jac(551,551)-k(34);
jac(546,551)=jac(546,551)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(551,551)=jac(551,551)-k(33);
jac(517,551)=jac(517,551)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(551,551)=jac(551,551)-k(31);
jac(548,551)=jac(548,551)+k(31);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(551,551)=jac(551,551)-k(25);
jac(565,551)=jac(565,551)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(551,551)=jac(551,551)-k(22);
jac(197,551)=jac(197,551)+k(22);
jac(644,551)=jac(644,551)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(551,551)=jac(551,551)-k(22);
jac(182,551)=jac(182,551)+k(22);
jac(581,551)=jac(581,551)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(551,551)=jac(551,551)-k(16);
jac(3,551)=jac(3,551)+k(16);
jac(577,551)=jac(577,551)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(551);
jac(3,551)=jac(3,551)-k(15)*y(3);
jac(551,3)=jac(551,3)-k(15)*y(551);
jac(551,551)=jac(551,551)-k(15)*y(3);
jac(641,3)=jac(641,3)+k(15)*y(551);
jac(641,551)=jac(641,551)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(551,551)=jac(551,551)-k(8);
jac(219,551)=jac(219,551)+k(8);
jac(227,551)=jac(227,551)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(551,551)=jac(551,551)-k(6);
jac(218,551)=jac(218,551)+k(6);
jac(543,551)=jac(543,551)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(552,552)=jac(552,552)-k(34);
jac(557,552)=jac(557,552)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(552,552)=jac(552,552)-k(34);
jac(536,552)=jac(536,552)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(552,552)=jac(552,552)-k(33);
jac(520,552)=jac(520,552)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(552,552)=jac(552,552)-k(33);
jac(548,552)=jac(548,552)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(552,552)=jac(552,552)-k(22);
jac(197,552)=jac(197,552)+k(22);
jac(529,552)=jac(529,552)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(552,552)=jac(552,552)-k(22);
jac(197,552)=jac(197,552)+k(22);
jac(547,552)=jac(547,552)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(552,552)=jac(552,552)-k(17);
jac(565,552)=jac(565,552)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(552,552)=jac(552,552)-k(16);
jac(3,552)=jac(3,552)+k(16);
jac(609,552)=jac(609,552)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(552);
jac(3,552)=jac(3,552)-k(9)*y(3);
jac(552,3)=jac(552,3)-k(9)*y(552);
jac(552,552)=jac(552,552)-k(9)*y(3);
jac(601,3)=jac(601,3)+k(9)*y(552);
jac(601,552)=jac(601,552)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(552,552)=jac(552,552)-k(8);
jac(209,552)=jac(209,552)+k(8);
jac(227,552)=jac(227,552)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(552,552)=jac(552,552)-k(6);
jac(192,552)=jac(192,552)+k(6);
jac(543,552)=jac(543,552)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(554,554)=jac(554,554)-k(33);
jac(114,554)=jac(114,554)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

jac(554,554)=jac(554,554)-k(22);
jac(32,554)=jac(32,554)+k(22);
jac(118,554)=jac(118,554)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(554,554)=jac(554,554)-k(16);
jac(3,554)=jac(3,554)+k(16);
jac(82,554)=jac(82,554)+k(16);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(1,1)=jac(1,1)-k(5)*y(554);
jac(1,554)=jac(1,554)-k(5)*y(1);
jac(554,1)=jac(554,1)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(1);
jac(592,1)=jac(592,1)+k(5)*y(554);
jac(592,554)=jac(592,554)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(7,7)=jac(7,7)-k(5)*y(554);
jac(7,554)=jac(7,554)-k(5)*y(7);
jac(554,7)=jac(554,7)-k(5)*y(554);
jac(554,554)=jac(554,554)-k(5)*y(7);
jac(107,7)=jac(107,7)+k(5)*y(554);
jac(107,554)=jac(107,554)+k(5)*y(7);

% rule    : R1_2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Lig(l1, l2) 

jac(554,554)=jac(554,554)-k(4);
jac(4,554)=jac(4,554)+k(4);
jac(37,554)=jac(37,554)+k(4);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(555,555)=jac(555,555)-k(34);
jac(570,555)=jac(570,555)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(555,555)=jac(555,555)-k(33);
jac(523,555)=jac(523,555)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(555,555)=jac(555,555)-k(31);
jac(553,555)=jac(553,555)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(555,555)=jac(555,555)-k(30);
jac(551,555)=jac(551,555)+k(30);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(555,555)=jac(555,555)-k(25);
jac(560,555)=jac(560,555)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(555,555)=jac(555,555)-k(22);
jac(32,555)=jac(32,555)+k(22);
jac(644,555)=jac(644,555)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(555,555)=jac(555,555)-k(22);
jac(182,555)=jac(182,555)+k(22);
jac(563,555)=jac(563,555)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(555,555)=jac(555,555)-k(16);
jac(3,555)=jac(3,555)+k(16);
jac(574,555)=jac(574,555)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(555);
jac(3,555)=jac(3,555)-k(15)*y(3);
jac(555,3)=jac(555,3)-k(15)*y(555);
jac(555,555)=jac(555,555)-k(15)*y(3);
jac(639,3)=jac(639,3)+k(15)*y(555);
jac(639,555)=jac(639,555)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(555,555)=jac(555,555)-k(8);
jac(219,555)=jac(219,555)+k(8);
jac(37,555)=jac(37,555)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(555,555)=jac(555,555)-k(6);
jac(218,555)=jac(218,555)+k(6);
jac(554,555)=jac(554,555)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(556,556)=jac(556,556)-k(33);
jac(524,556)=jac(524,556)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(556,556)=jac(556,556)-k(32);
jac(592,556)=jac(592,556)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

jac(556,556)=jac(556,556)-k(22);
jac(32,556)=jac(32,556)+k(22);
jac(537,556)=jac(537,556)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(556);
jac(197,556)=jac(197,556)-k(21)*y(197);
jac(556,197)=jac(556,197)-k(21)*y(556);
jac(556,556)=jac(556,556)-k(21)*y(197);
jac(557,197)=jac(557,197)+k(21)*y(556);
jac(557,556)=jac(557,556)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(556);
jac(32,556)=jac(32,556)-k(21)*y(32);
jac(556,32)=jac(556,32)-k(21)*y(556);
jac(556,556)=jac(556,556)-k(21)*y(32);
jac(535,32)=jac(535,32)+k(21)*y(556);
jac(535,556)=jac(535,556)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(556);
jac(2,556)=jac(2,556)-k(21)*y(2);
jac(556,2)=jac(556,2)-k(21)*y(556);
jac(556,556)=jac(556,556)-k(21)*y(2);
jac(541,2)=jac(541,2)+k(21)*y(556);
jac(541,556)=jac(541,556)+k(21)*y(2);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(556,556)=jac(556,556)-k(17);
jac(563,556)=jac(563,556)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(556,556)=jac(556,556)-k(16);
jac(3,556)=jac(3,556)+k(16);
jac(351,556)=jac(351,556)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(556);
jac(3,556)=jac(3,556)-k(9)*y(3);
jac(556,3)=jac(556,3)-k(9)*y(556);
jac(556,556)=jac(556,556)-k(9)*y(3);
jac(352,3)=jac(352,3)+k(9)*y(556);
jac(352,556)=jac(352,556)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(556,556)=jac(556,556)-k(8);
jac(16,556)=jac(16,556)+k(8);
jac(37,556)=jac(37,556)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

jac(556,556)=jac(556,556)-k(6);
jac(15,556)=jac(15,556)+k(6);
jac(554,556)=jac(554,556)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(558,558)=jac(558,558)-k(34);
jac(602,558)=jac(602,558)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(558,558)=jac(558,558)-k(33);
jac(349,558)=jac(349,558)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(558,558)=jac(558,558)-k(33);
jac(526,558)=jac(526,558)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(558,558)=jac(558,558)-k(30);
jac(601,558)=jac(601,558)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(558,558)=jac(558,558)-k(22);
jac(197,558)=jac(197,558)+k(22);
jac(352,558)=jac(352,558)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

jac(558,558)=jac(558,558)-k(22);
jac(32,558)=jac(32,558)+k(22);
jac(339,558)=jac(339,558)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(558,558)=jac(558,558)-k(16);
jac(3,558)=jac(3,558)+k(16);
jac(341,558)=jac(341,558)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(558,558)=jac(558,558)-k(10);
jac(3,558)=jac(3,558)+k(10);
jac(557,558)=jac(557,558)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(558,558)=jac(558,558)-k(8);
jac(37,558)=jac(37,558)+k(8);
jac(336,558)=jac(336,558)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(558,558)=jac(558,558)-k(6);
jac(335,558)=jac(335,558)+k(6);
jac(554,558)=jac(554,558)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(559,559)=jac(559,559)-k(34);
jac(612,559)=jac(612,559)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(559,559)=jac(559,559)-k(33);
jac(527,559)=jac(527,559)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(559,559)=jac(559,559)-k(33);
jac(348,559)=jac(348,559)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(559,559)=jac(559,559)-k(31);
jac(454,559)=jac(454,559)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(559,559)=jac(559,559)-k(30);
jac(609,559)=jac(609,559)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(559,559)=jac(559,559)-k(22);
jac(32,559)=jac(32,559)+k(22);
jac(338,559)=jac(338,559)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(559,559)=jac(559,559)-k(22);
jac(197,559)=jac(197,559)+k(22);
jac(351,559)=jac(351,559)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(559);
jac(3,559)=jac(3,559)-k(15)*y(3);
jac(559,3)=jac(559,3)-k(15)*y(559);
jac(559,559)=jac(559,559)-k(15)*y(3);
jac(557,3)=jac(557,3)+k(15)*y(559);
jac(557,559)=jac(557,559)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(559);
jac(3,559)=jac(3,559)-k(9)*y(3);
jac(559,3)=jac(559,3)-k(9)*y(559);
jac(559,559)=jac(559,559)-k(9)*y(3);
jac(341,3)=jac(341,3)+k(9)*y(559);
jac(341,559)=jac(341,559)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(559,559)=jac(559,559)-k(8);
jac(33,559)=jac(33,559)+k(8);
jac(209,559)=jac(209,559)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(559,559)=jac(559,559)-k(6);
jac(192,559)=jac(192,559)+k(6);
jac(82,559)=jac(82,559)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(561,561)=jac(561,561)-k(34);
jac(567,561)=jac(567,561)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(561,561)=jac(561,561)-k(33);
jac(639,561)=jac(639,561)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(561,561)=jac(561,561)-k(33);
jac(632,561)=jac(632,561)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(561,561)=jac(561,561)-k(30);
jac(642,561)=jac(642,561)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(561,561)=jac(561,561)-k(22);
jac(197,561)=jac(197,561)+k(22);
jac(633,561)=jac(633,561)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

jac(561,561)=jac(561,561)-k(22);
jac(32,561)=jac(32,561)+k(22);
jac(636,561)=jac(636,561)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(561,561)=jac(561,561)-k(16);
jac(3,561)=jac(3,561)+k(16);
jac(560,561)=jac(560,561)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(561,561)=jac(561,561)-k(16);
jac(3,561)=jac(3,561)+k(16);
jac(635,561)=jac(635,561)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(561,561)=jac(561,561)-k(8);
jac(37,561)=jac(37,561)+k(8);
jac(228,561)=jac(228,561)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(561,561)=jac(561,561)-k(6);
jac(227,561)=jac(227,561)+k(6);
jac(554,561)=jac(554,561)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(562,562)=jac(562,562)-k(34);
jac(568,562)=jac(568,562)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(562,562)=jac(562,562)-k(33);
jac(502,562)=jac(502,562)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(562,562)=jac(562,562)-k(33);
jac(574,562)=jac(574,562)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(562,562)=jac(562,562)-k(31);
jac(472,562)=jac(472,562)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(562,562)=jac(562,562)-k(31);
jac(559,562)=jac(559,562)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(562,562)=jac(562,562)-k(30);
jac(583,562)=jac(583,562)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(562,562)=jac(562,562)-k(22);
jac(32,562)=jac(32,562)+k(22);
jac(613,562)=jac(613,562)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(562,562)=jac(562,562)-k(22);
jac(197,562)=jac(197,562)+k(22);
jac(573,562)=jac(573,562)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(562);
jac(3,562)=jac(3,562)-k(15)*y(3);
jac(562,3)=jac(562,3)-k(15)*y(562);
jac(562,562)=jac(562,562)-k(15)*y(3);
jac(635,3)=jac(635,3)+k(15)*y(562);
jac(635,562)=jac(635,562)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(562);
jac(3,562)=jac(3,562)-k(15)*y(3);
jac(562,3)=jac(562,3)-k(15)*y(562);
jac(562,562)=jac(562,562)-k(15)*y(3);
jac(560,3)=jac(560,3)+k(15)*y(562);
jac(560,562)=jac(560,562)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(562,562)=jac(562,562)-k(8);
jac(33,562)=jac(33,562)+k(8);
jac(319,562)=jac(319,562)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(562,562)=jac(562,562)-k(6);
jac(257,562)=jac(257,562)+k(6);
jac(82,562)=jac(82,562)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(563,563)=jac(563,563)-k(33);
jac(308,563)=jac(308,563)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(563,563)=jac(563,563)-k(32);
jac(311,563)=jac(311,563)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(563,563)=jac(563,563)-k(31);
jac(556,563)=jac(556,563)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

jac(563,563)=jac(563,563)-k(22);
jac(32,563)=jac(32,563)+k(22);
jac(310,563)=jac(310,563)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(563);
jac(197,563)=jac(197,563)-k(21)*y(197);
jac(563,197)=jac(563,197)-k(21)*y(563);
jac(563,563)=jac(563,563)-k(21)*y(197);
jac(560,197)=jac(560,197)+k(21)*y(563);
jac(560,563)=jac(560,563)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(563);
jac(32,563)=jac(32,563)-k(21)*y(32);
jac(563,32)=jac(563,32)-k(21)*y(563);
jac(563,563)=jac(563,563)-k(21)*y(32);
jac(566,32)=jac(566,32)+k(21)*y(563);
jac(566,563)=jac(566,563)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(563);
jac(2,563)=jac(2,563)-k(21)*y(2);
jac(563,2)=jac(563,2)-k(21)*y(563);
jac(563,563)=jac(563,563)-k(21)*y(2);
jac(570,2)=jac(570,2)+k(21)*y(563);
jac(570,563)=jac(570,563)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(563,563)=jac(563,563)-k(16);
jac(3,563)=jac(3,563)+k(16);
jac(573,563)=jac(573,563)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(563);
jac(3,563)=jac(3,563)-k(15)*y(3);
jac(563,3)=jac(563,3)-k(15)*y(563);
jac(563,563)=jac(563,563)-k(15)*y(3);
jac(633,3)=jac(633,3)+k(15)*y(563);
jac(633,563)=jac(633,563)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(563,563)=jac(563,563)-k(8);
jac(315,563)=jac(315,563)+k(8);
jac(37,563)=jac(37,563)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

jac(563,563)=jac(563,563)-k(6);
jac(77,563)=jac(77,563)+k(6);
jac(554,563)=jac(554,563)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(564,564)=jac(564,564)-k(34);
jac(534,564)=jac(534,564)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(564,564)=jac(564,564)-k(33);
jac(644,564)=jac(644,564)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(564,564)=jac(564,564)-k(32);
jac(318,564)=jac(318,564)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(564,564)=jac(564,564)-k(31);
jac(529,564)=jac(529,564)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

jac(564,564)=jac(564,564)-k(22);
jac(197,564)=jac(197,564)+k(22);
jac(310,564)=jac(310,564)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(564);
jac(197,564)=jac(197,564)-k(21)*y(197);
jac(564,197)=jac(564,197)-k(21)*y(564);
jac(564,564)=jac(564,564)-k(21)*y(197);
jac(565,197)=jac(565,197)+k(21)*y(564);
jac(565,564)=jac(565,564)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(564);
jac(32,564)=jac(32,564)-k(21)*y(32);
jac(564,32)=jac(564,32)-k(21)*y(564);
jac(564,564)=jac(564,564)-k(21)*y(32);
jac(560,32)=jac(560,32)+k(21)*y(564);
jac(560,564)=jac(560,564)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(564);
jac(2,564)=jac(2,564)-k(21)*y(2);
jac(564,2)=jac(564,2)-k(21)*y(564);
jac(564,564)=jac(564,564)-k(21)*y(2);
jac(528,2)=jac(528,2)+k(21)*y(564);
jac(528,564)=jac(528,564)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(564,564)=jac(564,564)-k(16);
jac(3,564)=jac(3,564)+k(16);
jac(613,564)=jac(613,564)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(564);
jac(3,564)=jac(3,564)-k(15)*y(3);
jac(564,3)=jac(564,3)-k(15)*y(564);
jac(564,564)=jac(564,564)-k(15)*y(3);
jac(636,3)=jac(636,3)+k(15)*y(564);
jac(636,564)=jac(636,564)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(564,564)=jac(564,564)-k(8);
jac(54,564)=jac(54,564)+k(8);
jac(319,564)=jac(319,564)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(564,564)=jac(564,564)-k(6);
jac(118,564)=jac(118,564)+k(6);
jac(257,564)=jac(257,564)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(565,565)=jac(565,565)-k(34);
jac(560,565)=jac(560,565)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(565,565)=jac(565,565)-k(34);
jac(569,565)=jac(569,565)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(565,565)=jac(565,565)-k(33);
jac(626,565)=jac(626,565)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(565,565)=jac(565,565)-k(33);
jac(551,565)=jac(551,565)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(565,565)=jac(565,565)-k(31);
jac(552,565)=jac(552,565)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(565,565)=jac(565,565)-k(22);
jac(197,565)=jac(197,565)+k(22);
jac(564,565)=jac(564,565)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(565,565)=jac(565,565)-k(22);
jac(197,565)=jac(197,565)+k(22);
jac(581,565)=jac(581,565)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(565,565)=jac(565,565)-k(16);
jac(3,565)=jac(3,565)+k(16);
jac(583,565)=jac(583,565)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(565);
jac(3,565)=jac(3,565)-k(15)*y(3);
jac(565,3)=jac(565,3)-k(15)*y(565);
jac(565,565)=jac(565,565)-k(15)*y(3);
jac(642,3)=jac(642,3)+k(15)*y(565);
jac(642,565)=jac(642,565)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(565,565)=jac(565,565)-k(8);
jac(319,565)=jac(319,565)+k(8);
jac(227,565)=jac(227,565)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(565,565)=jac(565,565)-k(6);
jac(257,565)=jac(257,565)+k(6);
jac(543,565)=jac(543,565)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(567,567)=jac(567,567)-k(33);
jac(571,567)=jac(571,567)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(567,567)=jac(567,567)-k(33);
jac(631,567)=jac(631,567)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(567,567)=jac(567,567)-k(28);
jac(643,567)=jac(643,567)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(567,567)=jac(567,567)-k(27);
jac(561,567)=jac(561,567)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(567,567)=jac(567,567)-k(22);
jac(32,567)=jac(32,567)+k(22);
jac(633,567)=jac(633,567)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

jac(567,567)=jac(567,567)-k(22);
jac(32,567)=jac(32,567)+k(22);
jac(244,567)=jac(244,567)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(567,567)=jac(567,567)-k(16);
jac(3,567)=jac(3,567)+k(16);
jac(566,567)=jac(566,567)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(567,567)=jac(567,567)-k(16);
jac(3,567)=jac(3,567)+k(16);
jac(247,567)=jac(247,567)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(567,567)=jac(567,567)-k(8);
jac(37,567)=jac(37,567)+k(8);
jac(38,567)=jac(38,567)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(567,567)=jac(567,567)-k(6);
jac(37,567)=jac(37,567)+k(6);
jac(554,567)=jac(554,567)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(568,568)=jac(568,568)-k(33);
jac(499,568)=jac(499,568)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(568,568)=jac(568,568)-k(33);
jac(572,568)=jac(572,568)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(568,568)=jac(568,568)-k(31);
jac(468,568)=jac(468,568)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(568,568)=jac(568,568)-k(31);
jac(612,568)=jac(612,568)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(568,568)=jac(568,568)-k(28);
jac(582,568)=jac(582,568)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(568,568)=jac(568,568)-k(27);
jac(562,568)=jac(562,568)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(568,568)=jac(568,568)-k(22);
jac(32,568)=jac(32,568)+k(22);
jac(245,568)=jac(245,568)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(568,568)=jac(568,568)-k(22);
jac(32,568)=jac(32,568)+k(22);
jac(573,568)=jac(573,568)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(568);
jac(3,568)=jac(3,568)-k(15)*y(3);
jac(568,3)=jac(568,3)-k(15)*y(568);
jac(568,568)=jac(568,568)-k(15)*y(3);
jac(247,3)=jac(247,3)+k(15)*y(568);
jac(247,568)=jac(247,568)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(568);
jac(3,568)=jac(3,568)-k(15)*y(3);
jac(568,3)=jac(568,3)-k(15)*y(568);
jac(568,568)=jac(568,568)-k(15)*y(3);
jac(566,3)=jac(566,3)+k(15)*y(568);
jac(566,568)=jac(566,568)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(568,568)=jac(568,568)-k(8);
jac(33,568)=jac(33,568)+k(8);
jac(34,568)=jac(34,568)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(568,568)=jac(568,568)-k(6);
jac(33,568)=jac(33,568)+k(6);
jac(82,568)=jac(82,568)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(569,569)=jac(569,569)-k(34);
jac(566,569)=jac(566,569)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(569,569)=jac(569,569)-k(33);
jac(496,569)=jac(496,569)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(569,569)=jac(569,569)-k(33);
jac(546,569)=jac(546,569)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(569,569)=jac(569,569)-k(31);
jac(536,569)=jac(536,569)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(569,569)=jac(569,569)-k(29);
jac(565,569)=jac(565,569)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(569,569)=jac(569,569)-k(22);
jac(197,569)=jac(197,569)+k(22);
jac(534,569)=jac(534,569)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(569,569)=jac(569,569)-k(22);
jac(32,569)=jac(32,569)+k(22);
jac(581,569)=jac(581,569)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(569,569)=jac(569,569)-k(16);
jac(3,569)=jac(3,569)+k(16);
jac(582,569)=jac(582,569)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(569);
jac(3,569)=jac(3,569)-k(15)*y(3);
jac(569,3)=jac(569,3)-k(15)*y(569);
jac(569,569)=jac(569,569)-k(15)*y(3);
jac(643,3)=jac(643,3)+k(15)*y(569);
jac(643,569)=jac(643,569)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(569,569)=jac(569,569)-k(8);
jac(34,569)=jac(34,569)+k(8);
jac(227,569)=jac(227,569)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(569,569)=jac(569,569)-k(6);
jac(33,569)=jac(33,569)+k(6);
jac(543,569)=jac(543,569)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(571,571)=jac(571,571)-k(33);
jac(630,571)=jac(630,571)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(571,571)=jac(571,571)-k(28);
jac(258,571)=jac(258,571)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(571,571)=jac(571,571)-k(27);
jac(639,571)=jac(639,571)+k(27);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(571,571)=jac(571,571)-k(25);
jac(567,571)=jac(567,571)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(571,571)=jac(571,571)-k(22);
jac(2,571)=jac(2,571)+k(22);
jac(633,571)=jac(633,571)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

jac(571,571)=jac(571,571)-k(22);
jac(32,571)=jac(32,571)+k(22);
jac(251,571)=jac(251,571)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(571,571)=jac(571,571)-k(16);
jac(3,571)=jac(3,571)+k(16);
jac(570,571)=jac(570,571)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(571,571)=jac(571,571)-k(16);
jac(3,571)=jac(3,571)+k(16);
jac(254,571)=jac(254,571)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(571,571)=jac(571,571)-k(8);
jac(37,571)=jac(37,571)+k(8);
jac(44,571)=jac(44,571)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(571,571)=jac(571,571)-k(6);
jac(43,571)=jac(43,571)+k(6);
jac(554,571)=jac(554,571)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(573,573)=jac(573,573)-k(33);
jac(501,573)=jac(501,573)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(573,573)=jac(573,573)-k(32);
jac(84,573)=jac(84,573)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(573,573)=jac(573,573)-k(31);
jac(615,573)=jac(615,573)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(573,573)=jac(573,573)-k(31);
jac(351,573)=jac(351,573)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(573,573)=jac(573,573)-k(22);
jac(32,573)=jac(32,573)+k(22);
jac(584,573)=jac(584,573)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(573);
jac(197,573)=jac(197,573)-k(21)*y(197);
jac(573,197)=jac(573,197)-k(21)*y(573);
jac(573,573)=jac(573,573)-k(21)*y(197);
jac(562,197)=jac(562,197)+k(21)*y(573);
jac(562,573)=jac(562,573)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(573);
jac(32,573)=jac(32,573)-k(21)*y(32);
jac(573,32)=jac(573,32)-k(21)*y(573);
jac(573,573)=jac(573,573)-k(21)*y(32);
jac(568,32)=jac(568,32)+k(21)*y(573);
jac(568,573)=jac(568,573)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(573);
jac(2,573)=jac(2,573)-k(21)*y(2);
jac(573,2)=jac(573,2)-k(21)*y(573);
jac(573,573)=jac(573,573)-k(21)*y(2);
jac(572,2)=jac(572,2)+k(21)*y(573);
jac(572,573)=jac(572,573)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(573);
jac(3,573)=jac(3,573)-k(15)*y(3);
jac(573,3)=jac(573,3)-k(15)*y(573);
jac(573,573)=jac(573,573)-k(15)*y(3);
jac(634,3)=jac(634,3)+k(15)*y(573);
jac(634,573)=jac(634,573)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(573);
jac(3,573)=jac(3,573)-k(15)*y(3);
jac(573,3)=jac(573,3)-k(15)*y(573);
jac(573,573)=jac(573,573)-k(15)*y(3);
jac(563,3)=jac(563,3)+k(15)*y(573);
jac(563,573)=jac(563,573)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(573,573)=jac(573,573)-k(8);
jac(315,573)=jac(315,573)+k(8);
jac(33,573)=jac(33,573)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

jac(573,573)=jac(573,573)-k(6);
jac(77,573)=jac(77,573)+k(6);
jac(82,573)=jac(82,573)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(576,576)=jac(576,576)-k(34);
jac(252,576)=jac(252,576)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(576,576)=jac(576,576)-k(32);
jac(325,576)=jac(325,576)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(576,576)=jac(576,576)-k(31);
jac(431,576)=jac(431,576)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(576,576)=jac(576,576)-k(31);
jac(344,576)=jac(344,576)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(576,576)=jac(576,576)-k(22);
jac(182,576)=jac(182,576)+k(22);
jac(584,576)=jac(584,576)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(576);
jac(197,576)=jac(197,576)-k(21)*y(197);
jac(576,197)=jac(576,197)-k(21)*y(576);
jac(576,576)=jac(576,576)-k(21)*y(197);
jac(577,197)=jac(577,197)+k(21)*y(576);
jac(577,576)=jac(577,576)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(576);
jac(32,576)=jac(32,576)-k(21)*y(32);
jac(576,32)=jac(576,32)-k(21)*y(576);
jac(576,576)=jac(576,576)-k(21)*y(32);
jac(574,32)=jac(574,32)+k(21)*y(576);
jac(574,576)=jac(574,576)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(576);
jac(2,576)=jac(2,576)-k(21)*y(2);
jac(576,2)=jac(576,2)-k(21)*y(576);
jac(576,576)=jac(576,576)-k(21)*y(2);
jac(505,2)=jac(505,2)+k(21)*y(576);
jac(505,576)=jac(505,576)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(576);
jac(3,576)=jac(3,576)-k(15)*y(3);
jac(576,3)=jac(576,3)-k(15)*y(576);
jac(576,576)=jac(576,576)-k(15)*y(3);
jac(645,3)=jac(645,3)+k(15)*y(576);
jac(645,576)=jac(645,576)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(576);
jac(3,576)=jac(3,576)-k(15)*y(3);
jac(576,3)=jac(576,3)-k(15)*y(576);
jac(576,576)=jac(576,576)-k(15)*y(3);
jac(644,3)=jac(644,3)+k(15)*y(576);
jac(644,576)=jac(644,576)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

jac(576,576)=jac(576,576)-k(8);
jac(77,576)=jac(77,576)+k(8);
jac(219,576)=jac(219,576)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(576,576)=jac(576,576)-k(6);
jac(76,576)=jac(76,576)+k(6);
jac(218,576)=jac(218,576)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(647,647)=jac(647,647)-k(34);
jac(83,647)=jac(83,647)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(647,647)=jac(647,647)-k(33);
jac(301,647)=jac(301,647)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(647,647)=jac(647,647)-k(22);
jac(197,647)=jac(197,647)+k(22);
jac(78,647)=jac(78,647)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(647,647)=jac(647,647)-k(19);
jac(622,647)=jac(622,647)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(647,647)=jac(647,647)-k(16);
jac(3,647)=jac(3,647)+k(16);
jac(203,647)=jac(203,647)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(647,647)=jac(647,647)-k(16);
jac(3,647)=jac(3,647)+k(16);
jac(312,647)=jac(312,647)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(647,647)=jac(647,647)-k(8);
jac(62,647)=jac(62,647)+k(8);
jac(227,647)=jac(227,647)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(647,647)=jac(647,647)-k(6);
jac(61,647)=jac(61,647)+k(6);
jac(543,647)=jac(543,647)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(622,622)=jac(622,622)-k(34);
jac(633,622)=jac(633,622)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(622,622)=jac(622,622)-k(33);
jac(624,622)=jac(624,622)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(622,622)=jac(622,622)-k(32);
jac(647,622)=jac(647,622)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(622,622)=jac(622,622)-k(22);
jac(197,622)=jac(197,622)+k(22);
jac(627,622)=jac(627,622)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(622);
jac(197,622)=jac(197,622)-k(21)*y(197);
jac(622,197)=jac(622,197)-k(21)*y(622);
jac(622,622)=jac(622,622)-k(21)*y(197);
jac(642,197)=jac(642,197)+k(21)*y(622);
jac(642,622)=jac(642,622)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(622);
jac(32,622)=jac(32,622)-k(21)*y(32);
jac(622,32)=jac(622,32)-k(21)*y(622);
jac(622,622)=jac(622,622)-k(21)*y(32);
jac(643,32)=jac(643,32)+k(21)*y(622);
jac(643,622)=jac(643,622)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(622);
jac(2,622)=jac(2,622)-k(21)*y(2);
jac(622,2)=jac(622,2)-k(21)*y(622);
jac(622,622)=jac(622,622)-k(21)*y(2);
jac(258,2)=jac(258,2)+k(21)*y(622);
jac(258,622)=jac(258,622)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(622,622)=jac(622,622)-k(16);
jac(3,622)=jac(3,622)+k(16);
jac(581,622)=jac(581,622)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(622,622)=jac(622,622)-k(16);
jac(3,622)=jac(3,622)+k(16);
jac(260,622)=jac(260,622)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(622,622)=jac(622,622)-k(8);
jac(55,622)=jac(55,622)+k(8);
jac(227,622)=jac(227,622)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(622,622)=jac(622,622)-k(6);
jac(54,622)=jac(54,622)+k(6);
jac(543,622)=jac(543,622)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(627,627)=jac(627,627)-k(32);
jac(236,627)=jac(236,627)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(627,627)=jac(627,627)-k(32);
jac(78,627)=jac(78,627)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(627);
jac(197,627)=jac(197,627)-k(21)*y(197);
jac(627,197)=jac(627,197)-k(21)*y(627);
jac(627,627)=jac(627,627)-k(21)*y(197);
jac(636,197)=jac(636,197)+k(21)*y(627);
jac(636,627)=jac(636,627)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(627);
jac(32,627)=jac(32,627)-k(21)*y(32);
jac(627,32)=jac(627,32)-k(21)*y(627);
jac(627,627)=jac(627,627)-k(21)*y(32);
jac(244,32)=jac(244,32)+k(21)*y(627);
jac(244,627)=jac(244,627)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(627);
jac(2,627)=jac(2,627)-k(21)*y(2);
jac(627,2)=jac(627,2)-k(21)*y(627);
jac(627,627)=jac(627,627)-k(21)*y(2);
jac(251,2)=jac(251,2)+k(21)*y(627);
jac(251,627)=jac(251,627)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(627);
jac(197,627)=jac(197,627)-k(21)*y(197);
jac(627,197)=jac(627,197)-k(21)*y(627);
jac(627,627)=jac(627,627)-k(21)*y(197);
jac(622,197)=jac(622,197)+k(21)*y(627);
jac(622,627)=jac(622,627)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(32,32)=jac(32,32)-k(21)*y(627);
jac(32,627)=jac(32,627)-k(21)*y(32);
jac(627,32)=jac(627,32)-k(21)*y(627);
jac(627,627)=jac(627,627)-k(21)*y(32);
jac(633,32)=jac(633,32)+k(21)*y(627);
jac(633,627)=jac(633,627)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(2,2)=jac(2,2)-k(21)*y(627);
jac(2,627)=jac(2,627)-k(21)*y(2);
jac(627,2)=jac(627,2)-k(21)*y(627);
jac(627,627)=jac(627,627)-k(21)*y(2);
jac(628,2)=jac(628,2)+k(21)*y(627);
jac(628,627)=jac(628,627)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) 

jac(627,627)=jac(627,627)-k(16);
jac(3,627)=jac(3,627)+k(16);
jac(238,627)=jac(238,627)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) 

jac(627,627)=jac(627,627)-k(16);
jac(3,627)=jac(3,627)+k(16);
jac(310,627)=jac(310,627)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(627,627)=jac(627,627)-k(8);
jac(54,627)=jac(54,627)+k(8);
jac(55,627)=jac(55,627)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(627,627)=jac(627,627)-k(6);
jac(54,627)=jac(54,627)+k(6);
jac(118,627)=jac(118,627)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(629,629)=jac(629,629)-k(32);
jac(80,629)=jac(80,629)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(629,629)=jac(629,629)-k(31);
jac(276,629)=jac(276,629)+k(31);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(629,629)=jac(629,629)-k(26);
jac(634,629)=jac(634,629)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

jac(629,629)=jac(629,629)-k(22);
jac(2,629)=jac(2,629)+k(22);
jac(238,629)=jac(238,629)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(629);
jac(197,629)=jac(197,629)-k(21)*y(197);
jac(629,197)=jac(629,197)-k(21)*y(629);
jac(629,629)=jac(629,629)-k(21)*y(197);
jac(485,197)=jac(485,197)+k(21)*y(629);
jac(485,629)=jac(485,629)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(629);
jac(32,629)=jac(32,629)-k(21)*y(32);
jac(629,32)=jac(629,32)-k(21)*y(629);
jac(629,629)=jac(629,629)-k(21)*y(32);
jac(246,32)=jac(246,32)+k(21)*y(629);
jac(246,629)=jac(246,629)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(629);
jac(2,629)=jac(2,629)-k(21)*y(2);
jac(629,2)=jac(629,2)-k(21)*y(629);
jac(629,629)=jac(629,629)-k(21)*y(2);
jac(253,2)=jac(253,2)+k(21)*y(629);
jac(253,629)=jac(253,629)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(629,629)=jac(629,629)-k(16);
jac(3,629)=jac(3,629)+k(16);
jac(501,629)=jac(501,629)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(629);
jac(3,629)=jac(3,629)-k(15)*y(3);
jac(629,3)=jac(629,3)-k(15)*y(629);
jac(629,629)=jac(629,629)-k(15)*y(3);
jac(628,3)=jac(628,3)+k(15)*y(629);
jac(628,629)=jac(628,629)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(629,629)=jac(629,629)-k(8);
jac(55,629)=jac(55,629)+k(8);
jac(111,629)=jac(111,629)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(629,629)=jac(629,629)-k(6);
jac(54,629)=jac(54,629)+k(6);
jac(110,629)=jac(110,629)+k(6);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(630,630)=jac(630,630)-k(28);
jac(265,630)=jac(265,630)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(630,630)=jac(630,630)-k(27);
jac(637,630)=jac(637,630)+k(27);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(630,630)=jac(630,630)-k(26);
jac(571,630)=jac(571,630)+k(26);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(630,630)=jac(630,630)-k(25);
jac(631,630)=jac(631,630)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(630,630)=jac(630,630)-k(22);
jac(2,630)=jac(2,630)+k(22);
jac(628,630)=jac(628,630)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

jac(630,630)=jac(630,630)-k(22);
jac(2,630)=jac(2,630)+k(22);
jac(251,630)=jac(251,630)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(630,630)=jac(630,630)-k(16);
jac(3,630)=jac(3,630)+k(16);
jac(504,630)=jac(504,630)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(630,630)=jac(630,630)-k(16);
jac(3,630)=jac(3,630)+k(16);
jac(253,630)=jac(253,630)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(630,630)=jac(630,630)-k(8);
jac(43,630)=jac(43,630)+k(8);
jac(44,630)=jac(44,630)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(630,630)=jac(630,630)-k(6);
jac(43,630)=jac(43,630)+k(6);
jac(114,630)=jac(114,630)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(631,631)=jac(631,631)-k(33);
jac(630,631)=jac(630,631)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(631,631)=jac(631,631)-k(28);
jac(494,631)=jac(494,631)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(631,631)=jac(631,631)-k(27);
jac(632,631)=jac(632,631)+k(27);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(631,631)=jac(631,631)-k(26);
jac(567,631)=jac(567,631)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(631,631)=jac(631,631)-k(22);
jac(32,631)=jac(32,631)+k(22);
jac(628,631)=jac(628,631)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

jac(631,631)=jac(631,631)-k(22);
jac(2,631)=jac(2,631)+k(22);
jac(244,631)=jac(244,631)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(631,631)=jac(631,631)-k(16);
jac(3,631)=jac(3,631)+k(16);
jac(500,631)=jac(500,631)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(631,631)=jac(631,631)-k(16);
jac(3,631)=jac(3,631)+k(16);
jac(246,631)=jac(246,631)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(631,631)=jac(631,631)-k(8);
jac(43,631)=jac(43,631)+k(8);
jac(38,631)=jac(38,631)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(631,631)=jac(631,631)-k(6);
jac(37,631)=jac(37,631)+k(6);
jac(114,631)=jac(114,631)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(636,636)=jac(636,636)-k(34);
jac(244,636)=jac(244,636)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(636,636)=jac(636,636)-k(33);
jac(638,636)=jac(638,636)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(636,636)=jac(636,636)-k(32);
jac(320,636)=jac(320,636)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(636,636)=jac(636,636)-k(22);
jac(197,636)=jac(197,636)+k(22);
jac(627,636)=jac(627,636)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(636);
jac(197,636)=jac(197,636)-k(21)*y(197);
jac(636,197)=jac(636,197)-k(21)*y(636);
jac(636,636)=jac(636,636)-k(21)*y(197);
jac(642,197)=jac(642,197)+k(21)*y(636);
jac(642,636)=jac(642,636)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(636);
jac(32,636)=jac(32,636)-k(21)*y(32);
jac(636,32)=jac(636,32)-k(21)*y(636);
jac(636,636)=jac(636,636)-k(21)*y(32);
jac(561,32)=jac(561,32)+k(21)*y(636);
jac(561,636)=jac(561,636)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(636);
jac(2,636)=jac(2,636)-k(21)*y(2);
jac(636,2)=jac(636,2)-k(21)*y(636);
jac(636,636)=jac(636,636)-k(21)*y(2);
jac(632,2)=jac(632,2)+k(21)*y(636);
jac(632,636)=jac(632,636)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(636,636)=jac(636,636)-k(16);
jac(3,636)=jac(3,636)+k(16);
jac(491,636)=jac(491,636)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(636,636)=jac(636,636)-k(16);
jac(3,636)=jac(3,636)+k(16);
jac(564,636)=jac(564,636)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(636,636)=jac(636,636)-k(8);
jac(54,636)=jac(54,636)+k(8);
jac(228,636)=jac(228,636)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(636,636)=jac(636,636)-k(6);
jac(118,636)=jac(118,636)+k(6);
jac(227,636)=jac(227,636)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(644,644)=jac(644,644)-k(34);
jac(540,644)=jac(540,644)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(644,644)=jac(644,644)-k(32);
jac(323,644)=jac(323,644)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(644,644)=jac(644,644)-k(31);
jac(519,644)=jac(519,644)+k(31);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(644,644)=jac(644,644)-k(25);
jac(564,644)=jac(564,644)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

jac(644,644)=jac(644,644)-k(22);
jac(182,644)=jac(182,644)+k(22);
jac(310,644)=jac(310,644)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(644);
jac(197,644)=jac(197,644)-k(21)*y(197);
jac(644,197)=jac(644,197)-k(21)*y(644);
jac(644,644)=jac(644,644)-k(21)*y(197);
jac(551,197)=jac(551,197)+k(21)*y(644);
jac(551,644)=jac(551,644)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(644);
jac(32,644)=jac(32,644)-k(21)*y(32);
jac(644,32)=jac(644,32)-k(21)*y(644);
jac(644,644)=jac(644,644)-k(21)*y(32);
jac(555,32)=jac(555,32)+k(21)*y(644);
jac(555,644)=jac(555,644)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(644);
jac(2,644)=jac(2,644)-k(21)*y(2);
jac(644,2)=jac(644,2)-k(21)*y(644);
jac(644,644)=jac(644,644)-k(21)*y(2);
jac(523,2)=jac(523,2)+k(21)*y(644);
jac(523,644)=jac(523,644)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(644,644)=jac(644,644)-k(16);
jac(3,644)=jac(3,644)+k(16);
jac(576,644)=jac(576,644)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(644);
jac(3,644)=jac(3,644)-k(15)*y(3);
jac(644,3)=jac(644,3)-k(15)*y(644);
jac(644,644)=jac(644,644)-k(15)*y(3);
jac(638,3)=jac(638,3)+k(15)*y(644);
jac(638,644)=jac(638,644)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(644,644)=jac(644,644)-k(8);
jac(54,644)=jac(54,644)+k(8);
jac(219,644)=jac(219,644)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(644,644)=jac(644,644)-k(6);
jac(118,644)=jac(118,644)+k(6);
jac(218,644)=jac(218,644)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(646,646)=jac(646,646)-k(34);
jac(239,646)=jac(239,646)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(646,646)=jac(646,646)-k(31);
jac(224,646)=jac(224,646)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

jac(646,646)=jac(646,646)-k(22);
jac(182,646)=jac(182,646)+k(22);
jac(235,646)=jac(235,646)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(646,646)=jac(646,646)-k(20);
jac(645,646)=jac(645,646)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(646,646)=jac(646,646)-k(16);
jac(3,646)=jac(3,646)+k(16);
jac(325,646)=jac(325,646)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(646);
jac(3,646)=jac(3,646)-k(15)*y(3);
jac(646,3)=jac(646,3)-k(15)*y(646);
jac(646,646)=jac(646,646)-k(15)*y(3);
jac(324,3)=jac(324,3)+k(15)*y(646);
jac(324,646)=jac(324,646)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

jac(646,646)=jac(646,646)-k(8);
jac(72,646)=jac(72,646)+k(8);
jac(223,646)=jac(223,646)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(646,646)=jac(646,646)-k(6);
jac(71,646)=jac(71,646)+k(6);
jac(222,646)=jac(222,646)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(645,645)=jac(645,645)-k(34);
jac(250,645)=jac(250,645)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(645,645)=jac(645,645)-k(32);
jac(646,645)=jac(646,645)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(645,645)=jac(645,645)-k(31);
jac(282,645)=jac(282,645)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

jac(645,645)=jac(645,645)-k(22);
jac(182,645)=jac(182,645)+k(22);
jac(238,645)=jac(238,645)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(645);
jac(197,645)=jac(197,645)-k(21)*y(197);
jac(645,197)=jac(645,197)-k(21)*y(645);
jac(645,645)=jac(645,645)-k(21)*y(197);
jac(261,197)=jac(261,197)+k(21)*y(645);
jac(261,645)=jac(261,645)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(645);
jac(32,645)=jac(32,645)-k(21)*y(32);
jac(645,32)=jac(645,32)-k(21)*y(645);
jac(645,645)=jac(645,645)-k(21)*y(32);
jac(286,32)=jac(286,32)+k(21)*y(645);
jac(286,645)=jac(286,645)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(645);
jac(2,645)=jac(2,645)-k(21)*y(2);
jac(645,2)=jac(645,2)-k(21)*y(645);
jac(645,645)=jac(645,645)-k(21)*y(2);
jac(281,2)=jac(281,2)+k(21)*y(645);
jac(281,645)=jac(281,645)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(645,645)=jac(645,645)-k(16);
jac(3,645)=jac(3,645)+k(16);
jac(576,645)=jac(576,645)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(645);
jac(3,645)=jac(3,645)-k(15)*y(3);
jac(645,3)=jac(645,3)-k(15)*y(645);
jac(645,645)=jac(645,645)-k(15)*y(3);
jac(638,3)=jac(638,3)+k(15)*y(645);
jac(638,645)=jac(638,645)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

jac(645,645)=jac(645,645)-k(8);
jac(77,645)=jac(77,645)+k(8);
jac(223,645)=jac(223,645)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(645,645)=jac(645,645)-k(6);
jac(76,645)=jac(76,645)+k(6);
jac(222,645)=jac(222,645)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(638,638)=jac(638,638)-k(34);
jac(251,638)=jac(251,638)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(638,638)=jac(638,638)-k(32);
jac(324,638)=jac(324,638)+k(32);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(638,638)=jac(638,638)-k(25);
jac(636,638)=jac(636,638)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(638,638)=jac(638,638)-k(22);
jac(182,638)=jac(182,638)+k(22);
jac(627,638)=jac(627,638)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(638);
jac(197,638)=jac(197,638)-k(21)*y(197);
jac(638,197)=jac(638,197)-k(21)*y(638);
jac(638,638)=jac(638,638)-k(21)*y(197);
jac(641,197)=jac(641,197)+k(21)*y(638);
jac(641,638)=jac(641,638)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(638);
jac(32,638)=jac(32,638)-k(21)*y(32);
jac(638,32)=jac(638,32)-k(21)*y(638);
jac(638,638)=jac(638,638)-k(21)*y(32);
jac(639,32)=jac(639,32)+k(21)*y(638);
jac(639,638)=jac(639,638)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(638);
jac(2,638)=jac(2,638)-k(21)*y(2);
jac(638,2)=jac(638,2)-k(21)*y(638);
jac(638,638)=jac(638,638)-k(21)*y(2);
jac(637,2)=jac(637,2)+k(21)*y(638);
jac(637,638)=jac(637,638)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(638,638)=jac(638,638)-k(16);
jac(3,638)=jac(3,638)+k(16);
jac(645,638)=jac(645,638)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(638,638)=jac(638,638)-k(16);
jac(3,638)=jac(3,638)+k(16);
jac(644,638)=jac(644,638)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(638,638)=jac(638,638)-k(8);
jac(54,638)=jac(54,638)+k(8);
jac(223,638)=jac(223,638)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(638,638)=jac(638,638)-k(6);
jac(118,638)=jac(118,638)+k(6);
jac(222,638)=jac(222,638)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(639,639)=jac(639,639)-k(34);
jac(571,639)=jac(571,639)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(639,639)=jac(639,639)-k(33);
jac(637,639)=jac(637,639)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(639,639)=jac(639,639)-k(30);
jac(641,639)=jac(641,639)+k(30);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(639,639)=jac(639,639)-k(25);
jac(561,639)=jac(561,639)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

jac(639,639)=jac(639,639)-k(22);
jac(32,639)=jac(32,639)+k(22);
jac(638,639)=jac(638,639)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(639,639)=jac(639,639)-k(22);
jac(182,639)=jac(182,639)+k(22);
jac(633,639)=jac(633,639)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(639,639)=jac(639,639)-k(16);
jac(3,639)=jac(3,639)+k(16);
jac(286,639)=jac(286,639)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(639,639)=jac(639,639)-k(16);
jac(3,639)=jac(3,639)+k(16);
jac(555,639)=jac(555,639)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(639,639)=jac(639,639)-k(8);
jac(37,639)=jac(37,639)+k(8);
jac(223,639)=jac(223,639)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(639,639)=jac(639,639)-k(6);
jac(222,639)=jac(222,639)+k(6);
jac(554,639)=jac(554,639)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(643,643)=jac(643,643)-k(34);
jac(567,643)=jac(567,643)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(643,643)=jac(643,643)-k(33);
jac(494,643)=jac(494,643)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(643,643)=jac(643,643)-k(33);
jac(258,643)=jac(258,643)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(643,643)=jac(643,643)-k(29);
jac(642,643)=jac(642,643)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

jac(643,643)=jac(643,643)-k(22);
jac(197,643)=jac(197,643)+k(22);
jac(244,643)=jac(244,643)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

jac(643,643)=jac(643,643)-k(22);
jac(32,643)=jac(32,643)+k(22);
jac(622,643)=jac(622,643)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(643,643)=jac(643,643)-k(16);
jac(3,643)=jac(3,643)+k(16);
jac(248,643)=jac(248,643)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(643,643)=jac(643,643)-k(16);
jac(3,643)=jac(3,643)+k(16);
jac(569,643)=jac(569,643)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(643,643)=jac(643,643)-k(8);
jac(227,643)=jac(227,643)+k(8);
jac(38,643)=jac(38,643)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(643,643)=jac(643,643)-k(6);
jac(37,643)=jac(37,643)+k(6);
jac(543,643)=jac(543,643)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(642,642)=jac(642,642)-k(34);
jac(561,642)=jac(561,642)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(642,642)=jac(642,642)-k(34);
jac(643,642)=jac(643,642)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(642,642)=jac(642,642)-k(33);
jac(489,642)=jac(489,642)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(642,642)=jac(642,642)-k(33);
jac(641,642)=jac(641,642)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

jac(642,642)=jac(642,642)-k(22);
jac(197,642)=jac(197,642)+k(22);
jac(636,642)=jac(636,642)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

jac(642,642)=jac(642,642)-k(22);
jac(197,642)=jac(197,642)+k(22);
jac(622,642)=jac(622,642)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(642,642)=jac(642,642)-k(16);
jac(3,642)=jac(3,642)+k(16);
jac(492,642)=jac(492,642)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(642,642)=jac(642,642)-k(16);
jac(3,642)=jac(3,642)+k(16);
jac(565,642)=jac(565,642)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(642,642)=jac(642,642)-k(8);
jac(227,642)=jac(227,642)+k(8);
jac(228,642)=jac(228,642)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(642,642)=jac(642,642)-k(6);
jac(227,642)=jac(227,642)+k(6);
jac(543,642)=jac(543,642)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(641,641)=jac(641,641)-k(34);
jac(639,641)=jac(639,641)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(641,641)=jac(641,641)-k(34);
jac(258,641)=jac(258,641)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(641,641)=jac(641,641)-k(33);
jac(640,641)=jac(640,641)+k(33);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(641,641)=jac(641,641)-k(25);
jac(642,641)=jac(642,641)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

jac(641,641)=jac(641,641)-k(22);
jac(197,641)=jac(197,641)+k(22);
jac(638,641)=jac(638,641)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

jac(641,641)=jac(641,641)-k(22);
jac(182,641)=jac(182,641)+k(22);
jac(622,641)=jac(622,641)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(641,641)=jac(641,641)-k(16);
jac(3,641)=jac(3,641)+k(16);
jac(261,641)=jac(261,641)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(641,641)=jac(641,641)-k(16);
jac(3,641)=jac(3,641)+k(16);
jac(551,641)=jac(551,641)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(641,641)=jac(641,641)-k(8);
jac(227,641)=jac(227,641)+k(8);
jac(223,641)=jac(223,641)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(641,641)=jac(641,641)-k(6);
jac(222,641)=jac(222,641)+k(6);
jac(543,641)=jac(543,641)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(640,640)=jac(640,640)-k(34);
jac(637,640)=jac(637,640)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(640,640)=jac(640,640)-k(34);
jac(265,640)=jac(265,640)+k(34);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(640,640)=jac(640,640)-k(26);
jac(641,640)=jac(641,640)+k(26);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(640,640)=jac(640,640)-k(25);
jac(489,640)=jac(489,640)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

jac(640,640)=jac(640,640)-k(22);
jac(182,640)=jac(182,640)+k(22);
jac(638,640)=jac(638,640)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

jac(640,640)=jac(640,640)-k(22);
jac(182,640)=jac(182,640)+k(22);
jac(624,640)=jac(624,640)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(640,640)=jac(640,640)-k(16);
jac(3,640)=jac(3,640)+k(16);
jac(268,640)=jac(268,640)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(640,640)=jac(640,640)-k(16);
jac(3,640)=jac(3,640)+k(16);
jac(517,640)=jac(517,640)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(640,640)=jac(640,640)-k(8);
jac(222,640)=jac(222,640)+k(8);
jac(223,640)=jac(223,640)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(640,640)=jac(640,640)-k(6);
jac(222,640)=jac(222,640)+k(6);
jac(515,640)=jac(515,640)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(637,637)=jac(637,637)-k(34);
jac(630,637)=jac(630,637)+k(34);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(637,637)=jac(637,637)-k(30);
jac(640,637)=jac(640,637)+k(30);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(637,637)=jac(637,637)-k(26);
jac(639,637)=jac(639,637)+k(26);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(637,637)=jac(637,637)-k(25);
jac(632,637)=jac(632,637)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

jac(637,637)=jac(637,637)-k(22);
jac(2,637)=jac(2,637)+k(22);
jac(638,637)=jac(638,637)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(637,637)=jac(637,637)-k(22);
jac(182,637)=jac(182,637)+k(22);
jac(628,637)=jac(628,637)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(637,637)=jac(637,637)-k(16);
jac(3,637)=jac(3,637)+k(16);
jac(281,637)=jac(281,637)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(637,637)=jac(637,637)-k(16);
jac(3,637)=jac(3,637)+k(16);
jac(523,637)=jac(523,637)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(637,637)=jac(637,637)-k(8);
jac(43,637)=jac(43,637)+k(8);
jac(223,637)=jac(223,637)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(637,637)=jac(637,637)-k(6);
jac(222,637)=jac(222,637)+k(6);
jac(114,637)=jac(114,637)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(632,632)=jac(632,632)-k(34);
jac(631,632)=jac(631,632)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(632,632)=jac(632,632)-k(33);
jac(637,632)=jac(637,632)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(632,632)=jac(632,632)-k(30);
jac(489,632)=jac(489,632)+k(30);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(632,632)=jac(632,632)-k(26);
jac(561,632)=jac(561,632)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(632,632)=jac(632,632)-k(22);
jac(197,632)=jac(197,632)+k(22);
jac(628,632)=jac(628,632)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) 

jac(632,632)=jac(632,632)-k(22);
jac(2,632)=jac(2,632)+k(22);
jac(636,632)=jac(636,632)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(632,632)=jac(632,632)-k(16);
jac(3,632)=jac(3,632)+k(16);
jac(528,632)=jac(528,632)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(632,632)=jac(632,632)-k(16);
jac(3,632)=jac(3,632)+k(16);
jac(485,632)=jac(485,632)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(632,632)=jac(632,632)-k(8);
jac(43,632)=jac(43,632)+k(8);
jac(228,632)=jac(228,632)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(632,632)=jac(632,632)-k(6);
jac(227,632)=jac(227,632)+k(6);
jac(114,632)=jac(114,632)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(635,635)=jac(635,635)-k(34);
jac(247,635)=jac(247,635)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(635,635)=jac(635,635)-k(33);
jac(286,635)=jac(286,635)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(635,635)=jac(635,635)-k(33);
jac(485,635)=jac(485,635)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(635,635)=jac(635,635)-k(31);
jac(474,635)=jac(474,635)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(635,635)=jac(635,635)-k(30);
jac(492,635)=jac(492,635)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(635,635)=jac(635,635)-k(22);
jac(197,635)=jac(197,635)+k(22);
jac(634,635)=jac(634,635)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(635,635)=jac(635,635)-k(22);
jac(32,635)=jac(32,635)+k(22);
jac(491,635)=jac(491,635)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(635,635)=jac(635,635)-k(16);
jac(3,635)=jac(3,635)+k(16);
jac(562,635)=jac(562,635)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(635);
jac(3,635)=jac(3,635)-k(15)*y(3);
jac(635,3)=jac(635,3)-k(15)*y(635);
jac(635,635)=jac(635,635)-k(15)*y(3);
jac(561,3)=jac(561,3)+k(15)*y(635);
jac(561,635)=jac(561,635)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(635,635)=jac(635,635)-k(8);
jac(33,635)=jac(33,635)+k(8);
jac(228,635)=jac(228,635)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(635,635)=jac(635,635)-k(6);
jac(82,635)=jac(82,635)+k(6);
jac(227,635)=jac(227,635)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(634,634)=jac(634,634)-k(33);
jac(629,634)=jac(629,634)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(634,634)=jac(634,634)-k(32);
jac(81,634)=jac(81,634)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(634,634)=jac(634,634)-k(31);
jac(287,634)=jac(287,634)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

jac(634,634)=jac(634,634)-k(22);
jac(32,634)=jac(32,634)+k(22);
jac(238,634)=jac(238,634)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(634);
jac(197,634)=jac(197,634)-k(21)*y(197);
jac(634,197)=jac(634,197)-k(21)*y(634);
jac(634,634)=jac(634,634)-k(21)*y(197);
jac(635,197)=jac(635,197)+k(21)*y(634);
jac(635,634)=jac(635,634)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(634);
jac(32,634)=jac(32,634)-k(21)*y(32);
jac(634,32)=jac(634,32)-k(21)*y(634);
jac(634,634)=jac(634,634)-k(21)*y(32);
jac(247,32)=jac(247,32)+k(21)*y(634);
jac(247,634)=jac(247,634)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(634);
jac(2,634)=jac(2,634)-k(21)*y(2);
jac(634,2)=jac(634,2)-k(21)*y(634);
jac(634,634)=jac(634,634)-k(21)*y(2);
jac(254,2)=jac(254,2)+k(21)*y(634);
jac(254,634)=jac(254,634)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(634,634)=jac(634,634)-k(16);
jac(3,634)=jac(3,634)+k(16);
jac(573,634)=jac(573,634)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(634);
jac(3,634)=jac(3,634)-k(15)*y(3);
jac(634,3)=jac(634,3)-k(15)*y(634);
jac(634,634)=jac(634,634)-k(15)*y(3);
jac(633,3)=jac(633,3)+k(15)*y(634);
jac(633,634)=jac(633,634)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(634,634)=jac(634,634)-k(8);
jac(55,634)=jac(55,634)+k(8);
jac(33,634)=jac(33,634)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(634,634)=jac(634,634)-k(6);
jac(54,634)=jac(54,634)+k(6);
jac(82,634)=jac(82,634)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(633,633)=jac(633,633)-k(33);
jac(628,633)=jac(628,633)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(633,633)=jac(633,633)-k(32);
jac(83,633)=jac(83,633)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(633,633)=jac(633,633)-k(22);
jac(32,633)=jac(32,633)+k(22);
jac(627,633)=jac(627,633)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(633);
jac(197,633)=jac(197,633)-k(21)*y(197);
jac(633,197)=jac(633,197)-k(21)*y(633);
jac(633,633)=jac(633,633)-k(21)*y(197);
jac(561,197)=jac(561,197)+k(21)*y(633);
jac(561,633)=jac(561,633)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(633);
jac(32,633)=jac(32,633)-k(21)*y(32);
jac(633,32)=jac(633,32)-k(21)*y(633);
jac(633,633)=jac(633,633)-k(21)*y(32);
jac(567,32)=jac(567,32)+k(21)*y(633);
jac(567,633)=jac(567,633)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(633);
jac(2,633)=jac(2,633)-k(21)*y(2);
jac(633,2)=jac(633,2)-k(21)*y(633);
jac(633,633)=jac(633,633)-k(21)*y(2);
jac(571,2)=jac(571,2)+k(21)*y(633);
jac(571,633)=jac(571,633)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(633,633)=jac(633,633)-k(16);
jac(3,633)=jac(3,633)+k(16);
jac(634,633)=jac(634,633)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(633,633)=jac(633,633)-k(16);
jac(3,633)=jac(3,633)+k(16);
jac(563,633)=jac(563,633)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(633,633)=jac(633,633)-k(8);
jac(55,633)=jac(55,633)+k(8);
jac(37,633)=jac(37,633)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(633,633)=jac(633,633)-k(6);
jac(54,633)=jac(54,633)+k(6);
jac(554,633)=jac(554,633)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(628,628)=jac(628,628)-k(32);
jac(307,628)=jac(307,628)+k(32);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(628,628)=jac(628,628)-k(26);
jac(633,628)=jac(633,628)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(628,628)=jac(628,628)-k(22);
jac(2,628)=jac(2,628)+k(22);
jac(627,628)=jac(627,628)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(628);
jac(197,628)=jac(197,628)-k(21)*y(197);
jac(628,197)=jac(628,197)-k(21)*y(628);
jac(628,628)=jac(628,628)-k(21)*y(197);
jac(632,197)=jac(632,197)+k(21)*y(628);
jac(632,628)=jac(632,628)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(628);
jac(32,628)=jac(32,628)-k(21)*y(32);
jac(628,32)=jac(628,32)-k(21)*y(628);
jac(628,628)=jac(628,628)-k(21)*y(32);
jac(631,32)=jac(631,32)+k(21)*y(628);
jac(631,628)=jac(631,628)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(628);
jac(2,628)=jac(2,628)-k(21)*y(2);
jac(628,2)=jac(628,2)-k(21)*y(628);
jac(628,628)=jac(628,628)-k(21)*y(2);
jac(630,2)=jac(630,2)+k(21)*y(628);
jac(630,628)=jac(630,628)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(628,628)=jac(628,628)-k(16);
jac(3,628)=jac(3,628)+k(16);
jac(629,628)=jac(629,628)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(628,628)=jac(628,628)-k(16);
jac(3,628)=jac(3,628)+k(16);
jac(308,628)=jac(308,628)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(628,628)=jac(628,628)-k(8);
jac(55,628)=jac(55,628)+k(8);
jac(43,628)=jac(43,628)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(628,628)=jac(628,628)-k(6);
jac(54,628)=jac(54,628)+k(6);
jac(114,628)=jac(114,628)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(624,624)=jac(624,624)-k(34);
jac(628,624)=jac(628,624)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(624,624)=jac(624,624)-k(32);
jac(301,624)=jac(301,624)+k(32);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(624,624)=jac(624,624)-k(26);
jac(622,624)=jac(622,624)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(624,624)=jac(624,624)-k(22);
jac(182,624)=jac(182,624)+k(22);
jac(627,624)=jac(627,624)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(197,197)=jac(197,197)-k(21)*y(624);
jac(197,624)=jac(197,624)-k(21)*y(197);
jac(624,197)=jac(624,197)-k(21)*y(624);
jac(624,624)=jac(624,624)-k(21)*y(197);
jac(489,197)=jac(489,197)+k(21)*y(624);
jac(489,624)=jac(489,624)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(624);
jac(32,624)=jac(32,624)-k(21)*y(32);
jac(624,32)=jac(624,32)-k(21)*y(624);
jac(624,624)=jac(624,624)-k(21)*y(32);
jac(494,32)=jac(494,32)+k(21)*y(624);
jac(494,624)=jac(494,624)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(2,2)=jac(2,2)-k(21)*y(624);
jac(2,624)=jac(2,624)-k(21)*y(2);
jac(624,2)=jac(624,2)-k(21)*y(624);
jac(624,624)=jac(624,624)-k(21)*y(2);
jac(265,2)=jac(265,2)+k(21)*y(624);
jac(265,624)=jac(265,624)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(624,624)=jac(624,624)-k(16);
jac(3,624)=jac(3,624)+k(16);
jac(623,624)=jac(623,624)+k(16);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(624,624)=jac(624,624)-k(16);
jac(3,624)=jac(3,624)+k(16);
jac(267,624)=jac(267,624)+k(16);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(624,624)=jac(624,624)-k(8);
jac(55,624)=jac(55,624)+k(8);
jac(222,624)=jac(222,624)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(624,624)=jac(624,624)-k(6);
jac(54,624)=jac(54,624)+k(6);
jac(515,624)=jac(515,624)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(625,625)=jac(625,625)-k(34);
jac(504,625)=jac(504,625)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(625,625)=jac(625,625)-k(31);
jac(511,625)=jac(511,625)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(625,625)=jac(625,625)-k(29);
jac(517,625)=jac(517,625)+k(29);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(625,625)=jac(625,625)-k(25);
jac(496,625)=jac(496,625)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(625,625)=jac(625,625)-k(22);
jac(2,625)=jac(2,625)+k(22);
jac(623,625)=jac(623,625)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(625,625)=jac(625,625)-k(22);
jac(182,625)=jac(182,625)+k(22);
jac(540,625)=jac(540,625)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(625,625)=jac(625,625)-k(16);
jac(3,625)=jac(3,625)+k(16);
jac(266,625)=jac(266,625)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(625);
jac(3,625)=jac(3,625)-k(15)*y(3);
jac(625,3)=jac(625,3)-k(15)*y(625);
jac(625,625)=jac(625,625)-k(15)*y(3);
jac(265,3)=jac(265,3)+k(15)*y(625);
jac(265,625)=jac(265,625)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(625,625)=jac(625,625)-k(8);
jac(437,625)=jac(437,625)+k(8);
jac(222,625)=jac(222,625)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(625,625)=jac(625,625)-k(6);
jac(111,625)=jac(111,625)+k(6);
jac(515,625)=jac(515,625)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(626,626)=jac(626,626)-k(34);
jac(496,626)=jac(496,626)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(626,626)=jac(626,626)-k(34);
jac(528,626)=jac(528,626)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(626,626)=jac(626,626)-k(33);
jac(517,626)=jac(517,626)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(626,626)=jac(626,626)-k(31);
jac(520,626)=jac(520,626)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(626,626)=jac(626,626)-k(22);
jac(197,626)=jac(197,626)+k(22);
jac(623,626)=jac(623,626)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(626,626)=jac(626,626)-k(22);
jac(182,626)=jac(182,626)+k(22);
jac(564,626)=jac(564,626)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(626,626)=jac(626,626)-k(16);
jac(3,626)=jac(3,626)+k(16);
jac(490,626)=jac(490,626)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(626);
jac(3,626)=jac(3,626)-k(15)*y(3);
jac(626,3)=jac(626,3)-k(15)*y(626);
jac(626,626)=jac(626,626)-k(15)*y(3);
jac(489,3)=jac(489,3)+k(15)*y(626);
jac(489,626)=jac(489,626)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(626,626)=jac(626,626)-k(8);
jac(319,626)=jac(319,626)+k(8);
jac(222,626)=jac(222,626)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(626,626)=jac(626,626)-k(6);
jac(257,626)=jac(257,626)+k(6);
jac(515,626)=jac(515,626)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(623,623)=jac(623,623)-k(34);
jac(308,623)=jac(308,623)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(623,623)=jac(623,623)-k(32);
jac(303,623)=jac(303,623)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(623,623)=jac(623,623)-k(31);
jac(518,623)=jac(518,623)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

jac(623,623)=jac(623,623)-k(22);
jac(182,623)=jac(182,623)+k(22);
jac(310,623)=jac(310,623)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(623);
jac(197,623)=jac(197,623)-k(21)*y(197);
jac(623,197)=jac(623,197)-k(21)*y(623);
jac(623,623)=jac(623,623)-k(21)*y(197);
jac(626,197)=jac(626,197)+k(21)*y(623);
jac(626,623)=jac(626,623)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(623);
jac(32,623)=jac(32,623)-k(21)*y(32);
jac(623,32)=jac(623,32)-k(21)*y(623);
jac(623,623)=jac(623,623)-k(21)*y(32);
jac(496,32)=jac(496,32)+k(21)*y(623);
jac(496,623)=jac(496,623)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(623);
jac(2,623)=jac(2,623)-k(21)*y(2);
jac(623,2)=jac(623,2)-k(21)*y(623);
jac(623,623)=jac(623,623)-k(21)*y(2);
jac(625,2)=jac(625,2)+k(21)*y(623);
jac(625,623)=jac(625,623)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(623,623)=jac(623,623)-k(16);
jac(3,623)=jac(3,623)+k(16);
jac(497,623)=jac(497,623)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(623);
jac(3,623)=jac(3,623)-k(15)*y(3);
jac(623,3)=jac(623,3)-k(15)*y(623);
jac(623,623)=jac(623,623)-k(15)*y(3);
jac(624,3)=jac(624,3)+k(15)*y(623);
jac(624,623)=jac(624,623)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(623,623)=jac(623,623)-k(8);
jac(315,623)=jac(315,623)+k(8);
jac(222,623)=jac(222,623)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

jac(623,623)=jac(623,623)-k(6);
jac(77,623)=jac(77,623)+k(6);
jac(515,623)=jac(515,623)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(581,581)=jac(581,581)-k(34);
jac(563,581)=jac(563,581)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(581,581)=jac(581,581)-k(33);
jac(623,581)=jac(623,581)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(581,581)=jac(581,581)-k(32);
jac(312,581)=jac(312,581)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(581,581)=jac(581,581)-k(31);
jac(547,581)=jac(547,581)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

jac(581,581)=jac(581,581)-k(22);
jac(197,581)=jac(197,581)+k(22);
jac(310,581)=jac(310,581)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(581);
jac(197,581)=jac(197,581)-k(21)*y(197);
jac(581,197)=jac(581,197)-k(21)*y(581);
jac(581,581)=jac(581,581)-k(21)*y(197);
jac(565,197)=jac(565,197)+k(21)*y(581);
jac(565,581)=jac(565,581)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(581);
jac(32,581)=jac(32,581)-k(21)*y(32);
jac(581,32)=jac(581,32)-k(21)*y(581);
jac(581,581)=jac(581,581)-k(21)*y(32);
jac(569,32)=jac(569,32)+k(21)*y(581);
jac(569,581)=jac(569,581)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(581);
jac(2,581)=jac(2,581)-k(21)*y(2);
jac(581,2)=jac(581,2)-k(21)*y(581);
jac(581,581)=jac(581,581)-k(21)*y(2);
jac(546,2)=jac(546,2)+k(21)*y(581);
jac(546,581)=jac(546,581)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(581,581)=jac(581,581)-k(16);
jac(3,581)=jac(3,581)+k(16);
jac(578,581)=jac(578,581)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(581);
jac(3,581)=jac(3,581)-k(15)*y(3);
jac(581,3)=jac(581,3)-k(15)*y(581);
jac(581,581)=jac(581,581)-k(15)*y(3);
jac(622,3)=jac(622,3)+k(15)*y(581);
jac(622,581)=jac(622,581)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(581,581)=jac(581,581)-k(8);
jac(315,581)=jac(315,581)+k(8);
jac(227,581)=jac(227,581)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

jac(581,581)=jac(581,581)-k(6);
jac(77,581)=jac(77,581)+k(6);
jac(543,581)=jac(543,581)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(582,582)=jac(582,582)-k(34);
jac(568,582)=jac(568,582)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(582,582)=jac(582,582)-k(33);
jac(259,582)=jac(259,582)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(582,582)=jac(582,582)-k(33);
jac(495,582)=jac(495,582)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(582,582)=jac(582,582)-k(31);
jac(608,582)=jac(608,582)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(582,582)=jac(582,582)-k(31);
jac(620,582)=jac(620,582)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(582,582)=jac(582,582)-k(29);
jac(583,582)=jac(583,582)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(582,582)=jac(582,582)-k(22);
jac(32,582)=jac(32,582)+k(22);
jac(578,582)=jac(578,582)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(582,582)=jac(582,582)-k(22);
jac(197,582)=jac(197,582)+k(22);
jac(245,582)=jac(245,582)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(582);
jac(3,582)=jac(3,582)-k(15)*y(3);
jac(582,3)=jac(582,3)-k(15)*y(582);
jac(582,582)=jac(582,582)-k(15)*y(3);
jac(569,3)=jac(569,3)+k(15)*y(582);
jac(569,582)=jac(569,582)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(582);
jac(3,582)=jac(3,582)-k(15)*y(3);
jac(582,3)=jac(582,3)-k(15)*y(582);
jac(582,582)=jac(582,582)-k(15)*y(3);
jac(248,3)=jac(248,3)+k(15)*y(582);
jac(248,582)=jac(248,582)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(582,582)=jac(582,582)-k(8);
jac(257,582)=jac(257,582)+k(8);
jac(34,582)=jac(34,582)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(582,582)=jac(582,582)-k(6);
jac(33,582)=jac(33,582)+k(6);
jac(256,582)=jac(256,582)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(583,583)=jac(583,583)-k(34);
jac(582,583)=jac(582,583)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(583,583)=jac(583,583)-k(34);
jac(562,583)=jac(562,583)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(583,583)=jac(583,583)-k(33);
jac(577,583)=jac(577,583)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(583,583)=jac(583,583)-k(33);
jac(490,583)=jac(490,583)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(583,583)=jac(583,583)-k(31);
jac(609,583)=jac(609,583)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(583,583)=jac(583,583)-k(31);
jac(619,583)=jac(619,583)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(583,583)=jac(583,583)-k(22);
jac(197,583)=jac(197,583)+k(22);
jac(578,583)=jac(578,583)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(583,583)=jac(583,583)-k(22);
jac(197,583)=jac(197,583)+k(22);
jac(613,583)=jac(613,583)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(583);
jac(3,583)=jac(3,583)-k(15)*y(3);
jac(583,3)=jac(583,3)-k(15)*y(583);
jac(583,583)=jac(583,583)-k(15)*y(3);
jac(565,3)=jac(565,3)+k(15)*y(583);
jac(565,583)=jac(565,583)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(583);
jac(3,583)=jac(3,583)-k(15)*y(3);
jac(583,3)=jac(583,3)-k(15)*y(583);
jac(583,583)=jac(583,583)-k(15)*y(3);
jac(492,3)=jac(492,3)+k(15)*y(583);
jac(492,583)=jac(492,583)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(583,583)=jac(583,583)-k(8);
jac(257,583)=jac(257,583)+k(8);
jac(319,583)=jac(319,583)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(583,583)=jac(583,583)-k(6);
jac(257,583)=jac(257,583)+k(6);
jac(256,583)=jac(256,583)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(613,613)=jac(613,613)-k(34);
jac(245,613)=jac(245,613)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(613,613)=jac(613,613)-k(33);
jac(576,613)=jac(576,613)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(613,613)=jac(613,613)-k(32);
jac(321,613)=jac(321,613)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) 

jac(613,613)=jac(613,613)-k(31);
jac(338,613)=jac(338,613)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(613,613)=jac(613,613)-k(31);
jac(618,613)=jac(618,613)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(613,613)=jac(613,613)-k(22);
jac(197,613)=jac(197,613)+k(22);
jac(584,613)=jac(584,613)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(613);
jac(197,613)=jac(197,613)-k(21)*y(197);
jac(613,197)=jac(613,197)-k(21)*y(613);
jac(613,613)=jac(613,613)-k(21)*y(197);
jac(583,197)=jac(583,197)+k(21)*y(613);
jac(583,613)=jac(583,613)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(613);
jac(32,613)=jac(32,613)-k(21)*y(32);
jac(613,32)=jac(613,32)-k(21)*y(613);
jac(613,613)=jac(613,613)-k(21)*y(32);
jac(562,32)=jac(562,32)+k(21)*y(613);
jac(562,613)=jac(562,613)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(613);
jac(2,613)=jac(2,613)-k(21)*y(2);
jac(613,2)=jac(613,2)-k(21)*y(613);
jac(613,613)=jac(613,613)-k(21)*y(2);
jac(502,2)=jac(502,2)+k(21)*y(613);
jac(502,613)=jac(502,613)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(613);
jac(3,613)=jac(3,613)-k(15)*y(3);
jac(613,3)=jac(613,3)-k(15)*y(613);
jac(613,613)=jac(613,613)-k(15)*y(3);
jac(564,3)=jac(564,3)+k(15)*y(613);
jac(564,613)=jac(564,613)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(613);
jac(3,613)=jac(3,613)-k(15)*y(3);
jac(613,3)=jac(613,3)-k(15)*y(613);
jac(613,613)=jac(613,613)-k(15)*y(3);
jac(491,3)=jac(491,3)+k(15)*y(613);
jac(491,613)=jac(491,613)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

jac(613,613)=jac(613,613)-k(8);
jac(77,613)=jac(77,613)+k(8);
jac(319,613)=jac(319,613)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(613,613)=jac(613,613)-k(6);
jac(76,613)=jac(76,613)+k(6);
jac(257,613)=jac(257,613)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(615,615)=jac(615,615)-k(33);
jac(425,615)=jac(425,615)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(615,615)=jac(615,615)-k(32);
jac(87,615)=jac(87,615)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(615,615)=jac(615,615)-k(31);
jac(450,615)=jac(450,615)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(615,615)=jac(615,615)-k(22);
jac(32,615)=jac(32,615)+k(22);
jac(614,615)=jac(614,615)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(615);
jac(197,615)=jac(197,615)-k(21)*y(197);
jac(615,197)=jac(615,197)-k(21)*y(615);
jac(615,615)=jac(615,615)-k(21)*y(197);
jac(472,197)=jac(472,197)+k(21)*y(615);
jac(472,615)=jac(472,615)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(615);
jac(32,615)=jac(32,615)-k(21)*y(32);
jac(615,32)=jac(615,32)-k(21)*y(615);
jac(615,615)=jac(615,615)-k(21)*y(32);
jac(468,32)=jac(468,32)+k(21)*y(615);
jac(468,615)=jac(468,615)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(615);
jac(2,615)=jac(2,615)-k(21)*y(2);
jac(615,2)=jac(615,2)-k(21)*y(615);
jac(615,615)=jac(615,615)-k(21)*y(2);
jac(442,2)=jac(442,2)+k(21)*y(615);
jac(442,615)=jac(442,615)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(615);
jac(3,615)=jac(3,615)-k(15)*y(3);
jac(615,3)=jac(615,3)-k(15)*y(615);
jac(615,615)=jac(615,615)-k(15)*y(3);
jac(287,3)=jac(287,3)+k(15)*y(615);
jac(287,615)=jac(287,615)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(615);
jac(3,615)=jac(3,615)-k(9)*y(3);
jac(615,3)=jac(615,3)-k(9)*y(615);
jac(615,615)=jac(615,615)-k(9)*y(3);
jac(443,3)=jac(443,3)+k(9)*y(615);
jac(443,615)=jac(443,615)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(615,615)=jac(615,615)-k(8);
jac(315,615)=jac(315,615)+k(8);
jac(27,615)=jac(27,615)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

jac(615,615)=jac(615,615)-k(6);
jac(77,615)=jac(77,615)+k(6);
jac(86,615)=jac(86,615)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(621,621)=jac(621,621)-k(31);
jac(24,621)=jac(24,621)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Syk(tSH2, l~Y, a~Y) 

jac(621,621)=jac(621,621)-k(22);
jac(2,621)=jac(2,621)+k(22);
jac(234,621)=jac(234,621)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(621);
jac(3,621)=jac(3,621)-k(15)*y(3);
jac(621,3)=jac(621,3)-k(15)*y(621);
jac(621,621)=jac(621,621)-k(15)*y(3);
jac(45,3)=jac(45,3)+k(15)*y(621);
jac(45,621)=jac(45,621)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(621);
jac(3,621)=jac(3,621)-k(9)*y(3);
jac(621,3)=jac(621,3)-k(9)*y(621);
jac(621,621)=jac(621,621)-k(9)*y(3);
jac(25,3)=jac(25,3)+k(9)*y(621);
jac(25,621)=jac(25,621)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

jac(621,621)=jac(621,621)-k(8);
jac(1,621)=jac(1,621)+k(8);
jac(437,621)=jac(437,621)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(621,621)=jac(621,621)-k(6);
jac(6,621)=jac(6,621)+k(6);
jac(111,621)=jac(111,621)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(616,616)=jac(616,616)-k(32);
jac(621,616)=jac(621,616)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(616,616)=jac(616,616)-k(31);
jac(452,616)=jac(452,616)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(616,616)=jac(616,616)-k(22);
jac(2,616)=jac(2,616)+k(22);
jac(614,616)=jac(614,616)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(616);
jac(197,616)=jac(197,616)-k(21)*y(197);
jac(616,197)=jac(616,197)-k(21)*y(616);
jac(616,616)=jac(616,616)-k(21)*y(197);
jac(438,197)=jac(438,197)+k(21)*y(616);
jac(438,616)=jac(438,616)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(616);
jac(32,616)=jac(32,616)-k(21)*y(32);
jac(616,32)=jac(616,32)-k(21)*y(616);
jac(616,616)=jac(616,616)-k(21)*y(32);
jac(442,32)=jac(442,32)+k(21)*y(616);
jac(442,616)=jac(442,616)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(616);
jac(2,616)=jac(2,616)-k(21)*y(2);
jac(616,2)=jac(616,2)-k(21)*y(616);
jac(616,616)=jac(616,616)-k(21)*y(2);
jac(275,2)=jac(275,2)+k(21)*y(616);
jac(275,616)=jac(275,616)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(616);
jac(3,616)=jac(3,616)-k(15)*y(3);
jac(616,3)=jac(616,3)-k(15)*y(616);
jac(616,616)=jac(616,616)-k(15)*y(3);
jac(49,3)=jac(49,3)+k(15)*y(616);
jac(49,616)=jac(49,616)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(616);
jac(3,616)=jac(3,616)-k(9)*y(3);
jac(616,3)=jac(616,3)-k(9)*y(616);
jac(616,616)=jac(616,616)-k(9)*y(3);
jac(50,3)=jac(50,3)+k(9)*y(616);
jac(50,616)=jac(50,616)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

jac(616,616)=jac(616,616)-k(8);
jac(15,616)=jac(15,616)+k(8);
jac(437,616)=jac(437,616)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(616,616)=jac(616,616)-k(6);
jac(98,616)=jac(98,616)+k(6);
jac(111,616)=jac(111,616)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(617,617)=jac(617,617)-k(33);
jac(616,617)=jac(616,617)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(617,617)=jac(617,617)-k(32);
jac(35,617)=jac(35,617)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(617,617)=jac(617,617)-k(31);
jac(156,617)=jac(156,617)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(617,617)=jac(617,617)-k(22);
jac(32,617)=jac(32,617)+k(22);
jac(614,617)=jac(614,617)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(617);
jac(197,617)=jac(197,617)-k(21)*y(197);
jac(617,197)=jac(617,197)-k(21)*y(617);
jac(617,617)=jac(617,617)-k(21)*y(197);
jac(620,197)=jac(620,197)+k(21)*y(617);
jac(620,617)=jac(620,617)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(617);
jac(32,617)=jac(32,617)-k(21)*y(32);
jac(617,32)=jac(617,32)-k(21)*y(617);
jac(617,617)=jac(617,617)-k(21)*y(32);
jac(468,32)=jac(468,32)+k(21)*y(617);
jac(468,617)=jac(468,617)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(617);
jac(2,617)=jac(2,617)-k(21)*y(2);
jac(617,2)=jac(617,2)-k(21)*y(617);
jac(617,617)=jac(617,617)-k(21)*y(2);
jac(482,2)=jac(482,2)+k(21)*y(617);
jac(482,617)=jac(482,617)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(617);
jac(3,617)=jac(3,617)-k(15)*y(3);
jac(617,3)=jac(617,3)-k(15)*y(617);
jac(617,617)=jac(617,617)-k(15)*y(3);
jac(249,3)=jac(249,3)+k(15)*y(617);
jac(249,617)=jac(249,617)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(617);
jac(3,617)=jac(3,617)-k(9)*y(3);
jac(617,3)=jac(617,3)-k(9)*y(617);
jac(617,617)=jac(617,617)-k(9)*y(3);
jac(470,3)=jac(470,3)+k(9)*y(617);
jac(470,617)=jac(470,617)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

jac(617,617)=jac(617,617)-k(8);
jac(15,617)=jac(15,617)+k(8);
jac(34,617)=jac(34,617)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(617,617)=jac(617,617)-k(6);
jac(98,617)=jac(98,617)+k(6);
jac(33,617)=jac(33,617)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(620,620)=jac(620,620)-k(34);
jac(468,620)=jac(468,620)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(620,620)=jac(620,620)-k(33);
jac(498,620)=jac(498,620)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(620,620)=jac(620,620)-k(33);
jac(438,620)=jac(438,620)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(620,620)=jac(620,620)-k(31);
jac(456,620)=jac(456,620)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(620,620)=jac(620,620)-k(29);
jac(619,620)=jac(619,620)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(620,620)=jac(620,620)-k(22);
jac(197,620)=jac(197,620)+k(22);
jac(617,620)=jac(617,620)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(620,620)=jac(620,620)-k(22);
jac(32,620)=jac(32,620)+k(22);
jac(292,620)=jac(292,620)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(620);
jac(3,620)=jac(3,620)-k(15)*y(3);
jac(620,3)=jac(620,3)-k(15)*y(620);
jac(620,620)=jac(620,620)-k(15)*y(3);
jac(293,3)=jac(293,3)+k(15)*y(620);
jac(293,620)=jac(293,620)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(620);
jac(3,620)=jac(3,620)-k(9)*y(3);
jac(620,3)=jac(620,3)-k(9)*y(620);
jac(620,620)=jac(620,620)-k(9)*y(3);
jac(439,3)=jac(439,3)+k(9)*y(620);
jac(439,620)=jac(439,620)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(620,620)=jac(620,620)-k(8);
jac(192,620)=jac(192,620)+k(8);
jac(34,620)=jac(34,620)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(620,620)=jac(620,620)-k(6);
jac(33,620)=jac(33,620)+k(6);
jac(191,620)=jac(191,620)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(619,619)=jac(619,619)-k(34);
jac(472,619)=jac(472,619)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(619,619)=jac(619,619)-k(34);
jac(620,619)=jac(620,619)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(619,619)=jac(619,619)-k(33);
jac(487,619)=jac(487,619)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(619,619)=jac(619,619)-k(33);
jac(290,619)=jac(290,619)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(619,619)=jac(619,619)-k(31);
jac(455,619)=jac(455,619)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(619,619)=jac(619,619)-k(22);
jac(197,619)=jac(197,619)+k(22);
jac(618,619)=jac(618,619)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(619,619)=jac(619,619)-k(22);
jac(197,619)=jac(197,619)+k(22);
jac(292,619)=jac(292,619)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(619);
jac(3,619)=jac(3,619)-k(15)*y(3);
jac(619,3)=jac(619,3)-k(15)*y(619);
jac(619,619)=jac(619,619)-k(15)*y(3);
jac(294,3)=jac(294,3)+k(15)*y(619);
jac(294,619)=jac(294,619)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(619);
jac(3,619)=jac(3,619)-k(9)*y(3);
jac(619,3)=jac(619,3)-k(9)*y(619);
jac(619,619)=jac(619,619)-k(9)*y(3);
jac(434,3)=jac(434,3)+k(9)*y(619);
jac(434,619)=jac(434,619)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(619,619)=jac(619,619)-k(8);
jac(192,619)=jac(192,619)+k(8);
jac(319,619)=jac(319,619)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(619,619)=jac(619,619)-k(6);
jac(257,619)=jac(257,619)+k(6);
jac(191,619)=jac(191,619)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(618,618)=jac(618,618)-k(34);
jac(617,618)=jac(617,618)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) 

jac(618,618)=jac(618,618)-k(33);
jac(431,618)=jac(431,618)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(618,618)=jac(618,618)-k(32);
jac(230,618)=jac(230,618)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) 

jac(618,618)=jac(618,618)-k(31);
jac(376,618)=jac(376,618)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(618,618)=jac(618,618)-k(22);
jac(197,618)=jac(197,618)+k(22);
jac(614,618)=jac(614,618)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(618);
jac(197,618)=jac(197,618)-k(21)*y(197);
jac(618,197)=jac(618,197)-k(21)*y(618);
jac(618,618)=jac(618,618)-k(21)*y(197);
jac(619,197)=jac(619,197)+k(21)*y(618);
jac(619,618)=jac(619,618)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(618);
jac(32,618)=jac(32,618)-k(21)*y(32);
jac(618,32)=jac(618,32)-k(21)*y(618);
jac(618,618)=jac(618,618)-k(21)*y(32);
jac(472,32)=jac(472,32)+k(21)*y(618);
jac(472,618)=jac(472,618)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(618);
jac(2,618)=jac(2,618)-k(21)*y(2);
jac(618,2)=jac(618,2)-k(21)*y(618);
jac(618,618)=jac(618,618)-k(21)*y(2);
jac(427,2)=jac(427,2)+k(21)*y(618);
jac(427,618)=jac(427,618)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(618);
jac(3,618)=jac(3,618)-k(15)*y(3);
jac(618,3)=jac(618,3)-k(15)*y(618);
jac(618,618)=jac(618,618)-k(15)*y(3);
jac(232,3)=jac(232,3)+k(15)*y(618);
jac(232,618)=jac(232,618)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(618);
jac(3,618)=jac(3,618)-k(9)*y(3);
jac(618,3)=jac(618,3)-k(9)*y(618);
jac(618,618)=jac(618,618)-k(9)*y(3);
jac(401,3)=jac(401,3)+k(9)*y(618);
jac(401,618)=jac(401,618)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

jac(618,618)=jac(618,618)-k(8);
jac(15,618)=jac(15,618)+k(8);
jac(319,618)=jac(319,618)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(618,618)=jac(618,618)-k(6);
jac(98,618)=jac(98,618)+k(6);
jac(257,618)=jac(257,618)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(614,614)=jac(614,614)-k(32);
jac(234,614)=jac(234,614)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(614,614)=jac(614,614)-k(32);
jac(93,614)=jac(93,614)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(614,614)=jac(614,614)-k(31);
jac(150,614)=jac(150,614)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(614);
jac(197,614)=jac(197,614)-k(21)*y(197);
jac(614,197)=jac(614,197)-k(21)*y(614);
jac(614,614)=jac(614,614)-k(21)*y(197);
jac(618,197)=jac(618,197)+k(21)*y(614);
jac(618,614)=jac(618,614)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(614);
jac(32,614)=jac(32,614)-k(21)*y(32);
jac(614,32)=jac(614,32)-k(21)*y(614);
jac(614,614)=jac(614,614)-k(21)*y(32);
jac(617,32)=jac(617,32)+k(21)*y(614);
jac(617,614)=jac(617,614)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(614);
jac(2,614)=jac(2,614)-k(21)*y(2);
jac(614,2)=jac(614,2)-k(21)*y(614);
jac(614,614)=jac(614,614)-k(21)*y(2);
jac(616,2)=jac(616,2)+k(21)*y(614);
jac(616,614)=jac(616,614)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(614);
jac(197,614)=jac(197,614)-k(21)*y(197);
jac(614,197)=jac(614,197)-k(21)*y(614);
jac(614,614)=jac(614,614)-k(21)*y(197);
jac(292,197)=jac(292,197)+k(21)*y(614);
jac(292,614)=jac(292,614)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(614);
jac(32,614)=jac(32,614)-k(21)*y(32);
jac(614,32)=jac(614,32)-k(21)*y(614);
jac(614,614)=jac(614,614)-k(21)*y(32);
jac(615,32)=jac(615,32)+k(21)*y(614);
jac(615,614)=jac(615,614)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(614);
jac(2,614)=jac(2,614)-k(21)*y(2);
jac(614,2)=jac(614,2)-k(21)*y(614);
jac(614,614)=jac(614,614)-k(21)*y(2);
jac(425,2)=jac(425,2)+k(21)*y(614);
jac(425,614)=jac(425,614)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(614);
jac(3,614)=jac(3,614)-k(15)*y(3);
jac(614,3)=jac(614,3)-k(15)*y(614);
jac(614,614)=jac(614,614)-k(15)*y(3);
jac(56,3)=jac(56,3)+k(15)*y(614);
jac(56,614)=jac(56,614)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(614);
jac(3,614)=jac(3,614)-k(9)*y(3);
jac(614,3)=jac(614,3)-k(9)*y(614);
jac(614,614)=jac(614,614)-k(9)*y(3);
jac(57,3)=jac(57,3)+k(9)*y(614);
jac(57,614)=jac(57,614)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) 

jac(614,614)=jac(614,614)-k(8);
jac(15,614)=jac(15,614)+k(8);
jac(315,614)=jac(315,614)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) 

jac(614,614)=jac(614,614)-k(6);
jac(77,614)=jac(77,614)+k(6);
jac(98,614)=jac(98,614)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(584,584)=jac(584,584)-k(32);
jac(237,584)=jac(237,584)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(584,584)=jac(584,584)-k(32);
jac(79,584)=jac(79,584)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(584,584)=jac(584,584)-k(31);
jac(614,584)=jac(614,584)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(584,584)=jac(584,584)-k(31);
jac(610,584)=jac(610,584)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(584);
jac(197,584)=jac(197,584)-k(21)*y(197);
jac(584,197)=jac(584,197)-k(21)*y(584);
jac(584,584)=jac(584,584)-k(21)*y(197);
jac(613,197)=jac(613,197)+k(21)*y(584);
jac(613,584)=jac(613,584)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(584);
jac(32,584)=jac(32,584)-k(21)*y(32);
jac(584,32)=jac(584,32)-k(21)*y(584);
jac(584,584)=jac(584,584)-k(21)*y(32);
jac(245,32)=jac(245,32)+k(21)*y(584);
jac(245,584)=jac(245,584)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(584);
jac(2,584)=jac(2,584)-k(21)*y(2);
jac(584,2)=jac(584,2)-k(21)*y(584);
jac(584,584)=jac(584,584)-k(21)*y(2);
jac(252,2)=jac(252,2)+k(21)*y(584);
jac(252,584)=jac(252,584)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(584);
jac(197,584)=jac(197,584)-k(21)*y(197);
jac(584,197)=jac(584,197)-k(21)*y(584);
jac(584,584)=jac(584,584)-k(21)*y(197);
jac(578,197)=jac(578,197)+k(21)*y(584);
jac(578,584)=jac(578,584)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(584);
jac(32,584)=jac(32,584)-k(21)*y(32);
jac(584,32)=jac(584,32)-k(21)*y(584);
jac(584,584)=jac(584,584)-k(21)*y(32);
jac(573,32)=jac(573,32)+k(21)*y(584);
jac(573,584)=jac(573,584)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(584);
jac(2,584)=jac(2,584)-k(21)*y(2);
jac(584,2)=jac(584,2)-k(21)*y(584);
jac(584,584)=jac(584,584)-k(21)*y(2);
jac(501,2)=jac(501,2)+k(21)*y(584);
jac(501,584)=jac(501,584)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(3,3)=jac(3,3)-k(15)*y(584);
jac(3,584)=jac(3,584)-k(15)*y(3);
jac(584,3)=jac(584,3)-k(15)*y(584);
jac(584,584)=jac(584,584)-k(15)*y(3);
jac(238,3)=jac(238,3)+k(15)*y(584);
jac(238,584)=jac(238,584)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(3,3)=jac(3,3)-k(15)*y(584);
jac(3,584)=jac(3,584)-k(15)*y(3);
jac(584,3)=jac(584,3)-k(15)*y(584);
jac(584,584)=jac(584,584)-k(15)*y(3);
jac(310,3)=jac(310,3)+k(15)*y(584);
jac(310,584)=jac(310,584)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) 

jac(584,584)=jac(584,584)-k(8);
jac(77,584)=jac(77,584)+k(8);
jac(315,584)=jac(315,584)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~pY, g~pY) 

jac(584,584)=jac(584,584)-k(6);
jac(77,584)=jac(77,584)+k(6);
jac(76,584)=jac(76,584)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(612,612)=jac(612,612)-k(33);
jac(356,612)=jac(356,612)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(612,612)=jac(612,612)-k(33);
jac(575,612)=jac(575,612)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(612,612)=jac(612,612)-k(31);
jac(160,612)=jac(160,612)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(612,612)=jac(612,612)-k(28);
jac(608,612)=jac(608,612)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(612,612)=jac(612,612)-k(27);
jac(559,612)=jac(559,612)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(612,612)=jac(612,612)-k(22);
jac(32,612)=jac(32,612)+k(22);
jac(533,612)=jac(533,612)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(612,612)=jac(612,612)-k(22);
jac(32,612)=jac(32,612)+k(22);
jac(351,612)=jac(351,612)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(612);
jac(3,612)=jac(3,612)-k(15)*y(3);
jac(612,3)=jac(612,3)-k(15)*y(612);
jac(612,612)=jac(612,612)-k(15)*y(3);
jac(535,3)=jac(535,3)+k(15)*y(612);
jac(535,612)=jac(535,612)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(612);
jac(3,612)=jac(3,612)-k(9)*y(3);
jac(612,3)=jac(612,3)-k(9)*y(612);
jac(612,612)=jac(612,612)-k(9)*y(3);
jac(161,3)=jac(161,3)+k(9)*y(612);
jac(161,612)=jac(161,612)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(612,612)=jac(612,612)-k(8);
jac(33,612)=jac(33,612)+k(8);
jac(28,612)=jac(28,612)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(612,612)=jac(612,612)-k(6);
jac(27,612)=jac(27,612)+k(6);
jac(82,612)=jac(82,612)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(608,608)=jac(608,608)-k(34);
jac(612,608)=jac(612,608)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(608,608)=jac(608,608)-k(33);
jac(545,608)=jac(545,608)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(608,608)=jac(608,608)-k(33);
jac(359,608)=jac(359,608)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(608,608)=jac(608,608)-k(31);
jac(456,608)=jac(456,608)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(608,608)=jac(608,608)-k(29);
jac(609,608)=jac(609,608)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(608,608)=jac(608,608)-k(22);
jac(32,608)=jac(32,608)+k(22);
jac(585,608)=jac(585,608)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(608,608)=jac(608,608)-k(22);
jac(197,608)=jac(197,608)+k(22);
jac(533,608)=jac(533,608)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(608);
jac(3,608)=jac(3,608)-k(15)*y(3);
jac(608,3)=jac(608,3)-k(15)*y(608);
jac(608,608)=jac(608,608)-k(15)*y(3);
jac(536,3)=jac(536,3)+k(15)*y(608);
jac(536,608)=jac(536,608)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(608);
jac(3,608)=jac(3,608)-k(9)*y(3);
jac(608,3)=jac(608,3)-k(9)*y(608);
jac(608,608)=jac(608,608)-k(9)*y(3);
jac(362,3)=jac(362,3)+k(9)*y(608);
jac(362,608)=jac(362,608)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(608,608)=jac(608,608)-k(8);
jac(257,608)=jac(257,608)+k(8);
jac(28,608)=jac(28,608)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(608,608)=jac(608,608)-k(6);
jac(27,608)=jac(27,608)+k(6);
jac(256,608)=jac(256,608)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(611,611)=jac(611,611)-k(34);
jac(527,611)=jac(527,611)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(611,611)=jac(611,611)-k(34);
jac(359,611)=jac(359,611)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(611,611)=jac(611,611)-k(33);
jac(513,611)=jac(513,611)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(611,611)=jac(611,611)-k(31);
jac(369,611)=jac(369,611)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(611,611)=jac(611,611)-k(22);
jac(182,611)=jac(182,611)+k(22);
jac(338,611)=jac(338,611)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(611,611)=jac(611,611)-k(22);
jac(197,611)=jac(197,611)+k(22);
jac(512,611)=jac(512,611)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(611);
jac(3,611)=jac(3,611)-k(15)*y(3);
jac(611,3)=jac(611,3)-k(15)*y(611);
jac(611,611)=jac(611,611)-k(15)*y(3);
jac(520,3)=jac(520,3)+k(15)*y(611);
jac(520,611)=jac(520,611)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(611);
jac(3,611)=jac(3,611)-k(9)*y(3);
jac(611,3)=jac(611,3)-k(9)*y(611);
jac(611,611)=jac(611,611)-k(9)*y(3);
jac(363,3)=jac(363,3)+k(9)*y(611);
jac(363,611)=jac(363,611)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(611,611)=jac(611,611)-k(8);
jac(218,611)=jac(218,611)+k(8);
jac(209,611)=jac(209,611)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(611,611)=jac(611,611)-k(6);
jac(192,611)=jac(192,611)+k(6);
jac(264,611)=jac(264,611)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(609,609)=jac(609,609)-k(34);
jac(608,609)=jac(608,609)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(609,609)=jac(609,609)-k(34);
jac(559,609)=jac(559,609)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(609,609)=jac(609,609)-k(33);
jac(550,609)=jac(550,609)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(609,609)=jac(609,609)-k(33);
jac(611,609)=jac(611,609)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(609,609)=jac(609,609)-k(31);
jac(455,609)=jac(455,609)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(609,609)=jac(609,609)-k(22);
jac(197,609)=jac(197,609)+k(22);
jac(585,609)=jac(585,609)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(609,609)=jac(609,609)-k(22);
jac(197,609)=jac(197,609)+k(22);
jac(338,609)=jac(338,609)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(609);
jac(3,609)=jac(3,609)-k(15)*y(3);
jac(609,3)=jac(609,3)-k(15)*y(609);
jac(609,609)=jac(609,609)-k(15)*y(3);
jac(552,3)=jac(552,3)+k(15)*y(609);
jac(552,609)=jac(552,609)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(609);
jac(3,609)=jac(3,609)-k(9)*y(3);
jac(609,3)=jac(609,3)-k(9)*y(609);
jac(609,609)=jac(609,609)-k(9)*y(3);
jac(342,3)=jac(342,3)+k(9)*y(609);
jac(342,609)=jac(342,609)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(609,609)=jac(609,609)-k(8);
jac(257,609)=jac(257,609)+k(8);
jac(209,609)=jac(209,609)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(609,609)=jac(609,609)-k(6);
jac(192,609)=jac(192,609)+k(6);
jac(256,609)=jac(256,609)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(610,610)=jac(610,610)-k(32);
jac(331,610)=jac(331,610)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) 

jac(610,610)=jac(610,610)-k(32);
jac(590,610)=jac(590,610)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(610,610)=jac(610,610)-k(31);
jac(150,610)=jac(150,610)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(610);
jac(197,610)=jac(197,610)-k(21)*y(197);
jac(610,197)=jac(610,197)-k(21)*y(610);
jac(610,610)=jac(610,610)-k(21)*y(197);
jac(338,197)=jac(338,197)+k(21)*y(610);
jac(338,610)=jac(338,610)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(610);
jac(32,610)=jac(32,610)-k(21)*y(32);
jac(610,32)=jac(610,32)-k(21)*y(610);
jac(610,610)=jac(610,610)-k(21)*y(32);
jac(533,32)=jac(533,32)+k(21)*y(610);
jac(533,610)=jac(533,610)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(610);
jac(2,610)=jac(2,610)-k(21)*y(2);
jac(610,2)=jac(610,2)-k(21)*y(610);
jac(610,610)=jac(610,610)-k(21)*y(2);
jac(508,2)=jac(508,2)+k(21)*y(610);
jac(508,610)=jac(508,610)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(610);
jac(197,610)=jac(197,610)-k(21)*y(197);
jac(610,197)=jac(610,197)-k(21)*y(610);
jac(610,610)=jac(610,610)-k(21)*y(197);
jac(585,197)=jac(585,197)+k(21)*y(610);
jac(585,610)=jac(585,610)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(610);
jac(32,610)=jac(32,610)-k(21)*y(32);
jac(610,32)=jac(610,32)-k(21)*y(610);
jac(610,610)=jac(610,610)-k(21)*y(32);
jac(351,32)=jac(351,32)+k(21)*y(610);
jac(351,610)=jac(351,610)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(610);
jac(2,610)=jac(2,610)-k(21)*y(2);
jac(610,2)=jac(610,2)-k(21)*y(610);
jac(610,610)=jac(610,610)-k(21)*y(2);
jac(354,2)=jac(354,2)+k(21)*y(610);
jac(354,610)=jac(354,610)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(3,3)=jac(3,3)-k(15)*y(610);
jac(3,610)=jac(3,610)-k(15)*y(3);
jac(610,3)=jac(610,3)-k(15)*y(610);
jac(610,610)=jac(610,610)-k(15)*y(3);
jac(537,3)=jac(537,3)+k(15)*y(610);
jac(537,610)=jac(537,610)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(610);
jac(3,610)=jac(3,610)-k(9)*y(3);
jac(610,3)=jac(610,3)-k(9)*y(610);
jac(610,610)=jac(610,610)-k(9)*y(3);
jac(151,3)=jac(151,3)+k(9)*y(610);
jac(151,610)=jac(151,610)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) 

jac(610,610)=jac(610,610)-k(8);
jac(77,610)=jac(77,610)+k(8);
jac(16,610)=jac(16,610)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) 

jac(610,610)=jac(610,610)-k(6);
jac(15,610)=jac(15,610)+k(6);
jac(76,610)=jac(76,610)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(585,585)=jac(585,585)-k(34);
jac(351,585)=jac(351,585)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(585,585)=jac(585,585)-k(33);
jac(512,585)=jac(512,585)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(585,585)=jac(585,585)-k(32);
jac(587,585)=jac(587,585)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(585,585)=jac(585,585)-k(31);
jac(409,585)=jac(409,585)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(585,585)=jac(585,585)-k(22);
jac(197,585)=jac(197,585)+k(22);
jac(610,585)=jac(610,585)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(585);
jac(197,585)=jac(197,585)-k(21)*y(197);
jac(585,197)=jac(585,197)-k(21)*y(585);
jac(585,585)=jac(585,585)-k(21)*y(197);
jac(609,197)=jac(609,197)+k(21)*y(585);
jac(609,585)=jac(609,585)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(585);
jac(32,585)=jac(32,585)-k(21)*y(32);
jac(585,32)=jac(585,32)-k(21)*y(585);
jac(585,585)=jac(585,585)-k(21)*y(32);
jac(608,32)=jac(608,32)+k(21)*y(585);
jac(608,585)=jac(608,585)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(585);
jac(2,585)=jac(2,585)-k(21)*y(2);
jac(585,2)=jac(585,2)-k(21)*y(585);
jac(585,585)=jac(585,585)-k(21)*y(2);
jac(545,2)=jac(545,2)+k(21)*y(585);
jac(545,585)=jac(545,585)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(585);
jac(3,585)=jac(3,585)-k(15)*y(3);
jac(585,3)=jac(585,3)-k(15)*y(585);
jac(585,585)=jac(585,585)-k(15)*y(3);
jac(547,3)=jac(547,3)+k(15)*y(585);
jac(547,585)=jac(547,585)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(585);
jac(3,585)=jac(3,585)-k(9)*y(3);
jac(585,3)=jac(585,3)-k(9)*y(585);
jac(585,585)=jac(585,585)-k(9)*y(3);
jac(599,3)=jac(599,3)+k(9)*y(585);
jac(599,585)=jac(599,585)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(585,585)=jac(585,585)-k(8);
jac(16,585)=jac(16,585)+k(8);
jac(257,585)=jac(257,585)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

jac(585,585)=jac(585,585)-k(6);
jac(15,585)=jac(15,585)+k(6);
jac(256,585)=jac(256,585)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(588,588)=jac(588,588)-k(34);
jac(105,588)=jac(105,588)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(588,588)=jac(588,588)-k(33);
jac(594,588)=jac(594,588)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(588,588)=jac(588,588)-k(31);
jac(607,588)=jac(607,588)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(588,588)=jac(588,588)-k(22);
jac(197,588)=jac(197,588)+k(22);
jac(108,588)=jac(108,588)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(588);
jac(3,588)=jac(3,588)-k(15)*y(3);
jac(588,3)=jac(588,3)-k(15)*y(588);
jac(588,588)=jac(588,588)-k(15)*y(3);
jac(201,3)=jac(201,3)+k(15)*y(588);
jac(201,588)=jac(201,588)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(588,588)=jac(588,588)-k(10);
jac(3,588)=jac(3,588)+k(10);
jac(587,588)=jac(587,588)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(588,588)=jac(588,588)-k(8);
jac(8,588)=jac(8,588)+k(8);
jac(257,588)=jac(257,588)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(588,588)=jac(588,588)-k(6);
jac(7,588)=jac(7,588)+k(6);
jac(256,588)=jac(256,588)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(607,607)=jac(607,607)-k(34);
jac(103,607)=jac(103,607)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(607,607)=jac(607,607)-k(33);
jac(606,607)=jac(606,607)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(607,607)=jac(607,607)-k(22);
jac(197,607)=jac(197,607)+k(22);
jac(100,607)=jac(100,607)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(607,607)=jac(607,607)-k(12);
jac(588,607)=jac(588,607)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(607,607)=jac(607,607)-k(10);
jac(3,607)=jac(3,607)+k(10);
jac(202,607)=jac(202,607)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(607);
jac(3,607)=jac(3,607)-k(9)*y(3);
jac(607,3)=jac(607,3)-k(9)*y(607);
jac(607,607)=jac(607,607)-k(9)*y(3);
jac(414,3)=jac(414,3)+k(9)*y(607);
jac(414,607)=jac(414,607)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(607,607)=jac(607,607)-k(8);
jac(8,607)=jac(8,607)+k(8);
jac(192,607)=jac(192,607)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(607,607)=jac(607,607)-k(6);
jac(7,607)=jac(7,607)+k(6);
jac(191,607)=jac(191,607)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(606,606)=jac(606,606)-k(34);
jac(96,606)=jac(96,606)+k(34);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(606,606)=jac(606,606)-k(24);
jac(607,606)=jac(607,606)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(606,606)=jac(606,606)-k(22);
jac(182,606)=jac(182,606)+k(22);
jac(100,606)=jac(100,606)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(606,606)=jac(606,606)-k(12);
jac(594,606)=jac(594,606)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(606,606)=jac(606,606)-k(10);
jac(3,606)=jac(3,606)+k(10);
jac(418,606)=jac(418,606)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(606);
jac(3,606)=jac(3,606)-k(9)*y(3);
jac(606,3)=jac(606,3)-k(9)*y(606);
jac(606,606)=jac(606,606)-k(9)*y(3);
jac(417,3)=jac(417,3)+k(9)*y(606);
jac(417,606)=jac(417,606)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(606,606)=jac(606,606)-k(8);
jac(8,606)=jac(8,606)+k(8);
jac(177,606)=jac(177,606)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(606,606)=jac(606,606)-k(6);
jac(7,606)=jac(7,606)+k(6);
jac(176,606)=jac(176,606)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(594,594)=jac(594,594)-k(34);
jac(109,594)=jac(109,594)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(594,594)=jac(594,594)-k(31);
jac(606,594)=jac(606,594)+k(31);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(594,594)=jac(594,594)-k(24);
jac(588,594)=jac(588,594)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(594,594)=jac(594,594)-k(22);
jac(182,594)=jac(182,594)+k(22);
jac(108,594)=jac(108,594)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(594);
jac(3,594)=jac(3,594)-k(15)*y(3);
jac(594,3)=jac(594,3)-k(15)*y(594);
jac(594,594)=jac(594,594)-k(15)*y(3);
jac(593,3)=jac(593,3)+k(15)*y(594);
jac(593,594)=jac(593,594)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(594,594)=jac(594,594)-k(10);
jac(3,594)=jac(3,594)+k(10);
jac(304,594)=jac(304,594)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(594,594)=jac(594,594)-k(8);
jac(8,594)=jac(8,594)+k(8);
jac(218,594)=jac(218,594)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(594,594)=jac(594,594)-k(6);
jac(7,594)=jac(7,594)+k(6);
jac(264,594)=jac(264,594)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(596,596)=jac(596,596)-k(34);
jac(604,596)=jac(604,596)+k(34);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(596,596)=jac(596,596)-k(29);
jac(516,596)=jac(516,596)+k(29);

% rule    : R11
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(596,596)=jac(596,596)-k(25);
jac(360,596)=jac(360,596)+k(25);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(596,596)=jac(596,596)-k(24);
jac(544,596)=jac(544,596)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(596,596)=jac(596,596)-k(22);
jac(2,596)=jac(2,596)+k(22);
jac(595,596)=jac(595,596)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(596,596)=jac(596,596)-k(22);
jac(182,596)=jac(182,596)+k(22);
jac(539,596)=jac(539,596)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(596,596)=jac(596,596)-k(16);
jac(3,596)=jac(3,596)+k(16);
jac(180,596)=jac(180,596)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(596,596)=jac(596,596)-k(10);
jac(3,596)=jac(3,596)+k(10);
jac(511,596)=jac(511,596)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(596,596)=jac(596,596)-k(8);
jac(222,596)=jac(222,596)+k(8);
jac(141,596)=jac(141,596)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(596,596)=jac(596,596)-k(6);
jac(140,596)=jac(140,596)+k(6);
jac(515,596)=jac(515,596)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(597,597)=jac(597,597)-k(34);
jac(360,597)=jac(360,597)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(597,597)=jac(597,597)-k(34);
jac(526,597)=jac(526,597)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(597,597)=jac(597,597)-k(33);
jac(516,597)=jac(516,597)+k(33);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(597,597)=jac(597,597)-k(24);
jac(601,597)=jac(601,597)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(597,597)=jac(597,597)-k(22);
jac(197,597)=jac(197,597)+k(22);
jac(595,597)=jac(595,597)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~Y, a~pY) 

jac(597,597)=jac(597,597)-k(22);
jac(182,597)=jac(182,597)+k(22);
jac(339,597)=jac(339,597)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(597,597)=jac(597,597)-k(16);
jac(3,597)=jac(3,597)+k(16);
jac(363,597)=jac(363,597)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(597,597)=jac(597,597)-k(10);
jac(3,597)=jac(3,597)+k(10);
jac(520,597)=jac(520,597)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(597,597)=jac(597,597)-k(8);
jac(222,597)=jac(222,597)+k(8);
jac(336,597)=jac(336,597)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(597,597)=jac(597,597)-k(6);
jac(335,597)=jac(335,597)+k(6);
jac(515,597)=jac(515,597)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(605,605)=jac(605,605)-k(34);
jac(167,605)=jac(167,605)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(605,605)=jac(605,605)-k(33);
jac(180,605)=jac(180,605)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(605,605)=jac(605,605)-k(31);
jac(169,605)=jac(169,605)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(605,605)=jac(605,605)-k(29);
jac(195,605)=jac(195,605)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(605,605)=jac(605,605)-k(22);
jac(2,605)=jac(2,605)+k(22);
jac(599,605)=jac(599,605)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(605,605)=jac(605,605)-k(22);
jac(197,605)=jac(197,605)+k(22);
jac(143,605)=jac(143,605)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(605);
jac(3,605)=jac(3,605)-k(15)*y(3);
jac(605,3)=jac(605,3)-k(15)*y(605);
jac(605,605)=jac(605,605)-k(15)*y(3);
jac(544,3)=jac(544,3)+k(15)*y(605);
jac(544,605)=jac(544,605)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(605,605)=jac(605,605)-k(10);
jac(3,605)=jac(3,605)+k(10);
jac(545,605)=jac(545,605)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(605,605)=jac(605,605)-k(8);
jac(257,605)=jac(257,605)+k(8);
jac(141,605)=jac(141,605)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(605,605)=jac(605,605)-k(6);
jac(256,605)=jac(256,605)+k(6);
jac(140,605)=jac(140,605)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(599,599)=jac(599,599)-k(34);
jac(350,599)=jac(350,599)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(599,599)=jac(599,599)-k(33);
jac(361,599)=jac(361,599)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(599,599)=jac(599,599)-k(32);
jac(588,599)=jac(588,599)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(599,599)=jac(599,599)-k(31);
jac(196,599)=jac(196,599)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(599,599)=jac(599,599)-k(22);
jac(197,599)=jac(197,599)+k(22);
jac(151,599)=jac(151,599)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(599);
jac(197,599)=jac(197,599)-k(21)*y(197);
jac(599,197)=jac(599,197)-k(21)*y(599);
jac(599,599)=jac(599,599)-k(21)*y(197);
jac(342,197)=jac(342,197)+k(21)*y(599);
jac(342,599)=jac(342,599)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(32,32)=jac(32,32)-k(21)*y(599);
jac(32,599)=jac(32,599)-k(21)*y(32);
jac(599,32)=jac(599,32)-k(21)*y(599);
jac(599,599)=jac(599,599)-k(21)*y(32);
jac(362,32)=jac(362,32)+k(21)*y(599);
jac(362,599)=jac(362,599)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(599);
jac(2,599)=jac(2,599)-k(21)*y(2);
jac(599,2)=jac(599,2)-k(21)*y(599);
jac(599,599)=jac(599,599)-k(21)*y(2);
jac(605,2)=jac(605,2)+k(21)*y(599);
jac(605,599)=jac(605,599)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(599);
jac(3,599)=jac(3,599)-k(15)*y(3);
jac(599,3)=jac(599,3)-k(15)*y(599);
jac(599,599)=jac(599,599)-k(15)*y(3);
jac(598,3)=jac(598,3)+k(15)*y(599);
jac(598,599)=jac(598,599)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(599,599)=jac(599,599)-k(10);
jac(3,599)=jac(3,599)+k(10);
jac(585,599)=jac(585,599)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(599,599)=jac(599,599)-k(8);
jac(126,599)=jac(126,599)+k(8);
jac(257,599)=jac(257,599)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(599,599)=jac(599,599)-k(6);
jac(48,599)=jac(48,599)+k(6);
jac(256,599)=jac(256,599)+k(6);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(604,604)=jac(604,604)-k(28);
jac(596,604)=jac(596,604)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(604,604)=jac(604,604)-k(27);
jac(522,604)=jac(522,604)+k(27);

% rule    : R11
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(604,604)=jac(604,604)-k(25);
jac(357,604)=jac(357,604)+k(25);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(604,604)=jac(604,604)-k(24);
jac(603,604)=jac(603,604)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(604,604)=jac(604,604)-k(22);
jac(2,604)=jac(2,604)+k(22);
jac(539,604)=jac(539,604)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(604,604)=jac(604,604)-k(22);
jac(2,604)=jac(2,604)+k(22);
jac(116,604)=jac(116,604)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(604,604)=jac(604,604)-k(16);
jac(3,604)=jac(3,604)+k(16);
jac(173,604)=jac(173,604)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(604,604)=jac(604,604)-k(10);
jac(3,604)=jac(3,604)+k(10);
jac(507,604)=jac(507,604)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(604,604)=jac(604,604)-k(8);
jac(43,604)=jac(43,604)+k(8);
jac(141,604)=jac(141,604)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(604,604)=jac(604,604)-k(6);
jac(140,604)=jac(140,604)+k(6);
jac(114,604)=jac(114,604)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(603,603)=jac(603,603)-k(33);
jac(604,603)=jac(604,603)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(603,603)=jac(603,603)-k(28);
jac(544,603)=jac(544,603)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(603,603)=jac(603,603)-k(27);
jac(349,603)=jac(349,603)+k(27);

% rule    : R11
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(603,603)=jac(603,603)-k(25);
jac(602,603)=jac(602,603)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(603,603)=jac(603,603)-k(22);
jac(32,603)=jac(32,603)+k(22);
jac(539,603)=jac(539,603)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(603,603)=jac(603,603)-k(22);
jac(2,603)=jac(2,603)+k(22);
jac(352,603)=jac(352,603)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(603,603)=jac(603,603)-k(16);
jac(3,603)=jac(3,603)+k(16);
jac(167,603)=jac(167,603)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(603,603)=jac(603,603)-k(10);
jac(3,603)=jac(3,603)+k(10);
jac(541,603)=jac(541,603)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(603,603)=jac(603,603)-k(8);
jac(37,603)=jac(37,603)+k(8);
jac(141,603)=jac(141,603)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(603,603)=jac(603,603)-k(6);
jac(140,603)=jac(140,603)+k(6);
jac(554,603)=jac(554,603)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(602,602)=jac(602,602)-k(33);
jac(357,602)=jac(357,602)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(602,602)=jac(602,602)-k(33);
jac(603,602)=jac(603,602)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(602,602)=jac(602,602)-k(28);
jac(600,602)=jac(600,602)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(602,602)=jac(602,602)-k(27);
jac(558,602)=jac(558,602)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

jac(602,602)=jac(602,602)-k(22);
jac(32,602)=jac(32,602)+k(22);
jac(134,602)=jac(134,602)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(602,602)=jac(602,602)-k(22);
jac(32,602)=jac(32,602)+k(22);
jac(352,602)=jac(352,602)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(602,602)=jac(602,602)-k(16);
jac(3,602)=jac(3,602)+k(16);
jac(161,602)=jac(161,602)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(602,602)=jac(602,602)-k(10);
jac(3,602)=jac(3,602)+k(10);
jac(535,602)=jac(535,602)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(602,602)=jac(602,602)-k(8);
jac(37,602)=jac(37,602)+k(8);
jac(132,602)=jac(132,602)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(602,602)=jac(602,602)-k(6);
jac(131,602)=jac(131,602)+k(6);
jac(554,602)=jac(554,602)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(600,600)=jac(600,600)-k(34);
jac(602,600)=jac(602,600)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(600,600)=jac(600,600)-k(33);
jac(544,600)=jac(544,600)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(600,600)=jac(600,600)-k(33);
jac(360,600)=jac(360,600)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(600,600)=jac(600,600)-k(29);
jac(601,600)=jac(601,600)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(600,600)=jac(600,600)-k(22);
jac(32,600)=jac(32,600)+k(22);
jac(598,600)=jac(598,600)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

jac(600,600)=jac(600,600)-k(22);
jac(197,600)=jac(197,600)+k(22);
jac(134,600)=jac(134,600)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(600,600)=jac(600,600)-k(16);
jac(3,600)=jac(3,600)+k(16);
jac(362,600)=jac(362,600)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(600,600)=jac(600,600)-k(10);
jac(3,600)=jac(3,600)+k(10);
jac(536,600)=jac(536,600)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(600,600)=jac(600,600)-k(8);
jac(227,600)=jac(227,600)+k(8);
jac(132,600)=jac(132,600)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(600,600)=jac(600,600)-k(6);
jac(131,600)=jac(131,600)+k(6);
jac(543,600)=jac(543,600)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(601,601)=jac(601,601)-k(34);
jac(600,601)=jac(600,601)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(601,601)=jac(601,601)-k(34);
jac(558,601)=jac(558,601)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(601,601)=jac(601,601)-k(33);
jac(549,601)=jac(549,601)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(601,601)=jac(601,601)-k(33);
jac(597,601)=jac(597,601)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(601,601)=jac(601,601)-k(22);
jac(197,601)=jac(197,601)+k(22);
jac(598,601)=jac(598,601)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~pY) 

jac(601,601)=jac(601,601)-k(22);
jac(197,601)=jac(197,601)+k(22);
jac(339,601)=jac(339,601)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(601,601)=jac(601,601)-k(16);
jac(3,601)=jac(3,601)+k(16);
jac(342,601)=jac(342,601)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(601,601)=jac(601,601)-k(10);
jac(3,601)=jac(3,601)+k(10);
jac(552,601)=jac(552,601)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(601,601)=jac(601,601)-k(8);
jac(227,601)=jac(227,601)+k(8);
jac(336,601)=jac(336,601)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(601,601)=jac(601,601)-k(6);
jac(335,601)=jac(335,601)+k(6);
jac(543,601)=jac(543,601)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(598,598)=jac(598,598)-k(34);
jac(352,598)=jac(352,598)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(598,598)=jac(598,598)-k(33);
jac(595,598)=jac(595,598)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(598,598)=jac(598,598)-k(32);
jac(201,598)=jac(201,598)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(598,598)=jac(598,598)-k(22);
jac(197,598)=jac(197,598)+k(22);
jac(120,598)=jac(120,598)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(598);
jac(197,598)=jac(197,598)-k(21)*y(197);
jac(598,197)=jac(598,197)-k(21)*y(598);
jac(598,598)=jac(598,598)-k(21)*y(197);
jac(601,197)=jac(601,197)+k(21)*y(598);
jac(601,598)=jac(601,598)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(598);
jac(32,598)=jac(32,598)-k(21)*y(32);
jac(598,32)=jac(598,32)-k(21)*y(598);
jac(598,598)=jac(598,598)-k(21)*y(32);
jac(600,32)=jac(600,32)+k(21)*y(598);
jac(600,598)=jac(600,598)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(598);
jac(2,598)=jac(2,598)-k(21)*y(2);
jac(598,2)=jac(598,2)-k(21)*y(598);
jac(598,598)=jac(598,598)-k(21)*y(2);
jac(544,2)=jac(544,2)+k(21)*y(598);
jac(544,598)=jac(544,598)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(598,598)=jac(598,598)-k(16);
jac(3,598)=jac(3,598)+k(16);
jac(599,598)=jac(599,598)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(598,598)=jac(598,598)-k(10);
jac(3,598)=jac(3,598)+k(10);
jac(547,598)=jac(547,598)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(598,598)=jac(598,598)-k(8);
jac(126,598)=jac(126,598)+k(8);
jac(227,598)=jac(227,598)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(598,598)=jac(598,598)-k(6);
jac(48,598)=jac(48,598)+k(6);
jac(543,598)=jac(543,598)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(595,595)=jac(595,595)-k(34);
jac(116,595)=jac(116,595)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(595,595)=jac(595,595)-k(32);
jac(593,595)=jac(593,595)+k(32);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(595,595)=jac(595,595)-k(24);
jac(598,595)=jac(598,595)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(595,595)=jac(595,595)-k(22);
jac(182,595)=jac(182,595)+k(22);
jac(120,595)=jac(120,595)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(595);
jac(197,595)=jac(197,595)-k(21)*y(197);
jac(595,197)=jac(595,197)-k(21)*y(595);
jac(595,595)=jac(595,595)-k(21)*y(197);
jac(597,197)=jac(597,197)+k(21)*y(595);
jac(597,595)=jac(597,595)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(32,32)=jac(32,32)-k(21)*y(595);
jac(32,595)=jac(32,595)-k(21)*y(32);
jac(595,32)=jac(595,32)-k(21)*y(595);
jac(595,595)=jac(595,595)-k(21)*y(32);
jac(360,32)=jac(360,32)+k(21)*y(595);
jac(360,595)=jac(360,595)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(595);
jac(2,595)=jac(2,595)-k(21)*y(2);
jac(595,2)=jac(595,2)-k(21)*y(595);
jac(595,595)=jac(595,595)-k(21)*y(2);
jac(596,2)=jac(596,2)+k(21)*y(595);
jac(596,595)=jac(596,595)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(595,595)=jac(595,595)-k(16);
jac(3,595)=jac(3,595)+k(16);
jac(361,595)=jac(361,595)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(595,595)=jac(595,595)-k(10);
jac(3,595)=jac(3,595)+k(10);
jac(518,595)=jac(518,595)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(595,595)=jac(595,595)-k(8);
jac(126,595)=jac(126,595)+k(8);
jac(222,595)=jac(222,595)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(595,595)=jac(595,595)-k(6);
jac(48,595)=jac(48,595)+k(6);
jac(515,595)=jac(515,595)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(593,593)=jac(593,593)-k(34);
jac(113,593)=jac(113,593)+k(34);

% rule    : R10_1
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(593,593)=jac(593,593)-k(24);
jac(201,593)=jac(201,593)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~pY) 

jac(593,593)=jac(593,593)-k(22);
jac(182,593)=jac(182,593)+k(22);
jac(117,593)=jac(117,593)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(593,593)=jac(593,593)-k(19);
jac(595,593)=jac(595,593)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(593,593)=jac(593,593)-k(16);
jac(3,593)=jac(3,593)+k(16);
jac(594,593)=jac(594,593)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(593,593)=jac(593,593)-k(10);
jac(3,593)=jac(3,593)+k(10);
jac(591,593)=jac(591,593)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(593,593)=jac(593,593)-k(8);
jac(8,593)=jac(8,593)+k(8);
jac(222,593)=jac(222,593)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(593,593)=jac(593,593)-k(6);
jac(7,593)=jac(7,593)+k(6);
jac(515,593)=jac(515,593)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(591,591)=jac(591,591)-k(34);
jac(115,591)=jac(115,591)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

jac(591,591)=jac(591,591)-k(22);
jac(182,591)=jac(182,591)+k(22);
jac(119,591)=jac(119,591)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(591,591)=jac(591,591)-k(19);
jac(518,591)=jac(518,591)+k(19);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(591,591)=jac(591,591)-k(17);
jac(303,591)=jac(303,591)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(591,591)=jac(591,591)-k(16);
jac(3,591)=jac(3,591)+k(16);
jac(304,591)=jac(304,591)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(591);
jac(3,591)=jac(3,591)-k(9)*y(3);
jac(591,3)=jac(591,3)-k(9)*y(591);
jac(591,591)=jac(591,591)-k(9)*y(3);
jac(593,3)=jac(593,3)+k(9)*y(591);
jac(593,591)=jac(593,591)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(591,591)=jac(591,591)-k(8);
jac(5,591)=jac(5,591)+k(8);
jac(222,591)=jac(222,591)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

jac(591,591)=jac(591,591)-k(6);
jac(1,591)=jac(1,591)+k(6);
jac(515,591)=jac(515,591)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(592,592)=jac(592,592)-k(33);
jac(115,592)=jac(115,592)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

jac(592,592)=jac(592,592)-k(22);
jac(32,592)=jac(32,592)+k(22);
jac(119,592)=jac(119,592)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(592,592)=jac(592,592)-k(19);
jac(556,592)=jac(556,592)+k(19);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(592,592)=jac(592,592)-k(17);
jac(311,592)=jac(311,592)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(592,592)=jac(592,592)-k(16);
jac(3,592)=jac(3,592)+k(16);
jac(106,592)=jac(106,592)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(592);
jac(3,592)=jac(3,592)-k(9)*y(3);
jac(592,3)=jac(592,3)-k(9)*y(592);
jac(592,592)=jac(592,592)-k(9)*y(3);
jac(107,3)=jac(107,3)+k(9)*y(592);
jac(107,592)=jac(107,592)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(592,592)=jac(592,592)-k(8);
jac(5,592)=jac(5,592)+k(8);
jac(37,592)=jac(37,592)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

jac(592,592)=jac(592,592)-k(6);
jac(1,592)=jac(1,592)+k(6);
jac(554,592)=jac(554,592)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(589,589)=jac(589,589)-k(34);
jac(592,589)=jac(592,589)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(589,589)=jac(589,589)-k(33);
jac(591,589)=jac(591,589)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

jac(589,589)=jac(589,589)-k(22);
jac(197,589)=jac(197,589)+k(22);
jac(119,589)=jac(119,589)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(589,589)=jac(589,589)-k(19);
jac(547,589)=jac(547,589)+k(19);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(589,589)=jac(589,589)-k(17);
jac(312,589)=jac(312,589)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(589,589)=jac(589,589)-k(16);
jac(3,589)=jac(3,589)+k(16);
jac(587,589)=jac(587,589)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(9)*y(589);
jac(3,589)=jac(3,589)-k(9)*y(3);
jac(589,3)=jac(589,3)-k(9)*y(589);
jac(589,589)=jac(589,589)-k(9)*y(3);
jac(201,3)=jac(201,3)+k(9)*y(589);
jac(201,589)=jac(201,589)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(589,589)=jac(589,589)-k(8);
jac(5,589)=jac(5,589)+k(8);
jac(227,589)=jac(227,589)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

jac(589,589)=jac(589,589)-k(6);
jac(1,589)=jac(1,589)+k(6);
jac(543,589)=jac(543,589)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(590,590)=jac(590,590)-k(32);
jac(333,590)=jac(333,590)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) 

jac(590,590)=jac(590,590)-k(31);
jac(97,590)=jac(97,590)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(590);
jac(197,590)=jac(197,590)-k(21)*y(197);
jac(590,197)=jac(590,197)-k(21)*y(590);
jac(590,590)=jac(590,590)-k(21)*y(197);
jac(587,197)=jac(587,197)+k(21)*y(590);
jac(587,590)=jac(587,590)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(590);
jac(32,590)=jac(32,590)-k(21)*y(32);
jac(590,32)=jac(590,32)-k(21)*y(590);
jac(590,590)=jac(590,590)-k(21)*y(32);
jac(106,32)=jac(106,32)+k(21)*y(590);
jac(106,590)=jac(106,590)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(590);
jac(2,590)=jac(2,590)-k(21)*y(2);
jac(590,2)=jac(590,2)-k(21)*y(590);
jac(590,590)=jac(590,590)-k(21)*y(2);
jac(112,2)=jac(112,2)+k(21)*y(590);
jac(112,590)=jac(112,590)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(3,3)=jac(3,3)-k(15)*y(590);
jac(3,590)=jac(3,590)-k(15)*y(3);
jac(590,3)=jac(590,3)-k(15)*y(590);
jac(590,590)=jac(590,590)-k(15)*y(3);
jac(119,3)=jac(119,3)+k(15)*y(590);
jac(119,590)=jac(119,590)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(590);
jac(3,590)=jac(3,590)-k(9)*y(3);
jac(590,3)=jac(590,3)-k(9)*y(590);
jac(590,590)=jac(590,590)-k(9)*y(3);
jac(108,3)=jac(108,3)+k(9)*y(590);
jac(108,590)=jac(108,590)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~pY) 

jac(590,590)=jac(590,590)-k(8);
jac(77,590)=jac(77,590)+k(8);
jac(5,590)=jac(5,590)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) 

jac(590,590)=jac(590,590)-k(6);
jac(1,590)=jac(1,590)+k(6);
jac(76,590)=jac(76,590)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(587,587)=jac(587,587)-k(34);
jac(106,587)=jac(106,587)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(587,587)=jac(587,587)-k(33);
jac(304,587)=jac(304,587)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(587,587)=jac(587,587)-k(31);
jac(202,587)=jac(202,587)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(587,587)=jac(587,587)-k(22);
jac(197,587)=jac(197,587)+k(22);
jac(590,587)=jac(590,587)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(587);
jac(3,587)=jac(3,587)-k(15)*y(3);
jac(587,3)=jac(587,3)-k(15)*y(587);
jac(587,587)=jac(587,587)-k(15)*y(3);
jac(589,3)=jac(589,3)+k(15)*y(587);
jac(589,587)=jac(589,587)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(587);
jac(3,587)=jac(3,587)-k(9)*y(3);
jac(587,3)=jac(587,3)-k(9)*y(587);
jac(587,587)=jac(587,587)-k(9)*y(3);
jac(588,3)=jac(588,3)+k(9)*y(587);
jac(588,587)=jac(588,587)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(587,587)=jac(587,587)-k(8);
jac(5,587)=jac(5,587)+k(8);
jac(257,587)=jac(257,587)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

jac(587,587)=jac(587,587)-k(6);
jac(1,587)=jac(1,587)+k(6);
jac(256,587)=jac(256,587)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(586,586)=jac(586,586)-k(34);
jac(84,586)=jac(84,586)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(586,586)=jac(586,586)-k(33);
jac(302,586)=jac(302,586)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(586,586)=jac(586,586)-k(31);
jac(296,586)=jac(296,586)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(586,586)=jac(586,586)-k(31);
jac(587,586)=jac(587,586)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(586,586)=jac(586,586)-k(22);
jac(197,586)=jac(197,586)+k(22);
jac(79,586)=jac(79,586)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(586);
jac(3,586)=jac(3,586)-k(15)*y(3);
jac(586,3)=jac(586,3)-k(15)*y(586);
jac(586,586)=jac(586,586)-k(15)*y(3);
jac(203,3)=jac(203,3)+k(15)*y(586);
jac(203,586)=jac(203,586)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(586);
jac(3,586)=jac(3,586)-k(15)*y(3);
jac(586,3)=jac(586,3)-k(15)*y(586);
jac(586,586)=jac(586,586)-k(15)*y(3);
jac(312,3)=jac(312,3)+k(15)*y(586);
jac(312,586)=jac(312,586)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(586,586)=jac(586,586)-k(8);
jac(91,586)=jac(91,586)+k(8);
jac(257,586)=jac(257,586)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

jac(586,586)=jac(586,586)-k(6);
jac(72,586)=jac(72,586)+k(6);
jac(256,586)=jac(256,586)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(578,578)=jac(578,578)-k(34);
jac(573,578)=jac(573,578)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(578,578)=jac(578,578)-k(33);
jac(497,578)=jac(497,578)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(578,578)=jac(578,578)-k(32);
jac(586,578)=jac(586,578)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(578,578)=jac(578,578)-k(31);
jac(292,578)=jac(292,578)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(578,578)=jac(578,578)-k(31);
jac(585,578)=jac(585,578)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~pY, a~pY) 

jac(578,578)=jac(578,578)-k(22);
jac(197,578)=jac(197,578)+k(22);
jac(584,578)=jac(584,578)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(578);
jac(197,578)=jac(197,578)-k(21)*y(197);
jac(578,197)=jac(578,197)-k(21)*y(578);
jac(578,578)=jac(578,578)-k(21)*y(197);
jac(583,197)=jac(583,197)+k(21)*y(578);
jac(583,578)=jac(583,578)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(578);
jac(32,578)=jac(32,578)-k(21)*y(32);
jac(578,32)=jac(578,32)-k(21)*y(578);
jac(578,578)=jac(578,578)-k(21)*y(32);
jac(582,32)=jac(582,32)+k(21)*y(578);
jac(582,578)=jac(582,578)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(578);
jac(2,578)=jac(2,578)-k(21)*y(2);
jac(578,2)=jac(578,2)-k(21)*y(578);
jac(578,578)=jac(578,578)-k(21)*y(2);
jac(259,2)=jac(259,2)+k(21)*y(578);
jac(259,578)=jac(259,578)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(578);
jac(3,578)=jac(3,578)-k(15)*y(3);
jac(578,3)=jac(578,3)-k(15)*y(578);
jac(578,578)=jac(578,578)-k(15)*y(3);
jac(260,3)=jac(260,3)+k(15)*y(578);
jac(260,578)=jac(260,578)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(578);
jac(3,578)=jac(3,578)-k(15)*y(3);
jac(578,3)=jac(578,3)-k(15)*y(578);
jac(578,578)=jac(578,578)-k(15)*y(3);
jac(581,3)=jac(581,3)+k(15)*y(578);
jac(581,578)=jac(581,578)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(578,578)=jac(578,578)-k(8);
jac(315,578)=jac(315,578)+k(8);
jac(257,578)=jac(257,578)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

jac(578,578)=jac(578,578)-k(6);
jac(77,578)=jac(77,578)+k(6);
jac(256,578)=jac(256,578)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(580,580)=jac(580,580)-k(34);
jac(280,580)=jac(280,580)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(580,580)=jac(580,580)-k(34);
jac(271,580)=jac(271,580)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(580,580)=jac(580,580)-k(31);
jac(187,580)=jac(187,580)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(580,580)=jac(580,580)-k(22);
jac(182,580)=jac(182,580)+k(22);
jac(431,580)=jac(431,580)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(580,580)=jac(580,580)-k(22);
jac(182,580)=jac(182,580)+k(22);
jac(421,580)=jac(421,580)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(580);
jac(3,580)=jac(3,580)-k(15)*y(3);
jac(580,3)=jac(580,3)-k(15)*y(580);
jac(580,580)=jac(580,580)-k(15)*y(3);
jac(273,3)=jac(273,3)+k(15)*y(580);
jac(273,580)=jac(273,580)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(580);
jac(3,580)=jac(3,580)-k(9)*y(3);
jac(580,3)=jac(580,3)-k(9)*y(580);
jac(580,580)=jac(580,580)-k(9)*y(3);
jac(395,3)=jac(395,3)+k(9)*y(580);
jac(395,580)=jac(395,580)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(580,580)=jac(580,580)-k(8);
jac(177,580)=jac(177,580)+k(8);
jac(219,580)=jac(219,580)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(580,580)=jac(580,580)-k(6);
jac(218,580)=jac(218,580)+k(6);
jac(176,580)=jac(176,580)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(579,579)=jac(579,579)-k(34);
jac(505,579)=jac(505,579)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(579,579)=jac(579,579)-k(34);
jac(266,579)=jac(266,579)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(579,579)=jac(579,579)-k(31);
jac(580,579)=jac(580,579)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(579,579)=jac(579,579)-k(31);
jac(513,579)=jac(513,579)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(579,579)=jac(579,579)-k(22);
jac(182,579)=jac(182,579)+k(22);
jac(576,579)=jac(576,579)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(579,579)=jac(579,579)-k(22);
jac(182,579)=jac(182,579)+k(22);
jac(497,579)=jac(497,579)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(579);
jac(3,579)=jac(3,579)-k(15)*y(3);
jac(579,3)=jac(579,3)-k(15)*y(579);
jac(579,579)=jac(579,579)-k(15)*y(3);
jac(268,3)=jac(268,3)+k(15)*y(579);
jac(268,579)=jac(268,579)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(579);
jac(3,579)=jac(3,579)-k(15)*y(3);
jac(579,3)=jac(579,3)-k(15)*y(579);
jac(579,579)=jac(579,579)-k(15)*y(3);
jac(517,3)=jac(517,3)+k(15)*y(579);
jac(517,579)=jac(517,579)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(579,579)=jac(579,579)-k(8);
jac(218,579)=jac(218,579)+k(8);
jac(219,579)=jac(219,579)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(579,579)=jac(579,579)-k(6);
jac(218,579)=jac(218,579)+k(6);
jac(264,579)=jac(264,579)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(577,577)=jac(577,577)-k(34);
jac(574,577)=jac(574,577)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(577,577)=jac(577,577)-k(34);
jac(259,577)=jac(259,577)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(577,577)=jac(577,577)-k(33);
jac(579,577)=jac(579,577)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(577,577)=jac(577,577)-k(31);
jac(290,577)=jac(290,577)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(577,577)=jac(577,577)-k(31);
jac(550,577)=jac(550,577)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(577,577)=jac(577,577)-k(22);
jac(197,577)=jac(197,577)+k(22);
jac(576,577)=jac(576,577)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(577,577)=jac(577,577)-k(22);
jac(182,577)=jac(182,577)+k(22);
jac(578,577)=jac(578,577)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(577);
jac(3,577)=jac(3,577)-k(15)*y(3);
jac(577,3)=jac(577,3)-k(15)*y(577);
jac(577,577)=jac(577,577)-k(15)*y(3);
jac(261,3)=jac(261,3)+k(15)*y(577);
jac(261,577)=jac(261,577)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(577);
jac(3,577)=jac(3,577)-k(15)*y(3);
jac(577,3)=jac(577,3)-k(15)*y(577);
jac(577,577)=jac(577,577)-k(15)*y(3);
jac(551,3)=jac(551,3)+k(15)*y(577);
jac(551,577)=jac(551,577)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(577,577)=jac(577,577)-k(8);
jac(257,577)=jac(257,577)+k(8);
jac(219,577)=jac(219,577)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(577,577)=jac(577,577)-k(6);
jac(218,577)=jac(218,577)+k(6);
jac(256,577)=jac(256,577)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(574,574)=jac(574,574)-k(34);
jac(572,574)=jac(572,574)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(574,574)=jac(574,574)-k(33);
jac(505,574)=jac(505,574)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(574,574)=jac(574,574)-k(31);
jac(285,574)=jac(285,574)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(574,574)=jac(574,574)-k(31);
jac(348,574)=jac(348,574)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(574,574)=jac(574,574)-k(30);
jac(577,574)=jac(577,574)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(574,574)=jac(574,574)-k(22);
jac(32,574)=jac(32,574)+k(22);
jac(576,574)=jac(576,574)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(574,574)=jac(574,574)-k(22);
jac(182,574)=jac(182,574)+k(22);
jac(573,574)=jac(573,574)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(574);
jac(3,574)=jac(3,574)-k(15)*y(3);
jac(574,3)=jac(574,3)-k(15)*y(574);
jac(574,574)=jac(574,574)-k(15)*y(3);
jac(286,3)=jac(286,3)+k(15)*y(574);
jac(286,574)=jac(286,574)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(574);
jac(3,574)=jac(3,574)-k(15)*y(3);
jac(574,3)=jac(574,3)-k(15)*y(574);
jac(574,574)=jac(574,574)-k(15)*y(3);
jac(555,3)=jac(555,3)+k(15)*y(574);
jac(555,574)=jac(555,574)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(574,574)=jac(574,574)-k(8);
jac(33,574)=jac(33,574)+k(8);
jac(219,574)=jac(219,574)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(574,574)=jac(574,574)-k(6);
jac(218,574)=jac(218,574)+k(6);
jac(82,574)=jac(82,574)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(575,575)=jac(575,575)-k(33);
jac(506,575)=jac(506,575)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(575,575)=jac(575,575)-k(31);
jac(166,575)=jac(166,575)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(575,575)=jac(575,575)-k(28);
jac(545,575)=jac(545,575)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(575,575)=jac(575,575)-k(27);
jac(348,575)=jac(348,575)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(575,575)=jac(575,575)-k(22);
jac(32,575)=jac(32,575)+k(22);
jac(508,575)=jac(508,575)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(575,575)=jac(575,575)-k(22);
jac(2,575)=jac(2,575)+k(22);
jac(351,575)=jac(351,575)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(575);
jac(3,575)=jac(3,575)-k(15)*y(3);
jac(575,3)=jac(575,3)-k(15)*y(575);
jac(575,575)=jac(575,575)-k(15)*y(3);
jac(541,3)=jac(541,3)+k(15)*y(575);
jac(541,575)=jac(541,575)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(575);
jac(3,575)=jac(3,575)-k(9)*y(3);
jac(575,3)=jac(575,3)-k(9)*y(575);
jac(575,575)=jac(575,575)-k(9)*y(3);
jac(167,3)=jac(167,3)+k(9)*y(575);
jac(167,575)=jac(167,575)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(575,575)=jac(575,575)-k(8);
jac(33,575)=jac(33,575)+k(8);
jac(22,575)=jac(22,575)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(575,575)=jac(575,575)-k(6);
jac(21,575)=jac(21,575)+k(6);
jac(82,575)=jac(82,575)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(572,572)=jac(572,572)-k(33);
jac(503,572)=jac(503,572)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(572,572)=jac(572,572)-k(31);
jac(442,572)=jac(442,572)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(572,572)=jac(572,572)-k(31);
jac(575,572)=jac(575,572)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(572,572)=jac(572,572)-k(28);
jac(259,572)=jac(259,572)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(572,572)=jac(572,572)-k(27);
jac(574,572)=jac(574,572)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(572,572)=jac(572,572)-k(22);
jac(32,572)=jac(32,572)+k(22);
jac(252,572)=jac(252,572)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(572,572)=jac(572,572)-k(22);
jac(2,572)=jac(2,572)+k(22);
jac(573,572)=jac(573,572)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(572);
jac(3,572)=jac(3,572)-k(15)*y(3);
jac(572,3)=jac(572,3)-k(15)*y(572);
jac(572,572)=jac(572,572)-k(15)*y(3);
jac(254,3)=jac(254,3)+k(15)*y(572);
jac(254,572)=jac(254,572)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(572);
jac(3,572)=jac(3,572)-k(15)*y(3);
jac(572,3)=jac(572,3)-k(15)*y(572);
jac(572,572)=jac(572,572)-k(15)*y(3);
jac(570,3)=jac(570,3)+k(15)*y(572);
jac(570,572)=jac(570,572)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(572,572)=jac(572,572)-k(8);
jac(33,572)=jac(33,572)+k(8);
jac(437,572)=jac(437,572)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(572,572)=jac(572,572)-k(6);
jac(111,572)=jac(111,572)+k(6);
jac(82,572)=jac(82,572)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(570,570)=jac(570,570)-k(33);
jac(504,570)=jac(504,570)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(570,570)=jac(570,570)-k(31);
jac(541,570)=jac(541,570)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(570,570)=jac(570,570)-k(28);
jac(546,570)=jac(546,570)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(570,570)=jac(570,570)-k(27);
jac(555,570)=jac(555,570)+k(27);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(570,570)=jac(570,570)-k(25);
jac(566,570)=jac(566,570)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(570,570)=jac(570,570)-k(22);
jac(32,570)=jac(32,570)+k(22);
jac(540,570)=jac(540,570)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(570,570)=jac(570,570)-k(22);
jac(2,570)=jac(2,570)+k(22);
jac(563,570)=jac(563,570)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(570,570)=jac(570,570)-k(16);
jac(3,570)=jac(3,570)+k(16);
jac(572,570)=jac(572,570)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(570);
jac(3,570)=jac(3,570)-k(15)*y(3);
jac(570,3)=jac(570,3)-k(15)*y(570);
jac(570,570)=jac(570,570)-k(15)*y(3);
jac(571,3)=jac(571,3)+k(15)*y(570);
jac(571,570)=jac(571,570)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(570,570)=jac(570,570)-k(8);
jac(437,570)=jac(437,570)+k(8);
jac(37,570)=jac(37,570)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(570,570)=jac(570,570)-k(6);
jac(111,570)=jac(111,570)+k(6);
jac(554,570)=jac(554,570)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(566,566)=jac(566,566)-k(33);
jac(500,566)=jac(500,566)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(566,566)=jac(566,566)-k(33);
jac(570,566)=jac(570,566)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(566,566)=jac(566,566)-k(31);
jac(535,566)=jac(535,566)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(566,566)=jac(566,566)-k(28);
jac(569,566)=jac(569,566)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(566,566)=jac(566,566)-k(27);
jac(560,566)=jac(560,566)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(566,566)=jac(566,566)-k(22);
jac(32,566)=jac(32,566)+k(22);
jac(534,566)=jac(534,566)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(566,566)=jac(566,566)-k(22);
jac(32,566)=jac(32,566)+k(22);
jac(563,566)=jac(563,566)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(566,566)=jac(566,566)-k(16);
jac(3,566)=jac(3,566)+k(16);
jac(568,566)=jac(568,566)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(566);
jac(3,566)=jac(3,566)-k(15)*y(3);
jac(566,3)=jac(566,3)-k(15)*y(566);
jac(566,566)=jac(566,566)-k(15)*y(3);
jac(567,3)=jac(567,3)+k(15)*y(566);
jac(567,566)=jac(567,566)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(566,566)=jac(566,566)-k(8);
jac(34,566)=jac(34,566)+k(8);
jac(37,566)=jac(37,566)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(566,566)=jac(566,566)-k(6);
jac(33,566)=jac(33,566)+k(6);
jac(554,566)=jac(554,566)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(560,560)=jac(560,560)-k(34);
jac(566,560)=jac(566,560)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(560,560)=jac(560,560)-k(33);
jac(528,560)=jac(528,560)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(560,560)=jac(560,560)-k(33);
jac(555,560)=jac(555,560)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(560,560)=jac(560,560)-k(31);
jac(557,560)=jac(557,560)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(560,560)=jac(560,560)-k(30);
jac(565,560)=jac(565,560)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(560,560)=jac(560,560)-k(22);
jac(32,560)=jac(32,560)+k(22);
jac(564,560)=jac(564,560)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(560,560)=jac(560,560)-k(22);
jac(197,560)=jac(197,560)+k(22);
jac(563,560)=jac(563,560)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(560,560)=jac(560,560)-k(16);
jac(3,560)=jac(3,560)+k(16);
jac(562,560)=jac(562,560)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(560);
jac(3,560)=jac(3,560)-k(15)*y(3);
jac(560,3)=jac(560,3)-k(15)*y(560);
jac(560,560)=jac(560,560)-k(15)*y(3);
jac(561,3)=jac(561,3)+k(15)*y(560);
jac(561,560)=jac(561,560)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(560,560)=jac(560,560)-k(8);
jac(319,560)=jac(319,560)+k(8);
jac(37,560)=jac(37,560)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(560,560)=jac(560,560)-k(6);
jac(257,560)=jac(257,560)+k(6);
jac(554,560)=jac(554,560)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(557,557)=jac(557,557)-k(34);
jac(535,557)=jac(535,557)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(557,557)=jac(557,557)-k(33);
jac(525,557)=jac(525,557)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(557,557)=jac(557,557)-k(33);
jac(553,557)=jac(553,557)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(557,557)=jac(557,557)-k(30);
jac(552,557)=jac(552,557)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(557,557)=jac(557,557)-k(22);
jac(32,557)=jac(32,557)+k(22);
jac(529,557)=jac(529,557)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(557,557)=jac(557,557)-k(22);
jac(197,557)=jac(197,557)+k(22);
jac(556,557)=jac(556,557)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(557,557)=jac(557,557)-k(17);
jac(560,557)=jac(560,557)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(557,557)=jac(557,557)-k(16);
jac(3,557)=jac(3,557)+k(16);
jac(559,557)=jac(559,557)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(557);
jac(3,557)=jac(3,557)-k(9)*y(3);
jac(557,3)=jac(557,3)-k(9)*y(557);
jac(557,557)=jac(557,557)-k(9)*y(3);
jac(558,3)=jac(558,3)+k(9)*y(557);
jac(558,557)=jac(558,557)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(557,557)=jac(557,557)-k(8);
jac(209,557)=jac(209,557)+k(8);
jac(37,557)=jac(37,557)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(557,557)=jac(557,557)-k(6);
jac(192,557)=jac(192,557)+k(6);
jac(554,557)=jac(554,557)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(553,553)=jac(553,553)-k(34);
jac(541,553)=jac(541,553)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(553,553)=jac(553,553)-k(33);
jac(521,553)=jac(521,553)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(553,553)=jac(553,553)-k(30);
jac(548,553)=jac(548,553)+k(30);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(553,553)=jac(553,553)-k(25);
jac(557,553)=jac(557,553)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(553,553)=jac(553,553)-k(22);
jac(32,553)=jac(32,553)+k(22);
jac(519,553)=jac(519,553)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(553,553)=jac(553,553)-k(22);
jac(182,553)=jac(182,553)+k(22);
jac(556,553)=jac(556,553)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(553,553)=jac(553,553)-k(17);
jac(555,553)=jac(555,553)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(553,553)=jac(553,553)-k(16);
jac(3,553)=jac(3,553)+k(16);
jac(348,553)=jac(348,553)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(553);
jac(3,553)=jac(3,553)-k(9)*y(3);
jac(553,3)=jac(553,3)-k(9)*y(553);
jac(553,553)=jac(553,553)-k(9)*y(3);
jac(349,3)=jac(349,3)+k(9)*y(553);
jac(349,553)=jac(349,553)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(553,553)=jac(553,553)-k(8);
jac(214,553)=jac(214,553)+k(8);
jac(37,553)=jac(37,553)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(553,553)=jac(553,553)-k(6);
jac(177,553)=jac(177,553)+k(6);
jac(554,553)=jac(554,553)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(548,548)=jac(548,548)-k(34);
jac(553,548)=jac(553,548)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(548,548)=jac(548,548)-k(34);
jac(542,548)=jac(542,548)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(548,548)=jac(548,548)-k(33);
jac(514,548)=jac(514,548)+k(33);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(548,548)=jac(548,548)-k(25);
jac(552,548)=jac(552,548)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(548,548)=jac(548,548)-k(22);
jac(197,548)=jac(197,548)+k(22);
jac(519,548)=jac(519,548)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(548,548)=jac(548,548)-k(22);
jac(182,548)=jac(182,548)+k(22);
jac(547,548)=jac(547,548)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(548,548)=jac(548,548)-k(17);
jac(551,548)=jac(551,548)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(548,548)=jac(548,548)-k(16);
jac(3,548)=jac(3,548)+k(16);
jac(550,548)=jac(550,548)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(548);
jac(3,548)=jac(3,548)-k(9)*y(3);
jac(548,3)=jac(548,3)-k(9)*y(548);
jac(548,548)=jac(548,548)-k(9)*y(3);
jac(549,3)=jac(549,3)+k(9)*y(548);
jac(549,548)=jac(549,548)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(548,548)=jac(548,548)-k(8);
jac(214,548)=jac(214,548)+k(8);
jac(227,548)=jac(227,548)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(548,548)=jac(548,548)-k(6);
jac(177,548)=jac(177,548)+k(6);
jac(543,548)=jac(543,548)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(542,542)=jac(542,542)-k(34);
jac(541,542)=jac(541,542)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(542,542)=jac(542,542)-k(33);
jac(511,542)=jac(511,542)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(542,542)=jac(542,542)-k(29);
jac(548,542)=jac(548,542)+k(29);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(542,542)=jac(542,542)-k(25);
jac(536,542)=jac(536,542)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(542,542)=jac(542,542)-k(22);
jac(197,542)=jac(197,542)+k(22);
jac(538,542)=jac(538,542)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(542,542)=jac(542,542)-k(22);
jac(2,542)=jac(2,542)+k(22);
jac(547,542)=jac(547,542)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(542,542)=jac(542,542)-k(17);
jac(546,542)=jac(546,542)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(542,542)=jac(542,542)-k(16);
jac(3,542)=jac(3,542)+k(16);
jac(545,542)=jac(545,542)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(542);
jac(3,542)=jac(3,542)-k(9)*y(3);
jac(542,3)=jac(542,3)-k(9)*y(542);
jac(542,542)=jac(542,542)-k(9)*y(3);
jac(544,3)=jac(544,3)+k(9)*y(542);
jac(544,542)=jac(544,542)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(542,542)=jac(542,542)-k(8);
jac(22,542)=jac(22,542)+k(8);
jac(227,542)=jac(227,542)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(542,542)=jac(542,542)-k(6);
jac(21,542)=jac(21,542)+k(6);
jac(543,542)=jac(543,542)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(538,538)=jac(538,538)-k(32);
jac(329,538)=jac(329,538)+k(32);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(538,538)=jac(538,538)-k(25);
jac(531,538)=jac(531,538)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

jac(538,538)=jac(538,538)-k(22);
jac(2,538)=jac(2,538)+k(22);
jac(537,538)=jac(537,538)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(538);
jac(197,538)=jac(197,538)-k(21)*y(197);
jac(538,197)=jac(538,197)-k(21)*y(538);
jac(538,538)=jac(538,538)-k(21)*y(197);
jac(542,197)=jac(542,197)+k(21)*y(538);
jac(542,538)=jac(542,538)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(538);
jac(32,538)=jac(32,538)-k(21)*y(32);
jac(538,32)=jac(538,32)-k(21)*y(538);
jac(538,538)=jac(538,538)-k(21)*y(32);
jac(541,32)=jac(541,32)+k(21)*y(538);
jac(541,538)=jac(541,538)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(538);
jac(2,538)=jac(2,538)-k(21)*y(2);
jac(538,2)=jac(538,2)-k(21)*y(538);
jac(538,538)=jac(538,538)-k(21)*y(2);
jac(507,2)=jac(507,2)+k(21)*y(538);
jac(507,538)=jac(507,538)+k(21)*y(2);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(538,538)=jac(538,538)-k(17);
jac(540,538)=jac(540,538)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(538,538)=jac(538,538)-k(16);
jac(3,538)=jac(3,538)+k(16);
jac(508,538)=jac(508,538)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(538);
jac(3,538)=jac(3,538)-k(9)*y(3);
jac(538,3)=jac(538,3)-k(9)*y(538);
jac(538,538)=jac(538,538)-k(9)*y(3);
jac(539,3)=jac(539,3)+k(9)*y(538);
jac(539,538)=jac(539,538)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(538,538)=jac(538,538)-k(8);
jac(54,538)=jac(54,538)+k(8);
jac(22,538)=jac(22,538)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(538,538)=jac(538,538)-k(6);
jac(118,538)=jac(118,538)+k(6);
jac(21,538)=jac(21,538)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(531,531)=jac(531,531)-k(33);
jac(538,531)=jac(538,531)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(531,531)=jac(531,531)-k(32);
jac(133,531)=jac(133,531)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) 

jac(531,531)=jac(531,531)-k(22);
jac(32,531)=jac(32,531)+k(22);
jac(537,531)=jac(537,531)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(531);
jac(197,531)=jac(197,531)-k(21)*y(197);
jac(531,197)=jac(531,197)-k(21)*y(531);
jac(531,531)=jac(531,531)-k(21)*y(197);
jac(536,197)=jac(536,197)+k(21)*y(531);
jac(536,531)=jac(536,531)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(531);
jac(32,531)=jac(32,531)-k(21)*y(32);
jac(531,32)=jac(531,32)-k(21)*y(531);
jac(531,531)=jac(531,531)-k(21)*y(32);
jac(535,32)=jac(535,32)+k(21)*y(531);
jac(535,531)=jac(535,531)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(531);
jac(2,531)=jac(2,531)-k(21)*y(2);
jac(531,2)=jac(531,2)-k(21)*y(531);
jac(531,531)=jac(531,531)-k(21)*y(2);
jac(530,2)=jac(530,2)+k(21)*y(531);
jac(530,531)=jac(530,531)+k(21)*y(2);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(531,531)=jac(531,531)-k(17);
jac(534,531)=jac(534,531)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(531,531)=jac(531,531)-k(16);
jac(3,531)=jac(3,531)+k(16);
jac(533,531)=jac(533,531)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(531);
jac(3,531)=jac(3,531)-k(9)*y(3);
jac(531,3)=jac(531,3)-k(9)*y(531);
jac(531,531)=jac(531,531)-k(9)*y(3);
jac(134,3)=jac(134,3)+k(9)*y(531);
jac(134,531)=jac(134,531)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(531,531)=jac(531,531)-k(8);
jac(54,531)=jac(54,531)+k(8);
jac(28,531)=jac(28,531)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(531,531)=jac(531,531)-k(6);
jac(118,531)=jac(118,531)+k(6);
jac(27,531)=jac(27,531)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(532,532)=jac(532,532)-k(34);
jac(530,532)=jac(530,532)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(532,532)=jac(532,532)-k(33);
jac(511,532)=jac(511,532)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(532,532)=jac(532,532)-k(29);
jac(520,532)=jac(520,532)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(532,532)=jac(532,532)-k(22);
jac(182,532)=jac(182,532)+k(22);
jac(531,532)=jac(531,532)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(532,532)=jac(532,532)-k(22);
jac(32,532)=jac(32,532)+k(22);
jac(518,532)=jac(518,532)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(532,532)=jac(532,532)-k(17);
jac(496,532)=jac(496,532)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(532,532)=jac(532,532)-k(16);
jac(3,532)=jac(3,532)+k(16);
jac(359,532)=jac(359,532)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(532);
jac(3,532)=jac(3,532)-k(9)*y(3);
jac(532,3)=jac(532,3)-k(9)*y(532);
jac(532,532)=jac(532,532)-k(9)*y(3);
jac(360,3)=jac(360,3)+k(9)*y(532);
jac(360,532)=jac(360,532)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(532,532)=jac(532,532)-k(8);
jac(28,532)=jac(28,532)+k(8);
jac(222,532)=jac(222,532)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(532,532)=jac(532,532)-k(6);
jac(27,532)=jac(27,532)+k(6);
jac(515,532)=jac(515,532)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(530,530)=jac(530,530)-k(33);
jac(507,530)=jac(507,530)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(530,530)=jac(530,530)-k(28);
jac(532,530)=jac(532,530)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(530,530)=jac(530,530)-k(27);
jac(525,530)=jac(525,530)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(530,530)=jac(530,530)-k(22);
jac(2,530)=jac(2,530)+k(22);
jac(531,530)=jac(531,530)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(530,530)=jac(530,530)-k(22);
jac(32,530)=jac(32,530)+k(22);
jac(524,530)=jac(524,530)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(530,530)=jac(530,530)-k(17);
jac(500,530)=jac(500,530)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(530,530)=jac(530,530)-k(16);
jac(3,530)=jac(3,530)+k(16);
jac(356,530)=jac(356,530)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(530);
jac(3,530)=jac(3,530)-k(9)*y(3);
jac(530,3)=jac(530,3)-k(9)*y(530);
jac(530,530)=jac(530,530)-k(9)*y(3);
jac(357,3)=jac(357,3)+k(9)*y(530);
jac(357,530)=jac(357,530)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(530,530)=jac(530,530)-k(8);
jac(28,530)=jac(28,530)+k(8);
jac(43,530)=jac(43,530)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(530,530)=jac(530,530)-k(6);
jac(27,530)=jac(27,530)+k(6);
jac(114,530)=jac(114,530)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(525,525)=jac(525,525)-k(34);
jac(530,525)=jac(530,525)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(525,525)=jac(525,525)-k(33);
jac(521,525)=jac(521,525)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(525,525)=jac(525,525)-k(30);
jac(520,525)=jac(520,525)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(525,525)=jac(525,525)-k(22);
jac(2,525)=jac(2,525)+k(22);
jac(529,525)=jac(529,525)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(525,525)=jac(525,525)-k(22);
jac(197,525)=jac(197,525)+k(22);
jac(524,525)=jac(524,525)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(525,525)=jac(525,525)-k(17);
jac(528,525)=jac(528,525)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(525,525)=jac(525,525)-k(16);
jac(3,525)=jac(3,525)+k(16);
jac(527,525)=jac(527,525)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(525);
jac(3,525)=jac(3,525)-k(9)*y(3);
jac(525,3)=jac(525,3)-k(9)*y(525);
jac(525,525)=jac(525,525)-k(9)*y(3);
jac(526,3)=jac(526,3)+k(9)*y(525);
jac(526,525)=jac(526,525)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(525,525)=jac(525,525)-k(8);
jac(209,525)=jac(209,525)+k(8);
jac(43,525)=jac(43,525)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(525,525)=jac(525,525)-k(6);
jac(192,525)=jac(192,525)+k(6);
jac(114,525)=jac(114,525)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(521,521)=jac(521,521)-k(34);
jac(507,521)=jac(507,521)+k(34);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(521,521)=jac(521,521)-k(30);
jac(514,521)=jac(514,521)+k(30);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(521,521)=jac(521,521)-k(25);
jac(525,521)=jac(525,521)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(521,521)=jac(521,521)-k(22);
jac(2,521)=jac(2,521)+k(22);
jac(519,521)=jac(519,521)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(521,521)=jac(521,521)-k(22);
jac(182,521)=jac(182,521)+k(22);
jac(524,521)=jac(524,521)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(521,521)=jac(521,521)-k(17);
jac(523,521)=jac(523,521)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(521,521)=jac(521,521)-k(16);
jac(3,521)=jac(3,521)+k(16);
jac(509,521)=jac(509,521)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(521);
jac(3,521)=jac(3,521)-k(9)*y(3);
jac(521,3)=jac(521,3)-k(9)*y(521);
jac(521,521)=jac(521,521)-k(9)*y(3);
jac(522,3)=jac(522,3)+k(9)*y(521);
jac(522,521)=jac(522,521)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(521,521)=jac(521,521)-k(8);
jac(214,521)=jac(214,521)+k(8);
jac(43,521)=jac(43,521)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(521,521)=jac(521,521)-k(6);
jac(177,521)=jac(177,521)+k(6);
jac(114,521)=jac(114,521)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(514,514)=jac(514,514)-k(34);
jac(521,514)=jac(521,514)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(514,514)=jac(514,514)-k(34);
jac(511,514)=jac(511,514)+k(34);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(514,514)=jac(514,514)-k(25);
jac(520,514)=jac(520,514)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(514,514)=jac(514,514)-k(22);
jac(182,514)=jac(182,514)+k(22);
jac(519,514)=jac(519,514)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(514,514)=jac(514,514)-k(22);
jac(182,514)=jac(182,514)+k(22);
jac(518,514)=jac(518,514)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(514,514)=jac(514,514)-k(17);
jac(517,514)=jac(517,514)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(514,514)=jac(514,514)-k(16);
jac(3,514)=jac(3,514)+k(16);
jac(513,514)=jac(513,514)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(514);
jac(3,514)=jac(3,514)-k(9)*y(3);
jac(514,3)=jac(514,3)-k(9)*y(514);
jac(514,514)=jac(514,514)-k(9)*y(3);
jac(516,3)=jac(516,3)+k(9)*y(514);
jac(516,514)=jac(516,514)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(514,514)=jac(514,514)-k(8);
jac(214,514)=jac(214,514)+k(8);
jac(222,514)=jac(222,514)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(514,514)=jac(514,514)-k(6);
jac(177,514)=jac(177,514)+k(6);
jac(515,514)=jac(515,514)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(513,513)=jac(513,513)-k(34);
jac(509,513)=jac(509,513)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(513,513)=jac(513,513)-k(34);
jac(510,513)=jac(510,513)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(513,513)=jac(513,513)-k(31);
jac(187,513)=jac(187,513)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(513,513)=jac(513,513)-k(22);
jac(182,513)=jac(182,513)+k(22);
jac(344,513)=jac(344,513)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(513,513)=jac(513,513)-k(22);
jac(182,513)=jac(182,513)+k(22);
jac(512,513)=jac(512,513)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(513);
jac(3,513)=jac(3,513)-k(15)*y(3);
jac(513,3)=jac(513,3)-k(15)*y(513);
jac(513,513)=jac(513,513)-k(15)*y(3);
jac(514,3)=jac(514,3)+k(15)*y(513);
jac(514,513)=jac(514,513)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(513);
jac(3,513)=jac(3,513)-k(9)*y(3);
jac(513,3)=jac(513,3)-k(9)*y(513);
jac(513,513)=jac(513,513)-k(9)*y(3);
jac(188,3)=jac(188,3)+k(9)*y(513);
jac(188,513)=jac(188,513)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(513,513)=jac(513,513)-k(8);
jac(218,513)=jac(218,513)+k(8);
jac(214,513)=jac(214,513)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(513,513)=jac(513,513)-k(6);
jac(177,513)=jac(177,513)+k(6);
jac(264,513)=jac(264,513)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(510,510)=jac(510,510)-k(34);
jac(506,510)=jac(506,510)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(510,510)=jac(510,510)-k(31);
jac(179,510)=jac(179,510)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(510,510)=jac(510,510)-k(29);
jac(513,510)=jac(513,510)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(510,510)=jac(510,510)-k(22);
jac(182,510)=jac(182,510)+k(22);
jac(508,510)=jac(508,510)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(510,510)=jac(510,510)-k(22);
jac(2,510)=jac(2,510)+k(22);
jac(512,510)=jac(512,510)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(510);
jac(3,510)=jac(3,510)-k(15)*y(3);
jac(510,3)=jac(510,3)-k(15)*y(510);
jac(510,510)=jac(510,510)-k(15)*y(3);
jac(511,3)=jac(511,3)+k(15)*y(510);
jac(511,510)=jac(511,510)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(510);
jac(3,510)=jac(3,510)-k(9)*y(3);
jac(510,3)=jac(510,3)-k(9)*y(510);
jac(510,510)=jac(510,510)-k(9)*y(3);
jac(180,3)=jac(180,3)+k(9)*y(510);
jac(180,510)=jac(180,510)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(510,510)=jac(510,510)-k(8);
jac(218,510)=jac(218,510)+k(8);
jac(22,510)=jac(22,510)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(510,510)=jac(510,510)-k(6);
jac(21,510)=jac(21,510)+k(6);
jac(264,510)=jac(264,510)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(506,506)=jac(506,506)-k(31);
jac(172,506)=jac(172,506)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(506,506)=jac(506,506)-k(28);
jac(510,506)=jac(510,506)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(506,506)=jac(506,506)-k(27);
jac(509,506)=jac(509,506)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(506,506)=jac(506,506)-k(22);
jac(2,506)=jac(2,506)+k(22);
jac(508,506)=jac(508,506)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(506,506)=jac(506,506)-k(22);
jac(2,506)=jac(2,506)+k(22);
jac(354,506)=jac(354,506)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(506);
jac(3,506)=jac(3,506)-k(15)*y(3);
jac(506,3)=jac(506,3)-k(15)*y(506);
jac(506,506)=jac(506,506)-k(15)*y(3);
jac(507,3)=jac(507,3)+k(15)*y(506);
jac(507,506)=jac(507,506)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(506);
jac(3,506)=jac(3,506)-k(9)*y(3);
jac(506,3)=jac(506,3)-k(9)*y(506);
jac(506,506)=jac(506,506)-k(9)*y(3);
jac(173,3)=jac(173,3)+k(9)*y(506);
jac(173,506)=jac(173,506)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(506,506)=jac(506,506)-k(8);
jac(111,506)=jac(111,506)+k(8);
jac(22,506)=jac(22,506)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(506,506)=jac(506,506)-k(6);
jac(21,506)=jac(21,506)+k(6);
jac(110,506)=jac(110,506)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(503,503)=jac(503,503)-k(31);
jac(275,503)=jac(275,503)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(503,503)=jac(503,503)-k(31);
jac(506,503)=jac(506,503)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(503,503)=jac(503,503)-k(28);
jac(266,503)=jac(266,503)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(503,503)=jac(503,503)-k(27);
jac(505,503)=jac(505,503)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(503,503)=jac(503,503)-k(22);
jac(2,503)=jac(2,503)+k(22);
jac(252,503)=jac(252,503)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(503,503)=jac(503,503)-k(22);
jac(2,503)=jac(2,503)+k(22);
jac(501,503)=jac(501,503)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(503);
jac(3,503)=jac(3,503)-k(15)*y(3);
jac(503,3)=jac(503,3)-k(15)*y(503);
jac(503,503)=jac(503,503)-k(15)*y(3);
jac(253,3)=jac(253,3)+k(15)*y(503);
jac(253,503)=jac(253,503)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(503);
jac(3,503)=jac(3,503)-k(15)*y(3);
jac(503,3)=jac(503,3)-k(15)*y(503);
jac(503,503)=jac(503,503)-k(15)*y(3);
jac(504,3)=jac(504,3)+k(15)*y(503);
jac(504,503)=jac(504,503)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(503,503)=jac(503,503)-k(8);
jac(111,503)=jac(111,503)+k(8);
jac(437,503)=jac(437,503)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(503,503)=jac(503,503)-k(6);
jac(111,503)=jac(111,503)+k(6);
jac(110,503)=jac(110,503)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(499,499)=jac(499,499)-k(33);
jac(503,499)=jac(503,499)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(499,499)=jac(499,499)-k(31);
jac(482,499)=jac(482,499)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(499,499)=jac(499,499)-k(31);
jac(356,499)=jac(356,499)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(499,499)=jac(499,499)-k(28);
jac(495,499)=jac(495,499)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(499,499)=jac(499,499)-k(27);
jac(502,499)=jac(502,499)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(499,499)=jac(499,499)-k(22);
jac(2,499)=jac(2,499)+k(22);
jac(245,499)=jac(245,499)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(499,499)=jac(499,499)-k(22);
jac(32,499)=jac(32,499)+k(22);
jac(501,499)=jac(501,499)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(499);
jac(3,499)=jac(3,499)-k(15)*y(3);
jac(499,3)=jac(499,3)-k(15)*y(499);
jac(499,499)=jac(499,499)-k(15)*y(3);
jac(246,3)=jac(246,3)+k(15)*y(499);
jac(246,499)=jac(246,499)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(499);
jac(3,499)=jac(3,499)-k(15)*y(3);
jac(499,3)=jac(499,3)-k(15)*y(499);
jac(499,499)=jac(499,499)-k(15)*y(3);
jac(500,3)=jac(500,3)+k(15)*y(499);
jac(500,499)=jac(500,499)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(499,499)=jac(499,499)-k(8);
jac(111,499)=jac(111,499)+k(8);
jac(34,499)=jac(34,499)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(499,499)=jac(499,499)-k(6);
jac(33,499)=jac(33,499)+k(6);
jac(110,499)=jac(110,499)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(495,495)=jac(495,495)-k(34);
jac(499,495)=jac(499,495)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(495,495)=jac(495,495)-k(33);
jac(266,495)=jac(266,495)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(495,495)=jac(495,495)-k(31);
jac(498,495)=jac(498,495)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(495,495)=jac(495,495)-k(31);
jac(359,495)=jac(359,495)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(495,495)=jac(495,495)-k(29);
jac(490,495)=jac(490,495)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(495,495)=jac(495,495)-k(22);
jac(182,495)=jac(182,495)+k(22);
jac(245,495)=jac(245,495)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(495,495)=jac(495,495)-k(22);
jac(32,495)=jac(32,495)+k(22);
jac(497,495)=jac(497,495)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(495);
jac(3,495)=jac(3,495)-k(15)*y(3);
jac(495,3)=jac(495,3)-k(15)*y(495);
jac(495,495)=jac(495,495)-k(15)*y(3);
jac(493,3)=jac(493,3)+k(15)*y(495);
jac(493,495)=jac(493,495)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(495);
jac(3,495)=jac(3,495)-k(15)*y(3);
jac(495,3)=jac(495,3)-k(15)*y(495);
jac(495,495)=jac(495,495)-k(15)*y(3);
jac(496,3)=jac(496,3)+k(15)*y(495);
jac(496,495)=jac(496,495)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(495,495)=jac(495,495)-k(8);
jac(218,495)=jac(218,495)+k(8);
jac(34,495)=jac(34,495)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(495,495)=jac(495,495)-k(6);
jac(33,495)=jac(33,495)+k(6);
jac(264,495)=jac(264,495)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(493,493)=jac(493,493)-k(34);
jac(246,493)=jac(246,493)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(493,493)=jac(493,493)-k(33);
jac(263,493)=jac(263,493)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(493,493)=jac(493,493)-k(31);
jac(479,493)=jac(479,493)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(493,493)=jac(493,493)-k(29);
jac(488,493)=jac(488,493)+k(29);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(493,493)=jac(493,493)-k(26);
jac(248,493)=jac(248,493)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(493,493)=jac(493,493)-k(22);
jac(182,493)=jac(182,493)+k(22);
jac(243,493)=jac(243,493)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(493,493)=jac(493,493)-k(22);
jac(32,493)=jac(32,493)+k(22);
jac(267,493)=jac(267,493)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(493,493)=jac(493,493)-k(16);
jac(3,493)=jac(3,493)+k(16);
jac(495,493)=jac(495,493)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(493);
jac(3,493)=jac(3,493)-k(15)*y(3);
jac(493,3)=jac(493,3)-k(15)*y(493);
jac(493,493)=jac(493,493)-k(15)*y(3);
jac(494,3)=jac(494,3)+k(15)*y(493);
jac(494,493)=jac(494,493)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(493,493)=jac(493,493)-k(8);
jac(218,493)=jac(218,493)+k(8);
jac(38,493)=jac(38,493)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(493,493)=jac(493,493)-k(6);
jac(264,493)=jac(264,493)+k(6);
jac(37,493)=jac(37,493)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(488,488)=jac(488,488)-k(34);
jac(485,488)=jac(485,488)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(488,488)=jac(488,488)-k(34);
jac(493,488)=jac(493,488)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(488,488)=jac(488,488)-k(33);
jac(268,488)=jac(268,488)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(488,488)=jac(488,488)-k(31);
jac(486,488)=jac(486,488)+k(31);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(488,488)=jac(488,488)-k(26);
jac(492,488)=jac(492,488)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(488,488)=jac(488,488)-k(22);
jac(182,488)=jac(182,488)+k(22);
jac(491,488)=jac(491,488)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(488,488)=jac(488,488)-k(22);
jac(197,488)=jac(197,488)+k(22);
jac(267,488)=jac(267,488)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(488,488)=jac(488,488)-k(16);
jac(3,488)=jac(3,488)+k(16);
jac(490,488)=jac(490,488)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(488);
jac(3,488)=jac(3,488)-k(15)*y(3);
jac(488,3)=jac(488,3)-k(15)*y(488);
jac(488,488)=jac(488,488)-k(15)*y(3);
jac(489,3)=jac(489,3)+k(15)*y(488);
jac(489,488)=jac(489,488)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(488,488)=jac(488,488)-k(8);
jac(218,488)=jac(218,488)+k(8);
jac(228,488)=jac(228,488)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(488,488)=jac(488,488)-k(6);
jac(264,488)=jac(264,488)+k(6);
jac(227,488)=jac(227,488)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(486,486)=jac(486,486)-k(34);
jac(484,486)=jac(484,486)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(486,486)=jac(486,486)-k(34);
jac(479,486)=jac(479,486)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(486,486)=jac(486,486)-k(33);
jac(273,486)=jac(273,486)+k(33);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(486,486)=jac(486,486)-k(26);
jac(294,486)=jac(294,486)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(486,486)=jac(486,486)-k(22);
jac(182,486)=jac(182,486)+k(22);
jac(232,486)=jac(232,486)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(486,486)=jac(486,486)-k(22);
jac(197,486)=jac(197,486)+k(22);
jac(272,486)=jac(272,486)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(486,486)=jac(486,486)-k(18);
jac(488,486)=jac(488,486)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(486,486)=jac(486,486)-k(16);
jac(3,486)=jac(3,486)+k(16);
jac(487,486)=jac(487,486)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(486);
jac(3,486)=jac(3,486)-k(9)*y(3);
jac(486,3)=jac(486,3)-k(9)*y(486);
jac(486,486)=jac(486,486)-k(9)*y(3);
jac(476,3)=jac(476,3)+k(9)*y(486);
jac(476,486)=jac(476,486)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(486,486)=jac(486,486)-k(8);
jac(177,486)=jac(177,486)+k(8);
jac(228,486)=jac(228,486)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(486,486)=jac(486,486)-k(6);
jac(176,486)=jac(176,486)+k(6);
jac(227,486)=jac(227,486)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(484,484)=jac(484,484)-k(34);
jac(481,484)=jac(481,484)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(484,484)=jac(484,484)-k(33);
jac(278,484)=jac(278,484)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(484,484)=jac(484,484)-k(30);
jac(486,484)=jac(486,484)+k(30);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(484,484)=jac(484,484)-k(26);
jac(474,484)=jac(474,484)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(484,484)=jac(484,484)-k(22);
jac(2,484)=jac(2,484)+k(22);
jac(232,484)=jac(232,484)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(484,484)=jac(484,484)-k(22);
jac(197,484)=jac(197,484)+k(22);
jac(276,484)=jac(276,484)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(484,484)=jac(484,484)-k(18);
jac(485,484)=jac(485,484)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(484,484)=jac(484,484)-k(16);
jac(3,484)=jac(3,484)+k(16);
jac(427,484)=jac(427,484)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(9)*y(484);
jac(3,484)=jac(3,484)-k(9)*y(3);
jac(484,3)=jac(484,3)-k(9)*y(484);
jac(484,484)=jac(484,484)-k(9)*y(3);
jac(428,3)=jac(428,3)+k(9)*y(484);
jac(428,484)=jac(428,484)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(484,484)=jac(484,484)-k(8);
jac(21,484)=jac(21,484)+k(8);
jac(228,484)=jac(228,484)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(484,484)=jac(484,484)-k(6);
jac(89,484)=jac(89,484)+k(6);
jac(227,484)=jac(227,484)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(481,481)=jac(481,481)-k(33);
jac(274,481)=jac(274,481)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(481,481)=jac(481,481)-k(28);
jac(479,481)=jac(479,481)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(481,481)=jac(481,481)-k(27);
jac(484,481)=jac(484,481)+k(27);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(481,481)=jac(481,481)-k(26);
jac(483,481)=jac(483,481)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(481,481)=jac(481,481)-k(22);
jac(2,481)=jac(2,481)+k(22);
jac(249,481)=jac(249,481)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(481,481)=jac(481,481)-k(22);
jac(32,481)=jac(32,481)+k(22);
jac(276,481)=jac(276,481)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(481,481)=jac(481,481)-k(18);
jac(246,481)=jac(246,481)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(481,481)=jac(481,481)-k(16);
jac(3,481)=jac(3,481)+k(16);
jac(482,481)=jac(482,481)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(481);
jac(3,481)=jac(3,481)-k(9)*y(3);
jac(481,3)=jac(481,3)-k(9)*y(481);
jac(481,481)=jac(481,481)-k(9)*y(3);
jac(480,3)=jac(480,3)+k(9)*y(481);
jac(480,481)=jac(480,481)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(481,481)=jac(481,481)-k(8);
jac(21,481)=jac(21,481)+k(8);
jac(38,481)=jac(38,481)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(481,481)=jac(481,481)-k(6);
jac(89,481)=jac(89,481)+k(6);
jac(37,481)=jac(37,481)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(480,480)=jac(480,480)-k(33);
jac(51,480)=jac(51,480)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(480,480)=jac(480,480)-k(28);
jac(478,480)=jac(478,480)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(480,480)=jac(480,480)-k(27);
jac(428,480)=jac(428,480)+k(27);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(480,480)=jac(480,480)-k(26);
jac(469,480)=jac(469,480)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(480,480)=jac(480,480)-k(22);
jac(2,480)=jac(2,480)+k(22);
jac(40,480)=jac(40,480)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(480,480)=jac(480,480)-k(22);
jac(32,480)=jac(32,480)+k(22);
jac(58,480)=jac(58,480)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(480,480)=jac(480,480)-k(16);
jac(3,480)=jac(3,480)+k(16);
jac(382,480)=jac(382,480)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(480,480)=jac(480,480)-k(10);
jac(3,480)=jac(3,480)+k(10);
jac(481,480)=jac(481,480)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(480,480)=jac(480,480)-k(8);
jac(140,480)=jac(140,480)+k(8);
jac(38,480)=jac(38,480)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(480,480)=jac(480,480)-k(6);
jac(37,480)=jac(37,480)+k(6);
jac(380,480)=jac(380,480)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(478,478)=jac(478,478)-k(34);
jac(480,478)=jac(480,478)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(478,478)=jac(478,478)-k(33);
jac(270,478)=jac(270,478)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(478,478)=jac(478,478)-k(29);
jac(476,478)=jac(476,478)+k(29);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(478,478)=jac(478,478)-k(26);
jac(477,478)=jac(477,478)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(478,478)=jac(478,478)-k(22);
jac(182,478)=jac(182,478)+k(22);
jac(40,478)=jac(40,478)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) + Syk(tSH2, l~pY, a~Y) 

jac(478,478)=jac(478,478)-k(22);
jac(32,478)=jac(32,478)+k(22);
jac(422,478)=jac(422,478)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(478,478)=jac(478,478)-k(16);
jac(3,478)=jac(3,478)+k(16);
jac(423,478)=jac(423,478)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) 

jac(478,478)=jac(478,478)-k(10);
jac(3,478)=jac(3,478)+k(10);
jac(479,478)=jac(479,478)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(478,478)=jac(478,478)-k(8);
jac(184,478)=jac(184,478)+k(8);
jac(38,478)=jac(38,478)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(478,478)=jac(478,478)-k(6);
jac(37,478)=jac(37,478)+k(6);
jac(391,478)=jac(391,478)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(477,477)=jac(477,477)-k(34);
jac(469,477)=jac(469,477)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(477,477)=jac(477,477)-k(33);
jac(478,477)=jac(478,477)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(477,477)=jac(477,477)-k(33);
jac(207,477)=jac(207,477)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(477,477)=jac(477,477)-k(29);
jac(475,477)=jac(475,477)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(477,477)=jac(477,477)-k(22);
jac(197,477)=jac(197,477)+k(22);
jac(40,477)=jac(40,477)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(477,477)=jac(477,477)-k(22);
jac(32,477)=jac(32,477)+k(22);
jac(206,477)=jac(206,477)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(477,477)=jac(477,477)-k(16);
jac(3,477)=jac(3,477)+k(16);
jac(439,477)=jac(439,477)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(477,477)=jac(477,477)-k(10);
jac(3,477)=jac(3,477)+k(10);
jac(293,477)=jac(293,477)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(477,477)=jac(477,477)-k(8);
jac(335,477)=jac(335,477)+k(8);
jac(38,477)=jac(38,477)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(477,477)=jac(477,477)-k(6);
jac(37,477)=jac(37,477)+k(6);
jac(407,477)=jac(407,477)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(475,475)=jac(475,475)-k(34);
jac(473,475)=jac(473,475)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(475,475)=jac(475,475)-k(34);
jac(477,475)=jac(477,475)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(475,475)=jac(475,475)-k(33);
jac(476,475)=jac(476,475)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(475,475)=jac(475,475)-k(33);
jac(289,475)=jac(289,475)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) 

jac(475,475)=jac(475,475)-k(22);
jac(197,475)=jac(197,475)+k(22);
jac(205,475)=jac(205,475)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(475,475)=jac(475,475)-k(22);
jac(197,475)=jac(197,475)+k(22);
jac(206,475)=jac(206,475)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(475,475)=jac(475,475)-k(16);
jac(3,475)=jac(3,475)+k(16);
jac(434,475)=jac(434,475)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(475,475)=jac(475,475)-k(10);
jac(3,475)=jac(3,475)+k(10);
jac(294,475)=jac(294,475)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(475,475)=jac(475,475)-k(8);
jac(335,475)=jac(335,475)+k(8);
jac(228,475)=jac(228,475)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(475,475)=jac(475,475)-k(6);
jac(227,475)=jac(227,475)+k(6);
jac(407,475)=jac(407,475)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(473,473)=jac(473,473)-k(34);
jac(469,473)=jac(469,473)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(473,473)=jac(473,473)-k(33);
jac(284,473)=jac(284,473)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(473,473)=jac(473,473)-k(33);
jac(428,473)=jac(428,473)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(473,473)=jac(473,473)-k(30);
jac(475,473)=jac(475,473)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(473,473)=jac(473,473)-k(22);
jac(197,473)=jac(197,473)+k(22);
jac(59,473)=jac(59,473)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(473,473)=jac(473,473)-k(22);
jac(32,473)=jac(32,473)+k(22);
jac(205,473)=jac(205,473)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(473,473)=jac(473,473)-k(16);
jac(3,473)=jac(3,473)+k(16);
jac(471,473)=jac(471,473)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(473,473)=jac(473,473)-k(10);
jac(3,473)=jac(3,473)+k(10);
jac(474,473)=jac(474,473)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(473,473)=jac(473,473)-k(8);
jac(131,473)=jac(131,473)+k(8);
jac(228,473)=jac(228,473)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(473,473)=jac(473,473)-k(6);
jac(227,473)=jac(227,473)+k(6);
jac(441,473)=jac(441,473)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(471,471)=jac(471,471)-k(34);
jac(444,471)=jac(444,471)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(471,471)=jac(471,471)-k(33);
jac(426,471)=jac(426,471)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(471,471)=jac(471,471)-k(33);
jac(432,471)=jac(432,471)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(471,471)=jac(471,471)-k(31);
jac(402,471)=jac(402,471)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(471,471)=jac(471,471)-k(30);
jac(434,471)=jac(434,471)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(471,471)=jac(471,471)-k(22);
jac(32,471)=jac(32,471)+k(22);
jac(401,471)=jac(401,471)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(471,471)=jac(471,471)-k(22);
jac(197,471)=jac(197,471)+k(22);
jac(443,471)=jac(443,471)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(471);
jac(3,471)=jac(3,471)-k(15)*y(3);
jac(471,3)=jac(471,3)-k(15)*y(471);
jac(471,471)=jac(471,471)-k(15)*y(3);
jac(473,3)=jac(473,3)+k(15)*y(471);
jac(473,471)=jac(473,471)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(471,471)=jac(471,471)-k(10);
jac(3,471)=jac(3,471)+k(10);
jac(472,471)=jac(472,471)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(471,471)=jac(471,471)-k(8);
jac(319,471)=jac(319,471)+k(8);
jac(131,471)=jac(131,471)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(471,471)=jac(471,471)-k(6);
jac(257,471)=jac(257,471)+k(6);
jac(441,471)=jac(441,471)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(444,444)=jac(444,444)-k(33);
jac(382,444)=jac(382,444)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(444,444)=jac(444,444)-k(33);
jac(440,444)=jac(440,444)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(444,444)=jac(444,444)-k(31);
jac(446,444)=jac(446,444)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(444,444)=jac(444,444)-k(28);
jac(439,444)=jac(439,444)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(444,444)=jac(444,444)-k(27);
jac(471,444)=jac(471,444)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(444,444)=jac(444,444)-k(22);
jac(32,444)=jac(32,444)+k(22);
jac(470,444)=jac(470,444)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(444,444)=jac(444,444)-k(22);
jac(32,444)=jac(32,444)+k(22);
jac(443,444)=jac(443,444)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(444);
jac(3,444)=jac(3,444)-k(15)*y(3);
jac(444,3)=jac(444,3)-k(15)*y(444);
jac(444,444)=jac(444,444)-k(15)*y(3);
jac(469,3)=jac(469,3)+k(15)*y(444);
jac(469,444)=jac(469,444)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(444,444)=jac(444,444)-k(10);
jac(3,444)=jac(3,444)+k(10);
jac(468,444)=jac(468,444)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(444,444)=jac(444,444)-k(8);
jac(34,444)=jac(34,444)+k(8);
jac(131,444)=jac(131,444)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(444,444)=jac(444,444)-k(6);
jac(33,444)=jac(33,444)+k(6);
jac(441,444)=jac(441,444)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(446,446)=jac(446,446)-k(33);
jac(379,446)=jac(379,446)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(446,446)=jac(446,446)-k(33);
jac(445,446)=jac(445,446)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(446,446)=jac(446,446)-k(28);
jac(412,446)=jac(412,446)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(446,446)=jac(446,446)-k(27);
jac(402,446)=jac(402,446)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(446,446)=jac(446,446)-k(22);
jac(32,446)=jac(32,446)+k(22);
jac(384,446)=jac(384,446)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(446,446)=jac(446,446)-k(22);
jac(32,446)=jac(32,446)+k(22);
jac(405,446)=jac(405,446)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(446,446)=jac(446,446)-k(11);
jac(444,446)=jac(444,446)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(446,446)=jac(446,446)-k(10);
jac(3,446)=jac(3,446)+k(10);
jac(160,446)=jac(160,446)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(446);
jac(3,446)=jac(3,446)-k(9)*y(3);
jac(446,3)=jac(446,3)-k(9)*y(446);
jac(446,446)=jac(446,446)-k(9)*y(3);
jac(159,3)=jac(159,3)+k(9)*y(446);
jac(159,446)=jac(159,446)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(446,446)=jac(446,446)-k(8);
jac(28,446)=jac(28,446)+k(8);
jac(131,446)=jac(131,446)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(446,446)=jac(446,446)-k(6);
jac(27,446)=jac(27,446)+k(6);
jac(441,446)=jac(441,446)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(457,457)=jac(457,457)-k(34);
jac(467,457)=jac(467,457)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(457,457)=jac(457,457)-k(32);
jac(215,457)=jac(215,457)+k(32);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(457,457)=jac(457,457)-k(23);
jac(400,457)=jac(400,457)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(457,457)=jac(457,457)-k(22);
jac(182,457)=jac(182,457)+k(22);
jac(149,457)=jac(149,457)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(457);
jac(197,457)=jac(197,457)-k(21)*y(197);
jac(457,197)=jac(457,197)-k(21)*y(457);
jac(457,457)=jac(457,457)-k(21)*y(197);
jac(193,197)=jac(193,197)+k(21)*y(457);
jac(193,457)=jac(193,457)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(457);
jac(32,457)=jac(32,457)-k(21)*y(32);
jac(457,32)=jac(457,32)-k(21)*y(457);
jac(457,457)=jac(457,457)-k(21)*y(32);
jac(448,32)=jac(448,32)+k(21)*y(457);
jac(448,457)=jac(448,457)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(457);
jac(2,457)=jac(2,457)-k(21)*y(2);
jac(457,2)=jac(457,2)-k(21)*y(457);
jac(457,457)=jac(457,457)-k(21)*y(2);
jac(460,2)=jac(460,2)+k(21)*y(457);
jac(460,457)=jac(460,457)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(457,457)=jac(457,457)-k(10);
jac(3,457)=jac(3,457)+k(10);
jac(189,457)=jac(189,457)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(457,457)=jac(457,457)-k(10);
jac(3,457)=jac(3,457)+k(10);
jac(396,457)=jac(396,457)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(457,457)=jac(457,457)-k(8);
jac(48,457)=jac(48,457)+k(8);
jac(185,457)=jac(185,457)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(457,457)=jac(457,457)-k(6);
jac(47,457)=jac(47,457)+k(6);
jac(184,457)=jac(184,457)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(458,458)=jac(458,458)-k(33);
jac(461,458)=jac(461,458)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(458,458)=jac(458,458)-k(32);
jac(104,458)=jac(104,458)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(458,458)=jac(458,458)-k(22);
jac(32,458)=jac(32,458)+k(22);
jac(149,458)=jac(149,458)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(458);
jac(197,458)=jac(197,458)-k(21)*y(197);
jac(458,197)=jac(458,197)-k(21)*y(458);
jac(458,458)=jac(458,458)-k(21)*y(197);
jac(459,197)=jac(459,197)+k(21)*y(458);
jac(459,458)=jac(459,458)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(458);
jac(32,458)=jac(32,458)-k(21)*y(32);
jac(458,32)=jac(458,32)-k(21)*y(458);
jac(458,458)=jac(458,458)-k(21)*y(32);
jac(159,32)=jac(159,32)+k(21)*y(458);
jac(159,458)=jac(159,458)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(458);
jac(2,458)=jac(2,458)-k(21)*y(2);
jac(458,2)=jac(458,2)-k(21)*y(458);
jac(458,458)=jac(458,458)-k(21)*y(2);
jac(165,2)=jac(165,2)+k(21)*y(458);
jac(165,458)=jac(165,458)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(458,458)=jac(458,458)-k(10);
jac(3,458)=jac(3,458)+k(10);
jac(153,458)=jac(153,458)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(458,458)=jac(458,458)-k(10);
jac(3,458)=jac(3,458)+k(10);
jac(405,458)=jac(405,458)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(458,458)=jac(458,458)-k(8);
jac(126,458)=jac(126,458)+k(8);
jac(131,458)=jac(131,458)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(458,458)=jac(458,458)-k(6);
jac(48,458)=jac(48,458)+k(6);
jac(441,458)=jac(441,458)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(467,467)=jac(467,467)-k(32);
jac(23,467)=jac(23,467)+k(32);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(467,467)=jac(467,467)-k(23);
jac(155,467)=jac(155,467)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(467,467)=jac(467,467)-k(22);
jac(2,467)=jac(2,467)+k(22);
jac(149,467)=jac(149,467)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(467);
jac(197,467)=jac(197,467)-k(21)*y(197);
jac(467,197)=jac(467,197)-k(21)*y(467);
jac(467,467)=jac(467,467)-k(21)*y(197);
jac(466,197)=jac(466,197)+k(21)*y(467);
jac(466,467)=jac(466,467)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(467);
jac(32,467)=jac(32,467)-k(21)*y(32);
jac(467,32)=jac(467,32)-k(21)*y(467);
jac(467,467)=jac(467,467)-k(21)*y(32);
jac(165,32)=jac(165,32)+k(21)*y(467);
jac(165,467)=jac(165,467)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(467);
jac(2,467)=jac(2,467)-k(21)*y(2);
jac(467,2)=jac(467,2)-k(21)*y(467);
jac(467,467)=jac(467,467)-k(21)*y(2);
jac(171,2)=jac(171,2)+k(21)*y(467);
jac(171,467)=jac(171,467)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(467,467)=jac(467,467)-k(10);
jac(3,467)=jac(3,467)+k(10);
jac(145,467)=jac(145,467)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(467,467)=jac(467,467)-k(10);
jac(3,467)=jac(3,467)+k(10);
jac(388,467)=jac(388,467)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(467,467)=jac(467,467)-k(8);
jac(48,467)=jac(48,467)+k(8);
jac(141,467)=jac(141,467)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(467,467)=jac(467,467)-k(6);
jac(47,467)=jac(47,467)+k(6);
jac(140,467)=jac(140,467)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(466,466)=jac(466,466)-k(34);
jac(165,466)=jac(165,466)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(466,466)=jac(466,466)-k(33);
jac(178,466)=jac(178,466)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(466,466)=jac(466,466)-k(29);
jac(193,466)=jac(193,466)+k(29);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(466,466)=jac(466,466)-k(23);
jac(465,466)=jac(465,466)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(466,466)=jac(466,466)-k(22);
jac(197,466)=jac(197,466)+k(22);
jac(467,466)=jac(467,466)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(466,466)=jac(466,466)-k(22);
jac(2,466)=jac(2,466)+k(22);
jac(408,466)=jac(408,466)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(466,466)=jac(466,466)-k(10);
jac(3,466)=jac(3,466)+k(10);
jac(169,466)=jac(169,466)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(466,466)=jac(466,466)-k(10);
jac(3,466)=jac(3,466)+k(10);
jac(411,466)=jac(411,466)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(466,466)=jac(466,466)-k(8);
jac(335,466)=jac(335,466)+k(8);
jac(141,466)=jac(141,466)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(466,466)=jac(466,466)-k(6);
jac(140,466)=jac(140,466)+k(6);
jac(407,466)=jac(407,466)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(465,465)=jac(465,465)-k(34);
jac(159,465)=jac(159,465)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(465,465)=jac(465,465)-k(33);
jac(365,465)=jac(365,465)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(465,465)=jac(465,465)-k(33);
jac(466,465)=jac(466,465)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(465,465)=jac(465,465)-k(29);
jac(464,465)=jac(464,465)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(465,465)=jac(465,465)-k(22);
jac(197,465)=jac(197,465)+k(22);
jac(155,465)=jac(155,465)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(465,465)=jac(465,465)-k(22);
jac(32,465)=jac(32,465)+k(22);
jac(408,465)=jac(408,465)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(465,465)=jac(465,465)-k(10);
jac(3,465)=jac(3,465)+k(10);
jac(163,465)=jac(163,465)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(465,465)=jac(465,465)-k(10);
jac(3,465)=jac(3,465)+k(10);
jac(412,465)=jac(412,465)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(465,465)=jac(465,465)-k(8);
jac(335,465)=jac(335,465)+k(8);
jac(132,465)=jac(132,465)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(465,465)=jac(465,465)-k(6);
jac(131,465)=jac(131,465)+k(6);
jac(407,465)=jac(407,465)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(464,464)=jac(464,464)-k(34);
jac(459,464)=jac(459,464)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(464,464)=jac(464,464)-k(34);
jac(465,464)=jac(465,464)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(464,464)=jac(464,464)-k(33);
jac(368,464)=jac(368,464)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(464,464)=jac(464,464)-k(33);
jac(193,464)=jac(193,464)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(464,464)=jac(464,464)-k(22);
jac(197,464)=jac(197,464)+k(22);
jac(400,464)=jac(400,464)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(464,464)=jac(464,464)-k(22);
jac(197,464)=jac(197,464)+k(22);
jac(408,464)=jac(408,464)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(464,464)=jac(464,464)-k(10);
jac(3,464)=jac(3,464)+k(10);
jac(370,464)=jac(370,464)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) 

jac(464,464)=jac(464,464)-k(10);
jac(3,464)=jac(3,464)+k(10);
jac(403,464)=jac(403,464)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(464,464)=jac(464,464)-k(8);
jac(335,464)=jac(335,464)+k(8);
jac(336,464)=jac(336,464)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(464,464)=jac(464,464)-k(6);
jac(335,464)=jac(335,464)+k(6);
jac(407,464)=jac(407,464)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(459,459)=jac(459,459)-k(34);
jac(159,459)=jac(159,459)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(459,459)=jac(459,459)-k(33);
jac(372,459)=jac(372,459)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(459,459)=jac(459,459)-k(33);
jac(448,459)=jac(448,459)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(459,459)=jac(459,459)-k(30);
jac(464,459)=jac(464,459)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(459,459)=jac(459,459)-k(22);
jac(32,459)=jac(32,459)+k(22);
jac(400,459)=jac(400,459)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(459,459)=jac(459,459)-k(22);
jac(197,459)=jac(197,459)+k(22);
jac(458,459)=jac(458,459)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(459,459)=jac(459,459)-k(10);
jac(3,459)=jac(3,459)+k(10);
jac(162,459)=jac(162,459)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(459,459)=jac(459,459)-k(10);
jac(3,459)=jac(3,459)+k(10);
jac(402,459)=jac(402,459)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(459,459)=jac(459,459)-k(8);
jac(131,459)=jac(131,459)+k(8);
jac(336,459)=jac(336,459)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(459,459)=jac(459,459)-k(6);
jac(335,459)=jac(335,459)+k(6);
jac(441,459)=jac(441,459)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(463,463)=jac(463,463)-k(32);
jac(10,463)=jac(10,463)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(463);
jac(197,463)=jac(197,463)-k(21)*y(197);
jac(463,197)=jac(463,197)-k(21)*y(463);
jac(463,463)=jac(463,463)-k(21)*y(197);
jac(414,197)=jac(414,197)+k(21)*y(463);
jac(414,463)=jac(414,463)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(463);
jac(32,463)=jac(32,463)-k(21)*y(32);
jac(463,32)=jac(463,32)-k(21)*y(463);
jac(463,463)=jac(463,463)-k(21)*y(32);
jac(104,32)=jac(104,32)+k(21)*y(463);
jac(104,463)=jac(104,463)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(463);
jac(2,463)=jac(2,463)-k(21)*y(2);
jac(463,2)=jac(463,2)-k(21)*y(463);
jac(463,463)=jac(463,463)-k(21)*y(2);
jac(462,2)=jac(462,2)+k(21)*y(463);
jac(462,463)=jac(462,463)+k(21)*y(2);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(463,463)=jac(463,463)-k(13);
jac(149,463)=jac(149,463)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) 

jac(463,463)=jac(463,463)-k(10);
jac(3,463)=jac(3,463)+k(10);
jac(100,463)=jac(100,463)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) 

jac(463,463)=jac(463,463)-k(10);
jac(3,463)=jac(3,463)+k(10);
jac(99,463)=jac(99,463)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(463,463)=jac(463,463)-k(8);
jac(48,463)=jac(48,463)+k(8);
jac(8,463)=jac(8,463)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(463,463)=jac(463,463)-k(6);
jac(7,463)=jac(7,463)+k(6);
jac(47,463)=jac(47,463)+k(6);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(462,462)=jac(462,462)-k(24);
jac(104,462)=jac(104,462)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(462,462)=jac(462,462)-k(22);
jac(2,462)=jac(2,462)+k(22);
jac(463,462)=jac(463,462)+k(22);

% rule    : R5
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(462,462)=jac(462,462)-k(13);
jac(461,462)=jac(461,462)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(462,462)=jac(462,462)-k(10);
jac(3,462)=jac(3,462)+k(10);
jac(96,462)=jac(96,462)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(462,462)=jac(462,462)-k(10);
jac(3,462)=jac(3,462)+k(10);
jac(95,462)=jac(95,462)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(462,462)=jac(462,462)-k(8);
jac(8,462)=jac(8,462)+k(8);
jac(140,462)=jac(140,462)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(462,462)=jac(462,462)-k(6);
jac(7,462)=jac(7,462)+k(6);
jac(380,462)=jac(380,462)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(461,461)=jac(461,461)-k(32);
jac(462,461)=jac(462,461)+k(32);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(461,461)=jac(461,461)-k(24);
jac(458,461)=jac(458,461)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(461,461)=jac(461,461)-k(22);
jac(2,461)=jac(2,461)+k(22);
jac(149,461)=jac(149,461)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(461);
jac(197,461)=jac(197,461)-k(21)*y(197);
jac(461,197)=jac(461,197)-k(21)*y(461);
jac(461,461)=jac(461,461)-k(21)*y(197);
jac(372,197)=jac(372,197)+k(21)*y(461);
jac(372,461)=jac(372,461)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(461);
jac(32,461)=jac(32,461)-k(21)*y(32);
jac(461,32)=jac(461,32)-k(21)*y(461);
jac(461,461)=jac(461,461)-k(21)*y(32);
jac(381,32)=jac(381,32)+k(21)*y(461);
jac(381,461)=jac(381,461)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(461);
jac(2,461)=jac(2,461)-k(21)*y(2);
jac(461,2)=jac(461,2)-k(21)*y(461);
jac(461,461)=jac(461,461)-k(21)*y(2);
jac(171,2)=jac(171,2)+k(21)*y(461);
jac(171,461)=jac(171,461)+k(21)*y(2);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(461,461)=jac(461,461)-k(10);
jac(3,461)=jac(3,461)+k(10);
jac(152,461)=jac(152,461)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(461,461)=jac(461,461)-k(10);
jac(3,461)=jac(3,461)+k(10);
jac(383,461)=jac(383,461)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(461,461)=jac(461,461)-k(8);
jac(126,461)=jac(126,461)+k(8);
jac(140,461)=jac(140,461)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(461,461)=jac(461,461)-k(6);
jac(48,461)=jac(48,461)+k(6);
jac(380,461)=jac(380,461)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(460,460)=jac(460,460)-k(34);
jac(171,460)=jac(171,460)+k(34);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(460,460)=jac(460,460)-k(30);
jac(186,460)=jac(186,460)+k(30);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(460,460)=jac(460,460)-k(24);
jac(448,460)=jac(448,460)+k(24);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(460,460)=jac(460,460)-k(23);
jac(372,460)=jac(372,460)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(460,460)=jac(460,460)-k(22);
jac(2,460)=jac(2,460)+k(22);
jac(457,460)=jac(457,460)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(460,460)=jac(460,460)-k(22);
jac(182,460)=jac(182,460)+k(22);
jac(461,460)=jac(461,460)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(460,460)=jac(460,460)-k(10);
jac(3,460)=jac(3,460)+k(10);
jac(174,460)=jac(174,460)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) 

jac(460,460)=jac(460,460)-k(10);
jac(3,460)=jac(3,460)+k(10);
jac(389,460)=jac(389,460)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(460,460)=jac(460,460)-k(8);
jac(140,460)=jac(140,460)+k(8);
jac(185,460)=jac(185,460)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(460,460)=jac(460,460)-k(6);
jac(184,460)=jac(184,460)+k(6);
jac(380,460)=jac(380,460)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(448,448)=jac(448,448)-k(34);
jac(165,448)=jac(165,448)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(448,448)=jac(448,448)-k(33);
jac(460,448)=jac(460,448)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(448,448)=jac(448,448)-k(30);
jac(193,448)=jac(193,448)+k(30);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(448,448)=jac(448,448)-k(23);
jac(459,448)=jac(459,448)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(448,448)=jac(448,448)-k(22);
jac(182,448)=jac(182,448)+k(22);
jac(458,448)=jac(458,448)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(448,448)=jac(448,448)-k(22);
jac(32,448)=jac(32,448)+k(22);
jac(457,448)=jac(457,448)+k(22);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(448,448)=jac(448,448)-k(10);
jac(3,448)=jac(3,448)+k(10);
jac(447,448)=jac(447,448)+k(10);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) 

jac(448,448)=jac(448,448)-k(10);
jac(3,448)=jac(3,448)+k(10);
jac(168,448)=jac(168,448)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(448,448)=jac(448,448)-k(8);
jac(131,448)=jac(131,448)+k(8);
jac(185,448)=jac(185,448)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(448,448)=jac(448,448)-k(6);
jac(184,448)=jac(184,448)+k(6);
jac(441,448)=jac(441,448)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(456,456)=jac(456,456)-k(34);
jac(160,456)=jac(160,456)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(456,456)=jac(456,456)-k(33);
jac(366,456)=jac(366,456)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(456,456)=jac(456,456)-k(33);
jac(453,456)=jac(453,456)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(456,456)=jac(456,456)-k(29);
jac(455,456)=jac(455,456)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(456,456)=jac(456,456)-k(22);
jac(197,456)=jac(197,456)+k(22);
jac(156,456)=jac(156,456)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(456,456)=jac(456,456)-k(22);
jac(32,456)=jac(32,456)+k(22);
jac(409,456)=jac(409,456)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(456);
jac(3,456)=jac(3,456)-k(9)*y(3);
jac(456,3)=jac(456,3)-k(9)*y(456);
jac(456,456)=jac(456,456)-k(9)*y(3);
jac(163,3)=jac(163,3)+k(9)*y(456);
jac(163,456)=jac(163,456)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(456);
jac(3,456)=jac(3,456)-k(9)*y(3);
jac(456,3)=jac(456,3)-k(9)*y(456);
jac(456,456)=jac(456,456)-k(9)*y(3);
jac(412,3)=jac(412,3)+k(9)*y(456);
jac(412,456)=jac(412,456)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(456,456)=jac(456,456)-k(8);
jac(192,456)=jac(192,456)+k(8);
jac(28,456)=jac(28,456)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(456,456)=jac(456,456)-k(6);
jac(27,456)=jac(27,456)+k(6);
jac(191,456)=jac(191,456)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(455,455)=jac(455,455)-k(34);
jac(454,455)=jac(454,455)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(455,455)=jac(455,455)-k(34);
jac(456,455)=jac(456,455)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(455,455)=jac(455,455)-k(33);
jac(369,455)=jac(369,455)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(455,455)=jac(455,455)-k(33);
jac(194,455)=jac(194,455)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(455,455)=jac(455,455)-k(22);
jac(197,455)=jac(197,455)+k(22);
jac(376,455)=jac(376,455)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(455,455)=jac(455,455)-k(22);
jac(197,455)=jac(197,455)+k(22);
jac(409,455)=jac(409,455)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(455);
jac(3,455)=jac(3,455)-k(9)*y(3);
jac(455,3)=jac(455,3)-k(9)*y(455);
jac(455,455)=jac(455,455)-k(9)*y(3);
jac(370,3)=jac(370,3)+k(9)*y(455);
jac(370,455)=jac(370,455)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(455);
jac(3,455)=jac(3,455)-k(9)*y(3);
jac(455,3)=jac(455,3)-k(9)*y(455);
jac(455,455)=jac(455,455)-k(9)*y(3);
jac(403,3)=jac(403,3)+k(9)*y(455);
jac(403,455)=jac(403,455)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(455,455)=jac(455,455)-k(8);
jac(192,455)=jac(192,455)+k(8);
jac(209,455)=jac(209,455)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(455,455)=jac(455,455)-k(6);
jac(192,455)=jac(192,455)+k(6);
jac(191,455)=jac(191,455)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(454,454)=jac(454,454)-k(34);
jac(160,454)=jac(160,454)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(454,454)=jac(454,454)-k(33);
jac(449,454)=jac(449,454)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(454,454)=jac(454,454)-k(33);
jac(373,454)=jac(373,454)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(454,454)=jac(454,454)-k(30);
jac(455,454)=jac(455,454)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(454,454)=jac(454,454)-k(22);
jac(197,454)=jac(197,454)+k(22);
jac(450,454)=jac(450,454)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(454,454)=jac(454,454)-k(22);
jac(32,454)=jac(32,454)+k(22);
jac(376,454)=jac(376,454)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(454);
jac(3,454)=jac(3,454)-k(9)*y(3);
jac(454,3)=jac(454,3)-k(9)*y(454);
jac(454,454)=jac(454,454)-k(9)*y(3);
jac(402,3)=jac(402,3)+k(9)*y(454);
jac(402,454)=jac(402,454)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(454);
jac(3,454)=jac(3,454)-k(9)*y(3);
jac(454,3)=jac(454,3)-k(9)*y(454);
jac(454,454)=jac(454,454)-k(9)*y(3);
jac(162,3)=jac(162,3)+k(9)*y(454);
jac(162,454)=jac(162,454)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(454,454)=jac(454,454)-k(8);
jac(27,454)=jac(27,454)+k(8);
jac(209,454)=jac(209,454)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(454,454)=jac(454,454)-k(6);
jac(192,454)=jac(192,454)+k(6);
jac(86,454)=jac(86,454)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(450,450)=jac(450,450)-k(33);
jac(375,450)=jac(375,450)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(450,450)=jac(450,450)-k(32);
jac(101,450)=jac(101,450)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(450,450)=jac(450,450)-k(22);
jac(32,450)=jac(32,450)+k(22);
jac(150,450)=jac(150,450)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(450);
jac(197,450)=jac(197,450)-k(21)*y(197);
jac(450,197)=jac(450,197)-k(21)*y(450);
jac(450,450)=jac(450,450)-k(21)*y(197);
jac(454,197)=jac(454,197)+k(21)*y(450);
jac(454,450)=jac(454,450)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(450);
jac(32,450)=jac(32,450)-k(21)*y(32);
jac(450,32)=jac(450,32)-k(21)*y(450);
jac(450,450)=jac(450,450)-k(21)*y(32);
jac(160,32)=jac(160,32)+k(21)*y(450);
jac(160,450)=jac(160,450)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(450);
jac(2,450)=jac(2,450)-k(21)*y(2);
jac(450,2)=jac(450,2)-k(21)*y(450);
jac(450,450)=jac(450,450)-k(21)*y(2);
jac(166,2)=jac(166,2)+k(21)*y(450);
jac(166,450)=jac(166,450)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(450);
jac(3,450)=jac(3,450)-k(9)*y(3);
jac(450,3)=jac(450,3)-k(9)*y(450);
jac(450,450)=jac(450,450)-k(9)*y(3);
jac(153,3)=jac(153,3)+k(9)*y(450);
jac(153,450)=jac(153,450)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(450);
jac(3,450)=jac(3,450)-k(9)*y(3);
jac(450,3)=jac(450,3)-k(9)*y(450);
jac(450,450)=jac(450,450)-k(9)*y(3);
jac(405,3)=jac(405,3)+k(9)*y(450);
jac(405,450)=jac(405,450)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(450,450)=jac(450,450)-k(8);
jac(16,450)=jac(16,450)+k(8);
jac(27,450)=jac(27,450)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

jac(450,450)=jac(450,450)-k(6);
jac(15,450)=jac(15,450)+k(6);
jac(86,450)=jac(86,450)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(453,453)=jac(453,453)-k(34);
jac(166,453)=jac(166,453)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(453,453)=jac(453,453)-k(33);
jac(179,453)=jac(179,453)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(453,453)=jac(453,453)-k(29);
jac(194,453)=jac(194,453)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(453,453)=jac(453,453)-k(22);
jac(197,453)=jac(197,453)+k(22);
jac(452,453)=jac(452,453)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(453,453)=jac(453,453)-k(22);
jac(2,453)=jac(2,453)+k(22);
jac(409,453)=jac(409,453)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(453);
jac(3,453)=jac(3,453)-k(9)*y(3);
jac(453,3)=jac(453,3)-k(9)*y(453);
jac(453,453)=jac(453,453)-k(9)*y(3);
jac(169,3)=jac(169,3)+k(9)*y(453);
jac(169,453)=jac(169,453)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(453);
jac(3,453)=jac(3,453)-k(9)*y(3);
jac(453,3)=jac(453,3)-k(9)*y(453);
jac(453,453)=jac(453,453)-k(9)*y(3);
jac(411,3)=jac(411,3)+k(9)*y(453);
jac(411,453)=jac(411,453)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(453,453)=jac(453,453)-k(8);
jac(192,453)=jac(192,453)+k(8);
jac(22,453)=jac(22,453)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(453,453)=jac(453,453)-k(6);
jac(21,453)=jac(21,453)+k(6);
jac(191,453)=jac(191,453)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(452,452)=jac(452,452)-k(32);
jac(24,452)=jac(24,452)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(452,452)=jac(452,452)-k(22);
jac(2,452)=jac(2,452)+k(22);
jac(150,452)=jac(150,452)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(452);
jac(197,452)=jac(197,452)-k(21)*y(197);
jac(452,197)=jac(452,197)-k(21)*y(452);
jac(452,452)=jac(452,452)-k(21)*y(197);
jac(453,197)=jac(453,197)+k(21)*y(452);
jac(453,452)=jac(453,452)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(452);
jac(32,452)=jac(32,452)-k(21)*y(32);
jac(452,32)=jac(452,32)-k(21)*y(452);
jac(452,452)=jac(452,452)-k(21)*y(32);
jac(166,32)=jac(166,32)+k(21)*y(452);
jac(166,452)=jac(166,452)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(452);
jac(2,452)=jac(2,452)-k(21)*y(2);
jac(452,2)=jac(452,2)-k(21)*y(452);
jac(452,452)=jac(452,452)-k(21)*y(2);
jac(172,2)=jac(172,2)+k(21)*y(452);
jac(172,452)=jac(172,452)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(452);
jac(3,452)=jac(3,452)-k(9)*y(3);
jac(452,3)=jac(452,3)-k(9)*y(452);
jac(452,452)=jac(452,452)-k(9)*y(3);
jac(145,3)=jac(145,3)+k(9)*y(452);
jac(145,452)=jac(145,452)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(452);
jac(3,452)=jac(3,452)-k(9)*y(3);
jac(452,3)=jac(452,3)-k(9)*y(452);
jac(452,452)=jac(452,452)-k(9)*y(3);
jac(388,3)=jac(388,3)+k(9)*y(452);
jac(388,452)=jac(388,452)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY) 

jac(452,452)=jac(452,452)-k(8);
jac(15,452)=jac(15,452)+k(8);
jac(22,452)=jac(22,452)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(452,452)=jac(452,452)-k(6);
jac(98,452)=jac(98,452)+k(6);
jac(21,452)=jac(21,452)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~Y) 

jac(451,451)=jac(451,451)-k(34);
jac(452,451)=jac(452,451)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(451,451)=jac(451,451)-k(32);
jac(216,451)=jac(216,451)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(451,451)=jac(451,451)-k(22);
jac(182,451)=jac(182,451)+k(22);
jac(150,451)=jac(150,451)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(451);
jac(197,451)=jac(197,451)-k(21)*y(197);
jac(451,197)=jac(451,197)-k(21)*y(451);
jac(451,451)=jac(451,451)-k(21)*y(197);
jac(194,197)=jac(194,197)+k(21)*y(451);
jac(194,451)=jac(194,451)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(451);
jac(32,451)=jac(32,451)-k(21)*y(32);
jac(451,32)=jac(451,32)-k(21)*y(451);
jac(451,451)=jac(451,451)-k(21)*y(32);
jac(449,32)=jac(449,32)+k(21)*y(451);
jac(449,451)=jac(449,451)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(451);
jac(2,451)=jac(2,451)-k(21)*y(2);
jac(451,2)=jac(451,2)-k(21)*y(451);
jac(451,451)=jac(451,451)-k(21)*y(2);
jac(377,2)=jac(377,2)+k(21)*y(451);
jac(377,451)=jac(377,451)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(451);
jac(3,451)=jac(3,451)-k(9)*y(3);
jac(451,3)=jac(451,3)-k(9)*y(451);
jac(451,451)=jac(451,451)-k(9)*y(3);
jac(189,3)=jac(189,3)+k(9)*y(451);
jac(189,451)=jac(189,451)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(451);
jac(3,451)=jac(3,451)-k(9)*y(3);
jac(451,3)=jac(451,3)-k(9)*y(451);
jac(451,451)=jac(451,451)-k(9)*y(3);
jac(396,3)=jac(396,3)+k(9)*y(451);
jac(396,451)=jac(396,451)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY) 

jac(451,451)=jac(451,451)-k(8);
jac(15,451)=jac(15,451)+k(8);
jac(214,451)=jac(214,451)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(451,451)=jac(451,451)-k(6);
jac(98,451)=jac(98,451)+k(6);
jac(177,451)=jac(177,451)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) 

jac(449,449)=jac(449,449)-k(34);
jac(166,449)=jac(166,449)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(449,449)=jac(449,449)-k(33);
jac(377,449)=jac(377,449)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) 

jac(449,449)=jac(449,449)-k(30);
jac(194,449)=jac(194,449)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(449,449)=jac(449,449)-k(22);
jac(32,449)=jac(32,449)+k(22);
jac(451,449)=jac(451,449)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(449,449)=jac(449,449)-k(22);
jac(182,449)=jac(182,449)+k(22);
jac(450,449)=jac(450,449)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(449);
jac(3,449)=jac(3,449)-k(9)*y(3);
jac(449,3)=jac(449,3)-k(9)*y(449);
jac(449,449)=jac(449,449)-k(9)*y(3);
jac(168,3)=jac(168,3)+k(9)*y(449);
jac(168,449)=jac(168,449)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(449);
jac(3,449)=jac(3,449)-k(9)*y(3);
jac(449,3)=jac(449,3)-k(9)*y(449);
jac(449,449)=jac(449,449)-k(9)*y(3);
jac(447,3)=jac(447,3)+k(9)*y(449);
jac(447,449)=jac(447,449)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(449,449)=jac(449,449)-k(8);
jac(27,449)=jac(27,449)+k(8);
jac(214,449)=jac(214,449)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(449,449)=jac(449,449)-k(6);
jac(177,449)=jac(177,449)+k(6);
jac(86,449)=jac(86,449)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(447,447)=jac(447,447)-k(34);
jac(445,447)=jac(445,447)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(447,447)=jac(447,447)-k(33);
jac(389,447)=jac(389,447)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(447,447)=jac(447,447)-k(30);
jac(435,447)=jac(435,447)+k(30);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(447,447)=jac(447,447)-k(23);
jac(402,447)=jac(402,447)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(447,447)=jac(447,447)-k(22);
jac(32,447)=jac(32,447)+k(22);
jac(396,447)=jac(396,447)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(447,447)=jac(447,447)-k(22);
jac(182,447)=jac(182,447)+k(22);
jac(405,447)=jac(405,447)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(447,447)=jac(447,447)-k(11);
jac(432,447)=jac(432,447)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(447,447)=jac(447,447)-k(10);
jac(3,447)=jac(3,447)+k(10);
jac(449,447)=jac(449,447)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(447);
jac(3,447)=jac(3,447)-k(9)*y(3);
jac(447,3)=jac(447,3)-k(9)*y(447);
jac(447,447)=jac(447,447)-k(9)*y(3);
jac(448,3)=jac(448,3)+k(9)*y(447);
jac(448,447)=jac(448,447)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(447,447)=jac(447,447)-k(8);
jac(214,447)=jac(214,447)+k(8);
jac(131,447)=jac(131,447)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(447,447)=jac(447,447)-k(6);
jac(177,447)=jac(177,447)+k(6);
jac(441,447)=jac(441,447)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(445,445)=jac(445,445)-k(33);
jac(386,445)=jac(386,445)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(445,445)=jac(445,445)-k(28);
jac(411,445)=jac(411,445)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(445,445)=jac(445,445)-k(27);
jac(447,445)=jac(447,445)+k(27);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(445,445)=jac(445,445)-k(23);
jac(446,445)=jac(446,445)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(445,445)=jac(445,445)-k(22);
jac(32,445)=jac(32,445)+k(22);
jac(388,445)=jac(388,445)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(445,445)=jac(445,445)-k(22);
jac(2,445)=jac(2,445)+k(22);
jac(405,445)=jac(405,445)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(445,445)=jac(445,445)-k(11);
jac(440,445)=jac(440,445)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(445,445)=jac(445,445)-k(10);
jac(3,445)=jac(3,445)+k(10);
jac(166,445)=jac(166,445)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(445);
jac(3,445)=jac(3,445)-k(9)*y(3);
jac(445,3)=jac(445,3)-k(9)*y(445);
jac(445,445)=jac(445,445)-k(9)*y(3);
jac(165,3)=jac(165,3)+k(9)*y(445);
jac(165,445)=jac(165,445)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(445,445)=jac(445,445)-k(8);
jac(22,445)=jac(22,445)+k(8);
jac(131,445)=jac(131,445)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(445,445)=jac(445,445)-k(6);
jac(21,445)=jac(21,445)+k(6);
jac(441,445)=jac(441,445)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(440,440)=jac(440,440)-k(33);
jac(387,440)=jac(387,440)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(440,440)=jac(440,440)-k(31);
jac(445,440)=jac(445,440)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(440,440)=jac(440,440)-k(28);
jac(436,440)=jac(436,440)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(440,440)=jac(440,440)-k(27);
jac(432,440)=jac(432,440)+k(27);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(440,440)=jac(440,440)-k(23);
jac(444,440)=jac(444,440)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(440,440)=jac(440,440)-k(22);
jac(32,440)=jac(32,440)+k(22);
jac(50,440)=jac(50,440)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(440,440)=jac(440,440)-k(22);
jac(2,440)=jac(2,440)+k(22);
jac(443,440)=jac(443,440)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(440);
jac(3,440)=jac(3,440)-k(15)*y(3);
jac(440,3)=jac(440,3)-k(15)*y(440);
jac(440,440)=jac(440,440)-k(15)*y(3);
jac(52,3)=jac(52,3)+k(15)*y(440);
jac(52,440)=jac(52,440)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(440,440)=jac(440,440)-k(10);
jac(3,440)=jac(3,440)+k(10);
jac(442,440)=jac(442,440)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(440,440)=jac(440,440)-k(8);
jac(437,440)=jac(437,440)+k(8);
jac(131,440)=jac(131,440)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(440,440)=jac(440,440)-k(6);
jac(111,440)=jac(111,440)+k(6);
jac(441,440)=jac(441,440)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(436,436)=jac(436,436)-k(34);
jac(440,436)=jac(440,436)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(436,436)=jac(436,436)-k(33);
jac(392,436)=jac(392,436)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(436,436)=jac(436,436)-k(31);
jac(411,436)=jac(411,436)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(436,436)=jac(436,436)-k(29);
jac(433,436)=jac(433,436)+k(29);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(436,436)=jac(436,436)-k(23);
jac(439,436)=jac(439,436)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(436,436)=jac(436,436)-k(22);
jac(197,436)=jac(197,436)+k(22);
jac(50,436)=jac(50,436)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(436,436)=jac(436,436)-k(22);
jac(2,436)=jac(2,436)+k(22);
jac(410,436)=jac(410,436)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(436);
jac(3,436)=jac(3,436)-k(15)*y(3);
jac(436,3)=jac(436,3)-k(15)*y(436);
jac(436,436)=jac(436,436)-k(15)*y(3);
jac(207,3)=jac(207,3)+k(15)*y(436);
jac(207,436)=jac(207,436)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(436,436)=jac(436,436)-k(10);
jac(3,436)=jac(3,436)+k(10);
jac(438,436)=jac(438,436)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(436,436)=jac(436,436)-k(8);
jac(437,436)=jac(437,436)+k(8);
jac(335,436)=jac(335,436)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(436,436)=jac(436,436)-k(6);
jac(111,436)=jac(111,436)+k(6);
jac(407,436)=jac(407,436)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(433,433)=jac(433,433)-k(34);
jac(432,433)=jac(432,433)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(433,433)=jac(433,433)-k(34);
jac(436,433)=jac(436,433)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(433,433)=jac(433,433)-k(33);
jac(395,433)=jac(395,433)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(433,433)=jac(433,433)-k(31);
jac(435,433)=jac(435,433)+k(31);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(433,433)=jac(433,433)-k(23);
jac(434,433)=jac(434,433)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(433,433)=jac(433,433)-k(22);
jac(197,433)=jac(197,433)+k(22);
jac(430,433)=jac(430,433)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(433,433)=jac(433,433)-k(22);
jac(182,433)=jac(182,433)+k(22);
jac(410,433)=jac(410,433)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(433);
jac(3,433)=jac(3,433)-k(15)*y(3);
jac(433,3)=jac(433,3)-k(15)*y(433);
jac(433,433)=jac(433,433)-k(15)*y(3);
jac(289,3)=jac(289,3)+k(15)*y(433);
jac(289,433)=jac(289,433)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(433,433)=jac(433,433)-k(10);
jac(3,433)=jac(3,433)+k(10);
jac(290,433)=jac(290,433)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(433,433)=jac(433,433)-k(8);
jac(219,433)=jac(219,433)+k(8);
jac(335,433)=jac(335,433)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(433,433)=jac(433,433)-k(6);
jac(218,433)=jac(218,433)+k(6);
jac(407,433)=jac(407,433)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(430,430)=jac(430,430)-k(34);
jac(50,430)=jac(50,430)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(430,430)=jac(430,430)-k(32);
jac(217,430)=jac(217,430)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(430,430)=jac(430,430)-k(31);
jac(396,430)=jac(396,430)+k(31);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(430,430)=jac(430,430)-k(23);
jac(401,430)=jac(401,430)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(430,430)=jac(430,430)-k(22);
jac(182,430)=jac(182,430)+k(22);
jac(57,430)=jac(57,430)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(430);
jac(197,430)=jac(197,430)-k(21)*y(197);
jac(430,197)=jac(430,197)-k(21)*y(430);
jac(430,430)=jac(430,430)-k(21)*y(197);
jac(433,197)=jac(433,197)+k(21)*y(430);
jac(433,430)=jac(433,430)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(430);
jac(32,430)=jac(32,430)-k(21)*y(32);
jac(430,32)=jac(430,32)-k(21)*y(430);
jac(430,430)=jac(430,430)-k(21)*y(32);
jac(432,32)=jac(432,32)+k(21)*y(430);
jac(432,430)=jac(432,430)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(430);
jac(2,430)=jac(2,430)-k(21)*y(2);
jac(430,2)=jac(430,2)-k(21)*y(430);
jac(430,430)=jac(430,430)-k(21)*y(2);
jac(429,2)=jac(429,2)+k(21)*y(430);
jac(429,430)=jac(429,430)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(430);
jac(3,430)=jac(3,430)-k(15)*y(3);
jac(430,3)=jac(430,3)-k(15)*y(430);
jac(430,430)=jac(430,430)-k(15)*y(3);
jac(225,3)=jac(225,3)+k(15)*y(430);
jac(225,430)=jac(225,430)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(430,430)=jac(430,430)-k(10);
jac(3,430)=jac(3,430)+k(10);
jac(431,430)=jac(431,430)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(430,430)=jac(430,430)-k(8);
jac(48,430)=jac(48,430)+k(8);
jac(219,430)=jac(219,430)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(430,430)=jac(430,430)-k(6);
jac(47,430)=jac(47,430)+k(6);
jac(218,430)=jac(218,430)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(429,429)=jac(429,429)-k(34);
jac(387,429)=jac(387,429)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(429,429)=jac(429,429)-k(31);
jac(389,429)=jac(389,429)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(429,429)=jac(429,429)-k(30);
jac(395,429)=jac(395,429)+k(30);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(429,429)=jac(429,429)-k(23);
jac(426,429)=jac(426,429)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(429,429)=jac(429,429)-k(22);
jac(2,429)=jac(2,429)+k(22);
jac(430,429)=jac(430,429)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(429,429)=jac(429,429)-k(22);
jac(182,429)=jac(182,429)+k(22);
jac(424,429)=jac(424,429)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(429);
jac(3,429)=jac(3,429)-k(15)*y(3);
jac(429,3)=jac(429,3)-k(15)*y(429);
jac(429,429)=jac(429,429)-k(15)*y(3);
jac(279,3)=jac(279,3)+k(15)*y(429);
jac(279,429)=jac(279,429)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(429,429)=jac(429,429)-k(10);
jac(3,429)=jac(3,429)+k(10);
jac(280,429)=jac(280,429)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(429,429)=jac(429,429)-k(8);
jac(219,429)=jac(219,429)+k(8);
jac(140,429)=jac(140,429)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(429,429)=jac(429,429)-k(6);
jac(218,429)=jac(218,429)+k(6);
jac(380,429)=jac(380,429)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(426,426)=jac(426,426)-k(34);
jac(382,426)=jac(382,426)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(426,426)=jac(426,426)-k(33);
jac(429,426)=jac(429,426)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(426,426)=jac(426,426)-k(31);
jac(374,426)=jac(374,426)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(426,426)=jac(426,426)-k(30);
jac(398,426)=jac(398,426)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(426,426)=jac(426,426)-k(22);
jac(197,426)=jac(197,426)+k(22);
jac(424,426)=jac(424,426)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(426,426)=jac(426,426)-k(22);
jac(2,426)=jac(2,426)+k(22);
jac(401,426)=jac(401,426)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(426);
jac(3,426)=jac(3,426)-k(15)*y(3);
jac(426,3)=jac(426,3)-k(15)*y(426);
jac(426,426)=jac(426,426)-k(15)*y(3);
jac(428,3)=jac(428,3)+k(15)*y(426);
jac(428,426)=jac(428,426)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(426,426)=jac(426,426)-k(10);
jac(3,426)=jac(3,426)+k(10);
jac(427,426)=jac(427,426)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(426,426)=jac(426,426)-k(8);
jac(319,426)=jac(319,426)+k(8);
jac(140,426)=jac(140,426)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(426,426)=jac(426,426)-k(6);
jac(257,426)=jac(257,426)+k(6);
jac(380,426)=jac(380,426)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(424,424)=jac(424,424)-k(32);
jac(92,424)=jac(92,424)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(424,424)=jac(424,424)-k(31);
jac(383,424)=jac(383,424)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(424,424)=jac(424,424)-k(22);
jac(2,424)=jac(2,424)+k(22);
jac(57,424)=jac(57,424)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(424);
jac(197,424)=jac(197,424)-k(21)*y(197);
jac(424,197)=jac(424,197)-k(21)*y(424);
jac(424,424)=jac(424,424)-k(21)*y(197);
jac(426,197)=jac(426,197)+k(21)*y(424);
jac(426,424)=jac(426,424)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(424);
jac(32,424)=jac(32,424)-k(21)*y(32);
jac(424,32)=jac(424,32)-k(21)*y(424);
jac(424,424)=jac(424,424)-k(21)*y(32);
jac(382,32)=jac(382,32)+k(21)*y(424);
jac(382,424)=jac(382,424)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(424);
jac(2,424)=jac(2,424)-k(21)*y(2);
jac(424,2)=jac(424,2)-k(21)*y(424);
jac(424,424)=jac(424,424)-k(21)*y(2);
jac(387,2)=jac(387,2)+k(21)*y(424);
jac(387,424)=jac(387,424)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(424);
jac(3,424)=jac(3,424)-k(15)*y(3);
jac(424,3)=jac(424,3)-k(15)*y(424);
jac(424,424)=jac(424,424)-k(15)*y(3);
jac(58,3)=jac(58,3)+k(15)*y(424);
jac(58,424)=jac(58,424)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(424,424)=jac(424,424)-k(10);
jac(3,424)=jac(3,424)+k(10);
jac(425,424)=jac(425,424)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(424,424)=jac(424,424)-k(8);
jac(315,424)=jac(315,424)+k(8);
jac(140,424)=jac(140,424)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

jac(424,424)=jac(424,424)-k(6);
jac(77,424)=jac(77,424)+k(6);
jac(380,424)=jac(380,424)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(420,420)=jac(420,420)-k(34);
jac(424,420)=jac(424,420)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(420,420)=jac(420,420)-k(32);
jac(419,420)=jac(419,420)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(420,420)=jac(420,420)-k(31);
jac(393,420)=jac(393,420)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(420,420)=jac(420,420)-k(22);
jac(182,420)=jac(182,420)+k(22);
jac(57,420)=jac(57,420)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(420);
jac(197,420)=jac(197,420)-k(21)*y(197);
jac(420,197)=jac(420,197)-k(21)*y(420);
jac(420,420)=jac(420,420)-k(21)*y(197);
jac(398,197)=jac(398,197)+k(21)*y(420);
jac(398,420)=jac(398,420)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(420);
jac(32,420)=jac(32,420)-k(21)*y(32);
jac(420,32)=jac(420,32)-k(21)*y(420);
jac(420,420)=jac(420,420)-k(21)*y(32);
jac(423,32)=jac(423,32)+k(21)*y(420);
jac(423,420)=jac(423,420)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(420);
jac(2,420)=jac(2,420)-k(21)*y(2);
jac(420,2)=jac(420,2)-k(21)*y(420);
jac(420,420)=jac(420,420)-k(21)*y(2);
jac(392,2)=jac(392,2)+k(21)*y(420);
jac(392,420)=jac(392,420)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(420);
jac(3,420)=jac(3,420)-k(15)*y(3);
jac(420,3)=jac(420,3)-k(15)*y(420);
jac(420,420)=jac(420,420)-k(15)*y(3);
jac(422,3)=jac(422,3)+k(15)*y(420);
jac(422,420)=jac(422,420)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(420,420)=jac(420,420)-k(10);
jac(3,420)=jac(3,420)+k(10);
jac(421,420)=jac(421,420)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(420,420)=jac(420,420)-k(8);
jac(315,420)=jac(315,420)+k(8);
jac(184,420)=jac(184,420)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

jac(420,420)=jac(420,420)-k(6);
jac(77,420)=jac(77,420)+k(6);
jac(391,420)=jac(391,420)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(419,419)=jac(419,419)-k(34);
jac(92,419)=jac(92,419)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(419,419)=jac(419,419)-k(31);
jac(416,419)=jac(416,419)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(419,419)=jac(419,419)-k(22);
jac(182,419)=jac(182,419)+k(22);
jac(64,419)=jac(64,419)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(419);
jac(3,419)=jac(3,419)-k(15)*y(3);
jac(419,3)=jac(419,3)-k(15)*y(419);
jac(419,419)=jac(419,419)-k(15)*y(3);
jac(298,3)=jac(298,3)+k(15)*y(419);
jac(298,419)=jac(298,419)+k(15)*y(3);

% rule    : R5
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(419,419)=jac(419,419)-k(13);
jac(420,419)=jac(420,419)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(419,419)=jac(419,419)-k(10);
jac(3,419)=jac(3,419)+k(10);
jac(299,419)=jac(299,419)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(419,419)=jac(419,419)-k(8);
jac(91,419)=jac(91,419)+k(8);
jac(184,419)=jac(184,419)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

jac(419,419)=jac(419,419)-k(6);
jac(72,419)=jac(72,419)+k(6);
jac(391,419)=jac(391,419)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(416,416)=jac(416,416)-k(34);
jac(95,416)=jac(95,416)+k(34);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(416,416)=jac(416,416)-k(22);
jac(182,416)=jac(182,416)+k(22);
jac(99,416)=jac(99,416)+k(22);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(416,416)=jac(416,416)-k(13);
jac(393,416)=jac(393,416)+k(13);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(416,416)=jac(416,416)-k(11);
jac(419,416)=jac(419,416)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(416,416)=jac(416,416)-k(10);
jac(3,416)=jac(3,416)+k(10);
jac(418,416)=jac(418,416)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(416);
jac(3,416)=jac(3,416)-k(9)*y(3);
jac(416,3)=jac(416,3)-k(9)*y(416);
jac(416,416)=jac(416,416)-k(9)*y(3);
jac(417,3)=jac(417,3)+k(9)*y(416);
jac(417,416)=jac(417,416)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(416,416)=jac(416,416)-k(8);
jac(5,416)=jac(5,416)+k(8);
jac(184,416)=jac(184,416)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~Y) 

jac(416,416)=jac(416,416)-k(6);
jac(1,416)=jac(1,416)+k(6);
jac(391,416)=jac(391,416)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(413,413)=jac(413,413)-k(34);
jac(102,413)=jac(102,413)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(413,413)=jac(413,413)-k(33);
jac(416,413)=jac(416,413)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(413,413)=jac(413,413)-k(22);
jac(197,413)=jac(197,413)+k(22);
jac(99,413)=jac(99,413)+k(22);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(413,413)=jac(413,413)-k(13);
jac(406,413)=jac(406,413)+k(13);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(413,413)=jac(413,413)-k(11);
jac(415,413)=jac(415,413)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(413,413)=jac(413,413)-k(10);
jac(3,413)=jac(3,413)+k(10);
jac(202,413)=jac(202,413)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(413);
jac(3,413)=jac(3,413)-k(9)*y(3);
jac(413,3)=jac(413,3)-k(9)*y(413);
jac(413,413)=jac(413,413)-k(9)*y(3);
jac(414,3)=jac(414,3)+k(9)*y(413);
jac(414,413)=jac(414,413)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(413,413)=jac(413,413)-k(8);
jac(5,413)=jac(5,413)+k(8);
jac(335,413)=jac(335,413)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

jac(413,413)=jac(413,413)-k(6);
jac(1,413)=jac(1,413)+k(6);
jac(407,413)=jac(407,413)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(406,406)=jac(406,406)-k(34);
jac(405,406)=jac(405,406)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(406,406)=jac(406,406)-k(33);
jac(393,406)=jac(393,406)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(406,406)=jac(406,406)-k(32);
jac(413,406)=jac(413,406)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(406,406)=jac(406,406)-k(22);
jac(197,406)=jac(197,406)+k(22);
jac(404,406)=jac(404,406)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(406);
jac(197,406)=jac(197,406)-k(21)*y(197);
jac(406,197)=jac(406,197)-k(21)*y(406);
jac(406,406)=jac(406,406)-k(21)*y(197);
jac(403,197)=jac(403,197)+k(21)*y(406);
jac(403,406)=jac(403,406)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(406);
jac(32,406)=jac(32,406)-k(21)*y(32);
jac(406,32)=jac(406,32)-k(21)*y(406);
jac(406,406)=jac(406,406)-k(21)*y(32);
jac(412,32)=jac(412,32)+k(21)*y(406);
jac(412,406)=jac(412,406)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(406);
jac(2,406)=jac(2,406)-k(21)*y(2);
jac(406,2)=jac(406,2)-k(21)*y(406);
jac(406,406)=jac(406,406)-k(21)*y(2);
jac(411,2)=jac(411,2)+k(21)*y(406);
jac(411,406)=jac(411,406)+k(21)*y(2);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(406,406)=jac(406,406)-k(11);
jac(410,406)=jac(410,406)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(406,406)=jac(406,406)-k(10);
jac(3,406)=jac(3,406)+k(10);
jac(409,406)=jac(409,406)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(406);
jac(3,406)=jac(3,406)-k(9)*y(3);
jac(406,3)=jac(406,3)-k(9)*y(406);
jac(406,406)=jac(406,406)-k(9)*y(3);
jac(408,3)=jac(408,3)+k(9)*y(406);
jac(408,406)=jac(408,406)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(406,406)=jac(406,406)-k(8);
jac(16,406)=jac(16,406)+k(8);
jac(335,406)=jac(335,406)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY) 

jac(406,406)=jac(406,406)-k(6);
jac(15,406)=jac(15,406)+k(6);
jac(407,406)=jac(407,406)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(404,404)=jac(404,404)-k(32);
jac(14,404)=jac(14,404)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(404,404)=jac(404,404)-k(32);
jac(99,404)=jac(99,404)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(404);
jac(197,404)=jac(197,404)-k(21)*y(197);
jac(404,197)=jac(404,197)-k(21)*y(404);
jac(404,404)=jac(404,404)-k(21)*y(197);
jac(399,197)=jac(399,197)+k(21)*y(404);
jac(399,404)=jac(399,404)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(404);
jac(32,404)=jac(32,404)-k(21)*y(32);
jac(404,32)=jac(404,32)-k(21)*y(404);
jac(404,404)=jac(404,404)-k(21)*y(32);
jac(384,32)=jac(384,32)+k(21)*y(404);
jac(384,404)=jac(384,404)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(404);
jac(2,404)=jac(2,404)-k(21)*y(2);
jac(404,2)=jac(404,2)-k(21)*y(404);
jac(404,404)=jac(404,404)-k(21)*y(2);
jac(388,2)=jac(388,2)+k(21)*y(404);
jac(388,404)=jac(388,404)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(404);
jac(197,404)=jac(197,404)-k(21)*y(197);
jac(404,197)=jac(404,197)-k(21)*y(404);
jac(404,404)=jac(404,404)-k(21)*y(197);
jac(406,197)=jac(406,197)+k(21)*y(404);
jac(406,404)=jac(406,404)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(404);
jac(32,404)=jac(32,404)-k(21)*y(32);
jac(404,32)=jac(404,32)-k(21)*y(404);
jac(404,404)=jac(404,404)-k(21)*y(32);
jac(405,32)=jac(405,32)+k(21)*y(404);
jac(405,404)=jac(405,404)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(404);
jac(2,404)=jac(2,404)-k(21)*y(2);
jac(404,2)=jac(404,2)-k(21)*y(404);
jac(404,404)=jac(404,404)-k(21)*y(2);
jac(383,2)=jac(383,2)+k(21)*y(404);
jac(383,404)=jac(383,404)+k(21)*y(2);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(404,404)=jac(404,404)-k(11);
jac(57,404)=jac(57,404)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

jac(404,404)=jac(404,404)-k(10);
jac(3,404)=jac(3,404)+k(10);
jac(150,404)=jac(150,404)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(404);
jac(3,404)=jac(3,404)-k(9)*y(3);
jac(404,3)=jac(404,3)-k(9)*y(404);
jac(404,404)=jac(404,404)-k(9)*y(3);
jac(149,3)=jac(149,3)+k(9)*y(404);
jac(149,404)=jac(149,404)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(404,404)=jac(404,404)-k(8);
jac(16,404)=jac(16,404)+k(8);
jac(48,404)=jac(48,404)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) 

jac(404,404)=jac(404,404)-k(6);
jac(15,404)=jac(15,404)+k(6);
jac(47,404)=jac(47,404)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(399,399)=jac(399,399)-k(34);
jac(384,399)=jac(384,399)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(399,399)=jac(399,399)-k(33);
jac(396,399)=jac(396,399)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(399,399)=jac(399,399)-k(32);
jac(208,399)=jac(208,399)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(399,399)=jac(399,399)-k(22);
jac(197,399)=jac(197,399)+k(22);
jac(404,399)=jac(404,399)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(399);
jac(197,399)=jac(197,399)-k(21)*y(197);
jac(399,197)=jac(399,197)-k(21)*y(399);
jac(399,399)=jac(399,399)-k(21)*y(197);
jac(403,197)=jac(403,197)+k(21)*y(399);
jac(403,399)=jac(403,399)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(399);
jac(32,399)=jac(32,399)-k(21)*y(32);
jac(399,32)=jac(399,32)-k(21)*y(399);
jac(399,399)=jac(399,399)-k(21)*y(32);
jac(402,32)=jac(402,32)+k(21)*y(399);
jac(402,399)=jac(402,399)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(399);
jac(2,399)=jac(2,399)-k(21)*y(2);
jac(399,2)=jac(399,2)-k(21)*y(399);
jac(399,399)=jac(399,399)-k(21)*y(2);
jac(374,2)=jac(374,2)+k(21)*y(399);
jac(374,399)=jac(374,399)+k(21)*y(2);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(399,399)=jac(399,399)-k(11);
jac(401,399)=jac(401,399)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(399,399)=jac(399,399)-k(10);
jac(3,399)=jac(3,399)+k(10);
jac(376,399)=jac(376,399)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(399);
jac(3,399)=jac(3,399)-k(9)*y(3);
jac(399,3)=jac(399,3)-k(9)*y(399);
jac(399,399)=jac(399,399)-k(9)*y(3);
jac(400,3)=jac(400,3)+k(9)*y(399);
jac(400,399)=jac(400,399)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(399,399)=jac(399,399)-k(8);
jac(48,399)=jac(48,399)+k(8);
jac(209,399)=jac(209,399)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(399,399)=jac(399,399)-k(6);
jac(47,399)=jac(47,399)+k(6);
jac(192,399)=jac(192,399)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(397,397)=jac(397,397)-k(34);
jac(374,397)=jac(374,397)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(397,397)=jac(397,397)-k(34);
jac(385,397)=jac(385,397)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(397,397)=jac(397,397)-k(33);
jac(394,397)=jac(394,397)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(397,397)=jac(397,397)-k(22);
jac(182,397)=jac(182,397)+k(22);
jac(399,397)=jac(399,397)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(397,397)=jac(397,397)-k(22);
jac(197,397)=jac(197,397)+k(22);
jac(393,397)=jac(393,397)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(397,397)=jac(397,397)-k(11);
jac(398,397)=jac(398,397)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(397,397)=jac(397,397)-k(10);
jac(3,397)=jac(3,397)+k(10);
jac(369,397)=jac(369,397)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(397);
jac(3,397)=jac(3,397)-k(9)*y(3);
jac(397,3)=jac(397,3)-k(9)*y(397);
jac(397,397)=jac(397,397)-k(9)*y(3);
jac(368,3)=jac(368,3)+k(9)*y(397);
jac(368,397)=jac(368,397)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) 

jac(397,397)=jac(397,397)-k(8);
jac(209,397)=jac(209,397)+k(8);
jac(184,397)=jac(184,397)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(397,397)=jac(397,397)-k(6);
jac(192,397)=jac(192,397)+k(6);
jac(391,397)=jac(391,397)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(394,394)=jac(394,394)-k(34);
jac(389,394)=jac(389,394)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(394,394)=jac(394,394)-k(34);
jac(390,394)=jac(390,394)+k(34);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(394,394)=jac(394,394)-k(23);
jac(397,394)=jac(397,394)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(394,394)=jac(394,394)-k(22);
jac(182,394)=jac(182,394)+k(22);
jac(396,394)=jac(396,394)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(394,394)=jac(394,394)-k(22);
jac(182,394)=jac(182,394)+k(22);
jac(393,394)=jac(393,394)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(394,394)=jac(394,394)-k(11);
jac(395,394)=jac(395,394)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(394,394)=jac(394,394)-k(10);
jac(3,394)=jac(3,394)+k(10);
jac(187,394)=jac(187,394)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(394);
jac(3,394)=jac(3,394)-k(9)*y(3);
jac(394,3)=jac(394,3)-k(9)*y(394);
jac(394,394)=jac(394,394)-k(9)*y(3);
jac(186,3)=jac(186,3)+k(9)*y(394);
jac(186,394)=jac(186,394)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) 

jac(394,394)=jac(394,394)-k(8);
jac(214,394)=jac(214,394)+k(8);
jac(184,394)=jac(184,394)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(394,394)=jac(394,394)-k(6);
jac(177,394)=jac(177,394)+k(6);
jac(391,394)=jac(391,394)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(390,390)=jac(390,390)-k(34);
jac(386,390)=jac(386,390)+k(34);

% rule    : R13
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(390,390)=jac(390,390)-k(29);
jac(394,390)=jac(394,390)+k(29);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(390,390)=jac(390,390)-k(23);
jac(385,390)=jac(385,390)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(390,390)=jac(390,390)-k(22);
jac(182,390)=jac(182,390)+k(22);
jac(388,390)=jac(388,390)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(390,390)=jac(390,390)-k(22);
jac(2,390)=jac(2,390)+k(22);
jac(393,390)=jac(393,390)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(390,390)=jac(390,390)-k(11);
jac(392,390)=jac(392,390)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(390,390)=jac(390,390)-k(10);
jac(3,390)=jac(3,390)+k(10);
jac(179,390)=jac(179,390)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(390);
jac(3,390)=jac(3,390)-k(9)*y(3);
jac(390,3)=jac(390,3)-k(9)*y(390);
jac(390,390)=jac(390,390)-k(9)*y(3);
jac(178,3)=jac(178,3)+k(9)*y(390);
jac(178,390)=jac(178,390)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(390,390)=jac(390,390)-k(8);
jac(22,390)=jac(22,390)+k(8);
jac(184,390)=jac(184,390)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(390,390)=jac(390,390)-k(6);
jac(21,390)=jac(21,390)+k(6);
jac(391,390)=jac(391,390)+k(6);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(386,386)=jac(386,386)-k(28);
jac(390,386)=jac(390,386)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(386,386)=jac(386,386)-k(27);
jac(389,386)=jac(389,386)+k(27);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(386,386)=jac(386,386)-k(23);
jac(379,386)=jac(379,386)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(386,386)=jac(386,386)-k(22);
jac(2,386)=jac(2,386)+k(22);
jac(388,386)=jac(388,386)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(386,386)=jac(386,386)-k(22);
jac(2,386)=jac(2,386)+k(22);
jac(383,386)=jac(383,386)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(386,386)=jac(386,386)-k(11);
jac(387,386)=jac(387,386)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(386,386)=jac(386,386)-k(10);
jac(3,386)=jac(3,386)+k(10);
jac(172,386)=jac(172,386)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(386);
jac(3,386)=jac(3,386)-k(9)*y(3);
jac(386,3)=jac(386,3)-k(9)*y(386);
jac(386,386)=jac(386,386)-k(9)*y(3);
jac(171,3)=jac(171,3)+k(9)*y(386);
jac(171,386)=jac(171,386)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) 

jac(386,386)=jac(386,386)-k(8);
jac(22,386)=jac(22,386)+k(8);
jac(140,386)=jac(140,386)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(386,386)=jac(386,386)-k(6);
jac(21,386)=jac(21,386)+k(6);
jac(380,386)=jac(380,386)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(379,379)=jac(379,379)-k(33);
jac(386,379)=jac(386,379)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(379,379)=jac(379,379)-k(28);
jac(385,379)=jac(385,379)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(379,379)=jac(379,379)-k(27);
jac(374,379)=jac(374,379)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(379,379)=jac(379,379)-k(22);
jac(2,379)=jac(2,379)+k(22);
jac(384,379)=jac(384,379)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(379,379)=jac(379,379)-k(22);
jac(32,379)=jac(32,379)+k(22);
jac(383,379)=jac(383,379)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(379,379)=jac(379,379)-k(11);
jac(382,379)=jac(382,379)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(379,379)=jac(379,379)-k(10);
jac(3,379)=jac(3,379)+k(10);
jac(378,379)=jac(378,379)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(379);
jac(3,379)=jac(3,379)-k(9)*y(3);
jac(379,3)=jac(379,3)-k(9)*y(379);
jac(379,379)=jac(379,379)-k(9)*y(3);
jac(381,3)=jac(381,3)+k(9)*y(379);
jac(381,379)=jac(381,379)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) 

jac(379,379)=jac(379,379)-k(8);
jac(28,379)=jac(28,379)+k(8);
jac(140,379)=jac(140,379)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1, l2!1), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(379,379)=jac(379,379)-k(6);
jac(27,379)=jac(27,379)+k(6);
jac(380,379)=jac(380,379)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(378,378)=jac(378,378)-k(33);
jac(172,378)=jac(172,378)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) 

jac(378,378)=jac(378,378)-k(28);
jac(366,378)=jac(366,378)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(378,378)=jac(378,378)-k(27);
jac(373,378)=jac(373,378)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(378,378)=jac(378,378)-k(22);
jac(2,378)=jac(2,378)+k(22);
jac(156,378)=jac(156,378)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(378,378)=jac(378,378)-k(22);
jac(32,378)=jac(32,378)+k(22);
jac(375,378)=jac(375,378)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(378);
jac(3,378)=jac(3,378)-k(9)*y(3);
jac(378,3)=jac(378,3)-k(9)*y(378);
jac(378,378)=jac(378,378)-k(9)*y(3);
jac(157,3)=jac(157,3)+k(9)*y(378);
jac(157,378)=jac(157,378)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(378);
jac(3,378)=jac(3,378)-k(9)*y(3);
jac(378,3)=jac(378,3)-k(9)*y(378);
jac(378,378)=jac(378,378)-k(9)*y(3);
jac(379,3)=jac(379,3)+k(9)*y(378);
jac(379,378)=jac(379,378)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(378,378)=jac(378,378)-k(8);
jac(21,378)=jac(21,378)+k(8);
jac(28,378)=jac(28,378)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(378,378)=jac(378,378)-k(6);
jac(27,378)=jac(27,378)+k(6);
jac(89,378)=jac(89,378)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) 

jac(373,373)=jac(373,373)-k(34);
jac(378,373)=jac(378,373)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) 

jac(373,373)=jac(373,373)-k(33);
jac(377,373)=jac(377,373)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) 

jac(373,373)=jac(373,373)-k(30);
jac(369,373)=jac(369,373)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(373,373)=jac(373,373)-k(22);
jac(2,373)=jac(2,373)+k(22);
jac(376,373)=jac(376,373)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(373,373)=jac(373,373)-k(22);
jac(197,373)=jac(197,373)+k(22);
jac(375,373)=jac(375,373)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(373);
jac(3,373)=jac(3,373)-k(9)*y(3);
jac(373,3)=jac(373,3)-k(9)*y(373);
jac(373,373)=jac(373,373)-k(9)*y(3);
jac(371,3)=jac(371,3)+k(9)*y(373);
jac(371,373)=jac(371,373)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(373);
jac(3,373)=jac(3,373)-k(9)*y(3);
jac(373,3)=jac(373,3)-k(9)*y(373);
jac(373,373)=jac(373,373)-k(9)*y(3);
jac(374,3)=jac(374,3)+k(9)*y(373);
jac(374,373)=jac(374,373)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(373,373)=jac(373,373)-k(8);
jac(21,373)=jac(21,373)+k(8);
jac(209,373)=jac(209,373)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(373,373)=jac(373,373)-k(6);
jac(192,373)=jac(192,373)+k(6);
jac(89,373)=jac(89,373)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(371,371)=jac(371,371)-k(34);
jac(157,371)=jac(157,371)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(371,371)=jac(371,371)-k(33);
jac(174,371)=jac(174,371)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(371,371)=jac(371,371)-k(30);
jac(367,371)=jac(367,371)+k(30);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(371,371)=jac(371,371)-k(24);
jac(162,371)=jac(162,371)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(371,371)=jac(371,371)-k(22);
jac(2,371)=jac(2,371)+k(22);
jac(198,371)=jac(198,371)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(371,371)=jac(371,371)-k(22);
jac(197,371)=jac(197,371)+k(22);
jac(152,371)=jac(152,371)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(371,371)=jac(371,371)-k(12);
jac(340,371)=jac(340,371)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(371,371)=jac(371,371)-k(10);
jac(3,371)=jac(3,371)+k(10);
jac(373,371)=jac(373,371)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(371);
jac(3,371)=jac(3,371)-k(9)*y(3);
jac(371,3)=jac(371,3)-k(9)*y(371);
jac(371,371)=jac(371,371)-k(9)*y(3);
jac(372,3)=jac(372,3)+k(9)*y(371);
jac(372,371)=jac(372,371)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(371,371)=jac(371,371)-k(8);
jac(21,371)=jac(21,371)+k(8);
jac(336,371)=jac(336,371)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(371,371)=jac(371,371)-k(6);
jac(89,371)=jac(89,371)+k(6);
jac(335,371)=jac(335,371)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(367,367)=jac(367,367)-k(34);
jac(371,367)=jac(371,367)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(367,367)=jac(367,367)-k(34);
jac(364,367)=jac(364,367)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(367,367)=jac(367,367)-k(33);
jac(183,367)=jac(183,367)+k(33);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(367,367)=jac(367,367)-k(24);
jac(370,367)=jac(370,367)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(367,367)=jac(367,367)-k(22);
jac(182,367)=jac(182,367)+k(22);
jac(198,367)=jac(198,367)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(367,367)=jac(367,367)-k(22);
jac(197,367)=jac(197,367)+k(22);
jac(181,367)=jac(181,367)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(367,367)=jac(367,367)-k(12);
jac(363,367)=jac(363,367)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(367,367)=jac(367,367)-k(10);
jac(3,367)=jac(3,367)+k(10);
jac(369,367)=jac(369,367)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(367);
jac(3,367)=jac(3,367)-k(9)*y(3);
jac(367,3)=jac(367,3)-k(9)*y(367);
jac(367,367)=jac(367,367)-k(9)*y(3);
jac(368,3)=jac(368,3)+k(9)*y(367);
jac(368,367)=jac(368,367)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(367,367)=jac(367,367)-k(8);
jac(177,367)=jac(177,367)+k(8);
jac(336,367)=jac(336,367)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(367,367)=jac(367,367)-k(6);
jac(176,367)=jac(176,367)+k(6);
jac(335,367)=jac(335,367)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(364,364)=jac(364,364)-k(34);
jac(157,364)=jac(157,364)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(364,364)=jac(364,364)-k(33);
jac(175,364)=jac(175,364)+k(33);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(364,364)=jac(364,364)-k(29);
jac(367,364)=jac(367,364)+k(29);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(364,364)=jac(364,364)-k(24);
jac(163,364)=jac(163,364)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(364,364)=jac(364,364)-k(22);
jac(182,364)=jac(182,364)+k(22);
jac(154,364)=jac(154,364)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(364,364)=jac(364,364)-k(22);
jac(32,364)=jac(32,364)+k(22);
jac(181,364)=jac(181,364)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(364,364)=jac(364,364)-k(12);
jac(358,364)=jac(358,364)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(364,364)=jac(364,364)-k(10);
jac(3,364)=jac(3,364)+k(10);
jac(366,364)=jac(366,364)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(364);
jac(3,364)=jac(3,364)-k(9)*y(3);
jac(364,3)=jac(364,3)-k(9)*y(364);
jac(364,364)=jac(364,364)-k(9)*y(3);
jac(365,3)=jac(365,3)+k(9)*y(364);
jac(365,364)=jac(365,364)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(364,364)=jac(364,364)-k(8);
jac(177,364)=jac(177,364)+k(8);
jac(132,364)=jac(132,364)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(364,364)=jac(364,364)-k(6);
jac(176,364)=jac(176,364)+k(6);
jac(131,364)=jac(131,364)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(358,358)=jac(358,358)-k(34);
jac(355,358)=jac(355,358)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(358,358)=jac(358,358)-k(33);
jac(180,358)=jac(180,358)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(358,358)=jac(358,358)-k(31);
jac(364,358)=jac(364,358)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(358,358)=jac(358,358)-k(29);
jac(363,358)=jac(363,358)+k(29);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(358,358)=jac(358,358)-k(24);
jac(362,358)=jac(362,358)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(358,358)=jac(358,358)-k(22);
jac(182,358)=jac(182,358)+k(22);
jac(137,358)=jac(137,358)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(358,358)=jac(358,358)-k(22);
jac(32,358)=jac(32,358)+k(22);
jac(361,358)=jac(361,358)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(358);
jac(3,358)=jac(3,358)-k(15)*y(3);
jac(358,3)=jac(358,3)-k(15)*y(358);
jac(358,358)=jac(358,358)-k(15)*y(3);
jac(360,3)=jac(360,3)+k(15)*y(358);
jac(360,358)=jac(360,358)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(358,358)=jac(358,358)-k(10);
jac(3,358)=jac(3,358)+k(10);
jac(359,358)=jac(359,358)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(358,358)=jac(358,358)-k(8);
jac(218,358)=jac(218,358)+k(8);
jac(132,358)=jac(132,358)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(358,358)=jac(358,358)-k(6);
jac(264,358)=jac(264,358)+k(6);
jac(131,358)=jac(131,358)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(355,355)=jac(355,355)-k(33);
jac(173,355)=jac(173,355)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(355,355)=jac(355,355)-k(31);
jac(157,355)=jac(157,355)+k(31);

% rule    : R12_2
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(355,355)=jac(355,355)-k(28);
jac(358,355)=jac(358,355)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(355,355)=jac(355,355)-k(27);
jac(340,355)=jac(340,355)+k(27);

% rule    : R10_1
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(355,355)=jac(355,355)-k(24);
jac(161,355)=jac(161,355)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(355,355)=jac(355,355)-k(22);
jac(32,355)=jac(32,355)+k(22);
jac(353,355)=jac(353,355)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(355,355)=jac(355,355)-k(22);
jac(2,355)=jac(2,355)+k(22);
jac(137,355)=jac(137,355)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(355);
jac(3,355)=jac(3,355)-k(15)*y(3);
jac(355,3)=jac(355,3)-k(15)*y(355);
jac(355,355)=jac(355,355)-k(15)*y(3);
jac(357,3)=jac(357,3)+k(15)*y(355);
jac(357,355)=jac(357,355)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(355,355)=jac(355,355)-k(10);
jac(3,355)=jac(3,355)+k(10);
jac(356,355)=jac(356,355)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(355,355)=jac(355,355)-k(8);
jac(111,355)=jac(111,355)+k(8);
jac(132,355)=jac(132,355)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(355,355)=jac(355,355)-k(6);
jac(110,355)=jac(110,355)+k(6);
jac(131,355)=jac(131,355)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(353,353)=jac(353,353)-k(32);
jac(109,353)=jac(109,353)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(353,353)=jac(353,353)-k(31);
jac(152,353)=jac(152,353)+k(31);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(353,353)=jac(353,353)-k(24);
jac(350,353)=jac(350,353)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(353,353)=jac(353,353)-k(22);
jac(2,353)=jac(2,353)+k(22);
jac(151,353)=jac(151,353)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(353);
jac(197,353)=jac(197,353)-k(21)*y(197);
jac(353,197)=jac(353,197)-k(21)*y(353);
jac(353,353)=jac(353,353)-k(21)*y(197);
jac(340,197)=jac(340,197)+k(21)*y(353);
jac(340,353)=jac(340,353)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(353);
jac(32,353)=jac(32,353)-k(21)*y(32);
jac(353,32)=jac(353,32)-k(21)*y(353);
jac(353,353)=jac(353,353)-k(21)*y(32);
jac(355,32)=jac(355,32)+k(21)*y(353);
jac(355,353)=jac(355,353)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(353);
jac(2,353)=jac(2,353)-k(21)*y(2);
jac(353,2)=jac(353,2)-k(21)*y(353);
jac(353,353)=jac(353,353)-k(21)*y(2);
jac(173,2)=jac(173,2)+k(21)*y(353);
jac(173,353)=jac(173,353)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(353);
jac(3,353)=jac(3,353)-k(15)*y(3);
jac(353,3)=jac(353,3)-k(15)*y(353);
jac(353,353)=jac(353,353)-k(15)*y(3);
jac(116,3)=jac(116,3)+k(15)*y(353);
jac(116,353)=jac(116,353)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(353,353)=jac(353,353)-k(10);
jac(3,353)=jac(3,353)+k(10);
jac(354,353)=jac(354,353)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(353,353)=jac(353,353)-k(8);
jac(126,353)=jac(126,353)+k(8);
jac(111,353)=jac(111,353)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(353,353)=jac(353,353)-k(6);
jac(48,353)=jac(48,353)+k(6);
jac(110,353)=jac(110,353)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(350,350)=jac(350,350)-k(33);
jac(353,350)=jac(353,350)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(350,350)=jac(350,350)-k(32);
jac(105,350)=jac(105,350)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(350,350)=jac(350,350)-k(31);
jac(153,350)=jac(153,350)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(350,350)=jac(350,350)-k(22);
jac(32,350)=jac(32,350)+k(22);
jac(151,350)=jac(151,350)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(21)*y(350);
jac(197,350)=jac(197,350)-k(21)*y(197);
jac(350,197)=jac(350,197)-k(21)*y(350);
jac(350,350)=jac(350,350)-k(21)*y(197);
jac(341,197)=jac(341,197)+k(21)*y(350);
jac(341,350)=jac(341,350)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(350);
jac(32,350)=jac(32,350)-k(21)*y(32);
jac(350,32)=jac(350,32)-k(21)*y(350);
jac(350,350)=jac(350,350)-k(21)*y(32);
jac(161,32)=jac(161,32)+k(21)*y(350);
jac(161,350)=jac(161,350)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(350);
jac(2,350)=jac(2,350)-k(21)*y(2);
jac(350,2)=jac(350,2)-k(21)*y(350);
jac(350,350)=jac(350,350)-k(21)*y(2);
jac(167,2)=jac(167,2)+k(21)*y(350);
jac(167,350)=jac(167,350)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(350);
jac(3,350)=jac(3,350)-k(15)*y(3);
jac(350,3)=jac(350,3)-k(15)*y(350);
jac(350,350)=jac(350,350)-k(15)*y(3);
jac(352,3)=jac(352,3)+k(15)*y(350);
jac(352,350)=jac(352,350)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(350,350)=jac(350,350)-k(10);
jac(3,350)=jac(3,350)+k(10);
jac(351,350)=jac(351,350)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(350,350)=jac(350,350)-k(8);
jac(126,350)=jac(126,350)+k(8);
jac(33,350)=jac(33,350)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(350,350)=jac(350,350)-k(6);
jac(48,350)=jac(48,350)+k(6);
jac(82,350)=jac(82,350)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(347,347)=jac(347,347)-k(34);
jac(167,347)=jac(167,347)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(347,347)=jac(347,347)-k(33);
jac(346,347)=jac(346,347)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(347,347)=jac(347,347)-k(31);
jac(168,347)=jac(168,347)+k(31);

% rule    : R13_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(347,347)=jac(347,347)-k(30);
jac(195,347)=jac(195,347)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(347,347)=jac(347,347)-k(22);
jac(32,347)=jac(32,347)+k(22);
jac(343,347)=jac(343,347)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(347,347)=jac(347,347)-k(22);
jac(182,347)=jac(182,347)+k(22);
jac(350,347)=jac(350,347)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(347);
jac(3,347)=jac(3,347)-k(15)*y(3);
jac(347,3)=jac(347,3)-k(15)*y(347);
jac(347,347)=jac(347,347)-k(15)*y(3);
jac(349,3)=jac(349,3)+k(15)*y(347);
jac(349,347)=jac(349,347)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(347,347)=jac(347,347)-k(10);
jac(3,347)=jac(3,347)+k(10);
jac(348,347)=jac(348,347)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(347,347)=jac(347,347)-k(8);
jac(33,347)=jac(33,347)+k(8);
jac(185,347)=jac(185,347)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(347,347)=jac(347,347)-k(6);
jac(82,347)=jac(82,347)+k(6);
jac(184,347)=jac(184,347)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(343,343)=jac(343,343)-k(34);
jac(143,343)=jac(143,343)+k(34);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(343,343)=jac(343,343)-k(32);
jac(330,343)=jac(330,343)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(343,343)=jac(343,343)-k(31);
jac(189,343)=jac(189,343)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(343,343)=jac(343,343)-k(22);
jac(182,343)=jac(182,343)+k(22);
jac(151,343)=jac(151,343)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(343);
jac(197,343)=jac(197,343)-k(21)*y(197);
jac(343,197)=jac(343,197)-k(21)*y(343);
jac(343,343)=jac(343,343)-k(21)*y(197);
jac(195,197)=jac(195,197)+k(21)*y(343);
jac(195,343)=jac(195,343)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(343);
jac(32,343)=jac(32,343)-k(21)*y(32);
jac(343,32)=jac(343,32)-k(21)*y(343);
jac(343,343)=jac(343,343)-k(21)*y(32);
jac(347,32)=jac(347,32)+k(21)*y(343);
jac(347,343)=jac(347,343)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(343);
jac(2,343)=jac(2,343)-k(21)*y(2);
jac(343,2)=jac(343,2)-k(21)*y(343);
jac(343,343)=jac(343,343)-k(21)*y(2);
jac(346,2)=jac(346,2)+k(21)*y(343);
jac(346,343)=jac(346,343)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(343);
jac(3,343)=jac(3,343)-k(15)*y(3);
jac(343,3)=jac(343,3)-k(15)*y(343);
jac(343,343)=jac(343,343)-k(15)*y(3);
jac(345,3)=jac(345,3)+k(15)*y(343);
jac(345,343)=jac(345,343)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(343,343)=jac(343,343)-k(10);
jac(3,343)=jac(3,343)+k(10);
jac(344,343)=jac(344,343)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~pY, g~pY) 

jac(343,343)=jac(343,343)-k(8);
jac(77,343)=jac(77,343)+k(8);
jac(185,343)=jac(185,343)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(343,343)=jac(343,343)-k(6);
jac(76,343)=jac(76,343)+k(6);
jac(184,343)=jac(184,343)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(337,337)=jac(337,337)-k(34);
jac(137,337)=jac(137,337)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) 

jac(337,337)=jac(337,337)-k(33);
jac(343,337)=jac(343,337)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(337,337)=jac(337,337)-k(32);
jac(334,337)=jac(334,337)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(337,337)=jac(337,337)-k(31);
jac(198,337)=jac(198,337)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(337,337)=jac(337,337)-k(22);
jac(197,337)=jac(197,337)+k(22);
jac(151,337)=jac(151,337)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(337);
jac(197,337)=jac(197,337)-k(21)*y(197);
jac(337,197)=jac(337,197)-k(21)*y(337);
jac(337,337)=jac(337,337)-k(21)*y(197);
jac(342,197)=jac(342,197)+k(21)*y(337);
jac(342,337)=jac(342,337)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(337);
jac(32,337)=jac(32,337)-k(21)*y(32);
jac(337,32)=jac(337,32)-k(21)*y(337);
jac(337,337)=jac(337,337)-k(21)*y(32);
jac(341,32)=jac(341,32)+k(21)*y(337);
jac(341,337)=jac(341,337)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(337);
jac(2,337)=jac(2,337)-k(21)*y(2);
jac(337,2)=jac(337,2)-k(21)*y(337);
jac(337,337)=jac(337,337)-k(21)*y(2);
jac(340,2)=jac(340,2)+k(21)*y(337);
jac(340,337)=jac(340,337)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(337);
jac(3,337)=jac(3,337)-k(15)*y(3);
jac(337,3)=jac(337,3)-k(15)*y(337);
jac(337,337)=jac(337,337)-k(15)*y(3);
jac(339,3)=jac(339,3)+k(15)*y(337);
jac(339,337)=jac(339,337)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(337,337)=jac(337,337)-k(10);
jac(3,337)=jac(3,337)+k(10);
jac(338,337)=jac(338,337)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~pY) 

jac(337,337)=jac(337,337)-k(8);
jac(77,337)=jac(77,337)+k(8);
jac(336,337)=jac(336,337)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(337,337)=jac(337,337)-k(6);
jac(76,337)=jac(76,337)+k(6);
jac(335,337)=jac(335,337)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(334,334)=jac(334,334)-k(34);
jac(135,334)=jac(135,334)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(334,334)=jac(334,334)-k(33);
jac(330,334)=jac(330,334)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(334,334)=jac(334,334)-k(31);
jac(199,334)=jac(199,334)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(334,334)=jac(334,334)-k(22);
jac(197,334)=jac(197,334)+k(22);
jac(128,334)=jac(128,334)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(334);
jac(3,334)=jac(3,334)-k(15)*y(3);
jac(334,3)=jac(334,3)-k(15)*y(334);
jac(334,334)=jac(334,334)-k(15)*y(3);
jac(200,3)=jac(200,3)+k(15)*y(334);
jac(200,334)=jac(200,334)+k(15)*y(3);

% rule    : R5_2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) 

jac(334,334)=jac(334,334)-k(14);
jac(337,334)=jac(337,334)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(334,334)=jac(334,334)-k(10);
jac(3,334)=jac(3,334)+k(10);
jac(332,334)=jac(332,334)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

jac(334,334)=jac(334,334)-k(8);
jac(72,334)=jac(72,334)+k(8);
jac(336,334)=jac(336,334)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(334,334)=jac(334,334)-k(6);
jac(71,334)=jac(71,334)+k(6);
jac(335,334)=jac(335,334)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(332,332)=jac(332,332)-k(34);
jac(136,332)=jac(136,332)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(332,332)=jac(332,332)-k(33);
jac(328,332)=jac(328,332)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) 

jac(332,332)=jac(332,332)-k(31);
jac(211,332)=jac(211,332)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~pY) 

jac(332,332)=jac(332,332)-k(22);
jac(197,332)=jac(197,332)+k(22);
jac(331,332)=jac(331,332)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(332);
jac(3,332)=jac(3,332)-k(15)*y(3);
jac(332,3)=jac(332,3)-k(15)*y(332);
jac(332,332)=jac(332,332)-k(15)*y(3);
jac(322,3)=jac(322,3)+k(15)*y(332);
jac(322,332)=jac(322,332)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(332);
jac(3,332)=jac(3,332)-k(9)*y(3);
jac(332,3)=jac(332,3)-k(9)*y(332);
jac(332,332)=jac(332,332)-k(9)*y(3);
jac(334,3)=jac(334,3)+k(9)*y(332);
jac(334,332)=jac(334,332)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY, g~Y) 

jac(332,332)=jac(332,332)-k(8);
jac(72,332)=jac(72,332)+k(8);
jac(209,332)=jac(209,332)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(332,332)=jac(332,332)-k(6);
jac(71,332)=jac(71,332)+k(6);
jac(192,332)=jac(192,332)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(333,333)=jac(333,333)-k(31);
jac(12,333)=jac(12,333)+k(31);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(3,3)=jac(3,3)-k(15)*y(333);
jac(3,333)=jac(3,333)-k(15)*y(3);
jac(333,3)=jac(333,3)-k(15)*y(333);
jac(333,333)=jac(333,333)-k(15)*y(3);
jac(123,3)=jac(123,3)+k(15)*y(333);
jac(123,333)=jac(123,333)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(333);
jac(3,333)=jac(3,333)-k(9)*y(3);
jac(333,3)=jac(333,3)-k(9)*y(333);
jac(333,333)=jac(333,333)-k(9)*y(3);
jac(124,3)=jac(124,3)+k(9)*y(333);
jac(124,333)=jac(124,333)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) 

jac(333,333)=jac(333,333)-k(8);
jac(72,333)=jac(72,333)+k(8);
jac(5,333)=jac(5,333)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) 

jac(333,333)=jac(333,333)-k(6);
jac(1,333)=jac(1,333)+k(6);
jac(71,333)=jac(71,333)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) 

jac(331,331)=jac(331,331)-k(32);
jac(333,331)=jac(333,331)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(331,331)=jac(331,331)-k(31);
jac(18,331)=jac(18,331)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(331);
jac(197,331)=jac(197,331)-k(21)*y(197);
jac(331,197)=jac(331,197)-k(21)*y(331);
jac(331,331)=jac(331,331)-k(21)*y(197);
jac(332,197)=jac(332,197)+k(21)*y(331);
jac(332,331)=jac(332,331)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(331);
jac(32,331)=jac(32,331)-k(21)*y(32);
jac(331,32)=jac(331,32)-k(21)*y(331);
jac(331,331)=jac(331,331)-k(21)*y(32);
jac(136,32)=jac(136,32)+k(21)*y(331);
jac(136,331)=jac(136,331)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(331);
jac(2,331)=jac(2,331)-k(21)*y(2);
jac(331,2)=jac(331,2)-k(21)*y(331);
jac(331,331)=jac(331,331)-k(21)*y(2);
jac(142,2)=jac(142,2)+k(21)*y(331);
jac(142,331)=jac(142,331)+k(21)*y(2);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(3,3)=jac(3,3)-k(15)*y(331);
jac(3,331)=jac(3,331)-k(15)*y(3);
jac(331,3)=jac(331,3)-k(15)*y(331);
jac(331,331)=jac(331,331)-k(15)*y(3);
jac(127,3)=jac(127,3)+k(15)*y(331);
jac(127,331)=jac(127,331)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(331);
jac(3,331)=jac(3,331)-k(9)*y(3);
jac(331,3)=jac(331,3)-k(9)*y(331);
jac(331,331)=jac(331,331)-k(9)*y(3);
jac(128,3)=jac(128,3)+k(9)*y(331);
jac(128,331)=jac(128,331)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) + Rec(a, b~pY, g~Y) 

jac(331,331)=jac(331,331)-k(8);
jac(72,331)=jac(72,331)+k(8);
jac(16,331)=jac(16,331)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) + Rec(a, b~Y, g~pY) 

jac(331,331)=jac(331,331)-k(6);
jac(15,331)=jac(15,331)+k(6);
jac(71,331)=jac(71,331)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) 

jac(328,328)=jac(328,328)-k(34);
jac(142,328)=jac(142,328)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) 

jac(328,328)=jac(328,328)-k(31);
jac(216,328)=jac(216,328)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Syk(tSH2, l~Y, a~pY) 

jac(328,328)=jac(328,328)-k(22);
jac(182,328)=jac(182,328)+k(22);
jac(331,328)=jac(331,328)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(328);
jac(3,328)=jac(3,328)-k(15)*y(3);
jac(328,3)=jac(328,3)-k(15)*y(328);
jac(328,328)=jac(328,328)-k(15)*y(3);
jac(326,3)=jac(326,3)+k(15)*y(328);
jac(326,328)=jac(326,328)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(9)*y(328);
jac(3,328)=jac(3,328)-k(9)*y(3);
jac(328,3)=jac(328,3)-k(9)*y(328);
jac(328,328)=jac(328,328)-k(9)*y(3);
jac(330,3)=jac(330,3)+k(9)*y(328);
jac(330,328)=jac(330,328)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

jac(328,328)=jac(328,328)-k(8);
jac(72,328)=jac(72,328)+k(8);
jac(214,328)=jac(214,328)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(328,328)=jac(328,328)-k(6);
jac(71,328)=jac(71,328)+k(6);
jac(177,328)=jac(177,328)+k(6);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) 

jac(329,329)=jac(329,329)-k(25);
jac(133,329)=jac(133,329)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

jac(329,329)=jac(329,329)-k(22);
jac(2,329)=jac(2,329)+k(22);
jac(127,329)=jac(127,329)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(329,329)=jac(329,329)-k(17);
jac(316,329)=jac(316,329)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(329,329)=jac(329,329)-k(16);
jac(3,329)=jac(3,329)+k(16);
jac(142,329)=jac(142,329)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(329);
jac(3,329)=jac(3,329)-k(9)*y(3);
jac(329,3)=jac(329,3)-k(9)*y(329);
jac(329,329)=jac(329,329)-k(9)*y(3);
jac(129,3)=jac(129,3)+k(9)*y(329);
jac(129,329)=jac(129,329)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(329,329)=jac(329,329)-k(8);
jac(61,329)=jac(61,329)+k(8);
jac(22,329)=jac(22,329)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(329,329)=jac(329,329)-k(6);
jac(122,329)=jac(122,329)+k(6);
jac(21,329)=jac(21,329)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~Y) 

jac(326,326)=jac(326,326)-k(34);
jac(329,326)=jac(329,326)+k(34);

% rule    : R11
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(326,326)=jac(326,326)-k(25);
jac(322,326)=jac(322,326)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

jac(326,326)=jac(326,326)-k(22);
jac(182,326)=jac(182,326)+k(22);
jac(127,326)=jac(127,326)+k(22);

% rule    : R7
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(326,326)=jac(326,326)-k(17);
jac(323,326)=jac(323,326)+k(17);

% rule    : R6_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(326,326)=jac(326,326)-k(16);
jac(3,326)=jac(3,326)+k(16);
jac(328,326)=jac(328,326)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(326);
jac(3,326)=jac(3,326)-k(9)*y(3);
jac(326,3)=jac(326,3)-k(9)*y(326);
jac(326,326)=jac(326,326)-k(9)*y(3);
jac(327,3)=jac(327,3)+k(9)*y(326);
jac(327,326)=jac(327,326)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(326,326)=jac(326,326)-k(8);
jac(61,326)=jac(61,326)+k(8);
jac(214,326)=jac(214,326)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(326,326)=jac(326,326)-k(6);
jac(122,326)=jac(122,326)+k(6);
jac(177,326)=jac(177,326)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(323,323)=jac(323,323)-k(34);
jac(316,323)=jac(316,323)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(323,323)=jac(323,323)-k(31);
jac(326,323)=jac(326,323)+k(31);

% rule    : R11
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(323,323)=jac(323,323)-k(25);
jac(318,323)=jac(318,323)+k(25);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~pY) 

jac(323,323)=jac(323,323)-k(22);
jac(182,323)=jac(182,323)+k(22);
jac(314,323)=jac(314,323)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(323,323)=jac(323,323)-k(16);
jac(3,323)=jac(3,323)+k(16);
jac(325,323)=jac(325,323)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(323);
jac(3,323)=jac(3,323)-k(15)*y(3);
jac(323,3)=jac(323,3)-k(15)*y(323);
jac(323,323)=jac(323,323)-k(15)*y(3);
jac(324,3)=jac(324,3)+k(15)*y(323);
jac(324,323)=jac(324,323)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(323,323)=jac(323,323)-k(8);
jac(61,323)=jac(61,323)+k(8);
jac(219,323)=jac(219,323)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(323,323)=jac(323,323)-k(6);
jac(122,323)=jac(122,323)+k(6);
jac(218,323)=jac(218,323)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(318,318)=jac(318,318)-k(34);
jac(317,318)=jac(317,318)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~Y, a~pY) 

jac(318,318)=jac(318,318)-k(33);
jac(323,318)=jac(323,318)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY) 

jac(318,318)=jac(318,318)-k(31);
jac(322,318)=jac(322,318)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) 

jac(318,318)=jac(318,318)-k(22);
jac(197,318)=jac(197,318)+k(22);
jac(314,318)=jac(314,318)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(318,318)=jac(318,318)-k(16);
jac(3,318)=jac(3,318)+k(16);
jac(321,318)=jac(321,318)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(318);
jac(3,318)=jac(3,318)-k(15)*y(3);
jac(318,3)=jac(318,3)-k(15)*y(318);
jac(318,318)=jac(318,318)-k(15)*y(3);
jac(320,3)=jac(320,3)+k(15)*y(318);
jac(320,318)=jac(320,318)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(318,318)=jac(318,318)-k(8);
jac(61,318)=jac(61,318)+k(8);
jac(319,318)=jac(319,318)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(318,318)=jac(318,318)-k(6);
jac(122,318)=jac(122,318)+k(6);
jac(257,318)=jac(257,318)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(314,314)=jac(314,314)-k(32);
jac(313,314)=jac(313,314)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(314,314)=jac(314,314)-k(31);
jac(127,314)=jac(127,314)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(314);
jac(197,314)=jac(197,314)-k(21)*y(197);
jac(314,197)=jac(314,197)-k(21)*y(314);
jac(314,314)=jac(314,314)-k(21)*y(197);
jac(318,197)=jac(318,197)+k(21)*y(314);
jac(318,314)=jac(318,314)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(314);
jac(32,314)=jac(32,314)-k(21)*y(32);
jac(314,32)=jac(314,32)-k(21)*y(314);
jac(314,314)=jac(314,314)-k(21)*y(32);
jac(317,32)=jac(317,32)+k(21)*y(314);
jac(317,314)=jac(317,314)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(314);
jac(2,314)=jac(2,314)-k(21)*y(2);
jac(314,2)=jac(314,2)-k(21)*y(314);
jac(314,314)=jac(314,314)-k(21)*y(2);
jac(316,2)=jac(316,2)+k(21)*y(314);
jac(316,314)=jac(316,314)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

jac(314,314)=jac(314,314)-k(16);
jac(3,314)=jac(3,314)+k(16);
jac(237,314)=jac(237,314)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(314);
jac(3,314)=jac(3,314)-k(15)*y(3);
jac(314,3)=jac(314,3)-k(15)*y(314);
jac(314,314)=jac(314,314)-k(15)*y(3);
jac(236,3)=jac(236,3)+k(15)*y(314);
jac(236,314)=jac(236,314)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1!1, l2) 

jac(314,314)=jac(314,314)-k(8);
jac(315,314)=jac(315,314)+k(8);
jac(61,314)=jac(61,314)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) 

jac(314,314)=jac(314,314)-k(6);
jac(77,314)=jac(77,314)+k(6);
jac(122,314)=jac(122,314)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(313,313)=jac(313,313)-k(31);
jac(123,313)=jac(123,313)+k(31);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(313,313)=jac(313,313)-k(19);
jac(314,313)=jac(314,313)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

jac(313,313)=jac(313,313)-k(16);
jac(3,313)=jac(3,313)+k(16);
jac(74,313)=jac(74,313)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(313);
jac(3,313)=jac(3,313)-k(15)*y(3);
jac(313,3)=jac(313,3)-k(15)*y(313);
jac(313,313)=jac(313,313)-k(15)*y(3);
jac(73,3)=jac(73,3)+k(15)*y(313);
jac(73,313)=jac(73,313)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(313,313)=jac(313,313)-k(8);
jac(91,313)=jac(91,313)+k(8);
jac(61,313)=jac(61,313)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) 

jac(313,313)=jac(313,313)-k(6);
jac(72,313)=jac(72,313)+k(6);
jac(122,313)=jac(122,313)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) 

jac(309,309)=jac(309,309)-k(32);
jac(313,309)=jac(313,309)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(309,309)=jac(309,309)-k(31);
jac(119,309)=jac(119,309)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(309);
jac(197,309)=jac(197,309)-k(21)*y(197);
jac(309,197)=jac(309,197)-k(21)*y(309);
jac(309,309)=jac(309,309)-k(21)*y(197);
jac(312,197)=jac(312,197)+k(21)*y(309);
jac(312,309)=jac(312,309)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(309);
jac(32,309)=jac(32,309)-k(21)*y(32);
jac(309,32)=jac(309,32)-k(21)*y(309);
jac(309,309)=jac(309,309)-k(21)*y(32);
jac(311,32)=jac(311,32)+k(21)*y(309);
jac(311,309)=jac(311,309)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(309);
jac(2,309)=jac(2,309)-k(21)*y(2);
jac(309,2)=jac(309,2)-k(21)*y(309);
jac(309,309)=jac(309,309)-k(21)*y(2);
jac(306,2)=jac(306,2)+k(21)*y(309);
jac(306,309)=jac(306,309)+k(21)*y(2);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) 

jac(309,309)=jac(309,309)-k(19);
jac(310,309)=jac(310,309)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

jac(309,309)=jac(309,309)-k(16);
jac(3,309)=jac(3,309)+k(16);
jac(79,309)=jac(79,309)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(309);
jac(3,309)=jac(3,309)-k(15)*y(3);
jac(309,3)=jac(309,3)-k(15)*y(309);
jac(309,309)=jac(309,309)-k(15)*y(3);
jac(78,3)=jac(78,3)+k(15)*y(309);
jac(78,309)=jac(78,309)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(309,309)=jac(309,309)-k(8);
jac(91,309)=jac(91,309)+k(8);
jac(54,309)=jac(54,309)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) 

jac(309,309)=jac(309,309)-k(6);
jac(72,309)=jac(72,309)+k(6);
jac(118,309)=jac(118,309)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(306,306)=jac(306,306)-k(31);
jac(115,306)=jac(115,306)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Syk(tSH2, l~Y, a~Y) 

jac(306,306)=jac(306,306)-k(22);
jac(2,306)=jac(2,306)+k(22);
jac(309,306)=jac(309,306)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(306,306)=jac(306,306)-k(19);
jac(308,306)=jac(308,306)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(306,306)=jac(306,306)-k(16);
jac(3,306)=jac(3,306)+k(16);
jac(305,306)=jac(305,306)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!4, l2!1), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(306);
jac(3,306)=jac(3,306)-k(15)*y(3);
jac(306,3)=jac(306,3)-k(15)*y(306);
jac(306,306)=jac(306,306)-k(15)*y(3);
jac(307,3)=jac(307,3)+k(15)*y(306);
jac(307,306)=jac(307,306)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(306,306)=jac(306,306)-k(8);
jac(91,306)=jac(91,306)+k(8);
jac(43,306)=jac(43,306)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

jac(306,306)=jac(306,306)-k(6);
jac(72,306)=jac(72,306)+k(6);
jac(114,306)=jac(114,306)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(305,305)=jac(305,305)-k(31);
jac(90,305)=jac(90,305)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(305,305)=jac(305,305)-k(31);
jac(112,305)=jac(112,305)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(305,305)=jac(305,305)-k(22);
jac(2,305)=jac(2,305)+k(22);
jac(79,305)=jac(79,305)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(305);
jac(3,305)=jac(3,305)-k(15)*y(3);
jac(305,3)=jac(305,3)-k(15)*y(305);
jac(305,305)=jac(305,305)-k(15)*y(3);
jac(80,3)=jac(80,3)+k(15)*y(305);
jac(80,305)=jac(80,305)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(305);
jac(3,305)=jac(3,305)-k(15)*y(3);
jac(305,3)=jac(305,3)-k(15)*y(305);
jac(305,305)=jac(305,305)-k(15)*y(3);
jac(306,3)=jac(306,3)+k(15)*y(305);
jac(306,305)=jac(306,305)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(305,305)=jac(305,305)-k(8);
jac(91,305)=jac(91,305)+k(8);
jac(111,305)=jac(111,305)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

jac(305,305)=jac(305,305)-k(6);
jac(72,305)=jac(72,305)+k(6);
jac(110,305)=jac(110,305)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(302,302)=jac(302,302)-k(34);
jac(305,302)=jac(305,302)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(302,302)=jac(302,302)-k(31);
jac(299,302)=jac(299,302)+k(31);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) 

jac(302,302)=jac(302,302)-k(31);
jac(304,302)=jac(304,302)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Syk(tSH2, l~Y, a~pY) 

jac(302,302)=jac(302,302)-k(22);
jac(182,302)=jac(182,302)+k(22);
jac(79,302)=jac(79,302)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(302);
jac(3,302)=jac(3,302)-k(15)*y(3);
jac(302,3)=jac(302,3)-k(15)*y(302);
jac(302,302)=jac(302,302)-k(15)*y(3);
jac(300,3)=jac(300,3)+k(15)*y(302);
jac(300,302)=jac(300,302)+k(15)*y(3);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~pY) 

jac(3,3)=jac(3,3)-k(15)*y(302);
jac(3,302)=jac(3,302)-k(15)*y(3);
jac(302,3)=jac(302,3)-k(15)*y(302);
jac(302,302)=jac(302,302)-k(15)*y(3);
jac(303,3)=jac(303,3)+k(15)*y(302);
jac(303,302)=jac(303,302)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(302,302)=jac(302,302)-k(8);
jac(91,302)=jac(91,302)+k(8);
jac(218,302)=jac(218,302)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY, g~Y) 

jac(302,302)=jac(302,302)-k(6);
jac(72,302)=jac(72,302)+k(6);
jac(264,302)=jac(264,302)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(300,300)=jac(300,300)-k(34);
jac(80,300)=jac(80,300)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(300,300)=jac(300,300)-k(31);
jac(297,300)=jac(297,300)+k(31);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(300,300)=jac(300,300)-k(26);
jac(203,300)=jac(203,300)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

jac(300,300)=jac(300,300)-k(22);
jac(182,300)=jac(182,300)+k(22);
jac(75,300)=jac(75,300)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(300,300)=jac(300,300)-k(16);
jac(3,300)=jac(3,300)+k(16);
jac(302,300)=jac(302,300)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(300);
jac(3,300)=jac(3,300)-k(15)*y(3);
jac(300,3)=jac(300,3)-k(15)*y(300);
jac(300,300)=jac(300,300)-k(15)*y(3);
jac(301,3)=jac(301,3)+k(15)*y(300);
jac(301,300)=jac(301,300)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(300,300)=jac(300,300)-k(8);
jac(62,300)=jac(62,300)+k(8);
jac(218,300)=jac(218,300)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(300,300)=jac(300,300)-k(6);
jac(61,300)=jac(61,300)+k(6);
jac(264,300)=jac(264,300)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(297,297)=jac(297,297)-k(34);
jac(88,297)=jac(88,297)+k(34);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(297,297)=jac(297,297)-k(26);
jac(295,297)=jac(295,297)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~pY) 

jac(297,297)=jac(297,297)-k(22);
jac(182,297)=jac(182,297)+k(22);
jac(63,297)=jac(63,297)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(297,297)=jac(297,297)-k(18);
jac(300,297)=jac(300,297)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(297,297)=jac(297,297)-k(16);
jac(3,297)=jac(3,297)+k(16);
jac(299,297)=jac(299,297)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(297);
jac(3,297)=jac(3,297)-k(9)*y(3);
jac(297,3)=jac(297,3)-k(9)*y(297);
jac(297,297)=jac(297,297)-k(9)*y(3);
jac(298,3)=jac(298,3)+k(9)*y(297);
jac(298,297)=jac(298,297)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(297,297)=jac(297,297)-k(8);
jac(62,297)=jac(62,297)+k(8);
jac(177,297)=jac(177,297)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(297,297)=jac(297,297)-k(6);
jac(61,297)=jac(61,297)+k(6);
jac(176,297)=jac(176,297)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(295,295)=jac(295,295)-k(34);
jac(85,295)=jac(85,295)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(295,295)=jac(295,295)-k(33);
jac(297,295)=jac(297,295)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

jac(295,295)=jac(295,295)-k(22);
jac(197,295)=jac(197,295)+k(22);
jac(63,295)=jac(63,295)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(295,295)=jac(295,295)-k(18);
jac(203,295)=jac(203,295)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(295,295)=jac(295,295)-k(16);
jac(3,295)=jac(3,295)+k(16);
jac(296,295)=jac(296,295)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(295);
jac(3,295)=jac(3,295)-k(9)*y(3);
jac(295,3)=jac(295,3)-k(9)*y(295);
jac(295,295)=jac(295,295)-k(9)*y(3);
jac(204,3)=jac(204,3)+k(9)*y(295);
jac(204,295)=jac(204,295)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(295,295)=jac(295,295)-k(8);
jac(62,295)=jac(62,295)+k(8);
jac(192,295)=jac(192,295)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(295,295)=jac(295,295)-k(6);
jac(61,295)=jac(61,295)+k(6);
jac(191,295)=jac(191,295)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(291,291)=jac(291,291)-k(34);
jac(287,291)=jac(287,291)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) 

jac(291,291)=jac(291,291)-k(33);
jac(272,291)=jac(272,291)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(291,291)=jac(291,291)-k(32);
jac(295,291)=jac(295,291)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

jac(291,291)=jac(291,291)-k(22);
jac(197,291)=jac(197,291)+k(22);
jac(56,291)=jac(56,291)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(291);
jac(197,291)=jac(197,291)-k(21)*y(197);
jac(291,197)=jac(291,197)-k(21)*y(291);
jac(291,291)=jac(291,291)-k(21)*y(197);
jac(294,197)=jac(294,197)+k(21)*y(291);
jac(294,291)=jac(294,291)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(291);
jac(32,291)=jac(32,291)-k(21)*y(32);
jac(291,32)=jac(291,32)-k(21)*y(291);
jac(291,291)=jac(291,291)-k(21)*y(32);
jac(293,32)=jac(293,32)+k(21)*y(291);
jac(293,291)=jac(293,291)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(2,2)=jac(2,2)-k(21)*y(291);
jac(2,291)=jac(2,291)-k(21)*y(2);
jac(291,2)=jac(291,2)-k(21)*y(291);
jac(291,291)=jac(291,291)-k(21)*y(2);
jac(262,2)=jac(262,2)+k(21)*y(291);
jac(262,291)=jac(262,291)+k(21)*y(2);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(291,291)=jac(291,291)-k(18);
jac(260,291)=jac(260,291)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(291,291)=jac(291,291)-k(16);
jac(3,291)=jac(3,291)+k(16);
jac(292,291)=jac(292,291)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(291);
jac(3,291)=jac(3,291)-k(9)*y(3);
jac(291,3)=jac(291,3)-k(9)*y(291);
jac(291,291)=jac(291,291)-k(9)*y(3);
jac(206,3)=jac(206,3)+k(9)*y(291);
jac(206,291)=jac(206,291)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(291,291)=jac(291,291)-k(8);
jac(55,291)=jac(55,291)+k(8);
jac(192,291)=jac(192,291)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(291,291)=jac(291,291)-k(6);
jac(54,291)=jac(54,291)+k(6);
jac(191,291)=jac(191,291)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(288,288)=jac(288,288)-k(34);
jac(283,288)=jac(283,288)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(288,288)=jac(288,288)-k(34);
jac(262,288)=jac(262,288)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(288,288)=jac(288,288)-k(33);
jac(273,288)=jac(273,288)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(288,288)=jac(288,288)-k(22);
jac(197,288)=jac(197,288)+k(22);
jac(282,288)=jac(282,288)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(288,288)=jac(288,288)-k(22);
jac(182,288)=jac(182,288)+k(22);
jac(291,288)=jac(291,288)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(288,288)=jac(288,288)-k(18);
jac(261,288)=jac(261,288)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(288,288)=jac(288,288)-k(16);
jac(3,288)=jac(3,288)+k(16);
jac(290,288)=jac(290,288)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(288);
jac(3,288)=jac(3,288)-k(9)*y(3);
jac(288,3)=jac(288,3)-k(9)*y(288);
jac(288,288)=jac(288,288)-k(9)*y(3);
jac(289,3)=jac(289,3)+k(9)*y(288);
jac(289,288)=jac(289,288)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(288,288)=jac(288,288)-k(8);
jac(192,288)=jac(192,288)+k(8);
jac(223,288)=jac(223,288)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(288,288)=jac(288,288)-k(6);
jac(191,288)=jac(191,288)+k(6);
jac(222,288)=jac(222,288)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(283,283)=jac(283,283)-k(34);
jac(277,283)=jac(277,283)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(283,283)=jac(283,283)-k(33);
jac(278,283)=jac(278,283)+k(33);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(283,283)=jac(283,283)-k(30);
jac(288,283)=jac(288,283)+k(30);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~Y) 

jac(283,283)=jac(283,283)-k(22);
jac(32,283)=jac(32,283)+k(22);
jac(282,283)=jac(282,283)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(283,283)=jac(283,283)-k(22);
jac(182,283)=jac(182,283)+k(22);
jac(287,283)=jac(287,283)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(283,283)=jac(283,283)-k(18);
jac(286,283)=jac(286,283)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(283,283)=jac(283,283)-k(16);
jac(3,283)=jac(3,283)+k(16);
jac(285,283)=jac(285,283)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(283);
jac(3,283)=jac(3,283)-k(9)*y(3);
jac(283,3)=jac(283,3)-k(9)*y(283);
jac(283,283)=jac(283,283)-k(9)*y(3);
jac(284,3)=jac(284,3)+k(9)*y(283);
jac(284,283)=jac(284,283)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(283,283)=jac(283,283)-k(8);
jac(27,283)=jac(27,283)+k(8);
jac(223,283)=jac(223,283)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(283,283)=jac(283,283)-k(6);
jac(86,283)=jac(86,283)+k(6);
jac(222,283)=jac(222,283)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(278,278)=jac(278,278)-k(34);
jac(274,278)=jac(274,278)+k(34);

% rule    : R13_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(278,278)=jac(278,278)-k(30);
jac(273,278)=jac(273,278)+k(30);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(278,278)=jac(278,278)-k(26);
jac(283,278)=jac(283,278)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(278,278)=jac(278,278)-k(22);
jac(2,278)=jac(2,278)+k(22);
jac(282,278)=jac(282,278)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(278,278)=jac(278,278)-k(22);
jac(182,278)=jac(182,278)+k(22);
jac(276,278)=jac(276,278)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(278,278)=jac(278,278)-k(18);
jac(281,278)=jac(281,278)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(278,278)=jac(278,278)-k(16);
jac(3,278)=jac(3,278)+k(16);
jac(280,278)=jac(280,278)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(278);
jac(3,278)=jac(3,278)-k(9)*y(3);
jac(278,3)=jac(278,3)-k(9)*y(278);
jac(278,278)=jac(278,278)-k(9)*y(3);
jac(279,3)=jac(279,3)+k(9)*y(278);
jac(279,278)=jac(279,278)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(278,278)=jac(278,278)-k(8);
jac(21,278)=jac(21,278)+k(8);
jac(223,278)=jac(223,278)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(278,278)=jac(278,278)-k(6);
jac(89,278)=jac(89,278)+k(6);
jac(222,278)=jac(222,278)+k(6);

% rule    : R12_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(274,274)=jac(274,274)-k(28);
jac(269,274)=jac(269,274)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(274,274)=jac(274,274)-k(27);
jac(278,274)=jac(278,274)+k(27);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(274,274)=jac(274,274)-k(26);
jac(277,274)=jac(277,274)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(274,274)=jac(274,274)-k(22);
jac(2,274)=jac(2,274)+k(22);
jac(49,274)=jac(49,274)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(274,274)=jac(274,274)-k(22);
jac(2,274)=jac(2,274)+k(22);
jac(276,274)=jac(276,274)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(274,274)=jac(274,274)-k(18);
jac(253,274)=jac(253,274)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(274,274)=jac(274,274)-k(16);
jac(3,274)=jac(3,274)+k(16);
jac(275,274)=jac(275,274)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(274);
jac(3,274)=jac(3,274)-k(9)*y(3);
jac(274,3)=jac(274,3)-k(9)*y(274);
jac(274,274)=jac(274,274)-k(9)*y(3);
jac(51,3)=jac(51,3)+k(9)*y(274);
jac(51,274)=jac(51,274)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(274,274)=jac(274,274)-k(8);
jac(21,274)=jac(21,274)+k(8);
jac(44,274)=jac(44,274)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(274,274)=jac(274,274)-k(6);
jac(89,274)=jac(89,274)+k(6);
jac(43,274)=jac(43,274)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(269,269)=jac(269,269)-k(34);
jac(274,269)=jac(274,269)+k(34);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(269,269)=jac(269,269)-k(29);
jac(273,269)=jac(273,269)+k(29);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(269,269)=jac(269,269)-k(26);
jac(262,269)=jac(262,269)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(269,269)=jac(269,269)-k(22);
jac(182,269)=jac(182,269)+k(22);
jac(49,269)=jac(49,269)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(269,269)=jac(269,269)-k(22);
jac(2,269)=jac(2,269)+k(22);
jac(272,269)=jac(272,269)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(269,269)=jac(269,269)-k(18);
jac(263,269)=jac(263,269)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(269,269)=jac(269,269)-k(16);
jac(3,269)=jac(3,269)+k(16);
jac(271,269)=jac(271,269)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(269);
jac(3,269)=jac(3,269)-k(9)*y(3);
jac(269,3)=jac(269,3)-k(9)*y(269);
jac(269,269)=jac(269,269)-k(9)*y(3);
jac(270,3)=jac(270,3)+k(9)*y(269);
jac(270,269)=jac(270,269)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(269,269)=jac(269,269)-k(8);
jac(177,269)=jac(177,269)+k(8);
jac(44,269)=jac(44,269)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(269,269)=jac(269,269)-k(6);
jac(176,269)=jac(176,269)+k(6);
jac(43,269)=jac(43,269)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(263,263)=jac(263,263)-k(34);
jac(253,263)=jac(253,263)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(263,263)=jac(263,263)-k(31);
jac(269,263)=jac(269,263)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(263,263)=jac(263,263)-k(29);
jac(268,263)=jac(268,263)+k(29);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(263,263)=jac(263,263)-k(26);
jac(255,263)=jac(255,263)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(263,263)=jac(263,263)-k(22);
jac(182,263)=jac(182,263)+k(22);
jac(250,263)=jac(250,263)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(263,263)=jac(263,263)-k(22);
jac(2,263)=jac(2,263)+k(22);
jac(267,263)=jac(267,263)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(263,263)=jac(263,263)-k(16);
jac(3,263)=jac(3,263)+k(16);
jac(266,263)=jac(266,263)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(263);
jac(3,263)=jac(3,263)-k(15)*y(3);
jac(263,3)=jac(263,3)-k(15)*y(263);
jac(263,263)=jac(263,263)-k(15)*y(3);
jac(265,3)=jac(265,3)+k(15)*y(263);
jac(265,263)=jac(265,263)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(263,263)=jac(263,263)-k(8);
jac(218,263)=jac(218,263)+k(8);
jac(44,263)=jac(44,263)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(263,263)=jac(263,263)-k(6);
jac(264,263)=jac(264,263)+k(6);
jac(43,263)=jac(43,263)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(255,255)=jac(255,255)-k(34);
jac(254,255)=jac(254,255)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(255,255)=jac(255,255)-k(33);
jac(263,255)=jac(263,255)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(255,255)=jac(255,255)-k(31);
jac(262,255)=jac(262,255)+k(31);

% rule    : R13
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(255,255)=jac(255,255)-k(29);
jac(261,255)=jac(261,255)+k(29);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) 

jac(255,255)=jac(255,255)-k(22);
jac(197,255)=jac(197,255)+k(22);
jac(250,255)=jac(250,255)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(255,255)=jac(255,255)-k(22);
jac(2,255)=jac(2,255)+k(22);
jac(260,255)=jac(260,255)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(255,255)=jac(255,255)-k(16);
jac(3,255)=jac(3,255)+k(16);
jac(259,255)=jac(259,255)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(255);
jac(3,255)=jac(3,255)-k(15)*y(3);
jac(255,3)=jac(255,3)-k(15)*y(255);
jac(255,255)=jac(255,255)-k(15)*y(3);
jac(258,3)=jac(258,3)+k(15)*y(255);
jac(258,255)=jac(258,255)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(255,255)=jac(255,255)-k(8);
jac(257,255)=jac(257,255)+k(8);
jac(44,255)=jac(44,255)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(255,255)=jac(255,255)-k(6);
jac(256,255)=jac(256,255)+k(6);
jac(43,255)=jac(43,255)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(250,250)=jac(250,250)-k(32);
jac(239,250)=jac(239,250)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(250,250)=jac(250,250)-k(31);
jac(49,250)=jac(49,250)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

jac(250,250)=jac(250,250)-k(22);
jac(2,250)=jac(2,250)+k(22);
jac(238,250)=jac(238,250)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(250);
jac(197,250)=jac(197,250)-k(21)*y(197);
jac(250,197)=jac(250,197)-k(21)*y(250);
jac(250,250)=jac(250,250)-k(21)*y(197);
jac(255,197)=jac(255,197)+k(21)*y(250);
jac(255,250)=jac(255,250)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(250);
jac(32,250)=jac(32,250)-k(21)*y(32);
jac(250,32)=jac(250,32)-k(21)*y(250);
jac(250,250)=jac(250,250)-k(21)*y(32);
jac(254,32)=jac(254,32)+k(21)*y(250);
jac(254,250)=jac(254,250)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(250);
jac(2,250)=jac(2,250)-k(21)*y(2);
jac(250,2)=jac(250,2)-k(21)*y(250);
jac(250,250)=jac(250,250)-k(21)*y(2);
jac(253,2)=jac(253,2)+k(21)*y(250);
jac(253,250)=jac(253,250)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(250,250)=jac(250,250)-k(16);
jac(3,250)=jac(3,250)+k(16);
jac(252,250)=jac(252,250)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(250);
jac(3,250)=jac(3,250)-k(15)*y(3);
jac(250,3)=jac(250,3)-k(15)*y(250);
jac(250,250)=jac(250,250)-k(15)*y(3);
jac(251,3)=jac(251,3)+k(15)*y(250);
jac(251,250)=jac(251,250)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~pY) 

jac(250,250)=jac(250,250)-k(8);
jac(77,250)=jac(77,250)+k(8);
jac(44,250)=jac(44,250)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(250,250)=jac(250,250)-k(6);
jac(76,250)=jac(76,250)+k(6);
jac(43,250)=jac(43,250)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(243,243)=jac(243,243)-k(33);
jac(250,243)=jac(250,243)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(243,243)=jac(243,243)-k(32);
jac(240,243)=jac(240,243)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(243,243)=jac(243,243)-k(31);
jac(249,243)=jac(249,243)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

jac(243,243)=jac(243,243)-k(22);
jac(32,243)=jac(32,243)+k(22);
jac(238,243)=jac(238,243)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(243);
jac(197,243)=jac(197,243)-k(21)*y(197);
jac(243,197)=jac(243,197)-k(21)*y(243);
jac(243,243)=jac(243,243)-k(21)*y(197);
jac(248,197)=jac(248,197)+k(21)*y(243);
jac(248,243)=jac(248,243)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(243);
jac(32,243)=jac(32,243)-k(21)*y(32);
jac(243,32)=jac(243,32)-k(21)*y(243);
jac(243,243)=jac(243,243)-k(21)*y(32);
jac(247,32)=jac(247,32)+k(21)*y(243);
jac(247,243)=jac(247,243)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(243);
jac(2,243)=jac(2,243)-k(21)*y(2);
jac(243,2)=jac(243,2)-k(21)*y(243);
jac(243,243)=jac(243,243)-k(21)*y(2);
jac(246,2)=jac(246,2)+k(21)*y(243);
jac(246,243)=jac(246,243)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(243,243)=jac(243,243)-k(16);
jac(3,243)=jac(3,243)+k(16);
jac(245,243)=jac(245,243)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(243);
jac(3,243)=jac(3,243)-k(15)*y(3);
jac(243,3)=jac(243,3)-k(15)*y(243);
jac(243,243)=jac(243,243)-k(15)*y(3);
jac(244,3)=jac(244,3)+k(15)*y(243);
jac(244,243)=jac(244,243)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~pY) 

jac(243,243)=jac(243,243)-k(8);
jac(77,243)=jac(77,243)+k(8);
jac(38,243)=jac(38,243)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(243,243)=jac(243,243)-k(6);
jac(76,243)=jac(76,243)+k(6);
jac(37,243)=jac(37,243)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(240,240)=jac(240,240)-k(33);
jac(239,240)=jac(239,240)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(240,240)=jac(240,240)-k(31);
jac(39,240)=jac(39,240)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

jac(240,240)=jac(240,240)-k(22);
jac(32,240)=jac(32,240)+k(22);
jac(235,240)=jac(235,240)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(240,240)=jac(240,240)-k(20);
jac(243,240)=jac(243,240)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(240,240)=jac(240,240)-k(16);
jac(3,240)=jac(3,240)+k(16);
jac(242,240)=jac(242,240)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(3,3)=jac(3,3)-k(15)*y(240);
jac(3,240)=jac(3,240)-k(15)*y(3);
jac(240,3)=jac(240,3)-k(15)*y(240);
jac(240,240)=jac(240,240)-k(15)*y(3);
jac(241,3)=jac(241,3)+k(15)*y(240);
jac(241,240)=jac(241,240)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

jac(240,240)=jac(240,240)-k(8);
jac(72,240)=jac(72,240)+k(8);
jac(38,240)=jac(38,240)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(240,240)=jac(240,240)-k(6);
jac(71,240)=jac(71,240)+k(6);
jac(37,240)=jac(37,240)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(235,235)=jac(235,235)-k(32);
jac(70,235)=jac(70,235)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(235,235)=jac(235,235)-k(31);
jac(233,235)=jac(233,235)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(235);
jac(197,235)=jac(197,235)-k(21)*y(197);
jac(235,197)=jac(235,197)-k(21)*y(235);
jac(235,235)=jac(235,235)-k(21)*y(197);
jac(231,197)=jac(231,197)+k(21)*y(235);
jac(231,235)=jac(231,235)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(235);
jac(32,235)=jac(32,235)-k(21)*y(32);
jac(235,32)=jac(235,32)-k(21)*y(235);
jac(235,235)=jac(235,235)-k(21)*y(32);
jac(240,32)=jac(240,32)+k(21)*y(235);
jac(240,235)=jac(240,235)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(235);
jac(2,235)=jac(2,235)-k(21)*y(2);
jac(235,2)=jac(235,2)-k(21)*y(235);
jac(235,235)=jac(235,235)-k(21)*y(2);
jac(239,2)=jac(239,2)+k(21)*y(235);
jac(239,235)=jac(239,235)+k(21)*y(2);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(235,235)=jac(235,235)-k(20);
jac(238,235)=jac(238,235)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

jac(235,235)=jac(235,235)-k(16);
jac(3,235)=jac(3,235)+k(16);
jac(237,235)=jac(237,235)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(235);
jac(3,235)=jac(3,235)-k(15)*y(3);
jac(235,3)=jac(235,3)-k(15)*y(235);
jac(235,235)=jac(235,235)-k(15)*y(3);
jac(236,3)=jac(236,3)+k(15)*y(235);
jac(236,235)=jac(236,235)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) 

jac(235,235)=jac(235,235)-k(8);
jac(72,235)=jac(72,235)+k(8);
jac(55,235)=jac(55,235)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(235,235)=jac(235,235)-k(6);
jac(71,235)=jac(71,235)+k(6);
jac(54,235)=jac(54,235)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(233,233)=jac(233,233)-k(32);
jac(68,233)=jac(68,233)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(233);
jac(197,233)=jac(197,233)-k(21)*y(197);
jac(233,197)=jac(233,197)-k(21)*y(233);
jac(233,233)=jac(233,233)-k(21)*y(197);
jac(229,197)=jac(229,197)+k(21)*y(233);
jac(229,233)=jac(229,233)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(233);
jac(32,233)=jac(32,233)-k(21)*y(32);
jac(233,32)=jac(233,32)-k(21)*y(233);
jac(233,233)=jac(233,233)-k(21)*y(32);
jac(39,32)=jac(39,32)+k(21)*y(233);
jac(39,233)=jac(39,233)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(233);
jac(2,233)=jac(2,233)-k(21)*y(2);
jac(233,2)=jac(233,2)-k(21)*y(233);
jac(233,233)=jac(233,233)-k(21)*y(2);
jac(45,2)=jac(45,2)+k(21)*y(233);
jac(45,233)=jac(45,233)+k(21)*y(2);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(233,233)=jac(233,233)-k(20);
jac(56,233)=jac(56,233)+k(20);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(233,233)=jac(233,233)-k(18);
jac(235,233)=jac(235,233)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

jac(233,233)=jac(233,233)-k(16);
jac(3,233)=jac(3,233)+k(16);
jac(234,233)=jac(234,233)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(233);
jac(3,233)=jac(3,233)-k(9)*y(3);
jac(233,3)=jac(233,3)-k(9)*y(233);
jac(233,233)=jac(233,233)-k(9)*y(3);
jac(41,3)=jac(41,3)+k(9)*y(233);
jac(41,233)=jac(41,233)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) 

jac(233,233)=jac(233,233)-k(8);
jac(1,233)=jac(1,233)+k(8);
jac(55,233)=jac(55,233)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(233,233)=jac(233,233)-k(6);
jac(6,233)=jac(6,233)+k(6);
jac(54,233)=jac(54,233)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) 

jac(229,229)=jac(229,229)-k(34);
jac(39,229)=jac(39,229)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) 

jac(229,229)=jac(229,229)-k(33);
jac(224,229)=jac(224,229)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) 

jac(229,229)=jac(229,229)-k(22);
jac(197,229)=jac(197,229)+k(22);
jac(233,229)=jac(233,229)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(229,229)=jac(229,229)-k(20);
jac(232,229)=jac(232,229)+k(20);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) 

jac(229,229)=jac(229,229)-k(18);
jac(231,229)=jac(231,229)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(229,229)=jac(229,229)-k(16);
jac(3,229)=jac(3,229)+k(16);
jac(230,229)=jac(230,229)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(229);
jac(3,229)=jac(3,229)-k(9)*y(3);
jac(229,3)=jac(229,3)-k(9)*y(229);
jac(229,229)=jac(229,229)-k(9)*y(3);
jac(226,3)=jac(226,3)+k(9)*y(229);
jac(226,229)=jac(226,229)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y, g~Y) 

jac(229,229)=jac(229,229)-k(8);
jac(1,229)=jac(1,229)+k(8);
jac(228,229)=jac(228,229)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(229,229)=jac(229,229)-k(6);
jac(6,229)=jac(6,229)+k(6);
jac(227,229)=jac(227,229)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(226,226)=jac(226,226)-k(34);
jac(36,226)=jac(36,226)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(226,226)=jac(226,226)-k(33);
jac(221,226)=jac(221,226)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(226,226)=jac(226,226)-k(22);
jac(197,226)=jac(197,226)+k(22);
jac(41,226)=jac(41,226)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(226,226)=jac(226,226)-k(20);
jac(205,226)=jac(205,226)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(226,226)=jac(226,226)-k(16);
jac(3,226)=jac(3,226)+k(16);
jac(212,226)=jac(212,226)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Lyn(U, SH2) 

jac(226,226)=jac(226,226)-k(10);
jac(3,226)=jac(3,226)+k(10);
jac(229,226)=jac(229,226)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(226,226)=jac(226,226)-k(8);
jac(7,226)=jac(7,226)+k(8);
jac(228,226)=jac(228,226)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(226,226)=jac(226,226)-k(6);
jac(9,226)=jac(9,226)+k(6);
jac(227,226)=jac(227,226)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(221,221)=jac(221,221)-k(34);
jac(42,221)=jac(42,221)+k(34);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~pY), Lyn(U!5, SH2) 

jac(221,221)=jac(221,221)-k(23);
jac(226,221)=jac(226,221)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(221,221)=jac(221,221)-k(22);
jac(182,221)=jac(182,221)+k(22);
jac(41,221)=jac(41,221)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY), Lyn(U!5, SH2) 

jac(221,221)=jac(221,221)-k(20);
jac(225,221)=jac(225,221)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(221,221)=jac(221,221)-k(16);
jac(3,221)=jac(3,221)+k(16);
jac(217,221)=jac(217,221)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Lyn(U, SH2) 

jac(221,221)=jac(221,221)-k(10);
jac(3,221)=jac(3,221)+k(10);
jac(224,221)=jac(224,221)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(221,221)=jac(221,221)-k(8);
jac(7,221)=jac(7,221)+k(8);
jac(223,221)=jac(223,221)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(221,221)=jac(221,221)-k(6);
jac(9,221)=jac(9,221)+k(6);
jac(222,221)=jac(222,221)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(217,217)=jac(217,217)-k(34);
jac(25,217)=jac(25,217)+k(34);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(217,217)=jac(217,217)-k(31);
jac(213,217)=jac(213,217)+k(31);

% rule    : R10
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(217,217)=jac(217,217)-k(23);
jac(212,217)=jac(212,217)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(217,217)=jac(217,217)-k(22);
jac(182,217)=jac(182,217)+k(22);
jac(19,217)=jac(19,217)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(217);
jac(3,217)=jac(3,217)-k(15)*y(3);
jac(217,3)=jac(217,3)-k(15)*y(217);
jac(217,217)=jac(217,217)-k(15)*y(3);
jac(221,3)=jac(221,3)+k(15)*y(217);
jac(221,217)=jac(221,217)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(217,217)=jac(217,217)-k(10);
jac(3,217)=jac(3,217)+k(10);
jac(220,217)=jac(220,217)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(217,217)=jac(217,217)-k(8);
jac(7,217)=jac(7,217)+k(8);
jac(219,217)=jac(219,217)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(217,217)=jac(217,217)-k(6);
jac(9,217)=jac(9,217)+k(6);
jac(218,217)=jac(218,217)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(213,213)=jac(213,213)-k(34);
jac(20,213)=jac(20,213)+k(34);

% rule    : R10
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(213,213)=jac(213,213)-k(23);
jac(208,213)=jac(208,213)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~pY) 

jac(213,213)=jac(213,213)-k(22);
jac(182,213)=jac(182,213)+k(22);
jac(14,213)=jac(14,213)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(213,213)=jac(213,213)-k(11);
jac(217,213)=jac(217,213)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) 

jac(213,213)=jac(213,213)-k(10);
jac(3,213)=jac(3,213)+k(10);
jac(216,213)=jac(216,213)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(213);
jac(3,213)=jac(3,213)-k(9)*y(3);
jac(213,3)=jac(213,3)-k(9)*y(213);
jac(213,213)=jac(213,213)-k(9)*y(3);
jac(215,3)=jac(215,3)+k(9)*y(213);
jac(215,213)=jac(215,213)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(213,213)=jac(213,213)-k(8);
jac(7,213)=jac(7,213)+k(8);
jac(214,213)=jac(214,213)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(213,213)=jac(213,213)-k(6);
jac(9,213)=jac(9,213)+k(6);
jac(177,213)=jac(177,213)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(208,208)=jac(208,208)-k(34);
jac(26,208)=jac(26,208)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~pY) 

jac(208,208)=jac(208,208)-k(33);
jac(213,208)=jac(213,208)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) 

jac(208,208)=jac(208,208)-k(22);
jac(197,208)=jac(197,208)+k(22);
jac(14,208)=jac(14,208)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~pY) 

jac(208,208)=jac(208,208)-k(11);
jac(212,208)=jac(212,208)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~pY) + Lyn(U, SH2) 

jac(208,208)=jac(208,208)-k(10);
jac(3,208)=jac(3,208)+k(10);
jac(211,208)=jac(211,208)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(208);
jac(3,208)=jac(3,208)-k(9)*y(3);
jac(208,3)=jac(208,3)-k(9)*y(208);
jac(208,208)=jac(208,208)-k(9)*y(3);
jac(210,3)=jac(210,3)+k(9)*y(208);
jac(210,208)=jac(210,208)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~pY) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(208,208)=jac(208,208)-k(8);
jac(7,208)=jac(7,208)+k(8);
jac(209,208)=jac(209,208)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(208,208)=jac(208,208)-k(6);
jac(9,208)=jac(9,208)+k(6);
jac(192,208)=jac(192,208)+k(6);

% rule    : R19
% reaction: Syk(tSH2, l~pY, a~pY) -> Syk(tSH2, l~pY, a~Y) 

jac(197,197)=jac(197,197)-k(36);
jac(32,197)=jac(32,197)+k(36);

% rule    : R18
% reaction: Syk(tSH2, l~pY, a~pY) -> Syk(tSH2, l~Y, a~pY) 

jac(197,197)=jac(197,197)-k(35);
jac(182,197)=jac(182,197)+k(35);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(14);
jac(197,14)=jac(197,14)-k(21)*y(197);
jac(14,197)=jac(14,197)-k(21)*y(14);
jac(14,14)=jac(14,14)-k(21)*y(197);
jac(208,197)=jac(208,197)+k(21)*y(14);
jac(208,14)=jac(208,14)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(46);
jac(197,46)=jac(197,46)-k(21)*y(197);
jac(46,197)=jac(46,197)-k(21)*y(46);
jac(46,46)=jac(46,46)-k(21)*y(197);
jac(207,197)=jac(207,197)+k(21)*y(46);
jac(207,46)=jac(207,46)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(53);
jac(197,53)=jac(197,53)-k(21)*y(197);
jac(53,197)=jac(53,197)-k(21)*y(53);
jac(53,53)=jac(53,53)-k(21)*y(197);
jac(206,197)=jac(206,197)+k(21)*y(53);
jac(206,53)=jac(206,53)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(197,197)=jac(197,197)-k(21)*y(53);
jac(197,53)=jac(197,53)-k(21)*y(197);
jac(53,197)=jac(53,197)-k(21)*y(53);
jac(53,53)=jac(53,53)-k(21)*y(197);
jac(205,197)=jac(205,197)+k(21)*y(53);
jac(205,53)=jac(205,53)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(60);
jac(197,60)=jac(197,60)-k(21)*y(197);
jac(60,197)=jac(60,197)-k(21)*y(60);
jac(60,60)=jac(60,60)-k(21)*y(197);
jac(204,197)=jac(204,197)+k(21)*y(60);
jac(204,60)=jac(204,60)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(75);
jac(197,75)=jac(197,75)-k(21)*y(197);
jac(75,197)=jac(75,197)-k(21)*y(75);
jac(75,75)=jac(75,75)-k(21)*y(197);
jac(203,197)=jac(203,197)+k(21)*y(75);
jac(203,75)=jac(203,75)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(97);
jac(197,97)=jac(197,97)-k(21)*y(197);
jac(97,197)=jac(97,197)-k(21)*y(97);
jac(97,97)=jac(97,97)-k(21)*y(197);
jac(202,197)=jac(202,197)+k(21)*y(97);
jac(202,97)=jac(202,97)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~pY), Lyn(U, SH2!4) 

jac(197,197)=jac(197,197)-k(21)*y(117);
jac(197,117)=jac(197,117)-k(21)*y(197);
jac(117,197)=jac(117,197)-k(21)*y(117);
jac(117,117)=jac(117,117)-k(21)*y(197);
jac(201,197)=jac(201,197)+k(21)*y(117);
jac(201,117)=jac(201,117)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~pY), Lyn(U!4, SH2) 

jac(197,197)=jac(197,197)-k(21)*y(125);
jac(197,125)=jac(197,125)-k(21)*y(197);
jac(125,197)=jac(125,197)-k(21)*y(125);
jac(125,125)=jac(125,125)-k(21)*y(197);
jac(200,197)=jac(200,197)+k(21)*y(125);
jac(200,125)=jac(200,125)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(146);
jac(197,146)=jac(197,146)-k(21)*y(197);
jac(146,197)=jac(146,197)-k(21)*y(146);
jac(146,146)=jac(146,146)-k(21)*y(197);
jac(199,197)=jac(199,197)+k(21)*y(146);
jac(199,146)=jac(199,146)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(147);
jac(197,147)=jac(197,147)-k(21)*y(197);
jac(147,197)=jac(147,197)-k(21)*y(147);
jac(147,147)=jac(147,147)-k(21)*y(197);
jac(196,197)=jac(196,197)+k(21)*y(147);
jac(196,147)=jac(196,147)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(147);
jac(197,147)=jac(197,147)-k(21)*y(197);
jac(147,197)=jac(147,197)-k(21)*y(147);
jac(147,147)=jac(147,147)-k(21)*y(197);
jac(198,197)=jac(198,197)+k(21)*y(147);
jac(198,147)=jac(198,147)+k(21)*y(197);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(197,197)=jac(197,197)-k(21)*y(154);
jac(197,154)=jac(197,154)-k(21)*y(197);
jac(154,197)=jac(154,197)-k(21)*y(154);
jac(154,154)=jac(154,154)-k(21)*y(197);
jac(163,197)=jac(163,197)+k(21)*y(154);
jac(163,154)=jac(163,154)+k(21)*y(197);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(190,190)=jac(190,190)-k(34);
jac(168,190)=jac(168,190)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(190,190)=jac(190,190)-k(34);
jac(169,190)=jac(169,190)+k(34);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(190,190)=jac(190,190)-k(33);
jac(183,190)=jac(183,190)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~pY, a~pY) 

jac(190,190)=jac(190,190)-k(22);
jac(197,190)=jac(197,190)+k(22);
jac(189,190)=jac(189,190)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(190,190)=jac(190,190)-k(22);
jac(182,190)=jac(182,190)+k(22);
jac(196,190)=jac(196,190)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(190,190)=jac(190,190)-k(12);
jac(195,190)=jac(195,190)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~pY, a~pY) + Lyn(U, SH2) 

jac(190,190)=jac(190,190)-k(10);
jac(3,190)=jac(3,190)+k(10);
jac(194,190)=jac(194,190)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~pY, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(190);
jac(3,190)=jac(3,190)-k(9)*y(3);
jac(190,3)=jac(190,3)-k(9)*y(190);
jac(190,190)=jac(190,190)-k(9)*y(3);
jac(193,3)=jac(193,3)+k(9)*y(190);
jac(193,190)=jac(193,190)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~pY) 

jac(190,190)=jac(190,190)-k(8);
jac(192,190)=jac(192,190)+k(8);
jac(185,190)=jac(185,190)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~pY) 

jac(190,190)=jac(190,190)-k(6);
jac(191,190)=jac(191,190)+k(6);
jac(184,190)=jac(184,190)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(183,183)=jac(183,183)-k(34);
jac(174,183)=jac(174,183)+k(34);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(183,183)=jac(183,183)-k(34);
jac(175,183)=jac(175,183)+k(34);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~pY) 

jac(183,183)=jac(183,183)-k(24);
jac(190,183)=jac(190,183)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(183,183)=jac(183,183)-k(22);
jac(182,183)=jac(182,183)+k(22);
jac(189,183)=jac(189,183)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~pY) 

jac(183,183)=jac(183,183)-k(22);
jac(182,183)=jac(182,183)+k(22);
jac(181,183)=jac(181,183)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(183,183)=jac(183,183)-k(12);
jac(188,183)=jac(188,183)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~pY), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(183,183)=jac(183,183)-k(10);
jac(3,183)=jac(3,183)+k(10);
jac(187,183)=jac(187,183)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(183);
jac(3,183)=jac(3,183)-k(9)*y(3);
jac(183,3)=jac(183,3)-k(9)*y(183);
jac(183,183)=jac(183,183)-k(9)*y(3);
jac(186,3)=jac(186,3)+k(9)*y(183);
jac(186,183)=jac(186,183)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(183,183)=jac(183,183)-k(8);
jac(177,183)=jac(177,183)+k(8);
jac(185,183)=jac(185,183)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~pY) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(183,183)=jac(183,183)-k(6);
jac(176,183)=jac(176,183)+k(6);
jac(184,183)=jac(184,183)+k(6);

% rule    : R17
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(175,175)=jac(175,175)-k(34);
jac(170,175)=jac(170,175)+k(34);

% rule    : R13
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~pY) 

jac(175,175)=jac(175,175)-k(29);
jac(183,175)=jac(183,175)+k(29);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(175,175)=jac(175,175)-k(24);
jac(169,175)=jac(169,175)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~pY) 

jac(175,175)=jac(175,175)-k(22);
jac(182,175)=jac(182,175)+k(22);
jac(145,175)=jac(145,175)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~pY) + Syk(tSH2, l~Y, a~Y) 

jac(175,175)=jac(175,175)-k(22);
jac(2,175)=jac(2,175)+k(22);
jac(181,175)=jac(181,175)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(175,175)=jac(175,175)-k(12);
jac(180,175)=jac(180,175)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~pY) + Lyn(U, SH2) 

jac(175,175)=jac(175,175)-k(10);
jac(3,175)=jac(3,175)+k(10);
jac(179,175)=jac(179,175)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~pY), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(175);
jac(3,175)=jac(3,175)-k(9)*y(3);
jac(175,3)=jac(175,3)-k(9)*y(175);
jac(175,175)=jac(175,175)-k(9)*y(3);
jac(178,3)=jac(178,3)+k(9)*y(175);
jac(178,175)=jac(178,175)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~pY) 

jac(175,175)=jac(175,175)-k(8);
jac(177,175)=jac(177,175)+k(8);
jac(141,175)=jac(141,175)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~pY) 

jac(175,175)=jac(175,175)-k(6);
jac(176,175)=jac(176,175)+k(6);
jac(140,175)=jac(140,175)+k(6);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~pY) 

jac(170,170)=jac(170,170)-k(28);
jac(175,170)=jac(175,170)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~Y, a~Y) 

jac(170,170)=jac(170,170)-k(27);
jac(174,170)=jac(174,170)+k(27);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(170,170)=jac(170,170)-k(24);
jac(164,170)=jac(164,170)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(170,170)=jac(170,170)-k(22);
jac(2,170)=jac(2,170)+k(22);
jac(145,170)=jac(145,170)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(170,170)=jac(170,170)-k(22);
jac(2,170)=jac(2,170)+k(22);
jac(152,170)=jac(152,170)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(170,170)=jac(170,170)-k(12);
jac(173,170)=jac(173,170)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(170,170)=jac(170,170)-k(10);
jac(3,170)=jac(3,170)+k(10);
jac(172,170)=jac(172,170)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(170);
jac(3,170)=jac(3,170)-k(9)*y(3);
jac(170,3)=jac(170,3)-k(9)*y(170);
jac(170,170)=jac(170,170)-k(9)*y(3);
jac(171,3)=jac(171,3)+k(9)*y(170);
jac(171,170)=jac(171,170)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) 

jac(170,170)=jac(170,170)-k(8);
jac(21,170)=jac(21,170)+k(8);
jac(141,170)=jac(141,170)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) 

jac(170,170)=jac(170,170)-k(6);
jac(89,170)=jac(89,170)+k(6);
jac(140,170)=jac(140,170)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(164,164)=jac(164,164)-k(33);
jac(170,164)=jac(170,164)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(164,164)=jac(164,164)-k(28);
jac(169,164)=jac(169,164)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(164,164)=jac(164,164)-k(27);
jac(168,164)=jac(168,164)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(164,164)=jac(164,164)-k(22);
jac(32,164)=jac(32,164)+k(22);
jac(145,164)=jac(145,164)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) 

jac(164,164)=jac(164,164)-k(22);
jac(2,164)=jac(2,164)+k(22);
jac(153,164)=jac(153,164)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(164,164)=jac(164,164)-k(12);
jac(167,164)=jac(167,164)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~Y, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(164,164)=jac(164,164)-k(10);
jac(3,164)=jac(3,164)+k(10);
jac(166,164)=jac(166,164)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(164);
jac(3,164)=jac(3,164)-k(9)*y(3);
jac(164,3)=jac(164,3)-k(9)*y(164);
jac(164,164)=jac(164,164)-k(9)*y(3);
jac(165,3)=jac(165,3)+k(9)*y(164);
jac(165,164)=jac(165,164)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(164,164)=jac(164,164)-k(8);
jac(27,164)=jac(27,164)+k(8);
jac(141,164)=jac(141,164)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(164,164)=jac(164,164)-k(6);
jac(86,164)=jac(86,164)+k(6);
jac(140,164)=jac(140,164)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(158,158)=jac(158,158)-k(33);
jac(157,158)=jac(157,158)+k(33);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(158,158)=jac(158,158)-k(33);
jac(164,158)=jac(164,158)+k(33);

% rule    : R12_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~pY) 

jac(158,158)=jac(158,158)-k(28);
jac(163,158)=jac(163,158)+k(28);

% rule    : R12
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~pY), Syk(tSH2!5, l~pY, a~Y) 

jac(158,158)=jac(158,158)-k(27);
jac(162,158)=jac(162,158)+k(27);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(158,158)=jac(158,158)-k(22);
jac(32,158)=jac(32,158)+k(22);
jac(154,158)=jac(154,158)+k(22);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) 

jac(158,158)=jac(158,158)-k(22);
jac(32,158)=jac(32,158)+k(22);
jac(153,158)=jac(153,158)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(158,158)=jac(158,158)-k(12);
jac(161,158)=jac(161,158)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Syk(tSH2!2, l~pY, a~Y), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) 

jac(158,158)=jac(158,158)-k(10);
jac(3,158)=jac(3,158)+k(10);
jac(160,158)=jac(160,158)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(158);
jac(3,158)=jac(3,158)-k(9)*y(3);
jac(158,3)=jac(158,3)-k(9)*y(158);
jac(158,158)=jac(158,158)-k(9)*y(3);
jac(159,3)=jac(159,3)+k(9)*y(158);
jac(159,158)=jac(159,158)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) 

jac(158,158)=jac(158,158)-k(8);
jac(27,158)=jac(27,158)+k(8);
jac(132,158)=jac(132,158)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) 

jac(158,158)=jac(158,158)-k(6);
jac(86,158)=jac(86,158)+k(6);
jac(131,158)=jac(131,158)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(154,154)=jac(154,154)-k(33);
jac(145,154)=jac(145,154)+k(33);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(154,154)=jac(154,154)-k(32);
jac(138,154)=jac(138,154)+k(32);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(154,154)=jac(154,154)-k(22);
jac(32,154)=jac(32,154)+k(22);
jac(147,154)=jac(147,154)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(154);
jac(32,154)=jac(32,154)-k(21)*y(32);
jac(154,32)=jac(154,32)-k(21)*y(154);
jac(154,154)=jac(154,154)-k(21)*y(32);
jac(158,32)=jac(158,32)+k(21)*y(154);
jac(158,154)=jac(158,154)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Syk(tSH2!5, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(154);
jac(2,154)=jac(2,154)-k(21)*y(2);
jac(154,2)=jac(154,2)-k(21)*y(154);
jac(154,154)=jac(154,154)-k(21)*y(2);
jac(157,2)=jac(157,2)+k(21)*y(154);
jac(157,154)=jac(157,154)+k(21)*y(2);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(154,154)=jac(154,154)-k(12);
jac(137,154)=jac(137,154)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(154,154)=jac(154,154)-k(10);
jac(3,154)=jac(3,154)+k(10);
jac(156,154)=jac(156,154)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(154);
jac(3,154)=jac(3,154)-k(9)*y(3);
jac(154,3)=jac(154,3)-k(9)*y(154);
jac(154,154)=jac(154,154)-k(9)*y(3);
jac(155,3)=jac(155,3)+k(9)*y(154);
jac(155,154)=jac(155,154)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y, g~pY) 

jac(154,154)=jac(154,154)-k(8);
jac(15,154)=jac(15,154)+k(8);
jac(132,154)=jac(132,154)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(154,154)=jac(154,154)-k(6);
jac(98,154)=jac(98,154)+k(6);
jac(131,154)=jac(131,154)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(147,147)=jac(147,147)-k(32);
jac(146,147)=jac(146,147)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(147,147)=jac(147,147)-k(32);
jac(100,147)=jac(100,147)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(147);
jac(32,147)=jac(32,147)-k(21)*y(32);
jac(147,32)=jac(147,32)-k(21)*y(147);
jac(147,147)=jac(147,147)-k(21)*y(32);
jac(154,32)=jac(154,32)+k(21)*y(147);
jac(154,147)=jac(154,147)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(147);
jac(2,147)=jac(2,147)-k(21)*y(2);
jac(147,2)=jac(147,2)-k(21)*y(147);
jac(147,147)=jac(147,147)-k(21)*y(2);
jac(145,2)=jac(145,2)+k(21)*y(147);
jac(145,147)=jac(145,147)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(147);
jac(32,147)=jac(32,147)-k(21)*y(32);
jac(147,32)=jac(147,32)-k(21)*y(147);
jac(147,147)=jac(147,147)-k(21)*y(32);
jac(153,32)=jac(153,32)+k(21)*y(147);
jac(153,147)=jac(153,147)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(147);
jac(2,147)=jac(2,147)-k(21)*y(2);
jac(147,2)=jac(147,2)-k(21)*y(147);
jac(147,147)=jac(147,147)-k(21)*y(2);
jac(152,2)=jac(152,2)+k(21)*y(147);
jac(152,147)=jac(152,147)+k(21)*y(2);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) 

jac(147,147)=jac(147,147)-k(12);
jac(151,147)=jac(151,147)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) 

jac(147,147)=jac(147,147)-k(10);
jac(3,147)=jac(3,147)+k(10);
jac(150,147)=jac(150,147)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(147);
jac(3,147)=jac(3,147)-k(9)*y(3);
jac(147,3)=jac(147,3)-k(9)*y(147);
jac(147,147)=jac(147,147)-k(9)*y(3);
jac(149,3)=jac(149,3)+k(9)*y(147);
jac(149,147)=jac(149,147)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) 

jac(147,147)=jac(147,147)-k(8);
jac(15,147)=jac(15,147)+k(8);
jac(126,147)=jac(126,147)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) 

jac(147,147)=jac(147,147)-k(6);
jac(98,147)=jac(98,147)+k(6);
jac(48,147)=jac(48,147)+k(6);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(148,148)=jac(148,148)-k(14);
jac(100,148)=jac(100,148)+k(14);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(148,148)=jac(148,148)-k(12);
jac(124,148)=jac(124,148)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

jac(148,148)=jac(148,148)-k(10);
jac(3,148)=jac(3,148)+k(10);
jac(12,148)=jac(12,148)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(148);
jac(3,148)=jac(3,148)-k(9)*y(3);
jac(148,3)=jac(148,3)-k(9)*y(148);
jac(148,148)=jac(148,148)-k(9)*y(3);
jac(10,3)=jac(10,3)+k(9)*y(148);
jac(10,148)=jac(10,148)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) 

jac(148,148)=jac(148,148)-k(8);
jac(1,148)=jac(1,148)+k(8);
jac(8,148)=jac(8,148)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(148,148)=jac(148,148)-k(6);
jac(6,148)=jac(6,148)+k(6);
jac(7,148)=jac(7,148)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) 

jac(146,146)=jac(146,146)-k(32);
jac(148,146)=jac(148,146)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(146);
jac(32,146)=jac(32,146)-k(21)*y(32);
jac(146,32)=jac(146,32)-k(21)*y(146);
jac(146,146)=jac(146,146)-k(21)*y(32);
jac(138,32)=jac(138,32)+k(21)*y(146);
jac(138,146)=jac(138,146)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(146);
jac(2,146)=jac(2,146)-k(21)*y(2);
jac(146,2)=jac(146,2)-k(21)*y(146);
jac(146,146)=jac(146,146)-k(21)*y(2);
jac(144,2)=jac(144,2)+k(21)*y(146);
jac(144,146)=jac(144,146)+k(21)*y(2);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U!2, SH2) 

jac(146,146)=jac(146,146)-k(14);
jac(147,146)=jac(147,146)+k(14);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) 

jac(146,146)=jac(146,146)-k(12);
jac(128,146)=jac(128,146)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

jac(146,146)=jac(146,146)-k(10);
jac(3,146)=jac(3,146)+k(10);
jac(18,146)=jac(18,146)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(146);
jac(3,146)=jac(3,146)-k(9)*y(3);
jac(146,3)=jac(146,3)-k(9)*y(146);
jac(146,146)=jac(146,146)-k(9)*y(3);
jac(17,3)=jac(17,3)+k(9)*y(146);
jac(17,146)=jac(17,146)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) 

jac(146,146)=jac(146,146)-k(8);
jac(1,146)=jac(1,146)+k(8);
jac(126,146)=jac(126,146)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(146,146)=jac(146,146)-k(6);
jac(6,146)=jac(6,146)+k(6);
jac(48,146)=jac(48,146)+k(6);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(144,144)=jac(144,144)-k(22);
jac(2,144)=jac(2,144)+k(22);
jac(146,144)=jac(146,144)+k(22);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(144,144)=jac(144,144)-k(14);
jac(145,144)=jac(145,144)+k(14);

% rule    : R4_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(144,144)=jac(144,144)-k(12);
jac(139,144)=jac(139,144)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(144,144)=jac(144,144)-k(10);
jac(3,144)=jac(3,144)+k(10);
jac(24,144)=jac(24,144)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(144);
jac(3,144)=jac(3,144)-k(9)*y(3);
jac(144,3)=jac(144,3)-k(9)*y(144);
jac(144,144)=jac(144,144)-k(9)*y(3);
jac(23,3)=jac(23,3)+k(9)*y(144);
jac(23,144)=jac(23,144)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

jac(144,144)=jac(144,144)-k(8);
jac(1,144)=jac(1,144)+k(8);
jac(141,144)=jac(141,144)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(144,144)=jac(144,144)-k(6);
jac(6,144)=jac(6,144)+k(6);
jac(140,144)=jac(140,144)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(139,139)=jac(139,139)-k(31);
jac(144,139)=jac(144,139)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(139,139)=jac(139,139)-k(22);
jac(2,139)=jac(2,139)+k(22);
jac(128,139)=jac(128,139)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(139);
jac(3,139)=jac(3,139)-k(15)*y(3);
jac(139,3)=jac(139,3)-k(15)*y(139);
jac(139,139)=jac(139,139)-k(15)*y(3);
jac(129,3)=jac(129,3)+k(15)*y(139);
jac(129,139)=jac(129,139)+k(15)*y(3);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(139,139)=jac(139,139)-k(14);
jac(143,139)=jac(143,139)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(139,139)=jac(139,139)-k(10);
jac(3,139)=jac(3,139)+k(10);
jac(142,139)=jac(142,139)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

jac(139,139)=jac(139,139)-k(8);
jac(72,139)=jac(72,139)+k(8);
jac(141,139)=jac(141,139)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(139,139)=jac(139,139)-k(6);
jac(71,139)=jac(71,139)+k(6);
jac(140,139)=jac(140,139)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~Y, a~Y) 

jac(135,135)=jac(135,135)-k(33);
jac(139,135)=jac(139,135)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(135,135)=jac(135,135)-k(31);
jac(138,135)=jac(138,135)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(135,135)=jac(135,135)-k(22);
jac(32,135)=jac(32,135)+k(22);
jac(128,135)=jac(128,135)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(15)*y(135);
jac(3,135)=jac(3,135)-k(15)*y(3);
jac(135,3)=jac(135,3)-k(15)*y(135);
jac(135,135)=jac(135,135)-k(15)*y(3);
jac(130,3)=jac(130,3)+k(15)*y(135);
jac(130,135)=jac(130,135)+k(15)*y(3);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) 

jac(135,135)=jac(135,135)-k(14);
jac(137,135)=jac(137,135)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(135,135)=jac(135,135)-k(10);
jac(3,135)=jac(3,135)+k(10);
jac(136,135)=jac(136,135)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY, g~Y) 

jac(135,135)=jac(135,135)-k(8);
jac(72,135)=jac(72,135)+k(8);
jac(132,135)=jac(132,135)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(135,135)=jac(135,135)-k(6);
jac(71,135)=jac(71,135)+k(6);
jac(131,135)=jac(131,135)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(130,130)=jac(130,130)-k(33);
jac(129,130)=jac(129,130)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) 

jac(130,130)=jac(130,130)-k(22);
jac(32,130)=jac(32,130)+k(22);
jac(125,130)=jac(125,130)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY, g~Y), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(130,130)=jac(130,130)-k(16);
jac(3,130)=jac(3,130)+k(16);
jac(135,130)=jac(135,130)+k(16);

% rule    : R5_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~pY!5, g~pY), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y), Lyn(U, SH2!5) 

jac(130,130)=jac(130,130)-k(14);
jac(134,130)=jac(134,130)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!4), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(130,130)=jac(130,130)-k(10);
jac(3,130)=jac(3,130)+k(10);
jac(133,130)=jac(133,130)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY!3), Lig(l1!1, l2), Lyn(U!2, SH2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(130,130)=jac(130,130)-k(8);
jac(61,130)=jac(61,130)+k(8);
jac(132,130)=jac(132,130)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) -> Rec(a, b~Y!1, g~pY!2), Lyn(U!1, SH2), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) 

jac(130,130)=jac(130,130)-k(6);
jac(122,130)=jac(122,130)+k(6);
jac(131,130)=jac(131,130)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(125,125)=jac(125,125)-k(32);
jac(121,125)=jac(121,125)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(125);
jac(32,125)=jac(32,125)-k(21)*y(32);
jac(125,32)=jac(125,32)-k(21)*y(125);
jac(125,125)=jac(125,125)-k(21)*y(32);
jac(130,32)=jac(130,32)+k(21)*y(125);
jac(130,125)=jac(130,125)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(125);
jac(2,125)=jac(2,125)-k(21)*y(2);
jac(125,2)=jac(125,2)-k(21)*y(125);
jac(125,125)=jac(125,125)-k(21)*y(2);
jac(129,2)=jac(129,2)+k(21)*y(125);
jac(129,125)=jac(129,125)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) 

jac(125,125)=jac(125,125)-k(16);
jac(3,125)=jac(3,125)+k(16);
jac(128,125)=jac(128,125)+k(16);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(125,125)=jac(125,125)-k(14);
jac(120,125)=jac(120,125)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) 

jac(125,125)=jac(125,125)-k(10);
jac(3,125)=jac(3,125)+k(10);
jac(127,125)=jac(127,125)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(125,125)=jac(125,125)-k(8);
jac(61,125)=jac(61,125)+k(8);
jac(126,125)=jac(126,125)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(125,125)=jac(125,125)-k(6);
jac(48,125)=jac(48,125)+k(6);
jac(122,125)=jac(122,125)+k(6);

% rule    : R8
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(121,121)=jac(121,121)-k(19);
jac(125,121)=jac(125,121)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U!2, SH2) + Lyn(U, SH2) 

jac(121,121)=jac(121,121)-k(16);
jac(3,121)=jac(3,121)+k(16);
jac(124,121)=jac(124,121)+k(16);

% rule    : R5_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(121,121)=jac(121,121)-k(14);
jac(117,121)=jac(117,121)+k(14);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~Y), Lyn(U, SH2!3) + Lyn(U, SH2) 

jac(121,121)=jac(121,121)-k(10);
jac(3,121)=jac(3,121)+k(10);
jac(123,121)=jac(123,121)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(121,121)=jac(121,121)-k(8);
jac(61,121)=jac(61,121)+k(8);
jac(8,121)=jac(8,121)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~Y), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(121,121)=jac(121,121)-k(6);
jac(7,121)=jac(7,121)+k(6);
jac(122,121)=jac(122,121)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(117,117)=jac(117,117)-k(32);
jac(121,117)=jac(121,117)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(32,32)=jac(32,32)-k(21)*y(117);
jac(32,117)=jac(32,117)-k(21)*y(32);
jac(117,32)=jac(117,32)-k(21)*y(117);
jac(117,117)=jac(117,117)-k(21)*y(32);
jac(107,32)=jac(107,32)+k(21)*y(117);
jac(107,117)=jac(107,117)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(2,2)=jac(2,2)-k(21)*y(117);
jac(2,117)=jac(2,117)-k(21)*y(2);
jac(117,2)=jac(117,2)-k(21)*y(117);
jac(117,117)=jac(117,117)-k(21)*y(2);
jac(113,2)=jac(113,2)+k(21)*y(117);
jac(113,117)=jac(113,117)+k(21)*y(2);

% rule    : R8
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) 

jac(117,117)=jac(117,117)-k(19);
jac(120,117)=jac(120,117)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Lyn(U, SH2) 

jac(117,117)=jac(117,117)-k(16);
jac(3,117)=jac(3,117)+k(16);
jac(108,117)=jac(108,117)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY), Lyn(U, SH2!3) + Lyn(U, SH2) 

jac(117,117)=jac(117,117)-k(10);
jac(3,117)=jac(3,117)+k(10);
jac(119,117)=jac(119,117)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(117,117)=jac(117,117)-k(8);
jac(54,117)=jac(54,117)+k(8);
jac(8,117)=jac(8,117)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1, l2!1), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(117,117)=jac(117,117)-k(6);
jac(7,117)=jac(7,117)+k(6);
jac(118,117)=jac(118,117)+k(6);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(113,113)=jac(113,113)-k(24);
jac(107,113)=jac(107,113)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U!2, SH2), Lyn(U, SH2!4) + Syk(tSH2, l~Y, a~Y) 

jac(113,113)=jac(113,113)-k(22);
jac(2,113)=jac(2,113)+k(22);
jac(117,113)=jac(117,113)+k(22);

% rule    : R8
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(113,113)=jac(113,113)-k(19);
jac(116,113)=jac(116,113)+k(19);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(113,113)=jac(113,113)-k(16);
jac(3,113)=jac(3,113)+k(16);
jac(109,113)=jac(109,113)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY!3, g~pY!4), Lyn(U, SH2!3), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) 

jac(113,113)=jac(113,113)-k(10);
jac(3,113)=jac(3,113)+k(10);
jac(115,113)=jac(115,113)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(113,113)=jac(113,113)-k(8);
jac(8,113)=jac(8,113)+k(8);
jac(43,113)=jac(43,113)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1, l2!1), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(113,113)=jac(113,113)-k(6);
jac(7,113)=jac(7,113)+k(6);
jac(114,113)=jac(114,113)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(109,109)=jac(109,109)-k(31);
jac(96,109)=jac(96,109)+k(31);

% rule    : R10_1
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(109,109)=jac(109,109)-k(24);
jac(105,109)=jac(105,109)+k(24);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(109,109)=jac(109,109)-k(22);
jac(2,109)=jac(2,109)+k(22);
jac(108,109)=jac(108,109)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(109);
jac(3,109)=jac(3,109)-k(15)*y(3);
jac(109,3)=jac(109,3)-k(15)*y(109);
jac(109,109)=jac(109,109)-k(15)*y(3);
jac(113,3)=jac(113,3)+k(15)*y(109);
jac(113,109)=jac(113,109)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(109,109)=jac(109,109)-k(10);
jac(3,109)=jac(3,109)+k(10);
jac(112,109)=jac(112,109)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(109,109)=jac(109,109)-k(8);
jac(8,109)=jac(8,109)+k(8);
jac(111,109)=jac(111,109)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(109,109)=jac(109,109)-k(6);
jac(7,109)=jac(7,109)+k(6);
jac(110,109)=jac(110,109)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(105,105)=jac(105,105)-k(33);
jac(109,105)=jac(109,105)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(105,105)=jac(105,105)-k(31);
jac(103,105)=jac(103,105)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U!2, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(105,105)=jac(105,105)-k(22);
jac(32,105)=jac(32,105)+k(22);
jac(108,105)=jac(108,105)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(105);
jac(3,105)=jac(3,105)-k(15)*y(3);
jac(105,3)=jac(105,3)-k(15)*y(105);
jac(105,105)=jac(105,105)-k(15)*y(3);
jac(107,3)=jac(107,3)+k(15)*y(105);
jac(107,105)=jac(107,105)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(105,105)=jac(105,105)-k(10);
jac(3,105)=jac(3,105)+k(10);
jac(106,105)=jac(106,105)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(105,105)=jac(105,105)-k(8);
jac(8,105)=jac(8,105)+k(8);
jac(33,105)=jac(33,105)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(105,105)=jac(105,105)-k(6);
jac(7,105)=jac(7,105)+k(6);
jac(82,105)=jac(82,105)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(103,103)=jac(103,103)-k(33);
jac(96,103)=jac(96,103)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(103,103)=jac(103,103)-k(22);
jac(32,103)=jac(32,103)+k(22);
jac(100,103)=jac(100,103)+k(22);

% rule    : R4_2
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(103,103)=jac(103,103)-k(12);
jac(105,103)=jac(105,103)+k(12);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(103,103)=jac(103,103)-k(10);
jac(3,103)=jac(3,103)+k(10);
jac(101,103)=jac(101,103)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(103);
jac(3,103)=jac(3,103)-k(9)*y(3);
jac(103,3)=jac(103,3)-k(9)*y(103);
jac(103,103)=jac(103,103)-k(9)*y(3);
jac(104,3)=jac(104,3)+k(9)*y(103);
jac(104,103)=jac(104,103)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(103,103)=jac(103,103)-k(8);
jac(8,103)=jac(8,103)+k(8);
jac(27,103)=jac(27,103)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(103,103)=jac(103,103)-k(6);
jac(7,103)=jac(7,103)+k(6);
jac(86,103)=jac(86,103)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(101,101)=jac(101,101)-k(33);
jac(94,101)=jac(94,101)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) 

jac(101,101)=jac(101,101)-k(22);
jac(32,101)=jac(32,101)+k(22);
jac(97,101)=jac(97,101)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(101);
jac(3,101)=jac(3,101)-k(9)*y(3);
jac(101,3)=jac(101,3)-k(9)*y(101);
jac(101,101)=jac(101,101)-k(9)*y(3);
jac(103,3)=jac(103,3)+k(9)*y(101);
jac(103,101)=jac(103,101)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~pY, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(101);
jac(3,101)=jac(3,101)-k(9)*y(3);
jac(101,3)=jac(101,3)-k(9)*y(101);
jac(101,101)=jac(101,101)-k(9)*y(3);
jac(102,3)=jac(102,3)+k(9)*y(101);
jac(102,101)=jac(102,101)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(101,101)=jac(101,101)-k(8);
jac(5,101)=jac(5,101)+k(8);
jac(27,101)=jac(27,101)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y, g~Y) 

jac(101,101)=jac(101,101)-k(6);
jac(1,101)=jac(1,101)+k(6);
jac(86,101)=jac(86,101)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) 

jac(97,97)=jac(97,97)-k(32);
jac(12,97)=jac(12,97)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(97);
jac(32,97)=jac(32,97)-k(21)*y(32);
jac(97,32)=jac(97,32)-k(21)*y(97);
jac(97,97)=jac(97,97)-k(21)*y(32);
jac(101,32)=jac(101,32)+k(21)*y(97);
jac(101,97)=jac(101,97)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(97);
jac(2,97)=jac(2,97)-k(21)*y(2);
jac(97,2)=jac(97,2)-k(21)*y(97);
jac(97,97)=jac(97,97)-k(21)*y(2);
jac(94,2)=jac(94,2)+k(21)*y(97);
jac(94,97)=jac(94,97)+k(21)*y(2);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!2, l2!1), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(97);
jac(3,97)=jac(3,97)-k(9)*y(3);
jac(97,3)=jac(97,3)-k(9)*y(97);
jac(97,97)=jac(97,97)-k(9)*y(3);
jac(100,3)=jac(100,3)+k(9)*y(97);
jac(100,97)=jac(100,97)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(97);
jac(3,97)=jac(3,97)-k(9)*y(3);
jac(97,3)=jac(97,3)-k(9)*y(97);
jac(97,97)=jac(97,97)-k(9)*y(3);
jac(99,3)=jac(99,3)+k(9)*y(97);
jac(99,97)=jac(99,97)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) + Rec(a, b~Y, g~pY) 

jac(97,97)=jac(97,97)-k(8);
jac(15,97)=jac(15,97)+k(8);
jac(5,97)=jac(5,97)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) -> Rec(a!1, b~Y, g~pY), Lig(l1, l2!1) + Rec(a, b~Y, g~Y) 

jac(97,97)=jac(97,97)-k(6);
jac(1,97)=jac(1,97)+k(6);
jac(98,97)=jac(98,97)+k(6);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(94,94)=jac(94,94)-k(22);
jac(2,94)=jac(2,94)+k(22);
jac(97,94)=jac(97,94)+k(22);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(94);
jac(3,94)=jac(3,94)-k(9)*y(3);
jac(94,3)=jac(94,3)-k(9)*y(94);
jac(94,94)=jac(94,94)-k(9)*y(3);
jac(96,3)=jac(96,3)+k(9)*y(94);
jac(96,94)=jac(96,94)+k(9)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(94);
jac(3,94)=jac(3,94)-k(9)*y(3);
jac(94,3)=jac(94,3)-k(9)*y(94);
jac(94,94)=jac(94,94)-k(9)*y(3);
jac(95,3)=jac(95,3)+k(9)*y(94);
jac(95,94)=jac(95,94)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(94,94)=jac(94,94)-k(8);
jac(5,94)=jac(5,94)+k(8);
jac(21,94)=jac(21,94)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y, g~Y) 

jac(94,94)=jac(94,94)-k(6);
jac(1,94)=jac(1,94)+k(6);
jac(89,94)=jac(89,94)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) 

jac(90,90)=jac(90,90)-k(31);
jac(94,90)=jac(94,90)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY) + Syk(tSH2, l~Y, a~Y) 

jac(90,90)=jac(90,90)-k(22);
jac(2,90)=jac(2,90)+k(22);
jac(93,90)=jac(93,90)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(15)*y(90);
jac(3,90)=jac(3,90)-k(15)*y(3);
jac(90,3)=jac(90,3)-k(15)*y(90);
jac(90,90)=jac(90,90)-k(15)*y(3);
jac(88,3)=jac(88,3)+k(15)*y(90);
jac(88,90)=jac(88,90)+k(15)*y(3);

% rule    : R3
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY!4), Lyn(U!3, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(3,3)=jac(3,3)-k(9)*y(90);
jac(3,90)=jac(3,90)-k(9)*y(3);
jac(90,3)=jac(90,3)-k(9)*y(90);
jac(90,90)=jac(90,90)-k(9)*y(3);
jac(92,3)=jac(92,3)+k(9)*y(90);
jac(92,90)=jac(92,90)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY, g~Y), Lig(l1!1, l2) 

jac(90,90)=jac(90,90)-k(8);
jac(91,90)=jac(91,90)+k(8);
jac(21,90)=jac(21,90)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY, g~Y) 

jac(90,90)=jac(90,90)-k(6);
jac(72,90)=jac(72,90)+k(6);
jac(89,90)=jac(89,90)+k(6);

% rule    : R11_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(88,88)=jac(88,88)-k(26);
jac(85,88)=jac(85,88)+k(26);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) 

jac(88,88)=jac(88,88)-k(22);
jac(2,88)=jac(2,88)+k(22);
jac(63,88)=jac(63,88)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(88,88)=jac(88,88)-k(18);
jac(80,88)=jac(80,88)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(88,88)=jac(88,88)-k(16);
jac(3,88)=jac(3,88)+k(16);
jac(90,88)=jac(90,88)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(88);
jac(3,88)=jac(3,88)-k(9)*y(3);
jac(88,3)=jac(88,3)-k(9)*y(88);
jac(88,88)=jac(88,88)-k(9)*y(3);
jac(65,3)=jac(65,3)+k(9)*y(88);
jac(65,88)=jac(65,88)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(88,88)=jac(88,88)-k(8);
jac(62,88)=jac(62,88)+k(8);
jac(21,88)=jac(21,88)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(88,88)=jac(88,88)-k(6);
jac(61,88)=jac(61,88)+k(6);
jac(89,88)=jac(89,88)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(85,85)=jac(85,85)-k(33);
jac(88,85)=jac(88,85)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

jac(85,85)=jac(85,85)-k(22);
jac(32,85)=jac(32,85)+k(22);
jac(63,85)=jac(63,85)+k(22);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(85,85)=jac(85,85)-k(18);
jac(81,85)=jac(81,85)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(85,85)=jac(85,85)-k(16);
jac(3,85)=jac(3,85)+k(16);
jac(87,85)=jac(87,85)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(85);
jac(3,85)=jac(3,85)-k(9)*y(3);
jac(85,3)=jac(85,3)-k(9)*y(85);
jac(85,85)=jac(85,85)-k(9)*y(3);
jac(66,3)=jac(66,3)+k(9)*y(85);
jac(66,85)=jac(66,85)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(85,85)=jac(85,85)-k(8);
jac(62,85)=jac(62,85)+k(8);
jac(27,85)=jac(27,85)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(85,85)=jac(85,85)-k(6);
jac(61,85)=jac(61,85)+k(6);
jac(86,85)=jac(86,85)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(81,81)=jac(81,81)-k(33);
jac(80,81)=jac(80,81)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(81,81)=jac(81,81)-k(31);
jac(85,81)=jac(85,81)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) 

jac(81,81)=jac(81,81)-k(22);
jac(32,81)=jac(32,81)+k(22);
jac(75,81)=jac(75,81)+k(22);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY!3), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(81,81)=jac(81,81)-k(16);
jac(3,81)=jac(3,81)+k(16);
jac(84,81)=jac(84,81)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(81);
jac(3,81)=jac(3,81)-k(15)*y(3);
jac(81,3)=jac(81,3)-k(15)*y(81);
jac(81,81)=jac(81,81)-k(15)*y(3);
jac(83,3)=jac(83,3)+k(15)*y(81);
jac(83,81)=jac(83,81)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) 

jac(81,81)=jac(81,81)-k(8);
jac(62,81)=jac(62,81)+k(8);
jac(33,81)=jac(33,81)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1, l2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(81,81)=jac(81,81)-k(6);
jac(61,81)=jac(61,81)+k(6);
jac(82,81)=jac(82,81)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(75,75)=jac(75,75)-k(32);
jac(70,75)=jac(70,75)+k(32);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(75,75)=jac(75,75)-k(31);
jac(63,75)=jac(63,75)+k(31);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(75);
jac(32,75)=jac(32,75)-k(21)*y(32);
jac(75,32)=jac(75,32)-k(21)*y(75);
jac(75,75)=jac(75,75)-k(21)*y(32);
jac(81,32)=jac(81,32)+k(21)*y(75);
jac(81,75)=jac(81,75)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY!4), Lyn(U, SH2!2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(75);
jac(2,75)=jac(2,75)-k(21)*y(2);
jac(75,2)=jac(75,2)-k(21)*y(75);
jac(75,75)=jac(75,75)-k(21)*y(2);
jac(80,2)=jac(80,2)+k(21)*y(75);
jac(80,75)=jac(80,75)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~pY) + Lyn(U, SH2) 

jac(75,75)=jac(75,75)-k(16);
jac(3,75)=jac(3,75)+k(16);
jac(79,75)=jac(79,75)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~pY), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(75);
jac(3,75)=jac(3,75)-k(15)*y(3);
jac(75,3)=jac(75,3)-k(15)*y(75);
jac(75,75)=jac(75,75)-k(15)*y(3);
jac(78,3)=jac(78,3)+k(15)*y(75);
jac(78,75)=jac(78,75)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~pY) 

jac(75,75)=jac(75,75)-k(8);
jac(77,75)=jac(77,75)+k(8);
jac(62,75)=jac(62,75)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~pY), Lig(l1, l2!1) 

jac(75,75)=jac(75,75)-k(6);
jac(76,75)=jac(76,75)+k(6);
jac(61,75)=jac(61,75)+k(6);

% rule    : R14
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) 

jac(70,70)=jac(70,70)-k(31);
jac(68,70)=jac(68,70)+k(31);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~pY), Lyn(U, SH2!2) 

jac(70,70)=jac(70,70)-k(20);
jac(75,70)=jac(75,70)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~pY, g~Y) + Lyn(U, SH2) 

jac(70,70)=jac(70,70)-k(16);
jac(3,70)=jac(3,70)+k(16);
jac(74,70)=jac(74,70)+k(16);

% rule    : R6
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY!4, g~Y), Lyn(U, SH2!2), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(70);
jac(3,70)=jac(3,70)-k(15)*y(3);
jac(70,3)=jac(70,3)-k(15)*y(70);
jac(70,70)=jac(70,70)-k(15)*y(3);
jac(73,3)=jac(73,3)+k(15)*y(70);
jac(73,70)=jac(73,70)+k(15)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~pY, g~Y) 

jac(70,70)=jac(70,70)-k(8);
jac(72,70)=jac(72,70)+k(8);
jac(62,70)=jac(62,70)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~pY, g~Y), Lig(l1, l2!1) 

jac(70,70)=jac(70,70)-k(6);
jac(71,70)=jac(71,70)+k(6);
jac(61,70)=jac(61,70)+k(6);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) 

jac(68,68)=jac(68,68)-k(20);
jac(63,68)=jac(63,68)+k(20);

% rule    : R7_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~pY, g~Y), Lyn(U, SH2!2) 

jac(68,68)=jac(68,68)-k(18);
jac(70,68)=jac(70,68)+k(18);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

jac(68,68)=jac(68,68)-k(16);
jac(3,68)=jac(3,68)+k(16);
jac(69,68)=jac(69,68)+k(16);

% rule    : R3
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(68);
jac(3,68)=jac(3,68)-k(9)*y(3);
jac(68,3)=jac(68,3)-k(9)*y(68);
jac(68,68)=jac(68,68)-k(9)*y(3);
jac(67,3)=jac(67,3)+k(9)*y(68);
jac(67,68)=jac(67,68)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y, g~Y) 

jac(68,68)=jac(68,68)-k(8);
jac(1,68)=jac(1,68)+k(8);
jac(62,68)=jac(62,68)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) -> Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) + Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(68,68)=jac(68,68)-k(6);
jac(6,68)=jac(6,68)+k(6);
jac(61,68)=jac(61,68)+k(6);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(67,67)=jac(67,67)-k(20);
jac(60,67)=jac(60,67)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) 

jac(67,67)=jac(67,67)-k(16);
jac(3,67)=jac(3,67)+k(16);
jac(13,67)=jac(13,67)+k(16);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(67,67)=jac(67,67)-k(13);
jac(41,67)=jac(41,67)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Lyn(U, SH2!2) + Lyn(U, SH2) 

jac(67,67)=jac(67,67)-k(10);
jac(3,67)=jac(3,67)+k(10);
jac(68,67)=jac(68,67)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(67,67)=jac(67,67)-k(8);
jac(7,67)=jac(7,67)+k(8);
jac(62,67)=jac(62,67)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(67,67)=jac(67,67)-k(6);
jac(61,67)=jac(61,67)+k(6);
jac(9,67)=jac(9,67)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(60,60)=jac(60,60)-k(32);
jac(67,60)=jac(67,60)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(60);
jac(32,60)=jac(32,60)-k(21)*y(32);
jac(60,32)=jac(60,32)-k(21)*y(60);
jac(60,60)=jac(60,60)-k(21)*y(32);
jac(66,32)=jac(66,32)+k(21)*y(60);
jac(66,60)=jac(66,60)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(60);
jac(2,60)=jac(2,60)-k(21)*y(2);
jac(60,2)=jac(60,2)-k(21)*y(60);
jac(60,60)=jac(60,60)-k(21)*y(2);
jac(65,2)=jac(65,2)+k(21)*y(60);
jac(65,60)=jac(65,60)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) 

jac(60,60)=jac(60,60)-k(16);
jac(3,60)=jac(3,60)+k(16);
jac(64,60)=jac(64,60)+k(16);

% rule    : R5
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(60,60)=jac(60,60)-k(13);
jac(53,60)=jac(53,60)+k(13);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) 

jac(60,60)=jac(60,60)-k(10);
jac(3,60)=jac(3,60)+k(10);
jac(63,60)=jac(63,60)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(60,60)=jac(60,60)-k(8);
jac(48,60)=jac(48,60)+k(8);
jac(62,60)=jac(62,60)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~Y), Lyn(U, SH2!1) 

jac(60,60)=jac(60,60)-k(6);
jac(61,60)=jac(61,60)+k(6);
jac(47,60)=jac(47,60)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(53,53)=jac(53,53)-k(32);
jac(41,53)=jac(41,53)+k(32);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) 

jac(53,53)=jac(53,53)-k(32);
jac(60,53)=jac(60,53)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(53);
jac(32,53)=jac(32,53)-k(21)*y(32);
jac(53,32)=jac(53,32)-k(21)*y(53);
jac(53,53)=jac(53,53)-k(21)*y(32);
jac(40,32)=jac(40,32)+k(21)*y(53);
jac(40,53)=jac(40,53)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(53);
jac(2,53)=jac(2,53)-k(21)*y(2);
jac(53,2)=jac(53,2)-k(21)*y(53);
jac(53,53)=jac(53,53)-k(21)*y(2);
jac(46,2)=jac(46,2)+k(21)*y(53);
jac(46,53)=jac(46,53)+k(21)*y(2);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(53);
jac(32,53)=jac(32,53)-k(21)*y(32);
jac(53,32)=jac(53,32)-k(21)*y(53);
jac(53,53)=jac(53,53)-k(21)*y(32);
jac(59,32)=jac(59,32)+k(21)*y(53);
jac(59,53)=jac(59,53)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY!5), Lyn(U, SH2!2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(53);
jac(2,53)=jac(2,53)-k(21)*y(2);
jac(53,2)=jac(53,2)-k(21)*y(53);
jac(53,53)=jac(53,53)-k(21)*y(2);
jac(58,2)=jac(58,2)+k(21)*y(53);
jac(58,53)=jac(58,53)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~pY), Lyn(U!3, SH2) + Lyn(U, SH2) 

jac(53,53)=jac(53,53)-k(16);
jac(3,53)=jac(3,53)+k(16);
jac(57,53)=jac(57,53)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~pY), Lyn(U, SH2!2) + Lyn(U, SH2) 

jac(53,53)=jac(53,53)-k(10);
jac(3,53)=jac(3,53)+k(10);
jac(56,53)=jac(56,53)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2), Lyn(U, SH2!2) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(53,53)=jac(53,53)-k(8);
jac(48,53)=jac(48,53)+k(8);
jac(55,53)=jac(55,53)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) -> Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~pY!1, g~pY), Lyn(U, SH2!1) 

jac(53,53)=jac(53,53)-k(6);
jac(54,53)=jac(54,53)+k(6);
jac(47,53)=jac(47,53)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(46,46)=jac(46,46)-k(32);
jac(42,46)=jac(42,46)+k(32);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(46,46)=jac(46,46)-k(23);
jac(40,46)=jac(40,46)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(46,46)=jac(46,46)-k(22);
jac(2,46)=jac(2,46)+k(22);
jac(53,46)=jac(53,46)+k(22);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~pY, a~Y), Lyn(U!5, SH2) 

jac(32,32)=jac(32,32)-k(21)*y(46);
jac(32,46)=jac(32,46)-k(21)*y(32);
jac(46,32)=jac(46,32)-k(21)*y(46);
jac(46,46)=jac(46,46)-k(21)*y(32);
jac(52,32)=jac(52,32)+k(21)*y(46);
jac(52,46)=jac(52,46)+k(21)*y(32);

% rule    : R9
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY!6), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Syk(tSH2!6, l~Y, a~Y), Lyn(U!5, SH2) 

jac(2,2)=jac(2,2)-k(21)*y(46);
jac(2,46)=jac(2,46)-k(21)*y(2);
jac(46,2)=jac(46,2)-k(21)*y(46);
jac(46,46)=jac(46,46)-k(21)*y(2);
jac(51,2)=jac(51,2)+k(21)*y(46);
jac(51,46)=jac(51,46)+k(21)*y(2);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~pY), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(46,46)=jac(46,46)-k(16);
jac(3,46)=jac(3,46)+k(16);
jac(50,46)=jac(50,46)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(46,46)=jac(46,46)-k(10);
jac(3,46)=jac(3,46)+k(10);
jac(49,46)=jac(49,46)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~pY), Lyn(U!1, SH2) 

jac(46,46)=jac(46,46)-k(8);
jac(48,46)=jac(48,46)+k(8);
jac(44,46)=jac(44,46)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~pY), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(46,46)=jac(46,46)-k(6);
jac(47,46)=jac(47,46)+k(6);
jac(43,46)=jac(43,46)+k(6);

% rule    : R10
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(42,42)=jac(42,42)-k(23);
jac(36,42)=jac(36,42)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(42,42)=jac(42,42)-k(22);
jac(2,42)=jac(2,42)+k(22);
jac(41,42)=jac(41,42)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(42,42)=jac(42,42)-k(20);
jac(46,42)=jac(46,42)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(42,42)=jac(42,42)-k(16);
jac(3,42)=jac(3,42)+k(16);
jac(25,42)=jac(25,42)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Lyn(U, SH2) 

jac(42,42)=jac(42,42)-k(10);
jac(3,42)=jac(3,42)+k(10);
jac(45,42)=jac(45,42)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(42,42)=jac(42,42)-k(8);
jac(7,42)=jac(7,42)+k(8);
jac(44,42)=jac(44,42)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(42,42)=jac(42,42)-k(6);
jac(9,42)=jac(9,42)+k(6);
jac(43,42)=jac(43,42)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~Y, a~Y), Lyn(U!5, SH2) 

jac(36,36)=jac(36,36)-k(33);
jac(42,36)=jac(42,36)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U, SH2!2), Lyn(U!4, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(36,36)=jac(36,36)-k(22);
jac(32,36)=jac(32,36)+k(22);
jac(41,36)=jac(41,36)+k(22);

% rule    : R8_2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y!5, g~pY), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y), Lyn(U!5, SH2) 

jac(36,36)=jac(36,36)-k(20);
jac(40,36)=jac(40,36)+k(20);

% rule    : R6_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(36,36)=jac(36,36)-k(16);
jac(3,36)=jac(3,36)+k(16);
jac(31,36)=jac(31,36)+k(16);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2!4), Rec(a!4, b~Y, g~Y), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Lyn(U, SH2) 

jac(36,36)=jac(36,36)-k(10);
jac(3,36)=jac(3,36)+k(10);
jac(39,36)=jac(39,36)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a!1, b~pY!2, g~pY!3), Lig(l1!1, l2), Lyn(U, SH2!2), Syk(tSH2!3, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(36,36)=jac(36,36)-k(8);
jac(7,36)=jac(7,36)+k(8);
jac(38,36)=jac(38,36)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) -> Rec(a, b~pY!1, g~pY!2), Lyn(U, SH2!1), Syk(tSH2!2, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(36,36)=jac(36,36)-k(6);
jac(9,36)=jac(9,36)+k(6);
jac(37,36)=jac(37,36)+k(6);

% rule    : R16
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(31,31)=jac(31,31)-k(33);
jac(25,31)=jac(25,31)+k(33);

% rule    : R14
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(31,31)=jac(31,31)-k(31);
jac(26,31)=jac(26,31)+k(31);

% rule    : R9_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(31,31)=jac(31,31)-k(22);
jac(32,31)=jac(32,31)+k(22);
jac(19,31)=jac(19,31)+k(22);

% rule    : R6
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~pY!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U, SH2!4) 

jac(3,3)=jac(3,3)-k(15)*y(31);
jac(3,31)=jac(3,31)-k(15)*y(3);
jac(31,3)=jac(31,3)-k(15)*y(31);
jac(31,31)=jac(31,31)-k(15)*y(3);
jac(36,3)=jac(36,3)+k(15)*y(31);
jac(36,31)=jac(36,31)+k(15)*y(3);

% rule    : R3_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(31,31)=jac(31,31)-k(10);
jac(3,31)=jac(3,31)+k(10);
jac(35,31)=jac(35,31)+k(10);

% rule    : R2_1_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(31,31)=jac(31,31)-k(8);
jac(7,31)=jac(7,31)+k(8);
jac(34,31)=jac(34,31)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~pY, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(31,31)=jac(31,31)-k(6);
jac(9,31)=jac(9,31)+k(6);
jac(33,31)=jac(33,31)+k(6);

% rule    : R18
% reaction: Syk(tSH2, l~pY, a~Y) -> Syk(tSH2, l~Y, a~Y) 

jac(32,32)=jac(32,32)-k(35);
jac(2,32)=jac(2,32)+k(35);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(32,32)=jac(32,32)-k(21)*y(14);
jac(32,14)=jac(32,14)-k(21)*y(32);
jac(14,32)=jac(14,32)-k(21)*y(14);
jac(14,14)=jac(14,14)-k(21)*y(32);
jac(26,32)=jac(26,32)+k(21)*y(14);
jac(26,14)=jac(26,14)+k(21)*y(32);

% rule    : R16
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(26,26)=jac(26,26)-k(33);
jac(20,26)=jac(20,26)+k(33);

% rule    : R9_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~pY, a~Y) 

jac(26,26)=jac(26,26)-k(22);
jac(32,26)=jac(32,26)+k(22);
jac(14,26)=jac(14,26)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(26,26)=jac(26,26)-k(11);
jac(31,26)=jac(31,26)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) 

jac(26,26)=jac(26,26)-k(10);
jac(3,26)=jac(3,26)+k(10);
jac(30,26)=jac(30,26)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~pY, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(26);
jac(3,26)=jac(3,26)-k(9)*y(3);
jac(26,3)=jac(26,3)-k(9)*y(26);
jac(26,26)=jac(26,26)-k(9)*y(3);
jac(29,3)=jac(29,3)+k(9)*y(26);
jac(29,26)=jac(29,26)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~pY, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(26,26)=jac(26,26)-k(8);
jac(7,26)=jac(7,26)+k(8);
jac(28,26)=jac(28,26)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~pY, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(26,26)=jac(26,26)-k(6);
jac(9,26)=jac(9,26)+k(6);
jac(27,26)=jac(27,26)+k(6);

% rule    : R10
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~pY, a~Y) 

jac(20,20)=jac(20,20)-k(23);
jac(26,20)=jac(26,20)+k(23);

% rule    : R9_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) 

jac(20,20)=jac(20,20)-k(22);
jac(2,20)=jac(2,20)+k(22);
jac(14,20)=jac(14,20)+k(22);

% rule    : R4
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~pY, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!4, SH2), Syk(tSH2!2, l~Y, a~Y) 

jac(20,20)=jac(20,20)-k(11);
jac(25,20)=jac(25,20)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2!3), Rec(a!3, b~Y, g~Y), Syk(tSH2!2, l~Y, a~Y) + Lyn(U, SH2) 

jac(20,20)=jac(20,20)-k(10);
jac(3,20)=jac(3,20)+k(10);
jac(24,20)=jac(24,20)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY!5), Lyn(U!2, SH2), Syk(tSH2!5, l~Y, a~Y), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(20);
jac(3,20)=jac(3,20)-k(9)*y(3);
jac(20,3)=jac(20,3)-k(9)*y(20);
jac(20,20)=jac(20,20)-k(9)*y(3);
jac(23,3)=jac(23,3)+k(9)*y(20);
jac(23,20)=jac(23,20)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a!1, b~Y, g~pY!2), Lig(l1!1, l2), Syk(tSH2!2, l~Y, a~Y) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(20,20)=jac(20,20)-k(8);
jac(7,20)=jac(7,20)+k(8);
jac(22,20)=jac(22,20)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) -> Rec(a, b~Y, g~pY!1), Syk(tSH2!1, l~Y, a~Y) + Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(20,20)=jac(20,20)-k(6);
jac(9,20)=jac(9,20)+k(6);
jac(21,20)=jac(21,20)+k(6);

% rule    : R15
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(14,14)=jac(14,14)-k(32);
jac(11,14)=jac(11,14)+k(32);

% rule    : R9
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Syk(tSH2, l~Y, a~Y) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y, g~pY!4), Lyn(U!2, SH2), Syk(tSH2!4, l~Y, a~Y) 

jac(2,2)=jac(2,2)-k(21)*y(14);
jac(2,14)=jac(2,14)-k(21)*y(2);
jac(14,2)=jac(14,2)-k(21)*y(14);
jac(14,14)=jac(14,14)-k(21)*y(2);
jac(20,2)=jac(20,2)+k(21)*y(14);
jac(20,14)=jac(20,14)+k(21)*y(2);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(14,14)=jac(14,14)-k(11);
jac(19,14)=jac(19,14)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

jac(14,14)=jac(14,14)-k(10);
jac(3,14)=jac(3,14)+k(10);
jac(18,14)=jac(18,14)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!3, l2!1), Rec(a!3, b~Y!4, g~pY), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(14);
jac(3,14)=jac(3,14)-k(9)*y(3);
jac(14,3)=jac(14,3)-k(9)*y(14);
jac(14,14)=jac(14,14)-k(9)*y(3);
jac(17,3)=jac(17,3)+k(9)*y(14);
jac(17,14)=jac(17,14)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~pY), Lig(l1!1, l2) 

jac(14,14)=jac(14,14)-k(8);
jac(16,14)=jac(16,14)+k(8);
jac(7,14)=jac(7,14)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~pY) 

jac(14,14)=jac(14,14)-k(6);
jac(15,14)=jac(15,14)+k(6);
jac(9,14)=jac(9,14)+k(6);

% rule    : R5
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~pY), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(11,11)=jac(11,11)-k(13);
jac(14,11)=jac(14,11)+k(13);

% rule    : R4
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~pY, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(11,11)=jac(11,11)-k(11);
jac(13,11)=jac(13,11)+k(11);

% rule    : R3_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y, g~Y) + Lyn(U, SH2) 

jac(11,11)=jac(11,11)-k(10);
jac(3,11)=jac(3,11)+k(10);
jac(12,11)=jac(12,11)+k(10);

% rule    : R3
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) + Lyn(U, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(11);
jac(3,11)=jac(3,11)-k(9)*y(3);
jac(11,3)=jac(11,3)-k(9)*y(11);
jac(11,11)=jac(11,11)-k(9)*y(3);
jac(10,3)=jac(10,3)+k(9)*y(11);
jac(10,11)=jac(10,11)+k(9)*y(3);

% rule    : R2_1_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(11,11)=jac(11,11)-k(8);
jac(5,11)=jac(5,11)+k(8);
jac(7,11)=jac(7,11)+k(8);

% rule    : R2_op
% reaction: Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) 

jac(11,11)=jac(11,11)-k(6);
jac(1,11)=jac(1,11)+k(6);
jac(9,11)=jac(9,11)+k(6);

% rule    : R3_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) + Lyn(U, SH2) 

jac(9,9)=jac(9,9)-k(10);
jac(3,9)=jac(3,9)+k(10);
jac(6,9)=jac(6,9)+k(10);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y, g~Y) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2!2), Rec(a!2, b~Y!3, g~Y), Lyn(U!3, SH2) 

jac(1,1)=jac(1,1)-k(5)*y(9);
jac(1,9)=jac(1,9)-k(5)*y(1);
jac(9,1)=jac(9,1)-k(5)*y(9);
jac(9,9)=jac(9,9)-k(5)*y(1);
jac(11,1)=jac(11,1)+k(5)*y(9);
jac(11,9)=jac(11,9)+k(5)*y(1);

% rule    : R2
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) + Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2!3), Rec(a!3, b~Y!4, g~Y), Lyn(U!2, SH2), Lyn(U!4, SH2) 

jac(7,7)=jac(7,7)-k(5)*y(9);
jac(7,9)=jac(7,9)-k(5)*y(7);
jac(9,7)=jac(9,7)-k(5)*y(9);
jac(9,9)=jac(9,9)-k(5)*y(7);
jac(10,7)=jac(10,7)+k(5)*y(9);
jac(10,9)=jac(10,9)+k(5)*y(7);

% rule    : R1_2_op
% reaction: Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Lig(l1, l2) 

jac(9,9)=jac(9,9)-k(4);
jac(4,9)=jac(4,9)+k(4);
jac(7,9)=jac(7,9)+k(4);

% rule    : R3_op
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) -> Rec(a, b~Y, g~Y) + Lyn(U, SH2) 

jac(7,7)=jac(7,7)-k(10);
jac(3,7)=jac(3,7)+k(10);
jac(1,7)=jac(1,7)+k(10);

% rule    : R1_2
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1, l2!1), Lyn(U!2, SH2) 

jac(4,4)=jac(4,4)-k(3)*y(7);
jac(4,7)=jac(4,7)-k(3)*y(4);
jac(7,4)=jac(7,4)-k(3)*y(7);
jac(7,7)=jac(7,7)-k(3)*y(4);
jac(9,4)=jac(9,4)+k(3)*y(7);
jac(9,7)=jac(9,7)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) + Lig(l1, l2) -> Rec(a!1, b~Y!2, g~Y), Lig(l1!1, l2), Lyn(U!2, SH2) 

jac(4,4)=jac(4,4)-k(1)*y(7);
jac(4,7)=jac(4,7)-k(1)*y(4);
jac(7,4)=jac(7,4)-k(1)*y(7);
jac(7,7)=jac(7,7)-k(1)*y(4);
jac(8,4)=jac(8,4)+k(1)*y(7);
jac(8,7)=jac(8,7)+k(1)*y(4);

% rule    : R3
% reaction: Rec(a, b~Y, g~Y) + Lyn(U, SH2) -> Rec(a, b~Y!1, g~Y), Lyn(U!1, SH2) 

jac(3,3)=jac(3,3)-k(9)*y(1);
jac(3,1)=jac(3,1)-k(9)*y(3);
jac(1,3)=jac(1,3)-k(9)*y(1);
jac(1,1)=jac(1,1)-k(9)*y(3);
jac(7,3)=jac(7,3)+k(9)*y(1);
jac(7,1)=jac(7,1)+k(9)*y(3);

% rule    : R1_2
% reaction: Rec(a, b~Y, g~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1, l2!1) 

jac(4,4)=jac(4,4)-k(3)*y(1);
jac(4,1)=jac(4,1)-k(3)*y(4);
jac(1,4)=jac(1,4)-k(3)*y(1);
jac(1,1)=jac(1,1)-k(3)*y(4);
jac(6,4)=jac(6,4)+k(3)*y(1);
jac(6,1)=jac(6,1)+k(3)*y(4);

% rule    : R1
% reaction: Rec(a, b~Y, g~Y) + Lig(l1, l2) -> Rec(a!1, b~Y, g~Y), Lig(l1!1, l2) 

jac(4,4)=jac(4,4)-k(1)*y(1);
jac(4,1)=jac(4,1)-k(1)*y(4);
jac(1,4)=jac(1,4)-k(1)*y(1);
jac(1,1)=jac(1,1)-k(1)*y(4);
jac(5,4)=jac(5,4)+k(1)*y(1);
jac(5,1)=jac(5,1)+k(1)*y(4);
end


function obs=ode_obs(y)

global nobs
global var
obs=zeros(nobs,1);

t = y(655);

obs(1)=t; % [T]

end


main();

